/**
* WebViewer3D provides an HTML5-based 3D viewing component
*
* @module WebViewer3D
* @author DOGO, PHST
* @copyright	2014 Esko Stonecube Ltd. All rights reserved.
* @version  0.7.1
*/

define([
    "WV3Dmod/ViewerController",
    "WV3Dmod/Constants",
    "WV3Dmod/LogMessage",
    "WV3D/BuildVersion"
    ], function (
        WV3DViewerController,
        WV3DConstants,
        LogMessage,
        WV3DBuildVersion
        ) {

    'use strict';

    /**
	 * The WebViewer3DAPI object is the entry point for WebViewer3D. To use,
	 * create a new instance and provide the DOM element in which the viewer
	 * should be inserted. Dispose() function has to be called before this object
     * goes out of scope.
     * Warning: most of the API functions should be called asynchronuously, when the
     * scene has been loaded succesfully, therefore, most of them should be called in
     * __defaultClientLoadedSceneCallback or __clientLoadedSceneCallback
	 *
	 * @alias module:WebViewer3D
	 * @class WebViewer3DAPI
	 * @param {HTMLObject}
	 *            __context - a reference to the DOM element in which the viewer
	 *            should be inserted - can be null if no need for 3D canvas (e.g. Snapshots only)
	 * @param {function} __clientCallback - default callback function pointer to pass on
	 *            various error states back to client. eg., when loading of
	 *            scene is finished. callback function's argument 0 indicates
	 *            success with no errors. Otherwise indicates the error state
	 *            TODO Define arguments for error states (eg., 1 for error
	 *            during http request). This callback is a stateless callback
	 *            (not made through any object instance).
     *            The client callback gives also:
     *            arg[1] => the current file geometry percentage
     *            arg[2] => the current file texture percentage
     *            arg[3] => the current number of queried files
     *            arg[4] => the current number of successfully retrieved files
     *            E.G. the client just needs to test if (arg[1]=100 & arg[2]=100 & arg[3] = arg[4]) to
     *            know if "everything" (the entire scene) has been loaded (and
     *            displayed at least once).
     * @param {function} __defaultClientUIAddElementCallback - default callback funtion
     *            giving the current node path (arg[1]) and a boolean indicating
     *            if it is a leaf (arg[2])
     * @param {function} __clientLogMessageCallback - default callback funtion
     *            for output message from a string (arg[1])
	 */
    var WebViewer3DAPI = function(__context, __defaultClientLoadedSceneCallback, __defaultClientUIAddElementCallback, __clientLogMessageCallback)
    {
        console.log("ESKO HTML Viewer - Client V"+WV3DBuildVersion.SuiteNumber+"/"+WV3DBuildVersion.P4VChangelistNumber);
        LogMessage.SetCallbackFunction(__clientLogMessageCallback);
        LogMessage.Log("Creating WebViewer3DAPI Object");

        this._controller = new WV3DViewerController(__context, __defaultClientLoadedSceneCallback, __defaultClientUIAddElementCallback);
    }

    /**
    * Clear the scene and release all GPU resources. Has to be called before API object
    * goes out of scope to avoid memory leaks.
    *
    * @function Dispose
    */
    WebViewer3DAPI.prototype.Dispose = function()
    {
        LogMessage.Log("API Dispose");
        this._controller.Dispose();
    }

    /**
    * Set Log message callback
    *
    * @function SetLogMessageCallback
    * @param {function} __clientLogMessageCallback - default callback funtion
    *            for output message from a string (arg[1])
    */
    WebViewer3DAPI.prototype.SetLogMessageCallback = function(__clientLogMessageCallback)
    {
        LogMessage.SetCallbackFunction(__clientLogMessageCallback);
    }

    /**
    * Set end camera interaction callback
    *
    * @function SetEndCameraInteractionCallback
    * @param {function} __clientEndCameraInteractionCallback - callback function at the end of camera interaction
    */
    WebViewer3DAPI.prototype.SetEndCameraInteractionCallback = function(__clientEndCameraInteractionCallback)
    {
        this._controller.SetEndCameraInteractionCallback(__clientEndCameraInteractionCallback);
    }

    /**
    * Trigger a resize event that allows the WebGLViewer to adapt to the current context size
    * This function is useful only when you do not want to use window resize event trigger
    * User should use window.trigger('resize’) otherwise.
    * This Resize function is useful especially if the webgl canvas is embedded in an independant
    * and resizable dom element and avoid unnecessary refresh for the whole window. (i.e. a dockable
    * widget)
    * Warning: this function should not be called when Headless API is being used, since it might cause
    * scene not to be rendered.
    *
    * @function Resize
    */
    WebViewer3DAPI.prototype.Resize = function()
    {
        this._controller.get('_view').Resize();
    }

    /**
    * Function deprecated
    *
    * @function Initialize: <i> deprecated </i>
    */
    WebViewer3DAPI.prototype.Initialize = function()
    {
        LogMessage.Log("Initialize: Deprecated API function !!!!!!!!!!!!!!!! (No need to call that function anymore)");
    }

    /**
    * Load a scene from a URL that returns a JSON file containing the scene description
    *
    * @function LoadScene
    * @param {String} __sceneURL - URL to the scene data
    * @param {Number} __coplanarityEpsilon [0.02] - if > 0 used to tune tolerance for
    * smooth surfaces so that these are considered as one single highlighted surface, i.e., only
    * their silhouette will appear.
    * @param {function} __clientCallback - default callback function pointer to pass on
    *            various error states back to client. eg., when loading of
    *            scene is finished. callback function's argument 0 indicates
    *            success with no errors. Otherwise indicates the error state
    *            TODO Define arguments for error states (eg., 1 for error
    *            during http request). This callback is a stateless callback
    *            (not made through any object instance).
    *            The client callback gives also:
    *            arg[1] => the current file geometry percentage
    *            arg[2] => the current file texture percentage
    *            arg[3] => the current number of queried files
    *            arg[4] => the current number of successfully retrieved files
    *            E.G. the client just needs to test if (arg[1]=100 & arg[2]=100 & arg[3] = arg[4]) to
    *            know if "everything" (the entire scene) has been loaded (and
    *            displayed at least once).
    * @param {function} __defaultClientUIAddElementCallback - default callback funtion
    *            giving the current node path (arg[1]) and a boolean indicating
    *            if it is a leaf (arg[2])
    * @param {Number} __boundaryhardedges [true] - if true, builds the hard edges highlights at surface boundaries
    */
    WebViewer3DAPI.prototype.LoadScene = function(__sceneURL, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges)
    {
        LogMessage.Log("API Load Scene from URL");

        this._controller.LoadSceneFromURL(__sceneURL, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges);
    }

    /**
    * Load a scene from a local JSON file containing the scene description
    *
    * @function LoadSceneFromLocalFile
    * @param {File} __file - A File API file object
    * @param {Number} __coplanarityEpsilon [0.02] - if > 0 used to tune tolerance for
    * smooth surfaces so that these are considered as one single highlighted surface, i.e., only
    * their silhouette will appear.
    * @param {function} __clientCallback - default callback function pointer to pass on
    *            various error states back to client. eg., when loading of
    *            scene is finished. callback function's argument 0 indicates
    *            success with no errors. Otherwise indicates the error state
    *            TODO Define arguments for error states (eg., 1 for error
    *            during http request). This callback is a stateless callback
    *            (not made through any object instance).
    *            The client callback gives also:
    *            arg[1] => the current file geometry percentage
    *            arg[2] => the current file texture percentage
    *            arg[3] => the current number of queried files
    *            arg[4] => the current number of successfully retrieved files
    *            E.G. the client just needs to test if (arg[1]=100 & arg[2]=100 & arg[3] = arg[4]) to
    *            know if "everything" (the entire scene) has been loaded (and
    *            displayed at least once).
    * @param {function} __defaultClientUIAddElementCallback - default callback funtion
    *            giving the current node path (arg[1]) and a boolean indicating
    *            if it is a leaf (arg[2])
    * @param {Number} __boundaryhardedges [true] - if true, builds the hard edges highlights at surface boundaries
    */
    WebViewer3DAPI.prototype.LoadSceneFromLocalFile = function(__file, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges)
    {
        LogMessage.Log("API Load Scene from local file");

        var reader = new FileReader();
        var self = this;

        reader.onload = function(e) { self.LoadSceneFromObject(JSON.parse(reader.result), __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges); };

        reader.readAsText(__file);
    }

    /**
    * Load a scene from an object
    *
    * @function LoadSceneFromObject
    * @param {Object} __object - a JSON scene object (not a string!)
    * @param {Number} __coplanarityEpsilon [0.02] - if > 0 used to tune tolerance for
    * smooth surfaces so that these are considered as one single highlighted surface, i.e., only
    * their silhouette will appear.
    * @param {function} __clientCallback - default callback function pointer to pass on
    *            various error states back to client. eg., when loading of
    *            scene is finished. callback function's argument 0 indicates
    *            success with no errors. Otherwise indicates the error state
    *            TODO Define arguments for error states (eg., 1 for error
    *            during http request). This callback is a stateless callback
    *            (not made through any object instance).
    *            The client callback gives also:
    *            arg[1] => the current file geometry percentage
    *            arg[2] => the current file texture percentage
    *            arg[3] => the current number of queried files
    *            arg[4] => the current number of successfully retrieved files
    *            E.G. the client just needs to test if (arg[1]=100 & arg[2]=100 & arg[3] = arg[4]) to
    *            know if "everything" (the entire scene) has been loaded (and
    *            displayed at least once).
    * @param {function} __defaultClientUIAddElementCallback - default callback funtion
    *            giving the current node path (arg[1]) and a boolean indicating
    *            if it is a leaf (arg[2])
    * @param {Number} __boundaryhardedges [true] - if true, builds the hard edges highlights at surface boundaries
    */
    WebViewer3DAPI.prototype.LoadSceneFromObject = function(__object, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges)
    {
        LogMessage.Log("API Load Scene from object");

        this._controller.LoadSceneFromJSONObject(__object, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges);
    }

    /**
    * Reset visiblity of the main scene to its default
    * default visiblity of the main scene is true at startup
    *
    * @function ResetMainSceneVisibility
    */
    WebViewer3DAPI.prototype.ResetMainSceneVisibility = function()
    {
        LogMessage.Log("API Reset visiblity of main scene");

        this._controller.ResetMainSceneVisibility();
    }

    /**
    * Toggle visiblity of a node using its URL in the scene
    * default visiblity of a node is true at startup
    *
    * @function ToggleNodeVisibility
    * @param {String} __nodeURL - URL identifying a scene node
    * @param {Boolean} __recurseChildren - If true, traverse all children to toggle their visibility state
    * @param {Boolean} __recurseParents - If true, traverse all parents to toggle their visibility state
    */
    WebViewer3DAPI.prototype.ToggleNodeVisibility = function(__nodeURL, __recurseChildren, __recurseParents)
    {
        this._controller.ToggleNodeVisibility(__nodeURL, __recurseChildren, __recurseParents);
    }

    /**
    * Set visiblity state of a node using its URL in the scene
    * default visiblity of a node is true at startup
    *
    * @function SetNodeVisibility
    * @param {String} __nodeURL - URL identifying a scene node
    * @param {Boolean} __visible - Set the object visibility to __visible
    * @param {Boolean} __recurseChildren - If true, traverse all children to set their visibility to __visible
    * @param {Boolean} __recurseParents - If true, traverse all parents to set their visibility to __visible
    */
    WebViewer3DAPI.prototype.SetNodeVisibility = function(__nodeURL, __visible, __recurseChildren, __recurseParents)
    {
        this._controller.SetNodeVisibility(__nodeURL, __visible, __recurseChildren, __recurseParents);
    }

    /**
     * Set the angular position of the camera relative to the default front position of the scene
     *
     * @function SetCameraAngles
     * @param {Number} __azimuthAngle - the walk-around angle measured counterclockwise from the default front view
     * @param {Number} __elevationAngle - the elevantion angle - positive values lift the camera above the scene centre
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
    */
    WebViewer3DAPI.prototype.SetCameraAngles = function(__azimuthAngle, __elevationAngle, __redraw)
    {
        this._controller.SetCameraAngles(__azimuthAngle, __elevationAngle, __redraw);
    }

    /**
    * Get an object containing the angular position of the camera relative to the default front position of the scene
    *
    * @function GetCameraAngles
    * @returns {{azimuth: Number, elevation:Number} azimuth - the walk-around angle measured counterclockwise from the default
    * front view. elevation - the elevantion angle - positive values lift the camera above the scene centre
    *
    */
    WebViewer3DAPI.prototype.GetCameraAngles = function()
    {
        return this._controller.GetCameraAngles();
    }

    /**
     * Convenience function to reset the default camera to its default positions. This function call is same as calling
     * SetCameraAngles with 0.0 for both angles and SetCameraScale with 1.0.
     *
     * @function ResetCamera
     */
    WebViewer3DAPI.prototype.ResetCamera = function()
    {
        this._controller.ResetCamera();
    }

    /**
     * Control interactive functionality. Pass 'false' to not have any interactive control on the 3D view.
     * pass 'true' again to turn on interactivity.
     *
     * @function ToggleInteractiveCameraControl
     * @param __onOrOff
     */
    WebViewer3DAPI.prototype.ToggleInteractiveCameraControl = function(__onOrOff)
    {
        this._controller.SetInteractiveCameraControl(__onOrOff);
    }

    /**
     * Control interactive functionality. "WV3DConstants.interaction.NONE", "WV3DConstants.interaction.PAN", "WV3DConstants.interaction.SCALE",
     * "WV3DConstants.interaction.ROTATE", "WV3DConstants.interaction.ENVROT", "WV3DConstants.interaction.ALL" or an OR combination of these
     *
     * @function SetInteractiveCameraControlMask
     * @param {Number} __mask - an or combination of the following flags "WV3DConstants.interaction.NONE", "WV3DConstants.interaction.PAN",
     * "WV3DConstants.interaction.SCALE", "WV3DConstants.interaction.ROTATE", "WV3DConstants.interaction.ENVROT", "WV3DConstants.interaction.ALL".
     */
    WebViewer3DAPI.prototype.SetInteractiveCameraControlMask = function(__mask)
    {
        this._controller.SetInteractiveCameraControlMask(__mask);
    }

    /**
     * Set the size of the scene within the camera view. This is achieved by automatically
     * adjusting the camera properties (e.g. changing the camera distance for a perspective camera)
     *
     * @function SetCameraScale
     * @param {Number} __scale - the size of the scene bounding box in the view. A value of 1 will result in the scene filling the view in at least one dimension
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetCameraScale = function(__scale, __redraw)
    {
        this._controller.SetCameraScale(__scale, __redraw);
    }


    /**
     * Get the size of the scene within the camera view.
     *
     * @function GetCameraScale
     * @returns {Number} the size of the scene bounding box in the view. A value of 1 will result in the scene filling the view in at least one dimension
     */
    WebViewer3DAPI.prototype.GetCameraScale = function()
    {
        return this._controller.GetCameraScale();
    }

     /**
     * Set the panning offset of the camera.
     *
     * @function SetCameraPanOffset
     * @param {Number} __x_offset - horizontal offset
     * @param {Number} __x_offset - vertical offset
     */
    WebViewer3DAPI.prototype.SetCameraPanOffset = function(__x_offset, __y_offset, __redraw)
    {
        this._controller.SetCameraPanOffset(__x_offset, __y_offset, __redraw);
    }

    /**
     * Get the current panning offset of the current camera. Returns an object with properties "x" and "y" indicating the
     * offset in each of these dimensions.
     *
     * @function GetCameraPanOffset
     */
    WebViewer3DAPI.prototype.GetCameraPanOffset = function()
    {
        return this._controller.GetCameraPanOffset();
    }

    /**
     * Set the environment rotation.
     *
     * @function SetEnvironmentRotation
     * @param {Number} __quaternion - rotation as a quaternion
     */
    WebViewer3DAPI.prototype.SetEnvironmentRotation = function(__quaternion, __redraw)
    {
        this._controller.SetEnvironmentRotation(__quaternion, __redraw);
    }

    /**
     * Get the current environment rotation. Returns the rotation as a quaternion.
     *
     * @function GetEnvironmentRotation
     */
    WebViewer3DAPI.prototype.GetEnvironmentRotation = function()
    {
        return this._controller.GetEnvironmentRotation();
    }

    /**
     * Enables a specific rendering mode (for interaction) in which the best suitable scale of the scene
     * is applied to the current view. This neglects previously set scale factor.
     *
     * @function SetOptimisedViewRendering
     * @param {"x":number,"y":number} __border set the minimum gap between the projected bbox of the scene (+labels) and the borders of the image plane
     * @param {boolean} __enableOptimisation indicate the width of the viewport. default is the current canvas width.
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetOptimisedViewRendering = function(__border, __enableOptimisation, __redraw)
    {
        this._controller.SetOptimisedViewRendering(__border, __enableOptimisation, __redraw);
    }

    /**
     * Set up a perspective camera. The camera distance from the scene centre will be set automaticall based on the current scale setting.
     *
     * @function SetPerspectiveCamera
     * @param {Number} [__fieldOfView=45] - the field of view in degrees
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetPerspectiveCamera = function(__fieldOfView, __redraw)
    {
        this._controller.SetPerspectiveCamera(__fieldOfView, __redraw);
    }

    /**
    * Set up an orthographic camera. The viewport size will be set automaticall based on the current scale setting.
    *
    * @function SetOrthographicCamera
    * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
    */
    WebViewer3DAPI.prototype.SetOrthographicCamera = function(__redraw)
    {
        this._controller.SetOrthographicCamera(__redraw);
    }

    /**
    * Focus camera from a node's bbox using its URL in the scene
    * default focus of camera is on the root node of the scene
    *
    * @function SetCameraFocus
    * @param {String} __nodeURL - URL identifying the scene node
    * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
    */
    WebViewer3DAPI.prototype.SetCameraFocus = function(__nodeURL, __redraw)
    {
        this._controller.SetCameraFocus(__nodeURL, __redraw);
    }

    /**
     * SetHighlight enabled or disabled
     *
     * @function SetHighlights
     * @param {Boolean} [__enableHighlights=true] - enables the objects highlighting if true, disable it if false
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetHighlights = function(__enableHighlights, __redraw)
    {
        this._controller.SetHighlights(__enableHighlights, __redraw);
    }

    /**
     * SetHighlightsConfiguration enables and disables each highlight component
     * Call function SetHighlights first to enable highlights
     * By default, __enableHorizonHighlights and __enableHardEdgeHighlights are set to true
     * @function
     * @param {boolean} __enableHorizonHighlights - enables the horizon highlighting if true and highlighting is enabled too, disable it otherwise
     * @param {boolean} __enableHardEdgeHighlights - enables the hard edge highlighting if true and highlighting is enabled too, disable it otherwise
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetHighlightsConfiguration = function(__enableHorizonHighlights, __enableHardEdgeHighlights, __redraw)
    {
        this._controller.SetHighlightsConfiguration(__enableHorizonHighlights, __enableHardEdgeHighlights, __redraw)
    }

    /**
     * SetHighlightColor
     *
     * @function SetHighlightsColor
     * @param {Number} __red - red component (0 - 255)
     * @param {Number} __green - green component (0 - 255)
     * @param {Number} __blue - blue component (0 - 255)
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetHighlightsColor = function(__red, __green, __blue, __redraw)
    {
        this._controller.SetHighlightsColor(parseFloat(__red), parseFloat(__green), parseFloat(__blue), __redraw);
    }

    /**
     * Set the background color for the current scene
     * Setting an alpha lower than 255 might result in some visual artifacts in reflections since transparent objects on top of transparent background
     * is still experimental.
     * This function must be called when __clientCallback of API returns a fully loaded scene.
     *
     * @function SetBackgroundColor
     * @param {Number} __red - red component (0 - 255)
     * @param {Number} __green - green component (0 - 255)
     * @param {Number} __blue - blue component (0 - 255)
     * @param {Number} __alpha - __alpha component (0 - 255) (represent the opacity of the background, useful for snapshot)
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetBackgroundColor = function(__red, __green, __blue, __alpha, __redraw)
    {
        this._controller.SetBackgroundColor(parseFloat(__red), parseFloat(__green), parseFloat(__blue), parseFloat(__alpha), __redraw);
    }

    /**
     * Show or Hide all labels in the scene. Per label visibility settings are not altered with this function.
     *
     * @function SetLabelsVisibility
     * @param __show {Boolean} [__show=true] - true to show all labels in the scene and false otherwise
     * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
     */
    WebViewer3DAPI.prototype.SetLabelsVisibility = function(__show, __redraw)
    {
        this._controller.SetLabelsVisibility(__show, __redraw);
    }

    /**
     * Updates properties of Label assets for a particular node that are specified in JSON files.
     * The URL identifies the label asset for a particular node in the scene.
     *
     * @function UpdateLabelProperties
     * @param __nodeURL {String} - URL identifying the label in the scene.
     *                          eg., http://<yourhost>/path/to/jsonfile.json/InstanceGroup/MyLabel1
     * @param __properties {JSON Object} - Label properties as JSON object. Should include one or more of
     *                          'text', 'display_style', 'orientation', 'preferred_loc_v', 'preferred_loc_h', 'pixel_offset' and 'visible'
     *                          'display_style' should specify full styling and not a reference to scene's label display styles.
     *                          Valid values for 'visible' are true and false. A particular label's visibility can be controlled with this property.
     *                          A label that is already turned off particular node using "label_name":null mechanism in labelsettingsmap cannot be updated using this API
     *                          eg., JSON.parse({ "text": "new label text", "display_style": "{ "fontsize": 36, "fontface": "Arial", "borderColor": {"r":0, "g":0, "b":0, "a":1.0}, "backgroundColor": {"r":255, "g":255, "b":255, "a":1.0} },
     *                          "orientation": "horizontal", "preferred_loc_h": "right", "preferred_loc_v": "top" }")
     * @return false - If the __nodeURL is invalid OR if the label with __nodeURL is not found in the scene.
     *         true - In case of successful execution. (no other error case is hit)
     * @note Throws exceptions in the case of an undefined __nodeURL or __properties or if the label with __nodeURL is not found in the scenegraph
     */
     WebViewer3DAPI.prototype.UpdateLabelProperties = function(__nodeURL, __properties)
     {
         return this._controller.UpdateLabelProperties(__nodeURL, __properties);
     }

     /**
     * Updates properties of all label assets in the scene. A Convenience function that iterates
     * over all labels in the scene updating their display properties. Effectively same as calling the
     * UpdateLabelProperties() function for each label node in the scene. Typically used to update display_style
     * settings of labels in one go (eg., update fontsize to increase text size in all labels!)
     *
     * @function UpdateAllLabelsProperties
     * @param __properties {JSON Object} - Label properties as JSON object. Should include one or more of
     *                          'text', 'display_style', 'orientation', 'preferred_loc_v', 'preferred_loc_h', 'pixel_offset' and 'visible'
     *                          'display_style' should specify full styling and not a reference to scene's label display styles.
     *                          Valid values for 'visible' are true and false. A particular label's visibility can be controlled with this property.
     *                          A label that is already turned off particular node using "label_name":null mechanism in labelsettingsmap cannot be updated using this API
     *                          eg., JSON.parse({ "display_style": "{ "fontsize": 36 }" })
     * @return false if the __properties is not valid JSON (atleast one allowed parameters) or if there are no labels present in the scene. Otherwise returns true
     * @note Throws exception in the case of an undefined __properties object
     */
     WebViewer3DAPI.prototype.UpdateAllLabelsProperties = function(__properties)
     {
         return this._controller.UpdateAllLabelsProperties(__properties);
     }

    /**
     * SetReflections sets the reflection on, off, or relative to what is provided in the Json file.
     *
     * @function SetReflections
     * @param {string} __config - a string, "ON", "OFF", or "JSON" ("JSON" is already set by default)
     * @param {number} [__opacity = JSON-Value ] - [0 1] opacity of reflections (only if "ON" mode chosen)
     * @param {boolean} [__fade = JSON-Value ] - turns ON or OFF fading of reflections (only if "ON" mode chosen)
     * @param {number} [__position = JSON-Value ] - [0 1] position of reflections (only if "ON" mode chosen)
     * @param {number} [__fadingSize = JSON-Value ] - [1 2] fading size of reflections (only if "ON" mode chosen)
     * @param {boolean} [__redraw = true] - indicates if the scene has to be redrawn after this call. Default is true.
     */
    WebViewer3DAPI.prototype.SetReflections = function(__config, __opacity, __fade, __position, __fadingSize, __redraw)
    {
        LogMessage.Log("API SetReflections: "+__config);
        this._controller.SetReflections(__config, __opacity, __fade, __position, __fadingSize, __redraw);
    }

    /**
     * SetAnimations enabled or disabled
     *
     * @function SetAnimations
     * @param {boolean} [enableAnimations=true] - enables the camera animation if true, disable it if false
     */
    WebViewer3DAPI.prototype.SetAnimations = function(__enableAnimations)
    {
        LogMessage.Log("API enable/disable animations: "+__enableAnimations);

        this._controller.SetAnimations(__enableAnimations);
    }

    /**
     * SetInteractionOptimization enabled or disabled
     *
     * @function SetInteractionOptimization
     * @param {boolean} [__enableInteractionOptimization=true] - enables the scene optimization while interacting with the model if true, disable it if false
     */
    WebViewer3DAPI.prototype.SetInteractionOptimization = function(__enableInteractionOptimization)
    {
        LogMessage.Log("API enable/disable interaction optimization: "+__enableInteractionOptimization);

        this._controller.SetInteractionOptimization(__enableInteractionOptimization);
    }

    /**
     * SetQualityLevel
     *
     * @function SetQualityLevel sets maximum quality when no interaction with the model or when interaction optimization is disabled
     * by setting the number of transparency peels for transparent models. This quality level does not affect quality of opaque models.
     * @param {number} - an integer between 1 and 10 (included)
     * @param {boolean} [__redraw = true] - indicates if the scene has to be redrawn after this call. Default is true.
     */
    WebViewer3DAPI.prototype.SetQualityLevel = function(__qualityLevel, __redraw)
    {
        LogMessage.Log("API SetQualityLevel: "+__qualityLevel);

        this._controller.SetQualityLevel(__qualityLevel, __redraw);
    }

    /**
     * SetLightingContrast sets the lighting contrast (in designer quality only)
     * by reducing the directional light and increasing the ambient light components.
     *
     * @function SetLightingContrast
     * @param {number} - a floating number between 0 and 1 (included). default value is 0.5.
     * @param {boolean} [__redraw = true] - indicates if the scene has to be redrawn after this call. Default is true.
     */
    WebViewer3DAPI.prototype.SetLightingContrast = function(__lightingContrast, __redraw)
    {
        LogMessage.Log("API SetLightingContrast: "+__lightingContrast);

        this._controller.SetLightingContrast(__lightingContrast, __redraw);
    }

    /**
     * SetDefaultResourceRelativePath (optional function when loading scene from samples, useful to specify Resource folder location
     * not set to './Resources')
     *
     * @function SetDefaultResourceRelativePath
     * @param {String} ['./Resources'] - overwrite the default resource relative path
     */
    WebViewer3DAPI.prototype.SetDefaultResourceRelativePath = function(__path)
    {
        this._controller.SetDefaultResourceRelativePath(__path);
    }

    /**
    * Get an indication of the degree to which the current system is supported for 3D viewing
    *
    * @function GetSystemSupportLevel
    * @returns {{ "issues": Number, "support_level": supportLevel.FULL | supportLevel.PARTIAL | supportLevel.NONE}} An object containing "issues"
    * (Array of issues) and a "support_level" (supportLevel.FULL | supportLevel.PARTIAL | supportLevel.NONE) property
    */
    WebViewer3DAPI.prototype.GetSystemSupportLevel = function()
    {
        return this._controller.GetSystemSupportLevel();
    }

    /**
    * Get a technical report containing system info
    *
    * @function GetSystemReport
    * @returns {String} a report containing system info
    */
    WebViewer3DAPI.prototype.GetSystemReport = function()
    {
        return this._controller.GetSystemReport();
    }

    /**
    * Get a snapshot of the current viewer state as specified in format
    *
    * @function GetSnapshot
    * @param {string} [format = "png"] - image format (accepted: "png, jpeg")
    * @param {number} [width = context canvas width or 1] - image width in pixels (accepted: [1:2048] higher depend on graphics card and browser)
    * @param {number} [height = context canvas height or 1] - image height in pixels (accepted: [1:2048] higher depend on graphics card and browser)
    * @returns {Object} image data
    */
    WebViewer3DAPI.prototype.GetSnapshot = function(__format, __width, __height)
    {
        return this._controller.GetSnapshot(__format, __width, __height);
    }

    return WebViewer3DAPI;

});
