/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

define([
    "underscore",
    "backbone",
    "WV3Dmod/AssetLoader",
    "WV3Dmod/LogMessage"
    ], function (
        _,
        bb,
        AssetLoader,
        LogMessage
        ) {

    'use strict';

    /**
     * @alias module:WebViewer3D
     * @class WV3DAssetManager
     */
    var WV3DAssetManager = bb.Model.extend({


    initialize: function(__httprequestmanager)
    {
        this.set('_assetloader',new AssetLoader(__httprequestmanager));

        this.Clear();
    },

    /**
     * Clear the cache and reinit
     */
    Clear: function(resource)
    {
        this.get('_assetloader').ClearURLCache();

        this.set('_cache', {
                    "rootnodes":{}//in use: contains the loaded scenes using their absolute URL as a key
                });
        this.set('_pendingqueriedassets', []);

        this.set('_resolvedassetnumber', 0);
        this.set('_textureCounter', 0);
        this.set('_textureLoadedCounter', 0);
    },

    /**
     * Returns the set in which the resource shall be pushed (or not if already cached)
     */
    Getresourcelocation: function(resource)
    {

        // LogMessage.Log("AssetManager Getresourcelocation function");
        var cache = this.get('_cache');

        var resourcetypename = this.get('_assetloader').Getresourcelocation(resource);
        switch(resourcetypename)
        {
            //This is the location were we put our fetched scenes (one for each file actually)
            case "Reference": return cache.rootnodes;
            //This location is mainly for debug purposes
            case "NoLocation":
            default:
                if(this.get('_cache')[resourcetypename] === undefined)
                {
                    this.get('_cache')[resourcetypename] = [];
                }
                return this.get('_cache')[resourcetypename];
        }
    },

    /**
     * Request the key type of unique id we should be looking for when trying to cache a resource
     */
    RequestType: function(resource)
    {
        if (resource.hasOwnProperty('AssetType') && resource['AssetType']=="Scene")
        {
            return 'AssetId';
        }else
        {
            // LogMessage.Log("No unique name to reference this resource. Cannot say if in pending queue list.");
            return null;
        }
    },

    /**
     * Return the file name + extension from the URL
     *
     * @param {String} __string URL to the file
     */
    GetFileName: function(__string)
    {
        return this.get('_assetloader').GetFileName(__string);
    },

    /**
     * Return the asset name + extension from the URL
     *
     * @param {String} __string URL to the asset
     */
    GetAssetName: function(__string)
    {
        return this.get('_assetloader').GetAssetName(__string);
    },

    /**
     * Update a result query counter
     */
    IncrementResolvedQueryCounter: function()
    {
        this.set('_resolvedassetnumber', this.get('_resolvedassetnumber')+1);
    },

    /**
     * Load a url and return to the caller when data is ready
     */
    LoadURL:function (__sceneURL, __parentSceneURL)
    {
        this.get('_assetloader').LoadURL(__sceneURL, __parentSceneURL);
    },

    /**
     * Load a resource file in JSON format and put its elements in the cache
     */
    LoadAsset: function(__resource, __sceneURL)
    {

        LogMessage.Log("Asset Manager LoadAsset function");

        this.get('_assetloader').SetResource(__resource);

        this.Printresourceinfo();
        var scene_name = this.Loadroot(__sceneURL);
        this.Loadpendingqueriedassets(__sceneURL);

        this.IncrementResolvedQueryCounter();
        return scene_name;
    },

    /**
     * Load a resource file in JSON format and put its elements in the cache
     */
    Loadroot: function(__sceneURL)
    {
        LogMessage.Log("Asset Manager LoadAsset function");

        var root = this.get('_assetloader').Getrootnode();
        //Replace root name if it is an url
        if (__sceneURL) root.AssetId = __sceneURL;
        if (!root.AssetId) root.AssetId = "Default_scene_name";
        if (!root.ServerBuildNumber) LogMessage.Log("Asset generated with ESKO HTML Viewer - Server Component version anterior to 691651");
        else LogMessage.Log("Asset generated with ESKO HTML Viewer - Server Component V"+root.ServerBuildNumber);

        var location = this.Getresourcelocation(root);
        if(!this.Iscached(root, location))
        {
           this.get('_cache').rootnodes[root.AssetId] = root;
           this.Pushinpendinglist(root, location);
        }else
        {
            LogMessage.Log("Root already chached");
        }

        return root.AssetId;
    },

    /**
     * Load queried assets
     */
    Loadpendingqueriedassets: function(__sceneURL)
    {
        LogMessage.Log("Asset Manager Recurse function");

        while(this.get('_pendingqueriedassets').length > 0)
        {
            //Get first element
            var elttofetch = this.get('_pendingqueriedassets')[0];
            //Remove it from pending list
            this.get('_pendingqueriedassets').splice(0,1);
            //Add it to the cache
            this.Pushincache(elttofetch.res, elttofetch.loc)
            //Add its children to the pending list
            var list = this.get('_assetloader').GetRefChildrenFromName(elttofetch.res, __sceneURL);
            this.Pushlistinpendingqueue(list,__sceneURL);
        }
        LogMessage.Log("Size of pending list is: "+this.get('_pendingqueriedassets').length);
    },

    /**
     * Put list resources elements in the cache if not already in it
     */
    Pushlistinpendingqueue: function(list,__sceneURL)
    {
        for (var i = list.length - 1; i >= 0; i--)
        {
            var loc = this.Getresourcelocation(list[i]);
            if(!this.Isinpendinglist(list[i]) && !this.Iscached(list[i], loc))
            {
                this.Pushinpendinglist(list[i],loc);
            }
            //Query referenced urls and push them as nodes
            if (this.get('_assetloader').IsURL(list[i].AssetRef))
            {
                //Check if the current AssetRef is a URL and add it to the cache if needed
                LogMessage.Log("Loading external asset from URL");
                this.LoadURL(list[i].AssetRef, __sceneURL);
            }
        };
    },


    /**
     * Put a resource in the cache
     */
    Pushincache: function(resource, location)
    {
        var request_type = this.RequestType(resource);
        if(request_type === null)
        {
            // LogMessage.Log("No request type: object not cached");
            return false;
        }
        // Simple children are not added to the cache
        // But still need to be processed via the pending
        // query queue

        LogMessage.Log("AssetManager Pushincache function : Object \""+resource[request_type]+"\"");
        location[resource[request_type]]=resource;
    },

    /**
     * Returns true if an asset is already loaded in the cache plausible location
     */
    Iscached: function(resource, plausiblelocation)
    {

        // LogMessage.Log("AssetManager Iscached function");
        var request_type = this.RequestType(resource);
        if(request_type === null) return false;

        for (var i in plausiblelocation)
        {
            if(plausiblelocation.hasOwnProperty(i))
            {
                if (plausiblelocation[i][request_type] === resource[request_type])
                {
                    // LogMessage.Log("Object \""+i+"\" already cached !");
                    return true;
                }
            }
        }
    },

    /**
     * Put a resource in the cache
     */
    Pushinpendinglist: function(resource, location)
    {
        // LogMessage.Log("AssetManager PushinpendingList");
        this.get('_pendingqueriedassets').push({"res":resource,"loc":location});
    },

    /**
     * Put a resource in the cache
     */
    Isinpendinglist: function(resource)
    {
        // LogMessage.Log("AssetManager Isinpendinglist function");
        var request_type = this.RequestType(resource);
        if(request_type === null) return false;

        for (var i in this.get('_pendingqueriedassets'))
        {
            if(this.get('_pendingqueriedassets').hasOwnProperty(i))
            {
                if ( this.get('_pendingqueriedassets')[i].res[request_type] === resource[request_type])
                {
                    // LogMessage.Log("Object \""+i+"\" already in pending list !");
                    return true;
                }
            }
        }
        return false;
    },

    // /**
    //  * Return cached asset if present, else will load it into cache
    //  */
    // GetAssetList: function(assettype)
    // {
    //     LogMessage.Log("Asset Manager Get Asset function");
    //     var cache = this.get('_cache')[assettype];
    // },


    /**
     * Load a resource file in JSON format and put its elements in the cache
     */
    Printresourceinfo: function()
    {
        this.get('_assetloader').Printresourceinfo();
    },

    /**
     * Apply somefunction to the object
     */
    AssetManagerTestFunction: function()
    {
        LogMessage.Log("Asset Manager Test function");
    }

    });



    return WV3DAssetManager;

});