/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2014 Esko Stonecube Ltd. All rights reserved.
 */

define([
    "underscore",
    "backbone",
    "WV3Dlib/jquery.mousewheel",
    "WV3Dmod/hammerEXT",
    "WV3Dlib/three",
    "WV3Dmod/ViewerView",
    "WV3Dmod/ViewerModel",
    "WV3Dmod/SystemChecker",
    "WV3Dmod/HttpRequestManager",
    "WV3Dmod/Constants",
    "WV3Dmod/Math",
    "WV3Dmod/LogMessage"
    ], function (
        _,
        bb,
        JQMouseWheel,
        Hammer,
        THREE,
        WV3DViewerView,
        WV3DViewerModel,
        WV3DSystemChecker,
        WV3DHttpRequestManager,
        WV3DConstants,
        WV3DMath,
        LogMessage
        ) {

    'use strict';

    /**
     * @alias module:WebViewer3D
     * @class WV3DViewerController
     */
    var WV3DViewerController = bb.Model.extend({

        initialize: function(__context, __clientLoadedSceneCallback, __clientUIAddElementCallback)
        {
            LogMessage.Log("Creating WV3DViewerController Object");

            //Create an http request handler and give it to the model we created
            this.set('_httprequestmanager', new WV3DHttpRequestManager());

            //Create proxy canvas if no context have been set
            if(!__context)
            {
                __context = {clientWidth:1, clientHeight:1};
            }

            //Create one model + one view
            this.set('_model', new WV3DViewerModel(this.get('_httprequestmanager'), __clientLoadedSceneCallback));
            this.set('_view', new WV3DViewerView(__context, this.get('_model'), __clientUIAddElementCallback));

            //Maintain camera interactive state information
            this.set('_cameratransformstate', { pscale: 1.0, deltax: 0.0, deltay: 0.0 });
            this.set('_interactivecontrol', true);
            this.set('_interactivecontrolmask', WV3DConstants.interaction.ALL);

            //Attach 3D view to Hammer.js events
            if(typeof (__context.appendChild) === 'function')
            {
                //disable right-click default to avoid contextual menu opening
                __context["oncontextmenu"] = function(){return false;};
                //prevent 3d canvas from being selectable (causing cursor change and neighboring text selection when dragging)
                var disableselect = function(e) {return false};
                __context["onselectstart"] = disableselect;
                __context["onmousedown"] = disableselect;

                this.set('_hammer', new Hammer(__context));
                this.get('_hammer').get('pan').set( { threshold: 0, direction: Hammer.DIRECTION_ALL } );
                this.get('_hammer').get('tap').set( { taps: 2 } );
                this.get('_hammer').get('pinch').set( { enable: true } );
                this.get('_hammer').add( new Hammer.Pinch({ event: 'threepan', pointers: 3 }) );
                this.get('_hammer').on("panstart panmove panend", _.bind( this.InteractiveCameraMove, this ));
                this.get('_hammer').on("pinchstart pinchmove pinchend", _.bind( this.InteractiveCameraScale_touch, this));
                this.get('_hammer').on("tap", _.bind( this.InteractiveResetCamera, this ));
                this.get('_hammer').on("press", _.bind( this.InteractiveResetCamera, this ));
                this.get('_hammer').on("threepan", _.bind( this.InteractiveEnvironmentRotation_touch, this));

                //use jquery-mousewheel to handle mousewheel events for our 3d view.
                $(__context).on('mousewheel', _.bind( this.InteractiveCameraScale_mouse, this));
            }
        },

        /**
        * Clear the scene and release all GPU resources. Has to be called before API object
        * goes out of scope to avoid memory leaks.
        *
        * @function Dispose
        */
        Dispose: function()
        {
            this.get("_view").ReleaseDummyTextures();
            this.get("_view").ReleaseAllRenderTargets();
            this.get("_view").ReleaseScene();
        },

        /**
         * Set end camera interaction callback
         *
         * @function SetEndCameraInteractionCallback
         * @param {function} __clientEndCameraInteractionCallback - default callback funtion
         *            at the end of camera interaction
         */
        SetEndCameraInteractionCallback: function(__clientEndCameraInteractionCallback)
        {
            if((typeof __clientEndCameraInteractionCallback) === "function")
            {
                this.set("_clientEndCameraInteractionCallback",__clientEndCameraInteractionCallback);
            }
        },

        /**
         * Trigger a resize event that allow the WebGLViewer to adapt to the current context size
         * This function is useful only when you do not want to use window resize event trigger
         * User should use window.trigger('resize’) otherwise.
         * This Resize function is useful especially if the webgl canvas is embedded in an independant
         * and resizable dom element and avoid unnecessary refresh for the whole window. (i.e. a dockable
         * widget)
         *
         * @function
         */
        Resize: function(__path)
        {
            this.get('_view').Resize();
        },

        SetDefaultResourceRelativePath: function(__path)
        {
            this.get('_view').SetDefaultResourceRelativePath(__path);
        },

        /**
         * Reset scene and load new one from a URL resource
         *
         * @param {String} sceneURL URL to the scene data
         */
        LoadSceneFromURL: function (__sceneURL, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges)
        {
            var self = this;
            if(__clientLoadedSceneCallback)
                this.get("_model").set("_clientCallback",__clientLoadedSceneCallback);
            if(__clientUIAddElementCallback)
                this.get('_view').set("_clientUIAddElementCallback",__clientUIAddElementCallback);
            this.get('_model').ResetAssetManager();
            this.get('_view').ResetScene(__coplanarityEpsilon, __boundaryhardedges);
            this.AddSceneFromURL(__sceneURL);
        },

        /**
         * Load a scene from a URL resource
         *
         * @param {String} sceneURL URL to the scene data
         */
        AddSceneFromURL: function(__sceneURL)
        {
            this.get('_model').LoadSceneURL(__sceneURL);
        },

        /**
         * Reset the scene and load a new one from a JSON object
         *
         * @param {String} resource JSON format resource string
         */
        LoadSceneFromJSONObject: function(__resource, __coplanarityEpsilon, __clientLoadedSceneCallback, __clientUIAddElementCallback, __boundaryhardedges)
        {
            LogMessage.Log("Controller LoadResource function");

            if(__clientLoadedSceneCallback)
                this.get("_model").set("_clientCallback",__clientLoadedSceneCallback);
            if(__clientUIAddElementCallback)
                this.get('_view').set("_clientUIAddElementCallback",__clientUIAddElementCallback);
            this.get('_model').ResetAssetManager();
            this.get('_view').ResetScene(__coplanarityEpsilon, __boundaryhardedges);
            this.AddSceneFromJSONObject(__resource);
        },

        /**
         * Load a scene from a JSON string
         *
         * @param {String} resource JSON format resource string
         */
        AddSceneFromJSONObject: function(__resource)
        {
            LogMessage.Log("Controller LoadResource function");

            //As the current query is not from a URL we preincrement it now
            //The resolve query counter will be incremented in asset manager
            this.get('_model').get('_assetmanager').get('_assetloader').get('_httprequestmanager').IncrementQueryCounter();
            this.get('_model').LoadSceneJSON(__resource);
        },

        /**
         * Reload all textures
         *
         * @function ReLoad all textures (only material textures) that have been loaded by the json file.
         * @param {function} __callback - callback funtion used when rendering has been updated
         * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
         */
        ReloadAllTextures: function(__callback, __redraw)
        {
            LogMessage.Log("Controller ReLoadAllTextures function");
            this.get('_view').ReloadAllTextures(__callback, __redraw);
        },

        /**
         * Reload a texture
         *
         * @function ReloadTexture
         * @param {String} __textureURL - URL identifying the initial texture relative path from json to texture
         * @param {String} __newURL - URL identifying the new texture relative path from json to texture
         * @param {function} __callback - callback funtion used when rendering has been updated
         * @param {Boolean} [__keep_old_path = true] - indicates if the old VS new path has to be used now on to reference the texture
         * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
         */
        ReloadTexture: function(__textureURL, __newURL, __callback, __keep_old_path, __redraw)
        {
            LogMessage.Log("Controller ReLoadTexture function");
            this.get('_view').ReloadTexture(__textureURL, __newURL, __callback, __keep_old_path, __redraw);
        },

        /**
        * Reset visiblity of the main scene to its default (in json structure??? to define)
        * default visiblity of the main scene is true at startup
        *
        * @function
        */
        ResetMainSceneVisibility: function()
        {
            LogMessage.Log("Controller Reset visiblity of main scene");
            this.get('_view').ResetMainSceneVisibility();
        },


       /**
        * Toggle visiblity of a node using its URL in the scene
        * default visiblity of a node is true at startup
        *
        * @function
        * @param {String} __nodeURL - URL identifying a scene node
        * @param {Boolean} __recurseChildren - If true, traverse all children to toggle their visibility state
        * @param {Boolean} __recurseParents - If true, traverse all parents to toggle their visibility state
        */
        ToggleNodeVisibility: function(__nodeURL, __recurseChildren, __recurseParents)
        {
            this.get('_view').ToggleNodeVisibility(__nodeURL, __recurseChildren, __recurseParents);
        },


        /**
        * Set visiblity state of a node using its URL in the scene
        * default visiblity of a node is true at startup
        *
        * @function
        * @param {String} __nodeURL - URL identifying a scene node
        * @param {Boolean} __visible - Set the object visibility to __visible
        * @param {Boolean} __recurseChildren - If true, traverse all children to set their visibility to __visible
        * @param {Boolean} __recurseParents - If true, traverse all parents to set their visibility to __visible
        */
        SetNodeVisibility: function(__nodeURL, __visible, __recurseChildren, __recurseParents)
        {
            if(typeof __visible != "boolean")
            {
                throw ('Set visiblity: NaB Exception');
            }
            this.get('_view').SetNodeVisibility(__nodeURL, __visible, __recurseChildren, __recurseParents);
        },

        /*Panning using input event offset*/
        InteractiveCameraPanOffset: function(ev)
        {
            var curCameraPanOffset = this.get('_view').GetCameraPanOffset();
            var xcuroffset = curCameraPanOffset.x;
            var ycuroffset = curCameraPanOffset.y;
            var curCameraScale = this.get('_view').GetCameraScale();
            //Here we make sure that interactive panning will be function of the actual
            //scene size, so we introduce the scene bounding sphere radius in the calculation.
            //1/200 is a factor that can be tuned to change speed of panning.
            var speedFactor = 1/200;
            var resDeltaX = Math.abs(ev.deltaX - this.get('_cameratransformstate').deltax)/curCameraScale*this.get('_view').get('_3JSsceneBBox').getBoundingSphere().radius*speedFactor;
            var resDeltaY = Math.abs(ev.deltaY - this.get('_cameratransformstate').deltay)/curCameraScale*this.get('_view').get('_3JSsceneBBox').getBoundingSphere().radius*speedFactor;

            if(ev.deltaX > this.get('_cameratransformstate').deltax)
                xcuroffset -= resDeltaX;
            else
                xcuroffset += resDeltaX;

            if(ev.deltaY < this.get('_cameratransformstate').deltay)
                ycuroffset -= resDeltaY;
            else
                ycuroffset += resDeltaY;

            this.get('_cameratransformstate').deltax = ev.deltaX;
            this.get('_cameratransformstate').deltay = ev.deltaY;

            this.get('_view').SetCameraPanOffset({"x":xcuroffset,"y":ycuroffset});
        },

        /**
         * Hammer.js pan event handler for interactive camera rotation or panning (standing mode).
         *
         * @param ev event data as passed in by Hammer
         */
        InteractiveCameraMove: function(ev)
        {
            if(this.get('_interactivecontrol') == false) return;

            if(ev.type == "panstart")
            {
                this.set('_interaction_type', ev.srcEvent.shiftKey ? "PANNING" : (ev.srcEvent.ctrlKey || ev.button === 2 ? "ENVROT" : "ROTATION"));
                if( !(WV3DConstants.interaction.ROTATE & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "ROTATION") ) return;
                if( !(WV3DConstants.interaction.PAN & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "PANNING") ) return;
                if( !(WV3DConstants.interaction.ENVROT & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "ENVROT") ) return;
                this.get('_cameratransformstate').deltax = ev.deltaX;
                this.get('_cameratransformstate').deltay = ev.deltaY;
                this.get('_view').StartInteractiveFramerate();
            }

            if( !(WV3DConstants.interaction.ROTATE & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "ROTATION") ) return;
            if( !(WV3DConstants.interaction.PAN & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "PANNING") ) return;
            if( !(WV3DConstants.interaction.ENVROT & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "ENVROT") ) return;

            if(ev.type == "panend")
            {
                this.get('_view').EndInteractiveFramerate();
            }

            switch(this.get('_interaction_type'))
            {
                case "PANNING":
                    this.InteractiveCameraPanOffset(ev);
                break;

                case "ROTATION":
                    var curCameraAngles = this.get('_view').GetCameraAngles();
                    var curCameraScale = this.get('_view').GetCameraScale();
                    var resDeltaX = Math.abs(ev.deltaX - this.get('_cameratransformstate').deltax)/(curCameraScale*1.0);
                    var resDeltaY = Math.abs(ev.deltaY - this.get('_cameratransformstate').deltay)/(curCameraScale*1.0);

                    var newCameraAngles = curCameraAngles;

                    if(ev.deltaX > this.get('_cameratransformstate').deltax)
                        newCameraAngles.azimuth = curCameraAngles.azimuth -= resDeltaX;
                    else
                        newCameraAngles.azimuth = curCameraAngles.azimuth += resDeltaX;

                    if(newCameraAngles.azimuth < -360 || newCameraAngles.azimuth > 360)
                        newCameraAngles.azimuth = 0.0;

                    if(ev.deltaY < this.get('_cameratransformstate').deltay)
                        newCameraAngles.elevation = Math.min(Math.max(curCameraAngles.elevation -= resDeltaY, -90), 90);
                    else
                        newCameraAngles.elevation = Math.min(Math.max(curCameraAngles.elevation += resDeltaY, -90), 89.99);

                    this.get('_view').SetCameraAngles(newCameraAngles.azimuth, newCameraAngles.elevation);

                    this.get('_cameratransformstate').deltax = ev.deltaX;
                    this.get('_cameratransformstate').deltay = ev.deltaY;

                    break;

                case "ENVROT":
                    this.InteractiveEnvironmentRotation(ev);
                    break;

                default:
                    LogMessage.Log("Unrecognised interaction type");
            }

            if(ev.type == "panend")
            {
                if(this.get("_clientEndCameraInteractionCallback")) this.get("_clientEndCameraInteractionCallback")();
            }

            ev.preventDefault();
        },


        /**
         * Hammer.js pinch event handler for zoom in/out of camera, panning.
         *
         * @param ev event data as passed in by Hammer
         */
        InteractiveCameraScale_touch: function(ev)
        {
            if(this.get('_interactivecontrol') == false)
                return;

            if( !(WV3DConstants.interaction.SCALE & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "SCALING") ) return;

            if( !(WV3DConstants.interaction.PAN & this.get('_interactivecontrolmask')) && (this.get('_interaction_type') === "PANNING") ) return;

            if(ev.type == "pinchstart")
            {
                this.get('_cameratransformstate').deltax = ev.deltaX;
                this.get('_cameratransformstate').deltay = ev.deltaY;
                this.get('_cameratransformstate').pscale = this.get('_view').GetCameraScale() || 1;
                this.get('_view').StartInteractiveFramerate();
            }

            if(ev.type == "pinchend")
            {
                this.get('_view').EndInteractiveFramerate();
            }

            if(WV3DConstants.interaction.SCALE & this.get('_interactivecontrolmask'))
                this.SetCameraScale(Math.min(Math.max(this.get('_cameratransformstate').pscale * ev.scale, 0.1), 20));

            if(WV3DConstants.interaction.PAN & this.get('_interactivecontrolmask'))
                this.InteractiveCameraPanOffset(ev);

            if(ev.type == "pinchend")
            {
                if(this.get("_clientEndCameraInteractionCallback")) this.get("_clientEndCameraInteractionCallback")();
            }

            ev.preventDefault();
        },


        InteractiveCameraScale_mouse: function(ev)
        {
            if(this.get('_interactivecontrol') == false)
                return;

            if( !(WV3DConstants.interaction.SCALE & this.get('_interactivecontrolmask'))) return;

            //Activate Interactive framerate for half second
            if(this.get("_mousewheel_timeout"))
            {
                window.clearTimeout(this.get("_mousewheel_timeout"));
            }
            var self = this;
            var view = this.get('_view');
            view.StartInteractiveFramerate();
            var callback = function()
            {
                self.get('_view').EndInteractiveFramerate(true);
                if(self.get("_clientEndCameraInteractionCallback")) self.get("_clientEndCameraInteractionCallback")();
            }
            this.set("_mousewheel_timeout", window.setTimeout(callback, 500));

            var curCameraScale = view.GetCameraScale();
            var newScale = Math.min(Math.max(curCameraScale + curCameraScale * ev.deltaY * ev.deltaFactor / 1000.0, 0.1), 20)
            this.SetCameraScale(newScale); //Scaling co-efficient of 1000 (empirically chosen) makes mouse wheel scrolling smoother across devices

            ev.preventDefault();
        },

        /**
         * Hammer.js pinch event handler for environment rotation.
         *
         * @param ev event data as passed in by Hammer
         */
        InteractiveEnvironmentRotation_touch: function(ev)
        {
            if(this.get('_interactivecontrol') == false) return;
            if( !(WV3DConstants.interaction.ENVROT & this.get('_interactivecontrolmask')) ) return;

            if(ev.type == "panstart")
            {
                this.get('_cameratransformstate').deltax = ev.deltaX;
                this.get('_cameratransformstate').deltay = ev.deltaY;
                this.get('_view').StartInteractiveFramerate();
            }

            if(ev.type == "panend")
            {
                this.get('_view').EndInteractiveFramerate();
            }

            this.InteractiveEnvironmentRotation(ev);

            if(ev.type == "panend")
            {
                if(this.get("_clientEndCameraInteractionCallback")) this.get("_clientEndCameraInteractionCallback")();
            }

            ev.preventDefault();
        },

        InteractiveEnvironmentRotation: function(ev)
        {
            var curQuaternion = this.get('_view').GetEnvironmentRotation();
            var resDeltaX = ev.deltaX - this.get('_cameratransformstate').deltax;
            var resDeltaY = ev.deltaY - this.get('_cameratransformstate').deltay;

            var offsetQuaternion = new THREE.Quaternion();
            offsetQuaternion.setFromEuler(new THREE.Euler( resDeltaY / 100, resDeltaX / 100, 0, 'XYZ' ));

            //Put the current offset rotation into view space
            var viewRotation = new THREE.Quaternion();
            viewRotation.setFromRotationMatrix(this.get('_view').get('_3JScamera').matrixWorld);
            curQuaternion.multiply(viewRotation);

            //Apply rotation
            curQuaternion.multiply(offsetQuaternion);

            //Put current offset back into world space
            curQuaternion.multiply(viewRotation.conjugate());

            this.get('_view').SetEnvironmentRotation(curQuaternion);

            this.get('_cameratransformstate').deltax = ev.deltaX;
            this.get('_cameratransformstate').deltay = ev.deltaY;
        },

        SetEnvironmentRotation: function(__quaternion, __redraw)
        {
            if(!(__quaternion instanceof THREE.Quaternion))
            {
                throw ('SetEnvironmentRotation: Not A Quaternion Exception');
            }
            this.get("_view").SetEnvironmentRotation(__quaternion, __redraw);
        },

        /**
         * Convenience function to reset camera position. Mainly to be used from the
         * corresponding API call.
         */
        ResetEnvironmentRotation: function()
        {
            var resetQuat = new THREE.Quaternion();
            this.get("_view").SetEnvironmentRotation(resetQuat);
        },

        /**
         * Hammer.js double tap event handler to reset camera position.
         *
         * @param ev event data as passed in by Hammer
         */
        InteractiveResetCamera: function(ev)
        {
            if(this.get('_interactivecontrol') == false)
                return;

            var doScale = (WV3DConstants.interaction.SCALE & this.get('_interactivecontrolmask'));
            var doRotate = (WV3DConstants.interaction.ROTATE & this.get('_interactivecontrolmask'));
            var doPan = (WV3DConstants.interaction.PAN & this.get('_interactivecontrolmask'));
            var doRotateEnv = (WV3DConstants.interaction.ENVROT & this.get('_interactivecontrolmask'));

            var redraw = !doPan && !doRotate && !doRotateEnv;
            if(doScale) this.SetCameraScale(1.0, redraw);

            redraw = !doRotate && !doRotateEnv;
            if(doPan) this.SetCameraPanOffset(0.0,0.0, redraw);

            redraw = !doRotateEnv;
            if(doRotate) this.SetCameraAngles(0, 0, redraw);

            if(doRotateEnv) this.ResetEnvironmentRotation();

            if(!(WV3DConstants.interaction.NONE & this.get('_interactivecontrolmask')))
                if(this.get("_clientEndCameraInteractionCallback")) this.get("_clientEndCameraInteractionCallback")();

            ev.preventDefault();
        },


        /**
         * Turns on/off interactive control on 3D view. When pass 'false' all types of
         * mouse/touch interactions are ignored.
         */
        SetInteractiveCameraControl: function(__onOrOff)
        {
            this.set('_interactivecontrol', __onOrOff);
        },


        /**
         * Control interactive functionality. "NONE", "PAN", "SCALE", "ROTATE" or an OR combination of these
         *
         * @function SetInteractiveCameraControlMask
         * @param __mask
         */
        SetInteractiveCameraControlMask: function(__mask)
        {
            this.set('_interactivecontrolmask', __mask);
        },


        /**
         * Convenience function to reset camera position. Mainly to be used from the
         * corresponding API call.
         */
        ResetCamera: function()
        {
            this.SetCameraAngles(0, 0);
            this.SetCameraScale(1.0);
            this.SetCameraPanOffset(0.0,0.0);
        },


       /**
         * Set the angular position of the camera relative to the default front position of the scene
         *
         * @function
         * @param {Number} azimuthAngle - the walk-around angle measured counterclockwise from the default front view
         * @param {Number} elevationAngle - the elevantion angle - positive values lift the camera above the scene centre
         */
        SetCameraAngles: function(__azimuthAngle, __elevationAngle, __redraw)
        {
            if(((typeof __azimuthAngle) != "number") || ((typeof __elevationAngle) != "number"))
            {
                throw ('SetCameraAngles: NaN Exception');
            }
            this.get('_view').SetCameraAngles(__azimuthAngle, __elevationAngle, __redraw);
        },


       /**
         * Get the angular position of the camera relative to the default front position of the scene
         *
         * @function
         */
        GetCameraAngles: function()
        {
            return this.get('_view').GetCameraAngles();
        },


        /**
         * Set the size of the scene within the camera view. This is achieved by automatically
         * adjusting the camera properties (e.g. changing the camera distance for a perspective camera)
         *
         * @function
         * @param {Number} __scale - the size of the scene bounding box in the view. A value of 1 will result in the scene filling the view in at least one dimension
         * @param {Boolean} [__redraw = true] If true, redraw the scene after this call
         */
        SetCameraScale: function(__scale, __redraw)
        {
            if(((typeof __scale) != "number"))
            {
                throw ('SetCameraScale: NaN Exception');
            }
            if(__scale <= 0)
            {
                throw ('SetCameraScale: Out of range Exception');
            }
            this.get('_view').SetCameraScale(__scale, __redraw);
        },


        /**
         * Get the size of the scene within the camera view. A value of 1 means the scene is filling the view in at least one dimension.
         *
         * @function
         */
        GetCameraScale: function()
        {
            return this.get('_view').GetCameraScale();
        },

        /**
         * Set the panning offset of the camera.
         *
         * @function
         * @param {Number} __x_offset - horizontal offset
         * @param {Number} __x_offset - vertical offset
         */
        SetCameraPanOffset: function(__x_offset, __y_offset, __redraw)
        {
            if(((typeof __x_offset) != "number")||((typeof __y_offset) != "number"))
            {
                throw ('SetCameraPanOffset: NaN Exception');
            }
            this.get('_view').SetCameraPanOffset({"x":__x_offset, "y":__y_offset}, __redraw);
        },


        /**
         * Get the current panning offset of the current camera. Returns an object with propertie "x" and "y" indicating the
         * offset in each of these dimensions.
         *
         * @function
         */
        GetCameraPanOffset: function()
        {
            return this.get('_view').GetCameraPanOffset();
        },

        /**
         * Enables a specific rendering mode in which the best suitable scale of the scene
         * is applied to the current view. This neglects previously set scale factor.
         *
         * @function
         */
        SetOptimisedViewRendering: function(__border, __enableOptimisation, __redraw)
        {
            if(((typeof __border.x) != "number")||((typeof __border.y) != "number"))
            {
                throw ('SetOptimisedViewRendering: NaN Exception');
            }
            if(((typeof __enableOptimisation) != "boolean"))
            {
                throw ('SetOptimisedViewRendering: NaB Exception');
            }
            this.get('_view').SetOptimisedViewRendering(__border, __enableOptimisation, __redraw);
        },

        /**
         * Set up a perspective camera. The camera distance from the scene centre will be set automaticall based on the current scale setting.
         *
         * @function
         * @param {Number} __fieldOfView - the field of view in degrees
         */
        SetPerspectiveCamera: function(__fieldOfView, __redraw)
        {
            if(((typeof __fieldOfView) != "number"))
            {
                throw ('SetPerspectiveCamera: NaN Exception');
            }
            if(__fieldOfView <= 0)
            {
                throw ('SetPerspectiveCamera: Out of range (__fieldOfView) Exception');
            }
            this.get('_view').SetPerspectiveCamera(__fieldOfView, __redraw);
        },


        /**
         * Set up an orthographic camera. The viewport size will be set automaticall based on the current scale setting.
         *
         * @function
         */
        SetOrthographicCamera: function(__redraw)
        {
            this.get('_view').SetOrthographicCamera(__redraw);
        },


        /**
        * Focus camera from a node's bbox using its URL in the scene
        * default focus of camera is on the root node of the scene
        *
        * @function
        * @param {String} __nodeURL - URL identifying the scene node
        * @param {Boolean} __redraw - If true, redraw the scene after box update - default is true
        */
        SetCameraFocus: function(__nodeURL, __redraw)
        {
            this.get('_view').SetCameraFocus(__nodeURL, __redraw);
        },


        /**
         * SetHighlight enabled or disabled ()
         *
         * @function
         * @param {boolean} __enableHighlights - enables the objects highlighting if true, disable it if false
         */
        SetHighlights: function(__enableHighlights, __redraw)
        {
            if(((typeof __enableHighlights) != "boolean"))
            {
                throw ('SetHighLights: NaB Exception');
            }
            this.get('_view').SetHighlights(__enableHighlights, __redraw);
        },

        /**
         * SetHighlightsConfiguration enables and disables each highlight component
         * Call function SetHighlights first to enable highlights
         * By default, __enableHorizonHighlights and __enableHardEdgeHighlights are set to true
         * @function
         * @param {boolean} __enableHorizonHighlights - enables the horizon highlighting if true and highlighting is enabled too, disable it otherwise
         * @param {boolean} __enableHardEdgeHighlights - enables the hard edge highlighting if true and highlighting is enabled too, disable it otherwise
         */
        SetHighlightsConfiguration: function(__enableHorizonHighlights, __enableHardEdgeHighlights, __redraw)
        {
            if(((typeof __enableHorizonHighlights) != "boolean") || ((typeof __enableHardEdgeHighlights) != "boolean") )
            {
                throw ('SetHighLights: NaB Exception');
            }
            this.get('_view').SetHighlightsConfiguration(__enableHorizonHighlights, __enableHardEdgeHighlights, __redraw)
        },

        /**
        * SetHighlightColor
        *
        * @function
        * @param {Number} __red - red component
        * @param {Number} __green - green component
        * @param {Number} __blue - blue component
        */
        SetHighlightsColor: function(__red, __green, __blue, __redraw)
        {
            if(((typeof __red) != "number") || ((typeof __green) != "number") || ((typeof __blue) != "number"))
            {
                throw ('SetHighlightColor: NaN Exception');
            }

            this.get('_view').SetHighlightsColor(__red, __green, __blue);
        },

        /**
        * SetBackgroundColor
        *
        * @function
        * @param {Number} __red - red component
        * @param {Number} __green - green component
        * @param {Number} __blue - blue component
        * @param {Number} __alpha - __alpha component (0 - 255) (represent the opacity of the background, useful for snapshot)
        */
        SetBackgroundColor: function(__red, __green, __blue, __alpha, __redraw)
        {
            if(((typeof __red) != "number") || ((typeof __green) != "number") || ((typeof __blue) != "number") || ((typeof __alpha) != "number"))
            {
                throw ('SetBackgroundColor: NaN Exception');
            }
            if(__red < 0 || __red > 255 || __green < 0 || __green > 255 || __blue < 0 || __blue > 255 || __alpha < 0 || __alpha > 255)
            {
                throw ('SetBackgroundColor: Out of range Exception');
            }

            this.get('_view').SetBackgroundColor(__red / 255, __green / 255, __blue / 255, __alpha / 255, __redraw);
        },

        /**
         * Show or Hide Labels for particular node or for all labels in the scene when no node url is specified
         */
        SetLabelsVisibility: function(__visible, __redraw)
        {
            if(typeof __visible !== 'undefined')
                this.get('_view').SetLabelsVisible(__visible, __redraw);
        },

        /**
         * Update properties of Labels set through JSON file
         */
        UpdateLabelProperties: function(__nodeURL, __properties)
        {
            if(typeof __nodeURL === 'undefined' || !__nodeURL)
                throw "UpdateLabelProperties: invalid __nodeURL argument";

            if(typeof __properties === 'undefined' || !__properties)
                throw "UpdateLabelProperties: invalid __properties argument";

            return this.get('_view').UpdateLabelProperties(__nodeURL, __properties);
        },

        /**
        * Update properties of all labels in the scene in one go
        */
        UpdateAllLabelsProperties: function(__properties)
        {
            if(typeof __properties == 'undefined' || !__properties)
                throw "UpdateAllLabelsProperties: invalid __properties argument";

            return this.get('_view').UpdateAllLabelsProperties(__properties);
        },

        /**
         * Get the queried data info of a node using its URL
         *
         * @function
         * @param {string} __nodeURL -  the type of information we want to retrieve from node
         * @param {string} __datatype -  the type of information we want to retrieve from node
         * to choose among the following list:
         * - NODEOBJECTVIS returns (Boolean)
         * @returns object, boolean, string, depending on the type of info
         */
        Get: function(__nodeURL, __datatype)
        {
            return this.get('_view').Get(__nodeURL, __datatype);
        },

        /**
         * Prints a memory report
         * @function LogMemoryReport
         */
        LogMemoryReport: function()
        {
            return this.get('_view').LogMemoryReport();
        },

        /**
         * Print list of opaque scene vertices in world coordinates
         */
        PrintOpaqueSceneVertices: function()
        {
            return this.get('_view').PrintOpaqueSceneVertices();
        },

        /**
         * SetReflections
         *
         * @function SetReflections sets the reflection on, off, or relative to what is set in the Json file.
         * @param {string} - a string, "ON", "OFF", or "JSON" ("JSON" is already set by default)
         * @param {number} [0 1] - opacity of reflections (only if "ON" mode chosen)
         * @param {number} [0 1] - position of reflections (only if "ON" mode chosen)
         * @param {number} [1 2] - fading size of reflections (only if "ON" mode chosen)
         * @param {boolean} [optional - __redraw = true] - indicates if the scene has to be redrawn after this call. Default is true.
         */
        SetReflections: function(__config, __opacity, __fade, __position, __fadingSize, __redraw)
        {
            if(__config != "ON" && __config != "OFF" && __config != "JSON")
            {
                throw "SetReflections: Invalid parameter";
            }
            if(__config == "ON")
            {
                if((typeof __opacity)!=='number')
                {
                     throw "SetReflections: NaN Exception";
                }
                if(__opacity < 0 || __opacity > 1)
                {
                    throw "SetReflections: Out of range Exception";
                }
                if((typeof __fade)!=='boolean')
                {
                    throw "SetReflections: Invalid parameter";
                }
                if(__fade)
                {
                    if((typeof __position)!=='number' || (typeof __fadingSize)!=='number')
                    {
                        throw "SetReflections: NaN Exception";
                    }
                    if(__position < 0 || __position > 1 || __fadingSize < 1 || __fadingSize > 2)
                    {
                        throw "SetReflections: Out of range Exception";
                    }
                }
            }
            this.get('_view').SetReflections(__config, __opacity, __fade, __position, __fadingSize, __redraw);
        },

        /**
         * SetAnimations enabled or disabled
         *
         * @function
         * @param {boolean} [enableAnimations=true] - enables the camera animation if true, disable it if false
         */
        SetAnimations: function(__enableAnimations)
        {
            this.get('_view').SetAnimations(__enableAnimations);
        },

        /**
         * SetInteractionOptimization enabled or disabled
         *
         * @function SetInteractionOptimization
         * @param {boolean} [__enableInteractionOptimization=true] - enables the scene optimization while interacting with the model if true, disable it if false
         */
        SetInteractionOptimization: function(__enableInteractionOptimization)
        {
            this.get('_view').SetInteractionOptimization(__enableInteractionOptimization);
        },

        /**
         * SetQualityLevel
         *
         * @function SetQualityLevel sets maximum quality when no interaction with the model or when interaction optimization is disabled
         * by setting the number of transparency peels for transparent models. This quality level does not affect quality of opaque models.
         * @param {number} - an integer between 1 and 10 (included)
         */
        SetQualityLevel: function(__qualityLevel, __redraw)
        {
            if(typeof __qualityLevel !== 'number')
            {
                throw ('SetQualityLevel: NaN Exception');
            }
            if(__qualityLevel < 1 || __qualityLevel > 10)
            {
                throw "SetQualityLevel: Out of range Exception";
            }
            this.get('_view').SetQualityLevel(__qualityLevel, __redraw);
        },

        /**
         * SetLightingContrast
         *
         * @function SetLightingContrast sets the lighting contrast (in designer quality only)
         * by reducing the directional light and increasing the ambient light components.
         * @param {number} - a floating number between 0 and 1 (included)
         */
        SetLightingContrast: function(__lightingContrast, __redraw)
        {
            if(typeof __lightingContrast !== 'number')
            {
                throw ('SetLightingContrast: NaN Exception');
            }
            if(__lightingContrast < 0 || __lightingContrast > 1)
            {
                throw "SetLightingContrast: Out of range Exception";
            }
            this.get('_view').SetLightingContrast(__lightingContrast, __redraw);
        },

        /**
         * Implementation details to get an indication of the degree to which the current system is supported for 3D viewing
         *
         * @function
         * @param {Array} __issues - an array of issues causing the limited support level
         * @returns supportLevel.FULL | supportLevel.PARTIAL | supportLevel.NONE
         */
        GetSystemSupportLevel : function()
        {
            LogMessage.Log("ViewController: Get system support level");
            return WV3DSystemChecker.GetSystemSupportLevel();
        },

        /**
         * Implementation to get a technical report containing system info
         *
         * @function
         * @returns {String} a report containing system info
         */
        GetSystemReport : function()
        {
            LogMessage.Log("ViewController: Get readable system report");

            return WV3DSystemChecker.GetSystemReport();
        },


        /**
        * Get a snapshot of the current viewer state as specified in format
        *
        * @function
        * @param {string} [format="png"] - image format
        * @returns {Object} image data
        */
        GetSnapshot : function(__format, __width, __height)
        {
            return this.get('_view').GetSnapshot(__format, __width, __height);
        }

    });

    return WV3DViewerController;

});
