/**
 * @author DOGO & SVN
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "WV3Dmod/shaders/VertexShaderQuad",
        "WV3Dmod/shaders/ShaderUtils"
        ],
        function(THREE,
                 VertexShaderQuad,
                 ShaderUtils)
        {
        /**
         * @alias module:WebViewer3D
         * @class ColorBlendShaderMaterial: combines two textures into output using blending
         */

        var D2Shader = {

            uniforms:
            {
                "canvas_size": { type: "v2", value: null },

                "EnableBGBlend" : { type: "i", value: 0 }, //Enable final blending of opaque (with computation of opaque AND transparent alpha)
                "EnableBackdrop" : { type: "i", value: 0 }, //Enable rendering onto a background
                "EnableBackdropAlpha" : { type: "i", value: 0 }, //Enable rendering onto a transparent background
                "EnableMirror" : { type: "i", value: 0 }, //Enable rendering onto a transparent background

                "CX1RT" : { type: "t", value: null },    // color of new peel (or opaque peel if EnableBGBlend)
                "TransparencyStack" : { type: "t", value: null } , //Accumulated transparency
                "OpaqueRT" : { type: "t", value: null } , //Transparency of opaque scene (just a mask separating object from background)
                "ColorStackRT" : { type: "t", value: null } , // color of transparent scene (just a mask separating object from background)

                "Backdrop" : { type: "t", value: null },    //backdrop color
                "Backdrop_scale": { type: "v2", value: new THREE.Vector2(1, 1) },
                "Backdrop_offset": { type: "v2", value: new THREE.Vector2(0, 0) },
                "BGColor": { type: "c", value: new THREE.Color( 0xffffff ) }, //Background color
                "BGAlpha": { type: "f", value: 1. }, //Background alpha

                "reflection_opacity": {type: "f", value: 0.},
                "reflection_fade": {type: "i", value: 0},
                "reflection_fade_size": {type: "f", value: 0.},
                "reflection_fade_position": {type: "f", value: 0.}
            },

            vertexShader: VertexShaderQuad.vertexShader,

            fragmentShader:
            [
                "uniform vec2 canvas_size;",

                "uniform bool EnableBGBlend;",
                "uniform bool EnableBackdrop;",
                "uniform bool EnableBackdropAlpha;",
                "uniform bool EnableMirror;",

                "uniform sampler2D CX1RT;",
                "uniform sampler2D TransparencyStack;",
                "uniform sampler2D OpaqueRT;",
                "uniform sampler2D ColorStackRT;",

                "uniform sampler2D Backdrop;",
                "uniform vec2 Backdrop_scale;",
                "uniform vec2 Backdrop_offset;",
                "uniform vec3 BGColor;",
                "uniform float BGAlpha;",

                "uniform float reflection_opacity;",
                "uniform bool reflection_fade;",
                "uniform float reflection_fade_size;",
                "uniform float reflection_fade_position;",

                //When EnableBGBlend and EnableMirror are true, this shader uses mirrored parameters to blend
                //with the background on a similar fashion than what is being done in BackdropShader.js

                //At this point (when EnableBGBlend is enabled) ColorStack already contains the transparent peels
                //this shader allows to draw (blend) on-top-of/with it. However, alpha has been cleared from rendertarget
                //to allow rendering on a transparent background, so we need to set it properly here, even for transparent
                //objects already present in the render target (see grey variable below)

                "void main()",
                "{",
                    "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",

                    "vec3 cx = texture2D(CX1RT, loc).rgb;",
                    "vec3 trans = texture2D(TransparencyStack, loc).rgb;",
                    "vec3 tex_transop = texture2D(OpaqueRT, loc).rgb;",
                    "vec3 transop = (EnableBGBlend) ? tex_transop : vec3(1.0);",
                    "vec3 ct = texture2D(ColorStackRT, loc).rgb;",

                    //Supposedly background images are not transparent, but in the case we want to support
                    //the alpha channel, just uncomment one of the two lines below (and comment the current one)
                    // "vec4 backdrop = vec4(texture2D(Backdrop, loc * Backdrop_scale + Backdrop_offset).rgb, BGAlpha);",//Alpha channel set using BGAlpha defined by user
                    // "vec4 backdrop = texture2D(Backdrop, loc * Backdrop_scale + Backdrop_offset)",//Four channels textures
                    "vec4 backdrop = texture2D(Backdrop, loc * Backdrop_scale + Backdrop_offset);",
                    "if(!EnableBackdropAlpha) backdrop.a = 1.;",
                    "if(!EnableBackdrop) backdrop = vec4(BGColor, BGAlpha);",

                    //convert transparency stack into a grey-scale value to allow computing its opacity level
                    "float grey = dot(trans, vec3(0.299, 0.587, 0.114));",
                    "if(!EnableMirror)",
                    "{",
                        //This case should be calculated on a similar way the else branch computes its color + alpha
                        "gl_FragColor.rgb = EnableBGBlend? trans * (transop == vec3(0.) ? cx : backdrop.rgb) + ct : trans * cx;",
                        "gl_FragColor.a = EnableBGBlend? ( transop == vec3(0.) ? 1.0 : backdrop.a + 1. - grey) : 1.;",
                    "}",
                    "else",
                    "{",
                        "float refopacity = reflection_opacity;",
                        ShaderUtils.get('ReflectionFadeChunck'),

                        //This version is working quite well apparently, in all cases with no transparent background.
                        //With transparent background however, there seems to be two issues:
                        //- reflection tends to fade to much to transparent and the fading tends towards white
                        //- global background is not combined properly so that it is different from when reflections are disabled\
                        "vec3 color = trans * (transop == vec3(0.) ? cx : backdrop.rgb) + ct;",
                        "vec3 mirror_rgb_frag = (transop == vec3(0.))? color * refopacity + backdrop.rgb * (1.- refopacity) : (ct != vec3(0.)? color * refopacity + backdrop.rgb * (1.- refopacity) : backdrop.rgb);",
                        "float mirror_alpha_frag = (transop == vec3(0.))? refopacity + backdrop.a * (1.- refopacity) : (ct != vec3(0.) ? refopacity * (1. - grey) + backdrop.a * (1.- refopacity + grey) : backdrop.a);",
                        "gl_FragColor.rgb = mirror_rgb_frag;",
                        "gl_FragColor.a = mirror_alpha_frag;",

                    "}",
                "}"
            ].join("\n")

        }

        return D2Shader;

});