/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
    "WV3Dmod/shaders/VertexShaderPM",
    "WV3Dmod/shaders/ShaderUtils"
    ],
    function(THREE,
         VertexShaderPM,
         ShaderUtils)
    {
      /**
       * @alias module:WebViewer3D
       * @class Diffraction Shader for Printable Materials
       */

    var DiffractionShaderPM = {

          uniforms: THREE.UniformsUtils.merge( [

             VertexShaderPM.uniforms,

            {
              "uvdhTexture" : { type: "t", value: null },
              "diffFilterTexture" : { type: "t", value: null },
              "isFloorReflection": { type: "i", value: 0 },
              "normalMap_chan_0" : { type: "t", value: null },
              "normalMap_chan_1" : { type: "t", value: null },
              "spectrumTexture" : { type: "t", value: null },
              "envMapRotQuat": { type: "v4", value: new THREE.Vector4(0, 0, 0, 1) },

              "grating_spacing_scale" : { type: "f", value: 1.6 },
              "defaultDiffSpacing" : { type: "f", value: .5 },
              "defaultDiffHaze" : { type: "f", value: 1 },

              // default values taken from lights in Office environment
              "lightPos" : { type: "v3v", value: [ new THREE.Vector3( 0, 0, 0 ),
                                       new THREE.Vector3( 0, 0, 0 ),
                                       new THREE.Vector3( 0, 0, 0 ),
                                       new THREE.Vector3( 0, 0, 0 ) ] },
              "lightIntensity" : { type: "fv1", value: [ 0, 0, 0, 0] },
              "lightRadius" : { type: "fv1", value: [ 0, 0, 0, 0] },
              "lightHardness" : { type: "fv1", value: [ 0, 0, 0, 0] },

              "canvas_size": { type: "v2", value: new THREE.Vector2(1, 1) },
              "DX1RT" : { type: "t", value: null },
              "DX2RT" : { type: "t", value: null },
              "NBRTSRC" : { type: "i", value: 0 }

            }

          ] ),

          vertexShader: VertexShaderPM.vertexShader,

          fragmentShader: [

            "varying vec2 texcoordVarying;",
            "varying vec2 texcoordVaryingScaled;",
            "varying vec3 vNormal;",
            "varying vec4 vPosition;",
            "varying vec4 vWorldPosition;",

            "uniform sampler2D uvdhTexture;",
            "uniform sampler2D diffFilterTexture;",
            "uniform sampler2D normalMap_chan_0;",
            "uniform sampler2D normalMap_chan_1;",
            "uniform sampler2D spectrumTexture;",
            "uniform float grating_spacing_scale;",
            "uniform float defaultDiffSpacing;",
            "uniform float defaultDiffHaze;",
            "uniform float lightIntensity[4];",
            "uniform float lightRadius[4];",
            "uniform float lightHardness[4];",
            "uniform vec3 lightPos[4];",
            "uniform vec4 envMapRotQuat;",
            "uniform bool isFloorReflection;",

            // currently use global vars for this as they're calculated in a sub-subfunction and used in main,
            // and I'm not sure if inout params are supported in OpenGL 2 GLSL
            "vec3 tangent;",
            "vec3 binormal;",

            "mat3 cotangent_frame(vec3 N, vec3 p, vec2 uv)",
              "{",
                  // get edge vectors of the pixel triangle
                  "vec3 dp1 = dFdx( p );",
                  "vec3 dp2 = dFdy( p );",
                  "vec2 duv1 = dFdx( uv );",
                  "vec2 duv2 = dFdy( uv );",

                  // solve the linear system
                  "vec3 dp2perp = cross( dp2, N );",
                  "vec3 dp1perp = cross( N, dp1 );",
                  "tangent = normalize(dp2perp * duv1.x + dp1perp * duv2.x);",
                  "binormal = normalize(dp2perp * duv1.y + dp1perp * duv2.y);",

                  // construct a scale-invariant frame
                  "float invmax = inversesqrt( max( dot(tangent,tangent), dot(binormal,binormal) ) );",
                  "tangent *= invmax;",
                  "binormal *= invmax;",

                  "return mat3( tangent, binormal, N );",
              "}",

              "vec3 perturb_normal( vec3 N, vec3 V, vec2 texcoord)",
              "{",
                  // assume N, the interpolated vertex normal and
                  // V, the view vector (vertex to eye)
                  //(uv coordinates are not rescaled for bump maps)
                  "float chan_0 = texture2D(normalMap_chan_0, texcoord).r;",
                  "float chan_1 = texture2D(normalMap_chan_1, texcoord).r;",

                  "vec3 map = vec3(chan_0, chan_1, 0.);",

                  //Rescale map (from 0 to 1 into -1 to 1)
                  "map = map * 255./127. - 128./127.;",
                  // decode bump map
                  "vec2 tmp;",
                  "if(map.x >= 0.0) tmp.x = -1.0; else tmp.x = 1.0;",
                  "if(map.y >= 0.0) tmp.y = -1.0; else tmp.y = 1.0;",
                  "map = abs(map) * 9.96722625883599;",
                  "map = exp2(map);",
                  "map = (map * 0.001) - 0.001;",
                  "map = map * vec3(tmp,0.);",
                  "map.z = sqrt( 1. - dot( map.xy, map.xy ) );",

                  "mat3 TBN = cotangent_frame(N, -V, texcoord);",
                  "return normalize(TBN * map);",
              "}",

            // calculates the combined color for the diffraction from all lightsources, with double grating if needed.
              "vec4 diffraction_color( vec4 uvdh, vec3 worldP, vec3 eyeVector, vec3 pN, vec3 vNormalOriginal)",
              "{",

                "vec4 diffractColor = vec4(0., 0., 0., 0.);",
                "vec4 filt = texture2D(diffFilterTexture, texcoordVaryingScaled);",

                "#ifdef DOUBLE_GRATING",
                "vec4 originaluvdh = uvdh;",
                "for (int doubleGratingPass = 0; doubleGratingPass < 2; doubleGratingPass ++)",
                "{",
                "uvdh = originaluvdh;",//Recover initial values (for second pass)

                //For double gratings, the uvdh texture contains 2 direction pairs and we need to select
                "if(doubleGratingPass == 1)",
                    "uvdh.xy = uvdh.zw;",
                //For double gratings, set the spacing and haze factors to constants
                "uvdh.zw = vec2(defaultDiffSpacing, defaultDiffHaze);",
                "#endif",

                //recover the angle into .x from 2D triangle encoded 0 to 1 into linear -pi/2 to pi/2)
                "uvdh.y = uvdh.y - 0.5;",
                "float sign = 1.57079632679;",
                "if(uvdh.y < 0.0)",
                  "sign = -1.57079632679;",
                //recover the correct sign and scale
                "uvdh.x = (1.0 - uvdh.x) * sign;",

                //Make unit grating vector
                "float uvdhx_t = uvdh.x;",
                "uvdh.x = sin(uvdhx_t);",
                "uvdh.y = cos(uvdhx_t);",

                //Transform grating vector to world space
                "vec3 Gn = tangent * uvdh.x;",    //Gn -> Normalised grating vector
                "Gn = (binormal * uvdh.y) + Gn;",

                "for ( int lightIndex = 0; lightIndex < 4; lightIndex++)",
                "{",
                    "float hardness = 1.0 - uvdh.w;",   //hardness = 1 - haze //Recover initial value (for each light)
                    "vec3 illumVec = normalize(worldP - lightPos[ lightIndex ]);",

                    //ignore this light if it is behind the surface
                    "if(dot(illumVec, vNormalOriginal * -1.0) < 0.0)",
                        "continue;",

                    //Add the illumination vector to eye vector to make half vector
                    "vec3 H = illumVec + eyeVector;",

                    //Apply conical diffration restriction and use light parameters for soft roll-off
                    "float delta = abs(dot(Gn, H));",

                    //Use diffraction haze to modify light parameters
                    "hardness = min(lightHardness[ lightIndex ], hardness);",
                    "float radius = (uvdh.w * 0.1) + lightRadius[ lightIndex ];",

                    //Compute drop-off
                    "float intensityFact = 0.0;",
                    "if((delta - radius) < 0.0)",
                     "intensityFact = 1.0;",   //check if we are inside the hard light area?
                    "float hardRadius = radius * hardness;",
                    "float intensityScale = (radius - delta) / (radius - hardRadius);",
                    "if((delta - hardRadius) < 0.0)",   //check if we are inside the drop off zone
                      "intensityScale = 1.0;",
                    "intensityFact *= intensityScale;",

                    //Multiply in the light intensity
                    "intensityFact *= lightIntensity[ lightIndex ];",
                    //Multiply in the grating spacing and scale (from 0 to 1 into 0 to spacingScale)
                    "vec3 dG = (Gn * uvdh.z) * grating_spacing_scale;",//default spacing scale is 1.6

                    //Calculate the diffraction grating equation and scale wavelength to match texture 0 = 0, 1 = 2.048um
                    "float wavl = abs(dot(cross(H, pN), dG)) * 0.48828125;",

                    //Look up the RGB value for this wavelength and multiply by mask value
                    "vec2 lightSpecTexCoord = vec2(wavl, wavl);",
                    "vec4 lightSpectrum = texture2D(spectrumTexture, lightSpecTexCoord);",
                    "lightSpectrum = lightSpectrum * filt;",

                    "vec4 current_color = lightSpectrum * intensityFact;",
                    "diffractColor += current_color;",
                    // "diffractColor = current_color - current_color * diffractColor + diffractColor;",//Hard coded (1-dst,one) blending (check with Kris)
                "}",

                "#ifdef DOUBLE_GRATING",
                "}",
                "#endif",
                "return diffractColor; ",
              "}",

            ShaderUtils.get('DeclareDepthSamplersChunck'),
            ShaderUtils.get('EncodeDepthChunck'),
            ShaderUtils.get('DecodeDepthChunck'),
            ShaderUtils.get('EnvironmentRotationChunck'),

            "void main()",
            "{",

                // don't do anything if the resources (light, spectrum textur etc) are not loaded yet
                "#ifndef USE_HOLOGRAPHICS",
                "discard;",
                "#else",

                ShaderUtils.get('SelectPeelChunck'),

                "vec3 vNormalOriginal = normalize(vNormal);", // TODO: Does this work for holograms on backfaces? if not, we have to swap this for backfaces

                //Texture lookup to find the grating direction, grating spacing, mask and haze
                "vec4 uvdh = texture2D(uvdhTexture, texcoordVaryingScaled);",
                "#ifdef REMAPPED_DIFF_GRATING_ALPHA_CHANNEL",
                //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices)
                //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
                "uvdh.a = clamp((uvdh.a - 127./255.) * 255. / 127., 0.,1.);",
                "#endif",

                //Calculate illumination vector
                "vec3 worldP = vWorldPosition.xyz;",
                "vec3 eyeVector = normalize(worldP - cameraPosition.xyz);",

                "#ifdef USE_CHAN0_TEXTURE",
                  "#ifdef USE_CHAN1_TEXTURE",
                    // get surface normal and then perturb it using normal from normal map
                    // read the normal map channels depending on the texture format (uv coordinates are not rescaled for bump maps)
                    "vec3 pN = perturb_normal(normalize(vNormal), eyeVector, texcoordVarying);",
                  "#else",
                    "#define NO_BUMP",
                  "#endif",
                "#else",
                  "#define NO_BUMP",
                "#endif",

                "#ifdef NO_BUMP",
                  "vec3 pN = normalize(vNormal);",
                "#endif",

                "pN = reflect(eyeVector, pN);",
                "if(isFloorReflection);",
                "{",
                  "pN.y = -pN.y;",
                  "vNormalOriginal.y = -vNormalOriginal.y;",
                "}",

                "pN = rotate_vec_from_quat(envMapRotQuat, pN);",

                "gl_FragColor = diffraction_color( uvdh, worldP, eyeVector, pN, vNormalOriginal);",
                // "gl_FragColor.a = 1.0;",

                "#endif",
            "}"

          ].join("\n")


    }

    return DiffractionShaderPM;

});