/**
 * @authors DOGO, zz85 / https://github.com/zz85 | https://www.lab4games.net/zz85/blog
 *
 * Edge Detection Shader using Frei-Chen filter
 * Based on http://rastergrid.com/blog/2011/01/frei-chen-edge-detector
 *
 * @module WebViewer3D
 * @copyright   2016 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
		"WV3Dmod/shaders/VertexShaderQuad"],
	function(THREE,
			VertexShaderQuad)
	{
	    /**
	     * @alias module:WebViewer3D
	     * @class EdgeeShader
	     */

		var EdgeShader =
		{

			uniforms:
			{
				"depth_texture": { type: "t", value: null },
				"canvas_size": { type: "v2", value: null },
				"color":    { type: "v3", value: new THREE.Vector3( 0., 0., 0. ) }
			},

			vertexShader: VertexShaderQuad.vertexShader,

			fragmentShader:
			[
				"uniform sampler2D depth_texture;",
	            "uniform vec3 color;",
				"uniform vec2 canvas_size;",

				"vec2 vUv;",
				"vec2 texel;",

				"mat3 G[9];",

				// hard coded matrix values!!!! as suggested in https://github.com/neilmendoza/ofxPostProcessing/blob/master/src/EdgePass.cpp#L45

				"const mat3 g0 = mat3( 0.3535533845424652, 0, -0.3535533845424652, 0.5, 0, -0.5, 0.3535533845424652, 0, -0.3535533845424652 );",
				"const mat3 g1 = mat3( 0.3535533845424652, 0.5, 0.3535533845424652, 0, 0, 0, -0.3535533845424652, -0.5, -0.3535533845424652 );",
				"const mat3 g2 = mat3( 0, 0.3535533845424652, -0.5, -0.3535533845424652, 0, 0.3535533845424652, 0.5, -0.3535533845424652, 0 );",
				"const mat3 g3 = mat3( 0.5, -0.3535533845424652, 0, -0.3535533845424652, 0, 0.3535533845424652, 0, 0.3535533845424652, -0.5 );",
				"const mat3 g4 = mat3( 0, -0.5, 0, 0.5, 0, 0.5, 0, -0.5, 0 );",
				"const mat3 g5 = mat3( -0.5, 0, 0.5, 0, 0, 0, 0.5, 0, -0.5 );",
				"const mat3 g6 = mat3( 0.1666666716337204, -0.3333333432674408, 0.1666666716337204, -0.3333333432674408, 0.6666666865348816, -0.3333333432674408, 0.1666666716337204, -0.3333333432674408, 0.1666666716337204 );",
				"const mat3 g7 = mat3( -0.3333333432674408, 0.1666666716337204, -0.3333333432674408, 0.1666666716337204, 0.6666666865348816, 0.1666666716337204, -0.3333333432674408, 0.1666666716337204, -0.3333333432674408 );",
				"const mat3 g8 = mat3( 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408, 0.3333333432674408 );",

				"void main(void)",
				"{",
					"vUv = gl_FragCoord.xy;",
					"texel  = vec2(1.0 / canvas_size.x, 1.0 / canvas_size.y);",
					"G[0] = g0,",
					"G[1] = g1,",
					"G[2] = g2,",
					"G[3] = g3,",
					"G[4] = g4,",
					"G[5] = g5,",
					"G[6] = g6,",
					"G[7] = g7,",
					"G[8] = g8;",

					"mat3 I;",
					"float cnv[9];",
					"vec3 sample;",
					"float depth;",
					"vec3 value;",

					/* fetch the 3x3 neighbourhood and use the RGB vector's length as intensity value */

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[0][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[1][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0,-1.0)) ));",
					"sample = vec3(depth);",
					"I[2][0] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 0.0)) ));",
					"sample = vec3(depth);",
					"I[0][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 0.0)) ));",
					"sample = vec3(depth);",
					"value = sample;",
					"I[1][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 0.0)) ));",
					"sample = vec3(depth);",
					"I[2][1] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2(-1.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[0][2] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 0.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[1][2] = length(sample);",

					"depth = DecodeFloatRGBA(texture2D(depth_texture, texel * (vUv + vec2( 1.0, 1.0)) ));",
					"sample = vec3(depth);",
					"I[2][2] = length(sample);",

					/* calculate the convolution values for all the masks */
					"for (int i=0; i<9; i++) {",
						"float dp3 = dot(G[i][0], I[0]) + dot(G[i][1], I[1]) + dot(G[i][2], I[2]);",
						"cnv[i] = dp3 * dp3;",
					"}",

					"float M = (cnv[0] + cnv[1]) + (cnv[2] + cnv[3]);",
					"float S = (cnv[4] + cnv[5]) + (cnv[6] + cnv[7]) + (cnv[8] + M);",

					"float val = sqrt(M/S);",
					"if (val > 0.02) gl_FragColor = vec4(color, 1.);",
					"else discard;",
				"}",

			].join("\n")

		}

		return EdgeShader;
	}
);