/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
    "WV3Dmod/shaders/VertexShaderPM",
    "WV3Dmod/shaders/ShaderUtils"
    ],
    function(THREE,
         VertexShaderPM,
         ShaderUtils)
    {
      /**
       * @alias module:WebViewer3D
       * @class Irradiance Shader for Printable Materials
       */

    var IrradianceShaderPM = {

          uniforms: THREE.UniformsUtils.merge( [

        VertexShaderPM.uniforms,

        {
          "diffuseColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
          "opacity" : { type: "f", value: null },
          "diffuseTexture" : { type: "t", value: null },
          "normalMap_chan_0" : { type: "t", value: null },
          "normalMap_chan_1" : { type: "t", value: null },
          "isFloorReflection": { type: "i", value: 0 },
          "irradianceCubeMap" : { type: "t", value: null },
          "envMapRotQuat": { type: "v4", value: new THREE.Vector4(0, 0, 0, 1) },
          "shadowsShadingContrast" : { type: "f", value: 0.3 },

          "canvas_size": { type: "v2", value: new THREE.Vector2(1, 1) },
          "DX1RT" : { type: "t", value: null },
          "DX2RT" : { type: "t", value: null },
          "NBRTSRC" : { type: "i", value: 0 }
        }

      ] ),

      vertexShader: VertexShaderPM.vertexShader,

      fragmentShader: [

        "varying vec2 texcoordVarying;",
        "varying vec2 texcoordVaryingScaled;",
        "varying vec3 vNormal;",
        "varying vec4 vPosition;",
        "varying vec4 vWorldPosition;",

        "uniform vec3 diffuseColor;",
        "uniform float opacity;",
        "uniform sampler2D diffuseTexture;",
        "uniform sampler2D normalMap_chan_0;",
        "uniform sampler2D normalMap_chan_1;",
        "uniform bool isFloorReflection;",
        "uniform samplerCube irradianceCubeMap;",
        "uniform vec4 envMapRotQuat;",

        "uniform float shadowsShadingContrast;",

        "mat3 cotangent_frame(vec3 N, vec3 p, vec2 uv)",
        "{",
            // get edge vectors of the pixel triangle
            "vec3 dp1 = dFdx( p );",
            "vec3 dp2 = dFdy( p );",
            "vec2 duv1 = dFdx( uv );",
            "vec2 duv2 = dFdy( uv );",

            // solve the linear system
            "vec3 dp2perp = cross( dp2, N );",
            "vec3 dp1perp = cross( N, dp1 );",
            "vec3 T = normalize(dp2perp * duv1.x + dp1perp * duv2.x);",
            "vec3 B = normalize(dp2perp * duv1.y + dp1perp * duv2.y);",

            // construct a scale-invariant frame
            "float invmax = inversesqrt( max( dot(T,T), dot(B,B) ) );",
            "return mat3( T * invmax, B * invmax, N );",
        "}",

        "vec3 perturb_normal( vec3 N, vec3 V, vec2 texcoord )",
        "{",
            // assume N, the interpolated vertex normal and
            // V, the view vector (vertex to eye)
            //(uv coordinates are not rescaled for bump maps)
            "float chan_0 = texture2D(normalMap_chan_0, texcoord).r;",
            "float chan_1 = texture2D(normalMap_chan_1, texcoord).r;",

            "vec3 map = vec3(chan_0, chan_1, 0.);",

            //Rescale map (from 0 to 1 into -1 to 1)
            "map = map * 255./127. - 128./127.;",
            // decode bump map
            "vec2 tmp;",
            "if(map.x >= 0.0) tmp.x = -1.0; else tmp.x = 1.0;",
            "if(map.y >= 0.0) tmp.y = -1.0; else tmp.y = 1.0;",
            "map = abs(map) * 9.96722625883599;",
            "map = exp2(map);",
            "map = (map * 0.001) - 0.001;",
            "map = map * vec3(tmp,0.);",
            "map.z = sqrt( 1. - dot( map.xy, map.xy ) );",

            "mat3 TBN = cotangent_frame(N, -V, texcoord);",
            "return normalize(TBN * map);",
        "}",

        ShaderUtils.get('DeclareDepthSamplersChunck'),
        ShaderUtils.get('EncodeDepthChunck'),
        ShaderUtils.get('DecodeDepthChunck'),
        ShaderUtils.get('EnvironmentRotationChunck'),

        "void main()",
        "{",
            ShaderUtils.get('SelectPeelChunck'),

           "#ifdef USE_MAP",
             "vec4 diffColour = texture2D(diffuseTexture, texcoordVaryingScaled);",
              "#ifdef REMAPPED_DIFFUSE_ALPHA_CHANNEL",
               //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices)
               //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
                "diffColour.a = clamp((diffColour.a - 127./255.) * 255. / 127., 0.,1.);",
              "#endif",
           "#else",
             "vec4 diffColour = vec4(diffuseColor,opacity);",
           "#endif",

            "vec3 eyeVector = normalize(vec3(vWorldPosition) - cameraPosition.xyz);",

            "#ifdef USE_CHAN0_TEXTURE",
              "#ifdef USE_CHAN1_TEXTURE",
                // get surface normal and then perturb it using normal from normal map
                // read the normal map channels depending on the texture format (uv coordinates are not rescaled for bump maps)
                "vec3 pN = perturb_normal(normalize(vNormal), eyeVector, texcoordVarying);",
              "#else",
                "#define NO_BUMP",
              "#endif",
            "#else",
              "#define NO_BUMP",
            "#endif",

            "#ifdef NO_BUMP",
                "vec3 pN = normalize(vNormal);",
            "#endif",

           // do the irradiance lookup using the perturbed normal
           "#ifdef USE_IRRMAP",
              "if(isFloorReflection) pN.y = -pN.y;",
              "pN = rotate_vec_from_quat(envMapRotQuat, pN);",
              "vec4 irradiance = textureCube(irradianceCubeMap, pN);",
           "#else",
              "vec4 irradiance = diffColour;",
           "#endif",

           "float shadowFactor;",
           "if(shadowsShadingContrast > 0.3)",
             "shadowFactor = 1.0 + ((shadowsShadingContrast - 0.3) / 0.7);", // this gives a shadowFactor of 2.0 at the 100% setting
           "else ",
             "shadowFactor = shadowsShadingContrast / 0.3;", // this is to match traditional Visualizer behaviour at shadowsShadingContrast = 0.3, which is the default;

           "gl_FragColor = clamp(shadowFactor * irradiance + (1.0 - shadowFactor), 0.0, 1.0) * diffColour.a;",
           "gl_FragColor.a = diffColour.a;",
        "}"

   ].join("\n")
    }

    return IrradianceShaderPM;
});