
/**
 * @author SVN & DOGO (Original by RAVO)
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
    "WV3Dmod/shaders/VertexShaderPM",
    "WV3Dmod/shaders/ShaderUtils"
    ],
    function(THREE,
         VertexShaderPM,
         ShaderUtils)
    {
      /**
       * @alias module:WebViewer3D
       * @class Filter Shader for Printable Materials
       */

    var _uniforms =
    {
      "diffuseColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
      "specularColor" : { type: "c", value: new THREE.Color( 0xffffff ) },
      "shininess" : { type: "f", value: 0.5 },

      "diffuseTexture" : { type: "t", value: null },
      "specularTexture" : { type: "t", value: null },
      "internalSpecTex" : { type: "t", value: null },
      "normalMap_chan_0" : { type: "t", value: null },
      "normalMap_chan_1" : { type: "t", value: null },

      "isFloorReflection": { type: "i", value: 0 },
      "haze1CubeMap" : { type: "t", value: null },
      "haze2CubeMap" : { type: "t", value: null },
      "envMapRotQuat": { type: "v4", value: new THREE.Vector4(0, 0, 0, 1) },

      "canvas_size": { type: "v2", value: new THREE.Vector2(1, 1) },
      "DX1RT" : { type: "t", value: null },
      "DX2RT" : { type: "t", value: null },
      "NBRTSRC" : { type: "i", value: 0 }
    };

    var EXT_uniforms =
    {
      "haze3CubeMap" : { type: "t", value: null },
      "haze4CubeMap" : { type: "t", value: null },
      "haze5CubeMap" : { type: "t", value: null }
    };

    var fragmentShader_part1 =
    [
      "varying vec2 texcoordVarying;",
      "varying vec2 texcoordVaryingScaled;",
      "varying vec3 vNormal;",

      "varying vec4 vPosition;",
      "varying vec4 vWorldPosition;",

      "uniform vec3 diffuseColor;",
      "uniform vec3 specularColor;",
      "uniform float shininess;",

      "uniform sampler2D diffuseTexture;",
      "uniform sampler2D specularTexture;",
      "uniform sampler2D internalSpecTex;",
      "uniform sampler2D normalMap_chan_0;",
      "uniform sampler2D normalMap_chan_1;",

      "uniform vec4 envMapRotQuat;",

      "#ifdef USE_ENVMAP",
        "uniform bool isFloorReflection;",
        "uniform samplerCube haze1CubeMap;",
        "uniform samplerCube haze2CubeMap;",
    ].join("\n");

    var fragmentShader_ext1 =
    [
        "uniform samplerCube haze3CubeMap;",
        "uniform samplerCube haze4CubeMap;",
        "uniform samplerCube haze5CubeMap;"
    ].join("\n");

    var fragmentShader_part2 =
    [
      "#endif",

       // work out the contribution from each reflection/environment map based on the haze value
      "vec3 resolveHaze(float haze, vec3 reflSamples[5])",
      "{",
         "float haze4 = 4.0 * haze;",
         "vec3 result = reflSamples[0] * (1.0 - clamp(abs(0.0 - haze4), 0.0, 1.0));",
         "result +=     reflSamples[1] * (1.0 - clamp(abs(1.0 - haze4), 0.0, 1.0));",
         "result +=     reflSamples[2] * (1.0 - clamp(abs(2.0 - haze4), 0.0, 1.0));",
         "result +=     reflSamples[3] * (1.0 - clamp(abs(3.0 - haze4), 0.0, 1.0));",
         "result +=     reflSamples[4] * (1.0 - clamp(abs(4.0 - haze4), 0.0, 1.0));"
    ].join("\n");

    fragmentShader_part2Alt =
    [
      "#endif",

       // work out the contribution from each reflection/environment map based on the haze value
      "vec3 resolveHaze(float haze, vec3 reflSamples[5])",
      "{",
         "vec3 result = reflSamples[0] * (1.0 - clamp(abs(0.0 - haze), 0.0, 1.0));",
         "result +=     reflSamples[1] * (1.0 - clamp(abs(1.0 - haze), 0.0, 1.0));"
    ].join("\n");

    var fragmentShader_part3 =
    [
         "return result;",
      "}",

      "mat3 cotangent_frame(vec3 N, vec3 p, vec2 uv)",
      "{",
          // get edge vectors of the pixel triangle
          "vec3 dp1 = dFdx( p );",
          "vec3 dp2 = dFdy( p );",
          "vec2 duv1 = dFdx( uv );",
          "vec2 duv2 = dFdy( uv );",

          // solve the linear system
          "vec3 dp2perp = cross( dp2, N );",
          "vec3 dp1perp = cross( N, dp1 );",
          "vec3 T = normalize(dp2perp * duv1.x + dp1perp * duv2.x);",
          "vec3 B = normalize(dp2perp * duv1.y + dp1perp * duv2.y);",

          // construct a scale-invariant frame
          "float invmax = inversesqrt( max( dot(T,T), dot(B,B) ) );",
          "return mat3( T * invmax, B * invmax, N );",
      "}",

      "vec3 perturb_normal( vec3 N, vec3 V, vec2 texcoord )",
      "{",
          // assume N, the interpolated vertex normal and
          // V, the view vector (vertex to eye)
          //(uv coordinates are not rescaled for bump maps)
          "float chan_0 = texture2D(normalMap_chan_0, texcoord).r;",
          "float chan_1 = texture2D(normalMap_chan_1, texcoord).r;",

          "vec3 map = vec3(chan_0, chan_1, 0.);",

          //Rescale map (from 0 to 1 into -1 to 1)
          "map = map * 255./127. - 128./127.;",
          // decode bump map
          "vec2 tmp;",
          "if(map.x >= 0.0) tmp.x = -1.0; else tmp.x = 1.0;",
          "if(map.y >= 0.0) tmp.y = -1.0; else tmp.y = 1.0;",
          "map = abs(map) * 9.96722625883599;",
          "map = exp2(map);",
          "map = (map * 0.001) - 0.001;",
          "map = map * vec3(tmp,0.);",
          "map.z = sqrt( 1. - dot( map.xy, map.xy ) );",

          "mat3 TBN = cotangent_frame(N, -V, texcoord);",
          "return normalize(TBN * map);",
      "}",

      ShaderUtils.get('DeclareDepthSamplersChunck'),
      ShaderUtils.get('EncodeDepthChunck'),
      ShaderUtils.get('DecodeDepthChunck'),
      ShaderUtils.get('EnvironmentRotationChunck'),

      "void main()",
      "{",
          ShaderUtils.get('SelectPeelChunck'),

          "vec4 specularCol = vec4(specularColor, 1.0 - shininess);",
          "vec4 internalSpecCol = vec4(0.0, 0.0, 0.0, 1.0 - shininess);",

          "#ifdef USE_SPECULARMAP",
           "specularCol = texture2D(specularTexture, texcoordVaryingScaled);",
            "#ifdef REMAPPED_SPECULAR_ALPHA_CHANNEL",
             //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices)
             //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
              "specularCol.a = clamp((specularCol.a - 127./255.) * 255. / 127., 0.,1.);",
            "#endif",
          "#endif",

          "#ifdef USE_INT_SPEC_TEXTURE",
           "internalSpecCol = texture2D(internalSpecTex, texcoordVaryingScaled);",
            "#ifdef REMAPPED_INT_SPECULAR_ALPHA_CHANNEL",
             //remapping due to WebGL IE11 desktop, Safari & Chrome (mobile devices)
             //premultiplying and dividing rgb by alpha when uploading rgba texture to GPU
              "internalSpecCol.a = clamp((internalSpecCol.a - 127./255.) * 255. / 127., 0.,1.);",
            "#endif",
          "#endif",

          "vec3 eyeVector = normalize(vec3(vWorldPosition) - cameraPosition.xyz);",

          "#ifdef USE_CHAN0_TEXTURE",
            "#ifdef USE_CHAN1_TEXTURE",
              // get surface normal and then perturb it using normal from normal map
              // read the normal map channels depending on the texture format (uv coordinates are not rescaled for bump maps)
              "vec3 pN = perturb_normal(normalize(vNormal), eyeVector, texcoordVarying);",
            "#else",
              "#define NO_BUMP",
            "#endif",
          "#else",
            "#define NO_BUMP",
          "#endif",

          "#ifdef NO_BUMP",
              "vec3 pN = normalize(vNormal);",
          "#endif",

          "#ifdef USE_ENVMAP",
            "vec3 reflection = reflect(eyeVector, pN);",
            "if(isFloorReflection)",
            "{",
              "reflection.y = -reflection.y;",
            "}",

            "reflection = rotate_vec_from_quat(envMapRotQuat, reflection);",

            // Get reflection map samples
            "vec3 reflSamples[5], reflResult;",
            "reflSamples[0] = textureCube(haze1CubeMap, reflection).rgb;",
            "reflSamples[1] = textureCube(haze2CubeMap, reflection).rgb;",
    ].join("\n");

    var fragmentShader_ext3 =
    [
            "reflSamples[2] = textureCube(haze3CubeMap, reflection).rgb;",
            "reflSamples[3] = textureCube(haze4CubeMap, reflection).rgb;",
            "reflSamples[4] = textureCube(haze5CubeMap, reflection).rgb;"
    ].join("\n");

    var fragmentShader_part4 =
    [
            // calculate reflections using haze and reflection coefficients
            "reflResult = specularCol.rgb * resolveHaze(specularCol.w, reflSamples);",
            "reflResult += internalSpecCol.rgb * resolveHaze(internalSpecCol.w, reflSamples);",
          "#else",
            "vec3 reflResult = specularCol.rgb;",
          "#endif",

          "gl_FragColor.rgb = reflResult;",
          "gl_FragColor.a = 0.0;",
      "}"
    ].join("\n");

    var ReflectanceShaderPM = {

          vertexShader: VertexShaderPM.vertexShader,

          //On some mobile devices, textures units are limited
          //In this case, we downgrade quality of environment

          //Mobile/Compatibility version
          uniforms: THREE.UniformsUtils.merge( [
             VertexShaderPM.uniforms,
             _uniforms
          ] ),

          fragmentShader: [
            fragmentShader_part1,
            fragmentShader_part2Alt,
            fragmentShader_part3,
            fragmentShader_part4
          ].join("\n"),

          //Desktop/Full version
          EXTuniforms: THREE.UniformsUtils.merge( [
             VertexShaderPM.uniforms,
             _uniforms,
             EXT_uniforms
          ] ),

          EXTfragmentShader: [
            fragmentShader_part1,
            fragmentShader_ext1,
            fragmentShader_part2,
            fragmentShader_part3,
            fragmentShader_ext3,
            fragmentShader_part4
          ].join("\n")
    }

    return ReflectanceShaderPM;

});