
/**
 * @author DOGO
 * @module WebViewer3D
 * @copyright   2015 Esko Stonecube Ltd. All rights reserved.
 */

define(["WV3Dlib/three",
        "backbone",
        "WV3Dmod/SystemChecker",
        "WV3Dmod/LogMessage"
        ],
    function(THREE,
             bb,
             WV3DSystemChecker,
             LogMessage)
    {
      /**
       * @alias module:WebViewer3D
       * @class ShaderUtils gather all shader functions that can be reused in other shaders
       * Warning : this class is a singleton, any modification to one of its component will change for each reference to it.
       */

    var FullEncodeDepthChunck = [
      "vec4 EncodeFloatRGBA( float v ) {",
        "float remainder = v;",
        "vec4 res, r;",

        "res[0] = floor(remainder * 255.0) / 255.0;",
        "remainder -= res[0];",
        "res[1] = floor(remainder * 65535.0) / 65535.0;",
        "remainder -= res[1];",
        "res[2] = floor(remainder * 16777215.0) / 16777215.0;",

        "r[0] = clamp(floor((v) * 255.0),0.,255.);",
        "r[1] = clamp(floor((v - res[0]) * 65535.0),0.,255.);",
        "r[2] = clamp(floor((v - res[0] - res[1]) * 16777215.0),0.,255.);",
        "r[3] = clamp(floor((v - res[0] - res[1] - res[2]) * 4294967295.0),0.,255.);",

        "r[0] = r[0] / 255.0;",
        "r[1] = r[1] / 255.0;",
        "r[2] = r[2] / 255.0;",
        "r[3] = r[3] / 255.0;",
        "return r;",
      "}",
    ].join("\n");

    var FullDecodeDepthChunck = [
      "float DecodeFloatRGBA( vec4 rgba ) {",
        "return dot( rgba, vec4(1.0, 1./255., 1./65535., 1./16777215.) );",
      "}",
    ].join("\n");

    var NoEncodeDepthChunck = [
      "vec4 EncodeFloatRGBA( float v ) {",
        "return vec4(v, 0., 0., 0.);",
      "}",
    ].join("\n");

    var NoDecodeDepthChunck = [
      "float DecodeFloatRGBA( vec4 rgba ) {",
         "return rgba.r;",
      "}",
    ].join("\n");

     var Ipad2EncodeDepthChunck = [
      "vec4 EncodeFloatRGBA( float v ) {",
        "float remainder = v;",
        "vec4 res, r;",

        "res[0] = floor(remainder * 127.0) / 127.0;",
        "remainder -= res[0];",
        "res[1] = floor(remainder * 16383.0) / 16383.0;",
        "remainder -= res[1];",
        "res[2] = floor(remainder * 2097151.0) / 2097151.0;",

        "r[0] = clamp(floor((v) * 127.0),0.,127.);",
        "r[1] = clamp(floor((v - res[0]) * 16383.0),0.,127.);",
        "r[2] = clamp(floor((v - res[0] - res[1]) * 2097151.0),0.,127.);",

        "r.r = r[0] / 127.0;",
        "r.g = r[1] / 127.0;",
        "r.b = r[2] / 127.0;",
        "r.a = 1.;",
        "return r;",
      "}",
    ].join("\n");

    var Ipad2DecodeDepthChunck = [
      "float DecodeFloatRGBA( vec4 rgba ) {",
         "return rgba.r + rgba.g/127. + rgba.b/16383.;",
      "}",
    ].join("\n");

    var Simulate32BitEncodingChunck = [
      "depth = floor(depth * 255.) * 0.003921568627450980392156862745098;"
    ];

    var EncodeDepthChunck;
    var DecodeDepthChunck;
    var SimulateEncodingChunckD0, SimulateEncodingChunckD1, SimulateEncodingChunckD2;

    LogMessage.Log("Configuration set up for:")
    if(WV3DSystemChecker.get('_fullSupportFloatingPointTextures'))//Desktop platforms
    {
      if(WV3DSystemChecker.IsWindows())//Windows
      {
        LogMessage.Log("Windows");
        if(WV3DSystemChecker.IsIE())//tested ok
        {
          LogMessage.Log("IE");
          SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = "";
          EncodeDepthChunck = NoEncodeDepthChunck;
          DecodeDepthChunck = NoDecodeDepthChunck;
        }
        else //tested ok -- back of some printable parts (clearspeed) cannot be seen however
        {
          LogMessage.Log("Not IE");
          SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = Simulate32BitEncodingChunck;
          EncodeDepthChunck = FullEncodeDepthChunck;
          DecodeDepthChunck = FullDecodeDepthChunck;
        }
      }
      else if(WV3DSystemChecker.IsMac())//Mac -- tested on 10.10.4 -- back of some printable parts (clearspeed) cannot be seen however
      {
        LogMessage.Log("Mac");
        SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = "";
        EncodeDepthChunck = NoEncodeDepthChunck;
        DecodeDepthChunck = NoDecodeDepthChunck;
      }else
      {
        LogMessage.Log("Unsupported OS (for now): "+WV3DSystemChecker.get("_ua").os.family+" / "+WV3DSystemChecker.get("_ua").browser.family);
        SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = Simulate32BitEncodingChunck;
        EncodeDepthChunck = FullEncodeDepthChunck;
        DecodeDepthChunck = FullDecodeDepthChunck;
      }
    }
    else//All Mobile device ?
    {
      if (WV3DSystemChecker.IsAndroid())//Android -- to test and ajust relatively to the GPU (needs to be tested on some android devices)
      {
        LogMessage.Log("Android");
        SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = "";
        EncodeDepthChunck = Ipad2EncodeDepthChunck;
        DecodeDepthChunck = Ipad2DecodeDepthChunck;
      }else if(WV3DSystemChecker.IsiOSFirstGen())//Ipad 2 -- tested (still a few z-fighting issues on back of some printable parts, e.g. speed clear)
      {
        LogMessage.Log("iOS First Generation");
        SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = Simulate32BitEncodingChunck;
        EncodeDepthChunck = Ipad2EncodeDepthChunck;
        DecodeDepthChunck = Ipad2DecodeDepthChunck;
      }else //iPhone -- to test -- iPod -- tested ok (minor z-fighting issue on back of labels)
      {
        LogMessage.Log("Default Mobile Device");
        SimulateEncodingChunckD0 = SimulateEncodingChunckD1 = SimulateEncodingChunckD2 = "";
        EncodeDepthChunck = Ipad2EncodeDepthChunck;
        DecodeDepthChunck = Ipad2DecodeDepthChunck;
        //todo, try FullencodeB here and for IE and mac ?
      }
    }

    var ShaderUtils = bb.Model.extend({
        defaults:
        {

          EncodeDepthChunck: EncodeDepthChunck,
          DecodeDepthChunck: DecodeDepthChunck,
          SimulateEncodingChunckD2: SimulateEncodingChunckD2,
          SimulateEncodingChunckD1: SimulateEncodingChunckD1,
          SimulateEncodingChunckD0: SimulateEncodingChunckD0,

          DeclareDepthSamplerChunck:
          [
            "#ifdef GL_FRAGMENT_PRECISION_HIGH",
              "uniform highp sampler2D D0RT;",
            "#else",
              "uniform mediump sampler2D D0RT;",
            "#endif",
            "uniform vec2 canvas_size;",
          ].join("\n"),

          DeclareDepthSamplersChunck:
          [
            "uniform vec2 canvas_size;",
            "uniform int NBRTSRC;",
            "#ifdef GL_FRAGMENT_PRECISION_HIGH",
                "uniform highp sampler2D DX1RT;",
                "uniform highp sampler2D DX2RT;",
            "#else",
                "uniform mediump sampler2D DX1RT;",
                "uniform mediump sampler2D DX2RT;",
            "#endif"
          ].join("\n"),

          SelectPeelChunck:
          [
            "vec2 loc = vec2(gl_FragCoord.x/canvas_size.x, gl_FragCoord.y/canvas_size.y);",
            "vec4 dx1_rgba = texture2D(DX1RT, loc);",
            "vec4 dx2_rgba = texture2D(DX2RT, loc);",//Even if unused, needs to stay out of control flow
            "if(canvas_size.x > 0. && NBRTSRC > 0)",
            "{",
              "float z = DecodeFloatRGBA(EncodeFloatRGBA(1.0 - gl_FragCoord.z));",

              "if (NBRTSRC == 2)",
              "{",
                  "float zmax = DecodeFloatRGBA(dx2_rgba);",

                  "if (z >= zmax)",
                  "{",
                    "discard;",
                  "} ",
                  "float zmin = DecodeFloatRGBA(dx1_rgba);",

                  "if (z <= zmin)",
                  "{",
                    "discard;",
                  "} ",
              "}else if (NBRTSRC == 1){",
                  "float zmin = DecodeFloatRGBA(dx1_rgba);",

                  "if (z <= zmin)",
                  "{",
                    "discard;",
                  "} ",
              "}",
            "}",
          ].join("\n"),

          ReflectionFadeChunck:
          [
            "if(reflection_fade)",
            "{",
            //Calculate start and end of fading according to reflection_fade_size
            //reflection_fade_size 1. --> sharp line
            //reflection_fade_size 2. --> fade takes half the height of the screen
            //get the distance of loc.y to the start of the fade (loc.y - startfade)
            //interpolate linearly

              "float inv_position = 1. - reflection_fade_position;",
              "float offset = (reflection_fade_size - 1.) * 0.1;",
              "float fade_start = inv_position + offset;",
              "float fade_end = inv_position - offset;",
              "float factor = (fade_start - loc.y) / (fade_end - fade_start) + 1.;",

              "if (loc.y > fade_start) refopacity *= 1.;",//keep it as it is
              "else if (loc.y < fade_end) refopacity *= 0.;",//Beyond expected fade limit
              "else refopacity *= factor ;",//Faded using a linear function
            "}"
          ].join("\n"),

          ReflectionCosineFadeChunck:
          [
            "if(reflection_fade)",
            "{",
            //Calculate start and end of fading according to reflection_fade_size
            //reflection_fade_size 1. --> sharp line
            //reflection_fade_size 2. --> fade takes half the height of the screen
            //get the distance of loc.y to the start of the fade (loc.y - startfade)
            //divide by the length (startfade-endfade) to get a value between 0 and PI
            //Then calculate the opacity according to a cosine function

              "float inv_position = 1. - reflection_fade_position;",
              "float offset = (reflection_fade_size - 1.) * 0.1;",
              "float fade_start = inv_position + offset;",
              "float fade_end = inv_position - offset;",
              "float factor = (fade_start - loc.y) / (fade_start - fade_end) * 3.1415926535;",

              "if (loc.y > fade_start) refopacity *= 1.;",//keep it as it is
              "else if (loc.y < fade_end) refopacity *= 0.;",//Beyond expected fade limit
              "else refopacity *= cos(factor)*0.5 + 0.5 ;",//Faded using a cosine function
            "}"
          ].join("\n"),

          EnvironmentRotationChunck:
          [
            "vec3 rotate_vec_from_quat(vec4 envMapRotQuat, vec3 vector)",
            "{",
              //fast quaternion multiplication. source: https://molecularmusings.wordpress.com/2013/05/24/a-faster-quaternion-vector-multiplication/
              "vec3 tempVec = 2.0 * cross(envMapRotQuat.xyz, vector);",
              "vector = vector + envMapRotQuat.w * tempVec + cross(envMapRotQuat.xyz, tempVec);",
              "return vector;",
            "}",
          ].join("\n")

        }

    });

    return new ShaderUtils;//New is what makes it a singleton, see usage in shaders

});


