define([
		"utils/webcss!./wfeditor-style.css",
		"WFEditor/core/error/Debug",
		"WFEditor/core/message/MessageController",
		"component/errorhandler/ErrorHandler",
		"WFEditor/scene/Scene",
		"WFEditor/scene/SceneController",
		"WFEditor/core/undoredo/WFEditorUndoRedoController",
		"WFEditor/scene/SceneModel",
		"WFEditor/toolbar/ToolbarModel",
		"WFEditor/contextMenu/ContextMenuModel",
		"WFEditor/tools/ToolManager",
		"WFEditor/tools/GalleryNodeSelectorTool",
		"Mousetrap",
		"utils/Observable",
		"utils/Observer",
		"WFEditor/core/WFEditorContext",
		"component/loadingindicator/LoadingIndicatorSingleton",
		"jquery",
		"jqueryUI"
	],
	function (
		css,
		Debug,
		MessageController,
		ErrorHandler,
		Scene,
		SceneController,
		WFEditorUndoRedoController,
		SceneModel,
		ToolbarModel,
		ContextMenuModel,
		ToolManager,
		GalleryNodeSelectorTool,
		Mousetrap,
		Observable,
		Observer,
		WFEditorContext,
		LoadingIndicator,
		$
	) {
		"use strict";

		/**
		 * Defines the WFEditor core of the Workflow framework. Use this to initialize a workflow.
		 * Specify specific factories to create objects that overwrite framework provided objects.
		 *
		 * Once the Workflow model is initialized, load it by calling setWorkflow
		 * @class
		 * @exports WFEditor
		 * @mixes Observable
		 * @mixes Observer
		 * @param {JQueryObj}               context             - the context in which the Workflow needs to be rendered
		 * @param {AbstractWorkflowFactory} workflowFactory     - factory used to create the workflow model, must implement AbstractWorkflowFactory
		 * @param {AbstractViewFactory}     workflowViewFactory - factory used to create the view objects for the workflow
		 * @param {I18n}                    i18n                - internationalization used in the framework
		 * @param {ErrorHandler}			[errorHandler]		- ErrorHandler to be used in the wf
		 */
		var WFEditor = function (context, workflowFactory, workflowViewFactory, nodeGallery, i18n, errorHandler) {
			Observable.mixin(this, "WFEditor");
			Observer.mixin(this);

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._context = context;

			/**
			 * @member {AbstractWorkflowFactory} implementation of the factory that generates the workflow model.
			 * @private
			 */
			this._workflowFactory = workflowFactory;

			/**
			 * @member {AbstractWorkflowViewFactory} factory that generates the specific view elements
			 * @private
			 */
			this._workflowViewFactory = workflowViewFactory;

			/**
			 * @member {Workflow} the active workflow
			 * @private
			 */
			this._workflow = null;

			/**
			 * @member {ToolbarModel} model used by the toolbar
			 * @private
			 */
			this._toolbarModel = null;

			/**
			 * @member {ToolManager} main controller for all the interactive tools that use the scene
			 * @private
			 */
			this._toolManager = null;

			/**
			 * @member {I18n} Internationalization used in the framework
			 * @private
			 */
			this._i18n = i18n;

			/**
			 * @member {WFEditorUndoRedoController}
			 * @protected
			 */
			this._undoRedo = null;

			/**
			 * Init Code
			 */
			this._context.addClass("wfeditor");

			/**
			 * define elements
			 */
			this._rightPanel = undefined;
			this._centerPanel = $("<div/>").addClass("centerPanel");

			/**
			 * Hold a reference to the loading indicator element
			 */
			this._loadingIndicator = LoadingIndicator.element;

			// Loading indicator
			this._centerPanel.append(LoadingIndicator.element.addClass("right"));
			$.ajaxSetup({
				beforeSend: _.bind(function (jqXHR, settings) {
					jqXHR["_wfEditorLoadKey"] = settings.url;
					LoadingIndicator.start(settings.url);

				}, this),
				complete: _.bind(function (jqXHR, textStatus) {
					var url = jqXHR["_wfEditorLoadKey"];
					LoadingIndicator.stop(url);
				}, this),
				timeout: 60000
			}); // 1 min


			// Node gallery
			if (nodeGallery) {
				this._nodeGallery = nodeGallery;
				this._rightPanel = $("<div>").addClass("rightPanel");
				// Resizable
				this._rightPanel.resizable({
					handles: "w",
					minWidth: 150
				});

				this._nodeGallery.setContext(this._rightPanel);
				this._context.append(this._rightPanel);

				this._rightPanel.on('contextmenu', function (e) {
					e.preventDefault();
					e.stopPropagation();
				});
			}

			// The node gallery is added first (float right), then the center panel
			this._context.append(this._centerPanel);

			/**
			 * @member  {JQueryObj} context for the scene to render itself in.
			 * @private
			 */
			this._sceneDiv = $("<div>").addClass("scene");
			//append scene to the context.
			this._centerPanel.append(this._sceneDiv);


			/**
			 * @member {Scene} Scene view
			 * @private
			 */
			this._scene = null;

			/**
			 * @member {WFEditorContext}
			 * @private
			 */
			this._wfEditorContext = new WFEditorContext();

			this._wfEditorContext.set("errorHandler", errorHandler || new ErrorHandler());
			this._wfEditorContext.set("i18n", i18n);
			this._wfEditorContext.set("workflowFactory", workflowFactory);

			/**
			 * @member {JqueryObj} element that contains all the scene controls
			 * @private
			 */
			this._sceneControls = $("<div>").addClass("sceneControls disableTextSelection");

			this._centerPanel.append(this._sceneControls);

			// Init debugger
			this._debug = new Debug(this._wfEditorContext.get("errorHandler"));

			// Init message module
			this._messagebar = $("<div />").addClass("messagebar");
			this._sceneDiv.append(this._messagebar);
			this._messageController = new MessageController(this._messagebar, this._wfEditorContext.get("errorHandler"));

			// add fullscreen changed events
			this._isInFullScreen = window.document.fullscreenElement ||
				window.document.mozFullScreenElement ||
				window.document.webkitFullscreenElement ||
				window.document.msFullscreenElement;
			$(window.document).on('fullscreenchange', _.bind(this._handleFullScreenEvent, this));
			$(window.document).on('mozfullscreenchange', _.bind(this._handleFullScreenEvent, this));
			$(window.document).on('webkitfullscreenchange', _.bind(this._handleFullScreenEvent, this));
			$(window.document).on('MSFullscreenChange', _.bind(this._handleFullScreenEvent, this));

			/**
			 * @member {ShapeUtils.Rect} When toggling fullscreen we wish to have the same view als before 
			 * @private
			 */
			this._visibleRectBeforeFullScreenToggle = null;

		};

		/**
		 * Initializes the specific scene controller for the workflow editor. If no specific controller is
		 * needed, the default SceneController is used. the specific controller needs to extend
		 * SceneController.
		 *
		 * @param {AbstractWorkflowFactory} workflowFactory
		 * @param {SceneModel} sceneModel
		 * @param {Workflow} model
		 * @param {Scene} scene
		 * @param {WFEditorContext} WFEditorContext
		 * @returns {SceneController}
		 */
		WFEditor.prototype._initializeSceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			return new SceneController(workflowFactory, sceneModel, workflowModel, scene, WFEditorContext);
		};

		/**
		 * Initialize the WFEditorUndoRedoController
		 * @param {ToolbarModel} toolbarModel
		 * @returns {WFEditorUndoRedoController}
		 */
		WFEditor.prototype._initializeUndoRedoController = function (wFEditorContext) {
			return new WFEditorUndoRedoController(wFEditorContext);
		};

		/**
		 * Set the workflow that should be displayed. This will initialize a Scene and Scene controller
		 * @param {Workflow} model
		 * @param {Function} [whenWorkflowLoaded] - fired when all items are correctly set in the view
		 */
		WFEditor.prototype.setWorkflow = function (workflow, whenWorkflowLoaded) {
			this._stopObserveAll();
			this._wfEditorContext.set("workflow", workflow);
			var sceneModel = new SceneModel(workflow.getID(), this._wfEditorContext);
			this._sceneModel = sceneModel;
			//create a model for the toolbar
			this._toolbarModel = new ToolbarModel();
			//create a model for the contextMenu;
			this._contextMenuModel = new ContextMenuModel();
			//initialize the toolmanager
			this._toolManager = new ToolManager(sceneModel);

			this._workflow = workflow;
			this._wfEditorContext.set("toolManager", this._toolManager);
			this._wfEditorContext.set("contextMenuModel", this._contextMenuModel);
			this._wfEditorContext.set("toolbarModel", this._toolbarModel);
			this._wfEditorContext.set("sceneModel", this._sceneModel);

			//model for the workflow editor is created
			var scene = this._initializeScene(this._sceneDiv, sceneModel, workflow, this._workflowViewFactory, this._wfEditorContext);
			this._scene = scene;
			this._wfEditorContext.set("scene", this._scene);
			var keyboardHandler = new Mousetrap();

			// The WFEditor should only handle keyboard events if the the wf is in focus
			keyboardHandler.bind = function (shortcuts, fn, action) {
				Mousetrap.prototype.bind.call(this, shortcuts, function () {
					if (scene.hasFocus()) {
						fn.apply(this, arguments);
					}
				}, action);
			};
			this._wfEditorContext.set("keyboardHandler", keyboardHandler);



			// create undo redo controller
			this._undoRedo = this._initializeUndoRedoController(this._wfEditorContext);
			this._wfEditorContext.set("undoRedo", this._undoRedo);

			//create a view to display all the toolbar buttons, controller is constructed by the SceneController
			this._toolbar = this._initializeToolbar(this._toolbarModel, this._sceneControls, this._context);


			//initialize the controller for the workflow editor with the model and the view
			this._sceneController = this._initializeSceneController(this._workflowFactory, sceneModel, workflow, this._scene, this._wfEditorContext);
			this._wfEditorContext.set("sceneController", this._sceneController);
			if (this._nodeGallery) {
				this._nodeGallery.setNodeDefinitions(this._workflowFactory.getNodeDefinitions());

				// initialize the GalleryNodeSelectorTool
				new GalleryNodeSelectorTool(this._nodeGallery, this._sceneController, this._wfEditorContext);

			}

			if (whenWorkflowLoaded) {
				this._scene.whenWorkflowLoaded(whenWorkflowLoaded);
			}

			// Propagate events that could be useful for interacting components
			this._observe(this._scene, "workflowLoaded", function () {
				this._emit("workflowLoaded");
			});
			this._observe(this._scene, "focus", function () {
				this._emit("focus");
			});
			this._observe(this._scene, "blur", function () {
				this._emit("blur");
			});
			this._observe(this._scene, "requestNewWorkflow", this.loadNewWorkflow);

		};

		/**
		 * Request a new workflow to be loaded
		 * @param {String} id
		 */
		WFEditor.prototype.loadNewWorkflow = function (id) {
			//to be implemented by specific wf editor
		};

		/**
		 * Initialize the toolbar. Needs to be implemented by specific wf editor.
		 * @param {ToolbarModel} toolbarModel
		 * @param {JQueryObj} context
		 * @param {JQueryObj} globalContext - the context in which the toolbar has been added
		 */
		WFEditor.prototype._initializeToolbar = function (toolbarModel, context, globalContext) {
			//to be implemented by specific wf editor	
		};


		/**
		 * Initialize the Scene for the workflow editor.
		 * @param {JQueryObj}           context       	 - the context in which the workflow scene is drawn
		 * @param {SceneModel}	        sceneModel	  	 - the model of the scene
		 * @param {Workflow}            workflow  	 - model of the workflow which contains all the workflow items
		 * @param {AbstractViewFactory} workflowViewFactory   	 - factory used to generate all the view objects
		 * @param {WFEditorContext}		WFEditorContext
		 * @returns {Scene}
		 */
		WFEditor.prototype._initializeScene = function (context, sceneModel, workflow, workflowViewFactory, WFEditorContext) {
			return new Scene(context, sceneModel, workflow, workflowViewFactory, WFEditorContext);
		};

		/**
		 * Toggle Full Screen Mode
		 */
		WFEditor.prototype._toggleFullScreen = function () {
			this._visibleRectBeforeFullScreenToggle = this._scene.getVisibleRect().calculateOverlap(this._scene.getBoundingRect());
			if (this._isInFullScreen) {
				this._requestExitFullScreen();
			} else {
				this._requestFullScreen();
			}
		};

		/**
		 * Handle the fullscreen change event
		 * Checks whether or not we are in full screen and adapts workflow to it
		 */
		WFEditor.prototype._handleFullScreenEvent = function () {
			var fsElem = window.document.fullscreenElement || window.document.mozFullScreenElement || window.document.webkitFullscreenElement || window.document.msFullscreenElement;
			if (fsElem) {
				this._enterFullScreen();
			} else {
				this._exitFullScreen();
			}
			if (this._visibleRectBeforeFullScreenToggle) {
				this._sceneController.zoomToRect(this._visibleRectBeforeFullScreenToggle, 1);
			}
		};

		/**
		 * Request to enter Full Screen Mode
		 */
		WFEditor.prototype._requestFullScreen = function () {
			var isFullscreenAvailable = window.document.fullscreenEnabled || window.document.mozFullScreenEnabled || window.document.webkitFullscreenEnabled || window.document.msFullscreenEnabled;
			if (isFullscreenAvailable) {
				var body = window.document.body;
				if (body.requestFullscreen) {
					body.requestFullscreen();
				} else if (body.mozRequestFullScreen) {
					body.mozRequestFullScreen();
				} else if (body.webkitRequestFullscreen) {
					body.webkitRequestFullscreen();
				} else if (body.msRequestFullscreen) {
					body.msRequestFullscreen();
				}
			} else {
				this._enterFullScreen();
			}
		};

		/**
		 * Request to exit Full Screen Mode
		 */
		WFEditor.prototype._requestExitFullScreen = function () {
			var isFullscreenAvailable = window.document.fullscreenEnabled || window.document.mozFullScreenEnabled || window.document.webkitFullscreenEnabled || window.document.msFullscreenEnabled;
			this._context.removeClass("fullscreen");

			if (isFullscreenAvailable) {
				if (document.exitFullscreen) {
					document.exitFullscreen();
				} else if (document.mozCancelFullScreen) {
					document.mozCancelFullScreen();
				} else if (document.webkitExitFullscreen) {
					document.webkitExitFullscreen();
				} else if (document.msExitFullscreen) {
					document.msExitFullscreen();
				}
			} else {
				this._exitFullScreen();
			}
		};

		/**
		 * Enter Full Screen Mode (handle resizing)
		 */
		WFEditor.prototype._enterFullScreen = function () {
			this._isInFullScreen = true;
			this._context.addClass("fullscreen");
			$("body").addClass("wf-editor-is-fullscreen");
			this._emit("fullscreen.enter");
			this.resized();
		};

		/**
		 * Exit Full Screen Mode (handle resizing)
		 */
		WFEditor.prototype._exitFullScreen = function () {
			this._context.removeClass("fullscreen");
			$("body").removeClass("wf-editor-is-fullscreen");
			this._emit("fullscreen.exit");
			this._isInFullScreen = false;
			this.resized();
		};

		/**
		 * Let the WFEditor know that it has been resized
		 */
		WFEditor.prototype.resized = function () {
			if (this._scene) {
				this._scene.resized();
			}
		};

		/**
		 * Destroy the WFEditor
		 */
		WFEditor.prototype.destroy = function () {
			if (this._scene) {
				this._scene.destroy();
			}
			this._context.remove();
		}

		/**
		 * Sets the visible rect of the WF
		 * @param {ShapeUtils.Rect}
		 */
		WFEditor.prototype.setVisibleRect = function (rect) {
			this._sceneController.zoomToRect(rect, 1);
		};

		/** 
		 * @return {ShapeUtils.Rect} the visible rect of the WFEditor
		 */
		WFEditor.prototype.getVisibleRect = function (includeInsets) {
			if (this._scene) {
				return this._scene.getVisibleRect(includeInsets);
			}
		};

		/**
		 * focuses the WF
		 */
		WFEditor.prototype.focus = function () {
			if (this._scene) {
				this._scene.focus();
			}
		};

		/**
		 * Alas, some elements in parameter panels might not work correctly when removing and readding the workflow from/to the dom
		 * This method can be called from outside the WFEditor to let the custom parameter panels know they are being removed from the dom
		 * They should implement the beforeRemoveFromDom and afterAddToDom methods to handle these events correctly
		 */
		WFEditor.prototype.notifyRemoveFromDom = function () {
			if (this._scene) {
				this._scene.triggerRemoveFromDomEvent();
			}
		};

		/**
		 * Alas, some elements in parameter panels might not work correctly when removing and readding the workflow from/to the dom
		 * This method can be called from outside the WFEditor to let the custom parameter panels know they are being (re)-added to the dom
		 * They should implement the beforeRemoveFromDom and afterAddToDom methods to handle these events correctly	
		 */
		WFEditor.prototype.notifyAddToDom = function () {
			if (this._scene) {
				this._scene.triggerAddToDomEvent();
			}
		};

		return WFEditor;

	});