define(["WFEditor/core/WFEditor",
		"WFEditor/workflow/ModelConstants",
		"WFEditorWCR/data/WebCenterAPI",
		"WFEditorWCR/core/WebCenterWFEditorConstants",
		"WFEditorWCR/workflow/WebCenterWorkflowState",
		"WFEditorWCR/workflow/WebCenterWorkflowFactory",
		"WFEditorWCR/scene/WebCenterViewFactory",
		"WFEditorWCR/nodeGallery/WebCenterNodeGallery",
		"WFEditorWCR/scene/WebCenterSceneController",
		"WFEditorWCR/toolbar/WebCenterToolbar",
		"WFEditorWCR/scene/WebCenterScene",
		"WFEditorWCR/workflow/node/WebCenterNodeConstants",
		"WFEditorWCR/actionPopup/ImportWorkflowPopup",
		"WFEditorWCR/actionPopup/ExportWorkflowPopup",
		"WFEditorWCR/actionPopup/ResetToManualPopup",
		"WFEditorWCR/actionPopup/CancelWorkflowPopup",
		"component/errorhandler/ErrorHandler",
		"component/loadingindicator/LoadingIndicatorSingleton",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"utils/webcss!./wfeditorwcr-style.css",
		"jquery",
		"underscore"
	],
	function (WFEditor,
		ModelConstants,
		WebCenterAPI,
		WebCenterWFEditorConstants,
		WebCenterWorkflowState,
		WebCenterWorkflowFactory,
		WebCenterViewFactory,
		WebCenterNodeGallery,
		WebCenterSceneController,
		WebCenterToolbar,
		WebCenterScene,
		WebCenterNodeConstants,
		ImportWorkflowPopup,
		ExportWorkflowPopup,
		ResetToManualPopup,
		CancelWorkflowPopup,
		ErrorHandler,
		LoadingIndicator,
		Button,
		i18n,
		css,
		$,
		_
	) {
		"use strict";

		/**
		 * WFEditorWCR is an instance of the WebCenter HTML5 Workflow editor. To launch an instance of
		 * the WFEditor, use the constructor or use the data-init attribute in the context HTML-element
		 * on the page. WFEditor will initialize itself in this context. It needs the attribute tasktypeid
		 * set in the context-element to be able to start.
		 *
		 * @exports WFEditorWCR
		 * @class
		 * @augments {WFEditor}
		 *
		 * @param {HTMLObject} context    a reference to the div in which the workflow editor should be created
		 * @param {Map<String, Any>} privileges - The privileges for this workflow editor
		 * @param {String}     taskId - The id of the task to load in the Workflow Editor
		 * @param {String}     taskTypeId - The id of the task type to load in the Workflow Editor
		 * @param {String} [initialTaskStatus] - Initial Task Status ID of the workflow (needed when no report can be fetched)
		 * @param {String} [taskname] - Task name
		 */
		var WFEditorWCR = function (context, privileges, taskId, taskTypeId, initialTaskStatus, taskName) {
			/**
			 * @member {Map<String, Any>} privileges for the workflow
			 * @private
			 */
			this._privileges = privileges;

			/**
			 * @member {String} id of the task or task type to be loaded.
			 * @private
			 */
			this._taskId = taskId;

			/**
			 * @member {String} id of the task type to be loaded.
			 * @private
			 */
			this._taskTypeId = taskTypeId;

			/**
			 * @member {String | undefined} name of the task to be loaded.
			 * @private
			 */
			this._taskName = taskName;

			/**
			 * @member {String} id of the task status at the moment the WFEditor was launched
			 * @private
			 */
			this._initialTaskStatus = initialTaskStatus;

			/**
			 * @member {$.Deferred} xml of the definition of workflows in WebCenter
			 * @private
			 */
			this._workflowDefinitionPromise = null;

			/**
			 * @member {$.Deferred} xml of the definition of task types in WebCenter
			 * @private
			 */
			this._tasktypeDefinitionPromise = null;

			/**
			 * @member {ErrorHandler} error handler used in the wf
			 * @private
			 */
			var errorHandler = new ErrorHandler();

			/**
			 * @member {Boolean} is a save action already ongoing?
			 * @private
			 */
			this._saveOngoing = false;

			/**
			 * @member {String} config object with information of webcenter, loaction of the images, jsppages,...
			 * @private
			 */
			this._wcrConfig = window.webcenterConfig;

			// Add client browser information to wcrConfig
			this._wcrConfig.userAgent = typeof window.navigator !== "undefined" && typeof window.navigator.userAgent !== "undefined" ? window.navigator.userAgent : "";

			/**
			 * @member {WebCenterAPI} communication with WebCenter
			 */
			this._webCenterAPI = new WebCenterAPI(this._wcrConfig, errorHandler);

			//we create an instance of the WebCenterWorkflowFactory
			var workflowFactory = new WebCenterWorkflowFactory(this._webCenterAPI, errorHandler);

			//create an instance of the view factory
			var workflowViewFactory = new WebCenterViewFactory(errorHandler);

			var nodeGallery = null;
			if (this._privileges[ModelConstants.Privileges.addNodes]) {
				nodeGallery = new WebCenterNodeGallery();
			}

			// Empty the context
			$(context).empty();

			// we call the constructor of the framework worflow editor
			WFEditor.call(this, context, workflowFactory, workflowViewFactory, nodeGallery, i18n, errorHandler);

			// the WFEditor core is not initalized start loading the workflow (and feedback if available)
			this._loadAndInitalizeWorkflow();


			//>>excludeStart("PExclude", pragmas.productionExclude);
			// Make Development a bit more enjoyable... export WFEditorin non-optimized mode
			window.gWFEditor = this;
			//>>excludeEnd("PExclude");

			this._boundOnBeforeUnload = _.bind(this._onBeforeUnload, this);
			$(window).on("beforeunload", this._boundOnBeforeUnload);

			this._webCenterAPI.keepSessionAlive(true);
		};

		WFEditorWCR.prototype = Object.create(WFEditor.prototype);
		WFEditorWCR.prototype.constructor = WFEditorWCR;

		/**
		 * Initialize the the WFEditor with the data-init method of initialize static components
		 * @param {HTMLObject} context
		 */
		WFEditorWCR.wcrinit = function (context) {
			var tasktypeid = context.getAttribute("tasktypeid");
			var taskid = context.getAttribute("taskid");
			var initialTaskStatus = context.getAttribute("initialTaskStatus");
			var taskname = context.getAttribute("taskname");
			var privileges = {
				hasTaskEditRights: context.getAttribute("canedit") == "1" || context.getAttribute("canedit") == "true", // change task permissions in feedback mode or can edit in editor mode
				isMasterTask: context.getAttribute("mastertaskid") ? context.getAttribute("mastertaskid").length === 0 : true, // is a mastertask or not
				autorefresh: context.getAttribute("autorefresh") == "1" // should feedback be autorefreshed
			};

			var editableMode = !taskid && privileges.hasTaskEditRights; // editor mode if taskid is not defined and editable if canedit = 1

			privileges[ModelConstants.Privileges.zoomToActiveNode] = !!taskid; // only in feedback mode
			privileges[ModelConstants.Privileges.addEdges] = editableMode;
			privileges[ModelConstants.Privileges.removeEdges] = editableMode;
			privileges[ModelConstants.Privileges.moveEdges] = editableMode;
			privileges[ModelConstants.Privileges.canSave] = editableMode;
			privileges[ModelConstants.Privileges.addNodes] = editableMode;
			privileges[ModelConstants.Privileges.moveNodes] = true; // always allow nodes to be moved
			privileges[ModelConstants.Privileges.removeNodes] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.editLabels] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.editStickyNotes] = editableMode;
			privileges[WebCenterNodeConstants.Privileges.showWorkflowControls] = !!taskid && privileges.hasTaskEditRights;
			privileges[ModelConstants.Privileges.viewNodeConfiguration] = true;
			privileges[ModelConstants.Privileges.editNodeConfiguration] = editableMode;
			privileges[ModelConstants.Privileges.copyNodes] = editableMode;


			return new WFEditorWCR($(context), privileges, taskid, tasktypeid, initialTaskStatus, taskname);
		};

		/**
		 * Initializes the specific WebCenter controller for the workflow editor.
		 * @override
		 */
		WFEditorWCR.prototype._initializeSceneController = function (workflowFactory, sceneModel, workflowModel, scene, WFEditorContext) {
			return new WebCenterSceneController(workflowFactory, sceneModel, workflowModel, scene, WFEditorContext);
		};


		/**
		 * Fetch XML information from server and display the workflow
		 */
		WFEditorWCR.prototype._loadAndInitalizeWorkflow = function () {
			LoadingIndicator.start("loadAndInitializeWF", i18n.t("loadingWorkflowDefinition"));

			var workflowXMLPromise;

			if (this._taskId) {
				workflowXMLPromise = this._webCenterAPI.getTaskWorkflowXML(this._taskId);
			} else {
				workflowXMLPromise = this._webCenterAPI.getTaskTypeWorkflowXML(this._taskTypeId);
			}

			var workflowDefinitionPromise = this._webCenterAPI.getWorkflowDefinitions();
			var taskTypeDefinitionPromise = this._webCenterAPI.getTaskTypeDefinitions();
			var currentTaskTypeDefinition = this._webCenterAPI.getTaskTypeDefinition(this._taskTypeId);
			var workflowReportPromise = this._webCenterAPI.getWorkflowReport(this._taskId);

			// fetch WorkflowXML, WorkflowDefinitions and TaskTypeDefinitions in parallel
			$.when(workflowXMLPromise, workflowDefinitionPromise, taskTypeDefinitionPromise, currentTaskTypeDefinition)
				.done(_.bind(function (workflowXML, workflowDefinitionXML, tasktypeDefinitionXML, currentTaskTypeDefinitionXML) {
					var workflowPromise = this._workflowFactory.createWorkflow(this._taskTypeId,
						this._taskId,
						this._privileges,
						workflowXML[0],
						workflowDefinitionXML[0],
						tasktypeDefinitionXML[0],
						currentTaskTypeDefinitionXML[0],
						this._initialTaskStatus);
					workflowPromise.done(_.bind(function (workflow) {


						//>>excludeStart("PExclude", pragmas.productionExclude);
						// Make Development a bit more enjoyable... export workflow to window in non-optimized mode
						window.gWorkflow = workflow;
						//>>excludeEnd("PExclude");

						this.setWorkflow(workflow, _.bind(function () {
							// if there is a workflowReport, wait till it is loaded and start the feedbacktimer if needed
							if (workflowReportPromise) {
								$.when(workflowReportPromise, currentTaskTypeDefinition).done(_.bind(function (report, currentTaskDefinition) {
									this._workflowFactory.createWorkflowFeedback(report[0], currentTaskDefinition[0], workflow);

									// start feedback fetcher if needed
									this._startFeedbackTimer();
									//update the toolbar model with the created workflow feedback
									this._updateToolbarModel();
									// highlight active nodes (will only be visible in highlight mode)
									this._sceneController.resetHighlights();
								}, this));
							}
							this._wfEditorContext.get("keyboardHandler").bind("d e b u g enter", function () {
								workflow.toggleDebugMode();
							}, "keyup");

							LoadingIndicator.stop("loadAndInitializeWF");
						}, this));
					}, this));
				}, this));
		};

		/**
		 * If autorefresh is turned on, feedback should be fetched and added to the model at regular moments
		 * This method will start the feedbacktimer is autorefresh privilege is turned on
		 */
		WFEditorWCR.prototype._startFeedbackTimer = function () {
			if (!this._interval) {
				// Feedback should auto refresh, start feedback timer
				this._interval = setInterval(_.bind(function () {
					if (this._workflow && this._workflow.getPrivilege("autorefresh")) {
						this._reloadWorkflowFeedback();
					} else {
						clearInterval(this._interval);
						this._interval = null;
					}
				}, this), WebCenterWFEditorConstants.FeedbackUpdateInterval);
			}
		};

		/**
		 * Reload the feedback of the workflow and update the workflow model.
		 * @private
		 */
		WFEditorWCR.prototype._reloadWorkflowFeedback = function () {
			if (this._workflow) {
				var workflowReportPromise = this._webCenterAPI.getWorkflowReport(this._workflow.getTaskID());
				var currentTaskTypeDefinition = this._webCenterAPI.getTaskTypeDefinition(this._workflow.getTaskTypeID());

				$.when(workflowReportPromise, currentTaskTypeDefinition).done(_.bind(function (report, currentTaskDefinition) {
					// update feedback
					this._workflowFactory.createWorkflowFeedback(report[0], currentTaskDefinition[0], this._workflow);
					//update the toolbarmodel with the updated workflow feedback
					this._updateToolbarModel();
					// highlight active nodes (will only be visible in highlight mode)
					this._sceneController.resetHighlights();
				}, this));
			}
		};

		/**
		 * Reloads the feedback each interval ms until callback returns true
		 * @param {Number} interval   the interval between feedback calls
		 * @param {Function} callback The function to call to see if the reload of the workflow should stop
		 */
		WFEditorWCR.prototype._reloadWorkflowFeedbackUntil = function (interval, callback) {
			this._reloadWorkflowFeedback();
			var intervalId = setInterval(_.bind(function () {
				if (!callback()) {
					this._reloadWorkflowFeedback();
				} else {
					clearInterval(intervalId);
				}
			}, this), interval);
		};



		/**
		 * Request a new workflow to be loaded
		 * @param {String} id
		 */
		WFEditorWCR.prototype.loadNewWorkflow = function (id) {
			var currentUrl = document.location.href;
			var split = currentUrl.split("?");
			var mainUrl = split[0];
			var params = split[1];
			var newParams = [];
			_.each(params.split("&"), function (param) {
				var keyValue = param.split("=");
				if (keyValue[0] === "curProjTaskID") {
					newParams.push("curProjTaskID=" + id);
				} else {
					newParams.push(param);
				}
			}, this);

			document.location.href = mainUrl + "?" + newParams.join("&");
		};

		/**
		 * Initialize the toolbar for the workflow editor. Specific WebCenter implementation.
		 * @param   {ToolbarModel} toolbarModel
		 * @param   {JQueryObj} context
		 * @returns {WebCenterToolbar}
		 */
		WFEditorWCR.prototype._initializeToolbar = function (toolbarModel, context, globalContext) {
			var toolbar = new WebCenterToolbar(toolbarModel, context, globalContext);

			if (this._workflow && this._workflow.getPrivilege(WebCenterNodeConstants.Privileges.showWorkflowControls)) {
				toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowControlToolbar, "workflowStateActions");
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, "icon-WCR_WF-Hold2", _.bind(this._holdWorkflow, this), i18n.t("HoldWorkflowButtonTooltipText"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, "icon-WCR_WF-Resume2", _.bind(this._resumeWorkflow, this), i18n.t("ResumeWorkflowButtonTooltipText"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowControlToolbar, WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, "icon-WCR_WF-Cancel2", _.bind(this._cancelWorkflow, this), i18n.t("CancelWorkflowButtonTooltipText"));
			}

			toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowWFModesToolbar, "viewTitle");

			// add save button to the model
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave)) {

				// Bind ctrl+s | command+s keys to saving the workflow
				this._wfEditorContext.get("keyboardHandler").bind("mod+s", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					if (this._undoRedo.isWorkflowModified() && !this._saveOngoing) {
						this._saveCurrentWorkflow();
					}
				}, this));

				toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowActionsToolbar, "workflowTitle");
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowSaveToolbarButton, "icon-WCR_Save", _.bind(this._saveCurrentWorkflow, this), i18n.t("workflowSaveTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowImportToolbarButton, "icon-WCR_WF-Import", _.bind(this._importWorkflow, this), i18n.t("workflowImportTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowExportToolbarButton, "icon-WCR_WF-Export", _.bind(this._exportWorkflow, this), i18n.t("workflowExportTooltip"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowActionsToolbar, WebCenterWFEditorConstants.WorkflowClearToolbarButton, "icon-WCR_WF-Clear", _.bind(this._clearWorkflow, this), i18n.t("workflowClearTooltip"));

				this._undoRedo.setWorkflowModifiedCallback(_.bind(this._updateSaveButton, this));
			} else if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.zoomToActiveNode)) {
				this._highlightIcon = $("<i/>").addClass("icon containerIcon icon-normal icon-grey icon-WCR_WF-Show-Active-Node-cont").append(
					$("<i/>").addClass("icon subIcon icon-normal icon-default icon-WCR_WF-Show-Active-Node-sub"));
				toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowWFModesToolbar, WebCenterWFEditorConstants.WorkflowWFModesToolbar, "", _.bind(this._toggleHighlightNodes, this), i18n.t("workflowToggleHighlightTooltipOn"), {
					content: this._highlightIcon
				});
			}
			if (this._taskName) {
				toolbarModel.addComponent(WebCenterWFEditorConstants.TaskNameToolbarComponent, "taskName", this._taskName);
			}
			toolbarModel.addButton(WebCenterWFEditorConstants.WorkflowWFModesToolbar, WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, !this._isInFullScreen ? "icon-WCR_FullScreen" : "icon-WCR_NormalScreen", _.bind(this._toggleFullScreen, this), !this._isInFullScreen ? i18n.t("workflowToggleFullScreenTooltipOn") : i18n.t("workflowToggleFullScreenTooltipOff"));


			this._updateSaveButton();

			return toolbar;
		};

		/**
		 * Update the disabled state of the save button
		 * the button is disabled while a save is ongoing, or the workflow is not modified since the last save
		 */
		WFEditorWCR.prototype._updateSaveButton = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowSaveToolbarButton, this._undoRedo.isWorkflowModified() && !this._webCenterAPI.isSaveOngoing());
		};



		/**
		 * Update the toolbarmodel.
		 */
		WFEditorWCR.prototype._updateToolbarModel = function () {
			var workflowState = this._workflow.getWorkflowState();
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, workflowState !== WebCenterWorkflowState.STOPPING && workflowState === WebCenterWorkflowState.RUNNING);
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, workflowState !== WebCenterWorkflowState.STOPPING && workflowState === WebCenterWorkflowState.PAUSED);
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, workflowState !== WebCenterWorkflowState.STOPPING && (workflowState === WebCenterWorkflowState.RUNNING || workflowState === WebCenterWorkflowState.PAUSED));

			var workflowStatus = this._workflow.getTaskStatus();
			if (workflowStatus) {
				if (!this._toolbarModel.hasComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar)) {
					this._toolbarModel.addComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar, "workflowStateActions", workflowStatus);
				} else {
					this._toolbarModel.updateComponent(WebCenterWFEditorConstants.WorkflowStatusToolbar, workflowStatus);
				}

			}
		};

		/**
		 * @override
		 */
		WFEditorWCR.prototype._initializeScene = function (scene, sceneModel, workflow, workflowViewFactory, wfEditorContext) {
			return new WebCenterScene(scene, sceneModel, workflow, workflowViewFactory, this._webCenterAPI, wfEditorContext);
		};

		/**
		 * Save the current workflow
		 */
		WFEditorWCR.prototype._saveCurrentWorkflow = function () {
			// Check if this is a "default workflow"
			if (_.size(this._workflow.getEdges()) === 0) {
				// no edges, so check nodes
				var startNode = null;
				var completedNode = null;
				var errorContinueNode = null;
				if (_.every(this._workflow.getNodes(), function (node) {
						if (node.getNodeType() === WebCenterNodeConstants.Nodes.START) {
							if (startNode === null) {
								// Only 1 start node
								startNode = node;
								return true;
							} else {
								return false;
							}
						} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.COMPLETED) {
							if (completedNode === null) {
								// Only 1 completed node
								completedNode = node;
								return true;
							} else {
								return false;
							}
						} else if (node.getNodeType() == WebCenterNodeConstants.Nodes.ERRORCONTINUE) {
							if (errorContinueNode === null) {
								// Only 1 error continue node
								errorContinueNode = node;
								return true;
							} else {
								return false;
							}
						}
					})) {
					// we have a case of default workflow
					new ResetToManualPopup(this).open();
					return;
				}
			}

			// Normal save
			this.doSaveCurrentWorkflow();
		};

		/**
		 * Do save the current workflow
		 * @return {$.Deferred.<>} Promise resolved when save is finished.
		 */
		WFEditorWCR.prototype.doSaveCurrentWorkflow = function () {
			var deferred = $.Deferred();
			this._webCenterAPI.saveWorkflow(this._workflow).done(_.bind(function () {
				// workflow saved succesfully
				this._messageController.sendSuccessMessage(this._i18n.t("workflowSaveSuccessful"));
				this._undoRedo.addSaveMarker();
			}, this)).always(_.bind(function () {
				// save is finsihed or might have failed
				this._updateSaveButton();
				deferred.resolve();
			}, this));

			this._updateSaveButton();
			return deferred.promise();
		};

		/**
		 * Hold the workflow, this pauses the workflow. Only possible if the workflow is running.
		 */
		WFEditorWCR.prototype._holdWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.HOLD, false); //first disable the hold button to avoid double clicks

			this._webCenterAPI.holdWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("holdWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedback();
				}, this));
		};

		/**
		 * Resume the workflow, only possible if the workflow is paused
		 */
		WFEditorWCR.prototype._resumeWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.RESUME, false);

			this._webCenterAPI.resumeWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("resumeWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedback();
				}, this));
		};

		/**
		 * Cancel the workflow, only possible if the workflow is paused or in progress.
		 */
		WFEditorWCR.prototype.doCancelWorkflow = function () {
			this._toolbarModel.setButtonEnabled(WebCenterWFEditorConstants.WorkflowControlActionButtons.CANCEL, false);

			this._updateToolbarModel();

			this._webCenterAPI.cancelWorkflow(this._workflow)
				.done(_.bind(function () {
					this._messageController.sendSuccessMessage(i18n.t("cancelWorkflowSuccess"));
				}, this))
				.always(_.bind(function () {
					this._reloadWorkflowFeedbackUntil(2000, _.bind(function () {
						return this._workflow.getWorkflowState() === WebCenterWorkflowState.FINISHED;
					}, this));
				}, this));
		};

		/**
		 * opens a popup to cancel the workflow
		 */
		WFEditorWCR.prototype._cancelWorkflow = function () {
			new CancelWorkflowPopup(this).open();
		};


		/**
		 * Import a new workflow
		 */
		WFEditorWCR.prototype._importWorkflow = function () {
			new ImportWorkflowPopup(this).open();
		};

		/**
		 * Do the import of the workflow
		 */
		WFEditorWCR.prototype.doImportWorkflow = function () {
			$(window).off("beforeunload", this._boundOnBeforeUnload); // we don't want to see this 
			document.wfimportform.submit();
		};

		/**
		 * Export the current workflow
		 */
		WFEditorWCR.prototype._exportWorkflow = function () {
			if (this._undoRedo.isWorkflowModified() && !this._saveOngoing) {
				new ExportWorkflowPopup(this).open();
			} else {
				this.doExportWorkflow();
			}
		};

		/**
		 * Do export workflow
		 */
		WFEditorWCR.prototype.doExportWorkflow = function () {
			// Check if there is a workflow first
			this._webCenterAPI.getTaskTypeWorkflowXML(this.getTaskTypeID()).done(_.bind(function (xml) {
				var root = xml.getElementsByTagName("root")[0];
				if (root && root.childNodes.length === 0) {
					// still a default workflow
					if (this._wfEditorContext.get("errorHandler")) {
						this._wfEditorContext.get("errorHandler").warning(i18n.t("workflowExportDefaultError"));
					}
				} else {
					// Download as file
					var url = "DownloadWorkflowXML.jsp?returnasfile=1&tasktypeid=" + this.getTaskTypeID();
					window.location.href = url;
				}
			}, this));
		};

		/**
		 * Clear the current workflow
		 */
		WFEditorWCR.prototype._clearWorkflow = function () {
			if (this._sceneController && this._workflow) {
				var startNode = null;
				var completedNode = null;
				var errorContinueNode = null;

				var nodesToDelete = _.filter(this._workflow.getNodes(), function (node) {
					var toDelete = true;
					if (node.getNodeType() === WebCenterNodeConstants.Nodes.START) {
						// Keep 1 start node
						if (startNode == null) {
							startNode = node;
							toDelete = false;
						}
					} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.COMPLETED) {
						// Keep 1 completed node
						if (completedNode == null) {
							completedNode = node;
							toDelete = false;
						}
					} else if (node.getNodeType() === WebCenterNodeConstants.Nodes.ERRORCONTINUE) {
						// Keep 1 error continue node
						if (errorContinueNode == null) {
							errorContinueNode = node;
							toDelete = false;
						}
					}
					return toDelete;
				});

				// Remove them undoable
				if (_.size(nodesToDelete) > 0) {
					this._sceneController.removeNodes(this._workflow, nodesToDelete);
				}
			}
		};

		/**
		 * Resets a workflow back to a manual task
		 * @returns {$.Deferred} The promise resolved when clear is done
		 */
		WFEditorWCR.prototype.doResetToManualTask = function () {
			$(window).off("beforeunload", this._boundOnBeforeUnload); // we don't want to see this
			var deferred = $.Deferred();
			$.when(this._webCenterAPI.clearWorkflow(this.getTaskTypeID())).done(_.bind(function () {
				var context = this._context;
				context.empty();
				WFEditorWCR.wcrinit(context[0]);
			}, this)).always(function () {
				deferred.resolve();
			});
			return deferred.promise();
		};

		/**
		 * Enter Full Screen Mode (handle resizing)
		 * @override
		 */
		WFEditorWCR.prototype._enterFullScreen = function () {
			// make sure wcr menu doesn't pop pup over the fullscreen container
			$(".wcr .mainHeader, .wcr .footer").addClass('forcehide');

			WFEditor.prototype._enterFullScreen.call(this);

			// update toolbar
			if (this._toolbarModel) {
				this._toolbarModel.setButtonIcon(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, "icon-WCR_NormalScreen");
			}
		};

		/**
		 * Exit Full Screen Mode (handle resizing)
		 * @override
		 */
		WFEditorWCR.prototype._exitFullScreen = function () {
			// recover menu
			$(".wcr .mainHeader, .wcr .footer").removeClass('forcehide');

			WFEditor.prototype._exitFullScreen.call(this);

			// update toolbar
			if (this._toolbarModel) {
				this._toolbarModel.setButtonIcon(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, "icon-WCR_FullScreen");
			}
		};

		/**
		 * Get the current task type id
		 * @returns {String}
		 */
		WFEditorWCR.prototype.getTaskTypeID = function () {
			return this._taskTypeId;
		};

		/**
		 * Called before unloading of the page
		 */
		WFEditorWCR.prototype._onBeforeUnload = function () {
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave) && this._undoRedo.isWorkflowModified()) {
				return i18n.t("workflowSaveWarning");
			}
		};

		/**
		 * Highlight active nodes or remove highlight
		 * @param {Boolean} [force] - can force the editor in or out of highlight mode without toggling
		 */
		WFEditorWCR.prototype._toggleHighlightNodes = function (force) {
			if (force === true || (force !== false && this._scene.getSceneModel().isInHighlightMode())) {
				this._scene.getSceneModel().setHighlightMode(false);
				this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowWFModesToolbar, i18n.t("workflowToggleHighlightTooltipOn"));
				if (this._highlightIcon && this._highlightIcon.hasClass("icon-default")) {
					this._highlightIcon.removeClass("icon-default");
					this._highlightIcon.addClass("icon-grey");
				}
			} else {
				this._scene.getSceneModel().setHighlightMode(true);
				this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowWFModesToolbar, i18n.t("workflowToggleHighlightTooltipOff"));
				if (this._highlightIcon && this._highlightIcon.hasClass("icon-grey")) {
					this._highlightIcon.removeClass("icon-grey");
					this._highlightIcon.addClass("icon-default");
				}
			}
		};

		/**
		 * @override
		 */
		WFEditorWCR.prototype._handleFullScreenEvent = function () {
			WFEditor.prototype._handleFullScreenEvent.call(this);
			this._toolbarModel.setButtonTooltip(WebCenterWFEditorConstants.WorkflowFullScreenToolbarButton, !this._isInFullScreen ? i18n.t("workflowToggleFullScreenTooltipOn") : i18n.t("workflowToggleFullScreenTooltipOff"));

		};


		/**
		 * check if the WFEditor can be closed 
		 * @return {Boolean}
		 */
		WFEditorWCR.prototype.canClose = function () {
			if (this._workflow && this._workflow.getPrivilege(ModelConstants.Privileges.canSave) && this._undoRedo && this._undoRedo.isWorkflowModified()) {
				return window.confirm(i18n.t("workflowSaveWarning"));
			}
			return true;
		};

		return WFEditorWCR;
	});