define(["approval/ApprovalConstants",
		"approval/model/ApprovalModel",
		"approval/model/ApproveRejectViewModel",
		"approval/controller/SortableController",
		"approval/view/ApprovalView",
		"utils/webi18n!Approval",
		"jquery",
		"underscore",
		// dependencies below return no objects
		"wcr/hover",
		"wcr/date",
		"wcr/dateShift",
		"wcr/datetools",
		"wcr/webcenter_ajax"
	],
	function (ApprovalConstants,
		ApprovalModel,
		ApproveRejectViewModel,
		SortableController,
		ApprovalView,
		i18n,
		$,
		_) {

		"use strict";

		/**
		 * This is the entry point towards having an Approval setup page
		 * @param {string} displayId                   element used to render the approval
		 * @param {string} readURL                     url used to read information about approval
		 * @param {string} actionURL                   url used to store information
		 * @param {string} getProjectFolderSetupURL    url used to read project/folder approval setups
		 * @param {object} currentUser                 contains information about the logged in user
		 * @param {boolean} nextVersion                true if setup for next version is enabled
		 * @param {boolean} stagedApprovalAllowed      true if the license is available that allows to add stages
		 * @param {boolean} isViewFile                 true if file is viewable
		 * @param {boolean} isEncapsulated             true if the page should not reload when saving approval information
		 * @param {function} afterInitializationCallback callback that is called when approval has finished initializing
		 * @param {function} onUpdateCallback            callback tht is called whenever a client side change happens to the approval setup
		 */
		function Approval(displayId, readURL, actionURL, getProjectFolderSetupURL, currentUser, nextVersion, stagedApprovalAllowed, isViewFile, isEncapsulated, afterInitializationCallback, onUpdateCallback) {
			this.afterInitializationCallback = afterInitializationCallback || function () {};
			// get data 
			this.model = new ApprovalModel(readURL, nextVersion, actionURL, currentUser, stagedApprovalAllowed, getProjectFolderSetupURL, isViewFile);

			//>>excludeStart("PExclude", pragmas.productionExclude);
			window.model = this.model;
			//>>excludeEnd("PExclude");	

			var approveRejectViewModel = new ApproveRejectViewModel(actionURL, isEncapsulated, isViewFile, onUpdateCallback);

			// create view
			this.view = new ApprovalView(this.model, approveRejectViewModel, $("#" + displayId), currentUser, (readURL.indexOf("simplePage") !== -1));

			this._sortableController = new SortableController(this.view, this.model);
			this._sortableController.disable();
			this.model.on("initialized", function () {
				if (stagedApprovalAllowed &&
					(this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED ||
						this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED)) {
					this._sortableController.enable();
				} else {
					this._sortableController.disable();
				}
			}.bind(this));
			
			this.model.once("initialized", _.partial(this.afterInitializationCallback, this));

			if (!isEncapsulated) {
				window.onbeforeunload = _.bind(this.confirmExit, this); // confirmExit is the code called when somebody tries to get away from the page
				this.model.confirmNeeded = true; // by default, attempts to leave the page will throw a confirm box
			}
		}

		/** 
		 * Is called when navigating away from the page
		 * You don't always have to ask confirmation.  It's not needed when
		 * 1. The approval rule has not changed
		 * 2. The user exits the page using one of the action buttons (Save, Start Approval Cycle, etc)
		 */
		Approval.prototype.confirmExit = function () {
			if (this.model.confirmNeeded && this.model.approvalRuleChanged) {
				return i18n.t('CAppConfirmPageExit');
			}
		};

		/**
		 * Starts a very simple approval setup page with a single stage
		 */
		Approval.prototype.setupSingleStageApproval = function () {
			if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
				this.model.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_STARTED);
				this.model.addNewStage(0);
			}
		};

		return Approval;
	}
);
