define(["approval/ApprovalConstants",
		"approval/ApprovalUtils",
		"approval/model/ApprovalNotifications",
		"approval/model/ApprovalMember",
		"approval/model/ApprovalStage",
		"approval/stageFactory/SimpleApprovalStageFactory",
		"approval/stageFactory/ComplexApprovalStageFactory",
		"utils/Observable",
		"utils/Observer",
		"utils/dateShift",
		"wcr/webcenter_html",
		"wcr/webcenter_xml",
		"underscore"
	],
	function (ApprovalConstants,
		ApprovalUtils,
		ApprovalNotifications,
		ApprovalMember,
		ApprovalStage,
		SimpleApprovalStageFactory,
		ComplexApprovalStageFactory,
		Observable,
		Observer,
		dateShift,
		WCHTML,
		WCXML,
		_) {
		"use strict";

		/**
		 * The general data model for approval
		 * @param {string} 	readURL
		 * @param {boolean} nextVersion
		 * @param {string} 	actionURL
		 * @param {object}	currentUser
		 * @param {boolean} stagedApprovalAllowed
		 * @param {string} 	getProjectFolderSetupURL
		 */
		function ApprovalModel(readURL, nextVersion, actionURL, currentUser, stagedApprovalAllowed, getProjectFolderSetupURL, isViewFile) {
			this._id = _.uniqueId(); // An id to allow using one event bus for all stages

			Observable.mixin(this, ["Approval", this._id], "Approval");
			Observer.mixin(this);

			this.on("stage.*.changed.approvers.added", this._onStageApproverAdded.bind(this));

			this.onAll(["changed.*", "stage.*.changed.**"], function () {
				this.approvalRuleChanged = true;
			}.bind(this));

			/** 
			 * @member {Array.<ApprovalStage>} list op approvals
			 * @private
			 */
			this.steps = []; // The stage models
			this.actionURL = actionURL;
			this.currentUser = currentUser;
			this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_DEFINED;
			this.stagedApprovalAllowed = stagedApprovalAllowed;
			this.isViewFile = isViewFile;

			this._stageFactory = stagedApprovalAllowed ? new ComplexApprovalStageFactory(this) : new SimpleApprovalStageFactory(this);

			this.dueDateType = ApprovalConstants.DueDateType.NO; // default new approval setups to no due dates
			this.approvalMasterType = ApprovalConstants.ApprovalMasterType.USER;
			this.approvalMaster = this.currentUser.userId;
			this.approvalRuleChanged = false; // remembers whether somebody changed approval rule without saving.
			this.passwordValidation = false; // see function
			this.hasProjectApprovalSetup = false; // initialization, will be set by reading from xml (ajax return)
			this.hasFolderApprovalSetup = false;
			this.hasNextVersionSetup = false; // remembers whether there is a next version setup
			this.nextVersion = !!nextVersion;

			this.readOnly = !(currentUser && currentUser.canChangeSetup === true); // readOnly means that the user cannot change the approval setup (this is a right)

			this.canSubmitApprovalStatus = true;
			this.documentVersion = 1;
			this.uponNewVersion = 'InheritApproval';
			this.submitAllowed = true; //unit testing can switch off submission for easier local debugging
			this.submitDisabled = false; //if true submitting is not possible.  This is set to prevent 2 concurrent actions.  Not the same as submitAllowed which is purely for unit testing
			this.getProjectFolderSetupURL = getProjectFolderSetupURL;
			this.viewerApprovalCommentType = "Prompt";
			this.readURL = readURL;

			this.isMultiDocSetup = false;

			// in normal circumstances, you always get a readURL
			if (readURL) {
				// calls the back-end for an xml with the current approval setup
				this.readFromXML(this.readURL);
			} else {
				// blank approval model
				this._emit("update");
			}
		}

		/**
		 * Registers an error
		 * @param {object} errorObj:
		 *                  - errorMsg: general message tag
		 *                  - errors: Array.<object>
		 *                  	- tag {string} : error message
		 *                   	- parameters {Array.<string>}: parameters that have to given to the localize method
		 *                    	- inline {boolean} : if true, the tag is shown without translating
		 */
		ApprovalModel.prototype.setError = function (errorObj) {
			this.errorObj = errorObj;
			this._emit("error", errorObj);
		};

		/**
		 * The client-side ID of this ApprovalModel. This ID is not related with any backend ID's
		 * @returns {string}
		 */
		ApprovalModel.prototype.getId = function () {
			return this._id;
		};

		/**
		 * Callback that is fired when a new approver is added to a stage. If this approver is a new person than this callback will add him to the list of project members.
		 * @private
		 * @param   {Approver}   approver the newly added approver
		 */
		ApprovalModel.prototype._onStageApproverAdded = function (approver) {
			var memberObj = approver.memberObj;
			var isAdded = ApprovalUtils.findMemberObj(approver.memberObj.memberType, approver.memberObj.userId || approver.memberObj.groupId, this.getProjectMembers());
			if (!isAdded) {
				this.projectMembers.push(approver.memberObj);
				this._emit("member.added");
			}
		};

		/**
		 * Calls a web service via Ajax to read approval setup information
		 * @param {string} readURL
		 * @param {boolean} isProjectFolderApproval
		 */
		ApprovalModel.prototype.readApprovalFromURL = function (readURL, isProjectFolderApproval) {
			this.canSubmitApprovalStatus = true;

			$.ajax({
					url: readURL,
					type: "GET",
					data: "xml",
				})
				.done(_.bind(function (response) {
					this.reset();
					if (isProjectFolderApproval) {
						this.readProjectFolderReturnXML(response);
					} else {
						this.readReturnXML(response);
					}

					// Display the approval
					this.approvalRuleChanged = isProjectFolderApproval;
					this._emit("initialized");
				}, this))
				.fail(_.bind(function () {
					this.reset();
					this.setError("An error occured when loading the approval setup");
				}, this));
		};



		/**
		 * Ajax onSuccess call when reading approval setup xml from the back-end
		 * @param {xml} returnxml
		 */
		ApprovalModel.prototype.readReturnXML = function (returnxml) {
			var errorElem = WCXML.GetFirstFirstLevelChild(returnxml, "error"); // look whether the very first child is the <error> tag
			if (errorElem) { // if there is an error, look for errorMessage child
				var errorMsg = WCXML.GetElementText(errorElem, "errorMessage");
				this.setError({
					errors: [{
						tag: errorMsg,
						inline: true
					}]
				});
				return;
			} else {
				this.errorMsg = null;
			}
			var approvalSetupElem = WCXML.GetFirstFirstLevelChild(returnxml, "approvalSetup");
			if (approvalSetupElem) {

				if (this.notificationInfo) {
					this._stopObserveObject(this.notificationInfo);
				}
				// read notifications
				this.notificationInfo = ApprovalNotifications.fromXML(approvalSetupElem, this);

				this._observe(this.notificationInfo, "update", _.partial(this._emit, "changed.notifications"));

				// read general information
				var generalElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "general");
				if (generalElem) {
					this.uponNewVersion = WCXML.GetElementText(generalElem, "uponNewVersion");
					this.dueDateType = WCXML.GetElementText(generalElem, "dueDateType");
					if (this.dueDateType === '') {
						this.dueDateType = ApprovalConstants.DueDateType.NO;
					}
					if (this.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
						this.finalDueDate = parseInt(WCXML.GetElementText(generalElem, "finalDueDate"));
						if (this.finalDueDate) {
							this.finalDueDate = new Date(this.finalDueDate);
						} else {
							this.finalDueDate = null;
						}
					}
					this.approvalStatus = WCXML.GetElementText(generalElem, "generalApprovalStatus");
					this.viewerApprovalCommentType = WCXML.GetElementText(generalElem, "viewerApprovalCommentType");

					// read approval master
					this.approvalMasterType = WCXML.GetElementText(generalElem, "approvalMasterType");
					this.approvalMaster = WCXML.GetElementText(generalElem, "approvalMaster");

					if (this.approvalMasterType === "") { // default -> ME
						this.approvalMasterType = ApprovalConstants.ApprovalMasterType.USER;
						this.approvalMaster = this.currentUser.userId;
					}

					this.hasNextVersionSetup = WCXML.GetElementBoolean(generalElem, "hasNextDocumentVersionApprovalSetup");
					this.passwordValidation = WCXML.GetElementBoolean(generalElem, "passwordValidation");
					this.systemCondApprovalAllowed = WCXML.GetElementBoolean(generalElem, "systemCondApprovalAllowed");
				}

				// read project information
				var projectElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "project");
				if (projectElem) {
					this.projectId = WCXML.GetElementText(projectElem, "projectId");
					this.projectName = WCXML.GetElementText(projectElem, "projectName");
					this.projectManagerUserId = WCXML.GetElementText(projectElem, "projectManagerUserId");

					//[WCR10.2] check whether approval status submission is prevented by the parent project being inactive
					var inactiveProjFlag = WCXML.GetFirstFirstLevelChild(projectElem, "CannotSubmitAppStatusProjInactive");
					if (inactiveProjFlag) {
						this.canSubmitApprovalStatus = false;
					}

					// read project members.             
					var memberArr = WCXML.GetFirstFirstLevelChild(projectElem, "projectMembersWithAccess");
					if (memberArr) {
						this.projectMembers = ApprovalMember.readMembers(memberArr, true);
					}

					//read project roles with assignment data
					var roleAssignmentMapListElem = WCXML.GetFirstFirstLevelChild(projectElem, "project_role_assignment_map_list");
					if (roleAssignmentMapListElem) {
						this.roleAssignments = []; // dumping everybody who was there already, typical for Cancel
						this.assignedRolesID = [];
						this.readRoleAssignmentData(roleAssignmentMapListElem, this.roleAssignments, this.assignedRolesID);
					}

					// read people who did actions and are not (anymore) in projectMembersWithAccess
					var noMemberArr = WCXML.GetFirstFirstLevelChild(projectElem, "usersWithNoAccess");
					if (noMemberArr) {
						this.usersWithNoAccess = ApprovalMember.readMembers(noMemberArr, false);
					}
				}
				// end reading project information

				// read folder information
				var folderElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "folder");
				if (folderElem) {
					this.folderId = WCXML.GetElementText(folderElem, "folderId");
					this.folderName = WCXML.GetElementText(folderElem, "folderName");
					// in case you are editing a folder approval setup, you need to know whether there is a project approval setup
					this.hasProjectApprovalSetup = WCXML.GetElementBoolean(folderElem, "hasProjectApprovalSetup");
				}

				//read 'Multidocument Approval Setup' context flag
				var multiDocApprovalFlagElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "isMultiDocApprovalSetup");
				if (multiDocApprovalFlagElem) {
					//set the flag to true
					this.isMultiDocSetup = true;

					//also indicate that starting app cycle is allowed (permission checks are done when running the action)
					//starting approval cycle is not allowed for Template documents
					var isTemplateFlag = WCXML.GetFirstFirstLevelChild(multiDocApprovalFlagElem, "projIsTemplate");
					if (isTemplateFlag == null && (this.currentUser.canStartApproval == null || this.currentUser.canStartApproval === true)) {
						this.currentUser.canStartApproval = true;
					}

					//check whether the parent project/folder have approval setup that can be fallen back to
					this.hasProjectApprovalSetup = WCXML.GetElementBoolean(multiDocApprovalFlagElem, "hasProjectApprovalSetup");

					//also check for folder app. setup existence
					this.hasFolderApprovalSetup = WCXML.GetElementBoolean(multiDocApprovalFlagElem, "hasFolderApprovalSetup");
				}


				// read document information
				var documentElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "document");
				if (documentElem) {
					this.documentId = WCXML.GetElementText(documentElem, "documentId");
					this.documentName = WCXML.GetElementText(documentElem, "documentName");
					this.documentVersion = WCXML.GetElementText(documentElem, "documentVersion");
					this.hasFolderApprovalSetup = WCXML.GetElementBoolean(documentElem, "hasFolderApprovalSetup");
					if (this.hasProjectApprovalSetup !== true) {
						this.hasProjectApprovalSetup = WCXML.GetElementBoolean(documentElem, "hasProjectApprovalSetup");
					}
				}

				// read stages
				this.steps = [];
				// read approval steps information
				var stagesElement = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "stages");
				if (stagesElement) {
					_.each(WCXML.GetFirstLevelChilds(stagesElement, "stage"), function (stage) {
						this.addStage(ApprovalStage.fromXML(stage, this.projectMembers, this.usersWithNoAccess, this));
					}.bind(this));
				}
				this.approvalRuleChanged = false;

			} // end if approvalSetupElem
			if (this.nextVersion) {
				// for nextVersion, there is still done a call to the back-end, only this time it loads the 
				// next version.  Normally the back-end should respond with all statuses on NotStarted
				// the general status can also be NotDefined (if there is simply nothing defined). 
				// this can be slightly contradictorial since in that case hasNextVersion should be false.
				// the code below is probably for safety only.
				if (this.hasNextVersionSetup) {
					this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
				}
				// and all step statuses and approver's statuses must be 'NotStarted'
				_.each(this.steps, function (stepObj) {
					if (stepObj) {
						stepObj.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
						stepObj.approvers = _.chain(stepObj.approvers)
							.filter(function (approver) {
								// prevent that uninvited members that approved in the past will be shown in the next version setup
								// Note: Everyone that can approve, can be approval master
								return approver.memberObj && approver.memberObj.canBeApprovalMaster;
							})
							.each(function (approver) {
								approver.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
								approver.approvedVersion = null;
								approver.approvalDate = null;
								approver.approvalHour = null;
								approver.commentArr = [];
							}).value();
					}
				}.bind(this));
			}

			// store global approval object
			this.globalStatus = {
				status: this.approvalStatus,
				userStatus: undefined
			};
			if (this.globalStatus.status !== ApprovalConstants.ApprovalStatus.NOT_DEFINED && this.globalStatus.status !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				this.globalStatus.userStatus = this.currentUserApprovalStatus(0);
			}
		};

		/**
		 * Reads role assignments from xml input
		 * @param {xml} RoleAssignmentMapData input data contains role assignments
		 * @param {Array}    targetArray           array used to store the list of role assignments
		 * @param {Array}    assignedRoleGroupIDs  array used to store the role group id's
		 */
		ApprovalModel.prototype.readRoleAssignmentData = function (RoleAssignmentMapData, targetArray, assignedRoleGroupIDs) {
			if (!RoleAssignmentMapData) {
				return;
			}

			var extractUserFromXML = function (xml) {
				var firstName = WCXML.GetElementText(xml, "first_name");
				var lastName = WCXML.GetElementText(xml, "last_name");
				var userName = WCXML.GetElementText(xml, "name");
				return {
					firstName: firstName,
					lastName: lastName,
					userName: userName,
					name: lastName + (firstName ? ", " + firstName : " ") + "(" + userName + ")",
					isActive: WCXML.GetElementBoolean(xml, "is_active"),
					ID: WCXML.GetElementAttribute(xml, "id")
				};
			};

			//first initialize Role Assignment Map data
			var roleAssignmentMapList = WCXML.GetFirstLevelChilds(RoleAssignmentMapData, "project_role_assignment_map");

			//go over each of the Roles in the list
			_.each(roleAssignmentMapList, function (currentRoleMap) {
				var curRoleElem = WCXML.GetFirstFirstLevelChild(currentRoleMap, "role_group");

				//get the basic role data 
				var roleID = WCXML.GetElementAttribute(curRoleElem, "id");
				var roleName = WCXML.GetElementSelfTextContent(curRoleElem);

				var roleAssignmentMap = {};
				roleAssignmentMap.roleID = roleID;
				roleAssignmentMap.roleName = roleName;

				//keep track of this role as having been processed
				assignedRoleGroupIDs.push(roleID);

				//------------------------------------------------------
				//parse the User Assignments for the current Project Role
				var userRoleAssignments = [];
				var userRoleAssignmentMap = WCXML.GetFirstFirstLevelChild(currentRoleMap, "assigned_users_list");
				if (userRoleAssignmentMap) {
					var userAssignmentDataList = WCXML.GetFirstLevelChilds(userRoleAssignmentMap, "assigned_user");
					if (userAssignmentDataList) {
						_.each(userAssignmentDataList, function (currentUserRoleAssignment) {
							var userAssignmentData = extractUserFromXML(WCXML.GetFirstFirstLevelChild(currentUserRoleAssignment, "user"));
							userRoleAssignments.push(userAssignmentData);
						});
					}
				}

				//add the User Assignments list to the data of the currently processed Project Role
				if (userRoleAssignments && userRoleAssignments.length > 0) {
					roleAssignmentMap.userRoleAssignments = userRoleAssignments;
				}

				//------------------------------------------------------
				//parse the Group Assignments for the current Project Role
				var groupRoleAssignments = [];
				var groupRoleAssignmentMap = WCXML.GetFirstFirstLevelChild(currentRoleMap, "assigned_groups_list");
				if (groupRoleAssignmentMap) {
					var groupAssignmentDataList = WCXML.GetFirstLevelChilds(groupRoleAssignmentMap, "assigned_group");
					if (groupAssignmentDataList) {
						_.each(groupAssignmentDataList, function (currentGroupRoleAssignment) {
							var curGroupAssignmentGroupName = WCXML.GetElementText(currentGroupRoleAssignment, "assigned_group_name");
							var curGroupAssignmentGroupID = WCXML.GetElementAttribute(currentGroupRoleAssignment, "id");

							var groupAssignmentData = {};
							groupAssignmentData.name = curGroupAssignmentGroupName;
							groupAssignmentData.ID = curGroupAssignmentGroupID;

							//get Assignee info as well (if possible)
							var curGroupAssignmentUserAssigneeElem = WCXML.GetFirstFirstLevelChild(currentGroupRoleAssignment, "assigned_group_assignee");
							if (curGroupAssignmentUserAssigneeElem) {
								groupAssignmentData.assignee = extractUserFromXML(WCXML.GetFirstFirstLevelChild(curGroupAssignmentUserAssigneeElem, "user"));
							}

							groupRoleAssignments.push(groupAssignmentData);
						});
					}
				}

				//add the Group Assignments list to the data of the currently processed Project Role
				if (groupRoleAssignments && groupRoleAssignments.length > 0) {
					roleAssignmentMap.groupRoleAssignments = groupRoleAssignments;
				}

				//finally add the current Role Assignment Map to the parent list
				targetArray.push(roleAssignmentMap);
			}.bind(this));
		};

		/**
		 * Loads project approval setup
		 * @param {boolean} folderFlag If true, the folder approval setup will be loaded, else the project approval setup
		 */
		ApprovalModel.prototype.loadProjectFolderApproval = function (folderFlag) {
			// get the project approval setup via ajax
			var url = this.getProjectFolderSetupURL;
			if (!url) {
				throw ("getProjectFolderSetupURL is null");
			}
			if (url.indexOf("?") == -1) {
				url += "?";
			} else {
				url += "&";
			}

			if (folderFlag === true) {
				url += 'folder=true';
			} else {
				url += 'folder=false';
			}

			this.readApprovalFromURL(url, true);
		};

		/**
		 * Reads xml returning from Ajax call to back-end reading project or folder approval setup
		 * @param {xml} returnxml input xml data
		 */
		ApprovalModel.prototype.readProjectFolderReturnXML = function (returnxml) {
			var errorElem = WCXML.GetFirstFirstLevelChild(returnxml, "error"); // look whether the very first child is the <error> tag
			if (errorElem) { // if there is an error, look for errorMessage child
				var errorMsg = WCXML.GetElementText(errorElem, "errorMessage");
				this.setError({
					errorMsg: 'CAppCouldNotLoad',
					errors: [{
						tag: errorMsg,
						inline: true
					}]
				});
				return;
			} else {
				this.errorMsg = null;
			}

			this.reset();

			var approvalSetupElem = WCXML.GetFirstFirstLevelChild(returnxml, "approvalSetup");
			if (approvalSetupElem) {
				var stagesElement = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "stages");
				if (stagesElement) {
					var stagesArr = WCXML.GetFirstLevelChilds(stagesElement, "stage");
					if (stagesArr.length === 0) {
						this.setError({
							errorMsg: 'CAppNoProjectApproval'
						});
						return;
					}
					this.steps = [];
					_.each(stagesArr, function (stage) {
						this.addStage(ApprovalStage.fromXML(stage, this.projectMembers, this.usersWithNoAccess, this));
					}.bind(this));
				}
				this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
				this.notificationInfo.setNotificationType(ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS);
				// In case you wonder... Whatever the notifications in the folder or project, taking the default notifications is typically 
				// the right thing to do. It's definitely not smart calling these custom notifications while you just copied the setup 
				// from where the default notifications are got.
				var generalElem = WCXML.GetFirstFirstLevelChild(approvalSetupElem, "general");
				if (generalElem) {
					this.uponNewVersion = WCXML.GetElementText(generalElem, "uponNewVersion");
					this.dueDateType = WCXML.GetElementText(generalElem, "dueDateType");
					this.approvalMasterType = WCXML.GetElementText(generalElem, "approvalMasterType");
					this.approvalMaster = WCXML.GetElementText(generalElem, "approvalMaster");
					// other settings in generalElem don't make sense taking over
				}
				this.approvalRuleChanged = true;
			}
		};

		/**
		 * Edit the approval setup for the next version
		 * Sets the flag "nextVersion" and reads the next version's setup if it exists
		 */
		ApprovalModel.prototype.editSetupForNextVersion = function () {
			this.nextVersion = true;
			// figure out whether there exists a next version
			if (this.hasNextVersionSetup) {
				// append parameter nextDocVersion=true
				var url = this.readURL;
				if (url.indexOf("?") == -1) {
					url += "?";
				} else {
					url += "&";
				}
				url += "nextDocVersion=true";
				this.readApprovalFromURL(url, false);
				// repaint will happen caused by the return of the ajax call

			} else {
				// use the (cleared) current setup
				if (this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
					//create new approval setup
					this.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_STARTED);
					var stage = this._stageFactory.createApprovalStage();
					this.addStage(stage, 0);
				} else { //copy current setup
					_.each(this.steps, function (stepObj) {
						stepObj.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
						stepObj.id = null;
						_.each(stepObj.approvers, function (approver) {
							approver.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
							approver.approvedVersion = null;
							approver.approvalDate = null;
							approver.approvalHour = null;
							approver.commentArr = [];
							approver.approvedByUserId = null;
							approver.approvalTime = null;
						}.bind(this));
					}.bind(this));
					if (this.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
						this.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_STARTED);
					}
				}
				this.approvalRuleChanged = true;
				this._emit("initialized");
			}
		};

		/**
		 * Cancel the approval setup
		 * Read back the last saved approval setup from the back-end. All client changes are lost.
		 */
		ApprovalModel.prototype.cancel = function () {
			this.readFromXML(this.readURL);
		};

		/**
		 * Clears the approval setup at the client.  Nothing is done at the server until Save is clicked
		 */
		ApprovalModel.prototype.clear = function () {
			this.removeAllStages();
			this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_DEFINED;
			this.dueDateType = ApprovalConstants.DueDateType.NO;
			// put approval master to the current user
			if (this.currentUser && this.currentUser.userId) {
				this.approvalMasterType = ApprovalConstants.ApprovalMasterType.USER;
				this.approvalMaster = this.currentUser.userId;
			}
			if (this.documentId || this.folderId) {
				this.notificationInfo.setNotificationType(ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS);
			}

			this.approvalRuleChanged = true;

			this._emit("update");
		};


		/**
		 * Called when clicking Back to current version when editing the next version
		 * Clears the flag "nextVersion" and reads back the current version
		 */
		ApprovalModel.prototype.backToCurrentVersion = function () {
			this.nextVersion = false;
			this.readFromXML(this.readURL);
		};

		/**
		 * For the given roleId finds in this.roleAssignments the user object with the given userId
		 * @param userId
		 * @param roleId
		 */
		ApprovalModel.prototype.findRoleUserObj = function (userId, roleId) {
			try {
				var memberObj = null;
				if (!this.roleAssignments) {
					return null;
				}


				for (var role = 0; role < this.roleAssignments.length && !memberObj; role++) {
					if (this.roleAssignments[role].roleID == roleId) {
						// we found the role
						if (this.roleAssignments[role].userRoleAssignments) {
							// look at user assignments
							var userAssignments = this.roleAssignments[role].userRoleAssignments;
							for (var user = 0; user < userAssignments.length && !memberObj; user++) {
								if (userAssignments[user].ID == userId) {
									memberObj = ApprovalUtils.findMemberObj("User", userAssignments[user].ID, this.projectMembers);
								}
							}
						}
						if (this.roleAssignments[role].groupRoleAssignments) {
							// look at group assignments
							var groupAssignments = this.roleAssignments[role].groupRoleAssignments;
							for (var group = 0; group < groupAssignments.length && !memberObj; group++) {
								// find the group member
								var groupMember = ApprovalUtils.findMemberObj("Group", groupAssignments[group].ID, this.projectMembers);
								// look for the user within the group member
								memberObj = ApprovalUtils.findMemberObj("User", userId, groupMember.members);
							}
						}
					}
				}
				return memberObj; // can be null!
			} catch (err) {
				window.alert("Problem in findRoleUserObj: " + err);
			}
		};

		/**
		 * Generates the list of available actions
		 * @return {Object} result object with approvalText and a list of actions
		 */
		ApprovalModel.prototype.getApprovalActions = function (isPageList) {
			var result = {};
			result.list = [];
			result.text = undefined;
			result.showContent = true;

			// for pagelists, we still want the actions if the global status is pending, but they should be "approve remaining" etc.
			var globalStatus, globalUserStatus;
			if (isPageList && this.globalStatus) {
				globalStatus = this.globalStatus.status;
				globalUserStatus = this.globalStatus.userStatus;
			}

			try {

				//dont output the approval buttons if not applicable in this case    
				if ((!this.canSubmitApprovalStatus && globalStatus !== ApprovalConstants.ApprovalStatus.PENDING) || this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED || (this.globalStatus && this.globalStatus.status === ApprovalConstants.ApprovalStatus.NOT_STARTED)) {
					return result;
				}

				var step = 0;
				var user = this.currentUser;
				if (user) {
					// find pending step
					var userStatus = user.mustApprove;
					if (user.mustApprove == null) { // figure it out yourself
						if (this.forcedStep !== undefined) {
							step = this.forcedStep;
						} else {
							var found = false;
							for (step = 0; step < this.steps.length && !found; step++) {
								if (this.steps[step] && this.steps[step].approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
									found = true;
								}
							}
							if (found) {
								// look whether the current user needs to approve in this step
								step--;

								var appArr = this.steps[step].approvers;
								if (appArr && appArr.length > 0) {
									userStatus = this.currentUserApprovalStatus(step);
								}
							}
						}
						if (step >= this.steps.length && globalStatus != "Pending") {
							return result;
						} // none of the steps is pending, so you can't display approval actions
					}

					var suffix;
					if (userStatus == 'MustApprove' || globalUserStatus == "MustApprove") {
						result.text = 'CAppApprovalYouMustApprove';
						suffix = userStatus != 'MustApprove' ? "Remaining" : ""; // whether we want to approve current page or remaining pages (page list only)

						result.list.push({
							name: "Approve" + suffix,
							text: 'CAppApprove' + suffix,
							internalName: "Approved",
							step: step
						});
						result.list.push({
							name: "Reject" + suffix,
							text: 'CAppReject' + suffix,
							internalName: "Rejected",
							step: step
						});
						result.list.push({
							name: "Comment" + suffix,
							text: 'CAppComment' + suffix,
							internalName: "Comment",
							step: step
						});

					} else if (userStatus != null) {
						result.text = 'CAppYouHave' + userStatus;
					}


					// buttons for force approval and rejection
					suffix = (this.approvalStatus !== ApprovalConstants.ApprovalStatus.PENDING) ? "Remaining" : "";
					if (user.canForceApprove) {
						result.list.push({
							name: "ForceApprove" + suffix,
							text: 'CAppForceApprove' + suffix,
							internalName: "ForceApproved",
							step: step
						});
					}
					if (user.canForceReject) {
						result.list.push({
							name: "ForceReject" + suffix,
							text: 'CAppForceReject' + suffix,
							internalName: "ForceRejected",
							step: step
						});
					}
				} // end if user

			} catch (err) {
				window.alert("Problem in Approval.getApprovalActions: " + err);
			}
			result.showContent = true;
			return result;
		};

		/**************************************************************************************
		 * Determines the current user's (this.currentUser) approval status for this step
		 * Most important return value is "MustApprove".  Based on this, the approval buttons are shown
		 * Other return values are Approved, ApprovedConditional, Rejected, ForceApproved, ForceRejected, ForceApprovedConditional and null
		 * @param step: the step for which the check is done
		 * @param approverIdx: checks only against index approver in the approvers table of this step
		 * if approver is omitted, checks for the entire step
		 * @return the approval status of the current user
		 */
		ApprovalModel.prototype.currentUserApprovalStatus = function (step, approverIdx) {
			try {
				var approvalStatus = null;
				var user = this.currentUser;
				var appArr = this.steps[step].approvers;
				// run over all approvers for this step and look whether one is matching with the current user
				for (var approver = 0; approver < appArr.length; approver++) {
					if (approverIdx != null && approverIdx != approver) {
						continue;
					}
					// figure out whether the current user is this approver or member of this approver            
					if (appArr[approver].approvalType === ApprovalConstants.ApprovalType.USER) {
						if (appArr[approver].memberObj.userId == user.userId) {
							if (appArr[approver].approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
								approvalStatus = 'MustApprove';
							}
							if (approvalStatus != 'MustApprove') {
								approvalStatus = appArr[approver].approvalStatus;
							}
						}
					} else { // it's a group or role, figure out whether the current user is in there
						var foundInGroup = (appArr[approver].approvalType === ApprovalConstants.ApprovalType.ONE_OF_ROLE) ?
							this.findRoleUserObj(user.userId, appArr[approver].memberObj.groupId) :
							ApprovalUtils.findMemberObj("User", user.userId, appArr[approver].memberObj.members);
						if (foundInGroup && appArr[approver].approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
							approvalStatus = 'MustApprove';
						}
						if (approvalStatus != 'MustApprove' && appArr[approver].approvedByUserId == user.userId) {
							approvalStatus = appArr[approver].approvalStatus;
						}
					}
				}
				return approvalStatus;
			} catch (err) {
				window.alert("Problem in Approval.currentUserApprovalStatus: " + err);
				return 'MustApprove'; // show the buttons, even if not allowed.  Backend will error out if needed
			}
		};

		/**
		 * resets this model to an empty approval
		 */
		ApprovalModel.prototype.reset = function () {
			this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_DEFINED;
			this.documentVersion = 1;
			this.uponNewVersion = 'InheritApproval';
			this.removeAllStages();
			this.canSubmitApprovalStatus = true;
		};

		/**
		 * Gives each stage's due date an equal amount of time 
		 * @param   {int} days
		 * @param   {int} hours
		 * @returns {boolean}  True if change was succesful, false otherwise
		 */
		ApprovalModel.prototype.addTimeToStages = function (days, hours) {
			if (!isNaN(days) && !isNaN(hours)) {
				if (days < 0 || hours < 0 || hours >= 24) {
					return false;
				} else {
					var currentTime = new Date();
					_.each(this.steps, function (step) {
						if (step.canModifyDueDate()) {
							currentTime = dateShift.addDaysHours(currentTime, days, hours, null, true);
							step.setDueDate(currentTime);
						}
					}.bind(this));
					return true;
				}
			}
			return false;
		};

		/**
		 * Shifts the due dates of all stages with a given time period
		 * @param   {int} days
		 * @param   {int} hours
		 * @returns {boolean}  True if change was succesful, false otherwise
		 */
		ApprovalModel.prototype.shiftDueDatesOfStages = function (days, hours) {
			if (!isNaN(days) && !isNaN(hours)) {
				if (days * hours < 0 || hours >= 24 || hours <= -24) {
					return false;
				} else {
					_.each(this.steps, function (step) {
						if (!step.dueDate || step.dueDate.constructor != Date) {
							return; // should not happen
						}

						if (step.canModifyDueDate()) {
							if (days >= 0 && hours >= 0) {
								step.setDueDate(dateShift.addDaysHours(step.dueDate, days, hours, null, true));
							} else {
								step.setDueDate(dateShift.subtractDaysHours(step.dueDate, Math.abs(days), Math.abs(hours), null, true));
							}
						}
					}.bind(this));
					return true;
				}
			}
			return false;
		};

		/**
		 * Shift all due dates of the stages where the latest stage ends on the specified date
		 * @param   {Date}    finalDueDate
		 * @returns {boolean} True if change was succesful, false otherwise
		 */
		ApprovalModel.prototype.shiftDueDatesToLatest = function (finalDueDate) {
			// find the shift
			var lastStep = _.last(this.steps);
			var timeShift = dateShift.getWorkingTimeDifference(lastStep.dueDate, finalDueDate);
			// run over all steps which are not yet started, starting from the back, and calculate backward
			lastStep.setDueDate(finalDueDate);
			_.each(this.steps.slice().reverse(), function (step, i) {
				if (i > 0) {
					if (!step.dueDate || step.dueDate.constructor != Date) {
						return; // should not happen
					}
					if (step.canModifyDueDate()) {
						if (timeShift > 0) {
							step.setDueDate(dateShift.addDaysHours(step.dueDate, 0, 0, timeShift, true));
						} else {
							step.setDueDate(dateShift.subtractDaysHours(step.dueDate, 0, 0, -timeShift, true));
						}
					}
				}
			}.bind(this));
			return true;
		};

		/**
		 * Shift all due dates of the stages where the first stage ends on the specified date
		 * @param   {Date}    firstDueDate
		 * @returns {boolean} True if change was succesful, false otherwise
		 */
		ApprovalModel.prototype.shiftDueDatesToFirst = function (firstDueDate) {
			// find the shift
			var timeShift = dateShift.getWorkingTimeDifference(this.steps[0].dueDate, firstDueDate);

			// run over all steps and apply the shift
			_.each(this.steps, function (step) {
				if (!step.dueDate || step.dueDate.constructor != Date) {
					return; // should not happen
				}
				if (step.canModifyDueDate()) {
					if (timeShift > 0) {
						step.setDueDate(dateShift.addDaysHours(step.dueDate, 0, 0, timeShift, true));
					} else {
						step.setDueDate(dateShift.subtractDaysHours(step.dueDate, 0, 0, -timeShift, true));
					}
				}
			}.bind(this));
			return true;
		};

		/**
		 * Distributes the due dates evenly between a given start and enddate
		 * @param   {Date} startDate
		 * @param   {Date} endDate
		 * @returns {boolean}  True if change was succesful, false otherwise
		 */
		ApprovalModel.prototype.distributeDueDatesEvenly = function (startDate, endDate) {
			var timeShift = dateShift.getWorkingTimeDifference(startDate, endDate);
			if (timeShift < 0) {
				return false;
			}

			// determine amount of valid stages left					
			var validSteps = _.filter(this.steps, function (step) {
				if (!step.dueDate) {
					return false;
				}
				return step.canModifyDueDate();
			});

			if (validSteps.length > 0) {
				var curDate = startDate;
				_.each(validSteps, function (step) {
					curDate = dateShift.addDaysHours(curDate, 0, 0, timeShift / validSteps.length, true);
					step.setDueDate(curDate);
				}.bind(this));
			}
			return true;
		};

		/**
		 * Change the type of due date
		 * @param {string} newValue
		 */
		ApprovalModel.prototype.changeDueDateType = function (newValue) {
			var oldValue = this.dueDateType;

			if (oldValue !== newValue) {
				this.dueDateType = newValue;

				// when going from lead times to fixed due dates, calculate the due dates
				if (this.dueDateType === ApprovalConstants.DueDateType.FIXED) {
					var curTime = ApprovalUtils.getNewCurrentTime();

					if (oldValue === ApprovalConstants.DueDateType.LEAD_TIMES || oldValue === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
						_.each(this.steps, function (step) {
							if (step.canModifyDueDate()) {
								var leadTime = step.leadTime;
								if (leadTime && leadTime > 0) {
									curTime = dateShift.addDaysHours(curTime, 0, 0, leadTime, true); // adds time in working days and hours
									step.setDueDate(curTime);
								} else { // negative leadTime or no leadTime.  Just put this step's due date on the previous step's
									step.setDueDate(curTime);
								}
							}
						}.bind(this));
					} else if (oldValue == ApprovalConstants.DueDateType.NO) {
						// shift a day per step
						_.each(this.steps, function (step) {
							if (step.canModifyDueDate()) {
								curTime = dateShift.addDaysHours(curTime, 1, 0, 0, true);
								step.setDueDate(curTime);
							}
						}.bind(this));
					}
				} else if (this.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
					if (!this.finalDueDate) {
						this.finalDueDate = ApprovalUtils.getNewCurrentTime();
					}
				}

				this._emit("changed.duedatetype");
			}
		};

		/**
		 * Change the final due date
		 * @param {string} newDate
		 * @param {string} newHour
		 */
		ApprovalModel.prototype.setFinalDueDate = function (newFinalDueDate) {
			if (this.finalDueDate !== newFinalDueDate) {
				this.finalDueDate = newFinalDueDate;
				this._emit("changed.finalduedate");
			}
		};

		/**
		 * Change upon new version
		 * @param {string} neValue
		 */
		ApprovalModel.prototype.changeInheritApproval = function (newValue) {
			if (this.uponNewVersion !== newValue) {
				this.uponNewVersion = newValue;
				this._emit("changed.uponnewversion");
			}
		};

		/**
		 * Change approval master
		 * @param {string} neValue
		 */
		ApprovalModel.prototype.changeApprovalMaster = function (newValue) {
			if (newValue.substring(0, 4) == "$Me$") {
				if (this.currentUser) {
					this.approvalMasterType = ApprovalConstants.ApprovalMasterType.USER;
					this.approvalMaster = this.currentUser.userId;
				} else {
					this.setError({
						errorMsg: "No current user while trying to put the approval master on Me"
					});
				}
			} else if (newValue.substring(0, 4) == "$PM$") {
				this.approvalMasterType = ApprovalConstants.ApprovalMasterType.PROJECT_MANAGER;
				this.approvalMaster = null;
			} else if (newValue.substring(0, 10) == "$Uploader$") {
				this.approvalMasterType = ApprovalConstants.ApprovalMasterType.UPLOADER;
				this.approvalMaster = null;
			} else if (newValue.substring(0, 7) == "$Group$") {
				this.approvalMasterType = ApprovalConstants.ApprovalMasterType.GROUP;
				this.approvalMaster = newValue.substring(7, newValue.length);
			} else { // normal user
				this.approvalMasterType = ApprovalConstants.ApprovalMasterType.USER;
				this.approvalMaster = newValue;
			}
			this._emit("changed.approvalmaster");
		};

		/**
		 * Calls a web service via Ajax to read approval setup information 
		 */
		ApprovalModel.prototype.readFromXML = function (readURL) {
			if (!readURL) {
				return;
			}

			var url = readURL;
			if (this.nextVersion) {
				if (url.indexOf("?") == -1) {
					url += "?";
				} else {
					url += "&";
				}
				url += "nextDocVersion=true";
			}

			this.readApprovalFromURL(url);
		};

		/**
		 * Validates the approval setup
		 * @returns {object} errorObj - an object containing the errors and warnings
		 */
		ApprovalModel.prototype.validate = function () {
			var errorObj = {
				errorMsg: 'CAppCouldNotSave',
				errors: [],
				warnings: []
			};
			var today = new Date().getTime() - (7200000); // give 2 hours of tolerance
			var lastDueDate = -1;

			if (this.isComplexApproval()) {
				var stepsWithoutNames = _.some(this.steps, function (stage) {
					return _.isEmpty(stage.getStageName().trim());
				}.bind(this));

				// Check for empty stages 
				if (stepsWithoutNames) {
					errorObj.errors.push({
						tag: "CAppEmptyNameNotAllowed"
					});
				}
			}

			var stepsWithName = _.filter(this.steps, function (step) {
				return step.getStageName().trim();
			});

			// Check for duplicate stage names
			if ((_.uniq(stepsWithName, function (step) {
					return step.getStageName();
				})).length !== stepsWithName.length) {
				errorObj.errors.push({
					tag: "CAppSameNameNotAllowed"
				});

			}

			// validate the input, except when saving after clicking on Clear
			if (this.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
				// check no stages at all
				if (!this.steps || this.steps.length === 0) {
					// allow saving empty if the approval setup has been changed
					if (this.approvalRuleChanged !== true) {
						errorObj.errors.push({
							tag: 'CAppCannotSaveOrStartAnEmptyApprovalSetup'
						});
					}
				} else { // there is at least one stage

					for (var step = 0; step < this.steps.length; step++) {
						var stepObj = this.steps[step];
						// check empty approval stages
						if (!stepObj || !stepObj.approvers || stepObj.approvers.length === 0) {
							if (step === 0 && stepObj.name === "") { // simple approval setup
								errorObj.errors.push({
									tag: 'CAppPleaseSelectApprovers'
								});
							} else {
								errorObj.errors.push({
									tag: 'CAppEmptyStage',
									parameters: [(step + 1) + ": " + stepObj.name]
								});
							}
						}
						// check due dates in sequence and not in the past and give warning accordingly
						if (this.dueDateType === ApprovalConstants.DueDateType.FIXED && stepObj.canModifyDueDate()) {
							if (!stepObj.dueDate || stepObj.dueDate.constructor != Date) {
								errorObj.warnings.push({
									tag: 'CAppFixedDueDateEmpty'
								});
							} else { // now we are 100% sure stepObj.dueDate is a Date object
								// check due date in the past
								if (stepObj.dueDate.getTime() < today) {
									if (step === 0 && stepObj.name === "") {
										errorObj.warnings.push({
											tag: 'CAppDueDateInPast'
										});
									} else {
										errorObj.warnings.push({
											tag: 'CAppStageDueDateInPast',
											parameters: [(step + 1) + ": " + stepObj.name]
										});
									}
								}
								// check due date before dueDate in previous stage
								if (stepObj.dueDate.getTime() < lastDueDate) {
									// this can never happen for simple approval...
									errorObj.warnings.push({
										tag: 'CAppDueDateBeforePreviousStage',
										parameters: [(step + 1) + ": " + stepObj.name]
									});
								} else {
									lastDueDate = stepObj.dueDate.getTime();
								}
							}
						}
					}

					// Check lead times with final due date without having a final due date
					if (this.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
						if (!this.finalDueDate || this.finalDueDate.constructor != Date) {
							errorObj.errors.push({
								tag: 'CAppFinalDueDateEmpty'
							});
						}
						// check whether not in the past
						if (this.finalDueDate.constructor == Date) {
							if (this.finalDueDate.getTime() < today) {
								errorObj.warnings.push({
									tag: 'CAppDueDateInPast'
								});
							}
						}
					}
				}
			}

			return errorObj;
		};

		/**
		 * Converts approval setup stored in data model to xml according to <SetupApproval> scheme
		 * @returns {string} output xml
		 */
		ApprovalModel.prototype.writeToXML = function () {
			var xmlcode = "";
			xmlcode += '<approvalSetup>';

			// save back a minimalistic version of project, folder and document element.
			// this is probably never used by the back-end since the submission URL knows the documentVersionID and therefore
			// also the folder and project it is in (except in case of linked documents, but it really only needs the documentVersionID).
			// project information
			if (this.projectId) {
				xmlcode += '<project>';
				xmlcode += WCXML.ElementEncode('projectId', this.projectId);
				xmlcode += WCXML.ElementEncode('projectName', this.projectName);
				xmlcode += WCXML.ElementEncode('projectManagerUserId', this.projectManagerUserId);
				// project members not written back
				xmlcode += '</project>';
			}

			// folder information
			if (this.folderId) {
				xmlcode += '<folder>';
				xmlcode += WCXML.ElementEncode('folderId', this.folderId);
				xmlcode += WCXML.ElementEncode('folderName', this.folderName);
				xmlcode += '</folder>';
			}

			// document information
			if (this.documentId) {
				xmlcode += '<document>';
				xmlcode += WCXML.ElementEncode('documentId', this.documentId);
				xmlcode += WCXML.ElementEncode('documentName', this.documentName);
				xmlcode += WCXML.ElementEncode('documentVersion', this.documentVersion);
				xmlcode += '</document>';
			}

			// general information - Here it starts to become important

			xmlcode += '<general>';
			xmlcode += WCXML.ElementEncode('dueDateType', this.dueDateType);
			if (this.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE && this.finalDueDate && this.finalDueDate.constructor == Date) {
				xmlcode += WCXML.ElementEncode('finalDueDate', this.finalDueDate.getTime());
			}
			xmlcode += WCXML.ElementEncode('uponNewVersion', this.uponNewVersion);
			xmlcode += WCXML.ElementEncode('generalApprovalStatus', this.approvalStatus);
			//approval master type
			xmlcode += WCXML.ElementEncode('approvalMasterType', this.approvalMasterType);
			// approval master, can be null if type is ProjectManager or Uploader
			if (this.approvalMaster) {
				xmlcode += WCXML.ElementEncode('approvalMaster', this.approvalMaster);
			}

			// notifications (still part of general information)			
			xmlcode += this.notificationInfo.writeToXML();

			xmlcode += '</general>';

			// write stages
			if (this.steps && this.steps.length > 0) {
				xmlcode += '<stages>';
				for (var step = 0; step < this.steps.length; step++) {
					var stepObj = this.steps[step];
					if (stepObj) {
						xmlcode += stepObj.writeToXML(step);
					} else {
						throw ("Inconsistency in steps, stepObj == null");
					}
				}
				xmlcode += '</stages>';
			}

			xmlcode += '</approvalSetup>';

			return xmlcode;
		};

		/**
		 * Add a new stage on @param index
		 * @param {integer} index
		 */
		ApprovalModel.prototype.addNewStage = function (index) {
			var stage = this._stageFactory.createApprovalStage();
			this.addStage(stage, index);
		};

		/**
		 * Add a stage to the steps at the given index or at the end if index is undefined
		 * @param {ApprovalStage} stage
		 * @param {Integer} [index]
		 */
		ApprovalModel.prototype.addStage = function (stage, index) {
			if (this.steps.indexOf(stage) > -1) {
				throw "Stage already in steps";
			}
			if (typeof index === "number" && index >= 0 && index < this.steps.length) {
				this.steps.splice(index, 0, stage);
			} else {
				this.steps.push(stage);
			}
			this._emit("changed.stages.added", stage);
		};

		/**
		 * Adds a stage right after an other stage
		 * @param {ApprovalStage} stageAfter the stage that is in front of the new stage
		 */
		ApprovalModel.prototype.addStageAfter = function (stageAfter) {
			var stage = this._stageFactory.createApprovalStage(stageAfter);
			var indexOf = this.steps.indexOf(stageAfter);
			if (indexOf > -1) {
				this.addStage(stage, indexOf + 1);
			}
		};

		/**
		 * Remove a stage from the approval model		 * 
		 * @param {ApprovalStage} stage
		 */
		ApprovalModel.prototype.removeStage = function (stage) {
			var indexOfStage = this.steps.indexOf(stage);
			if (indexOfStage > -1) {
				this.steps.splice(indexOfStage, 1);
			}

			if (this.steps.length === 0) {
				this.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_DEFINED);
			}
			this._emit("changed.stages.deleted", stage);
		};

		/**
		 * Clears all stages
		 */
		ApprovalModel.prototype.removeAllStages = function () {
			var argumentsForEmit = ["changed.stages.deleted"];
			Array.prototype.push.apply(argumentsForEmit, this.steps);
			this.steps = [];
			this.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_DEFINED);
			this._emit.apply(this, argumentsForEmit);

		};

		/**
		 * Change the approval status
		 * @param {string} status
		 */
		ApprovalModel.prototype.setApprovalStatus = function (status) {
			if (this.approvalStatus !== status) {
				this.approvalStatus = status;
				this._emit("changed.approvalstatus");
			}
		};

		/**
		 * Returns the number of a particular stage
		 * @param {ApprovalStage} stage
		 * @return {Integer}
		 */
		ApprovalModel.prototype.getStageNumber = function (stage) {
			return this.steps.indexOf(stage) + 1;
		};

		/**
		 * Get the stage at the specified index
		 * @param   {integer} index
		 * @returns {ApprovalStage}
		 */
		ApprovalModel.prototype.getStageAtIndex = function (index) {
			return this.steps[index];
		};

		/**
		 * Get the amount of stages in this approval
		 * @returns {integer}
		 */
		ApprovalModel.prototype.getStageCount = function () {
			return this.steps.length;
		};

		/**
		 * Get the list of stages
		 * @returns {Array<ApprovalStage>}
		 */
		ApprovalModel.prototype.getStages = function () {
			return this.steps;
		};


		/**
		 * Move a stage from the specified index to an other specified index
		 * @param {integer} stageIndex
		 * @param {integer} newStageIndex
		 */
		ApprovalModel.prototype.moveStage = function (stageIndex, newStageIndex) {
			if (newStageIndex >= this.steps.length) {
				newStageIndex = this.steps.length - 1;
			}
			if (stageIndex === newStageIndex) {
				return;
			}
			var stageAtIndex = this.steps[stageIndex];
			if (stageAtIndex) {
				this.steps.splice(stageIndex, 1);
				this.steps.splice(newStageIndex, 0, stageAtIndex);
				this._emit("changed.stages.moved", stageAtIndex);
			}
		};

		/**
		 * Return the current user as a String
		 * @returns {string}
		 */
		ApprovalModel.prototype.getCurrentUserString = function () {
			var userString = '';
			if (this.currentUser && this.currentUser.userId) {
				var memberObj = ApprovalUtils.findMemberObj("User", this.currentUser.userId, this.projectMembers);
				if (memberObj) {
					userString += WCHTML.getUserNameString(memberObj.firstName, memberObj.lastName, memberObj.userName);
				}
			}
			return userString;
		};

		/**
		 * Checks whether newName is already in use as a name for a stage
		 * @return true if the name is in use, else false
		 */
		ApprovalModel.prototype.isValidStageName = function (step) {
			return !_.find(this.steps, function (step2) {
				return step && step2 && step2 !== step && step2.getStageName() === step.getStageName();
			});
		};

		/**
		 * Get the status of the approval
		 * @returns {string}
		 */
		ApprovalModel.prototype.getApprovalStatus = function () {
			if (this.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING && this.globalStatus && this.globalStatus.status === ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				return this.globalStatus.status;
			}
			return this.approvalStatus;
		};

		/**
		 * Generate a simple read only table
		 * @returns {Object}
		 */
		ApprovalModel.prototype.getApprovalDetailsData = function () {
			var d = {};
			d.isComplexApproval = this.isComplexApproval();

			d.steps = [];
			_.each(this.steps, function (stepObj, step) {
				// FOR EVERY STEP
				var s = {};
				d.steps.push(s);

				s.internalNumber = step;
				s.number = step + 1;
				s.name = stepObj.name;
				s.status = stepObj.approvalStatus;

				// add date when the step was already completed
				if (stepObj.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED && stepObj.approvalStatus !== ApprovalConstants.ApprovalStatus.PENDING) {
					var approvalTime = -1;
					_.each(stepObj.approvers, function (approver) {
						var stepApprovalTime = approver.approvalTime; // a date object
						if (stepApprovalTime && stepApprovalTime.constructor == Date) {
							stepApprovalTime = stepApprovalTime.getTime();
						} // convert to ms
						if (stepApprovalTime > approvalTime) {
							approvalTime = stepApprovalTime;
						}
					}.bind(this));
					if (approvalTime > 0) {
						s.approvalTime = new Date(approvalTime);
					}
				}

				// build up list of approvers
				s.approvers = [];
				if (stepObj.approvers) {
					for (var approver = 0; approver < stepObj.approvers.length; approver++) {
						// FOR EVERY APPROVER
						var a = {};
						s.approvers.push(a);

						var approverObj = stepObj.approvers[approver];
						if (!approverObj) {
							continue;
						}
						var memberObj = approverObj.memberObj;
						if (!memberObj) {
							continue;
						}

						a.user = ApprovalUtils.getUserRepresentation(memberObj);

						a.internalNumber = approver;
						a.type = approverObj.approvalType;

						if (a.isRole) {
							a.type = a.type.replace("Group", "Role");
						}

						a.typeLangTag = ApprovalUtils.getApprovalTypeLangTag(approverObj);

						a.status = approverObj.approvalStatus;
						// it can be that this is different from the current version
						if (approverObj.approvedVersion) {
							a.approvedVersion = approverObj.approvedVersion;
							a.isDifferentVersionApproved = (approverObj.approvedVersion != this.documentVersion);
						}

						a.approvalTime = approverObj.approvalTime;

						// Approved by
						if (approverObj.approvedByUserId) {
							var member = ApprovalUtils.findMemberObj("User", approverObj.approvedByUserId, this.projectMembers);
							if (!member) {
								// the user has no access anymore
								member = ApprovalUtils.findMemberObj("User", approverObj.approvedByUserId, this.usersWithNoAccess);
							}
							a.approvedBy = ApprovalUtils.getUserRepresentation(member);
						}


						// approval comment
						if (approverObj.commentArr && approverObj.commentArr.length > 0) {
							a.comments = approverObj.commentArr;
						}
					}
				}
			}.bind(this));
			return d;
		};

		/**
		 * Check if the approval has been started 
		 * @returns {Boolean}
		 */
		ApprovalModel.prototype.isApprovalStarted = function () {
			return !(this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED || this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED);
		};

		/** 
		 * Force approval to happen on a step
		 * This is used for pagelists, to approve other pages
		 * @param {Number} [stepnr] - index of step
		 */
		ApprovalModel.prototype.forceStep = function (stepnr) {
			this.forcedStep = stepnr;
		};

		/**
		 * Get the number of the currently active step 
		 * @returns {Number} the currently active step, if none is active the method returns -1
		 */
		ApprovalModel.prototype.getActiveStep = function () {
			for (var step = 0; step < this.steps.length; step++) {
				if (this.steps[step] && this.steps[step].approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
					return step;
				}
			}
			return -1;
		};

		/**
		 * Check if the current user is a Pending approver (the user has to approve in the currently active step)
		 * @returns {Boolean}
		 */
		ApprovalModel.prototype.isUserPendingApprover = function () {
			var step = this.getActiveStep();
			if (step >= 0) {
				return this.currentUserApprovalStatus(this.getActiveStep()) === "MustApprove";
			}
			return false;
		};

		/**
		 * Check if the user is in the approval in some way (in all possible steps, through group/role)
		 * @returns {Boolean}
		 */
		ApprovalModel.prototype.isUserApprover = function () {
			var userID = this.currentUser.userId;
			for (var i = 0; i < this.steps.length; i++) {
				var stepObj = this.steps[i];
				if (stepObj.approvers) {
					for (var j = 0; j < stepObj.approvers.length; j++) {
						var apprObj = stepObj.approvers[j];
						if ((apprObj.approvalType === ApprovalConstants.ApprovalType.USER && apprObj.memberObj.userId == userID) ||
							(apprObj.approvalType === ApprovalConstants.ApprovalType.ONE_OF_ROLE && this.findRoleUserObj(userID, apprObj.memberObj.groupId)) ||
							ApprovalUtils.findMemberObj("User", userID, apprObj.memberObj.members)) {
							return true;
						}
					}
				}
			}
			return false;
		};


		/**
		 * DO NOT USE THIS METHOD FOR NEW DEVELOPMENT
		 * Load approval info based on global information
		 **/
		ApprovalModel.getApprovalInfo = function (callback) {
			// only if in a single project
			if (window.gProjectId) {
				// first, get some data from the server with Ajax call
				var urlStr = "complexapproval.jsp?action=GetBootInfo&projectID=" + window.gProjectId + "&docVerID=" + window.gDocVersionId;
				$.ajax({
					url: urlStr,
					type: "GET",
					data: "xml",
					debug: false,
					timeout: 120000,
					synchronous: false,
					cache: false,
					success: function (xml, objHdl) {
						// use the data to build a ComplexApproval Object
						var xmlObj = WCXML.xml2obj(xml);
						if (!xmlObj.error) {
							var info = xmlObj.complexApprovalBootInfo;
							if (!info || info.isLinkedDocument == '1') {
								return;
							}

							// create user object to be used in complex approval
							var userObject = {
								userId: info.userId,
								userName: info.userName,
								lastName: info.lastName,
								firstName: info.firstName,
								isProjectManager: info.isProjectManager == '1',
								isAdmin: info.isAdmin == '1',
								canForceApprove: info.canForceApprove == '1',
								canForceReject: info.canForceReject == '1',
								canChangeSetup: info.canChangeSetup == '1',
								canStartApproval: info.canStartApproval == '1',
								canSeeOthers: info.canSeeOthers == '1',
								canCreateUsers: info.canCreateUsers == '1'
							};

							// create the parameters to create a complex approval object
							var readURL = "complexapproval.jsp?projectID=" + window.gProjectId + "&docVerID=" + window.gDocVersionId +
								"&returnURL=docdetailsapprovers.jsp&urlwanted=docdetailsapprovers.jsp&menu_file=" +
								"&docID=" + window.gDocId + "&folderID=";
							var readProjectFolderCAPP = readURL + "&action=Inherit";
							var actionURL = readURL;

							var nextversion = false;
							var isViewable = info.isViewable === '1';
							var stagedApprovalAllowed = info.stagedApprovalAllowed == '1';

							// load complex approval
							var appObject = new ApprovalModel(
								readURL,
								nextversion,
								actionURL,
								userObject,
								stagedApprovalAllowed,
								readProjectFolderCAPP
							);
							appObject.on("initialized", _.partial(callback, appObject));
							appObject.isViewFile = isViewable;
						}
					},
					error: function (status, objHdl, description) {
						if (description == 'SESSION_EXP' && window.loginForm != null) {
							// redirect to the login.jsp for logging in again. 
							window.loginForm.submit();
						} else {
							window.alert("Error in loadApprovalUser: " + status);
						}
					}
				});
			}
		};

		/**
		 * Get the list of all invited members in this project
		 * @returns {Array<Object>}
		 */
		ApprovalModel.prototype.getProjectMembers = function () {
			return this.projectMembers;
		};

		/**
		 * Get the project id
		 * @returns {String} 
		 */
		ApprovalModel.prototype.getProjectId = function () {
			return this.projectId;
		};

		/**
		 * Get the folder id
		 * @returns {String} 
		 */
		ApprovalModel.prototype.getFolderId = function () {
			return this.folderId;
		};

		/**
		 * Sends an event with the data to be submitted
		 * @param {object} data
		 */
		ApprovalModel.prototype.submit = function (data) {
			this._emit("submit", data);
		};

		/**
		 * Determines if staged approval is allowed.
		 * @returns {boolean}
		 */
		ApprovalModel.prototype.isStagedApprovalAllowed = function () {
			return this.stagedApprovalAllowed;
		};

		/**
		 * Determines if the approval is a complex approval (based on the number of steps in the approval)
		 * @returns {boolean}
		 */
		ApprovalModel.prototype.isComplexApproval = function () {
			return this.steps.length > 1;
		};

		/**
		 * Determines if a project approval setup is available
		 * @returns {boolean}
		 */
		ApprovalModel.prototype.isProjectSetupAvailable = function () {
			return (this.documentId || this.folderId || this.isMultiDocSetup === true) && this.hasProjectApprovalSetup;
		};

		/**
		 * Determines if a folder approval setup is available
		 * @returns {boolean}
		 */
		ApprovalModel.prototype.isFolderSetupAvailable = function () {
			return (this.documentId || this.isMultiDocSetup === true) && this.hasFolderApprovalSetup;
		};

		return ApprovalModel;
	});
