define(["approval/ApprovalConstants",
		"utils/Observable",
		"wcr/webcenter_xml",
		"underscore"
	],
	function (ApprovalConstants,
		Observable,
		WCXML,
		_) {
		"use strict";

		/**
		 * ApprovalNotifications is the model that holds all information regarding the notifications for the current Approval
		 * @param {ApprovalModel} approvalModel
		 */
		function ApprovalNotifications(approvalModel) {
			Observable.mixin(this, "notifications");

			this.approvalModel = approvalModel;
			this.notificationType = ApprovalConstants.NotificationType.NO_NOTIFICATIONS;
			this.notificationsArr = null;
			this.defaultNotificationsArr = null;
			this.projectNotificationsArr = null;
		}


		/**
		 * Creates a new ApprovalNotifications object from the given xml
		 * @param   {Object} xml
		 * @param   {ApprovalModel} approvalModel
		 * @returns {ApprovalNotifications} The model that holds all information regarding notifications
		 */
		ApprovalNotifications.fromXML = function (xml, approvalModel) {
			var notificationInfo = new ApprovalNotifications(approvalModel);
			notificationInfo.readNotificationInfoFromXML(xml);
			return notificationInfo;
		};

		/**
		 * This will extract all the required data from the given xml object.
		 * @param {Object} xml
		 */
		ApprovalNotifications.prototype.readNotificationInfoFromXML = function (xml) {
			var generalElem = WCXML.GetFirstFirstLevelChild(xml, "general");
			var projectElem = WCXML.GetFirstFirstLevelChild(xml, "project");
			var folderElem = WCXML.GetFirstFirstLevelChild(xml, "folder");

			if (generalElem) {
				this.notificationType = WCXML.GetElementText(generalElem, "notificationType");
				if (this.notificationType === ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS) {
					this.notificationsArr = this.readNotificationsFromXML(generalElem);
					if (this.notificationsArr.length === 0) {
						this.notificationType = ApprovalConstants.NotificationType.NO_NOTIFICATIONS;
					}
				} else if (!this.notificationType) {
					this.notificationType = ApprovalConstants.NotificationType.NO_NOTIFICATIONS;
				}
			}

			if (projectElem) {
				// read default notifications (found in the project section).  Remark that they can still be overwritten by folder
				// notifications ( found in the folder section)
				this.defaultNotificationsArr = this.readNotificationsFromXML(projectElem);
				if (!this.defaultNotificationsArr || this.defaultNotificationsArr.length === 0) {
					this.defaultNotificationsArr = null;
				}
			}

			if (folderElem) {
				// read notifications and overwrite the  project notifications if there are folder notifications
				var folderNotificationArr = this.readNotificationsFromXML(folderElem);

				// store the default notifications in the project notifications var
				this.projectNotificationsArr = this.defaultNotificationsArr;

				if (folderNotificationArr && folderNotificationArr.length > 0) {
					this.defaultNotificationsArr = folderNotificationArr;
				}
			}

			this._emit("update");
		};

		/**
		 * looks for <notifications> element in parentElem and convert its content to notificationArr
		 * used for reading default (folder and project) and custom notifications
		 * @param parentElem: the xml element in which you hope finding the <notifications> element
		 * @param notificationTypeArr: array of notification types (used as search keys)
		 * @return notifications array 
		 */
		ApprovalNotifications.prototype.readNotificationsFromXML = function (xml) {
			var notificationsArr = [];
			var notificationsElem = WCXML.GetFirstFirstLevelChild(xml, "notifications");
			if (notificationsElem) {

				for (var notificationType = 0; notificationType < ApprovalConstants.notificationTypeArr.length; notificationType++) {
					var notificationObj = {};
					var notificationElem = WCXML.GetFirstFirstLevelChild(notificationsElem, ApprovalConstants.notificationTypeArr[notificationType]);
					if (notificationElem) {
						notificationObj.projectManager = WCXML.GetElementBoolean(notificationElem, "projectManager");
						notificationObj.involvedPeople = WCXML.GetElementBoolean(notificationElem, "involvedPeople");
						notificationObj.projectMembers = WCXML.GetElementBoolean(notificationElem, "projectMembers");
					}
					notificationsArr.push(notificationObj); // always one push per notificationType even if notificationElem doesn't exist.
				}
			}

			return notificationsArr;
		};

		/**
		 * This will set the value of the notification at the given row and column
		 * @param {integer} row row of notification
		 * @param {integer} col column of notification
		 * @param {boolean} val the new value
		 */
		ApprovalNotifications.prototype.toggleNotification = function (row, col, val) {
			if (this.notificationsArr[row]) {
				var notificationObj = this.notificationsArr[row];
				switch (col) {
				case 0:
					notificationObj.projectManager = (val != null) ? val : (notificationObj.projectManager !== true);
					break;
				case 1:
					notificationObj.involvedPeople = (val != null) ? val : (notificationObj.involvedPeople !== true);
					break;
				case 2:
					notificationObj.projectMembers = (val != null) ? val : (notificationObj.projectMembers !== true);
					break;
				default:
					throw ("notification " + col + " does not exist");
				}
			}

			this._emit("update");
		};

		/**
		 * This will set the value of all the notifications in the given column
		 * @param {integer} col column
		 * @param {boolean} val the new value
		 */
		ApprovalNotifications.prototype.toggleNotificationColumn = function (col, val) {
			_.each(ApprovalConstants.notificationTypeArr, function (notification, row) {
				this.toggleNotification(row, col, val);
			}.bind(this));
		};

		/**
		 * This changes the current type of notifications
		 * @param {string} type
		 */
		ApprovalNotifications.prototype.setNotificationType = function (type) {
			this.notificationType = type;

			switch (this.notificationType) {
			case ApprovalConstants.NotificationType.NO_NOTIFICATIONS:
				// clear current notifications
				break;
			case ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS:
				// when dealing with a folder setup, show the project folder notifications
				if (this.projectNotificationsArr != null) {
					this.notificationsArr = this.projectNotificationsArr.slice();
				}
				break;
			case ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS:
				if (!this.notificationsArr) {
					// do a deep copy of the default notifications                
					if (this.defaultNotificationsArr) {
						this.notificationsArr = this.defaultNotificationsArr.slice();
					}
				}
				break;
			default:
				throw ("Notification type unknown" + this.notificationType);
			}

			this._emit("update");
		};

		/**
		 * This converts the model to an xml representation
		 * @returns {string} resulting xml data
		 */
		ApprovalNotifications.prototype.writeToXML = function () {
			var xmlcode = "";
			if (this.notificationType) {
				xmlcode += WCXML.ElementEncode("notificationType", this.notificationType);
			}
			if (this.notificationType === ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS && this.notificationsArr) {
				xmlcode += "<notifications>";
				_.each(ApprovalConstants.notificationTypeArr, function (notificationType, index) {
					var notificationObj = this.notificationsArr[index];
					if (notificationObj) {
						xmlcode += "<" + notificationType + ">";
						xmlcode += WCXML.ElementEncodeBoolean("projectManager", notificationObj.projectManager);
						xmlcode += WCXML.ElementEncodeBoolean("involvedPeople", notificationObj.involvedPeople);
						xmlcode += WCXML.ElementEncodeBoolean("projectMembers", notificationObj.projectMembers);
						xmlcode += "</" + notificationType + ">";
					}
				}.bind(this));
				xmlcode += "</notifications>";
			}
			return xmlcode;
		};


		return ApprovalNotifications;
	});
