define(["approval/model/Approver",
		"approval/ApprovalConstants",
		"utils/Observable",
		"utils/Observer",
		"utils/webi18n!Approval",
		"wcr/webcenter_xml",
		"underscore"
	],
	function (
		Approver,
		ApprovalConstants,
		Observable,
		Observer,
		i18n,
		WCXML,
		_) {
		"use strict";

		/**
		 * Represents an individual stage during approval
		 * @param   {ApprovalModel} approvalModel
		 */
		function ApprovalStage(approvalModel) {
			Observable.mixin(this, ["Approval", approvalModel.getId(), "stage", _.uniqueId()], "Approval");
			Observer.mixin(this);

			this.id = null;
			this.name = "";

			this.dueDate = null;
			this.leadTime = null;

			this.approvalStatus = ApprovalConstants.ApprovalStatus.NOT_STARTED;
			this.condApp = true;

			this.rejectHandle = 'RejectStopsApprovalAfterStep';
			this.requireReviewHandle = 'RequireReviewNo';

			this.approvers = [];
			this.approvalModel = approvalModel;

			// If stages change, the number might have changed
			this._observe(this.approvalModel, "changed.stages.*", _.partial(this._emit, "changed.number"));

			this._observe(this.approvalModel, "changed.stages.deleted", function () {
				if (_.contains(arguments, this)) {
					this._emit("deleted");
					this._stopObserveAll();
				}
			}.bind(this));

			// Special functionality to give this simple approval stage a name when adding a second stage
			this._observe(this.approvalModel, "changed.stages.added", function (newStage) {
				if (_.isEmpty(this.name) && this !== newStage && _.size(this.approvalModel.getStages()) === 2) {
					this.setStageName(i18n.t("CAppStage") + " 1");
				}
			}.bind(this));

			Object.defineProperty(this, "number", {
				get: function () {
					if (this.approvalModel) {
						return this.approvalModel.getStageNumber(this);
					} else {
						return "undefined";
					}
				}.bind(this)
			});

		}

		/**
		 * Reads approval stages (steps) from XML
		 * @param xml: parent element containing all the stages
		 * @param stepArr: array of stepObj in which the steps will be written
		 */
		ApprovalStage.fromXML = function (xml, projectMembers, usersWithNoAccess, approvalModel) {
			var stepObj = new ApprovalStage(approvalModel);
			stepObj.id = WCXML.GetElementText(xml, "stageId");
			stepObj.name = WCXML.GetElementText(xml, "stageName");

			// read due date/lead time.
			// Fixed due date takes precedence over lead time.
			var dueDate = parseInt(WCXML.GetElementText(xml, "stageDueTimeStamp"));
			if (dueDate) {
				stepObj.dueDate = new Date(dueDate);
				} else {
				stepObj.dueDate = null;
				}
			stepObj.leadTime = parseInt(WCXML.GetElementText(xml, "stageLeadTime"));

			// read approval information
			stepObj.approvalStatus = WCXML.GetElementText(xml, "stageApprovalStatus");
			stepObj.condApp = WCXML.GetElementBoolean(xml, "allowConditionalApproval");

			stepObj.rejectHandle = WCXML.GetElementText(xml, "uponRejection");
			stepObj.requireReviewHandle = WCXML.GetElementText(xml, "requireReviewState");

			var approversElem = WCXML.GetFirstFirstLevelChild(xml, "approvers");
			if (approversElem) {
				var approversElemArr = WCXML.GetFirstLevelChilds(approversElem, "approver");
				// run over all approvers, read <approver> element and put contents in approverObj
				_.each(approversElemArr, function (approverElem) {
					if (approverElem) {
						var approver = Approver.fromXML(approverElem, projectMembers, usersWithNoAccess);
						if (approver) {
							stepObj.approvers.push(approver);
						}
					}
				}.bind(this));
			}

			stepObj._emit("update");
			return stepObj;
		};

		/**
		 * Check if the dueDate property of this stage can be modified
		 * @returns {boolean}
		 */
		ApprovalStage.prototype.canModifyDueDate = function () {
			return this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED;
		};

		/**
		 * Check if the list of appropvers of this stage can be modified
		 * @returns {boolean}
		 */
		ApprovalStage.prototype.canModifyApprovers = function () {
			return this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED ||
				this.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING;
		};

		/**
		 * Writes the state of the stage to XML
		 * @param   {integer} stageNumber
		 * @returns {string}
		 */
		ApprovalStage.prototype.writeToXML = function (stageNumber) {
			var xmlcode = '<stage>';
			xmlcode += WCXML.ElementEncode('stageId', this.id);
			xmlcode += WCXML.ElementEncode('stageName', this.approvalModel.isStagedApprovalAllowed() ? this.name : '');
			xmlcode += WCXML.ElementEncode('stageNumber', stageNumber + 1);
			if (this.dueDate && this.dueDate.constructor == Date) {
				xmlcode += WCXML.ElementEncode('stageDueDateStamp', this.dueDate.getTime());
			}
			if (this.leadTime) {
				xmlcode += WCXML.ElementEncode('stageLeadTime', this.leadTime);
			}
			xmlcode += WCXML.ElementEncode('stageApprovalStatus', this.approvalStatus);
			xmlcode += WCXML.ElementEncode('uponRejection', this.rejectHandle);
			xmlcode += WCXML.ElementEncode('requireReviewState', this.requireReviewHandle);

			// when the conditional approval is not allowed on system level, put it on false
			if (this.approvalModel.systemCondApprovalAllowed) {
				xmlcode += WCXML.ElementEncode('allowConditionalApproval', this.condApp);
			} else {
				xmlcode += WCXML.ElementEncode('allowConditionalApproval', false);
			}


			// save approvers
			// Remark that we only save who need to approve. We don't write their current approval status
			// since this status must be the same as before (the client cannot change approval statuses on its own
			xmlcode += '<approvers>';
			if (this.approvers) {
				_.each(this.approvers, function (approverObj) {
					xmlcode += approverObj.writeToXML();
				});
			}
			xmlcode += '</approvers>';

			xmlcode += '</stage>';
			return xmlcode;
		};


		/**
		 * remove this Approval Stage
		 */
		ApprovalStage.prototype.remove = function () {
			this.approvalModel.removeStage(this);
		};

		/**
		 * Removes an approver at the specified position from this stage
		 * @param {integer} approverIndex
		 */
		ApprovalStage.prototype.removeApprover = function (approverIndex) {
			var appArr = this.approvers;
			if (!appArr) {
				throw ("appArr is null");
			}
			if (approverIndex < appArr.length && approverIndex >= 0) {
				appArr.splice(approverIndex, 1);
				this._emit("changed.approvers.removed");
			} else {
				throw ("approverIndex out of bounds of approvers array");
			}
		};

		/**
		 * Removes all approvers from this stage
		 */
		ApprovalStage.prototype.removeAllApprovers = function () {
			if (!_.isEmpty(this.approvers)) {
				this.approvers = [];
				this._emit("changed.approvers.removeall");
			}
		};

		/*
		 * Changes the type of approver at the specified position
		 * @param {integer} approverIndex
		 * @param {string} newGroupType  the new type of approver
		 */
		ApprovalStage.prototype.changeGroupApprover = function (approverIndex, newGroupType) {
			var approverObj = this.approvers[approverIndex];
			if (approverObj && newGroupType) {
				approverObj.approvalType = newGroupType;
				this._emit("changed.approvers.modified", approverObj);
			}
		};

		/**
		 * Sets a new due date
		 * @param {string} dueDate
		 */
		ApprovalStage.prototype.setDueDate = function (dueDate) {
			if (this.dueDate !== dueDate) {
				this.dueDate = dueDate;
				this._emit("changed.duedate");
			}
		};

		/**
		 * Sets the lead time
		 * @param {int} leadTime
		 */
		ApprovalStage.prototype.setLeadTime = function (leadTime) {
			if (this.leadTime !== leadTime) {
				this.leadTime = leadTime;
				this._emit("changed.leadtime");
			}
		};

		/**
		 * Get the name of this stage
		 * @returns {string}
		 */
		ApprovalStage.prototype.getStageName = function () {
			return this.name;
		};

		/**
		 * Change the name of the stage
		 * @param {string} stageName
		 */
		ApprovalStage.prototype.setStageName = function (stageName) {
			if (this.name === stageName || typeof stageName !== "string") {
				// nothing changed
				return;
			}
			this.name = stageName;
			this._emit("changed.name");
		};

		/**
		 * Checks wether or not thet stage has a valid stage name and returns an i18n key that indicates the error message to be shown.
		 * If it is a valid name, returns the empty string.
		 */
		ApprovalStage.prototype.hasValidStageName = function () {
			if (this.approvalModel.isComplexApproval() && (!this.name || !(this.name.trim()))) {
				return "CAppEmptyNameNotAllowed";
			} else if (!this.approvalModel.isValidStageName(this)) {
				return "CAppSameNameNotAllowed";
			} else {
				return "";
			}
		};

		/**
		 * Formats the current lead time of this stage in days and hours
		 * @returns {object} This object contains the properties 'days' and 'hours'
		 */
		ApprovalStage.prototype.getLeadTimeInDaysAndHours = function () {
			var leadTimeHours = this.leadTime / 3600000; // ms to hours conversion
			var leadTimeDays = Math.floor(leadTimeHours / 24);
			leadTimeHours = Math.round(leadTimeHours - 24 * leadTimeDays);
			return {
				days: leadTimeDays,
				hours: leadTimeHours
			};
		};

		/**
		 * Change the reject handle for this stage
		 * @param {String} newRejectHandle See ApprovalConstants.RejectHandler
		 */
		ApprovalStage.prototype.setRejectHandle = function (newRejectHandle) {
			if (this.rejectHandle !== newRejectHandle) {
				this.rejectHandle = newRejectHandle;
				this._emit("changed.rejecthandle");
			}
		};

		/**
		 * Change the require review handle for this stage
		 * @param {string} newRequireReviewHandle see ApprovalConstants.requireReviewHandlers
		 */
		ApprovalStage.prototype.setRequireReviewHandle = function (newRequireReviewHandle) {
			if (this.requireReviewHandle !== newRequireReviewHandle) {
				this.requireReviewHandle = newRequireReviewHandle;
				this._emit("changed.requirereviewhandle");
			}
		};

		/**
		 * Change the conditional approval boolean for this stage
		 * @param {boolean} newConditionalApproval
		 */
		ApprovalStage.prototype.setConditionalApproval = function (newConditionalApproval) {
			if (this.condApp !== newConditionalApproval) {
				this.condApp = newConditionalApproval;
				this._emit("changed.condapp");
			}
		};

		/**
		 * Adds approvers to the data model of a step 
		 * The approver to be added is the one selected in the add approver dropdown of parameter step
		 * @param step: Index of the step to which the approver will be added
		 */
		ApprovalStage.prototype.addApprover = function (member) {
			if (member) {
				var isNotAddedYet = !_.some(this.approvers, function (approver) {
					var memberObj = approver.memberObj;
					return (member.userId && member.userId === memberObj.userId) ||
						(member.groupId && member.groupId === memberObj.groupId);
				});
				if (isNotAddedYet) {
					var approvalType = member.userId ? "User" : member.isRole ? "OneOfRole" : "OneOfGroup";
					var approver = new Approver(approvalType, member);
					this.approvers.push(approver);
					this._emit("changed.approvers.added", approver);
				}
			}
		};

		/**
		 * Get the current list of approvers for this stage
		 * @returns {Array<Approver>}
		 */
		ApprovalStage.prototype.getApprovers = function () {
			return this.approvers;
		};

		/**
		 * Checks if the name of this stage can be changed.
		 * @returns {boolean} True if name can be changed, false otherwise.
		 */
		ApprovalStage.prototype.canModifyStageName = function () {
			return !this.approvalModel.readOnly &&
				this.approvalModel.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED &&
				this.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED;
		};

		return ApprovalStage;
	});
