define(["utils/Observable",
		"underscore"
	],
	function (
		Observable,
		_) {
		"use strict";

		/**
		 * ApproveRejectViewModel is used in conjunction with ApproveRejectView 
		 * @param {string} actionURL
		 * @param {boolean} isEncapsulated - indicates if it is used outside the approval setup page or not (e.g. viewers)
		 * @param {boolean} isViewFile
		 * @param {function} onUpdateCallback
		 */
		function ApproveRejectViewModel(actionURL, isEncapsulated, isViewFile, onUpdateCallback) {
			Observable.mixin(this, "approval");

			this.actionURL = actionURL;
			this.isEncapsulated = isEncapsulated;
			this.isViewFile = isViewFile;
			// Callbacks still used by viewer
			this.onUpdateCallback = onUpdateCallback;
			this.approveRejectCallback = null;
			this.pageRange = null;
		}

		/** 
		 * Set the page range for page list documents. It allows the approval af certain pages, instead of all at once.
		 * @param {String} [pageRange] - Range that needs to be approved. E.g. "1-3, 7, 9-10"
		 * @param {Boolean} [convert] - True when we get user input (e.g. "7-11" should become "6-10")
		 * @param {Boolean} [failWhenEmpty] - If true, an error will be thrown when the pageRange is empty
		 */
		ApproveRejectViewModel.prototype.setPageRange = function (pageRange, convert, failWhenEmpty) {
			var errMsg = "CAppInvalidPageRangeExample";
			var convertedRange = "";
			if (pageRange !== undefined) {
				pageRange = String(pageRange);
			}
			if (!pageRange || !pageRange.trim()) {
				if (failWhenEmpty) {
					return "CAppEmptyPageRange";
				}
				this.pageRange = null;
			} else {
				if (pageRange.trim() != "all") {
					var first = true;
					// split comma separated values, resulting in 1-3, 7, and 9-10
					var ranges = pageRange.trim().split(',');
					for (var i = 0; i < ranges.length; i++) {
						if (!first) {
							convertedRange += ",";
						} else {
							first = false;
						}
						// split range 1-3 into 1 and 3; this should always have 1 or 2 values (1 if single page, 2 if range)
						var values = ranges[i].trim().split('-');
						if (values.length < 1 || values.length > 2) {
							return errMsg;
						}
						for (var j = 0; j < values.length; j++) {
							var val = values[j].trim();
							if (isNaN(parseInt(val))) {
								return errMsg;
							}
							if (j === 1) {
								var prevVal = values[0].trim();
								if (parseInt(val) < parseInt(prevVal)) {
									return errMsg;
								}
								convertedRange += "-";
							}
							convertedRange += parseInt(values[j]) - 1;
						}
					}
				}
				if (convert) {
					this.pageRange = convertedRange;
				} else {
					this.pageRange = pageRange;
				}
			}
			return "";
		};

		/** 
		 * Add and extra action that will be executed before approval/rejection is submitted.
		 * @param {Funtion} [action] - action to be executed
		 */
		ApproveRejectViewModel.prototype.setBeforeSubmitAction = function (action) {
			this.beforesubmit = action;
		};

		/**
		 * Set if the position of the popup have to be adjusted. Set this when the buttons are on the bottom of the page.
		 * @param {Boolean} adjustPopupHeight - determines if the position will be ajusted
		 * @param {Function} callback         - called after the position of the popup is adjusted
		 */
		ApproveRejectViewModel.prototype.setAdjustPopupHeight = function (adjustPopupHeight, callback) {
			this.adjustPopupHeight = adjustPopupHeight;
			this.toggleAction = callback;
		};

		/**
		 * Sends an event with the data to be submitted
		 * @param {object} data
		 */
		ApproveRejectViewModel.prototype.submit = function (data) {
			this._emit("submit", data);
		};

		return ApproveRejectViewModel;
	});
