define(["approval/ApprovalConstants",
		"approval/ApprovalUtils",
		"wcr/webcenter_xml",
		"underscore"
	],
	function (ApprovalConstants,
		ApprovalUtils,
		WCXML,
		_) {
		"use strict";

		/**
		 * Constructor of an individual Approver. This object is valid for the current stage it is in.
		 * @param {ApprovalConstants.ApprovalType} approvalType - the type of approval
		 * @param {Object} memberObj
		 */
		function Approver(approvalType, memberObj) {
			this.approvalType = approvalType;
			this.approvalStatus = null;
			this.approvedVersion = null;
			this.approvedByUserId = null;
			this.approvalTime = null;
			this.memberObj = memberObj;
			this.commentArr = [];
		}

		/**
		 * Reads approval stages (steps) from XML
		 * @param xml: parent element containing all the stages
		 * @param stepArr: array of stepObj in which the steps will be written
		 */
		Approver.fromXML = function (xml, projectMembers, usersWithNoAccess) {
			var doAddApprover = true;
			var approverObj = new Approver();
			approverObj.approvalType = WCXML.GetElementText(xml, "approvalType");
			approverObj.approvalStatus = WCXML.GetElementText(xml, "approvalStatus");
			approverObj.approvedVersion = WCXML.GetElementText(xml, "approvedVersion");
			// read approval comments (the approvalComment element can be repeated!)
			var commentElemArr = WCXML.GetFirstLevelChilds(xml, "approvalComment");
			if (commentElemArr) {
				for (var comment = 0; comment < commentElemArr.length; comment++) {
					var commentObj = new Object;
					commentObj.commentText = WCXML.GetElementText(commentElemArr[comment], "comment");
					commentObj.commentByUserId = WCXML.GetElementText(commentElemArr[comment], "commentByUserId");
					var commentTime = parseInt(WCXML.GetElementText(commentElemArr[comment], "commentTime"));
					if (commentTime && !isNaN(commentTime)) {
						commentObj.commentTime = new Date(commentTime);
					} else {
						commentObj.commentTime = null;
					}
					commentObj.commentVersion = WCXML.GetElementText(commentElemArr[comment], "commentVersion");
					// hadw added if statement to cater for <comment></comment> which apparently is sent by the back-end
					if (commentObj.commentText) {
						approverObj.commentArr.push(commentObj);
					}
				}
			}

			approverObj.approvedByUserId = WCXML.GetElementText(xml, "approvalApprovedByUserId");
			var approvalTime = parseInt(WCXML.GetElementText(xml, "approvalTime"));
			if (approvalTime && !isNaN(approvalTime)) {
				approverObj.approvalTime = new Date(approvalTime);
			} else {
				approverObj.approvalTime = null;
			}

			// fetch approver member object 
			if (approverObj.approvalType === ApprovalConstants.ApprovalType.USER) {
				var userId = WCXML.GetElementText(xml, "userId");
				// go searching for the memberObj in the current project members             
				approverObj.memberObj = ApprovalUtils.findMemberObj("User", userId, projectMembers);
				// if you don't find this member, go looking in the users with no access
				if (!approverObj.memberObj) {
					approverObj.memberObj = ApprovalUtils.findMemberObj("User", userId, usersWithNoAccess);

					//JAMI: in case the memberObj is still not initialized > the User Approver has no VIEW rights for the parent folder/project - just skip him
					//NOTE: I consider this to be an intermediate fix only - we should record potential Approver's without view permissions properly, imo
					if (!approverObj.memberObj) {
						doAddApprover = false;
					}
				}
			} else {
				// group
				var groupId = WCXML.GetElementText(xml, "groupId");
				approverObj.memberObj = ApprovalUtils.findMemberObj("Group", groupId, projectMembers);
				if (!approverObj.memberObj) {
					approverObj.memberObj = ApprovalUtils.findMemberObj("Group", groupId, usersWithNoAccess);

					//JAMI: in case the memberObj is still not initialized > the Group Approver has no VIEW rights for the parent folder/project - just skip him
					//NOTE: I consider this to be an intermediate fix only - we should record potential Approver's without view permissions properly, imo
					if (!approverObj.memberObj) {
						doAddApprover = false;
					}
				}
			}

			if (doAddApprover) {
				return approverObj;
			}

			return null;
		};

		/**
		 * Write the state of the approver to XML
		 * @returns {string} 
		 */
		Approver.prototype.writeToXML = function () {
			var approvalType = this.approvalType;
			var xmlcode = '<approver>';
			xmlcode += WCXML.ElementEncode('approvalType', approvalType);
			if (approvalType === ApprovalConstants.ApprovalType.USER) {
				if (!this.memberObj) {
					window.alert("No memberObj for approver");
					return;
				}
				if (!this.memberObj.userId) {
					throw ("No userId for approver");
				}
				xmlcode += WCXML.ElementEncode('approverId', this.memberObj.userId);
			} else if (approvalType === ApprovalConstants.ApprovalType.ONE_OF_GROUP 
					   || approvalType === ApprovalConstants.ApprovalType.ALL_OF_GROUP 
					   || approvalType === ApprovalConstants.ApprovalType.ONE_OF_ROLE) {
				if (!this.memberObj.groupId) {
					throw ("No groupId for approver");
				}
				xmlcode += WCXML.ElementEncode('approverId', this.memberObj.groupId);
			} else {
				window.alert("Problem writing approvers.  Unexpected approvalType: " + approvalType);
			}
			xmlcode += '</approver>';

			return xmlcode;
		};

		return Approver;
	}
);
