define(["approval/ApprovalConstants",
		"approval/ApprovalUtils",
		"approval/view/ApproversTable",
		"component/tooltip/Tooltip",
		"Template",
		"tpl!approval/view/template/ApprovalStage",
		"tpl!approval/view/template/HourDropDown",
		"utils/Observer",
		"utils/webi18n!Approval",
		"approval/view/ApprovalXDrop",
		"utils/datetools",
		"jquery",
		"underscore"
	],
	function (ApprovalConstants,
		ApprovalUtils,
		ApproversTable,
		Tooltip,
		Template,
		tpl,
		hourTpl,
		Observer,
		i18n,
		ApprovalXDrop,
		datetools,
		$,
		_) {

		"use strict";

		/**
		 * This view represents an individual stage.
		 * @param {ApprovalStage} approvalStageModel
		 */
		function ApprovalStageView(approvalStageModel) {
			Observer.mixin(this);

			this.model = approvalStageModel;

			this._observe(this.model, "changed.number", this._updateNumber.bind(this));
			this._observe(this.model, "changed.number", this._renderRejectHandleOptions.bind(this));

			var languageCode = window.webcenterConfig.userLang || "en";
			this._helpArr = {};
			this._helpArr['CAppRejectionHandlerHelp'] = "./help/RejectionHandlerHelp_" + languageCode + ".htm";
			this._helpArr['CAppRequireReviewStateHelp'] = "./help.jsp?topic=common/wc/task/ta_wc_RequireAnnotationsReview-Approval&lang=" + languageCode;

			this.approvalsTableView = new ApproversTable(this.model);

			//initialize the template to be used for this popup
			this.t = new Template(i18n, tpl);
			this.t.bind("click", "div.panel-heading .action-remove-stage", this.removeStepDialog, this);
			this.t.bind("click", 'a.inline_help', function (element) {
				var helpSection = $(element).attr("help-info");
				var popw = window.open(this._helpArr[helpSection], helpSection);
				popw.focus();
			}, this);
			this.t.bind("change", '[name=ConditionalApproval]', this.changeCondApproval, this);
			this.t.bind("change", "[name=RejectionSelect]", this.changeRejection, this);
			this.t.bind("change", "[name=RequireReviewSelect]", this.changeRequireReview, this);
			this.t.bind("click", "[name=collapse-button]", function (element) {
				$("[name=collapse-target]", this.elem).collapse("toggle");
			}, this);
			this.t.bind("show.bs.collapse", "[name=collapse-target]", function (el, event) {
				if (event.target === el) {
					$("[name=collapse-button]", this.elem).removeClass("collapsed");
				}
			}, this);
			this.t.bind("hide.bs.collapse", "[name=collapse-target]", function (el, event) {
				if (event.target === el) {
					$("[name=collapse-button]", this.elem).addClass("collapsed");
				}
			}, this);

			this.t.bind("click", "[name=collapse-details-button]", function (element) {
				var mainPanel = $("[name=collapse-target]", this.elem);
				var targetPanel = $("[name=collapse-details-target]", this.elem);
				if (mainPanel.attr("aria-expanded") !== "true") {
					targetPanel.add(mainPanel).collapse("show");
				} else {
					targetPanel.collapse("toggle");
				}
			}, this);
			this.t.bind("click", "[name=xdrop-addon]", function (el, event) {
				event.stopPropagation();
				this._approverXDrop.toggle();
			}, this);


			this._observeAll(this.model.approvalModel, ["stage.*.changed.name", "changed.stages.**"], this._renderStageName.bind(this));
			this._observe(this.model.approvalModel, "changed.duedatetype", this._renderDueDate.bind(this));
			this._observeAll(this.model.approvalModel, ["changed.stages.deleted", "changed.stages.added"], this._updateActions.bind(this));
			this._observeAll(this.model, ["changed.duedate", "changed.leadtime"], this._renderDueDate.bind(this));
			this._observe(this.model, "deleted", function () {
				this.elem.remove();
				this._stopObserveAll();
			}.bind(this));

			this._approverXDrop = new ApprovalXDrop(approvalStageModel, approvalStageModel.approvalModel.getProjectId());

			this._approverXDrop.on("selected", approvalStageModel.addApprover.bind(approvalStageModel));

			this._observe(this.model, "changed.approvers.added", function () {
				this._approverXDrop.setValue(null);
			}.bind(this));
			this.elem = $("<div>");
			this.t.setContext(this.elem);
		}

		/**
		 * Callback that is fired whenever a change happens in the list of stages. This will update the number in the header of this stage.
		 * @private
		 */
		ApprovalStageView.prototype._updateNumber = function () {
			if (this.elem) {
				this.elem.find(".stageNumber").text(this.model.number);
			}
		};

		/** 
		 * Displays the header of the complete Complex Approval
		 */
		ApprovalStageView.prototype.render = function () {
			if (this.elem) {
				this._approverXDrop.element.detach();
				this._renderStep();
				this._renderApproverTable();
				this._renderDueDate();
				this._renderRejectHandleOptions();
				this._renderXDrop();
				this._renderStageName();
				this._updateActions();
			}
		};

		/**
		 * Displays one step of the Approval
		 * @param 
		 */
		ApprovalStageView.prototype._renderStep = function () {
			this._approverXDrop.element.detach();
			var step = this.model.number - 1;
			this.t.d.step = step;
			this.t.d.model = this.model;
			this.t.d.readOnly = this.model.approvalModel.readOnly;

			// editability
			var stepEditable = 2;
			if (this.model.approvalModel.readOnly) {
				stepEditable = 0;
			}
			if (this.model.approvalModel.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				stepEditable = 0;
			}
			if (this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED && this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.PENDING) {
				stepEditable = 0;
			}
			if (stepEditable === 2 && this.model.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
				stepEditable = 1;
			}

			this.t.d.stepEditable = stepEditable;
			this.t.d.approvalStatus = this.model.approvalStatus;

			this.t.d.images = ApprovalConstants.images;
			this.t.d.approvalTime = '';
			// step approval status
			if (this.model.approvalStatus != ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				if (this.model.approvalStatus != ApprovalConstants.ApprovalStatus.PENDING) {
					var approvalTime = -1;
					for (var approver = 0; approver < this.model.approvers.length; approver++) {
						var stepApprovalTime = this.model.approvers[approver].approvalTime; // a date object
						if (stepApprovalTime && stepApprovalTime.constructor == Date) {
							stepApprovalTime = stepApprovalTime.getTime(); // convert to ms 
						}
						if (stepApprovalTime > approvalTime) {
							approvalTime = stepApprovalTime;
						}
					}
					if (approvalTime > 0) {
						this.t.d.approvalTime = ApprovalUtils.getDateHourString(new Date(approvalTime));
					}
				}
			}

			this.t.d.showcollapse = false;
			if (this.model.approvalStatus != ApprovalConstants.ApprovalStatus.NOT_STARTED && this.model.approvalModel.readOnly === false) { // people who can edit must see the setup as readonly
				this.t.d.showcollapse = true;
				// conditional approval in compact form
				this.t.d.allowConditional = false;
				if (this.model.systemCondApprovalAllowed || this.model.condApp) {
					this.t.d.allowConditional = true;
				}

				// At rejection in compact form
				this.t.d.rejectHandle = null;
				if (this.model.rejectHandle != 'RejectStopsApprovalAfterStep' || this.model.approvalModel.isComplexApproval()) {
					this.t.d.rejectHandle = this.model.rejectHandle;
				}
			}

			if (this.model === _.last(this.model.approvalModel.steps)) {
				this.t.d.optionArr = ApprovalConstants.rejectionHandlersSimple;
			} else {
				this.t.d.optionArr = ApprovalConstants.rejectionHandlersComplex;
			}

			if (stepEditable) {
				this.t.d.requireReviewHandlers = ApprovalConstants.requireReviewHandlers;
			}

			var extraPanelClass = '';
			if (this.t.d.showcollapse && !this.t.d.showBottom) {
				extraPanelClass += " only-approvers-table";
			}
			extraPanelClass += ' panel-' + this.model.approvalStatus;

			this.t.d.extraPanelClass = extraPanelClass;
			this.t.d.extraPanelName = 'panel_' + step;

			this.t.render();

			var tooltips = $('[data-toggle="tooltip"]', this.elem);
			_.each(tooltips, function (tooltip) {
				new Tooltip(undefined, $(tooltip), this.elem, $("body"), false, $(tooltip).attr("data-title"), undefined, undefined, {
					prefPosition: $(tooltip).attr("data-placement")
				});
			}.bind(this));

		};

		/**
		 * Renders the table of approvers
		 * @private
		 */
		ApprovalStageView.prototype._renderApproverTable = function () {
			this.approvalsTableView.setElement($("[name=ApproversTable]", this.elem));
		};


		/**
		 * Displays the confirm dialog when attempting to remove a stage
		 * @param {jQuery} el
		 * @param {event}   e
		 */
		ApprovalStageView.prototype.removeStepDialog = function (el, e) {
			e.preventDefault();
			e.stopPropagation();
			this._confirm(i18n.t('CAppConfirmRemoveStep'), function () {
				this.model.remove();
			}.bind(this));

		};

		/**
		 * Display a browser native confirm dialog
		 * @private
		 * @param {string} message
		 * @param {function} confirmCallback
		 * @param {function} declineCallback
		 */
		ApprovalStageView.prototype._confirm = function (message, confirmCallback, declineCallback) {
			if (window.confirm(message)) {
				if (_.isFunction(confirmCallback)) {
					confirmCallback();
				}
			} else if (_.isFunction(declineCallback)) {
				declineCallback();
			}

		};

		/**
		 * sets the conditional approval flag of a stage in the data model
		 * @param elem: checkbox of the conditional approval
		 * @param step: the step
		 */
		ApprovalStageView.prototype.changeCondApproval = function (checkbox) {
			this.model.setConditionalApproval(checkbox.checked);
		};

		/**
		 * Copies the rejection handler from the UI to the data model when changed
		 */
		ApprovalStageView.prototype.changeRejection = function (select, event) {
			this.model.setRejectHandle(select.value);
		};

		/**
		 * Copies the require review state handler from the UI to the data model when changed
		 */
		ApprovalStageView.prototype.changeRequireReview = function (select) {
			this.model.setRequireReviewHandle(select.value);
		};


		/**
		 * Responsible for rendering the dates in the footer. This function will determine what kind of date is present: regular due dates, lead times, uneditable due dates, ...
		 * @private
		 */
		ApprovalStageView.prototype._renderDueDate = function () {
			var dueDateDiv = $("[name=due-date-footer]", this.elem);
			if (this.model.approvalModel.readOnly ||
				this.model.approvalModel.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED ||
				this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				dueDateDiv.html(this._renderReadOnlyFooter());
				dueDateDiv.removeClass("hidden");
			} else {
				var dueDateType = this.model.approvalModel.dueDateType;
				if (dueDateType === "FixedDueDates") {
					dueDateDiv.html(this._renderFixedDueDatesFooter());
					dueDateDiv.removeClass("hidden");
				} else if (dueDateType === "LeadTimes" || dueDateType === "LeadTimesFinalDueDate") {
					if (this.model.approvalStatus == 'NotStarted') {
						dueDateDiv.html(this._renderLeadTimesFooter());
					} else {
						dueDateDiv.html(this._renderFixedDueDatesFooter());
					}
					dueDateDiv.removeClass("hidden");
				} else {
					dueDateDiv.addClass("hidden");
				}
			}
		};

		/**
		 * Renders the xdrop
		 * @private
		 */
		ApprovalStageView.prototype._renderXDrop = function () {
			var footer = $("[name=footer]", this.elem);
			var xdrop = $(".approvers-xdrop", this.elem);
			xdrop.append(this._approverXDrop.element);
			footer.removeClass("hidden");
		};

		/**
		 * Renders the calendar and time input fields that control due date.
		 * @private
		 * @returns {jQuery}
		 */
		ApprovalStageView.prototype._renderFixedDueDatesFooter = function () {
			var content = $('<div>');
			var showDate = datetools.fixDateSummerWinterTime(this.model.dueDate, false);
			var tpl = new Template(i18n, hourTpl);
			tpl.d.d = {
				dueHour: showDate.getHours() * 2 + Math.floor(showDate.getMinutes() / 30),
				name: "hourdropdown",
				id: "CAppDueHour_" + (this.model.number - 1)
			};

			var calendar = $('<input>', {
				"class": "form-control",
				"data-sahi-id": "CAppDueDateText_" + (this.model.number - 1),
				value: datetools.getDateString(showDate),
				type: "text",
				name: "datepicker"
			});
			var hourdropdown = $(tpl.getRenderedHtml());
			var hourdropdowninput = hourdropdown.find("[name=hourdropdown]");
			var calendInputGroupAddon = $('<span class="form-datetime-btn input-group-addon">');
			calendInputGroupAddon.on('click', function () {
				$(calendar).datepicker("show");
			});
			content
				.append(hourdropdown)
				.append(
					$('<div class="input-group input-group-datepicker">')
					.append(calendar)
					.append(
						calendInputGroupAddon.append(
							$('<i class="icon icon-WCR_Calendar" >')
						)
					)
				);


			this._createCalendar(calendar, hourdropdowninput);

			var onChange = function () {
				var dateString = calendar.val();
				var doubleHour = hourdropdowninput.val();
				var formattedDateString = datetools.getDateString(dateString);
				if (formattedDateString) {
					var newDate = new Date(formattedDateString);
					var hours = Math.floor(doubleHour / 2);
					var minutes = (hours * 2 == doubleHour) ? 0 : 30;
					newDate.setHours(hours);
					newDate.setMinutes(minutes);
					newDate.setSeconds(0);
					var newDueDate = datetools.fixDateSummerWinterTime(newDate, true);
					if (isNaN(newDueDate.getTime())) {
						// Invalid date entered, reset calendar field to current model value
						calendar.val(datetools.getDateString(datetools.fixDateSummerWinterTime(this.model.dueDate, false)));
					} else {
						this.model.setDueDate(newDueDate);
			}
				} else {
					// No correct date entered, resetting to current model value
					calendar.val(datetools.getDateString(datetools.fixDateSummerWinterTime(this.model.dueDate, false)));
				}
			}.bind(this);

			calendar.change(onChange);
			hourdropdowninput.change(onChange);

			return content;
		};

		/**
		 * Renders the day and hours input fields that control the lead time of this stage
		 * @private
		 * @returns {jQuery}
		 */
		ApprovalStageView.prototype._renderLeadTimesFooter = function () {
			var content = $('<div>');

			if (!this.model.leadTime) {
				this.model.setLeadTime(24 * 3600000); // 1 day
			}

			var leadTime = this.model.getLeadTimeInDaysAndHours();

			var leadTimeHoursInput = $("<input>", {
				"class": "form-control",
				"data-sahi-id": "leadTimeHours_" + (this.model.number - 1),
				type: "text",
				value: leadTime.hours
			});

			var leadTimeDaysInput = $("<input>", {
				"class": "form-control",
				"data-sahi-id": "leadTimeDays_" + (this.model.number - 1),
				type: "text",
				value: leadTime.days
			});

			content
				.append(
					$('<div class="input-group input-group-leadtime">')
					.append(leadTimeHoursInput)
					.append(
						$('<span class="input-group-addon">').append(i18n.t("CAppHours"))
					)
				)
				.append(
					$('<div class="input-group input-group-leadtime">')
					.append(leadTimeDaysInput)
					.append(
						$('<span class="input-group-addon">').append(i18n.t("CAppDays"))
					)
				);

			var renderCurrentLeadTime = function () {
				var currentLeadTime = this.model.getLeadTimeInDaysAndHours();
				leadTimeDaysInput.val(currentLeadTime.days);
				leadTimeHoursInput.val(currentLeadTime.hours);
			}.bind(this);


			var onChange = function () {
				var days = parseInt(leadTimeDaysInput.val(), 10);
				var hours = parseInt(leadTimeHoursInput.val(), 10);

				if (isNaN(days) || days < 0) {
					renderCurrentLeadTime();
				} else if (isNaN(hours) || hours < 0) {
					renderCurrentLeadTime();
				} else {
					var maxHours = 8760; // = 365 days x 24 hours
					var totalHours = days * 24 + hours;
					if (totalHours <= maxHours) {
						this.model.setLeadTime(totalHours * 3600000);
					} else {
						var newLeadTime = maxHours * 3600000;
						if (newLeadTime !== this.model.leadTime) {
							this.model.setLeadTime(maxHours * 3600000);
						} else {
							renderCurrentLeadTime();
						}
					}

				}
			}.bind(this);

			leadTimeHoursInput.change(onChange);
			leadTimeDaysInput.change(onChange);

			return content;
		};

		/**
		 * Render the read only (uneditable) duedate or leadtime.
		 * @private
		 * @returns {jQuery}
		 */
		ApprovalStageView.prototype._renderReadOnlyFooter = function () {
			var content = $("<div>");
			if (this.model.dueDate) {
				return content.append(
					$("<span>")
					.append($("<strong>").text(i18n.t("CAppDueDate")))
					.append(ApprovalUtils.getDateHourString(this.model.dueDate))
				);
			} else if (this.model.leadTime) {
				var currentLeadTime = this.model.getLeadTimeInDaysAndHours();
				return content.append(
					$("<span>")
					.append($("<strong>").text(i18n.t("CAppLeadTime")))
					.append(currentLeadTime.days + i18n.t("CAppDaysAbbreviation") + " " + currentLeadTime.hours + i18n.t("CAppHoursAbbreviation"))
				);
			}
		};

		/**
		 * Create a calendar from the given input field
		 * @private
		 * @param {jQuery} calendarEl
		 * @param {jQuery} hourdropdownEl
		 */
		ApprovalStageView.prototype._createCalendar = function (calendarEl, hourdropdownEl) {
			$(calendarEl).datepicker({
				dateFormat: "M dd, yy",
				onClose: function () {
					var element = $(hourdropdownEl)[0];
					if (element && element.selectedIndex === 0) {
						element.selectedIndex = 37;
					}
				}
			});
		};

		/**
		 * Render reject handlees
		 * @private
		 */
		ApprovalStageView.prototype._renderRejectHandleOptions = function () {
			var select = $("[name=RejectionSelect]", this.elem);
			var options;
			if (this.model === _.last(this.model.approvalModel.steps)) {
				options = ApprovalConstants.rejectionHandlersSimple;
			} else {
				options = ApprovalConstants.rejectionHandlersComplex;
			}

			var optionElements = [];
			_.each(options, function (option) {
				var text;
				if (option === "RejectStopsApprovalAfterStep" && !this.model.approvalModel.isComplexApproval()) {
					text = i18n.t("CAppStopApprovalAfterEverybodyDecided");
				} else {
					text = i18n.t("CApp" + option);
				}
				optionElements.push($("<option>", {
					value: option,
					text: text
				}));
			}.bind(this));

			select.html(optionElements);
			select.val(this.model.rejectHandle);
		};

		/**
		 * Renders the stage name, this will be an input field when appropiate. Or an uneditable label otherwise.
		 * @private
		 */
		ApprovalStageView.prototype._renderStageName = function () {
			var container = $("[name=stage-name]", this.elem);
			if (container.length > 0) {
				container.empty();
				if (!this.model.approvalModel.isStagedApprovalAllowed()) {
					// Not allowed to show/change the name if staged approval is not allowed
					return;
				}
				if (this.model.canModifyStageName()) {
					container.addClass("editable");
					var isValidStageNameMsg = this.model.hasValidStageName();
					if (isValidStageNameMsg) {
						container.addClass("has-error");
					} else {
						container.removeClass("has-error");
					}
					container
						.append(
							$("<input>", {
								"class": "form-control stageNameEditable",
								type: "text",
								value: this.model.name,
								on: {
									change: function (e) {
										this.model.setStageName(e.currentTarget.value);
									}.bind(this)
								},
								"data-sahi-id": "stepName_" + (this.model.number - 1),
								placeholder: i18n.t("CAppStage")
							})
						)
						.append(
							$("<span>", {
								"class": "stageNameErrorMsg danger",
								text: i18n.t(isValidStageNameMsg)
							})
						);
				} else {
					container.removeClass("editable");
					container.append($("<label>", {
						"class": "stageName",
						text: this.model.name
					}));
				}
			}
		};

		/**
		 * Renders the stage name, this will be an input field when appropiate. Or an uneditable label otherwise.
		 * @private
		 */
		ApprovalStageView.prototype._updateActions = function () {
			var deleteAction = $(".action-remove-stage", this.elem);
			if (this.model.approvalModel.steps.length > 1) {
				deleteAction.removeClass("displayNone");
			} else {
				deleteAction.addClass("displayNone");
			}
		};

		return ApprovalStageView;
	});
