define([
		"approval/ApprovalConstants",
		"approval/view/ApproveButtonsToolbar",
		"approval/view/ApprovalStageView",
		"utils/webi18n!Approval",
		"utils/AnimationFrameRenderer",
		"jquery",
		"underscore"
	],
	function (
		ApprovalConstants,
		ApproveButtonsToolbar,
		ApprovalStageView,
		i18n,
		AnimationFrameRenderer,
		$,
		_) {
		"use strict";

		/**
		 * This view is used outside the approval setup page and shows depending on the status of the approval:
		 *      - the status
		 *      OR
		 *      - the action buttons for the approval (approve/reject/comment/force_approve/force_reject)
		 *      
		 * @class     
		 * @param {ApprovalModel} model
		 * @param {ApproveRejectViewModel} approveRejectViewModel
		 */
		function ApprovalStatusActionBar(model, approveRejectViewModel) {
			AnimationFrameRenderer.mixin(this);

			this.model = model;

			this._maxButtons = 6; //currently the maximum
			this._container = null;

			this.model.on("initialized", _.bind(function () {
				$("#ApprovalPanelDiv").empty();
			}, this));

			//Create sub-views
			this.approveButtonsToolbar = new ApproveButtonsToolbar(model, approveRejectViewModel);
		}

		/**
		 * Set the element where the component needs to be rendered
		 * @param {JQueryObject} elem
		 */
		ApprovalStatusActionBar.prototype.setElement = function (elem) {
			this._container = elem;
			this._container.addClass("approvalStatusActionBar");
		};

		/**
		 * Render
		 * (Before calling this, set the container of this component!)
		 * 
		 * @param {integer} maxButtons - maximum number of buttons that will be directly visible. 
		 *                             The remaining buttons will be hidden behind a "More..." button
		 */
		ApprovalStatusActionBar.prototype.render = function (maxButtons) {
			this._maxButtons = maxButtons;

			this._render();
		};

		/**
		 * The effective rendering of the component
		 * @private
		 */
		ApprovalStatusActionBar.prototype._doRender = function () {
			this._container.empty();
			if (this.model.approvalStatus && this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
				if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
					var buttons = $("<div>", {
						"class": "approval-buttons"
					});
					this.approveButtonsToolbar.setElement(buttons);
					this.approveButtonsToolbar.render(this._maxButtons, false);
					this._container.append(buttons);
				} else {
					this._container.append(this._renderStatusLabel());
				}
				this._container.append("&nbsp;");
				this._container.append(this._renderInfoElement());
			}			
		};


		/**
		 * Renders a label with the status of the approval
		 * @private
		 * @returns {JQueryObject}
		 */
		ApprovalStatusActionBar.prototype._renderStatusLabel = function () {
			return $("<span>", {
					'class': ApprovalConstants.statusLabels[this.model.approvalStatus]
				})
				.append($("<i>", {
					'class': ApprovalConstants.images[this.model.approvalStatus],
					title: this.model.approvalStatus
				}))
				.append(i18n.t("CApp" + this.model.approvalStatus));
		};

		/**
		 * Renders an info balloon
		 * @private
		 * @returns {JQueryObject}
		 */
		ApprovalStatusActionBar.prototype._renderInfoElement = function () {
			return $("<i>", {
				'class': "icon icon-normal icon-WCR_Info-Balloon show-cursor-pointer",
				title: i18n.t("CAppHelp"),
				on: {
					click: _.bind(this._openPopup, this)
				}
			});
		};

		/**
		 * Creates an overview of the approval stages
		 * @private
		 * @returns {JQueryObject}
		 */
		ApprovalStatusActionBar.prototype._getApprovalOverview = function () {
			var container = $("<div>");
			_.each(this.model.steps, _.bind(function (approvalStage) {
				var approvalStageView = new ApprovalStageView(approvalStage);
				approvalStageView.render();
				container.append(approvalStageView.elem);
			}, this));
			return container;
		};

		/**
		 * Opens the popup with the overview of the approval stages
		 * @private
		 */
		ApprovalStatusActionBar.prototype._openPopup = function () {
			if ($("#ApprovalPanelDiv").is(":empty")) {
				$("#ApprovalPanelDiv").append(this._getApprovalOverview());
			}

			var buttonObj = {};
			buttonObj[i18n.t("CAppClose")] = function () {
				$(this).dialog("close");
			};

			// get size of the browser's window and adapt the dialog size according to it.
			// Minimal size is 700x450px, which should be OK even for screens with 800x600 resolution.
			var viewportWidth = $(window).width();
			var viewportHeight = $(window).height();
			var dialogWidth = viewportWidth - viewportWidth / 10;
			var dialogHeight = viewportHeight - viewportHeight / 10;
			var dialogMinWidth = viewportWidth / 2;
			if (dialogWidth < 700) {
				dialogWidth = 700;
			}
			if (dialogHeight < 450) {
				dialogHeight = 450;
			}
			
			$("#ApprovalContainerDiv").css("min-width", dialogMinWidth + "px");
			$("#ApprovalContainerDiv").dialog({
				autoOpen: false,
				modal: false,
				closeOnEscape: true,
				draggable: true,
				height: 'auto',
				resizable: true,
				width: 'auto',
				buttons: buttonObj,
				maxHeight: dialogHeight,
				maxWidth: dialogWidth
			});
			
			$("#ApprovalContainerDiv").dialog("open");
		};


		return ApprovalStatusActionBar;
	});
