define(["Template",
		"approval/ApprovalConstants",
		"approval/ApprovalUtils",
		"approval/view/NotificationView",
		"approval/view/ApprovalStageContainer",
		"approval/view/ApproveRejectView",
		"approval/view/ApproveButtonsToolbar",
		"component/message/MessageComponent",
		"utils/Observer",
		"tpl!approval/view/template/ApprovalDueDateDialog",
		"tpl!approval/view/template/HourDropDown",
		"tpl!approval/view/template/Calendar",
		"utils/webi18n!Approval",
		"utils/AnimationFrameRenderer",
		"utils/dateShift",
		"utils/datetools",
		"jquery",
		"underscore",
		"wcr/webcenter_html",
		"wcr/webcenter_xml"
	],
	function (
		Template,
		ApprovalConstants,
		ApprovalUtils,
		NotificationView,
		ApprovalStageContainer,
		ApproveRejectView,
		ApproveButtonsToolbar,
		MessageComponent,
		Observer,
		ApprovalDueDateDialog,
		HourDropDownTmpl,
		CalendarTmpl,
		i18n,
		AnimationFrameRenderer,
		dateShift,
		datetools,
		$,
		_,
		WCHTML,
		WCXML) {
		"use strict";

		/**
		 * This view is responsible for rendering everything on the approval page. This includes all stages, the settings panel and multiple buttons at the top of the page
		 * @param {ApprovalModel} approvalModel
		 * @param {ApproveRejectViewModel} approveRejectViewModel
		 * @param {JQueryObj} displayElem
		 * @param {Object} currentUser
		 * @param {Boolean} isSimplePage - whether or not to show a simple view - 
		 								(should be refactored later to have two classes 'ApprovalView' and 'SimpleApprovalView', only one should be instantiated)
		 */
		function ApprovalView(approvalModel, approveRejectViewModel, displayElem, currentUser, isSimplePage) {
			AnimationFrameRenderer.mixin(this);
			Observer.mixin(this);

			this.model = approvalModel;
			this.currentUser = currentUser;


			this.displayElem = displayElem;
			this.loading(); // show that you are busy

			this.MSG_TAG = 'APPROVAL';
			this.stageViews = [];

			this._boundOnUpdate = this.onUpdate.bind(this);
			this._boundShowChangesApprovalMessage = _.debounce(this._toggleSaveButtonEnabled.bind(this), 10);

			this._stagesContainer = $("<div />", {
				"class": "col-lg-8 col-xs-12"
			});
			this._saveButton = null;
			this._messageModel = new MessageComponent.MessageModel();


			// Create sub-views
			this.approveRejectDialog = new ApproveRejectView(approvalModel, approveRejectViewModel);
			approveRejectViewModel.on("submit", this._submitForm.bind(this));
			this.approveButtonsToolbar = new ApproveButtonsToolbar(this.model, approveRejectViewModel);

			approvalModel.onAll(["update", "changed.approvalstatus", "initialized"], this._boundOnUpdate);
			approvalModel.onAll(["changed.*", "stage.*.changed.*", "stage.*.changed.approvers.*", "initialized"], this._boundShowChangesApprovalMessage);
			approvalModel.on("error", this._showErrors.bind(this));
			approvalModel.on("changed.stages.deleted", this._deleteStageView.bind(this));
			approvalModel.on("changed.stages.deleted", function(){
				var lastStage = _.last(this.model.steps);
				if(lastStage && lastStage.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED){
					// Request render because due date element will be no longer editable.
					this._renderDueDateElement();
				}
			}.bind(this));
			approvalModel.on("changed.stages.added", this._addStage.bind(this));
			approvalModel.on("changed.stages.added", function(){
				var secondLastStage = this.model.steps[this.model.steps.length - 2];
				var lastStage = _.last(this.model.steps);
				if((secondLastStage && secondLastStage.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED 
					  && lastStage.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED)){
					// Request render because due date elmeent becomes editable
					this._renderDueDateElement();
				}
			}.bind(this));
			approvalModel.on("submit", this._submitForm.bind(this));

			this._isSimplePage = isSimplePage === true;

			// Localized help  
			var languageCode = window.webcenterConfig.userLang || "en";
			this._helpArr = {};
			this._helpArr['CAppApprovalStatusHelp'] = "./help/ApprovalStatusHelp_" + languageCode + ".htm";
			this._helpArr['CAppDueDateHelp'] = "./help/DueDateHelp_" + languageCode + ".htm";
			this._helpArr['CAppApprovalMasterHelp'] = "./help/ApprovalMasterHelp_" + languageCode + ".htm";
			this._helpArr['CAppEditNextVersionHelp'] = "./help/EditNextVersionHelp_" + languageCode + ".htm";
			this._helpArr['CAppWhenNewVersionUploadedHelp'] = "./help/WhenNewVersionUploadedHelp_" + languageCode + ".htm";
		}

		/**
		 * Called when the model is updated
		 */
		ApprovalView.prototype.onUpdate = function () {
			this._loadingMsg.remove();
			this._render();
		};

		/**
		 * Shows a loading message. 
		 * (This message will be removed by calling the render method.)
		 */
		ApprovalView.prototype.loading = function () {
			this.displayElem.empty();
			this._loadingMsg = $("<div/>").html(i18n.t('CAppLoadingApprovalSetup'));
			this.displayElem.append(this._loadingMsg);
		};

		/**
		 * Displays the complete Approval (if not encapsulated)
		 */
		ApprovalView.prototype._doRender = function () {
			var body = $("<div/>", {
				"class": "container-fluid wcr-approval"
			});

			if (!this._messageWidget) {
				this._messageWidget = $("<div>", {
					"class": "messagebar messageWidget add-spacing-bottom"
				});
				new MessageComponent.MessageWidget(this._messageModel, this._messageWidget);
			}
			body.append(this._messageWidget);


			// header
			if (!this._isSimplePage) {
				if (!this._header) {
					this._header = $(".page-sub-title");
				}
				this._header.addClass("approvalTitle");
				body.append(this._header);

				if (this.model.documentId) {
					if (this._approvalStatusElem) {
						this._approvalStatusElem.remove();
					}
					if (!this.model.nextVersion) {
						this._approvalStatusElem = this._getStatusElement();
						this._approvalStatusElem.insertAfter(this._header);
					}
				}
			} //else approval status will be added later

			// body
			var form = this._getForm();
			body.append(form);


			if (this.model.errorObj) {
				this._showErrors(this.model.errorObj);
			} else {
				if (this.model.nextVersion) {
					var nextVersionMessage = $("<div>").addClass("next-version-message alert alert-warning").append($("<div>").text(i18n.t('CAppEditingNextVersion')));
					body.append(nextVersionMessage);
				}

				var buttonsContainer = $("<div/>", {
					"class": "row add-spacing-bottom add-spacing-top"
				}).appendTo(body);
				var switchToNextVersionButtonContainer = $("<div/>", {
					"class": "add-spacing-bottom"
				});
				if (!this._isSimplePage) {
					switchToNextVersionButtonContainer.addClass("nextVersionButtonOnTop");
					if (this.model.nextVersion) {
						switchToNextVersionButtonContainer.addClass("with-message");
					}
				} else if (this.model.documentId && !this.model.nextVersion) {
					switchToNextVersionButtonContainer.append(this._getStatusElement());
				}

				if (this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED &&
					this.model.documentId &&
					!this.model.readOnly) { //in the other case it is shown as 'action'
					// append infoballoon and button
					var helpBalloon = this._createHelpBalloon("CAppEditNextVersionHelp");
					switchToNextVersionButtonContainer.append($("<div/>").addClass("nextVersionButton").append(this._getSwitchToNextVersionBtn()).append(helpBalloon));
				}



				var actionBtns = $("<div />", {
					"class": "col-lg-8 col-lg-pull-4 col-xs-12"
				}).append(this._getActionButtons());

				buttonsContainer.append($("<div/>", {
					"class": "col-lg-4 col-lg-push-8 col-xs-12"
				}).append(switchToNextVersionButtonContainer));
				buttonsContainer.append(actionBtns);

				var contentContainer = $("<div/>", {
					"class": "row add-spacing-bottom"
				}).appendTo(body);

				contentContainer.append(this._stagesContainer);
				contentContainer.append($("<div/>", {
					"class": "col-lg-4 col-xs-12 approval-settings-panel"
				}).append(this._getGeneralSettingsPanel()));

				this.displayElem.empty();
				this.displayElem.append(body);

				//add stages
				_.each(this.stageViews, _.bind(function (stepView, i) {
					stepView.render();
				}, this));
			}
		};


		/**
		 * Creates the form with some hidden fields that are needed to submit the approval
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getForm = function () {
			this.form = $("<form>", {
				name: "actionform",
				id: "actionform",
				method: "post",
				action: this.model.actionURL
			});
			this.xmlContentField = $("<input>", {
				type: "hidden",
				id: "xmlcontent",
				name: "xmlcontent"
			}).appendTo(this.form);

			this.actionField = $("<input>", {
				type: "hidden",
				id: "actionField",
				name: "action"
			}).appendTo(this.form);
			this.passwordField = $("<input>", {
				type: "hidden",
				id: "password",
				name: "password"
			}).appendTo(this.form);
			this.commentField = $("<input>", {
				type: "hidden",
				id: "commentField",
				name: "comment"
			}).appendTo(this.form);

			return this.form;
		};

		/**
		 * Sets the values of the fields in the form and submits the form
		 * @private
		 * @param {object} data
		 *                   - {string} comment
		 *                   - {string} action
		 *                   - {string} password
		 */
		ApprovalView.prototype._submitForm = function (data) {
			this.commentField.val(data.comment);
			this.actionField.val(data.action);
			if (this.model.passwordValidation) {
				this.passwordField.val(data.password);
			}
			this.form.submit();
		};

		/**
		 * Creates a label with the status of the approval
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getStatusElement = function () {
			var container = $("<div />");
			container.addClass("approvalStatusLabelContainer");
			if (this._isSimplePage) {
				container.addClass("simple");
			}

			var helpBalloon = this._createHelpBalloon("CAppApprovalStatusHelp");
			var label = $("<span>", {
				text: i18n.t('CApp' + this.model.approvalStatus),
				"class": ApprovalConstants.statusLabels[this.model.approvalStatus] + " approvalStatusLabel"
			});

			container.append(label).append(helpBalloon);

			return container;
		};

		/**
		 * Creates a panel with the general settings (read-only or editable)
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getGeneralSettingsPanel = function () {
			if (this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED && !this.model.readOnly) {
				var collapseIconContainer = $("<div>", {
					"class": "collapseSettingsPanel"
				}).append($("<a>", {
					"class": "pull-left collapsed",
					"data-toggle": "collapse",
					"data-target": "#collapseSettings"
				}).append($("<i>", {
					"class": "icon icon-normal"
				})));

				var panel = $("<div/>", {
					"class": "panel panel-default"
				});
				$("<div/>", {
					"class": "panel-heading"
				}).append($("<h3>", {
					"class": "panel-title",
					text: i18n.t("CAppSettings")
				})).appendTo(panel);

				var panelBody = $("<div/>", {
					"class": "panel-body collapse",
					id: "collapseSettings"
				}).appendTo(panel);

				if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED && !this.model.readOnly) {
					// Add due date
					panelBody.append(this._getDueDateElement(false));
					if (this.model.isStagedApprovalAllowed()) {
						panelBody.append(this._getWhenNewVersionUploadedElement(false));
					}
					panelBody.append(this._getApprovalMasterElement(false));
					panelBody.append(this._getNotificationElement(false));
				} else if (this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED && !this.model.readOnly) {
					// Add due date
					panelBody.append(this._getDueDateElement(true));
					if (this.model.isStagedApprovalAllowed()) {
						panelBody.append(this._getWhenNewVersionUploadedElement(true));
					}
					panelBody.append(this._getApprovalMasterElement(true));
					panelBody.append(this._getNotificationElement(true));
				}
				var panelContainer = $("<div>", {
					class: "settingsPanel"
				}).append(panel);

				return $("<div>").append(collapseIconContainer).append(panelContainer);
			}
		};

		/**
		 * renders the due date element
		 */
		ApprovalView.prototype._renderDueDateElement = function () {
			var dueDateElement = this.displayElem.find(".approval-due-date-element");
			var finalDueDateElement = this.displayElem.find(".final-due-date-element");
			var newDueDateElement = null;

			if (!this.model.readOnly && this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
				newDueDateElement = this._getDueDateElement(this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED);
			}

			if (newDueDateElement) {
				dueDateElement.after(newDueDateElement);
				dueDateElement.remove();
				finalDueDateElement.remove();			
			}
		};

		/**
		 * Creates an element with due date information (can be editable)
		 * @private
		 * @param   {boolean} readonly - if true: show read only view
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getDueDateElement = function (readonly) {
			var label = i18n.t('CAppDueDate');
			var value = this.model.dueDateType;
			var valueLabel = i18n.t('CApp' + value);
			if (this.model.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
				valueLabel += " " + ApprovalUtils.getDateHourString(this.model.finalDueDate) + " ";
			}

			if (readonly) {
				return this._createFormEntryWithSelect(label, valueLabel, false);
			} else {
				var editable = true;

				var lastStep = _.last(this.model.steps);
				if (lastStep && lastStep.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
					editable = false;
				}
				var options = ApprovalConstants.dueDateOptions;
				if (!this.model.documentId && !this.model.isMultiDocSetup) {
					options = ApprovalConstants.dueDateOptionsFolderProject;
				}

				var element = null;
				var extraSettingsElement = null;

				var renderExtraSettingsElement = function () {
					// Add extra settings if necessary
					if (extraSettingsElement) {
						extraSettingsElement.remove();
						extraSettingsElement = null;
					}

					if (editable) {
						if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED &&
							this.model.isStagedApprovalAllowed() &&
							this.model.dueDateType === ApprovalConstants.DueDateType.FIXED) {
							extraSettingsElement = this._getDueDateRecalculateButton();
							element.append(extraSettingsElement);
						} else if (this.model.dueDateType === ApprovalConstants.DueDateType.LEAD_TIMES_WITH_FINAL_DATE) {
							extraSettingsElement = this._getFinalDueDateElement();
							extraSettingsElement.insertAfter(element);
						}
					}
				}.bind(this);

				this.model.on("changed.duedatetype", renderExtraSettingsElement);

				var onChange = _.bind(function (value) {
					this.model.changeDueDateType(value);
				}, this);

				element = this._createFormEntryWithSelect(label, valueLabel, editable, value, false, options, onChange, null, renderExtraSettingsElement, "CAppDueDateHelp", "approval_duedate_select");
				element.addClass("approval-due-date-element");
				return element;
			}
		};

		/**
		 * Creates settings element for 'when new version uploaded'
		 * @private
		 * @param   {boolean} readonly - if true: show read only view
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getWhenNewVersionUploadedElement = function (readOnly) {
			var label = i18n.t('CAppWhenNewVersionUploaded');
			var options = ['RestartApproval', 'RestartStage', 'InheritApproval'];
			var value = this.model.uponNewVersion;

			var onChange = _.bind(function (value) {
				this.model.changeInheritApproval(value);
			}, this);

			if (readOnly) {
				return this._createFormEntryWithSelect(label, i18n.t('CApp' + value), false);
			} else {
				return this._createFormEntryWithSelect(label, i18n.t('CApp' + value), true, value, true, options, onChange, null, null, "CAppWhenNewVersionUploadedHelp", "approval_new_version_select");
			}
		};

		/**
		 * Creates settings element for approval master
		 * @private
		 * @param   {boolean} readonly - if true: show read only view
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getApprovalMasterElement = function (readOnly) {
			var label = i18n.t('CAppApprovalMaster');

			var valueLabel = "";
			if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.USER && this.model.approvalMaster === this.currentUser.userId) {
				valueLabel = i18n.t('CAppMe');
			}
			if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.PROJECT_MANAGER) {
				valueLabel = i18n.t('CAppProjectManager');
			}
			if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.UPLOADER) {
				valueLabel = i18n.t('CAppUploader');
			}
			var memberObj;
			if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.USER && this.model.approvalMaster !== this.currentUser.userId) {
				memberObj = ApprovalUtils.findMemberObj("User", this.model.approvalMaster, this.model.projectMembers);
				var approvalMasterNoViewError = "";
				if (!memberObj) {
					memberObj = ApprovalUtils.findMemberObj("User", this.model.approvalMaster, this.model.usersWithNoAccess);
					approvalMasterNoViewError = $("<div>").append(
						$("<i>", {
							"class": "icon icon-warning icon-WCR_Msg-Warning",
							title: i18n.t('CAppAppMasterUserNoMember')
						})
					).html();
				}
				if (memberObj) {
					if (readOnly) {
						valueLabel += WCHTML.getUserNameStringActiveHTML(null, memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t('CAppUserDeactivated'), true);
					} else {
						valueLabel += WCHTML.getUserNameStringActiveText(memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t('CAppUserDeactivated'));
					}
					valueLabel += approvalMasterNoViewError;
				}
			}
			if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.GROUP) {
				memberObj = ApprovalUtils.findMemberObj("Group", this.model.approvalMaster, this.model.projectMembers);
				// group cannot be found, the group is not part of the project anymore
				if (memberObj === null) {
					valueLabel = "<i  class='icon icon-large icon-warning icon-WCR_Msg-Warning' ></i>" + WCXML.xmlencode(i18n.t('CAppAppMasterGroupNoMember'));
				} else {
					// App Master member is found. Depending on the type (Group or Role), show the correct label.
					var groupRoleTransl = i18n.t('CAppGroup');
					if (memberObj.isRole) {
						groupRoleTransl = i18n.t('CAppRole');
					}
					valueLabel = memberObj.groupName + ' (' + groupRoleTransl + ')';
				}
			}

			if (readOnly) {
				return this._createFormEntryWithSelect(label, valueLabel, false);
			} else {
				var value = this.model.approvalMasterType;
				if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.USER && this.model.approvalMaster === this.currentUser.userId) {
					value = "$Me$";
				} else if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.PROJECT_MANAGER) {
					value = "$PM$";
				} else if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.UPLOADER) {
					value = "$Uploader$";
				} else if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.GROUP) {
					value = "$Group$" + this.model.approvalMaster;
				} else if (this.model.approvalMasterType === ApprovalConstants.ApprovalMasterType.USER) {
					value = this.model.approvalMaster;
				}

				var editable = true;
				var alwaysEditMode = true;
				if (this.model.steps[0] && this.model.steps[0].approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
					editable = false;
					alwaysEditMode = false;
				}
				var options = ['$Me$', '$PM$', '$Uploader$'];
				var optionLabels = [i18n.t('CAppMe'), i18n.t('CAppProjectManager'), i18n.t('CAppUploader')];

				this.createMemberOptionArrays(this.model.projectMembers, 0, true, "", true, options, optionLabels);

				var onChange = _.bind(function (value) {
					this.model.changeApprovalMaster(value);
				}, this);

				return this._createFormEntryWithSelect(label, valueLabel, editable, value, alwaysEditMode, options, onChange, optionLabels, null, "CAppApprovalMasterHelp", "approval_master_select");
			}
		};

		/**
		 * HELPER METHOD to create form elements for the settings panel.
		 * Three different views are supported:
		 *     - read only: the label and the value are shown without editing capabilities
		 *     - editable: the label and value are shown as in read only. A edit pencil is added to enable the editing view.
		 *     - always editable: the setting is always shown in edit mode. The options are shown in a select element.
		 * 
		 * @private
		 * @param   {string}         label          - label for the setting
		 * @param   {string}         valueLabel     - label for the read only view
		 * @param   {boolean}        editable       - if true: edit pencil is shown
		 * @param   {string}         value          - value of the parameter
		 * @param   {boolean}        alwaysEditMode - always show the edit mode
		 * @param   {Array.<string>} options        - values for the select
		 * @param   {callback}       onChange       - callback executed when the value of the select changes
		 * @param   {Array.<string>} optionLabels   - labels for the select
		 * @param 	{callback}       onShown	    - callback executed when it's editable and select element is shown
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._createFormEntryWithSelect = function (label, valueLabel, editable, value, alwaysEditMode, options, onChange, optionLabels, onShown, helpTag, sahiId) {
			// Create select
			var selectElem;
			if (editable) {
				selectElem = $("<select>", {
					"class": "form-control",
					"data-sahi-id": sahiId
				});
				for (var i = 0; i < options.length; i++) {
					var text = "";
					if (optionLabels) {
						text = optionLabels[i];
					} else {
						text = i18n.t('CApp' + options[i]);
					}

					var optionElem = $("<option>", {
						value: options[i]
					}).html(text);
					if (options[i] === value) {
						optionElem.prop("selected", true);
					}
					selectElem.append(optionElem);
				}
				if (onChange) {
					selectElem.change(function (event) {
						var newValue = selectElem.val();
						onChange(newValue);
					});
				}
			}

			var formGroup = $("<div/>", {
				"class": "form-group"
			});
			$("<label>", {
				text: label
			}).appendTo(formGroup);

			var formEntry = $("<div/>", {
				"class": "form-entry"
			}).appendTo(formGroup);

			if (!alwaysEditMode) {
				var valueElem = $("<span>", {
					"class": "general-settings-form-entry-value"
				}).html(valueLabel).appendTo(formEntry);

				if (editable) {
					formEntry.append(selectElem);
					selectElem.hide();

					valueElem.append($("<a>", {
						"class": "btn btn-no-border btn-no-padding-right",
						"data-sahi-id": sahiId + "_edit"
					}).append($("<i>", {
						"class": "icon icon-WCR_Actions-Edit12"
					}))).on('click', function () {
						selectElem.show();
						valueElem.hide();
						onShown();
					});
				}
			} else {
				formEntry.append(selectElem);
			}

			if (helpTag) {
				var balloon = this._createHelpBalloon(helpTag);
				var inputGroup = $("<div />").addClass("input-group");
				inputGroup.append(formEntry.children());
				formEntry.append(inputGroup);
				inputGroup.append($("<div />").addClass("input-group-btn").append(balloon));
			}

			return formGroup;
		};

		/**
		 * Creates the notification settings panel
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getNotificationElement = function (readOnly) {
			this.notificationView = new NotificationView(this.model.notificationInfo, !readOnly);
			var notificationsElem = $("<div />");
			this.notificationView.setElement(notificationsElem);
			return notificationsElem;
		};

		/**
		 * Creates the action buttons based on the approval model
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getActionButtons = function () {
			var container = $('<div class="approval-action-buttons">');
			if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
				if (!this.model.readOnly) {
					// The user can start a simple (or complex) approval
					var primary = (this.model.approvalRuleChanged) ? false : true;
					this._addSetupApprovalButton(container, primary);
				} else {
					this._showWarnMessage(i18n.t('CAppNotDefined'), null, this.MSG_TAG);
				}

				// Start default approval cycle.  Only for documents and if there is a folder or project approval setup
				if ((this.model.documentId || this.model.isMultiDocSetup === true) &&
					this.currentUser && this.currentUser.canStartApproval === true && !this.model.nextVersion) {
					if (this.model.hasProjectApprovalSetup || this.model.hasFolderApprovalSetup) {
						this._addStartDefaultApprovalButton(container);
					}
				}

				if (!this.model.readOnly && this.model.approvalRuleChanged) { //if it is needed add save and revert buttons
					container.prepend(this._getSaveButton());
					container.append(this._getRevertButton(true));
				}

			} else if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
				this._addApprovalActions(container);
			} else if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED) {
				this._addSaveCancelStartApprovalButtons(container, true);
			}

			if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED && !this.model.readOnly) {
				this._addMoreActions(container);
			}

			return container;
		};

		/**
		 * Creates the button(s) to start an approval
		 * @private
		 */
		ApprovalView.prototype._addSetupApprovalButton = function (container, primary) {
			var classes = (primary) ? "btn btn-primary add-spacing-right" : "btn btn-default add-spacing-right";
			$('<button>', {
				"class": classes,
				type: "button",
				on: {
					click: _.bind(function () {
						if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_DEFINED) {
							this.model.setApprovalStatus(ApprovalConstants.ApprovalStatus.NOT_STARTED);
						}
						this.model.addNewStage(0);

					}, this)
				}
			}).append($("<i>", {
				"class": 'icon icon-WCR_Actions-Appr-Setup12'
			})).append(" " + i18n.t('CAppSetupSimpleApproval')).appendTo(container);
		};

		/**
		 * Creates the 'start default approval' button
		 * @private
		 */
		ApprovalView.prototype._addStartDefaultApprovalButton = function (container) {
			$('<button>', {
				"class": "btn btn-default add-spacing-right",
				type: "button",
				on: {
					click: _.bind(function () {
						this.save(true);
					}, this)
				}
			}).append($("<i>", {
				"class": "icon icon-WCR_Actions-Appr-Start12"
			})).append(" " + i18n.t('CAppStartDefaultApprovalCycle')).appendTo(container);
		};

		/**
		 * Creates the 'more actions' section
		 * @private
		 */
		ApprovalView.prototype._addMoreActions = function (container) {
			var wrapper = $("<div>", {
				"class": "feedback-with-actions"
			});
			var actions = $("<div>", {
				"class": "feedback-actions"
			}).append($("<p>", {
				text: "More Actions"
			}));
			var actionList = $("<ul>");

			// load project approval setup
			if (this.model.isProjectSetupAvailable()) {
				actionList.append($("<li>").append(this._getLoadProjectApprovalButton(false)));
			}

			// load folder approval setup
			if (this.model.isFolderSetupAvailable()) {
				actionList.append($("<li>").append(this._getLoadFolderApprovalButton(false)));
			}


			// Get setup for next version
			if (this.model.documentId) {
				actionList.append($("<li>").append($("<a>", {
					text: this.model.nextVersion ? i18n.t('CAppGoBackToCurrentVersion') : i18n.t('CAppEditApprovalSetupForNextVersion'),
					on: {
						click: _.bind(function () {
							if (this.model.nextVersion) {
								this.model.backToCurrentVersion();
							} else {
								this.model.editSetupForNextVersion();
							}
						}, this)
					}
				})));
			}

			if (actionList.children().length > 0) {
				container.append(wrapper.append(actions.append(actionList)));
			}

		};

		/**
		 * Creates the 'load project approval setup' button/link
		 * @private
		 * @param   {boolean} asButton - influences the rendering
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getLoadProjectApprovalButton = function (asButton) {
			if (this.model.isProjectSetupAvailable()) {

				var settings = {
					on: {
						click: _.bind(function () {
							var conf = true;
							if (this.model.steps.length > 0) {
								conf = this.confirmWithWarnings({
									question: 'CAppConfirmLoadProject'
								});
							}
							if (conf) {
								this.model.loadProjectFolderApproval(false);
							}
						}, this)
					}
				};

				if (asButton) {
					settings["class"] = "btn btn-default";
					settings.type = "button";
					settings.title = i18n.t('CAppLoadProjectApprovalSetup');
					return $("<button>", settings).append($("<i>", {
						"class": "icon icon-WCR_Actions-Project-Load12"
					}));
				} else {
					settings.text = i18n.t('CAppLoadProjectApprovalSetup');
					return $("<a>", settings);
				}
			}
		};

		/**
		 * Creates the 'load folder approval setup' button/link
		 * @private
		 * @param   {boolean} asButton - influences the rendering
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getLoadFolderApprovalButton = function (asButton) {
			if (this.model.isFolderSetupAvailable()) {
				var settings = {
					on: {
						click: _.bind(function () {
							var conf = true;
							if (this.model.steps.length > 0) {
								conf = this.confirmWithWarnings({
									question: 'CAppConfirmLoadProject'
								});
							}
							if (conf) {
								this.model.loadProjectFolderApproval(true);
							}
						}, this)
					}
				};

				if (asButton) {
					settings["class"] = "btn btn-default";
					settings.title = i18n.t('CAppLoadFolderApprovalSetup');
					settings.type = "button";
					return $("<button>", settings).append($("<i>", {
						"class": "icon icon-WCR_Actions-Folder-Load12"
					}));
				} else {
					settings.text = i18n.t('CAppLoadFolderApprovalSetup');
					return $("<a>", settings);
				}
			}
		};

		/**
		 * Creates the save, cancel, clear and start buttons
		 * @private
		 */
		ApprovalView.prototype._addSaveCancelStartApprovalButtons = function (container, leftRightLayout) {
			var loadAllowed = !_.some(this.model.steps, function (step) {
				return step.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED;
			});

			var leftSide = $("<div/>", {
				"class": "pull-left"
			});
			var rightSide = $("<div/>", {
				"class": "pull-right btn-group"
			});


			var saveBtn = this._getSaveButton();
			var revertBtn = this._getRevertButton(false);
			var startBtn = this._getStartApprovalButton();
			var clearBtn = this._getClearButton();

			var loadProjectBtn = null;
			var loadFolderBtn = null;

			if (loadAllowed) {
				loadProjectBtn = this._getLoadProjectApprovalButton(true);
				loadFolderBtn = this._getLoadFolderApprovalButton(true);
			}

			if (saveBtn) {
				leftSide.append(saveBtn);
			}
			if (startBtn) {
				leftSide.append(startBtn);
			}

			if (loadProjectBtn) {
				rightSide.append(loadProjectBtn);
			}
			if (loadFolderBtn) {
				rightSide.append(loadFolderBtn);
			}
			if (revertBtn) {
				rightSide.append(revertBtn);
			}
			if (clearBtn) {
				rightSide.append(clearBtn);
			}

			container.append(leftSide).append(rightSide);

			return container;
		};

		/**
		 * Creates the save button
		 * @private
		 * @returns {JQueryObj} 
		 */
		ApprovalView.prototype._getSaveButton = function () {
			if (!this.model.readOnly) {
				// Save button
				this._saveButton = $('<button>', {
					"class": "btn btn-primary add-spacing-right save-button",
					type: "button",
					on: {
						click: _.bind(function () {
							if (_.isEmpty(this.model.steps) && this.model.approvalRuleChanged === true) {
								var conf = this.confirmWithWarnings({
									question: 'CAppAreYouSureSaveEmpty'
								});
								if (!conf) {
									return false;
								}
							}
							this.validateAndSave(false);
						}, this)
					}
				}).append($("<i>", {
					"class": 'icon icon-WCR_Save12'
				})).append(" " + i18n.t('CAppSave'));
			} else {
				this._saveButton = null;
			}

			return this._saveButton;
		};

		/**
		 * Validates the model and saves it when no errors
		 * @param {boolean} startApprovalCycle - start the approval while saving
		 */
		ApprovalView.prototype.validateAndSave = function (startApprovalCycle) {
			var errorObj = this.model.validate();
			if (errorObj.errors.length > 0) {
				this._showErrors(errorObj);
			} else if (errorObj.warnings.length > 0) {
				var warningObj = {
					question: "CAppContinueAnyway",
					warnings: errorObj.warnings
				};
				if (this.confirmWithWarnings(warningObj)) {
					this.save(startApprovalCycle);
				}
			} else {
				this.save(startApprovalCycle);
			}
		};

		/**
		 * Save approval setup information 
		 * @param {boolean} startApprovalCycle - if true, the cycle is started too
		 */
		ApprovalView.prototype.save = function (startApprovalCycle) {
			var xmlcontent = null;
			if (!this.model.readOnly) {
				xmlcontent = this.model.writeToXML();
				if (!xmlcontent) {
					this.model.setError({
						errorMsg: 'CAppCouldNotSave'
					});
					return false; // something went wrong with conversion to xml.  So save nothing.
				}
				if (this.xmlContentField) {
					this.xmlContentField.val(xmlcontent);
				}
			} else {
				// no xmlcontent, so saving is not allowed
				if (!startApprovalCycle) {
					this.model.setError({
						errorMsg: 'CAppCannotSaveIfNothingChanged'
					});
					return;
				}
			}
			// set the action field
			if (this.actionField) {
				if (!this.model.nextVersion) {
					if (startApprovalCycle) {
						this.actionField.val("startCycle");
					} else {
						this.actionField.val("save");
					}
				} else {
					this.actionField.val("saveNextDocVersion");
				}
			} else {
				throw ("actionField is null");
			}

			if (!this.model.submitDisabled) {
				this.model.submitDisabled = true;
				if (this.model.submitAllowed) {
					this.model.confirmNeeded = false; // do not ask for confirmation of page leave
					this.form.submit();
				} else {
					// for unit test:make it visible
					if (xmlcontent && document.getElementById(this.model.saveFieldId)) {
						document.getElementById(this.model.saveFieldId).value = xmlcontent;
					}
					this.model.submitDisabled = false;
					return false;
				}
			} else {
				this.model.setError({
					errorMsg: 'CAppApprovalActionOngoing'
				});
			}
		};


		/**
		 * Creates the revert button
		 * @private
		 * @param   {boolean} buttonWithText - influences the rendering
		 * @returns {JQueryObj} 
		 */
		ApprovalView.prototype._getRevertButton = function (buttonWithText) {
			if (!this.model.readOnly) {
				// Revert button
				var revertBtn = $('<button>', {
					"class": "btn btn-default",
					type: "button",
					title: i18n.t('CAppRevert'),
					on: {
						click: _.bind(function () {
							this.model.cancel();
						}, this)
					}
				}).append($("<i>", {
					"class": 'icon icon-WCR_Revert12'
				}));

				if (buttonWithText) {
					revertBtn.append(" " + i18n.t('CAppRevert'));
					revertBtn.addClass("add-spacing-right");
				}

				return revertBtn;
			}
		};

		/**
		 * Creates the clear button
		 * @private
		 * @returns {JQueryObj} 
		 */
		ApprovalView.prototype._getClearButton = function () {
			if (!this.model.readOnly) {
				if (this.model.steps.length > 0 && this.model.steps[0].approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED) {
					return $('<button>', {
						"class": "btn btn-default",
						type: "button",
						title: i18n.t('CAppClear'),
						on: {
							click: _.bind(function () {
								this.model.clear();
							}, this)
						}
					}).append($("<i>", {
						"class": 'icon icon-WCR_Actions-Clear12'
					}));
				}
			}
		};

		/**
		 * Creates the start approval button
		 * @private
		 * @returns {JQueryObj} 
		 */
		ApprovalView.prototype._getStartApprovalButton = function () {
			// Start Approval Cycle button - only when this is a document approval setup
			if ((this.model.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED) &&
				(this.model.documentId || this.model.isMultiDocSetup === true) &&
				this.currentUser &&
				this.currentUser.canStartApproval === true &&
				!this.model.nextVersion) {
				return $('<button>', {
					"class": "btn btn-default add-spacing-right",
					type: "button",
					on: {
						click: _.bind(function () {
							this.validateAndSave(true);
						}, this)
					}
				}).append($("<i>", {
					"class": 'icon icon-WCR_Actions-Appr-Start12'
				})).append(" " + i18n.t('CAppStartApprovalCycle'));
			}
		};

		/**
		 * Creates the action buttons when approval cycle in ongoing
		 * @private
		 */
		ApprovalView.prototype._addApprovalActions = function (container) {
			var showStopAppButton = false;
			if (this.model.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING) {
				if (this.model.documentId && this.currentUser && this.currentUser.canStartApproval) {
					showStopAppButton = true;
				}
			}
			this.approveButtonsToolbar.setElement(container);
			this.approveButtonsToolbar.render(3, showStopAppButton);
		};


		/**
		 * Creates the button to switch between next version or to go back
		 * @private
		 */
		ApprovalView.prototype._getSwitchToNextVersionBtn = function () {
			var icon = (!this.model.nextVersion) ? "icon icon-WCR_Actions-Edit12" : "icon icon-WCR_Revert12";
			var text = (!this.model.nextVersion) ? i18n.t('CAppEditApprovalSetupForNextVersion') : i18n.t('CAppGoBackToCurrentVersion');

			return $("<button>", {
				"class": "btn btn-default",
				type: "button",
				on: {
					click: _.bind(function () {
						var conf = true;
						if (this.model.approvalRuleChanged) {
							var text = (this.model.newVersion) ? 'CAppConfirmBackToCurrent' : 'CAppConfirmGoToNextVersion';
							conf = this.confirmWithWarnings({
								question: text
							});
						}
						if (!conf) {
							return false;
						}

						if (!this.model.nextVersion) {
							this.model.editSetupForNextVersion();
						} else {
							this.model.backToCurrentVersion();
						}
					}, this)
				}
			}).append($("<i>", {
				"class": icon
			})).append(" " + text);
		};

		/**
		 * Creates the 'recalculate due date' button
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getDueDateRecalculateButton = function () {
			if ($("#dueDateRecalcDialog").length === 0) {
				var dialog = new Template(i18n, ApprovalDueDateDialog);
				dialog.d.defaultDueHour = 34; //17:00 is the default due hour when only giving a due date
				this.displayElem.append(dialog.getRenderedHtml());
			}

			return $("<button>", {
				"class": "btn btn-default add-spacing-top",
				type: "button",
				text: i18n.t('CAppDueDateCalculation'),
				on: {
					click: function () {
						this._createDueDateRecalcDialog();
						$("#dueDateRecalcDialog").dialog("open");
					}.bind(this)
				}
			});
		};

		/**
		 * Creates the recalculate due date dialog and makes sure it's only called once.
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._createDueDateRecalcDialog = _.once(function () {
			var buttonObj = {};
			buttonObj[i18n.t('CAppCancel')] = function () {
				$(this).dialog("close");
			};
			buttonObj[i18n.t('CAppOK')] = function (el) {
				// validate the operation, only close dialog when success
				if (this._doDueDateOperation()) {
					$("#dueDateRecalcDialog").dialog("close");
				}
			}.bind(this);

			//all but IE should have the dialog width set to AUTO
			var dialogWidthPref = "auto";

			//set the proper dialog width for IE
			if ($.browser.msie) {
				//have to use an explicit width to prevent unwanted strectching for IE
				dialogWidthPref = "700px";
			}

			$("#dueDateRecalcDialog").dialog({
				autoOpen: false,
				modal: false,
				closeOnEscape: true,
				draggable: true,
				height: 'auto',
				resizable: true,
				width: dialogWidthPref,
				maxWidth: 500,
				buttons: buttonObj
			});

			// add date pickers
			this._createCalendar("#shiftLatestToDate", "#shiftLatestToHour");
			this._createCalendar("#shiftFirstToDate", "#shiftFirstToHour");
			this._createCalendar("#distributeFromDate", "#distributeFromHour");
			this._createCalendar("#distributeToDate", "#distributeToHour");
		});

		/**
		 * Executes the selected operation in the due date dialog.
		 * @private
		 * @returns {boolean} True if succesful, false if invalid input has been entered and operation cannot be executed.
		 */
		ApprovalView.prototype._doDueDateOperation = function () {
			var closeWindow = true;
			if (this.model.dueDateType !== ApprovalConstants.DueDateType.FIXED) {
				// this should not happen because prevented by the UI
				throw ("You cannot change due dates when the due date type is not fixed due dates");
			}
			var radioSelects = document.getElementsByName("dueDateOperations");
			var currentSelect = _.find(radioSelects, function (select) {
				return select.checked;
			});
			var operation = currentSelect != null ? parseInt(currentSelect.value) : null;

			var days, hours;

			switch (operation) {
			case null:
				window.alert(i18n.t('CAppSelectOperation'));
				closeWindow = false;
				break; // do nothing
			case 0: // give each stage a number of days and hours
				days = parseInt(document.getElementById("giveDays").value, 10);
				hours = parseFloat(document.getElementById("giveHours").value); // allow decimals

				if (!this.model.addTimeToStages(days, hours)) {
					window.alert(i18n.t('CAppDaysAndHoursMustBePositive'));
					closeWindow = false;
				}
				break;
			case 1: // shift with fixed number of days
				days = parseInt(document.getElementById("shiftDays").value);
				hours = parseFloat(document.getElementById("shiftHours").value); // allow decimals

				if (!this.model.shiftDueDatesOfStages(days, hours)) {
					window.alert(i18n.t('CAppDaysAndHoursMustBeSameSign'));
					closeWindow = false;
				}
				break;
			case 2: // shift due dates to have the latest ending on
				if (!document.getElementById("shiftLatestToDate").value) {
					window.alert(i18n.t('CAppPleaseGiveADate'));
					closeWindow = false;
					break;
				}
				var finalDueDate = datetools.getDateString(document.getElementById("shiftLatestToDate").value);
				var finalDueHour = document.getElementById("shiftLatestToHour").value;
				finalDueDate = new Date(finalDueDate);
				if (finalDueHour) {
					finalDueDate.setHours(Math.floor(finalDueHour / 2));
				}
				if (Math.floor(finalDueHour / 2) * 2 == finalDueHour) {
					finalDueDate.setMinutes(0);
				} else {
					finalDueDate.setMinutes(30);
				}
				// adapt summer/winter time
				finalDueDate = datetools.fixDateSummerWinterTime(finalDueDate, true);
				this.model.shiftDueDatesToLatest(finalDueDate);
				break;
			case 3: // shift due dates to have the first starting on
				if (!this.model.steps || !this.model.steps[0] || !this.model.steps[0].dueDate) {
					window.alert(i18n.t('CAppFirstStepHasNoDueDate'));
					return;
				}
				if (this.model.steps[0].approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED) {
					window.alert(i18n.t('CAppCannotShiftIfStarted'));
					return;
				}
				if (!document.getElementById("shiftFirstToDate").value) {
					window.alert(i18n.t('CAppPleaseGiveADate'));
					closeWindow = false;
					break;
				}
				var firstDueDate = datetools.getDateString(document.getElementById("shiftFirstToDate").value);
				var firstDueHour = document.getElementById("shiftFirstToHour").value;
				firstDueDate = new Date(firstDueDate);
				if (firstDueHour) {
					firstDueDate.setHours(Math.floor(firstDueHour / 2));
				}
				if (Math.floor(firstDueHour / 2) * 2 == firstDueHour) {
					firstDueDate.setMinutes(0);
				} else {
					firstDueDate.setMinutes(30);
				}
				// adapt summer/winter time
				firstDueDate = datetools.fixDateSummerWinterTime(firstDueDate, true);
				this.model.shiftDueDatesToFirst(firstDueDate);
				break;
			case 4: // distribute evenly
				if (!document.getElementById("distributeFromDate").value || !document.getElementById("distributeToDate").value) {
					window.alert(i18n.t('CAppPleaseGiveADate'));
					closeWindow = false;
					break;
				}
				var startDate = new Date(datetools.getDateString(document.getElementById("distributeFromDate").value));
				var startHour = document.getElementById("distributeFromHour").value;
				if (startHour) {
					startDate.setHours(Math.floor(startHour / 2));
				}
				if (Math.floor(startHour / 2) * 2 == startHour) {
					startDate.setMinutes(0);
				} else {
					startDate.setMinutes(30);
				}
				// adapt summer/winter time
				startDate = datetools.fixDateSummerWinterTime(startDate, true);

				var endDate = new Date(datetools.getDateString(document.getElementById("distributeToDate").value));
				var endHour = document.getElementById("distributeToHour").value;
				if (endHour) {
					endDate.setHours(Math.floor(endHour / 2));
				}
				if (Math.floor(endHour / 2) * 2 == endHour) {
					endDate.setMinutes(0);
				} else {
					endDate.setMinutes(30);
				}
				// adapt summer/winter time
				endDate = datetools.fixDateSummerWinterTime(endDate, true);
				if (!this.model.distributeDueDatesEvenly(startDate, endDate)) {
					window.alert(i18n.t('CAppFirstDayLater'));
					closeWindow = false;
				}
				break;
			default:
				window.alert("this operation is not implemented");
			}
			return closeWindow;

		};

		/**
		 * Creates the date picker and hour selector to set a final due date
		 * @private
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._getFinalDueDateElement = function () {
			if (!this.model.finalDueDate || this.model.finalDueDate.constructor !== Date) {
				this.model.finalDueDate = new Date();
			}
			// adapt summer/winter time
			var showDate = datetools.fixDateSummerWinterTime(this.model.finalDueDate, false);

			var calendar = $('<input>', {
				"class": "form-control",
				"data-sahi-id" : "CAppFinalDueDateText",
				value: datetools.getDateString(showDate),
				type: "text",
				name: "datepicker"
			});

			var tpl = new Template(i18n, HourDropDownTmpl);
			tpl.d.d = {
				dueHour: showDate.getHours() * 2 + Math.floor(showDate.getMinutes() / 30),
				name: "hourdropdown",
				id: "CAppFinalDueHour"
			};
			var hourdropdown = $(tpl.getRenderedHtml());
			var hourdropdowninput = hourdropdown.find("[name=hourdropdown]");

			calendar.datepicker({
				dateFormat: "M dd, yy",
				onClose: function (dateText, inst) {
					var element = hourdropdowninput[0];
					if (element && element.selectedIndex === 0) {
						element.selectedIndex = (36 + 1);
					}
				}
			});

			var calendarInputGroup = $('<span class="form-datetime-btn input-group-addon">').on("click", function () {
				calendar.datepicker("show");
			});

			var formGroup = $("<div/>", {
					"class": "form-group final-due-date-element"
				})
				.append(
					$("<label>", {
						text: i18n.t('CAppFinalDueDate')
					})
				)
				.append(
					$("<div/>", {
						"class": "form-entry"
					})
					.append(
						$('<div class="clearfix final-due-date">')
						.append(
							$('<div class="form-date input-group">')
							.append(calendar)
							.append(
								calendarInputGroup.append(
									$('<i class="icon icon-WCR_Calendar" >')
								)
							)
						)
						.append(hourdropdown)
					)
				);


			var onChange = function () {
				var dateString = calendar.val();
				var formattedDateString = datetools.getDateString(dateString);
				if (formattedDateString) {
					var doubleHour = hourdropdowninput.val();
					var newDate = new Date(formattedDateString);
					var hours = Math.floor(doubleHour / 2);
					var minutes = (hours * 2 == doubleHour) ? 0 : 30;
					newDate.setHours(hours);
					newDate.setMinutes(minutes);
					newDate.setSeconds(0);
					var newFinalDueDate = datetools.fixDateSummerWinterTime(newDate, true);

					if (isNaN(newFinalDueDate.getTime())) {
						// Invalid date entered, reset calendar field to current model value
						calendar.val(datetools.getDateString(datetools.fixDateSummerWinterTime(this.model.finalDueDate, false)));
					} else {
						this.model.setFinalDueDate(newFinalDueDate);
					}
				} else {
					// Reset calendar with current model becaus we don't accept empty dates
					calendar.val(datetools.getDateString(datetools.fixDateSummerWinterTime(this.model.finalDueDate, false)));
				}
			}.bind(this);

			calendar.change(onChange);
			hourdropdowninput.change(onChange);

			return formGroup;
		};

		/** 
		 * This function creates a calendar object. 
		 * @param  idName id of the input field
		 * @param  idHour id of the selection of period in the day 
		 */
		ApprovalView.prototype._createCalendar = function (idName, idHour) {
			if ($(idName) !== null) {
				$(idName).datepicker({
					dateFormat: "M dd, yy",
					onClose: function (dateText, inst) {
						var element = $(idHour)[0];
						if (element && element.selectedIndex === 0) {
							element.selectedIndex = (36 + 1);
						}
					}
				});
			}
		};

		/**
		 * Toggles the Save button
		 * @private
		 */
		ApprovalView.prototype._toggleSaveButtonEnabled = function () {
			if (this._saveButton) {
				if (this.model.approvalRuleChanged) {
					this._saveButton.removeAttr("disabled");
				} else {
					this._saveButton.attr("disabled", "disabled");
				}
			}
		};

		/**
		 * Get the container for the stages
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype.getStagesContainer = function () {
			return this._stagesContainer;
		};

		/**
		 * Add a stage
		 * @private
		 * @param {ApprovalStage} stage
		 */
		ApprovalView.prototype._addStage = function (stage) {
			var index = this.model.getStageNumber(stage) - 1; // the index of the new stage
			this._addViewsAtIndex(index, stage);
		};

		/**
		 * Add a view for @param stage at index
		 */
		ApprovalView.prototype._addViewsAtIndex = function (index, stage) {
			var isAtEnd = this.stageViews.length === index;
			var approvalStageContainer = new ApprovalStageContainer(this.model, stage);
			this.stageViews.splice(index, 0, approvalStageContainer);
			if (index === 0) {
				this._stagesContainer.prepend(approvalStageContainer.element);
			} else if (isAtEnd) {
				this._stagesContainer.append(approvalStageContainer.element);
			} else {
				// not at beginning, nor at end, insert after the previous step
				this._getStageViewForStageAtIndex(index - 1).element.after(approvalStageContainer.element);
			}
			approvalStageContainer.render();
		};

		/**
		 * Get the stage view at @param index
		 * @private
		 * @returns {ApprovalStageView}
		 */
		ApprovalView.prototype._getStageViewForStageAtIndex = function (index) {
			var stage = this.model.getStageAtIndex(index);
			return _.find(this.stageViews, function (stageView) {
				return stageView.stageModel === stage;
			});
		};

		/**
		 * Delete all stages
		 * @private
		 */
		ApprovalView.prototype._deleteStageView = function () {
			var stages = arguments;
			_.each(stages, function (stage) {
				var stageViewIndex = _.findIndex(this.stageViews, function (stageView) {
					return stageView.stageModel === stage;
				});
				if (stageViewIndex > -1) {
					this.stageViews.splice(stageViewIndex, 1);
				}
			}.bind(this));

		};


		/**
		 * HELPER METHOD to make 2 arrays (labels and values) that will we used to construct a select element for approval master 
		 * (copied and adapted from old Complex Approval implementation)
		 * 
		 * @param   {Array}    memberArr  - array of project members
		 * @param   {number}   indent     - indentation of the text. Subsequent nesting will increase the indent automatically
		 * @param   {boolean}  masterOnly 
		 *                                @param   {string}   filter     - determines whether to display an item.  In order to display, an item must contain filter. If filter is null or the empty string, no filtering is done and everything is shown
		 * @param   {boolean} writeRoles - decides whether to also Role member assignments are added
		 */
		ApprovalView.prototype.createMemberOptionArrays = function (memberArr, indent, masterOnly, filter, writeRoles, outputValues, outputLabels) {
			//JAMI: first display Assigned Roles
			if (this.model.roleAssignments && this.model.roleAssignments.length > 0 && writeRoles) {
				var roleTagStr = i18n.t('CAppRole');
				var groupTagStr = i18n.t('CAppGroup');

				for (var assignedRoleIdx = 0; assignedRoleIdx < this.model.roleAssignments.length; assignedRoleIdx++) {
					var curRoleAssignmentMap = this.model.roleAssignments[assignedRoleIdx];

					var curRoleID = curRoleAssignmentMap.roleID;
					var curRoleName = curRoleAssignmentMap.roleName;

					//check whether the currently processed Role is among 'members that have access'
					var memberExists = ApprovalUtils.findMemberObj("Group", curRoleID, this.model.projectMembers);
					if (memberExists === null || memberExists === 'undefined') {
						//the current role was not found among the valid members -> skip it
						continue;
					}

					var assignedUsers = curRoleAssignmentMap.userRoleAssignments;
					var assignedGroups = curRoleAssignmentMap.groupRoleAssignments;

					outputValues.push("$Group$" + curRoleID);
					outputLabels.push(curRoleName + "<" + roleTagStr + ">");


					//now show the Role's User Assignees
					if (assignedUsers && assignedUsers.length > 0 && !masterOnly) {
						for (var assignedUserIdx = 0; assignedUserIdx < assignedUsers.length; assignedUserIdx++) {
							var curAssignedUserData = assignedUsers[assignedUserIdx];

							var curUserID = curAssignedUserData.ID;
							var curUserName = curAssignedUserData.name;

							//write the option (with indentation)
							outputValues.push(curUserID);
							outputLabels.push("&nbsp;&nbsp;&nbsp;&nbsp;" + curUserName);
						}
					}

					//also show the Role's Group Assignees
					if (assignedGroups && assignedGroups.length > 0 && !masterOnly) {
						for (var assignedGroupIdx = 0; assignedGroupIdx < assignedGroups.length; assignedGroupIdx++) {
							var curAssignedGroupData = assignedGroups[assignedGroupIdx];

							var curGroupID = curAssignedGroupData.ID;
							var curGroupName = curAssignedGroupData.name;

							var curAssigneeUserID = curAssignedGroupData.assigneeID;
							var curAssigneeUserName = curAssignedGroupData.assigneeName;

							//different results for a Group Role Assignment when a User Assignee Representative is specified
							var haveGroupAssigneeUser = false;
							if ((curAssigneeUserID && curAssigneeUserID.length > 0) && (curAssigneeUserName && curAssigneeUserName.length > 0)) {
								haveGroupAssigneeUser = true;
							}

							var optionValue = "&nbsp;&nbsp;&nbsp;&nbsp;";
							if (haveGroupAssigneeUser) {
								outputLabels.push(optionValue + curAssigneeUserName + ' @ ' + curGroupName + '<' + groupTagStr + '>');
							} else {
								outputLabels.push(optionValue + curGroupName + '<' + groupTagStr + '>');
							}

							if (haveGroupAssigneeUser) {
								//write the User-only option
								outputValues.push(curAssigneeUserID);
							} else {
								// HADW WCR12 16-May-2012: only show the one of group, choice moved to the table instead of the dropdown
								outputValues.push("$Group$" + curGroupID);
							}
						}
					}
				}
			}

			for (var member = 0; member < memberArr.length; member++) {
				var memberObj = memberArr[member];
				if (memberObj && (!masterOnly || memberObj.canBeApprovalMaster === true)) {
					var nameString = '';
					for (var i = 0; i < indent; i++) {
						nameString += "&nbsp;&nbsp;&nbsp;&nbsp;"; // add indent
					}

					var shouldShowIfInactive = memberObj.isActive || this.model.approvalMaster === memberObj.userId;

					if (memberObj.memberType === "User" && shouldShowIfInactive) {
						nameString += WCXML.xmlencode(WCHTML.getUserNameStringActiveText(memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t('CAppUserDeactivated')));

						if (!filter || nameString.toLowerCase().indexOf(filter.toLowerCase()) !== -1) {
							outputValues.push(memberObj.userId);
							outputLabels.push(nameString);
						}
					} else if (memberObj.memberType === "Group") {
						if (memberObj.isRole) {
							continue; // roles are already handled at the top
						}

						if (memberObj.groupName) {
							var groupRoleTransl = i18n.t('CAppGroup');
							if (memberObj.isRole) {
								groupRoleTransl = i18n.t('CAppRole');
							}
							nameString += memberObj.groupName + '<' + groupRoleTransl + '>';
						}
						if (!filter || nameString.toLowerCase().indexOf(filter.toLowerCase()) !== -1) {
							outputValues.push('$Group$' + memberObj.groupId);
							outputLabels.push(nameString);
						}
						if (memberObj.members && memberObj.members.length > 0) {
							this.createMemberOptionArrays(memberObj.members, indent + 1, masterOnly, filter, false, outputValues, outputLabels);
						}
					}

				}
			}
		};

		/**
		 * Shows an error message for each given error
		 * @private
		 * @param {Array.<Object>} errorObj (see approvalModel.setError for the structure of the errorObj)
		 */
		ApprovalView.prototype._showErrors = function (errorObj) {
			var message = (errorObj.errorMsg) ? i18n.t(errorObj.errorMsg) + "\n" : "";

			if (errorObj.errors) {
				for (var i = 0; i < errorObj.errors.length; i++) {
					var error = errorObj.errors[i];
					if (!error.parameters) {
						error.parameters = [];
					}
					if (error.inline) {
						message += "- " + error.tag;
					} else { // translate the message
						if (i > 0) {
							message += "\n";
						}
						error.parameters.unshift(error.tag);
						message += "- " + i18n.t.apply(this, error.parameters);
					}
				}
			}
			this._showErrorMessage(message, null, this.MSG_TAG);
		};

		/**
		 * Generates a confirm with the warnings
		 * @private
		 * @param {Array.<Object>} warnings 
		 *                   - tag: tag for translation
		 *                   - parameters: parameters to be added to the i18n 
		 */
		ApprovalView.prototype.confirmWithWarnings = function (warningObj) {
			var message = (warningObj.warningMsg) ? i18n.t(warningObj.warningMsg) + "\n" : "";

			// Add warnings to the message
			if (warningObj.warnings) {
				for (var i = 0; i < warningObj.warnings.length; i++) {
					var warning = warningObj.warnings[i];
					if (!warning.parameters) {
						warning.parameters = [];
					}
					if (i > 0) {
						message += "\n";
					}

					if (warning.inline) {
						message += warning.tag;
					} else {
						warning.parameters.unshift(warning.tag);
						message += i18n.t.apply(this, warning.parameters);
					}
				}
			}

			if (warningObj.question) {
				message += "\n" + i18n.t(warningObj.question);
			}
			return window.confirm(message);
		};

		/**
		 * Show a success message
		 * @param {string} message
		 * @param {string} url - action
		 * @param {boolean} autohide 
		 * @param {string} tag 
		 */
		ApprovalView.prototype._showSuccessMessage = function (message, url, autohide, tag) {
			this._messageModel.markObsolete(tag);
			var msg = new MessageComponent.Message(MessageComponent.MessageType.SUCCESS, message, url, autohide, tag);
			this._messageModel.addToMessageQueue(msg);
		};

		/**
		 * Show an error message
		 * @param {string} message
		 * @param {string} url - action
		 * @param {string} tag 
		 */
		ApprovalView.prototype._showErrorMessage = function (message, url, tag) {
			this._messageModel.markObsolete(tag);
			var msg = new MessageComponent.Message(MessageComponent.MessageType.ERROR, message, url, false, tag);
			this._messageModel.addToMessageQueue(msg);
		};

		/**
		 * Show a warning message
		 * @param {string} message
		 * @param {string} action
		 * @param {string} tag 
		 */
		ApprovalView.prototype._showWarnMessage = function (message, action, tag) {
			this._messageModel.markObsolete(tag);
			var msg = new MessageComponent.Message(MessageComponent.MessageType.WARN, message, action, false, tag);
			this._messageModel.addToMessageQueue(msg);
		};

		/**
		 * Generates a clickable help baloon that links to a help topic based on the given tag
		 * @private
		 * @param   {string} tag
		 * @returns {JQueryObj}
		 */
		ApprovalView.prototype._createHelpBalloon = function (tag) {
			var balloon = $('<a />').addClass('inline_help').attr('title', i18n.t('CAppHelp')).attr('help-info', tag);
			balloon.append($('<i />').addClass("icon icon-WCR_Info-Balloon"));

			balloon.on("click", _.bind(function (e) {
				var element = e.currentTarget;
				e.preventDefault();
				e.stopPropagation();
				var helpSection = $(element).attr("help-info");
				var popw = window.open(this._helpArr[helpSection], helpSection);
				popw.focus();
			}, this));

			return balloon;
		};

		return ApprovalView;
	});
