define(["underscore",
		"approval/view/approvalXdrop/ApprovalMemberXDropDataProvider",
		"approval/view/approvalXdrop/ApprovalAllUserXDropDataProvider",
		"component/xdrop/dataproviders/CompositeDataProvider",
		"component/xdrop/dataproviders/LimitResultsDataProvider",
		"component/xdrop/dataproviders/CreateUserDataProvider",
		"component/xdrop/dataproviders/CategoryDataProvider",
		"component/xdrop/renderers/UserRenderer",
		"component/xdrop/renderers/GroupRenderer",
		"component/xdrop/renderers/RoleRenderer",
		"component/xdrop/renderers/ButtonRenderer",
		"component/xdrop/renderers/CategoryTitleRenderer",
		"approval/view/approvalXdrop/ApprovalXDropConversionUtils",
		"approval/ApprovalUtils",
		"component/xdrop/XDrop",
		"utils/Observable",
		"utils/Observer",
		"utils/webi18n!Approval",
		"require"
	],
	function (_,
		ApprovalMemberXDropDataProvider,
		ApprovalAllUserXDropDataProvider,
		CompositeDataProvider,
		LimitResultsDataProvider,
		CreateUserDataProvider,
		CategoryDataProvider,
		UserRenderer,
		GroupRenderer,
		RoleRenderer,
		ButtonRenderer,
		CategoryTitleRenderer,
		ApprovalXDropConversionUtils,
		ApprovalUtils,
		XDrop,
		Observable,
		Observer,
		i18n,
		require) {

		"use strict";

		var renderers = {
			user: UserRenderer,
			group: GroupRenderer,
			role: RoleRenderer,
			button: ButtonRenderer,
			categoryTitle: CategoryTitleRenderer
		};


		/**
		 * This object wraps an XDrop object and provides it with the required dataproviders.
		 * @param {ApprovalStage}   approvalStage stage model
		 * @param {string} projectId
		 */
		var ApprovalXDrop = function (approvalStage, projectId) {
			Observable.mixin(this, "ApprovalXDrop");
			Observer.mixin(this);

			this._approvalStage = approvalStage;

			// Create static xdrop data provider with members in project
			this._projectMembersDataProvider = new ApprovalMemberXDropDataProvider(approvalStage);

			var dataproviders = [];

			var projectMemberCategory = new CategoryDataProvider({
				title: i18n.t("CAppXDropCurrentProjectCategory"),
				hideIfNoResults: false
			}, this._projectMembersDataProvider);

			dataproviders.push(projectMemberCategory);

			var createUserProvider;
			var currentUserObject = approvalStage.approvalModel.currentUser;
			if (currentUserObject.isProjectManager || currentUserObject.isAdmin) {
				// the current user is ADMIN or PM of this project
				
				// Create xdrop data provider with non project active members
				this._allActiveUserXDropDataProvider = new ApprovalAllUserXDropDataProvider(approvalStage, projectId, true, {maxResults: 20});
				var nonProjectActiveMemberCategory = new CategoryDataProvider({
					title: i18n.t("CAppXDropOtherCategory"),
					isCollapsed: true,
					hideIfNoResults: false,
					expandWhenQueryIsSet: true
				}, this._allActiveUserXDropDataProvider);
				dataproviders.push(nonProjectActiveMemberCategory);

				// Create xdrop data provider with non project inactive members
				this._allInactiveUserXDropDataProvider = new ApprovalAllUserXDropDataProvider(approvalStage, projectId, false, {maxResults: 10});
				var nonProjectInactiveMemberCategory = new CategoryDataProvider({
					title: i18n.t("CAppXDropDeactivatedUser"),
					isCollapsed: true,
					expandWhenQueryIsSet: true,
					hideIfNoResults: true
				}, this._allInactiveUserXDropDataProvider);
				dataproviders.push(nonProjectInactiveMemberCategory);

				if (currentUserObject.canCreateUsers || currentUserObject.isAdmin) {
					var folderId = approvalStage.approvalModel.getFolderId();
					// the current user can create (and invite) new users
					createUserProvider = new CreateUserDataProvider(projectId, folderId);
				}
			}

			var limitedResultSet = new LimitResultsDataProvider({
				maxResults: 50
			}, dataproviders);


			var composite;

			if (createUserProvider) {
				composite = new CompositeDataProvider(limitedResultSet, createUserProvider);
			} else {
				composite = limitedResultSet;
			}

			this._xdrop = new XDrop(composite, renderers, {
				placeholder: i18n.t("CAppXDropPlaceholder")
			});

			this.element = this._xdrop.getElement();
			this.element.attr("data-sahi-id", "AddApproverXDrop_" + (approvalStage.number -1));
			this._observe(this._xdrop, "selected", this._handleXDropSelection);

		};

	
		/**
		 * The selection of the XDrop has changed
		 * If a user is selected who is not yet in the project, he needs to be invited to the project
		 * The resultEntry of the XDrop is converted in to an approval member object
		 * @private
		 * @param   {ResultEntry} resultEntry 
		 */
		ApprovalXDrop.prototype._handleXDropSelection = function (resultEntry) {
			var approvalModel = this._approvalStage.approvalModel;

			if (resultEntry) {
				var member = ApprovalUtils.findMemberObj(((resultEntry.getType() === "group" || resultEntry.getType() === "role") ? "Group" : "User"),
					resultEntry.getData().getId(),
					approvalModel.getProjectMembers());
				
				var isProjectMember = true;

				if (!member) {
					// the user was not yet in the project!
					isProjectMember = false;
					member = ApprovalXDropConversionUtils.convertToMember(resultEntry.getData());					
				}

				if ("user" == resultEntry.getType()) {
					// When selecting a User we need to do some extra checks!
					var user = resultEntry.getData();
					if (!user.INCONTEXTCREATED) {
						// we didn't create the user in context...
						// check if the user is inactive or is not invited in the project yet
						if (!isProjectMember || !user.isActiveUser()) {
							// the selected user is not invited or is inactive 
							// we need to show the invitation popup
							var projectId = approvalModel.getProjectId();
							var folderId = approvalModel.getFolderId();

							// once the user clicks on the user name, we want to open the invite user popup
							require(["component/usercreation/InviteMemberPopup"], function (InviteMemberPopupView) {
								//If a deactivated user was already in the project (with a role possibly) there is no need to assign another role
								var assignNoRole = isProjectMember && !user.isActiveUser(); 
								var popup = new InviteMemberPopupView(resultEntry.getData(), projectId, folderId, assignNoRole);

								// Add callback to execute when a user is successfully invited
								popup.onSubmit(function () {
									// Since call to invite member is completed, so user suppose active.
									member.isActive = true;
									if(member.memberType === "User") {
										_.chain(approvalModel.getProjectMembers()).where({memberType: "Group", isRole: false}).pluck("members").flatten().where({userId: member.userId}).each(function(user){
											user.isActive = true;
										});
										_.chain(approvalModel.roleAssignments).pluck("userRoleAssignments").flatten().where({ID: member.userId}).each(function(user){
											user.isActive = true;
										});
										
									}
									this._emit("selected", member);
								}.bind(this));

								popup.onCancel(function () {
									// the popup was cancelled: revert selection in the XDrop
									this.setValue(null);
								}.bind(this));

								// open the popup
								popup.show();
							}.bind(this));

							// we are showing the poup, don't set the member as selected yet								
							return;
						}
					}
				}

				// it is safe to add the member to the approval stage
				this._emit("selected", member);
			}
		};

		/**
		 * Set the selected value of the xdrop
		 * @param {Object} value
		 */
		ApprovalXDrop.prototype.setValue = function (value) {
			this._xdrop.setValue(value);
		};

		/**
		 * Toggle the xdrop open/close
		 */
		ApprovalXDrop.prototype.toggle = function () {
			this._xdrop.toggle(!this._xdrop.isOpen());
		};
	

		return ApprovalXDrop;
	});