define([
		"approval/ApprovalConstants",
		"approval/view/ApproveRejectView",
		"component/actionmenu/ActionMenu",
		"utils/webi18n!Approval",
		"utils/AnimationFrameRenderer",
		"jquery",
		"underscore"
	],
	function (
		ApprovalConstants,
		ApproveRejectView,
		ActionMenu,
		i18n,
		AnimationFrameRenderer,
		$,
		_) {
		"use strict";

		/**
		 * This view shows the action buttons for the approval depending on the given models
		 * 
		 * @class
		 * @param {ApprovalModel} model
		 * @param {ApproveRejectViewModel} approveRejectViewModel
		 */
		function ApproveButtonsToolbar(model, approveRejectViewModel) {
			AnimationFrameRenderer.mixin(this);

			this.model = model;

			this._maxButtons = 6; //this is the maximum number (currently)
			this._showStopAppButton = false;
			this._container = null;

			//Create sub-views
			this._approveRejectDialog = new ApproveRejectView(model, approveRejectViewModel);
		}

		/**
		 * Set the element where the component needs to be rendered
		 * @param {JQueryObject} elem
		 */
		ApproveButtonsToolbar.prototype.setElement = function (elem) {
			this._container = elem;
		};

		/**
		 * Render
		 * (Before calling this, set the container of this component!)
		 * 
		 * @param {integer} maxButtons - maximum number of buttons that will be directly visible. 
		 *                             The remaining buttons will be hidden behind a "More..." button
		 * @param {boolean} showStopAppButton - if the "Stop Approval Cycle " button should be shown or not
		 */
		ApproveButtonsToolbar.prototype.render = function (maxButtons, showStopAppButton) {
			this._maxButtons = maxButtons;
			this._showStopAppButton = showStopAppButton;

			this._render();
		};

		/**
		 * The effective rendering of the component
		 * @private
		 */
		ApproveButtonsToolbar.prototype._doRender = function () {
			this._container.append(this._renderApprovalActionButtons());
		};


		/**
		 * Render the action buttons
		 * @private
		 * @returns {JQueryObject}
		 */
		ApproveButtonsToolbar.prototype._renderApprovalActionButtons = function () {
			var btnToolbar = $("<div/>", {
				"class": "btn-toolbar",
				role: "toolbar"
			});
			var actions = this.model.getApprovalActions();
			if (!actions.showContent) {
				// there is nothing to show
				return '';
			}
			var actionMenuContainer = $('<div>', {
				"class": "action-menu-component"
			}).appendTo(btnToolbar);
			// add buttons
			_.each(actions.list, _.bind(function (action, index) {
				var classes = "action ";
				var icon;
				switch (action.name) {
				case 'Approve':
					classes += "action-success";
					icon = ApprovalConstants.images.Approve + ' icon-white';
					break;
				case 'ForceApprove':
					icon = ApprovalConstants.images.ForceApprove + ' ' + ApprovalConstants.imageColors.ForceApprove;
					break;
				case 'Reject':
					classes += "action-danger";
					icon = ApprovalConstants.images.Reject + ' icon-white';
					break;
				case 'ForceReject':
					icon = ApprovalConstants.images.ForceReject + ' ' + ApprovalConstants.imageColors.ForceReject;
					break;
				case 'Comment':
					icon = ApprovalConstants.images.Comment;
					break;
				}

				var visibility = (index >= this._maxButtons) ? "-1" : "1";

				actionMenuContainer.append(
					$("<a>", {
						"class": classes,
						"data-icon": icon,
						"data-visibility": visibility,
						title: i18n.t(action.text),
						text: i18n.t(action.text),
						on: {
							click: _.bind(function (event) {
								this._approveRejectDialog.open(event, action.internalName, action.step);
							}, this)
						}
					})
				);

			}, this));

			if (this._showStopAppButton) {
				var visibility = (actions.list.length >= this._maxButtons) ? "-1" : "1";
				$("<a>", {
					"class": "action",
					'data-icon': "icon-WCR_Stopped",
					'data-visibility': visibility,
					title: i18n.t('CAppStopApprovalCycle'),
					text: " " + i18n.t('CAppStopApprovalCycle'),
					on: {
						click: _.bind(function () {
							if (!this.model.submitDisabled) {
								this.model.submitDisabled = true;
								this.model.confirmNeeded = false;
								this.model.submit({
									action: "stopCycle"
								});
							} else {
								this.model.setError({
									errorMsg: 'CAppApprovalActionOngoing'
								});
							}
						}, this)
					}
				}).appendTo(actionMenuContainer);
			}

			new ActionMenu(actionMenuContainer);

			return btnToolbar;
		};

		return ApproveButtonsToolbar;
	});
