define([
		"approval/ApprovalConstants",
		"approval/ApprovalUtils",
		"approval/view/CommentHoverView",
		"utils/Observer",
		"tpl!approval/view/template/ApproversTable",
		"tpl!approval/view/template/ApproversTableRoleHover",
		"utils/webi18n!Approval",
		"Template",
		"jquery",
		"underscore",
		"wcr/webcenter_html"
	],
	function (
		ApprovalConstants,
		ApprovalUtils,
		CommentHoverView,
		Observer,
		ApproversTableTempl,
		ApproversTableRoleHoverTempl,
		i18n,
		Template,
		$,
		_,
		WCHTML
	) {
		"use strict";

		/**
		 * Represents the view of the table of approvers in a stage
		 * @param {ApprovalStage} stageModel
		 */
		function ApproversTable(stageModel) {
			Observer.mixin(this);

			this.model = stageModel;
			this._observe(this.model, "changed.approvers.*", this.render, this);
			this._observe(this.model, "deleted", this._stopObserveAll, this);

			this._commentHoverView = new CommentHoverView(this.model);
		}

		ApproversTable.prototype.setElement = function (elem) {
			this.elem = elem;
			this.render();
		};

		/** 
		 * Displays the header of the complete Complex Approval
		 */
		ApproversTable.prototype.render = function () {
			if (!this.elem) {
				return;
			}

			// Generate table template
			var tmpl = new Template(i18n, ApproversTableTempl);
			tmpl.d.showRemoveColumn = this._showRemoveColumn();
			tmpl.d.showAllColumns = this._showAllColumns();
			tmpl.d.stageIndex = this.model.number - 1;
			tmpl.setContext(this.elem);
			tmpl.bind("click", "[name=approvers-remove-all]", function () {
				this.model.removeAllApprovers();
			}.bind(this));

			var table = $(tmpl.getRenderedHtml());
			var tableBody = table.find("[name=approvers-body]");
			if (!_.isEmpty(this.model.approvers)) {
				// Add all rows to table for each approver
				_.each(this.model.approvers, function (approverObj, index) {
					if (approverObj && approverObj.memberObj) {
						tableBody.append(this._generateRowElement(approverObj, index));
					}
				}.bind(this));
			} else {
				tableBody.append(
					$("<tr>").append(
						$('<td colspan="7">').append(
							$("<em>", {
								text: i18n.t("CAppNoApproversInStep")
							})
						)
					)
				);

			}

			this.elem.html(table);
		};

		/**
		 * Generates a row element for a specified approverObj at position "index"
		 * @private
		 * @param   {object}   approverObj
		 * @param   {integer} index
		 * @returns {jQuery el} <tr> tag
		 */
		ApproversTable.prototype._generateRowElement = function (approverObj, index) {
			var tr = $("<tr>");
			var memberObj = approverObj.memberObj;

			// Add "Remove" button
			if (this._showRemoveColumn()) {
				tr.append(
					$('<td class="td-centered">').append(
						$("<i>", {
							"class": "icon icon-normal icon-WCR_Remove show-cursor-pointer",
							title: i18n.t("CAppRemove"),
							on: {
								click: function () {
									this.model.removeApprover(index);
								}.bind(this)
							}
						})
					)
				);
			}

			// Lastname, firstname cell
			if (memberObj.lastName) {
				var userHTML = WCHTML.getUserNameStringActiveHTML(null, memberObj.firstName, memberObj.lastName, memberObj.userName, memberObj.isActive, i18n.t('CAppUserDeactivated'));
				tr.append(
					$('<td class="td-nowrap">').append($(userHTML))
				);
			} else if (memberObj.groupName) {
				if (memberObj.isRole) {
					tr.append(
						$("<td>", {
							"class": "td-nowrap",
							on: {
								mouseover: function (e) {
									this._showRoleHover(e, index);
								}.bind(this),
								mouseout: function () {
									delayhidemenu();
								}
							}
						}).append(
							$("<i>", {
								"class": "icon icon-normal icon-blue icon-WCR_Assignee-Role"
							})
						).append("&nbsp;" + memberObj.groupName)
					);
				} else {
					var groupCell = $('<td class="td-nowrap">').append(
						$("<i>", {
							"class": "icon icon-normal icon-green icon-WCR_Assignee-Group",
							title: i18n.t("CAppGroup")
						})
					).append("&nbsp;" + memberObj.groupName);
					if (memberObj.isMember && _.isEmpty(memberObj.members)) {
						groupCell.append(document.createTextNode(" " + i18n.t('CAppEmptyIndicator')));
					}
					tr.append(groupCell);
				}
			} else {
				tr.append($("<td><em>Approver with no name</em></td>"));
			}

			//determine whether the currently processed approval stage is not completed already
			//in that case show the Group/Role approver in a read only mode > no dropdown list
			var isStageCompleted = !this.model.canModifyApprovers();

			// group/role choice goes to read-only during running stage approval cycles	and when it's in read-only mode
			var isReadOnlyRow = approverObj.approvalType === "User" || this.model.approvalModel.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED || this.model.approvalModel.readOnly || isStageCompleted;

			//determine whether the currently processed approval stage is not completed already
			//in that case show the Group/Role approver in a read only mode > no dropdown list
			var dropdownCell = $("<td>");
			tr.append(dropdownCell);
			if (isReadOnlyRow) {
				dropdownCell.append(i18n.t(ApprovalUtils.getApprovalTypeLangTag(approverObj)));
			} else {
				var groupApproverSelect = $("<select>", {
					"class": "form-control",
					on: {
						change: function (e) {
							this.model.changeGroupApprover(index, e.target.value);
						}.bind(this)
					}
				});
				dropdownCell.addClass("type-drop-down");
				dropdownCell.append(groupApproverSelect);

				groupApproverSelect.attr("data-sahi-id", "CAppObjGroupApproverSelector_"+(this.model.number -1) + "_"+index)
				
				if (memberObj.isRole) {
					groupApproverSelect.append($("<option>", {
						value: "OneOfRole",
						text: i18n.t("CAppOneOfRole")
					}));
				}

				groupApproverSelect.append($("<option>", {
					value: "OneOfGroup",
					text: memberObj.isRole ? i18n.t("CAppAllOfRoleOneOfGroup") : i18n.t("CAppOneOfGroup")
				}));

				groupApproverSelect.append($("<option>", {
					value: "AllOfGroup",
					text: memberObj.isRole ? i18n.t("CAppAllOfRole") : i18n.t("CAppAllOfGroup")
				}));

				groupApproverSelect.val(approverObj.approvalType);
			}

			if (this._showAllColumns()) {
				// pending is not displayed if the stage is not pending
				var pendingForcedAway = approverObj.approvalStatus === ApprovalConstants.ApprovalStatus.PENDING && 
					this.model.approvalStatus !== ApprovalConstants.ApprovalStatus.PENDING;

				// approval status and approved version between brackets
				var approvalStatusAndVersion = $("<td>");
				tr.append(approvalStatusAndVersion);
				if (approverObj.approvalStatus && approverObj.approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED && !pendingForcedAway) {
					approvalStatusAndVersion.append(
						$("<i>", {
							title: i18n.t("CApp" + this.model.approvalStatus),
							"class": ApprovalConstants.images[approverObj.approvalStatus] + " " + ApprovalConstants.imageColors[approverObj.approvalStatus] + " add-spacing-right"
						})
					);
					if (approverObj.approvalStatus === ApprovalConstants.ApprovalStatus.FORCE_APPROVED || 
						approverObj.approvalStatus === ApprovalConstants.ApprovalStatus.FORCE_CONDITIONAL_APPROVED || 
						approverObj.approvalStatus === ApprovalConstants.ApprovalStatus.FORCE_REJECTED) {
						approvalStatusAndVersion.append(
							$("<span>").append(i18n.t("CApp" + approverObj.approvalStatus))
						);
					} else {
						approvalStatusAndVersion.append(i18n.t("CApp" + approverObj.approvalStatus));
					}

					if (approverObj.approvedVersion) {
						if (approverObj.approvedVersion !== this.model.approvalModel.documentVersion) {
							//show in bold red
							approvalStatusAndVersion.append(
								$("<span>").append("&nbsp;(" + approverObj.approvedVersion + ")")
							);
						} else {
							approvalStatusAndVersion.append("&nbsp;(" + approverObj.approvedVersion + ")");
						}
					}
				}

				// approval date/time
				var approvalTime = $("<td>");
				tr.append(approvalTime);
				if (approverObj.approvalTime) {
					approvalTime.addClass("td-date");
					approvalTime.append(ApprovalUtils.getDateHourString(approverObj.approvalTime));
				}

				// "By" field
				var approvalBy = $("<td>");
				tr.append(approvalBy);
				if (approverObj.approvedByUserId) {
					var approverMember = ApprovalUtils.findMemberObj("User", approverObj.approvedByUserId, this.model.approvalModel.projectMembers);
					if (!approverMember) { // approved by somebody who is no longer in the project?
						approverMember = ApprovalUtils.findMemberObj("User", approverObj.approvedByUserId, this.model.approvalModel.usersWithNoAccess);
					}

					if (approverMember) {
						approvalBy.text(WCHTML.getUserNameString(approverMember.firstName, approverMember.lastName, approverMember.userName));
					} else {
						approvalBy.text('User ID:' + approverObj.approvedByUserId);
					}
				}

				// Comment/condition
				var approvalComment = $("<td>");
				tr.append(approvalComment);
				if (!_.isEmpty(approverObj.commentArr)) {
					approvalComment.mouseover(function (e) {
						this._commentHoverView.open(index, e);
					}.bind(this));
					approvalComment.mouseout(function () {
						delayhidemenu();
					});
					var commentString = _.map(approverObj.commentArr, function (comment) {
						return comment.commentText;
					}).join("<br>");
					approvalComment.html(commentString);
				}
			}
			return tr;
		};

		/**
		 * This will display the popup that appears when hovering over a role
		 * @private
		 * @throws {Error} error is thrown when it couldn't find a role
		 * @param   {jQuery event}   event
		 * @param   {integer} approverIndex The index of the approver in the table
		 */
		ApproversTable.prototype._showRoleHover = function (event, approverIndex) {
			if (_.isEmpty(this.model.approvers)) {
				return;
			}

			var approverObj = this.model.approvers[approverIndex];
			if (!approverObj) {
				return;
			}

			var memberObj = approverObj.memberObj;
			if (!memberObj || !memberObj.isRole) {
				return;
			}
			var roleObj = _.find(this.model.approvalModel.roleAssignments, function (roleAssignment) {
				return memberObj.groupId === roleAssignment.roleID;
			});

			if (roleObj == null) {
				throw new Error('cannot find role');
			}

			var tmpl = new Template(i18n, ApproversTableRoleHoverTempl);
			tmpl.d.roleObj = roleObj;

			dropdownmenu(event.target, event, [tmpl.getRenderedHtml()], "auto", null, null, true, "#ffffcc", false);

		};

		/**
		 * Checks to see if all columns should be shown, such as: "Status", "Approval Date", "Approved By", "Comment/Condition"
		 * @private
		 * @returns {boolean} True if all columns should be shown
		 */
		ApproversTable.prototype._showAllColumns = function () {
			return this.model.approvalModel.steps[0].approvalStatus !== ApprovalConstants.ApprovalStatus.NOT_STARTED;
		};

		/**
		 * Checks to see if the remove column should be shown. If this is true than approvers can be removed from the table
		 * @private
		 * @returns {boolean} True if the remove column should be shown
		 */
		ApproversTable.prototype._showRemoveColumn = function () {
			return !this.model.approvalModel.readOnly &&
				this.model.approvalModel.approvalStatus === ApprovalConstants.ApprovalStatus.NOT_STARTED &&
				this.model.canModifyApprovers();
		};

		return ApproversTable;
	});
