define([
		"approval/ApprovalConstants",
		"utils/webi18n!Approval",
		"underscore",
		"wcr/webcenter_xml"
	],
	function (
		ApprovalConstants,
		i18n,
		_,
		WCXML
	) {
		"use strict";

		/**
		 * This view displays the model ApprovalNotifications
		 * @param {ApprovalNotifications} notificationModel
		 * @param {boolean} editable          True if the notification type should be changeable
		 */
		function NotificationView(notificationModel, editable) {
			this._model = notificationModel;
			this._showNotifications = true;
			this._editable = editable;
			this._editMode = false;

			this._model.on("update", _.bind(this.render, this));
		}

		/**
		 * This sets the context that will be used to display this view
		 * @param {jQuery} elem
		 */
		NotificationView.prototype.setElement = function (elem) {
			this.elem = elem;
			this.render();
		};

		/** 
		 * Displays this view
		 */
		NotificationView.prototype.render = function () {
			if (!this.elem) {
				return;
			}

			var html = $("<div />").addClass("form-group");

			if (this._editMode) {
				// Notifications
				html.append($("<label />").text(i18n.t('CAppNotifications')));
				this.notificationSelector = this._displayNotificationsSelector();
				html.append($("<div />").addClass("form-entry").html(this.notificationSelector));

				if (this._model.notificationType !== ApprovalConstants.NotificationType.NO_NOTIFICATIONS) {
					var toggleCont = $("<div />").addClass("form-group");
					toggleCont.append($("<label />").addClass("offset-label"));

					var toggle = $('<a href="#" />').click(_.bind(function (e) {
						this._toggleShowNotifications();
						e.preventDefault();
					}, this)).text(this._showNotifications ? i18n.t('CAppHideNotifications') : i18n.t('CAppShowNotifications'));

					toggleCont.append($("<div />").addClass("form-entry").append($("<p />").append(toggle)));

					html.append(toggleCont);
				}

				if (this._showNotifications) {
					html.append(this._displayNotifications());
				}
			} else {
				var notificationTypeTag = this._model.notificationType;

				$("<label>", {
					text: i18n.t('CAppNotifications')
				}).appendTo(html);

				var formEntry = $("<div/>", {
					"class": "form-entry"
				}).appendTo(html);

				var valueElem = $("<p>", {
					text: this._getLabel(notificationTypeTag)
				}).appendTo(formEntry);
				if (this._editable) {
					valueElem.append($("<a>", {
						"class": "btn btn-no-border",
						"data-sahi-id": "approval_notif_select_edit"
					}).append($("<i>", {
						"class": "icon icon-WCR_Actions-Edit12"
					}))).on('click', _.bind(function () {
						this._editMode = true;
						this.render();
					}, this));
				}
			}

			this.elem.html(html);
		};

		/**
		 * Displays the selector/dropdown of notifications
		 */
		NotificationView.prototype._displayNotificationsSelector = function () {
			var notificationOptions = [
				ApprovalConstants.NotificationType.NO_NOTIFICATIONS,
				ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS,
				ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS
			];
			if (!this._model.approvalModel.documentId && !this._model.approvalModel.folderId) { // you are editing the project notifications
				//JAMI: it looks odd, but makes sense > although the label will say "Project Notifications" the user can actually modify the settings 
				//(unlike for Folder/Document variant of the same option where it is only a read only option) and thus modify the Project Notification setup in general 
				//NOTE: for details consult WCR10_Capp_710 in Complex Approval FS
				notificationOptions = [
					ApprovalConstants.NotificationType.NO_NOTIFICATIONS,
					ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS
				];
			}

			//write the dropdown selector element
			var select = $("<select>", {
				"class": "form-control",
				"data-sahi-id": "approval_notif_select"
			});
			select.change(_.bind(this._changeNotificationSelector, this));

			//add the expected options
			_.each(notificationOptions, function (notificationOption) {
				var option = $("<option>", {
					value: notificationOption,
					text: this._getLabel(notificationOption)
				});

				if (this._model.notificationType === notificationOption) {
					option.prop("selected", "true");
				}
				select.append(option);
			}.bind(this));
			return select;
		};

		/**
		 * Displays the table of notifications
		 */
		NotificationView.prototype._displayNotifications = function () {
			var notificationsArr = null;
			var readOnly = false;
			if (this._model.notificationType === ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS) {
				// in case we are working with a folder setup, the default notifications are the ones from the project
				if (this._model.approvalModel.folderId != null && this._model.approvalModel.documentId == null) {
					notificationsArr = this._model.projectNotificationsArr;
				} else {
					notificationsArr = this._model.defaultNotificationsArr;
				}
				readOnly = true;
			}
			if (this._model.notificationType == ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS) {
				notificationsArr = this._model.notificationsArr;
			}

			if (!notificationsArr) {
				return '';
			}

			var html = $("<div />").addClass("form-group");
			html.append($("<label />").addClass("offset-label"));

			var table = $("<table>",{
				"class": "table",
				name: "CAppNotificationTable"
			});
			var headerRow = $("<tr />");
			table.append($("<thead />").append(headerRow));
			html.append($("<div />").addClass("form-entry").append(table));

			// header
			headerRow.append($("<th />").text(i18n.t('CAppNotification')));

			var htmlcode;

			var notificationTypeArr = ApprovalConstants.notificationTypeArr;
			var columns = ["projectManager", "involvedPeople", "projectMembers"];
			var row;
			for (var col = 0; col < columns.length; col++) {
				var allChecked = true;
				for (row = 0; row < notificationTypeArr.length; row++) {
					if (notificationsArr[row][columns[col]] === false) {
						allChecked = false;
						break;
					}
				}
				htmlcode = '<th class="td-checkbox"><div><div>';
				if (columns[col] == 'projectManager') {
					htmlcode += WCXML.xmlencode(i18n.t('CAppManager'));
				} else if (columns[col] == 'involvedPeople') {
					htmlcode += WCXML.xmlencode(i18n.t('CAppInvolvedPeople'));
				} else if (columns[col] == 'projectMembers') {
					htmlcode += WCXML.xmlencode(i18n.t('CAppMembers'));
				}
				htmlcode += '</div>';
				if (readOnly === false) {
					htmlcode += '<div><input id="notification_' + col + '_All" type="checkbox"';
					if (allChecked) {
						htmlcode += ' checked ';
					}
					htmlcode += '></div>';
				}
				htmlcode += '</div>';
				htmlcode += '</th>';

				var headerEntry = $(htmlcode);
				var currentCol = col;
				headerEntry.find('#notification_' + col + '_All').click(_.bind(function (col, elem) {
					this._model.toggleNotificationColumn(col, elem.is(":checked"));
				}, this, currentCol, headerEntry.find('#notification_' + col + '_All')));
				headerRow.append(headerEntry);
			} // end for col


			// row per notification type
			for (row = 0; row < notificationTypeArr.length; row++) {
				var curRow = row;
				htmlcode = '<tr>';
				htmlcode += '<td>' + WCXML.xmlencode(i18n.t('CApp' + notificationTypeArr[row])) + '</td>';
				var rowId = "notificationCheck_" + row;
				htmlcode += '<td class="td-checkbox"><input id="' + rowId + '_0" data-sahi-id=' + rowId + '_0" type="checkbox"';
				if (notificationsArr[row].projectManager === true) {
					htmlcode += ' checked ';
				}
				if (readOnly) {
					htmlcode += ' disabled="disabled" ';
				}
				htmlcode += '></td>';

				htmlcode += '<td class="td-checkbox"><input id="' + rowId + '_1" data-sahi-id=' + rowId + '_1" type="checkbox"';
				if (notificationsArr[row].involvedPeople === true) {
					htmlcode += ' checked ';
				}
				if (readOnly) {
					htmlcode += ' disabled="disabled" ';
				}
				htmlcode += '></td>';

				htmlcode += '<td class="td-checkbox"><input id="' + rowId + '_2" data-sahi-id=' + rowId + '_2" type="checkbox"';
				if (notificationsArr[row].projectMembers === true) {
					htmlcode += ' checked ';
				}
				if (readOnly) {
					htmlcode += ' disabled="disabled" ';
				}
				htmlcode += '"></td>';
				htmlcode += '</tr>';

				var tableRow = $(htmlcode);
				for (var i = 0; i < 3; i++) {
					var curI = i;
					tableRow.find('#notificationCheck_' + curRow + '_' + curI).click(_.bind(_.bind(function (row, col, elem) {
						this._model.toggleNotification(row, col, elem.is(":checked"));
					}, this, curRow, curI, tableRow.find('#notificationCheck_' + curRow + '_' + curI))));
				}

				table.append(tableRow);
			}

			return html;
		};


		/**
		 * Called when clicking on Show notifications or Hide notifications
		 * @private
		 */
		NotificationView.prototype._toggleShowNotifications = function () {
			this._showNotifications = !this._showNotifications;
			this.render();
		};


		/**
		 * Called when the user changes the selection in the selector between no Notification, Default Notification and Custom Notification
		 * @private
		 */
		NotificationView.prototype._changeNotificationSelector = function () {
			this._model.setNotificationType(this.notificationSelector.val());
		};


		/**
		 * Generates a translated label for the given notification type
		 * @private
		 * @param   {string} notificationType (See ApprovalConstants.NotificationType)
		 * @returns {string} translated label
		 */
		NotificationView.prototype._getLabel = function (notificationType) {
			if (notificationType === ApprovalConstants.NotificationType.DEFAULT_NOTIFICATIONS) { //'Project/Folder Notifications' options (inheriting) 		
				if (this._model.approvalModel.documentId &&
					this._model.approvalModel.folderId &&
					this._model.approvalModel.hasFolderApprovalSetup) {
					return i18n.t('CAppFolderNotifications');
				} else {
					return i18n.t('CAppProjectNotifications');
				}
			} else if (notificationType === ApprovalConstants.NotificationType.CUSTOM_NOTIFICATIONS) {
				if (!this._model.approvalModel.documentId && !this._model.approvalModel.folderId) {
					//'Project Notifications' option (Project Approval setup)
					return i18n.t('CAppProjectNotifications');
				} else {
					//'Custom Notifications' all other cases (Folder + Document Approval setup)
					return i18n.t('CAppCustomNotifications');
				}
			} else if (notificationType === ApprovalConstants.NotificationType.NO_NOTIFICATIONS) {
				return i18n.t('CAppNoNotifications');
			}
		};

		return NotificationView;
	}
);
