define(["underscore",
		"webcenterapi/data/WebCenterUser",
		"component/xdrop/dataproviders/WCRUserDataProvider",
		"component/xdrop/dataproviders/ResultEntry",
		"./ApprovalXDropConversionUtils",
		"approval/ApprovalUtils",
		"utils/Observer",
		"utils/FilterUtils",
		"utils/webi18n!XDrop"
	],
	function (_,
		WebCenterUser,
		WCRUserDataProvider,
		ResultEntry,
		ApprovalXDropConversionUtils,
		ApprovalUtils,
		Observer,
		FilterUtils,
		i18n) {
		"use strict";

		/**
		 * This class is a data provider for all WebCenter visible user for the xDrop that converts members as used in the approval module to result entries used for xdrop
		 * 
		 * @class ApprovalAllUserXDropDataProvider
		 * @augments WCRUserDataProvider
		 * 
		 * @constructor
		 * 
		 * @param {Object} approvalStage
		 * @param {String} projectID
		 * @param {Boolean} isActiveUsers
		 * @param {Object} options - Extra parameters as options.
		 * 
		 */
		var ApprovalAllUserXDropDataProvider = function (approvalStage, projectID, isActiveUsers, options) {
			WCRUserDataProvider.call(this, "user", isActiveUsers, options);
			Observer.mixin(this);

			this._approvalStage = approvalStage;
			this._projectID = projectID;

			this._observe(this._approvalStage.approvalModel, "stage.*.changed.approvers.*", function () {
				this.setQuery(this._query || "");
			}.bind(this));
			this._observe(approvalStage, "deleted", function () {
				this._stopObserveAll();
			}.bind(this));
		};

		ApprovalAllUserXDropDataProvider.prototype = Object.create(WCRUserDataProvider.prototype);
		ApprovalAllUserXDropDataProvider.prototype.constructor = ApprovalAllUserXDropDataProvider;

		/**
		 * @override
		 */
		ApprovalAllUserXDropDataProvider.prototype.setEntries = function () {
			this._resultSet.clear();

			//Remove members which already in project 
			var nonProjectMembers = _.filter(this._data, this._notInProject.bind(this));

			// Trim array of members more than max results count.
			var newData = nonProjectMembers;
			if (this.getMaxResults() >= 0) {
				newData = _.first(nonProjectMembers, this.getMaxResults());
			}

			//Create result entries map
			var resulEntriesMap = _.map(newData, function (entry) {
				return new ResultEntry(entry.type, entry.data);
			}.bind(this));

			//Apply filter on query
			resulEntriesMap = _.filter(resulEntriesMap, function (entry) {
				entry.getData().setExtraSAHIMarkers("not-invited"); // mark them for SAHI
				return entry.getData().match(this._query, true);
			}.bind(this));

			//Filter result entries on query string and add to result set
			this._resultSet.appendAll(resulEntriesMap);

			// Create label entry to display refining results is still possible by typing user name.
			if (this.getMaxResults() > 0 && nonProjectMembers.length > this.getMaxResults()) {
				var showMoreEntry = new ResultEntry("label", i18n.t("TypeToRefineResults"));
				showMoreEntry.setAllowSelection(false);
				this._resultSet.append(showMoreEntry);
			}

			this._resultSet.commitChanges();
		};

		/**
		 * Makes a param object to make AJAX call to fetch user list.
		 */
		WCRUserDataProvider.prototype._getUsersListParam = function () {

			//Get the userlist that is already invited in the project and filter based on search query
			var projectMembers = this._approvalStage.approvalModel.getProjectMembers();
			
			var projectMembersFilteredCount = _.reduce(projectMembers, function (memo, member) {
				if (member.memberType === "User") {
					return memo + (ApprovalXDropConversionUtils.createWebCenterUserFromMember(member).data.match(this._query) ? 1 : 0);
				} else if (member.isGroup) {
					return memo + _.reduce(ApprovalXDropConversionUtils.createWebCenterGroupFromMember(member).data.getMembers(), function (memo2, user) {
						return memo2 + (user.match(this._query) ? 1 : 0);
					}.bind(this), 0);
				}
			}.bind(this), 0);

			var maxResults = this.getMaxResults();
			if (maxResults > 0) {
				maxResults += projectMembersFilteredCount + 1;
			}

			var params = {
				"list": this._isActiveUsers ? "Active" : "Inactive",
				"filter": FilterUtils.preprocessPattern(this._query),
				"maxcount": maxResults,
				"json": 0
			};

			return params;
		};

		/**
		 * Checks if a result entry member is in project
		 * 
		 * @param resultEntry
		 * @returns
		 */
		ApprovalAllUserXDropDataProvider.prototype._notInProject = function (dataEntry) {
			// Get project members.
			var projectMembers = this._approvalStage.approvalModel.getProjectMembers();
			return !ApprovalUtils.findMemberObj("User", dataEntry.getData().getId(), projectMembers);

		};

		return ApprovalAllUserXDropDataProvider;
	});
