define(["underscore",
		"component/xdrop/dataproviders/XDropDataProvider",
		"approval/view/approvalXdrop/ApprovalXDropConversionUtils",
		"utils/Observer"
	],
	function (_,
		XDropDataProvider,
		ApprovalXDropConversionUtils,
		Observer) {

		"use strict";


		/**
		 * This class is a static data provider for the xDrop that converts members as used in the approval module to result entries used for xdrop
		 * @class ApprovalMemberXDropDataProvider
		 * @augments XDropDataProvider
		 */
		var ApprovalMemberXDropDataProvider = function (approvalStage) {
			XDropDataProvider.call(this);
			Observer.mixin(this);
			this._approvalStage = approvalStage;
			this.setQuery("");
			this._observe(this._approvalStage.approvalModel, "stage.*.changed.approvers.*", function () {
				this.setQuery(this._query || "");
			}.bind(this));
			this._observe(approvalStage, "deleted", function(){
				this._stopObserveAll();
			}.bind(this));
		};


		ApprovalMemberXDropDataProvider.prototype = Object.create(XDropDataProvider.prototype);
		ApprovalMemberXDropDataProvider.prototype.constructor = ApprovalMemberXDropDataProvider;


		/**
		 * Check to see if a given member is in the list of approvers in the current stage
		 * @private
		 * @param   {object}   member
		 * @returns {boolean} True if part of the approvers list, false otherwise.
		 */
		ApprovalMemberXDropDataProvider.prototype._memberInStage = function (member) {
			return _.some(_.pluck(this._approvalStage.getApprovers(), "memberObj"), function (stageMember) {
				return (member.userId && member.userId === stageMember.userId) ||
					(member.groupId && member.groupId === stageMember.groupId);
			});
		};


		ApprovalMemberXDropDataProvider.prototype.setMaxResults = function () {};

		/**
		 * @override
		 * Change the query of the xdrop
		 * @param   {string} queryString new query string
		 */
		ApprovalMemberXDropDataProvider.prototype.setQuery = function (queryString) {
			// Set the project members data as it might have changed
			this._query = queryString;
			this._resultSet.clear();
			this._resultSet.appendAll(
				_.filter(
					_.filter(this._getXDropReadyMembers(), this._notInStage.bind(this)),
					function (entry) {
						return entry.getData().match(queryString, true);
					}
				)
			);
			this._resultSet.commitChanges();
		};

		/**
		 * Get the list of members that can be shown in the xdrop
		 * @private
		 * @returns {object} xdrop compatible data object
		 */
		ApprovalMemberXDropDataProvider.prototype._getXDropReadyMembers = function () {
			var roleAssignments = _.map(_.clone(this._approvalStage.approvalModel.roleAssignments), _.clone);
			_.each(roleAssignments, function (roleAssignment) {
				roleAssignment.userRoleAssignments = _.filter(roleAssignment.userRoleAssignments, _.compose(_.negate(this._memberInStage.bind(this)), function (userRole) {
					return {
						userId: userRole.ID
					};
				}));
				roleAssignment.groupRoleAssignments = _.filter(roleAssignment.groupRoleAssignments, _.compose(_.negate(this._memberInStage.bind(this)), function (groupRole) {
					return {
						groupId: groupRole.ID
					};
				}));
			}, this);

			return ApprovalXDropConversionUtils.createMembersXDropData(this._approvalStage.approvalModel.getProjectMembers(), roleAssignments);
		};

		/**
		 * Checks if a result entry (or anything that has a getData() method which resturns a WebCenterUser,WebCenterGroup or WebCenterRole ) is in the stage
		 * !!! WARNING !!! this function messes with the @param resultEntry by removing Users that are already added to the approval stage from the member list
		 */
		ApprovalMemberXDropDataProvider.prototype._notInStage = function (resultEntry) {
			var approvalMember = ApprovalXDropConversionUtils.convertToMember(resultEntry.getData());
			var notAllGroupMemberInStage = false; // we also want to show the group if some members aren't in the stage.
			var approvalMemberInStage = this._memberInStage(approvalMember);

			if (approvalMember.groupName) {
				notAllGroupMemberInStage = _.some(resultEntry.getData().getMembers(), _.compose(_.negate(this._memberInStage.bind(this)), ApprovalXDropConversionUtils.convertToMember));
				if (approvalMemberInStage) {
					resultEntry.setAllowSelection(false);
				}
			}

			if (approvalMember.groupName) {
				resultEntry.getData()._members = _.filter(resultEntry.getData().getMembers(), _.compose(_.negate(this._memberInStage.bind(this)), ApprovalXDropConversionUtils.convertToMember));
			}

			return !approvalMemberInStage || notAllGroupMemberInStage;
		};

		return ApprovalMemberXDropDataProvider;
	}
);
