define(["webcenterapi/data/WebCenterUser",
	"webcenterapi/data/WebCenterGroup",
	"webcenterapi/data/WebCenterRole",
	"underscore",
	"component/xdrop/dataproviders/ResultEntry",
	"utils/FilterUtils"
], function (WebCenterUser,
	WebCenterGroup,
	WebCenterRole,
	_,
	ResultEntry,
	FilterUtils) {

	"use strict";


	/*
	 * This file includes some utility functions to transform Xdrop result entries containing 
	 * a WebCenterUser, WebCenterGroup and WebCenterRole data objects to 
	 * project member object as used in Approval.js and vice versa. These members are Plain javascript objects.
	 *
	 */
	
	
	function transformWebCenterUserToMember(data) {
		var member = {};
		member.userId = data.getId();
		member.userName = data.getName();
		member.firstName = data.getFirstName();
		member.lastName = data.getLastName();
		member.memberType = "User";
		member.isActive = data.isActiveUser();
		return member;
	}

	function createWebCenterUserFromMember(user) {
		return {
			type: "user",
			data: new WebCenterUser(user.userId || user.ID, user.userName, user.firstName, user.lastName, user.eMail, user.isActive)
		};
	}

	function createWebCenterGroupFromMember(group) {
		var members = _.map(group.members, function (user) {
			return new WebCenterUser(user.userId, user.userName, user.firstName, user.lastName, user.eMail, user.isActive);
		});
		
		var groupObject = new WebCenterGroup(group.groupId || group.ID, group.groupName || group.name, members);
		
		if (group.assignee){
			var assignee = _.find(members, function(user){
				return user.getId() === group.assignee.ID;
			});
			if (!assignee){
				assignee = createWebCenterUserFromMember(group.assignee).data;	
			}
			groupObject.setGroupAssignee(assignee);
		}
		return {
			type: "group",
			data: groupObject
		};
	}

	function createWebCenterRoleFromMember(role) {
		return {
			type: "role",
			data: new WebCenterRole(role.groupId, role.groupName)
		};
	}

	return {
		convertToMember: function (data) {
			var member = {};
			if (data instanceof WebCenterUser) {
				member = transformWebCenterUserToMember(data);
			} else if (data instanceof WebCenterGroup) {
				member.groupId = data.getId();
				member.groupName = data.getName();
				member.memberType = "Group";
				member.isRole = false;
				member.members = _.map(data.getMembers(), transformWebCenterUserToMember);
			} else if (data instanceof WebCenterRole) {
				member.groupId = data.getId();
				member.groupName = data.getName();
				member.memberType = "Group";
				member.isRole = true;
			} else {
				throw "Illegal argument: expected a WebCenterUser, WebCenterGroup or WebCenterRole";
			}
			return member;
		},
		transformWebCenterUserToMember: transformWebCenterUserToMember,

		// Function to convert the members to result entries
		createMembersXDropData: function (members, roleassignments) {

			var groupedMembers = _.groupBy(members, function (member) {
				return member.memberType === "Group" ? (member.isRole ? "role" : "group") : "user";
			});
			var xDropReadyMembers = [];

			xDropReadyMembers.push.apply(xDropReadyMembers, _.flatten(_.map(groupedMembers.role, function (role) {
				var roleAndAssignments = [];
				var roleMapping = _.find(roleassignments, function (roleassignment) {
					return roleassignment.roleID === role.groupId;
				});
				if (roleMapping) {
					roleAndAssignments.push.apply(roleAndAssignments, _.pluck(_.map(roleMapping.userRoleAssignments, createWebCenterUserFromMember), "data"));
					roleAndAssignments.push.apply(roleAndAssignments, _.pluck(_.map(roleMapping.groupRoleAssignments, createWebCenterGroupFromMember), "data"));
				}
				var webcenterRole = createWebCenterRoleFromMember(role);
				webcenterRole.data.setMembers(roleAndAssignments);
				return webcenterRole;


			})));

			xDropReadyMembers.push.apply(xDropReadyMembers, _.map(groupedMembers.user, createWebCenterUserFromMember));

			xDropReadyMembers.push.apply(xDropReadyMembers, _.map(groupedMembers.group, createWebCenterGroupFromMember));

			// make sure all groups and users are unique (especially users -> a user might be added to the project and to a group)
			return _.map(xDropReadyMembers, function (entry) {
				return new ResultEntry(entry.type, entry.data);
			});
		},
		
		createWebCenterGroupFromMember : createWebCenterGroupFromMember,
		createWebCenterUserFromMember: createWebCenterUserFromMember

	};
});
