define([], function() {
    /** 
     * Attribute.js
     * 
     * @author: JANH - September 2014
     * 
     * 
     * Definition of JavaScript class describing attribute objects used in AttributeCategory.js and 
     * AttributeCategoryLayout.js
     */

    "use strict";

    function Attribute(options) {
        this.id = options.id;
        this.prefix = options.prefix;
        this.name = options.name;
        this.description = options.description;
        this.unitId = options.unitId;
        this.rsId = options.rsId;
        this.type = options.type;
        this.value = '';
        this.hiddenDate = '';
        this.oldDate = '';
        if (options.value) {
        	this.setValue(options.value, options.valueHyperlink);
        } else {
        	this.setValue('');
        }
        this.placedOriginally = options.placedOriginally || false;
        this.placedArr = options.placedArr || new Array();
        this.isEditable = options.isEditable || false;
        this.isRequired = options.isRequired || false;

        this.rs = options.rs;
        this.restrictType = options.restrictType;

        /** @member {int} docRefMode - Mode of document reference attribute.
         * 		Supported values are 0..Latest Version; 1..Selected Version; 2..Selected Revision.
         * 		This member should be undefined if this object is not a document reference attribute. */
        this.docRefMode = undefined;
        if (this.isDocReference()) {
        	this.setDocRefMode(options.docRefMode);
        }

        this.isCheckBox = options.isCheckBox || false;
        
        /** @member {DocTableX} docTableObj - Reference to the defined document table. 
         * 		Used only in case of document reference! */
        if (this.isDocReference() && options.docTable) {
            this.docTableObj = options.docTable;
        }
        
        /** @member {Boolean} processingOnChangeEvent - Flag to remember whether the current change of
         * 		the attribute is done is part of processing of another attribute "on change" event or not
         * 		If it's TRUE, no emitting of "on change" events of given attribute should be performed.
         * 		Otherwise the event is emitted to all attribute categories on the page (standard behavior). 
         * 		See AttributeEventBus.js for implementation details. */
        this.processingOnChangeEvent = false;
    };

    /**
     * Function returning true if this attribute is of type "document reference".
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.isDocReference = function() {
        if (this.type) {
            return this.type.toLowerCase() == 'documentreference';
        } else {
            return this.unitId == '00001_0000000022';
        }
    };


    /**
     * Function returning true if this attribute is of type "text".
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.isText = function() {
        if (this.type) {
            return this.type.toLowerCase() == 'text';
        } else {
            return this.unitId == '00001_0000000001';
        }
    };

    
    /**
     * Function returning true if this attribute is of type "rich text".
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.isRichText = function() {
        if (this.type) {
            return this.type.toLowerCase() == 'richtext';
        } else {
            return this.unitId == '00001_0000000021';
        }
    };


    /**
     * Function returning true if this attribute is of type "date/time".
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.isDateTime = function() {
        if (this.type) {
       		return this.type.toLowerCase() == 'datetime';
        } else {
            return this.unitId == '00001_0000000020';
        }
    };
    

    /**
     * Function returning true if this attribute is of the same type as the attribute specified via
     * parameter 'other'.
     * 
     * @param {Attribute} other - The Attribute instance to compare this attribute with.
     * @returns {Boolean}
     */
    Attribute.prototype.isSameType = function(other) {
    	if (!other) {
    		return false;
    	}
        if (this.type && other.type) {
       		return this.type.toLowerCase() === other.type.toLowerCase();
        } else if (this.unitId) {
            return this.unitId === other.unitId;
        } else {
        	return false;
        }
    };

    /**
     * Function returning true if 'this' is the same attribute as the one specified via
     * parameter 'other'.
     * 
     * @param {Attribute} other - The Attribute instance to compare this attribute with.
     * @returns {Boolean}
     */
    Attribute.prototype.isSameInstanceAs = function(other) {
    	if (this === other) {
    		return true;
    	} else if (other && this.prefix && other.prefix && (this.prefix == other.prefix)) {
       		return true;
        } 
        return false;
    };

    /**
     * Function returning true if this attribute is of type "date/time" and its value is not a restricted set.
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.isDateTimeNotInRS = function() {
    	return this.isDateTime() && !this.rsId && !this.rs;
    };
    
    /**
     * Set whether the attribute is required or not.
     * 
     * @param {boolean} isRequired
     */
    Attribute.prototype.setIsRequired = function(isRequired) {
        this.isRequired = isRequired;
    };

    /**
     * Return true if the attribute is required, otherwise it returns false.
     * 
     * @returns {Boolean}
     */
    Attribute.prototype.getIsRequired = function() {
        if (!this.isRequired) return false;   // protection against undef and null (should always return true/false)
        return this.isRequired;
    };


    /**
     * Set the document reference attribute mode.
     * 
     * @param {int} mode - Supported values are 0..Latest Version; 1..Selected Version; 2..Selected Revision
     * 		and undefined in case of not supported input value.
     */
    Attribute.prototype.setDocRefMode = function(mode) {
    	if (!mode && (mode != 0)) {
    		// mode is undefined or null or empty string etc. - simply not defined.
    		this.docRefMode = undefined;
    	} else {
    		this.docRefMode = parseInt(mode);
    		if (isNaN(this.docRefMode)) {
    			// wrong value - pretend it's not initialized at all
    			this.docRefMode = undefined;
    		}
    	}
    };


    /**
     * Set the attribute value (and optionally also 'valueHyperlink') based on the 'newValue' parameter.
     * The hyper-link is used for document reference attributes only.
     * 
     * @param {String} newValue - New value to be set to this attribute.
     * @param {String} newValueHyperlink (optional) - New hyper-link value to be set to this attribute.
     * 		For document references the correct value must be always specified if the 'newValue' is something 
     * 		else than empty value!
     * @returns {Boolean} True if the attribute value was actually changed, false otherwise.
     */
    Attribute.prototype.setValue = function(newValue, newValueHyperlink) {
    	var oldValue = this.value;
    	if (!newValue) {
    		newValue = '';
    		newValueHyperlink = '';
    	}
    	if (this.value != newValue) {
    		this.value = newValue;
   			this.valueHyperlink = newValueHyperlink;
    	}
		if (this.isDateTimeNotInRS()) {
            this.processDateAttrValue();
        }
		return oldValue != this.value;
    };

    /**
     * Variation of setValue() method specialized for document reference attributes.
     * 
     * @param {DocTableX} docTable - object representing the new value of the document reference attribute.
     * @returns {Boolean} True if the attribute value was really changed, false otherwise.
     */
    Attribute.prototype.setDocReferenceValue = function(docTable) {
    	if (!this.isDocReference() || !docTable) {
    		return false;
    	}
    	var oldValue = this.value;
    	var newValue = docTable.getDocumentNames();
    	var newHyperlink = docTable.getHyperlinks();
    	
    	if (this.value != newValue) {
    		this.value = newValue;
   			this.valueHyperlink = newHyperlink;
    	}
    	return oldValue != this.value;
    };
    
    
    /**
     * Must be called after each date change
     * attObj.hiddenDate is used to paint a MM/DD/YYYY string in a hidden field every time the attribute 
     * cell is rendered.
     * 
     * @param {Attribute} attObj - Instance of an date attribute (Attribute class), which date should be processed.
     */
    Attribute.prototype.processDateAttrValue = function(attObj) {
    	if (!attObj) attObj = this;
        if (attObj.value) {
            var month = (parseInt(attObj.value.getMonth()) + 1) + ''; // getMonth puts January on 0, + '' converts it to a string                                    
            if (month.length <2) month = '0' + month;
            var day = attObj.value.getDate() + '';                                   
            if (day.length <2) day = '0' + day;
            attObj.hiddenDate = month + '/' + day + '/' + attObj.value.getFullYear();
            attObj.oldDate = attObj.hiddenDate;  // remember the old date
        } else {
            attObj.value = '';
            attObj.hiddenDate = '';
            attObj.oldDate = '';
        }
    };		


    /**
     * Copy the attribute value (including 'valueHyperlink', 'hiddenDate' and 'oldDate') from the specified
     * document reference property, which is a standard or simplified attribute object 'attObj') to this one.
     * The hyper-link is used for document reference attributes only.
     * The 'hiddenDate' and 'oldDate' is used for date&time attributes only.
     * 
     * @param {Attribute} attObj - Attribute object from which the value should be copied.
     */
    Attribute.prototype.copyValueFromDocRefProp = function(attObj) {
    	if (attObj) {
    		this.value = attObj.value;
    		this.isRequired = attObj.isRequired;
    		// document reference attribute specific value
    		this.valueHyperlink = (typeof attObj.valueHyperlink == 'undefined') ? '' : attObj.valueHyperlink;
    		// date&time specific values
    		this.hiddenDate = (typeof attObj.hiddenDate == 'undefined') ? '' : attObj.hiddenDate;
    		this.oldDate = (typeof attObj.oldDate == 'undefined') ? '' : attObj.oldDate;
    	}
    };

    
    /**
     * 
     */
    Attribute.prototype.setProcessingOnChangeEvent = function(value) {
    	this.processingOnChangeEvent = !!value;
    };		


    return Attribute;
});