/**
 * 3DView
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["underscore",
        "jquery",
		"3d-web-viewer/WebViewer3DAPIEXT",
		"WV3Dmod/Constants",
		"./3DViewerLauncher-view"
	],
	function (_, $, WebViewer3DAPIEXT, WV3DConstants, View3DView) {
		"use strict";
		/**
		 * @param {JQueryElement} element - element where the viewer should be shown
		 * @param {String} [docVersionId] - docVersion ID for which the 3D view should be shown. if not defined, it is taken from data-docVersionId
		 * @param {Function} [callback] - called when 3D has loaded. First parameter indicates succes. Second the error if there is any error.
		 * @param {Object} [position] - the initial camera position. See setCameraPosition.                             
		 */
		var Viewer3DLauncher = function (element, docVersionId, callback, position) {

			// private components
			this._View3DView = new View3DView(element);


			// Initialize the viewer.
			try {
				this._viewer = new WebViewer3DAPIEXT(this._View3DView.getViewerJQElement().get(0), this.processLoadingFeedback.bind(this));
				this._viewer.SetDefaultResourceRelativePath(window.webcenterConfig.requireBaseUrl + "/3d-web-viewer/Resources");

				// Set Transparency Level (between 1 and 10, recommended default = 7)
				// and Redraw flag (recommended default = true)
				this._viewer.SetQualityLevel(7, true);

				// Set Interaction Optimization Flag (recommended default = true)
				this._viewer.SetInteractionOptimization(true);

				if (!docVersionId) {
					this._docVersionId = this._View3DView.getJQElement().attr("data-docVersionId");
				} else {
					this._docVersionId = docVersionId;
				}
				
				//turn off reflections as it is known to cause performance issues on FF
				this._viewer.SetReflections("OFF");

				// A lot of display issues are solve by already resizing here.
				this._viewer.Resize();
				
				//furthermore need to register a 'window resize' callback for resizing the viewport
				//NOTE: _.throttle is used to prevent calling viewer.Resize() on every single window resize event
				var self = this;
				$(window).resize(_.throttle(function(){
					self._viewer.Resize();
				}, 40));

				this.load3DForDocVersion(this._docVersionId, callback, position);
			} catch (error) {
				this._View3DView.displayErrorMessageLangTag("COULD_NOT_LOAD_VIEWER_WEBGL", function (e) {
					window.open("https://wiki.esko.com/pages/viewpage.action?pageId=170058865");
				});
				if (window.console && window.console.error) {
					window.console.error(error);
				}
			}

		};

		/**
		 * Load 3D for given document version id
		 * @param {String} docVersionId - docVersion ID for which the 3D view should be shown.
		 * @param {Function} [callback] - called when 3D has loaded. First parameter indicates succes. Second the error if there is any error.
		 * @param {Object} [position] - the initial camera position. See setCameraPosition. 
		 */
		Viewer3DLauncher.prototype.load3DForDocVersion = function (docVersionId, callback, position) {

			// Set the docVersionID
			this._docVersionId = docVersionId;

			// Display loading message.
			this._View3DView.displayLoadingMessage();

			// Load the URL to the zipped JSON of the model asynchronously.
			var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			var self = this;
			$.getJSON(jspBaseUrl + "generate3DView.wcr?docVerID=" + this._docVersionId, function (data) {

				// Try to extract the URL from the returned result.
				if (data.result && data.result.pathTo3DJSON) {

					// Extract the URL to the model from the returned AJAXED JSON.
					var modelURL = data.result.pathTo3DJSON;

					// Apply desired settings to the viewer.
					self._viewer.LoadScene(modelURL);
					if (!position) {
						self._viewer.SetCameraScale(1.5);
						self._viewer.SetCameraAngles(30, 30);
					} else {
						self.setCameraPosition(position);
					}

					self._viewer.SetPerspectiveCamera(45);

					// Do some checks.
					var sp_level = self._viewer.GetSystemSupportLevel();
					if (sp_level["support_level"] != WV3DConstants.supportLevel.FULL) {
						alert(sp_level["issues"]["webgl"]);
					}

					// Report on the viewer.
					var system_report = self._viewer.GetSystemReport();

					if (console && console.log) {
						console.log(system_report);
					}
					
					
					// A lot of display issues are solve by already resizing here.
					self._viewer.Resize();

					if (callback) {
						callback(true);
					}
					


					// Check if we have an error result.
				} else if (data.error) {
					var errorMessage;
					// Parse and show the error message.
					if (data.error.msg) {
						self._View3DView.displayErrorMessage(data.error.msg);
						errorMessage = data.error.msg;
					} else if (data.error.addonMsg) {
						self._View3DView.displayErrorMessage(data.error.addonMsg);
						errorMessage = data.error.addonMsg;
					} else {
						self._View3DView.displayErrorMessageLangTag("UNKNOWNEXCEPTION_TAG");
					}

					if (callback) {
						callback(false, errorMessage);
					}
				}

			}).fail(function (jqxhr, textStatus, error) {
				// Parse and show the error message.
				self._View3DView.displayErrorMessage(error.message);

				if (callback) {
					callback(false, error.message);
				}
			});

			// Attach the viewer api as a property to the element.
			this._View3DView.getJQElement()[0].Viewer = this;
		};

		/**
		 * Get the current camera position if the 3D viewer is loaded or null otherwise.
		 * @returns {Object} @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		Viewer3DLauncher.prototype.getCameraPosition = function () {
			if (this._viewer) {
				var angles = this._viewer.GetCameraAngles();
				var scale = this._viewer.GetCameraScale();
				if (angles !== undefined && scale !== undefined && angles.azimuth !== undefined && angles.elevation !== undefined) {
					var position = {
						scale: scale,
						azimuth: angles.azimuth,
						elevation: angles.elevation
					};
					return position;
				}
			}
			return null;
		};

		/**
		 * Set the current camera position of the 3D widget if the 3D viewer is loaded
		 * @param {Object} position - 
		 *                 @example 
		 *                    {scale: 1.5,
		 *                    azimuth: 46.6,
		 *                    elevation: -52}
		 */
		Viewer3DLauncher.prototype.setCameraPosition = function (position) {
			if (this._viewer) {
				this._viewer.SetCameraScale(position.scale);
				this._viewer.SetCameraAngles(position.azimuth, position.elevation);
			}
		};

		/**
		 * Resize the viewer.
		 */
		Viewer3DLauncher.prototype.resize = function () {
			this._viewer.Resize();
		};

		/**
		 *
		 * @param arg0
		 * @param arg1
		 * @param arg2
		 * @param arg3
		 * @param arg4
		 */
		Viewer3DLauncher.prototype.processLoadingFeedback = function (arg0, arg1, arg2, arg3, arg4) {

			// When we get a progress update, postpone the automatically hiding of the progress bar.
			if (this._autoHideProgressAction) {
				clearTimeout(this._autoHideProgressAction);
			}

			// Check for error ...
			if (arg0 === 0) {

				// No error, findout progress ...
				var geometryLoadedP = arg1;
				var textureLoadedP = arg2;
				var numFilesToQuery = arg3;
				var numQueriedFiles = arg4;

				// Check if loading is done ...
				if (geometryLoadedP >= 100 && textureLoadedP >= 100 && numQueriedFiles >= numFilesToQuery) {

					// hide loading message.
					this._View3DView.hideAnyMessage();

				} else {

					// Determine percentage of files loaded.
					var fileLoadingP = (numQueriedFiles / numFilesToQuery) * 100;

					// Create global percentage number for these three criteria.
					var totalP = (geometryLoadedP + textureLoadedP + fileLoadingP) / 3;

					// Round this percentage to ...
					var rounded = parseFloat(Math.round(totalP * 10) / 10).toFixed(1);

					// Display progress...
					this._View3DView.displayLoadingMessage("(" + rounded + "%)");

					// Set that the progress update message should be hidden in three seconds.
					// This will get cancelled when this function, processLoadingFeedback, is called again within this method.
					// This way we avoid the scenario where the progress updater will keep to be shown when no new call this 
					// method will be made anymore from the 3d viewer (which appears to happen on FF).
					this._autoHideProgressAction = setTimeout((function () {
						this._View3DView.hideAnyMessage();
					}).bind(this), 5000);

				}
			} else {
				// Display progress...
				this._View3DView.displayErrorMessage("Error occured while loading 3D model. (code:" + arg0 + ")");
			}
		};

		/**
		 * Get a jQuery wrapped reference to the encapsulating div of this component.
		 *
		 * @returns {jQuery} The jQuery wrapped reference to the encapsulating div of this component.
		 */
		Viewer3DLauncher.prototype.getJQElement = function () {
			return this._View3DView.getJQElement();
		};

		/**
		 * Method to automatically apply this module to an input or select element ...
		 */
		Viewer3DLauncher.wcrinit = function (element) {
			new Viewer3DLauncher(element);
		};

		/**
		 * Method to automatically destroy this component once initialized for a div...
		 *
		 * @param div {DomElement} the DomElement to destroy again
		 */
		Viewer3DLauncher.wcrdestroy = function (element) {

		};

		return Viewer3DLauncher;

	});