define(["jquery",
		"underscore",
		"doctablex/DocTableXUtils",
		"doctablex/AJAXDataCache"
	],
	function (	$,
				_,
				DocTableXUtils,
				AJAXDataCache) {
		"use strict";

		/**
		 * This will be used to create some real buttons through inheritance
		 * 
		 * @exports AbstractButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped, so it can be added later
		 */
		var AbstractButton = function(docVerId, ctx) {
			// context for button and inline form part
			this.btnCtx = null;
			if(ctx){
				this.btnCtx = ctx;
			}
			
			this.baseUrl = window.webcenterConfig.jspBaseUrl || './';
			
			this.loaded = false;
			this.afterLoadCallbacks = [];

			// store parameters
			this.docVerId = docVerId;
			
			this.isVisible = false;
			
			// load info from server
			this.loadDocumentInfo();
		};
		
		/**
		 * Load data from server
		 * Default implementation is to load the document info 
		 */
		AbstractButton.prototype.loadDocumentInfo = function() {	
			if(this.docVerId){
				var docInfoLoader =  AJAXDataCache.getInstance().getDocInfoWithThumbnail(this.docVerId);
				$.when(docInfoLoader).done(_.bind(function(docInfo){
					if (AJAXDataCache.isEmptyValue(docInfo)) {
			            // when 'getDocInfoWithThumbnail' returns 'emptyObject' it means the document 
						// info cannot be loaded (probably because of not enough permissions)
						return;
					}
					this.loaded = true;
					this.updateDocumentInfo(docInfo);
					this.render();
					if(this.afterLoadCallbacks.length > 0){
						for(var i=0 ; i<this.afterLoadCallbacks.length ; i++){
							this.afterLoadCallbacks[i]();
						}
						this.afterLoadCallbacks = [];
					}
				}, this));
			}
		};
		
		/**
		 * Render the whole form and fill out previously filled values
		 */
		AbstractButton.prototype.render = function() {	
			// render button
			this.renderBtn(this.btnCtx);
		};
		
		/**
		 * Render the button in a given context. This means the button can be on several locations.
		 * 
		 * @param {JQuery object} ctx - context
		 */
		AbstractButton.prototype.renderBtn = function(ctx) {	
			if(ctx){
				if(this.loaded){
					// render button
					ctx.empty();
					if(this.isVisible){
						this._renderBtnImpl(ctx);
					}
				} else {
					this.afterLoadCallbacks.push(_.bind(this.renderBtn,this,ctx));
				}
			}
		};
		
		/**
		 * Implementation of rendering of button
		 * 
		 * @param ctx
		 */
		AbstractButton.prototype._renderBtnImpl = function(ctx) {	
			// override me
		};
		
		/**
		 * Open document in new tab
		 */
		AbstractButton.prototype.openInNewTab = function(url) {
			//open the URL in a new tab
			var tab = window.open(url, '_blank');
			if (tab != null) {
				tab.focus();	
			};
		};

		/**
		 * Set context in which the button should be rendered
		 * @param {JQuery Object} context
		 */
		AbstractButton.prototype.setBtnContext = function(context) {
			this.btnCtx = context;
			this.render();
		};
		
		/**
		 * Update document info to determine visibility.
		 * @param {Object} docInfo
		 */
		AbstractButton.prototype.updateDocumentInfo = function(docInfo) {
			// override me
		};

		return AbstractButton;
	});