define(["component/actionbuttons/AbstractButton",
        "jquery",
		"underscore",
		"utils/webi18n!ActionBtnStrings",
		"doctablex/AJAXDataCache",
		"doctablex/DocTableXUtils"
	],
	function (	AbstractButton,
				$,
				_,
				ActionBtnStrings,
				AJAXDataCache,
				DocTableXUtils) {
		"use strict";

		/**
		 * This will generate a button to open the document in ArtiosCAD
		 * It will only be visible for CAD file and only when the open in ArtiosCAD preference is configured
		 * 
		 * @exports OpenACADButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped now to be added later
		 */
		var OpenACADButton = function(docVerId, ctx) {
			AbstractButton.call(this, docVerId, ctx);
		};
		
		// extend from the AbstractButton
		OpenACADButton.prototype = Object.create(AbstractButton.prototype);
		OpenACADButton.prototype.constructor = OpenACADButton;
		
		/**
		 * Load data from server
		 * Load the document info and some global settings
		 * @override
		 */
		OpenACADButton.prototype.loadDocumentInfo = function() {	
			if(this.docVerId){
				var settingsLoader = AJAXDataCache.getInstance().getGlobalDocumentSettings();
				var docInfoLoader =  AJAXDataCache.getInstance().getDocInfoWithThumbnail(this.docVerId);
				$.when(settingsLoader, docInfoLoader).done(_.bind(function(settings, docInfo){
					if (AJAXDataCache.isEmptyValue(docInfo)) {
			            // when 'getDocInfoWithThumbnail' returns 'emptyObject' it means the document 
						// info cannot be loaded (probably because of not enough permissions)
						return;
					}
					this.loaded = true;
					this.updateDocumentInfo(docInfo,settings);
					this.render();
					if(this.afterLoadCallbacks.length > 0){
						for(var i=0 ; i<this.afterLoadCallbacks.length ; i++){
							this.afterLoadCallbacks[i]();
						}
						this.afterLoadCallbacks = [];
					}
				}, this));
			}
		};
		
		/**
		 * Render the button in a given context. This means the button can be on several locations.
		 * @override
		 * @param {JQuery object} ctx - context
		 */
		OpenACADButton.prototype._renderBtnImpl = function(ctx) {	
			var btn = $('<a/>')
					.addClass('action actionBtn')
					.attr('title', ActionBtnStrings.t('DOCOPENINACAD'));
			var url = '';
			btn.click(_.bind(function(){
				//accepted 'openMode' param values are: "Standard"; "Enterprise"
    			if (this.openType === "Standard") {
					//Open in a native app (ACAD file types need to be properly registered their MIME types) 
					//NOTE: Using 'Other' document type setting as we want to mimic the behavior
    				var menuParam = DocTableXUtils.getMenuFileParamValue();
    				url = this.baseUrl + "doctypeotherview.jsp?docVerID="+this.docVerId+"&menu_file="+menuParam;
    			} else if (this.openType === "Enterprise") {
    			    //use registered ACAD-Enterprise launcher protocol
    			    //NOTE: 'acade' protocol has to be enabled on the client system
    				url = encodeURI("acade:@"+this.projectName+":"+this.docName);
    			}
    			_.defer(function(){window.location = url;});
    			return false; // to prevent browsing to #
			},this));
			
			var icon = $('<i/>').addClass('icon icon-normal icon-WCR_Actions-ACAD12');
			btn.append(icon);
			ctx.replaceWith(btn);
		};

		/**
		 * Update Document info based on server response
		 * @override
		 */
		OpenACADButton.prototype.updateDocumentInfo = function(docInfo, config) {
			// set basic info
			this.docName = docInfo.docName;
			if (docInfo.projList.length>0) {
				this.projectName = docInfo.projList[0].name;
			}
			
			// set open type
			// need download permission
			if(docInfo.userPerms.download){
				// document needs to be ACAD document
				if((docInfo.docTypeID == '00001_0000000001') || (docInfo.docTypeID == '00001_0000000002') || (docInfo.docTypeID == '00001_0000000003')){
					// set open type
					this.openType = config.openCadType;
				}
			}
			
			this.isVisible = (this.openType && this.openType !== "None");
        };
		
        /*
    	 * This component is createable using data-init
    	 */
        OpenACADButton.wcrinit = function(div) {
        	var docVerId = $(div).attr("data-docverid");
    		new OpenACADButton(docVerId, $(div));
    	};
		
		return OpenACADButton;
	});