define(["component/actionbuttons/AbstractButton",
        "Template",
		"tpl!component/actionbuttons/InlineUploadNewVersionBlock",
		"jquery",
		"underscore",
		"utils/webi18n!ActionBtnStrings",
		"doctablex/AJAXDataCache",
		"doctablex/DocTableXUtils",
		"utils/WCEvent",
		"component/styledfileinput/StyledFileInput",
		"utils/webcss!./actionbtn-style.css"],
	function (	AbstractButton,
				Template,
				tpl,
				$,
				_,
				ActionBtnStrings,
				AJAXDataCache,
				DocTableXUtils,
				WCEvent,
				StyledFileInput) {
		"use strict";

		/**
		 * This will generate an upload new version button and the inline form field when the buttun is clicked.
		 * The button will only be shown when the user can really upload a new version (permissions, locks, AE pagelists, etc.)
		 * 
		 * @exports UploadNewVersionButton 
		 * @constructor
		 * @param {String} docVerId
		 * @param {String} (optional) ctx - context to render button in; can be skipped now to be added later
		 * @param {Object} currentUser - currentUser object
		 * @param {Object} options - various options
		 */
		var UploadNewVersionButton = function(docVerId, ctx, currentUser, options) {
			this.isExpanded = false;

			// context for button and inline form part
			this.rowCtx = null;
			
			// store parameters
			this.currentUser = currentUser;
			if(!this.currentUser){
				this.currentUser = gCurrentUser;
			}

			if(options){
				this.useInlineForm = options.useInlineForm;
				this.docReferenceMode = options.docReferenceMode;
				this.onCancelCallback = options.onCancelCallback;
			}
			
			if(this.useInlineForm){
				// initialize template
				this.t = new Template(ActionBtnStrings, tpl);
				this.t.d.formdata = {};
				
				// set bindings
				this.t.bind("click", "button.remove", this.clearRow, this);
				this.t.bind("change", ":input", _.bind(this.storeFormData, this));
			} else {
				// to have a shared datamodel with the template
				this.t = {d:{}};
			}
			
			if(options){
				if(options.alwaysUpdateAllProjects){
					this.t.d.alwaysUpdateAllProjects = true;
				}
				this.useInlineForm = options.useInlineForm;
				if(options.uniqueIdPrefix){
					this.t.d.idprefix = options.uniqueIdPrefix;
				} else {
					this.t.d.idprefix = '';
				}
			}

			AbstractButton.call(this, docVerId, ctx);
		};
		
		// extend from the AbstractButton
		UploadNewVersionButton.prototype = Object.create(AbstractButton.prototype);
		UploadNewVersionButton.prototype.constructor = UploadNewVersionButton;
		
		/**
		 * Load data from server
		 * Load the document info and some global settings
		 * @override
		 */
		UploadNewVersionButton.prototype.loadDocumentInfo = function() {	
			if(this.docVerId){
				var docInfoLoader =  AJAXDataCache.getInstance().getDocInfoWithThumbnail(this.docVerId);
				var settingsLoader = null;
				
				if(this.useInlineForm){
					settingsLoader = AJAXDataCache.getInstance().getGlobalDocumentSettings();
				}
				
				$.when(settingsLoader, docInfoLoader).done(_.bind(function(settings, docInfo){
					if (AJAXDataCache.isEmptyValue(docInfo)) {
			            // when 'getDocInfoWithThumbnail' returns 'emptyObject' it means the document 
						// info cannot be loaded (probably because of not enough permissions)
						return;
					}
					this.loaded = true;
					this.updateDocumentInfo(docInfo,settings);
					this.render();
					if(this.afterLoadCallbacks.length > 0){
						for(var i=0 ; i<this.afterLoadCallbacks.length ; i++){
							this.afterLoadCallbacks[i]();
						}
						this.afterLoadCallbacks = [];
					}
				}, this));
			}
		};
		
		/**
		 * Render the whole form and fill out previously filled values
		 * @override
		 */
		UploadNewVersionButton.prototype.render = function() {	
			// render button
			this.renderBtn(this.btnCtx);
			
			if(this.isExpanded && this.rowCtx){
				// render row content
				this.t.render();
				
				// fill out some values
				if(this.fileField){
					this.rowCtx.find("input[name^=newVersionDocFile_]").replaceWith(this.fileField);
				}
				if(this.t.d.duePeriod){
					 this.rowCtx.find("select[name^=due_periodOfDay]").val(this.t.d.duePeriod);
				}
				
				// Bootstrap the file input.
				StyledFileInput.wcrinit($(".newVersionDocFile", this.t.getContext())[0]);
				
				// Bootstrap the datepicker input.
				var datePickerElement = $(".longFormat_due_date_picker", this.t.getContext());
				datePickerElement.datepicker({
					dateFormat: "M dd, yy", 
					onClose: function(dateText, inst) {
						var element = $(".due_periodOfDay_picker", this.t.getContext())[0];
						if(element && element.selectedIndex == 0){
							element.selectedIndex = (36 + 1);
						}
						var dateVal = dateText;
						if (dateText != '')	{
							var date = new Date(dateText);
							dateVal = formatDate(date, "MM/dd/yyyy");
						}
						$(".due_date", this.t.getContext()).val(dateVal);																
					}.bind(this)
				});
			}
		};

		/**
		  * Render the button in a context
		  * @override
		  * @param {JQuery object} ctx - context
		  */
		UploadNewVersionButton.prototype._renderBtnImpl = function(ctx) {
			var btn = $('<a/>')
					.addClass('action actionBtn')
					.attr('title', ActionBtnStrings.t('DOCDETAILSCHECKIN_TAG'));
			btn.click(_.bind(function(){
				this.btnClicked();
				return false; // to prevent browsing to #
			},this));

			var icon = $('<i/>').addClass('icon icon-normal icon-WCR_Actions-Upload12');
			btn.append(icon);
			ctx.replaceWith(btn);
		};
		
		/**
		 * Render the whole form and fill out previously filled values
		 */
		UploadNewVersionButton.prototype.btnClicked = function() {	
			if(this.useInlineForm){
				// show inline form
				this.isExpanded = true;
				this.render();
				WCEvent.emit("btnClicked.UploadNewVersion."+this.docVerId);
			} else {
				var menuParam = DocTableXUtils.getMenuFileParamValue();
				// navigate to upload new version page
				var url = this.baseUrl + 'dodocdetailsactions.jsp?actionVal=DirectCheckinDoc&docVerID='+this.docVerId+'&menu_file='+menuParam;
				
				//open the URL in a new tab
				var tab = window.open(url, '_blank');
				if (tab != null) {
					tab.focus();	
				};
			}
		};
		
		/**
		 * Remove all fields
		 */
		UploadNewVersionButton.prototype.clearRow = function() {
			if(this.onCancelCallback){
				this.onCancelCallback();
			}
			this.rowCtx.empty();
			this.isExpanded = false;
		};
		
		/**
		 * The allowed characters for revisions
		 */
		UploadNewVersionButton.prototype.revisionLetters = ['A','B','C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 
		                                                    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'];
		/**
		 * Convert a revision number to a string (e.g. 2 --> B)
		 * After 'Z', it will just append a new character to the 'Z'. e.g. 26 -> 'Z'; 27 -> 'ZA'; 28 -> 'ZB': 'ZZ'+1 -> 'ZZA'
		 * 
		 * @param revisionNr
		 * @returns {String}
		 */
		UploadNewVersionButton.prototype.getRevisionLetter = function(revisionNr) {
			var revision = '';
			if(revisionNr && !isNaN(revisionNr)){
				while(revisionNr > this.revisionLetters.length){
					revision += this.revisionLetters[this.revisionLetters.length - 1];
					revisionNr -= this.revisionLetters.length;
				}
				if(revisionNr > 0){
					revision += this.revisionLetters[revisionNr - 1];
				}
			} else {
				revision += this.revisionLetters[0];
			}
			
			return revision;
		};
		
		/**
		 * Set context in which the form should be rendered
		 * @param {JQuery Object} context
		 */
		UploadNewVersionButton.prototype.setRowContext = function(context) {
			this.rowCtx = context;
			this.t.setContext(context);	
			this.render();
		};
		
		
		/**
		 * Store form fields in the object. This way, if we have to build it up from scratch (repaint), we can keep the values
		 * 
		 */
		UploadNewVersionButton.prototype.storeFormData = function() {
			this.fileField = this.rowCtx.find("input[name^=newVersionDocFile_]");
			this.t.d.formdata.newVersionDocDesc = this.rowCtx.find("input[name^=newVersionDocDesc_]").val();
			this.t.d.incrementRevision = (this.rowCtx.find("input[name^=incrementRevision_]").attr("checked") == "checked");
			this.t.d.updateAllProjects = (this.rowCtx.find("input[name^=updateAllProjects_]").attr("checked") == "checked");
			this.t.d.AppCycleFlag = (this.rowCtx.find("input[name^=AppCycleFlag]").attr("checked") == "checked");
			
			this.t.d.dueDate = this.rowCtx.find("input[name^=longFormat_due_date]").val();
			this.t.d.dueDateH1 = this.rowCtx.find("input[name^=due_date]").val();
			this.t.d.dueDateH2 = this.rowCtx.find("input[name^=save_due_date]").val();
			this.t.d.duePeriod = this.rowCtx.find("select[name^=due_periodOfDay]").val();
		};
		
        /**
		 * Update Document info and global settings based on server response
		 * @override
		 */
		UploadNewVersionButton.prototype.updateDocumentInfo = function(docInfo, settings) {
			//-------------------------
			// read document info
			//-------------------------
			this.t.d.docVerId = this.docVerId;
			this.t.d.docId = docInfo.docID;
			this.t.d.docName = docInfo.docName;
			this.t.d.projectList = docInfo.projList;
			var docRevLetter = null;
			try{
				docRevLetter = parseInt(docInfo.docRevIdx);
			}catch(ex){
				// parsing exception, use 0
				docRevLetter = 0;
			}
			this.t.d.lastRevision = this.getRevisionLetter(docRevLetter + 1); // +1, because we need the next revision

			// approval options
			if(docInfo.appStatus){
				this.t.d.approvalCycleState = docInfo.appStatus.cycleStatusStringCode;
			}
			
			this.t.d.showDueDate = docInfo.userPerms.changeApp?'1':'0';
			
			// Upload new version
            if(!docInfo.docIsUrl && !(docInfo.docTypeID == "00001_0000000014" && docInfo.docSourceId)){
    			if( (!docInfo.docCheckedOutOwner || (docInfo.docCheckedOutOwner == this.currentUser.userID)) && (docInfo.userPerms.isProjMgr || docInfo.userPerms.checkout)){
    				this.isVisible = true;
    			}
    		}
            
            //---------------------------
            // read info from sessions
            // Settings will only be loaded when inline form is used
            //---------------------------
            if(settings){
            	var approvalRestartOption = settings.approvalRestartOption;
    			if(approvalRestartOption == '1'){
    				this.t.d.initialStartApprovalValue = true;
    			} else if(approvalRestartOption == '2') {
    				this.t.d.initialStartApprovalValue = false;
    			} else if(settings.approvalCycleState == 'cycle_started' || settings.approvalCycleState == 'cycle_finished') {
    				this.t.d.initialStartApprovalValue = true;
    			} else {
    				this.t.d.initialStartApprovalValue = false;
    			}
    			
    			this.t.d.allowRaiseRevision = this.allowRevisionChange(settings);
    			this.t.d.approvalRestartEnableCheckbox = settings.approvalRestartUploaderCanOverrule == "1";
            }
            
            this.t.d.baseUrl = this.baseUrl;
        };

        /**
         * Helper method gathering logic around "Raise Revision" functionality (checkbox)
         * @param {Object} settings - settings loaded from the server
         * @returns {Boolean} - True if the raising of revision should be allowed, false otherwise.
         */
		UploadNewVersionButton.prototype.allowRevisionChange = function(settings)
        {
			// allow/draw "Raise Revision" checkbox only if it's enabled in configuration and 
			// (in case of document reference attribute value) the mode of document reference is 
			// set to "Latest Version" only
        	return (settings.allowRaiseRevision == '1') 
        			&& ((this.docReferenceMode === undefined) || (this.docReferenceMode == 0));
        };
        
        /**
		 * Validate the form fields
		 */
		UploadNewVersionButton.prototype.validate = function() {
			var errors = [];
			// only validat if expanded
	        if(this.isExpanded){
	        	// check file
	        	var fileField = this.rowCtx.find("input[name^=newVersionDocFile_]");
	        	var file = fileField.val();
	        	if(!file){
	        		var errorObj = {
	        			isBlocking : true,
	    				msg : ActionBtnStrings.t('DocRowFileNotChosen'),
	    				element : fileField
	    			};
	        		errors.push(errorObj);
	        	} else if (DocTableXUtils.getResolvedExtension(DocTableXUtils.getFileExtension(file)) != DocTableXUtils.getFileExtension(this.t.d.docName)){
	        		// check file extension 
	        		var errorObj = {
	        			isBlocking : true,
	    				msg : ActionBtnStrings.t('BADCHECKINEXT'),
	    				element : fileField
	    			};
	        		errors.push(errorObj);
	        	}
	        	
	        	// check new version comment
	        	var commentField = this.rowCtx.find("input[name^=newVersionDocDesc_]");
	        	var comment = $.trim(commentField.val());
	        	if(!comment){
	        		var errorObj = {
	        			isBlocking : true,
	    				msg : ActionBtnStrings.t('DOCDETAILSCHECKINCOMMENT_TAG'),
	    				element : commentField
	    			};
	    			errors.push(errorObj);
	        	}
	        }
	        return errors;
        };
        
        /*
    	 * This component is createable using data-init
    	 */
        UploadNewVersionButton.wcrinit = function(div) {
        	var docVerId = $(div).attr("data-docverid");
        	var inlineCtx = $(div).attr("data-inlinectx");
        	var isInline = $(div).attr("data-inline");
    		var btn = new UploadNewVersionButton(docVerId, $(div), window.gCurrentUser, {useInlineForm: (isInline=="true")});
    		btn.setRowContext($("#"+inlineCtx));
    	};
        
		return UploadNewVersionButton;
	});