define(["underscore",
        "jquery",
        "./ActionMenuBase",
        "utils/init",
		"utils/webi18n!ActionMenu"], function (
	_, $,
	ActionMenuBase,
	init,
	i18n) {
	"use strict";

	/**
	 * Action Menu component for displaying actions
	 * @class
	 * @constructor
	 * @exports ActionMenu
	 * @param {JQueryObj} context - div in which the action menu is defined
	 * @param {JQueryObj} [checkboxContainer] - container for which action menu is defined (this is used for data-require-selection)
	 * @param {String}	[checkboxSelector] - Selector string to select checkboxes in the checkboxContainer (this is used for data-require-selection)
	 */
	var ActionMenu = function (context, checkboxContainer, checkboxSelector) {
		// Initialize ActionMenuBase (call constructor of superclass)
		ActionMenuBase.call(this, context);

		//
		// Required selection handling
		//

		// Set the checkbox information (used for listening to selection)
		this._checkboxContainer = checkboxContainer;
		this._checkboxSelector = checkboxSelector;

		// Handle updates of the selection
		this.updateSelectionAttributes();
		$(this._checkboxContainer).on("change", this._checkboxSelector, _.bind(this.updateSelectionAttributes, this));
	};

	// ActionMenu extends ActionMenuBase and inherits all the methods...
	ActionMenu.prototype = Object.create(ActionMenuBase.prototype);
	ActionMenu.prototype.constructor = ActionMenu;


	/**
	 * Update the visibility of the data-require-selection actions
	 */
	ActionMenu.prototype.updateSelectionAttributes = function () {
		var itemsSelected = $(this._checkboxSelector, this._checkboxContainer).filter(":checked").length;

		_.each($(".action[data-require-selection-min],.action[data-require-selection-max]", this._context), function (action) {
			action = $(action);

			var min = 0;
			try {
				min = parseInt(action.attr("data-require-selection-min"), 10);
			} catch (ex) {
				// ignore
			}
			if (isNaN(min)) {
				min = 0;
			}


			var max = -1;
			try {
				max = parseInt(action.attr("data-require-selection-max"), 10);
			} catch (ex) {
				// ignore
			}

			if (isNaN(max)) {
				max = -1;
			}


			var disabled = false;
			var title = action.attr("data-title");

			if (max < 0) {
				if (itemsSelected < min) {
					disabled = true;
					title = i18n.t("ACTIONMENU_SELECT_X_OR_MORE_ITEMS_TO_ENABLE", min);
				}
			} else {
				// max defined
				if (min === max && itemsSelected != max) {
					disabled = true;
					title = i18n.t("ACTIONMENU_SELECT_X_ITEMS_TO_ENABLE", max);
				} else if (itemsSelected < min || itemsSelected > max) {
					disabled = true;
					title = i18n.t("ACTIONMENU_SELECT_X_TO_Y_ITEMS_TO_ENABLE", min, max);
				}
			}

			action.toggleClass("disabled", disabled).attr("title", title);


		}, this);
	};

	/**
	 * Set all the action buttons enabled or not
	 * @param {Boolean} enabled - True to follow their default behavior (e.g. selection based), false to disable them
	 */
	ActionMenu.prototype.setEnabled = function (enabled) {
		_.each($(".action", this._context), function (action) {
			$(action).toggleClass("disabled", !enabled);
		}, this);
		if (enabled) {
			this.updateSelectionAttributes();
		}
	};

	/**
	 * Which items does this action menu execute on?
	 * 
	 * Fill in the ActionExecuteEvent
	 * 
	 * @Override
	 * @param {ActionExecuteEvent} actionExecuteEvent - the event to fill up
	 */
	ActionMenu.prototype._fillActionExecuteEvent = function (actionExecuteEvent) {
		// Call super method
		ActionMenuBase.prototype._fillActionExecuteEvent.call(this, actionExecuteEvent);

		// Put in the items...
		actionExecuteEvent._items = _.map(this.getSelectedItems(), function (e) {
			// Pass more information in the future? Like the id's?
			return {
				checkbox: e
			};
		});
	};

	// Note: move the following methods to the Table ?

	/**
	 * Get an array with the values of the selected checkboxes
	 * @returns {Array.<jQuery>} array with values of selected checkboxes
	 */
	ActionMenu.prototype.getSelectedItems = function () {
		return _.map($(this._checkboxSelector, this._checkboxContainer).filter(":checked"), function (item) {
			return $(item);
		});
	};


	/**
	 * Init the component
	 * @param {JQueryObj,Node} element - element to initialize the action menu on
	 */
	ActionMenu.wcrinit = function (element) {
		if (!(element instanceof $)) {
			element = $(element);
		}

		var data = ActionMenuBase._wcrinitCollectSettings(element);

		// Initialize
		var actionMenu = new ActionMenu(element, data.checkboxContainer, data.checkboxSelector);

		// Find the parent Table component (if any)
		if (data.wcrtable.length > 0) {
			init.whenInitialized(data.wcrtable, function () {
				var tableComponent = data.wcrtable.get(0).Table;
				actionMenu.setTable(tableComponent);
			});
		}

		// Load & attach handler
		ActionMenuBase._wcrinitRegisterActionHandler(element, data, actionMenu);
	};

	return ActionMenu;
});