define(["./GlobalActionHandler",
        "./ActionExecuteEvent",
		"utils/webi18n!ActionMenu",
		"utils/ResponsiveFit",
		"require",
		"underscore",
		"jquery"],
	function (GlobalActionHandler,
		ActionExecuteEvent,
		i18n,
		ResponsiveFit,
		require,
		_,
		$) {
		"use strict";

		var ACTION_VISIBILITIES = {
			PRIMARY: 2,
			ALWAYS_VISIBLE: 1,
			DEFAULT: 0,
			SECONDARY: -1
		};


		/**
		 * Base class for Action Menu's.
		 * Use the implementation "ActionMenu" or "RowActionMenu" instead.
		 * 
		 * @class
		 * @constructor
		 * @exports ActionMenuBase
		 * @param {JQueryObj} context - div in which the action menu is defined
		 * @param {Boolean} smallmore - whether to use the very small more actions button
		 */
		var ActionMenuBase = function (context, smallmore, optimized) {
			this._context = context;
			this._table = null;
			this._smallmore = smallmore || false;
			this._optimized = optimized;

			// Action Handlers... These will be executed when an action is clicked.
			this._actionHandlers = [];

			this._initMoreContainer();

			// initialize the responsive fit (call constructor of superclass)
			ResponsiveFit.call(this, this._context, this._moreElement, this._moreContainer);

			// we want to maintain the original sequence
			this.setMaintainSequence(true);

			this._initialize();
			this._listenForEvents();

			// register the global action handler
			this.registerActionHandler(GlobalActionHandler.executeAction);
		};

		// ActionMenuBase extends ResponsiveFit
		ActionMenuBase.prototype = Object.create(ResponsiveFit.prototype);
		ActionMenuBase.prototype.constructor = ActionMenuBase;

		/**
		 * Initialize the more container
		 */
		ActionMenuBase.prototype._initMoreContainer = function () {
			if (this._optimized) {
				// Find the elements which are in the html
				this._moreElement = this._context.find("> .more");
				this._moreButton = this._moreElement.find("> .morebutton");
				this._moreContainer = this._moreElement.find("> .dropdown-menu");
				var optMoreText = this._moreButton.find("span.moretext");
				if (optMoreText.length > 0) {
					this._moreText = optMoreText;
				}
				var optMoreCaret = this._moreButton.find("span.caret");
				if (optMoreCaret.length > 0) {
					this._moreCaret = optMoreCaret;
				}
				var optMoreIcon = this._moreButton.find("i.icon");
				if (optMoreIcon.length > 0) {
					this._moreIcon = optMoreIcon;
				}
			} else {
				// Add the items in the html
				this._moreContainer = $("<ul>", {
					'class': "dropdown-menu"
				});
				this._moreButton = $("<a>", {
					'class': "btn btn-default dropdown-toggle morebutton",
					'href': "#",
					'data-toggle': "dropdown"
				});
				this._moreElement = $("<div>", {
					'class': "dropdown more"
				}).append(this._moreButton, this._moreContainer);


				this._updateMoreDisplayMode(this._smallmore);
				this._setMoreText(i18n.tFallback("ACTIONMENU_MORE_ACTIONS", "More..."));
				this._context.append(this._moreElement);
			}
		};

		ActionMenuBase.prototype._updateMoreDisplayMode = function (compact) {
			if (compact) {
				if (this._moreText) {
					this._moreText.detach();
				}
				if (this._moreCaret) {
					this._moreCaret.detach();
				}

				if (!this._moreIcon) {
					this._moreIcon = $("<i>", {
						"class": "icon icon-WCR_Actions-More12"
					});
				}

				if (this._moreButton.find(this._moreIcon).length === 0) {
					this._moreButton.prepend(this._moreIcon);
				}
			} else {
				if (this._moreIcon) {
					this._moreIcon.detach();
				}

				if (!this._moreText) {
					this._moreText = $("<span>");
				}

				if (!this._moreCaret) {
					this._moreCaret = $("<span>", {
						'class': "caret"
					});
				}

				if (this._moreButton.find(this._moreCaret).length === 0) {
					this._moreButton.prepend(this._moreCaret).prepend(this._moreText);
				}

			}
		};

		ActionMenuBase.prototype._setMoreText = function (text) {
			this._moreButton.prop("title", text);
			this._moreText.text(text);
		};

		ActionMenuBase.prototype._storeSequence = function () {
			if (!this._optimized) {
				ResponsiveFit.prototype._storeSequence.call(this);
			}
		};

		ActionMenuBase.prototype._storeSequence = function () {
			if (!this._optimized) {
				ResponsiveFit.prototype._storeSequence.call(this);
			}
		};


		/**
		 * Intialize the action menu. Make sure all low visibility actions are hidden
		 * Add primary classes if needed
		 * Apply show icon only if needed
		 */
		ActionMenuBase.prototype._initialize = function () {

			if (this._optimized) {
				return;
			}

			_.each(this._getMainChildren(), function (action) {
				action = $(action);
				var visibility = this._getActionVisibility(action);

				// primary action
				if (visibility >= ACTION_VISIBILITIES.PRIMARY) {
					action.addClass("action-primary");
				}


				action.attr("title", action.text());
				action.attr("data-title", action.text());

				if (action.attr("data-icon-only")) {
					action.text("");
				}

				this._addIconToChild(action);


				// secondary action
				if (visibility <= ACTION_VISIBILITIES.SECONDARY) {
					this._appendToMore(action);
				}
			}, this);

			this._updateMoreVisibility();

			this.resize();

			this._context.addClass("initialized");
		};

		/**
		 * Listen for resize and checkbox changed events
		 */
		ActionMenuBase.prototype._listenForEvents = function () {
			this._initResizeBehavior();

			// listen to action clicks in the action menu component
			this._context.on("click", "a.action", _.partial(function (event, actionmenu) {
				actionmenu._executeAction($(this), event);
			}, _, this));

			// Sometimes (inside a Table component for example), dropdowns are moved OUTSIDE of the action menu...
			// so we also need to listen to the items in the dropdown separately!
			this._moreContainer.on("click", "a.action", _.partial(function (event, actionmenu) {
				// Only execute events on the dropdown here if the dropdown is outside of the action menu when the event is called.
				// (Otherwise we will call the action twice)
				if (!$.contains(actionmenu._context.get(0), actionmenu._moreContainer.get(0))) {
					actionmenu._executeAction($(this), event);
				}
			}, _, this));
		};

		/**
		 * Listen for events fired from elements inside the table
		 * @private
		 */
		ActionMenuBase.prototype._listenForTableEvents = function () {
			if (this._table) {
				this._table.getContext().on("click", ".tableaction", _.partial(function (event, actionmenu) {
					actionmenu._executeAction($(this), event);
				}, _, this));
			}
		};

		/**
		 * Init the resizing behavior 
		 */
		ActionMenuBase.prototype._initResizeBehavior = function () {
			$(window).resize(_.throttle(_.bind(function () {
				this.resize();
			}, this), 40));
		};

		/**
		 * Can an element be moved to the main container
		 * @Override
		 * @param {JQueryObj} child
		 * @returns {Boolean}
		 */
		ActionMenuBase.prototype._canMoveToMain = function (child) {
			return this._getActionVisibility(child.find(".action")) >= ACTION_VISIBILITIES.DEFAULT;
		};

		/**
		 * Can an element be moved to the more container
		 * @Override
		 * @param {JQueryObj} action
		 * @returns {Boolean}
		 */
		ActionMenuBase.prototype._canMoveToMore = function (action) {
			return this._getActionVisibility(action) <= ACTION_VISIBILITIES.DEFAULT;
		};


		/**
		 * Add the icon element back to a certain element
		 * @param {JQueryObj} action - Action JQueryObject
		 */
		ActionMenuBase.prototype._addIconToChild = function (action) {

			if (action.attr("data-icon")) {
				// Only add the icon if there's not already one ...
				if ($("i", action).length === 0) {
					// Add the icon
					$(action).prepend($("<i>", {
						'class': action.attr("data-icon")
					}));
				}
			}
		};

		/**
		 * Move an action to the more container
		 * This will wrap the action in a <li> object
		 * It will also make sure the action text is displayed
		 * @Override
		 * @param {JQueryObj} child 
		 */
		ActionMenuBase.prototype._appendToMore = function (child) {
			if (child.attr("data-icon-only") && child.attr("data-icon-only") != "0") {
				child.text(child.attr("data-title"));
				this._addIconToChild(child);
			}

			child = $("<li>").append(child).attr("data-sequence", child.attr("data-sequence"));

			ResponsiveFit.prototype._appendToMore.call(this, child);
			return this._moreContainer.append(child);
		};

		/**
		 * Move an action to the more container
		 * This will wrap the action in a <li> object
		 * It will also make sure the action text is displayed
		 * @Override
		 * @param {JQueryObj} child 
		 */
		ActionMenuBase.prototype._prependToMore = function (child) {
			if (child.attr("data-icon-only") && child.attr("data-icon-only") != "0") {
				child.text(child.attr("title"));
				this._addIconToChild(child);
			}

			child = $("<li>").append(child).attr("data-sequence", child.attr("data-sequence"));
			return ResponsiveFit.prototype._prependToMore.call(this, child);
		};

		/**
		 * Move an action to the main container
		 * This will remove the wrapping  <li> object
		 * It will also make sure the data-icon-only is respected
		 * @Override
		 * @param {JQueryObj} child 
		 */
		ActionMenuBase.prototype._appendToMain = function (li) {

			var child = li.find(".action");
			li.remove();

			if (child.attr("data-icon-only")) {
				child.text("");
				this._addIconToChild(child);
			}

			return ResponsiveFit.prototype._appendToMain.call(this, child);
		};

		/**
		 * This will make the more button visible/invisible
		 * It will update the label to "Actions..." if there are no other actions in the main container
		 */
		ActionMenuBase.prototype._updateMoreVisibility = function () {
			ResponsiveFit.prototype._updateMoreVisibility.call(this);

			var mainChildren = this._getMainChildren().length;
			var moreChildren = this._getMoreChildren().length;

			if (moreChildren > 0) {
				var title;

				if (mainChildren > 0) {
					title = i18n.tFallback("ACTIONMENU_MORE_ACTIONS", "More...");
				} else if (mainChildren === 0) {
					title = i18n.tFallback("ACTIONMENU_ACTIONS", "Actions...");
				}

				this._setMoreText(title);
			}
		};


		/**
		 * Get the visibility property for an action
		 * @param {JQueryObj} action - action object
		 * @returns {ACTION_VISIBILITY}
		 */
		ActionMenuBase.prototype._getActionVisibility = function (action) {
			var visibility = 0;
			try {
				visibility = parseInt(action.attr("data-visibility"), 10);
			} catch (e) {
				// ignore parse exception
			}
			if (isNaN(visibility)) {
				visibility = 0;
			}

			return visibility;
		};

		/**
		 * Get the maximum with of the main container
		 * @Override
		 * @returns {Number}
		 */
		ActionMenuBase.prototype._getMaximumWidth = function () {
			var paddingLeft = 0,
				paddingRight = 0;
			try {
				//since we use the border-box model, we need to subtract padding from the width
				var computedStyle = getComputedStyle(this._container[0]);
				paddingLeft = parseFloat(computedStyle.getPropertyValue("padding-left"));
				paddingRight = parseFloat(computedStyle.getPropertyValue("padding-right"));
			} catch (e) {
				// ignore
			}

			return this._container.innerWidth() - paddingLeft - paddingRight;
		};

		/**
		 * Execute an action.
		 * 
		 * @param {jQuery} clickedLink - the link which got clicked
		 * @param {Event} event - the click (or other) event
		 */
		ActionMenuBase.prototype._executeAction = function (clickedLink, event) {
			var actionInfo = new ActionExecuteEvent();
			actionInfo.event = event;
			actionInfo.actionbutton = clickedLink;
			this._fillActionExecuteEvent(actionInfo);
			_.each(this._actionHandlers, function (handlerFunction) {
				handlerFunction.call(clickedLink, actionInfo);
			});
		};

		/**
		 * Which items does this action menu execute on?
		 * 
		 * Fill in the ActionExecuteEvent
		 * 
		 * @param {ActionExecuteEvent} actionExecuteEvent - the event to fill up
		 */
		ActionMenuBase.prototype._fillActionExecuteEvent = function (actionExecuteEvent) {
			actionExecuteEvent.action = actionExecuteEvent.actionbutton.attr("data-action");
			actionExecuteEvent.actionmenu = this;
			actionExecuteEvent.table = this._table;

			// Extend in subclasses!
		};

		/**
		 * Set the Table component for this action menu.
		 * Will be passed to the actionhandler when an action is executed.
		 * 
		 * @param {Table} table - the table component
		 */
		ActionMenuBase.prototype.setTable = function (table) {
			this._table = table;

			this._listenForTableEvents();
		};

		/**
		 * Registers an action handler.
		 * 
		 * @param {Function} actionHandler - a function to execute actions. It will have the following format: function(ActionExecuteEvent):void
		 */
		ActionMenuBase.prototype.registerActionHandler = function (actionHandler) {
			this._actionHandlers.push(actionHandler);
		};

		/**
		 * Get the context of the action menu
		 * @returns {JQueryObj}
		 */
		ActionMenuBase.prototype.getContext = function () {
			return this._context;
		};


		/**
		 * Called by subclasses to collect all the data attributes from wcrinit
		 * Note: static method
		 * 
		 * @param {jQuery} element
		 * @returns {Object}
		 */
		ActionMenuBase._wcrinitCollectSettings = function (element) {
			//
			// You can "hard-code" all the selectors per actionmenu
			//

			// The jQuery selector to get the container with the checkboxes
			var checkboxContainer = $(element.attr("data-container"));
			var checkboxContainerSelector = element.attr("data-container");
			// The jQuery selector to get a single row (or div)
			var itemrowSelector = element.attr("data-itemrow-selector");

			// The jQuery selector to get the checkbox inside that div
			var checkboxSelector = element.attr("data-checkbox-selector");

			//
			// OR you can define them on wcr-table level
			//
			var wcrtable = element.parents(".wcr-table").first();
			if (checkboxContainer.length === 0) {
				checkboxContainer = $(wcrtable.attr("data-container"));
				checkboxContainerSelector = wcrtable.attr("data-container");
			}
			if (!itemrowSelector) {
				itemrowSelector = wcrtable.attr("data-itemrow-selector");
			}
			if (!checkboxSelector) {
				checkboxSelector = wcrtable.attr("data-checkbox-selector");
			}

			//
			// OR you can just use the defaults in most cases...
			//
			if (checkboxContainer.length === 0) {
				checkboxContainer = wcrtable.find(".wcr-table-section").first();
				checkboxContainerSelector = ".wcr-table-section";

			}
			if (!itemrowSelector) {
				itemrowSelector = " > table.ordtable > tbody > tr";
			}
			if (!checkboxSelector) {
				checkboxSelector = "input[type='checkbox'][name^='select']";
			}

			return {
				wcrtable: wcrtable,
				checkboxContainer: checkboxContainer,
				checkboxContainerSelector: checkboxContainerSelector,
				itemrowSelector: itemrowSelector,
				checkboxSelector: checkboxSelector
			};
		};

		/**
		 * Called by subclasses to register the action handler from wcrinit
		 * Note: static method
		 * 
		 * @param {jQuery} element
		 * @param {Object} settings - see _wcrinitCollectSettings
		 * @param {ActionMenuBase} actionmenu - the action menu
		 */
		ActionMenuBase._wcrinitRegisterActionHandler = function (element, settings, actionmenu) {
			// data-actionhandler can point to a require module.
			// this module needs to have a function named "executeAction" which has one argument of type ActionExecuteEvent
			// this method will be called automatically when an action is executed.
			var handler = element.attr("data-actionhandler");
			if (!handler) {
				handler = settings.wcrtable.attr("data-actionhandler");
			}
			if (handler) {
				require([handler], function (handlerModule) {
					if (typeof (handlerModule.executeAction) === "function") {
						actionmenu.registerActionHandler(handlerModule.executeAction);
					}
				});
			}
		};

		return ActionMenuBase;
	});