define(["underscore",
        "jquery",
        "./ActionMenuBase",
        "utils/init",
		"utils/webi18n!ActionMenu"], function (
	_, $,
	ActionMenuBase,
	init,
	i18n) {

	/**
	 * Row Action Menu component for displaying actions in a certain row
	 * 
	 * @class
	 * @constructor
	 * @exports RowActionMenu
	 * @param {JQueryObj} context - div in which the action menu is defined
	 * @param {JQueryObj} [checkbox] - The element on which this action menu works... (most likely the checkbox in the row)
	 * @param {Boolean} [fillspace] - Whether the action tries to fill the available space or just shows the complete menu...
	 * @param {String} [deleteifempty] - jQuery selector of the parent element to delete if the row action menu does not contain any actions...
	 * @param {Boolean} [optimized] - whether the initial DOM transformations have already been done by XSL
	 */
	var RowActionMenu = function (context, checkbox, fillspace, deleteifempty, optimized) {
		// Delete a certain container element (found by deleteifempty jQuery selector) if there are no actions
		if(deleteifempty && $(".action", context).length==0) {
			context.parents(deleteifempty).first().remove();
		}
		
		
		// Initialize ActionMenuBase (call constructor of superclass)
		ActionMenuBase.call(this, context, true, optimized);

		// Option
		this.fillspace = fillspace;

		//
		// For passing the checkbox when executing the action
		//
		this._checkbox = checkbox;
	};

	// RowActionMenu extends ActionMenuBase and inherits all the methods...
	RowActionMenu.prototype = Object.create(ActionMenuBase.prototype);
	RowActionMenu.prototype.constructor = RowActionMenu;
	/** 
	 * @Override only execute resize logic when needed
	 */
	RowActionMenu.prototype._initResizeBehavior = function () {
		if (this.fillspace) {
			ActionMenuBase.prototype._initResizeBehavior.call(this);
		}
	};


	/** 
	 * @Override only execute resize logic when needed
	 */
	RowActionMenu.prototype.resize = function () {
		if (this.fillspace) {
			ActionMenuBase.prototype.resize.call(this);
		}
	};
	/**
	 * @Override
	 */
	RowActionMenu.prototype._getMaximumWidth = function () {
		if (this.fillspace) {
			var container = this._context.parent();
			var availablewidth = container.width() - $(".actioncell-content", container).outerWidth();
			return availablewidth;
		} else {
			// Huge number... (so no limit)
			return 100000;
		}
	};

	/**
	 * Can an element be moved to the more container
	 * @Override
	 * @param {JQueryObj} action
	 * @returns {Boolean}
	 */
	RowActionMenu.prototype._canMoveToMore = function (action) {
		if (this.fillspace) {
			return this._getActionVisibility(action) <= ACTION_VISIBILITIES.DEFAULT;
		} else {
			// "default visibility" actions will not be moved to "more" section
			return this._getActionVisibility(action) < ACTION_VISIBILITIES.DEFAULT;
		}
	};

	/**
	 * Which items does this action menu execute on?
	 * 
	 * Fill in the ActionExecuteEvent
	 * 
	 * @Override
	 * @param {ActionExecuteEvent} actionExecuteEvent - the event to fill up
	 */
	RowActionMenu.prototype._fillActionExecuteEvent = function (actionExecuteEvent) {
		// Call super method
		ActionMenuBase.prototype._fillActionExecuteEvent.call(this, actionExecuteEvent);

		// Put in the items...
		actionExecuteEvent._items = [{
			// Pass more information in the future? Like the id's?
			checkbox: this._checkbox
		}];
	};


	/**
	 * Init the component
	 * @param {JQueryObj,Node} element - element to initialize the action menu on
	 */
	RowActionMenu.wcrinit = function (element) {
		if (!(element instanceof $)) {
			element = $(element);
		}

		var data = ActionMenuBase._wcrinitCollectSettings(element);

		// Find the row this action menu works on
		var row = element.parents(data.checkboxContainerSelector + " " + data.itemrowSelector);

		// So, we have the row, now find the checkbox in this row
		var checkbox = row.find(data.checkboxSelector).first();

		// Whether we try to be responsive
		var fillspace = element.attr("data-fillspace") === "1";
		
		var deleteifempty = element.attr("data-deleteifempty") || "";
		
		var optimized = element.attr("data-optimized") === "1";

		// Initialize
		var rowActionMenu = new RowActionMenu(element, checkbox, fillspace, deleteifempty, optimized);

		// Find the parent Table component (if any)
		if (data.wcrtable.length > 0) {
			init.whenInitialized(data.wcrtable, function () {
				var tableComponent = data.wcrtable.get(0).Table;
				rowActionMenu.setTable(tableComponent);
			});
		}

		// Load & attach handler
		ActionMenuBase._wcrinitRegisterActionHandler(element, data, rowActionMenu);
	};

	return RowActionMenu;
});