define(["./ActionMenuBase", "./ActionMenu"], function (ActionMenuBase, ActionMenu) {
	"use strict";

	/**
	 * Extension of the ActionMenu
	 * Shows actions in a vertical action group
	 * It is not responsive, but has a hard limit of shown actions 
	 * @class
	 * @constructor
	 * @exports VerticalActionMenu
	 * @augments ActionMenu
	 * @param {JQueryObj} context - div in which the action menu is defined
	 * @param {Number} maxItems - maximum visible items
	 * @param {JQueryObj} [checkboxContainer] - container for which action menu is defined (this is used for data-require-selection)
	 * @param {String}	[checkboxSelector] - Selector string to select checkboxes in the checkboxContainer (this is used for data-require-selection)
	 */
	var VerticalActionMenu = function (context, maxItems, checkboxContainer, checkboxSelector) {
		this._maxItems = maxItems;

		context.addClass("vertical");
		ActionMenu.call(this, context, checkboxContainer, checkboxSelector);
	};

	VerticalActionMenu.prototype = Object.create(ActionMenu.prototype);
	VerticalActionMenu.prototype.constructor = VerticalActionMenu;

	/**
	 * @Override
	 */
	VerticalActionMenu.prototype._initResizeBehavior = function () {
		// no need for resize event handling	
	};

	/**
	 * Limit the number of visible items to  this._maxItems
	 * this stops the responsive script to be executed
	 * @override
	 */
	VerticalActionMenu.prototype.resize = function () {
		// limit visible items to maxItems	
		if (this._maxItems > 0) {
			var availableSlots = this._maxItems;
			var mainChildren = this._getMainChildren();

			var startMoveToMoreIndex = Math.min(mainChildren.length, availableSlots);

			if (this._getMoreChildren().length > 0 || (mainChildren.length - startMoveToMoreIndex) > 0) {
				startMoveToMoreIndex--;
			}


			// move excess items to more
			var i = 0;
			for (i = startMoveToMoreIndex; i < mainChildren.length; i++) {
				
				var child = $(mainChildren[i]);
				if (this._canMoveToMore(child)) {
					this._prependToMore(child);
					mainChildren[i] = null;
				} else {
					// we can't move the current item, see if some of the previous items can be moved
					var j = i-1;  
					
					while (j > 0 && (mainChildren[j] === null || !this._canMoveToMore($(mainChildren[j])))){
						j--;
					}
					if (j >= 0 && mainChildren[j]){
						var previousChildToMove = $(mainChildren[j]);
						if (this._canMoveToMore(previousChildToMove)){
							this._prependToMore(previousChildToMove);
							mainChildren[j] = null;
						}
					}
				}
			}

			var shownSlots = this._getMainChildren().length;
			if (this._getMoreChildren().length > 0) {
				shownSlots++;
			}

			availableSlots -= shownSlots;
			var moreChildren = this._getMoreChildren();
			while (availableSlots >= 0 && i < moreChildren.length) {
				if (availableSlots > 0 || this._getMoreChildren().length === 1) {
					var child = $(moreChildren[i]);
					if (this._canMoveToMain(child)) {
						this._appendToMain(child);
					}
				}
				i++;
			}

			this._updateMoreVisibility();
		}
	};

	/**
	 * Switch to compact more button when there is only icons
	 * @override 
	 */
	VerticalActionMenu.prototype._updateMoreVisibility = function () {
		ActionMenu.prototype._updateMoreVisibility.call(this);

		if (this._getMainChildren().length > 0 && _.every(this._getMainChildren(), function (element) {
				return $(element).attr("data-icon-only");
			})) {
			this._updateMoreDisplayMode(true);
		}
	};

	/**
	 * Init the component
	 * @param {JQueryObj,Node} element - element to initialize the action menu on
	 */
	VerticalActionMenu.wcrinit = function (element) {
		if (!(element instanceof $)) {
			element = $(element);
		}

		var data = ActionMenuBase._wcrinitCollectSettings(element);

		var maxItems;
		try {
			var maxItems = parseInt(element.attr("data-max-items"), 10);
		} catch (e) {
			maxItems = 0;
		}
		if (isNaN(maxItems)){
			maxItems = 0;
		}
		// Initialize
		var actionMenu = new VerticalActionMenu(element, maxItems, data.checkboxContainer, data.checkboxSelector);

		// Find the parent Table component (if any)
		if (data.wcrtable.length > 0) {
			init.whenInitialized(data.wcrtable, function () {
				var tableComponent = data.wcrtable.get(0).Table;
				actionMenu.setTable(tableComponent);
			});
		}

		// Load & attach handler
		ActionMenuBase._wcrinitRegisterActionHandler(element, data, actionMenu);
	};



	return VerticalActionMenu;

});