/**
 * A re-usable component to limit very long content.
 */
define(["jquery"], function($){
	
	
	/**
	 * Make a container of text with a certain limit on the height
	 * 
	 * @param {Element} div - the div
	 * @param {Number} maxheightmini - the collapsed height in pixels! (like 100)
	 * @param {Element} optWrapDiv - a div without styling which is placed around the "div" containing the content. 
	 * 						When passed, this component will use this div instead of wrapping one around the content. 
	 * 						Why? Because "wrap" is a slow operation on IE (=> 1ms each) and by using this it can be avoided.
	 */
	var OverflowContentContainer = function(div, maxheightmini, optWrapDiv){
		this.maxheightmini = maxheightmini;
		this.markerheight = 20;
		
		// Make the elements
		if (div instanceof $) {
			this.contentdiv = div;
		} else {
			this.contentdiv = $(div);
		}
		this.contentdiv.addClass("overflowcontentelem");
		if(optWrapDiv && optWrapDiv instanceof $) {
			this.wrapperdiv = optWrapDiv;
		} else if(optWrapDiv) {
			this.wrapperdiv = $(optWrapDiv);
		} else {
			this.contentdiv.wrap($("<div>"));
			this.wrapperdiv = this.contentdiv.parent();
		}
		this.wrapperdiv.addClass("overflowcontentcontainer");
		this.marker = $("<div>", {"class": "marker"});
		this.marker.css({"height":this.markerheight+"px"});
		this.wrapperdiv.append(this.marker);

		this.callbacks = [];
		
		this.collapse();
		
		var self = this;
		this.contentdiv.click(function(event){
			if(!$(event.target).is("a, .btn, .action, .dropdown, .no-toggle-overflow") && !$(event.target).parentsUntil(this).is("a, .btn, .action, .dropdown, .no-toggle-overflow")){
				// bubble the event if the content is not more than view area of the overflow container
				// We still allow the "marker" to overlap 10px with the content. 
				// Since it is a fade-out you don't notice these last 10 px anyway. (but if you do you could still click those to expand -> see other event handler)
				if(self.contentdiv.height() > self.maxheightmini-self.markerheight/2) {
					// otherwise, trigger the expand/collapse
					self.toggle("CLICK");
					event.preventDefault();
					event.stopPropagation();
				}
			}
		});
		this.marker.click(function(event){
			self.toggle("CLICK");
			event.preventDefault();
			event.stopPropagation();
		});
	};
	
	/**
	 * Register a listener.
	 * 
	 * The listener should have two parameters:
	 *  - expanded: The parameter will be true if this component went from collapsed => expanded (other way: false)
	 *  - cause: will pass a string which tells you the cause of the change. default: "CLICK"
	 *    (You pass the cause when calling the collapse/expand/toggle function)
	 * 
	 * @param {Function} callback
	 */
	OverflowContentContainer.prototype.onChanged = function(callback) {
		this.callbacks.push(callback);
	};
	
	/**
	 * (private) Fire a change
	 * 
	 * @param becameexpanded
	 * @param cause
	 */
	OverflowContentContainer.prototype._fireChange = function(becameexpanded, cause) {
		$.each(this.callbacks, function(i, callback) {
			callback(becameexpanded, cause);
		});
	};
	
	/**
	 * Collapse the Overflow container
	 * @param {String} cause - just a string... The cause of the collapse. Passed to the change listeners.
	 */
	OverflowContentContainer.prototype.collapse = function(cause) {
		if(!this.collapsed) {
			this.collapsed = true;
			this.wrapperdiv.addClass("collapsed").removeClass("expanded");
			this.wrapperdiv.css({"max-height":this.maxheightmini+"px"});
			this.marker.css({"top":(this.maxheightmini-this.markerheight)+"px"});
			this._fireChange(false, cause);
		}
	};
	
	/**
	 * Expand the Overflow container
	 * @param {String} cause - just a string... The cause of the expand. Passed to the change listeners.
	 */
	OverflowContentContainer.prototype.expand = function(cause) {
		if(this.collapsed) {
			this.collapsed = false;
			this.wrapperdiv.addClass("expanded").removeClass("collapsed");
			this.wrapperdiv.css({"max-height":"500px"});
			this.marker.css({"top":(500-this.markerheight)+"px"});
			this._fireChange(true, cause);
		}
	};
	
	/**
	 * Toggle Collapse/Expand
	 * @param {String} cause - just a string... The cause of the toggle. Passed to the change listeners.
	 */
	OverflowContentContainer.prototype.toggle = function(cause) {
		if(this.collapsed){
			this.expand(cause);
		} else {
			this.collapse(cause);
		}
	};
	
	/**
	 * Is the container expanded?
	 */
	OverflowContentContainer.prototype.isExpanded = function() {
		return !this.collapsed;
	};
	
	/**
	 * Return the wrapping jQuery element.
	 * 
	 * @returns {jQuery} the wrapping element.
	 */
	OverflowContentContainer.prototype.getJQElement = function() {
		return this.wrapperdiv;
	};
	
	OverflowContentContainer.wcrinit = function(div) {
    var maxheight = $(div).attr("data-height");
    new OverflowContentContainer(div, maxheight);
	};
	
	return OverflowContentContainer;
});