/**
 * Document Selector UI module
 * - takes care of the user interface for Document selection
 * 
 * @author JAMI
 * @since 20.2.2015
 */
define(["jquery",
        "utils/webi18n!DocumentSelector",
        "component/sdrop/SDrop",
        "./DocumentSelector-style"
        ], function ($, webI18n, SDrop) {
	
	"use strict";
	
	/*
	 * MODULE CONSTRUCTOR
	 */
	var DocumentSelectorView = function (projectId, folderId, documentId, documentVersionID, documentRevisionCode) {		
		// The baseUrl of the service.
        this._jspBaseUrl = window.webcenterConfig.jspBaseUrl;
        
        //initialize clean values for the DocumentSelectorView properties 
        //(NOTE: this is needed as after closing the dialog the properties remain > need to be flushed upon next dialog open)
        this._clearBasicProperties();
		
		// private variables
		var self = this;
		this._onDocumentSelectedCallbacks = [];
		this._onCancelClickedCallbacks = [];
		
		this._selectedProjectID = projectId;
		this._selectedFolderID = folderId;
		this._selectedDocumentID = documentId;
		this._selectedDocumentVersionIDs = []; //NOTE: as we could be selecting a whole Revision... 
		//... -> could end up multiple values (meaning that ALL Document Versions of the same Revision should be selected)
		
		//utility properties (need so to be able to reconstruct display state for  an existing selection)
		this._targetDocumentVersionID = documentVersionID; //helps us to single out A Document Version so to create an appropriate "Document Details" URL
		this._selectedDocumentRevision = documentRevisionCode; //in case we selected a whole Revision; only if this contains a valid value will we consider a whole Revision was selected
		this._selectedDocName = "";
		this._selectedDocVersionRevLabel = "";
			
		//selector data containers
		this._availableFolders = [];
		this._availableDocs = [];
		this._availableDocVersions = [];
		this._availableDocRevisions = [];
		
		//HTML elements
		this._wrapper = $("<div class='DocumentSelectorView-wrapper'></div>");
		
		//-------------------------------------------------------
		//Project Selector
		//-------------------------------------------------------
		var projectSelectorRow = $("<div>").addClass("form-group selector_row");
		//add "Project" label
		var projSelectorLabel = $("<label class='docname-label' style='clear: right;'/>").text(webI18n.t("PROJECT"));
		projectSelectorRow.append(projSelectorLabel);
		//Project Dropdown selector (as an SDrop)
		var _projSelectorEntryContainer = $("<div>").addClass("form-entry");
		this._projSelectorField = $("<select class='docname-input form-control' name='DocSelector_ProjID' id='DocSelector_ProjID'/>");
		_projSelectorEntryContainer.append(this._projSelectorField);
		projectSelectorRow.append(_projSelectorEntryContainer);
		//add the whole row to the wrapper
		this._wrapper.append(projectSelectorRow);
		
		//initialize the sDrop
		var initialSDropVal = this._selectedProjectID || null;
		new SDrop(this._projSelectorField, "", 
				   new SDropDataProvider({'dataSource':this._jspBaseUrl+"GetNameIDList.jsp?list=VisibleProjects"}), {       
            'onChangedByUser': this._projectSelectorChanged.bind(self),								   
            'autoload': true,
            'initialValue': initialSDropVal,
            'zIndex' : 1000,
            'autoWidth' : true,
            'minWidth' : 300
		});
		
		var projectValInitialized = (this._selectedProjectID && this._selectedProjectID.length > 0);
		
		//-------------------------------------------------------
		//Folder Selector
		//-------------------------------------------------------
		var folderSelectorRow = $("<div>").addClass("form-group selector_row");
		//add "Folder" label
		var folderSelectorLabel = $("<label class='docname-label'/>").text(webI18n.t("FOLDER"));
		folderSelectorRow.append(folderSelectorLabel);
		//Folder Dropdown selector
		var _folderSelectorEntryContainer = $("<div>").addClass("form-entry");
		this._folderSelectorEditField = $("<select autofocus class='docname-input form-control' name='DocSelector_FolderID' id='DocSelector_FolderID' />");
		this._folderSelectorEditField.change(this._folderSelectorChanged.bind(this));
		_folderSelectorEntryContainer.append(this._folderSelectorEditField);
		folderSelectorRow.append(_folderSelectorEntryContainer);
		//add the whole row to the wrapper
		this._wrapper.append(folderSelectorRow);
		
		//-------------------------------------------------------
		//Document Selector
		//-------------------------------------------------------
		var documentSelectorRow = $("<div>").addClass("form-group selector_row");
		//add "Document" label
		var documentSelectorLabel = $("<label class='docname-label'/>").text(webI18n.t("DOCUMENT"));
		documentSelectorRow.append(documentSelectorLabel);
		//Document Dropdown selector
		var _docSelectorEntryContainer = $("<div>").addClass("form-entry");
		this._documentSelectorEditField = $("<select class='docname-input form-control' name='DocSelector_DocID' id='DocSelector_DocID' value='" + this._selectedDocumentID + "'>");
		this._documentSelectorEditField.change(this._documentSelectorChanged.bind(this));
		if (!this._selectedDocumentID || this._selectedDocumentID == null) {
			//nothing selected yet > disable the control
			this._documentSelectorEditField.prop('disabled', 'disabled');
		}
		_docSelectorEntryContainer.append(this._documentSelectorEditField);
		documentSelectorRow.append(_docSelectorEntryContainer);
		//add the whole row to the wrapper
		this._wrapper.append(documentSelectorRow);
		
		//-------------------------------------------------------
		//Document Version/Revision Selector
		//-------------------------------------------------------
		var docVerSelectorRow = $("<div>").addClass("form-group selector_row");
		//add "Document Version/Revision" label
		var docVerSelectorLabel = $("<label class='docname-label'/>").text(webI18n.t("VERSION_REVISION"));
		docVerSelectorRow.append(docVerSelectorLabel);
		//Document Version/Revision Selector
		var _docVerSelectorEntryContainer = $("<div>").addClass("form-entry");
		this._docVerSelectorEditField = $("<select class='docname-input form-control' name='DocSelector_DocVerIDRev' id='DocSelector_DocVerIDRev' value='"+ this._selectedDocumentVersionID + "'>");
		this._docVerSelectorEditField.change(this._updateDocumentVersionRef.bind(this));
		if (!this._selectedDocumentVersionID || this._selectedDocumentVersionID == null) {
			//nothing selected yet > disable the control
			this._docVerSelectorEditField.prop('disabled', 'disabled');
		}
		_docVerSelectorEntryContainer.append(this._docVerSelectorEditField);
		docVerSelectorRow.append(_docVerSelectorEntryContainer);
		//add the whole row to the wrapper
		this._wrapper.append(docVerSelectorRow);
		
		//-------------------------------------------------------
		//Buttons Row
		//-------------------------------------------------------
		var buttonsRow = $("<div>").addClass("form-group selector_row mt10");
		
		//Select Button
		this._selectButton = $("<button class='btn btn-primary' style='display: block-inline;'/>").text(webI18n.t("SELECT"));
		
		//register event handler for the submit button click 
		this._selectButton.on('click', this._submitDocumentSelection.bind(this));
		buttonsRow.append(this._selectButton);
		
		//Cancel Button
		this._cancelButton = $("<button class='btn btn-default add-spacing-left' style='display: block-inline;'/>").text(webI18n.t("CANCEL"));
		
		this._cancelButton.on('click', this._cancelButtonClicked.bind(this));
		buttonsRow.append(this._cancelButton);
		
		this._wrapper.append(buttonsRow);
		
		//-------------------------------------------------------
		//UPDATING CONTROL STATES
		//-------------------------------------------------------
		//update the Folder Selector state
		if (projectValInitialized) {
			//initialize the Folder Selector > need to re-set the initial state
			//load the Folders for the selected Project
			this._loadVisibleProjectFolders();
			
			//fill in the Folder selector with the loaded set of visible Folders
			this._redrawVisibleProjectFoldersSelector(true);
			
			//NOTE: updating Folder selector state will ripple down to both the Document and DocVersion/Revision Selectors as well
		} else {
			//no Project Selected yet > disable the Folder Selector
			this._folderSelectorEditField.prop('disabled', 'disabled');
		}
	};
	
	/*
	 * MODULE PROTOTYPE METHODS 
	 */
	
	/*
	 * CHANGE EVENT HANDLERS 
	 */
	
	/**
	 * Triggered when the user clicks on 'Select' button
	 */
	DocumentSelectorView.prototype._submitDocumentSelection = function () {	
		//make sure we work with the latest values
		this._updateProjectRef();
		this._updateFolderRef();
		this._updateDocumentRef();
		this._updateDocumentVersionRef();
		
		//validate current output parameter values
		if (!this._validateOutputParams()) {
			alert(webI18n.t("INVALID_OUTPUT_PARMS"));
		} else {
			this._selectDocument(this._selectedProjectID, this._selectedFolderID, 
					this._selectedDocumentID, this._selectedDocumentVersionIDs, this._selectedDocumentRevision, 
					this._targetDocumentVersionID, this._selectedDocName, this._selectedDocVersionRevLabel);	
		}
	};
	
	
	/**
	 * Event callback method triggered when the "Project Selector" dropdown is changed
	 */
	DocumentSelectorView.prototype._projectSelectorChanged = function () {
		var selProjID = this._projSelectorField.find("option:selected").val();
		if (selProjID && selProjID !== null && selProjID.length > 0) {
			this._updateProjectRef();
			
			//load the Folders for the selected Project
			this._loadVisibleProjectFolders();
			
			//fill in the Folder selector with the loaded set of visible Folders
			this._redrawVisibleProjectFoldersSelector(false);
		}
	};
	
	/**
	 * Update "Selected Project ID" property (used as an output parameter)
	 */
	DocumentSelectorView.prototype._updateProjectRef = function () {
		var selProjID = this._projSelectorField.find("option:selected").val();
		if (selProjID && selProjID !== null && selProjID.length > 0) { 
			this._selectedProjectID = selProjID;
		}
	};
	
	/**
	 * Event callback method triggered when the "Folder Selector" dropdown is changed
	 */
	DocumentSelectorView.prototype._folderSelectorChanged = function () {
		var selFolderID = this._folderSelectorEditField.find("option:selected").val();

		if (selFolderID && selFolderID !== null && selFolderID.length > 0) {
			this._updateFolderRef();
				
			//load the visible Documents for the selected Project Folder
			this._loadVisibleDocuments();
			
			//fill in the Document selector with the loaded set of visible Documents
			this._redrawVisibleDocumentsSelector();
		}
		
	};
	
	/**
	 * Update "Selected Folder ID" property (used as an output parameter)
	 */
	DocumentSelectorView.prototype._updateFolderRef = function () {
		var selFolderID = this._folderSelectorEditField.find("option:selected").val();
		if (selFolderID && selFolderID !== null && selFolderID.length > 0) { 
			if (selFolderID == "NoFolderFilter") {
				this._selectedFolderID = "";
			} else {
				this._selectedFolderID = selFolderID;	
			}
		}
	};
	
	/**
	 * Event callback method triggered when the "Document Selector" dropdown is changed
	 */
	DocumentSelectorView.prototype._documentSelectorChanged = function () {
		var selDocumentID = this._documentSelectorEditField.find("option:selected").val();

		if (selDocumentID && selDocumentID !== null && selDocumentID.length > 0) {
			this._updateDocumentRef();
			
			//load Document Version/Revision data for the selected Document
			this._loadDocumentVersionsAndRevisions();
			
			//fill in the Document Version/Revision selector
			this._redrawDocumentVersionsAndRevisionsSelector();
		}
		
	};
	
	/**
	 * Update "Selected Document ID + Name" properties (used as an output parameters)
	 */
	DocumentSelectorView.prototype._updateDocumentRef = function () {
		var selDocumentID = this._documentSelectorEditField.find("option:selected").val();
		var selDocumentName = this._documentSelectorEditField.find("option:selected").text();

		if (selDocumentID && selDocumentID !== null && selDocumentID.length > 0) {
			if (selDocumentID == "N/A") {
				this._selectedDocumentID = "";
			} else {
				this._selectedDocumentID = selDocumentID;
				this._selectedDocName = selDocumentName;
			}
		}
	};
	
	/**
	 * Update "Selected Document Version ID/Revision Code" properties (used as an output parameters)
	 */
	DocumentSelectorView.prototype._updateDocumentVersionRef = function () {
		var selDocumentVersionID = this._docVerSelectorEditField.find("option:selected").val();

		if (selDocumentVersionID && selDocumentVersionID !== null && selDocumentVersionID.length > 0) {
			//clear the target array of DocVersionIDs > fill it up again based on the current value
			this._selectedDocumentVersionIDs = [];
			if (selDocumentVersionID == "AllVersions") {//Any Document Version
				var highestDocVersionID = ""; //NOTE: it is assumed that the DocVersions are stored in ascending order (highest at the end)
				for (var i=0; i < this._availableDocVersions.length; i++) {
					//NOTE: "AllVersions" setting means "No Document Version" filter > keep the array empty and just rely on the Document ID
					highestDocVersionID = this._availableDocVersions[i].id;
				}
				
				//clear the current Selected DocVersionID and SelectedRevisionCode
				this._selectedDocumentRevision = "";
				
				//update the utility DocVersionID (needed to display the DocDetails link which has to point to a specific Doc Version)
				this._targetDocumentVersionID = highestDocVersionID;
			} else if (selDocumentVersionID.indexOf("Rev_") == 0 ) {// Specific Revision (all the corresponding Document Versions)
				//strip the Revision prefix and only keep the Revision letter itself
				this._selectedDocumentRevision = selDocumentVersionID.substr(4);					

				//now add the DocVersionIDs for the selected Revision to the final list (and keep the highest ID reference)
				var highestDocVersionID = ""; //NOTE: it is assumed that the DocVersions are stored in ascending order (highest at the end)
				for (var i=0; i < this._availableDocVersions.length; i++) {
					if (this._availableDocVersions[i].revision == this._selectedDocumentRevision) {
						this._selectedDocumentVersionIDs.push(this._availableDocVersions[i].id);
						highestDocVersionID = this._availableDocVersions[i].id;	
					}
				}
				
				//update the utility properties
				this._targetDocumentVersionID = highestDocVersionID;
				
				//set the DocVerRev label value
				if (this._selectedDocumentRevision == "0") { // use localized "No Revision" label
					this._selectedDocVersionRevLabel = "("+webI18n.t("NO_REVISION")+")";	
				} else { //just use the Revision code
					this._selectedDocVersionRevLabel = "("+this._selectedDocumentRevision+")";	
				}
				
			} else { // Specific Document Version
				this._selectedDocumentVersionIDs.push(selDocumentVersionID);
				this._targetDocumentVersionID = selDocumentVersionID;
				
				//fetch the corresponding Version+Revision label
				for (var i=0; i < this._availableDocVersions.length; i++) {
					if (this._availableDocVersions[i].id == selDocumentVersionID) {
						//found the selected Doc Version > update the utility properties
						this._selectedDocVersionRevLabel = this._availableDocVersions[i].version;
						if (this._availableDocVersions[i].revision !== "0") {
							this._selectedDocVersionRevLabel = this._selectedDocVersionRevLabel+" ("+this._availableDocVersions[i].revision+")";
						}
					}
				}	
			}
		}	
	};
	
	/*
	 * SELECTOR ELEMENTS REDRAW FUNCTIONS
	 */
	
	/**
	 * Update the Folder Selector input with the available visible Folders for the currently select Project
	 * 
	 * @param initialStateProcessing - indicates whether this is the initialization phase of the popup (make sure not to clear the current "Folder ID" value)
	 */
	DocumentSelectorView.prototype._redrawVisibleProjectFoldersSelector = function (initialStateProcessing) {
		if (this._availableFolders.length == 0) {
			//load the Folders for the selected Project
			this._loadVisibleProjectFolders();
		}
		
		//clear out the Folder selector first
		this._folderSelectorEditField.find('option').remove();
		
		//clear the currently selected folderID (if not handling the initial state)
		if (!initialStateProcessing) {
			this._selectedFolderID = "";	
		}
		
		var projectNotVisible = false;
		
		//fill up the selector again
		if (this._availableFolders.length == 0) { // NO FOLDERS
			//this should not happen > at least the Root folder should always be visible, IF the selected Project is visible to the current User
			var invalidOptionElem = $("<option/>").text(webI18n.t("NO_FOLDER_VISIBLE"));
			invalidOptionElem.val("N/A");
			this._folderSelectorEditField.append(invalidOptionElem);
			
			//mark the invalid state
			projectNotVisible = true;
			
			//update the property
			this._updateFolderRef();
			
			//disable Document+DocVersionRevision selector
			this._folderSelectorEditField.prop('disabled', 'disabled');
			this._docVerSelectorEditField.prop('disabled', 'disabled');
			this._documentSelectorEditField.prop('disabled', 'disabled');
		} else if (this._availableFolders.length == 1) { // ONE FOLDER
			//we only have a single visible folder > just have this one option in there
			var onlyOptionFolderID = this._availableFolders[0].id;
			var onlyOptionFolderName = this._availableFolders[0].name;
			var onlyOptionElem = $("<option/>").text(onlyOptionFolderName).val(onlyOptionFolderID);
			this._folderSelectorEditField.append(onlyOptionElem);
			
			this._selectedFolderID = onlyOptionFolderID;
		} else { //MULTIPLE FOLDERS
			//start by having a "Whole Project" dummy option (aka. "do not apply any Folder filter on the Project Documents")
			var wholeProjectOptionElem = $("<option/>").text(webI18n.t("ANY_FOLDER"));
			wholeProjectOptionElem.val("NoFolderFilter");
			
			//make this option selected if no FolderID value is selected at the moment
			if (!this._selectedFolderID || this._selectedFolderID == null || this._selectedFolderID.length == 0) {
				wholeProjectOptionElem.attr('selected', 'selected');
			}
			this._folderSelectorEditField.append(wholeProjectOptionElem);
			
			//now add all the available visible Folders as individual select options
			for (var i=0; i < this._availableFolders.length; i++) {
				var curFolderID = this._availableFolders[i].id;
				var curFolderName = this._availableFolders[i].name;
				
				//create a new option element
				var curOptionElem = $("<option/>");
				curOptionElem.val(curFolderID);
				
				//initialize the option label
				if (curFolderName == "ROOT") {
					curFolderName = webI18n.t("ROOT_FOLDER");
				}
				curOptionElem.text(curFolderName);

				//make the current option selected if this is a case of re-opening an existing Selection
				if (this._selectedFolderID && this._selectedFolderID !== null && this._selectedFolderID == curFolderID) {
					curOptionElem.attr('selected', 'selected');
				}
				
				this._folderSelectorEditField.append(curOptionElem);
			}
		}
		
		//now pre-load visible Documents
		if (!projectNotVisible) {
			//enable the Folder Selector
			this._folderSelectorEditField.prop('disabled', false); 
			
			//load the visible Documents for the selected Project Folder
			this._loadVisibleDocuments();
			
			//fill in the Document selector with the loaded set of visible Documents
			this._redrawVisibleDocumentsSelector();
		} 
	};
	
	/**
	 * Update the Document Selector input with the available visible Document for the currently select Project (and Folder, if selected)
	 */
	DocumentSelectorView.prototype._redrawVisibleDocumentsSelector = function () {
		if (this._availableDocs.length == 0) {
			//load the visible Documents for the selected Project Folder
			this._loadVisibleDocuments();	
		}
		
		//clear out the Document selector first
		this._documentSelectorEditField.find('option').remove();
			
		var noDocumentAvailable = false;
		
		//fill up the Document Selector
		if (this._availableDocs.length == 0) {//NO DOCUMENTS
			//uh oh...no Documents (from the currently selected Project/Folder) are available to the current User
			var invalidOptionElem = $("<option/>").text(webI18n.t("NO_DOCUMENT_AVAILABLE"));
			invalidOptionElem.val("N/A");
			this._documentSelectorEditField.append(invalidOptionElem);
			this._documentSelectorEditField.prop('disabled', 'disabled');
			
			//mark the invalid state
			noDocumentAvailable = true;
			
			//disable DocVersionRevision selector
			this._docVerSelectorEditField.prop('disabled', 'disabled');
			//clear out the selected document property
			this._selectedDocumentID = null;
		} else if (this._availableDocs.length == 1) {//SINGLE DOCUMENT
			//we only have a single visible Document > just have this one option in there
			var onlyOptionDocID = this._availableDocs[0].id;
			var onlyOptionDocName = this._availableDocs[0].name;
			
			var onlyOptionElem = $("<option/>");
			onlyOptionElem.val(onlyOptionDocID).text(onlyOptionDocName);
			this._documentSelectorEditField.append(onlyOptionElem);
			
			//mark the fact that we have a specific "active Document" context at the moment
			this._selectedDocumentID = onlyOptionDocID;
		} else {//MULTIPLE DOCUMENTS
			//now add all the available visible Folders as individual select options
			for (var i=0; i < this._availableDocs.length; i++) {
				var curDocID = this._availableDocs[i].id;
				var curDocName = this._availableDocs[i].name;
				
				var curOptionElem = $("<option/>");
				curOptionElem.text(curDocName).val(curDocID);
				//make the current option selected if this is a case of re-opening an existing Selection
				if (this._selectedDocumentID && this._selectedDocumentID !== null && this._selectedDocumentID == curDocID) {
					curOptionElem.attr('selected', 'selected');
				}
				
				this._documentSelectorEditField.append(curOptionElem);
			}
			
			//mark the currently selected Doc ID
			var curSelectedDocID = this._documentSelectorEditField.find("option:selected").val();
			this._selectedDocumentID = curSelectedDocID;
		}
		
		if (!noDocumentAvailable) {
			//enable the Document and DocVersion Selector
			this._documentSelectorEditField.prop('disabled', false); 
			this._docVerSelectorEditField.prop('disabled', false); 		
		} 
		
		//load the Document Versions and Revisions
		this._loadDocumentVersionsAndRevisions();
		
		//fill in the Document Version/Revision selector
		this._redrawDocumentVersionsAndRevisionsSelector();
	};
	
	/**
	 * Update the Document Version/Revision Selector input with the available Document Versions (and Revisions if applicable) for the currently select Document
	 * NOTE: Project/Folder selection is no longer significant - Document Versions for the selected Document will be presented regardless of Project/Folder context (as a DocVersion can belong to multiples) 
	 */
	DocumentSelectorView.prototype._redrawDocumentVersionsAndRevisionsSelector = function () {
		
		if (this._availableDocVersions.length == 0) {
			//load the visible Document Versions/Revisions for the selected Project Folder
			this._loadDocumentVersionsAndRevisions();			
		}

		//clear out the DocVersion/Revision selector first
		this._docVerSelectorEditField.find('option').remove();
		
			
		//fill up the Document Selector
		if (this._availableDocVersions.length == 0) {//NO DOCUMENT VERSIONS/REVISIONS
			//uh oh...no Document Versions for the selected Document are available to the current User
			//this should not occur
			var invalidOptionElem = $("<option/>").text(webI18n.t("NO_DOCUMENT_VERSION_AVAILABLE"));
			invalidOptionElem.val("N/A");
			this._docVerSelectorEditField.append(invalidOptionElem);
			
			//mark the invalid state
			this._docVerSelectorEditField.prop('disabled', 'disabled');
		} else if (this._availableDocVersions.length == 1) {//SINGLE DOCUMENT VERSION/REVISION
			//we only have a single Document Version available > just have this one option in there
			var onlyOptionDocVerID = this._availableDocVersions[0].id;
			var onlyOptionDocVerNr = this._availableDocVersions[0].version;
			var onlyOptionDocRevisionCode = this._availableDocVersions[0].revision;
			
			var onlyOptionElem = $("<option/>");
			onlyOptionElem.val(onlyOptionDocVerID);
			
			//only show revisions code if it does contain some
			var onlyOptionDocVersionLangKey = "VERSION_CODE";
			if (onlyOptionDocRevisionCode && onlyOptionDocRevisionCode !== null && onlyOptionDocRevisionCode !== "0") {
				onlyOptionDocVersionLangKey = "VERSION_REVISION_CODE";
			}
			onlyOptionElem.text(webI18n.t(onlyOptionDocVersionLangKey, onlyOptionDocVerNr, onlyOptionDocRevisionCode));
			
			this._docVerSelectorEditField.append(onlyOptionElem);
			
			//mark the fact that we have a specific "active Document" context at the moment
			this._selectedDocumentVersionIDs.push(onlyOptionDocVerID);
			this._targetDocumentVersionID = onlyOptionDocVerID;
		} else {// MULTIPLE DOC VERSIONS/REVISIONS
			var docVersionValueSet = false;
			if (this._targetDocumentVersionID && this._targetDocumentVersionID != null && this._targetDocumentVersionID.length > 0) {
				docVersionValueSet = true;
			}	
			var docRevisionValueSet = false;
			if (this._selectedDocumentRevision && this._selectedDocumentRevision != null && this._selectedDocumentRevision.length > 0) {
				docRevisionValueSet = true;
			}
			
			//start by having a "All Versions" dummy option (aka. "do not apply any Doc. Version filter")
			var allVersionsOptionElem = $("<option/>").text(webI18n.t("ANY_DOC_VERSION"));
			allVersionsOptionElem.val('AllVersions');
			//make this option selected if - no Revision and no Doc Version ID are set
			if (!docVersionValueSet && !docRevisionValueSet) {
				allVersionsOptionElem.attr('selected', 'selected');
			}		
			this._docVerSelectorEditField.append(allVersionsOptionElem);
			
					
			//"All Versions of a Revision" options first (only if there are at least two revisions available)
			if (this._availableDocRevisions.length > 1) {
				for (var i=0; i < this._availableDocRevisions.length; i++) {
					var curRevCode = this._availableDocRevisions[i];
					
					var revOptionLabelTag = "";
					if (curRevCode == "0") { //NO REVISION
						revOptionLabelTag = "NO_REVISION"; 
					} else {
						revOptionLabelTag = "REVISION_CODE";
					}
					
					var curOptionElem = $("<option/>").text(webI18n.t(revOptionLabelTag, curRevCode));
					curOptionElem.val("Rev_"+curRevCode);
					//make this option selected if - Revision is set but Doc Version ID is not set
					if (!docVersionValueSet && docRevisionValueSet && curRevCode == this._selectedDocumentRevision) {
						curOptionElem.attr('selected', 'selected');
					}
					
					this._docVerSelectorEditField.append(curOptionElem);
				}
			}
			
			//individual Document Version options
			for (var i=0; i < this._availableDocVersions.length; i++) {
				var curDocVersionID = this._availableDocVersions[i].id;
				var curDocVersionNr = this._availableDocVersions[i].version;
				var curDocRevisionCode = this._availableDocVersions[i].revision;
				
				var curOptionElem = $("<option/>");
				curOptionElem.val(curDocVersionID);
				
				var optionLangKey = "VERSION_REVISION_CODE";
				if (curDocRevisionCode === "0") {
					optionLangKey = "VERSION_CODE";
				}
				curOptionElem.text(webI18n.t(optionLangKey, curDocVersionNr, curDocRevisionCode));
				
				//make the current option selected if this is a case of re-opening an existing Selection
				if ( !docRevisionValueSet && docVersionValueSet && curDocVersionID == this._targetDocumentVersionID) {
					curOptionElem.attr('selected', 'selected');
				}
				
				this._docVerSelectorEditField.append(curOptionElem);
			}	
		}
	};
	
	/*
	 * DATA LOADER FUNCTIONS
	 */
	
	/**
	 * Load Folders for the selected Project that are visible to the current User
	 */
	DocumentSelectorView.prototype._loadVisibleProjectFolders = function () {
		//clear out the current list of 'available Folders'
		this._availableFolders = [];
		
		if (this._selectedProjectID == null || this._selectedProjectID.length == 0) {
			return;
		}
		
		var self = this;
		$.ajax({
			url: this._jspBaseUrl+"GetNameIDList.jsp?",
			type: "GET",
			data:"list=VisibleProjectFolders&filter=*&projectID="+this._selectedProjectID,
			async:false, //enable this if required
			dataType: 'xml',
			timeout: 120000,
			success:function(data){
                // check for errors first
                var code = data.getElementsByTagName("code");
                if (code[0] && code[0].textContent === "001") {
                   	alert("Session Timeout. Cannot load visible folders for a Project");
                } else if (code[0] && code[0].textContent === "000") {
                   	alert("Cannot load visible folders for the selected Project");
                }

                //process the returned Folder ID/Name values
                var entries = data.getElementsByTagName("entry");
                if (entries && entries !== null) {
                    for(var i=0; i<entries.length; i++){
                        var curEntry = entries[i];
                        var folderID = curEntry.getAttribute("id");
                        var folderName = curEntry.textContent;
                        //add the current folder info to the target container
                        self._availableFolders.push({id: folderID, name: folderName});
                    }
                }
			},
			error:function(jqXHR, textStatus, errorThrown){
				  	alert("Cannot load visible folders for a Project");
		  	}
		});	
	};
	
	/**
	 * AJAX-Load Documents for the selected Project Folder (or across the entire Project if no Folder filter is applied; loads only for the visible Folders)
	 */
	DocumentSelectorView.prototype._loadVisibleDocuments = function () {
		//clear out the current list of 'available Documents'
		this._availableDocs = [];
		
		if (this._selectedProjectID == null || this._selectedProjectID.length == 0) {
			return;
		}
		
		//prepare the AJAX call parameters
		var loaderAjaxURL = this._jspBaseUrl+"GetNameIDList.jsp?";
		var ajaxParamDataSet = "list=VisibleProjectDocuments&filter=*&projectID="+this._selectedProjectID;
		if (this._selectedFolderID != null && this._selectedFolderID.length > 0) {
			ajaxParamDataSet = ajaxParamDataSet+"&folderID="+this._selectedFolderID;
		}
		
		var self = this;
		$.ajax({
			url: loaderAjaxURL,
			type: "GET",
			data: ajaxParamDataSet,
			async:false, //enable this if required
			dataType: 'xml',
			timeout: 120000,
			success:function(data){
				// check for errors first
                var code = data.getElementsByTagName("code");
                if (code[0] && code[0].textContent === "001") {
                   	alert("Session Timeout. Cannot load visible Documents for a Project/Folder");
                } else if (code[0] && code[0].textContent === "000") {
                   	alert("Cannot load visible Documents for a Project/Folder");
                }

                //process the returned Folder ID/Name values
                var entries = data.getElementsByTagName("entry");
                if (entries && entries !== null) {
                    for(var i=0; i<entries.length; i++){
                        var curEntry = entries[i];
                        var documentID = curEntry.getAttribute("id");
                        var documentName = curEntry.textContent;
                        //add the current folder info to the target container
                        self._availableDocs.push({id: documentID, name: documentName});
                    }
                }
			},
			error:function(jqXHR, textStatus, errorThrown){
				  	alert("Cannot load visible Documents for a Project/Folder");
		  	}
		});	
	};
	
	/**
	 * AJAX-Load Document Version and Revisions info for the selected Document
	 */
	DocumentSelectorView.prototype._loadDocumentVersionsAndRevisions = function () {
		//clear out the current list of 'available Document Versions and Revisions'
		this._availableDocVersions = [];
		this._availableDocRevisions = [];
		
		if (this._selectedDocumentID == null || this._selectedDocumentID.length == 0) {
			return;
		}	
		
		//prepare the AJAX call parameters
		var loaderAjaxURL = this._jspBaseUrl+"GetDocumentHistory.jsp?";
		var ajaxParamDataSet = "projectid="+this._selectedProjectID+"&docid="+this._selectedDocumentID;
		
		var self = this;
		$.ajax({
			url: loaderAjaxURL,
			type: "GET",
			data: ajaxParamDataSet,
			async:false, //enable this if required
			dataType: 'xml',
			timeout: 120000,
			success:function(data){
				// check for errors first
                var code = data.getElementsByTagName("code");
                if (code[0] && code[0].textContent === "001") {
                   	alert("Session Timeout. Cannot load Document Versions/Revisions for a Document");
                } else if (code[0] && code[0].textContent === "000") {
                	alert("Cannot load Document Versions/Revisions for a Document");
                }

                //process the returned Folder ID/Name values
                var docVersionEntries = data.getElementsByTagName("document_version");
                if (docVersionEntries && docVersionEntries !== null) {
                    for(var i=0; i< docVersionEntries.length; i++){
                        var curDocVersionEntry = docVersionEntries[i];
                        var documentVersionID = curDocVersionEntry.getElementsByTagName('docVersionID')[0].textContent;
                        var documentVersionNumber = curDocVersionEntry.getElementsByTagName('version_number')[0].textContent;
                        var documentRevisionLetter = curDocVersionEntry.getElementsByTagName('revision_letter')[0].textContent;
                        if (documentRevisionLetter.length == 0) {
                        	documentRevisionLetter = "0";
                        }
                       
                        //add the current Document Version info to the target container
                        self._availableDocVersions.push({id: documentVersionID, version: documentVersionNumber, revision: documentRevisionLetter});
                        
                        //also add the current version into the appropriate Revision letter container (if not there already)
                        if ($.inArray(documentRevisionLetter, self._availableDocRevisions) == -1) {
                        	self._availableDocRevisions.push(documentRevisionLetter);
                        }
                    }
                }
			},
			error:function(jqXHR, textStatus, errorThrown){
				  	alert("Cannot load Document Versions/Revisions for a Document");
		  	}
		});	
	};
	
	/**
	 * Executed when a Document or Document Revision / Version Selection is submitted by the User (via "Select" button)
	 * 
	 * @param documentName
	 */
	DocumentSelectorView.prototype._selectDocument = function (projectID, folderID, documentID, documentVersions, documentRevision, targetDocVersionID, documentName, docVerRevName) {				
		//trigger "Document Selected" event callback functions
		for (var i=0; i < this._onDocumentSelectedCallbacks.length; i++) {
			this._onDocumentSelectedCallbacks[i](projectID, folderID, documentID, documentVersions, documentRevision, targetDocVersionID, documentName, docVerRevName);
		};
	};
	
	/**
	 * Executed when a User clicks on the "Cancel" button
	 * 
	 */
	DocumentSelectorView.prototype._cancelButtonClicked = function () {				
		//trigger "Cancel Button Clicked" event callback functions
		for (var i=0; i < this._onCancelClickedCallbacks.length; i++) {
			this._onCancelClickedCallbacks[i]();
		};
	};
	
	/**
	 * Registers a callback function for "Cancel button clicked" event
	 * 
	 * @param onDocumentSelectedCallback
	 */
	DocumentSelectorView.prototype.onDocumentSelected = function(onDocumentSelectedCallback) {
		this._onDocumentSelectedCallbacks.push(onDocumentSelectedCallback);
	};
	
	/**
	 * Registers a callback function for "Document Selection Submit" event
	 * 
	 * @param onCancelClickedCallback
	 */
	DocumentSelectorView.prototype.onCancelClicked = function(onCancelClickedCallback) {
		this._onCancelClickedCallbacks.push(onCancelClickedCallback);
	};

	/**
	 * Returns a reference to the wrapper DIV element of this module
	 * 
	 * @returns JQuery object
	 */
	DocumentSelectorView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	/**
	 * Used to clear values of the output parameters
	 */
	DocumentSelectorView.prototype._clearBasicProperties = function () {
		this._selectedProjectID = "";
		this._selectedFolderID = "";
		this._selectedDocumentID = "";
		this._selectedDocumentVersionIDs = []; 
		this._targetDocumentVersionID = "";
		this._selectedDocumentRevision = "";
	};
	
	/**
	 * Validate that the target output parameters are OK for sending back to the requesting location
	 * 
	 * @returns {Boolean} > true if validated succesfully
	 */
	DocumentSelectorView.prototype._validateOutputParams = function () {
		if ((this._selectedProjectID && this._selectedProjectID !== null && this._selectedProjectID.length > 0)
			&& (this._selectedDocumentID && this._selectedDocumentID !== null && this._selectedDocumentID.length > 0)
			&& (this._targetDocumentVersionID && this._targetDocumentVersionID !== null && this._targetDocumentVersionID.length > 0)
		) {
			return true;
		} else {
			return false;
		}
	};
	
	//MODULE DEFINITION END
	return DocumentSelectorView;	
});
	
