define(["underscore", "jquery", "utils/Observable", "utils/webcss!./dropzone.css"], function (_, $, Observable) {

	"use strict";

	/**
	 * Abstract class for dropzones - takes care of events and adds classes to different states of the dropzone
	 * the classes are: dropzone - default class
	 * 					valid-dropzone - when items are  being dragged over the dropzone that can be dropped in the dropzone
	 *					invalid-dropzone - when non valid items are being dragged over the dropzone
	 *					dropzone-not-compatible - if the browser doesn't support html5 drag and drop
	 * @class
	 * @exports Dropzone
	 * @mixes Observable
	 * @param {Object} configuratin - {[element - <JQueryObject>]}
	 **/
	var Dropzone = function (configuration) {
		Observable.mixin(this, "dropzone");

		configuration = configuration || {};

		/**
		 * @member {JQueryElement}
		 * @private 
		 */
		this._element = (configuration.element || $("<div/>")).addClass("dropzone");

		this._overlay = $("<div/>").addClass("dropzone-overlay").addClass("dropzone-hidden");
		this._element.append(this._overlay);

		// detect HTML 5 Feature 
		this.browserCompatible = "draggable" in document.createElement("span");

		if (this.browserCompatible) {
			this._initializeEvents();

		} else {
			this._element.addClass("dropzone-not-compatible");
		}
	};

	/**
	 * Initializes the events for the dropzone
	 */
	Dropzone.prototype._initializeEvents = function () {

		this._element.on("dragenter", _.bind(function (event) {
			// validate if files are correct
			var isValid = this._validateDataTransfer(event.originalEvent);
			if (isValid === true) {
				this._element.addClass("valid-dropzone");
			} else if (isValid === false) {
				this._element.addClass("invalid-dropzone");
			}
			this._addOverlay();
			event.preventDefault();
		}, this));
		this._overlay.on("dragover", _.bind(function (e) {
			e.preventDefault();
		}, this));

		this._overlay.on("dragleave", _.bind(function (event) {
			this._element.removeClass("valid-dropzone");
			this._element.removeClass("invalid-dropzone");
			this._removeOverlay();
		}, this));
		this._overlay.on("drop", _.bind(function (event) {
			this._handleDrop(event.originalEvent);
			this._element.removeClass("valid-dropzone");
			this._element.removeClass("invalid-dropzone");
			this._removeOverlay();
			event.preventDefault();
		}, this));
	};

	/**
	 * Function called when files are dragged into the dropzone. This function is used to determine the class that will be added to the dropzone.
	 * if true is returned, the default "valid-dropzone" class will be added. If false is returned, "invalid-dropzone" is added. If undefined is returned, no class gets added
	 * The _handleDrop function will be called regardless of the returned value of this function.
	 * @param {DragEvent}
	 * @return {Boolean | undefined}
	 */
	Dropzone.prototype._validateDataTransfer = function (event) {
		// override me
		return true;
	};

	/**
	 * Handle the drop event. 
	 * @param {DragEvent}
	 */
	Dropzone.prototype._handleDrop = function (event) {
		// override me
	};

	/**
	 * adds an overlay to the dropzone that will handle the drop event
	 */
	Dropzone.prototype._addOverlay = function () {
		this._overlay.removeClass("dropzone-hidden");
	};
	
	/**
	 * removes the overlay
	 */
	Dropzone.prototype._removeOverlay = function () {
		this._overlay.addClass("dropzone-hidden");
	};

	/**
	 * returns the dropzone element
	 * @return {JQueryObject}
	 */
	Dropzone.prototype.getElement = function () {
		return this._element;
	};


	return Dropzone;
});
