define([
		"Template",
		"utils/Observable",
		"underscore",
		"jquery",
		"utils/webcss!./FilterList.css"
	],
	function (Template,
		Observable,
		_,
		$) {

		"use strict";

		/**
		 * @class
		 * @exports FilterList
		 * @mixes Observable
		 * 
		 * A component that contains a list and a input element to filter this list
		 * 
		 * @param {Function} filter - function that receives a String pattern and a page and should return a promise of entries
		 * @param {Object} options - options: filterplaceholder
		 */
		var FilterList = function (filter, options) {
			Observable.mixin(this, "filter-list");

			/*
			 * @member {Boolean} 
			 * @private
			 */
			this._isEnabled = true;
			/**
			 * @member {JQueryObject} the filter input field
			 * @private
			 */
			this._input = $('<input type="text" class="filter form-control" />');
			
			if(options.filterplaceholder) {
				this._input.attr({"placeholder": options.filterplaceholder});
			}

			/**
			 * @member {Integer} the current page of the filterlist
			 * @private
			 */
			this._page = 1;

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._div = $('<div/>').addClass("filter-list");

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._listContext = $('<div/>').addClass("list-group");

			this._div.append(this._input).append(this._listContext);

			/**
			 * @member {Function} the filter used to get data
			 * @private
			 */
			this.filterEntries = filter;

			// On input of the input filter, ask for data
			this._input.on("input", _.bind(function (e) {
				this._page = 1;
				this.filterEntries(this._input.val(), this._page).done(_.bind(function (values) {
					this.render(values);
				}, this));
			}, this));

			// if the scroll bar is at the end, ask for extra items
			this._listContext.on("scroll", _.partial(function (filterList, e) {
				if (this.scrollHeight - this.offsetHeight - this.scrollTop < 5) {
					filterList.renderNext();
				}
			}, this));

		};

		/**
		 * returns the ids of the entries visible in list
		 * @returns {Array.<String>}
		 */
		FilterList.prototype.getFilteredEntries = function () {
			var selection = [];
			this._listContext.find(".list-group-item").each(function () {
				var id = $(this).attr('data-id');
				selection.push(id);
			});
			return selection;
		};

		/**
		 * set context of the filter list
		 * @param {JqueryObject} div
		 */
		FilterList.prototype.setContext = function (div) {
			div.append(this._div);

		};

		/**
		 * gets the ids of the selected entries
		 * @returns {Array.<String>}
		 */
		FilterList.prototype.getSelected = function () {
			var selection = [];
			this._listContext.find(".list-group-item.active").each(function () {
				var id = $(this).attr('data-id');
				selection.push(id);
			});
			return selection;
		};


		/**
		 * returns the value of the input field
		 * @returns {String}
		 */
		FilterList.prototype.getFilter = function () {
			return this._input.val();
		};

		/**
		 * removes the read-onlyness of the filter
		 */
		FilterList.prototype.enable = function () {
			this._isEnabled = true;
			this._input.attr("disabled", false);
		};

		/**
		 * makes this filter list read only
		 */
		FilterList.prototype.disable = function () {
			this._isEnabled = false;
			this._input.attr("disabled", true);

		};

		/**
		 * render the template
		 * @param {Array.<Object>} values
		 */
		FilterList.prototype.render = function (entries) {
			this._page = 1;
			if (entries) {
				this._listContext.empty();
				this.addListItems(entries);
			} else {
				this.filterEntries(this.getFilter(), this._page).done(_.bind(function (entries) {
					this._listContext.empty();
					this.addListItems(entries);
				}, this));
			}

		};

		/**
		 * renders extra entries 
		 * @param {Array.<Object>} entries - list of objects containing an id and a label property
		 */
		FilterList.prototype.addListItems = function (entries) {
			_.each(entries, function (entry) {
				if (this._listContext.find(".list-group-item[data-id='" + entry.id + "']").size() > 0) {
					return;
				}
				var li = $("<a href='#'/>");
				li.addClass("list-group-item").text(entry.label).attr("data-id", entry.id).attr("data-label", entry.label);
				if (!this._isEnabled) {
					li.addClass("disabled");
				}
				li.on("click", _.partial(function (filterList, e) {
					if (filterList._isEnabled) {
						var elem = $(this);
						if (!e.ctrlKey) {
							filterList._listContext.find(".list-group-item.active").each(function () {
								$(this).removeClass("active");
							});
						}
						if (!elem.hasClass('active')) {
							elem.addClass('active');
						} else {
							elem.removeClass('active');
						}
						var id = elem.attr("data-id");
						var label = elem.attr("data-label");
						filterList._emit("itemSelected", id, label, e);
					}
					e.stopPropagation();
					e.preventDefault();
				}, this));
				this._listContext.append(li);
			}, this);
		};

		/**
		 * removes list items by id
		 * @param {Array.<String>} ids 
		 */
		FilterList.prototype.removeLiByIds = function (ids) {
			_.each(ids, function (id) {
				this._listContext.find(".list-group-item[data-id='" + id + "']").remove();
			}, this);
			if (this._listContext[0].scrollHeight - this._listContext.height() < 10) {
				this._emit("noScrollbar");
			}
		};

		/**
		 * removes list items by entry
		 * @param {Array.<String>} ids 
		 */
		FilterList.prototype.removeLiByEntries = function (entries) {
			_.each(entries, function (entry) {
				this._listContext.find(".list-group-item[data-id='" + entry.id + "']").remove();
			}, this);
			if (this._listContext[0].scrollHeight - this._listContext.height < 10) {
				// indicate that the elements fit the list item and no scrollbar is present 
				this._emit("noScrollbar");
			}
		};

		/**
		 * renders the next entries using the filter function
		 */
		FilterList.prototype.renderNext = function () {
			this._page++;
			this.filterEntries(this.getFilter(), this._page).done(_.bind(function (entries) {
				if (entries.length === 0) {
					this._page--;
				} else {
					this.addListItems(entries);
				}
			}, this));
		};
		
		FilterList.prototype.setFocus = function () {
			this._listContext.children('a').last().focus();
		};

		return FilterList;
	});