define(["underscore",
	"jquery",
	"utils/Observable",
	"utils/webcss!./inputdropdown-style.css"
], function (_, $, Observable) {

	"use strict";

	/**
	 * A component that combines an input and a dropdown 
	 *
	 * @class
	 * @exports InputDropdown
	 * @mixes Observable
	 * 
	 * @param {Object} options
	 */
	var InputDropdown = function (options) {
		Observable.mixin(this, "inputdropdown");

		/**
		 * @member {Object}
		 * @private
		 */
		this._options = options || {};
		/**
		 * @member {JQueryObject} the dropdown
		 * @private
		 */
		this._dropdownUL = this._options.ul || $("<ul/>");
		this._dropdownUL.addClass("dropdown-menu input-dropdown-menu");

		/**
		 * @member {JQueryObject} the input field
		 * @private
		 */
		this._input = $("<input type=text>").addClass("form-control input");
		this._input.on("change", _.bind(this._inputChanged, this));

		/**
		 * @member {JQueryObject} caret for dropdown toggle
		 * @private
		 */
		this._caret = $("<span/>").addClass("caret");

		/**
		 * @member {JQueryObject} 
		 * @private
		 */
		this._toggle = $("<div />").addClass("dropdown-toggle").attr("data-toggle", "dropdown").append(this._caret);

		/**
		 * @member {JQueryObject} 
		 * @private
		 */
		this._element = $("<div/>")
			.addClass("input-dropdown")
			.append(this._input)
			.append(this._toggle)
			.append(this._dropdownUL);

		/**
		 * @member {JQueryObject} 
		 * @private
		 */
		this._selectedLi = null;

		if (!this._options.hideOnInputFocus) {
			this._input.on("focus", _.bind(function () {
				if (this._toggle.attr("aria-expanded") !== "true") {
					this._toggle.dropdown("toggle");
				}
			}, this));
		}

	};

	/**
	 * Adds an option to the dropdown list
	 * @param {String}  val    - value of the option
	 * @param {String}  text   - text of the option
	 * @param {Boolean} isHtml - whether or not the text is html
	 */
	InputDropdown.prototype.addOption = function (val, text, isHtml) {
		var li = $("<li/>").attr("data-value", val);
		if (isHtml) {
			li.append($("<a href='#'/>").html(text));
		} else {
			li.append($("<a href='#'/>").text(text));
		}
		li.on("click", _.bind(this._liSelected, this));
		this._dropdownUL.append(li);
	};

	/**
	 * sets the options of the dropdown list. The options can be objects containing a value and a text property or an array of strings
	 * @param {Array.<Object> | Array.<String>} options 
	 */
	InputDropdown.prototype.setOptions = function (options) {
		this._dropdownUL.empty();
		_.each(options, function (option, index) {
			this.addOption(option.value || option, option.text || option);
		}, this);
	};

	/**
	 * returns the element of the input dropdown
	 * @returns {JQueryObject} 
	 */
	InputDropdown.prototype.getElement = function () {
		return this._element;
	};

	/**
	 * clears the dropdown
	 */
	InputDropdown.prototype.empty = function () {
		this._dropdownUL.empty();
	};

	/**
	 * sets the value of the dropdown or gets the value if @param {val} is undefined
	 * @param   {String}             [val] - value to set
	 * @returns {String | undefined} 
	 */
	InputDropdown.prototype.val = function (val) {
		if (val) {
			var li = $(this._dropdownUL.find("li[data-value]").filter(function(i, elem){
				return elem.getAttribute("data-value") === val;
			})[0]);
			if (li.length > 0) {
				this._selectedLi = li;
				this._input.val(li.text());
			}
		} else {
			if (this._selectedLi) {
				return this._selectedLi.val();
			} else {
				return this._input.val();
			}
		}
	};

	/**
	 * callback on input change
	 */
	InputDropdown.prototype._inputChanged = function (e) {
		var text = this._input.val();
		var li = this._getLiByText(text);
		if (li && (!this._selectedLi || (this._selectedLi && (li.get() !== this._selectedLi.get())))) {
			this._selectedLi = li;
			this._emit("changed.value", li.attr("data-value"));
			this._input.val(li.text());
		} else {
			this._selectedLi = null;
			this._emit("new.input", text);
		}
		e.preventDefault();
	};

	/**
	 * callback on li selected
	 * @param {Object} event 
	 */
	InputDropdown.prototype._liSelected = function (event) {
		var li = $(event.currentTarget);
		if ((li && !this._selectedLi) || (!li && this._selectedLi) || (li.get() !== this._selectedLi.get())) {
			this._selectedLi = li;
			this._emit("changed.value", li.attr("data-value"));
			this._input.val(li.text());
		}
		event.preventDefault();

	};

	/**
	 * gets the li by text content
	 * @param   {String}       text 
	 * @returns {JQueryObject} 
	 */
	InputDropdown.prototype._getLiByText = function (text) {
		var li;
		text = text.toLowerCase();
		this._dropdownUL.find("li").each(function () {
			if ($(this).text().toLowerCase() === text) {
				li = $(this);
			}
		});
		return li;
	};

	/**
	 * Disable the input dropdown
	 */
	InputDropdown.prototype.disable = function () {
		this._input.attr("disabled", true);
		this._toggle.removeAttr("data-toggle");
		this._toggle.css("cursor", "not-allowed");
	};

	/**
	 * Enable the input dropdown
	 */
	InputDropdown.prototype.enable = function () {
		this._input.attr("disabled", false);
		this._toggle.addAttr("data-toggle");
		this._toggle.css("cursor", "pointer");
	};

	/**
	 * focuses the input dropdown
	 */
	InputDropdown.prototype.focus = function () {
		this._input.focus();
	};

	/**
	 * Get input dropdown options
	 * @returns {Array.<Object>} array of objects containing val and text.
	 */
	InputDropdown.prototype.getDropdownOptions = function () {
		var options = [];
		this._dropdownUL.find("li").each(function () {
			options.push({
				val: $(this).attr("value"),
				text: $(this).text()
			});
		});
		return options;
	};



	return InputDropdown;

});