define([
		"jquery",
		"underscore",
		"utils/webi18n!JavaScriptEditor",
		"utils/webcss!./javascripteditor-style.css",
		"utils/ConvertUtils",
		"ace/ace",
		"lib/ace-modifications/mod-ace-all",
		"ace/ext/language_tools",
		"ace/mode/javascript",
		"ace/mode/javascript_worker",
		"ace/mode/javascript/jshint"
	],
	function ($,
		_,
		i18n,
		css,
		ConvertUtils,
		ace,
		modification,
		languageTools) {

		"use strict";
		var constants = {
			"API": "API",
			"VOID": "void", 
			"KEYWORDLIST": ["parseInt","parseFloat","isNaN","isFinite","URIError","TypeError","SyntaxError","StopIteration",
							"ReferenceError","RangeError","InternalError","EvalError","Error","RegExp","Number","Iteration",
							"Object","Date","Boolean","Array","while","var","typeof","try","throw","switch","return","new",
							"if","function","for","finally","else","do","continue","catch","case","break","const","undefined",
							"NaN","Infinity","null","false","true","API"]
		};
	
		

		/**
		 * Construction function for the javascript editor
		 * @param {String} editorDiv - The div that has to be used as editor div.
		 * @param {string} script    - The script
		 * @param {Object} options   - options for the panel. 
		 * Currently supported: {Number=100000} maxScriptLength - The maximum script length
		 *                      {Function} scrollbarVisibilityChanged - Callback to be executed when the scrollbars become visible/invisible
		 *                           
		 */
		var JavaScriptEditor = function (editorDiv, script, options) {

			/**
			 * @member {JQueryObj}
			 * @private
			 */
			this._editorDiv = editorDiv;

			this._infoBar = $("<div>").addClass("jsinfobar");
			this._editorDiv.after(this._infoBar);

			this._lineColumn = $("<span>").addClass("lineColumn");
			this._infoBar.append(this._lineColumn);
			this._linesChars = $("<span>").addClass("linesChars");
			this._infoBar.append(this._linesChars);
			this._lineCharsWarning = $("<i>").addClass("icon-small icon-WCR_Msg-Warning linesCharsWarning");
			this._infoBar.append(this._lineCharsWarning);

			/**
			 * @member {Object} options for the editor
			 * @private
			 */
			this._options = {
				maxScriptLength: 100000
			};
			this._options = _.defaults(options, this._options);

			/**
			 * @member {Object} object containing a getValue/setValue method for saving changes of the editor from/to
			 * @private
			 */
			this._parameter = null;

			/**
			* @member {Object} _variables
			* @private                    
				Structure of this object: (The example given is the structure of the API variable)
				{
					"api" : { type: "API"}, 
				}
			*/
			this._variables = { "api" : { type : constants.API}};

			/**
			 * @member {Array} this array will contain all possible methods, will be updated everytime a variable is added or changed
			 * @private
			 */
			this._methodList = [];

			/**
			 * enable basic autocomplete
			 */
			var langtools = languageTools;

			/**
			 *  standard: 3 completers, don't create a new completer for every instance because this will result in shared
			 *  autocompletion between both 
			 */
			var self = this;


			/**
			 * The ace editor 
			 */
			var editor = ace.edit(editorDiv[0]);
			this._editor = editor;
			editor.getSession().setMode("ace/mode/javascript");
			editor.getSession().getMode().$highlightRules.$keywordList = constants.KEYWORDLIST;

			/**
			 * this object will contain every possible method of an API object will dynamically be added from server
			 * @member {Object} 
			 */
			this._apiMethods = {};

			// update the api methods with a list from the server
			var url = window.webcenterConfig.jspBaseUrl + "JavaScriptAPIServlet.wcr";
			$.get(url, _.bind(function (data) {
				this._apiMethods = data;
				this._methodList = []; // empty the place holder apimethods with the methods received from the server
				this._variables = { "api" : { type : constants.API}}; // make sure to have an API variable

				//update the existing autocomplete once the api is loaded
				var lines = script.split("\n");
				for (var i = 0; i < lines.length; i++) {
					var currentType = this._findDynamicCurrentType(lines[i]);
					this._findVariables(lines[i], currentType);
				}
				this._updateMethodList();
			}, this));


			editor.setOptions({
				maxLines: 23,
				minLines: 23,
				autoScrollEditorIntoView: true,
				enableBasicAutocompletion: true,
				enableLiveAutocompletion: true,
				enableSnippets: false
			});

			editor.resize(true);

			//make sure that autocompletion continues after '.' character
			editor.commands.on("afterExec", _.bind(function (e, t) {
				//get the text of the current line
				var line = e.editor.session.getLine(e.editor.getSelectionRange().start.row);

				var currentType = this._findDynamicCurrentType(line);
				this._findVariables(line, currentType);

				//update methodlist and take this current type into account
				this._updateMethodList(currentType);
				
				var commandName = e.command.name;
				if (commandName === "insertstring" && e.args === ".") {
					e.editor.execCommand("startAutocomplete");
				}
			}, this));


			//handle the paste event
			editor.on("paste", _.bind(function (e) {

				//when pasting text variables should be updated
				var lines = e.text.split("\n");
				for (var i = 0; i < lines.length; i++) {
					var currentType = this._findDynamicCurrentType(lines[i]);
					this._findVariables(lines[i], currentType);
				}
				this._updateMethodList();
			}, this));

			/**
			 * The completer that will be used for autocompletion, only needs to be created once 
			 * (not on every focus event)
			 * After brackets / braces the prefix will be cleared
			 */
			var webCenterCompleter = {
				getCompletions: function (editor, session, pos, prefix, callback) {
					if (prefix.length === 0) {
						callback(null, []);
						return;
					}
					if(prefix === "." ){
						var i =0;
						callback(null, self._methodList.map(function (ea) {
							i--;
							return {
								name: "." + ea.name,
								value: "." + ea.name,
								meta: ea.meta,
								score: i
								};
						}));
					}else if(prefix[prefix.length-1] === "."){
						var j = 0;
						callback(null, self._methodList.map(function (ea) {
							j--;
							return {
								name: ea.name,
								value: prefix + ea.name,
								meta: ea.meta,
								score: j
								};
						}));
					}else{
						//find the last '.'
						var index = prefix.lastIndexOf(".");
						if(index !== -1){
							prefix = prefix.substring(0,index+1);
							var k = 0;
							callback(null, self._methodList.map(function (ea) {
								k--;
								return {
									name: ea.name,
									value: prefix + ea.name,
									meta: ea.meta,
									score: k
									};
							}));
						}
					}

					
				}
			};

			//make a context switch when focus switches => seperated auto completion in 2 windows
			editor.on("focus", _.bind(function (e) {
				//addCompleter ensures that only one webCenterCompleter will be in the completers list
				langtools.addCompleter(webCenterCompleter);

			}, this));

			//disable a warning message from ace
			editor.$blockScrolling = Infinity;

			//remove scrollbar when cursor is in last row
			$(".ace_editor").css("overflow", "hidden").css("margin", "0");


			// set the script
			editor.setValue(script, 1);

			// reset the undo manager of ace to have the script as "default" value
			var UndoManager = ace.require("ace/undomanager").UndoManager;
			editor.getSession().setUndoManager(new UndoManager());

			// listen to cursor changes
			editor.getSelection().on("changeCursor", _.bind(function () {
				this._updateInfoBar();
			}, this));

			// listen to scroll bar visibility changes
			editor.renderer.on("scrollbarVisibilityChanged", _.bind(function (e, renderer) {
				var callback = this._options['scrollbarVisibilityChanged'];
				if (typeof callback !== "undefined") {
					callback();
				}
			}, this));
			
			editor.getSession().$worker.call("setOptions",[{
				unused: true,
				esnext: true,
				moz: true,
				devel: false,
				browser: false,
				node: true,
				laxcomma: true,
				laxbreak: true,
				lastsemic: true,
				onevar: false,
				passfail: false,
				maxerr: 100,
				expr: true,
				multistr: true,
				globalstrict: false
			}]);

			// initial update
			this._updateInfoBar();
		};

		/**
		 * Returns the ace editor object
		 * @returns {Ace Editor} 
		 */
		JavaScriptEditor.prototype.getEditor = function () {
			return this._editor;
		};

		/**
		 * Removes the brackets in the input string returns true when something changed
		 * @private
		 * @param {String} string the string to check and remove the brackets from
		 */
		JavaScriptEditor.prototype._findAndRemoveBrackets = function (string) {

			var findBrackets = string.search(/\[[^\]]+\]/);
			if (findBrackets === -1) {
				return string;
			}
			var result = string.replace(/\[[^\]]+\];?/, "");

			return result;
		};

		/**
		 * Check if the current line has an assignment statement (e.g. var x =)
		 * if it does, finds the name and type of the variable and adds it to the list of known 
		 * variables
		 * @private
		 * @param {string} untrimmedLine 
		 * @param {string} currentType
		 */
		JavaScriptEditor.prototype._findVariables = function (untrimmedLine, currentType) {
			var line = untrimmedLine.trim();
			var findVar = line.search(/var.*=/);
			if (findVar !== -1 && typeof currentType !== "undefined") {
				var lineSplit = line.split("=");

				var variableName = lineSplit[0].split(" ")[1];

				this._variables[variableName.toLowerCase()] = {
					"type": currentType
				};

			}

		};

		/**
		 * Checks if the current line ends with a possible function 
		 * if it does, the auto completion will be updated to only show the methods of that object type
		 * @private
		 * @param {String} line 
		 */
		JavaScriptEditor.prototype._findDynamicCurrentType = function (line) {

			/*
				Explanation of the regex: 
					- Line ends in an array (no function call) -> \[[^]]*\]
					- Line ends in a function call (that can be indexed aswell) \([^)]*\)(\[[^]]*\])?
					function call can end in a ';' or a '.' 
					
			*/
			var findFunction = line.search(/\.|((\[[^]]*\]|\([^)]*\)(\[[^]]*\])?);?\.?$)/);
			if (findFunction != -1) {

				var interestingPartOfLine = line.toLowerCase();
				if (line.indexOf("=") !== -1) {
					interestingPartOfLine = interestingPartOfLine.split("=")[1];
				}

				var rightHandSide = interestingPartOfLine.replace(/\s|;/g, "");

				var splitted = rightHandSide.split(".");

				var identifier = splitted[0].replace(/\s/g, "");

				var removedBrackets = this._findAndRemoveBrackets(identifier);

				var currentType = this._findCurrentType(identifier, removedBrackets, splitted);

				return currentType;
			}
			return undefined;

		};


		/**
		 * Find the type of the currently used object
		 * @private
		 * @param   {String} identifier      
		 * @param   {String} removedBrackets 
		 * @param   {String} splitted  An array of all parts of the declaration
		 * @returns {String} the  current type or undefined when no type could be found
		 */
		JavaScriptEditor.prototype._findCurrentType = function (identifier, removedBrackets, splitted) {

			if (typeof removedBrackets !== "undefined" && typeof this._variables[removedBrackets] !== "undefined") {
				var currentType = this._variables[removedBrackets].type;

				if (removedBrackets !== identifier) {
					currentType = currentType.replace(/Array/, "");
				}

				//iterrate all parts to see what the result type is
				for (var i = 1; i < splitted.length; i++) {

					if (typeof (this._apiMethods[currentType]) !== "undefined") {
						var methodName = splitted[i];
						removedBrackets = this._findAndRemoveBrackets(methodName);

						var tempType = currentType;
						for (var j = 0; j < this._apiMethods[currentType].length; j++) {

							var method = this._apiMethods[currentType][j];
							if (methodName.contains(method.name.toLowerCase())) {
								tempType = method.returns;
							}

						}
						if (removedBrackets !== methodName) {
							tempType = tempType.replace(/Array/, "");
						}
						currentType = tempType;
					}

				}
				return currentType;

			} else {

				return undefined;
			}


		};

		/** find the name of the variable 
		 * @private
		 * @param {String} line      must be like "var %tmp% = APIFUNCTION"
		 * @param {number} startIndex The index where 'var' is found
		 */
		JavaScriptEditor.prototype._findTemporaryName = function (line, startIndex) {

			var start = line.indexOf(" ", startIndex);

			var end = line.indexOf(" ", start + 1);
			return line.substr(start + 1, end - start - 1);
		};

		/**
		 * Update the list of known methods (linked to the variables);
		 * @private
		 * @param {string} currentType the current type of which the methods should be shown aswell 
		 *                         (e.g. API.getProject(){Type Specific Methods})
		 *                         
		 */
		JavaScriptEditor.prototype._updateMethodList = function (currentType) {
			this._methodList = [];

			//update this so that it only shows the methods of the CURRENT TYPE
			if (typeof currentType !== "undefined" && typeof this._apiMethods[currentType] !== "undefined") {
				var apimethods = [];
				for (var i = 0; i < this._apiMethods[currentType].length; i++) {

					var method = this._apiMethods[currentType][i];
					var fullname = method.name;
					apimethods.push({
						name: fullname,
						value: fullname,
						meta: currentType,
						returns: method.returns
					});
				}
				this._methodList = _.sortBy(apimethods, "name");
			} 
		};

		/**
		 * Method that has to be called when the javascript editor has to be destroyed
		 */
		JavaScriptEditor.prototype.destroy = function () {


			if (typeof this._editor !== "undefined") {
				//when the panel is closed, also hide the auto completion
				if (typeof this._editor.completer !== "undefined" && typeof this._editor.completer.getPopup() !== "undefined") {
					this._editor.completer.getPopup().hide();
				}

				//there are some memory leaks when closing ace, this should fix that
				this._editor.destroy();
				this._editorDiv.remove();
				this._infoBar.remove();
			}

		};

		/**
		 * Method that has to be called when the javascript editor has to be validated
		 */
		JavaScriptEditor.prototype.validate = function () {
			if (typeof this._editor !== "undefined") {
				var val = this._editor.getValue();
				if (val.length > this._options.maxScriptLength) {
					window.alert(i18n.t("scriptLengthViolation"));
					return false;
				}
				return true;
			}
			return true;
		};

		/**
		 * This method resizes the editor
		 * @param {number} height The maximum height useable by the editor
		 */
		JavaScriptEditor.prototype.resize = function (height) {


			//make sure the ace editor also resizes when the popup is resized
			var lineHeight = this._editor.renderer.lineHeight;
			if (this._originalInfoBarHeight === undefined) {
				this._originalInfoBarHeight = this._infoBar.outerHeight(true); // first time
			}
			var infoBarHeight = this._originalInfoBarHeight;
			var rendererContainer = $(this._editor.container);
			var hScrollBar = rendererContainer.find(".ace_scrollbar-h:visible");
			// a scrollbar takes 17 pixels, that's either lineheight + 2 or scrollHeight -5.
			var realHScrollHeight = (hScrollBar.length > 0 ? lineHeight + 2 : 0);
			var heightLeft = height - infoBarHeight - realHScrollHeight;
			var maxLines = Math.floor((heightLeft) / lineHeight);

			this._editor.setOptions({
				maxLines: maxLines,
				minLines: maxLines
			});
			this._editor.resize();

			// fill space
			var newInfoBarHeight = height - (maxLines * lineHeight) - realHScrollHeight;
			this._infoBar.css({
				'height': newInfoBarHeight + "px",
				'line-height': newInfoBarHeight + "px"
			});
		};

		/**
		 * Specifies to which variable the content of the ace editor should be saved
		 * @param {Object} parameter the object must contain a method 'setValue' and 'getValue' which sets/gets the value
		 */
		JavaScriptEditor.prototype.saveChangesTo = function (parameter) {
			this._editor.on("change", _.bind(function (e) {
				this._updateInfoBar();
				var val = this._editor.getValue();
				if (val.length <= this._options.maxScriptLength) {
					// see afterexec above, this way the parameter always contains a valid length
					parameter.setValue(this._editor.getValue());
				}
			}, this));
		};

		/**
		 * Update the text in the info bar
		 * @private
		 */
		JavaScriptEditor.prototype._updateInfoBar = function () {
			var cursorPos = this._editor.getCursorPosition();

			var line = cursorPos.row + 1; // 0 based
			var column = cursorPos.column + 1; // 0 based

			var lines = this._editor.getSession().getLength();
			var charLength = this._editor.getValue().length;
			var maxCharLength = this._options.maxScriptLength;

			// warning
			this._lineCharsWarning.toggle(charLength > maxCharLength);

			// format them nicely
			line = ConvertUtils.format(line, 0, 3, '.', ',');
			column = ConvertUtils.format(column, 0, 3, '.', ',');
			lines = ConvertUtils.format(lines, 0, 3, '.', ',');
			charLength = ConvertUtils.format(charLength, 0, 3, '.', ',');
			maxCharLength = ConvertUtils.format(maxCharLength, 0, 3, '.', ',');

			// line/character count
			this._lineColumn.text(i18n.t("infoBarLineColumn", line, column));
			this._linesChars.text(i18n.t("infoBarLinesChars", lines, charLength, maxCharLength));
		};

		return JavaScriptEditor;


	});