/**
 * KeyWordSearchView 
 */

define([
		"utils/Observable",
		"./keywordsearchDataProvider",
		"jquery",
		"underscore",
		"component/sdrop/SDrop"],
	function (
		Observable,
		KeywordSearchDataProvider,
		$,
		_,
		SDrop) {

		"use strict";


		/**
		 * @class 
		 * @constructor
		 * @mixes Observable
		 * @param {JQueryObj} element - container for saved search
		 */
		var KeyWordSearchView = function (element) {
			Observable.mixin(this, "keywordSearchView");

			// If not wrapped by jQuery, do it.
			if (!(element instanceof $)) {
				element = $(element);
			}
			this._wrapper = element;
			// form
			this._form = this._wrapper.find("form");


			// search type field
			this._searchTypeInput = this._form.find("input[name='searchtypelist']");
			this._searchTypeIcon = this._form.find("i[name='searchTypeIcon']");

			this._searchTypeDropdown = this._form.find("a[name='searchTypeDropdown']");
			this._searchTypeDropdownOptions = this._form.find("ul");


			// saved search dropdown
			this._savedSearchDropdown = this._form.find("select[name='savedSearchID']");

			// keyword search input
			this._inputField = this._form.find("input[name='db_keyword']");

			this._searchButton = this._form.find("button[name='executeSearchButton']");
			this._searchButton.click(_.bind(function () {
				this._emit("clicked.search");
			}, this));
			this._sdrop = null;
			this._inputFieldUsed = true;

			// don't submit when empty search
			this._form.submit(_.bind(function (e) {
				if (this.isEmptySearch()) {
					e.preventDefault();
				}
			}, this));
		};

		/**
		 * Change the available search types
		 * @param {Array.<SearchType>} searchTypes - each searchtype is a plain object with properties: name, icon and id, placeholder
		 */
		KeyWordSearchView.prototype.setSearchTypes = function (searchTypes) {
			this._searchTypes = searchTypes;
			this._searchTypeDropdownOptions.empty();
			_.each(searchTypes, function (searchType) {
				this._searchTypeDropdownOptions.append(
					$("<li>")
					.append($("<a>")
						.text(" " + searchType.name)
						.prepend($("<i>", {
							'class': "icon-normal " + searchType.icon
						}))
						.click(_.bind(function () {
							this._emit("changed.searchType", searchType.id);
						}, this))
					)
				);
			}, this);
		};


		/**
		 * Change the currently active search type
		 * @param {SearchType} searchType- searchtype is a plain object with properties: name, icon, action, id and placeholder
		 */
		KeyWordSearchView.prototype.setActiveSearchType = function (searchType) {
			this._searchTypeIcon.removeClass().addClass(searchType.icon);
			this._form.attr("action", searchType.action);
			this._searchTypeDropdown.prop("title", searchType.name);
			this._searchTypeInput.val(searchType.searchtypelistValue);
			this._searchButton.prop("title", searchType.placeholder);
			this._inputField.prop("placeholder", searchType.placeholder);
		};


		/**
		 * Toggle between input field and sdrop dropdown
		 * @param {Boolean} toggle - true if sdrop should be enabled
		 * @param {Object} savedSearchOptions - set of options for saved search
		 * @param {Boolean} byUser - true if the search type was changed by the user (and a focus is needed)
		 * @param {String} [selectedSavedSearch] - id of the selected saved search
		 */
		KeyWordSearchView.prototype.toggleDropdown = function (toggle, savedSearchOptions, byUser, selectedSavedSearch) {
			this._inputField.toggleClass("hiddenElement", toggle);


			if (toggle) {
				if (!this._sdrop) {
					// initialize SDROP	
					var initialValue = null;
					if (selectedSavedSearch) {
						initialValue = selectedSavedSearch;
					}
					this._savedSearchDataProvider = new KeywordSearchDataProvider();
					this._sdrop = new SDrop(this._savedSearchDropdown, null, this._savedSearchDataProvider, {
						maxMatches: 500, // we want to keep as much as possible visible
						manageStyleAndWidth: false,
						enableEmptySelection: true,
						initialValue: initialValue,
						onChangedByUser: _.bind(function () {
							this._emit("changed.savedSearchSelection", this._sdrop.getSelectedItemID());
						}, this)
					});

				}
				this._savedSearchDataProvider.setData(savedSearchOptions);
			}

			if (this._sdrop) {
				this._sdrop.wrapper.toggleClass("hiddenElement", !toggle);
			}


			this._inputFieldUsed = !toggle;
			if (byUser) {
				if (toggle) {
					this._sdrop.wrapper.focus();
				} else {
					this._inputField.focus();
				}
			}
		};


		/**
		 * Make sure a search is selected when doing saved searches
		 * @returns {Boolean} true if keyword search, or a saved search is selected
		 */
		KeyWordSearchView.prototype.isEmptySearch = function () {

			if (this._inputFieldUsed) {
				return false;
			} else {
				return !this._savedSearchDropdown.val() || this._savedSearchDropdown.val() === "";

			}

		};

		/**
		 * Execute the search (submit the form)
		 */
		KeyWordSearchView.prototype.submitSearch = function () {
			if (!this.isEmptySearch()) {
				this._form.submit();
			}
		};

		/**
		 * Clear the input field of the keyword search
		 */
		KeyWordSearchView.prototype.clearSearchInput = function () {
			this._inputField.val("");
		};


		/**
		 * Get a jQuery wrapped reference to the encapsulating div of this component.
		 * 
		 * @returns {jQuery} The jQuery wrapped reference to the encapsulating div of this component.
		 */
		KeyWordSearchView.prototype.getJQElement = function () {
			return this._wrapper;
		};


		return KeyWordSearchView;

	});