define([
		"Template",
		"component/filterList/FilterList",
		"component/tooltip/Tooltip",
		"utils/Observer",
		"utils/Observable",
		"underscore",
		"jquery",
		"utils/webcss!./MultiSelectList.css"
	],
	function (Template, FilterList, Tooltip, Observer, Observable, _, $) {
		"use strict";

		/**
		 * @class
		 * @exports MultiSelectList
		 * @mixes Observer
		 * 
		 * A view-controller used to control and view the multi select model
		 * It uses two Filter Lists, one 'library' filter list that displays the not selected entries
		 * and one filter list to display the selected entries
		 * if a custom filter is given, the library filter list's filterEntries method will be overwritten by 
		 * a function that will call the custom filter and then filter out the entries that were already selected
		 * 
		 * @param {Function} libFilter        - filter used to filter the entries of the library
		 * @param {Function} selectedFilter   - filter used to filter the entries of the selected list
		 * @param {JQueryObject} context   - the context this element should be added to
		 * @param {Object}   options          - options: notSelectedLabel, selectedLabel, selectAllTooltip, deselectAllTooltip                                   
		 */
		var MultiSelectList = function (libFilter, selectedFilter, context, options) {
			Observer.mixin(this);
			Observable.mixin(this, "multiselectlist");

			this._options = options || {};

			/** 
			 * @member {JQueryObject} the containing element
			 * @private
			 */
			this._context = context;
			
			// Options for selected items filterlist
			var selectedFilterOptions = {};
			if(this._options.selFilterPlaceholder) {
				selectedFilterOptions.filterplaceholder = this._options.selFilterPlaceholder;
			}
			
			/**
			 * @member {FilterList} the filter list used for the selected entries
			 * @private
			 */
			this._selectedFilterList = new FilterList(selectedFilter, selectedFilterOptions);

			// Options for available items filterlist
			var availableFilterOptions = {};
			if(this._options.availFilterPlaceholder) {
				availableFilterOptions.filterplaceholder = this._options.availFilterPlaceholder;
			}
			
			/**
			 * @member {FilterList} the filter list used for the library, ie. the not selected entries
			 * @private
			 */
			this._notSelectedFilterList = new FilterList(libFilter, availableFilterOptions);

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._selectedDiv = $('<div/>').addClass('col-md-6 col-xs-12').attr({"name": "selecteditems"});

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._notSelectedDiv = $('<div/>').addClass('col-md-6 col-xs-12').attr({"name": "availableitems"});

			/**
			 * @member {Array.<Tooltip>} used for destroy method
			 * @private 
			 */
			this._tooltips = [];

			this._initFilterLists(this._options.selectedLabel || "", this._options.notSelectedLabel || "");

			/**
			 * @member {JQueryObject}
			 * @private
			 */
			this._buttonDiv = $('<div/>').addClass('btn-group multi-select-button-group');
			this._initButtons(this._options.selectAllTooltip || "", this._options.deselectAllTooltip || "");

			this._observe(this._selectedFilterList, "itemSelected", function (id, label, e) {
				this._emit("entryDeselected", [id], [label]);
				this._entryDeselected(id, label, e);
			});

			this._observe(this._notSelectedFilterList, "itemSelected", function (id, label, e) {
				this._emit("entrySelected", [id], [label]);
				this._entrySelected(id, label, e);

			});

			this._observe(this._notSelectedFilterList, "noScrollbar", function () {
				this._notSelectedFilterList.renderNext();
			});

			this._observe(this._selectedFilterList, "noScrollbar", function () {
				this._selectedFilterList.renderNext();
			});

			context.append($('<div class="multi-select">').append($('<div class="row">').append(this._notSelectedDiv).append(this._selectedDiv)).append($('<div class="row">').append(this._buttonDiv)));
		};


		/**
		 * called when an entry has been selected
		 * @param {String} id
		 * @param {String} label
		 * @param {Event} e
		 */
		MultiSelectList.prototype._entrySelected = function (id, label, e) {
			this._notSelectedFilterList.removeLiByIds([id]);
			this._selectedFilterList.addListItems([{
				"id": id,
				"label": label
			}]);
		};
	
		/**
		 * called when an entry has been deselected
		 * @param {String} id
		 * @param {String} label
		 * @param {Event} e
		 */
		MultiSelectList.prototype._entryDeselected = function (id, label, e) {
			this._selectedFilterList.removeLiByIds([id]);
			this._notSelectedFilterList.addListItems([{
				"id": id,
				"label": label
			}]);
		};

		/**
		 * sets the context of the multi select list
		 * @param {JQueryObject} context 
		 */
		MultiSelectList.prototype.setContext = function (context) {
			context.append(this._element);
		};

		/**
		 * initiate the filter lists 
		 * @param {String} selectedLabel - label above the selected filter list
		 * @param {String} notSelectedLabel - label above the library list
		 */
		MultiSelectList.prototype._initFilterLists = function (selectedLabel, notSelectedLabel) {
			this._selectedDiv.append($('<label/>').text(selectedLabel));
			this._selectedFilterList.setContext(this._selectedDiv);
			this._notSelectedDiv.append($('<label/>').text(notSelectedLabel));
			this._notSelectedFilterList.setContext(this._notSelectedDiv);

		};

		/**
		 * Initiate the buttons
		 */
		MultiSelectList.prototype._initButtons = function (selectAllTooltip, deselectAllTooltip) {

			// Deselect all button
			var deselectAllButton = $('<button/>').addClass('btn btn-default').html('<i class="icon icon-WCR_Table-Page-First12">').on('click', _.bind(function () {
				// Check if the user has used ctrl+click to select some of the selected entries
				var selection = this._selectedFilterList.getSelected();
				if (_.size(selection) > 0) {
					this._emit("entryDeselected", selection);
				} else {
					this._emit("deselectAll", this._selectedFilterList.getFilter());
				}
				this.render();
			}, this));
			this._buttonDiv.append(deselectAllButton);
			if (deselectAllTooltip) {
				this._tooltips.push(new Tooltip(null, deselectAllButton, $("body"), $("body"), false, deselectAllTooltip, undefined, undefined, this._options));
			}
			var selectAllButton = $('<button/>').addClass('btn btn-default').html('<i class="icon icon icon-WCR_Table-Page-Last12"></i>').on('click', _.bind(function () {
				var selection = this._notSelectedFilterList.getSelected();
				if (_.size(selection) > 0) {
					this._emit("entrySelected", selection);
				} else {
					this._emit("selectAll", this._notSelectedFilterList.getFilter());
				}
				this.render();
			}, this));

			this._buttonDiv.append(selectAllButton);
			if (selectAllTooltip) {
				this._tooltips.push(new Tooltip(null, selectAllButton, $("body"), $("body"), false, selectAllTooltip, undefined, undefined, this._options));
			}
		};


		/**
		 * enables multiselect
		 */
		MultiSelectList.prototype.enable = function () {
			this._selectedFilterList.enable();
			this._notSelectedFilterList.enable();
			this._buttonDiv.find("button").attr("disabled", false);
		};

		/**
		 * disables multiselect
		 */
		MultiSelectList.prototype.disable = function () {
			this._selectedFilterList.disable();
			this._notSelectedFilterList.disable();
			this._buttonDiv.find("button").attr("disabled", true);

		};

		/**
		 * rerenders the multi select list
		 */
		MultiSelectList.prototype.render = function () {
			this._selectedFilterList.render();
			this._notSelectedFilterList.render();
		};

		/**
		 * destroys the multi select list
		 */
		MultiSelectList.prototype.destroy = function () {
			_.each(this._tooltips, function (tooltip) {
				tooltip.destroy();
			});

		};
		
		/**
		 * returns selected filter list
		 * @returns {FilterList}
		 */		
		MultiSelectList.prototype.getSelectedFilterList = function () {
			return this._selectedFilterList;
		};

		return MultiSelectList;
	});
