define(["component/message/MessageComponent", "jquery"], function (MessageComponent, $) {
	"use strict";

	/**
	 * Generic component used to display success, error and warning messages on a normal webcenter page
	 * (this is a singleton)
	 */
	var PageMessageComponent = function () {
		this._messageModel = new MessageComponent.MessageModel();
		this._messageWidget = new MessageComponent.MessageWidget(this._messageModel, $("#globalPageMessageBar"));
	};


	/**
	 * Display a success message that autohides.
	 * @param {String} message
	 * @param {String}  [url] - URL to open
	 * @param {Boolean} [autohide] - if true, message is automatically hidden	 
	 * @param {Boolean} [tag] - tag for the message (so they can be marked obsolete later on)
	 */
	PageMessageComponent.prototype.sendSuccessMessage = function (message, url, autohide, tag) {
		if (!url){
			url = null;
		}
		var message = new MessageComponent.Message(MessageComponent.MessageType.SUCCESS, message, url, autohide, tag);
		this._messageModel.addToMessageQueue(message);
	};

	/**
	 * Display an error message that stays and if you click on it you go to the URL
	 * @param {String} message
	 * @param {String} [url] - URL to go to
	 * @param {String} [tag] - tag for the message (so they can be marked obsolete later on)
	 */
	PageMessageComponent.prototype.sendErrorMessage = function (message, url, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.ERROR, message, url, false, tag);
		this._messageModel.addToMessageQueue(message);
	};

	/**
	 * Display a warning message that stays.
	 * @param {String} message
	 * @param {Function} action - The action to execute
	 * @param {String} [tag] - tag for the message (so they can be marked obsolete later on)
	 */
	PageMessageComponent.prototype.sendWarnMessage = function (message, action, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.WARN, message, action, false, tag);
		this._messageModel.addToMessageQueue(message);
	};
    
    /**
	 * Display a info message that stays.
	 * @param {String} message
	 * @param {Function} action - The action to execute
	 * @param {String} [tag] - tag for the message (so they can be marked obsolete later on)
	 */
	PageMessageComponent.prototype.sendInfoMessage = function (message, action, autohide, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.INFO, message, action, autohide, tag);
		this._messageModel.addToMessageQueue(message);
	};
	
	/**
	 * Remove all messages with a certain tag from the Queue 
	 * @param {String} tag
	 */
	PageMessageComponent.prototype.markObsolete = function(tag){
		this._messageModel.markObsolete(tag);	
	};



	// singleton!	
	var instance = new PageMessageComponent();
	return instance;
});