define(["jquery",
        "underscore",
        "utils/Popup",
        "component/modal/Modal",
        "utils/webi18n!TextContentStrings",
        "utils/webi18n!PCMPasswordValidationPopup",
        "services/passwordValidationService",
        "utils/webcss!./PasswordValidationPopup.css"], 
	function($,
			_,
			Popup,
			ModalPopup,
			TextContentStrings,
			ValidationPopupBundle,
			passwordValidationService) {
		
		"use strict";
		
		/**
		 * View for the password validation popup
		 */
		var PasswordValidationPopupView = function() {
			
			// create submit and cancel buttons
			this._submitButton = null;
			this._cancelButton = null;
			this._passwordTextBox = null;
			this._invalidPasswordAlert = null;
			this._passwordInputHidden = null;
			this._errorAlert = null;
			
			// callbacks for submit and cancel
			this._onSubmitCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when submit or cancel button is clicked
			this._submitClickHandler = _.bind(function(event) {
				_.each(this._onSubmitCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			this._options = {};
			this._options.widthClass = "EnterPasswordPopup";
		};
		
		// This method is to provide view instance to handle password validation popup for viewer.
		PasswordValidationPopupView.prototype.getPasswordValidationPopupView = function() {
			// Create only one instance of popup.
			if (!this._popup) {
				this._constructPopup();
			}
			return this;
		};
		
		PasswordValidationPopupView.prototype._constructPopup = function() {
			// create the popup using model popup
			this._popup = new ModalPopup(this._getPopupContent(), this._options);
			this._popup.setTitle(WCXML.xmlencode(TextContentStrings.t("PCM_PASSWORD_POPUP_TITLE")));
			this._popup.setFooter(this._getFooterContent());
		};
		
		PasswordValidationPopupView.prototype._getPopupContent = function() {
			var popupContent = $('<div>');
			
			this._invalidPasswordAlert = $('<div>').addClass("alert alert-danger hidden").attr({'role': 'alert'}).text(ValidationPopupBundle.t("INVALID_PASSWORD_ENTRY_TAG")).hide();
			
			this._passwordTextBox = $('<input>').attr({'type': "password", 'name': 'PasswordValidationField', 'autofocus':'true'}).addClass("form-control");
			
			this._errorAlert = $('<div>').append($('<p>').text(ValidationPopupBundle.t("UNKNOWNEXCEPTION_TAG"))).hide();
			
			popupContent.append(
					$('<div>').append(this._invalidPasswordAlert, this._passwordTextBox),
					this._errorAlert
			);
			
			this._passwordTextBox.on('keypress', _.bind(function(event) {
				if(event.keyCode === $.ui.keyCode.ENTER) {
					this._submitButton.click();
				}
			}, this));
			
			this._passwordTextBox.on('cut copy paste', _.bind(function(event) {
				event.preventDefault();
			}, this));
			
			this._passwordTextBox.on('change keyup', _.bind(function(event) {
				this._setPasswordInputFieldDisabledOrEnabled();
				
				if(event.keyCode !== $.ui.keyCode.ENTER && !this._invalidPasswordAlert.hasClass('hidden')) {
					this._invalidPasswordAlert.addClass('hidden').hide();
				}
			}, this));
			
			return popupContent;
		};
				
		PasswordValidationPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'ActiveButtons'});
			
			this._submitButton = $('<input>').attr({'type': 'submit', 'value': WCXML.xmlencode(TextContentStrings.t("PCM_OK"))}).addClass("btn btn-primary");

			this._setPasswordInputFieldDisabledOrEnabled = function(){
				if(this._passwordTextBox.val()) {
					this._submitButton.removeClass('disabled');
					this._submitButton.prop("disabled",false);
				} else {
					this._submitButton.addClass('disabled');
					this._submitButton.attr({'disabled':'disabled'});
				}
			};
			
			this._setPasswordInputFieldDisabledOrEnabled();
			
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': WCXML.xmlencode(TextContentStrings.t("PCM_CANCEL"))}).addClass("btn btn-default");
			this._cancelButton.click(_.bind(function () {
				this._popup.close();
				this._cancelClickHandler();
			}, this));
			
			this._submitButton.click(_.bind(function() {
				var encodedPassword = WCXML.encode64(this._passwordTextBox.val());
				
				passwordValidationService.validatePassword(encodedPassword, _.bind(function(isValidationSuccessful) {
					if(isValidationSuccessful) {
						this._popup.close();
						this._submitClickHandler();
					} else {
						this._invalidPasswordAlert.removeClass('hidden').show();
					}
				}, this));
				
			}, this));
					
			footerButtons.append(this._cancelButton, this._submitButton);
			
			return footerButtons;
		};
		
		PasswordValidationPopupView.prototype.show = function () {
			this._popup.open();
		};
		
		PasswordValidationPopupView.prototype.onSubmit = function(callback) {
			this._onSubmitCallbacks.push(callback);
		};
		
		PasswordValidationPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		PasswordValidationPopupView.prototype.clearCallbacks = function() {
			this._onSubmitCallbacks = [];
			this._onCancelCallbacks = [];
		};
		
		return new PasswordValidationPopupView();
	}
);

