define(["jquery",
        "underscore",
        "web-viewer/layout/component/ModalPopup",
        "utils/ShapeUtils",
        "utils/webi18n!TextContentStrings",
        "utils/webi18n!PCMPasswordValidationPopup",
        "services/passwordValidationService",
        "web-viewer/layout/component/ViewerToolbarButton",
        "utils/webcss!./PasswordValidationPopup.css"], 
	function($,
			_,
			ModalPopup,
			ShapeUtils,
			TextContentStrings,
			ValidationPopupBundle,
			passwordValidationService,
			ViewerToolbarButton) {
		
		"use strict";
		
		/**
		 * View for the password validation popup in HTML5 viewer
		 * 
		 */
		var ViewerPasswordValidationPopupView = function() {
			// create submit and cancel buttons
			this._submitButton = null;
			this._cancelButton = null;
			this._passwordTextBox = null;
			this._invalidPasswordAlert = null;
			this._passwordInputHidden = null;
			this._errorAlert = null;
			
			// callbacks for submit and cancel
			this._onSubmitCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when submit or cancel button is clicked
			this._submitClickHandler = _.bind(function(event) {
				_.each(this._onSubmitCallbacks, _.bind(function(callback) {
					callback();
				}, this));
				this.clearCallbacks();
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
				this.clearCallbacks();
			}, this);
			
			this._options = {};
			this._options.widthClass = "EnterPasswordPopup";
			
			this._popup = null;
			
		};
		
		/**
		 * This method is to provide view instance to handle password validation popup for viewer.
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		ViewerPasswordValidationPopupView.prototype.getViewerPasswordValidationPopupView = function(f) {
			// If already exists, then don't create it.
				this._popup = null;
				this.f = f;
				this._constructPopup();
			return this;
		};
		
		ViewerPasswordValidationPopupView.prototype._constructPopup = function() {
			this._popup = new ModalPopup(this.f, "ViewerPasswordValidationPopup", this.f.getContext(), false, 'auto');
			this._popup.setRemoveFromDomAfterUsage(true);
			this._popup.setHeader($('<span>').text(WCXML.xmlencode(TextContentStrings.t("PCM_PASSWORD_POPUP_TITLE"))));
			
			this._submitAction = _.bind(function (event) {
				var encodedPassword = WCXML.encode64(this._passwordTextBox.val());
				
				passwordValidationService.validatePassword(encodedPassword, _.bind(function(isValidationSuccessful) {
					if(isValidationSuccessful) {
						this._popup.hide();
						this._submitClickHandler();
					} else {
						this._invalidPasswordAlert.removeClass('hidden').show();
						this._popup.recalculateSize(300);
					}
				}, this));
			}, this);
			
			this._popup.setOkAction(this._submitAction);
			this._popup.setOkBtnText(WCXML.xmlencode(TextContentStrings.t("PCM_SAVE")));
			this._popup.enableOkBtn(false);
			
			this._popup.setCancelAction(_.bind(function () {
				this._cancelClickHandler();
			}, this));
			
			this._popup.setContent(this._getPopupContent());
		};
		
		ViewerPasswordValidationPopupView.prototype._getPopupContent = function() {
			var popupContent = $('<div>').css({'padding': '10px', 'max-width': '300px'});
			
			this._invalidPasswordAlert = $('<div>').addClass("alert alert-danger hidden").attr({'role': 'alert'}).text(ValidationPopupBundle.t("INVALID_PASSWORD_ENTRY_TAG")).css({'margin-bottom': '5px', 'padding': '5px'}).hide();
			
			this._passwordTextBox = $('<input>').attr({'type': "password", 'name': 'PasswordValidationField', 'autofocus':'true'}).addClass("form-control");
			
			this._passwordTextBox.on('keypress', _.bind(function(event) {
				if(event.keyCode === $.ui.keyCode.ENTER) {
					this._submitAction();
					this._popup.enableOkBtn(false);
				}
			}, this));
			
			this._passwordTextBox.on('cut copy paste', _.bind(function(event) {
				event.preventDefault();
			}, this));
			
			this._passwordTextBox.on('change keyup', _.bind(function(event) {
				if(this._passwordTextBox.val()) {
					this._popup.enableOkBtn(true);
				} else {
					this._popup.enableOkBtn(false);
				}
				
				if(event.keyCode !== $.ui.keyCode.ENTER && !this._invalidPasswordAlert.hasClass('hidden')) {
					this._invalidPasswordAlert.addClass('hidden').hide();
					this._popup.recalculateSize(300);
				}
			}, this));
			
			
			popupContent.append(
					$('<div>').append(this._invalidPasswordAlert, this._passwordTextBox),
					$('<div>').addClass("PasswordUnknownErrorAlert").append($('<p>').text(ValidationPopupBundle.t("UNKNOWNEXCEPTION_TAG"))).hide()
			);
			
			return popupContent;
		};
		
		
		/**
		 * Show the popup
		 */
		ViewerPasswordValidationPopupView.prototype.show = function () {
			this._popup.show();
			this._popup.recalculateSize(300);
		};
		
		ViewerPasswordValidationPopupView.prototype.onSubmit = function(callback) {
			this._onSubmitCallbacks.push(callback);
		};
		
		ViewerPasswordValidationPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		ViewerPasswordValidationPopupView.prototype.clearCallbacks = function() {
			this._onSubmitCallbacks = [];
			this._onCancelCallbacks = [];
		};
		
		return new ViewerPasswordValidationPopupView();
	}
);

