define(["utils/Observer", "jquery", "underscore"], function (Observer, $, _) {

	"use strict";

	/**
	 * This class displays a progress bar. By default the progress goes from 0 to 1, but it can be set with the maxProgress property.
	 * It is also possible to give a array of sectionIds. This will lead to one progress bar that is divide into _.size(sectionIds) sections that can each have
	 * a progress and can each fail/complete.
	 * @class
	 * @exports ProgressBar
	 * @mixes Observer
	 * @param {Object} configuration - configuration object {
	 	[observable <Obervable>] - observable that emits progress events,
		[cssClass <String>] - class that should be added to the progressbar
		[maxProgress <String> = 1] - Number that indicates what 'completed' progress means
		[sections = <Array.<Object>>] - Array of ids, which are sections in the progress bar
	 }
	 **/
	var ProgressBar = function (configuration) {
		Observer.mixin(this);

		configuration = configuration || {};

		/**
		 * @member {JQueryObject}
		 * @private
		 */
		this._element = $("<div/>").addClass("progress");

		/**
		 * @member {Object.<JQueryObject>}
		 * @private
		 */
		this._progressBars = {};

		/** 
		 * @member {Array.<String>} ids of the sections in this progress bar
		 * @private
		 */
		this._sectionIds = [];
		
		/**
		 * @member {Object.<String>} map from section id to title
		 * @private
		 */
		this._sectionTitles = {};
		
		if (configuration.sections) {
			this._sectionIds = _.map(configuration.sections, function (section) {
				return section.id;
			});
			this._sectionTitles = _.object(this._sectionIds, _.map(configuration.sections, function (section) {
				return section.title || "";
			}));
		} else {
			this._sectionIds = ["myProgressBar"];
			this._sectionTitles = {
				myProgressBar: ""
			};
		}

		/**
		 * @member {Number} the amount of sections
		 * @private
		 */
		this._sectionsAmount = this._sectionIds.length;

		// create a progress bar section for each id
		_.each(this._sectionIds, function (id) {
			this._createProgressBarWithId(id, configuration.cssClass);
		}, this);

		/**
		 * @member {Number} The total amount of progress possible. defaults to 1
		 * @private
		 */
		this._maxProgress = configuration.maxProgress || 1;

		/**
		 * @member {Boolean} Should the percentage be shown in the progress bar?
		 * @private
		 */
		this._showPercentage = typeof configuration.showPercentage === "undefined" ? true : configuration.showPercentage;

		if (configuration.observable) {

			/**
			 * @member {Observable} if an observable is defined, listen to the events sent by the observable
			 * @private
			 */
			this._observable = configuration.observable;

			this._initializeListeners();
		}

	};

	/**
	 * initializes listeners
	 * events are: progress.[start | changed | completed | failed]
	 */
	ProgressBar.prototype._initializeListeners = function () {

		this._observe(this._observable, "progress.start", this.startProgress);
		this._observe(this._observable, "progress.changed", this.progressChanged);
		this._observe(this._observable, "progress.completed", this.stopProgress);
		this._observe(this._observable, "progress.failed", this.progressFailed);

	};

	/**
	 * Sets the progress of each section in the progress bar to 0 and removes classes
	 */
	ProgressBar.prototype.startProgress = function () {
		this._setProgress(0);
		this.removeClass("progress-bar-success");
		this.removeClass("progress-bar-danger");
	};

	/**
	 * sets the progress of a section. If no id is defined, the progress is set for all sections.
	 * The progress set will be divided by maxProgress to get the percentage of progress completed
	 * @param {Number} progress - the amount of progress to reach maxProgress. 
	 * @param {String} [id] - the id of the section
	 */
	ProgressBar.prototype.progressChanged = function (progress, id) {
		progress = progress / this._maxProgress;
		this._setProgress(progress, id);
	};

	/**
	 * Sets the progress of the section with id @param id, or of all sections if id is not defined, to 1 and adss the progress-bar-success class.
	 * @param {String} [id]
	 */
	ProgressBar.prototype.stopProgress = function (id) {
		this._setProgress(1, id);
		this.addClass("progress-bar-success", id);
	};

	/**
	 * Adds the progress-bar-danger class to the progress bar of the failed section or to all sections if id is not defined.
	 * @param {String} [id]
	 */
	ProgressBar.prototype.progressFailed = function (id) {
		this.addClass("progress-bar-danger", id);
	};

	/**
	 * sets the progress of the section with id @param id or to all sections if it isn't defined. progress should be between 0 and 1
	 * @param {Number} progress
	 * @param {String} [id]
	 */
	ProgressBar.prototype._setProgress = function (progress, id) {
		var ids = id ? [id] : this._sectionIds;
		if (progress > 1) {
			progress = 1;
		} else if (progress < 0) {
			progress = 0;
		}
		_.each(ids, function (id) {
			this._progressBars[id].css({
				width: (progress * 100) / this._sectionsAmount + "%"
			});

			if (this._showPercentage) {
				var text = this._sectionTitles[id];
				if (this._sectionTitles[id]) {
					text += " (";
				}
				text += Math.round(progress * 100) + "%";
				if (this._sectionTitles[id]) {
					text += ")";
				}
				this._progressBars[id].text(text);
			}
		}, this);


	};

	/**
	 * sets the maximum progress for the progress bar
	 * @param {Number} maxProgress
	 */
	ProgressBar.prototype.setMaxProgress = function (maxProgress) {
		this._maxProgress = maxProgress;
	};


	/** 
	 * gets the jquery element for this progress bar
	 * @return {JQueryObject}
	 */
	ProgressBar.prototype.getElement = function () {
		return this._element;
	};

	/** 
	 * adds a class to the progress bar with id @param id or to all if no id was specified
	 * @param {String} clazz
	 * @param {String} [id]
	 */
	ProgressBar.prototype.addClass = function (clazz, id) {
		var ids = id ? [id] : this._sectionIds;
		_.each(ids, function (id) {
			this._progressBars[id].addClass(clazz);
		}, this);
	};

	/**
	 * removes a class from the progress bar with id @param id or from all if no id was specified
	 * @param {String} clazz
	 * @param {String} [id]
	 */
	ProgressBar.prototype.removeClass = function (clazz, id) {
		var ids = id ? [id] : this._sectionIds;
		_.each(ids, function (id) {
			this._progressBars[id].removeClass(clazz);
		}, this);
	};

	/**
	 * creates a progress bar and appends it to the jquery element
	 * @param {String} id
	 * @param {String} [cssClass]
	 */
	ProgressBar.prototype._createProgressBarWithId = function (id, cssClass) {
		this._progressBars[id] = $("<div/>")
			.addClass("progress-bar progress")
			.addClass(cssClass || "")
			.attr("role", "progressbar")
			.text(this._sectionTitles[id])
			.appendTo(this._element);
	};



	return ProgressBar;
});
