define([
        "jquery",
        "underscore",
        "utils/webi18n!SavedSearchDialog",
		"wcr/webcenter_xml",
		"component/smartnames/savedsearchcontext/SavedSearchContextInfo",
		"component/modal/Modal"
   ],function($,
		   _,
		   i18n,
		   WCXML,
		   SavedSearchContextInfo,
		   ModalPopup){
	 
	"use strict";
	
	/**
	 * The constructor of SavedSearchDialog
	 * @param {Context} context 
	 *                          must contain projectID and savedSearchName attributes                        
	 */
	var SavedSearchDialog = function(context){
		//if context isn't defined savedSearchName and projectID MUST BE SET using setters
		if(context){
			/**
			* @member {String} The name of the search that will be used in this dialog
			* @private                  
			*/
			this._searchName = context.getAttribute("savedSearchName");

			/**
			* @member {String} The ID of the project from which this saved search will be executed
			* @private                     
			*/
			this._currentProjectID = context.getAttribute("projectID");
		}
		
		
		/**
		* @member {String} This is the title that the dialog will use, this value should be overridden in the implementing class
		* @private    
		*/
		this._dialogTitle="";
		
		/**
		* @member{ Number (0 or 1)}should be overridden if needed
		* @private    
		*/
		this._showSimpleGUI = 1;
		
		/**
		* @member {String} The HPSectionID of the reruncontainer on the page, when executed in a dialog the  default value is 00001_9000000000
		* @private    
		*/
		this._searchDialogHPSectionID = '00001_9000000000';
		
		/**
		* @member {Array.Button} A Cancel button is a standard button which is always available
		* @private    
		*/
		this._buttonObj = {};
		
	    this._buttonObj[i18n.t("CANCEL")]= function(){
	    	this.popup.close();
	    };
		
		
	};
	
	/**
	 * Sets the buttons for  this saved search dialog
	 * @param {Array.Button} buttonObj the buttons with their listener functions
	 */
	SavedSearchDialog.prototype.setButtonObj = function(buttonObj){
		this._buttonObj = buttonObj;
	};
	
	/**
	 * Sets the current project ID
	 * @param {String} projectID
	 */
	SavedSearchDialog.prototype.setProjectID = function(projectID){
		this._currentProjectID = projectID;	
	};
	
	/**
	 * Sets the savedSearchContext 
	 * @param {SavedSearchContextInfo} savedSearchContext
	 */
	SavedSearchDialog.prototype.setSavedSearchContext = function(savedSearchContext){
		this._savedSearchContext = savedSearchContext;	
	};
	
	/**
	 * Sets the title of the dialog
	 * @param {String} title
	 */
	SavedSearchDialog.prototype.setTitle = function(title){
		this._dialogTitle = title;	
	};
	
	/**
	 * Sets the name of the saved search
	 * @param {String} searchName
	 */
	SavedSearchDialog.prototype.setSearchName = function(searchName){
		this._searchName = searchName;
	};
		
	/**
	 * Get the name of the saved search that is executed in this dialog
	 * @returns {String} The name of the saved search
	 */
	SavedSearchDialog.prototype.getSearchName = function(){
		return this._searchName;
	};
		
	/**
	 * Once the ajax call to perform the saved search is finished, 
	 * this method will change the html code of the SavedSearchDialog to show the results
	 * @param returnHTML
	 * @param handle
	 */
	SavedSearchDialog.prototype.onSuccess = function(returnHTML,handle) {
		$("#savedSearchDialog").html(returnHTML);
				
		require(["utils/init"], function(init) {
			// Load and initialize require javascript for the savedsearch
			init.doinit($("#savedSearchDialog"), function(){
				// When loaded... do nothing...
			});
		});
					
		// make sure all abreviated descriptions appearing on mouse hovering in a small pop-up window 
		// are not hidden behind the search dialog (the jQuery search dialog has zIndex = 1002)
		var popitMenu = document.getElementById('popitmenu');
		if (popitMenu){
			popitMenu.style.zIndex = 1200;
		}
	};
	
	/**
	 * Executes the saved search
	 * @private
	 * @param {String} savedSearchContextUrlPart           
	 * @param {XML} savedSearchContextDynamicSmartNames 
	 */
	SavedSearchDialog.prototype._doSavedSearchAjaxCall = function(savedSearchContextUrlPart,savedSearchContextDynamicSmartNames){
		var url = window.webcenterConfig.jspBaseUrl + 'dosavedsearch.jsp?searchName=' + WCXML.urlencode(this.getSearchName()) + '&showSimpleGUI='+this._showSimpleGUI+
				'&twoColumnLayout=0&isInDialog=1&sectionID=' + this._searchDialogHPSectionID+ savedSearchContextUrlPart +'&ajax=1'; 
		var self = this;
		//use the webcenter_ajax function to make the ajax call 
		ajax({
			url: url,				
			type: "POST",
			data: "text",
			content: $.param({staticSmartNames: savedSearchContextDynamicSmartNames}),
			contentType: "application/x-www-form-urlencoded",
			synchronous: false,
			timeout: 30000,
			onSuccess: function(returnHTML,objHdl2) {					
				self.onSuccess(returnHTML,objHdl2);
			},		
			onError: function(xhr,ajaxOptions,thrownError){
				window.console.error("An error occurred when executing a saved search");
			},				
				objHdl: this
			});
	};
	
	/**
	 * This method will open the SavedSearchDialog
	 */
	SavedSearchDialog.prototype.openDialog = function(){
		if (!document.getElementById('savedSearchDialog')) {
			var $div = $('<div>').appendTo('body');
			$div.attr('id', 'savedSearchDialog');			
			$div.addClass("wcr savedsearchdialog fl-scrolls-body");
		}

		// Create the modal popup
		if (!this.popup) {
			var options = {};
			options.customClass = "savedsearch_modalpopup";
			options.onClose = function() {
				$("#savedSearchDialog").empty();
			};

			this.popup = new ModalPopup($("#savedSearchDialog"), options);
		}

		this._footerButtons = $('<div>').addClass("btn-group").attr({'role' : 'group'});

		// Create modal footer buttons from button objects
		var keys = _.keys(this._buttonObj);
		_.each(keys, function(key, index) {
			var buttonClass = (index === keys.length - 1) ? "btn btn-primary" : "btn btn-default add-spacing-right";
			var button = $('<input>').attr({'type' : 'button', 'value' : key}).addClass(buttonClass);
			button.click(_.bind(this._buttonObj[key], this));

			this._footerButtons.append(button);
		}.bind(this));

		// Set the buttons to popup footer
		this.popup.setFooter(this._footerButtons);

		// Set the popup title
		this.popup.setTitle(this._dialogTitle);

		var htmlcode = '<span class="darktext_greenlink_11">' + i18n.t("LOADINGSEARCH") + '</span>';
		$("#savedSearchDialog").html(htmlcode);		
		
		// Open the modal dialog
		this.popup.open();
		
		var savedSearchContextUrlPart = "";
		var savedSearchContextDynamicSmartNames = "";
		var savedSearchContextInfo = this._savedSearchContext;

		
		if(savedSearchContextInfo) {
			savedSearchContextUrlPart = savedSearchContextInfo.getContextUrlPart();
			var self = this;
			savedSearchContextInfo.getStaticSmartNamesXml(function(xml){
				savedSearchContextDynamicSmartNames = xml; 
				self._doSavedSearchAjaxCall.call(self,savedSearchContextUrlPart,savedSearchContextDynamicSmartNames);
			});
		}else{
			window.console.warn("Could not find context for Saved Search with SmartNames...");
			this._doSavedSearchAjaxCall.call(this,"","");
		}
		
	};

	return SavedSearchDialog;
});