define(["utils/Observable",
		"utils/ShapeUtils",
		"underscore",
		"jquery",
		"jqueryUI"
	],
	function (
		Observable,
		ShapeUtils,
		_,
		$) {

		"use strict";

		/**
		 * A Scrollbar uses a JQueryUI Slider to display a scrollbar for a certain "world"
		 * visibleRect is the part of the world (worldRect) that is visible.
		 *
		 * The scrollbar is observable and sends out "start", change" and "stop" events. Each event has a isInteractive as first parameter
		 * for the change event, the translation offset (ShapeUtils.Size) is the second argument
		 * @exports Scrollbar
		 * @class
		 *
		 * @param {"vertical"|"horizontal"} orientation - direction of the scrollbar
		 * @param {ShapeUtils.Rect} worldRect - the dimensions of the world
		 * @param {ShapeUtils.Rect} [visibleRect] - part of the world that is visible
		 */
		var Scrollbar = function (orientation, worldRect, visibleRect) {
			Observable.mixin(this, "scrollbar");

			/**
			 * @Member {"vertical"|"horizontal"}  orientation of the scrollbar
			 * @private
			 */
			this._orientation = orientation;

			/**
			 * @member {ShapeUtils.Rect} the rect of the world
			 * @private
			 */
			this._worldRect = worldRect;


			/**
			 * @member {Boolean} if true, the scrollbar position is changed from within the framework,
			 * if false, the change is done interactively by a user
			 * @private
			 */
			this._interactive = false;

			// if visibleRect is not defined, default to the worldRect
			if (!visibleRect) {
				visibleRect = worldRect.copy();
			}

			/**
			 * @member {ShapeUtils.Rect} the part of the world that is visible
			 * @private
			 */
			this._visibleRect = visibleRect;

			/**
			 * @member {Number} this is used to calculate the correct translation when the user moves the scrollbar
			 * @private
			 */
			this._previousValue = null;



			/**
			 * @member {JQueryObj} scrollbar element
			 */
			this.element = $("<div>").addClass("scrollbar").slider({
				orientation: this._orientation,
				range: false
			});


			// listen to events of the JQueryUI Slider
			this.element.on("slidestart", _.bind(function (event, ui) {
				event.stopPropagation(); // we don't want to trigger events on the scene
				this._interactive = true;
				this._previousValue = ui.value;
				this._emit("start", this.isChangedInteractively());
			}, this));
			this.element.on("slide", _.bind(this._sliderChanged, this));
			this.element.on("slidestop", _.bind(function (event) {
				event.stopPropagation(); // we don't want to trigger events on the scene
				var changedInteractively = this.isChangedInteractively();
				this._interactive = false;
				this._emit("stop", changedInteractively);
			}, this));

			// initialize the scrollbar
			this.render();
		};


		/**
		 * Set the dimensions of both the world and te visible part of the world
		 * @param {ShapeUtils.Rect} worldRect - the dimensions of the world
		 * @param {ShapeUtils.Rect} [visibleRect] - part of the world that is visible
		 */
		Scrollbar.prototype.setWorld = function (worldRect, visibleRect) {
			if (!this._interactive) {
				this._worldRect = worldRect;
				this._visibleRect = visibleRect;
				this.render();
			}
		};

		/**
		 * Set the dimensions of the world
		 * @param {ShapeUtils.Rect} worldRect - the dimensions of the world
		 */
		Scrollbar.prototype.setWorldRect = function (worldRect) {
			if (!this._interactive) {
				this._worldRect = worldRect;
				this.render();
			}
		};

		/**
		 * Set the visible part of the world
		 * @param {ShapeUtils.Rect} [visibleRect] - part of the world that is visible
		 */
		Scrollbar.prototype.setVisibleRect = function (visibleRect) {
			if (!this._interactive) {
				this._visibleRect = visibleRect;
				this.render();
			}
		};


		/**
		 * Calculate the size of the scroll bar and render it to project the visibleRect on the scrollbar
		 */
		Scrollbar.prototype.render = function () {
			var boundingMin, boundingMax, visibleMin, visibleMax, scrollbarSize;
			var total, range, rangeCenter, proportion, handleSize;
			var disabled = false;

			if (this._orientation === "horizontal") {
				// horizontal Scrollbar
				boundingMin = this._worldRect.left();
				boundingMax = this._worldRect.right();
				visibleMin = this._visibleRect.left();
				visibleMax = this._visibleRect.right();
				scrollbarSize = this.element.width();

				boundingMin = Math.min(boundingMin, visibleMin);
				boundingMax = Math.max(boundingMax, visibleMax);
				total = Math.abs(boundingMax - boundingMin);
				range = Math.abs(visibleMax - visibleMin);
				rangeCenter = visibleMin + range / 2;

				proportion = range / total;
				handleSize = Math.floor(proportion * scrollbarSize);

				disabled = (visibleMin <= boundingMin && visibleMax >= boundingMax);




				this.element.find(".ui-slider-handle").css({
					width: handleSize,
					"margin-left": Math.floor(-handleSize / 2)
				});

				this.element.slider({ // avoid getting JQueryUI error of non initialized slider
					orientation: this._orientation,
					range: false
				}).slider("option", {
					min: boundingMin,
					max: boundingMax,
					value: rangeCenter,
					disabled: disabled
				});
			} else if (this._orientation === "vertical") {
				//vertical Scrollbar

				// JQueryUI slider vertical goes from bottom to top
				// min should alway be smaller than max, so we turn around top and bottom 
				// we work in an "inverted" space
				// by using negative values, we can be sure that min is always smaller than max
				// we work in an inverted space
				boundingMin = this._worldRect.bottom();
				boundingMax = this._worldRect.top();
				visibleMin = this._visibleRect.bottom();
				visibleMax = this._visibleRect.top();
				scrollbarSize = this.element.height();

				boundingMin = Math.max(boundingMin, visibleMin);
				boundingMax = Math.min(boundingMax, visibleMax);
				total = Math.abs(boundingMin - boundingMax);
				range = Math.abs(visibleMin - visibleMax);
				rangeCenter = visibleMax + range / 2;

				proportion = range / total;
				handleSize = Math.ceil(proportion * scrollbarSize);

				disabled = (visibleMax <= boundingMax && visibleMin >= boundingMin);

				this.element.find(".ui-slider-handle").css({
					height: handleSize,
					"margin-bottom": Math.ceil(-handleSize / 2)
				});

				// by using negative values, we can be sure that min is always smaller than max				

				this.element.slider({ // avoid getting JQueryUI error of non initialized slider
					orientation: this._orientation,
					range: false
				}).slider("option", {
					min: -boundingMin,
					max: -boundingMax,
					value: -rangeCenter,
					disabled: disabled
				});
			}
		};

		/**
		 * Is the scrollbar being changed interactively?
		 * @returns {Boolean}
		 */
		Scrollbar.prototype.isChangedInteractively = function () {
			return this._interactive;
		};

		/**
		 * Handle slider canged events
		 *  (See JQueryUI Slider documentation)
		 *
		 * @param {Event} event
		 * @param {Object} ui - object with handle, value and values
		 *
		 */
		Scrollbar.prototype._sliderChanged = function (event, ui) {
			event.stopPropagation(); // we don't want to trigger events on the scene
			if (this._previousValue === null) {
				this._previousValue = ui.value;
			} else {
				var translation = this._previousValue - ui.value;
				var x = 0;
				var y = 0;

				if (this._orientation === "vertical") {
					// we working with vertical scrollbars, we use inverted values so JQueryUISlider works correctly
					// make sure we send out the correct transation
					y = -translation;
				} else {
					x = translation;
				}

				this._previousValue = ui.value;

				if (translation !== 0) {
					// send out event
					this._emit("change", this.isChangedInteractively(), new ShapeUtils.Size(x, y));
				}
			}
		};


		return Scrollbar;
	});