/**
 * Utils for Parameter Field implementations
 */
define([
	        "underscore", 
	        "utils/webi18n!SearchInputs",
	        
	        // Dependencies for type checking
	        "./dataobjects/FieldProperties",
	        "./dataobjects/TableDisplaySettings",
	        "./dataobjects/ExternalSearchSettings",
	        
	        // Smartname related dependencies
	        "component/smartnames/encoding/smartnameencoding",
	        
	        // Other utility imports
	        "utils/ConvertUtils"
        ], function(
    		_, 
    		i18n,
    		
    		 // Dependencies for type checking
    		FieldProperties,
    		TableDisplaySettings,
    		ExternalSearchSettings,
    		
    		// SmartName related dependencies
    		SmartNameEncoding,
    		
    		// Other utility imports
    		ConvertUtils
        ){
	
	
	
	//
	// This class should contain only functions which are useful for more than 80% of the fields...
	//
	// Do not pollute with functions which are specific for a small amount of fields...
	//
	
	var Utils = {};
	
	//
	//
	// -- Mostly used by Design Panel Implementations --
	//
	//
	
	/**
	 * Checks a string for given characters (e.g. to check if the string contains special characters)
	 * 
	 * @param {String} str - the string to check for specific characters
	 * @param {char[]} specialCharsArr - the array with the characters to check for
	 * @returns {Boolean} true if the string contains one of the characters...
	 */
	Utils.hasCharsInStr = function(str, charsArr){
		if(str != null) {
			for(var i=0; i < charsArr.length; i++) {
				if(str.indexOf(charsArr[i]) != -1) {
					return true;
				}
			}
		}
		return false;
	};
	
	/**
	 * Checks a SmartField for given characters (not in the SmartName names)
	 * (e.g. to check if the SmartField contains special characters)
	 * 
	 * @param {SmartField} smartfield - the SmartField to check for specific characters...
	 * @param {char[]} specialCharsArr - the array with the characters to check for
	 * @returns {Boolean} true if the text in the SmartField contains one of the characters...
	 */
	Utils.hasCharsInSmartField = function(smartfield, charsArr){
		// We only want to check for special characters in the text parts, not as part of the name of SmartNames...
		
		// Split the SmartName value in Text & SmartName pieces
		var splitInput = SmartNameEncoding.SAVEDSEARCH.decodeSmartNameString(smartfield.getValue());
		
		// Then loop through all the pieces, but ignore the SmartName pieces...
		for (var i = 0; i < splitInput.length; i++) {
			var part = splitInput[i];
			if(part.isTextPart()) {
				var text = part.getAsText();
				if(Utils.hasCharsInStr(text, charsArr)){
					return true;// contains special characters...
				}
			}
		}
		return false;
	};
	
	/**
	 * Checks if a SmartField contains a SmartName...
	 * 
	 * @param {SmartField} smartfield - the field to check...
	 * @returns {Boolean} true if the field contains a SmartName
	 */
	Utils.hasSmartNameInSmartField = function(smartfield) {
		return SmartNameEncoding.SAVEDSEARCH.containsSmartName(smartfield.getValue());
	};
	
	/**
	 * Convert an array of selected values to a lucene query
	 */
	Utils.arrayOfValuesToLuceneQuery = function(arr) {
		// 1) Escaping
		//  -> Escape existing quotes with \"    (note: only works if special characters are enabled!)
		//  -> Put quotes around the values (= escapes all other special characters + handles spaces)
		var escapedarr = _.map(arr, function(val) {return "\""+val.replace(/"/g, "\\\"")+"\"";});
		
		// 2) Join and return
		return escapedarr.join(" || ");
	};
	
	/**
	 * Convert a lucene query to an array of selected values (e.g. used in Asset Browser)
	 * 
	 * NOTE: this method makes a guess! The query needs to have a specific format!
	 *  => everything that is returned by "arrayOfValuesToLuceneQuery" should work! (and more)
	 * 
	 * But e.g. if you start using stars, this does not work. (and it is not intended to work)
	 */
	Utils.luceneQueryToArrayOfValues = function(query) {
		// Empty query
		if(query.trim().length === 0) {
			return [];
		}
		
		// Guess join logic
		var joinCharacters = " OR ";
		if(query.indexOf("||") !== -1) {
			joinCharacters = "||";// Could also have additional spaces (we will trim them later)
		}
		
		// Split (Note: this makes the assumption that the join characters do not occur in the values!)
		var values = query.split(joinCharacters);
		
		// Trim and remove start and ending quotes
		values = _.map(values, _.bind(function(val) {
			// First, trim the value
			val = val.trim();
			// If it is surrounded by qoutes... remove them and unescape the qoutes
			if(val.indexOf("\"") === 0 && val.lastIndexOf("\"") === val.length-1) {
				val = val.substring(1, val.length-1);
				val = val.replace(/\\"/g, "\"");
			}
			return val;
		}, this));
		
		return values;
	};
	
	//
	//
	// -- Mostly used by DataRenderers --
	//
	//
	
	/**
	 * Get the translated unit...
	 * 
	 * Fetch the localized unit variant from i18n or just use the unitLangKey code as a fallback (in case the language key doesn't match any label).
	 * 
	 * @returns {String} the translated unit
	 */
	Utils.getTranslatedUnit = function(unitKey) {
		// "SEARCHFIELD_UNIT_" = unit key prefix that is used to retrieve the correct language key 
		// (look at wcstrings_en.xml::<application name="SearchInputs">)
		return i18n.tFallback('SEARCHFIELD_UNIT_'+unitKey, unitKey) || "";
	};
	
	/**
	 * Get specific localized label from the SearchInputs bundle
	 * 
	 * @returns {String} the localized label
	 */
	Utils.getLocalizedLabel = function(langKey) {
		// (look at wcstrings_en.xml::<application name="SearchInputs">)
		return i18n.tFallback(langKey, langKey) || "";
	};
	
	/**
	 * Formats a number...
	 * @param {any} number - a number or a string representing a number
	 * @param {int} decimals - [OPTIONAL] the number of decimals we want
	 * @returns {String} the formatted number
	 */
	Utils.formatNumber = function(number, decimals) {
		if(decimals===undefined) {
			decimals = 3;
		}
		
		//if no decimals return whole number only
		if (Number(number).toFixed(decimals) % 1 === 0) {
			return Math.floor(number).toString();
		} else {
			return Number(number).toFixed(decimals);	
		}
	};
	
	/**
	 * Formats a number with units...
	 * @param {String} number - a number or a string representing a number
	 * @param {String} user_unitpref - imperial/metric; user's effective Units preference
	 * 
	 * @returns {String} the formatted number
	 */
	Utils.formatNumberWithUnits = function(number, user_unitpref) {
		var newValue = number;
		if (number != null && number.length > 0 && !isNaN(number)) {
			// convert the value to a number with the appropriate decimals magnitude
			if (Math.abs(number) > 1.0) {
				if (user_unitpref === "metric") {
					newValue = Number(number).toFixed(2);
				} else {
					newValue = Number(number).toFixed(3);
				}
			} else {
				// figure out what the precision is
				var dpPos = number.indexOf('.');
				if (dpPos >= 0) {
					dpPos++;
					var num0s = 0;
					while ((dpPos < number.length) && (number.charAt(dpPos) == '0')) {
						num0s++;
						dpPos++;
					}
					var numDP = num0s + 4;
					if (numDP > 10) {
						numDP = 10; //10 is the maximum DP size we allow
					}
					newValue = Number(number).toFixed(numDP);	
				};
			};
			
			//use whole numbers if possible > we don't want to show trailing .000s
			if (newValue % 1 === 0) {
				newValue = Math.floor(newValue).toString();
			}; 
		}
		
		return newValue;
	};
	
	/**
	 * Used to process a user field value
	 * 
	 * NOTE: Metric/Imperial unit conversion is done (for numeric fields)
	 * NOTE: Localization of unit types is done (for numeric fields)
	 * 
	 * @param {String} value - the value to display
	 * @param {TableDisplaySettings} tableDisplaySettings - the table display settings object
	 * @param {FieldProperties} fieldProperties - a fieldProperties object
	 * @param {ExternalSearchSettings} searchSettings - search related WCR settings container object
	 */
	Utils.getDisplayValue = function (value, tableDisplaySettings, fieldProperties, searchSettings) {
		//get the basic properties
		var dataType = fieldProperties.getDataType();
		var numeric = fieldProperties.isNumeric();
		
		//for empty value > make sure we keep it empty
		if (value === null || value.length === 0) {
			return "";
		}
		
		//------------------------------------
		//DATE
		if (dataType === "date" && !isNaN(value)) {
			if (Number(value) === 0) {
				return "";
			}
			
			//get the effective preferences the drive how to render Date value
			var dateRenderType = fieldProperties.displayConfig.getValue("date_render_type");
			if (!dateRenderType) {
				throw new Error("Missing 'date_render_type' display config parameter. Cannot render Date value properly.");
			}
			
			//now process the Date numeric value and format it as needed
			return Utils.getDateDisplayValue(Number(value), dateRenderType);
		}
		//------------------------------------
		//NUMERIC 
		else if (numeric && !isNaN(value)) {
			// We have a number, format the number and add the unit information...
			//get the Numeric-value related properties
			var decimals = fieldProperties.getDecimals();
			var unit = fieldProperties.displayUnit;
			var unitConversionFactor = fieldProperties.unitConversionFactor;
			if(unitConversionFactor===null || unitConversionFactor===undefined) {
				unitConversionFactor = 1;
			}
			var showUnits = tableDisplaySettings.showUnitsInValue;
			var userUnitPref = searchSettings.getUserUnitsPreference();
			var floatDecimalsMode = fieldProperties.displayConfig.getValue("float_decimals_mode");

			//format the numeric value
			return Utils.getNumericDisplayValue(value, decimals, showUnits, userUnitPref, unit, unitConversionFactor, floatDecimalsMode);
		}
		//------------------------------------------
		//STRING
		else {
			//no processing on the value required
			return value;
		};
	};
	
	/**
	 * Formats a date-data representing numeric value into a human-friendly string
	 * 
	 * @param {Number} value : numeric value representing the Timestamp of the targe Date
	 * @param {String} dateRenderType : code of what Date-readout type to use; supported values > "date", "date_time" 
	 * @returns {String}
	 */
	Utils.getDateDisplayValue = function(value, dateRenderType) {
		var dateFormattedVal = "";
		if (dateRenderType === "date") {
			dateFormattedVal = ConvertUtils.getLangFormattedDate(i18n, new Date(Number(value))); // DATE only
		} else if (dateRenderType === "date_time") {
			dateFormattedVal = ConvertUtils.getLangFormattedDateAndTime(i18n, new Date(Number(value))); // DATE + TIME readout
		} else {
			throw new Error("Unknown Date-type value rendering format was supplied. Value: " + dateRenderType);
		}
		
		return dateFormattedVal;
	};
	
	/**
	 * Formats a numeric-data value for rendering on a page
	 * 
	 * @param {any} value : string or number value to be processed
	 * @param {Number} decimals : how many decimal points to round down to (for float values) 
	 * @param {Boolean} showUnits : do render unit information along with the value 
	 * @param {String} userUntiPref : which unit system does the User use? Supported values: "metric"/"imperial" 
	 * @param {String} unitCode : code of the unit to accompany the value; is used to translate the unit into a localized value for rendering
	 * @param {number} unitConversionFactor : numeric conversion factor to apply to the value if needed; 1 if no conversion is needed
	 * @param {String} floatDecimalsMode : how to handle decimals for Float values
	 * @returns {String}
	 */
	Utils.getNumericDisplayValue = function(value, decimals, showUnits, userUntiPref, unitCode, unitConversionFactor, floatDecimalsMode) {
		var numberStr = value;
		var noDecimalsRoundoff = floatDecimalsMode && floatDecimalsMode === 'no_roundoff';
		
		//format the number with the required decimal precision
		if (unitCode) {
			// Do unit conversion
			numberStr = Utils.formatNumberWithUnits((Number(value) * unitConversionFactor).toString(), userUntiPref);
		} else if (!noDecimalsRoundoff) {
			numberStr = Utils.formatNumber(value, decimals);
		}
		
		if (unitCode && showUnits) {
			//append units
			var localizedUnitVal = Utils.getTranslatedUnit(unitCode);
					
			if (localizedUnitVal) {
				//real units (mm/in, etc.) should have a space between the value and the unit label
				//'abstract' units (s.a. %) have no space!!!
				if (localizedUnitVal !== '%') {
					numberStr = numberStr + " ";
				}
				numberStr = numberStr + localizedUnitVal;
			};
		}
		
		return numberStr;
	};
	
	/**
	 * Inspects a passed 'display string value' and if its too long generates an "old school" WebCenter cutoff Jquery DOM element
	 * 
	 * @param {String} displayValue - the original string value to be inspected
	 * @param {JQuery DOM Element} targetContainer - the DOM element to append the generated HTML into; JQuery object expected 
	 */
	Utils.truncateTextVal = function(displayValue, targetContainer) {
		var width = 45; // hardcoded value as it used to be
		var tolerance = 6; //set to the total length of the elipsis addition ' (...)'
		var needTruncate = displayValue && displayValue !== null && displayValue.length > (width + tolerance);
		if (needTruncate) {	
			var truncatedValContainer = $("<span>").append(displayValue.substring(1, width)+" ");
			//add the span with elipsis hover popup
			$("<a>", {"text": "(...)"})
					  .mouseout(_.partial(delayhidepopup, _))
					  .mouseover(_.partial(showmenu, _, displayValue))
					  .appendTo(truncatedValContainer);
			
			targetContainer.append(truncatedValContainer);
		} else {
			//just use the original value
			targetContainer.append(displayValue);
		};
	};
	
	/**
	 * Returns the function used to display text & numbers. (formats them correctly + adds units)
	 * 
	 * @param {TableDisplaySettings} tableDisplaySettings - the table display settings object
	 * @param {FieldProperties} fieldProperties - a fieldProperties object
	 * @param {ExternalSearchSettings} searchSettings - search related WCR settings container object
	 * @returns {Function} which accepts one argument. The value to format.
	 */
	Utils.getDisplayFunction = function(tableDisplaySettings, fieldProperties, searchSettings) {
		if(!(   tableDisplaySettings && tableDisplaySettings instanceof TableDisplaySettings && 
				fieldProperties && fieldProperties instanceof FieldProperties && searchSettings && searchSettings instanceof ExternalSearchSettings)) {
			throw new Error("Illegal arguments passed to Utils.getDisplayFunction");
		}
		
		// Underscore magic, 
		// Return the function Utils.getDisplayValue where this is this and the last two arguments are filled in already.
		
		return _.partial(
					_.bind(Utils.getDisplayValue, this),
					// prefilled arguments
					_, tableDisplaySettings, fieldProperties, searchSettings);
	};
	
	/**
	 * Returns the function used to display text, dates & numbers. (formats them correctly)
	 * 
	 * @param fieldProperties
	 * @param searchSettings
	 * @returns {Function} which accepts one argument. The value to format.
	 */
	Utils.getParameterFieldDisplayFunction = function(fieldProperties, searchSettings) {
		if(!(fieldProperties && fieldProperties instanceof FieldProperties && 
			searchSettings && searchSettings instanceof ExternalSearchSettings)) {
			throw new Error("Illegal arguments passed to Utils.getParameterFieldDisplayFunction");
		}
		
		//create a new TableDisplaySettings > only need it to set 'showUnitsInValue' property to FALSE (will never show the value and units together)
		var tblDisplaySettings = new TableDisplaySettings();
		tblDisplaySettings.showUnitsInValue = false;
		
		return Utils.getDisplayFunction(tblDisplaySettings, fieldProperties, searchSettings);
	};
	
	return Utils;
});