define(["dashboard/core/parameters/Parameter",
	"component/searchresults/model/SearchInstancedFieldConfiguration"
], function (Parameter,
	SearchInstancedFieldConfiguration) {
	"use strict";

	/**
	 * Parameter used to contain information about a search
	 * @class
	 * @constructor
	 * @augments Parameter
	 * @param {String} name
	 */
	var SearchContextParameter = function SearchContextParameter(name, context) {
		Parameter.call(this, name, context, "component/searchresults/dashboard/SearchContextParameter", "Search"); // FUTURE i18n

		var returnNull = function () {
			return null;
		};

		this._getResultsTable = returnNull;
		this._getSearchDesignPanel = returnNull;
		this._getCart = returnNull;
		this._getAttributeCategoryControl = returnNull;

		this._instancedFieldConfiguration = null;	
		
		this._isLoadingData = false;
	};

	SearchContextParameter.prototype = Object.create(Parameter.prototype);
	SearchContextParameter.prototype.constructor = SearchContextParameter;

	/**
	 * The value of the SearchContextParameter is the searchcontextparameter itself 
	 * @returns {SearchContextParameter} 	
	 */
	SearchContextParameter.prototype.getValue = function () {
		return this;
	};

	/**
	 * Set the FieldLayoutAPI
	 * @param {FieldLayoutAPI} fieldLayoutAPI [[Description]]
	 */
	SearchContextParameter.prototype.setFieldLayoutAPI = function (fieldLayoutAPI) {
		if (this._fieldLayoutAPI !== fieldLayoutAPI) {
			this._fieldLayoutAPI = fieldLayoutAPI;
			this._emitChanged();
		}
	};

	/**
	 * Get the field layout API
	 * @returns {FieldLayoutAPI} [[Description]]
	 */
	SearchContextParameter.prototype.getFieldLayoutAPI = function () {
		return this._fieldLayoutAPI;
	};
	/**
	 * Get the SearchSetup information
	 * @returns {SearchSetupInformation}
	 */
	SearchContextParameter.prototype.getSearchSetup = function () {
		return this._searchSetup;
	};

	/**
	 * Get the Cart 
	 * @returns {SearchDocCart} 
	 */
	SearchContextParameter.prototype.getCart = function () {
		return this._getCart();
	};

	/**
	 * Change the generator for the Cart
	 * @param {Function} generator - function should return SearchDocCart or null
	 */
	SearchContextParameter.prototype.setCartGenerator = function (generator) {
		this._getCart = generator;
	};

	/**
	 * Get the Attribute Category Control
	 * @returns {SearchResultsAttributeCategoryControl} 
	 */
	SearchContextParameter.prototype.getAttributeCategoryControl = function () {
		return this._getAttributeCategoryControl();
	};

	/**
	 * Set the generator of the Attribute Category Control
	 * @param {Function} controlGenerator - function should return SearchResultsAttributeCategoryControl or null
	 */
	SearchContextParameter.prototype.setAttributeCategoryControlGenerator = function (controlGenerator) {
		this._getAttributeCategoryControl = controlGenerator;
	};

	/**
	 * Set the SearchSetupInformation
	 * @param {SearchSetupInformation} searchSetup 
	 */
	SearchContextParameter.prototype.setSearchSetup = function (searchSetup) {
		if (this._searchSetup !== searchSetup) {
			this._searchSetup = searchSetup;
			this._emitChanged();
		}
	};


	/**
	 * Get the Search Data
	 * @returns {SearchModelFrontEndData}
	 */
	SearchContextParameter.prototype.getSearchData = function () {
		return this._searchData;
	};

	/**
	 * Set the Search Front End Data
	 * @param {SearchModelFrontEndData} searchData 
	 */
	SearchContextParameter.prototype.setSearchData = function (searchData) {
		if (this._searchData !== searchData) {
			this._searchData = searchData;
			this._emitChanged();
		}
	};

	/**
	 * Set a generator function for getting the SearchDesignPanel
	 * @param {Function} generator 	
	 */
	SearchContextParameter.prototype.setSearchDesignPanelGenerator = function (generator) {
		if (this._getSearchDesignPanel !== generator) {
			this._getSearchDesignPanel = generator;
			this._emitChanged();
		}
	};

	/**
	 * Get a reference to the SearchDesignPanel component
	 * @returns {SearchResultsParameterPanel} 
	 */
	SearchContextParameter.prototype.getSearchDesignPanel = function () {
		return this._getSearchDesignPanel();
	};


	/**
	 * Get a reference to the search results table component
	 * @returns {SearchResultsRerunSectionControl} [[Description]]
	 */
	SearchContextParameter.prototype.getSearchResultsTable = function () {
		return this._getResultsTable();
	};

	/**
	 * Set the generator function for the SearchResultsTable
	 */
	SearchContextParameter.prototype.setSearchResultsTableGenerator = function (generator) {
		if (this._getResultsTable !== generator) {
			this._getResultsTable = generator;
			this._emitChanged();
		}
	};

	/**
	 * Get the configuration of an instance
	 * @param {string} instanceId 
	 * @return {ValueData} config
	 */
	SearchContextParameter.prototype.getInstanceConfig = function (instanceId) {
		if (this.getInstancedFieldConfiguration()) {
			return this.getInstancedFieldConfiguration().getInstance(instanceId);
		}
		return null;
	};

	/**
	 * Get the Search Model Instanced Field Configuration object
	 * 
	 * the dashboard currently works with a clone of this object 
	 * in the future this might need to be changed.
	 * @returns {SearchInstancedFieldConfiguration} 
	 */
	SearchContextParameter.prototype.getInstancedFieldConfiguration = function () {
		if (!this._instancedFieldConfiguration && this._searchSetup) {
			this._instancedFieldConfiguration = this._searchSetup.getInstancedFieldConfiguration().clone();
		}
		return this._instancedFieldConfiguration;
	};

	/**
	 * Set the configuration of an instanced field
	 * @param {string} instanceId 
	 * @param {ValueData} config
	 */
	SearchContextParameter.prototype.setInstanceConfig = function (instanceId, config) {
		if (this.getInstancedFieldConfiguration()) {
			this.getInstancedFieldConfiguration().setInstance(instanceId, config);
		}
	};

	/**
	 * Remove an instance from the instanced field configuration
	 * @param {string} instanceId 
	 */
	SearchContextParameter.prototype.removeInstanceConfig = function (instanceId) {
		if (this.getInstancedFieldConfiguration()) {
			this.getInstancedFieldConfiguration().removeInstance(instanceId);
		}
	};

	/**
	 * Add an instance to the instanced field configuration
	 * @param {string} instanceId 
	 * @param {ValueData} config
	 */
	SearchContextParameter.prototype.addInstanceConfig = function (instanceId, config) {
		if (this.getInstancedFieldConfiguration()) {
			return this.getInstancedFieldConfiguration().addInstance(instanceId, config);
		}
		return null;
	};
	
	
	/**
	 * Indicate if data is being loaded
	 * @param {Boolean} isLoadingData 
	 */
	SearchContextParameter.prototype.setLoadingData = function(isLoadingData){
		if (this._isLoadingData != isLoadingData){
			
			this._isLoadingData = isLoadingData;
			this._emit("loadingData", isLoadingData);
		}
		
	};
	
	/**
	 * is data being loaded in the search?
	 * @returns {Boolean} 
	 */
	SearchContextParameter.prototype.isLoadingData = function(){
		return this._isLoadingData; 
	};

	return SearchContextParameter;

});