define(["dashboard/core/Context",
	"dashboard/core/parameters/Parameter",
	"./SearchContextParameter",
	"component/dashboard/i18n/WebCenterLanguageProvider",
	"dashboard/core/BlockDefinition",
	"./configuration/BlockDefinitionsJSON",
	"utils/webi18n!Dashboard",
	"underscore",
	"jquery",
	"lib/PromisePolyfill"
], function (
	Context,
	Parameter,
	SearchContextParameter,
	WebCenterLanguageProvider,
	BlockDefinition,
	blockDefinitions,
	dashboardBundle,
	_,
	$,
	PromisePolyfill) {
	"use strict";

	/**
	 * SearchResults dashboard iniializes a context for displaying search results.
	 * It can also be used to configure the search results dashboard
	 * @param {Object} searchInformation - { data: SearchModelFrontEndData, setup: SearchSetupInformation }
	 * @param   {JSONObject}   configuration - Layout of the blocks
	 * @param {Boolean} canConfigure - should the dashboard be opened in configuration mode?
	 * @param {Object} [smartnameInfo] - optional info for working in a smartname context
	 * @returns {Context}      context
	 
	 */
	var SearchResultsDashboard = function (searchInformation, configuration, canConfigure, smartnameInfo) {

		this._element = $("<div>");
		this._context = new Context();
		this._context.setLanguageProvider(new WebCenterLanguageProvider());

		this._context.toggleEditMode(canConfigure);

		//show loading indicator after 750ms loading
		setTimeout(_.bind(function () {
			this._context.getLanguageBundle("Dashboard").then(function (dashboard_bundle) {
				if (!this._dashboard) {
					this._element.text(dashboard_bundle.t("LOADING_BLOCK"));
				}
			}.bind(this));
		}, this), 500);
		this._element[0].__CONTEXT = this._context;


		this._searchContextParameter = new SearchContextParameter("searchContext", this._context);
		this._searchContextParameter.setSearchData(searchInformation.data);
		this._searchContextParameter.setSearchSetup(searchInformation.setup);

		if (searchInformation.getResultsTable) {
			this._searchContextParameter.setSearchResultsTableGenerator(searchInformation.getResultsTable);
		}
		if (searchInformation.getDesignPanel) {
			this._searchContextParameter.setSearchDesignPanelGenerator(searchInformation.getDesignPanel);
		}

		if (searchInformation.getCart) {
			this._searchContextParameter.setCartGenerator(searchInformation.getCart);
		}

		if (searchInformation.fieldLayoutAPI) {
			this._searchContextParameter.setFieldLayoutAPI(searchInformation.fieldLayoutAPI);
		}

		if (searchInformation.getAttributeCategoryControl) {
			this._searchContextParameter.setAttributeCategoryControlGenerator(searchInformation.getAttributeCategoryControl);
		}

		this._context.setParameter(this._searchContextParameter.getName(), this._searchContextParameter); //TODO check functionality if this takes a while

		// temporary way of passing smartname context info, until we develop a structured one solution fits all
		if (smartnameInfo) {
			var smartnamecontext = new Parameter("WCR_SMARTNAME_CONTEXT", null, "dashboard/core/parameters/Parameter");
			smartnamecontext.setValue(smartnameInfo);
			this._context.setParameter(smartnamecontext.getName(), smartnamecontext);
		}



		this._dashboardPromise = this._context.registerAndLoadBlock(configuration).then(function (dashboard) {
			this._element[0].__DASHBOARD = dashboard;
			this._dashboard = dashboard;
			this._element.html("");

			this._addMissingSearchBlocks();

			if (canConfigure) {
				var undoRedoBar = this.initUndoRedo().addClass("add-spacing-right");
				//Adding undo/redo bar
				this._element.append($("<div>", {
					'class': "clearfix add-spacing-bottom"
				}).append(undoRedoBar));
			}

			//Adding dashboard
			this._element.append(dashboard.getElement());

			return dashboard;
		}.bind(this));

		this.setBlockDefinitions(blockDefinitions);



	};

	/**
	 * Get the element of the SearchResults Dashboard
	 * @returns {JQueryObj} 
	 */
	SearchResultsDashboard.prototype.getElement = function () {
		return this._element;
	};

	/**
	 * Add all missing blocks that are needed for the current configuration of search
	 * 
	 * This method will add DesignPanel, Cart, AssetBrowser ImagePanel/SidePanel blocks
	 * @private
	 * @returns {Promise} resolved once the oeration has finished
	 */
	SearchResultsDashboard.prototype._addMissingSearchBlocks = function () {
		var searchSetup = this._searchContextParameter.getSearchSetup();
		var hasDesignPanelItems = searchSetup.hasDesignPanel();
		var hasassetbrowser = searchSetup.hasAssetBrowser();
		var hascart = searchSetup.getLayoutOptions().getOption("chkbox_ShowCart");
		var needsDesignPanel = hasDesignPanelItems || hasassetbrowser || searchSetup.hasSearchButton();

		if (needsDesignPanel || hascart) {

			var self = this;
			this._dashboard.whenFullyLoaded().then(function () {
				var blocksToAdd = [];

				if (needsDesignPanel) {
					blocksToAdd.push("SearchDesignPanel");
				}

				if (hasassetbrowser) {
					var assetBrowserStyle = "LeftAndImagePanel";
					
					if (self._context.isEditModeActive()){
						assetBrowserStyle = searchSetup.getLayoutOptions().getOption("SelectImageAreaStyle");
					}
					
					if (assetBrowserStyle === "LeftAndImagePanel" || assetBrowserStyle === "LeftPanelOnly"){
						blocksToAdd.push("AssetBrowserSidePanel");	
					} 
					
					if (assetBrowserStyle === "LeftAndImagePanel" || assetBrowserStyle === "ImagePanelOnly"){
						blocksToAdd.push("AssetBrowserImagePanel");
					}					
				}

				if (hascart) {
					blocksToAdd.push("Cart");
				}



				var addedBlocks = {};
				Promise.all(_.map(blocksToAdd, function (blockName) {
					var blockDefinition = blockDefinitions[blockName];

					return self._context.getAttachedBlocksByType(blockDefinition.type).then(function (blocks) {
						if (blocks.length === 0) {
							addedBlocks[blockName] = self._context.registerBlock(blockDefinition);
						}
					});
				})).then(function () {

					// all blocks are registered, not add them to the dashboard in the correct order
					var order = ["SearchDesignPanel", "AssetBrowserImagePanel", "Cart"];
					var blocksToPrepend = [];

					_.each(order, function (blockName) {
						if (addedBlocks[blockName]) {
							blocksToPrepend.push(addedBlocks[blockName]);
						}
					});

					return self._dashboard.prependBlocks(blocksToPrepend).then(function () {
						if (addedBlocks.AssetBrowserSidePanel) {


							// we need to create a column layout and a list layout
							var listLayout = self._context.loadBlock(self._context.registerBlock(blockDefinitions.ListLayout));
							var columnLayout = self._context.loadBlock(self._context.registerBlock(blockDefinitions.AssetBrowserColumnLayout));

							return Promise.all([listLayout, columnLayout, self._dashboard.getLayout()]).then(function (values) {

								var newListLayout = values[0];
								var newColumnLayout = values[1];
								var dashboardLayout = values[2];

								// get the current items of the dashboard
								var items = dashboardLayout.getItemList();

								var waitFor = [];

								// add the asset browser side panel in the column layout left column
								newColumnLayout.replaceColumnWithBlock(0, addedBlocks.AssetBrowserSidePanel);
								// add the new list layout in the right column
								newColumnLayout.replaceColumnWithBlock(1, newListLayout.getID());

								// add column layout to dashboard
								waitFor.push(dashboardLayout.prependBlock(newColumnLayout.getID()));

								// move items from dashboard to our new list layout
								waitFor.push(newListLayout.setItemList(items));

								return Promise.all(waitFor);
							});

						}
					});
				});
			});
		}
	};

	/**
	 * Are there any blocks of the given types attached?
	 * @param   {Array.<String>} types - key of block in BlockDefinitionJSON.js
	 * @returns {Promise.Object<String,Boolean>}   for each type in list a boolean true/false
	 */
	SearchResultsDashboard.prototype.hasBlocksOfType = function(types){
		var self = this;
		return Promise.all(_.map(types, function(blockName){
			return self._context.getAttachedBlocksByType(blockDefinitions[blockName].type).then(function(blocks){
				return blocks.length > 0;
			});
		
		})).then(function(values){
			return _.object(types, values);
		});	
	};

	/**
	 * Set the Block Definitions for the SearchResults Dashboard
	 * @param   {object}   blockDefinitions 
	 */
	SearchResultsDashboard.prototype.setBlockDefinitions = function (blockDefinitions) {
		var context = this._context;
		var self = this;
		context.getBlockDefinitionDataSource()
			.then(function (datasource) {
				var excludes = [];
				// filter out not allowed blocks
				if (self._searchContextParameter.getSearchData().informationtype !== "DOCUMENTS") {
					// following blocks are DOCUMENT search only, don't add them for other searches
					excludes = ["AssetBrowserImagePanel", "AssetBrowserSidePanel", "Cart"];

				}

				excludes.push("AssetBrowserColumnLayout");

				var filteredDefinitions = _.filter(blockDefinitions, function (item, key) {
					return !_.contains(excludes, key);
				});


				Promise.all(_.map(filteredDefinitions, function (item) {
					return context.getLanguageBundle(item.languageBundle).then(function (i18n) {
						if (item.displayNameTag) {
							item.displayName = i18n.tFallback(item.displayNameTag, item.displayName);
						}

						var blockdef = new BlockDefinition("", item.type, item);
				
						blockdef.setDescription(i18n.tFallback(item.descriptionTag, item.description));
						return blockdef;
					});
				})).then(function (blockDefinitionObjects) {
					datasource.setBlockDefinitions(blockDefinitionObjects);
				});
			});
	};



	/**
	 * Makes the undo/redo buttons
	 * @param   {Context}      context 
	 * @returns {JQueryObject} configurationbar
	 */
	SearchResultsDashboard.prototype.initUndoRedo = function () {
		var undoredo = this._context.getUndoRedoController();

		var configBar = $("<div>", {
			'class': "btn-group pull-right"
		});
		var commandOngoing = false;


		var undo = $("<a>", {
				'class': "btn btn-default",
				title: dashboardBundle.t("UNDO")
			}).append($("<i>", {
				'class': "icon icon-WCR_Actions-Undo12"
			}))
			.click(function () {
				commandOngoing = true;
				onUndoRedoChanged();
				Promise.resolve(undoredo.undo()).then(function () {
					commandOngoing = false;
					onUndoRedoChanged();
				});
			});

		var redo = $("<a>", {
				'class': "btn btn-default",
				title: dashboardBundle.t("REDO")
			}).append($("<i>", {
				'class': "icon icon-WCR_Actions-Redo12"
			}))
			.click(function () {
				commandOngoing = true;
				onUndoRedoChanged();
				Promise.resolve(undoredo.redo()).then(function () {
					commandOngoing = false;
					onUndoRedoChanged();
				});
			});

		var onUndoRedoChanged = _.debounce(function () {
			undo.toggleClass("disabled", commandOngoing || undoredo.undoCount() <= 0);
			undo.attr("title", dashboardBundle.t("UNDO") + " " + (undoredo.peekUndo() ? undoredo.peekUndo().getDescription() : ""));
			redo.toggleClass("disabled", commandOngoing || undoredo.redoCount() <= 0);
			redo.attr("title", dashboardBundle.t("REDO") + " " + (undoredo.peekRedo() ? undoredo.peekRedo().getDescription() : ""));
		}, 10);
		undoredo.on("changed", onUndoRedoChanged);
		onUndoRedoChanged();

		configBar.append(undo).append(redo);



		return configBar;

	};

	/** 
	 * returns the dashboard version as a promise
	 * @return {Promise}
	 */
	SearchResultsDashboard.prototype._getDashboardVersion = function () {
		return Promise.resolve($.get("dashboardversion.jsp"))
			.then(function (json) {
				return JSON.parse(json).number;
			}).catch(function () {
				return this._context.getLanguageBundle("Dashboard").then(function (dashboard_bundle) {
					return Promise.reject(dashboard_bundle.t("NOT_LOGGED_IN"));
				});
			}.bind(this));
	};


	/**
	 * Get the configuration of the dashboard
	 * @returns {Promise.<Object>} 
	 */
	SearchResultsDashboard.prototype.getConfiguration = function () {
		return this.whenFullyLoaded().then(function (dashboard) {
			return Promise.all([
				Promise.resolve(dashboard.getConfiguration(true)),
				this._getDashboardVersion()
			]);
		}.bind(this)).then(function (args) {
			var configuration = args[0];
			configuration.version = args[1];
			return configuration;
		});
	};

	/**
	 * Wait for the search results dashboard to be fully loaded
	 * @returns {Promise.<Dashboard>} 
	 */
	SearchResultsDashboard.prototype.whenFullyLoaded = function () {
		return this._dashboardPromise.then(function (dashboard) {
			return dashboard.whenFullyLoaded();
		});
	};

	/**
	 * Get a dataclone of the SearchInstancedFieldConfiguration object
	 * @returns {Object} 
	 */
	SearchResultsDashboard.prototype.getInstancedFieldConfigurationClone = function () {
		return this._searchContextParameter.getInstancedFieldConfiguration().getDataClone();
	};

	/**
	 * Get an array of all the instance ids in the parameters of the blocks attached in the context of the dashboard
	 * @returns {Array.<string>} instanceIds
	 */
	SearchResultsDashboard.prototype.getInstanceIds = function () {
		return this._context.getAttachedBlocks().then(function (blocks) {
			var instanceIds = [];
			_.each(blocks, function (block) {
				_.each(block.getParameterListByType("searchresults/dashboard/parameters/SearchIFCParameter"), function (sIFCParameter) {
					if (sIFCParameter.getInstanceId) {
						var instanceId = sIFCParameter.getInstanceId();
						if (instanceId) {
							instanceIds.push(instanceId);
						}
					}
				});
			});
			return instanceIds;
		});
	};

	/**
	 * Notify the dashbaord that data is being loaded
	 * @param {Boolean} isLoading 
	 */
	SearchResultsDashboard.prototype.setLoadingData = function(isLoading){
		this._searchContextParameter.setLoadingData(isLoading);	
	};
	
	/**
	 * Is the search results dashboard modified by the user?
	 * @returns {Boolean} 
	 */
	SearchResultsDashboard.prototype.isModified = function(){
		return this._context.isModified();
	};
	
	return SearchResultsDashboard;
});