define([
	"underscore",
	"jquery",
	"./configuration/DefaultSearchResultsConfiguration",
	"component/searchresults/dashboard/SearchResultsDashboard",
	"component/modal/Modal",
	"component/overlaymessage/OverlayMessage",
	"utils/webi18n!SearchResultsLayoutPopup",
	"utils/webi18n!Dashboard",
	"component/pagemessagecomponent/PageMessageComponent"
], function (
	_,
	$,
	defaultSearchResultsConfiguration,
	SearchResultsDashboard,
	Modal,
	OverlayMessage,
	searchResultsLayoutPopupBundle,
	dashboardBundle,
	pageMessageComponent
) {
	"use strict";


	/**
	 * Makes a dashboard popup with a configuration (layout) specified in searchDashboardConfiguration
	 * @param {object} searchInformation - { data: SearchModelFrontEndData, setup: SearchSetupInformation }
	 * @param {SearchDashboardConfiguration} searchDashboardConfiguration 
	 */
	var SearchSetupDashboardBuilder = function (searchInformation, searchDashboardConfiguration) {
		this._searchInformation = searchInformation;
		this._searchDashboardConfiguration = searchDashboardConfiguration;
		this._searchResultsDashboard = null;
		this._modal = null;

		this._isSavedSearchEdit = $('#isSavedSearchEdit').text();
	};

	/**
	 * Open the popup
	 */
	SearchSetupDashboardBuilder.prototype.open = function () {
		var configuration = defaultSearchResultsConfiguration;
		if (this._searchDashboardConfiguration.hasDashboard()) {
			configuration = this._searchDashboardConfiguration.getDashboard();
		}
		this._searchResultsDashboard = new SearchResultsDashboard(this._searchInformation, configuration, true);

		// Create the modal
		this._modal = new Modal(this._searchResultsDashboard.getElement(), {
			widthClass: "modal-xl",
			customCloseHandler: this._confirmUnsavedChangedAndClose.bind(this)
		});

		this._modal.setTitle(searchResultsLayoutPopupBundle.tFallback("SHOWDASHBOARDPOPUP", "Layout of Results Page"));
		this._modal.setHelpTopic("common/wc/concept/co_wc_search_results_dashboard");
		var footer = this.makeFooter();
		this._modal.setFooter(footer);

		this._modal.open(true);
	};

	/**
	 * If the builder modal is closed with unsaved changes to the dashboard,
	 * Ask the user what to do
	 * @private
	 * @param {Event} event 
	 */
	SearchSetupDashboardBuilder.prototype._confirmUnsavedChangedAndClose = function (event) {
		var self = this;
		if (self._searchResultsDashboard.isModified()) {
			event.preventDefault();
			event.stopPropagation();


			var confirmationModal = new Modal($("<div>", {
				text: dashboardBundle.t("UNSAVED_CHANGES_MESSAGE")
			}), {
				allowClose: false
			});
			confirmationModal.setTitle(dashboardBundle.t("UNSAVED_CHANGES_TITLE"));

			var modalFooter = $("<div>");
			confirmationModal.setFooter(modalFooter);

			// CANCEL
			modalFooter.append($("<a>", {
				"class": "btn btn-default",
				text: dashboardBundle.t("CANCEL_BLOCK_CONFIGURATION"),
				click: function () {
					confirmationModal.close();
				}
			}));

			// Close without Saving
			modalFooter.append($("<a>", {
				"class": "btn btn-default",
				text: dashboardBundle.t("UNSAVED_CHANGES_CLOSE_WITHOUT_SAVE"),
				click: function () {
					confirmationModal.close();
					self._modal.close();
				}
			}));

			// Close without Saving
			modalFooter.append($("<a>", {
				"class": "btn btn-primary",
				text: dashboardBundle.t("UNSAVED_CHANGES_CLOSE_WITH_SAVE"),
				click: function () {
					self._saveDashboardConfiguration();
					confirmationModal.close();
				}
			}));


			confirmationModal.open();
		}
	};

	/**
	 * Make the footer for the popup
	 * @param {Modal} modal
	 * @param {SearchResultsDashboard} searchResultsDashboard
	 * @returns {JQueryObject}   footer
	 */
	SearchSetupDashboardBuilder.prototype.makeFooter = function () {
		var ok = $('<input>').attr({
				'type': 'button',
				'value': searchResultsLayoutPopupBundle.tFallback("SAVECONFIGURATION", "OK")
			}).addClass("btn btn-primary")
			.click(function () {
				this._saveDashboardConfiguration();

			}.bind(this));

		var cancel = $('<input>').attr({
				'type': 'button',
				'value': searchResultsLayoutPopupBundle.tFallback("CANCEL", "Cancel")
			}).addClass("btn btn-default")
			.click(function () {
				this._modal.close();
			}.bind(this));
		var footer = $("<div>").append(cancel).append(ok);
		return footer;
	};


	/**
	 * Save the dashboard configuration
	 * @private
	 * @returns {Promise} 
	 */
	SearchSetupDashboardBuilder.prototype._saveDashboardConfiguration = function () {
		if (this._isSavedSearchEdit) {
			pageMessageComponent.markObsolete("SAVE_SAVED_SEARCH");
			pageMessageComponent.sendInfoMessage(searchResultsLayoutPopupBundle.tFallback("SAVE_SAVED_SEARCH_SAVING", "You have unsaved changes, please save to retain your changes."), null, false, "SAVE_SAVED_SEARCH");
		}

		var overlay = new OverlayMessage(this._modal.getModalDialog(), searchResultsLayoutPopupBundle.tFallback("SAVING_CONFIGURATION", "Saving Configuration..."));
		//Ok button clicked -> save configuration and close popup

		this._storeDashboardConfiguration()
			.then(this._storeInstancedFieldConfiguration.bind(this))
			.then(this._updateLayoutOptions.bind(this))
			.catch(function (reason) {
				if (!window.confirm(reason + " " + searchResultsLayoutPopupBundle.t("COULD_NOT_SAVE"))) {
					return Promise.reject();
				}
			}).then(function () {
				// we only close the popup once the dashboard is successfully saved in the search setup
				this._modal.close();
				overlay.hide();
			}.bind(this)).catch(function (e) {
				overlay.hide();
			});

	};

	/**
	 * Store the Dashboard Configuration
	 * @private
	 * @returns {Promise} 
	 */
	SearchSetupDashboardBuilder.prototype._storeDashboardConfiguration = function () {
		return this._searchResultsDashboard.getConfiguration().then(function (configuration) {
			//save layout of blocks
			if (configuration) {
				this._searchDashboardConfiguration.setDashboard(configuration);
			} else {
				this._searchDashboardConfiguration.setDashboard(null); // no dashboard defined 
			}
		}.bind(this));

	};

	/**
	 * Store the Instanced Field Configuration
	 * @private
	 * @returns {Promise} 
	 */
	SearchSetupDashboardBuilder.prototype._storeInstancedFieldConfiguration = function () {
		return this._searchResultsDashboard.getInstanceIds().then(function (instanceIdsFromDashboard) {
			var instancesFromIFC = this._searchResultsDashboard.getInstancedFieldConfigurationClone();
			_.each(Object.keys(instancesFromIFC), function (instanceIdFromIFC) {
				if (!_.contains(instanceIdsFromDashboard, instanceIdFromIFC)) {
					delete instancesFromIFC[instanceIdFromIFC];
				}
			});
			//save searchInstancedFieldConfiguration
			this._searchInformation.setup.getInstancedFieldConfiguration().setData(instancesFromIFC);

		}.bind(this));
	};

	/**
	 * Update the layout options of search based on the available blocks in the dashboard
	 * @private
	 * @returns {Promise}
	 */
	SearchSetupDashboardBuilder.prototype._updateLayoutOptions = function () {
		var searchSetup = this._searchInformation.setup;
		var layoutOptions = searchSetup.getLayoutOptions();

		return this._searchResultsDashboard.hasBlocksOfType(["Cart", "AssetBrowserSidePanel", "AssetBrowserImagePanel"]).then(function (blocks) {

			layoutOptions.setOption("chkbox_ShowCart", blocks.Cart);


			var cascadingdata = layoutOptions.getOption("cascading_attribute_trees");
			var hasassetbrowser = cascadingdata != null && cascadingdata.getSubList("treeElem") != null && cascadingdata.getSubList("treeElem").length > 0;

			if (hasassetbrowser) {
				if (blocks.AssetBrowserImagePanel && blocks.AssetBrowserSidePanel) {
					layoutOptions.setOption("SelectImageAreaStyle", "LeftAndImagePanel");
				} else if (blocks.AssetBrowserImagePanel) {
					layoutOptions.setOption("SelectImageAreaStyle", "ImagePanelOnly");
				} else if (blocks.AssetBrowserSidePanel) {
					layoutOptions.setOption("SelectImageAreaStyle", "LeftPanelOnly");
				}
			}
		});
	};




	return SearchSetupDashboardBuilder;
});