define(["./AbstractSearchBlock",
		"../parameters/SearchIFCParameter",
		"../parameters/SearchFacetParameter",
		"dashboard/core/parameters/LinkedParameter",
		"dashboard/core/parameters/NumberParameter",
		"dashboard/core/parameters/Parameter",
		"underscore",
		"utils/webcss!./AbstractSearchBlockStyle.css",
		"utils/webcss!./AbstractFacetBlockStyle.css",
		"utils/webi18n!AbstractFacetBlock"
	],
	function (AbstractSearchBlock,
		SearchIFCParameter,
		SearchFacetParameter,
		LinkedParameter,
		NumberParameter,
		Parameter,
		_,
		style,
		facetstyle,
		i18n) {
		"use strict";

		/**
		 * This is a Abstract Upper Class for all blocks related to a facetted search
		 * It wil automatically initialize the FacetParameter
		 * @abstract
		 * @augments AbstractSearchBlock
		 * @constructor
		 * @class
		 * @param {String} id 
		 * @param {Context} context
		 */
		var AbstractFacetBlock = function AbstractFacetBlock(id, context, parameterFieldType) {
			this._parameterFieldType = parameterFieldType;

			AbstractSearchBlock.call(this, id, context);

			/**
			 * @override
			 */
			this._showTransparentFrame = true;

			this._showLoadingIndicator = true;
		};

		AbstractFacetBlock.prototype = Object.create(AbstractSearchBlock.prototype);
		AbstractFacetBlock.prototype.constructor = AbstractFacetBlock;


		/**
		 * Define the search Facet Parameter. Also call _initializeDefaultParameters from AbstractSearchBlock.
		 * @private
		 * @override
		 */
		AbstractFacetBlock.prototype._initializeDefaultParameters = function (config) {
			AbstractSearchBlock.prototype._initializeDefaultParameters.call(this, config);

			return this._context.getLanguageBundle("AbstractFacetBlock").then(function (i18n) {
				// this is the instanced field parameter for db_facet
				// each block can add extra parameters from which the values will end up in the instanced field configuration
				// the value of these parameters can either be a string or Array.<ValueData>
				var facetParameter = new SearchIFCParameter("FacetParameter", this._context, i18n.t("FACET"), this._parameters.getParameter("search"), "db_facet");
				this._addParameter(facetParameter);

				var blockTitle = new LinkedParameter("title",
					this._context,
					"Title",
					this._parameters,
					"dashboard/core/parameters/StringParameter",
					"blockTitle",
					false);
				facetParameter.addParameter(blockTitle);

				// this parameter defines the field type of the parameter field (the value is set in the constructor of AbstractFacetBlock)
				var parameterFieldType = new Parameter("parameterFieldType", this._context, "ParameterFieldType");
				parameterFieldType.setValue(this._parameterFieldType);
				parameterFieldType.setConfigurableByUser(false);
				facetParameter.addParameter(parameterFieldType);

				// this parameter defines on whichs fields the facet block will work
				var facetIds = new SearchFacetParameter("facetids", this._context, i18n.t("FACET"), this._parameters.getParameter("search"));
				facetParameter.addParameter(facetIds);

				// if the facet ids parameter change, we want to automatically update the block title for user friendlyness
				this._initialTitle = this._parameters.getValue("blockTitle");
				this._observe(facetIds, "changed", this._onFacetIdsChanged);

				var numberOfFacetsParam = new NumberParameter("numberOfFacets", this._context, i18n.t("NUMBER_OF_FACETS"),
					10, {
						type: "int",
						min: 1,
						max: 100
					},
					i18n.t("NUMBER_OF_FACETS_HELP"), i18n.t("NUMBER_OF_FACETS_HELP_EXTENDED"));
				numberOfFacetsParam.setValue("10");
				facetParameter.addParameter(numberOfFacetsParam);

			}.bind(this));
		};
	
		/**
		 * FacetIds parameter was changed, update the blockTitle if needed
		 * @private
		 * @param {Array[ValueData]} oldValue 
		 * @param {Array[ValueData]} newValue 
		 */
		AbstractFacetBlock.prototype._onFacetIdsChanged = function (oldValue, newValue) {
			var currentTitle = this._parameters.getValue("blockTitle");
			var oldLabel = null;

			if (oldValue && oldValue[0]) {
				oldLabel = oldValue[0].getValue("label");
			}

			if (newValue && newValue[0]) {
				// a facet selection was made
				if (currentTitle === this._initialTitle || currentTitle === oldLabel || currentTitle === "") {
					var newTitle = newValue[0].getValue("label");
					this._parameters.setValue("blockTitle", newTitle);
				}
			}

		};


		/**
		 * listen to SearchFacetParameter changes
		 * @override
		 */
		AbstractFacetBlock.prototype._doInit = function (config, idMap) {
			return AbstractSearchBlock.prototype._doInit.apply(this, arguments).then(function () {
				this._parameters.getParameter("FacetParameter").doInit();
				this._observeParam("FacetParameter", this._render);
			}.bind(this));
		};

		/**
		 * Display the Search Design Panel (if available)
		 * @private
		 * @param {SearchContextParameter} 
		 */
		AbstractFacetBlock.prototype._doSearchRender = function (searchParameter) {
			var div = $("<div>");
			this._header = $("<div>", {
				'class': "facet-header"
			});
			var instanceId = this._parameters.getParameter("FacetParameter").getInstanceId();
			if (searchParameter && searchParameter.getValue() && searchParameter.getValue().getFieldLayoutAPI()) {
				searchParameter.getValue().getFieldLayoutAPI().makeParameterField(instanceId, div.get(0), function () {
					var header = $(".facet-header", div);
					this._header.append(header.children());
				}.bind(this));
			}
			this._element.html("").append(div);
		};

		/**
		 * Link this block to the SearchInstancedFieldConfiguration
		 */
		AbstractFacetBlock.prototype.onAttached = function () {
			AbstractSearchBlock.prototype.onAttached.call(this);
			this._parameters.getParameter("FacetParameter").attach();
		};

		/**
		 * Unlink this block from the SearchInstancedFieldConfiguration
		 */
		AbstractFacetBlock.prototype.onDetached = function () {
			AbstractSearchBlock.prototype.onDetached.call(this);
			this._parameters.getParameter("FacetParameter").detach();
		};

		/**
		 * @override
		 */
		AbstractFacetBlock.prototype.getTitleElement = function () {
			if (this._header) {
				return this._header;
			} else {
				return AbstractSearchBlock.prototype.getTitleElement.call(this);
			}
		};

		/**
		 * Set the order of parameters
		 * @private
		 */
		AbstractFacetBlock.prototype._updateParameterOrder = function () {
			this._parameters.moveParameterToEndOfList("blockShowFrame");
			this._parameters.moveParameterToEndOfList("blockFrameColor");
			this._parameters.moveParametertoFrontOfList("blockTitle");
			this._parameters.getParameter("blockTitle").setVisible(true);
		};

		/**
		 * The ShowFrame parameter has changed
		 * @private
		 * @param {Boolean} oldValue 
		 * @param {Boolean} newValue 
		 */
		AbstractFacetBlock.prototype._blockShowFrameChanged = function (oldValue, newValue) {
			AbstractSearchBlock.prototype._blockShowFrameChanged.apply(this, arguments);
			this._parameters.getParameter("blockTitle").setVisible(true);
		};

		return AbstractFacetBlock;
	});