define(["dashboard/core/Block",
		"dashboard/core/parameters/LinkedParameter",
		"component/overlaymessage/OverlayMessage",
		"underscore",
		"utils/webcss!./AbstractSearchBlockStyle.css"
	],
	function (Block,
		LinkedParameter,
		OverlayMessage,
		_,
		style) {
		"use strict";

		/**
		 * This is a Abstract Upper Class for all blocks related to a search
		 * It wil automatically initialize the search parameter
		 * @abstract
		 * @augments Block
		 * @constructor
		 * @class
		 * @param {String} id 
		 * @param {Context} context
		 */
		var AbstractSearchBlock = function AbstractSearchBlock(id, context) {
			Block.call(this, id, context);

			this._searchContext = null;
			this._searchContextObserver = null;

			this._showLoadingIndicator = false;
		};

		AbstractSearchBlock.prototype = Object.create(Block.prototype);
		AbstractSearchBlock.prototype.constructor = AbstractSearchBlock;


		/**
		 * Define the search Parameter link. If more parameters are needed, override this method, but call this implementation first.
		 * 
		 * @private
		 */
		AbstractSearchBlock.prototype._initializeDefaultParameters = function (config) {
			Block.prototype._initializeDefaultParameters.call(this, config);
			// add parameter linked to the Context 
			var searchParameter = new LinkedParameter("search", this._context, "Search", this._context.getParameterSet(), "component/searchresults/dashboard/SearchContextParameter");
			searchParameter.setConfigurableByUser(false);

			// by default initialize the parameter with the last added search parameter of the context
			var lastAddedSearchName = _.last(this._context.getParametersByType("component/searchresults/dashboard/SearchContextParameter"));
			if (lastAddedSearchName) {
				searchParameter.setLinkedName(lastAddedSearchName.getName());
			}

			this._addParameter(searchParameter);
		};

		/**
		 * The search context parameter was changed	
		 * @private
		 */
		AbstractSearchBlock.prototype._onSearchContextChanged = function () {
			if (this.getParameterByName("search").getValue() != this._searchContext) {
				if (this._searchContextObserver) {
					this._stopObserveSubscription(this._searchContextObserver);
					this._searchContextObserver = null;
				}

				this._searchContext = this.getParameterByName("search").getValue();
				this._searchContextObserver = this._observe(this._searchContext, "loadingData", this._updateLoadingIndicator);
				this._render();
			}
		};


		/**
		 * Show loading indicator if neeed
		 * @private
		 * @param {Boolean} isLoading 
		 */
		AbstractSearchBlock.prototype._updateLoadingIndicator = function (isLoading) {
			if (this._showLoadingIndicator && isLoading) {
				if (!this._overlayMessage) {
					this._overlayMessage = new OverlayMessage(this._element);
				}

				this._overlayMessage.showLoading();
			} else if (this._overlayMessage) {
				this._overlayMessage.hide();
			}
		};

		/**
		 * @override
		 * @private
		 */
		AbstractSearchBlock.prototype._doInit = function () {
			return this._context.getLanguageBundle("SearchDashboard").then(function (bundle) {
				this._i18n = bundle;


				this._observeParam("search", this._onSearchContextChanged);

				this._onSearchContextChanged();


			}.bind(this));

		};


		/**
		 * Show a message when not linked to a search
		 * @override
		 * @private
		 */
		AbstractSearchBlock.prototype._doRender = function () {
			var searchParameter = this._parameters.getParameter("search");

			if (!searchParameter || !searchParameter.getValue()) {
				this._element.text(this._i18n.t("LINK_BLOCK_TO_SEARCH"));
			} else {
				this._doSearchRender(searchParameter.getValue());
			}
		};


		/**
		 * Do the actual render, override this method to implement the block rendering
		 * @private
		 * @param {SearchContextParameter} searchParameter 
		 */
		AbstractSearchBlock.prototype._doSearchRender = function (searchParameter) {
			// override		
		};


		/**
		 * Utility method to indicate what information will be visible in the block once context is correctly set or information is available
		 * @private
		 * @param {String} icon  - icon class for the placeholder
		 * @param {String} label - translated placeholder label
		 */
		AbstractSearchBlock.prototype._showPlaceholder = function (icon, label) {
			this._element.addClass("searchBlockPlaceholder");
			this._element.text(label);
			this._element.prepend($("<i>", {
				'class': "icon icon-large add-spacing-right"
			}).addClass(icon));
		};

		return AbstractSearchBlock;
	});