define(["dashboard/core/Block",
		"dashboard/core/parameters/StringParameter",
		"dashboard/core/parameters/BooleanParameter",
		"../parameters/WCRSavedSearchParameter",
		"./SavedSearchCounterBlockStyles",
		"utils/HTTPUtils",
		"utils/Promisify",
		"utils/ColorUtils",
		"underscore",
		"jquery",
		"utils/webi18n!SavedSearchCounterBlock",
		"utils/webcss!./SavedSearchCounterBlock.css"
	],
	function (Block,
		StringParameter,
		BooleanParameter,
		WCRSavedSearchParameter,
		SavedSearchCounterBlockStyles,
		HTTPUtils,
		Promisify,
		ColorUtils,
		_,
		$,
		i18n) {
		"use strict";

		/**
		 * Basic implementation of an HTML Block
		 * This block can display a customizable chunk of HTML
		 *
		 * @class
		 * @constructor
		 * @augments Block
		 * @param {String} id - ID
		 * @param {Context} [context]
		 */
		var SavedSearchCounterBlock = function (id, context) {
			Block.call(this, id, context);
			this._count = "-";

			this._runSearchDebounced = _.debounce(this._runSearch, 50);
		};

		SavedSearchCounterBlock.prototype = Object.create(Block.prototype);
		SavedSearchCounterBlock.prototype.constructor = SavedSearchCounterBlock;

		/**
		 * @Override
		 */
		SavedSearchCounterBlock.prototype.getType = function () {
			return "component/searchresults/dashboard/blocks/SavedSearchCounterBlock";
		};


		/**
		 * Configure the default parameters of the block
		 * @override
		 */
		SavedSearchCounterBlock.prototype._initializeDefaultParameters = function () {
            Block.prototype._initializeDefaultParameters.apply(this, arguments);
			// saved search
			var savedSearch = new WCRSavedSearchParameter("savedSearch", this._context, i18n.t("SAVED_SEARCH")); 
			this._addParameter(savedSearch);

			// label
			var labelParameter = new StringParameter("label", this._context, i18n.t("LABEL"));
			this._addParameter(labelParameter);


			// color
			var colorParameter = new StringParameter("color", this._context, i18n.t("BACKGROUND_COLOR"));
			colorParameter.setConfigurationPanelComponentLocation("dashboard/builder/configuration/parameters/ColorPickerConfigurationPanel");
			colorParameter.setValue("#4cae4c"); // default color
			this._addParameter(colorParameter);

			// style
			var styleParameter = new StringParameter("style", this._context, i18n.t("STYLE"));
			styleParameter.setRestrictedSet(SavedSearchCounterBlockStyles);
			styleParameter.setValue("counter-default"); // default style
			this._addParameter(styleParameter);

			// run search on click
			var runSearchOnClick = new BooleanParameter("runSearchOnClick", this._context, i18n.t("RUN_SEARCH_ON_CLICK"));
			runSearchOnClick.setValue(true);
			this._addParameter(runSearchOnClick);
			
			
			var previousSearchName = null;
			if (savedSearch.getValue()){
				previousSearchName = savedSearch.getValue().getName();
			}

			// update the label and block title to the selected saved search
			this._observeParam("savedSearch", function(oldValue, newValue){
				if (newValue && this._isInitialized){
					var oldName = null;
					if (oldValue){
						oldName = oldValue.getName();
					}
					
					// update label
					if (!labelParameter.getValue() || (oldName === labelParameter.getValue())){
						labelParameter.setValue(newValue.getName());		
					}
					
					
					if (!this._parameters.getValue("blockTitle") || (oldName === this._parameters.getValue("blockTitle"))){
						this._parameters.setValue("blockTitle", newValue.getName());	
					}
				}
			});
		};


		/**
		 * @Override
		 */
		SavedSearchCounterBlock.prototype._doInit = function (config) {
			this._observeParam("savedSearch", this._runSearchDebounced);
			this._observeParam("label", this._render);
			this._observeParam("color", this._render);
			this._observeParam("style", this._render);

			this._count = "-";

			this._$counter = $("<div>", {
				'class': "count"
			});
			this._$label = $("<div>", {
				'class': "description",
				text: this._parameters.getValue("label")
			});
			this._element.append(this._$counter).append(this._$label);
			this._runSearchDebounced();
			this._render();

			// on click show the search results
			this._element.click(this._openSearchResults.bind(this));
		};


		/**
		 * Get the number of results of the saved search
		 * @private
		 */
		SavedSearchCounterBlock.prototype._runSearch = function () {
			this._render();

			if (this._parameters.getValue("savedSearch")) {
				var search = this._parameters.getValue("savedSearch");
				var params = {
					action: "count",
					searchName: search.getName()
				};
				
				if (this._context.getParameter("WCR_SMARTNAME_CONTEXT")){
					_.extend(params, this._context.getParameter("WCR_SMARTNAME_CONTEXT").getValue());
				}


				var url = HTTPUtils.constructURL("dosearch.wcr", _.pick(params, "menuitem3","menusel3", "menu_file"), window.webcenterConfig.jspBaseUrl);
				$.ajax({
					'url': url,
					'type': "POST",
					'data': params,
					'dataType': "JSON",
					'debug': false,
					'timeout': 120000,
					'synchronous': false,
					'success': function (json) {
						this._count = json.count;
						this._render();
					}.bind(this),
					'error': function () {
						this._count = "-";
						this._render();
					}.bind(this)
				});
			}
		};

		/**
		 * Submit search
		 * @private
		 * @param {JQueryEvent} e 
		 */
		SavedSearchCounterBlock.prototype._openSearchResults = function (e) {
			var search = this._parameters.getValue("savedSearch");
			if (search &&  this._parameters.getValue("runSearchOnClick")) {
				var smartnameInfo;
				
				if (this._context.getParameter("WCR_SMARTNAME_CONTEXT")){
					smartnameInfo = this._context.getParameter("WCR_SMARTNAME_CONTEXT").getValue();
				}
				
				var urlArgs = _.pick(smartnameInfo, "menuitem3", "menusel3", "menu_file", "projectID", "docVerID");
				urlArgs.searchName= search.getName();
				var form = $("<form>", {
					method: "post",
					'class': "hiddenElement",
					action: HTTPUtils.constructURL("dosavedsearch.jsp", urlArgs, window.webcenterConfig.jspBaseUrl)
				});
				
				if (e.ctrlKey || e.metaKey || e.which === 2) {
					form.attr("target", "_blank");
				}
				
				var data = {
					
				};
				
				_.extend(data, smartnameInfo);
				
				_.each(data, function(value, key){					
					form.append($("<input>", {name: key, value: value}));
				});
				
				$(document.body).append(form);
				form.submit();
			}
		};


		/**
		 * @Override
		 */
		SavedSearchCounterBlock.prototype._doRender = function () {
			// define the label
			var label = this._parameters.getValue("label");
			if (!label) {
				label = "";
			}

			// apply style
			this._element.removeClass().addClass("wcrsavedsearchcounterblock").addClass(this._parameters.getValue("style"));
			this._element.toggleClass("runonclick", this._parameters.getValue("runSearchOnClick"));
			// set color
			this._element.css({
				'background-color': this._parameters.getValue("color"),
				'color': ColorUtils.calculateContrastColor(this._parameters.getValue("color"))
			});

			// display results
			this._$counter.text(this._count);
			this._$label.text(label);
			this._element.attr("title", label);
		};


		return SavedSearchCounterBlock;
	});