define(["dashboard/core/parameters/Parameter",
		"component/searchresults/dataobjects/ValueData",
		"component/searchresults/dataobjects/ValueDataUtils",
		"underscore"
	],
	function (
		Parameter,
		ValueData,
		ValueDataUtils,
		_
	) {
		"use strict";

		/**
		 * Parameter to define the fields ids on which facetted search works
		 * the value is Array.<ValueData>
		 * @augments Parameter
		 * @param {String} name             - name of the parameter
		 * @param {String} userFriendlyName - user friendly representation of the parameter
		 * @param {SearchContextParameter} searchContext   
		 */
		var SearchFacetParameter = function (name, context, userFriendlyName, searchContext) {
			Parameter.call(this, name, context, "dashboard/core/parameters/SearchFacetParameter", userFriendlyName);



			this._searchType = null;
			this._searchContext = searchContext;

			this._facetedFieldType = null;

			// value is a list of valuedataobjects with facet id
			this.setValue([]);
		};

		SearchFacetParameter.prototype = Object.create(Parameter.prototype);
		SearchFacetParameter.prototype.constructor = SearchFacetParameter;

		/**
		 * Get the Search Type if it exists.
		 * @returns {string} 
		 */
		SearchFacetParameter.prototype.getSearchType = function () {
			if (this._searchContext.getValue()) {
				return this._searchContext.getValue().getSearchData().searchtype;
			} else {
				return null;
			}
		};

		/**
		 * Set the faceted Field Tye
		 * @param {String} facetedFieldType 
		 */
		SearchFacetParameter.prototype.setFacetedFieldType = function (facetedFieldType) {
			this._facetedFieldType = facetedFieldType;
		};

		/**
		 * Get the datatype of the faceted field
		 * not this info is not always available (in that case NULL is returned)
		 * @param {String} facetedFieldType 
		 */
		SearchFacetParameter.prototype.getFacetedFieldType = function () {
			return this._facetedFieldType;
		};



		/**
		 * Get the configuration panel
		 * @private
		 * @returns {string} [[Description]]
		 */
		SearchFacetParameter.prototype._getConfigurationPanelComponentLocation = function () {
			return "component/searchresults/dashboard/parameters/SearchFacetParameterConfigurationPanel";
		};

		/**
		 * @override
		 */
		SearchFacetParameter.prototype._getParameterSpecificConfig = function (willPersist) {
			var valueDataJsonList = [];
			_.each(this._value, function (valueData) {
				valueDataJsonList.push(ValueDataUtils.toJSON(valueData));
			});
			return {
				value: valueDataJsonList
			};
		};

		/**
		 * @override
		 */
		SearchFacetParameter.prototype.configure = function (config) {
			var value = [];
			_.each(config.value, function (json) {
				var valueData = new ValueData();
				ValueDataUtils.fromJSON(valueData, json);
				value.push(valueData);
			});

			// when configuring we don't know the type (and we don't care actually)
			this.setFacetedFieldType(null);
			this.setValue(value);
		};

		/**
		 * make sure the __type property is not saved in the valuedata
		 * __type contains the datatype of the faceted field
		 * 
		 * @override
		 * @param {[[Type]]} value [[Description]]
		 */
		SearchFacetParameter.prototype.setValue = function (value) {
			if (value[0] && value[0].getValue && value[0].getValue("__type")) {
				this.setFacetedFieldType(value[0].getValue("__type"));
				value[0].setValue("__type", null);
			}

			Parameter.prototype.setValue.apply(this, arguments);

		};


		return SearchFacetParameter;
	});