define(["dashboard/builder/configuration/ParameterConfigurationPanel",
		"dashboard/builder/configuration/parameters/CascadingDropdownConfigurationPanel",
		"utils/Promisify",
		"component/searchresults/dataobjects/ValueData",
		"component/searchresults/dataobjects/ValueDataUtils",
		"jquery",
		"underscore",
		"utils/webi18n!SearchInputs",
		"utils/AjaxCache"
	],
	function (ParameterConfigurationPanel,
		CascadingDropdownConfigurationPanel,
		Promisify,
		ValueData,
		ValueDataUtils,
		$,
		_,
		SearchInputs,
		AjaxCache) {
		"use strict";

		/**
		 * Configuration panel for a SearchFacetParameter
		 * @augments ParameterConfigurationPanel
		 * @param {SearchFacetParameter} parameter 
		 */
		var SearchFacetParameterConfigurationPanel = function (parameter) {
			ParameterConfigurationPanel.call(this, parameter);
			this._parameter = parameter;
		};

		SearchFacetParameterConfigurationPanel.prototype = Object.create(ParameterConfigurationPanel.prototype);
		SearchFacetParameterConfigurationPanel.prototype.constructor = SearchFacetParameterConfigurationPanel;

		/**
		 * @private
		 */
		SearchFacetParameterConfigurationPanel.prototype._init = function () {
			var searchType = this._parameter.getSearchType();
			if (searchType) {
				this._facetTypeSelection = new CascadingDropdownConfigurationPanel(this._parameter, []);
				this.registerSubpanel(this._facetTypeSelection);
				this._element.append(this._facetTypeSelection.getElement());
				var dataPromise = Promisify.deferred(AjaxCache.ajax({
					url: "searchdataservice.wcr",
					method: "POST",
					dataType: "json",
					data: {
						action: "getfacetlist",
						searchtype: searchType,
						ajax: "1"
					}
				}));
				Promise.resolve(dataPromise).then(function (filterListJSON) {
					// check for any errors while fetching the facet list
					if (filterListJSON.error) {
						this._showErrorMessage(filterListJSON.error);
						return;
					}
					
					var facetData = filterListJSON.result;
					var facetListByGroup = facetData.facets;
					var defaultFacet = facetData.defaultfacet;

					// Build data for CascadingDropdownConfigurationPanel
					var optionData = [];
					_.each(_.allKeys(facetListByGroup), function (key, index) {
						var jsonFacets = facetListByGroup[key].facets;
						if (!_.isEmpty(jsonFacets)) {
							var facets = {
								value: facetListByGroup[key].id,
								text: facetListByGroup[key].label,
								children: []
							};

							_.each(jsonFacets, function (facet) {
								var facetinfo = {
									value: facet.facetId,
									text: facet.label
								};
								if (!_.isEmpty(facet.variations)) {
									facetinfo.children = [];
									_.each(facet.variations, function (variation) {
										facetinfo.children.push({
											value: variation.facetId,
											text: variation.label
										});
									});
								}
								facets.children.push(facetinfo);
							});

							optionData.push(facets);
						}
					});

					var prevCascadingDropdownValues = [];

					// Build cascading dropdowns + restore previously selected values
					if (this._parameter.getValue() && this._parameter.getValue()[0]) {
						var previousFacetId = this._parameter.getValue()[0].getValue("id");
						prevCascadingDropdownValues = this._findFacetGroup(previousFacetId, facetListByGroup);
					} else if(defaultFacet) {
						prevCascadingDropdownValues = this._findFacetGroup(defaultFacet, facetListByGroup);
					}

					this._facetTypeSelection.setOptions(optionData, prevCascadingDropdownValues);
					this._facetTypeSelection.on("changed", this._facetTypeSelectionChanged.bind(this, facetListByGroup));
					this._facetTypeSelectionChanged(facetListByGroup);
					this._observeParam(function () {
						var cascadingDropdownValues = [];
						if (this._parameter.getValue() && this._parameter.getValue()[0]) {
							var previousFacetId = this._parameter.getValue()[0].getValue("id");
							cascadingDropdownValues = this._findFacetGroup(previousFacetId, facetListByGroup);
						}
						this._facetTypeSelection.setSelectedValues(cascadingDropdownValues);
					}.bind(this));
				}.bind(this));
			}
		};

		/**
		 * Display error message in the FacetListBlock
		 * @param error
		 */
		SearchFacetParameterConfigurationPanel.prototype._showErrorMessage = function (error) {
			var errorTag = null;

			switch (error.code) {
			case "PERMISSION_DENIED_TAG":
				errorTag = "PERMISSION_DENIED_TAG";
				break;
			case "AUTHENTICATIONFAILED_TAG":
				errorTag = "AJAX_ERROR_NO_SESSION_TAG";
				break;
			}

			if (errorTag) {
				var errorMessage = $("<div>", {
					"class": "alert alert-danger alert-dismissible",
					"role": "alert"
				}).append(
					$("<button>", {
						"type": "button",
						"class": "close",
						"data-dismiss": "alert",
						"aria-label": "Close"
					}).append($("<i>", {
						"class": "icon icon-small icon-WCR_Close12"
					})),
					$("<span>").text(SearchInputs.t(errorTag))
				);

				this._element.prepend(errorMessage);
			}
		};

		/**
		 * Update the parameter when the selection changes
		 * @private
		 */
		SearchFacetParameterConfigurationPanel.prototype._facetTypeSelectionChanged = function (groups) {
			// when the selection changes, we need to update the parameter

			// the 'values" array only contains one element as we only support facetting on 1 field at the time
			var values = this._facetTypeSelection.getSelectedValues();
			var valueData = new ValueData();
			if (values.length > 2) {
				valueData.setValue("id", values[2].value);
				valueData.setValue("label", values[1].label + " - " + values[2].label);
			} else if (values.length > 1) {
				valueData.setValue("id", values[1].value);
				valueData.setValue("label", values[1].label);
			}
			// temporary store the type of the faceted field
			valueData.setValue("__type", this._findFacetType(valueData.getValue("id"), groups));

			var previousValueData = this._parameter.getValue()[0];
			if (!previousValueData || !ValueDataUtils.equals(previousValueData, valueData)) {
				this._parameter.undoableSetValue([valueData]);
			}
		};

		/**
		 * Looks up the facet group of a given facet id.
		 * @private
		 * @param   {string} facetId
		 * @param   {object} groups
		 * @returns {Array} the corresponding groups
		 */
		SearchFacetParameterConfigurationPanel.prototype._findFacetGroup = function (searchFacetId, groups) {
			var result = [];
			_.each(groups, function (group) {
				if (result.length > 0) {
					return;
				}
				_.each(group.facets, function (facet) {
					if (result.length > 0) {
						return;
					}
					if (facet.facetId === searchFacetId) {
						result = [group.id, facet.facetId];
					} else {
						_.each(facet.variations, function (variation) {
							if (variation.facetId === searchFacetId) {
								result = [group.id, facet.facetId, variation.facetId];
							}
						});
					}
				});
			});
			return result;
		};

		/**
		 * Find the datatype of the faceted field
		 * @private
		 * @param   {String} searchFacetId 
		 * @param   {Object}  groups        
		 * @returns {String} the type of the field (or null if not found)
		 */
		SearchFacetParameterConfigurationPanel.prototype._findFacetType = function (searchFacetId, groups) {
			var type = null;
			_.each(groups, function (group) {
				if (type != null) {
					return;
				}
				_.each(group.facets, function (facet) {
					if (type != null) {
						return;
					}
					if (facet.facetId === searchFacetId) {
						type = facet.type;
					} else {
						_.each(facet.variations, function (variation) {
							if (variation.facetId === searchFacetId) {
								type = facet.type; // type is not defined in the variation
							}
						});
					}
				});
			});
			return type;

		};

		return SearchFacetParameterConfigurationPanel;
	});