define(["dashboard/core/parameters/CompositeParameter",
		"component/searchresults/dataobjects/ValueData",
		"component/searchresults/dataobjects/ValueDataUtils",
		"underscore"
	],
	function (CompositeParameter,
		ValueData,
		ValueDataUtils,
		_) {
		"use strict";

		/**
		 * Search Instanced Field Configuration Parameter
		 * 
		 * It is a parameter to configure an instanced field in the searchmodel	
		 * The parameter has subparameters that directly affect the configuration in the SearchInstancedFieldConfiguration
		 * The name of each subparameter is mapped on a property of the valuedata object. The value of the subparameter is put directly in the valuedata
		 * The value of subparameters should either be a String or Array.<ValueData>
		 * 
		 * When the block having this parameter is removed or detached, the instanced configuration is also removed/adapted
		 * The IFCParameter will continously sync changes in subparameters to the searchmodel
		 * on intiialization or when setting the instance ide, the configuration is pulled from the searchmodel and put in to the parameters
		 * 
		 * @augments CompositeParameter
		 * @param {String}                 name             - name of the parameter
		 * @param {String}                 userFriendlyName - user friendly representation of the parameter
		 * @param {SearchContextParameter} searchContext    - search context in which the instanced configuratio nwill be stored
		 * @param {String}                 instancedFieldId - the field which will be instanced by this parameter
		 */
		var SearchIFCParameter = function (name, context, userFriendlyName, searchContext, instancedFieldId) {
			CompositeParameter.call(this, name, context, "searchresults/dashboard/parameters/SearchIFCParameter", userFriendlyName);

			/**
			 * @memeber {String} the search field that will be instanced
			 */
			this._instancedFieldId = instancedFieldId;

			/**
			 * @member {SearchContextParameter} 
			 */
			this._searchContext = searchContext;
			this._searchContext.on("changed", this._syncWithSearchModel.bind(this));


			/**
			 * @member {Boolean} Prevent synchronization while initializing
			 */
			this._isInitialized = false;


			/**
			 * @member {Boolean} this flag indicates that a configuration action is ongoing, it will prevent update storms in the sub parameters
			 */
			this._isConfiguring = false;

			/**
			 * @member {string} 
			 */
			this._instanceId = null;

			/**
			 * @member {boolean} is the parameter attached to the board (or temporarily removed?)
			 */
			this._isAttached = true;

			this._instancesToBeRemoved = [];



			// when a subparameter changes, push to the search model
			this._parameters.on("changed.*", function () {
				if (!this._isConfiguring) {
					this._pushToSearchModel();
				}
			}.bind(this));
		};

		SearchIFCParameter.prototype = Object.create(CompositeParameter.prototype);
		SearchIFCParameter.prototype.constructor = SearchIFCParameter;

		/**
		 * Call once all subparameters are defined and configured
		 * An initial sync with the search modal will be executed
		 */
		SearchIFCParameter.prototype.doInit = function () {
			if (!this._isInitialized) {
				this._isInitialized = true;
				this._syncWithSearchModel();
			}

		};
		/**
		 * Set the configuration
		 * @param {object} configuration - { instanceId: <string>, inctanceConfig: <ValueData> }
		 */
		SearchIFCParameter.prototype.configure = function (configuration) {

			if (configuration.instanceId) {
				this._setInstanceId(configuration.instanceId);
			}



			if (configuration.valueData) {
				// make sure to ignore all incoming change requests
				this._isConfiguring = true;

				var valueData = ValueDataUtils.fromJSON(new ValueData(), configuration.valueData);

				this._pushValueDataToParameters(valueData);

				this._pushToSearchModel();

				this._isConfiguring = false;
			}
		};


		/**
		 * Get the Plain JSON object for the configuration of the parameter
		 * When persisting, we only store the instance id (the configuration of subparameters sits in the searchmodel in the instanced field configuration)
		 * When not persisting, we also need to store the configuration of subparameters, this is then used to inject in parameters when for example an undo action occurs
		 * @private
		 * @param   {Boolean} willPersist 
		 * @returns {object}  
		 */
		SearchIFCParameter.prototype._getParameterSpecificConfig = function (willPersist) {
			if (willPersist) {
				return {
					instanceId: this._instanceId
				};
			} else {
				return {
					instanceId: this._instanceId,
					valueData: ValueDataUtils.toJSON(this._getValueDataFromParameters())
				};
			}
		};


		/**
		 * Switch the instance ID of the instanced field
		 * This method will remove existing instancs if needed
		 * @param {String} instanceId 
		 */
		SearchIFCParameter.prototype._setInstanceId = function (instanceId) {
			if (this._instanceId !== instanceId) {
				this._removeInstanceFromSearchModel();

				this._instanceId = instanceId;
				this._emitChanged();
			}
			this._syncWithSearchModel();
		};

		/**
		 * Method that syncs with the search model
		 * 
		 * If the search context is set, the configuration will be synched with the SearchInstancedFieldConfiguration
		 * if a block is no longer attached, the linked instance is removed from the SearchInstancedFieldConfiguration
		 * @private
		 */
		SearchIFCParameter.prototype._syncWithSearchModel = function () {
			if (this._isInitialized && this._searchContext.getValue()) {
				this._cleanUpInstancesFromSearchModel();

				//we can only do something if there is a search context...
				if (this._isAttached) {
					// we are still an valid/active block
					if (this._instanceId) {
						this._pullFromSearchModel();
					} else {
						// we are not connected yet
						this._pushToSearchModel();
					}
				} else {
					// we should no longer be linked to the search model
					this._removeInstanceFromSearchModel();
				}
			}

		};

		/**
		 * Clean up previously linked instances from the searchmodel
		 * A double-check on unused instances will be performed when saving the dashboard
		 * @private
		 */
		SearchIFCParameter.prototype._cleanUpInstancesFromSearchModel = function () {
			if (this._isInitialized && this._searchContext.getValue()) {
				_.each(this._instancesToBeRemoved, function (instanceId) {
					this._searchContext.getValue().removeInstanceConfig(instanceId);
				}, this);

				this._instancesToBeRemoved = [];
			}

		};


		/**
		 * If the parameter is currently linked to the searchmodel. The linked instance is removed from the searchmodel
		 * @private
		 */
		SearchIFCParameter.prototype._removeInstanceFromSearchModel = function () {
			if (this._isInitialized && this._instanceId) {
				this._instancesToBeRemoved.push(this._instanceId);
				this._cleanUpInstancesFromSearchModel();

				this._instanceId = null;
			}
		};

		/**
		 * Push value data to the subparameters
		 * @private
		 * @param {ValueData} valueData 
		 */
		SearchIFCParameter.prototype._pushValueDataToParameters = function (valueData) {
			// run over sub parameters of the type Array.<ValueData>
			_.each(valueData.getSubListKeys(), function (subListKey) {
				if (this._parameters.getParameter(subListKey)) {
					this._parameters.setValue(subListKey, valueData.getSubList(subListKey));
				}
			}, this);

			// run over sub parameters of the type String 
			_.each(valueData.getValueKeys(), function (valueKey) {
				if (this._parameters.getParameter(valueKey)) {
					this._parameters.setValue(valueKey, valueData.getValue(valueKey));
				}
			}, this);
		};


		/**
		 * The parameter (or block with this parameter) was attached to the dashboard	
		 */
		SearchIFCParameter.prototype.attach = function () {
			this._isAttached = true;
			this._syncWithSearchModel();
		};

		/**
		 * The parameter (or block with this parameter) was detached from the dashboard
		 */
		SearchIFCParameter.prototype.detach = function () {
			this._isAttached = false;
			this._syncWithSearchModel();
		};

		/**
		 * Run over all sub parameters and build the ValueData object for the instanced configuration
		 * @private
		 * @returns {ValueData} ValueData for the instance in SearchInstancedFieldConfiguration
		 */
		SearchIFCParameter.prototype._getValueDataFromParameters = function () {
			var valueData = new ValueData();
			_.each(this.getParameterList(), function (parameter) {
				var value = parameter.getValue();

				if (_.isArray(value)) {
					valueData.setSubList(parameter.getName(), value);
				} else {
					valueData.setValue(parameter.getName(), value);
				}
			});

			return valueData;
		};

		/**
		 * Push the data from the sub parameters to the SearchModel
		 * @private
		 */
		SearchIFCParameter.prototype._pushToSearchModel = function () {
			if (this._isInitialized && this._searchContext.getValue()) {
				var newData = this._getValueDataFromParameters();
				if (this._instanceId) {
					// we already are connected
					this._searchContext.getValue().setInstanceConfig(this._instanceId, newData);
				} else {
					// we are not yet connected
					this._instanceId = this._searchContext.getValue().addInstanceConfig(this._instancedFieldId, newData);
				}
			}
		};

		/**
		 * Pull the data from the searchmodel to the parameters	
		 * @private
		 */
		SearchIFCParameter.prototype._pullFromSearchModel = function () {
			if (this._isInitialized && this._instanceId && this._searchContext.getValue()) {
				var valueData = this._searchContext.getValue().getInstanceConfig(this._instanceId);
				if (valueData) {
					this._pushValueDataToParameters(valueData);
				}
			}
		};

		/**
		 * Get the currently linked instancedid
		 * @returns {String} 
		 */
		SearchIFCParameter.prototype.getInstanceId = function () {
			return this._instanceId;
		};

		return SearchIFCParameter;
	});