/**
 * An object containing additional properties of a Search Field to be processed/rendered.
 * 
 * @author JAMI
 */
define(["underscore"], function(_){
	
	"use strict";
	
	/**
	 * @constructor
	 */
	var SearchFieldProperties = function(){
		
		// Unit code (originally retrieved from the DB) that can be used to fetch its localized variant from the language file (can be null)
		this.displayUnit = null;
		
		// Numeric Unit value conversion factor > used to convert a numeric unit field to the User's preferred unit-type (imperial VS metric)
		this.unitConversionFactor  = 1;
		
		// Data type of the field...
		this.dataType = "string";
		
		// Display Configuration > a ValueData object contain additional information about the field to configure the renderer/parameter field
		this.displayConfig = null;
		
		// ExpandMode instance... how to show additional information? (see package "expandmodes")
		// See showExpandedInformationArea function below...
		this.expandMode = null;
		
		// Child search instance. function to be called to do a child search. It should be set when creating the SearchFieldProperties
		this.childSearch = null;
		// Following lists are to be used to fill up static dropdowns for a parameter field of such a type
		// Both are optional (only specific fields support those) and both should have the following structure:
		// Array of ValueData objects. Expected values of each of the sub-list items: label; name; value (+ potentially more, see SearchValueListConstants.java)
		
		// Only if a parameter_field renderer of this field needs special static List Values (array of ValueData)
		this.staticListValues = null;
		
		// List Values that are loaded from the DB and make up the List Value items of this field (array of ValueData)
		this.listValues = null;
		
		// facets Ids for facet field
		this.facetIds = [];
		
		// functions to get additional information about the facets
		// get the facet name
		// Function(String):String
		this.getFacetName = null;
		// get the data for a facet value
		// Function(String):ValueData
		this.getFacetValueData = null;// TODO
		
		// Can the field have multiple values at the same time
		this.isMultiValued = null;
	};
	
	/**
	 * Is this field numeric?
	 * 
	 * @returns {Boolean} true if the field represents a number.
	 */
	SearchFieldProperties.prototype.isNumeric = function() {
		return this.dataType === "float" || this.dataType === "int";
	};
	
	/**
	 * Get the Data Type code
	 * 
	 * @returns {String} supported values: string; float; int; date
	 */
	SearchFieldProperties.prototype.getDataType = function() {
		return this.dataType;
	};
	
	/**
	 * When we format, how many decimals do we want?
	 * 
	 * @returns {Number} - the number of decimals.
	 */
	SearchFieldProperties.prototype.getDecimals = function() {
		if(this.dataType === "float") {
			return 3;
		}else{
			return 0;
		}
	};
	
	/**
	 * Returns a list of ValueData objects that are used to fill up static dropdown selector of a Parameter Field (s.a. Project/Task Status; Project Deliverables, etc.)
	 * 
	 * @returns {ValueData} object with a single "items" sublist; OPTIONAL > can be null
	 */
	SearchFieldProperties.prototype.getListValues = function() {
		return _.filter(
				this.staticListValues.concat(this.listValues), 
				// Some items might contain a deleted property. Don't show those in dropdowns.
				function(item){return item.getValue("deleted")!=="1" && item.getValue("enabled")!=="0";});
	};
	
	
	/**
	 * Utility method to show an additional area with information in the context of this field.
	 * 
	 * In most cases this will simply make a popup with the title being the name of the field.
	 * However, in some places it will actually do something different. 
	 * In List Search Results for example, it will show the content below the item, inside the table. (like for Bill of Materials)
	 * 
	 * The generateContent function takes two arguments:
	 * - the jQuery element to put the content into
	 * - close = the function to call to close the popup
	 * 
	 * Supported options:
	 *  - title => override the title of the popup (note: title is not guaranteed to be visible)
	 *  - subtitle => if not null, adds a subtitle.
	 *  - toggleid => pass an id. If you re-open the popup with the same toggleid while it is still open, it will close the popup instead.
	 *  - closebuttontitle => override the description of the close button (note: not guaranteed to be visible)
	 * 
	 * @param {Function(jQuery, Function())} generateContent - function which returns the content to show in the area/popup. 
	 * @param {Object} options - additional options
	 */
	SearchFieldProperties.prototype.showExpandedInformationArea = function(generateContent, options) {
		this.expandMode.show(generateContent, options);
	};
	
	/**
	 * Method to do a saved search that will be merged into the table
	 * @param {String} saved search to execute
	 * @param {String} projectId - context of the saved search
	 */
	SearchFieldProperties.prototype.doSavedSearch = function(savedSearch, projectId){
		if(this.childSearch) {
			return this.childSearch(savedSearch, projectId);
		}
	};
	
	// return the constructor
	return SearchFieldProperties;
});