/**
 * An object containing SearchSettings which will NOT change while on the page...
 */
define(["./ExternalSearchSettings"], function(ExternalSearchSettings){
	
	/**
	 * @constructor
	 * @param {Boolean} whether SmartNames are enabled
	 * @param {ExternalSearchSettings} settings which are more general in WebCenter (not specific to search)
	 */
	var SearchSettings = function(isSmartNamesEnabled, externalSettings, watchCriterionFunc, unwatchCriterionFunc){
		// Are SmartNames enabled?
		// Data will be "SmartNameEncoded" (different characters to escape) when this is enabled.
		this._isSmartNamesEnabled = isSmartNamesEnabled;
		
		//nested ExternalSearchSettings object that contains further indirect settings relevant to Search environment
		this._externalSettings = externalSettings;
		
		//
		// Methods for watching & unwatching other criterion objects... (read-only)
		//
		this._watchCriterionFunc = watchCriterionFunc;
		this._unwatchCriterionFunc = unwatchCriterionFunc;
	};
	
	/**
	 * @returns {Boolean}
	 */
	SearchSettings.prototype.isSmartNamesEnabled = function() {
		return this._isSmartNamesEnabled;
	};
	
	/**
	 * @returns {ExternalSearchSettings}
	 */
	SearchSettings.prototype.getExternalSettings = function() {
		return this._externalSettings;
	};
	
	/**
	 * Register a callback which needs to be called whenever another criterion changes values.
	 * The callback will always be called AT LEAST once.
	 * 
	 * In case the criterion does not exist, the callback will be called with null. Otherwise the ValueData (read-only) object will be passed.
	 * 
	 * @param {String} otherFieldId - the fieldid of the criterion to watch
	 * @param {function(ValueData)} callback - the callback which needs to be called when the information for the field changes.
	 */
	SearchSettings.prototype.watchCriterion = function(otherFieldId, callback) {
		this._watchCriterionFunc(otherFieldId, callback);
	};
	
	/**
	 * Unegister a watch callback.
	 * 
	 * @param {String} otherFieldId - the fieldid of the criterion to unwatch
	 * @param {function(ValueData)} callback - the same callback that was registered before
	 */
	SearchSettings.prototype.unwatchCriterion = function(otherFieldId, callback) {
		this._unwatchCriterionFunc(otherFieldId, callback);
	};
	
	// return the constructor
	return SearchSettings;
});