/**
 * Pass structured data between front-end and back-end.
 * 
 * Has a Java counterpart: "ValueData.java"
 * 
 * THE STRUCTURE OF THIS OBJECT IS VERY STRICT BECAUSE IT NEEDS TO BE ABLE TO TRANSFER TO THE SERVERSIDE.
 */
define(["jquery", "underscore"], function($, _){
	"use strict";
	
	/**
	 * @constructor
	 */
	var ValueData = function(){
		//
		// Internal data...
		//
		// Fields in this object can either contain a "string" 
		// or an array of ValueData objects. Nothing else is allowed.
		//
		// Use getters and setters to modify the data.
		//
		this._attr = {};
		this._lists = {};
	};
	
	/**
	 * Clears the ValueData
	 */
	ValueData.prototype.clear = function(){
		this._attr = {};
		this._lists = {};
	};
	
	/**
	 * Get the list of simple string values which are set
	 * 
	 * @returns {String[]} returns an array of strings with the keys... 
	 */
	ValueData.prototype.getValueKeys = function(){
		var keys = [];
		for(var key in this._attr) {
			if(this._attr.hasOwnProperty(key) 
				&& this._attr[key]!==undefined 
				&& this._attr[key]!==null 
				&& (typeof this._attr[key]) === "string") {
					keys.push(key);
			}
		}
		return keys;
	};
	
	/**
	 * Get a simple string value
	 * 
	 * @param {String} id - the id of the field containing the simple value
	 * @param {String} defaultVal - [OPTIONAL (default: null)] the value to use when this simple value is not defined.
	 * @returns {String} the value which is set (string) or null if no value is set.
	 */
	ValueData.prototype.getValue = function(id, defaultVal) {
		defaultVal = (defaultVal===undefined?null:defaultVal);
		//
		// Get the value from this._attr.
		// But do some checks first for safety
		//
		if(this._attr[id]===undefined || this._attr[id]===null) {
			// Return undefined if there is no value
			return defaultVal;
		}else{
			if((typeof this._attr[id]) === "string"){
				return this._attr[id];
			}else{
				throw new Error("Illegal state... Should never happen.");
			}
		}
	};
	
	/**
	 * Set a simple string value
	 * 
	 * @param {String} id
	 * @param {String} value - the new value. Setting "null" clears the simple value.
	 */
	ValueData.prototype.setValue = function(id, value) {
		if(!(value===undefined || value===null || (typeof value) === "string")) {
			throw new Error("Only allowed to set string values using \"ValueData.setValue\".");
		}
		
		this._attr[id] = value;
	};
	
	/**
	 * Get the list of sublist values which are set
	 * 
	 * @returns {String[]} returns an array of strings with the keys... 
	 */
	ValueData.prototype.getSubListKeys = function(){
		var keys = [];
		for(var key in this._lists) {
			if(this._lists.hasOwnProperty(key) 
					&& this._lists[key]!==undefined 
					&& this._lists[key]!==null 
					&& this._lists[key] instanceof Array) {
				keys.push(key);
			}
		}
		return keys;
	};
	
	/**
	 * Get a sublist
	 * 
	 * Sublist = Array of ValueData objects.
	 * 
	 * @param {String} id - the id of the field containing the simple value
	 * @returns {ValueData[]} returns an Array of ValueData objects.
	 */
	ValueData.prototype.getSubList = function(id) {
		//
		// Get the value from this._lists.
		// But do some checks first for safety
		//
		if(this._lists[id]===undefined || this._lists[id]===null) {
			// Return undefined if there is no value
			return null;
		}else{
			if(this._lists[id] instanceof Array){
				// Note: we could check here also that this is an array of ValueData objects...
				return this._lists[id];
			}else{
				throw new Error("Illegal state... Should never happen.");
			}
		}
	};
	
	/**
	 * Adds a new item to the list with a certain id.
	 * The list will be created if it does not exist.
	 * 
	 * @param {String} id - the id of the list
	 * @returns {ValueData} the new item which is added to the list.
	 */
	ValueData.prototype.addNewItemToList = function(id) {
		//
		// Set the value from this._lists.
		// But do some checks first for safety
		//
		if(this._lists[id]===undefined || this._lists[id]===null) {
			this._lists[id] = [];
		}
		
		var newObject = new ValueData();
		this._lists[id].push(newObject);
		return newObject;
	};
	
	/**
	 * Set a sublist
	 * 
	 * ! Should be an Array of ValueData objects!
	 * 
	 * @param {String} id - the id of the list
	 * @param {ValueData[]} value - the new value. An array of ValueData objects. Setting "null" clears the field containing the list.
	 */
	ValueData.prototype.setSubList = function(id, value) {
		//
		// Set the value from this._lists.
		// But do some checks first for safety
		//
		if(!(value===undefined || value===null || value instanceof Array)) {
			throw new Error("Only allowed to set list values using \"ValueData.setSubList\".");
		}
		// Should we check if all items are ValueData ?????
		// Or do we assume the person which uses this class makes sure it is like that?
		
		this._lists[id] = value;
	};
	
	return ValueData;
});