/**
 * Some utils to work with a ValueData object
 * 
 * Has a Java counterpart: "ValueDataUtils.java"
 */
define(["./ValueData"], function(ValueData){
	
	var utils = {};
		
	/**
	 * Convert the ValueData to JSON
	 * 
	 * Note: after calling this method, 
	 * you still need to use JSON.stringify(obj) 
	 * to convert the value to a string.
	 * 
	 * @param {ValueData} data
	 * @returns {Object} the JSON
	 */
	utils.toJSON = function(data) {
		if(!(data && data instanceof ValueData)) {
			throw new Error("Illegal argument in ValueDataUtils.toJSON");
		}
		
		var attr = null;
		var lists = null;
		// Copy the simple string values
		var vkeys = data.getValueKeys();
		for(var i=0;i<vkeys.length;i++) {
			var key = vkeys[i];
			if(attr===null){
				attr = {};
			}
			attr[key] = data.getValue(key);
		}
		// Copy the sublists (which are arrays of ValueData)
		var slkeys = data.getSubListKeys();
		for(var i=0;i<slkeys.length;i++) {
			var key = slkeys[i];
			var sl = data.getSubList(key);
			var newsl = [];
			for(var a=0;a<sl.length;a++) {
				newsl.push(utils.toJSON(sl[a]));
			}
			if(lists===null){
				lists = {};
			}
			lists[key] = newsl;
		}
		
		var obj = {};
		if(attr!==null){
			obj["attr"]=attr;
		}
		if(lists!==null){
			obj["list"]=lists;
		}
		
		return obj;
	};
	
	/**
	 * Import the ValueData from JSON
	 * 
	 * Note: json should be already parsed using $.parseJSON(<string>);
	 * 
	 * @param {ValueData} data - an empty ValueData object which will be filled
	 * @param {Object} obj
	 */
	utils.fromJSON = function(data, obj) {
		if(!(data && data instanceof ValueData)) {
			throw new Error("Illegal argument in ValueDataUtils.fromJSON");
		}
		
		var attr = obj["attr"];
		
		if(attr){
			for(var key in attr) {
				if(attr.hasOwnProperty(key)) {
					data.setValue(key, attr[key]);
				}
			}
		}
		
		var lists = obj["list"];
		
		if(lists){
			for(var key in lists) {
				if(lists.hasOwnProperty(key)) {
					var jsonarr = lists[key];
					var sublist = [];
					for(var i=0;i<jsonarr.length;i++) {
						var item = jsonarr[i];
						var valdataItem = new ValueData();
						utils.fromJSON(valdataItem, item);
						sublist.push(valdataItem);
					}
					
					data.setSubList(key, sublist);
				}
			}
		}
	};
		
	/**
	 * Make a deep clone of the ValueData
	 * 
	 * @param {ValueData} data
	 * @returns {ValueData} the clone
	 */
	utils.clone = function(data) {
		if(!(data && data instanceof ValueData)) {
			throw new Error("Illegal argument in ValueDataUtils.clone");
		}
		
		var clone = new ValueData();
		utils.fromJSON(clone, utils.toJSON(data));
		return clone;
	};
		
	/**
	 * Checks if the two instances of ValueData are equal.
	 * 
	 * [Currently, this method is implemented for the JavaScript version of ValueData only]
	 * 
	 * @param {ValueData} data
	 * @param {ValueData} other
	 * @returns {boolean} whether the two instances are equal.
	 */
	utils.equals = function(data, other) {
		if(!(data && data instanceof ValueData)) {
			throw new Error("Illegal argument in ValueDataUtils.equals");
		}
		if(!(other && other instanceof ValueData)) {
			throw new Error("Illegal argument in ValueDataUtils.equals");
		}
		
		var equals = true;
		
		// Check the simple string values
		var vkeysA = data.getValueKeys();
		var vkeysB = other.getValueKeys();
		equals = equals && (vkeysA.length === vkeysB.length);
		vkeysA.sort();
		vkeysB.sort();
		for(var i=0;i<vkeysA.length;i++) {
			var key = vkeysA[i];
			var keyB = vkeysB[i];
			equals = equals && (key === keyB);
			equals = equals && (data.getValue(key) === other.getValue(key));
		}
		// Check the sublists (which are arrays of ValueData)
		var slkeysA = data.getSubListKeys();
		var slkeysB = other.getSubListKeys();
		equals = equals && (slkeysA.length === slkeysB.length);
		slkeysA.sort();
		slkeysB.sort();
		for(var i=0;i<slkeysA.length;i++) {
			var key = slkeysA[i];
			var keyB = slkeysB[i];
			equals = equals && (key === keyB);
			var slA = data.getSubList(key);
			var slB = other.getSubList(key);
			equals = equals && (slA.length == slB.length);
			for(var a=0;a<slA.length;a++) {
				equals = equals && utils.equals(slA[a], slB[a]);
			}
		}
		
		return equals;
	};
	

	/**
	 * Set an array of strings as value of a certain keuy
	 * @param   {ValueData}      data    
	 * @param   {String}         key     
	 * @param   {Array.<String>} strings 
	 */
	utils.setStringList = function(data, key, strings){
		data.setSubList(key, _.map(strings, function(string){
			var data = new ValueData();
			data.setValue("v", string);
			return data;
		}));
	};
	
	/**
	 * Read an array of strings (set using setStringList)
	 * @param   {ValueData}      data 
	 * @param   {String}         key  
	 * @returns {Array.<String>} 
	 */
	utils.getStringList = function(data, key){
		var list = data.getSubList(key);
		if (list){
			return _.map(list, function(entry){
				return entry.getValue("v");
			});	
		} else {
			return [];
		}
	};
	
	return utils;
});
