/**
 * "Approval Status" Display Method
 * 
 * Shows an icon-based representation of the effective Approval Status of the Document 
 * and/or the "Approval Actions" available to the currently logged in User
 * 
 * @author JAMI
 */
define(["jquery", 
        "underscore", 
        "utils/webi18n!SearchInputs",
        "wcr/approvalInfoPopup"], 
        function($,
        		 _, 
        		 i18n){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
			
			//retrieve the required parameters
			var docVersionPropsList = data.getSubList("docversioninfo");
			var parentProjectPropsList = data.getSubList("projectinfo");
			
			if (!docVersionPropsList || docVersionPropsList == null || !parentProjectPropsList || parentProjectPropsList == null) {
				throw new Error("Missing information about document version & project.");
			} else {
				docVersionPropsList = docVersionPropsList[0];
				parentProjectPropsList = parentProjectPropsList[0];
			}
			
			//document related properties
			var docVersionID = docVersionPropsList.getValue("id"); 
			var docName = docVersionPropsList.getValue("document_name");
			var docApprovalStateID = docVersionPropsList.getValue("approval_state");
			var appCycleStatusName = docVersionPropsList.getValue("approval_cycle_state");
			var isDocURL = !(docVersionPropsList.getValue("document_is_url") && docVersionPropsList.getValue("document_is_url") === "0");
			
			//project related properties
			var isParentProjActive = !(parentProjectPropsList.getValue("is_active") && parentProjectPropsList.getValue("is_active") === "0");
			var isTemplate = !(parentProjectPropsList.getValue("is_template") && parentProjectPropsList.getValue("is_template") === "0");
			
			//get specifics in case of multi-value (currently only a single App Status per row is supported)
			var appStatusvalues = data.getSubList(fieldname);
			if(appStatusvalues){
				// We have data (should always be only a single value, but we iterate anyways)
				for(var i=0;i<appStatusvalues.length;i++) {
					
					//create the container div 
					var approvalStatusCellContainer = $("<div>");
					
					//center-align the container if using list layout
					if (tableDisplaySettings.resultsStyle === "list") {
						approvalStatusCellContainer.attr("align", "center");
					};
					
					$div.append(approvalStatusCellContainer);
					
					

					
					var approvalPropsList = appStatusvalues[i];
					var isApprovalOwner = !(approvalPropsList.getValue("is_approval_owner") && approvalPropsList.getValue("is_approval_owner") === "0");
					var hasChangeApprovalPerm = !(approvalPropsList.getValue("has_change_approval_permission") && approvalPropsList.getValue("has_change_approval_permission") === "0");
					var isUserPMofParentProject = !(approvalPropsList.getValue("is_project_manager_current_project") && approvalPropsList.getValue("is_project_manager_current_project") === "0");
					var isDocumentApprover = !(approvalPropsList.getValue("is_doc_approver") && approvalPropsList.getValue("is_doc_approver") === "0");
					var hasDefaultApprovalSetup = !(approvalPropsList.getValue("def_approval_setup_exists") && approvalPropsList.getValue("def_approval_setup_exists") === "0");
					
					var isAdmin = searchSettings.getIsAdmin();
					
					//--------------------------------------------------------------
					//Approval Status Icon (only if a valid app. cycle state exists)
					if (appCycleStatusName  &&
						(appCycleStatusName === "cycle_started" || appCycleStatusName === "cycle_finished" || appCycleStatusName === "cycle_stopped")) 
					{	
						var appStatusIconContainer = $("<div>");
						var approvalIconElem = $("<i>");
						
						//------------------------------------------------------------
						//Approval Details popup (if Document Version ID was supplied)
						if (docVersionID && docVersionID.length > 0) {
							//prepare popup header label
							var headerText = i18n.t("APPROVAL_SETUP");
							if (docName && docName.length > 0) {
								headerText = headerText.concat("   "+docName);
							}
							
							//event handlers
							approvalIconElem.on("click", function (event) {
								showapprovaldetails(this, event, docVersionID, headerText);
							});
							
							approvalIconElem.on("mouseout", function (event) {
								dynamichide(event);
							});
						}
						
						//set the proper icon-class and hover-label
						var iconClassVal = "icon ";
						var hoverLabelVal = "";
						if (appCycleStatusName && appCycleStatusName === "cycle_stopped") {
							iconClassVal = iconClassVal.concat("icon-WCR_Stopped icon-blue");
							hoverLabelVal = i18n.t("APPROVAL_STATUS_HOLD");
						} else {
							if (docApprovalStateID && docApprovalStateID.length > 0) {
								switch (docApprovalStateID) {
									case "00001_0000000001" : //Rejected
										iconClassVal = iconClassVal.concat("icon-WCR_Rejected icon-red");
										hoverLabelVal = i18n.t("APPROVAL_REJECTED");
										break;
									case "00001_0000000002" : //Pending
										iconClassVal = iconClassVal.concat("icon-WCR_Pending icon-blue");
										hoverLabelVal = i18n.t("APPROVAL_PENDING");
										break;
									case "00001_0000000003" : //Approved
										iconClassVal = iconClassVal.concat("icon-WCR_Approved icon-green");
										hoverLabelVal = i18n.t("APPROVAL_APPROVED");
										break;
									case "00001_0000000004" : //Force Rejected
										iconClassVal = iconClassVal.concat("icon-WCR_ForceReject icon-red");
										hoverLabelVal = i18n.t("APPROVAL_FORCED_REJ");
										break;
									case "00001_0000000005" : //Force Approved
										iconClassVal = iconClassVal.concat("icon-WCR_ForceApprove icon-green");
										hoverLabelVal = i18n.t("APPROVAL_FORCED_APP");
										break;
									case "00001_0000000006" : //Approved with Comments
										iconClassVal = iconClassVal.concat("icon-WCR_ApprovedConditional icon-green");
										hoverLabelVal = i18n.t("APPROVAL_APPROVED_COMMENTS");
										break;
									case "00001_0000000007" : //Force Approved with Comments
										iconClassVal = iconClassVal.concat("icon-WCR_ApprovedConditional icon-green");
										hoverLabelVal = i18n.t("FORCED_APP_COMMENTS");
										break;
								}
							}
						}
						
						//set the icon and hover text to the Icon elem
						approvalIconElem.addClass(iconClassVal);
						if (hoverLabelVal && hoverLabelVal.length > 0) {
							approvalIconElem.attr("alt", hoverLabelVal).attr("title", hoverLabelVal);
						}
						
						//add the icon element into the container
						appStatusIconContainer.append(approvalIconElem);
						approvalStatusCellContainer.append(appStatusIconContainer);
					}
					
					//--------------------------------------------------------------
					//Approval Actions/Links
					if (!isDocURL && isParentProjActive) {
						var approvalActionLinksContainer = $("<div>").appendTo(approvalStatusCellContainer);
						
						//note: whatever the action we always want to just forward to "Document Approval" page 
						var approvalActionLinkElem = $("<a>", {"href" : "docdetailsapprovers.jsp?docVerID="+docVersionID+"&menu_file="+searchSettings.getMenuFileURL()})
						.addClass("search_result_link")
						.appendTo(approvalActionLinksContainer);
						
						//change the 
						if (!docApprovalStateID || docApprovalStateID.length === "0" || appCycleStatusName === "no_approval_rule") {
							//Approval has not been setup

							if (hasChangeApprovalPerm || isUserPMofParentProject || isAdmin) {
								//'Setup Approval Cycle' link > only if the User has enough permissions	
								approvalActionLinkElem.text(i18n.t("DOCDETAILSSETUPAPPROVAL_TAG"));
								// append '&autoSetupSingleStage=1' to auto create an approval setup with 1 stage
								approvalActionLinkElem.attr("href", approvalActionLinkElem.attr("href") + "&autoSetupSingleStage=1");
							} else if (hasDefaultApprovalSetup && isApprovalOwner && !hasChangeApprovalPerm && !isTemplate 
										&& !(isUserPMofParentProject || isAdmin)) {
								//'Start Approval Cycle' link (only shown for a User Member without 'Update Approval Cycle' who is the "Version Author") 
								//AND when the project/folder has an approval setup defined
								approvalActionLinkElem.text(i18n.t("APPROVAL_STATUS_STARTCYCLE"));
							}
						} else {
							//Approval has been setup
							
							if (appCycleStatusName === "cycle_stopped" && isApprovalOwner && !isTemplate) {
								//Allow to start and clear a cycle for approval owners when	at least one approver is specified
								approvalActionLinkElem.text(i18n.t("APPROVAL_STATUS_STARTCYCLE"));
							} else if (appCycleStatusName === "cycle_started" && 
										(isDocumentApprover ||(isUserPMofParentProject && (searchSettings.getIsForceApprovalAllowed() || searchSettings.getIsForceRejectionAllowed())))) {
								//If status is pending and a user is an approver of the doc then show Approve/Reject link
								//DT7642: if the user is project manager, also allow him/her to approve the document
								approvalActionLinkElem.text(i18n.t("DOCDETAILSAPPROVE_TAG"));
							}
						}
					}
				}
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
            
            return {overflowsnever: true};
		}
	};
});