/**
 * "Bill of Materials" Display Method
 * 
 * Relevant for Project Search.
 * Displays an overview of BOM-Documents status (total doc count, per-approval status overview, etc.) for the specific Project.
 * Also registers expandable view for each of the icons to open more details.
 * 
 * @author JAMI
 */
define(["jquery", "underscore", "utils/webi18n!SearchInputs"], function($, _, i18n){
	
	/**
	 * Generate the content of the bill of materials row...
	 * 
	 * @param {FieldProperties} fieldProperties - the field properties object...
	 * @param {String} projectId - the current project id - used to build the query
	 * @param {String} approvalStatusId - the clicked approval status id - used to build the query
	 * @param {String} approvalCycle - the current approval cycle - used to build the query
	 * @param {jQuery} hourGlass - the Hour Glass jQuery element 
	 */
	function showBomDetails(fieldProperties, subtitle, projectId, approvalStatusId, approvalCycle, hourGlass){
		var workerURL = 'bomdeliverabledetail.jsp?projectID=' + projectId + '&approvalStatusId=' + approvalStatusId  + '&approvalCycle=' + approvalCycle + '&menu_file=projsearchresults';   
	    
		var expandoptions = {
			subtitle:subtitle,
			closebuttontitle:i18n.t("BOM_HIDEDOCS"),
			popupcssclass: "bom_search_popup"
		};
		
		hourGlass.css("visibility", "visible");
		ajax({
  			url: workerURL,
  			contentType: "application/x-www-form-urlencoded",
  			type: "GET",
  			data: "text",
  			debug: false,
  			synchronous: false,
  			onError: function(status,objHdl, description) {
      			// Code on failure goes here
  				hourGlass.css("visibility", "hidden");
				
  				// Expand the row / open the popup and...
				fieldProperties.showExpandedInformationArea(function(div) {
					// Display an error message
					div.text(errorMessage);
				});
  			},                                  
  			onSuccess: function(xml) {
  				hourGlass.css("visibility", "hidden");
  				
  				fieldProperties.showExpandedInformationArea(function(div) {
  					div.html(xml);
  					require(["utils/init"],function (init) {
  						init.doinit(div);
  					});
  				}, expandoptions);
	        } 
		});
	};
	
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties.
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);
					
			var projectID = data.getSubList("projectinfo")[0].getValue("id");
			
			var deliverablesData = data.getSubList(fieldname);
			if (deliverablesData) {
				deliverablesData = deliverablesData[0];
				
				var bomInfoContainer = $("<div>").appendTo($div);
				
				var delvsTotal = deliverablesData.getValue("deliverables_total");
				var showAll = deliverablesData.getValue("show_all") && deliverablesData.getValue("show_all") === "1";
				
				// make the hour glass element
				var hourglass = $("<td>")
					.css("visibility", "hidden")
					.append('<img src="images/ajax_loader.gif"  height="15"/>');
				
				//run over each BOM Approval Status info
				
				//NOTE: using <table> to retain the layout
				var delvsList = deliverablesData.getSubList("deliverables");
				if (delvsList) {
					var BOMTable = $("<table>").addClass("BOM_table").appendTo(bomInfoContainer);
					
					//-----------------------------
					//FIRST ROW > BOM Doc Status ICONS  with links
					var BOMIconTableRow = $("<tr>").appendTo(BOMTable);
					
					for(var i=0;i<delvsList.length;i++) {
						var curDelvAppStatusInfo = delvsList[i];
						var appStatusID = curDelvAppStatusInfo.getValue("status_id");
						var appCycleVal = curDelvAppStatusInfo.getValue("approval_cycle");
						var iconFileName = curDelvAppStatusInfo.getValue("icon");
						var labelVal = curDelvAppStatusInfo.getValue("label");
						
						//get icon class instead of file name
						var iconClass = iconFileName;
						if (iconClass == "rejected.gif") {
							iconClass = "icon-WCR_Rejected icon-red";
						} else if (iconClass == "approved.gif") {
							iconClass = "icon-WCR_Approved icon-green";
						} else if (iconClass == "pending.gif") {
							iconClass = "icon-WCR_Pending icon-blue";
						} else if (iconClass == "stopped-approval.png") {
							iconClass = "icon-WCR_Stopped icon-blue";
						} else if (iconClass == "rejectforce.gif") {
							iconClass = "icon-WCR_ForceReject icon-red";
						} else if (iconClass == "approveforce.gif") {
							iconClass = "icon-WCR_ForceApproved icon-green";
						} else if (iconClass == "approvedcond.gif") {
							iconClass = "icon-WCR_ApprovedConditional icon-green";
						} else if (iconClass == "fappcond.gif") {
							iconClass = "icon-WCR_ApprovedConditional icon-green";
						} else if (iconClass == "setup-approval-cycle.png") {
							iconClass = "icon-WCR_Configuration";
						}
						
						var curBOMIconCell = $("<td>")
						//.addClass("BOM_table_cell")
						.on("click", _.partial(showBomDetails, fieldProperties, labelVal, projectID, appStatusID, appCycleVal, hourglass))
						.addClass("action no-toggle-overflow add-padding-right")
						.appendTo(BOMIconTableRow);
						
						//add App Status icon for the currently processed App Status type					
						$("<i>", {
							"class" : "icon icon-normal "+iconClass,
							"title" : labelVal,
							"style" : "cursor: pointer"
						})
						.appendTo(curBOMIconCell);
					}
					
					//All BOM docs
					if (delvsTotal && Number(delvsTotal) > 0 && showAll) {
						var allBOMDocsIconCell = $("<td>")
						//.addClass("BOM_table_cell")
						.on("click", _.partial(showBomDetails, fieldProperties, null, projectID, "All", -1, hourglass))
						.addClass("action no-toggle-overflow")
						.appendTo(BOMIconTableRow);
						
						$("<i>", {
							"class" : "icon icon-normal icon-green icon-WCR_Actions-BOM",
							"title" : i18n.t("BOM_ALLDOCS"),
							"style" : "cursor: pointer"
						})
						.appendTo(allBOMDocsIconCell);
					}
					
					//------------------------
					// Deliverables hour glass (hidden at first)
					if (delvsTotal && Number(delvsTotal) > 0) {
						// add hour glass (separate cell)
						hourglass.appendTo(BOMIconTableRow);
					}
					
					
					//---------------------------------------------------------------
					//SECOND LINE > counts for the corresponding BOM Doc Status group
					var BOMCountTableRow = $("<tr>").appendTo(BOMTable);
					
					for(var i=0;i<delvsList.length;i++) {
						var curDelvAppStatusInfo = delvsList[i];
						var countVal = curDelvAppStatusInfo.getValue("count");
						
						 $("<td>")
						 .text(countVal).appendTo(BOMCountTableRow)
						 .addClass("add-padding-right");
					}
					
					//All BOM docs
					if (delvsTotal && Number(delvsTotal) > 0 && showAll) {
						$("<td>").text(delvsTotal).appendTo(BOMCountTableRow);
					}
					
				}

			}
			
			return {overflowsnever: true};
		}
	};
});