/**
 * Drilldown field Display Method
 * 
 * Renders as a + or - button, depending if the search row is expanded or not.
 * 
 */
define(["jquery",
	"utils/webi18n!SearchInputs",
	"wcr/webcenter_xml",
	"wcr/docSearchOperations",
	"wcr/SendDocDownloadLink",
	"lib/PromisePolyfill",
	"underscore",
	"utils/idUtils",
	"utils/HTTPUtils",
	"component/loadingindicator/LoadingIndicator"
], function (
	$,
	i18n,
	WCXML,
	DocSearcOperations,
	sendDocDownloadLink,
	Promise,
	_,
	idUtils,
	HTTPUtils,
	LoadingIndicator) {

	"use strict";


	/**
	 * Creates an icon with a certain class 
	 */
	var createIcon = function (icon, title) {
		return $("<a>", {
			'class': "btn btn-no-border",
			title: title
		}).append(
			$("<i>", {
				'class': "icon icon-normal icon-default " + icon
			}));
	};

	var createExpandIcon = _.partial(createIcon, "icon-WCR_Plus-square", i18n.t("EXPAND"));
	var createShrinkIcon = _.partial(createIcon, "icon-WCR_Minus-square", i18n.t("COLLAPSE"));


	function makeUrlPageRelativeToInstance(url) {
		return url.substr(window.webcenterConfig.webInstAbsPath.length +1 );
	}

	/**
	 * returns a promise for the saved search
	 * @param {String} savedSearch - name of the saved search
	 * @param {String} projectId - id of the project this search should be executed in
	 */
	var getSavedSearchHTML = _.memoize(function (projectId, savedSearch, timestamp, searchSettings) {
		var sectionId = idUtils.newClientSideId("drilldown");
		var sourcePageUrl = makeUrlPageRelativeToInstance(window.location.pathname) + (window.location.search ? window.location.search : "");
		var url = HTTPUtils.constructURL(window.webcenterConfig.jspBaseUrl + "dosavedsearch.jsp", {
			'searchName': savedSearch,
			"ajax": 1,
			"sectionID": sectionId,
			"projectID": projectId,
			"isDrilldown": 1,
			useDefaultDrilldown: savedSearch ? 0 : 1,
			sourcePageUrl: sourcePageUrl,
			showSimpleGUI: searchSettings.preferences.simpleGUI
		});

		return new Promise(function (resolve, reject) {
			document.body.classList.add("progress-cursor");
			$.get(url).done(function (data) {
				resolve(data);
			}).fail(function (data) {
				reject(data);
			});
		});
	}, function (projectId, savedSearch, timestamp) {
		return projectId + "-" + savedSearch + "-" + timestamp;
	});



	/**
	 * returns a function that, when called, shows the saved search
	 * @param {$.Element} cellDiv - the div in which to render the icon
	 * @param {String} savedSearch 
	 * @param {FieldProperties} fieldProperties
	 * @param {String} projectId;
	 */
	var showSavedSearch = function (cellDiv, savedSearch, fieldProperties, projectId, timestamp, searchSettings) {

		var loadingIndicator = new LoadingIndicator;
		var isCancelled = false;

		/**
		 * function to display the html of the saved search
		 */
		var displayHtml = function (html) {
			if (!isCancelled) {
				fieldProperties.showExpandedInformationArea(function (expandedDiv, closeFn) {
					setShrinkIcon(cellDiv, closeFn); // set '-' icon
					expandedDiv.html(html); // set the html
					loadingIndicator.stop();
					requirejs(["utils/init"], function (init) { // init
						init.doinit(expandedDiv);
						document.body.classList.remove("progress-cursor");
					});
				}, {
					onclose: _.partial(setExpandIcon, cellDiv, savedSearch, fieldProperties, projectId, timestamp),
					hideshrinkbtn: true,
					modaldialogclass: "render-drilldown-popup_open"
				});
			}
		};



		var cancelLoadingOfHtml = function (closeFn) {
			return function () {
				isCancelled = true;
				document.body.classList.remove("progress-cursor");
				closeFn();
			};
		};

		var setIconWhileLoading = function () {
			fieldProperties.showExpandedInformationArea(function (expandedDiv, closeFn) {
				setShrinkIcon(cellDiv, cancelLoadingOfHtml(closeFn)); // set '-' icon

				expandedDiv.append(loadingIndicator.element);
				loadingIndicator.start();
			}, {
				onclose: _.partial(setExpandIcon, cellDiv, savedSearch, fieldProperties, projectId, timestamp),
				hideshrinkbtn: true,
				modaldialogclass: "render-drilldown-popup_loading"
			});

		};

		return function (e) {
			setIconWhileLoading();
			getSavedSearchHTML(projectId, savedSearch, timestamp, searchSettings).then(displayHtml);
			e.preventDefault();
		};
	};

	/**
	 * sets the '+' icon
	 */
	var setExpandIconForMergedChildSearch = function (cellDiv, savedSearch, fieldProperties, projectId, timestamp, showFn, closeFn) {
		cellDiv.empty();
		createExpandIcon().appendTo(cellDiv).on("click", function (e) {
			if (showFn) {
				showFn();
				setShrinkIcon(cellDiv, function () {
					setExpandIconForMergedChildSearch(cellDiv, savedSearch, fieldProperties, projectId, timestamp, showFn, closeFn);
					closeFn();
				});
			} else {
				var fns = fieldProperties.doSavedSearch(savedSearch, projectId, undefined);
				setShrinkIcon(cellDiv, function () {
					setExpandIconForMergedChildSearch(cellDiv, savedSearch, fieldProperties, projectId, timestamp, fns.show, fns.close);
					fns.close();
				});
			}
		});
	};

	/**
	 * sets the '+' icon
	 */
	var setExpandIcon = function (cellDiv, savedSearch, fieldProperties, projectId, timestamp, searchSettings) {
		cellDiv.empty();
		createExpandIcon().appendTo(cellDiv).on("click", showSavedSearch(cellDiv, savedSearch, fieldProperties, projectId, timestamp, searchSettings));
	};

	/**
	 * sets the '-' icon
	 */
	var setShrinkIcon = function (cellDiv, closeSavedSearch) {
		cellDiv.empty();
		createShrinkIcon().appendTo(cellDiv).on("click", closeSavedSearch);
	};


	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function (div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var cellDiv = $(div);

			var projectId = data.getSubList("projectinfo")[0].getValue("id");

			var savedSearch = fieldProperties.displayConfig.getValue("savedsearch");

			var useParentLayout = fieldProperties.displayConfig.getValue("useParentLayout") === "1";

			// Timestamp is used to invalidate cache if a rerun has happend.
			var timestamp = Date.now();

			if (useParentLayout) {
				setExpandIconForMergedChildSearch(cellDiv, savedSearch, fieldProperties, projectId, timestamp);
			} else {
				setExpandIcon(cellDiv, savedSearch, fieldProperties, projectId, timestamp, searchSettings);
			}

			return {
				overflowsnever: true
			};
		}

	};
});
