/**
 * "Project Status" Display Method
 * 
 * Shows an advanced readout of a Project Status - supports project status icons and/or color-based status name label
 * 
 * @author JAMI
 */
define(["jquery","utils/ColorUtils", "utils/webi18n!SearchInputs"], function($, ColorUtils, i18n){
	return {
		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function(div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);			
			var values = data.getSubList(fieldname);
			
			if(values){
				// We have data
				for(var i=0;i<values.length;i++) {
					
					var projStatusCellContainer = $("<div>").appendTo($div);
					
					//retrieve the required parameters
					var statusIconURL = values[i].getValue("icon_url");
					var statusName = values[i].getValue("translation");
					var statusColorRGBCode = values[i].getValue("color_rgb");
					
					//render style options
					//NOTE: "show_status_icon" is only considered if not set to "auto" 
					//> in that case use the User's general 'Search::Show Thumbnails' preference instead
					var showStatusIcon = tableDisplaySettings.showThumbnails; 
					var showIconPref = fieldProperties.displayConfig.getValue("show_status_icon");
					if (showIconPref && showIconPref !== "auto") {
						showStatusIcon = showIconPref === "1";
					}
					
					var showStatusName = fieldProperties.displayConfig.getValue("show_label") === "1";
					var showStatusColorLabel = fieldProperties.displayConfig.getValue("show_color_label") === "1";
					
					var doRenderStatusName = false;
					//---------------------------------
					//Status Icon (and Name, optionally)
					if (showStatusIcon) {
						if (statusIconURL) {
							//show Status Icon if its URL can be retrieved
							$("<img>", {
								"src"   :"images/statuses/"+statusIconURL,
								"alt"   :i18n.t("ICONMISSING_TAG"), 
								"title" :statusName,
								"border":0
							}).appendTo(projStatusCellContainer);
								
							if (showStatusName) {
								//add a line break if Status Name should also be shown
								$("<br\>").appendTo(projStatusCellContainer);
								
								doRenderStatusName = true;
							}	
						} else {
							//Icon URL cannot be retrieved > fall back to the Status Name readout
							doRenderStatusName = true;
						}
					}
					//-----------
					//Color Label
					else if (showStatusColorLabel) {						
						//calculate foreground color code
						var foregroundColorRGBCode = ColorUtils.calculateForegroundColor(statusColorRGBCode);
						
						var colorLabelContainer = $("<div>", {"class" : "projectStatus dropdown"})
						.css("background-color", statusColorRGBCode)
						.css("color", foregroundColorRGBCode)
						.css("display", "block")
						.appendTo(projStatusCellContainer);
						
						//add the button>icon + label for the status
						$("<a>")
						.addClass("btn currentProjectStatus btn-no-action")
						.text(statusName)
						.appendTo(colorLabelContainer);
					}
					//--------------------------
					//Simple Status Name readout
					else {  
						doRenderStatusName = true;
					}
					
					//Simple Status Name readout
					if (doRenderStatusName) {
						projStatusCellContainer.append(statusName);
					};
				};
			}else{
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
            return {overflowsnever: true};
		}
	};
});