/**
 * "User/Group/Role" Display Method
 * 
 * Shows a user friendly label of the referenced object with a link that opens "details popup"
 * 
 * @author JAMI
 */
define(["jquery", "underscore", "../../Utils", "utils/webi18n!SearchInputs", "wcr/webcenter_html"], function ($, _, Utils, i18n) {
	"use strict";

	/**
	 * Generates HTML for a User-member reference
	 * 
	 * @param {ValueData} userData : contains all the needed information about the User to be shown
	 * @param {Boolean} showIcon : should we also show a User icon in front of the User name/link?
	 * @param {JQuery DOM Element} parentContainer : container (usually a div) to append the target User reference to
	 * @param {Boolean} isUserRep : are we rendering a User representative
	 * @return {JQuery DOM Element}
	 */
	function renderUserRef(userData, showIcon, parentContainer, isUserRep) {
		var userLastName = userData.getValue("user_lastname");
		var userFirstName = userData.getValue("user_firstname");
		var userLoginName = userData.getValue("user_loginname");
		var userIsActive = userData.getValue("user_is_active") != "false";
		var userID = userData.getValue("id");

		// userID will be "" string for <nobody>
		if (userID) {
			var html = WCHTML.getUserNameStringActiveHTML(userID, userFirstName, userLastName, userLoginName, userIsActive, i18n.t("DEACTIVATED"), !showIcon);

			//add into the parent container
			$(html).appendTo(parentContainer);

			// add "@" divider to be shown between the User representative and its parent Group (that is written separately)
			if (isUserRep) {
				parentContainer.append(" @ ");
			}
		}
	}

	/**
	 * Generates HTML for a Group-member reference
	 * 
	 * @param {ValueData} groupData : contains all the needed information about the Group to be shown
	 * @param {Boolean} showIcon : should we also show a Group icon in front of the User name/link?
	 * @param {JQuery DOM Element} parentContainer : container (usually a div) to append the target Group reference to
	 */
	function renderGroupRef(groupData, showIcon, parentContainer) {
		var groupDisplayVal = groupData.getValue("group_name");
		var groupID = groupData.getValue("id");

		if (!showIcon) {
			groupDisplayVal = groupDisplayVal.concat(" (" + i18n.t("GROUP") + ")");
		}

		//Group-type icon
		if (showIcon) {
			var iconElem = $("<i>").addClass("icon icon-normal icon-green icon-WCR_Assignee-Group");
			parentContainer.append(iconElem);
			$(" ").appendTo(parentContainer);
		}

		//Group name + link
		var objRefLinkContainer = $("<a>", {
			"href": "#",
			"text": groupDisplayVal
		});
		objRefLinkContainer.on("click", function (event) {
			event.preventDefault();
			ShowGroupDetails(groupID, null, 0);
		});

		//add into the parent container
		objRefLinkContainer.appendTo(parentContainer);
	}

	/**
	 * Generates HTML for a Role-member reference
	 * 
	 * @param {ValueData} roleData : contains all the needed information about the Role to be shown
	 * @param {Boolean} showIcon : should we also show a Role icon in front of the User name/link?
	 * @param {JQuery DOM Element} parentContainer : container (usually a div) to append the target Role reference to
	 * @param {String} projectID : parent Project ID reference used for properly showing Role assignment details
	 */
	function renderRoleRef(roleData, showIcon, parentContainer, projectID) {
		var roleDisplayVal = roleData.getValue("group_name");
		var roleID = roleData.getValue("id");

		if (!showIcon) {
			roleDisplayVal = roleDisplayVal.concat(" (" + i18n.t("ROLE") + ")");
		}

		//Group-type icon
		if (showIcon) {
			var iconElem = $("<i>").addClass("icon icon-normal icon-blue icon-WCR_Assignee-Role");
			parentContainer.append(iconElem);
			$(" ").appendTo(parentContainer);
		}

		//Group name + link
		var objRefLinkContainer = $("<a>", {
			"href": "#",
			"text": roleDisplayVal
		});
		objRefLinkContainer.on("click", function (event) {
			event.preventDefault();
			ShowGroupDetails(roleID, projectID, 1);
		});

		//add into the parent container
		objRefLinkContainer.appendTo(parentContainer);
	}

	/**
	 * Utility function used to generate HTML of User/Group/Role information to render and add to a specified DOM element
	 * 
	 * @param {JQuery DOM Element} targetContainerDiv - the div to append the generated HTML into; JQuery object expected
	 * @param {ValueData} dataObj - contains all the relevant properties to control the content of the target HTML data
	 * @param {Boolean} showIconsSetting - flag that indicates whether User/Group/Role icons should also be shown
	 * @param {String} projectID : parent Project ID reference used for properly showing Role assignment details
	 */
	function renderWCRMemberObj(targetContainerDiv, dataObj, showIconsSetting, projectID) {
		var objType = dataObj.getValue("obj_type");
		var isRoleType = objType === "role" || objType === "roleassignments";

		var memberObjContainer = $("<div>").appendTo(targetContainerDiv);

		//---------------------------------------------------------------
		//Render the appropriate member based on the User/Group/Role type
		if (objType === "user") {
			renderUserRef(dataObj, showIconsSetting, memberObjContainer);
		} else if (objType === "group") {
			//----------------------------------------------------------------------------------------
			//special case when dealing with Group Assignment (from a Role) with a User representative
			var curGroupUserMembers = dataObj.getSubList("userassignee");
			if (curGroupUserMembers !== null && curGroupUserMembers.length > 0) {
				//we do have assignments > should only be a single User (we don't support multiple User representatives of a single Group)
				var curGroupUserRepData = curGroupUserMembers[0];

				if (curGroupUserRepData) {
					//add the User representative display data into the container
					renderUserRef(curGroupUserRepData, showIconsSetting, memberObjContainer, true);
				}
			}

			renderGroupRef(dataObj, showIconsSetting, memberObjContainer);
		} else if (isRoleType) {
			renderRoleRef(dataObj, showIconsSetting, memberObjContainer, projectID);
		}
	}

	return {

		/**
		 * Render the value in the cell...
		 * 
		 * @param {DOMElement} div - the div to render into
		 * @param {String} fieldname - the fieldname shown in the column
		 * @param {ValueData} data - the information for this row. Format: an object of key => array of strings (currently only ROMD data)
		 * @param {TableDisplaySettings} tableDisplaySettings - some information about how the field is displayed
		 * @param {FieldProperties} fieldProperties - custom properties of the processed Search Field
		 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
		 * @returns {Object} [OPTIONAL] to return special properties. 
		 *        Like: "overflowsnever = true" when this field will never contain too much content.
		 */
		render: function (div, fieldname, data, tableDisplaySettings, fieldProperties, searchSettings) {
			var $div = $(div);

			var values = data.getSubList(fieldname);
			var showIconsSetting = fieldProperties.displayConfig.getValue("show_icon") !== "0";
			//NOTE: Project ID only needed when rendering a Project Role details link > in other cases it is not used
			var projectID = data.getSubList("projectinfo") ? data.getSubList("projectinfo")[0].getValue("id") : "";

			if (values) {
				// We have data
				// Add multiple rows to the cell...
				for (var i = 0; i < values.length; i++) {

					var objType = values[i].getValue("obj_type");
					var isRoleType = objType === "role" || objType === "roleassignments";

					//------------------------------------------------------------------------------
					//if showing Role assignment in grid layout > use a different rendering approach
					if (tableDisplaySettings.resultsStyle === "grid" && isRoleType) {
						//get the "inline role assignment data" value
						var inlineRoleAssignmentVal = values[i].getValue("inline_value");
						//use Util method to truncate the value if needed
						Utils.truncateTextVal(inlineRoleAssignmentVal, $div);
					} else {
						//render the current WCR Member (User/Group/Role) Object and add it to the target container
						renderWCRMemberObj($div, values[i], showIconsSetting, projectID);

						//handle Role assignments, if relevant
						if (isRoleType) {
							//lookup Role assignment data
							var curRoleAssignments = values[i].getSubList("assignee");

							if (curRoleAssignments !== null && curRoleAssignments.length > 0) {
								//we do have assignments > process those one by one
								for (var j = 0; j < curRoleAssignments.length; j++) {
									//render the assignee obj
									renderWCRMemberObj($div, curRoleAssignments[j], showIconsSetting, projectID);
								}
							}
						}
					}
				}
			} else {
				// There is no data for this field... (or at least not for this document/project)
				// Just leave it empty
			}
		}

	};
});