define([
	"jquery",
	"underscore",
	"utils/AttributeEventBus",
	"component/searchresults/Utils"
], function (
	$,
	_,
	AttributeEventBus,
	utils) {
	"use strict";

	/**
	 * Implements a wrapper for the cascading dropdown attributes configured with lists in the ListManager project.
	 *
	 * @constructor
	 */
	var CascadingDropdownParameterField = function (div, fieldId, settings) {
		this.div = $(div);
		this.fieldId = fieldId;
		this.settings = settings;
		this.changeCallbacks = [];
		this._openDropdownCallbacks = [];
		this._onClearCallbacks = [];
		this._onChangeRequestCallbacks = [];
		this._currentValue = "";

		this._initialize();
	};

	/**
	 * (private) Initialize the field.
	 */
	CascadingDropdownParameterField.prototype._initialize = function () {
		var self = this;

		var wrapper = $("<div>", {
			"class": "input-group"
		}).css({"width":"100%"});

		// Make the input...
		this.input = $("<input>", {
			"type": "text",
			"class": "form-control attachleft search-input-lg"
		});
		wrapper.append(this.input);

		// Make dropdown button...
		this.dropdown = $("<label>", {
			'class': "input-group-addon"
		}).append(
			$("<i>", {
				'class': "icon icon-WCR_Down"
			})
		);
		this.dropdown.click(function (e) {
			_.each(self._openDropdownCallbacks, function (callback) {
				callback(e);
			});
		});
		wrapper.append(this.dropdown);

		// Add the field
		this.div.append(wrapper);

		//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
		this.input.on("keyup", function (event) {
			self._onIntermediateUpdate(event);

			//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
			if (event.keyCode == 13) {
				$(this).blur();
			}
		});

		//blur=get out of the field >>> trigger final update event
		this.input.on("blur", function (event) {
			self._onFinalUpdate(event);
		});

	};

	//
	// Internal event handling
	//
	CascadingDropdownParameterField.prototype._onIntermediateUpdate = function (e) {
		this._triggerChange(true, e);
	};

	CascadingDropdownParameterField.prototype._onFinalUpdate = function (e) {
		//console.log(this.fieldId + " has changed to " + this.input.val());
		this._triggerChange(false, e);
	};

	/**
	 * Get the value by filling the value object.
	 * Also do the validation in the mean time and
	 * use the feedback object to give the information back.
	 * 
	 * @param {ValueData} value - the object to put all the value information in
	 * @param {ValidationFeedback} feedback - the object 
	 */
	CascadingDropdownParameterField.prototype.getValueData = function (value, feedback) {
		value.clear();
		// Make sure there are quotes around the value if needed 
		// (alternative would be to set flag "isDropdown" to true => but that would only work for a single value)
		var query = utils.arrayOfValuesToLuceneQuery(utils.luceneQueryToArrayOfValues(this.input.val()));
		value.setValue("value", query);
	};

	/**
	 * Set the value, update the field to represent this value.
	 * 
	 * @param {ValueData} data - the object which contains all the information about the value
	 * @returns {Boolean} true if the field can represent the data which is set
	 */
	CascadingDropdownParameterField.prototype.setValueData = function (data) {
		var newValue = data.getValue("value") || "";
		
		// Do not show quotes in the user interface in case of a single value... (even though we search for them)
		var values = utils.luceneQueryToArrayOfValues(newValue);
		if(values.length>1) {
			// Multi value => Ok... show in input with quotes and || signs.
			this.setValue(utils.arrayOfValuesToLuceneQuery(values));
		} else if (values.length>0) {
			// Single value => Special case!! Do not show quotes in the input EVEN though we will actually search for them!
			this.setValue(values[0]);
		} else {
			// No values
			this.setValue("");
		}
		
		_.each(this._onChangeRequestCallbacks, function (callback) {
			callback(newValue);
		});
		return true;
	};

	/**
	 * Clear the Field (called by "Clear this form" functionality) 
	 */
	CascadingDropdownParameterField.prototype.clear = function () {
		_.each(this._onClearCallbacks, function (callback) {
			callback();
		});
		this._onFinalUpdate();
	};

	/**
	 * Does this field contain a SmartName?
	 * 
	 * @returns {bool} true when this field contains a SmartName
	 */
	CascadingDropdownParameterField.prototype.valueContainsSmartName = function () {
		return false;
	};

	/**
	 * Does this field contain a special search character?
	 * 
	 * @param {char[]} specialCharacters - an array of special characters...
	 */
	CascadingDropdownParameterField.prototype.valueContainsSpecialSearchCharacters = function (specialCharacters) {
		return false;
	};

	/**
	 * Registers a new function to call when the field changes...
	 * 
	 * @param {Function} callback - a function to call when the field changes. 
	 * Function has one parameter, which should be true when the user is not finished with modifying the field.
	 */
	CascadingDropdownParameterField.prototype.onChange = function (callback) {
		this.changeCallbacks.push(callback);
	};

	/**
	 * (private) Trigger a change event
	 */
	CascadingDropdownParameterField.prototype._triggerChange = function (intermediatechange, event) {
		_.each(this.changeCallbacks, function (callback) {
			callback(intermediatechange, event);
		});
	};

	/**
	 * Destroys this field...
	 * Remove listeners and clean up the div.
	 */
	CascadingDropdownParameterField.prototype._destroy = function () {
		this.div.empty();
		this.changeCallbacks = [];
		this._openDropdownCallbacks = [];
		this._onClearCallbacks = [];
		this._onChangeRequestCallbacks = [];
		this._currentValue = "";
	};

	/**
	 * Register a listener that is called when the user tries to open the dropdown
	 * @param {Function} callback
	 */
	CascadingDropdownParameterField.prototype.onOpenDropdown = function (callback) {
		this._openDropdownCallbacks.push(callback);
	};

	/**
	 * Register a listener that is called when the user tries to clear the form
	 * @param {Function} callback
	 */
	CascadingDropdownParameterField.prototype.onClear = function (callback) {
		this._onClearCallbacks.push(callback);
	};

	/**
	 * Register a listener that is called when the value of this ParameterField is changed externally, usually by the search framework.
	 * @param {Function} callback
	 */
	CascadingDropdownParameterField.prototype.onChangeRequest = function (callback) {
		this._onChangeRequestCallbacks.push(callback);
	};

	/**
	 * Set the new value
	 * @param {String} newValue
	 */
	CascadingDropdownParameterField.prototype.setValue = function (newValue) {		
		this.input.val(newValue);
		if (newValue !== this._currentValue) {
			this._currentValue = newValue;
			this._onFinalUpdate();
		}
	};



	// ======================================================================= //

	/**
	 * Factory of CascadingDropdownParameterField
	 * @param {SearchResultsAttributeCategoryControl} searchResultsAttributeCategoryControl 
	 * @param {AttributeCategory} attributeCategory                 
	 * @param {Attribute} attObj                               
	 * @param {int} row                                   column of attribute dropdown
	 * @param {int} col                                   row of attribute dropdown
	 */
	var CascadingDropdownParameterFieldFactory = function (searchResultsAttributeCategoryControl, attributeCategory, attObj, row, col) {
		this._searchResultsAttributeCategoryControl = searchResultsAttributeCategoryControl;
		this._attributeCategory = attributeCategory;
		this._attObj = attObj;
		this._row = row;
		this._col = col;
		this._currentValue = "";
	};

	/** 
	 * Make a new field of this type
	 * Returned object should implement certain functions
	 * @param {DOMElement} div - The dom element in which we should render the field
	 * @param {String} fieldName - The name of the field for which this field is created
	 * @param {SearchSettings} settings - Information about settings which are enabled
	 *     - E.g. are SmartNames enabled?
	 * @param {DataProvider} data - interface to have access to data
	 * @param {FieldProperties} fieldProperties - additional properties of the field
	 */
	CascadingDropdownParameterFieldFactory.prototype.make = function (div, fieldName, settings, dataProvider, fieldProperties) {
		var cascadingDropdownParameterField = new CascadingDropdownParameterField(div, fieldName, settings);

		this._attributeCategory.registerAttributeElement(this._attObj.id, cascadingDropdownParameterField.input[0]);

		// ============================================================================= //
		// Hook all listeners to ParameterField, redirect requests to AttributeCategory  //
		// ============================================================================= //
		cascadingDropdownParameterField.onOpenDropdown(function (event) {
			// pass request to attribute category to open dropdown
			openDropdown(event, cascadingDropdownParameterField.dropdown[0]);
		}.bind(this));

		cascadingDropdownParameterField.onChange(function (immediate, event) {
			if (immediate && event) {
				openDropdown(event, cascadingDropdownParameterField.input[0]);
			}
		}.bind(this));

		cascadingDropdownParameterField.onChangeRequest(function (newValue) {
			if (newValue) {
				var newValues = utils.luceneQueryToArrayOfValues(newValue);
				// If more than 1 value is selected, enable multivalue mode
				if (newValues.length > 1) {
					this._attObj.columnObj.searchMultiple = true;
					this._attObj.columnObj.autoSelected = false;
					this._attObj.value = newValues.join(this._attributeCategory.multiSep);
				} else {
					this._attObj.columnObj.searchMultiple = false;
					this._searchResultsAttributeCategoryControl.onCascadingDropdownsLoaded(function () {
						this._attributeCategory.selectDBValue(this._attObj.id, newValues[0], false, "");
					}.bind(this));
				}
			}
		}.bind(this));

		cascadingDropdownParameterField.onClear(function () {
			this._attObj.columnObj.searchMultiple = false;
			var listObj = this._attObj.listObj;
			this._attributeCategory.clearListValues(listObj.name, listObj.type, listObj.counter, true);
		}.bind(this));

		var openDropdown = function (event, elem) {
			this._attributeCategory.getDBValues(elem, event, this._row, this._col, false, true);
		}.bind(this);

		// ======================================== //
		// Hook all listeners to AttributeCategory  //
		// ======================================== //
		AttributeEventBus.getInstance().onChangeAttrValueListener(function (attUID, attObj) {
			if (attObj === this._attObj) {
				cascadingDropdownParameterField.setValue(attObj.value);
			}
		}.bind(this));

		this._attributeCategory.onListValuesDeselected(function () {
			cascadingDropdownParameterField.setValue("");
		}.bind(this));
		
		this._attributeCategory.onSearchSelectMultiple(function (attObj, selectedValues) {
            if (attObj === this._attObj) {
                cascadingDropdownParameterField.setValue(utils.arrayOfValuesToLuceneQuery(selectedValues));
            }
        }.bind(this));

		return cascadingDropdownParameterField;
	};

	/**
	 * Destroy a field created by this factory...
	 * @param obj - the object which was returned by this factory
	 */
	CascadingDropdownParameterFieldFactory.prototype.destroy = function (obj) {
		obj._destroy();
	};

	return CascadingDropdownParameterFieldFactory;
});
