define([
	"jquery",
	"underscore",
	"utils/AttributeEventBus",
	"component/searchresults/Utils"
], function (
	$,
	_,
	AttributeEventBus,
	utils) {
	"use strict";

	/**
	 * Implements a wrapper for the cascading dropdown attributes configured with front-end excel lists that are configured 
	 * in the custom folder.
	 *
	 * @constructor
	 */
	var CascadingDropdownXmlParameterField = function (div, fieldId, settings, selectHtml) {
		this.div = $(div);
		this.fieldId = fieldId;
		this.settings = settings;
		this.changeCallbacks = [];
		this._onClearCallbacks = [];
		this._selectHtml = selectHtml;
		this._currentValue = "";

		this._initialize();
	};

	/**
	 * (private) Initialize the field.
	 */
	CascadingDropdownXmlParameterField.prototype._initialize = function () {
		this.div.html(this._selectHtml);
	};

	//
	// Internal event handling
	//
	CascadingDropdownXmlParameterField.prototype._onIntermediateUpdate = function (e) {
		this._triggerChange(true, e);
	};

	CascadingDropdownXmlParameterField.prototype._onFinalUpdate = function (e) {
		// window.console.log(this.fieldId + " has changed to \"" + this._currentValue + "\"");
		this._triggerChange(false, e);
	};

	//
	// Set & Get the value
	//

	/**
	 * Get the value by filling the value object.
	 * Also do the validation in the mean time and
	 * use the feedback object to give the information back.
	 * 
	 * @param {ValueData} value - the object to put all the value information in
	 * @param {ValidationFeedback} feedback - the object 
	 */
	CascadingDropdownXmlParameterField.prototype.getValueData = function (value, feedback) {
		value.clear();
		var values = utils.luceneQueryToArrayOfValues(this._currentValue);
		if(values.length===1) {
			value.setValue("value", utils.arrayOfValuesToLuceneQuery([values[0]]));
		} else {
			value.setValue("value", "");
		}
	};

	/**
	 * Set the value, update the field to represent this value.
	 * 
	 * @param {ValueData} data - the object which contains all the information about the value
	 * @returns {Boolean} true if the field can represent the data which is set
	 */
	CascadingDropdownXmlParameterField.prototype.setValueData = function (data) {
		this.setValue(data.getValue("value") || "");

		// Update the select dropdown with new value
		var values = utils.luceneQueryToArrayOfValues(this._currentValue);
		if (values.length === 1) {
			this.findSelect().val(values[0]);
		} else {
			this.findSelect().val("");
		}
		this.findSelect().trigger("change");
		return true;
	};

	/**
	 * Clear the Field (called by "Clear this form" functionality) 
	 */
	CascadingDropdownXmlParameterField.prototype.clear = function () {
		this._currentValue = "";
		_.each(this._onClearCallbacks, function (callback) {
			callback();
		});
		this._onFinalUpdate();
	};

	/**
	 * Does this field contain a SmartName?
	 * 
	 * @returns {bool} true when this field contains a SmartName
	 */
	CascadingDropdownXmlParameterField.prototype.valueContainsSmartName = function () {
		return false;
	};

	/**
	 * Does this field contain a special search character?
	 * 
	 * @param {char[]} specialCharacters - an array of special characters...
	 */
	CascadingDropdownXmlParameterField.prototype.valueContainsSpecialSearchCharacters = function (specialCharacters) {
		return false;
	};

	//
	// Change listeners
	//

	/**
	 * Registers a new function to call when the field changes...
	 * 
	 * @param {Function} callback - a function to call when the field changes. 
	 * Function has one parameter, which should be true when the user is not finished with modifying the field.
	 */
	CascadingDropdownXmlParameterField.prototype.onChange = function (callback) {
		this.changeCallbacks.push(callback);
	};

	/**
	 * Trigger a change event
	 * @private
	 * @param {boolean}   intermediatechange True if this should result in a new search rerun
	 * @param {DOM-Event} event              The event that triggered this change     
	 */
	CascadingDropdownXmlParameterField.prototype._triggerChange = function (intermediatechange, event) {
		_.each(this.changeCallbacks, function (callback) {
			callback(intermediatechange, event);
		});
	};

	/**
	 * Finds the select object of this ParameterField. This element is generated by AttributeCategory in function 'renderRestrictedList'.
	 * @returns {JQuery} the select element
	 */
	CascadingDropdownXmlParameterField.prototype.findSelect = function () {
		return this.div.find("select.form-control");
	};

	/**
	 * Sets the new value of this ParameterField. This will not update the select dropdown, because it is already handled in AttributeCategory.js
	 * @param {String} newValue
	 */
	CascadingDropdownXmlParameterField.prototype.setValue = function (newValue) {
		if (this._currentValue !== newValue) {
			if (newValue) {
				this._currentValue = newValue;
			} else {
				this._currentValue = "";
			}
			this._onFinalUpdate();
		}
	};

	/**
	 * Add a listener to when this ParameterField is requested to clear its value
	 * @param {Function} callback
	 */
	CascadingDropdownXmlParameterField.prototype.onClear = function (callback) {
		this._onClearCallbacks.push(callback);
	};

	/**
	 * Destroys this field...
	 * Remove listeners and clean up the div.
	 */
	CascadingDropdownXmlParameterField.prototype._destroy = function () {
		this.div.empty();
		this.changeCallbacks = [];
		this._onClearCallbacks = [];
		this._currentValue = "";
	};

	// ======================================================================= //

	/**
	 * Factory of CascadingDropdownXmlParameterField
	 * @param {AttributeCategory} attributeCategory
	 * @param {Object}            cellObj           contains the attribute
	 * @param {int}               row               row of attribute
	 * @param {int}               col               column of attribute
	 */
	var CascadingDropdownXmlParameterFieldFactory = function (attributeCategory, cellObj, row, col) {
		this._attributeCategory = attributeCategory;
		this._cellObj = cellObj;
		this._row = row;
		this._col = col;
	};

	/** 
	 * Make a new field of this type
	 * Returned object should implement certain functions
	 * @param {DOMElement} div - The dom element in which we should render the field
	 * @param {String} fieldName - The name of the field for which this field is created
	 * @param {SearchSettings} settings - Information about settings which are enabled
	 *     - E.g. are SmartNames enabled?
	 * @param {DataProvider} data - interface to have access to data
	 * @param {FieldProperties} fieldProperties - additional properties of the field
	 */
	CascadingDropdownXmlParameterFieldFactory.prototype.make = function (div, fieldName, settings, dataProvider, fieldProperties) {
		// Generate the dropdown html + construct new ParameterField
		var selectHtml = this._attributeCategory.renderRestrictedList(this._cellObj, this._row, this._col, true);
		var cascadingDropdownXmlParameterField = new CascadingDropdownXmlParameterField(div, fieldName, settings, selectHtml);

		this._attributeCategory.registerAttributeElement(this._cellObj.attObj.id, cascadingDropdownXmlParameterField.div);

		// Listen to clear request of ParameterField and propagate request to AttributeCategory
		cascadingDropdownXmlParameterField.onClear(function () {
			var listObj = this._cellObj.attObj.listObj;
			this._attributeCategory.deselectListAttributeValues(listObj);
			this._attributeCategory.updateSearchDropdowns(listObj);
		}.bind(this));

		// Listen to changes to attribute and propagate them to ParameterField
		AttributeEventBus.getInstance().onChangeAttrValueListener(function (attID, attObj) {
			if (attObj === this._cellObj.attObj) {
				cascadingDropdownXmlParameterField.setValue(attObj.value.trim());
			}
		}.bind(this));

		return cascadingDropdownXmlParameterField;
	};

	/**
	 * Destroy a field created by this factory...
	 * @param obj - the object which was returned by this factory
	 */
	CascadingDropdownXmlParameterFieldFactory.prototype.destroy = function (obj) {
		obj._destroy();
	};

	return CascadingDropdownXmlParameterFieldFactory;
});
