/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore",
        "./popups/CharacteristicsSearchPopup",
        "utils/webi18n!SearchInputs"
        ], function(
        		$, _,
        		CharacteristicsSearchPopup,
        		SearchInputs) {
	
	"use strict";

    /**
     * @constructor
     */
    var CharacteristicsParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    CharacteristicsParameterField.prototype._initialize = function(){
    	
    	/*
    	 * Object to store the char id and name
    	 * {searchtype: "AND", chars: [{id:"00002_0000000001", name:"char1"}, {id:"00002_0000000002", name:"char2"}]}
    	 */
    	this._charsData = {};
    	
    	// Div to display selected characteristics
    	this.input = $("<div>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    		"class": "characteristics"
    	});
    	
    	/*
    	 * Method to execute, when a characteristic(s) is selected
    	 */
    	this._updateInputText = _.bind(function() {
    		this.input.empty();
    		
    		var charInput = $("<span>");
        	
    		// Localized string for search type (AND/OR)
        	var connector = null;
    		if(this._charsData.searchtype == "OR") {
    			connector = SearchInputs.t("SEARCH_OPERATOR_OR");
    		} else {
    			connector = SearchInputs.t("SEARCH_OPERATOR_AND");
    		}
    		
    		var charsList = this._charsData.chars;
    		if(charsList) {
    			for(var i=0; i<charsList.length; i++) {
        			var char = charsList[i];
        			
        			if(char) {
        				if(i!==0) {
            				charInput.append(" ", $("<span>", {"class":"connector", "text":connector}), " ");
            			}
        				
        				charInput.append($("<span>", {"text":char.name}));
        			}
        		}
    		}
    		
    		
    		this.input.append(charInput);
    		if(charInput) {
    			$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("EDIT")});
    		} else {
    			$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("ADD")});
    		}
    		
    	}, this);
    	
    	// create add button...
    	this._addButton = $("<a>", {"class": "btn btn-no-border"}).append($("<i>", {"name": "addorediticon", "class": "icon icon-WCR_Actions-Edit12", "title": SearchInputs.t("ADD")}));
    	
    	this._addButton.click(_.bind(function() {
    		if(!this._charsPopup) {
    			// Create a characteristics search popup
    			this._charsPopup = new CharacteristicsSearchPopup(this._charsData);
    			this._charsPopup.onSubmit(_.bind(function(charsData) {
    				this._charsData = charsData;
    				this._updateInputText();
    				this._triggerChange(true);
    			}, this));
    			
    			this._charsPopup.open(this._charsData);
    			
    		} else {
    			this._charsPopup.open(this._charsData);
    		}
    		
    	}, this));

		// add the button and input
    	this.div.append($("<table>").append($("<tr>").append($("<td>", {"class": "add-padding-right"}).append(this._addButton), $("<td>").append(this.input))));
    	
    };
    
    //
    // Internal event handling
    //
    CharacteristicsParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    CharacteristicsParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    CharacteristicsParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	
    	if(this._charsData.searchtype) {
    		// Set search type
    		value.setValue("search_type", this._charsData.searchtype);
    		
    		// Set characteristic
    		if(this._charsData.chars) {
    			for(var i=0; i<this._charsData.chars.length; i++) {
        			var char = this._charsData.chars[i];
        			if(char) {
        				var charData = value.addNewItemToList("char_search_criterion");
        				charData.setValue("id", char.id);
        				charData.setValue("description", char.name);
        			}
        		}
    		}
    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    CharacteristicsParameterField.prototype.setValueData = function (data) {
    	this._charsData = {};
    	
    	// Get search type
    	this._charsData.searchtype = data.getValue("search_type");
    	
    	var charsList = data.getSubList("char_search_criterion");
    	this._charsData.chars = [];
    	
    	// Get characteristic
    	if(charsList) {
    		for(var i=0; i<charsList.length; i++) {
        		var char = charsList[i];
        		
        		if(char) {
        			this._charsData.chars.push({"id": char.getValue("id"), "name": char.getValue("description")});
        		}
        	}
    	}
    	
    	// update the field input to display
    	this._updateInputText();
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    CharacteristicsParameterField.prototype.clear = function() {
    	this.input.empty();
    	$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("ADD")});
    	this._charsData = {};
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    CharacteristicsParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    CharacteristicsParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    CharacteristicsParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    CharacteristicsParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    CharacteristicsParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new CharacteristicsParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});