/**
 * Implements a simple Date Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils",
        "./popups/DateSearchPopup",
        "utils/webi18n!SearchInputs"
        ], function(
        		$, _, utils,
        		DateSearchPopup,
        		SearchInputs) {
	
		"use strict";

    /**
     * A Date parameter field
     * 
     * @constructor
     */
    var DateParameterField = function(div, fieldId, settings, dataProvider, fieldProperties) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.changeCallbacks = [];
    	
    	this._fieldProperties = fieldProperties;
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    DateParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	this._showFutureContext = false;
    	
    	if(this._fieldProperties.displayConfig.getValue("mode") == "past_and_future") {
    		this._showFutureContext = true;
    	}
    	
    	this._showLabel = this._fieldProperties.displayConfig.getValue("labels");
    	
    	var cssClass = "form-control search-input-lg attachleft";
    	
    	if(this.fieldId == "db_approval_date") {
    		cssClass = "form-control search-input-md";
    	}
    	
    	// Make the dropdown...
    	this.select = $("<select>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    		"class": cssClass
    	});

    	// Add the dropdown options
    	this._addSelectOptions();
    	
		// add the field to wrapper
    	this.div.append(this.select);
    	
    	
    	/*
    	 * Acts on a selection from a date element 
    	 * 
    	 * @param selectCtrl
    	 * @param moreOptionsUrl
    	 */
		this._handleDateAction = _.bind(function(selectCtrl) {
			// handle only when more_options is selected in the dropdown
			// need to open date popup for more options
			if (selectCtrl.value == 'more_options') {
				selectCtrl.options[0].selected = true;
				
				/*
				 * Callback to be executed when option is added using more_options popup
				 * 
				 * optionData is data required to create the dropdown option. for example:
				 * optionData = { operator: "last_n_days", value="5" }
				 * optionData = { operator: "next_n_days", value="5" }
				 * optionData = { operator: "date_range", fromdate: {"day": "21", "month": "04", "year": "2016"}, todate: {"day": "21", "month": "04", "year": "2016"} }
				 */
				var addMoreOptionCallback = _.bind(function(optionData) {
					var operator = optionData.operator;
					// Add the option to the dropdown and trigger the change
					if(operator == "last_n_days" || operator == "next_n_days") {
						this._appendNDaysOptions(optionData);
						this._triggerChange(true);
					} else if (operator == "date_range") {
						this._appendDateRangeOption(optionData);
						this._triggerChange(true);
					}
				}, this);
				
				// Create a DateSearchPopup if not created before, open the popup otherwise
				if(!this._dateSearchPopup) {
					// These options are needed to create popup
					var options = {};
					options.showFutureContext = this._showFutureContext.toString();
					options.showLabel = this._showLabel;
					options.fieldId = this.fieldId;
					
					//open the popup
					this._dateSearchPopup = new DateSearchPopup(options);
					this._dateSearchPopup.onSubmit(addMoreOptionCallback);
					this._dateSearchPopup.open();
				} else {
					this._dateSearchPopup.open();
				}
			}
		}, this);
    	
		
    	// listen to certain events on the field...
    	this.select.on("change", function(event){
    		self._onIntermediateUpdate();
    		self._handleDateAction(this);
    	});
    	
    	//blur=get out of the field >>> trigger final update event
    	this.select.on("blur", _.bind(this._onFinalUpdate, this));
    	
    };
    
    /**
     * Method to create and append options to the dropdown 
     */
    DateParameterField.prototype._addSelectOptions = function() {
    	
    	// create empty option
    	this.select.append($("<option>", {"value": ""}).text("").prop("selected", true));
    	
    	this._dateContextOption('today', SearchInputs.t('ADVSEARCH_TODAY'));
    	this._dateContextOption('yesterday', SearchInputs.t('ADVSEARCH_YESTERDAY'));
    	
    	if(this._showFutureContext) {
    		this._dateContextOption('tomorrow', SearchInputs.t('ADVSEARCH_TOMORROW'));
    	}
    	
    	this._dateContextOption('before_today', SearchInputs.t('ADVSEARCH_BEFORETODAY'));
    	
    	if(this._showFutureContext) {
    		this._dateContextOption('after_today', SearchInputs.t('ADVSEARCH_AFTERTODAY'));
    	}
    	
    	this._dateContextOption('this_week', SearchInputs.t('ADVSEARCH_THISWEEK'));
    	this._dateContextOption('last_week', SearchInputs.t('ADVSEARCH_LASTWEEK'));
    	
    	if(this._showFutureContext) {
    		this._dateContextOption('next_week', SearchInputs.t('ADVSEARCH_NEXTWEEK'));
    	}
    	
    	
    	this._dateContextOption('this_month', SearchInputs.t('ADVSEARCH_THISMONTH'));
    	this._dateContextOption('last_month', SearchInputs.t('ADVSEARCH_LASTMONTH'));
    	
    	if(this._showFutureContext) {
    		this._dateContextOption('next_month', SearchInputs.t('ADVSEARCH_NEXTMONTH'));
    	}
    	
    	this._dateContextOption('this_year', SearchInputs.t('ADVSEARCH_THISYEAR'));
    	this._dateContextOption('last_year', SearchInputs.t('ADVSEARCH_LASTYEAR'));
    	
    	if(this._showFutureContext) {
    		this._dateContextOption('next_year', SearchInputs.t('ADVSEARCH_NEXTYEAR'));
    	}
    	
    	// create more_options option
    	this.select.append($("<option>").attr({"value": "more_options"}).text(SearchInputs.t('ADVSEARCH_DATEMOREOPTIONS')));
    	
    };
    
    /**
     * Create and append last_n_days or next_n_days option to the dropdown
     * 
     * @param optionData - optionData is data required to create the dropdown option. for example:
	 * 						optionData = { operator: "last_n_days", value="5" }
	 * 						optionData = { operator: "next_n_days", value="5" }
     */
    DateParameterField.prototype._appendNDaysOptions = function(optionData) {
    	// Add an  Last-N-Days entry to the menu if the user has specified a Last-N-Days criterion
    	
    	var operator = optionData.operator;
    	
    	var optionText = null;
    	if(operator == 'last_n_days') {
    		optionText = SearchInputs.t('ADVSEARCH_LAST');
    	} else if(operator == 'next_n_days') {
    		optionText = SearchInputs.t('ADVSEARCH_NEXT');
    	}
    	
    	var dayCount = optionData.value;
		
		this.select.append(
				$("<option>").attr({"selected": "selected", "value": this._getOptionValue(optionData)})
				.text(optionText + " " + dayCount + " " + SearchInputs.t('ADVSEARCH_DAYSUNIT')));
    };
    
    /**
     * Create and append date_range option to the dropdown
     * 
     * @param optionData - optionData is data required to create the dropdown option. for example:
	 * 						optionData = { operator: "date_range", fromdate: {"day": "21", "month": "04", "year": "2016"}, todate: {"day": "21", "month": "04", "year": "2016"} }
     */
    DateParameterField.prototype._appendDateRangeOption = function(optionData) {
    	// Add a Date-Range entry to the menu if the user has specified a Date-Range criterion
    	
    	if(optionData.operator == 'date_range') {
    		var start = optionData.fromdate;
    		var end = optionData.todate;
    		
    		var dateText = "";
    		
    		var hasStartDate = false;
    		var hasEndDate = false;
    		
    		if(start && start.day && start.month && start.year) {
    			hasStartDate = true;
    		}
    		
    		if(end && end.day && end.month && end.year) {
    			hasEndDate = true;
    		}
    		
    		if(hasStartDate) {
    			if(!hasEndDate) {
    				dateText = dateText.concat(SearchInputs.t('ADVSEARCH_AFTERDATE') + " ");
    			}
    			dateText = dateText.concat(start.month + "/" + start.day + "/" + start.year);
    			if(hasEndDate) {
    				dateText = dateText.concat(" - ");
    			}
    		}
    		
    		if(hasEndDate) {
    			if(!hasStartDate) {
    				dateText = dateText.concat(SearchInputs.t('ADVSEARCH_BEFOREDATE') + " ");
    			}
    			dateText = dateText.concat(end.month + "/" + end.day + "/" + end.year);
    		}
    		
    		this.select.append(
    				$("<option>").attr({"selected": "selected", "value": this._getOptionValue(optionData)}).text(dateText)
    		);
    	}
    	
    };
    
    /**
     * Create an option and append to the dropdown 
     * 
     * @param dateContext - operator like today, tomorrow etc.
     * @param langStringKey - localized string for option text
     */
    DateParameterField.prototype._dateContextOption = function(dateContext, langStringKey) {
    	var optionData = {};
    	optionData.operator = dateContext;
    	
    	var optionValue = this._getOptionValue(optionData);
    	
    	var option = $('<option>').attr("value", optionValue).text(langStringKey);
    	this.select.append(option);
    };
    
    //
    // Internal event handling
    //
    DateParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    DateParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    DateParameterField.prototype.getValueData = function (value, feedback) {
    	// Add information to ValueData object
    	
    	value.clear();
    	
    	// The value string will be of json format
    	var selectedOptionVal = this.select.val();
    	
    	if(selectedOptionVal && selectedOptionVal != "more_options") {
    		// parse the value string
    		var selectedOptionData = JSON.parse(selectedOptionVal);
    		
    		// Check and update valueData for last_n_days and next_n_days
    		if(selectedOptionData.operator && selectedOptionData.value) {
    			value.setValue("operator", selectedOptionData.operator);
    			value.setValue("value", selectedOptionData.value);
    			
    		} 
    		// Check and update valueData for a date range
    		else if(selectedOptionData.operator && (selectedOptionData.fromdate || selectedOptionData.todate)) {
    			value.setValue("operator", selectedOptionData.operator);
    			
    			if(selectedOptionData.fromdate) {
    				var fromDateData = value.addNewItemToList("from_date_criterion");
    				fromDateData.setValue("year", selectedOptionData.fromdate.year);
    				fromDateData.setValue("month", selectedOptionData.fromdate.month);
    				fromDateData.setValue("day", selectedOptionData.fromdate.day);
    			}
    			
    			if (selectedOptionData.todate) {
    				var toDateData = value.addNewItemToList("to_date_criterion");
    				toDateData.setValue("year", selectedOptionData.todate.year);
    				toDateData.setValue("month", selectedOptionData.todate.month);
    				toDateData.setValue("day", selectedOptionData.todate.day);
    			}
    		}
    		// Check and update valueData for contextual date
    		else {
    			value.setValue("operator", selectedOptionData.operator);
    		}
    	} else if(selectedOptionVal === "") {
    		value.setValue("operator", "");
    	}
    	
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    DateParameterField.prototype.setValueData = function (data) {
    	// Get information from ValueData object
    	
    	// create optionData object for creating the option for the dropdown
    	var optionData = {};
    	optionData.operator = data.getValue("operator");
    	
    	if(optionData.operator == "next_n_days" || optionData.operator == "last_n_days") {
    		optionData.value = data.getValue("value");
    	} else if (optionData.operator == "date_range") {
    		if(data.getSubList("from_date_criterion")) {
    			var fromDateData = data.getSubList("from_date_criterion")[0];
    			optionData.fromdate = {};
        		optionData.fromdate.day = fromDateData.getValue("day");
        		optionData.fromdate.month = fromDateData.getValue("month");
        		optionData.fromdate.year = fromDateData.getValue("year");
    		}
    		
    		if(data.getSubList("to_date_criterion")) {
    			var toDateData = data.getSubList("to_date_criterion")[0];
        		optionData.todate = {};
        		optionData.todate.day = toDateData.getValue("day");
            	optionData.todate.month = toDateData.getValue("month");
            	optionData.todate.year = toDateData.getValue("year");
        	}
    	}
    	
    	var optionValue = null;
    	if(optionData.operator === "") {
    		optionValue = "";
    	} else {
    		// Get the option value in xml format
    		optionValue = this._getOptionValue(optionData);
    	}
    	
    	if(optionData.operator == "next_n_days" || optionData.operator == "last_n_days") {
    		// add if _n_days options is present in saved search
        	this._appendNDaysOptions(optionData);
    	} else if (optionData.operator == "date_range") {
    		// add if date_range option is present in saved search
        	this._appendDateRangeOption(optionData);
    	}
    	
    	// Set the select dropdown to the current option
    	this.select.val(optionValue);
    	this._triggerChange(true);
    	
    	return true;
    };
    
    /**
     * Create the option value in json format
     * 
     * @param optionData
     * @returns {String} - the json value for the option.
     */
    DateParameterField.prototype._getOptionValue = function(optionData) {
    	return JSON.stringify(optionData);
    };
    
    /**
     * Enable or disable the date dropdown
     */
    DateParameterField.prototype.setEnabled = function(isEnable) {
    	if(isEnable) {
    		this.select.prop("disabled", false);
    	} else {
    		this.select.prop("disabled", true);
    	}
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    DateParameterField.prototype.clear = function() {
    	if(this.smartNamesEnabled) {
    		// no need to implement, will not happen on Search Setup page.
    	} else{
	    	this.select.val("empty");
	    	this._triggerChange(false);
    	}
    };
    
    /**
     * Focus this field
     */
    DateParameterField.prototype.focus = function() {
    	this.select.focus();
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    DateParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    DateParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	//date field should never contain special Lucene characters
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    DateParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    DateParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    DateParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new DateParameterField(div, fieldName, settings, dataProvider, fieldProperties);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});