/**
 * Implements a field that consists of a single dropdown selector filled up with static options.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils",
        "../../dataobjects/FieldProperties",
        "../../dataobjects/ValueData",
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield"
        ], function(
        	$, 
        	_, 
        	utils,
        	FieldProperties,
        	ValueData,
        	SmartField) {

    /**
     * @constructor
     * 
     * @param {DOMElement} div - The dom element in which we should render the field
     * @param {String} fieldName - The name of the field for which this field is created
     * @param {SearchSettings} settings - Information about settings which are enabled
     *      - E.g. are SmartNames enabled?
     * @param {DataProvider} data - interface to have access to data
     * @param {FieldProperties} fieldProperties - additional properties of the field
     */
    var DropdownParameterField = function(div, fieldId, settings, dataProvider, fieldProperties) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.fieldProperties = fieldProperties;
    	this.dropdownOptions = fieldProperties.getListValues();
    	this.dropdownValueKey = fieldProperties.displayConfig.getValue("list_option_value_prop");
    	if(!this.dropdownValueKey) {
    		this.dropdownValueKey = "id";
    	}
    	//NOTE: make sure to use "Date+Time" render mode for any Date-type values shown in this Parameter Field
    	this.fieldProperties.displayConfig.setValue("date_render_type", "date_time");
    	this.isSmartnamesEnabled = this.settings.isSmartNamesEnabled() && !this.fieldProperties.displayConfig.getValue("sn_not_supported");
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    DropdownParameterField.prototype._initialize = function(){    	
    	// Make the dropdown selector...
    	this.select = $("<select>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    		"class":"form-control search-input-lg attachleft"
    	});
    	
    	// Get the display function (Used to display text & numbers. For numbers, formats them correctly)
		var displayFunc = utils.getParameterFieldDisplayFunction(this.fieldProperties, this.settings.getExternalSettings());
    	
    	_.each(this.dropdownOptions, function (valueData) {
    		//use either "id" property (default) or "name" as the value attribute of the option
    		//NOTE: fallback to id if name not found (usually for the static values that don't really have a name property)
    		var optVal = valueData.getValue(this.dropdownValueKey)?valueData.getValue(this.dropdownValueKey):valueData.getValue("id");
    		
    		$("<option>", {"value" : optVal, "text" : displayFunc(valueData.getValue("label"))}).appendTo(this.select);
    	}, this);
    	
    	//if we're dealing with a numeric attribute RS that uses units > show those next to the selector field
    	if (this.fieldProperties.displayUnit) {
    		var selectWrapper = $("<div>", {"class": "inputfield"}).append(this.select);

    		//units readout
			var localizedUnitVal = utils.getTranslatedUnit(this.fieldProperties.displayUnit);
	    	var unitslabel = $("<div>", {"class": "unitlabel add-padding-left"}).text(localizedUnitVal).appendTo(this.div);
	    	
    		//add the select and 'suffix it' by the units readout in a dedicated container
    		$("<div>", {"class": "fieldwithunitcontainer"}).append(selectWrapper).append(unitslabel).appendTo(this.div);
    	} else {
    		//only the drop-down/SN-Field
    		this.select.appendTo(this.div);
    	}

    	//---------------
    	//Callbacks
    	if(!this.isSmartnamesEnabled) {
    		// We have a normal input
    		this.select.change(_.bind(this._onFinalUpdate, this));
    	} else {
    		// We have a SmartField input
    		
    		// Convert the input into a SmartField
    		this.smartfield = new SmartField(this.select);
    		
    		// listen to certain events on the field...
    		this.smartfield.onValueChange(_.bind(this._onIntermediateUpdate, this));
    	}
    };
    
    //
    // Internal event handling
    //
    DropdownParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    DropdownParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    DropdownParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	if(this.isSmartnamesEnabled) {
    		value.setValue("valueSN", this.smartfield.getValue());
    		if (!this.smartfield.isSmartFieldInputActive()) {
    			//smartfield mode enabled, but the field is still using the dropdown selector
    			value.setValue("isDropdown", "true");
    		}
    	} else {
    		value.setValue("value", this.select.val());
    		value.setValue("isDropdown", "true");
    	}
    	
    };
    
    /**
     * Set the value, update the selector to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    DropdownParameterField.prototype.setValueData = function (data) {
    	if(this.isSmartnamesEnabled) {
    		// SmartName mode...
    		var valueSN = data.getValue("valueSN");
    		this.smartfield.setValue(valueSN || "");
    	} else {
    		// Normal mode...
    		this.select.val(data.getValue("value"));
    	}
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    DropdownParameterField.prototype.clear = function() {  	
    	this.select.find('option:eq(0)').prop('selected', true);
	    this._triggerChange(false);
    };
    
    /**
     * Focus this field
     */
    DropdownParameterField.prototype.focus = function() {
    	if(this.isSmartnamesEnabled) {
    		this.smartfield.focus();
    	} else {
    		this.select.focus();
    	}
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    DropdownParameterField.prototype.valueContainsSmartName = function() {
    	return this.isSmartnamesEnabled && utils.hasSmartNameInSmartField(this.smartfield);
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    DropdownParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	// Never check for special characters. We should never do that in SmartFields and dropdowns...
    	// This ParameterField is always one of those two.
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    DropdownParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    DropdownParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    DropdownParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {SearchFieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new DropdownParameterField(div, fieldName, settings, dataProvider, fieldProperties);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});