/**
 * Implements a Facet List field for use in dashboards
 */
define([
		"jquery",
		"underscore",
		"./facetutils/AbstractFacetParameterField",
		"utils/ColorUtils",
		"utils/webi18n!SearchDashboard"
	],
	function (
		$,
		_,
		AbstractFacetParameterField,
		ColorUtils,
		i18n) {
		"use strict";

		/**
		 * Displays facet information in a list
		 * @constructor
		 * @augments AbstractFacetParameterField
		 * @param {JQueryObj,HTMLObject} div - element in which the parameter field should be rendered
		 * @param {String} fieldId - field ID of the parameterfield	
		 * @param {SearchSettings} settings
		 * @param {FieldProperties} fieldProperties
		 */
		var FacetParameterField = function (div, fieldId, settings, fieldProperties) {
			AbstractFacetParameterField.call(this, div, fieldId, settings, fieldProperties);
		};

		FacetParameterField.prototype = Object.create(AbstractFacetParameterField.prototype);
		FacetParameterField.prototype.constructor = FacetParameterField;


		/**
		 * (private) Initialize the field.
		 */
		FacetParameterField.prototype._initialize = function () {

			this.div.css({
				"min-width": "150px"
			}); // set a minimal width to this div... (you can set the "absolute width also)


			// on click on an item in the list, select the value
			this._list = $("<div>", {
				'class': "list-group facet-list"
			}).on("click", ".list-group-item", function (event) {
				event.preventDefault();
				event.stopPropagation();

				if ($(event.target).data("value")) {
					this.selectFacetValue($(event.target).data("value"));
				}
			}.bind(this)).appendTo(this.div);
		};

		/**
		 * Update the rendering of the field
		 */

		FacetParameterField.prototype._doRender = function () {

			var facets = _.map(this._facetData, function (facetEntry) {
				var value = facetEntry.getValue("id");
				var facet = this.getFacetValueData(this._facetIDs[0], value);
				
				if (!this._facetSelection || !this._facetSelection.isSelectedSingleValue(facet.getValue("id"))) {
					// we only render unselected facet values
					var facetEl = $("<a>", {
						text: this.getLabelForFacet(facet),
						'class': "list-group-item facetLabelValue",
						data: {
							value: facet.getValue("id")
						}
					});
					var badge = $("<span>", {
						'class': "badge",
						text: facet.getValue("count", "0")
					}).appendTo(facetEl);
					
					
					// is a custom color defined for the value?
					var color = facet.getValue("color_rgb");
					if (color) {
						var  contrastColor = ColorUtils.calculateContrastColor(color, 0.75);
					
						badge.css({
							'background-color': color,
							'color': contrastColor
						});
					}

					
					return facetEl;
				}
			}, this);
			
			if(!this._facetData || this._facetData.length === 0) {
				facets.push($("<div>", {
					text: i18n.t("FACETFIELD_NOVALUES"),
					'class': "list-group-item facetLabelValue"}));
			}

			this._list.empty().append(_.compact(facets));
			
			this._checkForNumberOfValuesExceeded();
		};
	
		/**
		 * @override
		 */	
		FacetParameterField.prototype._checkForNumberOfValuesExceeded = function(){
			this._warning.removeClass();
			this._warning.removeAttr("title");
			if(this._maxFacetsExceeded){
				this._warning.addClass("icon icon-grey icon-WCR_Info-square12 pull-right");
				this._warning.attr("title",i18n.t("WARNING_MORE_FACETS"));
				this.div.append(this._warning);
			}
		};

		//
		// Factory...
		// Provides methods to create new parameter fields
		// & provides some general settings about the implementation
		//
		var factory = {
			/** 
			 * Make a new field of this type
			 * Returned object should implement certain functions
			 * @param {DOMElement} div - The dom element in which we should render the field
			 * @param {String} fieldName - The name of the field for which this field is created
			 * @param {SearchSettings} settings - Information about settings which are enabled
			 *     - E.g. are SmartNames enabled?
			 * @param {DataProvider} data - interface to have access to data
			 * @param {FieldProperties} fieldProperties - additional properties of the field
			 */
			make: function (div, fieldName, settings, dataProvider, fieldProperties) {
				return new FacetParameterField(div, fieldName, settings, fieldProperties);
			},
			/**
			 * Destroy a field created by this factory...
			 * @param obj - the object which was returned by this factory
			 */
			destroy: function (obj) {
				obj._destroy();
			}
		};
		return factory;
	});