/**
 * Implements a Facet Counter bar field for use in dashboards
 */
define([
		"jquery",
		"underscore",
		"./facetutils/AbstractFacetParameterField",
		"utils/ColorUtils",
		"utils/webi18n!SearchDashboard",
		"utils/webcss!component/searchresults/dashboard/blocks/SavedSearchCounterBlock.css"
	],
	function (
		$,
		_,
		AbstractFacetParameterField,
		ColorUtils,
		i18n,
		css) {
		"use strict";

		/**
		 * Displays facet information counter bar
		 * @constructor
		 * @augments AbstractFacetCounterBarParameterField
		 * @param {JQueryObj,HTMLObject} div - element in which the parameter field should be rendered
		 * @param {String} fieldId - field ID of the parameterfield	
		 * @param {SearchSettings} settings
		 * @param {FieldProperties} fieldProperties
		 */
		var FacetCounterBarParameterField = function (div, fieldId, settings, fieldProperties) {
			AbstractFacetParameterField.call(this, div, fieldId, settings, fieldProperties);
		};

		FacetCounterBarParameterField.prototype = Object.create(AbstractFacetParameterField.prototype);
		FacetCounterBarParameterField.prototype.constructor = FacetCounterBarParameterField;


		/**
		 * (private) Initialize the field.
		 */
		FacetCounterBarParameterField.prototype._initialize = function () {

			this.div.addClass("facetcounters");

			// on click on an item in the list, select the value
			var self = this;
			this.div.on("click", ".wcrsavedsearchcounterblock", function (event) {
				event.preventDefault();
				event.stopPropagation();
				var value = $(event.currentTarget).data("value");
				if (value) {

					if (self._facetSelection) {
						if (self._facetSelection.isSelectedSingleValue(value)) {
							self.clear();

						} else {
							self._facetSelection.clear();
							self.selectFacetValue(value);
						}
					} else {
						self.selectFacetValue(value);
					}

				}
			}).appendTo(this.div);
			
		};

		/**
		 * Update the rendering of the field
		 */

		FacetCounterBarParameterField.prototype._doRender = function () {

			var facets = _.map(this._facetData, function (facetEntry) {
				var value = facetEntry.getValue("id");
				var facet = this.getFacetValueData(this._facetIDs[0], value);

				var title = this.getLabelForFacet(facet);
				var counter = $("<div>", {
					'class': "wcrsavedsearchcounterblock runonclick",
					title: title,
					data: {
						value: facet.getValue("id")
					}
				});

				counter.append($("<div>", {
					'class': "count",
					text: facet.getValue("count", "0")
				}));
				counter.append($("<div>", {
					'class': "description",
					text: title
				}));

				// apply style
				counter.addClass(this._fieldProperties.instanceConfig.getValue("counterStyle"));

				var color = facet.getValue("color_rgb", "#fafafa");
				var contrastColor = ColorUtils.calculateContrastColor(color, 0.75);

				counter.css({
					"background-color": color,
					"color": contrastColor
				});

				// indicate selected
				if (this._facetSelection) {
					var isSelected = this._facetSelection.isSelectedSingleValue(facet.getValue("id"));
					var fade = this._facetSelection.getSelection().length > 0;
					counter.toggleClass("faded", fade && !isSelected);
					if (isSelected) {
						counter.addClass("active");
					}
				}
				return counter;

			}, this);
			
			// If no counter is present, add "No Values"
			if(facets.length === 0) {
				var counter = $("<div>", {
					'class': "wcrsavedsearchcounterblock novalues",
					title: i18n.t("FACETFIELD_NOVALUES")
				});

				counter.append($("<div>", {
					'class': "description",
					text: i18n.t("FACETFIELD_NOVALUES")
				}));
				
				facets.push(counter);
			}

			this.div.empty().append(_.compact(facets));

			this._checkForNumberOfValuesExceeded();	
		};

		//
		// Factory...
		// Provides methods to create new parameter fields
		// & provides some general settings about the implementation
		//
		var factory = {
			/** 
			 * Make a new field of this type
			 * Returned object should implement certain functions
			 * @param {DOMElement} div - The dom element in which we should render the field
			 * @param {String} fieldName - The name of the field for which this field is created
			 * @param {SearchSettings} settings - Information about settings which are enabled
			 *     - E.g. are SmartNames enabled?
			 * @param {DataProvider} data - interface to have access to data
			 * @param {FieldProperties} fieldProperties - additional properties of the field
			 */
			make: function (div, fieldName, settings, dataProvider, fieldProperties) {
				return new FacetCounterBarParameterField(div, fieldName, settings, fieldProperties);
			},
			/**
			 * Destroy a field created by this factory...
			 * @param obj - the object which was returned by this factory
			 */
			destroy: function (obj) {
				obj._destroy();
			}
		};
		return factory;
	});