/**
 * Implements First or Any level for the Design Panel.
 */
define([
		"require",
        "jquery",
        "underscore",
        "../../Utils",
        "utils/webi18n!SearchInputs",

        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding"
        ],
	function (
		require,
		$,
		_,
		utils,
		i18n,
		SmartField,
		SmartNameEncoding) {
		"use strict";

		/**
		 * @constructor
		 */
		var FirstAnyChoiceParameterField = function (div, fieldId, settings, dataProvider, fieldProperties) {
			this.div = $(div);
			this.fieldId = fieldId;
			this.settings = settings;
			this.dataProvider = dataProvider;
			this.fieldProperties = fieldProperties;

			this.changeCallbacks = [];

			this._initialize();
		};

		/**
		 * (private) Initialize the field.
		 */
		FirstAnyChoiceParameterField.prototype._initialize = function () {
			var self = this;

			// Create the Input field for First or Any
			this.constructFieldElements();

			if (!this.settings.isSmartNamesEnabled()) {
				// We have a normal input

				// listen to certain events on the field...

				//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
				this.input.on("keyup", function (event) {
					self._onIntermediateUpdate();

					//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
					if (event.keyCode == 13) {
						$(this).blur();
					}
				});

				//blur=get out of the field >>> trigger final update event
				this.input.on("blur", _.bind(this._onFinalUpdate, this));

			} else {
				// We have a SmartField input

				// Convert the input into a SmartField
				this.smartField = new SmartField(this.input);

				// listen to certain events on the field...
				this.smartField.onValueChange(_.bind(this._onIntermediateUpdate, this));
			}

			// on click event when option first is clicked 
			this.first.on("click", _.bind(function () {
				this._onFirstAnyTypeChanged(this.first);
			}, this));

			// on click event when option any is clicked 
			this.any.on("click", _.bind(function () {
				this._onFirstAnyTypeChanged(this.any);
			}, this));

			// on click event when option none is clicked 
			this.none.on("click", _.bind(function () {
				this._onFirstAnyTypeChanged(this.none);
			}, this));

		};

		/**
		 * Make the First, Any or None choice field 
		 * 
		 * This method will create fields [this.first] , [this.any] , [this.none] , [this.input] - This fields are public and it will accessible all methods
		 * 
		 */
		FirstAnyChoiceParameterField.prototype.constructFieldElements = function () {

			var table = $("<table>").addClass("parameterFieldTable");
			var row = $("<tr>");
			var colButtons = $("<td>");
			var colInputs = $("<td>").addClass("widecell");

			row.append(colButtons, colInputs);
			table.append(row);

			var buttonDiv = $("<div>").addClass("btn-group searchtogglegroup");
			colButtons.append(buttonDiv);

			// Make the first button
			var firstTitle = this.fieldProperties.displayConfig.getValue("first_tooltip");
			var firstIcon = this.fieldProperties.displayConfig.getValue("first_icon");
			this.first = $("<div>", {
				"data-sahi-id":this.fieldId+"_first",// For automated testing
				"align": "top",
				"value": "FIRST",
				"class": "btn btn-default active",
				"title": i18n.t(firstTitle)
			}).css({
				"cursor": "pointer"
			}).append($("<i>").addClass(firstIcon));

			buttonDiv.append(this.first);

			// Make the any button
			var anyTitle = this.fieldProperties.displayConfig.getValue("any_tooltip");
			var anyIcon = this.fieldProperties.displayConfig.getValue("any_icon");
			this.any = $("<div>", {
				"data-sahi-id":this.fieldId+"_any",// For automated testing
				"align": "top",
				"value": "ANY",
				"class": "btn btn-default",
				"title": i18n.t(anyTitle)
			}).css({
				"cursor": "pointer"
			}).append($("<i>").addClass(anyIcon));

			buttonDiv.append(this.any);

			// Make the none button
			var noneTitle = this.fieldProperties.displayConfig.getValue("none_tooltip");
			var noneIcon = this.fieldProperties.displayConfig.getValue("none_icon");
			this.none = $("<div>", {
				"data-sahi-id":this.fieldId+"_none",// For automated testing
				"align": "top",
				"value": "NONE",
				"class": "btn btn-default",
				"title": i18n.t(noneTitle)
			}).css({
				"cursor": "pointer"
			}).append($("<i>").addClass(noneIcon));

			buttonDiv.append(this.none);

			// Make the none text
	    	this.noneLabel = $("<div>", {
	    		"data-sahi-id":this.fieldId+"_text_none",// For automated testing
	    		"class" : "search_metext"
	    	}).text(i18n.t(noneTitle)).hide();
	    	
	    	colInputs.append(this.noneLabel);
	    	
			// Make the input...
			this.input = $("<input>", {
				"data-sahi-id":this.fieldId,// For automated testing
				"type": "text",
				"align": "top",
				"value": "",
				"class": "form-control search-input-lg"
			}).attr("size", "30");

			colInputs.append(this.input);

			// Add tooltip for the button options
			require(["jqueryUI"], _.bind(function () {
				this.first.tooltip({
					position: {
						my: "center bottom-5",
						at: "center top"
					}
				});
				this.any.tooltip({
					position: {
						my: "center bottom-5",
						at: "center top"
					}
				});
				this.none.tooltip({
					position: {
						my: "center bottom-5",
						at: "center top"
					}
				});
			}, this));

			// add the fields
			this.div.append(table);

		};

		/**
		 * Just Highlight the Given button
		 * @param {JQuery} activeElement - The First or Any button that's been selected
		 */
		FirstAnyChoiceParameterField.prototype._highLightActiveElement = function (activeElement) {
			this.first.removeClass("active");
			this.any.removeClass("active");
			this.none.removeClass("active");

			activeElement.addClass("active");
		};

		/**
		 *  This function is called when a first/any type selector is changed
		 *  
		 * @param {JQuery} element - The option button selected "First" or "Any" 
		 *        
		 */
		FirstAnyChoiceParameterField.prototype._onFirstAnyTypeChanged = function (element) {
			var val = element.val();
			if ("FIRST" == val) {
				// Highlight First Option
				this._highLightActiveElement(this.first);
			} else if ("ANY" == val) {
				// Highlight Any Option
				this._highLightActiveElement(this.any);
			} else if ("NONE" == val) {
				// Highlight Any Option
				this._highLightActiveElement(this.none);
			}

			if ("FIRST" == val || "ANY" == val) {
    	    	if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.clear();
        			this.smartField.getJQElement().show();
				} else {
					this.input.val("");
					this.input.show();
				}
   	    		this.noneLabel.hide();
			}
			else {
   	    		if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.clear();
        			this.smartField.getJQElement().hide();
				} else {
					this.input.val("");
        			this.input.hide();
				}
   	    		this.noneLabel.show();				
			}
			// call the update method to notify the changes
			this._onFinalUpdate();
		};

		//
		// Internal event handling
		//
		FirstAnyChoiceParameterField.prototype._onIntermediateUpdate = function () {
			this._triggerChange(true);
		};

		FirstAnyChoiceParameterField.prototype._onFinalUpdate = function () {
			this._triggerChange(false);
		};

		//
		// Set & Get the value
		//

		/**
		 * Get the value by filling the value object.
		 * Also do the validation in the mean time and
		 * use the feedback object to give the information back.
		 * 
		 * @param {ValueData} value - the object to put all the value information in
		 * @param {ValidationFeedback} feedback - the object 
		 */
		FirstAnyChoiceParameterField.prototype.getValueData = function (value, feedback) {

			value.clear();

			if (this.first.hasClass("active")) {
				value.setValue("modus", "FIRST");
			} else if (this.any.hasClass("active")) {
				value.setValue("modus", "ANY");
			} else {
				value.setValue("modus", "NONE");
			}

			if (this.settings.isSmartNamesEnabled()) {
				value.setValue("valueSN", this.smartField.getValue());
			} else {
				value.setValue("value", this.input.val());
			}
		};

		/**
		 * Set the value, update the field to represent this value.
		 * 
		 * @param {ValueData} data - the object which contains all the information about the value
		 * @returns {Boolean} true if the field can represent the data which is set
		 */
		FirstAnyChoiceParameterField.prototype.setValueData = function (data) {
			var modus = data.getValue("modus");

			if (modus == "FIRST") {
				this._highLightActiveElement(this.first);
			} else if (modus == "ANY") {
				this._highLightActiveElement(this.any);
			} else if (modus == "NONE") {
				this._highLightActiveElement(this.none);
			}

			if (modus == "FIRST" || modus == "ANY") {
				if (this.settings.isSmartNamesEnabled()) {
					// SmartName mode...
					var valueSN = data.getValue("valueSN");
					this.smartField.setValue(valueSN || "");
				} else {
					// Normal mode...
					var value = data.getValue("value");
					this.input.val(value || "");
				}
				this.noneLabel.hide();				
			}
			else {
				if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.getJQElement().hide();
				} else{
					this.input.hide();
				}
				this.noneLabel.show();				
			}

			return true;
		};

		/**
		 * Clear the Field (called by "Clear this form" functionality) 
		 */
		FirstAnyChoiceParameterField.prototype.clear = function () {
			this.input.val("");
			this.input.show();
			this._highLightActiveElement(this.first);
	    	this.noneLabel.hide();
			this._triggerChange(false);
		};

		/**
		 * Does this field contain a SmartName?
		 * 
		 * @returns {boolean} true when this field contains a SmartName
		 */
		FirstAnyChoiceParameterField.prototype.valueContainsSmartName = function () {
			return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this.smartField);
		};

		/**
		 * Does this field contain a special search character?
		 * 
		 * @param {char[]} specialCharacters - an array of special characters...
		 */
		FirstAnyChoiceParameterField.prototype.valueContainsSpecialSearchCharacters = function (specialCharacters) {
			if (this.settings.isSmartNamesEnabled()) {
				// Don't check for special characters in a SmartField...
				// The hint would otherwise also show up if you use the autocomplete to type a SmartName...
				// (For the autocomplete you start typing a SmartName with "[" , which is a special character)
				//return utils.hasCharsInSmartField(this.smartfield, specialCharacters);
				return false;
			} else {
				return utils.hasCharsInStr(this.input.val(), specialCharacters);
			}
		};

		//
		// Change listeners
		//

		/**
		 * Registers a new function to call when the field changes...
		 * 
		 * @param {Function} callback - a function to call when the field changes. 
		 * Function has one parameter, which should be true when the user is not finished with modifying the field.
		 */
		FirstAnyChoiceParameterField.prototype.onChange = function (callback) {
			this.changeCallbacks.push(callback);
		};

		/**
		 * (private) Trigger a change event
		 */
		FirstAnyChoiceParameterField.prototype._triggerChange = function (intermediatechange) {
			$.each(this.changeCallbacks, function (i, callback) {
				callback(intermediatechange);
			});
		};

		//
		// Destructor
		//

		/**
		 * Destroys this field...
		 * Remove listeners and clean up the div.
		 */
		FirstAnyChoiceParameterField.prototype._destroy = function () {
			this.div.empty();
			this.changeCallbacks = [];
		};

		//
		// Factory...
		// Provides methods to create new parameter fields
		// & provides some general settings about the implementation
		//
		var factory = {
			/** 
			 * Make a new field of this type
			 * Returned object should implement certain functions
			 * @param {DOMElement} div - The dom element in which we should render the field
			 * @param {String} fieldName - The name of the field for which this field is created
			 * @param {SearchSettings} settings - Information about settings which are enabled 
			 *     - E.g. are SmartNames enabled?
			 * @param {DataProvider} data - interface to have access to data
			 * @param {FieldProperties} fieldProperties - additional properties of the field
			 */
			make: function (div, fieldName, settings, dataProvider, fieldProperties) {
				return new FirstAnyChoiceParameterField(div, fieldName, settings, dataProvider, fieldProperties);
			},
			/**
			 * Destroy a field created by this factory...
			 * @param obj - the object which was returned by this factory
			 */
			destroy: function (obj) {
				obj._destroy();
			}
		};
		return factory;
	});