/**
 * Implements a Float Field for the Search Setup and Design Panel.
 */
define(["jquery", "underscore", "../../Utils"], function($, _, utils) {

    /**
     * @constructor
     */
    var FloatParameterField = function(div, fieldId, settings, dataProvider, unitKey) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	// This field does not have SmartNames enabled at the moment.
    	this.smartNamesEnabled = false;//settings.isSmartNamesEnabled;
    	this.changeCallbacks = [];
    	this.unitKey = unitKey;
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    FloatParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	// Class for style in design panel
    	this.div.parents(".inputcell").first().addClass("multifields");
    	
    	// Make the select
    	this.select = $("<select>",  {
    		"data-sahi-id":"select_"+this.fieldId,// For automated testing
    		"class":"form-control search-input-sm"
    	});
    	this.optionequals = $("<option>", {"value": "equals", "text": "="});
    	this.select.append(this.optionequals);
    	this.optionrange = $("<option>", {"value": "range", "text": ">="});
    	this.select.append(this.optionrange);
    	
    	// Make the first input
    	this.input1 = $("<input>", {
    		"data-sahi-id":"input_1_"+this.fieldId,// For automated testing
    		"class":"paramfield_floatinput1 form-control  search-input-sm"
    	}).attr({
    		"size":"6",
    		"value":""
    	});
    	
    	// Make the second input
    	this.input2 = $("<input>", {
    		"data-sahi-id":"input_2_"+this.fieldId,// For automated testing
    		"class":"paramfield_floatinput2 form-control  search-input-sm"
    	}).attr({
    		"size":"6",
    		"value":""
    	});
    	
    	// Make the operator label
    	this.operatorlabel = $("<label>", {
    		"data-sahi-id":"label_"+this.fieldId,// For automated testing
    		"text":"??"
    	});
    	
    	// Make the units span
    	this.unitslabel = $("<span>");
    	if(this.unitKey) {
    		this.unitslabel.text(utils.getTranslatedUnit(this.unitKey));
    	}
    	
    	//
    	// Add the fields (and layout them
    	//

    	// Make a table to put the fields next to each other
    	this.table = $("<table>").attr({"cellspacing":"0", "cellpadding":"0"});
    	this.row = $("<tr>");
    	this.table.append(this.row);
    	function addCell(component, spaceAfter, cssclass){
    		var cell = $("<td>", {"class":cssclass?cssclass:"darktext_greenlink_11"}).append(component).css({"padding-right":spaceAfter?spaceAfter:"0px"});
    		self.row.append(cell);
    	};
    	addCell(this.select, "5px");
    	addCell(this.input1, "20px", "darktext_darklink_11");
    	addCell(this.operatorlabel, "5px");
    	addCell(this.input2, "5px", "darktext_darklink_11");
    	addCell(this.unitslabel);
    	
    	// add the table
    	this.div.append(this.table);
    	
    	// initialize the mode
    	this.setMode("equals");
    	
    	if(!this.smartNamesEnabled) {
    		// We have a normal input
	    	// listen to certain events on the fields...
    		
    		//'Criterion Search Mode' dropdown
    		this.select.on("change", function(){
    			self.handleToleranceValue();
    			self._onIntermediateUpdate();
        	});
    		
    		//value/greater than value
	    	this.input1.on("keyup", function(event){
	    		var validated = true;
	    		validated = self.validatePartialNumericInput($(this), true);
	    		self._onIntermediateUpdate();
	    		 
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    		return validated;
	    		
	    	});
	    	this.input1.on("blur", _.bind(this._onFinalUpdate, this));
	    	
	    	//tolerance/less than value
	    	this.input2.on("keyup", function(event){
	    		//tolerance should not be a negative number 
	    		//('tolerance' context is active only when 'equals' mode is currently selected)
	    		var allowNegativeVal = (self.select.val() === 'range');
	    		
	    		var validated = true;
	    		validated = self.validatePartialNumericInput($(this), allowNegativeVal);
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    		return validated;
	    	});
	    	this.input2.on("blur", _.bind(this._onFinalUpdate, this));
	    	
    	} else {
    		throw new Error("Not implemented yet...");
    	}
    };
    
    //
    // Internal event handling
    //
    FloatParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    FloatParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Controlling & validating the field
    //
    /**
     * Set the mode of the field
     * @param mode (equals/range)
     */
    FloatParameterField.prototype.setMode = function(mode) {
    	//only update the selector state if the target mode value is passed explicetly
    	if (!mode) {
    		mode = $("option:selected", this.select).val();
    	} else {
    		this.select.val(mode);	
    	}
    	
    	//update the field label
    	if(mode==="equals") {
    		this.operatorlabel.text("+/-");
    	} else {
    		this.operatorlabel.text("<=");
    	}
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    FloatParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	if(this.smartNamesEnabled) {
    		throw new Error("Not implemented yet...");
    	}else{
    		//TODO: should validation be done here?
    		
    		// Put the values in the object
    		value.setValue("mode", $("option:selected", this.select).val());
    		value.setValue("unit", this.unitKey);
    		
    		//set the value properties according to the selected 'mode' (equals mode as default)
    		var input1ValKey = "value";
    		var input2ValKey = "tolerance";
    		if (this.select.val() === 'range') {
    			input1ValKey = "gt_value";
    			input2ValKey = "lt_value";
    		}
    		value.setValue(input1ValKey, this.input1.val());
    		value.setValue(input2ValKey, this.input2.val());
    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     */
    FloatParameterField.prototype.setValueData = function (data) {
    	var foundUnsupportedMode = false;
    	
    	if(this.smartNamesEnabled) {
    		throw new Error("Not implemented yet...");
    	} else {    		
    		// Get the mode and check if it is valid...
    		var targetModeVal = data.getValue("mode", "equals").toLowerCase();
    		if(!(targetModeVal === "equals" || targetModeVal === "range")) {
    			targetModeVal = "equals";
    			// rerun with equals mode!
    			foundUnsupportedMode = true;
    		}
    		
    		// Put the values in the object (and initialize the 'mode' UI state)
    		this.setMode(targetModeVal);
    		
    		if (targetModeVal === 'equals') {
    			this.input1.val(data.getValue("value", ""));
        		this.input2.val(data.getValue("tolerance", ""));
    		} else if (targetModeVal === 'range') {
    			this.input1.val(data.getValue("gt_value", ""));
        		this.input2.val(data.getValue("lt_value", ""));
    		}	
    	}
    	
    	return !foundUnsupportedMode;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    FloatParameterField.prototype.clear = function() {
    	this.input1.val("");
    	this.input2.val("");
    	this.setMode("equals");
    	this._triggerChange(false);
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    FloatParameterField.prototype.valueContainsSmartName = function() {
    	// Note: This field does not implement SmartNames yet
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {Function} checkString - function to check a certain string for special characters
     */
    FloatParameterField.prototype.valueContainsSpecialSearchCharacters = function(checkString) {
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    FloatParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    FloatParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    //VALIDATION functions
    //
    
    /**
     * Checks whether the tolerance (only for 'equals' criterion search mode) is a valid=positive number
     * Also makes sure to switch the field labels to reflect the selected criterion search mode 
     */
    FloatParameterField.prototype.handleToleranceValue = function ()
	{      	
		if(this.select.val() === "equals")
		{
			var toleranceFieldValue = this.input2.val();
			if (toleranceFieldValue.length > 0 && !isNaN(toleranceFieldValue) && toleranceFieldValue < 0) {
	            // a switch to 'equals' mode leaves the tolerance field with a negative numeric value --> 
				// inform the user and clear out the field
	            alert(utils.getLocalizedLabel("SEARCHCRITERION_TOLERANCE_NEGATIVE_FEEDBACK"));
	            this.input2.val("");
			}
	   }
		
		//update the field labels based on the selected criterion search mode
		this.setMode();
	};
	
	/**
	 * Use validatePartialNumericInput to validate
	 * numbers that may not yet be fully typed (including just a decimal
	 * point so far)
	 * 
	 * @param element - JQuery object; HTML element that is being validated
	 * @param allowNegative - boolean; do we allow negative numbers in this field?
	 * 
	 * @returns {Boolean}
	 */
	FloatParameterField.prototype.validatePartialNumericInput = function (element, allowNegative)
	{
	   var value = element.val();
	   
	   // Blank field is allowed
	   if(value == "") {
		   return true;   
	   }
	   
	   // The user has only entered a decimal point and/or negative sign so far
	   if(allowNegative)
	   {
	      if(value == "-")
	      return true;
	      if(value == "-.")
	      return true;
	   }
	   
	   if(value == ".") {
		   return true;   
	   }
	   
	   //make sure the value is valid Float number
	   var checkNum = parseFloat(value);
	   if(isNaN(checkNum))
	   {
	      // They did not enter a number...
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_FLOAT_NUMBER_FEEDBACK"));
	      element.val("");
	      return false;
	   }
	   
	   if(checkNum != value)
	   {
	      // They entered something with a number at the front - e.g. 12Y >>>
		   //revert to the numeric part only
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_FLOAT_NUMBER_FEEDBACK"));
	      element.val(checkNum);
	      return false;
	   }
	   
	   //make sure a non-negative is not there at the moment (if not allowed)
	   if(!allowNegative &&(checkNum < 0))
	   {
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_NEGATIVE_NUMBER_FEEDBACK"));
	      return false;
	   }
	   
	   return true;
	};
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    FloatParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new FloatParameterField(div, fieldName, settings, dataProvider, fieldProperties.displayUnit);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        },
        /**
         * The constructor of this ParameterField... 
         * Used for extending this Parameterfield.
         */
        constructor: FloatParameterField
    };
    return factory;
});