/**
 * Implements a simple Text Field for the Design Panel.
 */
define(["jquery", "underscore", "../../Utils"], function($, _, utils) {

    /**
     * @constructor
     */
    var FloatPercentParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	// This field does not have SmartNames enabled at the moment.
    	this.smartNamesEnabled = false;//settings.isSmartNamesEnabled;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    FloatPercentParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	// Class for style in design panel
    	this.div.parents(".inputcell").first().addClass("multifields");
    	
    	// Make the first input
    	this.input1 = $("<input>", {
    		"data-sahi-id":"input_1_"+this.fieldId,// For automated testing
    		"class":"paramfield_floatinput1 form-control search-input-sm"
    	}).attr({
    		"size":"3",
    		"value":""
    	});
    	
    	// Make the second input
    	this.input2 = $("<input>", {
    		"data-sahi-id":"input_2_"+this.fieldId,// For automated testing
    		"class":"paramfield_floatinput2 form-control search-input-sm"
    	}).attr({
    		"size":"3",
    		"value":""
    	});
    	
    	//
    	// Add the fields (and layout them
    	//

    	// Make a table to put the fields next to each other
    	this.table = $("<table>").attr({"cellspacing":"0", "cellpadding":"0"});
    	this.row = $("<tr>");
    	this.table.append(this.row);
    	function addCell(component, spaceAfter){
    		var cell = $("<td>", {"class":"darktext_darklink_11"}).append(component).css({"padding-right":spaceAfter?spaceAfter:"0px"});
    		self.row.append(cell);
    	};
    	addCell($("<span>", {"html": "&nbsp;&gt;="}), "5px");
    	addCell(this.input1, "20px");
    	addCell($("<span>", {"text": "<"}), "5px");
    	addCell(this.input2, "5px");
    	addCell($("<span>", {"text": "%"}));
    	
    	// add the table
    	this.div.append(this.table);
    	
    	if(!this.smartNamesEnabled) {
    		// We have a normal input
    		
	    	this.input1.on("keyup", function(event){
	    		var validated = true;
	    		validated = self.validatePartialNumericInput($(this), false);
	    		self._onIntermediateUpdate();
	    		 
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    		return validated;
	    	});
	    	this.input1.on("blur", _.bind(this._onFinalUpdate, this));
	    	
	    	this.input2.on("keyup", function(event){
	    		var validated = true;
	    		validated = self.validatePartialNumericInput($(this), false);
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    		return validated;
	    	});
	    	this.input2.on("blur", _.bind(this._onFinalUpdate, this));
    	} else {
    		throw new Error("Not implemented yet...");
    	}
    };
    
    //
    // Internal event handling
    //
    FloatPercentParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    FloatPercentParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Controlling & validating the field
    //
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    FloatPercentParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	if(this.smartNamesEnabled) {
    		throw new Error("Not implemented yet...");
    	}else{
    		//TODO: should validation be done here?
    		
    		// Put the values in the object
    		value.setValue("mode", "range");
    		value.setValue("unit", "%");
    		value.setValue("gt_value", this.input1.val());
    		value.setValue("lt_value", this.input2.val());
    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     */
    FloatPercentParameterField.prototype.setValueData = function (data) {
    	if(data.getValue("mode")==="equals") {
    		// We cannot display that value
    		// Keep current value & rerun
    		return false;
    	}
    	
    	if(this.smartNamesEnabled) {
    		throw new Error("Not implemented yet...");
    	} else {   		
    		// Put the values in the object
   			this.input1.val(data.getValue("gt_value", ""));
        	this.input2.val(data.getValue("lt_value", ""));
    	}
    	
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    FloatPercentParameterField.prototype.clear = function() {
    	this.input1.val("");
    	this.input2.val("");
    	this._triggerChange(false);
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    FloatPercentParameterField.prototype.valueContainsSmartName = function() {
    	// Note: This field does not support SmartNames yet
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {Function} checkString - function to check a certain string for special characters
     */
    FloatPercentParameterField.prototype.valueContainsSpecialSearchCharacters = function(checkString) {
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    FloatPercentParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    FloatPercentParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    //VALIDATION functions
    //
	
	/**
	 * Use validatePartialNumericInput to validate
	 * numbers that may not yet be fully typed (including just a decimal
	 * point so far)
	 * 
	 * @param element - JQuery object; HTML element that is being validated
	 * @param allowNegative - boolean; do we allow negative numbers in this field?
	 * 
	 * @returns {Boolean}
	 */
    FloatPercentParameterField.prototype.validatePartialNumericInput = function (element, allowNegative)
	{
	   var value = element.val();
	   
	   // Blank field is allowed
	   if(value == "") {
		   return true;   
	   }
	   
	   // The user has only entered a decimal point and/or negative sign so far
	   if(allowNegative)
	   {
	      if(value == "-")
	      return true;
	      if(value == "-.")
	      return true;
	   }
	   
	   if(value == ".") {
		   return true;   
	   }
	   
	   //make sure the value is valid Float number
	   var checkNum = parseFloat(value);
	   if(isNaN(checkNum))
	   {
	      // They did not enter a number...
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_FLOAT_NUMBER_FEEDBACK"));
	      element.val("");
	      return false;
	   }
	   
	   if(checkNum != value)
	   {
	      // They entered something with a number at the front - e.g. 12Y >>>
		   //revert to the numeric part only
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_FLOAT_NUMBER_FEEDBACK"));
	      element.val(checkNum);
	      return false;
	   }
	   
	   //make sure a non-negative is not there at the moment (if not allowed)
	   if(!allowNegative &&(checkNum < 0))
	   {
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_NEGATIVE_NUMBER_FEEDBACK"));
	      return false;
	   }
	   
	   return true;
	};
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    FloatPercentParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new FloatPercentParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        },
        /**
         * The constructor of this ParameterField... 
         * Used for extending this Parameterfield.
         */
        constructor: FloatPercentParameterField
    };
    return factory;
});