/**
 * This controls a field in search which is a dropdown...
 * 
 * It has some special features like: (if enabled)
 *  - SmartNames (by using the SmartField component)
 *  - Entering a lucene query (multi values)
 */
define([
        "jquery",
        "underscore",
        "../../Utils",

        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding",

        "utils/webi18n!SearchInputs"
        ], function (
	$, _, utils,
	SmartField,
	SmartNameEncoding,
	SearchInputs) {
	"use strict";

	/**
	 * @constructor
	 * 
	 * @param {DOMElement} div - The dom element in which we should render the field
	 * @param {String} fieldName - The name of the field for which this field is created
	 * @param {SearchSettings} settings - Information about settings which are enabled
	 *      - E.g. are SmartNames enabled?
	 * @param {DataProvider} data - interface to have access to data
	 * @param {FieldProperties} fieldProperties - additional properties of the field
	 */
	var FreeQueryDropdownField = function (div, fieldId, settings, dataProvider, fieldProperties) {
		this.div = $(div);
		this.fieldId = fieldId;
		this.settings = settings;
		this.fieldProperties = fieldProperties;

		this.dropdownOptions = fieldProperties.getListValues();
		this.dropdownValueKey = fieldProperties.displayConfig.getValue("list_option_value_prop");
		if (!this.dropdownValueKey) {
			this.dropdownValueKey = "id";
		}
		this.displayConfig = fieldProperties.displayConfig;

		// If it is a smartname field
		this.isSmartnamesEnabled = this.settings.isSmartNamesEnabled() && !this.displayConfig.getValue("sn_not_supported");
		// true, if it is a freequery text input field
		this._stateInputField = false;

		this.changeCallbacks = [];

		this._initialize();
	};

	/**
	 * (private) Initialize the field.
	 */
	FreeQueryDropdownField.prototype._initialize = function () {

		// Make the input div...
		this.inputdiv = $("<div>", {
			"class": "input-group searchdropdownfielddiv full-width"
		});


		/*
		 * The normal dropdown
		 */
		this._dropdown = $("<select>", {
			"data-sahi-id":this.fieldId,// For automated testing
			"class": "dropdownqueryfield dropdown form-control attachleft search-input-lg"
		});
		this._dropdown.change(_.bind(function () {
			this._updateValue(true);
		}, this));

		// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
		var displayFunc = utils.getParameterFieldDisplayFunction(this.fieldProperties, this.settings.getExternalSettings());

		_.each(this.dropdownOptions, function (valueData) {
			//use either "id" property (default) or "name" as the value attribute of the option
			//NOTE: fallback to id if name not found (usually for the static values that don't really have a name property)
			var value = valueData.getValue(this.dropdownValueKey) ? valueData.getValue(this.dropdownValueKey) : valueData.getValue("id");
			var name = valueData.getValue("name");
			var label = displayFunc(valueData.getValue("label"));

			var option = $("<option>", {
				"value": value,
				"text": label
			});

			if (name) {
				option.attr({
					"querydisplayvalue": name
				});
			}

			option.appendTo(this._dropdown);
		}, this);


		/*
		 * The free query text input field
		 */
		this._inputfield = $("<input>").attr({
			"data-sahi-id":this.fieldId+"_freequery",// For automated testing
			'type':'text',
			'class':'form-control attachleft'
		}).val("").hide();
		this._inputfield.on('input', _.bind(function () {
			// false = do not rerun while typing
			this._updateValue(false);
		}, this));
		this._inputfield.on('blur', _.bind(function () {
			// When we blur the input field, also remove the trailing OR from the input
			this._inputfield.val(this._inputfield.val().replace(/ OR\s*$/, ""));

			// potentially trigger a rerun
			this._updateValue(true);
		}, this));
		this._inputfield.on('focus', _.bind(function () {
			// Add the "OR" again, and trigger autocomplete...
			var reducedval = this._inputfield.val().replace(/ OR\s*$/, "");
			if (reducedval.length > 0) {
				this._inputfield.val(reducedval + " OR ");

				// Make sure the cursor is put at the end of the input... (otherwise, cursor jumps to beginning of input in IE)
				// Source: http://stackoverflow.com/questions/511088/use-javascript-to-place-cursor-at-end-of-text-in-text-input-element

				setTimeout(_.bind(function () {
					this._inputfield.get(0).selectionStart = this._inputfield.get(0).selectionEnd = 10000;
				}, this), 0);
			}
			this._inputfield.autocomplete("search", "");
		}, this));


		this._fields = $("<div>", {
			"class": "searchdropdownfield inputdiv main-content"
		}).append(this._dropdown, this._inputfield);

		// Enable autocomplete feature for the free text input field
		this._initializeAutocomplete();

		// make the switch button (to toggle between normal select and free text input)
		this._switchbutton = $("<div>", {
			"class": "icon icon-WCR_Search-Toggle-TextInput12"
		}).addClass("searchdropdownfieldswitchbutton");

		this._addon = $("<div>", {
			"class": "input-group-addon btn btn-default"
		}).attr("title", SearchInputs.t("SEARCH_PROJECT_STATUS_TOGGLE_FIELD")).append(this._switchbutton);
		this._addon.click(_.bind(function () {
			this._switchInputField(!this._stateInputField);
		}, this));

		// Append the select, input and switch to input-group div
		this.inputdiv.append(this._fields, this._addon);

		// Switch to smartname input, if enabled
		if (this.isSmartnamesEnabled) {
			this._setSmartNameMode();
		}

		// append the field to parameter field wrapper
		this.div.append(this.inputdiv);

	};

	/**
	 * Private method that should be called when the value in the field is
	 * modified... (or could be modified)
	 * 
	 * @param finalChange {boolean}
	 */
	FreeQueryDropdownField.prototype._updateValue = function (finalChange) {
		this._triggerChange(!finalChange);
	};

	/**
	 * Internal method used to switch between input field and dropdown
	 * 
	 * Most of the time only the first argument is used... (others are undefined)
	 * 
	 * @param toinputfield {Bool} Whether we need to show the dropdown or the input field
	 * @param withValueToInputField {String} some initial value in the input field. 
	 * 										ONLY USED during initialization!!! 
	 * 										(Should be undefined in other cases)
	 * @param dontUpdateToDropdown {Bool} most of the time undefined. 
	 * 									ONLY USED to switch to the dropdown RIGHT BEFORE conversion to SmartField. 
	 * 									This bool makes sure the value is not changed 
	 * 									(we still want to use the value from the inputfield in the SmartField). 
	 */
	FreeQueryDropdownField.prototype._switchInputField = function (toinputfield, withValueToInputField, dontUpdateToDropdown) {

		if (this.isSmartnamesEnabled) {
			// Should never happen...
			throw new Error("Illegal state! Cannot switch input for a SmartField.");
		} else {
			if (this._stateInputField === toinputfield) {
				// we are already in that state...
			} else {
				this._stateInputField = toinputfield;
				var opt;
				if (toinputfield) {
					this._inputfield.show();
					this._dropdown.hide();

					// Set the value of the input field...
					if (withValueToInputField !== undefined) {
						this._inputfield.val(withValueToInputField);
					} else {
						// Switching from dropdown to input field... 
						// Use the data-querydisplayvalue to know what to show
						opt = this._dropdown.find(":selected");

						if (opt.length === 1 && opt.attr("querydisplayvalue")) {
							this._inputfield.val(opt.attr("querydisplayvalue"));
						} else {
							this._inputfield.val("");
						}
					}

					// Value might have changed
					this._updateValue(true);

				} else {
					this._inputfield.hide();
					this._dropdown.show();

					// Switching from input field to dropdown...
					// Search for a value in the select which has the correct data-querydisplayvalue
					opt = $("option", this._dropdown).filter(_.bind(function (i, e) {
						var displayValue = $(e).attr("querydisplayvalue");
						return displayValue === this._inputfield.val();
					}, this));

					if (opt.length === 1) {
						this._dropdown.val(opt.attr("value"));
					}

					// Value might have changed
					if (!dontUpdateToDropdown) {
						this._updateValue(true);
					}
				}
			}
		}
	};

	/**
	 * Method which initializes the autocompletion in the "inputfield".
	 * 
	 * Very naive implementation!!
	 * Limitations:
	 *  - Only completes things at the end of the input field.
	 *  - Only works if you separate values with " OR "
	 * 
	 * So: does not complete values (or not correctly) if you use AND or special characters or || to separate values...
	 */
	FreeQueryDropdownField.prototype._initializeAutocomplete = function () {
		var availableItems = [];

		$(">option[querydisplayvalue]", this._dropdown).each(function (i, e) {
			availableItems.push($(e).attr("querydisplayvalue"));
		});


		// BASED ON CODE AT: https://jqueryui.com/autocomplete/#multiple
		// (MOSTLY COPIED)
		function split(val) {
			return val.split(/ OR /);
		}

		function extractLast(term) {
			return split(term).pop();
		}

		var self = this;
		this._inputfield
			// don't navigate away from the field on tab when
			// selecting an item
			.bind(
				"keydown",
				function (event) {
					if (event.keyCode === $.ui.keyCode.TAB && $(this).autocomplete("instance").menu.active) {
						event.preventDefault();
					}
				}).autocomplete({
				classes: {
					"ui-autocomplete": "freequerydropdown"
				},
				minLength: 0,
				autoFocus: true,
				delay: 0,
				source: function (request, response) {
					// delegate back to autocomplete, but
					// extract the last term
					response($.ui.autocomplete.filter(
						availableItems,
						extractLast(request.term)));
				},
				focus: function () {
					// prevent value inserted on focus
					return false;
				},
				select: function (event, ui) {
					var terms = split(this.value);
					// remove the current input
					terms.pop();
					// add the selected item
					terms.push(ui.item.value);
					// add placeholder to get the
					// comma-and-space at the end
					terms.push("");
					this.value = terms.join(" OR ");
					self._updateValue(false);

					// We just added a value, open search popup again for the next value
					_.defer(function () {
						self._inputfield.autocomplete("search", "");
					});
					return false;
				}
			});

		$("ul.ui-autocomplete.ui-menu").addClass("freequerydropdown");
	};


	/**
	 * Method which should be called when we go into SmartName mode
	 */
	FreeQueryDropdownField.prototype._setSmartNameMode = function () {

		// Switch value to correct encoding
		this._updateValue(false);

		// Update field
		if (this._stateInputField) {
			// We are in the input field mode...
			// Switch back to the "dropdown field" first
			this._switchInputField(false, undefined, true);
		}

		// Switch field to SmartField with the correct initial value
		// smartname encoded value
		this._dropdown.attr("data-smartnameval", this._value);
		this._dropdownsmartfield = new SmartField(this._dropdown);
		this._dropdownsmartfield.onValueChange(_.bind(this._onIntermediateUpdate, this));

		// hide the toggle button
		this._addon.hide();
	};


	//
	// Internal event handling
	//
	FreeQueryDropdownField.prototype._onIntermediateUpdate = function () {
		this._triggerChange(true);
	};

	FreeQueryDropdownField.prototype._onFinalUpdate = function () {
		this._triggerChange(false);
	};

	//
	// Set & Get the value
	//

	/**
	 * Get the value by filling the value object.
	 * Also do the validation in the mean time and
	 * use the feedback object to give the information back.
	 * 
	 * @param {ValueData} value - the object to put all the value information in
	 * @param {ValidationFeedback} feedback - the object 
	 */
	FreeQueryDropdownField.prototype.getValueData = function (value, feedback) {
		value.clear();

		// Update value (get the value from different places depending on the state we are in)
		if (this.isSmartnamesEnabled) {
			// We are in SmartName mode...
			// SmartField is loaded...
			this._value = this._dropdownsmartfield.getValue();
		} else {
			// We are not in Query mode...
			if (this._stateInputField) {
				this._value = this._inputfield.val();
			} else {
				this._value = this._dropdown.val();
			}
		}

		// Note, make sure there is NEVER a trailing " OR ", it will break searching.
		var fieldValue = this._value.replace(/ OR\s*$/, "");

		if (this.isSmartnamesEnabled) {
			value.setValue("valueSN", fieldValue);
			if (!this._dropdownsmartfield.isSmartFieldInputActive()) {
				//smartfield mode enabled, but the field is still using the dropdown selector
				value.setValue("isDropdown", "true");
			}
		} else {
			value.setValue("value", fieldValue);
			value.setValue("isDropdown", "true");
		}
	};

	/**
	 * Set the value, update the field to represent this value.
	 * 
	 * @param {ValueData} data - the object which contains all the information about the value
	 * @returns {Boolean} true if the field can represent the data which is set
	 */
	FreeQueryDropdownField.prototype.setValueData = function (data) {
		if (this.isSmartnamesEnabled) {
			// SmartName mode...
			var valueSN = data.getValue("valueSN");
			this._dropdownsmartfield.setValue(valueSN || "");
		} else {
			// Normal mode...
			var initialValue = data.getValue("value");

			// find selected option in dropdown
			var valueSelected = false;

			// select value based on value
			var option = $(">option", this._dropdown).filter(function (i, e) {
				return $(e).prop("value") === initialValue;
			});

			// Found a option in dropdown
			if (option.length > 0) {
				valueSelected = true;
			}

			// if a value was found in the dropdown... then keep the dropdown
			// otherwise, switch to the input field
			if (!valueSelected && initialValue !== "") {
				// Note: this also means that if you delete a project status, 
				// the search will switch to the input field. 
				// This input field will then show the id... (that id is no longer in the dropdown)
				this._switchInputField(true, initialValue);
			} else {
				this._dropdown.val(initialValue || "");
			}
		}

		return true;
	};

	/**
	 * Clear the Field (called by "Clear this form" functionality) 
	 */
	FreeQueryDropdownField.prototype.clear = function () {
		if (this.isSmartnamesEnabled) {
			// no need to implement, will not happen on Search Setup page.
		} else {
			this._inputfield.val("");
			this._dropdown.val("");
			this._stateInputField = false;

			this._triggerChange(false);
		}
	};

	/**
	 * Does this field contain a SmartName?
	 * 
	 * @returns {bool} true when this field contains a SmartName
	 */
	FreeQueryDropdownField.prototype.valueContainsSmartName = function () {
		return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this._dropdownsmartfield);
	};

	/**
	 * Does this field contain a special search character?
	 * 
	 * @param {char[]} specialCharacters - an array of special characters...
	 */
	FreeQueryDropdownField.prototype.valueContainsSpecialSearchCharacters = function (specialCharacters) {
		// Never check for special characters. We should never do that in SmartFields and dropdowns...
		// This ParameterField is always one of those two.
		return false;
	};

	//
	// Change listeners
	//

	/**
	 * Registers a new function to call when the field changes...
	 * 
	 * @param {Function} callback - a function to call when the field changes. 
	 * Function has one parameter, which should be true when the user is not finished with modifying the field.
	 */
	FreeQueryDropdownField.prototype.onChange = function (callback) {
		this.changeCallbacks.push(callback);
	};

	/**
	 * (private) Trigger a change event
	 */
	FreeQueryDropdownField.prototype._triggerChange = function (intermediatechange) {
		$.each(this.changeCallbacks, function (i, callback) {
			callback(intermediatechange);
		});
	};

	//
	// Destructor
	//

	/**
	 * Destroys this field...
	 * Remove listeners and clean up the div.
	 */
	FreeQueryDropdownField.prototype._destroy = function () {
		this.div.empty();
		this.changeCallbacks = [];
	};

	//
	// Factory...
	// Provides methods to create new parameter fields
	// & provides some general settings about the implementation
	//
	var factory = {
		/** 
		 * Make a new field of this type
		 * Returned object should implement certain functions
		 * @param {DOMElement} div - The dom element in which we should render the field
		 * @param {String} fieldName - The name of the field for which this field is created
		 * @param {SearchSettings} settings - Information about settings which are enabled
		 *     - E.g. are SmartNames enabled?
		 * @param {DataProvider} data - interface to have access to data
		 * @param {FieldProperties} fieldProperties - additional properties of the field
		 */
		make: function (div, fieldName, settings, dataProvider, fieldProperties) {
			return new FreeQueryDropdownField(div, fieldName, settings, dataProvider, fieldProperties);
		},
		/**
		 * Destroy a field created by this factory...
		 * @param obj - the object which was returned by this factory
		 */
		destroy: function (obj) {
			obj._destroy();
		}
	};
	return factory;
});