/**
 * Implements a Facet Graph field for use in dashboards
 */
define([
    "jquery",
    "underscore",
    "./facetutils/AbstractFacetParameterField",
    "utils/webi18n!SearchDashboard",
    "utils/Promisify"
], function (
    $,
    _,
    AbstractFacetParameterField,
    i18n,
    Promisify) {
    "use strict";

	//Global counter for how many graphs are allready created on the page.
	var globalGraphCounter;

    /**
     * @constructor
     * @augments AbstractFacetParameterField
     * @param {JQueryObj,HTMLObject} div - element in which the parameter field should be rendered
     * @param {String} fieldId - field ID of the parameterfield	
     * @param {SearchSettings} settings
     * @param {FieldProperties} fieldProperties
     */
    var GraphParameterField = function (div, fieldId, settings, fieldProperties) {
        AbstractFacetParameterField.call(this, div, fieldId, settings, fieldProperties);

		// Generate a unique id for this chart + keep track of it
		var uniquechartindex = globalGraphCounter || 0;
		globalGraphCounter = uniquechartindex + 1;
		this._globalGraphId = "graph-" + uniquechartindex;
    };

    GraphParameterField.prototype = Object.create(AbstractFacetParameterField.prototype);
    GraphParameterField.prototype.oncstructor = GraphParameterField;



    /**
     * (private) Initialize the graph
     */
    GraphParameterField.prototype._initialize = function () {

        this._chartElement = $("<div>");
        this.div.append(this._chartElement);

        // We store the id's in our own datastructure
        // map of numeric index used in this component => to actual selection value
        this._rememberedidsmap = {SERIEDATA:{}};
        // Keep the full names in here as the name we provide to c3 is cut off at a certain amount of chars
        this._rememberednamesmap =  {SERIEDATA:{}};
        
        // Some charttypes are handled a bit differently
        var charttype = this._fieldProperties.instanceConfig.getValue("chartType") || "pie";
        this._hasrotatedaxes = false;
        if(charttype === "hbar") {
        	this._hasrotatedaxes = true;
        	charttype = "bar";
        }
        this._dataAsSeries;
        if(charttype == "pie" || charttype == "donut") {
        	this._dataAsSeries = true;
        } else {
        	this._dataAsSeries = false;
        	this._seriename = this._fieldProperties.getFacetName(this._facetIDs[0]);
        	this._rememberedcolorsmap = {SERIEDATA:{}};
        }
		
		
		Promisify.require("c3WithStyling").then(function (c3) {
			var graphinitdata = {
	                bindto: this._chartElement.get(0),
	                data: {
	                    labels: {
	                    	format: function (v, id, i, j) {
	                    		// If the value is "0", do not show the label.
	                    		return v==0?"":v;
	                    	}
	                    },
	                    columns: [],
	                    type: charttype,
	                    empty: {
	                        label: {
	                            text: i18n.t("FACETFIELD_NOVALUES")
	                        }
	                    },
	                    onclick: this._chartOnClick.bind(this),
	                    color: _.bind(function (color, d) {
	                    	if(!this._dataAsSeries) {
	                    		// Note: pie and donut have their color set using the "normal" way.
	                    		
		                        // d will be 'id' when called for legends
	                    		if(d.id && d.id === 'SERIEDATA') {
	                    			// So, we need to provide the color
	                    			var customColor = this._rememberedcolorsmap.SERIEDATA[d.x];
	                    			if(customColor) {
	                    				// We have a custom color => return it
	                    				return d3.rgb(customColor);
	                    			}
	                    		}
	                    	}
	                    	return color;
	                    }, this)
	                },
	                pie: {
	                    label: {
	                        format: function (value, ratio, id) {
	                            return value;
	                        }
	                    }
	                },
	                donut: {
	                    label: {
	                        format: function (value, ratio, id) {
	                            return value;
	                        }
	                    }
	                },
	                bar: {
	                    width: {
	                    	ratio: 0.85
	                    }
	                },
	                axis: {
	                    y: {
	                    	tick: {
	                            format: function (x) {
	                            	// do not show labels on the y axis with commas
	                                return (x == Math.floor(x)) ? x : "";
	                            }
	                        }
	                    }
	                },
	                onrendered: function () {
	                    if (!this._dataAsSeries) {
	                        // Hide the ticks on y-axis which do not have a label (see format above)
	                        var yTicks = $(this.div).find(".c3-axis-y .tick");
	                        _.each(yTicks, function (tick, index) {
	                            if ($(tick).text() === "") {
	                                $(tick).hide();
	                            }
	                        });
	                    }
	                    
	                    // Generate the stripe pattern and add this to the graph...
                    	// Note: make sure to do this only once!
                        if(!this._defsgenerated) {
                        	this._defsgenerated = true;
                        	var defs = d3.select(this._chartElement[0]).select("svg defs");

                        	// The pattern is {width} x {width} pixels (keeps repeating) where {fill} pixels are fully filled and then we have a {width}-{fill} pixel transparent line.  
	                        var patternwidth = 4;
	                        var patternfill = 3;
	                        // After that we rotate the pattern by 45%
	                        var patternrotation = 45;
	                        // What does the pattern look like?
	                        var pattern = defs.append("pattern").attr("id", this._globalGraphId+"-pattern-stripe").attr("width", patternwidth).attr("height", patternwidth).attr("patternUnits", "userSpaceOnUse").attr("patternTransform", "rotate("+patternrotation+")");
	                        // Well, we start with everything white (=visible)
	                        pattern.append("rect").attr("width", patternwidth).attr("height", patternwidth).attr("transform", "translate(0,0)").attr("fill", "white");
	                        // Then, we draw a horizontal grey line (=half visible)
	                        pattern.append("rect").attr("width", patternwidth-patternfill).attr("height", patternwidth).attr("transform", "translate(0,0)").attr("fill", "#666");
	                        // Note that the pattern is rotated 45° so the lines are not really horizontal/vertical...
	                        
	                        // Then we make a svg mask which is simply a big rectangle where we keep repeating this pattern.
	                        // Note that the width is 200%.
	                        // Why? For pie charts, (0,0) is at the center, so if we would only draw from (0,0) in positive x and y, we would only show the bottom right part of the pie with stripes.
	                        // For bar charts, positive x and y seem to be enough though.
	                        defs.append("mask").attr("id", this._globalGraphId+"-mask-stripe")
	                            .append("rect").attr("x", "-100%").attr("y", "-100%").attr("width", "200%").attr("height", "200%").attr("fill", "url(#"+this._globalGraphId+"-pattern-stripe)");
                        }
                        
                        // Stripe the selection 
                        _.each(_.keys(this._rememberedidsmap.SERIEDATA), function(key) {
                        	var id = this._rememberedidsmap.SERIEDATA[key];
                        	// So, we need to find the bar/pie piece where we need to apply this mask.
                        	
                        	// For bar, this is easy, we can look it up by the name of the series and then we simply need the bar with a certain index.
                        	var bar = $(".c3-chart-bars .c3-target-SERIEDATA .c3-bar-"+key, this._chartElement);
                        	// For pie, we can lookup by the index, because we put in the index as the id (in the render method)
                        	var pie = $(".c3-chart-arcs .c3-target-"+key+" .c3-arc", this._chartElement);
                        	
                        	// Is this value selected?
							var selected = null;
							if(this._facetSelection){
								 selected = this._facetSelection.isSelectedSingleValue(id);
							}
                        	if(bar.length>0) {
                        		bar.css({"mask":selected?"url(#"+this._globalGraphId+"-mask-stripe)":""});
                        	}
                        	if(pie.length>0) {
                        		pie.css({"mask":selected?"url(#"+this._globalGraphId+"-mask-stripe)":""});
                        	}
                        }, this);
                        
                        // Add a tooltip to the legend in a pie/donut chart
                        var legendtooltipmsg = i18n.t("SEARCH_GRAPH_LEGEND_TOOLTIP");
                        if(legendtooltipmsg && legendtooltipmsg.trim()) {
	                        // So, we need to find the legend... But it does not have a class. So, instead take the parent of the first legend "item" we find.
	                        var legenditem = d3.select(this._chartElement[0]).select("svg").select(".c3-legend-item").node();
	                        if(legenditem !== null) {
	                        	var legend = legenditem.parentNode;
	                        	// Check if the tooltip is already there
		                        var existingtitle = d3.select(legend).select("title").node();
			                    if(existingtitle === null) {
			                    	// If not, add it.
			                    	d3.select(legend).append("title").text(legendtooltipmsg);
			                    }
		                    }
                        }
	                }.bind(this),
	                tooltip: {
	                	format: {
	                		title: function(d) {
	                			if(this._dataAsSeries) {
	                				// Pie chart -> this method is not called (there is no title)
	                				return d;
	                			} else {
	                				// Bar chart -> The title contains the value of the x-axis
	                				return this._rememberednamesmap.SERIEDATA[d];
	                			}
	                		}.bind(this),
	                		name: function(name, ratio, id, index) {
	                			if(this._dataAsSeries) {
	                				// Pie chart -> can only have one series...
	                				// So, the name = the label!
	                				return this._rememberednamesmap.SERIEDATA[id];
	                			} else {
	                				// Bar chart -> just return "name" (which is the name of the series)
	                				return name;
	                			}
	                		}.bind(this),
	                		value: function(value, ratio, id) {
	                			if(this._dataAsSeries) {
	                				return d3.format('%')(ratio);
	                			} else {
	                				return value;
	                			}
	                		}.bind(this)
	                	}
	                }
	            };
			
			if(!this._dataAsSeries) {
				graphinitdata.data.x = 'x';
				graphinitdata.axis.x = {type:'category', tick:{rotate: 90, multiline: false, 
					format: _.bind(function(x) {
						var label = this._rememberednamesmap.SERIEDATA[x];
						return this._getCutofflabel(label);
					}, this)
				}};
				graphinitdata.legend = {show:false};
				graphinitdata.data.names = {
					SERIEDATA: this._seriename
				};
				graphinitdata.data.columns = [["SERIEDATA"]];
				graphinitdata.data.types = {SERIEDATA:graphinitdata.data.type};
				delete graphinitdata.data.type;
				
				if(this._hasrotatedaxes) {
					graphinitdata.axis.rotated = true;
				}
			}
			
            this._chart = c3.generate(graphinitdata);
			this._chart.element.classList.add(this._globalGraphId);
            
			this.resize();
      

            this.render();
        }.bind(this));

    };
	
	/**
	 * [[Description]]
	 * @private
	 * @param {object}   d       [[Description]]
	 * @param {[[Type]]} element [[Description]]
	 */
	GraphParameterField.prototype._chartOnClick = function (d, element) {
		var id = null;
		if(this._dataAsSeries) {
			id = this._rememberedidsmap.SERIEDATA[d.id];
		} else {
			id = this._rememberedidsmap.SERIEDATA[d.x];
		}
		this.toggleSelectionForValue(id);
	};
	
	/**
	 * Util function - get a label that is at most 21 characters long.
	 * 
	 * @param label
	 * @returns the shorter label
	 */
	GraphParameterField.prototype._getCutofflabel = function (label) {
		// Shorten labels if too long
        var cutofflabel = ""+label;
        var CUTOFFATCHARS = 20;
        if(cutofflabel.length>CUTOFFATCHARS) {
        	cutofflabel = cutofflabel.substring(0, CUTOFFATCHARS);
        	cutofflabel+="…";
        }
        return cutofflabel;
	};

    /**
     * Update the Graph
     * @private
     */
    GraphParameterField.prototype._doRender = function () {
        if (this._chart) {
            var chartData = {};
            
            // The previous ids in the graph...
        	var oldIds = this._rememberedidsmap.SERIEDATA;
        	var oldKeyLookup = _.invert(this._rememberedidsmap.SERIEDATA);
        	
        	// Reset the list of ids
        	this._rememberedidsmap.SERIEDATA = {};
        	// Reset the list of non-cut-off labels
        	this._rememberednamesmap.SERIEDATA = {};
            
            var names, column;
            if(this._dataAsSeries) {
            	chartData.columns = [];
            	chartData.colors = {};
            	chartData.names = {};
            	chartData.unload = _.pluck(this._chart.data(), "id");
            } else {
            	names = ["x"];
            	column = ["SERIEDATA"];
            	this._rememberedcolorsmap.SERIEDATA = {};
            }
            
            var nextidx = 0;
            var entryIdx = 0;
            _.each(this._facetData, function (facetEntry) {
                var id = facetEntry.getValue("id");
                var facetChunk = this.getFacetValueData(this._facetIDs[0], id);
                var count = facetChunk.getValue("count", "0");
                var label = this.getLabelForFacet(facetChunk);
                var color = facetChunk.getValue("color_rgb");
                
                // So we need numeric id's for all values 
                // (because they will be used in html id's -> see striping pattern for selection)
                var idx = null;
                if(this._dataAsSeries) {
                	// For PIE/DONUT this is tricky !!
                    // For a value that was in the graph we need to keep the same id.
                	// For other values, just create a new index.
                	if(_.has(oldKeyLookup, id)) {
                    	// So, this value was also in the previous graph
                    	idx = oldKeyLookup[id];
                    } else {
                    	// This value was not in the previous graph
                    	// Check if "nextidx" index is available... 
                    	while(_.has(oldIds, ""+nextidx)) {
                    		// This index might be used by a previous value
                    		// (We need to keep the same indexes for those values)
                    		nextidx++;
                    	}
                    	// a free index...
                    	idx = nextidx;
                    	nextidx++;
                    }
                } else {
                	// For bar we can simply start counting from 0 again...
                	// (There is no way of telling c3 that a certain bar has moved)
                	idx = entryIdx;
                	entryIdx++;
                }
                
                // Store the id in the map
                // map of numeric index used in this component => to actual selection value
                this._rememberedidsmap.SERIEDATA[idx] = id;
                
                // Store the full label so we can show it in the tooltip
                this._rememberednamesmap.SERIEDATA[idx] = label;
                
                if(this._dataAsSeries) {
                	// For pie and such
	                chartData.columns.push([idx, count]);
	                if (color) {
	                    chartData.colors[idx] = color;
	                }
	                chartData.names[idx] = this._getCutofflabel(label);
	
	                chartData.unload = _.without(chartData.unload, idx);
                } else {
                	// For bar and such
                	this._rememberedcolorsmap.SERIEDATA[idx] = color;
                	names.push(label);
                	column.push(count);
                }
            }, this);

            if(!this._dataAsSeries) {
            	if(this._facetData.length === 0) {
            		// No results
            		chartData.unload = ["SERIEDATA"];
            	} else {
            		// Otherwise, add the data...
            		chartData.columns = [names, column];
            	}
            }
            
            this._chart.load(chartData);
			
			this._checkForNumberOfValuesExceeded();
			
			//Allow clicking of the background for selection
			d3.selectAll('.'+this._globalGraphId).selectAll('.c3-event-rect').on("click", function(data, index){
				if(data.value > 0){ //Only do selection if the number of results for this value is not zero
					this._chartOnClick(data, index);					
				}
			}.bind(this));
			
        }
		
    };
	
	/**
	 * Resize the chart
	 */
	GraphParameterField.prototype.resize = function(){
		// IE has timing issues with c3 on load... because of that we need the defer...
		this._resize();
		_.defer(this._resize.bind(this));
	};
	
	/**
	 * Implementation of resizing behavior
	 * @private
	 */
	GraphParameterField.prototype._resize = function(){
		if (this._chart){
			 // Get the user configured Graph width and Height value
			var graphWidth = this._fieldProperties.instanceConfig.getValue("graphWidth");
			var graphHeight = this._fieldProperties.instanceConfig.getValue("graphHeight");
			  // Resize the graph only if user has defined Width & Height
			if(graphWidth && graphHeight) {

				//convert as a pixel value
				var newHeight = parseInt(graphHeight, 10);
				var newWidth  =  parseInt(graphWidth, 10);

				this._chart.resize({ 
					height: newHeight, 
					width: newWidth
					});
			} else {
				this._chart.resize();
			}
		}
	};

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function (div, fieldName, settings, dataProvider, fieldProperties) {
            return new GraphParameterField(div, fieldName, settings, fieldProperties);
        },
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function (obj) {
            obj._destroy();
        },
        /**
         * The constructor of this ParameterField... 
         * Used for extending this Parameterfield.
         */
        constructor: GraphParameterField
    };
    return factory;
});
