/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore",
        "./popups/InkSearchPopup",
        "utils/webi18n!SearchInputs"
        ], function(
        		$, _,
        		InkSearchPopup,
        		SearchInputs) {
	
	"use strict";

    /**
     * @constructor
     */
    var InkParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    InkParameterField.prototype._initialize = function(){
    	
    	/*
    	 * Object to store the ink information
    	 * {
    	 * 		ink_search_mode: "CMYKONLY",
    	 * 		number_inks: "3",
    	 * 		number_inks_operator: "LESSTHAN",
    	 * 		number_printing_inks: "5",
    	 * 		number_printing_inks_operator: "EQUALS",
    	 * 		ink_rows: [
    	 * 					{ink_name: "Green", ink_group: "Any", ink_type: "Regular"},
    	 * 					{ink_name: "Red", ink_group: "Designer", ink_type: "Varnish"}
    	 * 				]
    	 * }
    	 */
    	this._inksData = {};
    	
    	// Make a div to display ink...
    	this.input = $("<div>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    	});
    	
    	/*
    	 * Method to execute, when a ink(s) is selected
    	 */
    	this._updateInputText = _.bind(function() {
    		this.input.empty();
    		
    		if(!$.isEmptyObject(this._inksData)) {
    			var mode = this._inksData.ink_search_mode;
    			var isUpdateAddIconTitle = false;
    			
    			// If CMYKonly mode is selected
    			if(mode && mode == "CMYKONLY") {
    				this.input.text(SearchInputs.t("GRAPHICS_SEARCH_CMYKONLY"));
    				isUpdateAddIconTitle = true;
    				
    			} 
    			// If Custom mode is selected
    			else {
    				var getOperatorSymbol = function(operatorStr) {
    					if(operatorStr == "EQUALS") {
    						return "=";
    					} else if(operatorStr == "LESSTHAN") {
    						return "≤";
    					} else if(operatorStr == "GREATERTHAN") {
    						return "≥";
    					}
    				};
    				
    				var inktable = $("<table>");
    				
    				// Add number of inks information
    				if(this._inksData.number_inks && this._inksData.number_inks_operator) {
    					var numberink = getOperatorSymbol(this._inksData.number_inks_operator) + " " + this._inksData.number_inks;
    					inktable.append($("<tr>").append($("<td>", {"class": "add-padding-right"}).text(SearchInputs.t("GRAPHICS_SEARCH_NRINKS")), $("<td>", {"class": "add-padding-left"}).text(numberink)));
    					isUpdateAddIconTitle = true;
    				}
    				
    				// Add number of printing inks information
    				if(this._inksData.number_printing_inks && this._inksData.number_printing_inks_operator) {
    					var numberprintink = getOperatorSymbol(this._inksData.number_printing_inks_operator) + " " + this._inksData.number_printing_inks;
    					inktable.append($("<tr>").append($("<td>", {"class": "add-padding-right"}).text(SearchInputs.t("GRAPHICS_SEARCH_NRPRINTINGINKS")), $("<td>", {"class": "add-padding-left"}).text(numberprintink)));
    					isUpdateAddIconTitle = true;
    				}
    				
    				// Add ink information
    				if(this._inksData.ink_rows) {
    					for(var i=0; i<this._inksData.ink_rows.length; i++) {
    						var inkrow = this._inksData.ink_rows[i];
							
    						// ink information to be displayed
    						var inkrowtext = inkrow.ink_name || "";
    						
							if(inkrow.ink_group != "Any") {
								inkrowtext = inkrowtext ? (inkrowtext + " - " + inkrow.ink_group) : inkrow.ink_group;
							}
							if(inkrow.ink_type != "Any") {
								inkrowtext = inkrowtext ? (inkrowtext + " - " + inkrow.ink_type) : inkrow.ink_type;
							}
							
							var rowtext = (i === 0) ? (SearchInputs.t("GRAPHICS_SEARCH_LISTINKS") + ":") : "";
							
							inktable.append($("<tr>").append($("<td>", {"class": "add-padding-right"}).text(rowtext), $("<td>", {"class": "add-padding-left"}).text(inkrowtext)));
    					}
    					isUpdateAddIconTitle = true;
    				}
    				
    				this.input.append(inktable);
    			}
    			
    			if(isUpdateAddIconTitle) {
    				$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("EDIT")});
    			} else {
    				$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("ADD")});
    			}
    			
    		}
    		
    	}, this);
    	
    	// create add button...
    	this._addButton = $("<a>", {"class": "btn btn-no-border"}).append($("<i>", {"name": "addorediticon", "class": "icon icon-WCR_Actions-Edit12", "title": SearchInputs.t("ADD")}));
    	
    	this._addButton.click(_.bind(function() {
    		if(!this._inksPopup) {
    			// Create a ink search popup
    			this._inksPopup = new InkSearchPopup();
    			this._inksPopup.onSubmit(_.bind(function(inksData) {
    				this._inksData = inksData;
    				this._updateInputText();
    				this._triggerChange(true);
    			}, this));
    			this._inksPopup.open(this._inksData);
    		} else {
    			this._inksPopup.open(this._inksData);
    		}
    	}, this));

		// add the button
    	this.div.append($("<table>").append($("<tr>").append($("<td>", {"class": "add-padding-right"}).append(this._addButton), $("<td>").append(this.input))));
    };
    
    //
    // Internal event handling
    //
    InkParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    InkParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    InkParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	
    	// Set the ink search mode in value data
    	if(this._inksData.ink_search_mode) {
    		value.setValue("ink_search_mode", this._inksData.ink_search_mode);
    	}
    	
    	// If custom option is selected
    	if(this._inksData.ink_search_mode == "CUSTOM") {
    		
    		// Add number of inks information to valuedata
    		if(this._inksData.number_inks && this._inksData.number_inks_operator) {
    			// Create a int_search_criterion valuedata object
    			var inkNumberData = value.addNewItemToList("int_search_criterion");
    			inkNumberData.setValue("name", "number_inks");
    			
    			if(this._inksData.number_inks_operator == "EQUALS") {
    				inkNumberData.setValue("mode", "EQUALS");
    				inkNumberData.setValue("value", this._inksData.number_inks);
    				
    			} else if(this._inksData.number_inks_operator == "GREATERTHAN") {
    				inkNumberData.setValue("mode", "RANGE");
    				inkNumberData.setValue("gt_value", this._inksData.number_inks);
    				
    			} else if(this._inksData.number_inks_operator == "LESSTHAN") {
    				inkNumberData.setValue("mode", "RANGE");
    				inkNumberData.setValue("lt_value", this._inksData.number_inks);
    			}
        	}
    		
    		// Add number of printing inks information to valuedata
    		if(this._inksData.number_printing_inks && this._inksData.number_printing_inks_operator) {
    			// Create a int_search_criterion valuedata object
        		var printInkNumberData = value.addNewItemToList("int_search_criterion");
        		printInkNumberData.setValue("name", "number_printing_inks");
    			
    			if(this._inksData.number_printing_inks_operator == "EQUALS") {
    				printInkNumberData.setValue("mode", "EQUALS");
    				printInkNumberData.setValue("value", this._inksData.number_printing_inks);
    				
    			} else if(this._inksData.number_printing_inks_operator == "GREATERTHAN") {
    				printInkNumberData.setValue("mode", "RANGE");
    				printInkNumberData.setValue("gt_value", this._inksData.number_printing_inks);
    				
    			} else if(this._inksData.number_printing_inks_operator == "LESSTHAN") {
    				printInkNumberData.setValue("mode", "RANGE");
    				printInkNumberData.setValue("lt_value", this._inksData.number_printing_inks);
    			}
        	}
        	
    		// Add ink information as text_search_criterion valuedata object
    		// The value will be of format : ink1__*__* or ink1__Pantone__Opaque [* represents any group or any type]
        	if(this._inksData.ink_rows && this._inksData.ink_rows.length > 0) {
        		for(var i=0; i<this._inksData.ink_rows.length; i++) {
        			var inkRow = this._inksData.ink_rows[i];
        			
        			if(inkRow) {
        				var inkRowData = value.addNewItemToList("text_search_criterion");
        				var inkvalue = "";
        				
        				inkvalue = inkvalue + inkRow.ink_name;
        				inkvalue = (inkRow.ink_group == "Any") ? inkvalue + "__*" : inkvalue + "__" + inkRow.ink_group;
        				inkvalue = (inkRow.ink_type == "Any") ? inkvalue + "__*" : inkvalue + "__" + inkRow.ink_type;
        				
        				inkRowData.setValue("key", "ink");
        				inkRowData.setValue("value", inkvalue);
        			}
        		}
        	}
    	}
    	
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    InkParameterField.prototype.setValueData = function (data) {
    	this._inksData = {};
    	
    	// Get the ink search mode
    	if(data.getValue("ink_search_mode")) {
    		this._inksData.ink_search_mode = data.getValue("ink_search_mode");
    	}
    	
    	// If search mode is custom
    	if(this._inksData.ink_search_mode == "CUSTOM") {
    		var numberInkList = data.getSubList("int_search_criterion");
    		
    		if(numberInkList && numberInkList.length > 0) {
    			
    			// Iterate for each int_search_criterion valuedata object
    			for(var i=0; i<numberInkList.length; i++) {
    				var numberData = numberInkList[i];
    				
    				// Get the number of inks information from valuedata object
    				if(numberData.getValue("name") == "number_inks") {
    					if(numberData.getValue("mode").toUpperCase() == "EQUALS") {
    						this._inksData.number_inks = numberData.getValue("value");
    						this._inksData.number_inks_operator = "EQUALS";
    					} else if(numberData.getValue("mode").toUpperCase() == "RANGE" && numberData.getValue("gt_value")) {
    						this._inksData.number_inks = numberData.getValue("gt_value");
    						this._inksData.number_inks_operator = "GREATERTHAN";
    					} else if(numberData.getValue("mode").toUpperCase() == "RANGE" && numberData.getValue("lt_value")) {
    						this._inksData.number_inks = numberData.getValue("lt_value");
    						this._inksData.number_inks_operator = "LESSTHAN";
    					}
    				} 
    				
    				// Get the number of printing inks information from the valuedata object
    				if(numberData.getValue("name") == "number_printing_inks") {
    					if(numberData.getValue("mode").toUpperCase() == "EQUALS") {
    						this._inksData.number_printing_inks = numberData.getValue("value");
    						this._inksData.number_printing_inks_operator = "EQUALS";
    					} else if(numberData.getValue("mode").toUpperCase() == "RANGE" && numberData.getValue("gt_value")) {
    						this._inksData.number_printing_inks = numberData.getValue("gt_value");
    						this._inksData.number_printing_inks_operator = "GREATERTHAN";
    					} else if(numberData.getValue("mode").toUpperCase() == "RANGE" && numberData.getValue("lt_value")) {
    						this._inksData.number_printing_inks = numberData.getValue("lt_value");
    						this._inksData.number_printing_inks_operator = "LESSTHAN";
    					}
    				}
    				
    			}
    		}
    		
    		// Get the ink information from text_search_criterion valuedata object
    		var inkRowList = data.getSubList("text_search_criterion");
    		if(inkRowList && inkRowList.length > 0) {
    			this._inksData.ink_rows = [];
    			for(var index=0; index<inkRowList.length; index++) {
    				var inkRowData = inkRowList[index];
    				
    				if(inkRowData) {
    					var inkName = inkRowData.getValue("inkName");
    					var inkGroup = inkRowData.getValue("inkGroup");
    					var inkType = inkRowData.getValue("inkType");
    					
    					this._inksData.ink_rows.push({"ink_name": inkName,"ink_group": inkGroup, "ink_type": inkType});
    				}
    			}
    		}
    	}
    	
    	// Show the ink information on the search setup page
    	this._updateInputText();
    	
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    InkParameterField.prototype.clear = function() {
    	this.input.empty();
    	$("i[name='addorediticon']", this._addButton).attr({"title": SearchInputs.t("ADD")});
    	this._triggerChange(false);
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    InkParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    InkParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    InkParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    InkParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    InkParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new InkParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});