/**
 * Implements a Float Field for the Search Setup and Design Panel.
 */
define(["lib/oop", "jquery", "underscore", "../../Utils", "./ParameterField_float_percent"], function(oop, $, _, utils, FloatPercentParameterField_factory) {

    /**
     * @constructor
     */
    var IntegerPercentParameterField = function(div, fieldId, settings, dataProvider, unitKey) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	// This field does not have SmartNames enabled at the moment.
    	this.smartNamesEnabled = false;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    // inherit functions from FloatPercentParameterField
    IntegerPercentParameterField.prototype = Object.create(FloatPercentParameterField_factory.constructor.prototype);
    IntegerPercentParameterField.prototype.constructor = IntegerPercentParameterField;
    
    //
    //VALIDATION functions
    //
	
	/**
	 * Use validatePartialNumericInput numbers that may not yet be fully typed
	 * 
	 * @param element - JQuery object; HTML element that is being validated
	 * @param allowNegative - boolean; do we allow negative numbers in this field?
	 * 
	 * @returns {Boolean}
	 */
	IntegerPercentParameterField.prototype.validatePartialNumericInput = function (element, allowNegative)
	{
	   var value = element.val();
	   
	   // Blank field is allowed
	   if(value == "") {
		   return true;   
	   }
	   
	   // The user has only entered a negative sign so far
	   if(allowNegative)
	   {
	      if(value == "-")
	      return true;
	   }
	   
	   //decimal point is not allowed for an Integer input
	   if(value == ".") {
		   alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_INTEGER_NUMBER_FEEDBACK"));
		   return false;   
	   }
	   
	   //make sure the value is valid Integer number
	   var checkNum = parseInt(value);
	   if(isNaN(checkNum))
	   {
	      // They did not enter a number...
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_INTEGER_NUMBER_FEEDBACK"));
	      return false;
	   }
	   
	   if(checkNum != value)
	   {
	      // They entered something with a number at the front - e.g. 12Y >>>
		   //revert to the numeric part only
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_INTEGER_NUMBER_FEEDBACK"));
	      element.val(checkNum);
	      return false;
	   }
	   
	   //make sure a non-negative is not there at the moment (if not allowed)
	   if(!allowNegative && checkNum < 0)
	   {
	      alert(utils.getLocalizedLabel("SEARCHCRITERION_INVALID_NEGATIVE_NUMBER_FEEDBACK"));
	      return false;
	   }
	   
	   return true;
	};

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new IntegerPercentParameterField(div, fieldName, settings, dataProvider, fieldProperties.displayUnit);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});