/**
 * Implements a simple Text Field for the Design Panel.
 */
define(["jquery",
        "underscore",
        "../../Utils",
        "packaging-content-management/TextContentSearchStatementFields"], 
function($,
		_,
		utils,
		TextContentSearchStatementFields) {

    /**
     * @constructor
     */
    var OperatorParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    OperatorParameterField.prototype._initialize = function(){
		
		this._operatorButton = $('<button>').attr({
			"data-sahi-id":this.fieldId,// For automated testing
			'type': 'button'
		}).addClass('btn btn-primary btn-sm attachleft').text('OR');
		
		this._operatorButton.on('click', _.bind(function() {
			if ('AND' === this._operatorButton.text()) {
				this._operatorButton.text('OR');
			} else {
				this._operatorButton.text('AND');
			}
			this._onFinalUpdate();
		}, this));
		
		this.input = this._operatorButton;
    	
		// add the field
    	this.div.append(this.input);
    };
    
    //
    // Internal event handling
    //
    OperatorParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    OperatorParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    OperatorParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	value.setValue("operator", this._operatorButton.text());
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    OperatorParameterField.prototype.setValueData = function (data) {
    	var operator = data.getValue("operator");
		this._operatorButton.text(operator || "OR");
    	
		return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    OperatorParameterField.prototype.clear = function() {
    	this._operatorButton.text("OR");
    	this._triggerChange(false);
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    OperatorParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    OperatorParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
		return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    OperatorParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    OperatorParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    OperatorParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new OperatorParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});