/**
 * Implements User or Me Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils",
        "utils/webi18n!SearchInputs",
        
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding"
        ], function(
        		$, _, utils,
        		i18n,
        		SmartField,
        		SmartNameEncoding) {

    /**
     * @constructor
     */
    var RevisionParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    RevisionParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	// Create the Input field for Revision field
    	this._constructRevisionField();
    	
    	if(!this.settings.isSmartNamesEnabled()) {
    		// We have a normal input
    		
	    	// listen to certain events on the field...
    		
	    	//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
	    	this.input.on("keyup", function(event){
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    	});
	    	
	    	//blur=get out of the field >>> trigger final update event
	    	this.input.on("blur", _.bind(this._onFinalUpdate, this));
	    	
    	} else {
    		// We have a SmartField input
    		
    		// Convert the input into a SmartField
    		this.smartField = new SmartField(this.input);
    		
    		this.smartField.disable();
    		
    		// listen to certain events on the field...
    		this.smartField.onValueChange(_.bind(this._onIntermediateUpdate, this));
    	}
    	
    	// On change event when user select revision option value
    	this.select.on("change", _.bind(function () {
    		this._onChangeRevisionOption(this.select);
    	}, this));
    	
    };
    
    /**
     * Make the Revision Field Parameter input
     * 
     * This method will create fields [this.input] , [this.select] - This fields are public and it will accessible all methods
     */
    RevisionParameterField.prototype._constructRevisionField = function() {
    	
    	var table = $("<table>").addClass("parameterFieldTable");
    	var row = $("<tr>");
    	var colRevSelect = $("<td>");
    	
    	var colRevLetterLabel = $("<td>").addClass("parameterFieldRevisionLabel");
    	colRevLetterLabel.text(i18n.t('REVISION_NUMBER') + ":");
    	
    	var colRevInputs = $("<td>").addClass("widecell");
    	
    	row.append(colRevSelect, colRevLetterLabel, colRevInputs);
    	table.append(row);
    	
    	var selectDiv = $("<div>");
    	colRevSelect.append(selectDiv);
    	
    	this.select = $("<select>", {
    		"data-sahi-id":this.fieldId+"_option",// For automated testing
			"class" : "form-control attachleft search-input-md",
		});
    	
    	//Load the revision options to the revision dropdown
    	this._loadRevisionOptions(this.select);
    	
    	selectDiv.append(this.select);
    	
    	this.input = $("<input>", {
    		"data-sahi-id":this.fieldId,// For automated testing
			"type" : "text",
			"class" : "form-control search-input-sm",
			"data-smartfieldwidth" : "70px"
		});
    	this.input.css("width", "70px");
    	
    	this.input.attr('disabled', 'disabled');
    	
    	colRevInputs.append(this.input);
    	
    	this.div.append(table);
    };
    
    /**
	 * This method used load Revision options 
	 * @param {JQuery} select - Option "Revision" dropdown
	 */
    RevisionParameterField.prototype._loadRevisionOptions = function(select) {
		
    	var latestRevisionOption = $("<option>").val("0");
    	latestRevisionOption.text(i18n.t('DOCSEARCH_LATEST_REVISION'));
		select.append(latestRevisionOption);
		
		var allRevisionsOption = $("<option>").val("1");
		allRevisionsOption.text(i18n.t('DOCSEARCH_ALL_REVISIONS'));
		select.append(allRevisionsOption);
		
		var baseRevisionOption = $("<option>").val("2");
		baseRevisionOption.text(i18n.t('DOCSEARCH_BASE_REVISION'));
		select.append(baseRevisionOption);
		
		var specificRevisionOption = $("<option>").val("3");
		specificRevisionOption.text(i18n.t('DOCSEARCH_SPECIFIC_REVISION'));
		select.append(specificRevisionOption);

	};
    
    /** 
    * Used on the Document Search Page and Search Results Page.
    * Enables or Disables (and clears) the Revision Number drop down depending on he value of revisionOptionValue
    * 
    * @param {JQuery} element - The Revision Option dropdown
    */
    RevisionParameterField.prototype._onChangeRevisionOption = function(element) {

    	var revisionOptionValue = element.val();
    	
    	if (revisionOptionValue != null) {
    		
    		if(!this.settings.isSmartNamesEnabled()) {
    			if (revisionOptionValue == '3') {
    				// Specific Revision selected
    				this.input.removeAttr('disabled');
    			} else {
    				this.input.val('');
    				this.input.attr('disabled', 'disabled');
    			}
    		} 
				
    		if(this.settings.isSmartNamesEnabled()) {
    			if (revisionOptionValue == '3') {
    				// Specific Revision selected
    				this.smartField.enable();
    			} else {
    				this.smartField.clear();
    				this.smartField.disable();
    			}
    		} 
		}
    	// call the update method to notify the changes
    	this._onFinalUpdate();
    };
    
    //
    // Internal event handling
    //
    RevisionParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    RevisionParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    RevisionParameterField.prototype.getValueData = function (value, feedback) {
    	
            value.clear();
			value.setValue("revision_option",this.select.val());
			
			if(this.settings.isSmartNamesEnabled()) {
	    		value.setValue("revision_letterSN", this.smartField.getValue());
	    	}else{
	    		value.setValue("revision_letter", this.input.val());
	    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    RevisionParameterField.prototype.setValueData = function (data) {

    	var revision_option = data.getValue("revision_option");
    	
    	if (revision_option != null) {
    		
    		this.select.val(revision_option);
        	
        	if (revision_option == '3') {
        		if(this.settings.isSmartNamesEnabled()) {
        			this.smartField.enable();
            		// SmartName mode...
            		var revision_letterSN = data.getValue("revision_letterSN");
            		this.smartField.setValue(revision_letterSN || "");
            	} else {
            		// Normal mode...
            		this.input.removeAttr('disabled');
            		var revision_letter = data.getValue("revision_letter");
            		this.input.val(revision_letter || "");
            		
            	}
        	} else {
        		if(this.settings.isSmartNamesEnabled()) {
        			this.smartField.disable();
        		} else{
        			this.input.attr('disabled', 'disabled');
        		}
        	}
    	}
    	return true;
    };
    
    /**
     * Ensures the revision number isn't blank  
     * 
     * @returns {Boolean}
     */
    RevisionParameterField.prototype.validate = function() {
    	
    	// Revision Number can't be blank on Doc search screen or results screen, if it's enabled.
    	// i.e. 'Specific Revision' has been selected from the revision options drop down 
    	// It can be :-
    	// (1) 1-4 characters long, but only the last character in the string can be in the range a-z A-Z
    	//     Any preceding characters have to be equal to z (or Z)
    	// or ...
    	// (2) A Smart Name
    	// 
    	
    	if (this.input) {
    		
    		if (!this.smartField) {
    			// Will only execute this code if we're on the Document design, results or My Work screens.
    			if (!this.input.prop('disabled')) {
    				if (this.input.val() === '') {
    					alert(i18n.t('REVISION_LETTER_CANNOT_EMPTY')); 
    					return false;				
    				}
    			}
    		} else {
    			if(!this.smartField.isDisabled()) {
    				if(this.smartField.getValue() === '') {
    					alert(i18n.t('REVISION_LETTER_CANNOT_EMPTY')); 
    					return false;
    				}
    			}
    		}
    	}
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    RevisionParameterField.prototype.clear = function() {
    	if(this.settings.isSmartNamesEnabled()) {
    		// no need to implement, will not happen on Search Setup page.
    	} else{
    		$(this.select).val("0");
    		this.input.prop("disabled", true);
	    	this.input.val("");
	    	this._triggerChange(false);
    	}
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {boolean} true when this field contains a SmartName
     */
    RevisionParameterField.prototype.valueContainsSmartName = function() {
    	return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this.smartField);
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    RevisionParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	if(this.settings.isSmartNamesEnabled()) {
    		// Don't check for special characters in a SmartField...
    		// The hint would otherwise also show up if you use the autocomplete to type a SmartName...
    		// (For the autocomplete you start typing a SmartName with "[" , which is a special character)
    		//return utils.hasCharsInSmartField(this.smartfield, specialCharacters);
    		return false;
    	}else{
    		return utils.hasCharsInStr(this.input.val(), specialCharacters);
    	}
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    RevisionParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    RevisionParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    RevisionParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new RevisionParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});