/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils", 
        
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding"
        ], function(
        		$, _, utils,
        		SmartField,
        		SmartNameEncoding) {

    /**
     * @constructor
     */
    var TextParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    TextParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	// Make the input...
    	this.input = $("<input>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    		"type":"text",
    		"value":"",
    		"class":"form-control attachleft search-input-lg"
    	}).attr("size", "30");

		// add the field
    	this.div.append(this.input);
    	
    	if(!this.settings.isSmartNamesEnabled()) {
    		// We have a normal input
    		
	    	// listen to certain events on the field...
    		
	    	//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
	    	this.input.on("keyup", function(event){
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    	});
	    	
	    	//blur=get out of the field >>> trigger final update event
	    	this.input.on("blur", _.bind(this._onFinalUpdate, this));
    	} else {
    		// We have a SmartField input
    		
    		// Convert the input into a SmartField
    		this.smartfield = new SmartField(this.input);
    		
    		// listen to certain events on the field...
    		this.smartfield.onValueChange(_.bind(this._onIntermediateUpdate, this));
    	}
    };
    
    //
    // Internal event handling
    //
    TextParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    TextParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    TextParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	if(this.settings.isSmartNamesEnabled()) {
    		value.setValue("valueSN", this.smartfield.getValue());
    	}else{
    		value.setValue("value", this.input.val());
    	}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    TextParameterField.prototype.setValueData = function (data) {
    	if(this.settings.isSmartNamesEnabled()) {
    		// SmartName mode...
    		var valueSN = data.getValue("valueSN");
    		this.smartfield.setValue(valueSN || "");
    	} else {
    		// Normal mode...
    		var value = data.getValue("value");
    		this.input.val(value || "");
    	}
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    TextParameterField.prototype.clear = function() {
    	if(this.settings.isSmartNamesEnabled()) {
    		// no need to implement, will not happen on Search Setup page.
    	} else{
	    	this.input.val("");
	    	this._triggerChange(false);
    	}
    };
    
    /**
     * Focus this field
     */
    TextParameterField.prototype.focus = function() {
    	if(this.settings.isSmartNamesEnabled()) {
    		this.smartfield.focus();
    	} else {
    		this.input.focus();
    	}
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    TextParameterField.prototype.valueContainsSmartName = function() {
    	return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this.smartfield);
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    TextParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	if(this.settings.isSmartNamesEnabled()) {
    		// Don't check for special characters in a SmartField...
    		// The hint would otherwise also show up if you use the autocomplete to type a SmartName...
    		// (For the autocomplete you start typing a SmartName with "[" , which is a special character)
    		//return utils.hasCharsInSmartField(this.smartfield, specialCharacters);
    		return false;
    	}else{
    		return utils.hasCharsInStr(this.input.val(), specialCharacters);
    	}
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    TextParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    TextParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    TextParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new TextParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        },
        /**
         * The constructor of this ParameterField... 
         * Used for extending this Parameterfield.
         */
        constructor: TextParameterField
    };
    return factory;
});