/**
 * Implements a simple Text Field for the Design Panel.
 */
define(["jquery",
        "underscore",
        "../../Utils",
        "packaging-content-management/TextContentSearchStatementFields"], 
function($,
		_,
		utils,
		TextContentSearchStatementFields) {

    /**
     * @constructor
     */
    var TextContentParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    	
    	// Watch value change of element type field to disable/enable statement fields.
    	this.settings.watchCriterion("ElementType", _.bind(function (data) {
    		if (data && "NUTRITIONFACTS" == data.getValue("value")) {
    			// If element type is selected as nutrition facts, then table content search should disable statement section.
    			this._statementParameterFields.disable();
			} else {
				// Enable statement fields once element type value is changed to other.
				this._statementParameterFields.enable();
			}
    	}, this));
    };
    
    /**
     * (private) Initialize the field.
     */
    TextContentParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	
		// Create Statement field
		this._statementParameterFields = new TextContentSearchStatementFields();
		this.input = this._statementParameterFields.getJQElement();
		
		this._textContentElement = this._statementParameterFields.getTextContentElement();
		this._textStatusElement = this._statementParameterFields.getTextStatusElement();

		// add the field
    	this.div.append(this.input);
    	
    	// listen to certain events on the field...
		
    	//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
    	this._textContentElement.on("keyup", function(event){
    		self._onIntermediateUpdate();
    		
    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
    		if (event.keyCode == 13) {
    			$(this).blur();
    		}
    	});
    	
    	//blur=get out of the field >>> trigger final update event
    	this._textContentElement.on("blur", _.bind(this._onFinalUpdate, this));
    	
    	this._textStatusElement.on("change", _.bind(this._onFinalUpdate, this));
    };
    
    //
    // Internal event handling
    //
    TextContentParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    TextContentParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    TextContentParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
		value.setValue("textContent", this._textContentElement.val());
		value.setValue("textStatus", this._textStatusElement.val());
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    TextContentParameterField.prototype.setValueData = function (data) {
		var textContent = data.getValue("textContent");
		var textStatus = data.getValue("textStatus");
		
		this._textContentElement.val(textContent || "");
		this._textStatusElement.val(textStatus || "");
		
		// Setting status on the UI
		this._statementParameterFields.setStatusValue(textStatus || "");
    	
		return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    TextContentParameterField.prototype.clear = function() {
    	this._textContentElement.val("");
    	this._textStatusElement.val("");
	    this._triggerChange(false);
    };
    
    /**
     * Focus this field
     */
    TextContentParameterField.prototype.focus = function() {
    	this._textContentElement.focus();
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    TextContentParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    TextContentParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
		return utils.hasCharsInStr(this._textContentElement.val(), specialCharacters);
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    TextContentParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    TextContentParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    TextContentParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new TextContentParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});