/**
 * Implements a simple Text Field for the Design Panel.
 */
define(["jquery",
        "underscore",
        "../../Utils",
        "packaging-content-management/text-content-services/elementTypeService"], 
function($,
		_,
		utils,
		elementTypeService) {

    /**
     * @constructor
     */
    var TextContentParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	this._elementTypeLoadReq = new $.Deferred();
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    TextContentParameterField.prototype._initialize = function(){
    	var self = this._elementTypeLoadReq;
    	var element = $('<select>').attr({
    		"data-sahi-id":this.fieldId,// For automated testing
    		"class":"form-control attachleft search-dropdown-lg"
    	});
    	
    	elementTypeService.getElementTypes(function(elementList) {
    		element.append($('<option>', {value: '', text: ''}));
    		for (var elementTypeCode in elementList) {
    			if (elementList.hasOwnProperty(elementTypeCode)){
    				element.append($('<option>', {value: elementTypeCode.trim().replace(/\_/g, ''), text: elementList[elementTypeCode].elementHumanReadable}));
    			}
    		}
    		self.resolve(element);
		});
    		
    	this.input = element;

		// add the field
    	this.div.append(this.input);
    	
    	//change=get out of the field >>> trigger final update event
    	this.input.on("change", _.bind(this._onFinalUpdate, this));
    };
    
    //
    // Internal event handling
    //
    TextContentParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    TextContentParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    TextContentParameterField.prototype.getValueData = function (value, feedback) {
    	value.clear();
    	value.setValue("value", this.input.val());
    	value.setValue("isDropdown", "true");
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    TextContentParameterField.prototype.setValueData = function (data, value) {
    	$.when(this._elementTypeLoadReq).done(function() {
			var value = data.getValue("value");
			this.input.val(value || "");
	    	return true;
    	}.bind(this));
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    TextContentParameterField.prototype.clear = function() {
    	this.input.val("");
	    this._triggerChange(false);
    };
    
    /**
     * Focus this field
     */
    TextContentParameterField.prototype.focus = function() {
    	this.input.focus();
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    TextContentParameterField.prototype.valueContainsSmartName = function() {
    	return false;
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    TextContentParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	// Not required to check, its a drop-down with fixed values
		return false;
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    TextContentParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    TextContentParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    TextContentParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new TextContentParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});