/**
 * Implements User or Me Field for the Design Panel.
 */
define([
        "jquery", 
        "underscore", 
        "../../Utils",
        "utils/webi18n!SearchInputs",
        
        // Smartname related stuff
        "component/smartnames/smartfield/smartfield",
        "component/smartnames/encoding/smartnameencoding"
        ], function(
        		$, _, utils,
        		i18n,
        		SmartField,
        		SmartNameEncoding) {

    /**
     * @constructor
     */
    var UserMeChoiceParameterField = function(div, fieldId, settings, dataProvider) {
    	this.div = $(div);
    	this.fieldId = fieldId;
    	this.settings = settings;
    	this.changeCallbacks = [];
    	
    	this._initialize();
    };
    
    /**
     * (private) Initialize the field.
     */
    UserMeChoiceParameterField.prototype._initialize = function(){
    	var self = this;
    	
    	// Create the Input field for User or Me choice
    	this.constructFieldElements();
    	
    	if(!this.settings.isSmartNamesEnabled()) {
    		// We have a normal input
    		
	    	// listen to certain events on the field...
    		
	    	//keyup event >>> check whether Enter key was pressed (final update then) > otherwise intermediate
	    	this.input.on("keyup", function(event){
	    		self._onIntermediateUpdate();
	    		
	    		//if "Enter" key was pressed > blur the current element (so that it would trigger its submit)
	    		if (event.keyCode == 13) {
	    			$(this).blur();
	    		}
	    	});
	    	
	    	//blur=get out of the field >>> trigger final update event
	    	this.input.on("blur", _.bind(this._onFinalUpdate, this));
	    	
    	} else {
    		// We have a SmartField input
    		
    		// Convert the input into a SmartField
    		this.smartField = new SmartField(this.input);
    		
    		// listen to certain events on the field...
    		this.smartField.onValueChange(_.bind(this._onIntermediateUpdate, this));
    	}
    	
    	// on click event when option me is clicked 
    	this.me.on("click", _.bind(function () {
    		this._onUserMeTypeChanged(this.me);
    	}, this));
    	
    	// on click event when option user is clicked 
    	this.user.on("click", _.bind(function () {
    		this._onUserMeTypeChanged(this.user);
    	}, this));
    	
    };
  
    /**
     * Make the User or Me choice field 
     * 
     * This method will create fields [this.me] , [this.user] , [this.input] - This fields are public and it will accessible all methods
     * 
     */
    UserMeChoiceParameterField.prototype.constructFieldElements = function(){
    	
    	var table = $("<table>").addClass("parameterFieldTable");
    	var row = $("<tr>");
    	var colButtons = $("<td>");
    	var colInputs = $("<td>").addClass("widecell");
    	
    	row.append(colButtons, colInputs);
    	table.append(row);
    	
    	var buttonDiv = $("<div>").addClass("btn-group searchtogglegroup");
    	colButtons.append(buttonDiv);
    	
    	// Make the me button
    	this.me = $("<div>", {
    		"data-sahi-id":this.fieldId+"_type_Me",// For automated testing
    		"align":"top",
    		"value": "me",
    		"class":"btn btn-default",
    		"title" : i18n.t('FILTER_MYTASKS')
    		});
    	
    	this.me.css({"cursor": "pointer"});
    	this.me.append($("<i>").addClass("icon icon-WCR_User-alt"));
    	
    	buttonDiv.append(this.me);

    	// Make the user button
    	this.user = $("<div>", {
    		"data-sahi-id":this.fieldId+"_type_User",// For automated testing
    		"align":"top",
    		"value": "user",
    		"class":"btn btn-default active",
    		"title" : i18n.t('USER')
    		});
    	
    	this.user.css({"cursor": "pointer"});
    	this.user.append($("<i>").addClass("icon icon-WCR_Menu-Users"));
    	
    	buttonDiv.append(this.user);
    	
    	// Make the input...
    	this.input = $("<input>", {
    		"data-sahi-id":this.fieldId,// For automated testing
    		"type":"text",
    		"align":"top",
    		"value":"",
    		"class":"form-control search-input-lg"
    	}).attr("size", "30");
    	
    	colInputs.append(this.input);

    	this.meLabel = $("<div>", {
    		"id":this.fieldId+"_text_me",
    		"class" : "search_metext"
    	}).text(i18n.t('FILTER_MYTASKS')).hide();
    	
    	colInputs.append(this.meLabel);
    	
    	// Add tooltip for the button options
    	require(["jqueryUI"], _.bind(function(){
    		this.me.tooltip({position:{my: "center bottom-5", at: "center top"}});
    		this.user.tooltip({position:{my: "center bottom-5", at: "center top"}});
		}, this));
    	
		// add the fields
    	this.div.append(table);
    	
    };
    
    /**
	 * Just Highlight the Given button
	 * @param {JQuery} activeElement - The User or Me button that's been selected
	 */
    UserMeChoiceParameterField.prototype._highLightActiveElement = function (activeElement) {
		
		this.me.removeClass("active");
		this.user.removeClass("active");
		
		activeElement.addClass("active");
	};
    
    /**
     *  This function is called when a user/me type selector is changed
     *  if me is selected it will fill the smart name into the text field and hide it
     *  
     * @param {JQuery} element - The option button selected "User" or "Me" 
     *        
     */
    UserMeChoiceParameterField.prototype._onUserMeTypeChanged = function(element) {
    	    var val = element.val();
   	    	if ("me" == val) {
   	    		if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.clear();
        			this.smartField.getJQElement().hide();
				} else {
					this.input.val("");
        			this.input.hide();
				}
   	    		// Highlight Me Option
   	    		this._highLightActiveElement(this.me);
   	    		this.meLabel.show();
    	    } else {
    	    	if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.clear();
        			this.smartField.getJQElement().show();
				} else {
					this.input.val("");
					this.input.show();
				}
    	    	// Highlight User Option
   	    		this._highLightActiveElement(this.user); 
    	    	this.meLabel.hide();
    	    }
    	    
    		// call the update method to notify the changes
    		this._onFinalUpdate();
    };
    
    //
    // Internal event handling
    //
    UserMeChoiceParameterField.prototype._onIntermediateUpdate = function(){
    	this._triggerChange(true);
    };
    
    UserMeChoiceParameterField.prototype._onFinalUpdate = function(){
    	this._triggerChange(false);
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    UserMeChoiceParameterField.prototype.getValueData = function (value, feedback) {
    	
    	value.clear();
    	
    	if(this.me.hasClass("active")) {
			value.setValue("user_type","ME");
		}else {
			value.setValue("user_type","USER");
			
			if(this.settings.isSmartNamesEnabled()) {
	    		value.setValue("user_nameSN", this.smartField.getValue());
	    	}else{
	    		value.setValue("user_name", this.input.val());
	    	}
		}
    };
    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} data - the object which contains all the information about the value
     * @returns {Boolean} true if the field can represent the data which is set
     */
    UserMeChoiceParameterField.prototype.setValueData = function (data) {

			var user_type = data.getValue("user_type");
			if (user_type == "ME") {
				// Highlight Me Option
   	    		this._highLightActiveElement(this.me);
				
				if (this.settings.isSmartNamesEnabled()) {
        			this.smartField.getJQElement().hide();
				} else{
					this.input.hide();
				}
				this.meLabel.show();
				
			} else {
				// Highlight User Option
   	    		this._highLightActiveElement(this.user);
				this.meLabel.hide();
				
				if (this.settings.isSmartNamesEnabled()) {
					// SmartName mode...
					var valueSN = data.getValue("user_nameSN");
					this.smartField.setValue(valueSN || "");
				} else {
					// Normal mode...
					var value = data.getValue("user_name");
					this.input.val(value || "");
				}
			}
	
    	return true;
    };
    
    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    UserMeChoiceParameterField.prototype.clear = function() {
	    	this.input.val("");
	    	this.input.show();
	    	this._highLightActiveElement(this.user); 
	    	this.meLabel.hide();
	    	this._triggerChange(false);
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {boolean} true when this field contains a SmartName
     */
    UserMeChoiceParameterField.prototype.valueContainsSmartName = function() {
    	return this.settings.isSmartNamesEnabled() && utils.hasSmartNameInSmartField(this.smartField);
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param {char[]} specialCharacters - an array of special characters...
     */
    UserMeChoiceParameterField.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	if(this.settings.isSmartNamesEnabled()) {
    		// Don't check for special characters in a SmartField...
    		// The hint would otherwise also show up if you use the autocomplete to type a SmartName...
    		// (For the autocomplete you start typing a SmartName with "[" , which is a special character)
    		//return utils.hasCharsInSmartField(this.smartfield, specialCharacters);
    		return false;
    	}else{
    		return utils.hasCharsInStr(this.input.val(), specialCharacters);
    	}
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes. 
     * Function has one parameter, which should be true when the user is not finished with modifying the field.
     */
    UserMeChoiceParameterField.prototype.onChange = function(callback) {
    	this.changeCallbacks.push(callback);
    };
    
    /**
     * (private) Trigger a change event
     */
    UserMeChoiceParameterField.prototype._triggerChange = function(intermediatechange) {
    	$.each(this.changeCallbacks, function(i, callback) {
    		callback(intermediatechange);
    	});
    };
    
    //
    // Destructor
    //
    
    /**
     * Destroys this field...
     * Remove listeners and clean up the div.
     */
    UserMeChoiceParameterField.prototype._destroy = function() {
    	this.div.empty();
    	this.changeCallbacks = [];
    };

    //
    // Factory...
    // Provides methods to create new parameter fields
    // & provides some general settings about the implementation
    //
    var factory = {
        /** 
         * Make a new field of this type
         * Returned object should implement certain functions
         * @param {DOMElement} div - The dom element in which we should render the field
         * @param {String} fieldName - The name of the field for which this field is created
         * @param {SearchSettings} settings - Information about settings which are enabled
         *     - E.g. are SmartNames enabled?
         * @param {DataProvider} data - interface to have access to data
         * @param {FieldProperties} fieldProperties - additional properties of the field
         */
        make: function(div, fieldName, settings, dataProvider, fieldProperties) {
        	return new UserMeChoiceParameterField(div, fieldName, settings, dataProvider);
        }, 
        /**
         * Destroy a field created by this factory...
         * @param obj - the object which was returned by this factory
         */
        destroy: function(obj) {
        	obj._destroy();
        }
    };
    return factory;
});