/**
 * Implements a simple Text Field for the Design Panel.
 */
define([
	"jquery",
	"underscore",
	"./FacetSelectionCriterionUtil",
	"./FacetFieldHeader",
	"../../../dataobjects/ValueData",
	"utils/webi18n!SearchDashboard"
], function (
	$,
	_,
	FacetSelectionCriterionUtil,
	FacetFieldHeader,
	ValueData,
	i18n) {
	"use strict";


	/**
	 * Upper class for ParameterFields that display Facet Information
	 * 
	 * @constructor
	 * @param {JQueryObj,HTMLObject} div - element in which the parameter field should be rendered
	 * @param {String} fieldId - field ID of the parameterfield	
	 * @param {SearchSettings} settings
	 * @param {FieldProperties} fieldProperties
	 */
	var AbstractFacetParameterField = function (div, fieldId, settings, fieldProperties) {
		this.div = $(div);
		this.fieldId = fieldId;
		this.settings = settings;
		this.changeCallbacks = [];
		this._fieldProperties = fieldProperties;
		
		/**
		 * @member {Array.<String>} list of fields on which the parameterfield facets
		 * currently this is an array with only one item (we don't support combined facets yet)
		 */
		this._facetIDs = this._fieldProperties.facetIds;
		this._maxFacetsExceeded = false;

		this._header = new FacetFieldHeader(this._fieldProperties.instanceConfig.getValue("title")  || "Facet", this._fieldProperties.isMultiValued, this.getFacetValueData.bind(this));
		this.div.append(this._header.getElement());
		
		//placeholder div for warning icon
		this._warning = $("<div>");
		this.div.append(this._warning);

		this._facetSelection = null;
		this._header.on("deselectFacetValue", this.deselectFacetValues.bind(this));
		this._header.on("deselectAllFacets", this.clear.bind(this));
		// add operator change
		this._header.on("changeOperator", this.changeOperator.bind(this));
		// used to make sure the warning is only rendered once
		
		this._initialize();
		this.render();
	};

	/**
	 * (private) Initialize the field.
	 */
	AbstractFacetParameterField.prototype._initialize = function () {
		// Implement by subclass
	};

	//
	// Set & Get the value
	//

	/**
	 * Get the value by filling the value object.
	 * Also do the validation in the mean time and
	 * use the feedback object to give the information back.
	 * 
	 * @param {ValueData} value - the object to put all the value information in
	 * @param {ValidationFeedback} feedback - the object 
	 */
	AbstractFacetParameterField.prototype.getValueData = function (value, feedback) {
		if (this._facetSelection) {
			FacetSelectionCriterionUtil.setSelection(this._facetIDs, value, this._facetSelection);
		}
	};


	/**
	 * Set the value, update the field to represent this value.
	 * 
	 * @param {ValueData} data - the object which contains all the information about the value
	 * @returns {Boolean} true if the field can represent the data which is set
	 */
	AbstractFacetParameterField.prototype.setValueData = function (data) {
		this._facetSelection = FacetSelectionCriterionUtil.getSelection(this._facetIDs, data);
		this._header.setSelection(this._facetSelection);
		this.render();
		return true;
	};

	/**
	 * Select a single facet value
	 * @param {String} value 
	 */
	AbstractFacetParameterField.prototype.selectFacetValue = function (value) {
		if (!this._facetSelection) {
			this._facetSelection = FacetSelectionCriterionUtil.getSelection(this._facetIDs, new ValueData());
		}

		this._facetSelection.selectSingleValue(value);
		this._header.setSelection(this._facetSelection);

		this._triggerChange(false);
	};

	/**
	 * Select a combo of facet values
	 * @param {Object} values {facetId: value}	
	 */
	AbstractFacetParameterField.prototype.selectFacetValues = function (values) {
		if (!this._facetSelection) {
			this._facetSelection = FacetSelectionCriterionUtil.getSelection(this._facetIDs, new ValueData());
		}

		this._facetSelection.select(values);
		this._header.setSelection(this._facetSelection);

		this._triggerChange(false);
	};

	/**
	 * Deselect a singel Facet Value
	 * @param {String} value 
	 */
	AbstractFacetParameterField.prototype.deselectFacetValue = function (value) {
		if (this._facetSelection) {
			this._facetSelection.deselectSingleValue(value);
			this._header.setSelection(this._facetSelection);
			this._triggerChange(false);
		}
	};
	
	/**
	 * Get the information for a certain facet value
	 * @param facetid
	 * @param value
	 */
	AbstractFacetParameterField.prototype.getFacetValueData = function(facetid, value) {
		return this._fieldProperties.getFacetValueData(facetid, value);
	};
	
	/**
	 * Get the label which should be displayed for a certain facet entry
	 * @param   {ValueData} facetInfo 
	 * @returns {String}    label to display
	 */
	AbstractFacetParameterField.prototype.getLabelForFacet = function(facetInfo){
		return facetInfo.getValue("translation", facetInfo.getValue("label", facetInfo.getValue("id")));
	};
	
	

	/**
	 * Deselect a set of Facet Values
	 * @param {Object} valuesMap 
	 */
	AbstractFacetParameterField.prototype.deselectFacetValues = function (valuesMap) {

		if (this._facetSelection) {
			this._facetSelection.deselect(valuesMap);
			this._header.setSelection(this._facetSelection);
			this._triggerChange(false);
		}
	};
	
	/**
	 * Selection of operator choice - "OR" or "AND"
	 * @param {String} value 
	 */
	AbstractFacetParameterField.prototype.changeOperator = function (operator) {
		if (!this._facetSelection) {
			this._facetSelection = FacetSelectionCriterionUtil.getSelection(this._facetIDs, new ValueData());
		}
		this._header.setSelection(this._facetSelection);
		this._header.setOperatorSelection(operator);
		this._triggerChange(false);
		
	};

	/**
	 * Toggle selection of a certain value
	 * @param {String} value 
	 */
	AbstractFacetParameterField.prototype.toggleSelectionForValue = function (value) {
		if (this._facetSelection && this._facetSelection.isSelectedSingleValue(value)) {
			this.deselectFacetValue(value);
		} else {
			this.selectFacetValue(value);
		}
	};


	/**
	 * Clear the Field (called by "Clear this form" functionality) 
	 */
	AbstractFacetParameterField.prototype.clear = function () {
		if (this._facetSelection) {
			this._facetSelection.clear();

			this._triggerChange(false);
		}
	};

	/**
	 * Does this field contain a SmartName?
	 * 
	 * @returns {bool} true when this field contains a SmartName
	 */
	AbstractFacetParameterField.prototype.valueContainsSmartName = function () {
		return false;
	};

	/**
	 * Does this field contain a special search character?
	 * 
	 * @param {char[]} specialCharacters - an array of special characters...
	 */
	AbstractFacetParameterField.prototype.valueContainsSpecialSearchCharacters = function (specialCharacters) {
		return false;
	};


	AbstractFacetParameterField.prototype.render = function () {
		this._header.render();
		if (!this._didRequestAnimationFrame) {
			var self = this;
			this._didRequestAnimationFrame = window.requestAnimationFrame(function () {
				self._doRender();
				self._didRequestAnimationFrame = null;
			});
		}
	};


	AbstractFacetParameterField.prototype._doRender = function () {
		// IMPLEMENT in subclass

	};

	/**
	 * Facet information was updated
	 * @param {ValueData} facetInfo 
	 */
	AbstractFacetParameterField.prototype.doFacetUpdate = function (facetInfo) {
		this._facetData = facetInfo.getSubList("facet") || [];
		this._header.setFacetData(this._facetData);
		this._maxFacetsExceeded = (facetInfo.getValue("maxFacetsExceeded")==="true") ? true : false;					
		
		this.render();
	};

	//
	// Change listeners
	//

	/**
	 * Registers a new function to call when the field changes...
	 * 
	 * @param {Function} callback - a function to call when the field changes. 
	 * Function has one parameter, which should be true when the user is not finished with modifying the field.
	 */
	AbstractFacetParameterField.prototype.onChange = function (callback) {
		this.changeCallbacks.push(callback);
	};

	/**
	 * (private) Trigger a change event
	 */
	AbstractFacetParameterField.prototype._triggerChange = function (intermediatechange) {
		this.render();

		$.each(this.changeCallbacks, function (i, callback) {
			callback(intermediatechange);
		});
	};

	//
	// Destructor
	//

	/**
	 * Destroys this field...
	 * Remove listeners and clean up the div.
	 */
	AbstractFacetParameterField.prototype._destroy = function () {
		this.div.empty();
		this.changeCallbacks = [];
	};
	
	/**
	 * When the maximum number of values is exceeded, show a warning icon.
	 * @private
	 */
	AbstractFacetParameterField.prototype._checkForNumberOfValuesExceeded = function(){
		this.div.removeClass("dashboardInfoIndicationParent");
		this._warning.removeClass();
		this._warning.removeAttr("title");
		if(this._maxFacetsExceeded){
			this.div.addClass("dashboardInfoIndicationParent");
			this._warning.addClass("icon icon-grey icon-WCR_Info-square12 dashboardInfoIndication");
			this._warning.attr("title",i18n.t("WARNING_MORE_FACETS"));
			this.div.append(this._warning);
		}
	};


	return AbstractFacetParameterField;
});