define(["underscore",
		"jquery",
		"utils/Observable",
		"utils/ColorUtils",
		        "utils/webi18n!SearchInputs"
	],
	function (_,
		$,
		Observable,
		ColorUtils,
		i18n) {
		"use strict";
		/**
		 * This display the list of selected facet tokens
		 * @param {String}  name          - Title of the header
		 * @param {boolean} isMultiValued - If true the operator switch will be shown
		 * @param {Function} getFacetValueData - The call to get data for a certain facet value
		 * @class FacetFieldHeader
		 */
		var FacetFieldHeader = function (name, isMultiValued, getFacetValueData) {
			Observable.mixin(this, "FacetFieldHeader");

			this._didRequestAnimationFrame = null;
			this._element = $("<div>", {'class':"facet-field-header"});

			this._isMultiValued = isMultiValued;
			this._getFacetValueData = getFacetValueData;
			
			this._header = $("<div>", {
				'class': "facet-header"
			});
			
			//Making the oparator switch only if multiple selections are allowed at the same time
			if(this._isMultiValued){
				this._operatorSelection = $("<div>", {
					 text : i18n.t("SEARCH_OPERATOR_OR"),
					 operator : "OR",
					 // default operator in WebCenter is OR
					class : "label label-primary pull-right labelAsButton"
				});
							
				this._header.append(this._operatorSelection);

				// click event for operator
				this._operatorSelection.click(function (event) {
					var operator = $(event.target).attr("operator");
					if (operator === "OR") {
						operator = "AND";
					} else {
						operator = "OR";
					}
					this._setOperatorText(operator);
					this._emit("changeOperator", operator);
				}.bind(this));
			}
	    	
			this._deselectAll = $("<i>", {
				'class': "icon icon-WCR_Filter-Remove12",
				title: i18n.tFallback("DESELECT_ALL", "Deselect All")
			});

			this._title = $("<h4>", {
				text: name,
				title: name
			});
			
			
			this._header.prepend(this._title.append(this._deselectAll));
			this._element.append(this._header);
			
			// while initialize hide the deselect option
			this._deselectAll.hide();


			this._tokens = $("<div>", {
				'class': "facet-tokens"
			}).appendTo(this._element);

			this._facetSelection = null;
			this._facetData = null;


			// notify about deselecting elements
			this._element.on("click", "i.delete-token", function (event) {
				event.stopPropagation();
				event.preventDefault();

				if (this._facetSelection) {
					this._emit("deselectFacetValue", $(event.target).data("values"));
				}
			}.bind(this));
			this._deselectAll.on("click", function (event) {
				event.preventDefault();
				event.stopPropagation();
				this._emit("deselectAllFacets");
			}.bind(this));
		};

		/**
		 * Get the element of the FacetFieldHeader
		 * @returns {JQueryObj} [[Description]]
		 */
		FacetFieldHeader.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Set the selection object
		 * @param {FacetSelection} facetSelection 
		 */
		FacetFieldHeader.prototype.setSelection = function (facetSelection) {
			this._facetSelection = facetSelection;

			this.render();

		};
		
		/**
		 * set the operator choice
		 * @param operatorSelection
		 */
		FacetFieldHeader.prototype.setOperatorSelection = function (operatorSelection) {
			if(this._facetSelection) {
				this._facetSelection.setOperator(operatorSelection);
				this.setSelection(this._facetSelection);
				this.render();
			}
		};
		
		/**
		 * Set the operator text i.e "OR" or "AND"
		 * 
		 * @param operator
		 */
		FacetFieldHeader.prototype._setOperatorText = function (operator) {
			
			if(operator === "OR") {
				this._operatorSelection.text(i18n.t("SEARCH_OPERATOR_OR"));
			}else if(operator === "AND"){
				this._operatorSelection.text(i18n.t("SEARCH_OPERATOR_AND"));
			}else{
				throw new Error("only OR and AND are allowed operators");
			}
			this._operatorSelection.attr("operator",operator);
		};

		/**
		 * Set the Facet Data
		 * @param {Array.<ValueData>} facetData 
		 */
		FacetFieldHeader.prototype.setFacetData = function (facetData) {
			this._facetData = facetData;

			this.render();
		};

		/**
		 * Trigger a render of the facet field header
		 */
		FacetFieldHeader.prototype.render = function () {
			if (!this._didRequestAnimationFrame) {
				var self = this;
				this._didRequestAnimationFrame = window.requestAnimationFrame(function () {
					self._doRender();
					self._didRequestAnimationFrame = null;
				});
			}
		};

		/**
		 * Get the labe lfor a token
		 * @private
		 * @param   {object} selection 
		 * @returns {String} 
		 */
		FacetFieldHeader.prototype._getLabelForSelection = function (selection) {
			return _.compact(_.map(this._facetSelection.getFacetIds(), function (id) {
				var facetInfo = this._getFacetValueData(id, selection.values[id]);
				if(facetInfo) {
					return facetInfo.getValue("translation", facetInfo.getValue("label", facetInfo.getValue("id")));
				} else {
					return "??";
				}
			}, this)).join(", ");
		};

		/**
		 * Update the rendering of the field header
		 * @private
		 */
		FacetFieldHeader.prototype._doRender = function () {
			var tokens = $("<div>", {
				'class': "facet-tokens"
			});

			if (this._facetSelection && this._facetData) {
				_.each(this._facetSelection.getSelection(), function (selection, index) {
					var facetId = this._facetSelection.getFacetIds()[0];
					var facetInfo = this._getFacetValueData(facetId, selection.values[facetId]);

					var token = $("<div>", {
						'class': "facet-token",
						text: this._getLabelForSelection(selection)
					});
					var deleteIcon = $('<i/>', {
						'class': 'icon icon-small icon-WCR_Close12 delete-token',
						title:  i18n.tFallback("DESELECT", "Deselect")
					}).data("values", selection.values);


					var count = 0;
					var color = null;
					var contrastColor = null;
					if (facetInfo) {
						count = facetInfo.getValue("count", "0");
						color = facetInfo.getValue("color_rgb", null);
						if (color){
							contrastColor = ColorUtils.calculateContrastColor(color, 0.75);
						}
					}

					if (color) {
						token.addClass("colored");
						token.css({
							'background-color': color,
							'color': contrastColor
						});
						deleteIcon.css({
							'color': contrastColor
						});
					}

					token.append($("<span>", {
						'class': "badge",
						text: count
					}));

					tokens.append(token.append(deleteIcon));
				}, this);
			}
			this._tokens.replaceWith(tokens);
			this._tokens = tokens;
			if(this._facetSelection) {
				if(this._isMultiValued){
					var operator = this._facetSelection.getOperator() || "OR";
					this._setOperatorText(operator);
				}
				
				// Show the deselect option based on facet selection
				if(this._facetSelection.getSelection() && this._facetSelection.getSelection().length > 0) {
					this._deselectAll.show();
				} else {
					this._deselectAll.hide();
				}
			}
		};

		return FacetFieldHeader;
	});
