define(["underscore",
		"../../../dataobjects/ValueDataUtils"
	],
	function (_,
		ValueDataUtils) {
		"use strict";

		/**
		 * DataModel for the selection for a certain facet
		 * @param {String|Array.<String>} facetId... faceted fields	
		 */
		var FacetSelection = function (facetId) {
			if (_.isArray(facetId)) {
				this._facetIds = facetId;
			} else {
				this._facetIds = _.toArray(arguments);
			}

			this._operator = "OR";
			this._selection = [];

		};
		/**
		 * Get the list of faceted fields
		 * @returns {Array.<String>} 
		 */
		FacetSelection.prototype.getFacetIds = function () {
			return this._facetIds;
		};

		/**
		 * Clear the selection
		 */
		FacetSelection.prototype.clear = function () {
			this._selection = [];
		};

		/**
		 * Set the operator
		 * @param {String} operator - AND or OR
		 */
		FacetSelection.prototype.setOperator = function (operator) {
			if (!_.contains(["OR", "AND"], operator)) {
				throw new Error("only OR and AND are allowed operators");
			}
			this._operator = operator;
		};

		/**
		 * Get the operator
		 * @returns {String} 
		 */
		FacetSelection.prototype.getOperator = function () {
			return this._operator;
		};

		/**
		 * Internal metho to make a valuesMap from a single value
		 * @private
		 * @param   {String} value 
		 * @returns {Object} valuesMap {facetId: value}
		 */
		FacetSelection.prototype._convertValueToValuesMap = function (value) {
			var values = {};
			values[this._facetIds[0]] = value;
			return values;
		};
		/**
		 * Set the selection
		 * @param {Array.<Object>} selection - object: {inverse: boolean, values: {id:value, id: value}}
		 */
		FacetSelection.prototype.setSelection = function (selection) {
			this._selection = selection;
		};

		/**
		 * Get the selection
		 * @returns {Array.<Object>} selection - {inverse: boolean, values: {id:value, id: value}}
		 */
		FacetSelection.prototype.getSelection = function () {
			return this._selection;
		};

		/**
		 * Select a single value
		 * @param {String} value 
		 */
		FacetSelection.prototype.selectSingleValue = function (value, inverse) {
			this.select(this._convertValueToValuesMap(value), inverse);
		};

		/**
		 * Select a value combo
		 * @param {Object}   valuesMap {facetId: value, facetId: value}
		 * @param {Boolean} [inverse]
		 */
		FacetSelection.prototype.select = function (valuesMap, inverse) {
			var selection = this.isSelected(valuesMap);
			if (!selection) {
				this._selection.push({
					inverse: inverse || false,
					values: valuesMap
				});
			} else {
				// exists make sure to update inverse flag
				selection.inverse = inverse || false;
			}
		};

		/**
		 * Deselect a single value
		 * @param {String} value 
		 */
		FacetSelection.prototype.deselectSingleValue = function (value) {
			this.deselect(this._convertValueToValuesMap(value));
		};

		/**
		 * Check if a single value is selected
		 * @param   {String}  value 
		 * @returns {Object} selection object if selected 
		 */
		FacetSelection.prototype.isSelectedSingleValue = function (value) {
			return this.isSelected(this._convertValueToValuesMap(value));
		};

		/**
		 * Check if a certain value is selected
		 * @param   {Object}  valuesMap 
		 * @returns {Object}  selection object if selected
		 */
		FacetSelection.prototype.isSelected = function (valuesMap) {

			return _.find(this._selection, function (selectionEntry) {
				return _.isEqual(selectionEntry.values, valuesMap);
			});
		};

		/**
		 * Deselect a value combo
		 * @param  {Object}   valuesMap {facetId: value, facetId: value}
		 */
		FacetSelection.prototype.deselect = function (valuesMap) {

			this._selection = _.filter(this._selection, function (selectionEntry) {
				return !_.isEqual(selectionEntry.values, valuesMap);
			});

		};

		/**
		 * Convert FacetSelection Object to ValueData
		 * @param {ValueData} valueData 
		 */
		FacetSelection.prototype.toValueData = function (valueData) {

			ValueDataUtils.setStringList(valueData, "facetids", this._facetIds);
			valueData.setValue("operator", this.getOperator());
			valueData.setSubList("selection", null);
			_.each(this.getSelection(), function (selectionEntry) {
				var selectionData = valueData.addNewItemToList("selection");

				selectionData.setValue("inverse",
					selectionEntry.inverse ? "1" : "0");
				var selectionEntryData = selectionData.addNewItemToList("values");
				_.each(selectionEntry.values, function (value, key) {
					selectionEntryData.setValue(key, value);
				});
			});
		};


		/**
		 * Initialize FacetSelection object from value data
		 * @param   {ValueData}      valueData 
		 * @returns {FacetSelection} 
		 */
		FacetSelection.fromValueData = function (valueData) {
			var selection = new FacetSelection(ValueDataUtils.getStringList(valueData, "facetids"));

			selection.setOperator(valueData.getValue("operator") || "OR");

			selection.setSelection(_.map(valueData.getSubList("selection"), function (selectionData) {
				var values = {};
				if (selectionData.getSubList("values") && selectionData.getSubList("values")[0]) {
					var valuesData = selectionData.getSubList("values")[0];
					_.each(valuesData.getValueKeys(), function (key) {
						values[key] = valuesData.getValue(key);
					});
				}

				return {
					inverse: selectionData.getValue("inverse") === "1",
					values: values
				};
			}));

			return selection;
		};



		return FacetSelection;
	});