define(["jquery",
        "underscore",
        "component/modal/Modal",
        "component/multiSelect/view/MultiSelectList",
        "wcr/FilterUtils",
        "utils/Observer",
        "utils/webi18n!SearchInputs",
        "wcr/webcenter_xml"], 
	function($,
			_,
			ModalPopup,
			MultiSelectList,
			FilterUtils,
			Observer,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Popup to select attributes based on attribute category
		 */
		var AttributesSearchPopupView = function(attrsData) {
			
			if(attrsData && !$.isEmptyObject(attrsData)) {
				if(attrsData.categoryid) {
					this._acatid = attrsData.categoryid;
				} else {
					this._acatid = null;
				}
				
				if(attrsData.attributeids && attrsData.attributeids.length > 0) {
					this._attributeids = attrsData.attributeids;
				} else {
					this._attributeids = [];
				}
			}
			
			// Add and Cancel buttons
			this._addButton = null;
			this._cancelButton = null;
			
			// callbacks for add and cancel
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function(event, selAttrsData) {
				_.each(this._onAddCallbacks, _.bind(function(callback) {
					callback(selAttrsData);
				}, this));
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			// Construct the popup
			this._constructPopup();
		};
		
		/**
		 * Create the popup
		 */
		AttributesSearchPopupView.prototype._constructPopup = function() {
			
			// Create only one instance of popup.
			if(!this._popup) {
				// Get the popup html content
				this._popupContent = this._getPopupContent();
				
				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.setTitle(SearchInputs.t("SEARCHATTRIBUTESPROMPT"));
				this._popup.setFooter(this._getFooterContent());
			}
		};
		
		/**
		 * Construct the html structure of the popup
		 * 
		 * @returns
		 */
		AttributesSearchPopupView.prototype._getPopupContent = function() {
			
			// popup html content
			var popupContent = $('<div>').attr({"name": "attributes_search_popup"});
			
			// Dropdown to select attribute category
			this._acatSelect = $("<select>", {"name": "actionlist", "class": "form-control"});
			
			this._acatSelect.change(_.bind(function(event) {
				// Clear the attributes html container
				this._attrSelectDiv.empty();
				
				this._attributeids = [];
				
				// Get and render the attributes for selected category
				this._getAndRenderAttributes($(event.target).attr("value"));
			}, this));
			
			var acatSection = $("<div>", {"class": "add-spacing-bottom"}).append(
					$("<label>").append(SearchInputs.t("ATTSEARCHTITLE")),
					this._acatSelect
			);
			
			// Append the acat dropdown to the popup body
			popupContent.append(acatSection);
			
			// Container to render the attributes
			this._attrSelectDiv = $("<div>", {"name": "attributes_container"});
			
			// Append the container to popup body
			popupContent.append(this._attrSelectDiv);
			
			// Get the attribute categories and populate the acat select options
			this._getAttributeCategories();
			
			
			return popupContent;
		};
		
		
		/**
		 * Method the get attribute categories using Ajax call 
		 * and populate the options of acat select dropdown
		 */
		AttributesSearchPopupView.prototype._getAttributeCategories = function() {
			/*
			 * Array to hold the attribute category information
			 * 
			 * [
			 * 		{id: , name: },
			 * 		{id: , name: }
			 * ]
			 */
			this._acatData = [];
			
			/*
			 * Get the attribute categories using ajax call,
			 * populate the options of select dropdown of acat
			 * and render multi select for attributes in currently selected acat
			 */
			$.when($.ajax({url: "GetAttributeCategoryList.jsp", method: "POST", dataType: "xml"})).then(_.bind(function(acatXMLDoc, textStatus, jqXHR) {
				
				var acatXML = $(acatXMLDoc).children().get(0);
				
				// check the response if user session is expired, redirect to login page
				if(acatXML.tagName === "error") {
					var errorCode = $(acatXML).find('code').text();
					if(errorCode && '000' === errorCode) {
	            		document.location.reload();
	            	}
				}
				// If attribute categories exists load information into this._acatData object
				if(acatXML.tagName == "category_list") {
					var acats = $(acatXML).children();
					
					for(var i=0; i<acats.length; i++) {
						this._acatData.push({
							id : $(acats[i]).attr("id"),
							name : $(acats[i]).find("name").text()
						});
					}
					
					// Render the options of acat select dropdown
					this._renderACatSelectOptions();
				} 
				// If no acat exists, disable the dropdown and show message
				else {
					this._acatSelect.prop("disabled", true);
					this._attrSelectDiv.empty();
					this._attrSelectDiv.append($("<span>").text(SearchInputs.t("NOATTCATEGORY")));
				}
				
			}, this));
		};
		
		
		/**
		 * Method to render the acat dropdown options
		 * and also load attributes multiselect for currently selected acat (will be the first option of the dropdown)
		 */
		AttributesSearchPopupView.prototype._renderACatSelectOptions = function() {
			for(var i=0; i<this._acatData.length; i++) {
				var option = $("<option>", {"value": this._acatData[i].id}).text(this._acatData[i].name);
				
				// If acatid is not defined, select first option and load attributes for first option
				// or
				// If acatid is defined, select that attribute category and load attributes for that option
				if((this._acatid && this._acatid == this._acatData[i].id) || (!this._acatid && i === 0)) {
					option.prop("selected", true);
					this._acatSelect.val(this._acatData[i].id);
					this._getAndRenderAttributes(this._acatData[i].id);
				}
				
				this._acatSelect.append(option);
			}
		};
		
		
		/**
		 * Method to get attributes for selected attribute category
		 * 
		 * @param acatid - id of selected attribute category
		 */
		AttributesSearchPopupView.prototype._getAndRenderAttributes = function(acatid) {
			
			/*
			 * Array to store the attributes information
			 * This object will be used to get all selected attributes on clicking "Add" button in popup
			 * 
			 * [
			 * 		{id: , label: , selected: },
			 * 		{id: , label: , selected: }
			 * ]
			 */
			this._availableAttrs = [];
			
			/*
			 * Ajax call to get attributes of selected attribute category
			 */
			$.when($.ajax({url: "GetAttributes.jsp", data: {"ajax": "1", "categoryid": acatid}, method: "POST", dataType: "xml"})).then(_.bind(function(attrXMLDoc, textStatus, jqXHR) {
				var attrXML = $(attrXMLDoc).children().get(0);
				
				// check the response if user session is expired, redirect to login page
				if(attrXML.tagName === "error") {
					var errorCode = $(attrXML).find('code').text();
					if(errorCode && '000' === errorCode) {
	            		document.location.reload();
	            	}
				}
				
				// If attributes exists, load the information
				if(attrXML.tagName == "attribute_category") {
					var attrNodes = null;
					
					// Get the atts node of xml (it has the attributes information)
					$.each($(attrXML).children(), _.bind(function(index, child) {
						if(child.tagName == "atts") {
							attrNodes = $(child).children();
							return false;
						}
					}, this));
					
					if(attrNodes) {
						// For each attribute, get id and name (label)
						for(var i=0; i<attrNodes.length; i++) {
							// Add the attribute information to this._availableAttrs object
							var attrid = $(attrNodes[i]).find("id").text();
							var attrname = $(attrNodes[i]).find("an").text();
							if(attrid && attrname) {
								this._availableAttrs.push({
									id: attrid,
									label: attrname,
									selected: false
								});
							}
						}
					}
					
					// Clear the attributes multiselect container
					this._attrSelectDiv.empty();
					
					// select the attributes in MultiSelectList if loading from saved search
					this._doAttributeSelection();
					
					// Render the multiselect for the attributes
					this._getAttributesContent(this._attrSelectDiv, this._availableAttrs);
					
				} 
				// Otherwise show a message
				else {
					this._attrSelectDiv.empty();
					this._attrSelectDiv.append($("<span>").text(SearchInputs.t("NOATTRSAVAIL")));
				}
			}, this));
		};
		
		
		/**
		 * Method to render multiselect for attributes
		 * 
		 * @param attrSelectDiv - html container/wrapper for creating multiselect
		 * @param data - attributes information object (this._availableAttrs object)
		 */
		AttributesSearchPopupView.prototype._getAttributesContent = function(attrSelectDiv, data) {
			var container = attrSelectDiv;
			var observer = {};
			Observer.mixin(observer);
	
			// High value to "disable" paging
			var maxRows = 5000;
	
			// The function used to get data for the library list. This could also
			// get data from the server
			var getLibraryData = function(pattern, page) {
				var dfr = $.Deferred();
				// variable for paging
				var index = 0;
	
				var regex = FilterUtils.generateRegex(pattern, true);
	
				dfr.resolve(_.filter(data, _.bind(function(object) {
					index++;
					if (regex.test(object.label) && !object.selected) {
						// Only get the not selected objects that are on the right
						// page
						return index >= (maxRows * (page - 1))
								&& index <= maxRows * page;
					}
				}, this)));
				
				return dfr.promise();
			};
	
			// get the selected data
			var getSelectedData = function(pattern, page) {
				var dfr = $.Deferred();
				// variable for paging
				var index = 0;
	
				var regex = FilterUtils.generateRegex(pattern, true);
	
				// First sort based on index, then page
				dfr.resolve(_.filter(_.sortBy(data, "index"), _.bind(function(object) {
					index++;
					if (regex.test(object.label) && object.selected) {
						// Only get the not selected objects that are on the right
						// page
						return index >= (maxRows * (page - 1))
								&& index <= maxRows * page;
					}
				}, this)));
				
				return dfr.promise();
			};
	
			// Create the multiselect list
			var multiSelect = new MultiSelectList(getLibraryData, getSelectedData,
					container, {
						notSelectedLabel : SearchInputs.t("AVAILABLE_ATTRIBUTES"),
						selectedLabel : SearchInputs.t("SELECTED_ATTRIBUTES"),
						selectAllTooltip : SearchInputs.t("SELECTALL_FILTERED"),
						deselectAllTooltip : SearchInputs.t("DESELECTALL_FILTERED"),
						availFilterPlaceholder: SearchInputs.t("ATTRS_AVAILABLE_FILTER"),
						selFilterPlaceholder: SearchInputs.t("ATTRS_SELECTED_FILTER")
					});
			
			/*
			 * Method to get a new index for selected item
			 */
			var getLastObjectIndex = function() {
				var lastIndex = -1;
				
				_.each(data, function(object, i) {
					if(object.index > lastIndex) {
						lastIndex = object.index;
					}
				});
				
				return lastIndex;
			};
			
			/*
			 * Method to remove index for deselected item and 
			 * Update the index for rest of the selected items with object index greater than deselected item index
			 */
			var removeObjectIndex = function(obj) {
				var removeIndex = obj.index;
				
				_.each(data, function(object, i) {
					if(object.index > removeIndex) {
						--object.index;
					}
				});
				
				delete obj.index;
			};
	
			// Handling of a entry selection
			observer._observe(multiSelect, "entrySelected", function(ids) {
				_.each(ids, function(id) {
					for(var i=0; i<data.length; i++) {
						if(id == data[i].id) {
							var obj = data[i];
							obj.selected = true;
							obj.index = getLastObjectIndex() + 1;
						}
					}
				}, this);
			});
	
			// Handling of a entry deselection
			observer._observe(multiSelect, "entryDeselected", function(ids) {
				_.each(ids, function(id) {
					for(var i=0; i<data.length; i++) {
						if(id == data[i].id) {
							var obj = data[i];
							obj.selected = false;
							removeObjectIndex(obj);
						}
					}
				}, this);
			});
	
			// Handling of all entries being selected
			observer._observe(multiSelect, "selectAll", function(pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				
				var objIndex = getLastObjectIndex();
				_.each(data, function(obj) {
					if (regex.test(obj.label) && !obj.selected) {
						obj.selected = true;
						obj.index = ++objIndex;
						return true;
					}
				});
			});
	
			// deselect all entries that have a label
			observer._observe(multiSelect, "deselectAll", function(pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				_.each(data, function(obj) {
					if (regex.test(obj.label) && obj.selected) {
						obj.selected = false;
						delete obj.index;
						return true;
					}
				});
			});
	
			// Filter-lists should get a height
			container.find(".filter-list .list-group").css({
				height : 300
			});
	
			multiSelect.render();
		};
			
		/**
		 * Mark the previously selected items as selected in "this._availableAttrs".
		 * Make sure to also fill in the "index" so that they stay in the same order...
		 * 
		 * Note that the items might not necessary be visible yet (due to paging functionality).
		 */
		AttributesSearchPopupView.prototype._doAttributeSelection = function() {
			// id => {id: , index: } for all selected items (in this list it might be that not all of them exist)
			var allSelectedAttrs = _.indexBy(_.map(this._attributeids, function(id, index) {
				return {id:id, index:index};
			}), _.property("id"));
			
			// filter the list above to contain only the existing ones
			// in the same order + get all the information for those attributes from "this._availableAttrs"
			var attrs = _.filter(this._availableAttrs, function(attr) {
				return allSelectedAttrs.hasOwnProperty(attr.id);
			}, this).sort(function(a, b) {return allSelectedAttrs[a.id].index-allSelectedAttrs[b.id].index;});
			
			// put them all as "selected" + enter the correct index
			_.each(attrs, function(e, i) {
				e.selected = true;
				e.index = i;
			}, this);
		};
		
		/**
		 * Create add and cancel buttons of the popup
		 * 
		 * @returns
		 */
		AttributesSearchPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'FooterButtons'});
			
			this._addButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("ADD")}).addClass("btn btn-primary");
			
			this._addButton.click(_.bind(function(event) {
				/*
				 * Object to store the selected attributes information
				 * 
				 * {
				 * 		categoryid: , 
				 * 		attributeids: []
				 * }
				 */
				var selAttrsData = {};
				
				// Set acatid to the object
				selAttrsData.categoryid = this._acatSelect.val();
				selAttrsData.attributeids = [];
				
				var data = this._availableAttrs || [];
				// The data should be sorted by index property which specifies order of selection
				_.filter(_.sortBy(data, "index"), function(option) {
					if(option && option.selected) {
						selAttrsData.attributeids.push(option.id);
					}
				});
				
				this._addClickHandler(event, selAttrsData);
				this._popup.close();
			}, this));
			
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CANCEL")}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function(event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));
			
			this._clearButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CLEARALL")}).addClass("btn btn-default add-spacing-right");
			this._clearButton.click(_.bind(function(event) {
				this._resetPopup();
			}, this));
			
			footerButtons.append(this._clearButton, this._cancelButton, this._addButton);
			
			return footerButtons;
		};
		
		/**
		 * Method to reset and open the popup
		 * Also initializes the date picker
		 */
		AttributesSearchPopupView.prototype.show = function (selAttrsData) {
			if(selAttrsData) {
				this._initPopup(selAttrsData);
			}
			
			this._popup.open();
		};
		
		/**
		 * Initialize the popup
		 */
		AttributesSearchPopupView.prototype._initPopup = function(selAttrsData) {
			// If some selected attributes exists, initialize the popup accordingly
			if(selAttrsData && !$.isEmptyObject(selAttrsData)) {
				if(selAttrsData.categoryid) {
					this._acatid = selAttrsData.categoryid;
				} else {
					this._acatid = null;
				}
				
				if(selAttrsData.attributeids && selAttrsData.attributeids.length > 0) {
					this._attributeids = selAttrsData.attributeids;
				} else {
					this._attributeids = [];
				}
				
				if(this._acatid) {
					this._acatSelect.val(this._acatid);
					this._getAndRenderAttributes(this._acatid);
				} else if(this._acatSelect.children().length > 0) {
					var firstOption = $("option:first", this._acatSelect);
					this._acatSelect.val(firstOption.attr("value"));
					this._getAndRenderAttributes(this._acatSelect.val());
				} else {
					this._getAttributeCategories();
				}
			} 
			// Otherwise, reset the popup
			else {
				this._resetPopup();
			}
		};
		
		AttributesSearchPopupView.prototype._resetPopup = function() {
			this._acatid = null;
			this._attributeids = [];
			this._acatSelect.empty();
			this._getAttributeCategories();
		};
		
		/**
		 * To add callback for add button
		 * @param callback
		 */
		AttributesSearchPopupView.prototype.onSubmit = function(callback) {
			this._onAddCallbacks.push(callback);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		AttributesSearchPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		return AttributesSearchPopupView;
	}
);

