define(["jquery",
        "underscore",
        "component/modal/Modal",
        "utils/webi18n!SearchInputs",
        "wcr/date",
        "wcr/popup",
        "wcr/webcenter_html",
        "wcr/webcenter_xml",
        "wcr/hover"], 
	function($,
			_,
			ModalPopup,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Popup to enter characteristics search inputs
		 * 
		 * @param searchtype [Object] - search type to preselect 
		 * @param chars [Object] - characteristics to preselect 
		 */
		var CharacteristicsSearchPopupView = function(searchtype, chars) {
			
			// Variables to initialize popup
			this._searchtype = searchtype;
			this._chars = chars;
			
			// Add and Cancel buttons
			this._addButton = null;
			this._cancelButton = null;
			
			// callbacks for add and cancel
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function(event, charsData) {
				_.each(this._onAddCallbacks, function(callback) {
					callback(charsData);
				});
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, function(callback) {
					callback();
				});
			}, this);
			
			
			this._constructPopup();
		};
		
		/**
		 * Construct the html structure of the popup
		 */
		CharacteristicsSearchPopupView.prototype._constructPopup = function() {
			// Create only one instance of popup.
			if (!this._popup) {
				// Get the popup html content
				this._popupContent = this._getPopupContent();
				
				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.setTitle(SearchInputs.t("CHARSEARCHTITLE"));
				this._popup.setFooter(this._getFooterContent());
			}
		};
		
		/**
		 * HTML content for popup body
		 * @returns
		 */
		CharacteristicsSearchPopupView.prototype._getPopupContent = function() {
			
			// popup html content
			var popupContent = $("<div>", {"class": "characteristics_search_popup"});
			
			this._createSearchTypeSelect(popupContent);
			
			this._createFilterInput(popupContent);
			
			this._createCharsSelect(popupContent);
			
			return popupContent;
		};
		
		
		CharacteristicsSearchPopupView.prototype._createSearchTypeSelect = function(popupContent) {
			/*
			 * add search type select dropdown
			 */
			this._searchTypeSelect = $("<select>", {"name": "search_type"});
			
			var orOption = $("<option>", {"value": "OR"}).text(SearchInputs.t("SEARCHCHARCONTAINSANY"));
			var andOption = $("<option>", {"value": "AND"}).text(SearchInputs.t("SEARCHCHARCONTAINSALL"));
			
			this._searchTypeSelect.append(orOption, andOption);
			
			if(this._searchtype && this._searchtype == "OR") {
				orOption.prop("selected", true);
			} else if(this._searchtype && this._searchtype == "AND") {
				andOption.prop("selected", true);
			}
			
			var searchTypeContent = $("<div>", {"class": "add-spacing-bottom"}).append(
					$("<span>", {"class": "add-spacing-right"}).text(SearchInputs.t("CHARSEARCHTITLE_PART1")),
					this._searchTypeSelect,
					$("<span>", {"class": "add-spacing-left"}).text(SearchInputs.t("CHARSEARCHTITLE_PART2"))
			);
			
			popupContent.append(searchTypeContent);
		};
		
		
		CharacteristicsSearchPopupView.prototype._createFilterInput = function(popupContent) {
			/*
			 * add find characteristics filter
			 */
			this._filterInput = $("<input>", {"name": "char_finder_input", "type": "text", "class": "form-control", "placeholder": SearchInputs.t("CHAR_FINDER_TITLE")});
			this._filterInput.keyup(_.bind(function(event) {
				// Get the filter keyword
				var filterRxp = $(event.target).attr("value");
				
				// Store the char id and name to revert the changes later using undo button
				this._filteredChars = [];
				
				// If some filter value is entered, hide characteristics tree and show list and apply filter
				if (filterRxp) {
					this._charTree.hide();
					this._charList.show();
					
					// Apply filter on the list
					$(".list_char", this._charList).each(_.bind(function(i, element) {
						try {
							var charid = ($(element).attr("id")).substring(5);
							var charname = ($(element).attr("name")).substring(5);
							if (charname && WCHTML.passesFilter(charname, filterRxp)) {
								$(element).show();
								this._filteredChars.push({"id": charid, "name": charname});
							} else {
								$(element).hide();
							}
						} catch (err) {  
							//some error during filtering(like during parsing bad ID value), rather do nothing
						}
					}, this));
				} 
				// Show characteristics tree and hide list
				else {
					this._charTree.show();
					this._charList.hide();
				}	
			}, this));
			
			/*
			 * Method to add an option if doesn't exists
			 */
			this._addSelectedOption = _.bind(function(charid, charname) {
				var existingOption = $("option[value='"+ charid +"']", this._selectedCharsInput);
				if(!existingOption.length) {
					// Add an option to selected characteristics
					var option = $("<option>", {"value": charid}).text(charname).prop("selected", true);
					this._selectedCharsInput.append(option);
				}
			}, this);
			
			/*
			 * Method to remove an option if exists
			 */
			this._removeSelectedOption = _.bind(function(charid) {
				var existingOption = $("option[value='"+ charid +"']", this._selectedCharsInput);
				if(existingOption.length) {
					// Remove the option from selected characteristics
					existingOption.remove();
				}
			}, this);
			
			// button to add all filtered characteristics
			this._selectAllButton = $("<input>", {"name": "filterbuttons", "type": "button", "value": SearchInputs.t("ADD"), "class": "btn btn-default btn-no-icon"});
			this._selectAllButton.click(_.bind(function(event) {
				if(this._filterInput.val()) {
					// Show the undo button
					this._undoButton.show();
					
					for(var i=0; i<this._filteredChars.length; i++) {
						var charid = this._filteredChars[i].id;
						var charname = this._filteredChars[i].name;
						
						// Add option to the selected input
						this._addSelectedOption(charid, charname);
						
						// Mark checkbox of characteristic in the tree hierarchy as checked
						$("#TreeCheck" + charid, this._charTree).prop("checked", true);
						// Mark checkbox of characteristic in the filter list as checked
						$("#ListCheck" + charid, this._charList).prop("checked", true);
					}
					
					// Hide the list, Show the tree and reset the filter input
					this._charList.hide();
					this._charTree.show();
					this._filterInput.val("");
				}
			}, this));
			
			// button to undo all filtered characteristics
			this._undoButton = $("<input>", {"name": "filterbuttons", "type": "button", "value": SearchInputs.t("CHAR_FINDER_UNDO"), "class": "btn btn-default btn-no-icon"}).hide();
			this._undoButton.click(_.bind(function(event) {
				// Hide the undo button
				this._undoButton.hide();
				
				for(var i=0; i<this._filteredChars.length; i++) {
					var charid = this._filteredChars[i].id;
					
					// Remove option from the selected input
					this._removeSelectedOption(charid);
					
					// Mark checkbox of characteristic in the tree hierarchy as unchecked
					$("#TreeCheck" + charid, this._charTree).prop("checked", false);
					// Mark checkbox of characteristic in the filter list as unchecked
					$("#ListCheck" + charid, this._charList).prop("checked", false);
				}
				
				// Reset the filtered characteristics
				this._filteredChars = [];
				// Hide the list, Show the tree and reset the filter input
				this._charList.hide();
				this._charTree.show();
				this._filterInput.val("");
			}, this));
			
			var filterContent = $("<div>", {"class": "add-spacing-bottom"}).append(
					$("<div>", {"class": "input-group"}).append(
							this._filterInput,
							$("<div>", {"class": "input-group-btn"}).append(this._selectAllButton, this._undoButton)
					)
			);
			
			popupContent.append(filterContent);
		};
		
		
		CharacteristicsSearchPopupView.prototype._createCharsSelect = function(popupContent) {
			/*
			 * Display characteristics
			 */
			this._charsContentDiv = $("<div>", {"class": "add-scroll-to-tree"});
			
			// Ajax call to load characteristics
			$.when($.ajax({url: "GetCharacteristicsTree.jsp", method: "POST", dataType: "xml"})).then(_.bind(function(data, textStatus, jqXHR) {
				
				/*
				 * XML structure of the characteristics 
				 * 
				 * <char_tree>
				 * 		<elem>
				 * 			<name>char1</name>
				 * 			<id>00002_0000000001</id>
				 * 			<children>
				 * 				<elem>
				 * 					<name>char1a</name>
				 * 					<id>00002_0000000006</id>
				 * 				</elem>
				 * 				<elem>
				 * 					<name>char1b</name>
				 * 					<id>00002_0000000007</id>
				 * 				</elem>
				 * 			</children>
				 * 		</elem>
				 * 		<elem>
				 * 			<name>char3</name>
				 * 			<id>00002_0000000003</id>
				 * 			<children>
				 * 				<elem>
				 * 					<name>char3a</name>
				 * 					<id>00002_0000000009</id>
				 * 				</elem>
				 * 				<elem>
				 * 					<name>char3b</name>
				 * 					<id>00002_0000000010</id>
				 * 						<children>
				 * 						<elem>
				 * 							<name>abcdefghijklmnopqrstuvwxyz0123456789</name>
				 * 							<id>00002_0000000201</id>
				 * 						</elem>
				 * 					</children>
				 * 				</elem>
				 * 			</children>
				 * 		</elem>
				 * </char_tree>
				 */
				
				// Get the <char_tree> xml dom from document object
				this._charsXML = $(data).children();
				this._charsContentDiv.empty();
				
				// create the characteristics tree hierarchy
				this._charTree = $("<div>", {"name": "characteristics_tree"});
				this._getCharInputs(this._charsXML, this._charTree, false);
				this._charsContentDiv.append(this._charTree);
				
				// create the characteristics list (for filter)
				this._charList = $("<div>", {"name": "characteristics_list"}).hide();
				this._getCharInputs(this._charsXML, this._charList, true);
				this._charsContentDiv.append(this._charList);
			}, this));
			
			
			this._selectedCharsInput = $("<select>", {"class": "form-control", "name": "charSelect"}).attr({"size": "15"});
			
			var charsSelectContent = $("<div>").append(
					$("<div>", {"class": "row"}).append(
							$("<div>", {"class": "col-xs-12 col-sm-6"}).append(this._charsContentDiv),
							$("<div>", {"class": "col-xs-12 col-sm-6"}).append(this._selectedCharsInput)
					)
			);
			
			popupContent.append(charsSelectContent);
		};
		
		
		/**
		 * Recursive method to create the characteristics hierarchy and list (for filter)
		 * 
		 * @param charsXML
		 * @param parentDiv
		 * @param isList
		 */
		CharacteristicsSearchPopupView.prototype._getCharInputs = function(charsXML, parentDiv, isList) {
			
			// Get <elem> nodes of the xml
			var charElems = $(charsXML).children();

							
			this._toggleIconClickHandler = _.bind(function(event) {
				var icon = $(event.target);
				if (icon.hasClass("icon-WCR_Plus-square12")) {
					icon.removeClass("icon-WCR_Plus-square12").addClass(
							"icon-WCR_Minus-square12");
					$("#" + "ChildChar" + (icon.attr("id")).substring(3),
							this._charTree).show();
				} else {
					icon.removeClass("icon-WCR_Minus-square12").addClass(
							"icon-WCR_Plus-square12");
					$("#" + "ChildChar" + (icon.attr("id")).substring(3),
							this._charTree).hide();
				}
			}, this);

			this._checkBoxInputClickHandler = _.bind(function(event) {
				var checkbox = $(event.target);

				var chardiv = null;
				var charid = null;
				var charname = null;

				// If the checkbox is from filter list, sync the checkbox
				// from tree hierarchy
				if (checkbox.hasClass("charlist_checkbox")) {
					chardiv = $("#List_"
							+ (checkbox.attr("id")).substring(9),
							this._charList);
					charid = (chardiv.attr("id")).substring(5);
					charname = (chardiv.attr("name")).substring(5);

					// If checkbox is checked add an option to selected
					// chars input
					if (checkbox.prop("checked")) {
						this._addSelectedOption(charid, charname);
						$("#TreeCheck" + charid, this._charTree).prop(
								"checked", true);
					}
					// If checkbox is unchecked remove the option from
					// selected chars input
					else {
						this._removeSelectedOption(charid);
						$("#TreeCheck" + charid, this._charTree).prop(
								"checked", false);
					}

				}
				// If the checkbox is from tree hierarchy, sync the checkbox
				// from filter list
				else {
					chardiv = $("#" + (checkbox.attr("id")).substring(9),
							this._charTree);
					charid = chardiv.attr("id");
					charname = chardiv.attr("name");

					if (checkbox.prop("checked")) {
						this._addSelectedOption(charid, charname);
						$("#ListCheck" + charid, this._charList).prop(
								"checked", true);
					} else {
						this._removeSelectedOption(charid);
						$("#ListCheck" + charid, this._charList).prop(
								"checked", false);
					}
				}

			}, this);

			// For each <elem>
			for(var i=0; i<charElems.length; i++) {
				var charElem = charElems[i];
				
				// Get the <id>, <name> and <children> nodes if any
				var childElems = $(charElem).children();
				var childCharElems = null;
				var charId = null;
				var charName = null;
				for(var j=0; j<childElems.length; j++) {
					var child = childElems[j];
					
					if(child.tagName == "id") {
						charId = $(child).text();
					} else if(child.tagName == "name") {
						charName = $(child).text();
					} else if(child.tagName == "children") {
						childCharElems = child;
					}
					
					if(charId && charName && childCharElems) {
						break;
					}
				}
				
				// Div for a characteristic (has id and name attribute)
				var charDiv = $("<div>");
				if(!isList) {
					charDiv.attr({"id": charId, "name": charName}).addClass("tree_char");
				} else {
					charDiv.attr({"id": "List_" + charId, "name": "List_" + charName}).addClass("list_char");
				}
				
				var charNodeDiv = $("<div>").addClass("char-node");
				
				// If the characteristic is a leaf node of the tree hierarchy or incase of filter list > add class child-node (do not add expand/collapse icon)
				if(!childCharElems || isList) {
					charNodeDiv.addClass("child-node");
				} 
				// Add expand/collapse icon if the characteristic has child characteristics in tree hierarchy
				else {
					var toggleIcon = $("<i>", {"id": "Img" + charId, "class": "icon icon-normal icon-WCR_Plus-square12"});
					toggleIcon.click(this._toggleIconClickHandler);
					
					charNodeDiv.append(toggleIcon);
				}
				
				var labelDiv = $("<div>", {"class": "labeldiv"});
				
				var preSelectChar = this._isDoPreSelectChar(charId);
				
				// checkbox to select/deselect the characteristic
				var checkBoxInput = $("<input>", {"type": "checkbox", "value": charId});
				if(!isList) {
					checkBoxInput.attr({"id": "TreeCheck" + charId}).addClass("chartree_checkbox");
				} else {
					checkBoxInput.attr({"id": "ListCheck" + charId}).addClass("charlist_checkbox");
				}
				
				if(preSelectChar) {
					checkBoxInput.prop("checked", true);
					this._addSelectedOption(charId, charName);
				}
				
				
				checkBoxInput.click(this._checkBoxInputClickHandler);
				
				// Get the name of the characteristic
				var charNameSpan = this._getCharNameContent(charName);
				
				labelDiv.append($("<table>").append($("<tr>").append(
						$("<td>").append(checkBoxInput),
						$("<td>").append(charNameSpan)
				)));
				
				charNodeDiv.append(labelDiv);
				charDiv.append(charNodeDiv);
				
				// If child characteristics exists, make a recursive call to build the tree hierarchy
				if(childCharElems && $(childCharElems).children().length > 0) {
					if(!isList) {
						var childDiv = $("<div>", {"id": "ChildChar" + charId, "class": "add-spacing-left add-padding-left"}).hide();
						charDiv.append(childDiv);
						this._getCharInputs(childCharElems, childDiv, false);
					} else {
						this._getCharInputs(childCharElems, parentDiv, true);
					}
				}
				
				parentDiv.append(charDiv);
			}
			
		};
		
		/**
		 * Method to create the characteristic name
		 * 
		 * @param charName
		 * @returns
		 */
		CharacteristicsSearchPopupView.prototype._getCharNameContent = function(charName) {
			var charNameSpan = $("<span>");
			
			if(charName.length < 20) {
				charNameSpan.text(charName);
			} else {
				charNameSpan.append($("<span>").text(charName.substring(0, 19)));
				var cutCharName = $("<span>").text("(...)");
				
				cutCharName.mouseout(_.bind(function(event) {
					delayhidepopup();
				}, this));
				
				cutCharName.mouseover(_.bind(function(event) {
					showmenu(event, charName);
					$("#popitmenu").css({"z-index": "10000", "word-break": "break-all"});
				}, this));
				
				charNameSpan.append(cutCharName);
			}
			
			return charNameSpan;
		};
		
		
		CharacteristicsSearchPopupView.prototype._isDoPreSelectChar = function(charId) {
			var preSelect = false;
			
			if(this._chars && this._chars.length > 0) {
				for(var i=0; i<this._chars.length; i++) {
					if(this._chars[i].id == charId) {
						preSelect = true;
						break;
					}
				}
			}
			
			return preSelect;
		};
		
		/**
		 * Create add and cancel buttons of the popup
		 * @returns
		 */
		CharacteristicsSearchPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'ActiveButtons'});
			
			this._addButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("ADD")}).addClass("btn btn-primary");
			this._addButton.click(_.bind(function(event) {
				var charsData = {};
				
				// Add the search type
				charsData.searchtype = this._searchTypeSelect.val();
				charsData.chars = [];
				
				// Add the selected characteristics
				var options = this._selectedCharsInput.children();
				for(var i=0; i<options.length; i++) {
					var option = options[i];
					charsData.chars.push({"id": $(option).attr("value"), "name": $(option).text()});
				}
				
				this._addClickHandler(event, charsData);
				this._popup.close();
			}, this));
			
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CANCEL")}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function(event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));
			
			this._clearAllButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CLEARALL")}).addClass("btn btn-default add-spacing-right");
			this._clearAllButton.click(_.bind(function(event) {
				this._resetPopup();
			}, this));
			
			footerButtons.append(this._clearAllButton, this._cancelButton, this._addButton);
			
			return footerButtons;
		};
		
		/**
		 * Method to initialize and open the popup
		 */
		CharacteristicsSearchPopupView.prototype.show = function (charsData) {
			// Check if user is logged in
			$.when($.ajax({url: "checkifloggedin.jsp", method: "POST", dataType: "xml", data: {"ajax": "1"}})).always(_.bind(function(data, textStatus, jqXHR) {
				var xml = $(data).children().get(0);
				if(xml && xml.tagName === "error") {
					var errorCode = $(xml).find("code").text();
					if(errorCode && errorCode === "001") {
						document.location.reload();
					}
				} else {
					this._initPopup(charsData);
					this._popup.open();
				}
			}, this));
		};
		
		/**
		 * Method to initialize
		 */
		CharacteristicsSearchPopupView.prototype._initPopup = function (charsData) {
			
			// Reset the popup
			this._resetPopup();
			
			// Initialize search type select
			var searchtype = charsData.searchtype;
			if(searchtype) {
				this._searchTypeSelect.val(searchtype);
				$("option[value='" + searchtype + "']", this._searchTypeSelect).prop("selected", true);
			}
			
			// Initialize characteristics
			var chars = charsData.chars;
			if(chars) {
				for(var i=0; i<chars.length; i++) {
					var char = chars[i];
					
					// Add an option to selected characteristics list
					this._addSelectedOption(char.id, char.name);
					
					// Mark checkbox of characteristic in the tree hierarchy as checked
					$("#TreeCheck" + char.id, this._charTree).prop("checked", true);
					// Mark checkbox of characteristic in the filter list as checked
					$("#ListCheck" + char.id, this._charList).prop("checked", true);
				}
				
				this._addButton.attr({"value": SearchInputs.t("CHANGE")});
			}
			
			
		};
		
		CharacteristicsSearchPopupView.prototype._resetPopup = function () {
			if(this._popup && this._charTree && this._charList) {
				this._charTree.show();
				this._charList.hide();
				this._filterInput.val("");
				this._filteredChars = [];
				this._selectedCharsInput.empty();
				this._undoButton.hide();
				this._searchTypeSelect.val("OR");
				$("option[value='OR']", this._searchTypeSelect).prop("selected", true);
				$("option[value='AND']", this._searchTypeSelect).prop("selected", false);
				$(".chartree_checkbox", this._charTree).prop("checked", false);
				$(".charlist_checkbox", this._charList).prop("checked", false);
				this._addButton.attr({"value": SearchInputs.t("ADD")});
			}
		};
		
		/**
		 * To add callback for add button
		 * @param callback
		 */
		CharacteristicsSearchPopupView.prototype.onSubmit = function(callback) {
			this._onAddCallbacks.push(callback);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		CharacteristicsSearchPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		return CharacteristicsSearchPopupView;
	}
);

