define(["jquery",
        "underscore",
        "component/modal/Modal",
        "utils/webi18n!SearchInputs",
        "wcr/date",
        "wcr/webcenter_xml"], 
	function($,
			_,
			ModalPopup,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Popup to enter date search inputs when more_options is selected
		 * 
		 * @param options [Object] - options 
		 */
		var DateSearchPopupView = function(showFutureContext, showLabel, fieldId) {
			
			this._showFutureContext = showFutureContext == "true" ? true : false;
			this._showLabel = showLabel;
			this._fieldId = fieldId;
			
			// Add and Cancel buttons
			this._addButton = null;
			this._cancelButton = null;
			
			// callbacks for add and cancel
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function(event, optionData) {
				_.each(this._onAddCallbacks, _.bind(function(callback) {
					callback(optionData);
				}, this));
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			
			this._constructPopup();
		};
		
		/**
		 * Construct the html structure of the popup
		 */
		DateSearchPopupView.prototype._constructPopup = function() {
			
			// Create only one instance of popup.
			if(!this._popup) {
				// Get the popup html content
				this._popupContent = this._getPopupContent();
				
				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.setTitle(SearchInputs.t("ADVSEARCH_MOREOPTIONS"));
				this._popup.setFooter(this._getFooterContent());
			}
		};
		
		DateSearchPopupView.prototype._getPopupContent = function() {
			
			// popup html content
			var popupContent = $('<div>', {"name": "date_search_popup", "class": "wcr-form form-no-labels"});
			
			// error block
			this._errorMsg = $("<div>").attr({"name": "error_msg"});
			popupContent.append(this._errorMsg);
			
			/*
			 * Error message html blocks
			 */
			
			this._negativeNumErrorMsg = this._getErrorContent("NEGATIVEINTEGERRESTRICTIONERROR");
				
			this._nonNumericErrorMsg = this._getErrorContent("NUMERICRESTRICTIONERROR");
			
			this._invalidStartDateMsg = this._getErrorContent("INVALIDSTARTDATE");
			
			this._invalidEndDateMsg = this._getErrorContent("INVALIDENDDATE");
			
			this._endDateBeforeStartDateMsg = this._getErrorContent("STARTENDDATE");
			
			this._invalidStartAndEndDateMsg = this._getErrorContent("INVALIDSTARTANDENDDATE");
			
			/*
			 * Method to validate value while entering in input field
			 */
			this._validatePartialInt = function(element, allowNegative) {
				var value = element.value;
				// Blank field is allowed
				if (value === "") {
                    return true;
                }
					
				// The user has only entered a negative sign so far
				if (allowNegative && (value == "-")) {
                    return true;
                }
				
				var checkNum = parseInt(value);
				if (isNaN(checkNum)) {
					// They did not enter an integer...
					element.value = "";
					return false;
				}
				if (checkNum != value || value.indexOf(".") != -1) {
					// They entered something with a number at the front -
					// e.g. 12Y
					
					// They entered something with a decimal point.
					// JavaScript thinks 100.0 is an int but Java
					// on the server disagrees.
					element.value = checkNum;
					return false;
				}
				if (!allowNegative && (checkNum < 0)) {
					element.value = checkNum.toString().substring(1);
					return false;
				}
				return true;
			};
			
			
			/*
			 * Method to validate value when adding finally
			 */
			this._validateInt = function(element) {
				var value = element.value;
				
				// Blank field is allowed
				if(value === "") {
					return true;
				}
					
				var checkNum = parseInt(value);
				if(isNaN(checkNum) || checkNum != value || value.indexOf(".") != - 1) {
					// They did not enter an integer...
					// They entered something with a number at the front -
					// e.g. 12Y
					// They entered something with a decimal point.
					// JavaScript thinks 100.0 is an int but Java
					// on the server disagrees.
					return false;
				}
				
				return true;
			};
			
			
			/*
			 * Method to format date to "MM/dd/yyyy" format
			 */
			this._getFormattedDate = function(dateText) {
				var dateVal = null;
				if(dateText) {
					var date = new Date(dateText);
					dateVal = formatDate(date, "MM/dd/yyyy");
				}
				return dateVal;
			};
			
			
			/*
			 * Method to execute when add button is clicked
			 */
			this._doAdd = _.bind(function(event) {
				var isSubmit = false;
				
				var optionData = {};
				
				// Create option data for last_n_days
				if(this._lastndaysradio.prop("checked")) {
					isSubmit = this._doAddLastNDays();
					if(isSubmit) {
						optionData.operator = "last_n_days";
						optionData.value = this._lastndaysinput.attr("value");
					}
					
				} 
				// Create option data for next_n_days
				else if(this._nextndaysradio && this._nextndaysradio.prop("checked")) {
					isSubmit = this._doAddNextNDays();
					if(isSubmit) {
						optionData.operator = "next_n_days";
						optionData.value = this._nextndaysinput.attr("value");
					}
					
				} 
				// Create option data for date_range
				else if (this._daterangeradio.prop("checked")) {
					isSubmit = this._doAddDateRange();
					if(isSubmit) {
						optionData.operator = "date_range";
						var fromDateValue = this._getFormattedDate(this._fromdateinput.attr("value"));
						if(fromDateValue) {
							optionData.fromdate = {};
							optionData.fromdate.day = fromDateValue.substring(3,5);
							optionData.fromdate.month = fromDateValue.substring(0,2);
							optionData.fromdate.year = fromDateValue.substring(6);
						}
						
						var toDateValue = this._getFormattedDate(this._todateinput.attr("value"));
						if(toDateValue) {
							optionData.todate = {};
							optionData.todate.day = toDateValue.substring(3,5);
							optionData.todate.month = toDateValue.substring(0,2);
							optionData.todate.year = toDateValue.substring(6);
						}
					}
				}
				
				// Execute add button callbacks and close the popup
				if(isSubmit) {
					this._addClickHandler(event, optionData);
					this._popup.close();
				}
			}, this);
			
			/*
			 * Method to validate input entered for last_n_days while adding
			 */
			this._doAddLastNDays = _.bind(function() {
				var dayCount = this._lastndaysinput.attr("value");
				// Check for non-empty value
				if(isEmpty(dayCount)) {
					this._errorMsg.append(this._nonNumericErrorMsg);
					return false;
				}
				// Check for positive integer
				if(!this._validateInt(this._lastndaysinput.get(0))) {
					this._errorMsg.append(this._negativeNumErrorMsg);
					return false;
				}
				
				return true;
			}, this);
			
			/*
			 * Method to validate input entered for next_n_days while adding
			 */
			this._doAddNextNDays = _.bind(function() {
				var dayCount = this._nextndaysinput.attr("value");
				// Check for non-empty value
				if(isEmpty(dayCount)) {
					this._errorMsg.append(this._nonNumericErrorMsg);
					return false;
				}
				// Check for positive integer
				if(!this._validateInt(this._nextndaysinput.get(0))) {
					this._errorMsg.append(this._negativeNumErrorMsg);
					return false;
				}
				
				return true;
			}, this);
			
			/*
			 * Method to validate input entered for date_range while adding
			 */
			this._doAddDateRange = _.bind(function() {
				
				/*
				 * To use a Date() object for comparison we have to realize that this object takes
				 * a range of values for its month from 0-11, not 1-12.
				 * 
				 * So, a Date() for Jan 14th 2013 should be created as follows - myDate.setFullYear(2013,0,14);
				 * The date returned from the object on this screen will be in the format 01/14/2013
				 * 
				 * Hence, we need to adjust the month before using it to create the Date() object, and subsequently
				 * using that object in the date comparison. Otherwise you will get strange results.
				 * 
				 */
				/**
				 * To use a Date() object for comparison we have to realize that this object takes
				 * a range of values for its month from 0-11, not 1-12.
				 * 
				 * Also prepend a '0' for months (0 - 9) as a string
				 * 
				 * @param {String} - month
				 */
				var adjustMonthForDateObject = function(month) {
					// Parse the month string
					var monthNum = parseInt(month);
					
					if(!isNaN(monthNum)) {
						// the month should be in range 0-11 and not 1-12
						var adjustedMonth = monthNum - 1;
						
						var adjustedMonthStr = '' + adjustedMonth;
						
						// Prepend a '0' character before the month if it is less than 10
						if (adjustedMonth < 10) {
							adjustedMonthStr = '0' + adjustedMonth;
						}
						
						// Return as string
						return adjustedMonthStr;
					}
				};
				
				// startDateTime and/or endDateTime have their value set to the string 'Invalid Date'
				// if anything untoward is entered in the respective text field.
				var startDate = this._getFormattedDate(this._fromdateinput.attr("value"));
				var startDateTime = null;
				if(startDate) {
					startDateTime = new Date(startDate.substring(6), adjustMonthForDateObject(startDate.substring(0,2)), startDate.substring(3,5));
					
					if (startDateTime!= null && startDateTime == 'Invalid Date') {
						this._errorMsg.append(this._invalidStartDateMsg);
						return false;
					}
				}
				
				
				var endDate = this._getFormattedDate(this._todateinput.attr("value"));
				var endDateTime = null;
				if(endDate) {
					endDateTime = new Date(endDate.substring(6), adjustMonthForDateObject(endDate.substring(0,2)), endDate.substring(3,5));
					
					if (endDateTime!= null && endDateTime == 'Invalid Date') {
						this._errorMsg.append(this._invalidEndDateMsg);
						return false;
					}
				}
				
				if (startDateTime != null && endDateTime != null && startDateTime > endDateTime) {
					this._errorMsg.append(this._endDateBeforeStartDateMsg);
					return false;
				} else if(!startDateTime && !endDateTime) {
					this._errorMsg.append(this._invalidStartAndEndDateMsg);
					return false;
				}
				
				return true;
			}, this);
			
			
			/*
			 * Append last_n_days field to popup
			 */
			popupContent.append(this._constructLastNDaysField());
			
			/*
			 * Append last_n_days field to popup
			 */
			if(this._showFutureContext) {
				popupContent.append(this._constructNextNDaysField());
			}
			
			/*
			 * Append last_n_days field to popup
			 */
			popupContent.append(this._constructDateRangeField());
			
			
			return popupContent;
		};
		
		
		DateSearchPopupView.prototype._getErrorContent = function(labelText) {
			var error = $("<div>").addClass("alert alert-danger alert-dismissible").append(
					$("<button>").addClass("close").attr({"data-dismiss": "alert", "aria-label": "Close"}).append($("<i>").addClass("icon icon-small icon-WCR_Close12")),
					$("<p>").text(SearchInputs.t(labelText))
			);
			
			return error;
		};
		
		/**
		 * Construct html content for last_n_days field
		 * 
		 * @returns
		 */
		DateSearchPopupView.prototype._constructLastNDaysField = function() {
			/*
			 * Create last_n_days radio and input field
			 */
			this._lastndaysradio = $("<input>").attr({"type": "radio", "name": "lastndays"}).prop("checked", true);
			this._lastndaysradio.click(_.bind(function(event) {
				// Remove error message
				this._errorMsg.empty();
				
				// Enable input
				this._lastndaysinput.prop("disabled", false);
				
				if(this._showFutureContext) {
					// Disable and uncheck next_n_days field
					this._nextndaysinput.prop("disabled", true);
					this._nextndaysradio.prop("checked", false);
				}
				
				// Disable and uncheck date_range field
				this._fromdateinput.prop("disabled", true);
				this._todateinput.prop("disabled", true);
				this._daterangeradio.prop("checked", false);
			}, this));
			
			this._lastndaysinput = $("<input>").attr({"type": "text", "name": "last_n_days", "size": "2"}).prop("disabled", false);
			this._lastndaysinput.keyup(_.bind(function(event) {
				var isValidInput = this._validatePartialInt(event.target, false);
				
				if(!isValidInput) {
					this._errorMsg.append(this._negativeNumErrorMsg);
				} else {
					this._errorMsg.empty();
				}
			}, this));
			
			var lastndaysfield = $("<div>").addClass("form-group").append(
					$("<label>").addClass("offset-label"),
					$("<div>").addClass("form-entry").append(
							$("<div>").addClass("radio with-form-control").append(
									$("<label>").append(
											this._lastndaysradio,
											$("<span>").text(this._getLastNDaysLabel(this._showLabel))
									),
									this._lastndaysinput,
									$("<span>").addClass("add-spacing-left").text(SearchInputs.t("ADVSEARCH_DAYSUNIT"))
							)
					)
			);
			
			return lastndaysfield;
		};
		
		/**
		 * Construct html content for next_n_days field
		 * 
		 * @returns
		 */
		DateSearchPopupView.prototype._constructNextNDaysField = function() {
			/*
			 * Create next_n_days radio and input field
			 */
			this._nextndaysradio = $("<input>").attr({"type": "radio", "name": "nextndays"}).prop("checked", false);
			this._nextndaysradio.click(_.bind(function(event) {
				// Remove error message
				this._errorMsg.empty();
				
				// Enable input
				this._nextndaysinput.prop("disabled", false);
				
				// Disable and uncheck last_n_days field
				this._lastndaysinput.prop("disabled", true);
				this._lastndaysradio.prop("checked", false);
				
				// Disable and uncheck date_range field
				this._fromdateinput.prop("disabled", true);
				this._todateinput.prop("disabled", true);
				this._daterangeradio.prop("checked", false);
			}, this));
			
			this._nextndaysinput = $("<input>").attr({"type": "text", "name": "last_n_days", "size": "2"}).prop("disabled", true);
			this._nextndaysinput.keyup(_.bind(function(event) {
				var isValidInput = this._validatePartialInt(event.target, false);
				
				if(!isValidInput) {
					this._errorMsg.append(this._negativeNumErrorMsg);
				} else {
					this._errorMsg.empty();
				}
			}, this));
			
			var nextndaysfield = $("<div>").addClass("form-group").append(
					$("<label>").addClass("offset-label"),
					$("<div>").addClass("form-entry").append(
							$("<div>").addClass("radio with-form-control").append(
									$("<label>").append(
											this._nextndaysradio,
											$("<span>").text(this._getNextNDaysLabel(this._showLabel))
									),
									this._nextndaysinput,
									$("<span>").addClass("add-spacing-left").text(SearchInputs.t("ADVSEARCH_DAYSUNIT"))
							)
					)
			);
			
			return nextndaysfield;
		};
		
		/**
		 * Construct html content for date_range field
		 * 
		 * @returns
		 */
		DateSearchPopupView.prototype._constructDateRangeField = function() {
			/*
			 * Create date_range radio and from date and to date input fields
			 */
			this._daterangeradio = $("<input>").attr({"type": "radio", "name": "daterange"}).prop("checked", false);
			this._daterangeradio.click(_.bind(function(event) {
				// Hide error message
				this._errorMsg.empty();
				
				// Enable input
				this._fromdateinput.prop("disabled", false);
				this._todateinput.prop("disabled", false);
				
				// Disable and uncheck last_n_days field
				this._lastndaysinput.prop("disabled", true);
				this._lastndaysradio.prop("checked", false);
				
				if(this._showFutureContext) {
					// Disable and uncheck next_n_days field
					this._nextndaysinput.prop("disabled", true);
					this._nextndaysradio.prop("checked", false);
				}
			}, this));
			
			this._fromdateinput = $("<input>").attr({"type": "text", "id": this._fieldId + "_fromdatefield"}).addClass("form-control").prop("disabled", true);
			this._fromdateinput.change(_.bind(function() {
				// Hide error message
				this._errorMsg.empty();
			}, this));
			
			this._todateinput = $("<input>").attr({"type": "text", "id": this._fieldId + "_todatefield"}).addClass("form-control").prop("disabled", true);
			this._todateinput.change(_.bind(function() {
				// Hide error message
				this._errorMsg.empty();
			}, this));
			
			var daterangefield = $("<div>").addClass("form-group").append(
					$("<label>").addClass("offset-label"),
					$("<div>").addClass("form-entry").append(
							$("<table>").append($("<tr>").append(
									$("<td>").addClass("td-nowrap add-padding-right").append($("<div>").addClass("radio").append(
											$("<label>").append(
													this._daterangeradio,
													$("<span>").text(this._getDateRangeLabel(this._showLabel))
											)
									)),
									$("<td>").append($("<div>").addClass("input-group").append(
											this._fromdateinput,
											$("<label>").attr({"for": this._fieldId + "_fromdatefield"}).addClass("form-datetime-btn input-group-addon").append($("<i>").addClass("icon icon-WCR_Calendar"))
									)),
									$("<td>").append($("<span>").addClass("add-spacing-left add-spacing-right").text(SearchInputs.t("ANDBETWEENDATES"))),
									$("<td>").append($("<div>").addClass("input-group").append(
											this._todateinput,
											$("<label>").attr({"for": this._fieldId + "_todatefield"}).addClass("form-datetime-btn input-group-addon").append($("<i>").addClass("icon icon-WCR_Calendar"))
									))
							))
					)
			);
			
			return daterangefield;
		};
		
		
		/**
		 * Create add and cancel buttons of the popup
		 * @returns
		 */
		DateSearchPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'ActiveButtons'});
			
			this._addButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("ADVSEARCH_ADDDATE")}).addClass("btn btn-primary");
			this._addButton.click(_.bind(function(event) {
				this._doAdd(event);
			}, this));
			
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CANCEL")}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function(event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));
			
			footerButtons.append(this._cancelButton, this._addButton);
			
			return footerButtons;
		};
		
		/**
		 * Method to reset and open the popup
		 * Also initializes the date picker
		 */
		DateSearchPopupView.prototype.show = function () {
			this._resetPopup();
			this._popup.open();
			
			require(["jqueryUI"], _.bind(function(){
				$(this._fromdateinput).datepicker({dateFormat: "M dd, yy"});
				$(this._todateinput).datepicker({dateFormat: "M dd, yy"});
			}, this));
		};
		
		/**
		 * Reset the popup
		 */
		DateSearchPopupView.prototype._resetPopup = function() {
			this._errorMsg.empty();
			this._lastndaysradio.prop("checked", true);
			this._lastndaysinput.attr({"value": ""}).prop("disabled", false);
			
			if(this._showFutureContext) {
				this._nextndaysradio.prop("checked", false);
				this._nextndaysinput.attr({"value": ""}).prop("disabled", true);
			}
			
			this._daterangeradio.prop("checked", false);
			this._fromdateinput.attr({"value": ""}).prop("disabled", true);
			this._todateinput.attr({"value": ""}).prop("disabled", true);
		};
		
		/**
		 * Option label for last_n_days option
		 * 
		 * @param showLabel
		 * @returns {String}
		 */
		DateSearchPopupView.prototype._getLastNDaysLabel = function(showLabel) {
			var label = "";
			
			switch(showLabel) {
			case "show_create_date_labels":
				label = SearchInputs.t("ADVSEARCH_CREATEDLASTNDAYS");
				break;
			case "show_mod_date_labels":
				label = SearchInputs.t("ADVSEARCH_MODIFIEDLASTNDAYS");
				break;
			case "show_due_date_labels":
				label = SearchInputs.t("ADVSEARCH_DUELASTNDAYS");
				break;
			case "show_start_date_labels":
				label = SearchInputs.t("ADVSEARCH_STARTEDLASTNDAYS");
				break;
			case "show_estimated_date_labels":
				label = SearchInputs.t("ADVSEARCH_ESTIMATEDLASTNDAYS");
				break;
			case "show_completed_date_labels":
				label = SearchInputs.t("ADVSEARCH_COMPLETEDLASTNDAYS");
				break;
			default:
				label = showLabel + " " + SearchInputs.t("ADVSEARCH_LAST");
				break;
			}
			
			return label;
		};
		
		/**
		 * Option label for next_n_days option
		 * 
		 * @param showLabel
		 * @returns {String}
		 */
		DateSearchPopupView.prototype._getNextNDaysLabel = function(showLabel) {
			var label = "";
			
			switch(showLabel) {
			case "show_create_date_labels":
				label = SearchInputs.t("ADVSEARCH_CREATEDNEXTNDAYS");
				break;
			case "show_mod_date_labels":
				label = SearchInputs.t("ADVSEARCH_MODIFIEDNEXTNDAYS");
				break;
			case "show_due_date_labels":
				label = SearchInputs.t("ADVSEARCH_DUENEXTNDAYS");
				break;
			case "show_start_date_labels":
				label = SearchInputs.t("ADVSEARCH_STARTEDNEXTNDAYS");
				break;
			case "show_estimated_date_labels":
				label = SearchInputs.t("ADVSEARCH_ESTIMATEDNEXTNDAYS");
				break;
			case "show_completed_date_labels":
				label = SearchInputs.t("ADVSEARCH_COMPLETEDNEXTNDAYS");
				break;
			default:
				label = showLabel + " " + SearchInputs.t("ADVSEARCH_NEXT");
				break;
			}
			
			return label;
		};
		
		
		/**
		 * Option label for date_range option
		 * 
		 * @param showLabel
		 * @returns {String}
		 */
		DateSearchPopupView.prototype._getDateRangeLabel = function(showLabel) {
			var label = "";
			
			switch(showLabel) {
			case "show_create_date_labels":
				label = SearchInputs.t("CREATEDBETWEEN");
				break;
			case "show_mod_date_labels":
				label = SearchInputs.t("MODIFIEDBETWEEN");
				break;
			case "show_due_date_labels":
				label = SearchInputs.t("DUEBETWEEN");
				break;
			case "show_start_date_labels":
				label = SearchInputs.t("STARTEDBETWEEN");
				break;
			case "show_estimated_date_labels":
				label = SearchInputs.t("ESTIMATEDBETWEEN");
				break;
			case "show_completed_date_labels":
				label = SearchInputs.t("COMPLETEDBETWEEN");
				break;
			default:
				label = showLabel + " " + SearchInputs.t("BETWEEN");
				break;
			}
			
			return label;
		};
		
		/**
		 * To add callback for add button
		 * @param callback
		 */
		DateSearchPopupView.prototype.onSubmit = function(callback) {
			this._onAddCallbacks.push(callback);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		DateSearchPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		return DateSearchPopupView;
	}
);

