define(["jquery",
        "underscore",
        "component/modal/Modal",
        "./InkSearchRowView",
        "utils/webi18n!SearchInputs",
        "wcr/webcenter_xml"], 
	function($,
			_,
			ModalPopup,
			InkSearchRowView,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Constructor
		 * 
		 * Popup view to enter ink search inputs
		 */
		var InkSearchPopupView = function() {
			
			// Add and Cancel buttons
			this._addButton = null;
			this._cancelButton = null;
			
			// callbacks for add and cancel
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function(event, inksData) {
				_.each(this._onAddCallbacks, _.bind(function(callback) {
					callback(inksData);
				}, this));
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			// Create the popup
			this._constructPopup();
		};
		
		/**
		 * Create the modal popup
		 */
		InkSearchPopupView.prototype._constructPopup = function() {
			
			// Create only one instance of popup.
			if(!this._popup) {
				// Get the popup html content
				this._popupContent = this._getPopupContent();
				
				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.setTitle(SearchInputs.t("SEARCH_INKS"));
				this._popup.setFooter(this._getFooterContent());
			}
		};
		
		/**
		 * Construct the html structure of the popup
		 * @returns
		 */
		InkSearchPopupView.prototype._getPopupContent = function() {
			
			// popup html content
			var popupContent = $('<div>', {"class": "ink_search_popup"});
			
			// error block
			this._errorMsg = $("<div>").attr({"name": "error_msg"});
			popupContent.append(this._errorMsg);
			
			/*
			 * Error message html blocks
			 */
			this._negativeNumErrorMsg = this._getErrorContent("NEGATIVEINTEGERRESTRICTIONERROR");
			
			/*
			 * Method to validate the number entered in input fields
			 */
			this._validateNonNegativeInt = function(element) {
				var value = $(element).val();
				
				// Blank field is allowed
				if (value === "") {
                    return true;
                }
				
				var checkNum = parseInt(value);
				
				if (isNaN(checkNum) || checkNum != value || checkNum < 0 || value.indexOf(".") != -1) {
					// They did not enter an integer...
					// They entered something with a number at the front - e.g. 12Y
					// they cannot enter a negative value
					// They entered something with a decimal point.
					// JavaScript thinks 100.0 is an int but Java on the server disagrees.
					return false;
				}
				
				return true;
			};
			
			/*
			 * Create the CMYKOnly option
			 */
			this._cmykOnlyOption = $("<div>", {"name": "InkSearchOption", "value": "CMYKInkSearch", "class": "btn btn-default"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_CMYKONLY"));
			
			this._cmykOptionHandler = _.bind(function(event) {
				this._setActiveOption(this._cmykOnlyOption);
				this._clearCustomOptionFields();
				$(".customoptionfields", this._popupContent).prop("disabled", true);
				$("a[name='customoptionbuttons']", this._popupContent).hide();
				$(".addinkbuttondiv", this._popupContent).hide();
			}, this);
			
			this._cmykOnlyOption.click(this._cmykOptionHandler);
			
			/*
			 * Create the Custom option
			 */
			this._customOption = $("<div>", {"name": "InkSearchOption", "value": "CustomInkSearch", "class": "btn btn-default"}).text(SearchInputs.t("GRAPHICS_INKCRITERIA_CUSTOM"));
			
			this._customOptionHandler = _.bind(function(event) {
				this._setActiveOption(this._customOption);
				$(".customoptionfields", this._popupContent).prop("disabled", false);
				$("a[name='customoptionbuttons']", this._popupContent).show();
				$(".addinkbuttondiv", this._popupContent).show();
				this._detachInkRowDeleteButton();
			}, this);
			
			this._customOption.click(this._customOptionHandler);
			
			
			/*
			 * Div to show the CMYKOnly option and Custom option
			 */
			var optionsContentDiv = $("<div>", {"class": "form-group"}).append(
					$("<label>", {"class": "offset-label"}),
					$("<div>", {"class": "form-entry"}).append(
							$("<div>", {"class": "btn-group searchtogglegroup"}).append(this._cmykOnlyOption, this._customOption)
					)
			);
			
			/*
			 * Div to show fields under Custom option
			 */
			this._customContentFields = $("<div>", {"name": "customink_fields", "class": "customink_fields"});
			
			/*
			 * Div to show Custom option fields
			 */
			var customOptionFields = $("<div>", {"class": "form-group"}).append(
					$("<label>", {"class": "offset-label"}),
					$("<div>", {"class": "form-entry"}).append(this._customContentFields));
			
			// Construct the number of inks and number of printing inks fields
			this._customContentFields.append(this._getNumberInkFields());
			
			// Construct the ink information fields
			this._customContentFields.append(this._getInkTableFields());
			
			// Append the CMYKOnly and Custom options to the popup body
			popupContent.append(optionsContentDiv, customOptionFields);
			
			return popupContent;
		};
		
		/**
		 * Construct error html content
		 * 
		 * @param labelText
		 * @returns
		 */
		InkSearchPopupView.prototype._getErrorContent = function(labelText) {
			var error = $("<div>").addClass("alert alert-danger alert-dismissible").append(
					$("<button>").addClass("close").attr({"data-dismiss": "alert", "aria-label": "Close"}).append($("<i>").addClass("icon icon-small icon-WCR_Close12")),
					$("<p>").text(SearchInputs.t(labelText))
			);
			
			return error;
		};
		
		/**
		 * Construct number of inks and number of printing inks fields
		 * @returns
		 */
		InkSearchPopupView.prototype._getNumberInkFields = function() {
			
			var numberFieldsDiv = $("<div>", {"name": "number_of_inks"});
			
			// Number of inks select dropdown for mode (=, >=, <=)
			this._numberOfInksSelect = $("<select>", {"class": "form-control customoptionfields"});
			this._numberOfInksSelect.append($("<option>", {"value": "EQUALS"}).text("="));
			this._numberOfInksSelect.append($("<option>", {"value": "GREATERTHAN"}).text("≥"));
			this._numberOfInksSelect.append($("<option>", {"value": "LESSTHAN"}).text("≤"));
			
			// Field to enter non-negative number integer for number of inks
			this._numberOfInksInput = $("<input>", {"type": "text", "id": "number_inks", "name": "number_inks", "size": "6", "class": "form-control customoptionfields"});
			this._numberOfInksInput.on("input propertychange change", _.bind(function(event) {
				if(!this._validateNonNegativeInt(event.target)) {
					this._errorMsg.append(this._negativeNumErrorMsg);
					this._numberOfInksInput.val("");
				} else {
					this._errorMsg.empty();
				}
			}, this));
			
			
			// Number of printing inks select dropdown for mode (=, >=, <=)
			this._numberOfPrintingInksSelect = $("<select>", {"class": "form-control customoptionfields"});
			this._numberOfPrintingInksSelect.append($("<option>", {"value": "EQUALS"}).text("="));
			this._numberOfPrintingInksSelect.append($("<option>", {"value": "GREATERTHAN"}).text("≥"));
			this._numberOfPrintingInksSelect.append($("<option>", {"value": "LESSTHAN"}).text("≤"));
			
			// Field to enter non-negative number integer for number of printing inks
			this._numberOfPrintingInksInput = $("<input>", {"type": "text", "id": "number_printing_inks", "name": "number_printing_inks", "size": "6", "class": "form-control customoptionfields"});
			this._numberOfPrintingInksInput.on("input propertychange change", _.bind(function(event) {
				if(!this._validateNonNegativeInt(event.target)) {
					this._errorMsg.append(this._negativeNumErrorMsg);
					this._numberOfPrintingInksInput.val("");
				} else {
					this._errorMsg.empty();
				}
			}, this));
			
			numberFieldsDiv.append($("<table>", {"class": "number_of_inks"}).append(
					$("<tr>").append(
							$("<td>", {"class": "add-padding-right td-label td-nowrap"}).append($("<label>", {"for": "number_inks"}).append(SearchInputs.t("GRAPHICS_INKCRITERIA_NRINKS"))),
							$("<td>").append(this._numberOfInksSelect),
							$("<td>").append(this._numberOfInksInput)
					),
					$("<tr>").append(
							$("<td>", {"class": "add-padding-right td-label td-nowrap"}).append($("<label>", {"for": "number_printing_inks"}).append(SearchInputs.t("GRAPHICS_INKCRITERIA_NRPRINTINGINKS"))),
							$("<td>").append(this._numberOfPrintingInksSelect),
							$("<td>").append(this._numberOfPrintingInksInput)
					)
				));
			
			
			return numberFieldsDiv;
		};
		
		/**
		 * Construct html for adding ink information
		 * @returns
		 */
		InkSearchPopupView.prototype._getInkTableFields = function() {
			
			var tableFieldsDiv = $("<div>", {"name": "ink_table"});
			
			// Add a button to create new Ink row
			this._addInkButton = $("<a>", {"class": "btn btn-default"}).append(
					$("<i>", {"class": "icon icon-WCR_Actions-Add"}), $("<span>").text(SearchInputs.t("GRAPHICS_INKCRITERIA_ADDINK")));
			
			// Method to create new ink row when Add Ink button is clicked
			this._addInkRowHandler = _.bind(function(event) {
				// Create a new ink row
				var inkRow = new InkSearchRowView();
				inkRow.onDelete(this._inkRowOnDeleteHandler);
				this._inkTableBody.append(inkRow.getInkRowJQElement());
				this._inkRows.push(inkRow);
				
				// If only one row exists, detach the delete button
				this._detachInkRowDeleteButton();
				
				return inkRow;
			}, this);
			
			this._addInkButton.click(this._addInkRowHandler);
			
			tableFieldsDiv.append($("<h4>").text(SearchInputs.t("GRAPHICS_SEARCH_JOBCONTAINSINKS") + ":"));
			
			this._inkTableBody = $("<tbody>");
			
			tableFieldsDiv.append($("<div>", {"class": "ink_rows_table"}).append($("<table>", {"class": "table"}).append(
					$("<thead>").append($("<tr>").append(
							$("<th>").text(SearchInputs.t("GRAPHICS_INKCRITERIA_INKNAME")),
							$("<th>").text(SearchInputs.t("GRAPHICS_INKCRITERIA_INKGROUP")),
							$("<th>").text(SearchInputs.t("GRAPHICS_INKCRITERIA_INKTYPE")),
							$("<th>")
					)),
					this._inkTableBody
			)));
			
			tableFieldsDiv.append($("<div>", {"class": "addinkbuttondiv"}).append(this._addInkButton));
			
			// Array to hold the ink row (InkSearchRowView) objects
			this._inkRows = [];
			
			// Method to delete a ink row
			this._inkRowOnDeleteHandler = _.bind(function(event, thisInkRow) {
				var objIndex = -1;
				
				$.each(this._inkRows, _.bind(function (index, inkRow) {
					if (inkRow === thisInkRow) {
						objIndex = index;
						return false;
					}
				}, this));
				
				// Remove the ink row (InkSearchRowView) object from the this._inkRows array
				if(objIndex > -1) {
					this._inkRows.splice(objIndex, 1);
				}
				
				// Remove the ink row from the html DOM
				thisInkRow.getInkRowJQElement().remove();
				
				// Detach the delete button if only one row exists
				this._detachInkRowDeleteButton();
			}, this);
			
			/*
			 * Method to detach/attach the delete button from html DOM
			 * if only one row exists detach delete button
			 * else attach the delete button
			 */
			this._detachInkRowDeleteButton = _.bind(function() {
				if(this._inkTableBody.children().length === 1) {
					_.each(this._inkRows, function(inkRow, index) {
						inkRow.detachDeleteButton();
					});
					
				} else if(this._inkTableBody.children().length > 1) {
					_.each(this._inkRows, function(inkRow, index) {
						inkRow.attachDeleteButton();
					});
				}
			}, this);
			
			// Create a ink row initially
			this._addInkRowHandler();
			
			return tableFieldsDiv;
		};
		
		/**
		 * Create add and cancel buttons of the popup
		 * @returns
		 */
		InkSearchPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'ActiveButtons'});
			
			// Button to add the ink information on Search setup page
			this._addButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("ADD")}).addClass("btn btn-primary");
			this._addButton.click(_.bind(function(event) {
				
				/*
		    	 * Object to store the ink information
		    	 * {
		    	 * 		ink_search_mode: "CMYKONLY",
		    	 * 		number_inks: "3",
		    	 * 		number_inks_operator: "LESSTHAN",
		    	 * 		number_printing_inks: "5",
		    	 * 		number_printing_inks_operator: "EQUALS",
		    	 * 		ink_rows: [
		    	 * 					{ink_name: "Green", ink_group: "Any", ink_type: "Regular"},
		    	 * 					{ink_name: "Red", ink_group: "Designer", ink_type: "Varnish"}
		    	 * 				]
		    	 * }
		    	 */
				var inksData = {};
				
				if(this._cmykOnlyOption.hasClass("active")) {
					inksData.ink_search_mode = "CMYKONLY";
					
				} else if(this._customOption.hasClass("active")) {
						inksData.ink_search_mode = "CUSTOM";
						
						if(this._numberOfInksInput.val()) {
							inksData.number_inks = this._numberOfInksInput.val();
							inksData.number_inks_operator = this._numberOfInksSelect.val();
						}
						
						if(this._numberOfPrintingInksInput.val()) {
							inksData.number_printing_inks = this._numberOfPrintingInksInput.val();
							inksData.number_printing_inks_operator = this._numberOfPrintingInksSelect.val();
						}
						
						if(this._inkRows && this._inkRows.length > 0) {
							for(var i=0; i<this._inkRows.length; i++) {
								var inkRowData = this._inkRows[i].getInkRowData();
								
								if(inkRowData) {
									if(!inksData.ink_rows) {
										inksData.ink_rows = [];
									}
									
									inksData.ink_rows.push(inkRowData);
								}
							}
						}
					}
				
					this._addClickHandler(event, inksData);
					this._popup.close();
			}, this));
			
			// Button to clear all fields
			this._resetButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("GRAPHICS_INKCRITERIA_RESET")}).addClass("btn btn-default add-spacing-right");
			this._resetButton.click(_.bind(function(event) {
				this._resetPopup();
			}, this));
			
			// Button to cancel the popup
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CANCEL")}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function(event) {
				this._cancelClickHandler(event);
				this._popup.close();
			}, this));
			
			footerButtons.append(this._resetButton, this._cancelButton, this._addButton);
			
			return footerButtons;
		};
		
		/**
		 * Method to reset/initalize and open the popup
		 */
		InkSearchPopupView.prototype.show = function (inksData) {
			// Initialize the popup (incase of existing information from saved search)
			this._initPopup(inksData);
			// Open the modal popup
			this._popup.open();
		};
		
		/**
		 * Initialize the popup
		 */
		InkSearchPopupView.prototype._initPopup = function(inksData) {
			// If ink information exists, initialize the fields
			if(!$.isEmptyObject(inksData)) {
				// Set the option to CMYKOnly
				if(inksData.ink_search_mode == "CMYKONLY") {
					this._cmykOptionHandler();
					
				} 
				// Set the option to Custom
				else if(inksData.ink_search_mode == "CUSTOM") {
					this._setActiveOption(this._customOption);
					
					// populate the number of inks fields and dropdown
					if(inksData.number_inks && this._numberOfInksSelect) {
						this._numberOfInksSelect.prop("disabled", false);
						this._numberOfInksSelect.val(inksData.number_inks_operator);
						this._numberOfInksInput.prop("disabled", false);
						this._numberOfInksInput.val(inksData.number_inks);
					}
					
					// populate the number of printing inks fields and dropdown
					if(inksData.number_printing_inks && this._numberOfPrintingInksSelect) {
						this._numberOfPrintingInksSelect.prop("disabled", false);
						this._numberOfPrintingInksSelect.val(inksData.number_printing_inks_operator);
						this._numberOfPrintingInksInput.prop("disabled", false);
						this._numberOfPrintingInksInput.val(inksData.number_printing_inks);
					}
					
					this._inkTableBody.empty();
					this._inkRows = [];
					
					// populate the ink row information
					if(inksData.ink_rows && inksData.ink_rows.length > 0) {
						for(var i=0; i<inksData.ink_rows.length; i++) {
							var inkRowData = inksData.ink_rows[i];
							var inkRow = this._addInkRowHandler();
							inkRow.setInkRowData(inkRowData.ink_name, inkRowData.ink_group, inkRowData.ink_type);
						}
					} else {
						this._addInkRowHandler();
					}
				}
				
			} 
			// Reset the popup
			else {
				this._resetPopup();
			}
		};
		
		/**
		 * Reset the popup
		 */
		InkSearchPopupView.prototype._resetPopup = function() {
			// Select the custom option
			this._setActiveOption(this._customOption);
			
			// clear the fields under custom option
			this._clearCustomOptionFields();
			
			// Enable all fields under custom option
			this._customOptionHandler();
		};
		
		/**
		 * Clear the fields under custom option
		 */
		InkSearchPopupView.prototype._clearCustomOptionFields = function() {
			// reset the number of inks and number of printing inks
			this._numberOfInksSelect.val("EQUALS");
			this._numberOfInksInput.val("");
			this._numberOfPrintingInksSelect.val("EQUALS");
			this._numberOfPrintingInksInput.val("");
			
			// clear the inks table and add a single row
			this._inkTableBody.empty();
			this._inkRows = [];
			this._addInkRowHandler();
		};
		
		/**
		 * Set active option (CMYK Only/Custom)
		 */
		InkSearchPopupView.prototype._setActiveOption = function(option) {
			this._cmykOnlyOption.removeClass("active");
			this._customOption.removeClass("active");
			option.addClass("active");
		};
		
		/**
		 * To add callback for add button
		 * @param callback
		 */
		InkSearchPopupView.prototype.onSubmit = function(callback) {
			this._onAddCallbacks.push(callback);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		InkSearchPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		return InkSearchPopupView;
	}
);

