define(["jquery",
        "underscore",
        "component/modal/Modal",
        "component/multiSelect/view/MultiSelectList",
        "wcr/FilterUtils",
        "utils/Observer",
        "utils/webi18n!SearchInputs",
        "wcr/webcenter_xml"], 
	function($,
			_,
			ModalPopup,
			MultiSelectList,
			FilterUtils,
			Observer,
			SearchInputs) {
		
		"use strict";
		
		/**
		 * Popup to select specifications based on task types
		 */
		var SpecificationsSearchPopupView = function(specsData, isAdmin) {
			
			this._tasktypeid = null;
			this._specificationids = [];
			this._filterbytasktype = true;
			
			if(specsData && !$.isEmptyObject(specsData)) {
				if(specsData.categoryid) {
					this._tasktypeid = specsData.categoryid;
				}
				
				if(specsData.attributeids && specsData.attributeids.length > 0) {
					this._specificationids = specsData.attributeids;
				}
				
				if(_.isBoolean(specsData.filterbytasktype) && specsData.filterbytasktype) {
					this._filterbytasktype = true;
				}
			}
			
			this._isAdminUser = isAdmin;
			
			// Add and Cancel buttons
			this._addButton = null;
			this._cancelButton = null;
			
			// callbacks for add and cancel
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
			
			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function(event, selSpecsData) {
				_.each(this._onAddCallbacks, _.bind(function(callback) {
					callback(selSpecsData);
				}, this));
			}, this);
			
			this._cancelClickHandler = _.bind(function(event) {
				_.each(this._onCancelCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			
			// Construct the popup
			this._constructPopup();
		};
		
		/**
		 * Create the popup
		 */
		SpecificationsSearchPopupView.prototype._constructPopup = function() {
			
			// Create only one instance of popup.
			if(!this._popup) {
				// Get the popup html content
				this._popupContent = this._getPopupContent();
				
				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.setTitle(SearchInputs.t("SEARCHSPECIFICATIONPROMPT"));
				this._popup.setFooter(this._getFooterContent());
			}
		};
		
		/**
		 * Construct the html structure of the popup
		 * 
		 * @returns
		 */
		SpecificationsSearchPopupView.prototype._getPopupContent = function() {
			
			// popup html content
			var popupContent = $('<div>').attr({"name": "specifications_search_popup"});
			
			// Container to render the specifications (multiselectlist)
			this._specsSelectDiv = $("<div>", {"name": "specifications_container"});
			
			// Show the Task Type select and filter by specs checkbox only for admin user
			if(this._isAdminUser) {
				
				/**
				 * Method to get the selected specifications when changing task type or enabling filter by specs checkbox
				 */
				this._setNewSpecIds = _.bind(function() {
					var newselections = [];
					
					// Get the selected attributes in same order
					var selSpecsDiv = $("div[name='selecteditems']", this._specsSelectDiv);
					if(selSpecsDiv) {
						var selSpecsList = $(".list-group-item", selSpecsDiv);
						
						if(selSpecsList.length > 0) {
							for(var i=0; i<selSpecsList.length; i++) {
								newselections.push($(selSpecsList[i]).attr("data-id"));
							}
						}
					}
					
					this._specificationids = newselections;
				}, this);
				
				
				// Dropdown to select task type
				this._taskTypeSelect = $("<select>", {"name": "CascadingTaskType", "class": "form-control"});
				this._taskTypeSelect.change(_.bind(function(event) {
					this._setNewSpecIds();
					
					// Clear the attributes html container
					this._specsSelectDiv.empty();
					
					// Get and render the specifications for selected task type
					this._getAndRenderSpecifications($(event.target).val());
				}, this));
				
				
				var taskTypeSection = $("<div>", {"class": "add-spacing-bottom"}).append(
						$("<label>").append(SearchInputs.t("SEARCH_TASKSPECIFICATIONS_CASCADING")),
						this._taskTypeSelect,
						$("<p>", {"class": "help-block"}).text(SearchInputs.t("SEARCH_TASKSPECIFICATIONS_CASCADING_HINT"))
				);
				
				// Append the acat dropdown to the popup body
				popupContent.append(taskTypeSection);
				
				// Checkbox to filter task specifications by selected Task Type
				this._filterTaskSpecs = $("<input>", {"type": "checkbox", "name": "CascadingTaskTypeFilter"}).prop("checked", false).css({"margin-top": "2px"});
				this._filterTaskSpecs.change(_.bind(function(event) {
					this._setNewSpecIds();
					this._getAndRenderSpecifications(this._taskTypeSelect.val());
				}, this));
				
				// By default enable the checkbox
				if(!this._filterbytasktype) {
					this._filterTaskSpecs.prop("checked", false);
				} else {
					this._filterTaskSpecs.prop("checked", true);
				}
				
				var filterCheckbox = $("<div>", {"class": "checkbox"}).append($("<label>").append(
						this._filterTaskSpecs,
						$("<span>").text(SearchInputs.t("SEARCH_TASKSPECIFICATIONS_CASCADING_FILTER")).css({"font-weight": "500"})
				));
				
				popupContent.append(filterCheckbox);
			} else {
				this._getAndRenderSpecifications();
			}
			
			// Append the container to popup body
			popupContent.append(this._specsSelectDiv);
			
			if(this._isAdminUser) {
				// Get the task types and populate the task type select options
				this._getTaskTypes();
			} else {
				this._getAndRenderSpecifications();
			}
			
			return popupContent;
		};
		
		
		/**
		 * Method the get task types using Ajax call 
		 * and populate the options of task type select dropdown
		 */
		SpecificationsSearchPopupView.prototype._getTaskTypes = function() {
			/*
			 * Array to hold the task type information
			 * 
			 * [
			 * 		{id: , name: },
			 * 		{id: , name: }
			 * ]
			 */
			this._taskTypesData = [];
			
			/*
			 * Get the task types using ajax call,
			 * populate the options of select dropdown of task types
			 * and render multi select for specifications in currently selected task type
			 */
			$.when($.ajax({url: "GetSearchSpecificationsTaskTypes.jsp", method: "POST", dataType: "xml"})).then(_.bind(function(taskTypesXMLDoc, textStatus, jqXHR) {
				
				var taskTypesXML = $(taskTypesXMLDoc).children().get(0);
				
				// check the response if user session is expired, redirect to login page
				if(taskTypesXML.tagName === "error") {
					var errorCode = $(taskTypesXML).find('code').text();
					if(errorCode && '000' === errorCode) {
	            		document.location.reload();
	            	}
				}
				
				var searchIndexWorkflows = $(taskTypesXML).find("search_index_workflows").get(0);
				
				var isSearchIndexWorkflows = $(searchIndexWorkflows).text() == "true" ? true : false;
				
				var taskTypeList = $(taskTypesXML).find("wc_taskType_List").get(0);
				
				// If attribute categories exists load information into this._acatData object
				if(taskTypeList) {
					var taskTypes = $(taskTypeList).find("wctask_type");
					
					for(var i=0; i<taskTypes.length; i++) {
						var taskType = taskTypes[i];
						
						var hasworkflow = $(taskType).find("has_workflow").text() === "1" ? true : false;
						
						this._taskTypesData.push({
							id : $(taskType).attr("id"),
							name : $(taskType).find("task_type_name").text(),
							hasWorkflow : hasworkflow
						});
					}
					
					// Render the options of task types select dropdown
					this._renderTaskTypeSelectOptions(isSearchIndexWorkflows);
				} 
				// If no task type exists, disable the dropdown and load only the specifications
				else {
					this._taskTypeSelect.prop("disabled", true);
					this._getAndRenderSpecifications();
				}
				
			}, this));
		};
		
		
		/**
		 * Method to render the task types dropdown options
		 * and also load specifications multiselect for currently selected task type
		 */
		SpecificationsSearchPopupView.prototype._renderTaskTypeSelectOptions = function(isSearchIndexWorkflows) {
			// clear the dropdown
			this._taskTypeSelect.empty();
			
			var defaultOption = $("<option>", {"value": ""}).text(SearchInputs.t("SEARCH_TASKSPECIFICATIONS_CASCADING_ANY")).prop("selected", true);
			this._taskTypeSelect.append(defaultOption);
			this._taskTypeSelect.val("");
			
			for(var i=0; i<this._taskTypesData.length; i++) {
				var tasktype = this._taskTypesData[i];
				
				if(!tasktype.hasWorkflow || (tasktype.hasWorkflow && isSearchIndexWorkflows)) {
					var option = $("<option>", {"value": this._taskTypesData[i].id}).text(this._taskTypesData[i].name);
					this._taskTypeSelect.append(option);
					
					// If tasktypeid is defined, select that task type and load specifications for that option
					if(this._tasktypeid && this._tasktypeid == this._taskTypesData[i].id) {
						option.prop("selected", true);
						this._taskTypeSelect.val(this._taskTypesData[i].id);
					}
				}
			}
			
			this._getAndRenderSpecifications(this._taskTypeSelect.val());
		};
		
		
		/**
		 * Method to get specifications for selected task type only if filter checkbox is checked
		 * 
		 * @param tasktypeid - id of selected task type
		 */
		SpecificationsSearchPopupView.prototype._getAndRenderSpecifications = function(taskTypeId) {
			
			/*
			 * Array to store the specifications information
			 * This object will be used to get all selected specifications on clicking "Add" button in popup
			 * 
			 * [
			 * 		{id: , label: , selected: },
			 * 		{id: , label: , selected: }
			 * ]
			 */
			this._availableSpecs = [];
			
			// Array to hold consolidated specifications information 
			// for currently selected Task Type and if some pre-selected specs are unavailable
			this._loadedSpecs = [];
			
			if(!this._availableAllSpecs) {
				this._availableAllSpecs = [];
			} else if(this._allSpecsCached) {
				// Copy the cached all specs (some specs might have selected and set to true for selected property)
				this._availableAllSpecs = $.extend(true, [], this._allSpecsCached);
			}
			
			/*
			 * Load specification for below conditions
			 * 1. For a specific task type (filter option is checked)
			 * or
			 * 2. All specifications (only load if they are not loaded before)
			 * 
			 */
			if(this._availableAllSpecs.length === 0 || (taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked"))) {
				
				$.when(this._getSpecifications(taskTypeId)).then(_.bind(function(specifications) {
					
					// Copy the retrived specs
					this._loadedSpecs = $.extend(true, [], specifications);
					
					// Incase of saved search there might be other specs selected which does not belong to the selected task type
					// Find these specs (since we don't have their names, also we need to retain these selections)
					var specsIdsWithNoName = [];
					
					for(var i=0; i<this._specificationids.length; i++) {
						var isExists = false;
						for(var j=0; j<specifications.length; j++) {
							if(this._specificationids[i] == specifications[j].id) {
								isExists = true;
								break;
							}
						}
						
						if(!isExists) {
							specsIdsWithNoName.push(this._specificationids[i]); 
						}
					}
					
					/**
					 * Method to load the multiselectlist
					 */
					var loadMultiSelect = _.bind(function(specsData) {
						// If attributes exists, load multiselectlist
						if(specsData.length > 0) {
							
							if(taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked")) {
								this._availableSpecs = $.extend(true, [], specsData);
							} else {
								this._availableAllSpecs = $.extend(true, [], specsData);
							}
							
							// Clear the specifications multiselect container
							this._specsSelectDiv.empty();
							
							// Render the multiselect for the attributes
							if(taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked")) {
								this._getSpecificationsContent(this._specsSelectDiv, this._availableSpecs);
							} else {
								if(!this._allSpecsCached) {
									this._allSpecsCached = $.extend(true, [], this._availableAllSpecs);
								}
								this._getSpecificationsContent(this._specsSelectDiv, this._availableAllSpecs);
							}
						} 
						// Otherwise show a message
						else {
							// Clear the specifications multiselect container
							this._specsSelectDiv.empty();
							this._specsSelectDiv.append($("<span>").text(SearchInputs.t("NOSPECIFICATIONSAVAILABLE")));
						}
					},this);
					
					
					// If some specification ids exists without their names
					if(specsIdsWithNoName.length > 0) {
						/**
						 * Method to consolidate the specs of currently selected task type AND the specs which were selected from other task type(s)
						 */
						var onSpecsLoaded = _.bind(function(loadedspecs, specsIdsWithNoName) {
							for(var index=0; index<specsIdsWithNoName.length; index++) {
								$.each(loadedspecs, _.bind(function(i, spec) {
									if(specsIdsWithNoName[index] == spec.id) {
										this._loadedSpecs.push(spec);
									}
								}, this));
							}
						}, this);
						
						
						// If all specs are already loaded, get the names of specs
						if(this._availableAllSpecs.length > 0) {
							onSpecsLoaded(this._availableAllSpecs, specsIdsWithNoName);
							loadMultiSelect(this._loadedSpecs);
							
						} 
						// Otherwise, get all specifications and then get the names of specs
						else {
							$.when(this._getSpecifications()).then(_.bind(function(loadedSpecsData) {
								onSpecsLoaded(loadedSpecsData, specsIdsWithNoName);
								loadMultiSelect(this._loadedSpecs);
							}, this));
						}
					} 
					// Otherwise, just load the multiselect.
					else {
						loadMultiSelect(this._loadedSpecs);
					}
					
				}, this));
				
			} else if(this._availableAllSpecs.length > 0 && !(taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked"))) {
				// Clear the specifications multiselect container
				this._specsSelectDiv.empty();
				
				// Render the multiselect for the attributes
				this._getSpecificationsContent(this._specsSelectDiv, this._availableAllSpecs);
			}
			
		};
		
		
		/**
		 * Method to load the all specs or specs for a task type using AJAX call
		 * 
		 * @param taskTypeId
		 * @returns promise {Object} - promise object
		 */
		SpecificationsSearchPopupView.prototype._getSpecifications = function(taskTypeId) {
			var deferred = $.Deferred();
			
			var ajaxdata = {};
			var ajaxurl = null;
			
			if(taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked")) {
				ajaxurl = "GetSearchSpecificationsForTaskType.jsp";
				ajaxdata.tasktypeid = taskTypeId;
				
			} else {
				ajaxurl = "GetAttributeListXML.jsp";
				ajaxdata.type = "task";
			}
			
			/*
			 * Ajax call to get specifications of selected task type
			 */
			$.when($.ajax({url: ajaxurl, data: ajaxdata, method: "POST", dataType: "xml"})).then(_.bind(function(specsXMLDoc, textStatus, jqXHR) {
				var specsXML = $(specsXMLDoc).children().get(0);
				
				var specifications = [];
				
				// If attributes exists, load the information
				if(specsXML.tagName == "specifications" || specsXML.tagName == "attributes") {
					var specNodes = null;
					
					if(specsXML.tagName == "specifications") {
						specNodes = $(specsXML).children();
						
					} else if(specsXML.tagName == "attributes") {
						$.each($(specsXML).children(), _.bind(function(index, child) {
							if(child.tagName == "attribute_list") {
								specNodes = $(child).children();
								return false;
							}
						}, this));
					}
					
					
					if(specNodes && specNodes.length > 0) {
						
						// For each attribute, get id and name (label)
						for(var i=0; i<specNodes.length; i++) {
							var specId = $(specNodes[i]).attr("id");
							var specName = null;
							
							if(taskTypeId && this._isAdminUser && this._filterTaskSpecs.prop("checked")) {
								specName = $(specNodes[i]).attr("name");
							} else {
								specName = $(specNodes[i]).find("name").text();
							}
							
							// Add the specification information to this._availableSpecs object
							var specObj = {
									id: specId,
									label: specName,
									selected: false
								};
							
							specifications.push(specObj);
						}
					}
				}
				
				deferred.resolve(specifications);
			}, this));
			
			return deferred.promise();
		};
		
		
		/**
		 * Method to render multiselect for attributes
		 * 
		 * @param specsSelectDiv - html container/wrapper for creating multiselect
		 * @param data - specifications information object (this._availableSpecs object)
		 */
		SpecificationsSearchPopupView.prototype._getSpecificationsContent = function(specsSelectDiv, data) {
			
			// Before creating multiselectlist, mark sepecifications to preselect if any
			this._setSelectedSpecs(data);
			
			var container = specsSelectDiv;
			var observer = {};
			Observer.mixin(observer);
	
			var maxRows = 100;
	
			// The function used to get data for the library list. This could also
			// get data from the server
			var getLibraryData = function(pattern, page) {
				var dfr = $.Deferred();
				// variable for paging
				var selected = 0;
	
				var regex = FilterUtils.generateRegex(pattern, true);
	
				dfr.resolve(_.filter(data, _.bind(function(object) {
					if (regex.test(object.label) && !object.selected) {
						selected++;
						// Only get the not selected objects that are on the right
						// page
						return selected >= (maxRows * (page - 1))
								&& selected <= maxRows * page;
					}
				}, this)));
				
				return dfr.promise();
			};
	
			// get the selected data
			var getSelectedData = function(pattern, page) {
				var dfr = $.Deferred();

				var regex = FilterUtils.generateRegex(pattern, true);	
				
				var selected = 0;
				
				// First sort based on index, then page
				dfr.resolve(_.filter(_.sortBy(data, "index"), _.bind(function(object) {
					if (regex.test(object.label) && object.selected) {
						selected++;
						// Only get the not selected objects that are on the right
						// page
						return selected >= (maxRows * (page - 1))
								&& selected <= maxRows * page;
					}
				}, this)));
				
				return dfr.promise();
			};
	
			// Create the multiselect list
			var multiSelect = new MultiSelectList(getLibraryData, getSelectedData,
					container, {
						notSelectedLabel : SearchInputs.t("AVAILABLE_SPECIFICATIONS"),
						selectedLabel : SearchInputs.t("SELECTED_ATTRIBUTES"),
						selectAllTooltip : SearchInputs.t("SELECTALL_FILTERED"),
						deselectAllTooltip : SearchInputs.t("DESELECTALL_FILTERED"),
						availFilterPlaceholder: SearchInputs.t("SPECS_AVAILABLE_FILTER"), 
						selFilterPlaceholder: SearchInputs.t("SPECS_SELECTED_FILTER")
					});
			
			/*
			 * Method to get a new index for selected item
			 */
			var getLastObjectIndex = function() {
				var lastIndex = -1;
				
				_.each(data, function(object, i) {
					if(object.index > lastIndex) {
						lastIndex = object.index;
					}
				});
				
				return lastIndex;
			};
			
			/*
			 * Method to remove index for deselected item and 
			 * Update the index for rest of the selected items with object index greater than deselected item index
			 */
			var removeObjectIndex = function(obj) {
				var removeIndex = obj.index;
				
				_.each(data, function(object, i) {
					if(object.index > removeIndex) {
						--object.index;
					}
				});
				
				delete obj.index;
			};
	
			// Handling of a entry selection
			observer._observe(multiSelect, "entrySelected", function(ids) {
				_.each(ids, function(id) {
					for(var i=0; i<data.length; i++) {
						if(id == data[i].id) {
							var obj = data[i];
							obj.selected = true;
							obj.index = getLastObjectIndex() + 1;
						}
					}
				}, this);
			});
	
			// Handling of a entry deselection
			observer._observe(multiSelect, "entryDeselected", function(ids) {
				_.each(ids, function(id) {
					for(var i=0; i<data.length; i++) {
						if(id == data[i].id) {
							var obj = data[i];
							obj.selected = false;
							removeObjectIndex(obj);
						}
					}
				}, this);
			});
	
			// Handling of all entries being selected
			observer._observe(multiSelect, "selectAll", function(pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				
				var objIndex = getLastObjectIndex();
				_.each(data, function(obj) {
					if (regex.test(obj.label) && !obj.selected) {
						obj.selected = true;
						obj.index = ++objIndex;
						return true;
					}
				});
			});
	
			// deselect all entries that have a label
			observer._observe(multiSelect, "deselectAll", function(pattern) {
				var regex = FilterUtils.generateRegex(pattern, true);
				_.each(data, function(obj) {
					if (regex.test(obj.label) && obj.selected) {
						obj.selected = false;
						delete obj.index;
						return true;
					}
				});
			});
	
			// Filter-lists should get a height
			container.find(".filter-list .list-group").css({
				height : 300
			});
	
			multiSelect.render();
		};
		
		
		/**
		 * Method to set selected for specifications to pre-select
		 * Also, maintain the order by using index
		 * 
		 * @param data
		 */
		SpecificationsSearchPopupView.prototype._setSelectedSpecs = function(data) {
			if(data && data.length > 0 && this._specificationids.length > 0) {
				_.each(data, _.bind(function(specObj, index) {
					// -1 if the not in present, > -1 if present
					var isInSelectedSpecs = $.inArray(specObj.id, this._specificationids);
					
					if(isInSelectedSpecs > -1) {
						specObj.selected = true;
						specObj.index = this._specificationids.indexOf(specObj.id);
					}
				}, this));
			}
		};
		
		
		/**
		 * Create add and cancel buttons of the popup
		 * 
		 * @returns
		 */
		SpecificationsSearchPopupView.prototype._getFooterContent = function() {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({'role': 'group', 'name': 'FooterButtons'});
			
			this._addButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("ADD")}).addClass("btn btn-primary");
			
			this._addButton.click(_.bind(function(event) {
				/*
				 * Object to store the selected specifications information
				 * 
				 * {
				 * 		categoryid: , 
				 * 		attributeids: [],
				 * 		filterbytasktype: true/false
				 * }
				 */
				var selSpecsData = {};
				// Initially set to default values
				selSpecsData.categoryid = "";
				selSpecsData.attributeids = [];
				selSpecsData.filterbytasktype = false;
				
				var data = this._availableAllSpecs || [];
				if(this._isAdminUser && this._filterTaskSpecs.prop("checked")) {
					if(this._taskTypeSelect.val()) {
						data = this._availableSpecs;
						selSpecsData.categoryid = this._taskTypeSelect.val();
					}
					
					selSpecsData.filterbytasktype = true;
				}
				
				// The data should be sorted by index property which specifies order of selection
				_.filter(_.sortBy(data, "index"), function(option) {
					if(option && option.selected) {
						selSpecsData.attributeids.push(option.id);
					}
				});
				
				this._addClickHandler(event, selSpecsData);
				this._popup.close();
			}, this));
			
			this._cancelButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CANCEL")}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function(event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));
			
			this._clearButton = $('<input>').attr({'type': 'button', 'value': SearchInputs.t("CLEARALL")}).addClass("btn btn-default add-spacing-right");
			this._clearButton.click(_.bind(function(event) {
				this._resetPopup();
			}, this));
			
			footerButtons.append(this._clearButton, this._cancelButton, this._addButton);
			
			return footerButtons;
		};
		
		/**
		 * Method to reset and open the popup
		 * Also initializes the date picker
		 */
		SpecificationsSearchPopupView.prototype.show = function (selSpecsData) {
			if(selSpecsData) {
				this._initPopup(selSpecsData);
			}
			this._popup.open();
		};
		
		/**
		 * Initialize the popup
		 */
		SpecificationsSearchPopupView.prototype._initPopup = function(selSpecsData) {
			this._loadedSpecs = [];
			
			// If some selected specifications exists, initialize the popup accordingly
			if(selSpecsData && !$.isEmptyObject(selSpecsData)) {
				if(selSpecsData.categoryid) {
					this._tasktypeid = selSpecsData.categoryid;
				} else {
					this._tasktypeid = null;
				}
				
				if(selSpecsData.attributeids && selSpecsData.attributeids.length > 0) {
					this._specificationids = selSpecsData.attributeids;
				} else {
					this._specificationids = [];
				}
				
				if(_.isBoolean(selSpecsData.filterbytasktype) && selSpecsData.filterbytasktype) {
					this._filterbytasktype = true;
					if(this._isAdminUser) {
						this._filterTaskSpecs.prop("checked", true);
					}
				} else {
					this._filterbytasktype = false;
					if(this._isAdminUser) {
						this._filterTaskSpecs.prop("checked", false);
					}
				}
				
				this._specidscached = this._specificationids ? $.extend(true, [], this._specificationids) : [];
				
				if(this._tasktypeid && this._taskTypeSelect) {
					this._taskTypeSelect.val(this._tasktypeid);
					this._getAndRenderSpecifications(this._tasktypeid);
				} else {
					if(!this._isAdminUser) {
						this._getAndRenderSpecifications();
					} else {
						this._getTaskTypes();
					}
				}
			}
			// Otherwise, reset the popup
			else {
				this._resetPopup();
			}
		};
		
		SpecificationsSearchPopupView.prototype._resetPopup = function() {
			this._tasktypeid = null;
			this._specificationids = [];
			this._filterbytasktype = true;
			this._loadedSpecs = [];
			this._specidscached = [];
			
			if(this._isAdminUser) {
				this._filterTaskSpecs.prop("checked", true);
				this._getTaskTypes();
			} else {
				this._getAndRenderSpecifications();
			}
		};
		
		/**
		 * To add callback for add button
		 * @param callback
		 */
		SpecificationsSearchPopupView.prototype.onSubmit = function(callback) {
			this._onAddCallbacks.push(callback);
		};
		
		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		SpecificationsSearchPopupView.prototype.onCancel = function(callback) {
			this._onCancelCallbacks.push(callback);
		};
		
		return SpecificationsSearchPopupView;
	}
);

