/**
 * Implements a "Parameter Field" Layout Handler.
 * 
 * Handles the adding & removing of rows for Items (override to define "items")
 * 
 * @author JEPE
 */
define(["jquery", "underscore", "./SearchSetupTableRow"], function($, _, SearchSetupTableRow) {
	
	/**
	 * Abstract super class for objects that control the rows for lists of items on the search setup page. E.g. attributes or roles.
	 * 
	 * Expects format of the fieldid to be "{prefix}{id of 16 digits}".
	 * 
	 * @param {DOMElement} div - the element in which to use this layout. It is the row that contains the link to edit the list.
	 * @param {FieldLayoutAPI} api - the api to add new rows and such.
	 * @param {String} prefix - the prefix of the id's of the fields
	 * @param {String} orderOption - name of the SEQUENCELIST type option in the layout options containing the order of the criteria on the search setup page
	 * @param {boolean} reorderAllObjectModelListsAlso - 
	 * 					Whether to also keep the "display columns" in the object model in the search setup order. 
	 * 					This affects the order in Excel and in the available dropdown (top parameter panel). 
	 */
	var AbstractListSearchSetupLayout = function(div, api, prefix, orderOption, reorderAllObjectModelListsAlso){
		this._div = $(div);
		this._api = api;
		this._prefix = prefix;
		this._orderOption = orderOption || null;
		if(this._orderOption && this._api.getLayoutOptions().getOptionType(this._orderOption) !== "ITEMSEQUENCE") {
			// Either this option does not exist or it is not of the correct type...
			throw new Error("Illegal argument passed to AbstractListSearchSetupLayout");
		}
		this._reorderAllObjectModelListsAlso = reorderAllObjectModelListsAlso;
		
		this._rows = [];// array of {control:<SearchSetupTableRow>, td:<jQuery>}
	};
	
	//
	// Utility functions...
	//
	
	/**
	 * Looks through the model for all fields which match the expected id format.
	 * 
	 * Expects format of the fieldid to be "{prefix}{id of 16 digits}".
	 * 
	 * @returns {Array} ids - the array of ids (16 digit ids)
	 */
	AbstractListSearchSetupLayout.prototype.getConfiguredFieldIds = function() {
		// Look for all id's in the model which have the correct format.
		var allfieldids = _.filter(this._api.getListOfAllCriterionIds(), _.bind(function(fieldid) {
			return fieldid.indexOf(this._prefix) === 0 && fieldid.length===this._prefix.length+16;
		}, this));
		
		// Extract the ids...
		var ids = _.map(allfieldids, _.bind(function(fieldid) {
			return fieldid.substring(this._prefix.length);
		}, this));
		
		// Order them according to the "search setup screen sequence"...
		// This is saved in a layout option
		// NOTE: this option does not necessary contain all the fields! (e.g. new criteria can be added by asset browser)
		// Those will simply be added at the end.
		// NOTE: this option could also contain MORE criteria, in which case these will be added to the list
		if(this._orderOption !== null) {
			var lastsearchsetuporder = this._api.getLayoutOptions().getOption(this._orderOption) || [];
		
			ids = _.union(ids, lastsearchsetuporder);
			
			// Now order...
			var orderlookupmap = _.invert(lastsearchsetuporder);// (lookup by key, instead of index)
			ids.sort(function(a, b) {
				var aWasOnSetup = _.has(orderlookupmap, a);
				var bWasOnSetup = _.has(orderlookupmap, b);
				
				var aScreenSeq = parseInt(orderlookupmap[a]);
				var bScreenSeq = parseInt(orderlookupmap[b]);
				
				if (!aWasOnSetup && !bWasOnSetup) {
					return 0;// both were added on the search +results+ page...
				} else if(aWasOnSetup && bWasOnSetup){
					// both were added on the search +setup+ page
					if(aScreenSeq === bScreenSeq) {
						return 0;// should never happen
					}else if(aScreenSeq < bScreenSeq) {
						return -1;// last time on the search setup, a was shown above b
					}else {
						return 1;// last time on the search setup, a was shown below b
					}
				} else if(aWasOnSetup) {
					return -1;// only a was added on the search setup
				} else {
					return 1;// only b was added on the search setup
				}
			});
		}
		
		return ids;
	};
	
	/**
	 * Register the click listener on the link to edit the configuration
	 */
	AbstractListSearchSetupLayout.prototype.bindEdit = function() {
		$("a", this._div).click(_.bind(this.edit, this));
	};
	
	/**
	 * Function to update the table and id's shown
	 * 
	 * Note: in case id's are removed, the caller is responsible to remove the data from the object model also.
	 * This method only removes and adds the controls.
	 * 
	 * @param {Array} newids - the new id's to show
	 */
	AbstractListSearchSetupLayout.prototype.updateRowsByIds = function(newids) {
		// If there are too much rows, delete some
		while(this._rows.length > newids.length) {
			this.deleteRow(_.last(this._rows));
		}
		
		// If there are not enough rows, create some
		while(this._rows.length < newids.length) {
			this.addRowAtEnd();
		}
		
		// Now, recreate all controls in the rows with the new id's...
		var i = 0;
		_.each(this._rows, _.bind(function(row) {
			this.configureRow(row, newids[i]);
			i++;
		}, this));
	};
	
	/**
	 * Keep track of the order on the search setup page in the object model, so we can restore it next time
	 * 
	 * Note: can be called too much (e.g. while we are still building the setup by using updateRowsByIds).
	 */
	AbstractListSearchSetupLayout.prototype._updateObjectModelOrder = function() {
		if(this._orderOption !== null) {
			var newids = this.getRowIds();
			
			// Keep track of the order in the object model
			this._api.getLayoutOptions().setOption(this._orderOption, newids);
			
			if(this._reorderAllObjectModelListsAlso) {
				// Also reorder the different criteria... (if enabled)
				// This affects the order of the columns in Excel & in the available dropdown (design panel) ...
				var fullids = _.map(newids, function(id) {return this._prefix+id;}, this);
				this._api.updateObjectModelOrder(fullids);
			}
		}
	};
	
	/**
	 * Get the list of configured ids.
	 */
	AbstractListSearchSetupLayout.prototype.getRowIds = function() {
		// They are stored in rows in the "id" property.
		// This property can be null but we are not interested in those.
		return _.map(_.filter(this._rows, function(row) {return row.id!==null;}), function(row) {return row.id;});
	};
	
	/**
	 * Delete the row identified by rowinfo
	 */
	AbstractListSearchSetupLayout.prototype.deleteRow = function(rowinfo) {
		this._rows = _.without(this._rows, rowinfo);
		rowinfo.control.cleanup();
		rowinfo.tr.remove();
		rowinfo.id=null;
		this._toggleBorderBelowLast(true);
		
		// And store this order in the layout options part of the object model, so we can restore it next time
		this._updateObjectModelOrder();
	};
	
	/**
	 * (Private) util function to add/remove the border on the last row
	 */
	AbstractListSearchSetupLayout.prototype._toggleBorderBelowLast = function(borderon) {
		this._div.toggleClass("borderabove", this._rows.length!=0);
		if(this._rows.length!=0) {
			this._rows[this._rows.length-1].tr.toggleClass("borderbelow", borderon);
		}
	};
	
	/**
	 * Make a new empty row at the end.
	 * 
	 * @return {Object} rowinfo - an object with control (SearchSetupTableRow) & tr (jQuery) properties. Identifies the row.
	 */
	AbstractListSearchSetupLayout.prototype.addRowAtEnd = function() {
		// remove the border on the last row (new row will be added)
		this._toggleBorderBelowLast(false);
		
		// Make the rowinfo
		var row = $("<tr>");
		var control = new SearchSetupTableRow(row, this._api);
		var rowinfo = {
			control:control,
			tr:row,
			id:null
		};
		
		// Add the row at the end
		var lastrow = this._rows.length>0?this._rows[this._rows.length-1].tr:this._div;
		this._rows.push(rowinfo);
		lastrow.after(row);
		
		// Add the border to the last row
		this._toggleBorderBelowLast(true);
		
		// Return the rowinfo
		return rowinfo;
	};
	
	/**
	 * Change the id of a certain row.
	 * Can be used to re-initialize a row also!
	 * 
	 * Note: in case you replace the controls in a row with ones of a different id, 
	 * the caller is responsible to remove the data from the object model also.
	 * This method only removes the controls. => so you could also move them.
	 * 
	 * @param {Object} rowinfo - the object identifying the row (returned by addRowAtEnd)
	 * @param {String} newid - the id with which to configure the row.
	 */
	AbstractListSearchSetupLayout.prototype.configureRow = function(rowinfo, newid) {
		// Cleanup the controls in case there were already controls there...
		rowinfo.control.cleanup();
		
		// Configure the row...
		rowinfo.control.configure(this._prefix+newid, true, true, true, true);
		this.makeLabel(rowinfo, this._prefix+newid, rowinfo.control.getLabelContainer());

		// Keep track of the id
		rowinfo.id = newid;
		
		// And store this order in the layout options part of the object model, so we can restore it next time
		this._updateObjectModelOrder();
	};
	
	/**
	 * Called when a label need to be generated for a certain row
	 * 
	 * @param {Object} rowinfo - the object identifying the row (returned by addRowAtEnd)
	 */
	AbstractListSearchSetupLayout.prototype.makeLabel = function(rowinfo, fieldid, cell) {
		throw new Error("Need to override this method!");
	};
	
	/**
	 * Remove the given Item from the SEQUENCELIST in the layout options
	 * 
	 * @param {settingOption} name of the SEQUENCELIST type option in the layout options containing the order of the criteria on the search setup page
	 * @param {item} Id of item which should remove from the List
	 */
	AbstractListSearchSetupLayout.prototype.tryToRemoveItem = function(settingOption, item) {
		
		if(settingOption) {
			var itemList = this._api.getLayoutOptions().getOption(settingOption);
			
			if(itemList) {
				var itemIndex = itemList.indexOf(item);
				if (itemIndex > -1) {
					itemList.splice(itemIndex, 1);
					this._api.getLayoutOptions().setOption(settingOption,itemList);
				}
			}
		}
	};
	
	/**
	 * Called when you click the link to edit the items that are shown
	 */
	AbstractListSearchSetupLayout.prototype.edit = function() {
		throw new Error("Need to override this method!");
	};
	
	// Return the constructor... not a factory. 
	return AbstractListSearchSetupLayout;
});