/**
 * Implements a "Parameter Field" Layout Handler.
 * 
 * Handles the adding & removing of rows for Attributes
 * 
 * @author JEPE
 */
define(["jquery", 
        "underscore", 
        "./AbstractListSearchSetupLayout", 
        "../parameterfields/popups/AttributesSearchPopup"], 
        function($, 
        		_, 
        		AbstractListSearchSetupLayout,
        		AttributesSearchPopup) {
	
	/**
	 * The object controlling the rows for attributes on the search setup page
	 * 
	 * @param {DOMElement} div - the element in which to use this layout. In our case this is the ROW with the attributes link.
	 * @param {FieldLayoutAPI} api - the api to add new rows and such.
	 * @param {String} prefix - the prefix of the id's for attribute fields
	 * @param {String} mode - either "attributes" or "specifications"
	 */
	var AttributeSearchSetupLayout = function(div, api, prefix, mode){
		// name of "layout option" which contains the order of these criteria on the search setup
		this._extended = prefix!=="db_attr_";
		var orderOption = !this._extended ? "attr_search_screen_sequence" : "extended_attr_search_screen_sequence";
		
		// super call
		AbstractListSearchSetupLayout.call(this, div, api, prefix, orderOption, true);
		
		// Is admin user?
		this._isAdmin = api.getExternalSearchSettings().getIsAdmin();
		
		// Initialize
		this._setting = null;
		this._shownacatid = "";
		this._shownids = [];
		this._isattrs = mode!=="specifications";
		this._init();
		this.bindEdit();
	};
	
	AttributeSearchSetupLayout.prototype = Object.create(AbstractListSearchSetupLayout.prototype);
	AttributeSearchSetupLayout.prototype.constructor = AttributeSearchSetupLayout;
	
	/**
	 * Initialize, look up which id's are in the saved search and make rows for them...
	 */
	AttributeSearchSetupLayout.prototype._init = function() {
		this._shownids = this.getConfiguredFieldIds();
		
		// Now, set the new ids...
		this.updateRowsByIds(this._shownids);
	};
	
	/**
	 * Called when a label need to be generated for a certain row
	 */
	AttributeSearchSetupLayout.prototype.makeLabel = function(rowinfo, fieldid, cell) {
		var label = $("<label>");
		// Might return instantly (if we already downloaded the information for this role)
		this._api.getSetupLabel(fieldid, _.bind(function(setuplabel) {
			label.text(setuplabel);
		}, this));
		cell.empty().append(label);
	};
	
	/**
	 * Called when you click the link to edit the attributes that are shown
	 */
	AttributeSearchSetupLayout.prototype.edit = function() {
		
		/*
		 * Object to store selected Attribute Category/Task Type and its Attributes/Specifications respectively
		 * categoryid is selected attribute category id or task type id
		 * 
		 * Attribute selection
		 * {
		 * 		categoryid: ,
		 * 		attributeids: []
		 * }
		 * 
		 * Specification selection
		 * {
		 * 		categoryid: ,
		 * 		attributeids: [],
		 * 		filterbytasktype: true/false
		 * }
		 */ 
		
		if(!this._attrsData) {
			this._attrsData = {};
			
			this._layoutOptions = this._api.getLayoutOptions();
			
			if(this._layoutOptions) {
				var categoryId = this._layoutOptions.getOption(!this._extended ? "category_id" : "extended_category_id");
				
				if(categoryId) {
					this._attrsData.categoryid = categoryId;
				} else {
					this._attrsData.categoryid = "";
				}
				
				if(!this._isattrs && !this._extended) {
					var filterByTaskType = this._layoutOptions.getOption("chkbox_FilterSpecsByTaskType");
					this._attrsData.filterbytasktype = filterByTaskType;
				}
			}
			
			// this._shownids are attribute ids from saved search in order
			if(this._shownids.length > 0) {
				this._attrsData.attributeids = this._shownids;
			}
		}
		
		
		// Create popup if doesn't exists, else open the popup
		if(!this._popup) {
			var options = {};
			options.isattrsview = this._isattrs;
			options.isadmin = this._isAdmin;
			
			// Create a popup
			this._popup = new AttributesSearchPopup(this._attrsData, options);
			
			this._popup.onSubmit(_.bind(function(selAttrsData) {
				// Assign selected acat and its attrs to object
				this._attrsData = selAttrsData;
				
				var newids = this._attrsData.attributeids;
				
				// Update the controls
				this.updateRowsByIds(newids);
				
				if(!this._extended) {
					this._layoutOptions.setOption("category_id", this._attrsData.categoryid);
					
					if(!this._isattrs) {
						this._layoutOptions.setOption("chkbox_FilterSpecsByTaskType", this._attrsData.filterbytasktype);
					}
				} else {
					this._layoutOptions.setOption("extended_category_id", this._attrsData.categoryid);
				}
				
				// Which id's are deleted?
				// The new controls are there now...
				// However, we still have the old data in the object model...
				var deletedids = _.difference(this._shownids, newids);
				_.each(deletedids, _.bind(function(id) {
					this._api.tryDeleteAllInformationForField(this._prefix+id);
				}, this));
				
				this._shownids = newids;
				
			}, this));
			
			// Open the popup
			this._popup.open();
			
		} else {
			this._popup.open(this._attrsData);
		}
	};
	
	
	return {
		/**
		 * Main function which will be called to layout...
		 * 
		 * @param {DOMElement} div - the div in which the layout needs to be put...
		 * @param {FieldLayoutAPI} api - all the functions which can be used...
		 */
		layout:function(div, api) {
			var prefix = $(div).attr("data-fieldidprefix");
			var mode = $(div).attr("data-mode");
			new AttributeSearchSetupLayout(div, api, prefix, mode);
		}
	};
});