/**
 * Implements a "Parameter Field" Layout Handler.
 * 
 * Simple... Components are added in the xsl and we just make the Parameter Fields here...
 * 
 * But, there could be complex implementations which add/remove parameter field components & criteria
 */
define(["jquery",
        "underscore",
        "./components/SearchSetupFieldPanelVisibilitySettings",
        "./components/SearchSetupFieldColumnSettings",
        "./components/SearchSetupFieldSortCriterionSettings"
        ], function($, _,
        		SearchSetupFieldPanelVisibilitySettings, 
        		SearchSetupFieldColumnSettings, 
        		SearchSetupFieldSortCriterionSettings){
	
	/*
	 * NOTE: this is the default one and it will always be created...
	 * 
	 * Other implementations will need to add 
	 *   data-searchsetuplayout="type"
	 * to the row/div they want to be instantiated in.
	 */
	
	return {
		/**
		 * Main function which will be called to layout...
		 * 
		 * @param {DOMElement} div - the div in which the layout needs to be put...
		 * @param {FieldLayoutAPI} api - all the functions which can be used...
		 */
		layout:function(div, api) {
			// 
			// The Default Layout simply looks at the html generated by xsl and puts 
			// the design panel fields in the pre-created cells.
			//
			// The default layout does not look at which criterion exist...
			// The default layout can also not delete or add new criteria
			//
			$(".searchdesignpanelfield", this.div).each(function(i, e) {
				var fieldid = $(e).attr("data-fieldid");
				var fieldid_data = $(e).attr("data-fieldid_data");
				var fieldinitialfocus = $(e).attr("data-fieldinitialfocus") === "1";
				
				// Usually, all controls in a single on one row work on the same field. 
				// But not always... (see bill of materials for project search)
				var fieldid_parameterfieldcontrol = fieldid;
				var fieldid_designpanelcontrol =  fieldid_data || fieldid;
				var fieldid_columndisplaycontrol = fieldid_data || fieldid;
				var fieldid_sortingcontrol = fieldid_data || fieldid;
				
				var row = $($(e).parents("tr")[0]);
				
				// Make the design panel component
				api.makeParameterField(fieldid_parameterfieldcontrol, e);
				if(fieldinitialfocus) {
					api.focusParameterField(e);
				}
				
				// Make the design panel settings controller for this field...
				var cell_visibility_select = $(".genericinput_visibility_select", row).get(0);
				var cell_visibility_label = $(".genericinput_visibility_label", row).get(0);
				var cell_visibility_row = $(".genericinput_visibility_row", row).get(0);
				var cell_visibility_column = $(".genericinput_visibility_column", row).get(0);
				if(!!fieldid_data) {
					// We have a separate id for the data & the design panel...
					// In that case we have difficulty with the design panel, because the label field should work for both id's!
					// So, simply hide the controls for the design panel!
					cell_visibility_select = $("<div>").get(0);
					cell_visibility_row = $("<div>").get(0);
					cell_visibility_column = $("<div>").get(0);
				}
				
				var control_visibility = new SearchSetupFieldPanelVisibilitySettings(cell_visibility_select, cell_visibility_label, cell_visibility_column, cell_visibility_row);
				api.registerFieldDesignPanelSettingsController(fieldid_designpanelcontrol, control_visibility);
				
				// Make the column display settings controller for this field...
				var cell_columndisplay_checked = $(".genericinput_columndisplay_checked", row).get(0);
				var cell_columndisplay_sequence = $(".genericinput_columndisplay_sequence", row).get(0);
				var cell_columndisplay_editable = $(".genericinput_columndisplay_editable", row).get(0);
				var control_display = new SearchSetupFieldColumnSettings(cell_columndisplay_checked, cell_columndisplay_sequence, cell_columndisplay_editable);
				api.registerFieldColumnDisplaySettingsController(fieldid_columndisplaycontrol, control_display);
				
				// Make the sorting settings controller for this field...
				var cell_sorting_checked = $(".genericinput_sorting_checked", row).get(0);
				var cell_sorting_number = $(".genericinput_sorting_number", row).get(0);
				var control_sorting = new SearchSetupFieldSortCriterionSettings(cell_sorting_checked, cell_sorting_number);
				api.registerFieldSortingSettingsController(fieldid_sortingcontrol, control_sorting);
			});
			
		}
	};
});