define([
	"jquery",
	"underscore",
	"./displaypreferences/CascadingAttributesSetupTable",
	"component/sdrop/SDrop"
], function (
	$,
	_,
	CascadingAttributesSetupTable,
	SDrop
) {

	/**
	 * The object controlling the section for display preferences on the search setup page
	 * 
	 * @param {DOMElement} div - the element in which to use this layout.
	 * @param {FieldLayoutAPI} api - the api to get to display preferences and other things.
	 */
	var DisplayPreferencesSearchSetupLayout = function (div, api) {
		this._div = $(div);
		this._api = api;
		this._data = this._api.getLayoutOptions();
		this._settinglist = this._data.getAvailableSettings();

		this._initialize();

		this._updatingModel = false;
	};

	/**
	 * Initialize all the controls in the display preferences section.
	 */
	DisplayPreferencesSearchSetupLayout.prototype._initialize = function () {
		// Initialize the component for the cascading attributes table
		this._cascadingtable = new CascadingAttributesSetupTable($("#cascadingTreeSetupTable"));

		// Initialize the SDrop for selecting templates (on pages where we have the template selector)
		this._initializeTemplateSDrop();
		this._drilldownSDropProvider = new StaticSDropDataProvider();

		var savedSearchSelect = $("#selectDrillDownSavedSearchName");
		this._savedSearchMap = this._createSavedSearchesArray(savedSearchSelect);
		this._drilldownSDropProvider.setData(this._savedSearchMap);
		this._initializeDrillDownSDrop(this._drilldownSDropProvider, savedSearchSelect);

		// Watch for changes in the inputs
		_.each(this._settinglist, function (setting) {
			var input = $("#" + setting.id);
			if (setting.type === "BOOLEAN") {
				input.on("click", _.bind(this._onChange, this, setting.id));
			} else if (setting.type === "STRING") {
				input.on("change", _.bind(this._onChange, this, setting.id));
			}
		}, this);
		// also for the cascading table...
		this._cascadingtable.onChange(_.bind(this._onChange, this, "cascading_attribute_trees"));

		// Update the controls to match the state in the model.
		this._updateView();

		this._data.onChange(_.bind(this._updateView, this));
	};

	/**
	 * Updates the controls the user sees based on what is in the object model.
	 * 
	 * In the future this method could maybe be called multiple times during the lifetime of the component...
	 */
	DisplayPreferencesSearchSetupLayout.prototype._updateView = function () {
		//
		// Default code for BOOLEAN & STRING types
		//

		if (this._updatingModel) {
			return;
		}

		// Go through all the settings and update the controls.
		_.each(this._settinglist, function (setting) {
			// Get the value the option should have
			var value = this._data.getOption(setting.id);

			// Look for a checkbox or radiobutton or input or select with the same id as the setting
			// If there is none, the default code cannot handle this setting
			var input = $("#" + setting.id);

			// Look for the control and configure it
			if (setting.type === "BOOLEAN") {
				// So, this is a boolean setting...
				// Assume it is represented by a checkbox or a radio button
				if (input.length !== 0 && input.prop("checked") !== value) {
					input.prop("checked", value);
				}
			} else if (setting.type === "STRING") {
				// So, this is a string setting...
				// Assume it is represented by an input or a dropdown or an SDrop
				if (input.length !== 0 && input.val() !== value) {
					if (input.get(0).SDrop) {
						// SDrop dropdown
						input.get(0).SDrop.populate(value);
					} else {
						// normal dropdown
						input.val(value);
					}
				}
			} else {
				// For the other data types, there is no default code...
				// You need custom code to handle those...
			}
		}, this);


		//
		// Code for all other fields...
		//

		// Set the information for the cascading attributes.
		var cascadingdata = this._data.getOption("cascading_attribute_trees");
		if (cascadingdata) {
			this._cascadingtable.setData(cascadingdata);
		}

		// Show the right controls...
		this._handleNewPreferencesActions();
	};

	/**
	 * Called when the control for a certain preference changes...
	 * 
	 * @param {String} settingid - the "id" of the setting
	 */
	DisplayPreferencesSearchSetupLayout.prototype._onChange = function (settingid) {
		var recursiveChangeEvent = this._updatingModel;
		if (!recursiveChangeEvent) {
			this._updatingModel = true;
		}

		// The setting
		var setting = _.findWhere(this._settinglist, {
			id: settingid
		});

		// The input
		var input = $("#" + setting.id);

		// Look for the control and configure it
		if (input.length !== 0 && setting.type === "BOOLEAN") {
			// So, this is a boolean setting...
			// Assume it is represented by a checkbox or a radio button
			this._data.setOption(setting.id, input.prop("checked"));

			// If it is a radio button, we also need to update the "linked" display preferences...
			// These do not get an "update" event automatically...
			if (input.is(":radio") && !recursiveChangeEvent) {
				var allradiobtns = $(":radio[name='" + input.attr("name") + "']");
				allradiobtns.each(function () {
					if (this != input[0]) {
						$(this).triggerHandler("click");
					}
				});
			}
		} else if (input.length !== 0 && setting.type === "STRING") {
			// So, this is a string setting...
			// Assume it is represented by an input or a dropdown or an SDrop
			this._data.setOption(setting.id, input.val());
		} else {
			// For the other data types, there is no default code...
			// You need custom code to handle those...
		}

		if (setting.id === "cascading_attribute_trees") {
			this._data.setOption(setting.id, this._cascadingtable.getData());
		}

		if (!recursiveChangeEvent) {
			this._data.commitChanges();
			this._updatingModel = false;


			// Show the right controls...
			this._handleNewPreferencesActions();
		}
	};

	/**
	 * Handles actions that need to be done when preferences have changed
	 */
	DisplayPreferencesSearchSetupLayout.prototype._handleNewPreferencesActions = function () {

		var chkbox_CustomActions = document.getElementById('chkbox_CustomActions');
		if (chkbox_CustomActions && chkbox_CustomActions.checked) {
			$("#selectCustomActionTagsSpan").show();
		} else {
			$("#selectCustomActionTagsSpan").hide();
		}

		var chkbox_CreateProject = document.getElementById('chkbox_CreateProject');
		if (chkbox_CreateProject && chkbox_CreateProject.checked) {

			$("#CreateProjectDetails").show();
			var radiobtn_SelectTemplate = document.getElementById('radiobtn_SelectTemplate');
			var radiobtn_CharacteristicsPath = document.getElementById('radiobtn_CharacteristicsPath');

			if (radiobtn_SelectTemplate && radiobtn_SelectTemplate.checked) {
				$("#selectTemplateId").show();
				this._templateSDrop.redraw();

			} else {
				$("#selectTemplateId").hide();
			}

			if (radiobtn_CharacteristicsPath && radiobtn_CharacteristicsPath.checked) {
				$("#selectCharPathRowId").show();
			} else {
				$("#selectCharPathRowId").hide();
			}

			if (!((radiobtn_SelectTemplate && radiobtn_SelectTemplate.checked) ||
					(radiobtn_CharacteristicsPath && radiobtn_CharacteristicsPath.checked))) {
				document.getElementById('radiobtn_NoTemplate').checked = true;
			}
		} else {
			$("#CreateProjectDetails").hide();
		}

		var selectStatusStyle = document.getElementById('SelectProjectStatusStyle');
		if (selectStatusStyle && selectStatusStyle.value.indexOf('Calendar') == 0) { // starts with Calendar
			$("#SetCalendarSize").show();
			// set default value
			if (!$("#selectCalendarSize").val() || $("#selectCalendarSize").val() == '') {
				$("#selectCalendarSize").val('500');
			}

			if ($("#selectCalendarSize").val() && !isNaN($("#selectCalendarSize").val()) && $("#selectCalendarSize").val() < 200) {
				$("#selectCalendarSize").val('200');
			}

			$("#SetProjectStatusLabelStyle").show();

			// only show forward planning option when estimations are made
			if (selectStatusStyle.value == 'Calendar') {
				$("#SetApplyForwardPlanning").hide();
			} else {
				$("#SetApplyForwardPlanning").show();
			}


			$("#SetShowCalendarStatusIcon").show();

			// also select status to be shown as a column
			if ($("#db_status_checked") && $("#db_status_checked")[0]) {
				if (!$("#db_status_checked").attr("checked")) { // need this check, otherwise counter will go up at every change
					$("#db_status_checked").attr("checked", true);
					$("#db_status_checked").triggerHandler("updatesequence");
				}
			}

			$("#chkbox_ShowDrillDownColumn").prop("checked", false).closest("tr").hide();

			// Enable the project status view checkbox for calendar options
			this._api.setColumnDisplayView("db_status", true);

		} else {
			$("#SetCalendarSize").hide();
			$("#SetShowCalendarStatusIcon").hide();
			$("#SetApplyForwardPlanning").hide();
			$("#SetProjectStatusLabelStyle").hide();
			$("#chkbox_ShowDrillDownColumn").closest("tr").show();
		}

		var showDrillDownColumn = $("#chkbox_ShowDrillDownColumn");
		var useParentLayout = $("#useParentLayoutDiv");
		if (showDrillDownColumn && this._drillDownSavedSearchSDrop && useParentLayout) {
			if (showDrillDownColumn[0].checked) {
				this._drillDownSavedSearchSDrop.show();
				if (selectStatusStyle && selectStatusStyle.value.indexOf('Calendar') === 0) {
					selectStatusStyle.value = -1;
					$(selectStatusStyle).trigger("change");
				}
			} else {
				this._drillDownSavedSearchSDrop.hide();
			}
		}

		var drilldownSelect = $("#selectDrillDownSavedSearchName");
		if (drilldownSelect && showDrillDownColumn && showDrillDownColumn[0]) {
			var selectItemVal = drilldownSelect.val();
			var selectedItem = this._drilldownSDropProvider.getById(selectItemVal);

			if (selectItemVal && selectedItem && selectedItem.isProjectSearch && showDrillDownColumn[0].checked) {
				useParentLayout.show();
			} else {
				useParentLayout.hide();
				var userParentLayoutCheckbox = useParentLayout.find("input")[0];
				if (userParentLayoutCheckbox.checked) {
					userParentLayoutCheckbox.checked = false;
					this._onChange(userParentLayoutCheckbox.getAttribute("name"));
				}
			}
		}

	};

	//
	// -- Utility methods below --
	//

	/**
	 * Initialize the SDrop for selecting a template for "creating project from selected".
	 */
	DisplayPreferencesSearchSetupLayout.prototype._initializeTemplateSDrop = function () {
		if ($('#selectTemplateId').length === 0) {
			return; // We don't have this specific display preference on this page...
		}

		// Show the template row
		$("#selectTemplateRowId").show();

		// only create one instance of the sDrop for templates
		if (!this._templateSDrop) {
			this._templateSDrop = new SDrop($("#selectTemplateId"), null,
				new SDropDataProvider({
					'dataSource': "GetNameIDList.jsp?list=Templates"
				}), {
					'autoload': false,
					'manageStyleAndWidth': false,
					'initializeDataProvider': false,
					'initialValue': null
				}
			);
		}
		this._templateSDrop.redraw();

		// load templates
		this._templateSDrop.searchFor('');
	};


	/**
	 * Creates an array of saved search objects, containing the id, name and an 'isProjectSearch' property indicating if it is a project search or not.
	 * @return {Array.<Object>}
	 */
	DisplayPreferencesSearchSetupLayout.prototype._createSavedSearchesArray = function (select) {
		return _.map(select.find("option"), function (elem) {
			return {
				id: elem.value,
				name: elem.textContent,
				isProjectSearch: elem.getAttribute("data-isprojectsearch") === "true"
			};
		});
	};

	/**
	 * Initializes the SDrop for selecting a saved search used in the drill down
	 * @private
	 */
	DisplayPreferencesSearchSetupLayout.prototype._initializeDrillDownSDrop = function (sdropProvider, select) {
		if (select && select.length > 0) {
			this._drillDownSavedSearchSDrop = new SDrop(select, null, sdropProvider, {
				manageStyleAndWidth: false,
				enableEmptySelection: true
			});
		}
	};

	return {
		/**
		 * Main function which will be called to layout...
		 * 
		 * @param {DOMElement} div - the div in which the layout needs to be put...
		 * @param {FieldLayoutAPI} api - all the functions which can be used...
		 */
		layout: function (div, api) {
			new DisplayPreferencesSearchSetupLayout(div, api);
		}
	};

});
