/**
 * Implements a "Parameter Field" Layout Handler.
 * 
 * Handles the adding & removing of rows for Roles
 * 
 * @author JEPE
 */
define(["jquery", 
        "underscore", 
        "./AbstractListSearchSetupLayout", 
        "utils/webi18n!SearchInputs",
        "component/sdrop/SDrop"], 
function($, _, AbstractListSearchSetupLayout, bundle, SDrop) {
	
	/**
	 * The object controlling the rows for roles on the search setup page
	 * 
	 * @param {DOMElement} div - the element in which to use this layout. In our case this is the ROW with the "add role" link.
	 * @param {FieldLayoutAPI} api - the api to add new rows and such.
	 */
	var RolesSearchSetupLayout = function(div, api){
		// super call
		AbstractListSearchSetupLayout.call(this, div, api, "db_role", "roles_search_screen_sequence", true);
		
		// Initialize
		this._init();
		this.bindEdit();
	};
	
	RolesSearchSetupLayout.prototype = Object.create(AbstractListSearchSetupLayout.prototype);
	RolesSearchSetupLayout.prototype.constructor = RolesSearchSetupLayout;
	
	/**
	 * Initialize, look up which id's are in the saved search and make rows for them...
	 */
	RolesSearchSetupLayout.prototype._init = function() {
		// Get the configured ids
		var shownids = this.getConfiguredFieldIds();
		// And make rows for each one of them
		this.updateRowsByIds(shownids);
	};
	
	/**
	 * Called when a label need to be generated for a certain row
	 */
	RolesSearchSetupLayout.prototype.makeLabel = function(rowinfo, fieldid, cell) {
		// Get the label... 
		var label = $("<label>");
		// Might return instantly (if we already downloaded the information for this role)
		this._api.getSetupLabel(fieldid, _.bind(function(setuplabel) {
			label.text(setuplabel);
		}, this));
		
		// Fill the cell
		cell.empty()
			.append(this._getDeleteButton(rowinfo, fieldid))
			.append("&nbsp;&nbsp;")
			.append(label);
	};
	
	/**
	 * Get a delete button for a certain row
	 * @param {Object} rowinfo - the unique identifier for the row
	 * @param {String} fieldid - the id of the field which is in the row to delete (can be undefined or null)
	 */
	RolesSearchSetupLayout.prototype._getDeleteButton = function(rowinfo, fieldid){
		return $("<i class='icon icon-WCR_Remove searchrolesremovebutton'>").click(_.bind(function() {
			this.deleteRow(rowinfo);
			if(fieldid) {
				// Delete the roleId from role sequence list
				this.tryToRemoveItem(this._orderOption,fieldid.replace(this._prefix,""));
				this._api.tryDeleteAllInformationForField(fieldid);
			}
		}, this));
	};
	
	/**
	 * Called when you click the link to add a new role...
	 */
	RolesSearchSetupLayout.prototype.edit = function() {
		// - Make a new row
		var rowinfo = this.addRowAtEnd();
		
		// - Add the delete button
		rowinfo.control.getLabelContainer().append(this._getDeleteButton(rowinfo, null));
		
		// - Initialize the SDrop data provider if needed...
		if(!this._roleDataprovider) {
			this._roleDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=Role"});
		}
		
		// - Add a select in the input cell
		var inputcontainer = rowinfo.control.getInputContainer();
		var selector = $('<select />');
		inputcontainer.append(selector);
		
		// - Convert the select to an SDrop
		new SDrop(selector, "", this._roleDataprovider, {'onChangedByUser': _.bind(function(){
			var newid = selector.val();
			
			var shownids = this.getRowIds();
			if(_.contains(shownids, newid)) {
				alert(bundle.t("WCREPORT_ROLE_ALREADY_SELECTED"));
				return;
			}
			
			// Delete the SDrop
			selector[0].SDrop.destroy();
			
			// Configure the row
			this.configureRow(rowinfo, newid);
			
			// Enable the view checkbox
			this._api.setColumnDisplayView(this._prefix+newid, true);
		}, this)});
	};
	
	
	return {
		/**
		 * Main function which will be called to layout...
		 * 
		 * @param {DOMElement} div - the div in which the layout needs to be put...
		 * @param {FieldLayoutAPI} api - all the functions which can be used...
		 */
		layout:function(div, api) {
			new RolesSearchSetupLayout(div, api);
		}
	};
});