/**
 * Implements a "Parameter Field" Layout Handler.
 * 
 * Handles the adding & removing of rows for Text Management
 * 
 * @author PASH
 */
define(["packaging-content-management/text-content-services/languageService",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "./SearchSetupTableRow",
        "utils/webi18n!TextContentSearchStrings",
		"underscore",
		"jquery"], 
function(languageService,
		LanguageUtils,
		SearchSetupTableRow,
		webi18n,
		_,
		$){
	
	// For automatically focussing the first row when you enter the page.
	var firstrowcreated = false;
	
	/**
	 * Class to prepare layout for the statement search of text content search page.
	 * 
	 * @param {FieldLayoutAPI} api
	 * @param {JQuery TR} parentRow
	 */
	var TextContentStatementsLayout = function(api, parentRow) {
		this._parentRow = $(parentRow);		// This row is created in XSL contains header as Statement.
		this._api = api;
		this._indexedRowStore = [];			// Array to keep track of add/remove buttons for all the row object
		this._rowCounter = 0;				// Helps to manage remove button from the last row
		
		this._constructStatementRow();
		
		// Watch for element type value change.
		this._api.watch("ElementType", _.bind(function (data) {
			// Get value of element type field.
			var elementType = data.getValue("value");
			if ("NUTRITIONFACTS" == elementType) {
				// If value is nutrition facts then destroy all the rows of statement.
				$.each(this._indexedRowStore, _.bind(function (index, statementRow) {
					statementRow.destroyRow();
				}, this));
				
				this._rowCounter = 0;
				this._indexedRowStore.length = 0;
				this._parentRow.hide();
			} else {
				// Create or show statement rows.
				this._parentRow.show();
				if (this._indexedRowStore.length == 0) {
					this.addStatementRow();
				}
			}
		}, this));
	};
	
	/**
	 * This method starts creation of rows based on the field IDs stored in list of criteria </br>
	 * If list does not contains any field Id, then empty row will be generated by default.
	 */
	TextContentStatementsLayout.prototype._constructStatementRow = function() {
		this.fieldIds = this._api.getListOfAllCriterionIds();
		
		var generateField = true;
		for ( var i = 0; i < this.fieldIds.length; i++) {
			// Text content statement field IDs are combination of prefix, language and content type separated by underscore.
			var fieldIdParts = this.fieldIds[i].split("_");
			if ("textcontent" === fieldIdParts[0]) {
				var fieldId = this.fieldIds[i];
				this.addStatementRow(fieldId);
				generateField = false;
			}
		}
		if (generateField) {
			this.addStatementRow();
		}
	};
	
	/**
	 * This method generates single row for desired fieldId.
	 * @param {String} fieldId
	 */
	TextContentStatementsLayout.prototype.addStatementRow = function(fieldId) {
		if (!fieldId) {
			fieldId = 'textcontent_lang_Any';
		}
		
		var placeHolder = this._parentRow;
		if (this._indexedRowStore.length > 0) {
			placeHolder = this._indexedRowStore.getLast().getRow();
		}
		
		var textContentStatementRow = new TextContentStatementRow(this, this._api, fieldId, placeHolder);
		this._rowCounter++;
		this._indexedRowStore.push(textContentStatementRow);
		
		$.each(this._indexedRowStore, function(index, row) {
			row.showRemoveRowOperation();
			row.showOperatorLabel();
			row.hideAddRowOperation();
		});
		
		if (1 === this._rowCounter) {
			this._indexedRowStore.getLast().hideRemoveRowOperation();
		}
		this._indexedRowStore.getLast().hideOperatorLabel();
		this._indexedRowStore.getLast().showAddRowOperation();
	};
	
	/**
	 * This method is called when delete button is clicked to destroy current row.
	 * @param {TextContentStatementRow} statementRow
	 */
	TextContentStatementsLayout.prototype.destroyStatementRow = function(statementRow) {
		
		this._indexedRowStore = jQuery.grep(this._indexedRowStore, _.bind(function(row, index) {
			if (row === statementRow) {
				statementRow.destroyRow();
				this._rowCounter--;
			}
			row.hideAddRowOperation();
			row.showOperatorLabel();
			return row !== statementRow;
		}, this));
		
		// User is not allowed to delete last row, so hide remove option for last row.
		if (1 === this._rowCounter) {
			this._indexedRowStore.getLast().hideRemoveRowOperation();
		}
		this._indexedRowStore.getLast().hideOperatorLabel();
		this._indexedRowStore.getLast().showAddRowOperation();
	};
	
	/**
	 * This class represents single row for text content statement and provides all the facility to perform action on statement row.
	 * 
	 * @param {TextContentStatementsLayout} statementLayout
	 * @param {FieldLayoutAPI} api
	 * @param {String} fieldId
	 * @param {JQuery TR} placeHolder
	 */
	var TextContentStatementRow = function(statementLayout, api, fieldId, placeHolder) {
		this._statementLayout = statementLayout;
		this._api = api;
		this._fieldId = fieldId;
		this._row = $('<tr>');
		this._row.addClass('altEvenRow');
		$(placeHolder).after(this._row);
		
		this._rowIdentifier = '';
		
		this._isLanguagesLoaded = new $.Deferred();
		this._languageSelect = this._constructLanguageSelect();
		this._contentTypeSelect = this._constructContentTypeSelect();
		
		this._addRowActionButton = $('<a>').addClass('btn btn-default').attr({'type':'button'});
		this._addRowActionButton.addClass("btn-row-action").append(
				$('<i>').addClass('icon icon-WCR_Add action-icon-font')
		);
		
		this._removeRowActionButton = $('<a>').addClass('btn btn-default').attr({'type':'button'});
		this._removeRowActionButton.addClass("btn-row-action").append(
				$('<i>').addClass('icon icon-WCR_Close action-icon-font')
		);
		
		// Adding click event to call method to add statement row.
		this._addRowActionButton.on('click', _.bind(function() {
			this._statementLayout.addStatementRow();
		}, this));
		
		// Adding click event to call method to remove statement row.
		this._removeRowActionButton.on('click', _.bind(function() {
			this._statementLayout.destroyStatementRow(this);
		}, this));
		
		this._operatorDiv = $('<div>').hide();
		
		this._constructRow();
		
		this._addLanguageListener();
		this._addContentTypeListener();
	};
	
	/**
	 * @returns {JQuery TR} - current row 
	 */
	TextContentStatementRow.prototype.getRow = function() {
		return this._row;
	};
	
	/**
	 * Method to destroy current row. </br>
	 * This method make sure to clean all the registry for the criteria, visibility and other settings before removing row.
	 */
	TextContentStatementRow.prototype.destroyRow = function() {
		this._api.destroyParameterField(this._inputParameterFieldsColumn[0]);
		this._SearchSetupTableRow.cleanup();
		
		this._api.tryDeleteAllInformationForField(this._fieldId);
		this._row.remove();
	};

	/**
	 * Method to hide option for adding a row. Add row option is only valid for last row.
	 */
	TextContentStatementRow.prototype.hideOperatorLabel = function() {
		this._operatorDiv.hide();
	};
	
	/**
	 * Since add row operation is possible of last row, so this method will help to provide option.
	 */
	TextContentStatementRow.prototype.showOperatorLabel = function() {
		this._operatorDiv.show();
	};

	/**
	 * Method to hide option for adding a row. Add row option is only valid for last row.
	 */
	TextContentStatementRow.prototype.hideAddRowOperation = function() {
		this._addRowActionButton.hide();
	};
	
	/**
	 * Since add row operation is possible of last row, so this method will help to provide option.
	 */
	TextContentStatementRow.prototype.showAddRowOperation = function() {
		this._addRowActionButton.show();
	};

	/**
	 * User is not allowed to delete statement row if there is only 1 row left. </br>
	 * So remove option will be hidden with this method.
	 */
	TextContentStatementRow.prototype.hideRemoveRowOperation = function() {
		this._removeRowActionButton.hide();
	};

	/**
	 * This method is to show remove option when more then 1 statement row are available.
	 */
	TextContentStatementRow.prototype.showRemoveRowOperation = function() {
		this._removeRowActionButton.show();
	};
	
	/**
	 * This method is the actual place where statement row is created. </br>
	 * Previous values are respected, Means values will be filled with previous values in case of refine search criteria or refreshing the page.
	 */
	TextContentStatementRow.prototype._constructRow = function() {
		this._languageSelect.detach();// keep change events after recreating the row
		this._contentTypeSelect.detach();// keep change events after recreating the row
		this._addRowActionButton.detach();// keep click events after recreating the row
		this._removeRowActionButton.detach();// keep click events after recreating the row
		this._row.empty();// empty the row (note: will remove all listeners to elements that are still in the row)
		this._SearchSetupTableRow = new SearchSetupTableRow(this._row, this._api);
		
		// Fill in all the cells except for the actual input
		this._SearchSetupTableRow.configure(this._fieldId, false, true, true, true);
		
		//
		// Fill in the actual input cell
		//
		var fieldIdParts = this._fieldId.split("_");
		this._rowIdentifier = "lang" === fieldIdParts[1] ? '': fieldIdParts[1];
		var valueContentType = fieldIdParts[2];
		
		var table = $('<table>');
		
		this._inputParameterFieldsColumn = $('<td>').addClass('text-status-input').attr('colspan', '2');
		
		table.append(
			$('<tbody>').append(
				$('<tr>').append(
					$('<td>').append(this._constructLanguageCol(this._rowIdentifier)),
					$('<td>').append(this._constructContentTypeCol(valueContentType)),
					$('<td>').append(
						this._removeRowActionButton
					)
				),
				$('<tr>').append(
						this._inputParameterFieldsColumn,
					$('<td>').append(
						this._addRowActionButton,
						this._operatorDiv
					)
				)
			)
		);
		
		this._SearchSetupTableRow.getInputContainer().append(table);
		
		// Make the design panel component
		this._api.makeParameterField(this._fieldId, this._inputParameterFieldsColumn[0]);
		
		// Automatically focus the first row when first entering the page...
		if(!firstrowcreated) {
			firstrowcreated = true;
			this._api.focusParameterField(this._inputParameterFieldsColumn[0]);
		}
		
		this._api.makeParameterField('textcontentoperator', this._operatorDiv.get(0));
	};
	
	/**
	 * This method constructs language drop down. </br>
	 * Executes only once for each row.
	 * @returns
	 */
	TextContentStatementRow.prototype._constructLanguageSelect = function() {
		var languageSelect = $('<select>').addClass('form-control');
		
		languageSelect.attr({
			'type' : 'select',
			'aria-describedby': 'sizing-addon3',
		});
		
		languageService.getBothAllLanguages(_.bind(function (masterLangs, transLangs) {
			languageSelect.append($('<option>', {value: '', text: ''}));
			var combinedLang = LanguageUtils.mergeBothLanguagesTogether(masterLangs, transLangs);
			_.each(combinedLang.values(), function(langObj){
					var code = langObj.langCode.toLowerCase().trim().replace(/\-/g, '');
					var description = langObj.langDesc;
					languageSelect.append($('<option>', {value: code, text:description}));
			});
			this._isLanguagesLoaded.resolve('');
		}, this));
		
		return languageSelect;
	};

	/**
	 * This method constructs content type drop down. </br>
	 * Executes only once for each row.
	 * @returns
	 */
	TextContentStatementRow.prototype._constructContentTypeSelect = function() {
		var contentTypeSelect = $('<select>').addClass('form-control search-input-sm');
		contentTypeSelect.attr({
			'type' : 'select',
			'aria-describedby' : 'sizing-addon3',
		});
		
		contentTypeSelect.append(
			$('<option>', {value: 'Any', text: ''}),
			$('<option>', {value: 'Master', text: webi18n.t('MASTER')}),
			$('<option>', {value: 'Translated', text: webi18n.t('TRANSLATED')})
		);
		
		return contentTypeSelect;
	};
	
	/**
	 * This method returns a DIV with language drop down to place in current row of statement.
	 * @param valueLanguage
	 * @returns
	 */
	TextContentStatementRow.prototype._constructLanguageCol = function(valueLanguage) {
		var languageSelectDiv = $('<div>').addClass('input-group input-statement-field');
		
		$.when(this._isLanguagesLoaded).done(_.bind(function() {
			this._languageSelect.val(valueLanguage);
	    	return true;
    	}, this));

		languageSelectDiv.append(
				$('<span>').addClass('input-group-addon').attr({'id': 'sizing-addon3'}).text(webi18n.t('STATEMENT_LANGUAGE')),
				this._languageSelect
		);
		return languageSelectDiv;
	};
	
	/**
	 * This method returns a DIV with content type drop down to place in current row of statement.
	 * @param {String} valueContentType
	 * @returns
	 */
	TextContentStatementRow.prototype._constructContentTypeCol = function(valueContentType) {
		var contentTypeSelectDiv = $('<div>').addClass('input-group input-statement-field');
		
		this._contentTypeSelect.val(valueContentType);
		
		contentTypeSelectDiv.append(
			$('<span>').addClass('input-group-addon').attr({'id': 'sizing-addon3'}).text(webi18n.t('STATEMENT_CONTENTTYPE')),
			this._contentTypeSelect
		);
		
		return contentTypeSelectDiv;
	};
	
	/**
	 * Method is to add event listener for language change.
	 */
	TextContentStatementRow.prototype._addLanguageListener = function() {
		var self = this;
		this._languageSelect.change(function() {
			self._identityModifier(this);
		});
	};
 
	/**
	 * Method is to add event listener for content type change.
	 */
	TextContentStatementRow.prototype._addContentTypeListener = function() {
		var self = this;
		this._contentTypeSelect.change(function() {
			self._identityModifier(this);
		});
 	};
 	
 	/** 
 	 * This method is to generate row identifier based on the selection of language and content type belongs to the same row 
 	 **/
	TextContentStatementRow.prototype._identityModifier = function() {
 		var lang = this._languageSelect.val();
		var type = this._contentTypeSelect.val();
		
		if (!lang) {
			lang = 'lang';
		}
		if (!type) {
			type = 'Any';
		}
		
		var rowIdentifier = 'textcontent_' + lang + '_' + type;
		
		this._api.destroyParameterField(this._inputParameterFieldsColumn[0]);
		this._SearchSetupTableRow.cleanup();
		
		this._api.tryMoveAllInformationForField(this._fieldId, rowIdentifier);
		this._api.tryDeleteAllInformationForField(this._fieldId);
		
		this._fieldId = rowIdentifier;
		this._constructRow();
 	};
 	
	return {
		/**
		 * Main function which will be called to layout...
		 * 
		 * @param {DOMElement} div - the div in which the layout needs to be put...
		 * @param {FieldLayoutAPI} api - all the functions which can be used...
		 */
		layout:function(div, api) {
			new TextContentStatementsLayout(api, div);
		}
	};
});