/**
 * Search Utility class to provide common functionalities of search.
 * @author PASH + JEPE
 */
define(["jquery",
        "underscore",
        "./components/SearchSetupFieldColumnSettings",
        "./components/SearchSetupFieldPanelVisibilitySettings",
        "./components/SearchSetupFieldSortCriterionSettings"],
function($,
		_,
		SearchSetupFieldColumnSettings,
		SearchSetupFieldPanelVisibilitySettings,
		SearchSetupFieldSortCriterionSettings
		) {
	"use strict";
	
	/**
	 * This class expects one row which suppose to have 2 column for label and parameter fields. </br>
	 * So that columns for option of search panel design and search column settings can be generated.
	 * 
	 * @param {jQuery} row : current row where column needs to be generated.
	 * @param {FieldLayoutAPI} api : the api passed to the layouts...
	 */
	var SearchSetupTableRow = function(row, api) {
		this._row = row;
		this._api = api;
		this._construct();
	};
	
	//
	// Constructing the cells in this row
	//
	
	/**
	 * Common function to generate 'Search Results Options' columns in specified row for the search criteria
	 */
	SearchSetupTableRow.prototype._construct = function() {
		// Whether the user is logged in as an admin
		var isAdmin = this._api.getExternalSearchSettings().getIsAdmin();
		
		// 
		// Generate the cells in the row...
		//
		this._inputcontainer = $('<div>').addClass('itemwrapper');
		this._labelcontainer = $('<td>').addClass("labelcell");
		
		this._row.append(this._labelcontainer, $('<td>').addClass('inputcell').append(this._inputcontainer));
		
		this._columnViewList = $('<td>').attr({"class": "darktext_darklink_11", "align": "center"});		// View(List)
		this._columnColumnOrder = $('<td>').attr({"class": "darktext_darklink_11", "align": "center"});	// Column Order
		this._columnEditable = $('<td>').attr({"class": "Editable", "align": "center"});	// Editable
		this._columnSortOrder = $('<td>').attr({"class": "darktext_darklink_11", "align": "center"});	// Sort
		this._columnOrder = $('<td>').attr({"class": "darktext_darklink_11", "align": "center"});	// Order
		
		/** Creating 4 extra columns for search panel design */
		if($('#db_show_search_panel_design:checked').length > 0) {
			this._columnVisibility = $('<td>').addClass("darktext_darklink_11");		// Visibility
			this._columnLable = $('<td>').addClass("darktext_darklink_11");				// Label
			this._columnRowPosition = $('<td>').addClass("darktext_darklink_11");		// Row
			this._columnColumnPosition = $('<td>').addClass("darktext_darklink_11");	// Column
			this._row.append(this._columnVisibility).append(this._columnLable).append(this._columnRowPosition).append(this._columnColumnPosition);
		}
		
		this._row.append(this._columnViewList);
		this._row.append(this._columnColumnOrder);
	
		// Editable column is available only for admin users.
		if (isAdmin) {
			this._row.append(this._columnEditable);
		}
		
		this._row.append(this._columnSortOrder);
		this._row.append(this._columnOrder);
	};
	
	//
	// Controlling the components in those cells
	//
	
	/**
	 * Configure the row to be linked to a certain fieldid and become functional.
	 * 
	 * @param {String} fieldId - the id of the field to link to this row
	 */
	SearchSetupTableRow.prototype.configure = function (fieldId, doField, doDesignPanel, doColumnDisplay, doSorting) {
		// Create the parameter field
		if(!this._initedField && doField) {
			this._initedField = true;
			this._inputdiv = $("<div>");
			this._inputcontainer.empty().append(this._inputdiv);
			this._api.makeParameterField(fieldId, this._inputdiv.get(0));
		}
		
		// Register panel visibility settings
		if(!this._initedDesignPanel && doDesignPanel && $('#db_show_search_panel_design:checked').length > 0) {
			this._initedDesignPanel = true;
			
			var visibilityCol = this.getVisibilityCol().get(0);
			var labelCol = this.getLabelCol().get(0);
			var columnPositionCol = this.getColumnPositionCol().get(0);
			var rowPositionCol = this.getRowPositionCol().get(0);
			
			this._panelVisibilitySettings = new SearchSetupFieldPanelVisibilitySettings(visibilityCol, labelCol, columnPositionCol, rowPositionCol);
			this._api.registerFieldDesignPanelSettingsController(fieldId, this._panelVisibilitySettings);
		}
		
		// Register display column settings
		if(!this._initedColumnDisplay && doColumnDisplay) {
			this._initedColumnDisplay = true;
			
			var viewListCheckCol = this.getViewListCheckBoxCol().get(0);
			var columnOrderFieldCol = this.getColumnOrderFieldCol().get(0);
			var editableFieldCol = this.getEditableFieldCol().get(0);
			
			this._displayColumnSettings = new SearchSetupFieldColumnSettings(viewListCheckCol, columnOrderFieldCol, editableFieldCol);
			this._api.registerFieldColumnDisplaySettingsController(fieldId, this._displayColumnSettings);
		}
		
		// Register sort criterion settings
		if(!this._initedSorting && doSorting){
			this._initedSorting = true;
			
			var sortCheckBoxCol = this.getSortCheckBoxCol().get(0);
			var sortOrderCol = this.getSortOrderCol().get(0);
			
			this._sortCriterionSettings = new SearchSetupFieldSortCriterionSettings(sortCheckBoxCol, sortOrderCol);
			this._api.registerFieldSortingSettingsController(fieldId, this._sortCriterionSettings);
		}
	};
	
	/**
	 * Remove the link to a fieldid.
	 * 
	 * Optionally also delete the criteria in the model.
	 */
	SearchSetupTableRow.prototype.cleanup = function () {
		if(this._initedField) {
			this._initedField = false;
			this._api.destroyParameterField(this._inputdiv.get(0));
		}
		
		if(this._initedDesignPanel) {
			this._initedDesignPanel = false;
			this._api.unregisterFieldDesignPanelSettingsController(this._panelVisibilitySettings);
		}
		
		if(this._initedColumnDisplay) {
			this._initedColumnDisplay = false;
			this._api.unregisterFieldColumnDisplaySettingsController(this._displayColumnSettings);
		}
		
		if(this._initedSorting) {
			this._initedSorting = false;
			this._api.unregisterFieldSortingSettingsController(this._sortCriterionSettings);
		}
	};
	
	//
	// Getters
	//

	/**
	 * Container to put the input in
	 * @returns container in which to put the input
	 */
	SearchSetupTableRow.prototype.getInputContainer = function() {
		return this._inputcontainer;
	};
	
	/**
	 * Container to put the label in
	 * @returns container in which to put the label
	 */
	SearchSetupTableRow.prototype.getLabelContainer = function() {
		return this._labelcontainer;
	};
	
	/**
	 * Placeholder for view (list) setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getViewListCheckBoxCol = function() {
		return this._columnViewList;
	};

	/**
	 * Placeholder for column order setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getColumnOrderFieldCol = function() {
		return this._columnColumnOrder;
	};

	/**
	 * Placeholder for editable field
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getEditableFieldCol = function() {
		return this._columnEditable;
	};

	/**
	 * Placeholder for visibility setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getVisibilityCol = function() {
		return this._columnVisibility;
	};

	/**
	 * Placeholder for label setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getLabelCol = function() {
		return this._columnLable;
	};

	/**
	 * Placeholder for row position setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getRowPositionCol = function() {
		return this._columnRowPosition;
	};

	/**
	 * Placeholder for column position setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getColumnPositionCol = function() {
		return this._columnColumnPosition;
	};
	
	/**
	 * Placeholder for Sort setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getSortCheckBoxCol = function() {
		return this._columnSortOrder;
	};

	/**
	 * Placeholder for Sort Order setting
	 * @returns table column
	 */
	SearchSetupTableRow.prototype.getSortOrderCol = function() {
		return this._columnOrder;
	};
	
	return SearchSetupTableRow;
});






	