/**
 * Inputs for column view/list settings on the search setup page.
 * 
 * Created from JavaScript and filled with the information from the model.
 * This class will keep the model up to date.
 */
define([], function(){
	
	/**
	 * View/list checkbox and related fields in Search Setup page.
	 * 
	 * @param {DOMElement} viewListHolder - element to put the view/list checkbox in
	 * @param {DOMElement} columnOrderHolder - element to put the column order input in
	 * @param {DOMElement} editableHolder - element to put the editable checkbox in
	 */
	var SearchSetupFieldColumnSettings = function(viewListHolder, columnOrderHolder, editableHolder) {
		// The listener we attach to the model.
		this._listener = _.bind(this._updateFromModel, this);
		
		// Will be filled in when the field is configured
		this._searchfield = null;
		this._data = null;
		
		// The divs in which we put the information
		this._viewList = $(viewListHolder);
		this._columnOrder = $(columnOrderHolder);
		this._editable = $(editableHolder);
		
		// Initialize this component
		this._initialize();
	};
	
	//
	// Private methods
	//
	
	/**
	 * Initialize this component...
	 */
	SearchSetupFieldColumnSettings.prototype._initialize = function() {
		// Make the view/list checkbox
		this._viewListCheckBox = $('<input>').attr({
			'type' : 'checkbox',
			'class' : 'searchsetup_checked',
			'value' : 'ON'
		}).css('visibility', 'hidden');
		this._viewList.append(this._viewListCheckBox);
		
		// Make the order input
		this._columnOrderInput = $('<input>').attr({
			'type' : 'text',
			'class' : 'searchsetup_sequence',
			'size' : '3'
		}).css('visibility', 'hidden');
		this._columnOrder.append(this._columnOrderInput);
		
		// Make the editable checkbox
		this._editableInput = $('<input>').attr({
			'type' : 'checkbox',
			'class' : 'searchsetup_editable',
			'size' : '3'
		}).css('visibility', 'hidden');
		this._editable.append(this._editableInput);
		
		// Update the object model if you change something...
		this._viewListCheckBox.on("click", _.bind(this._updateToModel, this));
		this._columnOrderInput.on("change", _.bind(this._updateToModel, this));
		this._editableInput.on("click", _.bind(this._updateToModel, this));
	};

	/**
	 * Called when the SearchColumnSettings changes...
	 *  => Update the values which are shown
	 *  
	 * Also called for the initial values...
	 */
	SearchSetupFieldColumnSettings.prototype._updateFromModel = function() {
		this._viewListCheckBox.prop('checked', this._data.getEnabled());
		if(this._columnOrderInput.val() !== this._data.getSequence()) {
			this._columnOrderInput.val(this._data.getSequence());
		}
		 
		if(this._editableInput.prop('checked') !== this._data.isEditable()) {
			this._editableInput.prop('checked', this._data.isEditable());
		}
		
		if (this._viewListCheckBox.prop('checked')) {
			this._columnOrderInput.css('visibility', 'visible');
		}else{
			this._columnOrderInput.css('visibility', 'hidden');
		}
	};

	/**
	 * Called when the user changes any of the inputs...
	 * Update the model in that case...
	 */
	SearchSetupFieldColumnSettings.prototype._updateToModel = function() {
		this._data.setEnabled(this._viewListCheckBox.prop('checked'));
		this._data.setSequence(this._columnOrderInput.val());
		this._data.setEditable(this._editableInput.prop('checked'));
		this._data.commitChanges();
	};

	//
	// Implement the interface
	//

	/**
	 * Configure this control. 
	 * This method is called after the api.registerFieldColumnDisplaySettingsController is called.
	 * 
	 * @param {SearchField} searchfield - common information about the field. Like, can it even be made visible?
	 * @param {SearchDisplayColumn} displayColumn - the information in the object model
	 */
	SearchSetupFieldColumnSettings.prototype.configure = function(searchfield, displayColumn) {
		
		// Store the information
		this._searchfield = searchfield;
		displayColumn.take();
		this._data = displayColumn;
		
		this._fieldId = searchfield.fieldId;
		
		// If we support column settings
		if (searchfield.supportsViewList) {
			// Make the view/list checkbox visible
			this._viewListCheckBox.css("visibility", "visible");
			
			// Make sure changes in the model are reflected in this object
			this._data.onChange(this._listener);
			this._updateFromModel();
		}
		if (searchfield.fieldId === "db_name") {
			// Exception... 
			// Cannot configure db_name... Just show some inputs that do not do anything.
			this._columnOrder.empty().append($('<input type="text" size="3" class="searchsetup_sequence" value="1" disabled="disabled" data-sahi-id="db_name_sequence">'));
			this._viewList.empty().append($('<input type="checkbox" class="searchsetup_checked" checked="checked" disabled="disabled" data-sahi-id="db_name_checked">'));
		}
		
		// If we support batch actions
		if(this._searchfield.supportsBatchActions) {
			this._editableInput.css('visibility', 'visible');
		}
		
		// Add id's for automated testing
		this._viewListCheckBox.attr("data-sahi-id", this._fieldId+"_checked");
		this._columnOrderInput.attr("data-sahi-id", this._fieldId+"_sequence");
		this._editableInput.attr("data-sahi-id", this._fieldId+"_editable");
	};

	/**
	 * Destroy this control. 
	 * This method is called after the api.unregisterFieldColumnDisplaySettingsController is called.
	 */
	SearchSetupFieldColumnSettings.prototype.destroy = function() {
		this._data.removeOnChange(this._listener);
		this._data.release();
		this._data = null;
		this._viewList.empty();
		this._columnOrder.empty();
		this._editable.empty();
	};
	
	return SearchSetupFieldColumnSettings;
});