/**
 * Inputs for design panel visibility settings on the search setup page.
 * 
 * Created from JavaScript and filled with the information from the model.
 * This class will keep the model up to date.
 */
define(["jquery", "underscore", "utils/webi18n!SearchInputs"], function($, _, bundle){
	
	/**
	 * Visibility dropdown & related fields in the Search Setup page
	 * 
	 * @param {DOMElement} selectdiv - element to put the visibility dropdown in
	 * @param {DOMElement} labeldiv - element to put the label input in
	 * @param {DOMElement} columndiv - element to put the column input in
	 * @param {DOMElement} rowdiv - element to put the row input in
	 */
	var SearchSetupFieldPanelVisibilitySettings = function(selectdiv, labeldiv, columndiv, rowdiv) {
		// The listener we attach to the model.
		this._listener = _.bind(this._updateFromModel, this);
		
		// Will be filled in when the field is configured
		this._searchfield = null;
		this._data = null;
		
		// The divs in which we put the information
		this._selectdiv = $(selectdiv);
		this._labeldiv = $(labeldiv);
		this._columndiv = $(columndiv);
		this._rowdiv = $(rowdiv);
		
		// Initialize this component
		this._initialize();
	};
	
	//
	// Private methods
	//
	
	/**
	 * Initialize this component...
	 */
	SearchSetupFieldPanelVisibilitySettings.prototype._initialize = function() {
		this._dropdown = $('<select>', {
			'class' : 'searchsetup_designpanel_select'// for automated testing
		})
			.append($('<option>', {value: 'Hidden', text: bundle.t('ADVSEARCH_VISIBILITY_HIDDEN')}))
			.append($('<option>', {value: 'Available', text: bundle.t('ADVSEARCH_VISIBILITY_AVAILABLE')}))
			.append($('<option>', {value: 'Visible', text: bundle.t('ADVSEARCH_VISIBILITY_VISIBLE')}))
			.css('visibility', 'hidden');
		this._selectdiv.append(this._dropdown);
		
		this._label = $('<input>').attr({
			'type' : 'text',
			'size' : '20',
			'value' : '',
			'class' : 'searchsetup_designpanel_label'// for automated testing
		}).css('visibility', 'hidden');
		this._labeldiv.append(this._label);
		
		this._column = $('<input>').attr({
			'value' : '',
			'type' : 'text',
			'size' : '5',
			'class' : 'searchsetup_designpanel_col'// for automated testing
		}).css('visibility', 'hidden');
		this._columndiv.append(this._column);
		
		this._row = $('<input>').attr({
			'value' : '',
			'type' : 'text',
			'size' : '5',
			'class' : 'searchsetup_designpanel_row'// for automated testing
		}).css('visibility', 'hidden');
		this._rowdiv.append(this._row);
		
		// Update the object model if you change something...
		this._dropdown.on("change", _.bind(this._updateToModel, this));
		this._label.on("keyup", _.bind(this._updateToModel, this));
		this._column.on("keyup", _.bind(function(){
			// Note: Global method used to validate this field... but it does not have side effects... (it does not access any html elements)
			// We should probably move it to RequireJS though...
			var ok = validatePartialInt(this._column.get(0), false, bundle.t('SEARCHCRITERION_INVALID_INTEGER_NUMBER_FEEDBACK'));
			
			if(ok){
				// update the model
				this._updateToModel();
			}else{
				return false;// undo change
			}
		}, this));
		this._row.on("keyup", _.bind(function(){
			// Note: Global method used to validate this field... but it does not have side effects... (it does not access any html elements)
			// We should probably move it to RequireJS though...
			var ok = validatePartialInt(this._row.get(0), false, bundle.t('SEARCHCRITERION_INVALID_INTEGER_NUMBER_FEEDBACK'));
			
			if(ok){
				// update the model
				this._updateToModel();
			}else{
				return false;// undo change
			}
		}, this));
	};
	
	/**
	 * Called when the SearchDesignPanelItem changes...
	 *  => Update the values which are shown
	 */
	SearchSetupFieldPanelVisibilitySettings.prototype._updateFromModel = function() {
		if(this._dropdown.val() !== this._data.getOriginalVisibility()) {
			this._dropdown.val(this._data.getOriginalVisibility());
		}
		if(this._label.val() !== this._data.getLabel()) {
			this._label.val(this._data.getLabel());
		}
		
		if ('Visible' === this._data.getOriginalVisibility()) {
			this._column.css('visibility', 'visible');
			this._row.css('visibility', 'visible');
		} else {
			this._column.css('visibility', 'hidden');
			this._row.css('visibility', 'hidden');
		}
		
		if(this._column.val() !== this._data.getColumn()) {
			this._column.val(this._data.getColumn());
		}
		if(this._row.val() !== this._data.getRow()) {
			this._row.val(this._data.getRow());
		}
	};
	
	/**
	 * Called when the user changes any of the inputs...
	 * Update the model in that case...
	 */
	SearchSetupFieldPanelVisibilitySettings.prototype._updateToModel = function() {
		this._data.setOriginalVisibility(this._dropdown.val());
		this._data.setLabel(this._label.val());
		this._data.setColumn(this._column.val());
		this._data.setRow(this._row.val());
		this._data.commitChanges();
	};
	
	//
	// Implement the interface
	//
	
	/**
	 * Configure this control. 
	 * This method is called after the api.registerFieldDesignPanelSettingsController is called.
	 * 
	 * @param {SearchField} searchfield - common information about the field. Like, can it even be made visible?
	 * @param {SearchDesignPanelItem} designPanelItem - the information in the object model
	 */
	SearchSetupFieldPanelVisibilitySettings.prototype.configure = function(searchfield, designPanelItem) {
		
		// Store the information
		this._searchfield = searchfield;
		designPanelItem.take();
		this._data = designPanelItem;
		
		this._fieldId = searchfield.fieldId;
		
		// If we support design panel settings
		if (searchfield.supportsDesignPanel) {
			// Make the label & dropdown visible
			this._dropdown.css('visibility', 'visible');
			this._label.css('visibility', 'visible');
			
			// Make sure changes in the model are reflected in this object
			this._data.onChange(this._listener);
			this._updateFromModel();
		}
		
		// Add id's for automated testing
		this._dropdown.attr("data-sahi-id", this._fieldId+"_visibility");
		this._label.attr("data-sahi-id", this._fieldId+"_label");
		this._column.attr("data-sahi-id", this._fieldId+"_column");
		this._row.attr("data-sahi-id", this._fieldId+"_row");
	};
	
	
	/**
	 * Destroy this control. 
	 * This method is called after the api.unregisterFieldDesignPanelSettingsController is called.
	 */
	SearchSetupFieldPanelVisibilitySettings.prototype.destroy = function() {
		this._data.removeOnChange(this._listener);
		this._data.release();
		this._data = null;
		this._selectdiv.empty();
		this._labeldiv.empty();
		this._columndiv.empty();
		this._rowdiv.empty();
	};
	
	return SearchSetupFieldPanelVisibilitySettings;
});