/**
 * Inputs for sort order settings on the search setup page.
 * 
 * Created from JavaScript and filled with the information from the model.
 * This class will keep the model up to date.
 */
define([], function(){
	
	/**
	 * Sorting check box & related fields in the Search Setup page.
	 * 
	 * @param {DOMElement} sortCheckBoxHolder - element to put the sort checkbox in
	 * @param {DOMElement} sortOrderHolder - element to put the sort order input in
	 */
	var SearchSetupFieldSortCriterionSettings = function(sortCheckBoxHolder, sortOrderHolder) {
		// The listener we attach to the model.
		this._listener = _.bind(this._updateFromModel, this);
		
		// Will be filled in when the field is configured
		this._searchfield = null;
		this._data = null;
		
		// The divs in which we put the information
		this._sortCheckBoxHolder = $(sortCheckBoxHolder);
		this._sortOrderHolder = $(sortOrderHolder);
		
		// Current status
		this._currentSortOrder = "asc";
		this._currentSortNumber = "";
		
		// Initialize this component
		this._initialize();
	};

	
	//
	// Private methods
	//
	
	/**
	 * Initialize this component...
	 */
	SearchSetupFieldSortCriterionSettings.prototype._initialize = function(fieldId) {
		// Make the sort checkbox
		this._sortCheckBox = $('<input>').attr({
			'type' : 'checkbox',
			'value' : 'ON'
		}).css({
			'display' : 'none'
		});
		this._sortCheckBoxHolder.append(this._sortCheckBox);
		
		// The sort arrows...
		this._makeJSSortOrderField(fieldId);

		// Update the object model if you change something...
		this._sortCheckBox.on("click", _.bind(this._updateToModel, this));
	};
	
	/**
	 * Make the arrows for changeing the sort order
	 */
	SearchSetupFieldSortCriterionSettings.prototype._makeJSSortOrderField = function(fieldId) {
		// Order Display
		this._sortOrderField = $('<span>');
		
		// Ascending image
		this._orderDisplayImgSortAsc = $('<i>').attr({
			'class' : 'icon icon-normal icon-WCR_Table-Sort-Up',
			'title' : 'Ascending'
		}).css({
			'display' : 'none',
			'cursor' : 'pointer'
		}).click(_.bind(this._toggleOrder, this));
		
		// Descending image
		this._orderDisplayImgSortDesc = $('<i>').attr({
			'class' : 'icon icon-normal icon-WCR_Table-Sort-Down',
			'title' : 'Descending'
		}).css({
			'display' : 'none',
			'cursor' : 'pointer'
		}).click(_.bind(this._toggleOrder, this));
		
		this._sortOrderHolder.append(this._sortOrderField, "&nbsp;&nbsp;", $('<span>').append(this._orderDisplayImgSortAsc, this._orderDisplayImgSortDesc));
	};
	
	/**
	 * Toggle the order when clicking on the order images
	 */
	SearchSetupFieldSortCriterionSettings.prototype._toggleOrder = function(){
		if(this._currentSortOrder === "asc") {
			this._currentSortOrder = "desc";
		}else{
			this._currentSortOrder = "asc";
		}
		this._updateToModel();
	};

	/**
	 * Called when the SearchColumnSettings changes...
	 *  => Update the values which are shown
	 *  
	 * Also called for the initial values...
	 */
	SearchSetupFieldSortCriterionSettings.prototype._updateFromModel = function() {
		this._sortCheckBox.prop("checked", this._data.getEnabled());
		if(this._data.getEnabled()) {
			if(this._data.getOrder()==="asc") {
				this._orderDisplayImgSortAsc.show();
				this._orderDisplayImgSortDesc.hide();
			} else {
				this._orderDisplayImgSortDesc.show();
				this._orderDisplayImgSortAsc.hide();
			}
			this._sortOrderField.text(""+this._data.getNumber());
		}else{
			this._orderDisplayImgSortDesc.hide();
			this._orderDisplayImgSortAsc.hide();
			this._sortOrderField.text("");
		}
		
		this._currentSortOrder = this._data.getOrder();
		this._currentSortNumber = ""+this._data.getNumber();
	};

	/**
	 * Called when the user changes any of the inputs...
	 * Update the model in that case...
	 */
	SearchSetupFieldSortCriterionSettings.prototype._updateToModel = function() {
		this._data.setEnabled(this._sortCheckBox.prop("checked"));
		this._data.setOrder(this._currentSortOrder);
		this._data.setNumber(this._currentSortNumber);
		this._data.commitChanges();
	};

	//
	// Implement the interface
	//

	/**
	 * Configure this control. 
	 * This method is called after the api.registerFieldSortingSettingsController is called.
	 * 
	 * @param {SearchField} searchfield - common information about the field. Like, can it even be made visible?
	 * @param {SearchSortingCriterion} sortingSettings - the information in the object model
	 */
	SearchSetupFieldSortCriterionSettings.prototype.configure = function(searchfield, sortingSettings) {

		// Store the information
		this._searchfield = searchfield;
		sortingSettings.take();
		this._data = sortingSettings;

		this._fieldId = searchfield.fieldId;
		
		// If we support sorting settings
		if (searchfield.supportsSorting) {
			// Make the sort checkbox visible
			this._sortCheckBox.show();
			
			// Make sure changes in the model are reflected in this object
			this._data.onChange(this._listener);
			this._updateFromModel();
		}
		
		// Add id's for automated testing
		this._sortCheckBox.attr("data-sahi-id", this._fieldId + "_sort");
		this._sortOrderField.attr("data-sahi-id", this._fieldId + "_sort_number_disp");
		this._orderDisplayImgSortAsc.attr("data-sahi-id", this._fieldId + "_sort_asc");
		this._orderDisplayImgSortDesc.attr("data-sahi-id", this._fieldId + "_sort_desc");
	};

	/**
	 * Destroy this control. 
	 * This method is called after the api.unregisterFieldSortingSettingsController is called.
	 */
	SearchSetupFieldSortCriterionSettings.prototype.destroy = function() {
		this._data.removeOnChange(this._listener);
		this._data.release();
		this._data = null;
		this._sortCheckBoxHolder.empty();
		this._sortOrderHolder.empty();
	};
	
	return SearchSetupFieldSortCriterionSettings;
});