define([
        "jquery", 
        "underscore", 
        "../../../dataobjects/ValueData"
        ], function(
        	$, 
        	_, 
        	ValueData
        ) {
	
	/**
	 * A Javascript file controlling the "cascading attributes configuration table".
	 * 
	 * @param {jQuery} table - the table jQuery element
	 */
	var CascadingAttributesSetupTable = function(table) {
		this._changelisteners = [];
		
		this._table = table;
		// Note: currently the basic structure of the table (= without content) is still generated in xsl...
		this._rowcontainer = $(" > tbody", this._table);
		this._addButtonsRow = $(" > tr", this._rowcontainer);
		this._addButton = $(".add-new-to-list", this._addButtonsRow);
		
		// Keep a list of rows in the cascading tree setup
		this._cascadingrows = [];
		
		// Initialize (and clear)
		this._clear();
	};
	
	/**
	 * Register a change listener.
	 * @param {Function) callback - the callback which will be executed when something changed in the table
	 */
	CascadingAttributesSetupTable.prototype.onChange = function(callback) {
		this._changelisteners.push(callback);
	};
	
	/**
	 * Execute all registered change listeners
	 */
	CascadingAttributesSetupTable.prototype._cascadingAttributesChanged = function() {
		_.each(this._changelisteners, function(callback) {
			callback();
		}, this);
	};
	
	/**
	 * Clear the content of the table
	 */
	CascadingAttributesSetupTable.prototype._clear = function() {
		// Clear the container
		this._rowcontainer.empty();
		
		// Re-add & configure the "add" button
		this._rowcontainer.append(this._addButtonsRow);
		this._addButton.click(_.bind(function() {
			// Add a new empty item
			var data = new ValueData();
			data.setValue("type", "list");
			this._addListOrAttributeToTree(data);
			this._cascadingAttributesChanged();
		}, this));
		
		// Also clear the private list of rows we keep
		this._cascadingrows = [];
	};
	
	
	/**
	 * Get the information in the table
	 * 
	 * @return {ValueData} - the tree configuration
	 */
	CascadingAttributesSetupTable.prototype.getData = function() {
		var data = new ValueData();
		for(var i=0;i<this._cascadingrows.length;i++) {
			var row = this._cascadingrows[i];
			var rowdata = data.addNewItemToList("treeElem");
			rowdata.setValue("type", row.type);
			rowdata.setValue("name", row.input_name.val());
			rowdata.setValue("depth", row.input_depth.val());
			rowdata.setValue("height", row.input_height.val());
		}
		return data;
	};
	
	/**
	 * Set the information in the table
	 * @param {ValueData} data - the tree configuration
	 */
	CascadingAttributesSetupTable.prototype.setData = function(data) {
		this._clear();
		
		// Check if we have data
		if(!data) return;
		
		// Loop through all the "treeElem" nodes
		var items = data.getSubList("treeElem") || [];
		for(var i=0;i<items.length;i++) {
			var item = items[i];
			this._addListOrAttributeToTree(item);
		}
	};
	
	/** 
	* Adds a line to the list of items in the left tree
	* 
	* The "type" should always be filled in with either "list" or "attribute"
	* @param {ValueData} rowdata - The data to edit in the row
	*/
	CascadingAttributesSetupTable.prototype._addListOrAttributeToTree = function(rowdata) {
		// Get the data
		var item_type = rowdata.getValue("type");
		var item_name = rowdata.getValue("name") || "";
		var item_depth = rowdata.getValue("depth") || "";
		var item_height = rowdata.getValue("height") || "";
		
		// construct a row
		var r = {
			// the row
			row: $("<tr>", {"class":"treeconfigtablerow"}),
			
			// the type for the row
			type: item_type,
			
			// all the cells
			cell_delete: $("<td>", {"class":"treeconfigtablecelldelete"}),
			cell_move: $("<td>", {"class":"treeconfigtablecellmove"}),
			cell_type: $("<td>", {"class":"treeconfigtablecelltype"}),
			cell_name: $("<td>", {"class":"treeconfigtablecellname"}),
			cell_depth: $("<td>", {"class":"treeconfigtablecelldepth"}),
			cell_height: $("<td>", {"class":"treeconfigtablecellheight"}),
			
			// the buttons
			btn_delete: $("<a>", {"title":"Remove", "class":"btn btn-xs btn-noborder"}).append($("<i>", {"class":"icon icon-normal icon-WCR_Remove12"})),
			btn_moveup: $("<a>", {"title":"Move Up", "class":"btn btn-xs btn-noborder"}).append($("<i>", {"class":"icon icon-normal icon-WCR_Up-S"})),
			btn_movedown: $("<a>", {"title":"Move Down", "class":"btn btn-xs btn-noborder"}).append($("<i>", {"class":"icon icon-normal icon-WCR_Down-S"})),
			
			// the inputs
			input_name: $("<input>", {"class":"form-control", "type":"text"}).attr("size", "30").val(item_name),
			input_depth: $("<input>", {"class":"form-control", "type":"text"}).attr("size", "4").val(item_depth),
			input_height: $("<input>", {"class":"form-control", "type":"text"}).attr("size","4").val(item_height)
		};
		
		// fill the row
		r.row.append(r.cell_delete, r.cell_move, r.cell_type, r.cell_name, r.cell_depth, r.cell_height);
		r.cell_delete.append(r.btn_delete);
		r.cell_move.append(r.btn_moveup, r.btn_movedown);
		r.cell_type.text(item_type);
		r.cell_name.append(r.input_name);
		if (item_type == 'list') {
			r.cell_depth.append(r.input_depth);
		}
		this._row_unique_id_counter = !this._row_unique_id_counter?1:this._row_unique_id_counter+1;
		var id_for_label = "listcellheight_" + this._row_unique_id_counter;// only reason for this id => to make the <label> work!
		r.cell_height.append($("<div>", {"class":"input-group"}).append(r.input_height.attr("id", id_for_label), $("<label>", {"class":"input-group-addon", "for":id_for_label, "text":"px"})));
		
		// attach listeners
		r.btn_delete.click(_.bind(this._deleteTreeRow, this, r));
		r.btn_moveup.click(_.bind(this._moveRow, this, r, -1));
		r.btn_movedown.click(_.bind(this._moveRow, this, r, 1));
		r.input_name.change(_.bind(this._cascadingAttributesChanged, this));
		r.input_depth.change(_.bind(this._cascadingAttributesChanged, this));
		r.input_height.change(_.bind(this._cascadingAttributesChanged, this));
		
		// add the row to the html
		var rowCount = $('#cascadingTreeSetupTable > tbody > tr').length;
		$('#cascadingTreeSetupTable > tbody > tr').eq(rowCount - 1).before(r.row);
		
		// add the information to the array and update the model
		this._cascadingrows.push(r);
	};
	
	/** 
	* Deletes a line to the list of items in the left tree
	* @param {Object} info - object with all the information about the row
	*/
	CascadingAttributesSetupTable.prototype._deleteTreeRow = function(info) {
		info.row.remove();
		
		// remove the information from the array and update the model
		this._cascadingrows = _.without(this._cascadingrows, info);
		this._cascadingAttributesChanged();
	};
	
	/**
	 * Moves a line up or down in the left tree
	 * @param {Object} info - object with all the information about the row
	 * @param {int} direction - the direction to move the row (-1 = up, +1 = down)
	 */
	CascadingAttributesSetupTable.prototype._moveRow = function(info, direction) {
		var index = _.indexOf(this._cascadingrows, info);
		if(index+direction < 0 || index+direction > this._cascadingrows.length-1) {
			// We want it to move outside the table... Don't do anything
		} else {
			var otherinfo = this._cascadingrows[index+direction];
			if(direction===-1) {
				otherinfo.row.before(info.row);
			} else if(direction===1) {
				otherinfo.row.after(info.row);
			} else {
				return;
			}
			this._cascadingrows[index+direction] = info;
			this._cascadingrows[index] = otherinfo;
		}
		this._cascadingAttributesChanged();
	};
	
	return CascadingAttributesSetupTable;
});