define([
	"component/searchresults/internal/SearchResultsControl",
	"component/searchresults/dashboard/SearchResultsDashboard",
	"component/searchresults/model/SearchModelFrontEndData",
	"component/searchresults/model/io/SearchModelTransferedJsonWriter",
	"lib/PromisePolyfill",
	"utils/HTTPUtils",
	"utils/idUtils",
	"component/searchresults/internal/ChildSearchResultsRerunSectionControl",
	"underscore"
], function (
	SearchResultsControl,
	SearchResultsDashboard,
	SearchModelFrontEndData,
	SearchModelTransferedJsonWriter,
	Promise,
	HTTPUtils,
	idUtils,
	ChildSearchResultsRerunSectionControl,
	_) {

	"use strict";

	/**
	 * @augments {SearchResultsControl}
	 * @class
	 * @param {JQueryNode} childSearchHTML - the html returned by the dosavedsearch.jsp call for this child search
	 * @param {ChildSearchRenderer} childTableRenderer, the renderer used to render the saved search into the parent table.
	 * @param {String} parentHpSectionId - used to adjust the checkboxes of the saved search, so that they will be enabled if the 'check all' checkbox is checked from the parent
	 */
	var ChildSearchResultsControl = function (childSearchHTML, childTableRenderer, parentHpSectionId) {
		var $childSearchHTML = $(childSearchHTML);
		var $div = $childSearchHTML.filter("[data-init='component/searchresults/internal/SearchResultsControl']");

		if ($div[0]) {
			$childSearchHTML.filter("form").appendTo(childTableRenderer.endRow);
			$div.find("#advancedsearchform").find("input").appendTo(childTableRenderer.endRow);

			$div.find("[name='selectproject']").attr("data-hpSectionId", parentHpSectionId);

			// The JSON information which is passed...
			var searchDataJSON = $.parseJSON($div.attr("data-searchdata"));

			// The Textarea in which the Search Setup Model is put.
			var setupTextarea = $("> textarea[name=searchSetupModel]", $div);

			// Read the passed JSON data about the fields etc
			var searchData = new SearchModelFrontEndData(searchDataJSON);
			this._childTableRenderer = childTableRenderer;
			this._dashboardElement = childTableRenderer.dashboardRow;

			SearchResultsControl.call(this, $div[0], searchData, setupTextarea);
		} else {
			childTableRenderer.addNoChildrenRow();
		}
	};

	ChildSearchResultsControl.prototype = Object.create(SearchResultsControl.prototype);
	ChildSearchResultsControl.prototype.construct = ChildSearchResultsControl;

	/**
	 * @override
	 */
	ChildSearchResultsControl.prototype._initSearchResultsRerunSectionControl = function (searchresultstable, mainsearchresultscomponent, setupInfo, searchData) {
		return new ChildSearchResultsRerunSectionControl(searchresultstable, mainsearchresultscomponent, setupInfo, searchData, this._childTableRenderer);
	};

	/**
	 * @override
	 */
	ChildSearchResultsControl.prototype._doDashboardInit = function () {
		return SearchResultsControl.prototype._doDashboardInit.apply(this, arguments);
	};

	/**
	 * @override
	 */
	ChildSearchResultsControl.prototype._getDashboardElement = function () {
		this._dashboardElement.removeClass("no-dashboard").show(); // this method is called if there is a dasbhoard, so remove the no-dashboard class
		return this._dashboardElement.find("div");
	};

	/**
	 * @Override don't display search results table in child search dashboard
	 * @private
	 */
	ChildSearchResultsControl.prototype._getDashboardContext = function () {
		var context = SearchResultsControl.prototype._getDashboardContext.call(this);
		// We do not want to display the results as part of the dashboard, use dummy div
		context.getResultsTable = function () {
			return {
				getElement: function () {
					return $("<div/>");
				}
			};
		};

		return context;
	};


	/**
	 * @override
	 * use the parent's searchsetup model for the child search
	 */
	ChildSearchResultsControl.prototype._getRerunParameters = function (params) {
		SearchResultsControl.prototype._getRerunParameters.call(this, params);

		var serializedSearchSetupModel = {};
		SearchModelTransferedJsonWriter.write(this.searchSetup, serializedSearchSetupModel);

		params.searchSetupModel = JSON.stringify(serializedSearchSetupModel);
	};

	/** 
	 * do a child search
	 */
	ChildSearchResultsControl.doSearch = function (row, projectId, savedSearch, timestamp, setupInfo, searchSettings) {
		return getSavedSearchHTML(projectId, savedSearch, timestamp, setupInfo, searchSettings);
	};


	/** 
	 * gets the HTML from the saved search.
	 * The _.memoize will cache the results based on the project Id, savedSearch name and a timestamp.
	 *
	 */
	var getSavedSearchHTML = _.memoize(function (projectId, savedSearch, timestamp, setupinfo, searchSettings) {
		var serializedSearchSetupModel = {};
		SearchModelTransferedJsonWriter.write(setupinfo, serializedSearchSetupModel); // serialize the setup information
		var hpSectionId = idUtils.newClientSideId("inline-drilldown");
		var url = HTTPUtils.constructURL(window.webcenterConfig.jspBaseUrl + "dosavedsearch.jsp", {
			'searchName': savedSearch,
			"ajax": 1,
			"isDrilldown": 1,
			"sectionID": hpSectionId,
			"projectID": projectId,
			useDefaultDrilldown: savedSearch ? 0 : 1,
			showSimpleGUI: searchSettings.preferences.simpleGUI
		});

		// searialize it to be form url encoded
		var searchSetupInfoSerialized = $("<form>").append($("<input>", {
			name: "searchSetupModel",
			value: JSON.stringify(serializedSearchSetupModel)
		})).serialize();

		return new Promise(function (resolve, reject) {
			$.post(url, searchSetupInfoSerialized).done(function (data) {
				resolve(data);
			}).fail(function (data) {
				reject(data);
			});
		});
	}, function (projectId, savedSearch, timestamp) {
		return projectId + "-" + savedSearch + "-" + timestamp;
	});

	/**
	 * (internal) Do a rerun because the paging has changed
	 * 
	 * Called when the user clicks one of the paging links.
	 * 
	 * @param startIndex - index of the first item of the results
	 * @param endIndex - index of the last item of the results
	 */
	ChildSearchResultsControl.prototype.changePageAndRerun = function (startIndex, endIndex) {
		var paramFormElem = document.getElementById(this.hpSectionId + '_display_params_form');
		if (paramFormElem) {
			paramFormElem.StartIndex.value = startIndex;
			paramFormElem.EndIndex.value = endIndex;
		}

		// We do not want to empty the rows when we da a page rerun.
		this.doRerun(false, {
			doNotEmptyTable: true
		});
	};

	return ChildSearchResultsControl;
});
