/**
 * Defines the API one can use to make/remove design panel components from/to the search setup page.
 */
define(["underscore",
        "../model/SearchSetupInformation",
        "../model/SearchModelFrontEndData",
        "./SearchParameterFieldControl"
        ], function(_,
        		SearchSetupInformation,
        		SearchModelFrontEndData,
        		SearchParameterFieldControl
        ) {
	
	/**
	 * @constructor
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (which values are set?)
	 * @param {SearchModelFrontEndData} searchData - information about the search data send from the server
	 * @param {SearchParameterFieldControl} paramFieldControl - the controller which manages ParameterFields.
	 */
	var FieldLayoutAPI = function(setupInfo, searchData, paramFieldControl){
		// Check constructor arguments
		if(!( 
				setupInfo && setupInfo instanceof SearchSetupInformation &&
				searchData && searchData instanceof SearchModelFrontEndData && 
				paramFieldControl && paramFieldControl instanceof SearchParameterFieldControl)) {
			throw new Error("Illegal arguments passed to FieldLayoutAPI");
		}
		
		this._setupInfo = setupInfo;
		this._searchData = searchData;
		this._paramFieldControl = paramFieldControl;
	};
	
	/**
	 * <p>Makes sure the information for a certain fieldid is loaded before using it.</p>
	 * 
	 * <p>Note that all calls to this function which work on the same field will be guaranteed to complete in the same order.</p>
	 * 
	 * @param {String} fieldId - the field id
	 * @param {Function<SearchField>} callback - the callback that will be executed (might be called immediately!)
	 */
	FieldLayoutAPI.prototype._withSearchField = function(fieldId, callback) {
		// Check first if we have the information for this field...
		this._searchData.requireFieldInformationAvailable(fieldId, _.bind(function(){
			// Lookup the field
			var searchfield = this._searchData.getFieldInformation(fieldId, this._setupInfo);
			// Do the callback (in the correct oder) and pass the searchfield
			callback.call(this, searchfield);
		}, this));
	};
	
	/**
	 * Get the current list of all criterion id's which are in the search setup (they have some information)
	 * @returns {String[]} the ids
	 */
	FieldLayoutAPI.prototype.getListOfAllCriterionIds = function() {
		return _.union(
				this.getListOfCriterionIds(),
				this.getListOfColumnDisplayCriterionIds(),
				this.getListOfDesignPanelCriterionIds(),
				this.getListOfSortingCriterionIds()
			);
	};
	
	/**
	 * Get the current list of criterion id's which are in the search setup (they have some information)
	 * @returns {String[]} the ids
	 */
	FieldLayoutAPI.prototype.getListOfCriterionIds = function() {
		var list = [];
		this._setupInfo.getCriteriaList().each(function(criterion) {
			list.push(criterion.getFieldId());
		});
		return list;
	};
	
	/**
	 * Get the current list of column display criterion id's which are in the search setup (they have some information)
	 * @returns {String[]} the ids
	 */
	FieldLayoutAPI.prototype.getListOfColumnDisplayCriterionIds = function() {
		var list = [];
		this._setupInfo.getDisplayColumns().each(function(displayColumn) {
			list.push(displayColumn.getFieldId());
		});
		return list;
	};
	
	/**
	 * Set the view checkbox by default by setting the SearchDisplayColumn settings
	 * 
	 * @param fieldId
	 * @param value
	 */
	FieldLayoutAPI.prototype.setColumnDisplayView = function(fieldId, value) {
		// Lookup the data
		var criterion = this._setupInfo.getDisplayColumns().findByFieldId(fieldId);
		if(!criterion) {
			criterion = this._setupInfo.getDisplayColumns().createItem(fieldId);
			this._setupInfo.getDisplayColumns().add(criterion);
		}
		criterion.setEnabled(value);
	};
	
	/**
	 * Get the current list of design panel criterion id's which are in the search setup (they have some information)
	 * @returns {String[]} the ids
	 */
	FieldLayoutAPI.prototype.getListOfDesignPanelCriterionIds = function() {
		var list = [];
		this._setupInfo.getDesignPanelItems().each(function(designPanelItem) {
			list.push(designPanelItem.getFieldId());
		});
		return list;
	};
	
	/**
	 * Get the current list of sorting criterion id's which are in the search setup (they have some information)
	 * @returns {String[]} the ids
	 */
	FieldLayoutAPI.prototype.getListOfSortingCriterionIds = function() {
		var list = [];
		this._setupInfo.getSortingCriteriaList().each(function(sortingCriterion) {
			list.push(sortingCriterion.getFieldId());
		});
		return list;
	};
	
	/**
	 * Make a Parameter Field (and a criterion if there does not exist one)
	 * 
	 * @param {String} fieldId - the id of the field to make the parameter field for
	 * @param {DOMElement} div - the div to put the ParameterField in
	 */
	FieldLayoutAPI.prototype.makeParameterField = function(fieldId, div, callback) {
		// Check first if we have the information for this field...
		this._withSearchField(fieldId, function(){
			// Make the field
			this._paramFieldControl.makeParameterField(fieldId, div);
            if(callback){
                callback();
            }
		});
	};
	
	/**
	 * Destroy a Parameter Field
	 * 
	 * @param {DOMElement} div - the div in which we placed the ParameterField.
	 */
	FieldLayoutAPI.prototype.destroyParameterField = function(div) {
		// TODO after 16.1: Need to call _withSearchField for correctness (but at the moment it can never go wrong)
		this._paramFieldControl.destroyParameterField(div);
	};
	
	/**
	 * Focus a Parameter Field
	 * 
	 * @param {DOMElement} div - the div in which we placed the ParameterField.
	 */
	FieldLayoutAPI.prototype.focusParameterField = function(div) {
		// TODO after 16.1: Need to call _withSearchField for correctness (but at the moment it can never go wrong)
		this._paramFieldControl.focusParameterField(div);
	};
	
	/**
	 * Register a Column Display Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the ColumnDisplaySettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {String} fieldId
	 * @param {ColumnDisplaySettingsController} control 
	 */
	FieldLayoutAPI.prototype.registerFieldColumnDisplaySettingsController = function(fieldId, control) {
		// Check if we get correct parameters...
		if(!(
				// fieldId should be an non-null string
				fieldId && typeof(fieldId) === "string" &&
				
				// columnDisplaySettings needs to be an object...
				control && 
				typeof(control)==="object" &&
				
				// and have a method configure(SearchField, SearchDisplayColumn)
				typeof(control.configure)==="function" &&
				
				// and have a method destroy()
				typeof(control.destroy)==="function"
				
				// those are all the requirements...
				)) {
			throw new Error("Illegal object passed to registerFieldColumnDisplaySettingsController in FieldLayoutAPI");
		}
		
		// Check first if we have the information for this field...
		this._withSearchField(fieldId, function(searchfield){
			// Lookup the data
			var criterion = this._setupInfo.getDisplayColumns().findByFieldId(fieldId);
			if(!criterion) {
				criterion = this._setupInfo.getDisplayColumns().createItem(fieldId);
				this._setupInfo.getDisplayColumns().add(criterion);
			}
			
			// configure the component
			control.configure(searchfield, criterion);
		});
	};
	
	/**
	 * Unregister a Column Display Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the ColumnDisplaySettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {ColumnDisplaySettingsController} control
	 */
	FieldLayoutAPI.prototype.unregisterFieldColumnDisplaySettingsController = function(control) {
		// destroy the component
		control.destroy();
	};
	
	/**
	 * Register a Design Panel Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the DesignPanelSettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {String} fieldId
	 * @param {DesignPanelSettingsController} control 
	 */
	FieldLayoutAPI.prototype.registerFieldDesignPanelSettingsController = function(fieldId, control) {
		// Check if we get correct parameters...
		if(!(
				// fieldId should be an non-null string
				fieldId && typeof(fieldId) === "string" &&
				
				// designPanelSettings needs to be an object...
				control && 
				typeof(control)==="object" &&
				
				// and have a method configure(SearchField, SearchDesignPanelItem)
				typeof(control.configure)==="function" && 

				// and have a method destroy()
				typeof(control.destroy)==="function" 
				
				// those are all the requirements...
				)) {
			throw new Error("Illegal object passed to registerFieldDesignPanelSettingsController in FieldLayoutAPI");
		}
		
		// Check first if we have the information for this field...
		this._withSearchField(fieldId, function(searchfield){
			// Lookup the data
			var criterion = this._setupInfo.getDesignPanelItems().findByFieldId(fieldId);
			if(!criterion) {
				criterion = this._setupInfo.getDesignPanelItems().createItem(fieldId);
				this._setupInfo.getDesignPanelItems().add(criterion);
			}
			
			// configure the component
			control.configure(searchfield, criterion);
		});
	};
	
	/**
	 * Unregister a Design Panel Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the DesignPanelSettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {DesignPanelSettingsController} control
	 */
	FieldLayoutAPI.prototype.unregisterFieldDesignPanelSettingsController = function(control) {
		// destroy the component
		control.destroy();
	};
	
	/**
	 * Register a Sorting Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the SortingSettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {String} fieldId
	 * @param {SortingSettingsController} control 
	 */
	FieldLayoutAPI.prototype.registerFieldSortingSettingsController = function(fieldId, control) {
		// Check if we get correct parameters...
		if(!(
				// fieldId should be an non-null string
				fieldId && typeof(fieldId) === "string" &&
				
				// sortingSettings needs to be an object...
				control && 
				typeof(control)==="object" &&
				
				// and have a method configure(SearchField, SearchSortingCriterion)
				typeof(control.configure)==="function" &&

				// and have a method destroy()
				typeof(control.destroy)==="function"
				
				// those are all the requirements...
				)) {
			throw new Error("Illegal object passed to registerFieldSortingSettingsController in FieldLayoutAPI");
		}

		// Check first if we have the information for this field...
		this._withSearchField(fieldId, function(searchfield){
			// Lookup the data
			var criterion = this._setupInfo.getSortingCriteriaList().findByFieldId(fieldId);
			if(!criterion) {
				criterion = this._setupInfo.getSortingCriteriaList().createItem(fieldId);
				this._setupInfo.getSortingCriteriaList().add(criterion);
			}
			
			// configure the component
			control.configure(searchfield, criterion);
		});
	};
	
	/**
	 * Unregister a Sorting Settings Controller for a certain fieldid.
	 * 
	 * The controller needs to implement the SortingSettingsController interface.
	 * See UDD diagram for an overview of the interface and the available implementations.
	 * 
	 * @param {SortingSettingsController} control
	 */
	FieldLayoutAPI.prototype.unregisterFieldSortingSettingsController = function(control) {
		// destroy the component
		control.destroy();
	};
	
	/**
	 * Tries to copy the criterion values from one criterion to another.
	 * Note, will fail silently if some Parameter Panel is editing the criterion you try to copy to.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryCopyAllInformationForField = function(fromFieldId, toFieldId) {
		this.tryCopyCriterion(fromFieldId, toFieldId);
		this.tryCopyDisplayColumnCriterion(fromFieldId, toFieldId);
		this.tryCopyDesignPanelCriterion(fromFieldId, toFieldId);
		this.tryCopySortingCriterion(fromFieldId, toFieldId);
	};
	
	/**
	 * Tries to copy the criterion values from one criterion to another.
	 * Note, will fail silently if the destination criterion already exists.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryCopyCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getCriteriaList().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getCriteriaList().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getCriteriaList().createItem(toFieldId);
			destCriterion.copyFrom(srcCriterion);
			this._setupInfo.getCriteriaList().add(destCriterion);
		}
	};
	
	/**
	 * Tries to copy the criterion values from one criterion to another.
	 * Note, will fail silently if the destination criterion already exists.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryCopyDisplayColumnCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getDisplayColumns().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getDisplayColumns().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getDisplayColumns().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getDisplayColumns().add(destCriterion);
		}
	};
	
	/**
	 * Tries to copy the criterion values from one criterion to another.
	 * Note, will fail silently if the destination criterion already exists.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryCopyDesignPanelCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getDesignPanelItems().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getDesignPanelItems().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getDesignPanelItems().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getDesignPanelItems().add(destCriterion);
		}
	};
	
	/**
	 * Tries to copy the criterion values from one criterion to another.
	 * Note, will fail silently if the destination criterion already exists.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryCopySortingCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getSortingCriteriaList().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getSortingCriteriaList().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getSortingCriteriaList().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getSortingCriteriaList().add(destCriterion);
		}
	};
	
	/**
	 * Tries to move the criterion values from one criterion to another.
	 * Note, will copy if cannot move.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryMoveAllInformationForField = function(fromFieldId, toFieldId) {
		this.tryMoveCriterion(fromFieldId, toFieldId);
		this.tryMoveDisplayColumnCriterion(fromFieldId, toFieldId);
		this.tryMoveDesignPanelCriterion(fromFieldId, toFieldId);
		this.tryMoveSortingCriterion(fromFieldId, toFieldId);
	};
	
	/**
	 * Tries to move the criterion values from one criterion to another.
	 * Note, will copy if cannot move.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryMoveCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getCriteriaList().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getCriteriaList().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getCriteriaList().createItem(toFieldId);
			destCriterion.copyFrom(srcCriterion);
			this._setupInfo.getCriteriaList().tryToRemoveItem(srcCriterion);
			this._setupInfo.getCriteriaList().add(destCriterion);
		}
	};
	
	/**
	 * Tries to move the criterion values from one criterion to another.
	 * Note, will copy if cannot move.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryMoveDisplayColumnCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getDisplayColumns().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getDisplayColumns().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getDisplayColumns().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getDisplayColumns().tryToRemoveItem(srcCriterion);
			this._setupInfo.getDisplayColumns().add(destCriterion);
		}
	};
	
	/**
	 * Tries to move the criterion values from one criterion to another.
	 * Note, will copy if cannot move.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryMoveDesignPanelCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getDesignPanelItems().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getDesignPanelItems().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getDesignPanelItems().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getDesignPanelItems().tryToRemoveItem(srcCriterion);
			this._setupInfo.getDesignPanelItems().add(destCriterion);
		}
	};
	
	/**
	 * Tries to move the criterion values from one criterion to another.
	 * Note, will copy if cannot move.
	 * 
	 * @param fromFieldId
	 * @param toFieldId
	 */
	FieldLayoutAPI.prototype.tryMoveSortingCriterion = function(fromFieldId, toFieldId) {
		var srcCriterion = this._setupInfo.getSortingCriteriaList().findByFieldId(fromFieldId);
		var destCriterion = this._setupInfo.getSortingCriteriaList().findByFieldId(toFieldId);
		if(srcCriterion && !destCriterion) {
			destCriterion = this._setupInfo.getSortingCriteriaList().createItem(toFieldId);
			this._withSearchField(toFieldId, function(searchfield){
				destCriterion.safeCopyFrom(searchfield, srcCriterion);
			});
			this._setupInfo.getSortingCriteriaList().tryToRemoveItem(srcCriterion);
			this._setupInfo.getSortingCriteriaList().add(destCriterion);
		}
	};
	
	/**
	 * Tries to delete all "criteria information" for a field.
	 * Note, will fail silently if some component is still editing the criterion.
	 * 
	 * @param fieldId
	 */
	FieldLayoutAPI.prototype.tryDeleteAllInformationForField = function(fieldId) {
		this.tryDeleteCriterion(fieldId);
		this.tryDeleteDisplayColumnCriterion(fieldId);
		this.tryDeleteDesignPanelCriterion(fieldId);
		this.tryDeleteSortingCriterion(fieldId);
	};
	
	/**
	 * Tries to delete a criterion.
	 * Note, will fail silently if some Parameter Panel is still editing the criterion.
	 * 
	 * @param fieldId
	 */
	FieldLayoutAPI.prototype.tryDeleteCriterion = function(fieldId) {
		var criterion = this._setupInfo.getCriteriaList().findByFieldId(fieldId);
		if(criterion) {
			this._setupInfo.getCriteriaList().tryToRemoveItem(criterion);
		}
	};
	
	/**
	 * Tries to delete a display column criterion.
	 * Note, will fail silently if some component is still editing the criterion.
	 * 
	 * @param fieldId
	 */
	FieldLayoutAPI.prototype.tryDeleteDisplayColumnCriterion = function(fieldId) {
		var criterion = this._setupInfo.getDisplayColumns().findByFieldId(fieldId);
		if(criterion) {
			this._setupInfo.getDisplayColumns().tryToRemoveItem(criterion);
		}
	};
	
	/**
	 * Tries to delete a design panel criterion.
	 * Note, will fail silently if some component is still editing the criterion.
	 * 
	 * @param fieldId
	 */
	FieldLayoutAPI.prototype.tryDeleteDesignPanelCriterion = function(fieldId) {
		var criterion = this._setupInfo.getDesignPanelItems().findByFieldId(fieldId);
		if(criterion) {
			this._setupInfo.getDesignPanelItems().tryToRemoveItem(criterion);
		}
	};
	
	/**
	 * Tries to delete a sorting criterion.
	 * Note, will fail silently if some component is still editing the criterion.
	 * 
	 * @param fieldId
	 */
	FieldLayoutAPI.prototype.tryDeleteSortingCriterion = function(fieldId) {
		var criterion = this._setupInfo.getSortingCriteriaList().findByFieldId(fieldId);
		if(criterion) {
			this._setupInfo.getSortingCriteriaList().tryToRemoveItem(criterion);
		}
	};
	
	/**
	 * Get the label to use for a certain fieldId.
	 * 
	 * @param {String} fieldId - the fieldid to look up the label for
	 * @param {Function} callback - function with one argument - the label to use.
	 */
	FieldLayoutAPI.prototype.getSetupLabel = function(fieldId, callback) {
		// Check first if we have the information for this field...
		this._withSearchField(fieldId, function(searchfield){
			callback(searchfield.getDefaultLabel());
		});
	};
	
	/**
	 * <p>Get some external settings/preferences.<br/>This object contains information about general WebCenter settings that are not related to the currently processed Search Setup</p>
	 * 
	 * Tip: See "wcrSettings" in SearchModelFrontEndData.java for available settings.
	 * 
	 * @returns {ExternalSearchSettings} the external settings
	 */
	FieldLayoutAPI.prototype.getExternalSearchSettings = function() {
		return this._searchData.getSearchSettings();
	};
	
	/**
	 * Get the layout options part of the object model.<br/>
	 * In the layout options you can set or get any preference of how the search results or setup should be shown.
	 * 
	 * @returns {SearchLayoutOptions} - the layout options
	 */
	FieldLayoutAPI.prototype.getLayoutOptions = function() {
		return this._setupInfo.getLayoutOptions();
	};
	
	/**
	 * Change the order of display columns & sorting criteria & design panel items in the object model.
	 * (For altering the order in Excel & the available dropdown on the search results)
	 * 
	 * @param {Array{String}} fieldids - the fieldids in the order they should be in the object model 
	 */
	FieldLayoutAPI.prototype.updateObjectModelOrder = function(fieldids) {
		this._setupInfo.getDisplayColumns().reorderItemsByIds(fieldids);
		this._setupInfo.getCriteriaList().reorderItemsByIds(fieldids);
		this._setupInfo.getDesignPanelItems().reorderItemsByIds(fieldids);
		this._setupInfo.getSortingCriteriaList().reorderItemsByIds(fieldids);
	};
	
	/**
	 * Watch an object in the list by an id.
	 * This will not prevent the object from being removed from the list and should not be used if you need to "synchronize" with the information.
	 * 
	 * You will never get write access to an item in the list by calling the watch method.
	 * 
	 * The callback will be called also immediately when the watch function is called.
	 * 
	 * @param fieldid
	 * @param callback
	 */
	FieldLayoutAPI.prototype.watch = function (fieldId, callback) {
		this._setupInfo.getCriteriaList().watch(fieldId, callback);
	};
	
	/**
	 * Unwatch changes to an object in the list by an id. (inverse of watchWithoutReference)
	 * 
	 * @param fieldid
	 * @param callback
	 */
	FieldLayoutAPI.prototype.unwatch = function(fieldid, callback) {
		this._setupInfo.getCriteriaList().unwatch(fieldId, callback);
	};
	
	
	return FieldLayoutAPI;
});