/**
 * Controls the layouts...
 */
define(["jquery", 
        "underscore",
        
        // Search Layout Loader
        "./render/SearchLayoutLoader"
        ], function($, _,
        		SearchLayoutLoader){
	
	/**
	 * @constructor
	 */
	var SearchLayoutControl = function(){
		this._api = null;
		
		// Loading state
		this._layoutStillLoading = 0;
		this._loadingcallbacks = [];
	};
	
	/**
	 * Set the FieldLayoutAPI instance to pass to the layouts.
	 * This api allows them to make and remove ParameterFields and all kinds of components.
	 * 
	 * @param {FieldLayoutAPI} api - the api
	 */
	SearchLayoutControl.prototype.setApiInstance = function(api) {
		this._api = api;
	};
	
	/**
	 * Register a new layout... A layout needs to be implemented to make Parameter Fields using JavaScript...
	 * 
	 * @param {String} layoutMethod - the way to layout
	 * @param {Element} div - the element to execute the layout in
	 * @param {Object} additionalData - some additional data which can be passed to the layouts
	 */
	SearchLayoutControl.prototype.registerLayout = function(layoutMethod, div, additionalData) {
		// Check arguments
		if(!(	div && 1 === div.nodeType /* check if it is a HTML Element */ && 
				layoutMethod && typeof(layoutMethod) === "string")) {
			throw new Error("Illegal arguments passed to SearchLayoutControl:registerLayout");
		}
		if(!this._api) {
			throw new Error("Illegal state: api is null in SearchLayoutControl:registerLayout");
		}
		
		this._layoutStillLoading++;
		var layoutImpl = new SearchLayoutLoader(div, layoutMethod, this._api, additionalData);
		layoutImpl.whenLoaded(_.bind(function(){
			this._layoutStillLoading--;
			this._checkAllLoaded();
		}, this));
	};
	
	//
	// Callbacks...
	//
	
	/**
	 * Are all the layouts loaded?
	 * 
	 * @returns {Boolean}
	 */
	SearchLayoutControl.prototype.isLoaded = function(){
		return this._layoutStillLoading===0;
	};
	
	/**
	 * Will wait till all fields are loaded and then invoke the callback.
	 * The callback will be invoked instantly if all fields are already loaded.
	 * 
	 * Note... The Design Panel could go back into loading state if you add a new field to it...
	 * The callback will only be executed the first time it is done loading after you attach it!
	 */
	SearchLayoutControl.prototype.whenLoadedOnce = function(callback){
		if(this.isLoaded()) {
			callback();
		}else{
			this._loadingcallbacks.push(callback);
		}
	};
	
	/**
	 * Called internally to check if all components are loaded...
	 * 
	 * Does the loaded callbacks...
	 */
	SearchLayoutControl.prototype._checkAllLoaded = function() {
		if(this.isLoaded()) {
			$.each(this._loadingcallbacks, function(i, callback) {
				callback();
			});
			this._loadingcallbacks = [];
		}
	};
	
	
	return SearchLayoutControl;
});