/**
 * Controls the design panel components
 */
define(["jquery", 
        "underscore",
        
        // The thing used to render a field
        "./render/ParameterField",
        
        // Search Model
        "../model/SearchSetupInformation",
        "../model/SearchModelFrontEndData",
        
        // Data objects
        "../dataobjects/ValueData",
        "../dataobjects/SearchSettings",
        "../dataobjects/DataProvider",
        
        // Keeps track of special characters usage
        "./functionality/SpecialCharactersHintController"
        ], function($, _,
        		ParameterField,
        		SearchSetupInformation,
        		SearchModelFrontEndData,
        		ValueData,
        		SearchSettings,
        		DataProvider,
        		SpecialCharactersHintController
        ){
	
	/**
	 * @constructor
	 * 
	 * @param {jQuery} div - a jQuery element that contains all the design panel html.
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (which values are set?)
	 * @param {SearchModelFrontEndData} searchData - information about the search data send from the server
	 */
	var SearchParameterFieldControl = function(div, setupInfo, searchData, isSearchResults){
		// Check constructor arguments
		if(!(	div && div instanceof $ && 
				setupInfo && setupInfo instanceof SearchSetupInformation &&
				searchData && searchData instanceof SearchModelFrontEndData)) {
			throw new Error("Illegal arguments passed to SearchParameterFieldControl");
		}
		
		// Init fields
		this.div = div;
		this._setupInfo = setupInfo;
		this._searchData = searchData;
		this._isSearchResults = isSearchResults;
		
		// Loading state
		this._componentsStillLoading = 0;
		this._loadingcallbacks = [];
		
		// Rerun callbacks...
		this._rerunCallbacks = [];
		
		// Replaced implementations of ParameterFields factories
		this._replacedFactoryMap = {};
		
		//
		// Initialize special characters hint util
		//
		if(!this._isSearchResults) {
			// Special characters hint controller
			this._charHint = new SpecialCharactersHintController(div, setupInfo);
		}
		
		// Keep a list of DOMElement containing ParameterFields to be able to do "Clear this Form".
		this._list = [];
	};
	
	//
	// Adding or removing Parameter Field.
	//
	
	/**
	 * Make a Parameter Field (and a criterion if there does not exist one)
	 * 
	 * @param {String} fieldid - the id of the field on which the design panel component works...
	 * @param {DOMElement} container - the div in which we want to show the design panel component
	 */
	SearchParameterFieldControl.prototype.makeParameterField = function(fieldid, container) {
		this._componentsStillLoading ++;
		
		// Search Settings + External Settings
		var isSmartNamesEnabled = // If the search has SmartNames enabled
			this._setupInfo.getFeatures().getUseSmartNames() &&
			// And the user is an admin
			this._searchData.getSearchSettings().getIsAdmin() &&
			// And we are not on the search results page
			!this._isSearchResults;
		var externalSearchSettings = this._searchData.getSearchSettings();
		var criteriaList = this._setupInfo.getCriteriaList();
		var watchFunc = _.bind(criteriaList.watch, criteriaList);
		var unwatchFunc = _.bind(criteriaList.unwatch, criteriaList);
		var searchSettings = new SearchSettings(isSmartNamesEnabled, externalSearchSettings, watchFunc, unwatchFunc);
		
		// Do we need to use a special type of factory?
		// Only used for cascading dropdowns at the moment
		var overrulefactory = undefined;
		if(_.has(this._replacedFactoryMap, fieldid)) {
			overrulefactory = this._replacedFactoryMap[fieldid];
		}
		
		//
		// Make the Parameter Field component
		//
		var paramfield = new ParameterField(fieldid, container, this._setupInfo, this._searchData, searchSettings, overrulefactory);
		
		// For unloading...
		container.ParameterFieldComponent = paramfield;
		this._list.push(container);
		
		// "Loading Done" callback
		paramfield.onDoneLoading(_.bind(function(){
			// Field has been loaded...
			this._componentsStillLoading--;
			this._checkAllLoaded();
		}, this));
		
		// "Requires rerun" callback
		paramfield.onRequiresRerun(_.bind(function(intermediateUpdate){
			// Field requires rerun...
			this._triggerNeedsRerun(intermediateUpdate);
		}, this));
		
		// "Special Characters" callback
		if(!this._isSearchResults) {
			paramfield.onStartsUsingSpecialCharacters(_.bind(function(inInitialValue) {
				// Field starts using special characters
				if(inInitialValue) {
					// We are already using special characters in the initial value...
					this._charHint.setDontAskAgain();
				}else{
					// We just started using special characters in this field
					this._charHint.trigger();
				}
			}, this));
		}
	};
	
	/**
	 * Destroy a Parameter Field
	 * @param {DOMElement} container - the div in which we show the design panel component
	 */
	SearchParameterFieldControl.prototype.destroyParameterField = function(container) {
		var comp = container.ParameterFieldComponent;
		if(!comp.isLoaded()) {
			this._componentsStillLoading--;
			this._checkAllLoaded();
		}
		
		this._list.splice(_.indexOf(this._list, container), 1);
		comp.destroy();
	};
	
	/**
	 * Focus a Parameter Field
	 * @param {DOMElement} container - the div in which we show the design panel component
	 */
	SearchParameterFieldControl.prototype.focusParameterField = function(container) {
		var paramfield = container.ParameterFieldComponent;
		if(paramfield) {
			paramfield.onDoneLoading(_.bind(function(){
				paramfield.focus();
			}, this));
		}
	};
	
	/**
	 * <p>Replaces all previously and future created components for this fieldid with the component provided by the new factory.</p>
	 * 
	 * <p>Only use this way of creating ParameterFields if the ParameterFields are not modular enough to be created in the normal way. (Cascading dropdowns)</p>
	 * 
	 * @param fieldid
	 * @param newfactory - the factory to use for new components of this fieldid
	 */
	SearchParameterFieldControl.prototype.replaceImplementations = function(fieldid, newfactory) {
		// Look for all the ParameterFields with this id
		var replaceList = [];
		_.each(this._list, function(container) {
			if(container.ParameterFieldComponent.getFieldId() === fieldid) {
				replaceList.push(container);
			}
		}, this);
		
		// Now start replacing them
		_.each(replaceList, function(container) {
			// Destroy the old component nicely
			this.destroyParameterField(container);
			
			// Store the fact that all new items of this specific field (by fieldid) should use the new factory
			this._replacedFactoryMap[fieldid] = newfactory;
			
			// Make the component again
			this.makeParameterField(fieldid, container);
		}, this);
	};
	
	/**
	 * Clear this form (clear all the design panel components)
	 */
	SearchParameterFieldControl.prototype.clearForm = function() {
		// Note: only clears the inputs which are already loaded!
		_.each(this._list, _.bind(function(e) {
			var comp = e.ParameterFieldComponent;
			comp.clearField();
		}, this));
	};
	
	/**
	 * Validate the each parameter validate method before submit the search
	 * 
	 * @returns {Boolean} - returns true when all the parameter field validation true , if one of field validation is fails returns false
	 */
	SearchParameterFieldControl.prototype.validateAll = function() {
		// Note: only validate the inputs which are already loaded!
		var validate = true;
		_.each(this._list, _.bind(function(e) {
			var comp = e.ParameterFieldComponent;
			if(!comp.validate()){
				validate = false;
			}
			
		}, this));
		return validate;
	};
	
	/**
	 * Notify all the ParameterFields that a resize has happened.
	 * 
	 * Note that this will only be called on the fields that are already created. (which makes sense)
	 */
	SearchParameterFieldControl.prototype.resizeAll = function() {
		_.each(this._list, _.bind(function(e) {
			var comp = e.ParameterFieldComponent;
			comp.resize();
		}, this));
	};
	
	//
	// Callbacks...
	//
	
	/**
	 * Are all the fields loaded?
	 * 
	 * @returns {Boolean}
	 */
	SearchParameterFieldControl.prototype.isLoaded = function(){
		return this._componentsStillLoading===0;
	};
	
	/**
	 * Will wait till all fields are loaded and then invoke the callback.
	 * The callback will be invoked instantly if all fields are already loaded.
	 * 
	 * Note... The Design Panel could go back into loading state if you add a new field to it...
	 * The callback will only be executed the first time it is done loading after you attach it!
	 */
	SearchParameterFieldControl.prototype.whenLoadedOnce = function(callback){
		if(this.isLoaded()) {
			callback();
		}else{
			this._loadingcallbacks.push(callback);
		}
	};
	
	/**
	 * Will be called when one of the fields needs a rerun...
	 * 
	 * WARNING: Might be called multiple times in a very short amount of time (if multiple fields require rerun)
	 * 
	 * NOTE: You MIGHT want to wait (not needed) until all design panel components are loaded before doing the actual rerun.
	 * 
	 * @param {Function} callback - the callback to call when a rerun is needed. Has one parameter: intermediateUpdate - true if the input is still changing.
	 */
	SearchParameterFieldControl.prototype.onNeedsRerun = function(callback){
		this._rerunCallbacks.push(callback);
	};
	
	
	
	//
	// Private - Triggering callbacks
	//
	
	/**
	 * Used internally by this component...
	 * 
	 * Called when one of the fields needs a rerun.
	 * 
	 * @param intermediateUpdate - true if the input is still changing
	 */
	SearchParameterFieldControl.prototype._triggerNeedsRerun = function(intermediateUpdate){
		for(var i=0;i<this._rerunCallbacks.length;i++) {
			this._rerunCallbacks[i](intermediateUpdate);
		}
	};
	
	/**
	 * Called internally to check if all components are loaded...
	 * 
	 * Does the loaded callbacks...
	 */
	SearchParameterFieldControl.prototype._checkAllLoaded = function() {
		if(this.isLoaded()) {
			$.each(this._loadingcallbacks, function(i, callback) {
				callback();
			});
			this._loadingcallbacks = [];
		}
	};
	
	return SearchParameterFieldControl;
});