/**
 * 
 * ## Search Results Internal Javascript Functionality ##
 * 
 * [Use]
 * You should NOT use this component directly... 
 * This component expects a complete search results created by xsl, so it is not a re-usable component on it's own...
 * 
 * [How it works]
 * This component is attached to every Search Results Page...
 * It is initialized automatically using data-init and imports all the required javascript.
 * (look for data-init="component/searchresults/internal/SearchResultsControl" in the xsl)
 * 
 * [This is a work in progress...]
 * At the moment though (WCR 14.1) this component only contains the javascript functionality for 
 *  - "asset browser (partially)" &
 *  - "top panel cascading dropdowns" &
 *  - "new search fields design panel"
 *
 * NOTE: Gets only initialized once for a search results "group". 
 * (not re-initialized on re-run. See SearchResultsRerunSectionControl if you do want to re-initialize on re-run) 
 *
 */
define([
	// Design Panel & Layout Controller
	"./SearchParameterFieldControl",
	"./FieldLayoutAPI",

	// Model
	"../model/SearchSetupInformation",
	"../model/SearchModelFrontEndData",
	"../model/SearchField",

	// Model IO
	"../model/io/SearchModelTransferedJsonReader",
	"../model/io/SearchModelTransferedJsonWriter",

	// All functionality of the Results Table (editable actions, rendering, ...)
	"./SearchResultsRerunSectionControl",

	// Other functionality
	"./functionality/SearchResultsParameterPanel",
	"./attributecategory/SearchResultsAttributeCategoryControl",

	"component/searchresults/dashboard/configuration/DefaultSearchResultsConfiguration",
	"component/searchresults/dashboard/SearchResultsDashboard",

	"../Utils",

	"utils/HTTPUtils",

	"./expandmodes/ExpandModeModal",
	
	// Browser History Mgr
	"./functionality/SearchBrowserHistoryManager",

	// Cart Display
	"./components/cart/SearchDocCart",

	// Language Strings...
	"utils/webi18n!SearchInputs",

	"underscore",
	"jquery",

	// IE6 & IE7 JSON compatibility
	"wcr/json3.min",




], function (
	SearchParameterFieldControl,
	FieldLayoutAPI,
	SearchSetupInformation,
	SearchModelFrontEndData,
	SearchField,
	SearchModelTransferedJsonReader,
	SearchModelTransferedJsonWriter,
	SearchResultsRerunSectionControl,
	SearchResultsParameterPanel,
	SearchResultsAttributeCategoryControl,
	defaultSearchResultsConfiguration,
	SearchResultsDashboard,
	Utils,
	HTTPUtils,
	ExpandModeModal,
	SearchBrowserHistoryManager,
	SearchDocCart,
	bundle,
	_,
	$
) {
	"use strict";
	
	// Possible places where a search can be used
	var CONTEXT_DEFAULT = "default";
	var CONTEXT_MYWORK = "mywork";
	var CONTEXT_SELECTITEMPOPUP = "selectitempopup";
	var CONTEXT_SANDBOX = "sandbox";
	var CONTEXT_CHILDSEARCH = "childsearch";
	
	/**
	 * Main control component for the search results...
	 * 
	 * @constructor
	 * @param {Element} searchresultscontainer - the DOM Element containing the search results html.
	 * @param {SearchModelFrontEndData} searchData - data which is passed from the front-end
	 * @param {jQuery} setupTextarea - the textarea which contains a serialized "SearchSetupInformation" object 
	 *                                 with all information about the search Setup...
	 */
	var SearchResultsControl = function (searchresultscontainer, searchData, setupTextarea) {
		// Checking inputs
		if (!(searchresultscontainer && 1 === searchresultscontainer.nodeType /* check if it is a HTML Element */ &&
				searchData && searchData instanceof SearchModelFrontEndData &&
				setupTextarea && setupTextarea instanceof $)) {
			throw new Error("Illegal arguments passed to SearchResultsControl");
		}

		// Keep the div we are working in
		this.$div = $(searchresultscontainer);

		// Store the reference in the table
		this.$div.get(0).SearchResultsControl = this;

		// unique id for the search on this page...
		this.hpSectionId = this.$div.attr("data-hpsectionid");
		// hpSection prefix (for old javascript functionality)
		this.hpSectionPrefix = (this.hpSectionId ? this.hpSectionId + "_" : "");
		
		//
		// Where are we???
		// 
		// (FUTURE: make sure there is a cleaner way of doing this?)
		this._searchcontext = CONTEXT_DEFAULT;
		if($("#hpSection_container_"+this.hpSectionId).length>0) {
			// We are in a MyWork section
			this._searchcontext = CONTEXT_MYWORK;
		} else if (this.hpSectionId === '00001_9000000000') {
			// We are in a document/project source
			this._searchcontext = CONTEXT_SELECTITEMPOPUP;
		} else if (this.hpSectionId === '00001_8900000000') {
			// We are in a external system popup (iframe) => also known as "sandbox mode"
			this._searchcontext = CONTEXT_SANDBOX;
		} else if (this.hpSectionId.indexOf('00004_')===0 || this.hpSectionId.indexOf('00003_')===0) {
			// We are in a child saved search (drill down)
			this._searchcontext = CONTEXT_CHILDSEARCH;
		}
		
		//
		// Should we remember the configuration of THIS search when using the browser back button?
		// Well... That depends...
		// Rule of thumb: When we use the browser to go back to this page, can we be 100% we are in the same search?
		//
		// So
		//  * If we are in a document/project source popup
		//    If you use the back button to go back to the page containing the popup, the popup will be closed anyway.
		//    Or it could also be that there are multiple popups with the same id.
		//    Certainly do NOT remember the search configuration
		//  * If we are in an iframe (sandbox mode), do not remember history (you cannot use the back button anyway)
		//  * If we are in a child search (drill down), id's might be re-used on the same page when using the back button.
		//    If such an id would be re-used and we would remember history, 
		//    the design panel would unexpectedly show up for a different search (with the same id)
		//  * If we are in MyWork, this means that even if the user uses the back button, it will still be rerun automatically...
		//    So, no need to remember history => we will get the most recent search from the session.
		//    (We could remember it though, but we would simply execute the same search twice)
		//  * If we are in a project details or document details context.
		//    No issue there. If you return (with the browser back button) to the same page, it will contain the same search in the same project.
		//    So, ok to remember history
		//  * If we are in a "ad hoc" search or any search in the menu => Ok to remember history. (there is only one search on the page => we cannot get confused)
		// 
		this._rememberBrowserBack = this._searchcontext === CONTEXT_DEFAULT;

		// Used when we submit several ajax searches in quick succession 
		// We ensure that only the last request in the series updates the screen.
		// The rest are "discarded".
		this.rerunCounter = 0;

		// reRun url (where to submit the page to
		this.reRunURL = this.$div.attr("data-rerunurl");

		// the search results table & rerun container functionality
		this._reruncontainer = $(".searchreruncontainer", this.$div);
		
		// the object that handles the history of this search (using the browser back button)
		this._restored = false;
		if(this._rememberBrowserBack) {
			this._browserhistorymgr = new SearchBrowserHistoryManager(this.hpSectionId);
			
			// restore the search if we used the back button
			var restoreData = this._browserhistorymgr.getRestoreData();
			if(restoreData) {
				setupTextarea.text(restoreData);
				this._restored = true;
			}
		}

		// Information about the search setup
		// Read the passed JSON for the model...
		this.setupTextarea = setupTextarea;
		this.searchSetup = new SearchSetupInformation();
		SearchModelTransferedJsonReader.read(this.searchSetup, $.parseJSON(this.setupTextarea.text()));

		// Information about the fields, unit conversions and other "static" information
		this.searchData = searchData;

		// read facet information
		var additionalFieldDataString = $(".searchreruncontainer", this.$div).attr("data-additionalfielddata");
		this._parseAdditionalFieldData(additionalFieldDataString);

		// Make new parameter field controller
		this.parameterfields = new SearchParameterFieldControl(this.$div, this.searchSetup, this.searchData, true);
		this.parameterfields.onNeedsRerun(_.bind(function (intermediateUpdate) {
			if(!intermediateUpdate) {
				// Rerun with some timeout since multiple fields might Request a rerun at the same time...
				// (If they are reacting to the same event...)
				// Note: "doRerun" method (see below) will then get rid of the multiple reruns
				setTimeout(_.bind(this.doAutomaticRerun, this), 100);
			}
			
			// Even if this is an intermediate update => already make the search button available:
			this._topparameterpanel.setNeedsRerun(true);
		}, this));

		// Make the layout api
		this._layoutApi = new FieldLayoutAPI(this.searchSetup, this.searchData, this.parameterfields);

		// Make & initialize the Search Results Rerun Section javascript.
		this.rerunsectioncontrol = this._initSearchResultsRerunSectionControl(this._reruncontainer.get(0), this, this.searchSetup, this.searchData);
		// hide the search results table by default... dashboard will unhide if needed
		this.rerunsectioncontrol.getElement().addClass("hidden");

		// Register clear this form link
		// Note: look anywhere on the page! (but data-targetsection needs to match hpSectionId)
		$(".action-searchresultsclearform")
			.filter(_.bind(function (i, e) {
				return $(e).attr("data-targetsection") === this.hpSectionId;
			}, this))
			.click(_.bind(this.clearForm, this));

		this._isInitialized = false;
		
		
		// Make the top parameter panel
		this._topparameterpanel = new SearchResultsParameterPanel(this.$div.get(0), this.hpSectionId, this._layoutApi, this.searchSetup, this.searchData);
		// When clicking the search button, we need to rerun
		this._topparameterpanel.onSearchButtonClicked(_.bind(this.doRerun, this, true));

		// For debugging purposes only !
		this.DEBUGOBJ = {
			// Get full access to the internal data structure
			model: this.searchSetup,
			// Get full access to the available data
			data: this.searchData,
			// Get full access to the layout api
			api: this._layoutApi,
		};
		if (!window.DEBUGSEARCH) {
			window.DEBUGSEARCH = {};
		}
		window.DEBUGSEARCH[this.hpSectionPrefix + "results"] = this.DEBUGOBJ;

		this._doDashboardInit();
		
		if(this._restored) {
			// Note: we could also restore the page (in which case "true" is not correct)
			this.doRerun(true);
		}
	};

	/**
	 * Initialize all the components in the Search
	 * 
	 */
	SearchResultsControl.prototype._doInitialization = function () {
		if (!this._isInitialized) {
			this._isInitialized = true;

			// initialize Cart
			this._cart = new SearchDocCart(this.$div);
			this._cart.initialize();

			// Create Attribute Category functionality (asset browser/cascading dropdowns)
			this.attributeCategoryFunctionality = new SearchResultsAttributeCategoryControl(this.$div, this.searchSetup, this.searchData, this.parameterfields, this.DEBUGOBJ, this.doAutomaticRerun.bind(this), this._distributeResizeEvent.bind(this));
			this.attributeCategoryFunctionality.load();

			// Show the div
			$(".loadingcontainer", this.$div).remove();
			$(".searchloadingarea", this.$div).css("display", "");
		}
	};

	/**
	 * Get the Dashboard Configuration
	 * @private
	 * @returns {Object} plain Object with configuration of dashboard
	 */
	SearchResultsControl.prototype._getDashboardConfiguration = function () {

		var dashboardConfiguration = this.searchSetup.getDashboardConfiguration().getDashboard();

		if (!dashboardConfiguration) {
			dashboardConfiguration = defaultSearchResultsConfiguration;
		}

		return dashboardConfiguration;

	};

	/**
	 * Get the context information for the dashboard
	 * @private
	 * @returns {object} 
	 */
	SearchResultsControl.prototype._getDashboardContext = function () {
		return {
			setup: this.searchSetup,
			data: this.searchData,
			getResultsTable: function () {
				this._doInitialization();
				this.rerunsectioncontrol.getElement().removeClass("hidden");
				// FUTURE: The dashboard needs to be able to tell us when a column might have changed size!!
				// Now we are just guessing + resizing way too much things probably
				_.defer(this._distributeResizeEvent.bind(this));
				return this.rerunsectioncontrol;
			}.bind(this),
			getDesignPanel: function () {
				this._doInitialization();
				// FUTURE: The dashboard needs to be able to tell us when a column might have changed size!!
				// Now we are just guessing + resizing way too much things probably
				_.defer(this._distributeResizeEvent.bind(this));
				return this._topparameterpanel;
			}.bind(this),
			fieldLayoutAPI: this._layoutApi,
			getCart: function () {
				this._doInitialization();
				// FUTURE: The dashboard needs to be able to tell us when a column might have changed size!!
				// Now we are just guessing + resizing way too much things probably
				_.defer(this._distributeResizeEvent.bind(this));
				return this._cart;
			}.bind(this),
			getAttributeCategoryControl: function () {
				this._doInitialization();
				// FUTURE: The dashboard needs to be able to tell us when a column might have changed size!!
				// Now we are just guessing + resizing way too much things probably
				_.defer(this._distributeResizeEvent.bind(this));
				return this.attributeCategoryFunctionality;
			}.bind(this)


		};
	};

	/**
	 * init the dashboard
	 * @return {SearchResultsDashboard}
	 */
	SearchResultsControl.prototype._doDashboardInit = function () {
		// get the smartname info context information
		var smartnameInfo = {
			projectID: $('#' + this.hpSectionPrefix + 'projectID').val(),
			docVerID: $('#' + this.hpSectionPrefix + 'docVerID').val(),
			staticSmartNames: $('#' + this.hpSectionPrefix + 'staticSmartNames').val()
		};
		var queryParams = HTTPUtils.getQueryStringParams();
		_.each(["menuitem3", "menusel3", "menu_file"], function (param) {
			smartnameInfo[param] = queryParams[param];
		});
		smartnameInfo = _.pick(smartnameInfo, function (value, key) {
			return value;
		});


		// initialize the dashboard
		this._dashboard = new SearchResultsDashboard(this._getDashboardContext(), this._getDashboardConfiguration(), false, smartnameInfo);

	
		
		this._getDashboardElement().append(this._dashboard.getElement());
		

		this._dashboard.whenFullyLoaded().then(function () {
			this._doInitialization();
			
			// Also call the resize on the parameterfields after the dashboard is fully loaded
			this._distributeResizeEvent();
			
			// HOTFIX HF_20170811_CS00896686
			// Only initialize CardView when Dashboard is loaded and everything is in the DOM.
			if(this.rerunsectioncontrol.onDashboardReady) {
				// Not needed for merged drilldown searches (or at least: not yet needed in 16.1.1)
				this.rerunsectioncontrol.onDashboardReady();
			}
		}.bind(this));

		return this._dashboard;
	};

	/**
	 * init the searchResultsRerunSectionControl
	 * @param {Element} searchresultstable - The DOM element contain the results table html.
	 * @param {Object} mainsearchresultscomponent - Needs to have ONE function: doRerunAfterBatchActions
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (columns to show, ...)
	 * @param {SearchModelFrontEndData} searchData - Data which is passed from the server...
	 * @return {SearchResultsRerunSectionControl}
	 */
	SearchResultsControl.prototype._initSearchResultsRerunSectionControl = function (searchresultstable, mainsearchresultscomponent, setupInfo, searchData) {
		return new SearchResultsRerunSectionControl(searchresultstable, mainsearchresultscomponent, setupInfo, searchData);
	};

	/**
	 * Clear the form...
	 */
	SearchResultsControl.prototype.clearForm = function () {
		// Clear the "top parameter fields"
		this.parameterfields.clearForm();

		// Clear the "attribute category"
		if (this.attributeCategoryFunctionality) {
			this.attributeCategoryFunctionality.clearForm();
		}

		// Rerun
		this.doRerun();
	};

	/** 
	 * returns the element in which to display the dashboard
	 * @return {JQueryElement}
	 */
	SearchResultsControl.prototype._getDashboardElement = function () {
		return $(".searchdashboardcontainer", this.$div);
	};

	/**
	 * Method is called before every search submit...
	 * 
	 * Convert the object model we use internally to a string which will be send to the backend...
	 * 
	 * @param {Boolean} doValidation - whether to do validation
	 */
	SearchResultsControl.prototype.doBeforeSubmit = function (doValidation) {
		// 1) Serialize the model
		var serializedSearchSetupModel = {};
		SearchModelTransferedJsonWriter.write(this.searchSetup, serializedSearchSetupModel);

		// 2) Put it in the textarea before submitting the search...
		this.setupTextarea.text(JSON.stringify(serializedSearchSetupModel));
		
		if(doValidation) {
			
			// Validation for all the parameter fields .. like revision 
			var validation = this.parameterfields.validateAll();
			
			return validation;
		} else {
			
			return true;
		}
	};

	/**
	 * (internal)
	 * When the search panel is cleared, re-initialize some functionality
	 */
	SearchResultsControl.prototype._onSearchPanelCleared = function () {
		// At the moment, simply refresh all search top panel related functionality...
		this._initialize();
	};

	/**
	 * Get the hp section id (only for old javascript!!!)
	 * @returns
	 */
	SearchResultsControl.prototype.getLegacyHpSectionId = function () {
		return this.hpSectionId;
	};


	/**
	 * Toggle the loader icon for search
	 * 
	 * @param showIt
	 */
	SearchResultsControl.prototype._showHourGlassSearch = function (showIt) {
		this._dashboard.setLoadingData(showIt);
	};

	/**
	 * Reset search to the first page (need to rerun 
	 * 
	 * @param hpSectionId
	 */
	SearchResultsControl.prototype._resetPageIndex = function () {
		var paramFormElem = document.getElementById(this.hpSectionId + '_display_params_form');
		if (paramFormElem) {
			paramFormElem.StartIndex.value = 1;
			paramFormElem.EndIndex.value = paramFormElem.pageSize.value;
		}
	};

	/**
	 * @public
	 * 
	 * Rerun this search.
	 * 
	 * @param {boolean} resetPage - whether to show page one of search again (default: true)
	 * @param {Object} [updateContentsParam] - object to pass to the updateContent function
	 */
	SearchResultsControl.prototype.doRerun = function (resetPage, updateContentsParam) {
		if (resetPage === undefined) {
			resetPage = true;
		}

		// Update the content of the hidden textarea with the JSON version of the object model
		
		// Check the validation before submit the search. if validation fails do not submit
		if(!this.doBeforeSubmit(true)){
			return false;
		}
		
		// Store the current state (for when the user uses the browser back button)
		if(this._rememberBrowserBack) {
			this._browserhistorymgr.storeState(this.setupTextarea.text());
		}

		// Avoid rerunning twice at the same time... Might cause problems...
		// Can be caused by doAutomaticRerun happening twice because of different events (e.g. click & blur)

		// Search parms have changed. The URL needs to have the start and end index reset to their initial values
		if (resetPage) {
			this._resetPageIndex();
		}

		// 
		// The parameters to pass to the request
		//
		var params = {};
		this._getRerunParameters(params);
		
		// Notify the top parameter panel that we are doing a rerun
		setTimeout(_.bind(function() {
			// Only do this after 150 ms though. 
			// Why? Because clicking the search button 
			// => might cause a blur even on the input 
			// => which would enable the button again 100ms later.
			this._topparameterpanel.setNeedsRerun(false);
		}, this), 150);

		// Submit...
		this._showHourGlassSearch(true);
		try {
			// Give each rerun a number so that we know the order in which they happened.
			var sectionRerunOrderIndex = ++this.rerunCounter;

			ajax({
				url: this.reRunURL,
				content: $.param(params, true),
				contentType: "application/x-www-form-urlencoded",
				type: "POST",
				data: "text",
				objHdl: null,
				debug: false,
				synchronous: false,
				onError: _.bind(function (status, objHdl, description) {
					// only update the screen or My Work section if it's the returned xml
					// from the "latest" request for the screen or My Work section
					if (this.rerunCounter == sectionRerunOrderIndex) {
						this._showHourGlassSearch(false);

						if (description == 'SESSION_EXP') {
							// the session got expired - show the dialog to log in again 
							sessionExpired(workerURL, this.hpSectionId);
						} else {
							this._showError(bundle.t("HP_LOADING_SECTION_FAILED") + ": " + status);
						}
					}
				}, this),
				onSuccess: _.bind(function (xml, objHdl) {
					// only update the screen or My Work section if it's the returned xml
					// from the "latest" request for the screen or My Work section
					if (this.rerunCounter == sectionRerunOrderIndex) {
						this._showHourGlassSearch(false);

						// Check what we got back...
						if ($(xml).is("error")) {
							// We got a SDK error response back...
							this._showError(bundle.t("HP_LOADING_SECTION_FAILED") + ": " + $("message", $(xml)).text());
						} else {
							// Update the content in the rerun_container
							this._updateContent(xml, updateContentsParam);
						}
					}
					
					//automatically scroll to the top of the selection, fixes some render issues					
					if (!resetPage){
						$(".searchreruncontainer", this.$div).get(0).scrollIntoView(true);
					}
				}, this)
			});
		} catch (err) {
			this._showHourGlassSearch(false);
			alert('Problem in keep session alive: ' + err);
		}


	};

	/**
	 * @param {Object} params - object to fill with parameters for the rerun call
	 */
	SearchResultsControl.prototype._getRerunParameters = function (params) {
		// 
		params.actionVal = "executeSavedSearchWithParms";

		// pass 'sectionID' parameter.
		params.sectionID = this.hpSectionId;

		// get the 'showSimpleGUI' from search settings
		var showSimpleGUI = this.searchData.getSearchSettings().getIsSimpleGUI();

		//JAMI: do not pass "Show Simple GUI" parameter when in a 'non-My-Work' section context 
		// --> in that case the HPSectionID is set to '00000_0000000000'
		// KSJ: test on global variable defined on mytodolist page; if defined and true, add parameter. 
		// This is needed because the search pages doesn't use the general mechanism for HP section reload where this parameter is added automatically
		params.showSimpleGUI = (this.hpSectionId != '00000_0000000000' && typeof showSimpleGUI !== "undefined" && showSimpleGUI) ? "1" : null;

		// Whether we are in a dialog
		params.isInDialog = $('#' + this.hpSectionPrefix + 'isInDialogFlag').val() === "1" ? "1" : null;

		// smartname context
		params.projectID = $('#' + this.hpSectionPrefix + 'projectID').val();
		params.docVerID = $('#' + this.hpSectionPrefix + 'docVerID').val();
		params.staticSmartNames = $('#' + this.hpSectionPrefix + 'staticSmartNames').val();

		// Add some other basic settings 
		// Note: In MyWork these are preferences - see also "defaultURLParameters_" in xsl for MyWork
		// Note 2: Notice the different id (always starts with "_")
		var paramFormElem = $('#' + this.hpSectionId + '_display_params_form').get(0);
		params.StartIndex = paramFormElem.StartIndex.value;
		params.EndIndex = paramFormElem.EndIndex.value;
		params.showThumbnails = paramFormElem.showThumbnails.value;
		params.resultsStyle = paramFormElem.resultsStyle.value;

		params.searchReRun = "1";

		// Remove the empty or null or undefined items !
		_.each(params, function (val, key) {
			if (_.has(params, key)) {
				if (!val) {
					delete params[key];
				}
			}
		});

		// Pass the object model with all the criteria information
		params.searchSetupModel = $("#searchresultscontrol" + this.hpSectionId + " > textarea[name='searchSetupModel']").text();
	};

	/**
	 * @public
	 * 
	 * Rerun this search if the search button is hidden.
	 */
	SearchResultsControl.prototype.doAutomaticRerun = function () {
		// When the search button is hidden...
		if (!this.searchSetup.hasSearchButton()) {
			this.doRerun();
		} else {
			this._topparameterpanel.setNeedsRerun(true);
		}
	};

	/**
	 * (internal) Do a rerun because the paging has changed
	 * 
	 * Called when the user clicks one of the paging links.
	 * 
	 * @param startIndex - index of the first item of the results
	 * @param endIndex - index of the last item of the results
	 */
	SearchResultsControl.prototype.changePageAndRerun = function (startIndex, endIndex) {
		var paramFormElem = document.getElementById(this.hpSectionId + '_display_params_form');
		if (paramFormElem) {
			paramFormElem.StartIndex.value = startIndex;
			paramFormElem.EndIndex.value = endIndex;
		}

		this.doRerun(false);
	};

	/**
	 * (internal) Do a rerun because the show/hide thumbnail has changed
	 * 
	 * Called when the user clicks on the "show/hide thumbnails" toggle.
	 * 
	 * @param {boolean} showThumbnails - whether to show thumbnails
	 * @param endIndex - index of the last item of the results
	 */
	SearchResultsControl.prototype.changeThumbnailsAndRerun = function (showThumbnails) {
		var paramFormElem = document.getElementById(this.hpSectionId + '_display_params_form');
		if (paramFormElem) {
			paramFormElem.showThumbnails.value = showThumbnails ? "1" : "0";
		}

		this.doRerun(false);
	};

	/**
	 * (internal) Do a rerun because the results style has changed
	 * 
	 * Called when the user clicks one of the display style buttons.
	 * 
	 * @param {String} newResultsStyle - the new display style to use ("grid", "card" or "list")
	 */
	SearchResultsControl.prototype.changeResultsStyleAndRerun = function (newResultsStyle) {
		var paramFormElem = document.getElementById(this.hpSectionId + '_display_params_form');
		if (paramFormElem) {
			paramFormElem.resultsStyle.value = newResultsStyle;
		}

		this.doRerun(false);
	};

	/**
	 * Get the values of a restricted set. (For Batch Actions) 
	 * 
	 * @param attributeId
	 * @returns Array of Strings; the values of a restricted set IF attributeId is a restricted set
	 */
	SearchResultsControl.prototype.getRestrictedSetValues = function (attributeId) {
		//retrieve the SearchField for the passed Attribute 
		//could be an Attribute Criterion in either the main or extended section
		var searchfield = this.searchData.getFieldInformation("db_attr_" + attributeId, this.searchSetup);
		if (!searchfield) {
			//not found in the main section > try the extended
			searchfield = this.searchData.getFieldInformation("db_attr_ext_" + attributeId, this.searchSetup);
		}

		if (!searchfield) {
			return [];
		}

		// Get the display function (Used to display text & numbers. For numbers, formats them correctly + adds units)
		var fieldProperties = this.makeFieldDataForFieldExternal(searchfield);
		var displayFunc = Utils.getParameterFieldDisplayFunction(fieldProperties, this.searchData.getSearchSettings());

		//get List Values for the Search Field (taken from 'id' attribute of the ValueData object)
		if (searchfield && searchfield.listValueID) {
			var curFieldOptions = this.searchData.listValuesMap[searchfield.listValueID];
			return $.map(curFieldOptions, function (val, i) {
				return displayFunc(val.getValue("id"));
			});
		}
	};

	/**
	 * Get the list of available Project Statuses. (For Batch Actions) 
	 *
	 * @returns Array of ValueData objects; contains the following Attributes 
	 * --> name: custom Status name; label: translated label (same as name for a custom status)
	 */
	SearchResultsControl.prototype.getProjectStatusValues = function () {
		//retrieve the SearchField for Project Status
		var searchfield = this.searchData.getFieldInformation("db_status", this.searchSetup);

		//get List Values for the Search Field (only non-deleted ones)
		if (searchfield && searchfield.listValueID) {
			return $.map(this.searchData.listValuesMap[searchfield.listValueID], function (val, i) {
				if (val.getValue("deleted") !== "1") {
					return val;
				}
			});
		}
	};

	/**
	 * <p>Make a FieldProperties object...</br>
	 * This object contains information about the field like units</p>
	 * 
	 * @param {SearchField} searchfield - the SearchField
	 * @param {SearchSetupInformation} searchSetup - information about the currently processed Search Setup
	 * @returns {FieldProperties} a FieldProperties object
	 */
	SearchResultsControl.prototype.makeFieldDataForFieldExternal = function (searchField) {
		if (!(searchField && searchField instanceof SearchField)) {
			throw new Error("Illegal arguments passed to SearchResultsControl.makeFieldDataForFieldExternal");
		}

		//dummy "expand model" > not used
		var expandModalPopupObj = new ExpandModeModal("N/A");
		return this.searchData.makeFieldDataForField(searchField, expandModalPopupObj, this.searchSetup.getLayoutOptions());
	};

	/**
	 * Add items to the cart on top of the search results.
	 * 
	 * @param {Array} selectedIds - array of doc version id's
	 * @param {Array} checkboxes - array of checkbox elements to deselect
	 */
	SearchResultsControl.prototype.addItemsToSearchResultsCart = function (selectedIds, checkboxes) {
		this._cart.addSelectedToCart(selectedIds, checkboxes);
	};


	/**
	 * Parse addtional field (facet) data
	 * @private
	 * @param {String} fieldDataString 
	 */
	SearchResultsControl.prototype._parseAdditionalFieldData = function (fieldDataString) {
		if (fieldDataString) {
			var additionalFieldData = null;
			try {
				additionalFieldData = JSON.parse(fieldDataString);
			} catch (e) {
				if (console && console.error) {
					console.error("Error while loading Facet Data", e);
				}
			}

			this.searchData.setAdditionalFieldData(additionalFieldData);
		}
	};

	/**
	 * (internal) Called by search.js when the ajax result for the re-run is back
	 */
	SearchResultsControl.prototype._updateContent = function (xml, parameters) {
		this.rerunning = false;

		// read addionatal data and update facets
		var additionalFieldDataString = $(".searchreruncontainer", $(xml)).attr("data-additionalfielddata");
		this._parseAdditionalFieldData(additionalFieldDataString);

		this.rerunsectioncontrol.updateContent(xml, parameters);
		
		// Force a resize of the search results.
		this.forceResize();
	};

	/**
	 * (internal) Called by search.js when the ajax result for the re-run is back and contains error
	 */
	SearchResultsControl.prototype._showError = function (errormsg) {
		this.rerunning = false;
		this.rerunsectioncontrol.showError(errormsg);
	};
	
	/**
	 * Forces Search to resize.
	 * 
	 * Called when the size of the search results table changes OR when the browser resizes and thus the whole search might need to be resized.
	 * 
	 * This is not needed on mobile since there search will try to fit the available space and shrink all components as needed.
	 * 
	 * On desktop however, search will allow components to take up more space than is available when needed. (causing a scrollbar on the window)
	 * However, if they don't need more space, they should not use more space.
	 *  => this method makes sure the shrinking also happens.
	 */
	SearchResultsControl.prototype.forceResize = function() {
		// Trick to resize the full dashboard
		// Because the size of the results table might have changed => we need to resize the graphs and other things also.
		
		// 1) First force the dashboard to fit a very small width
		// This might cause some things to start overlapping with each other...
		$(".searchresultsflexiblecontainerchild", this.$div).css({"max-width":"30px"});
		
		// Resize all the parameter fields... => Graphs will now adapt to the available space. This means they become very small.
		// The problem if we do not do this collapse and resize step is => graphs can only become bigger because they have a fixed calculated width. (so the browser cannot shrink them)
		this._distributeResizeEvent();
		
		// 2) Now, restore the table style. This will cause the dasboard to become bigger IF this is really needed.
		$(".searchresultsflexiblecontainerchild", this.$div).css({"max-width":""});
		
		// Resize the parameter fields... The size of the dashboard might have changed...
		this._distributeResizeEvent();
	};
	
	SearchResultsControl.prototype._distributeResizeEvent = function() {
		this.parameterfields.resizeAll();
		// NOTE: SHOULD also call resize on "asset browser image panel" here => it also has a calculated width in px.
		// Otherwise, putting a graph above the asset browser image panel => search can not shrink the results anymore (bug)
		
		// Also resize the scrollbar on the search results table (note: might be triggered on multiple tables)
		$(".wcr-table-section", this.$div).trigger("contentchange");
	};

	/**
	 * (internal)
	 * Destroy all functionality in the Search Results table...
	 * 
	 * (Use for cleanup)
	 */
	SearchResultsControl.prototype._destroy = function () {
		this.rerunsectioncontrol._destroy();
	};

	/*
	 * This component is createable using data-init (and should be created that way)
	 */

	SearchResultsControl.wcrinit = function (div) {
		// The JSON information which is passed...
		var searchDataJSON = $.parseJSON($(div).attr("data-searchdata"));

		// The Textarea in which the Search Setup Model is put.
		var setupTextarea = $("> textarea[name=searchSetupModel]", $(div));

		// Read the passed JSON data about the fields etc
		var searchData = new SearchModelFrontEndData(searchDataJSON);

		new SearchResultsControl(div, searchData, setupTextarea);
	};

	SearchResultsControl.wcrdestroy = function (div) {
		div.SearchResultsControl._destroy();
	};

	return SearchResultsControl;
});
