/**
 * 
 * ## Search Results Rerun Container Internal Javascript Functionality ##
 * 
 * [Use]
 * You should NOT use this component directly... 
 * This component expects a complete search results table created by xsl, so it is not a re-usable component on it's own...
 * 
 * [How it works]
 * This component is attached to every Search Results Table...
 * It is initialized automatically using data-init and imports all the required javascript.
 * (look for data-init="component/searchresults/internal/SearchResultsRerunSectionControl" in the xsl)
 * 
 * [This is a work in progress...]
 * At the moment though (WCR 14.1) this component only contains the javascript functionality for "batch actions" & "create project from documents".
 *
 * NOTE: Gets initialized again every re-run for a search results "group". 
 * (See SearchResultsControl if you don't want to re-initialize on re-run)
 */
define([
	// Actions
	"./actions/SearchResultsActionHandler",

	// Re-sorting & Paging
	"./functionality/SearchResultsResortHandler",
	"./functionality/SearchResultsPaginationHandler",
	"./functionality/SearchResultsResultsStyleHandler",

	// Handles the displaying of the results
	"./render/SearchTableRenderer",

	// Model
	"../model/SearchSetupInformation",
	// Additional information passed from the server
	"../model/SearchModelFrontEndData",

	// Data-Init
	"utils/init"
], function (
	SearchResultsActionHandler,
	SearchResultsResortHandler,
	SearchResultsPaginationHandler,
	SearchResultsResultsStyleHandler,
	SearchTableRenderer,
	SearchSetupInformation,
	SearchModelFrontEndData,
	init
) {

	/**
	 * Object which controls all JavaScript for the search results table
	 * 
	 * @constructor
	 * @param {Element} searchresultstable - The DOM element contain the results table html.
	 * @param {Object} mainsearchresultscomponent - Needs to have ONE function: doRerunAfterBatchActions
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (columns to show, ...)
	 * @param {SearchModelFrontEndData} searchData - Data which is passed from the server...
	 */
	var SearchResultsRerunSectionControl = function (searchresultstable, mainsearchresultscomponent, setupInfo, searchData) {
		// Check input
		if (!(searchData && searchData instanceof SearchModelFrontEndData)) {
			throw new Error("Illegal argument passed: searchData");
		}
		if (!(setupInfo && setupInfo instanceof SearchSetupInformation)) {
			throw new Error("Illegal argument passed: setupInfo");
		}
		if (!(searchresultstable && 1 === searchresultstable.nodeType /* check if it is a HTML Element */ )) {
			throw new Error("Illegal argument passed: searchresultstable");
		}
		if (!(mainsearchresultscomponent && (typeof mainsearchresultscomponent.doRerun) === "function")) {
			throw new Error("Illegal argument passed: mainsearchresultscomponent");
		}

		this.$div = $(searchresultstable);

		// Store the reference in the table
		this.$div.get(0).SearchResultsRerunSectionControl = this;

		// Store the reference to the main search component
		this._maincomponent = mainsearchresultscomponent;

		// Init fields
		this._searchData = searchData;
		this._setupInfo = setupInfo;
		
		// HOTFIX HF_20170811_CS00896686
		this._childrenRenderReady = false;
		this._childrenDashboardReady = false;

		// Initialize search results table action handler...
		this._actionhandler = new SearchResultsActionHandler(this.$div, mainsearchresultscomponent, this, this._searchData.getSearchSettings());

		// Initialize sorting functionality
		// Note: this functionality still uses the hpSectionId...
		this._sortinghandler = new SearchResultsResortHandler(this.$div, this._setupInfo.getSortingCriteriaList(), _.bind(this._maincomponent.doRerun, this._maincomponent));

		// Initialize pagination functionality
		this._paginationhandler = this._initPaginationHandler();

		// Initialize display style change functionality
		this._resultsstylehandler = new SearchResultsResultsStyleHandler(this.$div, _.bind(this._maincomponent.changeThumbnailsAndRerun, this._maincomponent), _.bind(this._maincomponent.changeResultsStyleAndRerun, this._maincomponent));

		// Initialize the table renderer
		this._tablerenderer = this._initSearchTableRenderer();

		// Initialize the javascript for the Search Results Table
		this._initialize();
	};

	/**
	 * Get a reference to the main element of the search results rerun section control
	 * @returns {JQueryObj} 
	 */
	SearchResultsRerunSectionControl.prototype.getElement = function () {
		return this.$div;
	};

	/**
	 * init the searchTableRenderer
	 * @returns {SearchTableRenderer}
	 */
	SearchResultsRerunSectionControl.prototype._initSearchTableRenderer = function () {
		return new SearchTableRenderer(this._searchData, this._setupInfo);
	};

	/**
	 * @return {SearchResultsPaginationHandler}
	 */
	SearchResultsRerunSectionControl.prototype._initPaginationHandler = function(){
		return new SearchResultsPaginationHandler(this.$div, _.bind(this._maincomponent.changePageAndRerun, this._maincomponent));
	};
	
	/**
	 * HOTFIX HF_20170811_CS00896686
	 * 
	 * - to be called when the dashboard is ready and completely put in the DOM.
	 */
	SearchResultsRerunSectionControl.prototype.onDashboardReady = function() {
		this._childrenDashboardReady = true;
		this._initializeChildComponents();
	};
	
	/**
	 * HOTFIX HF_20170811_CS00896686
	 * 
	 * Tries to initialize subcomponents but ONLY succeeds if the results are already inside the DOM.
	 */
	SearchResultsRerunSectionControl.prototype._initializeChildComponents = function() {
		// If in DOM and ready ...
		if(this._childrenRenderReady && this._childrenDashboardReady) {
			// Run all data-init stuff (the table + action menu + card view)
			$("*[data-init-whensearchready]", this.$div).each(function() {
				$(this).attr("data-init", $(this).attr("data-init-whensearchready"));
			});
			init.doinit(this.$div.get(0));
		}
	};

	/**
	 * (internal)
	 * Initialize all the functionality in the Search Results Table
	 * @param {Object} parameters, parameters to pass to the render method
	 */
	SearchResultsRerunSectionControl.prototype._initialize = function (parameters) {
		// Run all data-init stuff (the table + action menu)
		this._childrenRenderReady = true;
		this._initializeChildComponents();

		// Attach the pagination listeners
		this._paginationhandler.initialize();

		// Notify the pagination handler that the number of results might have changed
		this._paginationhandler.resultsChanged();

		// Attach the show/hide thumbnail & grid/list/card listeners
		this._resultsstylehandler.initialize();


		this._render(this.$div, parameters);

		// Attach the re-sort listeners
		// Happens after displaying the table + calendar view
		this._sortinghandler.initialize();

		// size of this element might have changed...
		// mainly for drill down searches where this search might be inside the table of another one.
		this.$div.trigger("contentchange");
	};

	/**
	 * @param {JQueryElement} div 
	 * @param {Object} parameters
	 */
	SearchResultsRerunSectionControl.prototype._render = function (div, parameters) {
		// Look for the place to put the actual table with results... 
		var tables = $(".searchresultstable", div);
		$.each(tables, _.bind(function (ti, t) {
			// Display the results...
			this._tablerenderer.render($(t), parameters);
		}, this));
	};

	/**
	 * Update the content with new xml and initialize all javascript again...
	 * 
	 * @param {Node} xml
	 * @param {Object} parameters
	 */
	SearchResultsRerunSectionControl.prototype.updateContent = function (xml, parameters) {
		this._destroy();

		var tempdiv = $("<div>").html(xml);
		if ($(".searchreruncontainer", tempdiv).length > 0) {
			// Replace the contents of the rerun container ONLY! (xml contains too much information)
			$(".searchreruncontainer", tempdiv).children().clone().appendTo(this.$div.empty());
		} else {
			// When there are no results found, no table is returned, just a bit of xml.
			this.$div.empty().html(xml);
		}

		// Initialize all javascript functionality on the search results table again...
		this._initialize(parameters);
	};

	/**
	 * Remove all the content and show an error message
	 * 
	 * @param errormsg
	 */
	SearchResultsRerunSectionControl.prototype.showError = function (errormsg) {
		this._destroy();
		this.$div.empty();
		this.$div.append($('<div class="alert alert-danger"/>').append(errormsg));
	};

	/**
	 * (internal)
	 * Destroy all functionality in the Search Results table...
	 * 
	 * (Use for cleanup)
	 */
	SearchResultsRerunSectionControl.prototype._destroy = function () {

	};

	return SearchResultsRerunSectionControl;
});
