/**
 * 
 * ## Search Setup Internal Javascript Functionality ##
 * 
 * [Use]
 * You should NOT use this component directly... 
 * This component expects the complete search setup created by xsl, so it is not a re-usable component on it's own...
 * 
 * [How it works]
 * This component is attached to every Search Setup Page...
 * It is initialized automatically using data-init and imports all the required javascript.
 * (look for data-init="component/searchresults/internal/SearchSetupControl" in the xsl)
 */
define([
        "underscore",
        // Design Panel & Layout Controller
        "./SearchParameterFieldControl",
        "./SearchLayoutControl",
        "./FieldLayoutAPI",
        
        // Model
        "../model/SearchSetupInformation",
        "../model/SearchModelFrontEndData",
        
        // Model IO
        "../model/io/SearchModelTransferedJsonReader",
        "../model/io/SearchModelTransferedJsonWriter",
        
        // Search Setup JavaScript (for inputs and stuff)
        "./functionality/SetupControlDesignPanel",
        "./functionality/SetupControlDisplayColumn",
        "./functionality/SetupControlSorting",
    	
    	// Browser History Mgr
    	"./functionality/SearchBrowserHistoryManager",
        
        "component/searchresults/dashboard/SearchSetupDashboardBuilder",

	
        // IE6 & IE7 JSON compatibility
        "wcr/json3.min"
        ], function(
                _,
        		SearchParameterFieldControl,
        		SearchLayoutControl,
        		FieldLayoutAPI,
        		SearchSetupInformation,
        		SearchModelFrontEndData,
        		SearchModelTransferedJsonReader,
        		SearchModelTransferedJsonWriter,
        		SetupControlDesignPanel,
        		SetupControlDisplayColumn,
        		SetupControlSorting,
        		SearchBrowserHistoryManager,
        		SearchSetupDashboardBuilder
        		){
	
	/**
	 * Main control component for the search results...
	 * 
	 * @constructor
	 * @param {Element} searchsetupcontainer - the DOM Element containing the search setup html.
	 * @param {SearchModelFrontEndData} searchData - data which is passed from the front-end
	 * @param {jQuery} setupTextarea - the textarea which contains a serialized "SearchSetupInformation" object 
	 *                                 with all information about the search Setup...
	 */
	var SearchSetupControl = function(searchsetupcontainer, searchData, setupTextarea){
		// Checking inputs
		if(!(	searchsetupcontainer && 1 === searchsetupcontainer.nodeType /* check if it is a HTML Element */ &&
				searchData && searchData instanceof SearchModelFrontEndData &&
				setupTextarea && setupTextarea instanceof $)) {
			throw new Error("Illegal arguments passed to SearchSetupControl");
		}
		
		var self = this;
		
		// Keep the div we are working in
		this.$div = $(searchsetupcontainer);
		
		// Show the div
		$(".loadingcontainer", this.$div).remove();
		$(".searchloadingarea", this.$div).css("display","");
		
		// Store the reference in the table
		this.$div.get(0).SearchSetupControl = this;
		
		// the object that handles the history of this search (using the browser back button)
		this._browserhistorymgr = new SearchBrowserHistoryManager("");
		
		// restore the search if we used the back button
		var restoreData = this._browserhistorymgr.getRestoreData();
		if(restoreData) {
			setupTextarea.text(restoreData);
		}

		// Information about the search setup
		// Read the passed JSON for the model...
		this._setupTextarea = setupTextarea;
		this._searchSetup = new SearchSetupInformation();
		SearchModelTransferedJsonReader.read(this._searchSetup, $.parseJSON(this._setupTextarea.text()));
		
		// Information about the fields, unit conversions and other "static" information
		this._searchData = searchData;
		
		var searchInformation = {
			data: this._searchData,
			setup: this._searchSetup
		};
        
		
		// Graphic configuration link
		this.configpopup = new SearchSetupDashboardBuilder(searchInformation, this._searchSetup.getDashboardConfiguration());
		var graphicconfigurelink = $(".btn.configuredashboard", this.$div);
		graphicconfigurelink.click(_.bind(function(){
			this.configpopup.open();       
		},this));
		
		// Save this search link
		var savelink = $(".btn.advancedsearchsave", this.$div);
		savelink.click(_.bind(this.saveAdvanced, this));
		
		// Do search link
		var searchlink = $(".doadvancedsearchbtn", this.$div);
		searchlink.click(_.bind(this.searchAdvanced, this));
		
		// Initialize the JavaScript for all fields...
		self._initialize();
	};
	
	/**
	 * (internal)
	 * Initialize all the functionality
	 */
	SearchSetupControl.prototype._initialize = function(){
		//
		// Handle to the db_show_search_panel_design checkbox
		// => link to "Search Layout Option" "chkbox_ShowSearchPanelDesign" in the object model
		//
		// (note: first step in "initialize" since the value of the checkbox is accessed by some of the layouts)
		this._designPanelCheckbox = $("#db_show_search_panel_design");
		this._designPanelOptionName = "chkbox_ShowSearchPanelDesign";
		this._designPanelCheckbox.prop("checked", this._searchSetup.getLayoutOptions().getOption(this._designPanelOptionName));
		this._designPanelCheckbox.click(_.bind(this.toggleDesignPanel, this));
		
		// Initialize the SmartName checkbox
		this._smartnameCheckbox = $("#db_smart_name_context");
		this._smartnameCheckbox.prop("checked", this._searchSetup.getFeatures().getUseSmartNames());
		this._smartnameCheckbox.click(_.bind(this.toggleSmartnames, this));

		// Initialize the SmartName checkbox
		this._advancedCharsCheckbox = $("#db_adv_char_search");
		this._advancedCharsCheckbox.prop("checked", this._searchSetup.getFeatures().getUseAdvancedChars());
		this._advancedCharsCheckbox.click(_.bind(function() {
			// Update the model
			this._searchSetup.getFeatures().setUseAdvancedChars(!this._searchSetup.getFeatures().getUseAdvancedChars());
		}, this));
		this._searchSetup.getFeatures().onUseAdvancedCharsChange(_.bind(function() {
			// Update the checkbox to reflect the model
			this._advancedCharsCheckbox.prop("checked", this._searchSetup.getFeatures().getUseAdvancedChars());
		}, this));
		require(["jqueryUI"], function(){
			var tooltiptext = $(".advsearchchartooltip").attr("data-tooltiptext");
			$(".advsearchchartooltip").attr("title", "-").tooltip({'content':$("<span>").text(tooltiptext).html().replace(/\\n/g, "<br/>"), 'tooltipClass':"search-advancedchars-tooltip", position:{my: "center bottom-5", at: "center top"}});
		});
		
		// Make new parameter field controller
		this._parameterfields = new SearchParameterFieldControl(this.$div, this._searchSetup, this._searchData, false);
		
		// Make the layout api
		var layoutApi = new FieldLayoutAPI(this._searchSetup, this._searchData, this._parameterfields);
		
		// Make the layouts controller
		this._layouts = new SearchLayoutControl();
		this._layouts.setApiInstance(layoutApi);
		this._layouts.registerLayout("defaultxsl", this.$div.get(0));
		
		// Initialize custom layouts
		$(".customsetuplayout").each(_.bind(function(i, e) {
			var nameoflayout = $(e).attr("data-nameoflayout");
			this._layouts.registerLayout(nameoflayout, e);
		}, this));
		
		// Initialize the display preferences
		if($("#LayoutPreferencesPanel").get(0)) {
			this._layouts.registerLayout("displaypreferences", $("#LayoutPreferencesPanel").get(0));
		}
		
		//
		// Initialize other JavaScript for the Search Setup page
		//
		
		// Initialize Design Panel Setup related JavaScript (column & row numbering)
		this._designPanelFunc = new SetupControlDesignPanel(this.$div, this._searchData, this._searchSetup);
		// Initialize Display Column related JavaScript (sequence numbering)
		this._displayColumnFunc = new SetupControlDisplayColumn(this.$div, this._searchData, this._searchSetup);
		// Initialize Sorting related JavaScript (order)
		this._sortingFunc = new SetupControlSorting(this.$div, this._searchData, this._searchSetup);
		
		// For debugging purposes only !
		if(!window.DEBUGSEARCH) {
			window.DEBUGSEARCH = {};
		}
		window.DEBUGSEARCH["setup"] = {
			// Get full access to the internal data structure
			model: this._searchSetup,
			// Get full access to the available data
			data: this._searchData,
			// Get full access to the layout api
			api: layoutApi,
		};
	};
	
	/**
	 * Method is called before every search submit...
	 * 
	 * Convert the object model we use internally to a string which will be send to the backend...
	 * 
	 * @param {Boolean} doValidation - whether to do validation
	 */
	SearchSetupControl.prototype.doBeforeSubmit = function(doValidation){
		// 1) Serialize the model
		var serializedSearchSetupModel = {};
		SearchModelTransferedJsonWriter.write(this._searchSetup, serializedSearchSetupModel);
		
		// 2) Put it in the textarea before submitting the search...
		this._setupTextarea.text(JSON.stringify(serializedSearchSetupModel));
		
		// 3) Validation
		if(doValidation) {
			var validation = true;
			validation = this._designPanelFunc.validate() && validation;
			
			// Validation for all the parameter fields .. like revision 
			validation = this._parameterfields.validateAll() && validation;
			// TODO REFACTOR - this is currently a global method in the xsl... => should be moved as a method in this class
			// Need to introduce calls "validate()" in the ParameterFields + in the Layout Options section
			validation = validateDesignPanel() && validation;
			return validation;
		}else{
			return true;
		}
	};
	
	/**
	 * Toggle the design panel. 
	 * This will cause a reload of the page.
	 */
	SearchSetupControl.prototype.toggleDesignPanel = function() {
		try {
			document.getElementById("loadMessage").style.display = "block";	  
		} catch (Exception){} // ignore - not all search pages have a "loadMessage" element
		
		// change the setting in the object model
		var oldValue = this._searchSetup.getLayoutOptions().getOption(this._designPanelOptionName);
		this._searchSetup.getLayoutOptions().setOption(this._designPanelOptionName, !oldValue);
		this._designPanelCheckbox.prop("checked", !oldValue);
		
        this._submitAdvancedSearch("ChangeSearchPanelDesign", false);
	};
	


	/**
	 * Handles toggle events on the "Smart Name Context" checkbox of the Search Setup page
	 * 
	 * @param selElem the checkbox we clicked on
	 */
	SearchSetupControl.prototype.toggleSmartnames = function(selElem) {
		var smartNameChecked = !this._searchSetup.getFeatures().getUseSmartNames();
		this._submitAdvancedSearch(smartNameChecked?'SmartNamesEnable':'SmartNamesDisable', false);
	};
	
	/**
	 * Search (when on the advanced search setup page)
	 */
	SearchSetupControl.prototype.searchAdvanced = function(event) {
		event.preventDefault();
		this._submitAdvancedSearch("Search", true);
	};
	
	/**
	 * Save the search setup page (when on the advanced search setup page)
	 */
	SearchSetupControl.prototype.saveAdvanced = function(event) {
		event.preventDefault();		
        this._submitAdvancedSearch("Save", true);
	};
    
    /**
     * Submit the advanced search form
     * @private
     * @param {String}   actionVal
     * @param {Boolean} doValidation Enable validation on the form
     */
    SearchSetupControl.prototype._submitAdvancedSearch = function (actionVal, doValidation) {
        _.defer(_.bind(function () {
            // Set the correct action
            document.advancedsearchform.actionVal.value = actionVal;

            // put the correct information in the form & validate
            if (this.doBeforeSubmit(doValidation)) {
            	// store the setup in the browser history
            	this._browserhistorymgr.storeState(this._setupTextarea.text());
            	
                // post the request
                document.advancedsearchform.submit();
            }
        }, this));
    };
	
	/*
	 * This component is createable using data-init (and should be created that way)
	 */	
	SearchSetupControl.wcrinit = function(div) {
		// The JSON information which is passed...
		var searchDataJSON = $.parseJSON($(div).attr("data-searchdata"));
		
		// The Textarea in which the Search Setup Model is put.
		var setupTextarea = $("> textarea[name=searchSetupModel]", $(div));
		
		// Read the passed JSON data about the fields etc
		var searchData = new SearchModelFrontEndData(searchDataJSON);
		
		new SearchSetupControl(div, searchData, setupTextarea);
	};
	
	return SearchSetupControl;
});
