define(["underscore",
        "jquery",
        
        // Action Handlers... (can be re-used in other places)
        "./SearchResultsActionsOnDocuments",
        "./SearchResultsActionsOnProjects",
        "./SearchResultsBatchActions",
        
        // For type checking
        "../../dataobjects/ExternalSearchSettings"
        ], function(
        		_,
        		$,
        		SearchResultsActionsOnDocuments,
        		SearchResultsActionsOnProjects,
        		SearchResultsBatchActions,
        		ExternalSearchSettings
        		){
	/**
	 * @param {jQuery} div - the container in which the table will be created
	 * @param {SearchResultsControl} control - the control component
	 * @param {SearchResultsRerunSectionControl} rerunsection - the rerun section component
	 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
	 */
	var SearchResultsActionHandler = function($div, control, rerunsection, searchSettings) {
		// Check input
		if(!($div && $div instanceof $)) {
			throw new Error("Illegal argument passed: $div");
		}
		if(!(control && typeof(control.doRerun) === "function")) {
			throw new Error("Illegal argument passed: control");
		}
		if(!(control && typeof(control.getRestrictedSetValues) === "function")) {
			throw new Error("Illegal argument passed: control");
		}
		if(!(control && typeof(control.getProjectStatusValues) === "function")) {
			throw new Error("Illegal argument passed: control");
		}
		if(!(searchSettings && searchSettings instanceof ExternalSearchSettings)) {
			throw new Error("Illegal argument passed: searchSettings");
		}
		
		// Set vars
		this.$div = $div;
		this.actionHandlers = [];
		this.context = this.makeActionCallbackAPI();
		this._registerContainer();
		
		//
		// Callbacks...
		//
		
		// callback for when something changed in the batch actions...
		var reruncallafterbatchaction = _.bind(function(){this.doRerun();}, control);
		// callback to get the restricted set values (if the attribute is a restricted set!)
		var restricedsetcallback = _.bind(control.getRestrictedSetValues, control);
		// callback to get the restricted set values (if the attribute is a restricted set!)
		var projstatuscallback = _.bind(control.getProjectStatusValues, control);
		// add documents to the search cart callback
		var adddocstosearchcart = _.bind(control.addItemsToSearchResultsCart, control);
		
		//
		// Initialize Action Handlers
		//
		
		// Document Actions Handler
		this.actionHandlers.push(new SearchResultsActionsOnDocuments(this.$div, adddocstosearchcart));
		
		// Project Actions Handler
		this.actionHandlers.push(new SearchResultsActionsOnProjects(this.$div));
		
		// Batch Actions Handler
		this.actionHandlers.push(new SearchResultsBatchActions(searchSettings, reruncallafterbatchaction, restricedsetcallback, projstatuscallback));
		
	};
	
	/**
	 * Register a container which contains links to actions...
	 */
	SearchResultsActionHandler.prototype._registerContainer = function(){
		// Action links... (action menu)
		this.$div.on("click", "a.action", _.partial(function(event, actionHandler) {
			var elem = $(this);
			var actionname = elem.attr("data-action");
			actionHandler.handleAction(actionname, elem, event);
		}, _, this));
		
		// Edit buttons in table header...
		this.$div.on("click", ".searchheadereditbutton", _.partial(function(event, actionHandler) {
			actionHandler.handleAction("searchaction", $(this), event);
		}, _, this));
	};
	
	/**
	 * Method to execute an action on the Search Results table.
	 * 
	 * @param {String} actionname
	 * @param {jQuery} elem
	 */
	SearchResultsActionHandler.prototype.handleAction = function(actionname, elem, event) {
		// Try all handlers to execute the action...
		for(var i=0;i<this.actionHandlers.length;i++) {
			this.actionHandlers[i].executeAction(actionname, elem, this.context, event);
		}
	};
	

	
	/**
	 * (internal) The context which are passed to the modules with functionality...
	 * 
	 * This is the only context the sub-modules have... (except for what is passed in the constructor)
	 * 
	 * Re-implementing these functions would allows you to re-use the functionality in different places!
	 * (like Cart or ... )
	 */
	SearchResultsActionHandler.prototype.makeActionCallbackAPI = function() {
		return {
			/**
			 * Util method to parse json...
			 * 
			 * @returns null if the string cannot be parsed as json (and shows an alert)
			 */
			getActionOptions: _.bind(function (e){
				try {
					return $.parseJSON($(e).attr("data-jsonoptions"));
				} catch (err) {
					// should NEVER occur... Means it is not a JSON which is passed... (maybe an error in escaping?)
					alert("SearchResultsRerunSectionControl, internal error: Could not parse JSON... "+jsonattr);
					return null;
				}
			}, this),
			
			/**
			 * Returns an array of {id, isselected, elem} objects where id is the documentversionid/projectid and elem is the jQuery element
			 */
			getDocumentsOnPage: _.bind(function() {
				return $("input[type=checkbox][name=selectedDocs]", this.$div).map(function(i, e) {
					var $e = $(e);
					return {
						id: $e.attr("value"),// the documentversionid
						isselected:$e.prop("checked"),
						elem: $e// the checkbox
					};
				});
			}, this),
			
			/**
			 * Returns an array of {id, isselected, elem} objects where id is the documentversionid/projectid and elem is the jQuery element
			 */
			getProjectsOnPage: _.bind(function() {
				return _.map(_.uniq($("input[type=checkbox][name=selectproject]", this.$div), function(element){
					return element.getAttribute("value");
				}), function(e) {
					var $e = $(e);
					return {
						id: $e.attr("value"),// the projectid
						isselected:$e.prop("checked"),
						elem: $e// the checkbox
					};
				});
			}, this)
		};
	};
	
	return SearchResultsActionHandler;
});