/*
 * author Jeroen Penninck, 2015
 * 
 * Search Actions on Documents.
 */

define(["utils/webi18n!SearchActions"], function(langBundle){
	
	/**
	 * Constructor
	 */
	var SearchResultsActionsOnDocuments = function($div, addDocsToSearchCart){
		// container div
		this.div = $div;
		
		// callback to add documents to the cart of this search results
		this._addDocsToSearchCart = addDocsToSearchCart;
	};
	
	/**
	 * (Public) Execute a search action button action
	 * 
	 * @param actionname - the action to execute. The executer should check if it can execute this action. 
	 * @param elem - The element which was clicked on
	 * @param context - An object which represents the context. 
	 *                Contains methods: getActionOptions, getDocumentsOnPage, getProjectsOnPage. More info, see calling class...
	 * @param event - The event which caused this action to be executed
	 */
	SearchResultsActionsOnDocuments.prototype.executeAction = function (actionname, elem, context, event) {
		var documents = context.getDocumentsOnPage();
		var selecteddocs = _.filter(documents, function(doc){return doc.isselected;});
		var nr_selected_docs = selecteddocs.length;
		
		//
		// Check number of documents selected
		//
		
		switch(actionname) {
		case "view_and_annotate":
		case "compare":
		// Please select at least one Document to open in the Viewer
		{
			if (nr_selected_docs == 0) {
				alert(langBundle.t("SELECT_MIN_ONE_DOC_VIEWER"));
				return;
			}
		}
		break;
		case "approve_reject":
		case "download":
		case "add_to_cart":
		case "send_download_link":
		case "create_project_from_documents":
		// Please select a Document.
		{
			if(nr_selected_docs===0) {
				// No documents selected...
				alert(langBundle.t('SELECTDOC'));
				return;
			}
		}
		break;
		}
		
		//
		// Execute the actions...
		//
		if(actionname==="approve_reject") {
			// NOT IN USE
			this.submitSimpleAction("ApproveRejectSelection", event);
		}else if(actionname==="view_and_annotate") {
			this.submitSimpleAction("OpenInViewerSelection", event);
		}else if(actionname==="compare") {
			this.submitSimpleAction("CompareViewer", event);
		}else if(actionname==="download") {
			//need to check whether the User selected any locked documents > if so, show a warning/info message
			var selectedlockeddocs = _.filter(selecteddocs, function(doc) {return doc.elem.attr("is_locked")==="1";});
			
			// Check and warn if locked documents are found...
			// Use can still decide to continue
			if(!(
					// No locked documents
					(selectedlockeddocs.length==0) ||
					// User confirms locked documents are ok
					confirm(langBundle.t("CHECKED_OUT_DOWNLOAD_WARNING_MULTIDOC")))) {
				// bail out if the user chose to not DL after all
				return;
			}
    		
    		this.submitSimpleAction("DownloadDocuments", event);
		}else if(actionname==="add_to_cart") {
			var docVersionIDs = _.map(selecteddocs, function(doc) {return doc.id;});
			var checkboxes = _.map(selecteddocs, function(doc) {return doc.elem.get(0);});
			
			this._addDocsToSearchCart(docVersionIDs, checkboxes);
			
			// Note, there is also a call:
			// this.submitSimpleAction("AddDocumentToCart");
			// That one is probably not used?
		}else if(actionname==="send_download_link") {
			//prepare the list of Document Version IDs
      		var docVersionIDs = _.map(selecteddocs, function(doc) {return doc.id;});
      		var docVersionNameMap = _.map(selecteddocs, function(doc) {
      			return {id: doc.id, name: doc.elem.attr('fullname')};
      		});
      
      		//execute the UC itself
      		// WARNING: USES GLOBAL OBJECT WHICH IS CREATED IN XSL.
		  	sendDlLinkMgr.generateDocumentBatchDLLinkMail(docVersionIDs, docVersionNameMap);
		}else if(actionname==="create_project_from_documents") {
			var templateinformation = context.getActionOptions(elem);
			if(templateinformation!==null){
				// Get the array of all selected documents in the search results...
				var selectedDocs = $.map(context.getDocumentsOnPage(), function(e, i){
					// is the document selected?
					if(e.isselected) {
						// add the id to the array
						return e.id;
					} else {
						// else: remove element
						return null;
					}
				});
				
				// Do the action
				SearchResultsActionsOnDocuments.createProjectFromDocuments(selectedDocs, templateinformation);
			}
		}
		// other actions... => see SearchResultsBatchActions... (actionname="searchaction")
	};
	
	/**
	 * ACTION: Submit a simple action
	 * This is done by submitting a form, but could become an ajax call in the future...
	 * 
	 * @param action
	 */
	SearchResultsActionsOnDocuments.prototype.submitSimpleAction = function(action, event) {
		// Note: we could get rid of this form quite easily...
		// Just use ajax calls... We have all the required information...
		var actionform = $(".searchresultsactionform", this.div);
		var actionvalueinput = $("[name=actionVal]", actionform);
		actionvalueinput.val(action);
		var clone = actionform.clone();
		clone.hide();
		clone.appendTo($("body")).get(0).submit();
		if(event) {
			event.stopPropagation();
			event.preventDefault();
		}
	};
	
	/**
	 * ACTION: Create a Project From Documents
	 *  (Note: no dependencies on search whatsoever => please keep it that way)
	 * 
	 * (static utility function)
	 * 
	 * @param selectedDocs An array of docVerId's
	 * @param templateinformation An object which might contain a templateId or a templatecharacteristicpath field or none of those.
	 */
	SearchResultsActionsOnDocuments.createProjectFromDocuments = function(selectedDocs, templateinformation) {
		// Setup the information which needs to be passed to the call
		var parameters = {
				// docVersionId => array of all docVersionId's which are selected
				docVersionId:selectedDocs
		};
		
		// Add template information to the call
		if(templateinformation.templateId){
			// We want to use a template
			parameters.templateId = templateinformation.templateId;
		}else if(templateinformation.templatecharacteristicpath){
			// We want to use a characteristic path
			parameters.templatecharacteristicpath = templateinformation.templatecharacteristicpath;
		} else {
			// We want the user to be able to select a template...
		}
		
		// Do the call
		window.location = "projmgmntnewproject.jsp?"+$.param(parameters, true);
	};
		
	return SearchResultsActionsOnDocuments;
});