/*
* author Hans Dewitte, 2012
* Tried Object oriented Javascript but walked away from it for following reasons:
* The calling code must call with new setAttributes, otherwise the keyword this refers to the global document --> OO has no advantages anyway
* When using new from the calling, it would probably work but you have to remember the object somewhere. this makes it much more complex
* Therefore decided (given lack of time) to keep things simple
*/
/*
 * author Jeroen Penninck, 2015
 * Refactored to use require, made a bit Object Oriented.
 * 
 * However, all methods are still global, see below...
 */
/* How to make REALLY object oriented (and scoped)
 *   -> all methods should be added to the SearchResultsBatchActions prototype (simple: do a search replace on window)
 *   -> all javascript concatenation should be converted to use jQuery
 *      -> because all click listeners have to be registered with local functions
 *   -> all function calls to local methods should be prefixed with "this." 
 *   -> all globals in this file should be prefixed with "this."
 */

define(
[
	//search framework modules
    "utils/webi18n!SearchActions",
    "component/sdrop/SDrop",
        
    //legacy imports
    "wcr/modalpopup",
    "wcr/webcenter_xml",
    "wcr/webcenter_html",
    "wcr/ManageRoles",
    "wcr/webcenter_ajax",
    "wcr/ObjectArrayCloneUtil"     
], function (
        searchActionsLangBundle,
        SDrop
) {
	
	// Note: JEPE: This class does not reference any search specific stuff and can be re-used anywhere... 
	// (As long as you implement the "context" correctly and pass the correct arguments to the constructor...)
	
	// GLOBALS
	// TODO REFACTOR: do not make this global (need refactoring of all the global methods below)
	var gSearchActionsObj = null;
	var gRoleDataprovider = null;
	var gGroupDataprovider = null;
	var gPMUserDataprovider = null;
	var gUserDataprovider = null;
	var cMaxAjax = 12;
	
	// Should be above the search results document source popup...
	var zIndex = 2000;
	var sdropZIndex = 2100;	
	
	/**
	 * Constructor
	 * 
	 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
	 * @param completionCallback {Function(Boolean)} calls back when the user cancels (false) or closes after a batch action (true)
	 * @param restricedSetCallback {Function(String)} function which will be called to get the values of a restricted set.
	 * @param projstatuscallback {Function()} function which will be called to get ValueData > data of the available Project Statuses
	 */
	var SearchResultsBatchActions = function(searchSettings, completionCallback, restricedSetCallback, projstatuscallback){
		// Information about the general preferences...
		this.searchSettings = searchSettings;
		
		// Method to call to rerun this section
		this.completionCallback = completionCallback;
		
		//Methods to retrieve values for specific Batch Actions
		this.restricedSetCallback = restricedSetCallback;
		this.projstatuscallback = projstatuscallback;
		
		// Initialize the data providers
		this.initializeDataProviders();
		
		// Initialize the popup content container...
		this.initializeSACTContainer();
	};
	
	/**
	 * Initial logic (to be run as soon as the component is loaded) to prepare required variables and states
	 * 
	 * => Make SDrop data providers
	 */
	SearchResultsBatchActions.prototype.initializeDataProviders = function() {
		// Initialize Data Providers for SDrop
		if(typeof SDropDataProvider !== "undefined"){
			gRoleDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=Role"});
			gGroupDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=Group"});
			gPMUserDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=PMUser"});
			gUserDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=User"});
		}
	};
	
	/**
	 * Initial logic (to be run as soon as the component is loaded) to prepare required variables and states
	 * 
	 * => Make SACT popup container (TODO: store it in this object instead of accessing it by id)
	 */
	SearchResultsBatchActions.prototype.initializeSACTContainer = function() {
		if($("#ContainerDiv").length == 0){
		    var container = '<div id="ContainerDiv" style="display:none;"></div>';
			$('body').append(container);
		}
	};
	
	/**
	 * (Public) Execute a search action button action
	 * 
	 * @param actionname - the action to execute. The executer should check if it can execute this action. 
	 * @param elem - The element which was clicked on
	 * @param context - An object which represents the context. 
	 *                Contains methods: register, getActionOptions, getDocumentsOnPage, getProjectsOnPage. More info, see calling class...
	 */
	SearchResultsBatchActions.prototype.executeAction = function (actionname, elem, context, event) {
		if(actionname==="searchaction") {
			var actionDataContainer = context.getActionOptions(elem);
			if(actionDataContainer!==null){
				actionDataContainer.feedbackElem = elem;
				actionDataContainer.event = event;
				
				// Add the restricted set values into the actionJson
				var restrictedSetValues = null;
				if(actionDataContainer.operation == 'setAttribute') {
					restrictedSetValues = this.restricedSetCallback(actionDataContainer.attributeId);
				} 
				if(restrictedSetValues && restrictedSetValues.length>0) {
					actionDataContainer.rsetValues = restrictedSetValues;
				}

				// Add the Role Preference into the actionJson
				if(actionDataContainer.operation == 'setRole') {
					actionDataContainer.inviteGroup = this.searchSettings.getRoleAssignInviteGroupDefault();
				}
				
				// Add Project Status data into the actionJson
				if (actionDataContainer.operation == 'setStatus') {
					var projStatuses = this.projstatuscallback() || [];
					actionDataContainer.values = [];
					actionDataContainer.labels = [];
					
					for(var i=0;i<projStatuses.length;i++) {
						actionDataContainer.values.push(projStatuses[i].getValue("name"));
						actionDataContainer.labels.push(projStatuses[i].getValue("label"));
					}
				}
				
				if (actionDataContainer.operation == 'manageProjectMembers') {
					actionDataContainer.haveTMicense = this.searchSettings.getHasTaskManagementLicense()?"true":"false";
					actionDataContainer.isBOMEnabled = this.searchSettings.getBOMEnabled()?"true":"false";
				}
				
				if(actionDataContainer.attributeType && actionDataContainer.attributeType === "units") {
					var unitPref = this.searchSettings.getUserUnitsPreference();
					var actualType = unitPref==="metric"?actionDataContainer.attributeTypeMetric:actionDataContainer.attributeTypeImperial;
					actionDataContainer.attributeType = actualType;
					actionDataContainer.attributeUnit = actualType;
				}
				
				// Note: we should not have to put this in here if all methods were local...
				// TODO REFACTOR: remove
				actionDataContainer.completionCallback = this.completionCallback;
				
				// Method to call the functionality (open popup)
				this.setAttributes(actionDataContainer, context.getDocumentsOnPage(), context.getProjectsOnPage());
			}
		}
		if(actionname === "UnlinkChildProject"){
			var actionJson = context.getActionOptions(elem);
			var projects = context.getProjectsOnPage();
			var selectedproj = _.filter(projects, function(proj){return proj.isselected;});
			this.DeleteChild(actionJson,selectedproj);
			
		}
	};
	
	/**
	 * Removes the selected children from the current project. Uses an ajax call
	 * @param {JSObject} actionJson like {'projectId' : ... , 'successMessage' : ..., 'hpSectiondId' : ... } 
	 * @param selectedProjects the selected projects in the saved search
	 */
	SearchResultsBatchActions.prototype.DeleteChild = function(actionJson, selectedProjects){
		var projectID = actionJson.projectId;
		var successMessage = actionJson.successMessage;
      	
		var url = "UpdateProjectRelation.jsp?projectid="+projectID+"&action=remove";
			selectedProjects.each(function(project){
				url += "&childprojectid=" + project.id;
		});
		
		var self = this;
		$.post(url,"",function(data){
			var xml = $(data);
			require(["component/pagemessagecomponent/PageMessageComponent"], function(PageMessageComponent) {
				// Load and initialize require javascript for the successmessages
				if(xml.find("error").length > 0){
					PageMessageComponent.markObsolete("programManagementError");
					PageMessageComponent.sendErrorMessage(xml.find("message").text(),undefined, "programManagementError");
				}else{
					PageMessageComponent.markObsolete("programManagementError");
					PageMessageComponent.sendSuccessMessage(successMessage,undefined, true );
				}		
				//wait until the children are removed to update the container
				self.completionCallback(true);
			});
		});
	};
	
	
	/**
	* Sets attributes and (later) other project properties on a WebCenter Search result
	* Assumed input:
	* _searchActionsObj = {
	*						operation: what needs to be done. Currently supported: setAttribute,setAttributeCategory.  Will be extended later with f.e. setProjectDueDate
	* 						}
	* The code also assumes to run on a WebCenter search result page with following features
	* The operation happens on selected elements where the checkboxes have names selectproject for project search results and selectedDocs for document search results
	* These checkboxes have additional attributes projectid and documentid(only for document search result) and fullname: readable name of project for Proj search and document for doc search
	* 
	*/
	SearchResultsBatchActions.prototype.setAttributes = function(_searchActionDataContainer, documentsOnPage, projectsOnPage) {
	
		if (!_searchActionDataContainer) throw new Error("\n_searchActionsObj " + searchActionsLangBundle.t('NullProblem'));
		if (gSearchActionsObj && gSearchActionsObj.todoList && gSearchActionsObj.todoList.length > 0) {
			alert(searchActionsLangBundle.t('PreviousAttrChange')); 
			return;
		}
	
		gSearchActionsObj = _searchActionDataContainer;  // make global
		
		// get the list of checkboxes. Can be for project or documents. Tasks not yet supported
		var itemArr = null;
		if (projectsOnPage.length==0) {
			// a document search
			itemArr = documentsOnPage;
			gSearchActionsObj.type = 'documents';
		} else {
			// a project search
			itemArr = projectsOnPage;
			gSearchActionsObj.type = 'projects';
		}
	
		var todoList = new Array();
		gSearchActionsObj.todoList = todoList;
		gSearchActionsObj.somethingDone = false;  //  remember whether some action was done successfully. If not, the search results page will not be repainted
		gSearchActionsObj.nAjaxSent = 0;
		gSearchActionsObj.nSubAjaxErrors = new Array(); // if multiple calls per project, this will keep track of SDK errors per project
		
		$.each(itemArr, function(idx, item) {
			if (item.isselected) {
				// get the checkbox (jQuery)
				var elem = item.elem;
				
				// for project operations documentId and docVerId will not be used, so it doesn't matter what value they have
				todoList.push({projectId:elem.attr('projectid'),documentId:elem.attr('documentid'), docVerId:elem.attr("value"), name:elem.attr('fullname'), done:false, idx:idx, newValue:null });  
			}
		});
	
		if (todoList.length>0) {
			// create a dialog of type dropdownmenu
			
			var advanced= false;
			var htmlcode = displayDialog(advanced);  // create html code to fill the dialog
	
			// show generated html in a popup
			$("#ContainerDiv").html(htmlcode);
			
			$("#ContainerDiv").dialog({
				title: searchActionsLangBundle.t(gSearchActionsObj.operation),
				autoOpen: false,
				closeOnEscape: true,
				modal: true,
				draggable: true,
				resizable: true,
				width: 700,
				height: 'auto',
				close: function(event, ui) {
					window.closeAndReload();
				}, 
				// Add a class to the dialog so we can set the zIndex later...
				dialogClass: "searchbatchactionsdialog",// jQuery UI < 1.12   <= currently we are still on this version (dialogClass will become deprecated in 1.12)
				classes: {// jQueryUI >= 1.12
					"ui-dialog":"searchbatchactionsdialogfg",
					"ui-widget-overlay":"searchbatchactionsdialogbg"
				}
			});
			
			 // initialize calendar for date field in dialog
			if (gSearchActionsObj.attributeType && gSearchActionsObj.attributeType.toLowerCase() == 'datetime' || 
					gSearchActionsObj.inputType && gSearchActionsObj.inputType.toLowerCase() == 'datetime') { 
		
				$("#newDateAll").datepicker({
					dateFormat: "M dd, yy",
					onClose: function(dateText, inst){}
				});	
				
				// Trick jQueryUI into not putting the focus on the date picker when the popup opens...
				// Simply add the "autofocus" attribute to a random div inside the popup.
				$("#ContainerDiv .searchresultsbatchactions").attr("autofocus", "autofocus");
			}
			
			$("#ContainerDiv").dialog("open");
			
			// Set the zIndex of the dialog. We need to do this because we are mixing jQuery and bootstrap dropdowns when in a document source!
			if($(".searchbatchactionsdialogfg").length>0) {
				// jQuery UI >= 1.12
				$(".searchbatchactionsdialogfg").css({"z-index":zIndex+1});
				$(".searchbatchactionsdialogbg").css({"z-index":zIndex});
			} else {
				// jQuery UI < 1.12   <= currently we are still on this version
				$(".searchbatchactionsdialog").css({"z-index":(zIndex+1)});
				$(".searchbatchactionsdialog").prev(".ui-widget-overlay").css({"z-index":(zIndex)});
			}
			
			// initialization for select PM if needed (user is selected by default)
			if (gSearchActionsObj.inputType && gSearchActionsObj.inputType.toLowerCase() == 'projmgr') { 
				// initialize SDrop for manager
				changePMType('All');
			}
			
			// initialize role assignee widget if needed.
			if (gSearchActionsObj.inputType && gSearchActionsObj.inputType.toLowerCase() == 'role') { 
				initRoleAssign();
			}
			
			if (gSearchActionsObj.operation == 'setAttributeCategory') getAttributeCategories(); // fetch the attribute categories
		} else {
			 alert(searchActionsLangBundle.t('SetAttributesNothingSelected'));
		}
	
	};
	
	
	/*
	 * ---- After this - all the global methods (used in click listeners...) --- (TODO: REFACTOR)
	 */

	/**
	 * @return HTML code to display the dialog
	 */
	window.displayDialog = function (advanced) {
		var htmlcode = '';
		try {
			var todoList = gSearchActionsObj.todoList;
			var halfhours = 36;
			var changeMethod;
			var attUnit;
			
			htmlcode += '<div class="searchresultsbatchactions">';
			
			htmlcode += '<div class="wcr-form form-no-labels">';
			htmlcode += '<div class="form-section">';
			
			// top line with input field
			htmlcode += '<div  class="form-group">';
			
			htmlcode += '<label class="offset-label"></label>';
			htmlcode += '<div class="form-entry full-width">';
			
			switch (gSearchActionsObj.operation) {
			case 'setAttribute':
				htmlcode += '<div class="labeldiv">Set ' + WCXML.xmlencode(gSearchActionsObj.displayName) + '  to </div>';
				break;
			case 'setAttributeCategory':
				htmlcode += '<div class="labeldiv">Set ' + searchActionsLangBundle.t('AttributeCategory') + '  to </div>';
				break;
			default:
				htmlcode += '';
				break;	
			}
			
			var inputType = 'text';
			
			// see if it's a restricted set
			if (gSearchActionsObj.operation == 'setAttribute') {
				var optArr = gSearchActionsObj.rsetValues;
				if(optArr) {
					inputType = 'restricted';
					gSearchActionsObj.values = [""].concat(optArr);
				}
			}
			
			if (gSearchActionsObj.operation == 'setAttributeCategory') {
				inputType = 'restricted'; 
				gSearchActionsObj.values = []; // values are fetched with ajax later
			}
			
			if (gSearchActionsObj.operation == 'manageProjectMembers') {
				inputType = 'projMemberWPermissions'; 
			}
			
			// see if it is a date
			if (gSearchActionsObj.attributeType && gSearchActionsObj.attributeType.toLowerCase() == 'datetime') inputType='datetime';	
			
			// make sure inputtype is set correctly
			if(!gSearchActionsObj.inputType)
				gSearchActionsObj.inputType = inputType;
			else{
				if(gSearchActionsObj.inputType == 'status'){
					gSearchActionsObj.inputType = 'restricted';
				}
					
				inputType = gSearchActionsObj.inputType;
			}
				
			
			attUnit = gSearchActionsObj.attributeUnit ? gSearchActionsObj.attributeUnit.toLowerCase() : '';
			if (!advanced) {
				
				switch(inputType) {	
				case 'datetime':
					htmlcode += '<div class="form-datetime">';
					htmlcode += '<div class="form-date"><div class="input-group"><input type="text" size="16" id="newDateAll" name="newDateAll" valign="middle" class="form-control"></input>';
					htmlcode += '<label class="input-group-addon" for="newDateAll"><i id="CalendarImageAll" class="icon icon-WCR_Calendar"></i></label></div></div>';
					htmlcode += '<div class="form-time">' + WCHTML.displayHourDropdown(halfhours,'newHourAll','newHourAll',null) + '</div>';
					htmlcode += '</div>';
					break;
				case 'restricted':
					htmlcode += '<select id="newValueAll" class="form-control">';
					for (var opt=0; opt<gSearchActionsObj.values.length;opt++) {
						if(gSearchActionsObj.hasOwnProperty("labels")) {
							htmlcode += '<option value="'+WCXML.xmlencode(gSearchActionsObj.values[opt])+'">' + WCXML.xmlencode(gSearchActionsObj.labels[opt])+'</option>';
						} else {
							htmlcode += '<option value="'+WCXML.xmlencode(gSearchActionsObj.values[opt])+'">' + WCXML.xmlencode(gSearchActionsObj.values[opt])+'</option>';
						}
					}
					htmlcode += '</select>';
					break;
				case 'projmgr': // Select Project manager, either user or group, through an SDrop
					// draw 2 radio buttons and empty select
					htmlcode += '<div id="newValueAllDiv" style="display:inline-block; white-space: nowrap;">'
						+'<table></tr>'
						+'<td class="add-padding-right"><div class="radio"><label><input type="radio" name="newPMIsGroupAll" value="0" checked="checked" onclick="changePMType(\'All\')"></input>'+searchActionsLangBundle.t('USER')+'</label></div></td>'
						+'<td class="add-padding-left"><div class="radio"><label><input type="radio" name="newPMIsGroupAll" value="1" onclick="changePMType(\'All\')"></input>'+searchActionsLangBundle.t('GROUP')+'</label></div></td>'
						+'</tr></table>'
						+'<div class="add-spacing-top"><select id="newValueAll" class="form-control"></select></div></div>';
					break;
				case 'projMemberWPermissions':
					// draw 3 radio buttons and empty select
					htmlcode += '<div id="topProjMemberManageDiv" style="display: inline-block; white-space: nowrap;">'
						+'<table></tr>'
						+'<td class="add-padding-right"><div class="radio"><label><input type="radio" name="projMemberTypeRB" value="0" onclick="changeManageProjMemberType();"></input>'+searchActionsLangBundle.t('USER')+'</label></div></td>'
						+'<td class="add-padding-left add-padding-right"><div class="radio"><label><input type="radio" name="projMemberTypeRB" value="1" onclick="changeManageProjMemberType();"></input>'+searchActionsLangBundle.t('GROUP')+'</label></div></td>'
						+'<td class="add-padding-left"><div class="radio"><label><input type="radio" name="projMemberTypeRB" value="2" onclick="changeManageProjMemberType();"></input>'+searchActionsLangBundle.t('ROLE')+'</label></div></td>'
						+'</tr></table>'
						+'<div class="add-spacing-top"><select style="display:none" id="ManageProjectMemberSelector" class="form-control"></select></div>'
						+'</div>';
					break;
				case 'role':
					// draws a table with titles. Selectors will be added by initRoleAssign
					htmlcode += '<div class="wcr-table"><div class="wcr-table-section" id="newValueAllDiv"></div></div>'
						+'<div><div class="radio"><label><input type="radio" name="AssignmentType" checked="checked" value="0"></input>'+searchActionsLangBundle.t('ADD_MEMBERS')+'</label></div>'
						+'<div class="radio"><label><input type="radio" name="AssignmentType" value="1"></input>'+searchActionsLangBundle.t('REPLACE_MEMBERS')+'</label></div></div>';
					break;
				default: // text etc
					changeMethod = 'setAttributesValidateInput(this,\''+ attUnit + '\')';
					htmlcode += '<div class="input-group full-width"><input type="text" value="" id="newValueAll" onchange="'+ changeMethod +'" class="form-control">';
					
					if (attUnit) {
						switch (attUnit) {
						case 'datetime':
						case 'text':
						case 'richtext':
						case 'number':
						case 'float':
						break;
						default:
							htmlcode += '<span class="input-group-addon">' + gSearchActionsObj.attributeUnit + '</span>';
							break;
						} // end switch
					}
					
					htmlcode += '</div>';
					break;
				}
				
				if (gSearchActionsObj.operation!='setAttributeCategory' && gSearchActionsObj.operation!='manageProjectMembers' && gSearchActionsObj.operation!='setRole') 
					htmlcode += '<div class="add-spacing-top"><a href="#" onclick="showAdvancedDialog();event.preventDefault();">'+searchActionsLangBundle.t('ADVANCED')+'</a></div>';  // Advanced link
			} else {
				// Simple
				htmlcode += '<div><a href="#" onclick="showSimpleDialog();event.preventDefault();">'+searchActionsLangBundle.t('SIMPLE')+'</a></div>';  // Simple link
			
			}
			
			htmlcode += '</div></div>';
			
			//add extra 'Set Permission' controls row if handling 'Manage Project members/Permissions' bulk operation
			if (inputType == 'projMemberWPermissions') {			
				htmlcode += generateProjectMemberPermissionControls(gSearchActionsObj.haveTMicense, gSearchActionsObj.haveBOM);
			}
			
			// Start button
			htmlcode += '<div class="form-group"><label class="offset-label"></label><div class="form-entry"><input type="button" class="btn btn-primary" value="'+WCXML.xmlencode(searchActionsLangBundle.t('START'))+'" name="StartButton" onclick="executeTodoList(0)" >';
			// Close button
			htmlcode += '<input type="button" class="btn btn-default closeBtn" value="'+WCXML.xmlencode(searchActionsLangBundle.t('CLOSE'))+'" onclick="$(\'#ContainerDiv\').dialog(\'close\');"></div></div>';	
			
			// Main table 
			htmlcode += '<div class="form-group"><label class="offset-label"></label><div class="form-entry full-width">';
			
			htmlcode += '<div class="wcr-table"><div class="wcr-table-section"><table class="ordtable">';
			// Header
			htmlcode += '<thead><tr>';
			htmlcode += '<th class="td-narrow td-status">'+ searchActionsLangBundle.t('Status') +'</th>';
			if (gSearchActionsObj.type == 'projects') {
				htmlcode += '<th>'+ searchActionsLangBundle.t('Project') +'</th>';
			} else {
				htmlcode += '<th>'+ searchActionsLangBundle.t('Document') +'</th>';
			}
			if (advanced) {  // for attributes, put the attribute name as column header of the input column
				if (gSearchActionsObj.displayName) {
					htmlcode += '<th>'+  WCXML.xmlencode(gSearchActionsObj.displayName) + '</th>';
				} else {
					htmlcode += '<th>'+searchActionsLangBundle.t('VALUE')+'</th>';   // to do localization
				}
			}
			htmlcode += '<th>Comment</th></tr></thead>' ;	  // to do localization
			
			// Item list	
			htmlcode += '<tbody>';
			for (var item=0; item<todoList.length; item++) {
				var htmlcode2 = ''; // speed on IE
				
				htmlcode2 += '<tr>';
				
				htmlcode2 += '<td class="td-status"><i class="icon icon-WCR_Status-OK" style="visibility:hidden;" id="icon_'+item+'" title="'+ searchActionsLangBundle.t('STATUS_OK') +'"></i></td>';
				
				htmlcode2 += '<td>'+ WCXML.xmlencode(todoList[item].name)+'</td>';    // to do: check encoding, migth be over-encoded
				if (advanced) {		
					htmlcode2 += '<td class="td-nowrap">';
					switch(inputType) {	
					case 'datetime':
						htmlcode2 += '<div class="form-datetime">';
						htmlcode2 += '<div class="form-date"><div class="input-group"><input type="text" size="16" id="newDate_' + item +'" name="newDate_' + item +'" valign="middle" onchange="changeAdvancedValue('+item+')" value="'+$("#newDateAll").val()+'" class="form-control"></input>';
						htmlcode2 += '<label class="input-group-addon" for="newDate_' + item +'"><i id="CalendarImageAll" class="icon icon-WCR_Calendar"></i></label></div></div>';
						var currentHours = $("#newHourAll").val();
						htmlcode2 += '<div class="form-time">' + WCHTML.displayHourDropdown((currentHours==''?halfhours:currentHours),'newHour_'+ item,'newHour_' +item,null) + '</div>';
						htmlcode2 += '</div>';
						break;
					case 'restricted':
						htmlcode2 += '<select id="newValue_' + item +'" onchange="changeAdvancedValue('+item+')" class="form-control">';
						for (var opt=0; opt<gSearchActionsObj.values.length;opt++) {
							htmlcode2 += '<option value="' + WCXML.xmlencode(gSearchActionsObj.values[opt])+'"'
										+ ((gSearchActionsObj.values[opt] == todoList[item].newValue)?' selected="selected"':'')
										+'>' + gSearchActionsObj.values[opt]+'</option>';
						}
						htmlcode2 += '</select>';
						break;
					case 'projmgr':
						// draw 2 radio buttons and select with 1 value (which was selected before advanced was click). All are diabled
						// then, add a link which will make everythin editable
						var origSelected = $('input[name=newPMIsGroupAll]:checked').val();
						var origVal = $("#newValueAll option[value='"+todoList[item].newValue+"']").text();
						htmlcode2 += '<div id="newValue_'+item+'Div" style="display:inline-block; white-space: nowrap;">'
							+'<table><tr>'
							+'<td class="add-padding-right"><div class="radio"><label><input type="radio" disabled="disabled" name="newPMIsGroup_'+item+'" ' + ((origSelected == "1")? '' : ' checked="checked"')
							+'value="0" onclick="changePMType('+item+')"></input>'+searchActionsLangBundle.t('USER')+'</label></div></td>'
							+'<td class="add-padding-left add-padding-right"><div class="radio"><label><input type="radio" disabled="disabled" name="newPMIsGroup_'+item+'" ' + ((origSelected == "1")? ' checked="checked"' : '')
							+'value="1" onclick="changePMType('+item+')"></input>'+searchActionsLangBundle.t('GROUP')+'</label></div></td>'
							+'<td class="add-padding-left add-padding-right"><select disabled="disabled" loaded="0" id="newValue_'+item+'" class="form-control"><option value="'+todoList[item].newValue+'">'+WCXML.xmlencode(origVal)+'</option></select></td>'
							+'<td class="add-padding-left"><a id="changelink_'+item+'" href="javascript:clickPMType('+item+')">change</a></td>'
							+'</tr></table></div>';
						break;
					default:
						htmlcode2 += '<input type="text" value="'+ todoList[item].newValue+'" id="newValue_'+item+'" onchange="changeAdvancedValue('+item+')" class="form-control">';
						break;
					}  // end switch
					htmlcode2 += '</td>';
				}
				htmlcode2 += '<td id="feedback_'+item+'"></td>';
				
				htmlcode2 += '</tr>';
				
				htmlcode += htmlcode2;
			}
			htmlcode += '</tbody>';
			htmlcode += '</table></div></div>';
			
			htmlcode += '</div></div>';
			
			// Start and Close button
			htmlcode += '<div class="form-group"><label class="offset-label"></label><div class="form-entry"><input type="button" class="btn btn-primary" value="'+WCXML.xmlencode(searchActionsLangBundle.t('START'))+'" name="StartButton" onclick="executeTodoList(0)" >'
				+'<input type="button" class="btn btn-default closeBtn" value="'+WCXML.xmlencode(searchActionsLangBundle.t('CLOSE'))+'" onclick="$(\'#ContainerDiv\').dialog(\'close\');"></div></div>';
			
			// close wcr-form and form-section
			htmlcode+='</div></div>';
			
			htmlcode+='</div>';
			
			
		} catch(err) {
			alert(searchActionsLangBundle.t('DisplayDialogProblem') + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
		return htmlcode;
	};

	/**
	 * @author BVON
	 * 
	 * Will fill up the SDrop to select project manager, based on the selected radio button
	 * 
	 * @param index
	 * @param initValue
	 */
	window.changePMType = function(index, initValue) {
		var identifier;
		if(index != 'All'){
			identifier = '_' + index;
		}else{
			identifier = index;
		}
		var selected = $('input[name=newPMIsGroup'+identifier+']:checked').val();
	
		//load PM SDrop
		var selector = document.getElementById('newValue'+identifier);
		// destroy previous SDrop and reset value
		$(selector).val("");
		if(selector.SDrop){
			if(selected == "1"){
				selector.SDrop.setDataProvider(gGroupDataprovider,true);
			}else{
				selector.SDrop.setDataProvider(gPMUserDataprovider,true);
			}
		} else {
			if(selector){
				var dataProvider = ((selected == "1")? gGroupDataprovider : gPMUserDataprovider);	
				var options = {'selectorContainer':'body', 'onChangedByUser':  changeAdvancedValue.bind(this,index), minWidth: 300, maxWidth: 300};
				options['zIndex'] = sdropZIndex;
				if(initValue){
					options['initialValue'] = initValue;
					options['autoload'] = true;
				}
				new SDrop($(selector), "", dataProvider, options);
				$(selector).parent().css('display','inline-block');
			}
		}
	};

	/**
	 * @author BVON
	 * 
	 * Will create the necessary info to create a assigneeController. Then draws the controller, 
	 * adds it to the popup and stores the object in the HTML as well.
	 * 
	 */
	window.initRoleAssign = function(){
		// make datastructures needed for the ManageRoles entry
		var userDataprovider = new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=User"});
		var roles = new Array();
		roles.push({id:gSearchActionsObj.roleId, name:gSearchActionsObj.rolename, assignedGroups:"", assignedUsers:"", showActions:true});
		var roleMgr = new roleManagement ("", roles, searchActionsLangBundle.asMap(), "", gSearchActionsObj.inviteGroup);
		roleMgr.sdropWidth = 300;
		roleMgr.userDataProvider = userDataprovider;
		roleMgr.selectorContainer = 'body';
	
		$("#newValueAllDiv").append(roleMgr.getAssigneeTable());
		$("#newValueAllDiv")[0].roleMgr = roleMgr;
		
	};

	/**
	 * @author BVON
	 * 
	 * Will make the project manager selector editable and initialize it for a certain project (in advanced mode)
	 * 
	 * @param index
	 */
	window.clickPMType = function(index){
		// add '_' to index, because all fields contain the underscore
		var identifier = '_' + index;
		var elem = document.getElementById("newValue"+identifier);
		if(elem){
			$("#changelink"+identifier).hide();
			$("input[name=newPMIsGroup"+identifier+"]").removeAttr('disabled');
			$(elem).removeAttr('disabled');
			if(elem.loaded!="1"){
				changePMType(index, $(elem).val());
				elem.loaded="1";
			}
		}
	};

	/**
	 * @author JAMI
	 * @since 18.12.2013
	 * 
	 * Used when changing the 'Select Project Member Type' radiobutton selection 
	 * > will initalize the appropriate sDrop for further selection
	 * 
	 * 0 = User
	 * 1 = Group
	 * 2 = Role
	 */
	window.changeManageProjMemberType = function(){
		var selected = $('input[name=projMemberTypeRB]:checked').val();
	
		//load Project Member SDrop
		var selector = document.getElementById('ManageProjectMemberSelector');
		var finalProvider = null;
		if (selected === "1") {
			finalProvider = gGroupDataprovider;
		} else if (selected === "2") {
			finalProvider = gRoleDataprovider;
		} else {
			finalProvider = gUserDataprovider;
		}
		
		// destroy previous SDrop and reset value
		$(selector).val("");
		if(selector.SDrop){
			selector.SDrop.setDataProvider(finalProvider,true);
		} else {
			if(selector){
				new SDrop($(selector), "", finalProvider, {'selectorContainer':'body', 'onChangedByUser': projectMemberSelectorChanged.bind(this,selector,selected), minWidth: 300, maxWidth: 300, zIndex:sdropZIndex});
			}
		}
		
		//show the sDrop control
		$('#ManageProjectMemberSelector').show();
	};

	/**
	 * Event handler that manages state of Project Member Permission controls based on the entered value in Project Member selector
	 * 
	 * @param selectorElem - the sDrop element
	 * @param memberTypeCode - code of selected 'Project Member Type' (0 - User; 1 - Group; 2 - Role)
	 */
	window.projectMemberSelectorChanged = function(selectorElem, memberTypeCode) {
		// Get the selected id.
		var selection = $(selectorElem).val();
		
		if ((memberTypeCode == "0" && selection !== "NOUSER") ||
			(memberTypeCode == "1" && selection !== "NOGROUP") ||
			(memberTypeCode == "2" && selection !== "NOROLE")) {
			//some actual User/Group/Role was selected > show the permissions controls
			$('.proj_member_perm_row').show();
		}
	};


	window.closeAndReload = function () {
		// empty the content before hiding, to get rid of the buttons which keep hanging
		$("#ContainerDiv").html('');
		
		gSearchActionsObj.todoList = new Array();
	
		// Reload the page to show the result
		gSearchActionsObj.completionCallback(gSearchActionsObj.somethingDone);
	};

	/**
	* Gets list of attribute categories via Ajax. Probably limited to Admin 
	*/
	window.getAttributeCategories = function () {
		try {
			//var url='attcategorymgr.jsp?XSLPage=copyXML';
			var url='GetAttributeCategoryList.jsp';
			
			ajax({
					url: url,
					type: "GET",
					data: "xml",				
					synchronous: false,
					timeout: 10000,	// 10seconds
					onSuccess: function(xml) {
						try {
							var htmlcode=''; 
							
							var categoryListElem = WCXML.GetFirstFirstLevelChild(xml,'category_list'); 
							var categoryArr = WCXML.GetFirstLevelChilds(categoryListElem,'category');
							//var dropdownElem = document.getElementById('newValueAll');		
							if (categoryArr && categoryArr.length>0 ) {
								htmlcode += '<select id="newValueAll" class="form-control">';
								for (var category=0; category<categoryArr.length; category++) {								
									var categoryId = WCXML.GetElementAttribute(categoryArr[category],"id");
									var name = WCXML.GetElementText(categoryArr[category],"name");
									htmlcode+= '<option value="'+categoryId+'">'+WCXML.xmlencode(name)+'</option>';
								}
								htmlcode+= '</select>';
								//dropdownElem.innerHTML = htmlcode;
								$('#newValueAll').replaceWith(htmlcode);   // DT109809: innerHTML doesn't work on IE because you cannot replace the content of a select in IE
							}
							
							
						} catch(err) {
							alert(searchActionsLangBundle.t('AttrCatListProblem') + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
						}
						
					},
					onError: function(errMsg) {
						alert(searchActionsLangBundle.t('AttrCatProblem'));	
					}            
				});
			
			
			
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn') + ' getAttributeCategories ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	* Algorithm to avoid too many calls at the same time
	* Keep track of how many are still pending (keep a counter)
	* When too many sent, suspend for 0.5 seconds and count again
	*/
	window.executeTodoList = function (startIndex) {
		try {
			var baseUrl;
			var url;		
					
			var newValueAll = '';
			var newValueAllElem = document.getElementById("newValueAll");
			if (newValueAllElem) newValueAll = newValueAllElem.value;
			var todoList = gSearchActionsObj.todoList;
			
			// don't redo validation when it's a continuation of the work
			if(!startIndex){
				// first do validation for specific operations if needed
				switch (gSearchActionsObj.operation) {
					case 'manageProjectMembers':
						if(!$("#ManageProjectMemberSelector").val()){
							alert(searchActionsLangBundle.t('NOMEMBERTYPE'));
							return;
						}
						
						if($("input[name=permSetupFolderName]").length == 0){
							alert(searchActionsLangBundle.t('NOFOLDER'));
							return;
						}
						break;
					case 'setPM':
						if(newValueAllElem){
							if(newValueAll == ''){
								alert(searchActionsLangBundle.t('NOPMVALUE'));
								return;
							}
						} else {
							// check all values
							var fields = $("#ContainerDiv").find("select");
							for(var i=0 ; i<fields.length ; i++){
								if(fields[i].value == ''){
									alert(searchActionsLangBundle.t('NOPMVALUEALL'));
									return;
								}
							}
						}
						break;
				}
				
				// now do general validation
				if (newValueAllElem && newValueAll == '') {
					var okToContinue = confirm(searchActionsLangBundle.t('NoValuePrompt'));
					if (!okToContinue) return;
				}
				
				if (!todoList) throw new Error("todoList " + searchActionsLangBundle.t('DoesNotExist'));
				
				if(gSearchActionsObj.operation == 'setDueDate'){
					var all = document.getElementById("newDateAll");
					if(all){
						var newValue = findDateHourTime(document.getElementById("newDateAll"),document.getElementById("newHourAll"));
						if(newValue!==null && newValue < new Date().getTime()){
							alert(searchActionsLangBundle.t("CANNOT_SAVE__DUE_DATE_IN_PAST"));
							return;
						}
						if(newValue===null) {
							alert(searchActionsLangBundle.t("NODUEDATEENTERED"));
							return;
						}
					}else{
						// advanced
						for (var item=0; item<todoList.length; item++) {
							if(todoList[item].newValue && todoList[item].newValue < new Date().getTime()){
								alert(searchActionsLangBundle.t("CANNOT_SAVE__DUE_DATE_IN_PAST"));
								return;
							}
							if(todoList[item].newValue===null) {
								alert(searchActionsLangBundle.t("NODUEDATEENTERED"));
								return;
							}
						}
					}
				}
				
				if(gSearchActionsObj.inputType == 'datetime' && gSearchActionsObj.operation !== 'setDueDate'){
					var all = document.getElementById("newDateAll");
					if(all){
						var newValue = findDateHourTime(document.getElementById("newDateAll"),document.getElementById("newHourAll"));
						if (newValue === null) {
							var okToContinue = confirm(searchActionsLangBundle.t('NoValuePrompt'));
							if (!okToContinue) return;
						}
					}else{
						// advanced
						// no warning is given when one of the dates is not filled in (it will be cleared!)
					}
				}
				
				startIndex = 0;
			}
			
			// disable all inputs and hide images when hiding the start button
			$("#ContainerDiv :input").not(".closeBtn").attr("disabled","disabled");
			$("#ContainerDiv").find("i").not("[id^=icon_]").hide();
			$("#ContainerDiv").find("a").hide();
			$("input[name=StartButton]").hide();
			
			for (var item=startIndex; item<todoList.length; item++) {
				
				var urlArr = new Array();
				
				if (gSearchActionsObj.nAjaxSent > cMaxAjax) {
					// number of ongoing ajax calls too large, continue later
					setTimeout(executeTodoList.bind(this,item),200);
					return;  
				}
				// To do: encoding
				// To do: send large amounts not in one go.  What happens is that probably Tomcat batches them and then Ajax calls fail because of timeouts
				// Time out already put much higher, but still not a good method.
				if (todoList[item].done) continue;
				
				
				var newValue =  todoList[item].newValue;
				if (newValue === null) {
					newValue = newValueAll;
					if ( gSearchActionsObj.inputType == 'datetime') {
						newValue = findDateHourTime(document.getElementById("newDateAll"),document.getElementById("newHourAll"));
						if(newValue===null && gSearchActionsObj.operation !== 'setDueDate') {
							// fix: clear value when no date is entered (don't show "undefined error")
							newValue="";
						}
					}
				}
				// make sure values are properly encoded
				newValue = encodeURIComponent(newValue);
				
				switch (gSearchActionsObj.operation) {
				case 'setAttribute':
					baseUrl = 'SetAttribute.jsp?';
					url = baseUrl + 'projectId=' + todoList[item].projectId + '&attributeId='+ gSearchActionsObj.attributeId + '&newvalue=' + newValue;
					if (gSearchActionsObj.type == 'documents') url += '&documentId=' + todoList[item].documentId;				
					urlArr.push(url);
					break;
				case 'setAttributeCategory':  
					if (gSearchActionsObj.type == 'projects') {
						url = 'doprojectdetailsactions.jsp?actionVal=SetProjectCategory&category=' + newValue +  '&projectID=' + todoList[item].projectId;				
					} else { 				
						url = 'dodocdetailsactions.jsp?actionVal=SetCategory&category=' + newValue +  '&docVerID=' + todoList[item].docVerId;
					}
					urlArr.push(url);
					break;
				case 'setPM':  
					var newValueIsGroupSelect = $('input[name=newPMIsGroup_'+item+']:checked').val();
					if (newValueIsGroupSelect === undefined) {
						newValueIsGroupSelect = $('input[name=newPMIsGroupAll]:checked').val();
					}
					url = 'CreateProject.jsp?action=modify&projectID=' + todoList[item].projectId + '&managerid=' + newValue + "&ismanagergroup=" + newValueIsGroupSelect;				
					urlArr.push(url);
					break;
				case 'setStatus':  
					url = 'CreateProject.jsp?action=modify&projectID=' + todoList[item].projectId + '&newprojectstatus=' + newValue;	
					urlArr.push(url);
				break;
				case 'setDueDate':  
					url = 'CreateProject.jsp?action=modify&projectID=' + todoList[item].projectId + '&duedate=' + newValue;	
					urlArr.push(url);
				break;
				case 'manageProjectMembers':
					var selectedType = $('input[name=projMemberTypeRB]:checked').val();
					var memberStr = selectedType=="0"?"&usermemberid=":selectedType=="1"?"&groupmemberid=":"&roleid=";
					memberStr += $("#ManageProjectMemberSelector").val();
					var permissions = ["view", "add", "update", "delete", "download", "lock", "changetask", "changeapproval", "changebom"];
					
					$("input[name=permSetupFolderName]").each(function(){
						var foldername = $(this)[0].value;
						var folderStr = foldername=="ROOT"?"":"&foldername="+encodeURIComponent(foldername);
						var prefix = foldername + "_";
						var permissionStr = "";
						for(var p=0 ; p<permissions.length ; p++){
							var permElem = document.getElementById(prefix + permissions[p]);
							if(permElem){
								permissionStr += "&"+permissions[p]+"=" + (permElem.checked? '1' : '0');
							}
						}
						url = 'UpdateProjectMember.jsp?&projectid=' + todoList[item].projectId + "&inviteOnFail=1" + folderStr + memberStr + permissionStr;
						urlArr.push(url);
					});
					break;
				case 'setRole':
					var assignmentType = $('input[name=AssignmentType]:checked').val();
					var firstRoleMember = true;
					var roleMgr = $("#newValueAllDiv")[0].roleMgr;
					var controllers = roleMgr.assigneeControllers[gSearchActionsObj.roleId];
					
					for(var i=0 ; i<controllers.length; i++){
						var controller = controllers[i];
						var assignmentStr = "";
						if(controller.groupId){
							if(controller.userId){
								assignmentStr += "&newusermemberid=" + controller.userId;
								if($(controller.inviteGroupCheckbox).is(":checked")){
									assignmentStr += "&newgroupmemberid=" + controller.groupId; 
								}
							}else{
								assignmentStr += "&newgroupmemberid=" + controller.groupId;
							}
						}else{
							assignmentStr += "&newusermemberid=" + controller.userId;
						}
						var actionStr = "action=" + ((!firstRoleMember || assignmentType==0)?"add":"replace");
						
						url = 'UpdateProjectMemberAssignment.jsp?'+actionStr+'&projectid=' + todoList[item].projectId + "&rolename=" + encodeURIComponent(gSearchActionsObj.rolename) + assignmentStr;
						urlArr.push(url);
						firstRoleMember = false;
					}
					
					break;
				default:
					throw new Error(searchActionsLangBundle.t('UnknownOperation') + " : " + gSearchActionsObj.operation);
				
				}; // end switch
				
				// Note: if there are multiple actions on 1 project, they will be executed synchronously (2nd action waits for first to complete, etc...)
				gSearchActionsObj.nSubAjaxErrors[item] = new Array();
				doSDKAjax(urlArr, item, 0);
			}
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn') + ' executeTodoList ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	 * Make the actual SDK call through Ajax. With the subindex, it indicates which subcall should be executed. When the call returns, 
	 * it will call the next subcall for this project.
	 * Note: if there are multiple actions on 1 project, they will be executed synchronously (2nd action waits for first to complete, etc...)
	 * 
	 * @param urlArr 
	 * 					Array that contains all urls for which an ajax call is needed
	 * @param projInd
	 * 					Index of project to which call is done
	 * @param subInd
	 * 					Index of call that should be made at this moment
	 */
	window.doSDKAjax = function(urlArr, projInd, subInd){
		if(subInd == 0){
			var icon = document.getElementById('icon_'+projInd);
			icon.src = './images/ajax_loader.gif';
			icon.style.visibility = 'visible';
		}
	
		ajax({
			url: urlArr[subInd],
			type: "GET",
			data: (gSearchActionsObj.operation =='setAttributeCategory'? "text":"xml"),
			objHdl: projInd,
			synchronous: false,
			timeout: 100000,	// 100 seconds
			onSuccess: function(item, urlArray, current){ 
				var returnFunc = function(xml) {
					try {
						gSearchActionsObj.nAjaxSent--;
						var nonSDKErrorMsg = "";
						var errorElem = null;
						var msgElem =  document.getElementById('feedback_'+this.objHdl);
						if (gSearchActionsObj.operation !='setAttributeCategory') {
							errorElem = WCXML.GetFirstFirstLevelChild(xml,"error");
						} else {
							//JAMI: setting attribute category is not an SDK call > any error returns as a HTML-based text response with <span>ERROR_MSG</span>
							nonSDKErrorMsg = getNonSDKTextResponseErrorMsg(xml);
						}
						
						var errMsg = null;
						if (errorElem || nonSDKErrorMsg.length > 0) {
							
							if (gSearchActionsObj.operation !='setAttributeCategory') {
								errMsg = WCXML.GetElementText(errorElem,"message");
							} else {
								errMsg = nonSDKErrorMsg;
							}
							
							gSearchActionsObj.nSubAjaxErrors[item].push(errMsg);
						} else {
							gSearchActionsObj.somethingDone = true;
						}
						
						if(urlArray.length==1){
							if (errMsg) 
								msgElem.innerHTML = '<p class="text-danger"><strong>' + errMsg + "</strong></p>";
						}else{
							msgElem.innerHTML = '<p><strong>' + (current + 1) + "/" + urlArray.length
							+ ', '+gSearchActionsObj.nSubAjaxErrors[item].length + ' ' + searchActionsLangBundle.t('ERRORS') + '</strong></p>';
							for(var errInd=0 ; errInd<gSearchActionsObj.nSubAjaxErrors[item].length ; errInd++){
								msgElem.innerHTML += '<p class="text-danger"><strong>'+gSearchActionsObj.nSubAjaxErrors[item][errInd] + '</strong></p>';
							}
						}
						
						if((current + 1) == urlArray.length){
							var icon2 = document.getElementById('icon_'+this.objHdl);
							if(gSearchActionsObj.nSubAjaxErrors[item].length==0){
								icon2.className = 'icon icon-WCR_Status-OK';
								icon2.title = searchActionsLangBundle.t('STATUS_OK');
							}else{
								icon2.className = 'icon icon-WCR_Status-Failed';
								icon2.title = searchActionsLangBundle.t('STATUS_FAILED');
							}
							gSearchActionsObj.todoList[this.objHdl].done = true;
						}else{
							doSDKAjax(urlArray, item, (current+1));
						}			
					} catch(err) {
						//alert('Problem showing feedback at line "  + err.lineNumber + ' : ' + err.message);
					}
					
				};
				
				return returnFunc;
			}(projInd, urlArr, subInd),
			onError: function(item, urlArray, current){ 
				var returnFunc = function(errMsg) {
					gSearchActionsObj.nAjaxSent--;
					var msgElem =  document.getElementById('feedback_'+this.objHdl);
					switch (errMsg) {
					case 998:
					case '998':
						errMsg = '998: You got logged off (time out?)';
						break;
					default:
						// just say what http error it is
						errMsg = 'HTTP' + errMsg;
					}
					if(urlArray.length==1){
						msgElem.innerHTML = '<p class="text-danger"><strong>' + errMsg + "</strong></p>";
					} else {
						gSearchActionsObj.nSubAjaxErrors[item].push(errMsg);
						msgElem.innerHTML = '<p><strong>' + (current + 1) + "/" + urlArray.length
						+ ', '+gSearchActionsObj.nSubAjaxErrors[item].length+' ' + searchActionsLangBundle.t('ERRORS') + '</strong></p>';
						for(var errInd=0 ; errInd<gSearchActionsObj.nSubAjaxErrors[item].length ; errInd++){
							msgElem.innerHTML += '<p class="text-danger"><strong>'+gSearchActionsObj.nSubAjaxErrors[item][errInd] + '</strong></p>';
						}
					}
					
					if((current + 1) == urlArray.length){
						var icon2 = document.getElementById('icon_'+this.objHdl);
						icon2.className = 'icon icon-WCR_Status-Failed';
						icon2.title = searchActionsLangBundle.t('STATUS_FAILED');
						gSearchActionsObj.todoList[this.objHdl].done = true;// Even mark it as done if it failed. Just to avoid to retry endlessly
					}else{
						doSDKAjax(urlArray, item, (current+1));
					}	
					//alert("ajax call returned with error. \nError Message: " + errMsg + "\ncalled URL: " + url);
				};
				
				return returnFunc;
			}(projInd,urlArr,subInd)
		});
		gSearchActionsObj.nAjaxSent++;
	};
	
	window.changeAdvancedValue = function(item) {
		try {		
			var todoList = gSearchActionsObj.todoList;
			
			switch (gSearchActionsObj.inputType) {
			case 'text':
			case 'restricted':
			case 'projmgr':
				var elem = document.getElementById('newValue_'+item);
				if (elem) todoList[item].newValue =  elem.value;
				break;
			case 'datetime':
				var dateField = document.getElementById('newDate_'+item);
				var hourField = document.getElementById('newHour_'+item);
				todoList[item].newValue = findDateHourTime(dateField,hourField);
				
				
				break;
			default:
				throw new Error(searchActionsLangBundle.t('UnknownInputType'));
			} // end switch
			
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn') + ' changeAdvancedValue ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
	
	};

	window.findDateHourTime = function(dateField,hourField) {  // candidate to move to WCHTML 
		var newValue = null;
		try {
			
			if (dateField && hourField) {
				var dateString = dateField.value;
	
				if (dateString) {
					
					var newDate = new Date(dateString);  // normalized date string is used to create a Date object.
					
					if ( (newDate.constructor != Date) || isNaN(newDate.getTime()) ) {
						// entered something which can't be interpreted as a date
						// make it empty
						dateField.value = "";
						
						 
					} else {    
	
						// add hours and minutes
						var halfhours = 18;
						if (hourField.value != null && hourField.value!="") halfhours = parseInt(hourField.value);
						if (halfhours == -1) {
							halfhours = 18;
							hourField.value = halfhours;
						}
						
						var hours = Math.floor(halfhours/2);
						var minutes = (hours * 2 == halfhours)? 0:30;
						newDate.setHours(hours); newDate.setMinutes(minutes);
						newValue = newDate.getTime();
					}
				} else { // this is not a date --> somebody emptied the field --> put it on "No date"
					dateField.value = "";
				}
				
			}
			
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn')+ ' findDateHourTime ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
		return newValue;  // can be null
	};


	window.showAdvancedDialog = function () {
		try {		
			var todoList = gSearchActionsObj.todoList;
			
			// copy content of the new value field to all new fields (by puttin them in todoList[item].newValue before asking a repaint
			var newValueAll = "";
			if ( gSearchActionsObj.inputType == 'datetime'){
				newValueAll = findDateHourTime(document.getElementById("newDateAll"),document.getElementById("newHourAll"));
			} else {
				var newValueAllElem = document.getElementById("newValueAll");
				if (newValueAllElem) newValueAll = newValueAllElem.value;
			}
					
			for (var item=0; item<todoList.length; item++) {
				todoList[item].newValue =  newValueAll;
			}
			
			var htmlcode = displayDialog(true);  // create html code to fill the dialog
	
			$("#ContainerDiv").html(htmlcode);
			
			for (var item=0; item<todoList.length; item++) {
				$("#newDate_"+item).datepicker({
						dateFormat: "M dd, yy",
						onClose: function(dateText, inst){}
				});	
			}	
	
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn') + ' showAdvancedDialog ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
	};
	
	window.showSimpleDialog = function () {
		try {	
			var todoList = gSearchActionsObj.todoList;	
			// clear contents of all values
			var newValueAllElem = document.getElementById("newValueAll");
			if (newValueAllElem) newValueAllElem.value="";
			for (var item=0; item<todoList.length; item++) {
				todoList[item].newValue =  "";
			}
			
			var htmlcode = displayDialog(false);  // create html code to fill the dialog
			$("#ContainerDiv").html(htmlcode);	
	
			if (gSearchActionsObj.inputType && gSearchActionsObj.inputType.toLowerCase() == 'projmgr') { 
				// initialize SDrop for manager
				changePMType('All');
			} else if (gSearchActionsObj.attributeType && gSearchActionsObj.attributeType.toLowerCase() == 'datetime' || 
					gSearchActionsObj.inputType && gSearchActionsObj.inputType.toLowerCase() == 'datetime') { 
				$("#newDateAll").datepicker({
					dateFormat: "M dd, yy",
					onClose: function(dateText, inst){}
				});	
			}
	
		} catch (err) {
			alert(searchActionsLangBundle.t('ProblemIn')+ ' showAdvancedDialog ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	* Validates the input in a field to change attribute values
	* @param elem {Object}  the DOM object of the input field
	* @param attUnit {String} lowercase value of the attribute unit/type
	*/
	window.setAttributesValidateInput = function (elem,attUnit) {
		try {
			switch (attUnit) {
				case 'number':
					// check whether this is an integer
					WCHTML.checkNumericInput(elem,"Integer",true,'');
					break;  
				case 'float':
				//
				// All numeric units...
				// NOTE: see table in database: "webcenter.Conversions" for the full list of units.
				//
				case 'mm':
				case 'mm2':
				case 'cm':
				case 'cm2':
				case 'm':
				case 'm2':
				case 'in':
				case 'in2':
				case 'ft':
				case 'ft2':
				case 'ft3':
				case 'in3':
				case 'cm3':
				case 'dm3':
				case 'm3':
				case 'lbs':
				case 'gr':
				case 'kg':
					WCHTML.checkNumericInput(elem,null,true,'');
					break;
				default:
					break; // do nothing for the rest, typically text
			}
		
		} catch(err) {	
			alert(searchActionsLangBundle.t('ProblemIn')+ ' setAttributesValidateInput ' + searchActionsLangBundle.t('AtLine') + err.lineNumber + ' : ' + err.message);	
		}
	};

	/**
	 * Generates HTML data for 'Project Member Permission setup' controls
	 * 
	 * @param hasTMLicense
	 * @param haveBOMFunctionality
	 * @returns {String} HTML structure
	 */
	window.generateProjectMemberPermissionControls = function (hasTMLicense, haveBOMFunctionality) {
		var resultHTML = "";
		//first write the header row
		resultHTML += '<div class="proj_member_perm_row" style="display:none"><div class="form-group"><label class="offset-label"></label>';
			
		resultHTML += '<div class="form-entry full-width">';
		
		resultHTML += '<div class="wcr-table"><div class="wcr-table-section"><table class="ordtable">';
		resultHTML += generatePermissionTableHeaderRow(hasTMLicense, haveBOMFunctionality);
		
		//now write the ROOT permission matrix controls
		resultHTML += '<tbody id="permissionTableBody">';
		resultHTML += generatePermissionMatrixControlsRow(hasTMLicense, haveBOMFunctionality, 'ROOT');
		
		//add 'Add Folder' link to the end of the list
		var colspanForLinkRow = 9;
		if (hasTMLicense) {
			colspanForLinkRow = colspanForLinkRow + 1;
		}
		if (haveBOMFunctionality) {
			colspanForLinkRow = colspanForLinkRow + 1;
		}
		resultHTML += '<tr id="addNewFolderLinkRow">';
		resultHTML += '<td colspan="'+colspanForLinkRow+'">';
		resultHTML += '<div style="float: left;"><div class="input-group"><label class="input-group-addon" for="newPermFolderName" style="width: auto;">' + searchActionsLangBundle.t('ADD_FOLDER')+'</label><input class="form-control" id="newPermFolderName" size="30"/></div></div>';
		resultHTML += '<div style="float: left; padding-top: 3px;" class="add-padding-left"><i id="newPermFolderImg" class="icon icon-WCR_Add12" onclick="addNewFolderPermRow(this, '+hasTMLicense+','+haveBOMFunctionality+', false);" title="' + searchActionsLangBundle.t('ADD') + '" style="cursor: pointer;"/></div>';
		resultHTML += '</td></tr>';
		
		resultHTML += '</tbody></table></div></div>';
		
		resultHTML += '</div></div></div>';
		
		return resultHTML;
	};

	/**
	 * Generates HTML representing the 'table header' row for 'Set Member Permissions' controls table
	 * @param hasTMLicense
	 * @param haveBOMFunctionality
	 * @returns {String} HTML 
	 */
	window.generatePermissionTableHeaderRow = function(hasTMLicense, haveBOMFunctionality) {
		var resultHTML = '';
		
		resultHTML += '<thead id="PermsTableHeader"><tr id="SelectMemberPermsRow">';
		resultHTML += '<th>'+searchActionsLangBundle.t('FOLDER')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('FULL_PERMS')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('VIEW')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('ADD')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('UPDATEPERM')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('DELETEPERM')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('DOWNLOADPERM')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('CHECKOUTPERM')+'<br/>'+searchActionsLangBundle.t('START_STOP_APP_PERM')+'</th>';
		resultHTML += '<th class="td-checkbox">'+searchActionsLangBundle.t('CHANGEAPPROVALPERM')+'</th>';
		
		if (hasTMLicense) {
			resultHTML += '<th>'+searchActionsLangBundle.t('CHANGETASKPERM')+'</th>';
			}
		
		if (haveBOMFunctionality) {
			resultHTML += '<th>'+searchActionsLangBundle.t('CHANGEBOM')+'</th>';
		}
	
		resultHTML += '</thead></tr>';
		
		return resultHTML;
	};

	/**
	 * Generates a custom 'permission matrix row' for a specific Project Folder (ROOT folder is supported)
	 * 
	 * @param folderName
	 * @param hasTMLicense
	 * @param haveBOMFunctionality
	 * @param doShow - boolean; determines whether to hide or show the target row thats being generated
	 * @returns {String} HTML 
	 */
	window.generatePermissionMatrixControlsRow = function(hasTMLicense, haveBOMFunctionality,  folderName, doShow) {
		var isRootFolder = false;
		var folderNameLabel = folderName;
		if (!folderName) {
			folderName = 'ROOT';
			folderNameLabel = searchActionsLangBundle.t('ROOTCHAR');
		}	
		if (folderName.toUpperCase() === 'ROOT') {
			folderName = 'ROOT'; // make sure the case is right here
			folderNameLabel = searchActionsLangBundle.t('ROOTCHAR');
			isRootFolder = true;
		}
		
		folderName = WCXML.xmlencode(folderName);
		onclickFolderName = WCXML.xmlencode(WCHTML.encodeForJScript(folderName));
		
		var resultHTML = '';
		resultHTML += '<tr class="proj_member_perm_row"';
		if (!doShow) {
			resultHTML += 'style="display:none"';
		}
		resultHTML += '>';
		resultHTML += '<td>';
		//insert MINUS icon that allows removing of this whole row from the list of Folder Permissions
		resultHTML += '<i class="icon icon-WCR_Remove12" class="'+folderName+'_removeButton"  onclick="removeFolderPermissionRow(this, '+hasTMLicense+', '+haveBOMFunctionality+');" title="' + searchActionsLangBundle.t('REMOVE') + '" style="cursor:Pointer; padding-right:10px"/>';
		resultHTML +=  folderNameLabel+'<input type="hidden" name="permSetupFolderName" value="'+folderName+'"/></td>';
		
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_selectAllPerms" id="'+folderName+'_selectAllPerms" value="" alt="Check/Clear All" onclick="handleFullPermissionsCheckboxInRow(this, \''+onclickFolderName+'\');"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_view" id="'+folderName+'_view" value="1" ';
		if (isRootFolder) {
			//VIEW permission is always on for the ROOT folder
			resultHTML += 'checked="checked" disabled="disabled"';
		}
		resultHTML += '></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_add" id="'+folderName+'_add" value="1"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_update" id="'+folderName+'_update" value="1"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_delete" id="'+folderName+'_delete" value="1"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_download" id="'+folderName+'_download" value="1"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_lock" id="'+folderName+'_lock" value="1"></td>';
		resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_changeapproval" id="'+folderName+'_changeapproval" value="1"></td>';
		if (hasTMLicense) { // only show Task Change for the ROOT folder (+only if TM license is available) 
			resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_changetask" id="'+folderName+'_changetask" value="1"';
			if (!isRootFolder) {
				resultHTML += ' disabled=disabled';
			}
			resultHTML += '></td>';
		}
		
		if ( haveBOMFunctionality) { // only show Change BOM for the ROOT folder (+only if BOM functionality is enabled)
			resultHTML += '<td class="td-checkbox"><input type="checkbox" name="'+folderName+'_changebom" id="'+folderName+'_changebom" value="1"';
			if (!isRootFolder) {
				resultHTML += ' disabled=disabled';
			}
			resultHTML += '></td>';
		}
	
		resultHTML += '</tr>';
		
		return resultHTML;
	};

	/**
	 * Handle change of 'All Permissions' checkbox in 'Set Project Member Permission for a Folder' row
	 * 
	 * @param elem - the checkbox that was updated
	 * @param folderNamePrefix - helps locating other checkboxes from this permission row
	 */
	window.handleFullPermissionsCheckboxInRow = function(chkBoxElem, folderNamePrefix) {
		var isMainToggleOn = chkBoxElem.checked;
		var chBxArray =  $(chkBoxElem).closest('tr').find(':checkbox');
		//iterate over each of the checkboxes and set their value accordingly
		chBxArray.each(function() {
			if (!this.disabled && isMainToggleOn !== this.checked) {
				this.checked =  isMainToggleOn;
			}
		});
	};

	/**
	 * Removes 'Folder Permissions setup' table row that contains the Minus Button where it was triggered
	 * 
	 * @param sourceMinusImgElem
	 */
	window.removeFolderPermissionRow = function(sourceMinusImgElem, hasTMLicense, haveBOMFunctionality) {
		//check if ROOT folder is not being removed
		var permSetupFolderName = $(sourceMinusImgElem).closest('tr').find('input[name=permSetupFolderName]').val();
		if (permSetupFolderName === searchActionsLangBundle.t('ROOTCHAR')) {
			//removing ROOT folder row > add an extra link to easily get it back
			var extraLinkHTML = '<a style="padding-left:10px" id="addRootFolderLink" href="#" onclick="addNewFolderPermRow(this, '+hasTMLicense+', '+haveBOMFunctionality+', true);event.preventDefault();">'+searchActionsLangBundle.t('ADD_ROOT_FOLDER')+'</a>';
			$('#newPermFolderImg').after(extraLinkHTML);	
		}
		
		//remove the current row
		$(sourceMinusImgElem).closest('tr').remove();
	};

	/**
	 * Adds a new 'Folder Permissions setup' row to the appropriate table 
	 * 
	 * @param sourceAddFolderPermRowLinkElem
	 */
	window.addNewFolderPermRow = function(sourceAddFolderPermRowLinkElem, hasTMLicense, haveBOMFunctionality, addingRoot) {
		var newFolderNameVal = $('#newPermFolderName').val();
		var newPermRowHTML = generatePermissionMatrixControlsRow(hasTMLicense, haveBOMFunctionality, newFolderNameVal, true);
		
		// check for duplicates, case sensitive (2 folders with different cases are allowed)
		var rows = $("input[name=permSetupFolderName]");
		for(var i=0 ; i<rows.length ; i++){
			if(rows[i].value.toLowerCase() == newFolderNameVal.toLowerCase()){
				alert(searchActionsLangBundle.t('FOLDER_EXISTS'));
				return;
			}
		}
	
		if (addingRoot || (newFolderNameVal && newFolderNameVal.toUpperCase() === "ROOT")) {
			//insert the ROOT Folder row at the beginning
			var headerRow = $('#PermsTableHeader');
			headerRow.after(newPermRowHTML);
			//and remove the 'Add ROOT folder' link
			$('#addRootFolderLink').remove();
		} else {
			//cannot create a new Folder Row unless a name is specified
			if (!newFolderNameVal) {
				alert(searchActionsLangBundle.t('NO_FOLDER_NAME'));
				return;
			}
			
			//locate the parent table that holds Member Permission controls 
			//var parentTable = $(sourceMinusImgElem).closest('table');
			var lastRow = $('#addNewFolderLinkRow');
			
			//insert the HTML at the end of the table (but keep the 'Add New Folder' link as last)
			lastRow.before(newPermRowHTML);
		}
		
		$('#newPermFolderName').val('');
	};
	
	return SearchResultsBatchActions;
});