define([
	"underscore",
	"utils/idUtils",
	"./AssetBrowserModel",
	"./AssetBrowserTreePane",
	"./AssetBrowserImagePane",
	"./AssetBrowserVisibilityDropdown",
	"./AssetBrowserSearchIntegration",
	"utils/Observable"
], function (
	_,
	idUtils,
	AssetBrowserModel,
	AssetBrowserTreePane,
	AssetBrowserImagePane,
	AssetBrowserVisibilityDropdown,
	AssetBrowserSearchIntegration,
	Observable
) {
	"use strict";


	/**
	 * @param {SearchCategoryInfo} info - an object containing information about the Attribute Category.
	 * @param {SearchSetupInformation} searchSetup - the object model of search (contains criteria + the layout options of the search (also contains the configured lists))
	 * @param {Function} autoRerun - the function to call when something changes in search and we need to rerun
	 */
	var AssetBrowser = function (info, searchSetup, autoRerun) {
		// Some information
		this._info = info;
		this._searchSetup = searchSetup;
		this._config = this._searchSetup.getLayoutOptions();
		this._autoRerun = autoRerun;

		// Generate an unique id for this Asset Browser
		this._id = idUtils.newClientSideId("assetbrowser") + "_"; // Needs to be exactly 17 characters!

		// Keep track of whether the Asset Browser is initialized
		this._configurationinitialized = false;
		this._hasvalidconfiguration = false;

		// DOM elements
		this._leftpanel = null;
		this._imagepanel = null;
		this._dropdownarea = null;

		// Asset Browser Object Model
		this._model = new AssetBrowserModel(this._id);

		// This object is observable!
		Observable.mixin(this, "assetbrowser");

		// Config - the width of the left panel
		this._model.panelWidth = parseInt(this._config.getOption("SelectTreePanelWidth"), 10) - 30;

		// Config - Whether to show the dropdown to switch
		this._dropdownvisible = this._config.getOption("chkbox_ImageAreaAllowChange");


		// No valid configuration loaded yet
		this._hasvalidconfiguration = false;
		this._ready = false;

		// Start loading the Attribute Category
		this._info.load(_.bind(function () {
			// Read the cascading list
			var cascadingConfig = this._config.getOption("cascading_attribute_trees");

			this._model.cascadingLists = [];
			this._model.cascadingListsByAttrCatListId = {};
			this._model.lists = this._info.getAttributeCategory().lists;
			var usedListIdx = {};
			if (cascadingConfig !== null && cascadingConfig.getSubList("treeElem") !== null && cascadingConfig.getSubList("treeElem").length !== 0) {
				_.each(cascadingConfig.getSubList("treeElem"), function (treeElem) {
					var type = treeElem.getValue("type");
					var listName = treeElem.getValue("name");
					var depth = treeElem.getValue("depth");
					if (!depth) {
						depth = 0; // meaning all columns
					}
					var height = treeElem.getValue("height");
					if (!height) {
						height = 0; // meaning no maximum
					}

					// Note: for copied lists, need special naming pattern in the saved search
					if (listName && type == 'list') {
						var listIdx = -1;
						for (var list2 = 0; list2 < this._model.lists.length; list2++) {
							var extendedName = this._model.lists[list2].name;
							var counter = this._model.lists[list2].counter;
							if (counter && counter > 1) {
								extendedName += ' (' + counter + ')';
							}
							if (extendedName == listName) {
								listIdx = list2;
								break;
							}
						}

						if (listIdx >= 0) {
							if (_.has(usedListIdx, "" + listIdx)) {
								// Skip this list... It is already in use.
								// Why would you show the same list TWICE in the same attribute category??
								// Unless it is copied -> in which case it is not considered the same list
								return;
							} else {
								usedListIdx["" + listIdx] = true;
							}

							// This Asset Browser has at least one valid list
							this._hasvalidconfiguration = true;

							// Make an object representing this cascading list and put it in the model
							this._model.cascadingLists.push({
								cascadingOrderIndex: this._model.cascadingLists.length, 
								label: listName,
								listName: listName,
								listIdx: listIdx,
								depth: depth,
								height: height,
								treeRootNode: null // Here we will put the actual tree (in memory) => but we first need to load the list!
							});
						}
					}
				}, this);
			}

			if (this._model.cascadingLists.length === 0) {
				return; // No configured lists...
			}

			// Start loading the lists also
			this._info.loadWithLists(this._model.cascadingLists, _.bind(function () {
				// Lists are also loaded now!

				// Make the "in memory" tree for every cascading list
				_.each(this._model.cascadingLists, function (cascadingList) {
					this._buildCascadingTreeRepresentation(cascadingList);
				}, this);

				// Make the root node in the Model also (just put a node above all the roots of all the cascading lists)
				this._model.rootNode = {};
				this._model.rootNode.idx = null;
				this._model.rootNode.items = _.map(this._model.cascadingLists, _.property("treeRootNode"));
				this._model.rootNode.itemsByIndex = _.indexBy(this._model.rootNode.items, "idx");

				// Put also some derived information in the model (for fast lookup)
				this._model.cascadingListsByAttrCatListId = _.indexBy(this._model.cascadingLists, "listIdx");

				// The "tree" is fully ready into model now. (Just the data, no view)
				
				// Handle the searching on the selection and the selection from the search criteria:
				new AssetBrowserSearchIntegration(this._searchSetup, this._model, this._autoRerun);

				// Set the initial focus in the tree
				this._model.setFocusTreeId('allLists');

				// Which panels to show... Left panel? Image panel?
				this._model.changeDisplayVisibilityOption(this._config.getOption("SelectImageAreaStyle"));


				// Asset Browser is now ready!
				// All the data is available => we can now generate some views that work on the model!

				// Loading done
				this._ready = true;
				this._emit("ready");

			}, this));
		}, this));
	};

	/**
	 * Build a real tree (in memory) from the "combinations" in the Attribute Category List
	 */
	AssetBrowser.prototype._buildCascadingTreeRepresentation = function (cascadingList) {
		// 1) Find the matching list
		var listIdx = cascadingList.listIdx;
		var listObj = this._model.lists[listIdx];

		// The top level of the tree
		var root = {
			idx: listIdx,
			items: [],
			itemsByIndex: {}
		};

		// 2) Where are we currently in the tree?

		// We are currently on the root level
		var where = [];
		// For every level, keep the object
		var stack = [root];

		// What is the actual depth that will be shown?
		var nColumns = listObj.columns.length;
		var depth = cascadingList.depth;
		if (depth > 0 && depth < nColumns) {
			nColumns = depth;
		}

		// Loop through all combinations and build the tree
		
		// How to access the combinations?
		//   listObj.combinations = what Attribute Category uses and loads => can already be loaded (for lists that load "Immediate" or if the design panel was initiated already)
		//    => we need to make sure we do not touch this! Because otherwise we might break the cascading dropdowns which can be used together with the asset browser.
		// For the "On the Fly" lists, we load those ourself + we load only the columns we really need. (if depth is configured)
		//    => so, we should not use "combinations" for that (Attribute Category will be missing the other columns)
		//    => so, we introduced "listObj.assettreecombinations" => only loaded for "On The Fly"
		var combinations = listObj["assettreecombinations"];
		if(!combinations) {
			combinations = listObj.combinations;
		}
		
		_.each(combinations, function (combination) {
			var valueArr = combination.split("~");
			if (valueArr.length > nColumns) {
				// This combination has more columns than we want to show
				valueArr = valueArr.slice(0, nColumns);
			}

			// Compare with "where". Are we deeper? Are we in a different place? Are we in one of the parents?
			var foundDifference = false;
			var equalPath = []; // The path from the root till the place where some different "folder" shows up
			var differentPath = []; // The different "folder" and down...
			for (var i = 0; i < valueArr.length; i++) {
				if (!foundDifference) {
					if (i >= where.length) {
						// The new combination is a child of the old one
						foundDifference = true;
					} else if (parseInt(where[i]) != parseInt(valueArr[i])) {
						// The new combination has a different value at index i
						foundDifference = true;
					}
				}
				if (!foundDifference) {
					equalPath.push(valueArr[i]);
				} else {
					differentPath.push(valueArr[i]);
				}
			}

			if (!foundDifference) {
				// This combination is still the same as the last one.
				return; // Go to next combination
			}

			// 1) Exit the children that are no longer in the "equalsPath"
			where = where.slice(0, equalPath.length);
			stack = stack.slice(0, equalPath.length+1);


			// 2) Enter (or make) the children that are in the "differentPath"
			_.each(differentPath, function (enterChild) {
				var parent = stack[stack.length - 1];
				var newChild = null;
				if (_.has(parent.itemsByIndex, enterChild)) {
					// We visited that child already once before...
					// Use that element instead
					newChild = parent.itemsByIndex[enterChild];
				} else {
					// First time we visit this "folder"
					// Make a new child in the tree
					newChild = {
						idx: parseInt(enterChild),
						items: [],
						itemsByIndex: {}
					};
					parent.itemsByIndex[enterChild] = newChild;
					parent.items.push(newChild);
				}
				where.push(enterChild);
				stack.push(newChild);
			}, this);
		}, this);

		// Second iteration => clean up all elements that have an empty value and no children!
		function removeEmptyValueLeafs(item, depth) {
			_.each(item.items, function (child) {
				// Do I have children myself? If so, visit those first.
				removeEmptyValueLeafs.call(this, child, depth + 1);

				// Do I still have children?
				if (child.items.length === 0) {
					// No more children...
					// What is my label?
					var label = this._model.getLabelImpl(listObj, depth + 1, child.idx);

					if (label === '' || label === ' ') {
						// Delete myself from the tree
						item.items = _.without(item.items, child);
						delete item.itemsByIndex["" + child.idx];
					}
				}
			}, this);
		}

		// Note: root is considered depth: 0 (first column: depth = 1)
		removeEmptyValueLeafs.call(this, root, 0);


		// Ok! So we have a tree now
		// Store it inside the cascading list
		cascadingList.treeRootNode = root;
	};

	/**
	 * Check whether the Asset Browser has all it's loading done
	 */
	AssetBrowser.prototype.whenReady = function (callback) {
		if (this._ready) {
			callback();
		} else {
			this.on("ready", callback);
		}
	};

	/**
	 * Whether this asset browser has a correct configuration in the search
	 * @returns {Boolean}
	 */
	AssetBrowser.prototype.isCorrectlyConfigured = function () {
		if (!this._ready) {
			throw new Error("Illegal state! Configuration has not been loaded yet.");
		}
		return this._hasvalidconfiguration;
	};
	
	/**
	 * <p>Make the visibility dropdown</p>
	 * 
	 * <p>NOTE: Might actually create nothing if the configuration was set to not show the dropdown!!!</p>
	 */
	AssetBrowser.prototype.makeVisibilityDropdown = function (div) {
		if (!this.isCorrectlyConfigured()) {
			throw new Error("Illegal state! Trying to initialize an Asset Browser that is not correctly configured. Check function isConfigured first!");
		}
		// Put it there
		if(this._dropdownvisible) {
			this._objDropdown = new AssetBrowserVisibilityDropdown(div, this._model);
		}
	};
	
	/**
	 * <p>Make the image pane</p>
	 * 
	 * <p>NOTE: Depending on whether the Tree Pane is configured to be visible in search, this component might look a bit different or not be shown.</p>
	 * 
	 * @param div
	 */
	AssetBrowser.prototype.makeImagePane = function (div) {
		if (!this.isCorrectlyConfigured()) {
			throw new Error("Illegal state! Trying to initialize an Asset Browser that is not correctly configured. Check function isConfigured first!");
		}
		
		new AssetBrowserImagePane(div, this._model);
	};
	
	/**
	 * <p>Make the left tree pane</p>
	 * 
	 * <p>NOTE: Depending on whether the Tree Pane is configured to be visible in search, this component might look a bit different or not be shown.</p>
	 */
	AssetBrowser.prototype.makeTreePane = function (div) {
		if (!this.isCorrectlyConfigured()) {
			throw new Error("Illegal state! Trying to initialize an Asset Browser that is not correctly configured. Check function isConfigured first!");
		}
		
		// Change the div's a bit...
		$(div)
			.attr("id", this._id + "leftTreePane")
			.attr("valign", "top")
			.addClass("searchassetbrowserLeftTreePane")
			.css({
				"width": this._config.getOption("SelectTreePanelWidth") + "px",
				"min-width": this._config.getOption("SelectTreePanelWidth") + "px",
			});
		
		new AssetBrowserTreePane(div, this._model);
	};

	/**
	 * Called at end of initialisation or when clearing the form
	 */
	AssetBrowser.prototype.clear = function () {
		// Clear the selection
		this._model.clearSelection();
		
		// Clear the expanded folders
		this._model.collapseAll();
		
		// Do we want to reset the focus?
		this._model.resetFocus();
	};

	return AssetBrowser;
});