define([
	"underscore",
	"utils/Observable"
], function (
	_,
	Observable
) {
	"use strict";

	/**
	 * Make a new Asset Browser Model
	 */
	var AssetBrowserModel = function (id) {
		// An unique id for this Asset Browser
		this.id = id;

		// All lists in the Attribute Category
		this.lists = null;

		// Objects with all information about the cascading trees used in the Asset Browser
		this.cascadingLists = null;

		// Inverse lookup... Lookup a cascading list by the id of the list in the Attribute Category
		this.cascadingListsByAttrCatListId = null;

		// Make a root node (above all the cascading lists even)
		// Note, in the tree... at all levels. Property itemsByIndex => uses strings as keys (not numbers)
		this.rootNode = null;

		// Configuration
		// -> The width of the left panel
		this.panelWidth = -1;

		// -- private --

		// Current state
		this._treeandimagevisibilityoption = null;

		// Current selection (contains ids -> see further)
		this._selection = [];

		// Current expanded rows in the tree (contains ids -> see further)
		this._expandedids = [];

		// This object is observable!
		Observable.mixin(this, "assetbrowsermodel");
	};
	
	//
	// -- Checkbox utils --
	//

	// private - classes for the glyph icons
	var classEmpty = "icon icon-WCR_Search-Tree-Deselected12";
	var classChecked = "icon icon-WCR_Search-Tree-Selected12";
	var classIndeterminate = "icon icon-WCR_Search-Tree-ChildSelected12";
	var classesAll = classEmpty+" "+classChecked+" "+classIndeterminate;
	
	/**
	 * Return the checkbox html
	 */
	AssetBrowserModel.prototype.utilCheckboxMakeHtml = function(id, clickAction) {
		// Old code - working with real checkboxes
		//return '<input type="checkbox" id="' + id + '" onclick="' + clickAction + '"/>';
		
		// New code - working with glyph icons
		return '<i class="'+classEmpty+'" id="' + id + '" onclick="' + clickAction + '" style="cursor:pointer;"/>';
	};
	
	/**
	 * Set the checkbox state
	 * 
	 * @param {jQuery} chkbxElem
	 * @param {boolean} checked
	 * @param {boolean} indeterminate
	 */
	AssetBrowserModel.prototype.utilCheckboxSetState = function(chkbxElem, checked, indeterminate) {
		// Old code - working with real checkboxes
		//chkbxElem.prop("checked", checked);
		//chkbxElem.prop("indeterminate", indeterminate);
		
		// New code - working with glyph icons
		chkbxElem.removeClass(classesAll).addClass(indeterminate?classIndeterminate:(checked?classChecked:classEmpty));
	};
	

	//
	// -- ID handling utils --
	//

	/**
	 * Convert an id "0_12_3" to an array [0,12,3]
	 */
	AssetBrowserModel.prototype.convertIdToArray = function (id) {
		if (id === "allLists") {
			// Special case
			return [];
		}
		return _.map(id.split("_"), function (e) {
			return parseInt(e, 10);
		});
	};

	/**
	 * Convert an array [0,12,3] to an id "0_12_3"
	 * 
	 */
	AssetBrowserModel.prototype.convertArrayToId = function (arr) {
		if (arr.length === 0) {
			// Special case
			return "allLists";
		}
		return _.map(arr, function (e) {
			return "" + e;
		}).join("_");
	};

	//
	// -- Get labels for a certain id --
	//


	/**
	 * Get a label (optimized, but more complex to use)
	 * 
	 * @param selListObj
	 * @param depth
	 * @param index
	 */
	AssetBrowserModel.prototype.getLabelImpl = function (selListObj, depth, index) {
		if (depth === -1) {
			// Label of allLists
			return null;
		} else if (depth === 0) {
			// Label of the list
			return selListObj.assetbrowserlabel ? selListObj.assetbrowserlabel : selListObj.name;
		} else {
			// Label of some column in the list
			return selListObj.columns[depth - 1].values[index];
		}
	};

	/**
	 * Get a label for a certain id
	 * 
	 * @param {String} id
	 * @param {String} the label
	 */
	AssetBrowserModel.prototype.getLabel = function (id) {
		var idarr = this.convertIdToArray(id);
		var selListObj = this.lists[idarr[0]];
		return this.getLabelImpl(selListObj, idarr.length - 1, idarr[idarr.length - 1]);
	};

	/**
	 * Get all the labels for the full path of the id
	 * 
	 * @param {String} id
	 * @param {String} the label
	 */
	AssetBrowserModel.prototype.getLabels = function (id) {
		var idarr = this.convertIdToArray(id);
		var selListObj = this.lists[idarr[0]];

		return _.map(idarr, function (e, i) {
			return this.getLabelImpl(selListObj, i, e);
		}, this);
	};

	//
	// -- Getting a node in the tree by a certain id
	//
	AssetBrowserModel.prototype.getTreeNodeById = function (id) {
		// The id = a path in the tree
		var idarr = this.convertIdToArray(id);

		// The node we are currently in
		var node = this.rootNode;

		// Follow the path in the tree
		for (var i = 0; i < idarr.length; i++) {
			node = node.itemsByIndex["" + idarr[i]];
		}
		return node;
	};

	//
	// -- Handling of visibility --
	//

	/**
	 * Change the display visibility option
	 */
	AssetBrowserModel.prototype.changeDisplayVisibilityOption = function (treeandimagevisibilityoption) {
		this._treeandimagevisibilityoption = treeandimagevisibilityoption;
		this._emit("visibility");
	};

	/**
	 * Whether the tree panel is configured as visible
	 */
	AssetBrowserModel.prototype.isTreePanelVisible = function () {
		return this._treeandimagevisibilityoption === "LeftAndImagePanel" || this._treeandimagevisibilityoption === "LeftPanelOnly";
	};

	/**
	 * Whether the tree panel is configured as visible
	 */
	AssetBrowserModel.prototype.isImagePanelVisible = function () {
		return this._treeandimagevisibilityoption === "LeftAndImagePanel" || this._treeandimagevisibilityoption === "ImagePanelOnly";
	};

	/**
	 * Get the display visibility option
	 * @returns
	 */
	AssetBrowserModel.prototype.getDisplayVisibilityOption = function () {
		return this._treeandimagevisibilityoption;
	};

	//
	// -- Which "folders" are expanded? --
	//

	/**
	 * Get the expanded folder ids
	 */
	AssetBrowserModel.prototype.getExpandedIds = function () {
		return this._expandedids;
	};

	/**
	 * Expand or collapse a certain row
	 * 
	 * @param {String} id - the row to expand
	 */
	AssetBrowserModel.prototype.doExpand = function (id, expanded) {
		var wasExpanded = this.isExpanded(id);
		if (expanded === wasExpanded) {
			return;
		}
		if (expanded) {
			this._expandedids.push(id);
			this._emit("expandedchange", id, true);
		} else {
			this._expandedids = _.without(this._expandedids, id);
			this._emit("expandedchange", id, false);
		}
	};
	
	/**
	 * Check whether the row with a certain id is expanded
	 * 
	 * @param id
	 * @returns {Boolean}
	 */
	AssetBrowserModel.prototype.isExpanded = function (id) {
		return _.indexOf(this._expandedids, id) !== -1;
	};

	/**
	 * Toggle the expansion of a certain row
	 * 
	 * @param id
	 */
	AssetBrowserModel.prototype.toggleExpansion = function (id) {
		this.doExpand(id, !this.isExpanded(id));
	};

	/**
	 * Expand the asset browser to show the complete selection
	 */
	AssetBrowserModel.prototype.expandAllSelection = function () {
		// Get the selection from the model
		var selection = this.getSelection();

		// Expand according to the selection
		for (var i = 0; i < selection.length; i++) {
			var idarr = this.convertIdToArray(selection[i]);
			var partialidarr = [];

			// Go thru' each level expanding the tree
			for (var j = 0; j < idarr.length; j++) {
				partialidarr.push(idarr[j]);
				if (j == idarr.length - 1) {
					// Last level = already selected
				} else {
					// Some level => expand! 
					this.doExpand(this.convertArrayToId(partialidarr), true);
				}
			}
		}
	};
	
	/**
	 * Collapse the whole tree
	 */
	AssetBrowserModel.prototype.collapseAll = function () {
		_.each(this._expandedids, function(id) {
			this.doExpand(id, false);
		}, this);
	};

	//
	// -- What items are currently selected? --
	//

	/**
	 * Get the selection
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @returns {Array}
	 */
	AssetBrowserModel.prototype.getSelection = function () {
		return this._selection;
	};

	/**
	 * Set the selection
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * WARNING: Using this method could get the Asset Browser in some illegal state!
	 * Make sure that the provided selection is the "minimal selection"!
	 * 
	 * @param selection
	 */
	AssetBrowserModel.prototype.setSelection = function (selection) {
		this.clearSelection();
		this._selection = selection || [];
		_.each(this._selection, function (id) {
			this._emit("selectionchange", id, true);
		}, this);
		this._emit("selectionchangefinal");
	};

	/**
	 * Clears the selection
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 */
	AssetBrowserModel.prototype.clearSelection = function () {
		var selectionBefore = this._selection;
		this._selection = [];
		_.each(selectionBefore, function (id) {
			this._emit("selectionchange", id, false);
		}, this);
		this._emit("selectionchangefinal");
	};

	/**
	 * Is a certain id selected?
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @param {String} id
	 * @returns {Boolean}
	 */
	AssetBrowserModel.prototype.isSelected = function (id) {
		return _.indexOf(this._selection, id) !== -1;
	};
	
	/**
	 * Is a certain id selected?
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @param {String} id
	 * @returns {Boolean}
	 */
	AssetBrowserModel.prototype.isSelectedOrParent = function (id) {
		return this.findSelectedParentId(id)!==null;
	};
	
	/**
	 * Has this item selected children?
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @param {String} id
	 * @returns {Boolean}
	 */
	AssetBrowserModel.prototype.hasSelectedChildren = function (id) {
		var found = false;
		_.each(this._selection, function (selectID) {
			if(selectID.indexOf(id+"_")===0 && id!==selectID) {
				found = true;
			}
		}, this);
		return found;
	};
	
	/**
	 * For a checkbox, find the parent that is selected (could also be the element itself)
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @param id
	 * @returns parentid - the id of the parent that is selected or null if none of the parent elements are selected
	 */
	AssetBrowserModel.prototype.findSelectedParentId = function(id) {
		var parentid = null;
		_.each(this._selection, function (selectID) {
			if(id.indexOf(selectID+"_")===0 || id === selectID) {
				parentid = selectID;
			}
		}, this);
		return parentid;
	};

	/**
	 * Select a checkbox in the list...
	 * 
	 * Note: selection is stored as "minimal selection" => only the top most element where all the children are selected is stored!
	 * 
	 * @param id
	 * @param select - true = select; false = deselect
	 */
	AssetBrowserModel.prototype.doSelect = function (id, select) {
		
		// Check if this item was already part of the "minimal selection"
		var idx = _.indexOf(this._selection, id);

		if (select) {
			// Select... We want to keep the "minimal selection" (= always select the top most element => consider all below selected!)
			// => so if we can select a parent, do so
			
			if (this.isSelectedOrParent(id)) {
				// Already selected
				return;
			}

			// First, deselect all the children
			_.each(this._selection, function (selectID) {
				// if name of iterated checkbox starts with name of selected checkbox, it's a child
				if ((selectID.indexOf(id + '_') === 0) && id != selectID) {
					// Remove it from the selection
					this._selection = _.without(this._selection, selectID);
					this._emit("selectionchange", selectID, false);
				}
			}, this);
			
			// Go through all the parents to see if we should not select a parent instead
			var idarr = this.convertIdToArray(id);
			for(var depth = idarr.length-1;depth>1;depth--) {
				idarr.splice(idarr.length-1,1);
				var parentid = this.convertArrayToId(idarr);
				var parentNode = this.getTreeNodeById(parentid);
				
				// Find the DIRECT children selected in this parent (all the other ones have already been removed)
				var selectedSiblings = _.filter(this._selection, function (selectID) {
					return selectID.indexOf(parentid + '_') === 0;
				}, this);
				
				// Check if all the children are selected except for the newly selected one
				if(parentNode.items.length-1 === selectedSiblings.length) {
					// So, selecting this checkbox would result in all the children of the parent being selected...
					// So, select the parent instead.
					
					// First, we need to deselect all the children at this level...
					this._selection = _.difference(this._selection, selectedSiblings);
					_.each(selectedSiblings, function (selectID) {
						this._emit("selectionchange", selectID, false);
					}, this);
				} else {
					// We really do need to select this checkbox (parent does not have all other children selected)
					// => So, break to do the actual selection
					break;
				}
				
				// We should select the parent instead of this element + continue loop
				id = parentid;
			}
			
			// Ok, we got out of the loop...
			// Select the one with "id" => that is the topmost parent we can select
			// However, we cannot select the "list" or the root above the list...
			if(this.convertIdToArray(id).length>1) {
				this._selection.push(id);
				this._emit("selectionchange", id, true);
			}
			
		} else {
			if (idx !== -1) {
				// This selection was part of the minimal selection (= always select the top most element => consider all below selected!)
				// => Easy, we just have to remove it
				this._selection.splice(idx, 1);
				this._emit("selectionchange", id, false);
			} else {
				// This selection was not part of the minimal selection
				// Instead, some parent has all his children selected
				// Find the parent in the selection list
				var parentId = this.findSelectedParentId(id);
				var parentSelectionIdx = _.indexOf(this._selection, parentId);
				if(parentId!==null) {
					// Ok, so this parent had all his children selected, but now we deselected a child
					this._selection.splice(parentSelectionIdx, 1);
					this._emit("selectionchange", parentId, false);
					
					// Get the ids
					var parentidarr = this.convertIdToArray(parentId);
					var idarr = this.convertIdToArray(id);
					
					// Get the node in the tree
					var node = this.getTreeNodeById(parentId);
					var prefix = parentId;
					
					// Select all levels down
					for(var depth=parentidarr.length;depth<idarr.length;depth++) {
						// At this level we deselected the following:
						var deselectIdx = idarr[depth];
						
						// All the other ones we need to select
						_.each(node.items, function(child) {
							var childidx = child.idx;
							var childid = prefix+"_"+childidx;
							if(childidx!==deselectIdx) {
								// Select it
								this._selection.push(childid);
								this._emit("selectionchange", childid, true);
							}
						}, this);
						
						// For the next level...
						prefix = prefix+"_"+deselectIdx;
						node = node.itemsByIndex[""+deselectIdx];
					}
				}
			}
		}
		this._emit("selectionchangefinal");
	};

	/**
	 * Toggle the selection of a certain item.
	 */
	AssetBrowserModel.prototype.toggleSelection = function (id) {
		this.doSelect(id, !this.isSelectedOrParent(id));
	};

	//
	// -- The "focus" of the Asset Browser => what is shown in the Image Panel
	//

	/**
	 * Get the element which is currently focussed (for the image panel)
	 */
	AssetBrowserModel.prototype.getFocusTreeId = function () {
		return this._currentFocusTreeId;
	};

	/**
	 * Set the element which is currently focussed (for the image panel)
	 * @param newFocusTreeId
	 */
	AssetBrowserModel.prototype.setFocusTreeId = function (newFocusTreeId) {
		var before = this._currentFocusTreeId;
		this._currentFocusTreeId = newFocusTreeId;
		this._emit("focuschange", this._currentFocusTreeId, before);
	};

	/**
	 * Clear the focus
	 */
	AssetBrowserModel.prototype.resetFocus = function () {
		this.setFocusTreeId("allLists");
	};

	return AssetBrowserModel;
});