define(["underscore", "../../Utils", "../../dataobjects/ValueData"], function(_, utils, ValueData) {
	
	/**
	 * The component that is responsible for making the selection do something with search... and the other way around!
	 * 
	 * NOTE: Except for the configuration (AssetBrowser.js), this is the only place where the asset browser code interacts with search in any way!
	 * 
	 * NOTE 2: The model contains all the information we need (we don't care about the other views on the model like a tree or image pane).
	 * 
	 * @param {SearchSetupInformation} searchModel - the model of search
	 * @param {AssetBrowserModel} assetBrowserModel - the model of the Asset Browser
	 * @param {Function} autoRerunFunction - a function to be called to rerun
	 */
	var AssetBrowserSearchIntegration = function (searchModel, assetBrowserModel, autoRerunFunction) {
		this._searchModel = searchModel;
		this._assetBrowserModel = assetBrowserModel;
		this._autoRerun = autoRerunFunction;
		
		// Whether some selection in some list of the Cascading tree needs updating.
		this._cascadingListSelectionDirty = new Array(this._assetBrowserModel.cascadingLists.length);
		this._cascadingListSearchDirty = new Array(this._assetBrowserModel.cascadingLists.length);
		for(var i=0;i<this._cascadingListSelectionDirty.length;i++) {
			this._cascadingListSelectionDirty[i] = true;// We need to update the selection at least once in the beginning
			this._cascadingListSearchDirty[i] = false;// Assume the values in search are correct
		}
		
		// Whether we are currently updating the search object model OR asset browser model
		// If so, ignore events that it throws!
		this._updatingModels = false;
		
		// All the criteria from the Search Object Model we are using
		this._criteriaMap = {};
		
		//
		// Listen and use the Search Object Model (+ fill the criteria map)
		//
		// For each cascading list
		for(var i=0;i<this._assetBrowserModel.cascadingLists.length;i++) {
			var cascadingList = this._assetBrowserModel.cascadingLists[i];
			// Find the matching Attribute Category list
			var listObj = this._assetBrowserModel.lists[cascadingList.listIdx];
			
			// For all the columns in the list...
			for(var col=0;col<listObj.columns.length;col++) {
				// find the attribute
				var colObj = listObj.columns[col];
				if (!colObj) continue;
				var attObj = colObj.attObj;
				if (!attObj) continue;
				
				// find the criterion
				var fieldid = "db_attr_"+attObj.id;
				var criterion = this._searchModel.getCriteriaList().findByFieldId(fieldid);
				if(criterion===null) {
					// Make a new criterion if it does not exist yet
					criterion = this._searchModel.getCriteriaList().createItem(fieldid);
					var vd = new ValueData();
					vd.setValue("value", "");
					criterion.setValue(vd, false);
					this._searchModel.getCriteriaList().add(criterion);
				}
				
				// "Lock" it, it should not be removed because we are keeping a reference!
				criterion.take();
				
				// Put it in the map we have
				this._criteriaMap[attObj.id] = criterion;
				
				// Listen to changes in the search object model!
				criterion.onChange(_.bind(function(idx) {
					if(this._updatingModels) {
						// We are causing these change events, ignore them :)
						return;
					}
					
					// We need to update the selection in the asset browser
					this._cascadingListSelectionDirty[idx] = true;
					
					// Make sure to do the update only once (could be multiple attributes in the same list that are changed)
					_.defer(_.bind(this._updateAssetBrowserSelection, this));
				}, this, i));
			}
		}
		
		// When the Selection in the Asset Browser Model changes => update search
		this._assetBrowserModel.on("selectionchange", _.bind(function(id, show) {
			if(this._updatingModels) {
				// We are causing these change events, ignore them :)
				return;
			}
			var idarr = this._assetBrowserModel.convertIdToArray(id);
			var listIdx = idarr[0];
			var cascadingListIdx = this._assetBrowserModel.cascadingListsByAttrCatListId[listIdx].cascadingOrderIndex;
			this._cascadingListSearchDirty[cascadingListIdx]=true;
		}, this));
		
		this._assetBrowserModel.on("selectionchangefinal", _.bind(function() {
			if(this._updatingModels) {
				// We are causing these change events, ignore them :)
				return;
			}
			this._updateSearchQueries();
		}, this));
		
		// Update the Asset Browser with the initial selection from search
		this._updateAssetBrowserSelection();
	};
	
	/**
	 * Search Model => Asset Browser Model
	 * 
	 * Copies the selection that was/is in Search to the Asset Browser
	 * 
	 * Calls update on each list in the asset browser - if necessary for that list
	 */
	AssetBrowserSearchIntegration.prototype._updateAssetBrowserSelection = function() {
		this._updatingModels = true;
		try{
			for(var i=0;i<this._assetBrowserModel.cascadingLists.length;i++) {
				if(this._cascadingListSelectionDirty[i]) {
					this._cascadingListSelectionDirty[i] = false;
					this._fromSearchToAssetBrowser(this._assetBrowserModel.cascadingLists[i]);
				}
			}
			
			// Show the selected items in the tree
			this._assetBrowserModel.collapseAll();
			this._assetBrowserModel.expandAllSelection();
		} finally {
			this._updatingModels = false;
		}
	};
	
	/**
	 * Search Model => Asset Browser Model
	 * 
	 * Copies the selection that was/is in Search to the Asset Browser
	 * 
	 * Calls update on each list in the asset browser - if necessary for that list
	 */
	AssetBrowserSearchIntegration.prototype._updateSearchQueries = function() {
		this._updatingModels = true;
		try {
			for(var i=0;i<this._assetBrowserModel.cascadingLists.length;i++) {
				if(this._cascadingListSearchDirty[i]) {
					this._cascadingListSearchDirty[i] = false;
					this._fromAssetBrowserToSearch(this._assetBrowserModel.cascadingLists[i]);
				}
			}
		} finally {
			this._updatingModels = false;
		}
		
		//
		// Do a rerun if needed
		//
		this._autoRerun();
	};
	
	
	/**
	 * Search Model => Asset Browser Model
	 * 
	 * Copies the selection that was/is in Search to the Asset Browser
	 * 
	 * There can be multiple cascading lists. This method will only update one of them at a time.
	 * 
	 * @param {Object} cascadingList - a reference to the cascadingList object in the Asset Browser
	 */
	AssetBrowserSearchIntegration.prototype._fromSearchToAssetBrowser = function(cascadingList) {
		// Get the index of the list (in the Attribute Category)
		var listIdx = cascadingList.listIdx;
		
		// Get the Attribute Category list (useful for labels and such)
		var listObj = this._assetBrowserModel.lists[listIdx];
		
		// Get the actual depth of the asset browser
		var nColumns = listObj.columns.length;
		var depth = cascadingList.depth;
		if (depth > 0 && depth < nColumns) {
			nColumns = depth;
		}
		
		// Note: keys in the map are numbers, but of datatype string
		var selectedIdsPerLevelMap = [];
		var selectedIdsPerLevelCount = [];
		
		// For all the columns in the list...
		for(var col=0;col<nColumns;col++) {
			var currentLevelSelectedIdsMap = {};
			
			// find the attribute
			var colObj = listObj.columns[col];
			if (!colObj) {
				// How can a list not have a column??
				// We still need an entry in the map though since we access it by column index.
				selectedIdsPerLevelMap.push({});
				selectedIdsPerLevelCount.push(0);
				continue;
			}
			var attObj = colObj.attObj;
			if (!attObj) {
				// Some column in the list is in the asset browser BUT it does not have a matching attribute in the attribute category.
				// This will probably behave rather strange...
				if(!this._warnedcolumns) {
					console.log("Warning - Column of list used in Asset Browser which does not have a matching Attribute in the Attribute Category. This might behave in unexpected ways. (column "+colObj.columnName+")");
				}
				// We still need an entry in the map though since we access it by column index.
				selectedIdsPerLevelMap.push({});
				selectedIdsPerLevelCount.push(0);
				continue;
			}
			
			// Get the search criterion & the values
			var searchCriterion = this._criteriaMap[attObj.id];
			var searchQuery = searchCriterion.getValue().getValue("value");
			var valueArr = utils.luceneQueryToArrayOfValues(searchQuery);
			
			// However, we need the id's...
			var foundIds = 0;
			var idLookupMap = _.invert(colObj.values);
			_.each(valueArr, function(value) {
				if(_.has(idLookupMap, value)) {
					currentLevelSelectedIdsMap[idLookupMap[value]] = true;
					foundIds++;
				}
			});
			
			// Push the values
			selectedIdsPerLevelMap.push(currentLevelSelectedIdsMap);
			selectedIdsPerLevelCount.push(foundIds);
		}
		// Don't show the warnings again and again...
		this._warnedcolumns = true;
		
		// Now, loop through the tree and build the selection
		function buildSelection(node, depth, prefix) {
			var selection = [];
			var alldirectchildrenselected = true;
			
			// Check all the children of this node
			_.each(node.items, function(item) {
				// My index
				var childIdx = ""+item.idx;
				var childId = prefix+"_"+childIdx;
				
				// Selection in the child tree
				var childSelection = buildSelection(item, depth+1, childId);
				
				// Am I selected?
				var meSelected = _.has(selectedIdsPerLevelMap[depth], childIdx) || selectedIdsPerLevelCount[depth] === 0;
				
				// If all my direct children are selected (ignoring empty values) and I am selected too
				// => don't use the child selection and consider myself selected
				if(meSelected && childSelection.alldirectchildrenselected) {
					selection.push(childId);
				} else if(meSelected){
					// Add the selected children
					selection = selection.concat(childSelection.selection);
					alldirectchildrenselected = false;
				} else {
					// Me and my sub tree are not selected
					alldirectchildrenselected = false;
				}
			});
			
			return {selection: selection, alldirectchildrenselected:alldirectchildrenselected};// some children are selected
		}
		var selectionData = buildSelection(cascadingList.treeRootNode, 0, ""+listIdx);
		var selection = null;
		if(!selectionData.alldirectchildrenselected) {
			// Some checkboxes are checked in the tree, but not all of them.
			selection = selectionData.selection;
		} else if(selectedIdsPerLevelCount[0] > 0) {
			// Exception for first level checkboxes.
			// If the user selected everything... (in the cascading dropdowns in the design panel or when loading a saved search with some values set) 
			// Do not select the parent (we don't have a parent anymore)
			// Instead, keep all the checkboxes selected...
			selection = selectionData.selection;
		} else {
			// Everything is selected, but this is because the design panel has nothing filled in...
			// Do not check all checkboxes!
			selection = [];
		}
		
		// If there are multiple lists in the asset browser => be careful! Above code only generates the selection for ONE list
		// Replace ONLY the selection of this list in the Asset Browser Model
		_.each(this._assetBrowserModel.getSelection(), function(selectId) {
			if(selectId.indexOf(""+listIdx+"_") != 0) {
				// Selection is not for this list => make sure to not lose it!!
				selection.push(selectId);
			}
		}, this);
		
		// Set the selection in the Asset Browser Object Model
		this._assetBrowserModel.setSelection(selection);
	};
	
	/**
	 * Asset Browser Model => Search Model
	 * 
	 * Copies the selection that was/is in the Asset Browser and applies it to Search
	 * 
	 * There can be multiple cascading lists. This method will only update one of them at a time.
	 * 
	 * @param {Object} cascadingList - a reference to the cascadingList object in the Asset Browser
	 */
	AssetBrowserSearchIntegration.prototype._fromAssetBrowserToSearch = function(cascadingList) {
		// Get the index of the list (in the Attribute Category)
		var listIdx = cascadingList.listIdx;
		
		// Get the Attribute Category list (useful for labels and such)
		var listObj = this._assetBrowserModel.lists[listIdx];
		
		// From the model, extract the selection which applies to this list
		var prefix = ""+listIdx+"_";
		var selection = _.filter(this._assetBrowserModel.getSelection(), _.bind(function(id) {
			// The ids are in the format: 1_12_14_6 => where each number represents some selected index at it's level
			// The first index = the list
			return id.indexOf(prefix)===0;
		}, this));
		
		// Search depth - how much attributes to search on
		var columnSelectionDepth = 0;
		_.each(selection, function(id) {
			// Count the number of levels down this selection is by counting the underscores
			var depth = id.split("_").length-1;
			if(depth>columnSelectionDepth) {
				columnSelectionDepth = depth;
			}
		});
		
		
		//
		// Convert "tree selection" => "attribute criteria"
		//

		// Note when a parent is selected => all the children selected also! (not shown, but that is the interpretation)
		// However, we only need to actually add them in search for the levels where we have some other selection... (=> columnSelectionDepth)
		// If we would not have some other selection we would be OR-ing all possible values anyway => so we can leave the criterion empty.

		
		// Note: Limitation of the Asset Browser "by design"
		// We will not search for empty values... However, this can lead to inconsistent results!
		// But, even if we could search on those, both adding the empty values or not adding them could lead to "invalid" search results.
		//
		// E.g.: You selected some item on the lowest level, but not the other ones.
		//       Some other selection might have "all" on the lowest level (so selected a parent) and add some empty value
		//       => This means you will also see items with the parent attribute of the first item, but with the attribute where you made the selection "empty"
		// E.g.: You selected some item on the lowest level, but not the other ones.
		//       Some other selection might have "all" on the lowest level (so selected a parent) and it did NOT add the empty value
		//       => This means that you will not find all the items where the parent is selected => you will not find the items where the last attribute it empty!
		//
		// So summary: 
		// The issue is actually more general... 
		// Selection cannot be represented in search if there are identical children in different parents.
		// If we really want correct selection, we should make a new criterion in the search object model specific for the Asset Browser.
		
		
		// Here we will collect all the attribute values (we will OR them at the end)
		var attrValues = [];
		
		// Initialize all of them to an empty array
		for (var col = 0; col < columnSelectionDepth; col++) {
			attrValues[col] = [];
		}
		
		// Now, loop through the selection
		_.each(selection, _.bind(function(id) {
			// Convert the id to an array of indices
			var idarr = this._assetBrowserModel.convertIdToArray(id);
			
			// The nodes of the tree of the previously processed level
			var treeNodes = [cascadingList.treeRootNode];
			
			// And loop through all the columns...
			for(var col=0;col<columnSelectionDepth;col++) {
				if(col+1<idarr.length) {
					// -> Parent attributes of the selected item OR the selected item (simply search for this value)
					attrValues[col].push(idarr[col+1]);
					
					// Prepare for the next level... Put the matching node in the tree in the treeNodes array
					treeNodes = [treeNodes[0].itemsByIndex[idarr[col+1]]];
				} else {
					// -> Child attributes of the selected item (add all)
					var childTreeNodes = [];
					// Go through all the parents, and find all the children
					_.each(treeNodes, _.bind(function(parentNode) {
						// Add all children of this node to the array of child nodes
						Array.prototype.push.apply(childTreeNodes, parentNode.items);
					}, this));
					
					// Add all the values to the search query
					_.each(childTreeNodes, _.bind(function(node) {
						// Search on this value for this column...
						attrValues[col].push(node.idx);
					}, this));
					
					// Prepare for the next level...
					treeNodes = childTreeNodes;
				}
			}
			
		}, this));
		
		
		//
		// Convert "attribute criteria" => "lucene queries"
		//
		
		// Ok, so now we have all the "id's" of the values we need to search on per attribute...
		for (var col = 0; col < listObj.columns.length; col++) {
			var query = null;
			if(col < columnSelectionDepth) {
				// => First, lookup the actual strings and remove duplicates
				var values = _.map(_.unique(attrValues[col]), _.bind(function(idx) {
					return this._assetBrowserModel.getLabelImpl(listObj, col+1, idx);
				}, this));
				
				// ... and make the query
				query = utils.arrayOfValuesToLuceneQuery(values);
			} else {
				// These attributes need to be cleared (search for all)
				query = "";
			}
			
			// find the attribute id
			var colObj = listObj.columns[col];
			if (!colObj) continue;
			var attObj = colObj.attObj;
			if (!attObj) continue;
			
			// Set the attribute in search...
			var criterion = this._criteriaMap[attObj.id];
			var val = new ValueData();
			val.setValue("value", query);
			criterion.setValue(val, false);
		}

		// Commit the changes in the model
		this._searchModel.getCriteriaList().commitChanges();
	};

	return AssetBrowserSearchIntegration;
});