define([
	"jquery",
	"underscore",
	"wcr/webcenter_xml",
	"./AssetBrowserSelectionDisplay"
], function (
	$,
	_,
	WCXML,
	AssetBrowserSelectionDisplay
) {
	"use strict";

	/**
	 * @param {DOMElement} div - the DOM element to put this component in
	 * @param {AssetBrowserModel} model - the model for this Asset Browser
	 */
	var AssetBrowserTreePane = function (div, model) {
		this._div = $(div);
		this._m = model;

		// Because of old code... Need some way of accessing this object using a global.
		if (!window["assetbrowsertree"]) {
			window["assetbrowsertree"] = {};
		}
		window["assetbrowsertree"][this._m.id] = this;
		this._functionPrefix = "window.assetbrowsertree['" + this._m.id + "']";
		// End of old code workaround

		// Make the selection component (only once, in constructor)
		this._selectioncomponentdiv = $('<div id="' + this._m.id + 'CurrentSelectionDiv" class="searchassetbrowserCurrentSelectionDiv" style="width:' + (this._m.panelWidth + 20) + ';overflow-x:auto">');
		this._selectiondisplay = new AssetBrowserSelectionDisplay(this._selectioncomponentdiv.get(0), this._m);

		// Initialize the Asset Browser Left Tree Panel
		this._initialize();
		
		//
		// Mirror the view with the current and future states of the model =>
		//

		// Listen to the object model for the item that needs to have the focus
		this._m.on("focuschange", _.bind(this._changeFocusHighlight, this));
		// And simulate the events we might have missed (before this object was constructed)
		this._changeFocusHighlight(this._m.getFocusTreeId(), "allLists");

		// When the selection changes in the model, update the checkboxes
		this._m.on("selectionchange", _.bind(this._changeSelectionView, this));
		// And simulate the events we might have missed (before this object was constructed)
		_.each(this._m.getSelection(), function(id) {
			this._changeSelectionView(id, true);
		}, this);

		// When the expand/collapse changes in the model, update the expand/collapse in the view
		this._m.on("expandedchange", _.bind(this._changeExpandView, this));
		// And simulate the events we might have missed (before this object was constructed)
		_.each(this._m.getExpandedIds(), function(id) {
			this._changeExpandView(id, true);
		}, this);
		
		// When the visibility changes in the model, update the visibility of the view
		this._m.on("visibility", _.bind(this._updateVisibility, this));
		// And simulate the events we might have missed (before this object was constructed)
		this._updateVisibility();
		
	};


	// CONSTANTS
	// icon for tree view in search results
	var cCollapseIcon = "icon icon-WCR_Right-S";
	// icon for tree view in search results
	var cExpandIcon = "icon icon-WCR_Down-S";


	/**
	 * Initialize the left tree pane => render
	 */
	AssetBrowserTreePane.prototype._initialize = function () {
		var htmlcode = '';
		htmlcode += '<table id="' + this._m.id + 'mainCascadingTreesTable" width="100%">';

		// Current selection
		htmlcode += '<tr><td colspan="2" id="' + this._m.id + 'CurrentSelectionTopLeft">';
		// -> Here we will put the current selection table component
		htmlcode += '</td></tr>';

		// grey line above
		htmlcode += '<tr><td colspan="2" class="grey2_background" style="padding: 0pt;font-size:0;"></td></tr>';

		for (var list = 0; list < this._m.cascadingLists.length; list++) {
			var listIdx = this._m.cascadingLists[list].listIdx;
			var listObj = this._m.lists[listIdx];
			var htmlcode2 = '';
			var label = this._m.cascadingLists[list].label;

			if (listObj) {
				htmlcode2 += '<tr>';
				var clickMethod = this._functionPrefix + '._toggleExpand(\'' + listIdx + '\');';
				htmlcode2 += '<td width="20px"><i class="' + cCollapseIcon + '" style="cursor:pointer;" onclick="' + clickMethod + '" id="' + this._m.id + 'Img_' + listIdx + '"/></td>'; // triangle icon
				htmlcode2 += '<td id="' + this._m.id + 'td_' + listIdx + '">';
				htmlcode2 += '<span style="cursor:default; onclick="' + clickMethod + 'return false;"><nobr>' + WCXML.xmlencode(label) + '</nobr></span>';
				htmlcode2 += '</td>';
				htmlcode2 += '</tr>';

				htmlcode2 += '<tr style="display:none" id="' + this._m.id + 'List_' + listIdx + '"><td/>';
				htmlcode2 += '<td><div id="' + this._m.id + 'DivList_' + listIdx + '" style="';
				if (this._m.cascadingLists[list].height) {
					var h = this._m.cascadingLists[list].height;
					htmlcode2 += 'max-height: ' + h + 'px; height: expression(this.scrollHeight > ' + (h - 1) + '? \'' + h + 'px\' : \'auto\' );overflow-y:auto;';
				}

				htmlcode2 += 'max-width:' + this._m.panelWidth + 'px; min-width:' + this._m.panelWidth + 'px; width:expression(this.scrollWidth > ' + (this._m.panelWidth - 1) + '? \'' + this._m.panelWidth + 'px\' : \'auto\' );overflow-x:auto; ">';
				htmlcode2 += this._generateFullTree(this._m.cascadingLists[list]);
				htmlcode2 += '</div></td></tr>';

				// grey line below
				htmlcode2 += '<tr><td colspan="2" class="grey2_background" style="padding: 0pt;font-size:0;"></td></tr>';
			}
			htmlcode += htmlcode2;
		}

		htmlcode += '</table>';

		// Remove the selection component first
		this._selectioncomponentdiv.detach();

		// Put the new content of the left panel
		this._div.html(htmlcode);

		// Put the selection component in the right place again
		$("#" + this._m.id + "CurrentSelectionTopLeft").append(this._selectioncomponentdiv);
	};

	/**
	 * Generate the HTML for the tree
	 */
	AssetBrowserTreePane.prototype._generateFullTree = function (cascadingList) {
		var listIdx = cascadingList.listIdx;
		var listObj = this._m.lists[listIdx];

		return '<table>' + this._generateTree(listObj, cascadingList.treeRootNode, "" + listIdx, 0) + "</table>";
	};

	/**
	 * Generate the HTML for one level of the tree (recursive)
	 * 
	 * @param {Object} listObj - the Attribute Category list object (can be used for more efficient access to the labels)
	 * @param {Object} parent - the parent "object" in the tree. => We build an in-memory representation of the tree already before!
	 * @param {String} prefix - the prefix at this level. Every node in the tree has an id. => Prefix = the id of the parent = prefix of the id of the child
	 * @param {int} depth - the depth we are in the tree (starts at -1 for the root)
	 */
	AssetBrowserTreePane.prototype._generateTree = function (listObj, parent, prefix, depth) {
		var htmlcode = '';

		// Loop through all the items in this element
		_.each(parent.items, function (item) {
			// Make my identifier (1_12_3)
			var myId = prefix + "_" + item.idx;
			var myDepth = depth + 1;

			// Can we expand?
			var canExpand = item.items.length > 0;

			// click methods
			var expandRowAction = this._functionPrefix + '._toggleExpand(\'' + myId + '\');';
			var selectRowAction = this._functionPrefix + '._toggleSelect(\'' + myId + '\');';
			var defaultClickAction = canExpand ? expandRowAction : selectRowAction;

			// start building the html for this row
			htmlcode += '<tr>';

			// Collapse icon
			if (canExpand) {
				htmlcode += '<td><i class="' + cCollapseIcon + '" style="cursor:pointer;" id="' + this._m.id + 'Img_' + myId + '" onclick="' + expandRowAction + '"/></td>'; // triangle icon
			} else {
				htmlcode += '<td></td>';
			}

			// Checkbox
			htmlcode += '<td>'+this._m.utilCheckboxMakeHtml(this._m.id + 'Chkbox_' + myId, selectRowAction)+'</td>';

			// Label
			var label = WCXML.xmlencode(this._m.getLabelImpl(listObj, myDepth, item.idx));
			htmlcode += '<td id="' + this._m.id + 'td_' + myId + '">';
			htmlcode += '<span style="cursor:default;" class="current-selection-td-details" onclick="' + defaultClickAction + '"><nobr>' + label + '</nobr></span>';
			htmlcode += '</td>';

			htmlcode += '</tr>';

			// Children ?
			if (canExpand) {
				htmlcode += '<tr style="display:none" id="' + this._m.id + 'List_' + myId + '"><td/><td/><td><table>';
				htmlcode += this._generateTree(listObj, item, myId, myDepth);
				htmlcode += '</table></td></tr>';
			}
		}, this);

		return htmlcode;
	};

	//
	// ---
	// Handle Showing what is currently expanded
	// ---
	//

	/**
	 * -- Listener on the view --
	 * 
	 * Function that is called when you want to toggle the expansion for a certain row
	 * 
	 * @param id - the id representing an index in the tree (see also AssetBrowserModel)
	 */
	AssetBrowserTreePane.prototype._toggleExpand = function (id) {
		// Toggle the expansion of a row in the model (events will update the views)
		this._m.toggleExpansion(id);

		// Focus this element!
		this._m.setFocusTreeId(id);
	};

	/**
	 * -- Listener on model --
	 * 
	 * Called when the "expansion" changes in the model
	 */
	AssetBrowserTreePane.prototype._changeExpandView = function (id, expanded) {
		if (id != 'allLists') {
			var elem = $("#" + this._m.id + 'List_' + id);
			var imgElem = $("#" + this._m.id + 'Img_' + id);

			if (expanded) {
				elem.css("display", "");
				imgElem.attr("class", cExpandIcon);
			} else {
				elem.css("display", "none");
				imgElem.attr("class", cCollapseIcon);
			}
		}
	};

	//
	// ---
	// Handle Showing the current state of the selection
	// ---
	//

	/**
	 * -- Listener on the view --
	 * 
	 * Function that is called when you want to toggle the selection for a certain row
	 * 
	 * @param id - the id representing an index in the tree (see also AssetBrowserModel)
	 */
	AssetBrowserTreePane.prototype._toggleSelect = function (id) {
		// Toggle the selection in the model (events will update the views)
		this._m.toggleSelection(id);
	};

	/**
	 * -- Listener on model --
	 * 
	 * Called when the selection changes in the model
	 */
	AssetBrowserTreePane.prototype._changeSelectionView = function (id, selected) {
		// 1) Go through all parents
		//    => Show as some selected or not selected
		var idarr = this._m.convertIdToArray(id);
		var parentid = idarr[0];
		var parentsHaveSelectedChildren = selected;
		for(var i=1;i<idarr.length-1;i++) {
			parentid+="_"+idarr[i];
			// Note: only execute hasSelectedChildren if we are not sure whether we have selected children
			parentsHaveSelectedChildren = parentsHaveSelectedChildren || this._m.hasSelectedChildren(parentid);
			var chkbxElem = $("#" + this._m.id + "Chkbox_" + parentid);
			this._m.utilCheckboxSetState(chkbxElem, false, parentsHaveSelectedChildren);
		}
		
		// 2) Find myself and show as selected
		this._m.utilCheckboxSetState($("#" + this._m.id + "Chkbox_" + id), selected, false);
		
		// 3) Find all children and show them as selected
		this._m.utilCheckboxSetState($("[id^=" + this._m.id + "Chkbox_" + id+"_]"), selected, false);
	};



	//
	// ---
	//  Handle Highlighting "focussed" elements
	// ---
	//

	/**
	 * -- Listener on model --
	 * 
	 * When the focus changes in the model, this method will update the shown focus in the left tree
	 */
	AssetBrowserTreePane.prototype._changeFocusHighlight = function (newFocusId, oldFocusId) {
		// Reset background color of element no longer focussed
		if (oldFocusId && oldFocusId != 'allLists') {
			this._setFocusHighlight(oldFocusId, '');
		}

		// Only show the "focus" if we have an image panel
		if (this._m.isImagePanelVisible()) {
			this._setFocusHighlight(newFocusId, '#d0d0d0');
		}
	};

	/**
	 * Find an element in the tree by id and set the highlight
	 * 
	 * Change Background color of selected element
	 * elemId looks like List_0_1
	 * We need to find the element td0_0_1
	 */
	AssetBrowserTreePane.prototype._setFocusHighlight = function (focusId, color) {
		$("#" + this._m.id + 'td_' + focusId).css("background", color);
	};
	
	//
	// ---
	// Handle Visibility Changes
	// ---
	//
	
	/**
	 * -- Listener on model --
	 * 
	 * When the visibility changes in the model, hide or display this component
	 */
	AssetBrowserTreePane.prototype._updateVisibility = function() {
		if(this._m.isTreePanelVisible()) {
			this._div.css("display","");
			
			// Might be that the image panel is now shown... Update the highlight
			this._changeFocusHighlight(this._m.getFocusTreeId(), this._m.getFocusTreeId());
		} else {
			this._div.css("display","none");
		}
	};

	// Return constructor
	return AssetBrowserTreePane;
});