define([
	"underscore",
	"jquery",
	"require",
	"utils/idUtils",
	"utils/Observable",
	"wcr/ObjectArrayCloneUtil"
], function (
	_,
	$,
	require,
	// Some util to generate new ids (used for Calendar View)
	idUtils,
	Observable,
	ObjectHandler
) {
	"use strict";

	//
	// NOTE: uses global "gCurrentUser"
	//

	/**
	 * Constructs a wrapper around the attribute category.
	 * 
	 * @param {jQuery} div - the div containing the full search. (needed because the attribute category needs to be put in some hidden div somewhere)
	 * @param categoryid - the id of the category or task type
	 * @param isTaskType - whether it is a task type
	 */
	var SearchCategoryInfo = function (div, categoryid, isTaskType) {
		// Checking inputs
		if (!(div && div instanceof $ &&
				categoryid && _.isString(categoryid) &&
				_.isBoolean(isTaskType))) {
			throw new Error("Illegal arguments passed to SearchCategoryInfo");
		}
		
		
		this._categoryid = categoryid;
		this._isTaskType = isTaskType;

		// The Attribute Category needs some unique id for it to work. (before 16.1 => some combination with hpSectionId)
		this._id = idUtils.newClientSideId("search_attr_cat");
		this._prefix = "searchattrcat_" + this._id + "_";

		// Put the id on the element we want to put the Attribute Category in
		div.attr("id", this.getAttributeCategoryDomElementId());

		// Debugging information
		this._debug = {
			log: ""
		};

		// Loading information
		this._isLoading = false;
		this._doneLoading = false;

		// List loading information
		this._listidcounter = 0;
		this._listsLoadingAll = {}; // Map every list we need to load to some unique id
		this._listsLoadingDone = [];

		// The attribute category
		this._category = null;

		// This object is observable!
		Observable.mixin(this, "searchcategoryinfo");
	};

	/**
	 * Get the id of the attribute category info object.
	 * This id is automatically generated and unique on the page.
	 */
	SearchCategoryInfo.prototype.getId = function () {
		return this._id;
	};

	/**
	 * Get the id of the attribute category "div" where the category is attached to.
	 * @returns {String}
	 */
	SearchCategoryInfo.prototype.getAttributeCategoryDomElementId = function () {
		return this._prefix + "ACatPanel";
	};

	/**
	 * Get the attribute category.
	 * 
	 * (Only call this AFTER loading is done)
	 * 
	 * @returns
	 */
	SearchCategoryInfo.prototype.getAttributeCategory = function () {
		if (this._doneLoading) {
			return this._category;
		} else {
			throw new Error("Tried to access Attribute Category before done loading.");
		}
	};

	/**
	 * Get debugging information related to this category.
	 */
	SearchCategoryInfo.prototype.getDebugInfo = function () {
		return this._debug;
	};

	/**
	 * Log a message related to things working on this category
	 */
	SearchCategoryInfo.prototype.log = function (txt) {
		this._debug.log += "\n" + txt;
	};

	/**
	 * Load a attribute category OR check whether the attribute category is already loaded.
	 * 
	 * Callback will always be called.
	 * 
	 * @param callback - the callback to execute if/when the attribute category is loaded (might be immediately)
	 */
	SearchCategoryInfo.prototype.load = function (callback) {
		// Register a callback to be called when loading is done.
		if (this._doneLoading) {
			callback();
		} else {
			this.once("loadingbasedone", callback);
		}

		// Do not load when already called
		if (this._isLoading || this._doneLoading) {
			return;
		}
		this._isLoading = true;

		//
		// Do the actual loading.
		//
		var args = {};
		args[this._isTaskType ? "tasktypeid" : "categoryid"] = this._categoryid;

		var url = 'GetAttributeCategoryXML.jsp?' + $.param(args, true);

		var currentUser = ObjectHandler.getCloneOfObject(window.gCurrentUser);
		currentUser.canEditAttributes = false;

		var aCatOptions = {};
		aCatOptions.currentUser = currentUser;
		aCatOptions.ACatObj = this.getAttributeCategoryDomElementId();
		aCatOptions.isTemplate = false;
		aCatOptions.ACatPanel = null; // no display
		aCatOptions.readURL = url;
		aCatOptions.type = ""; // not known here
        aCatOptions.disableSynchronizingAttributes = true;
		aCatOptions.prefix = this._prefix;

		// Load the attribute category asynchronously.
		require(["attributecategory/AttributeCategory"], _.bind(function (AttributeCategory) {
			// Create the attribute category.
			var aCatObj = new AttributeCategory(aCatOptions);
			var time0 = new Date();
			// attribute categories load asynchronous. So you need to wait and potentially even retry
			// find attributes (for starters visible?) which need to get filled.
			setTimeout(_.bind(function checkIfDone() {
				var loadingReady = true;
				if (!aCatObj.loadingDone) {
					loadingReady = false;
				}
				if (!loadingReady) {
					// Attribute Category is not done loading...
					setTimeout(_.bind(checkIfDone, this), 100);
					return;
				} else {
					var listArr = aCatObj.lists;
					if (listArr) {
						//check whether lists are loaded
						for (var list = 0; list < listArr.length; list++) {
							var listObj = listArr[list];
							if (listObj) {
								if (listObj.loadingType != 'OnTheFly' && (!listObj.combinations || listObj.combinations.length === 0)) {
									loadingReady = false;
								}
							}
						}
						if (!loadingReady) {
							// Lists are not done loading...
							setTimeout(_.bind(checkIfDone, this), 100);
							return;
						}
					}

					// Done loading!
					this.log('loading Ready after ' + (new Date() - time0) + 'ms' + new Date());

					// Change the state
					this._doneLoading = true;
					this._isLoading = false;
					this._category = aCatObj;
					// Call all the listeners...
					this._emit("loadingbasedone");
				}
			}, this), 100);
		}, this));
	};

	/**
	 * Load a list OR check whether the list is already loaded.
	 * 
	 * Callback will always be called.
	 * 
	 * @param list - small objects containing the index of the list (in the attribute category) to load and the depth {listIdx:2, depth:5}
	 * @param callback - the callback to execute if/when the list is loaded (might be immediately)
	 */
	SearchCategoryInfo.prototype.loadList = function (list, callback) {
		// First make sure the Attribute Category is loaded
		this.load(_.bind(function () {
			// Get the actual number of columns of the list we need to load
			var listObj = this._category.lists[list.listIdx];

			if (listObj.loadingType != 'OnTheFly') {
				callback();
				return;
			}

			var nColumns = listObj.columns.length;
			if (list["depth"] && list.depth < nColumns) {
				nColumns = list.depth;
			}

			var listLoadId = listObj.name + "~~" + nColumns;
			
			// Check if we already loaded this.
			if(_.has(this._listsLoadingDone, listLoadId)) {
				callback();
				return;
			}

			// Check if we are already loading this.
			if (_.has(this._listsLoadingAll, listLoadId)) {
				// Already called this before
				var counter = this._listsLoadingAll[listLoadId];
				this.once("loadinglistdone." + counter, _.bind(function () {
					callback();
				}, this));
				return;
			}

			// First time this list was requested... 
			// Generate a new id
			var listCounter = this._listidcounter;
			this._listidcounter++;
			// Register it
			this._listsLoadingAll[listLoadId] = listCounter;


			// Build the URL
			var url = window.webcenterConfig.jspBaseUrl + 'GetRowsFromList.jsp?';
			var params = {
				"listType": "WCR_DB_List",
				"outputType": "JSON",
				"indexed": "1",
				"list": listObj.name,
				"maxrows": "10000", // To do: make this configurable
				"maxcolrows": "1000",
				"output": []
			};

			for (var column = 0; column < nColumns; column++) {
				var columnObj = listObj.columns[column];
				params.output.push(columnObj.columnName);
			}
			url = url + $.param(params, true);

			// Function to call when loading is done
			var loadingDone = _.bind(function () {
				this._listsLoadingDone[listLoadId] = true;
				this._emit("loadinglistdone." + listCounter);
				callback();
				loadingDone = null;
			}, this);

			// ajax call
			$.ajax({
				url: url,
				type: "GET",
				dataType: "text"
			}).done(_.bind(function (jsonContent) {
				try {
					var jsonObj = $.parseJSON(jsonContent);

					if (!jsonObj.success || jsonObj.success == "0") {
						window.alert('Cannot get values ' + jsonObj.error.code + ' : ' + jsonObj.error.message);
					} else {
						for (var column2 = 0; column2 < nColumns; column2++) {
							var columnObj2 = listObj.columns[column2];
							columnObj2.values = jsonObj.columns[column2].values;
						}
						// Because we might not load all the columns => make sure to not use the "combinations" field
						listObj.assettreecombinations = jsonObj.combinations;
					}

					loadingDone();
				} catch (err) {
					window.alert("Error processing return of ajax call to GetRowsFromList.jsp in SearchCategoryInfo.loadList: " + err);
					loadingDone();
				}
			}, this)).fail(_.bind(function (errObj) {
				window.alert("Ajax call unsuccessful. Status: " + errObj.status + "\n" + errObj.responseText);
			}, this));
		}, this));
	};

	/**
	 * Load the attribute category and also some lists OR check whether the attribute category AND all the lists are already loaded.
	 * 
	 * Callback will always be called.
	 * 
	 * @param listsToLoad - array of small objects containing the index of the list (in the attribute category) to load and the depth {listIdx:2, depth:5}
	 * @param callback - the callback to execute if/when all the lists are loaded (might be immediately)
	 */
	SearchCategoryInfo.prototype.loadWithLists = function (listsToLoad, callback) {
		var needToLoad = listsToLoad.length;

		_.each(listsToLoad, function (list) {
			this.loadList(list, _.bind(function () {
				// Done loading this list
				needToLoad--;
				if (needToLoad === 0) {
					// When all loaded, callback
					callback();
				}
			}, this));
		}, this);
	};

	return SearchCategoryInfo;
});