/**
 * Module which controls the Cascading Dropdowns in the search control panel + Asset Browser
 * 
 * 
 */
define([
        "underscore",
        "./SearchCategoryInfo",
        "./AssetBrowser",
        "component/searchresults/implementations/parameterfields/ParameterField_cascading_dropdown",
        "component/searchresults/implementations/parameterfields/ParameterField_cascading_dropdown_xml"
    ],
    function (
        _,
        SearchCategoryInfo,
        AssetBrowser,
        CascadingDropdownParameterFieldFactory,
        CascadingDropdownXmlParameterFieldFactory
    ) {
        "use strict";
    
        /**
         * Constructor of the object controlling the cascading dropdowns in this search.
         * 
         * @param {jQuery} div - the container for the search
         * @param {SearchSetupInformation} setupInfo - the configuration of the search
         * @param {SearchModelFrontEndData} searchData - other data related to this search
         * @param {SearchParameterFieldControl} searchParameterFieldControl - Controls all ParameterField instances
         * @param {Object} DEBUG - some object to store debug information in (used for logging)
         * @param {Function} rerunAutomaticCall - rerun function (but only if no search button)
         * @param {Function} resizeCallback - function to call when the size of search might have changed
         */
        var SearchResultsAttributeCategoryControl = function (div, setupInfo, searchData, searchParameterFieldControl, DEBUG, rerunAutomaticCall, resizeCallback) {
            // Store the div containing the full search
            this._div = div;

            // Store the object model
            this._setupInfo = setupInfo;

            // Store the other information
            this._searchData = searchData;

            this._searchParameterFieldControl = searchParameterFieldControl;

            // Logger (used for writing debug messages to)
            this.DEBUG = DEBUG;

            // Rerun method of the search we are in.
            // This method needs to be called to rerun the search for this section
            this._rerunSearchResultsAutomaticCall = rerunAutomaticCall;
            
            // Resize the search we are in
            this._resizeCallback = resizeCallback;

            // The asset browsers in this search
            this._assetBrowser = null;

            this._cascadingDropdownsLoaded = false;

            this._cascadingDropdownsLoadedCallbacks = [];
        };

        /**
         * Load and initialize all Attribute Category functionality
         */
        SearchResultsAttributeCategoryControl.prototype.load = function () {
            // Do we need to initialize?
            
            // Constants
            var ATTR_PREFIX_NORMAL = "db_attr_";
            var ATTR_PREFIX_EXTENDED = "db_attr_ext_";

            // Are there cascading attributes in the design panel?
            var needscascadingattrs = this.hasdesignpanelitems(ATTR_PREFIX_NORMAL);
            var needsextcascadingattrs = this.hasdesignpanelitems(ATTR_PREFIX_EXTENDED);

            // Is there an asset browser?
            var cascadingtreessetup = this._setupInfo.getLayoutOptions().getOption("cascading_attribute_trees");
            var hasassetbrowser = ((cascadingtreessetup ? cascadingtreessetup.getSubList("treeElem") : []) || []).length > 0;

            // If we've come from a project or document search we may have an attribute category
            // that will be used for configuring cascading dropdowns (for attributes) and/or the asset tree
            // Or
            // If we've come from a task search we may have a task type id that will be used
            // for configuring cascading dropdowns (for specifications)
            //
            // In addition, if we've come from a Document or Task search we may have an extended (project)
            // attribute category that will be used for configuring cascading dropdowns (for project attributes)
            
            if (needscascadingattrs || hasassetbrowser) {
                // Load the normal attribute category / task type
                var categoryid = this._setupInfo.getLayoutOptions().getOption("category_id");
                var istasktype = this._searchData.isTaskSearch();
                this._initCategory(categoryid, istasktype, false, needscascadingattrs, hasassetbrowser, ATTR_PREFIX_NORMAL);
            }

            if (needsextcascadingattrs) {
                // Load the extend attribute category
                var extcategoryid = this._setupInfo.getLayoutOptions().getOption("extended_category_id");
                var extistasktype = false;
                this._initCategory(extcategoryid, extistasktype, true, true, false, ATTR_PREFIX_EXTENDED);
            }
        };

        /**
         * Called when the user clicked "clear this form"
         */
        SearchResultsAttributeCategoryControl.prototype.clearForm = function () {
            // Cascading fields => are already cleared by the SearchParameterFieldControl

            // Asset browser => clear here
            if (this._assetBrowser) {
                this._assetBrowser.clear();
            }
        };

        /**
         * Initialize all the functionality for one attribute category
         * 
         * @param {string} categoryid - the attribute category id
         * @param {boolean} isTaskType - whether this attribute category is a task type
         * @param {boolean} isExtended - whether this is an attribute category linked to extended criteria
         * @param {boolean} doCascading - whether we should enable cascading attributes
         * @param {boolean} doAssetBrowser - whether to make an asset browser
         * @param {string} attributePrefix - the prefix of the attribute field element "db_attr" or "db_attr_ext"
         */
        SearchResultsAttributeCategoryControl.prototype._initCategory = function (categoryid, isTaskType, isExtended, doCascading, doAssetBrowser, attributePrefix) {
            if (!categoryid) {
                return; // No Attribute Category configured
            }

            // We need some hidden cell to put the Attribute Category in...
            var categoryhiddendiv = $("<td>").css({
                "padding": "0px"
            });
            $(".attributecategorycells", this._div).append($("<tr>").append(categoryhiddendiv));

            // Make a new SearchCategoryInfo object
            var categoryinfo = new SearchCategoryInfo(categoryhiddendiv, categoryid, isTaskType);

            // Store some information in the global debug object for debugging purposes.
            this.DEBUG[categoryinfo.getId()] = categoryinfo.getDebugInfo();


            //
            // Initialize the Asset Browser
            //
            if (doAssetBrowser) {
                // Make the Asset Browser object
                var assetBrowser = new AssetBrowser(categoryinfo, this._setupInfo, this._rerunSearchResultsAutomaticCall);
                assetBrowser.whenReady(_.bind(function () {
                    // Asset Browser is ready
                    if (assetBrowser.isCorrectlyConfigured()) {
                        // This is a working asset browser
                        // Put it in the page

                        // Where to put this dropdown?
                        var dropdownarea;
                        var headerDropdownPosition = $("#headerAssetViewDropdownContainer");
                        if (headerDropdownPosition.length > 0) {
                            dropdownarea = headerDropdownPosition;
                        } else {
                            dropdownarea = $(".assetViewDropdownContainer", this._div);
                        }

                        // Put the dropdown there
                        assetBrowser.makeVisibilityDropdown(dropdownarea.get(0));

                        // Where to put the images?
                        var imagearea = $(".searchassetbrowserImageArea", this._div);

                        // Put the image panel there
                        assetBrowser.makeImagePane(imagearea.get(0));

                        // Where to put the left tree?
                        var leftarea = $(".assetViewTreeContainer", this._div);

                        // Put the left tree there
                        assetBrowser.makeTreePane(leftarea.get(0));

                        // Put it in "this", so it is available for "clear this form"
                        this._assetBrowser = assetBrowser;
                        
                        // Asset browser might have loaded with some delay => tell search that the size might have changed.
                        this._resizeCallback();
                    }
                }, this));
            }

            //
            // Initialize the Cascading Attributes
            //
            this._initCascadingDropdowns(categoryinfo, attributePrefix);
        };

		/**
		 * Get the Asset Browser ImagePanel Div
		 * @returns {JQueryObj} 
		 */
		SearchResultsAttributeCategoryControl.prototype.getImagepanel = function () {
			return $(".searchassetbrowserImageArea", this._div);
		};
	
	
		/**
		 * Get the Asset Browser Side Panel Div
		 * @returns {JQueryObj} 
		 */
		SearchResultsAttributeCategoryControl.prototype.getSidePanel = function () {
			return $(".assetViewTreeContainer", this._div);
		};

        //
        // UTILITY functions
        //
        /**
         * Utility function to check whether the design panel contains a fieldid with a certain prefix.
         */
        SearchResultsAttributeCategoryControl.prototype.hasdesignpanelitems = function (prefix) {
            var found = false;
            this._setupInfo.getDesignPanelItems().each(function (designpanelitem) {
                found = found || (designpanelitem.getFieldId().lastIndexOf(prefix, 0) === 0);
            });
            return found;
        };

        /**
         * Initialize all cascading dropdowns on search page
         * @private
         * @param {SearchCategoryInfo} categoryinfo    contains the attribute category
         * @param {string}             attributePrefix prefix of the attribute field element "db_attr" or "db_attr_ext"
         */
        SearchResultsAttributeCategoryControl.prototype._initCascadingDropdowns = function (categoryinfo, attributePrefix) {
            categoryinfo.load(function () {
                // Get the Attribute Category object
                var aCatObj = categoryinfo.getAttributeCategory();

                if (aCatObj.lists) {
                    // we have lists available
                    _.each(aCatObj.cells, function (rowArr, row) {
                        _.each(rowArr, function (cellObj, col) {
                            if (cellObj) {
                                // Replace each cell with a proper ParameterField implementation
                                var attObj = cellObj.attObj;
                                if (attObj && attObj.listObj && attObj.columnObj) {
                                    var fieldId = attributePrefix + attObj.id;
                                    var factory;
                                    if (attObj.listObj.type === "database") {
                                    	// "database" cascading dropdowns
                                        factory = new CascadingDropdownParameterFieldFactory(this, aCatObj, attObj, row, col);
                                    } else {
                                    	// "Excel" cascading dropdowns
                                        factory = new CascadingDropdownXmlParameterFieldFactory(aCatObj, cellObj, row, col);
                                    }
                                    this._searchParameterFieldControl.replaceImplementations(fieldId, factory);
                                }
                            }
                        }.bind(this));
                    }.bind(this));
                    this._fireCascadingDropdownsLoaded();
                }
            }.bind(this));
        };

        /**
         * Register a callback that will be called when this object is done loading cascading dropdowns on search result page
         * @param {Function} callback
         */
        SearchResultsAttributeCategoryControl.prototype.onCascadingDropdownsLoaded = function (callback) {
            if (this._cascadingDropdownsLoaded) {
                callback();
            } else {
                this._cascadingDropdownsLoadedCallbacks.push(callback);
            }
        };

        /**
         * Fires the callbacks when cascading dropdowns are fully loaded
         * @private
         */
        SearchResultsAttributeCategoryControl.prototype._fireCascadingDropdownsLoaded = function () {
            this._cascadingDropdownsLoaded = true;
            _.each(this._cascadingDropdownsLoadedCallbacks, function (callback) {
                callback();
            });
        };

        return SearchResultsAttributeCategoryControl;
    });
