/**
 * JS functionality for Document Search results page context handling of Cart manipulation
 * Author: JAMI, HADW, JEPE
 */
define(["jquery", "underscore", "component/actionmenu/HeaderActionMenu", "component/table/Table", "../../actions/SearchResultsActionsOnDocuments", "utils/webi18n!SearchInputs"], function($, _, HeaderActionMenu, Table, SearchResultsActionsOnDocuments, i18n){

	/**
	 * @constructor
	 * @param {jQuery} div - the div containing the complete search results
	 */
	var SearchDocCart = function (div) {
		// The div containing the complete search results
		this._div = div;
		// The div in which the cart should be displayed. (if any)
		this._cartdiv = $(".cartDisplay", this._div);
		
		// Information about create project from documents in cart
		var createprojectsettingsstring = this._cartdiv.attr("data-createprojectsettings");
		this._createprojectlinkenabled = !!createprojectsettingsstring;
		if(this._createprojectlinkenabled) {
			this._createprojectsettings = $.parseJSON(createprojectsettingsstring);
		}
	};
	
	/**
	 * Called from SearchResultsControl to initialize the cart
	 */
	SearchDocCart.prototype.initialize = function(){
		// Show the content of the cart (if enabled)
		this._showCartContent();
		
		// When something changes the cart on this page... (can be outside of search!)
		// => update the content of the cart in search results...
		$(document.body).on("cartupdated", _.bind(function(){
			this._showCartContent();
		}, this));
	};
	
	/**
	 * Get the DOM element of the cart
	 * @returns {JQueryObj} 
	 */
	SearchDocCart.prototype.getElement= function(){
		return this._cartdiv;
	};
	
	/**
	 * Render the Cart area HTML content within a Document Search Results page
	 */
	SearchDocCart.prototype._showCartContent = function () {
		try {
			//only proceed if the search results page supports showing Cart content
			if (this._cartdiv.length!=1) return;
			
			// Show "Loading" message
			this._cartdiv.empty();
			var loadingMsgElem = $('<span>').appendTo(this._cartdiv);
			$('<label>').addClass('form-label').css({'clear' : 'right'}).text(i18n.t("SEARCH_CART_LOADING")).appendTo(loadingMsgElem);
			
			$.ajax({
				url: "cartmgr.jsp?XSLPage=cartmgr_XML&ajax=1",
				method: "GET",
				timeout: 30000,	// 30 seconds
				success: _.bind(function(xml) {
					try {
						var pageElem = WCXML.GetFirstFirstLevelChild(xml,'webcenter_page');						

						var docVersionListElem = WCXML.GetFirstFirstLevelChild(pageElem,'doc_version_list');	
						var docVersionList = WCXML.GetFirstLevelChilds(docVersionListElem,"doc_version");
						
						var cartElem = WCXML.GetFirstFirstLevelChild(pageElem,'cart');
						var cartItemListElem = WCXML.GetFirstFirstLevelChild(cartElem,'cart_item_list');
						var cartItemList = WCXML.GetFirstLevelChilds(cartItemListElem,'cart_item');
						
						//write the HTML contents for the cart section
						var allcontent = $('<div>', {"class":"searchresultscart"});
						var content = $('<div>', {"class":"content"});
						allcontent.append(content);
						
						if (docVersionList && docVersionList.length>0 && cartItemList && cartItemList.length>0) {
							//cart contains some elements
							
							// cart title
							var title = $('<h3>', {"text":i18n.t("CUR_IN_YOUR_CART")}).appendTo(content);
							
							// actions bar
							var actions = $('<div>', {"class":"action-menu-component header-action-buttons"}).appendTo(title);
							
							//add "Create Project from Cart" action link if needed
							if(this._createprojectlinkenabled){
								$('<a>',{
									"class":"action",
									"data-icon":"icon icon-WCR_Actions-CreateProject12",
								    href: '#',
								    click: _.bind(this.createProjectFromCart, this, true)
								}).text(i18n.t("CREATE_PROJ_FROM_CART")).appendTo(actions);
							}
							
							//add "Remove selected Documents from Cart" action link
							$('<a>',{
								"class":"action",
								"data-require-selection-min":"1",
								"data-icon":"icon icon-WCR_Actions-Cart-Remove12",
							    href: '#',
							    click: _.bind(this.removeSelectedFromCart, this, true)
							}).text(i18n.t("REMFROMCART")).appendTo(actions);
							
							//add "Clear Cart" action link
							$('<a>',{
								"class":"action",
								"data-icon":"icon icon-WCR_Actions-Delete12",
							    href: '#',
							    click: _.bind(this.clearCart, this, true)
							}).text(i18n.t("CLEARCART")).appendTo(actions);

							//---------------------------------------------------
							//create new table row for showing the Cart Documents
							var cartItemsTable = $('<div class="wcr-table">').appendTo(content);
							var cartItemsTableSection = $('<div class="wcr-table-section no-overflow">').appendTo(cartItemsTable);
							var cartItemsContainer = $('<div class="cartoverflow clearfix">').appendTo(cartItemsTableSection);
							
							for (var docIdx=0; docIdx<docVersionList.length; docIdx++) {
								var docVerElem = docVersionList[docIdx];
								if (docVerElem) {
									var docName = '';
									var docVersionId = WCXML.GetElementAttribute(docVerElem,'id');
									
									// Retrieve the cart item id for each doc version in the cart
									var cartItemId = null;
									for (var ci=0; ci<cartItemList.length; ci++) {
									  var cartItemElem = cartItemList[ci];	
								      var cartItemDocVersionId = WCXML.GetElementText(cartItemElem,'doc_version_id',false, true);
									  if (docVersionId === cartItemDocVersionId) {
									      cartItemId = WCXML.GetElementAttribute(cartItemElem,'id');
										  break;
									  }
									}
									
									var thumbnail_url =  WCXML.GetElementText(docVerElem,"thumbnail_url");
									var docElem =  WCXML.GetFirstFirstLevelChild(docVerElem,'document');
									if (docElem) {
										docName =  WCXML.GetElementText(docElem,"name");										
									}
									
									//create the HTML data for the current Cart Document item
									var item = $("<div>", {"class":"searchcartitem no-toggle-overflow"}).appendTo(cartItemsContainer);
									
									var selectCheckbox = $('<input>').attr('type', 'checkbox').attr('id', 'cartDocChk_'+docVersionId).attr('name', 'cartDocChk').attr('cartitemid', cartItemId).val(docVersionId);
									item.append($("<div>", {"class":"itemcheckbox"}).append(selectCheckbox));
									
									var makeShowThumbnailCallback = function(thumbnail_url, docName, docVersionId){
										return function(event) {showthumb(event, thumbnail_url, WCXML.xmlencode(docName), '202', null, 'DownloadViewerThumbnail.jsp?docversionid='+docVersionId+'&amp;pix=500&amp;ajax=1', 500);};
									}
									
									var curDocumentThumbnailCellElem = $('<div>', {"class":"itemthumbnail"}).appendTo(item);
									$('<img>').attr('src', thumbnail_url).attr('height', '50')
									.mouseout(function() {delayhidethumb();})
									.mouseover(makeShowThumbnailCallback(thumbnail_url, docName, docVersionId))
									.appendTo(curDocumentThumbnailCellElem);
									
									$('<div>', {"class":"itemname", "title":docName}).text(docName).appendTo(item);															
								}
							}
							

							// Initialize components...
							_.defer(_.bind(function(){
								// Initialize the header action javascript
								var actionmenu = new HeaderActionMenu(actions, content, "input[type=checkbox][name=cartDocChk]");
								
								// Initialize the table javascript
								var table = new Table(cartItemsTable.get(0), ".searchcartitem", "input[type=checkbox][name=cartDocChk]", true);
								actionmenu.setTable(table);
							}, this));
							
						} else { // the cart is empty
							content.append($("<div>", {"class":"no-results", "text":i18n.t("CARTEMPTY")}));
						}
						
						//paste the generated HTML contents into the prepared container on the page
						this._cartdiv.empty();
						this._cartdiv.append(allcontent);
						
					} catch(err) {
						alert('showCartContent::Error when processing return XML on line' + err.lineNumber + '. Error details : ' + err.message);
					}
					
				}, this),
				error: _.bind(function(errMsg) {
					
				}, this)
			});
		
		} catch (err) {
			alert('Problem in showCartContents at line ' + err.lineNumber + ' : ' + err.message);	
		}
	};


	/**
	 * Add the selected Documents (in the Document Search results table) into the Cart
	 * 
	 * @param {Array} selectedIds - an array of document version id's
	 * @param {Array} checkboxes - an array of checkbox elements to deselect
	 */
	SearchDocCart.prototype.addSelectedToCart = function (selectedIds, checkboxes) {
		try {
			var url =  'dodocsearchactions.jsp?actionVal=AddDocumentToCart&ajax=1';
			
			if (selectedIds.length>0) {
				$.ajax({
					url: url,
					method: "POST",
					data: $.param({selectedDocs:selectedIds}, true),
					cache:false,
					timeout: 30000,	// 30 seconds
					success: _.bind(function(xml) {
						try {
							// clear checkboxes relating to documents added to the cart
							for (var docIdx=0; docIdx< checkboxes.length; docIdx++) {
								var docElem = checkboxes[docIdx];
								if (docElem && docElem.checked) {
									docElem.checked = false;
									$(docElem).trigger("change");
								}
							}
						} catch(err) {
							alert('addSelectedToCart:: Error when processing return XML on line' + err.lineNumber + '. Error details : ' + err.message);
						}
						
						// Cart has changed... Notify everyone
						//  - update the cart badge in the header of the page... 
						//  - updates the cart in search results (Note: there can be multiple searches with cart on my work)
						$(document.body).triggerHandler("cartupdated");
					}, this),
					error: _.bind(function(errMsg) {
						
					}, this)
				});
			} else {
				alert(i18n.t("NOTHING_SELECTED"));
			};
		
		} catch (err) {
			alert('addSelectedToCart:: Error on line' + err.lineNumber + '. Error details : ' + err.message);
		};
	};

	/**
	 * Remove the selected Documents (in the Cart Documents overview) from your Cart
	 */
	SearchDocCart.prototype.removeSelectedFromCart = function  () {
		
		try {
			//only proceed if the search results page supports showing Cart content
			if (this._cartdiv.length!=1) return;
			
	        // List of "cart item id's" (not the same a doc version id's)
	        var selectedIds = $("[name=cartDocChk]:checked", this._cartdiv).map(function(){return $(this).attr("cartitemid");}).get();
			
			// construct url
			var url =  'docartactions.jsp?actionVal=MultIDocRemoveFromCart&ajax=1';
						
			if (selectedIds.length>0) {
				$.ajax({
					url: url,
					method: "POST",
					data: $.param({selectdoc:selectedIds}, true),
					cache:false,
					timeout: 30000,	// 30 seconds
					success: _.bind(function(xml) {
						// Cart has changed... Notify everyone
						//  - update the cart badge in the header of the page... 
						//  - updates the cart in search results (Note: there can be multiple searches with cart on my work)
						$(document.body).triggerHandler("cartupdated");
					}, this),
					error: _.bind(function(errMsg) {
						
					}, this)
				});
			} else {
				alert(i18n.t("NOTHING_SELECTED"));
			}
		
		} catch (err) {
			alert('removeSelectedFromCart::Error on line' + err.lineNumber + '. Error details : ' + err.message);	
		}

	};
		
	/**
	 * Clears the entire Cart (including all the Projects! these are not shown atm)
	 */
	SearchDocCart.prototype.clearCart = function  () {
		
		try {
			if (this._cartdiv.length!=1) return;
			
			// construct url
			var url =  'docartactions.jsp?actionVal=ClearCart&ajax=1';

			$.ajax({
				url: url,
				method: "POST",
				cache:false,
				timeout: 30000,	// 30 seconds
				success: _.bind(function(xml) {
					// Cart has changed... Notify everyone
					//  - update the cart badge in the header of the page... 
					//  - updates the cart in search results (Note: there can be multiple searches with cart on my work)
					$(document.body).triggerHandler("cartupdated");
				}, this),
				error: _.bind(function(errMsg) {
					
				}, this)
			});	
		} catch (err) {
			alert('Problem in removeSelectedFromCart at line ' + err.lineNumber + ' : ' + err.message);	
		}

	};

	/**
	 * Creates a new Project using the current Documents in the Cart (if the cart is empty an error message is shown).
	 */
	SearchDocCart.prototype.createProjectFromCart = function () {
		try {
			if (this._cartdiv.length!=1) return;
			if (!this._createprojectlinkenabled) return;
			
			// Get all the id's
			var ids = $("[name=cartDocChk]", this._cartdiv).map(function(){return this.value;}).get();
			
			if(ids.length>0){
				// Call the static method in SearchResultsActionsOnDocuments
				SearchResultsActionsOnDocuments.createProjectFromDocuments(ids, this._createprojectsettings);
			}else{
				alert(i18n.t("CARTEMPTY"));
			}
		} catch(err) {
			alert('createProjectFromCart::Error on line' + err.lineNumber + '. Error details : ' + err.message);	
		}
	};
	
	return SearchDocCart;
});