/**
 * One of the "ExpandModes". This one shows the information in a row in the list search results.
 */
define(["underscore", "jquery"], function(_, $) {
	
	/**
	 * Constructor
	 * 
	 * @param {jQuery} row - the row which should be show/hidden when the content is shown/hidden
	 * @param {jQuery} cell - the cell in which the content should be shown
	 * @param {String} defaultTitle - the title to use if the person using the expandarea does not define a title.
	 */
	var ExpandModeListViewRow = function(row, cell, defaultTitle) {
		if(!(row && row instanceof $)) {
			throw new Error("Illegal arguments passed to ExpandModeListViewRow constructor");
		}
		if(!(cell && cell instanceof $)) {
			throw new Error("Illegal arguments passed to ExpandModeListViewRow constructor");
		}
		if(defaultTitle && !_.isString(defaultTitle)) {
			throw new Error("Illegal arguments passed to ExpandModeListViewRow constructor");
		}
		this._defaultTitle = defaultTitle || "";
		this._row = row;
		this._cell = cell;
		this._lastToggleId = null;
	};
	
	/**
	 * <implements interface> Show an expanded information area with the specified content
	 * 
	 * The generateContent function takes two arguments:
	 * - the jQuery element to put the content into
	 * - close = the function to call to close the popup
	 * 
	 * Supported options:
	 *  - title => override the title of the popup (note: title is not guaranteed to be visible)
	 *  - subtitle => if not null, adds a subtitle.
	 *  - toggleid => pass an id. If you re-open the popup with the same toggleid while it is still open, it will close the popup instead.
	 *  - closebuttontitle => override the description of the close button (note: not guaranteed to be visible)
	 *  - onclose => function to call when this expand mode is closed
	 * 
	 * @param {Function(jQuery, Function())} generateContent - function which returns the content to show in the popup. 
	 * @param {Object} options - additional options
	 */
	ExpandModeListViewRow.prototype.show = function(generateContent, options) {
		// Check the arguments
		if(!(generateContent && typeof(generateContent) === "function")) {
			throw new Error("Illegal arguments passed to ExpandModeListViewRow.show");
		}
		if(!(options && typeof(options)==="object")) {
			options = {};
		}
		
		// Make the content for the modal
		var content = $("<div>");
		
		// Cleanup the previous row...
		this._close();

		generateContent(content, _.bind(this._close, this));
		
		
		this._onclose = options["onclose"];
		
		// Check if we are not closing the view...
		if(options["toggleid"] && this._lastToggleId) {
        	return;// we are trying to reopen the same one while it was still open => simply close it
        }
        this._lastToggleId = options["toggleid"] || null;
		
		// Build the contents of the additional row... (with the close button...)
		var container = $("<div>");
		var mainrow = $("<div class='RowExpandSectionHeader'>");
		if(!options.hideshrinkbtn) {
			var closeicon = $('<i class="icon icon-WCR_Minus-square">');
			var closelink = $('<a class="RowExpandSectionCloseButton">').append(closeicon);
			if(options["closebuttontitle"]) {
				closelink.prop("title", options["closebuttontitle"]);
			}
			closelink.on("click", _.bind(this._close, this));
		
			mainrow.append(closelink);
		 
		} else {
			mainrow.addClass("no-shrink-btn");
		}
		var title = options["title"] ? options["title"] : this._defaultTitle;
		var subtitle = options["subtitle"] ? options["subtitle"] : "";
		
		if(title && subtitle) {
			mainrow.append($("<h4>").text(title + " - " + subtitle));
		} else {
			mainrow.append($("<h4>").text(title));
		}
		
		container.append(mainrow);
		container.append(content);
		
		this._cell.append(container);
		this._row.show();
	};
	
	/**
	 * Private method... Close the row "popup" view
	 */
	ExpandModeListViewRow.prototype._close = function() {
		if(this._onclose) {
			this._onclose();
			this._onclose = null;
		}
		this._cell.empty();
		this._row.hide();
	};
	
	/**
	 * sets the default title of the expand mode
	 * @param {String} title
	 */
	ExpandModeListViewRow.prototype.setDefaultTitle = function(title) {
		this._defaultTitle = title;
	};
	
	return ExpandModeListViewRow;
});