/**
 * One of the "ExpandModes". This one shows the information in a popup.
 */
define(["jquery", "component/modal/Modal", "underscore"], function($, Modal, _) {
	
	/**
	 * Constructor
	 * 
	 * @param {String} defaultTitle - the title to use if the person using the popup does not define a title.
	 */
	var ExpandModeModal = function(defaultTitle) {
		if(defaultTitle && !_.isString(defaultTitle)) {
			throw new Error("Illegal arguments passed to ExpandModeModal constructor");
		}
		this._defaultTitle = defaultTitle || "";
	};
	
	/**
	 * <implements interface> Show an expanded information area with the specified content
	 * 
	 * The generateContent function takes two arguments:
	 * - the jQuery element to put the content into
	 * - close = the function to call to close the popup
	 * 
	 * Supported options:
	 *  - title => override the title of the popup (note: title is not guaranteed to be visible)
	 *  - subtitle => if not null, adds a subtitle.
	 *  - toggleid => pass an id. If you re-open the popup with the same toggleid while it is still open, it will close the popup instead.
	 *  - closebuttontitle => override the description of the close button (note: not guaranteed to be visible)
	 *  - onclose => function to be called when the modal is closed
	 *  - modaldialogclass => class for the modal dialog
	 * 
	 * @param {Function(jQuery, Function())} generateContent - function which returns the content to show in the popup. 
	 * @param {Object} options - additional options
	 */
	ExpandModeModal.prototype.show = function(generateContent, options) {
		// Check the arguments
		if(!(generateContent && typeof(generateContent) === "function")) {
			throw new Error("Illegal arguments passed to ExpandModeModal.show");
		}
		if(!(options && typeof(options)==="object")) {
			options = {};
		}
		
		// For passing the close function before we actually have the Modal...
		// (so before the close function actually exists...)
		var closeContext = {};
		var closeFunction = _.bind(function(){this.close();}, closeContext);
		
		// Make the content for the modal
		var content = $("<div>");
		generateContent(content, closeFunction);
		
		var popupOptions = {};
		if(options.popupcssclass) {
			popupOptions.customClass = options.popupcssclass;
		}
		
		popupOptions.widthClass = options.modaldialogclass;
		
		popupOptions.onClose = options.onclose;
		
		// Make the modal
		var modal = new Modal(content, popupOptions);
		
		// Initialize the close function
		closeContext.close = _.bind(modal.close, modal);

		// Set a title
        modal.setTitle((options["title"]?options["title"]:this._defaultTitle) + (options["subtitle"]?" - "+options["subtitle"]:""));
        
        modal.open();
	};
	
	/**
	 * Sets the default title of the expand mode
	 * @param {String} title;
	 */
	ExpandModeModal.prototype.setDefaultTitle = function(title) {
		this._defaultTitle = title;
	};
	
	return ExpandModeModal;
});