define(["jquery", "underscore", "utils/webi18n!SearchInputs"], function($, _, bundle){
	
	/**
	 * The component creating the Search Results parameter panel (design panel)
	 */
	var SearchResultsParameterPanel = function(div, hpSectionId, api, searchSetup, searchData) {
		this._div = $(div);
		this._api = api;
		this._hpSectionId = hpSectionId; // Cleanup once other Javascript is no longer using the search button to rerun
		this._searchSetup = searchSetup;
		this._searchData = searchData;

		this._parameterstable = $(".designpaneltable > .designpanelsection", this._div);
		this._searchbuttontable = $(".searchbuttontable tr", this._div);
		this._lastRow = null;
		this._availableRowsMap = {};
		
		// Information about the search button (not set yet)
		this._isSearchButtonVisible = null;
		this._searchButton = null;
		
		this._initialize();
	};
	
	/**
	 * Get the element of the SearchResultsPanel
	 * @returns {JQueryObj}
	 */
	SearchResultsParameterPanel.prototype.getElement = function(){
		return $(".outerTdSearchParamsForm", this._div);
	};
	
	/**
	 * Used to make the search button show whether a rerun is needed.
	 * 
	 * @param needsrerun
	 */
	SearchResultsParameterPanel.prototype.setNeedsRerun = function(needsrerun) {
		this._searchButton.toggleClass("searchbuttondisabled", !needsrerun);
	};
	
	/**
	 * Register a callback to be executed when the searchbutton is clicked.
	 */
	SearchResultsParameterPanel.prototype.onSearchButtonClicked = function(callback) {
		this._searchButton.click(callback);
	};
	
	
	//
	// Below: private functions that create the design panel
	//
	
	/**
	 * Makes the design panel html
	 */
	SearchResultsParameterPanel.prototype._initialize = function() {
		// Get the list of design panel items
		var listvisible = [];
		var listavailable = [];
		this._searchSetup.getDesignPanelItems().each(function(item){
			var visibility = item.getOriginalVisibility();
			if(visibility === "Available") {
				listavailable.push(item);
			}else if(visibility === "Visible"){
				listvisible.push(item);
			}
		});
		
		// Make the "visible" design panel rows
		if(listvisible.length>0) {
			// 1) Sort the "visible list" on row, then column
			listvisible.sort(function compare(a,b) {
				var aRow = parseInt(a.getRow(), 10);
				var bRow = parseInt(b.getRow(), 10);
				var aCol = parseInt(a.getColumn(), 10);
				var bCol = parseInt(b.getColumn(), 10);
				
				if (aRow < bRow) {
					return -1;
				} else if (aRow > bRow) {
					return 1;
				} else {
					if(aCol < bCol) {
						return -1;
					} else if(aCol > bCol) {
						return 1;
					} else {
						return 0;
					}
				}
			});
			
			// 2) Add the visible rows...
			var row = null;
			var lastRowIndex = -1;
			if(listvisible.length>0) {
				row = this._appendRow(true);
				lastRowIndex = listvisible[0].getRow();
			}
			
			_.each(listvisible, function(item) {
				if(lastRowIndex !== item.getRow()) {
					row = this._appendRow(true);
					lastRowIndex = item.getRow();
				}
				this._putDesignPanelItem(row, item.getFieldId());
			}, this);
		}
		
		// Make the "Available rows + available dropdown"
		if(listavailable.length>0) {
			// 1) Dropdown
			// note: "availableparamsselect" class is for automated testing
			var dropdown = $('<select class="form-control availableparamsselect">');
			dropdown.append($('<option value="">').text(bundle.t("SELECT_CRITERION")));
			dropdown.on("change", _.bind(function() {
				// The user selected a value from the dropdown
				var fieldid = dropdown.val();
				if(_.has(this._availableRowsMap, fieldid)) {
					// ok, that is a valid fieldid...
					// make the row visible
					this._availableRowsMap[fieldid].css({"display":""});
					
					// also keep track of this in the object model! (for if we reload the search)
					var designPanelItem = this._searchSetup.getDesignPanelItems().findByFieldId(fieldid);
					designPanelItem.setVisibility("Visible");
					
					// Go back to nothing selected
					dropdown.val("");
				}
				
			}, this));
			
			// 2) All available rows
			_.each(listavailable, function(item) {
				// Do we need to show this "available" item? (happens if we go back to the search results page)
				var isShow = item.getVisibility()==="Visible";
				
				// Already make the row, but keep it hidden (if not selected before)
				var availablerow = this._appendRow(isShow);
				this._putDesignPanelItem(availablerow, item.getFieldId());
				
				// Add it to the map to be able to look it up later
				this._availableRowsMap[item.getFieldId()] = availablerow;
				
				// Get the label
				var searchfield = this._searchData.getFieldInformation(item.getFieldId(), this._searchSetup);
				var customizedlabel = this._searchSetup.getDesignPanelItems().getCustomizedLabel(searchfield);
				
				// Add the option to the dropdown
				dropdown.append($('<option>').attr("value", item.getFieldId()).text(customizedlabel));
			}, this);
			
			// Add dropdown to the page
			this._searchbuttontable.append($("<td>").append(dropdown));
		}
		
		// Some other stuff... "Completing the design panel"
		
		// 1) Make the search button!
		// Note: because some JavaScript still checks if the search button is visible, it needs to be ALWAYS on the page
		var hasvisibleitemsdesignpanel = listvisible.length>0;
		var hasavailableitemsdesignpanel = listavailable.length>0;
		
		this._isSearchButtonVisible = this._searchSetup.hasSearchButton();

		this._searchButton = $('<input name="Submit" type="button" class="buttonprimary SearchPanelSearchButton">').attr("value", bundle.t("SEARCHACTION"));
		this.setNeedsRerun(false);
		
		// This is this here because it is used by quite some Javascript functionality to check if "automatic rerun" should happen.
		// Leaving it here for now...
		this._searchButton.attr("id", this._hpSectionId+"_SearchPanelSearchButton");
		if(this._isSearchButtonVisible) {
			this._searchButton.css({"visibility":"visible"});
		} else {
			this._searchButton.css({"visibility":"hidden", "height":"0px"});
		}
		// End of code for old JS
		
		this._searchbuttontable.prepend($("<td>", 
				{"class":"SearchButtonSection "+(this._isSearchButtonVisible?"SearchButtonVisible":"SearchButtonHidden")})
				.append(this._searchButton));
		
		// 2) Make the parameter panel visible...
		if(this._isSearchButtonVisible || hasvisibleitemsdesignpanel || hasavailableitemsdesignpanel) {
			$(".outerTdSearchParamsForm", this._div).css({"display":"block"});
		}
	};
	
	/**
	 * Adds a new row to the design panel and returns the jQuery element
	 * @param {boolean} isVisible - whether the row should be visible
	 * @returns {jQuery}
	 */
	SearchResultsParameterPanel.prototype._appendRow = function(isVisible) {
		var row = $("<tr>", {"class":"designpanelrow"});
		if(!isVisible) {
			row.css({"display":"none"});
		}
		if(this._lastRow === null) {
			this._parameterstable.prepend(row);
		} else {
			this._lastRow.after(row);
		}
		this._lastRow = row;
		return row;
	};
	
	/**
	 * Add a design panel input to the row.
	 * @param {jQuery} row - the row to add the input in
	 * @param {String} fieldid - the id of the field to add in the cell.
	 */
	SearchResultsParameterPanel.prototype._putDesignPanelItem = function(row, fieldid) {
		/*
		 * Adds the following html structure to the row:
		 
	      <td class="labelcell">
			<label>... label ...</label>
		  </td>
		  <td class="inputcell">
		  	<div class="itemwrapper">
			  	<div class="searchdesignpanelfield" data-fieldid="... fieldid ...">
			  		... loader image / component ...
			  	</div>
			</div>
		  </td>
		 */
		
		var celllabel = $("<td>", {"class":"labelcell"});
		var cellinput = $("<td>", {"class":"inputcell"});
		
		var searchfield = this._searchData.getFieldInformation(fieldid, this._searchSetup);
		var customizedlabel = this._searchSetup.getDesignPanelItems().getCustomizedLabel(searchfield);
		
		var label = $("<label>", {"text":customizedlabel});
		var inputdiv = $("<div>", {"class":"searchdesignpanelfield"});
		
		// Add all the elements
		cellinput.append($("<div>", {"class":"itemwrapper"}).append(inputdiv));
		celllabel.append(label);
		
		// add fieldid for automated testing
		inputdiv.attr("data-fieldid", fieldid);
		
		// start loading the field inside the div
		this._api.makeParameterField(fieldid, inputdiv.get(0));
		
		row.append(celllabel, cellinput);
	};
	
	return SearchResultsParameterPanel;
});