define([
        "jquery",
        "underscore",
        "../../model/SearchSortingCriteriaList"
        ], function(
    		// For input checking
        	$,
        	_,
    		SearchSortingCriteriaList
        ){
	
	/**
	 * Constructor
	 * @param {jQuery} div
	 * @param {SearchSortingCriteriaList} searchSetupSort - the sorting part of the setup object model
	 * @param {Function} rerunCallback - the callback to call to rerun the search
	 */
	var SearchResultsResortHandler = function($div, searchSetupSort, rerunCallback) {
		if(!(	$div && $div instanceof $ && 
				searchSetupSort && searchSetupSort instanceof SearchSortingCriteriaList &&
				rerunCallback && _.isFunction(rerunCallback))) {
			throw new Error("Illegal arguments passed to SearchResultsResortHandler");
		}
		
		this.div = $div;
		this._searchSetupSort = searchSetupSort;
		this.rerunCallback = rerunCallback;
	};
	
	/**
	 * Initialize...
	 * 
	 * Registers the listeners on the table...
	 */
	SearchResultsResortHandler.prototype.initialize = function(){
		// Register listeners! => sorting headers
		$("th.column-sort[data-columnid]", this.div).click(_.partial(function(event, resorthandler){
			var header = $(this);
			var columnName = header.attr("data-columnid");
			resorthandler.sortSearchResults(header, event, columnName);
		}, _, this));
		
		// Register listeners! => sorting dropdowns...
		$(".searchresultssortingdropdown", this.div).change(_.partial(function(event, resorthandler){
			var dropdown = $(this);
			resorthandler.sortGridSearchResults(dropdown);
		}, _, this));
	};
	
	/**
	 * (private) Make changes to the object model & rerun
	 * @param fieldId
	 * @param newOrder
	 */
	SearchResultsResortHandler.prototype._applySort = function (fieldId, newOrder) {
		// Find the sort number of the one we clicked...
		var sortingCriterion = this._searchSetupSort.findByFieldId(fieldId);
		var oldClickedColumnNumberStr = sortingCriterion!==null?sortingCriterion.getNumber():"";
		var oldClickedColumnNumber = oldClickedColumnNumberStr?parseInt(oldClickedColumnNumberStr):-1;
	    
	    // Adjust the sort order for the non clicked columns
		this._searchSetupSort.each(function(otherSortingCriterion) {
			var numberStr = otherSortingCriterion.getNumber();
			var number = parseInt(numberStr);
			if (numberStr !== "") {
				if (sortingCriterion === null || (number < oldClickedColumnNumber)) {
					otherSortingCriterion.setNumber(""+(number+1));
				}
			}
		});
		
		if(sortingCriterion==null) {
			sortingCriterion = this._searchSetupSort.createItem(fieldId);
			this._searchSetupSort.add(sortingCriterion);
		}
		// Adjust the hidden sort number and order for the clicked column and 
		// also set the hidden sort checkbox to checked
		sortingCriterion.setEnabled(true);
		sortingCriterion.setOrder(newOrder);
		sortingCriterion.setNumber("1");
		
		this._searchSetupSort.commitChanges();
		
		// Re-submit the search
		this.rerunCallback();
	};
	
	/**
	* Used in search results screen (List mode), when the user clicks on a results table column header
	* Adjusts the sort information in the object model and re-submits the search.
	* 
	* @param {jQuery} header - the header element (with the classes)
	* @param {ClickEvent} event
	* @param fieldId : the clicked column (for instance db_name)
	*/
	SearchResultsResortHandler.prototype.sortSearchResults = function(header, event, fieldId) {
		if (event && ($(event.target).is(".searchheadereditbutton") || $(event.target).parents().is(".searchheadereditbutton"))){ 
			// do not sort when clicking on an action
			return; 
		}
		
		var newSortOrder = "";
		
		// Change the clicked header's image
		if(header.hasClass("sort-asc")) {
			// was sorted descending
			newSortOrder = "desc";
		}else if(header.hasClass("sort-desc")) {
			// was sorted ascending
			newSortOrder = "asc";
		}else{
			// did not have an order yet...
			newSortOrder = "asc";
		}
		
		// Apply new classes to display the order
		// Note: we do not really have to update this since we will submit anyway...
		// Note2: we don't update the icon of the other columns here...
		header.removeClass("sort-available");
		header.removeClass("sort-asc");
		header.removeClass("sort-desc");
		header.addClass("sort-"+newSortOrder);
		
		// Make the changes in the object model & rerun
		this._applySort(fieldId, newSortOrder);
	};

	/**
	* Used in search results screen (Grid mode), when the user selects an option from the Sort By drop down.
	* Adjusts the sort information in the object model and re-submits the search.
	* 
	* @param changedDropdown : the sorting dropdown which was changed
	*/
	SearchResultsResortHandler.prototype.sortGridSearchResults = function(changedDropdown){
		var selectedOption = $(":selected", changedDropdown);
		var fieldId = selectedOption.attr("data-fieldId");
		var order = selectedOption.attr("data-order");
		
		if (fieldId) {
			this._applySort(fieldId, order);
		}
	};
	
	return SearchResultsResortHandler;
});
