define([
        "jquery", 
        "underscore",
        "component/smartnames/encoding/smartnameencoding",
        "utils/webi18n!SearchInputs"
       ], function(
    		   $, 
    		   _,
    		   SmartNameEncoding,
    		   bundle
       ){
	
	/**
	 * JavaScript which controls the inputs for design panel on search setup page
	 * 
	 * @param {jQuery} $div - the search setup div
	 * @param {SearchModelFrontEndData} searchData - the information about all fields
	 * @param {SearchSetupInformation} searchSetup - the model with all information about the design panel items
	 */
	var SetupControlDesignPanel = function($div, searchData, searchSetup){
		// Store information
		this.$div = $div;
		this._searchData = searchData;
		this._searchSetup = searchSetup;
		// Store the SearchDesignPanelItems part of the model separately for easy access
		this._model = this._searchSetup.getDesignPanelItems();
		
		// Initialize
		this._init();
	};
	
	/**
	 * Validation of the design panel
	 */
	SetupControlDesignPanel.prototype.validate = function(){
		// Validate the design panel
		return this._validateDesignPanel();
	};
	
	
	/**
	 * (private)
	 * @returns whether smartnames are enabled
	 */
	SetupControlDesignPanel.prototype._isSmartNamesEnabled = function(){
		return this._searchSetup.getFeatures().getUseSmartNames();
	};
	
	/**
	 * Initialize listeners... (new architecture)
	 */
	SetupControlDesignPanel.prototype._init = function() {
		this._lastknownstate = {};
		
		// Store the initial state
		this._model.each(_.bind(function(item){
			var fieldId = item.getFieldId();
			var visibility = item.getOriginalVisibility();
			this._lastknownstate[fieldId] = visibility;
		}, this));
		
		this._model.onChange(_.bind(function(){
			// Something changed in the model...
			// Find out what
			var _newstate = {};
			this._model.each(_.bind(function(item){
				var fieldId = item.getFieldId();
				var visibility = item.getOriginalVisibility();
				if(visibility !== this._lastknownstate[fieldId]) {
					// Field changed it's visibility
					this._lastknownstate[fieldId] = visibility;// make sure we don't call triggerVisibilityChanged for the same thing twice
					this._triggerVisibilityChanged(item);
				}
				_newstate[fieldId] = visibility;
			}, this));
			this._lastknownstate = _newstate;
			this._model.commitChanges();
		}, this));
		
		// Also listen to whether the values in the criteria contain SmartNames
		this._searchSetup.getCriteriaList().onSmartNameChange(_.bind(function(){
			// Some field now has or no longer has a SmartName...
			
			// Loop through all visible design panel items
			this._model.each(_.bind(function(item){
				var fieldId = item.getFieldId();
				var visibility = item.getOriginalVisibility();
				if(visibility==="Visible" || visibility==="Available") {
					var criterion = this._searchSetup.getCriteriaList().findByFieldId(fieldId);
					if(criterion.getContainsSmartName()) {
						// criterion contains SmartName and field is in design panel
						alert(bundle.t('SMART_NAME_INSERT_VISIBILITY_ERR_MSG'));
						item.setOriginalVisibility("Hidden");
					}
				}
				this._lastknownstate[fieldId] = visibility;
			}, this));
		}, this));
	};
	
	
	/**
	 * Called by new architecture when some field changed it's visibility
	 * 
	 * @param {SearchDesignPanelItem} item - the item for which the visibility changed
	 */
	SetupControlDesignPanel.prototype._triggerVisibilityChanged = function(item){
		var newVisibility = item.getOriginalVisibility();
		if(newVisibility === "Available" || newVisibility === "Visible") {
			// Not allowed if we have SmartNames... Check for it here...
			// Look up the matching criterion
			var matchingCriterion = this._searchSetup.getCriteriaList().findByFieldId(item.getFieldId());
			// Does it contain a SmartName?
			var hasSmartName = matchingCriterion?matchingCriterion.getContainsSmartName():false;
			if(hasSmartName) {
				// Show an error
				alert(bundle.t('SMART_NAME_VISIBILITY_ERR_MSG'));
				// and set it back to Hidden
				item.setOriginalVisibility("Hidden");
			}
		}
		
		if(newVisibility === "Hidden" || newVisibility === "Available") {
			// Clear the numbers
			item.setRow("");
			item.setColumn("");
		} else {
			// First check if we already have some numbers filled in in the model through some other means.
			var oldrow = item.getRow();
			var oldcol = item.getColumn();
			if(oldrow && oldcol) {
				// Check if the place is ok (nothing else is there)
				var placeIsEmpty = true;
				this._processItems(function(fieldId, visibility, row, col) {
					if(fieldId!==item.getFieldId() && visibility === "Visible" && row === oldrow && col === oldcol) {
						// There is already a component at that position!
						placeIsEmpty = false;
					}
				});
				if(placeIsEmpty) {
					// Ok, place empty
					// Keep the position
					return;
				}
			}
			
			// Get the next numbers
			item.setRow("");
			item.setColumn("");
			var next = this._getNextRowAndColumnNumber();
			item.setRow(""+next.row);
			item.setColumn(""+next.col);
		}
	};
	
	/**
	 * Loop through all the information
	 */
	SetupControlDesignPanel.prototype._processItems = function(callback) {
    	// Loop through fields in new architecture
    	this._model.each(_.bind(function(item){
			var fieldId = item.getFieldId();
			var visibility = item.getOriginalVisibility();
			item.setRow((""+item.getRow()).trim());
			item.setColumn((""+item.getColumn()).trim());
			var row = item.getRow();
			var col = item.getColumn();
			
			callback(fieldId, visibility, row, col);
    	}, this));
	};
    
    /**
     * JavaScript to get the next row & column number
	 * 
	 * Moved from searchtemplate.xsl:DesignPanelScript
     */
    SetupControlDesignPanel.prototype._getNextRowAndColumnNumber = function() {
    	var result = {'row':1, 'col':0};
    	
    	// For each field...
    	this._processItems(_.bind(function(fieldId, visibility, row, col) {
    		// Check visibility and find largest row & col number
    		if (visibility === "Visible") {
	    		if (this._isNumeric(row, false, false) && this._isNumeric(col, false, false)) {
					if (+row >= result.row) {
						if (+row > result.row) {
							result.row = +row;
							result.col = +col;
						} else if (+col > result.col) {
							result.col = +col;
						}
					}
				}
    		}
    	}, this));

    	// Pick the one after the last one found...
    	result.col++;
		if (result.col > 3)
		{
			result.col = 1;
			result.row++;
		}
		
		// Return the result
		return result; 
    };
    
    /**
     * JavaScript to validate the design panel in the old architecture
	 * 
	 * Moved from searchtemplate.xsl:DesignPanelScript
     */
    SetupControlDesignPanel.prototype._validateDesignPanel = function(){
		var rowColArr = new Array();
		var arrCt = 0;

		var items = [];
		this._processItems(_.bind(function(fieldId, visibility, row, col){
			items.push({row:row, col:col, visibility:visibility});
		}, this));
		
		
		for(var i=0;i<items.length;i++) {
			var item = items[i];
			if(item.visibility==="Visible") {
				if (!this._isNumeric(item.row, false, false))
			    {
			    	alert(bundle.t('SEARCH_DESIGN_PANEL_ROW'));
			    	return false;
			    }
			    if (item.row < 1 )
			    {
			    	alert(bundle.t('SEARCH_DESIGN_PANEL_ROW_RANGE'));
			    	return false;
			    }
			    
			    if (!this._isNumeric(item.col, false, false))
			    {
		    		alert(bundle.t('SEARCH_DESIGN_PANEL_COLUMN'));
		    		return false;
			    }
			    if (item.col < 1 )
			    {
			    	alert(bundle.t('SEARCH_DESIGN_PANEL_COLUMN_RANGE'));
			    	return false;
			    }
			    
				// store row/column values for uniqueness check
				rowColArr[arrCt++] = (""+item.row) + ";" + (""+item.col);
			}
		}

		// are the values of the selected row / column pair unique ? 
		for (var i=0; i < rowColArr.length; i++) {
			for (var j=i+1; j < rowColArr.length; j++) {
			  	if (rowColArr[i] == rowColArr[j]) {
			  		alert(bundle.t('SEARCH_DESIGN_PANEL_UNIQUE'));
					return false;
			  	}
	        }
		}
		
		// Validation ok
		return true;
    };
	
	
	/**
	 * Some util method to check if a number is numeric.
	 * 
	 * Moved from searchtemplate.xsl. Can probably be moved to some easy to use util function.
	 */
    SetupControlDesignPanel.prototype._isNumeric = function(sText, decimals, negatives) {
		var isNumber=true;
		var numDecimals = 0;
		var validChars = "0123456789";
		if (decimals)  validChars += ".";
		if (negatives) validChars += "-";
		var thisChar;
		
		if (sText.length == 0)
		{
			isNumber = false;
		}
		else
		{
			for (var i = 0; i < sText.length && isNumber == true; i++) 
			{  
				thisChar = sText.charAt(i); 
				if (negatives && thisChar == "-" && i > 0) isNumber = false;
				if (decimals && thisChar == ".")
				{
					numDecimals = numDecimals + 1;
					if (i==0 || i == sText.length-1) isNumber = false;
					if (numDecimals > 1) isNumber = false;
				}
				if (validChars.indexOf(thisChar) == -1) isNumber = false;
			}
		}
		return isNumber;
	};
	
	
	return SetupControlDesignPanel;
	
});