define(["underscore", "utils/Map"], function (_, Map) {
	"use strict";

	/**
	 * JavaScript which controls the inputs for columns on search setup page
	 * 
	 * @param {jQuery} $div - the search setup div
	 * @param {SearchModelFrontEndData} searchData - the information about all fields
	 * @param {SearchSetupInformation} searchSetup - the model with all information about the design panel items
	 */
	var SetupControlDisplayColumn = function ($div, searchData, searchSetup) {
		// Store information
		this.$div = $div;
		this._searchData = searchData;
		this._searchSetup = searchSetup;
		// Store the SearchDisplayColumns part of the model separately for easy access
		this._model = this._searchSetup.getDisplayColumns();

		// Keep track of the old values
		this._oldValues = new Map();
		this._lastknowncheckboxstate = new Map();
		
		// Only used during deletion of columns...
		this._itemsPendingRemoval = [];

		// Initialize
		this._init();

		this.startColumnSequenceFields();
	};

	/**
	 * Initialize listeners... (new architecture)
	 */
	SetupControlDisplayColumn.prototype._init = function () {

		// Store the initial state
		this._model.each(_.bind(function (item) {
			var fieldId = item.getFieldId();
			this._lastknowncheckboxstate.add(fieldId, item.getEnabled());
		}, this));

		// Listen to changes in the model
		this._model.onChange(_.bind(function () {
			// Something changed in the model...

			// First check if sorting is ok...
			// Check the case where the columns were already moved by some other code (so everything is still consistent)
			// If so, we will not call any code to reorder the numbers...
			var numbers = [1];
			var max = 1;
			var min = 1;
			this._processItems(function (otheritem) {
				try {
					if (otheritem.getEnabled()) {
						var value = +(otheritem.getSequence());
						if(!isNaN(value)) {
							numbers.push(value);
							if (value > max) {
								max = value;
							}
							if (value < min) {
								min = value;
							}
						}
					}
				} catch (e) {}
			});

			function sortNumber(a, b) {
				return a - b;
			}
			var sortingOk = _.reduce(numbers.sort(sortNumber), function(memo, num, index) {return memo && num===(index+1);}, true);

			if (sortingOk) {
				// So, sorting is correct...
				// Just update the internal state
				this._processItems(function (otheritem) {
					try {
						if (!otheritem.getEnabled()) {
							otheritem.setSequence("-1");
						}
						otheritem.setOldValue(otheritem.getSequence());
					} catch (e) {}
				});
				this._lastknowncheckboxstate = new Map();
				this._model.each(_.bind(function (item) {
					var fieldId = item.getFieldId();
					this._lastknowncheckboxstate.add(fieldId, item.getEnabled());
				}, this));
			} else {
				// Find out what changed
				var _newstate = new Map();
				this._model.each(_.bind(function (item) {
					var fieldId = item.getFieldId();
					var fieldNewState = item.getEnabled();
					if (fieldNewState === false && !this._lastknowncheckboxstate.has(fieldId)) {
						// We started editing a new field... Just ignore.
					} else if (fieldNewState !== (this._lastknowncheckboxstate.get(fieldId) || false)) {
						// Field changed it's enabled state
						item.setSequence("-1");
						this.showHideColumnSequenceField(item.getFieldId());
					} else if (fieldNewState === true && item.getSequence() !== this._oldValues.get(fieldId)) {
						// Field changed it's sequence number
						this.updateColumnSequenceFields(item.getFieldId());
					}
					_newstate.add(fieldId, fieldNewState);
				}, this));
				this._itemsPendingRemoval = [];
				this._lastknowncheckboxstate.each(_.bind(function (key, value) {
					if (!_newstate.has(key)) {
						// This item was removed completely...
						this._itemsPendingRemoval.push(key);
					}
				}, this));
				while(this._itemsPendingRemoval.length>0) {
					// Fill up the gaps...
					this.updateColumnSequenceFields(this._itemsPendingRemoval.pop(), true);
				}
				this._itemsPendingRemoval = [];
				this._lastknowncheckboxstate = _newstate;
			}

			this._model.commitChanges();
		}, this));
	};

	/**
	 * @author Hans Dewitte
	 * Shows or hides a column sequence field based on the checkbox value to the left of it
	 * Used as onclick for the view checkbox (but only makes sense on the main search page, not on the attributes page)
	 */
	SetupControlDisplayColumn.prototype.showHideColumnSequenceField = function (fieldId) {
		try {
			var item = this._getControl(fieldId);
			if (!item) {
				return;
			}

			if (item.getEnabled()) {
				// give it the highest number in the sequence
				var highestNumber = 1; // Always start from 2... => db_name is hardcoded as 1.
				this._processItems(function (item) {
					var value = parseInt(item.getSequence());
					if (value > highestNumber) {
						highestNumber = value;
					}
				});

				item.setSequence(highestNumber + 1);
				item.setOldValue(highestNumber + 1);
			} else {
				item.setSequence(-1);
				this.updateColumnSequenceFields(fieldId, true);
			}
		} catch (err) {
			window.alert('Problem in showHideColumnSequenceField at line ' + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	 * @author Hans Dewitte
	 * To be called after each change on the visible columns and their sequence
	 * Makes the right columns visible and re-orders them apprpriately
	 * 
	 * @param {String} fieldId - the field which changed
	 * @param {Boolean} takenOut - whether the field was taken out
	 */
	SetupControlDisplayColumn.prototype.updateColumnSequenceFields = function (fieldId, takenOut) {
		try {
			// find new and old value of the changed field
			var curitem = this._getControl(fieldId);

			var oldValue = parseInt(curitem.getOldValue());
			var newValueString = curitem.getSequence();
			var newValue = parseInt(newValueString, 10);
			// check sanity of newValue
			if (newValue !== parseInt(newValueString, 10)) {
				window.alert("Please enter a number");
				// put back the old value
				oldValue = curitem.getOldValue();
				curitem.setSequence(oldValue);
				return;
			}
			var highestNumber = 1;
			this._processItems(function (item) {
				var value = parseInt(item.getOldValue());
				if (value > highestNumber) {
					highestNumber = value;
				}
			});
			if (newValue > highestNumber) {
				newValue = highestNumber;
			}

			if (takenOut) { // column taken out --> shift all columns with value higher than newvalue up
				this._processItems(_.bind(function (item) {
					var curVal = parseInt(item.getSequence());
					if (curVal > oldValue) { // will automatically skip hidden once since their curVal is never greater than oldValue
						item.setSequence(curVal - 1);
						item.setOldValue(curVal - 1);
					}
				}, this));
			} else if (newValue < 2) {
				window.alert("Column number must be 2 or higher");
				// put back on old value
				oldValue = curitem.getOldValue();
				curitem.setSequence(oldValue);
			} else {
				if (oldValue < newValue) {
					// example: put column 3 to 6. Then columns 4, 5 and 6 need to move 1 down, all the rest remains the same
					this._processItems(_.bind(function (item) {
						var curVal = parseInt(item.getSequence());
						if (curVal > oldValue && curVal <= newValue) { // will automatically skip hidden once since their curVal is never greater than oldValue
							item.setSequence(curVal - 1);
							item.setOldValue(curVal - 1);
						}
					}, this));
				} else if (oldValue > newValue) {
					// example: put column 6 to 3. Then columns 3, 4 and 5 need to move 1 up, all the rest remains the same
					this._processItems(_.bind(function (item) {
						var curVal = parseInt(item.getSequence());
						if (curVal >= newValue && curVal < oldValue) { // will automatically skip hidden once since their curVal is never greater than oldValue
							item.setSequence(curVal + 1);
							item.setOldValue(curVal + 1);
						}
					}, this));
				}
				curitem.setSequence(newValue);
				curitem.setOldValue(newValue);
			}
		} catch (err) {
			window.alert('Problem in updateColumnSequenceFields at line ' + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	 * Sets the column order fields in a good initial condition, with all visible fields having a number and all invisible fields on -1 
	 * also sets oldValue for all fields
	 * Must be called when the document is loaded.
	 */
	SetupControlDisplayColumn.prototype.startColumnSequenceFields = function () {
		try {
			// Collect all display columns...
			var allItems = [];
			this._processItems(_.bind(function (item) {
				if (item.getEnabled()) {
					allItems.push(item);
				} else {
					// Set non-visible ones to -1
					item.setSequence(-1);
					item.setOldValue(-1);
				}
			}, this));
			
			// Sort them by sequence number (but put "not filled in" (-1) at the end)
			allItems.sort(function(itema, itemb) {
				var a = parseInt(itema.getSequence());
				var b = parseInt(itemb.getSequence());
				if(a==b) {
					return 0;
				} else if(a===-1) {
					return 1;
				} else if(b===-1) {
					return -1;
				} else {
					return a-b;
				}
			});
			
			// Re-assign sequence numbers to fill holes
			var current = 2;
			_.each(allItems, function (item) {
				var value = parseInt(item.getSequence());
				if (value != current) {
					item.setSequence(current);
				}
				item.setOldValue(current);
				current++;
			}, this);
		} catch (err) {
			window.alert('Problem in startColumnSequenceFields at line ' + err.lineNumber + ' : ' + err.message);
		}
	};

	/**
	 * Loop through all the information
	 */
	SetupControlDisplayColumn.prototype._processItems = function (callback) {
		this._model.each(_.bind(function (displayColumn) {
			callback(this._getControl(displayColumn.getFieldId()));
		}, this));
		_.each(this._itemsPendingRemoval, function(fieldId) {
			callback(this._getControl(fieldId));
		}, this);
	};

	SetupControlDisplayColumn.prototype._getControl = function (fieldId) {
		var displayColumn = this._model.findByFieldId(fieldId);
		var item = {
			// Set & get the previous sequence values
			setOldValue: _.bind(function (value) {
				if (value === undefined) {
					this._oldValues.remove(fieldId);
				} else {
					this._oldValues.add(fieldId, value);
				}
			}, this),
			getOldValue: _.bind(function (value) {
				return this._oldValues.get(fieldId);
			}, this)
		};

		if (displayColumn) {
			item.getFieldId = _.bind(displayColumn.getFieldId, displayColumn);
			item.getEnabled = _.bind(displayColumn.getEnabled, displayColumn);
			item.setSequence = _.bind(displayColumn.setSequence, displayColumn);
			item.getSequence = _.bind(displayColumn.getSequence, displayColumn);
		} else {
			// If we deleted some row, we still need to provide some information to reorder the other rows.
			var temp_sequence = "-1";
			item.getFieldId = function () {
				return fieldId;
			};
			item.getEnabled = function () {
				return false;
			};
			item.setSequence = function (value) {
				temp_sequence = value;
			};
			item.getSequence = function () {
				return temp_sequence;
			};
		}
		return item;
	};


	return SetupControlDisplayColumn;

});