define([], function(){
	
	/**
	 * JavaScript which controls the inputs for columns on search setup page
	 * 
	 * @param {jQuery} $div - the search setup div
	 * @param {SearchModelFrontEndData} searchData - the information about all fields
	 * @param {SearchSetupInformation} searchSetup - the model with all information about the design panel items
	 */
	var SetupControlSorting = function($div, searchData, searchSetup) {
		// Store information
		this.$div = $div;
		this._searchData = searchData;
		this._searchSetup = searchSetup;
		// Store the SearchDisplayColumns part of the model separately for easy access
		this._model = this._searchSetup.getSortingCriteriaList();
		
		// Some information to detect changes
		this._lastknownstate = {};
		this.sortedFieldIds = [];
		this.boxesSelected = 0;
		
		// Initialize
		this._init();
	};
	
	/**
	 * Saves a copy of the current state in the object model.
	 * Used to compare when something changes in the object model and where to introduce/remove numbers...
	 */
	SetupControlSorting.prototype._saveCurrentState = function() {
		this.sortedFieldIds = [];
		this._lastknownstate = {};
		this._model.each(_.bind(function(item){
			var fieldId = item.getFieldId();
			if(item.getEnabled()) {
				this._lastknownstate[fieldId] = item.getNumber();
				this.sortedFieldIds.push(fieldId);
			}
		}, this));
		this.boxesSelected = this.sortedFieldIds.length;
	};
	
	
	/**
	 * Initialize
	 */
	SetupControlSorting.prototype._init = function() {
		// Store the initial state
		this._saveCurrentState();
		
		// Listen to changes in the model
		this._model.onChange(_.bind(function(){
			// Something changed in the model...
			
			// A. Did we do multiple updates at once? (Check if the numbers are still ok)
			// This can be the case if we change the id of a field in the object model but kept the numbers.
			var numbers = []; 
			
			// A.1) collect all numbers
			this._model.each(_.bind(function(item){
				if(item.getEnabled()){
					try{
						numbers.push(parseInt(item.getNumber()));
					}catch(e){
						numbers.push(-1);
					}
				}
			}, this));
			
			// A.2) check them
			function sortNumber(a,b) {return a - b;}
			numbers = numbers.sort(sortNumber);
			if(numbers.length === 0 || (numbers[0] === 1 && numbers[numbers.length-1] === numbers.length && _.uniq(numbers, true).length === numbers.length)) {
				//
				// This is a correct order... (changed by some other code)
				//
				this._saveCurrentState();
			} else {
				//
				// Order is not correct
				//
				
				// Find out what changed (note: we only care about the enabled/disabled state)
				var _newstate = {};
				this._model.each(_.bind(function(item){
					var fieldId = item.getFieldId();
					
					var enabled = item.getEnabled();
					var wasEnabled = this._lastknownstate.hasOwnProperty(fieldId);
					
					// Did we change the enabled state?
					if(enabled !== wasEnabled){
						// call some method
						this.boxSelected(fieldId);
					}
					
					// Also store new state
					if(enabled) {
						_newstate[fieldId] = item.getNumber();
					}
				}, this));
				for(var fieldId in this._lastknownstate) {
					if(this._lastknownstate.hasOwnProperty(fieldId) && !_newstate.hasOwnProperty(fieldId)) {
						// This item was removed completely...
						this.boxSelected(fieldId);
					}
				}
				this._lastknownstate = _newstate;
			}
			
			this._model.commitChanges();
		}, this));
	};
    
    /*
    * This function is called when a search criterion is selected for ordering
    */
	SetupControlSorting.prototype.boxSelected = function (fieldId) {
		var control = this._getControl(fieldId);
    	if(control.getEnabled()) {
    		this.boxesSelected++;
    		control.setNumber(this.boxesSelected);
    		control.setOrder('asc');
    		this.sortedFieldIds.push(fieldId);
    	} else {
    		this.boxesSelected--;
    		// Update the other fields that are being displayed
    		this.updateNumberFields(control.getNumber());
    		control.setNumber('');
    		control.setOrder('');
    	}
    };
    
    /*
    * This function changes the number values for the selected sorting criteria
    */
    SetupControlSorting.prototype.updateNumberFields = function (number) {
    	for(var i = this.sortedFieldIds.length-1; i >= 0; i--) {
    		if(this.sortedFieldIds[i] != null) {
    			var control = this._getControl(this.sortedFieldIds[i]);
    			if(!control.getEnabled()) {
    				this.sortedFieldIds.splice(i, 1);
    			} else if(parseInt(control.getNumber()) > parseInt(number)) {
        			control.setNumber(control.getNumber()-1);
        		}
    		}
    	}
    };

    SetupControlSorting.prototype._getControl = function(fieldId) {
		var sortingCriterion = this._model.findByFieldId(fieldId);
		var item = {};
		
		if(sortingCriterion) {
			item.getFieldId = _.bind(sortingCriterion.getFieldId, sortingCriterion);
			item.getEnabled = _.bind(sortingCriterion.getEnabled, sortingCriterion);
			item.setNumber = _.bind(sortingCriterion.setNumber, sortingCriterion);
			item.getNumber = _.bind(sortingCriterion.getNumber, sortingCriterion);
			item.setOrder = _.bind(sortingCriterion.setOrder, sortingCriterion);
			item.getOrder = _.bind(sortingCriterion.getOrder, sortingCriterion);
		}else{
			// If we deleted some row, we still need to provide some information to reorder the other rows.
			var temp_number = this._lastknownstate[fieldId];
			var temp_order = "";
			item.getFieldId = function(){return fieldId;};
			item.getEnabled = function(){return false;};
			item.setNumber = function(value){temp_number = value;};
			item.getNumber = function(){return temp_number;};
			item.setOrder = function(value){temp_order = value;};
			item.getOrder = function(){return temp_order;};
		}
		return item;
    };
	
	return SetupControlSorting;
});