/**
 * CellRenderer: Renders a cell in the Search Results Table.
 * 
 * Looks for implementations in a certain folder...
 *
 * @author JEPE
 */
define([
        "../../dataobjects/TableDisplaySettings",
        "../../dataobjects/FieldProperties",
        "../../dataobjects/ValueData",
        "../../dataobjects/ExternalSearchSettings",

        // All the implementations
        "../../implementations/datarenderers/all"
        ], function (
	TableDisplaySettings,
	FieldProperties,
	ValueData,
	ExternalSearchSettings,
	rendererfactories
) {

	/**
	 * Constructor
	 * 
	 * @param {Element} div - the div to render in
	 * @param {String} fieldDisplayMethod - the way to display this value...
	 * @param {String} fieldid - the id of the field to display
	 * @param {ValueData} data - the available data about the document/project/task we want to display
	 * @param {TableDisplaySettings} tableDisplaySettings - the settings to display the table... 
	 * @param {FieldProperties} fieldProperties - field specific information (currently only units)
	 * @param {ExternalSearchSettings} searchSettings - general WCR settings, not related to a specific Search Setup
	 */
	var CellRenderer = function (div, fieldDisplayMethod, fieldid, data, tableDisplaySettings, fieldProperties, searchSettings) {
		// Checking Parameters
		if (!(
				div && 1 === div.nodeType /* check if it is a HTML Element */ &&
				fieldid && (typeof fieldid === "string") &&
				data && data instanceof ValueData &&
				tableDisplaySettings && tableDisplaySettings instanceof TableDisplaySettings &&
				fieldProperties && fieldProperties instanceof FieldProperties)) {
			throw new Error("Wrong parameters passed to constructor of CellRenderer");
		}

		// Store div
		this.$div = $(div);

		// Some variables...
		var self = this;
		this.fieldid = fieldid;
		this.data = data;
		this.tableDisplaySettings = tableDisplaySettings;
		this.fieldProperties = fieldProperties;
		this.searchSettings = searchSettings;

		// Loading information
		// Note: we are no longer dynamically loading the render implementations... 
		// But we just keep this code here, because it is easier to support "missing implementations" (they never finish loading)
		this.doneloading = false;
		this.whenloadedcallbacks = [];

		// Information returned from the field about what is rendered...
		this.returninfo = {};

		if (fieldDisplayMethod) {
			// Load Renderer
			if (rendererfactories.hasOwnProperty(fieldDisplayMethod)) {
				// Renderer was already cached (happens when we do a rerun of search => same fields are rendered)
				self.rendermodule = rendererfactories[fieldDisplayMethod];
				self._render();
			} else {
				// The renderer was not added to the datarenderers/all file...
				self._showError("Cannot find render type \"" + fieldDisplayMethod + "\".");
			}
		} else {
			// Missing DisplayMethod parameter...
			// We could also ignore this, but this should never happen, so show an error
			self._showError("not configured");
		}
	};

	/**
	 * 
	 * @param callback
	 */
	CellRenderer.prototype.whenLoaded = function (callback) {
		if (!this.doneloading) {
			this.whenloadedcallbacks.push(callback);
		} else {
			callback();
		}
	};

	/**
	 * (private)
	 * Trigger the callbacks which are waiting for this component to load.
	 */
	CellRenderer.prototype._triggerLoadedCallbacks = function () {
		// Done loading... Notify listeners...
		if (!this.doneloading) {
			this.doneloading = true;
			for (var i = 0; i < this.whenloadedcallbacks.length; i++) {
				this.whenloadedcallbacks[i]();
			}
			this.whenloadedcallbacks = [];
		}
	};

	/**
	 * No value available for this field...
	 */
	CellRenderer.prototype._renderEmpty = function () {
		this.$div.empty();
		this._triggerLoadedCallbacks();
	};

	/**
	 * Render, update the cell in the table...
	 */
	CellRenderer.prototype._render = function () {
		this.$div.empty();
		if (this.rendermodule.hasOwnProperty("render") && typeof (this.rendermodule["render"]) === "function") {
			try {
				this.returninfo = this.rendermodule.render(this.$div.get(0), this.fieldid, this.data, this.tableDisplaySettings, this.fieldProperties, this.searchSettings);
			} catch (e) {
				this._showError("Cannot render cell", e);
			}
			this._triggerLoadedCallbacks();
		} else {
			this._showError("Display Method does not provide render");
		}
	};

	/**
	 * Cannot render... Show an error message in the field
	 * 
	 * @param msg
	 */
	CellRenderer.prototype._showError = function (msg, thrown) {
		this.$div.empty();
		this.$div.append($('<i>', {
			"class": "icon icon-WCR_Msg-Warning",
			"title": msg
		}));
		this._triggerLoadedCallbacks();
		if (thrown) {
			console.log("[ERROR] " + msg + " -- See error below.");
			console.log(thrown);
		}
	};

	/**
	 * Might this field contain a lot of data? 
	 * Or should we never hide part of the content?
	 * 
	 * ONLY call this when the component is loaded! See "whenLoaded".
	 */
	CellRenderer.prototype.getMightOverflow = function () {
		if (this.doneloading) {
			return !((this.returninfo || {})["overflowsnever"] || false);
		} else {
			throw new Error("getMightOverflow should only be called AFTER the field is loaded!");
		}
	};

	/**
	 * Does this field want specific css rules to be set on the column?
	 * 
	 * ONLY call this when the component is loaded! See "whenLoaded".
	 */
	CellRenderer.prototype.getParentCss = function () {
		if (this.doneloading) {
			return ((this.returninfo || {})["cellcss"] || {});
		} else {
			throw new Error("getParentCss should only be called AFTER the field is loaded!");
		}
	};

	// Return the constructor
	return CellRenderer;
});