define(["jquery",
		"underscore",
		"component/searchresults/model/SearchModelFrontEndData",
		"component/searchresults/model/SearchSetupInformation",
		"component/searchresults/internal/render/SearchTableRenderer",
		"utils/webi18n!searchinputs"
	],
	function ($,
		_,
		SearchModelFrontEndData,
		SearchSetupInformation,
		SearchTableRenderer,
		i18n) {

		"use strict";


		/**
		 * Table Renderer used to render search results of a drilldown into the parent search's table
		 * @class
		 * @augments SearchTableRenderer
		 * @param {Object} renderProperties
		 * @param {JQuery.element} row - the table row of the parent
		 * @param {parentExpandRow} parentExpandRow - the expandRow of the parent
		 * @param {SearchModelFrontEndData} searchData - Data which is passed from the server...
		 * @param {SearchSetupInformation} setupInfo - information about the search setup (columns to show, configured labels...)
		 */
		var ChildSearchRenderer = function (renderProperties, row, parentExpandRow, loadingIndicatorRow) {

			/** 
			 * @member {Object} renderProperties - used to show/hide on render
			 * @private
			 */
			this._renderProperties = renderProperties;

			/** 
			 * @member {Array} childtablerenders, used to hide them when clicking '-' icon
			 */
			this.children = [];

			/** 
			 * @member {Integer} level of indentation
			 */
			this.level = renderProperties.level;

			/**
			 * @member {JQueryElement} Expand row of the parent row, used to clone to add expand rows to the child rows 
			 */
			this.parentExpandRow = parentExpandRow;

			/**
			 * @member {JQueryElement} row that contains the loadingIndicator
			 */
			this.loadingIndicatorRow = loadingIndicatorRow;

			/**
			 * @member {JQueryElement} the end row of the renderer, all search results should be added before this row.
			 */
			this.endRow = $("<tr>");

			/**
			 * @member {JQueryElement} row that will contain the  dashboard if present
			 */
			this.dashboardRow = loadingIndicatorRow.clone().removeClass().show().addClass("drilldown-dashboard-row no-dashboard");


			this.dashboardRow.find("td").empty().append(this._setIndentationMargin($("<div class='inline-drilldown-dashboard'>")));

			// Add the appropriate classes to handle the open/closed behaviour
			addClassesToRows(this.level, [this.dashboardRow]);


			this.dashboardRow.hide();

			loadingIndicatorRow.after(this.endRow);
			this.loadingIndicatorRow.after(this.dashboardRow); // add dashboard row
			/**
			 * @member {JQueryElement} the 'load more' element
			 */
			this._pager = null;

			SearchTableRenderer.call(this, new SearchModelFrontEndData(), new SearchSetupInformation());

		};

		ChildSearchRenderer.prototype = Object.create(SearchTableRenderer.prototype);
		ChildSearchRenderer.prototype.constructor = ChildSearchRenderer;


		/**
		 * renders the rows into the parent table.
		 * @overrie
		 */
		ChildSearchRenderer.prototype.render = function (html, parameters) {
			this.div = html; // used for the _initialize method
			this._initialize();

			this.loadingIndicatorRow.hide();



			var rows = this._getRows(this.level, html); // filter the rows from the HTML

			if (!parameters || !parameters.doNotEmptyTable) {
				this.dashboardRow.nextUntil(this.endRow).remove(); // Remove all row that were previously added
			}
			if (this._pagingRow) { // We first remove the pagingRow (if it was there before the previous rerun)
				this._pagingRow.remove();
			}
			if (this._pager) { // Then add it again if it should still be there with the current rerun
				this._pagingRow = this.parentExpandRow.clone().show();
				this._pagingRow.find("td.contentcell").empty().append(this._pager);
				rows.push(this._pagingRow);
			}

			// Add the appropriate classes to handle the open/closed behaviour
			addClassesToRows(this.level, rows);

			// Insert the rows between the dashboard row and the endrow
			this.insertTableRows(rows);


			this._renderProperties.isRendered = true;

			_.each(rows, function (row) {
				if (!this._renderProperties.showOnRender) { // hide them if necessary
					row.hide();
				}

			}, this);

			// trigger change to make sure the 'select all' functionality is correct
			this.dashboardRow.nextUntil(this.endRow).find("input[name='selectproject']").first().trigger("change");

		};

		/**
		 * Adds a "no children found" row to the table
		 *
		 */
		ChildSearchRenderer.prototype.addNoChildrenRow = function () {
			this.dashboardRow.nextUntil(this.endRow).remove();
			var noChildrenFoundRow = this.parentExpandRow.clone().removeClass("expandablerow").addClass("no-children-row");
			addClassesToRows(this.level, [noChildrenFoundRow]);
			noChildrenFoundRow.find(".contentcell").empty().append(this._setIndentationMargin($("<div>")).text(i18n.t("NO_CHILDREN_FOUND")));
			this.dashboardRow.after(noChildrenFoundRow);
			noChildrenFoundRow.show();
			this.loadingIndicatorRow.hide();
			this._renderProperties.isRendered = true;

		};

		/**
		 * Gives the row some indentaion based on the level
		 * @param {JQueryElement} row
		 * @param {Integer} [mod = 5] - margin-left = (mod * level) "em"
		 */
		ChildSearchRenderer.prototype._setIndentationMargin = function (row, mod) {
			if (!mod) {
				mod = 5;
			}
			return row.css({
				"margin-left": (this.level * mod) + "em"
			});
		};

		/*
		 * Adds classes to the rows to handle the open/closed functionality
		 */
		function addClassesToRows(level, rows) {
			var classes = getClasses(level);
			_.each(rows, function (row) {
				_.each(classes, function (clazz) {
					$(row).addClass(clazz);
				});
				$(row).addClass("drilldown-indentation-level_open_" + level);
			}, this);

		}

		/**
		 * This method will insert rows before expandableRow.
		 * @param {Array.<$.Element>} rows - the rows that will be added 
		 */
		ChildSearchRenderer.prototype.insertTableRows = function (rows) {
			_.each(rows, function (row) {
				this.endRow.before(row); // add the row
				this.endRow.before(this.parentExpandRow.clone()); // add the expandrow for the 
			}, this);
			var entries = this._getEntries(rows);
			this._renderAllCells(entries, this);
		};

		/**
		 * Iterates over all search results and adds/removes columns that the parent does (not) have. 
		 * @param {Array.<String>} headers - columns in the parent search
		 * @param {Integer} level - indentation of the seach
		 * @param {String} html - the html returned by a saved search
		 */
		ChildSearchRenderer.prototype._getRows = function (level, html) {
			var $html = $(html);

			var rows = $html.find("> tbody > tr[data-rowdisplaydata*='{']");
			var self = this;
			var rowsWithIndentation = rows.each(function (i, elem) {
				var $row = $(elem);
				self._setIndentationMargin($row.find(".display-name").find("> table"));
				self._setIndentationMargin($row.find(".display-name.name-only").find(".drilldown"));
			});
			return rowsWithIndentation;
		};

		/*
		 * creates an array of classes needed to handle the open/close functionality of @param level 
		 */
		var getClasses = function (level) {
			var classes = [];
			_.times(level - 1, function (i) {
				classes.push("drilldown-indentation-level_open_" + (i + 1));
			});
			return classes;

		};

		/**
		 * Hides all the children of this search.
		 */
		ChildSearchRenderer.prototype.hideChildren = function () {
			cancelOngoingChildSearches(this); // Make sure child searches that are ongoing aren't shown when they are loaded

			var classes = getClasses(this.level);

			// All child rows should not display if any of the ancestor + icons are pressed
			var childRows = this.parentExpandRow.nextUntil(this.endRow);
			_.each(classes, function (clazz) {
				childRows.removeClass(clazz);
			});
			childRows.hide();
			// deselect all checkboxes
			childRows.find("input[name='selectproject']:checked").prop('checked', false);
			childRows.find("input[name='selectproject']").first().trigger("change");


		};

		/**
		 * shows all the open children
		 */
		ChildSearchRenderer.prototype.showChildren = function () {
			var classes = getClasses(this.level);
			var childRows = this.parentExpandRow.nextUntil(this.endRow);
			_.each(classes, function (clazz) {
				// All rows that are visible when opening my own level, should also be visible when opening an ancestor level
				childRows.filter(".drilldown-indentation-level_open_" + this.level).addClass(clazz);
			}, this);
			childRows.filter(".drilldown-indentation-level_open_" + this.level + ":not(.no-dashboard)").show();
			childRows.find("input[name='selectproject']").first().trigger("change");
		};

		/** 
		 * calling this function will set the showOnRender property of all child ChildSearchRenderer to false, so that they won't show when they are loaded.
		 */
		function cancelOngoingChildSearches(childTableRenderer) {
			if (childTableRenderer.table) {
				childTableRenderer.children.forEach(function (renderer) {
					if (!renderer.isRendered) {
						renderer.showOnRender = false;
					} else {
						cancelOngoingChildSearches(renderer);
					}
				});
			}
		}

		/**
		 * sets the html of a search rerun
		 */
		ChildSearchRenderer.prototype.setSearchHTML = function (html) {
			var pager = html.find(".searchresultspagination");
			if (pager.length > 0) {
				var curPage = Number(pager.attr("data-curpage"));
				var pageCount = Number(pager.attr("data-pageCount"));
				if (curPage < pageCount) {
					this._pager = $("<a class='load-more-results-pager searchresultspagelink'>").attr("data-gotopage", curPage + 1).text(i18n.t("LOAD_MORE_RESULTS"));
					this._setIndentationMargin(this._pager);
					pager.append(this._pager);
				} else {
					this._pager = null;
				}
			} else {
				this._pager = null;
			}
		};

		/** 
		 * set search data
		 */
		ChildSearchRenderer.prototype.setSearchData = function (searchData) {
			this._searchData = searchData;
		};

		/** 
		 * set search setup info
		 */
		ChildSearchRenderer.prototype.setSetupInfo = function (setupInfo) {
			this._setupInfo = setupInfo;
		};


		return ChildSearchRenderer;



	});
