/*
 * Controls a single parameter field...
 */
define(["jquery", 
        "underscore",
        
        // Used to load the parameter field implementation
        "./ParameterFieldLoader",
        
        // Search Model
        "../../model/SearchSetupInformation",
        "../../model/SearchCriterionInfo",
        "../../model/SearchModelFrontEndData",
        
        // Data objects
        "../../dataobjects/ValueData",
        "../../dataobjects/SearchSettings",
        "../../dataobjects/DataProvider",
        "../../dataobjects/ValidationFeedback",
        
        // The ExpandMode to pass in the FieldInformation object
        "../expandmodes/ExpandModeModal"
        ], function($, _,
        		ParameterFieldLoader,
        		SearchSetupInformation,
        		SearchCriterionInfo,
        		SearchModelFrontEndData,
        		ValueData,
        		SearchSettings,
        		DataProvider,
        		ValidationFeedback,
        		ExpandModeModal
        ){
	
	/*
	 * Controls a single parameter field...
	 * 
	 * Generic Object... 
	 * For actual implementations of different fields, see implementations/parameterfields folder.
	 * 
	 * Uses the ParameterFieldLoader to load the implementations...
	 * 
	 * This controller is responsible for 
	 *  - keeping the value in the component up to date
	 *  - and inverse... update the value in the object model for search
	 *  - call individual methods of the field implementation
	 * 
	 * Links the parameter field to the data in the model...
	 * 
	 * You can listen to when a rerun should take place...
	 */
	
	/**
	 * @constructor
	 * 
	 * @param {String} fieldid - the id of the field for which we are making a parameter field
	 * @param {DOMElement} container - the container to put the field in
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (which values are set?)
	 * @param {SearchModelFrontEndData} searchData - information about the search data send from the server
	 * @param {SearchSettings} searchSettings - the settings of the search...
	 * @param {Object} overrulefactory - [Optional] a different factory to use to create the ParameterField
	 */
	var ParameterField = function(fieldid, container, setupInfo, searchData, searchSettings, overruleFactory) {
		// Check constructor arguments
		if(!(	fieldid && (typeof fieldid === "string") && 
				container && 1 === container.nodeType /* check if it is a HTML Element */ && 
				setupInfo && setupInfo instanceof SearchSetupInformation &&
				searchData && searchData instanceof SearchModelFrontEndData &&
				searchSettings && searchSettings instanceof SearchSettings)) {
			throw new Error("Illegal arguments passed to SearchResultsDesignPanelFieldController");
		}
		
		// Make two divs... (one for the component, one for error messages)
		this.container = $(container);
		$(container).wrapInner($("<div>"));
		this.componentDiv = $("> div", $(container));
		this.feedbackDiv = $("<div>");
		$(container).append(this.feedbackDiv);
		
		
		// Store the setup info & search data objects...
		this._setupInfo = setupInfo;
		this._searchData = searchData;
		
		// Look up the field and see what kind of param field we need to use...
		this._searchfield = this._searchData.getFieldInformation(fieldid, this._setupInfo);
		var fieldeditmethod = this._searchfield?this._searchfield["parameterFieldType"]:"";
			
		// Make a ExpandMode instance
		var expandMode = new ExpandModeModal(this._setupInfo.getDesignPanelItems().getCustomizedLabel(this._searchfield));
		
		// Look up the field properties...
		this._fieldProperties = this._searchData.makeFieldDataForField(this._searchfield, expandMode, this._setupInfo.getLayoutOptions());

		// Object so the field can get additional data
		var dataProvider = new DataProvider(null, null);// TODO!

		// Validation Feedback
		this._feedback = new ValidationFeedback();
		
		//
		// Initial settings
		//
		this._loading = true;
		this._ignoreCriterionChanges = false;
		this._ignoreFieldChange = false;
		
		//
		// Callbacks
		//
		this._doneLoadingCallbacks = [];
		this._requiresRerunCallbacks = [];
		this._startsUsingSpecialCharsCallbacks = [];
		
		
		//
		// Start loading the implementation of the field...
		//
		this.fieldloader = new ParameterFieldLoader(this.componentDiv.get(0), fieldeditmethod, fieldid, searchSettings, dataProvider, this._fieldProperties, overruleFactory);
		
		// Set the initial value of the field
		this.fieldloader.withField(_.bind(function(fieldcomponent) {
			// DONE LOADING...
			
			// Store the component
			this.fieldcomponent = fieldcomponent;
			
			// Link the field to the object model and listen to it and work with it...
			if(!this.fieldloader.inErrorState()) {
				this._initializeField(fieldid, fieldcomponent);
			}
			
			// Event, done loading
			this._triggerDoneLoading();
		}, this));
	};
	
	/**
	 * Get the fieldId of this ParameterField.
	 */
	ParameterField.prototype.getFieldId = function() {
		return this._searchfield.fieldId;
	};
	
	//
	// Internal logic...
	//
	
	/**
	 * Initializes the implementation when it is loaded...
	 */
	ParameterField.prototype._initializeField = function(fieldId){
		// Listen to the field implementation...
		this.fieldcomponent.onChange(_.bind(this._onValueUpdate, this));
		
		//
		// Look up the criterion which is being edited...
		//
		var emptyCriterion = false;
		
		var fieldIdForCriterion = fieldId;
		if (this._searchfield.instancingType === "MULTI_FIELD_SINGLE_SELECTION"){
			fieldIdForCriterion = this._searchfield.instancingSource;
		}
		this._criterion = this._setupInfo.getCriteriaList().findByFieldId(fieldIdForCriterion);
		if(!this._criterion) {
			// If there is no such criterion... make an empty one
			this._criterion = new SearchCriterionInfo(fieldIdForCriterion);
			this._setupInfo.getCriteriaList().add(this._criterion);
			emptyCriterion = true;
		}
		
		// NOTE: We will keep a reference to this criterion, we need to call "take()"!
		this._criterion.take();
		
		// When the criterion changes because of some external effect, we will update our value...
		this._onChangeFunction = _.bind(function(){
			if(!this._ignoreCriterionChanges) {
				// Change in the criterion was caused by some external component...
				// Update value in the field.
				
				// But ignore change events thrown by the field...
				this._ignoreFieldChange = true;
				var validValueSet = this.fieldcomponent.setValueData(this._criterion.getValue());
				this._ignoreFieldChange = false;
				
				if(!validValueSet) {
					// The external component put an illegal value in the criterion...
					// Update the value in the criterion... (and override the value of the other component)
					this._onValueUpdate();
				}
			}
		}, this);
		this._criterion.onChange(this._onChangeFunction);
		
		
		var validValueSet = true;
		if(!emptyCriterion) {
			// Set the initial value (only if we have a valid value to set...)
			this._ignoreFieldChange = true;
			validValueSet = this.fieldcomponent.setValueData(this._criterion.getValue());
			this._ignoreFieldChange = false;
		}
		
		
		// Special characters in initial value?
		// Test if the initial value contains special characters...
		this._currentHasSpecialCharacters = this.checkSpecialCharacters();
		if(this._currentHasSpecialCharacters){
			this._triggerStartsUsingSpecialCharacters(true);
		}
		
		if(!validValueSet) {
			// The value which is saved is not valid for this field... (Cannot be represented)
			// Override the value in the criterion...
			this._onValueUpdate(false);
		}
		
		if(emptyCriterion) {
			// Fill in the empty value in the model...
			// (Get the value from the component...)
			this._onValueUpdate(true);
		}
		
		if (this._fieldProperties.facetIds.length > 0){
			this._searchData.observeAdditionalFieldData(fieldId, this.fieldcomponent.doFacetUpdate.bind(this.fieldcomponent));
			this.fieldcomponent.doFacetUpdate(this._searchData.getAdditionalFieldData(fieldId));
		}
	};
	
	/**
	 * (private) 
	 * Only to be called after the field it loaded.
	 * 
	 * Gets the value from the field & updates the feedback...
	 * 
	 * @returns ValueData value
	 */
	ParameterField.prototype._extractValue = function() {
		var retVal = new ValueData();
		
		if (this._searchfield.instancingType === "MULTI_FIELD_SINGLE_SELECTION"){
			// there is a shared selection, the parameter field should work on this shared selection data
			retVal = this._criterion.getValue();	
		}
		
		this._feedback.clear();
		this.fieldcomponent.getValueData(retVal, this._feedback);
		this._updateFeedback();
		return retVal;
	};
	
	/**
	 * Update the feedback...
	 * 
	 * Display the errors below the field...
	 */
	ParameterField.prototype._updateFeedback = function() {
		this.feedbackDiv.empty();
		$.each(this._feedback.getErrors(), function(i, err) {
			this.feedbackDiv.append($("<div>", {text:err}));
		});
	};
	
	/**
	 * Validate the Parameter field input
	 * @returns
	 */
	ParameterField.prototype.validate = function() {
		if(!this._loading) {
			return this.fieldcomponent.validate();
		} else {
			return true;
		}
	};
	
	/**
	 * Clear the ParameterField.
	 */
	ParameterField.prototype.clearField = function() {
		if(!this._loading) {
			this.fieldcomponent.clear();
		} else {
			// This component is still loading... Cannot clear this input...
		}
	};
	
	/**
	 * Trigger a resize on the ParameterField.
	 */
	ParameterField.prototype.resize = function() {
		if(!this._loading) {
			this.fieldcomponent.resize();
		} else {
			// This component is still loading... Cannot resize this input...
		}
	};
	
	ParameterField.prototype.focus = function() {
		if(!this._loading) {
			this.fieldcomponent.focus();
		} else {
			// This component is still loading... Cannot focus this input...
		}
	};
	
	/**
	 * Called for every little value change (like typing a letter)
	 * 
	 * @param {Boolean} intermediateUpdate - whether the change is final and should cause a rerun.
	 */
	ParameterField.prototype._onValueUpdate = function(intermediateUpdate){
		if(this._ignoreCriterionChanges || this._ignoreFieldChange) {
			// We are already causing a criterion change (avoid recursive loops)
			// Or, we are changing the value in the field ourself and don't want the change event
			return;
		}
		
		this._ignoreCriterionChanges = true;
		this._criterion.setValue(this._extractValue(), this.fieldcomponent.valueContainsSmartName());
		this._criterion.commitChanges();
		this._ignoreCriterionChanges = false;
		
		// Special characters?
		if(!this._currentHasSpecialCharacters) {
			// We were not using special characters in this field yet...
			this._currentHasSpecialCharacters = this.checkSpecialCharacters();
			if(this._currentHasSpecialCharacters){
				// But we are now...
				this._triggerStartsUsingSpecialCharacters(false);
			}
		}
		
		this._triggerRequiresRerun(intermediateUpdate);
	};
	
	
	
	//
	// Listen to when this component is fully loaded...
	//
	
	/**
	 * Will be called when this field is loaded...
	 * 
	 * @param {Function} callback - callback which will be called when this field is loaded...
	 */
	ParameterField.prototype.onDoneLoading = function(callback){
		if(!this._loading) {
			callback();
		}else{
			this._doneLoadingCallbacks.push(callback);
		}
	};
	
	/**
	 * Internal - trigger the "DoneLoading" event.
	 */
	ParameterField.prototype._triggerDoneLoading = function(){
		if(this._loading) {
			this._loading = false;
			for(var i=0;i<this._doneLoadingCallbacks.length;i++) {
				this._doneLoadingCallbacks[i]();
			}
			this._doneLoadingCallbacks = [];
		}
	};
	
	/**
	 * Is this ParameterField loaded?
	 * @returns {Boolean}
	 */
	ParameterField.prototype.isLoaded = function(){
		return !this._loading;
	};
	
	//
	// Listen to when this fields needs a rerun
	//
	
	/**
	 * Will be called when this field requires a rerun
	 * 
	 * @param {Function} callback - callback which will be called when this field requires a rerun... Has one parameter => intermediateUpdate - true when the field is still changing.
	 */
	ParameterField.prototype.onRequiresRerun = function(callback) {
		this._requiresRerunCallbacks.push(callback);
	};
	
	/**
	 * Internal - trigger the "RequiresRerun" event.
	 * 
	 * @param intermediateUpdate - true if the field is still changing.
	 */
	ParameterField.prototype._triggerRequiresRerun = function(intermediateUpdate){
		for(var i=0;i<this._requiresRerunCallbacks.length;i++) {
			this._requiresRerunCallbacks[i](intermediateUpdate);
		}
	};
	
	//
	// Listen to when the user types special characters in this field for the first time...
	// 
	
	/**
	 * A list of special characters
	 */
	ParameterField.prototype._specialCharacters = new Array('&', '|', '(', ')', '~', '\\');
	
	ParameterField.prototype.checkSpecialCharacters = function(callback){
		// Ask the field and provide the function to check
		return this.fieldcomponent.valueContainsSpecialSearchCharacters(this._specialCharacters);
	};
	
	/**
	 * Will be called when this field starts using special characters for the first time
	 * 
	 * The callback has one parameter: fromInitialValue
	 * It will be true when the callback is called because the initial value already contains special characters...
	 * If false, the user typed the special characters later on...
	 * 
	 * @param {Function} callback
	 */
	ParameterField.prototype.onStartsUsingSpecialCharacters = function(callback){
		this._startsUsingSpecialCharsCallbacks.push(callback);
	};
	
	/**
	 * Internal - trigger the "StartsUsingSpecialCharacters" event.
	 * 
	 * @param {Boolean} fromInitialValue - whether the field started using special characters from it's initial value
	 *                                     or when the value is modified by the user...
	 */
	ParameterField.prototype._triggerStartsUsingSpecialCharacters = function(fromInitialValue){
		for(var i=0;i<this._startsUsingSpecialCharsCallbacks.length;i++) {
			this._startsUsingSpecialCharsCallbacks[i](fromInitialValue);
		}
		this._startsUsingSpecialCharsCallbacks = [];
	};
	
	ParameterField.prototype.destroy = function() {
		this.fieldloader.destroy();
		
		if(this.fieldloader.isLoaded()) {
			this._criterion.removeOnChange(this._onChangeFunction);
			
			// NOTE: We are no longer keeping a reference to this criterion, we need to call release()
			this._criterion.release();
			this._criterion = null;
		}
		
		this._doneLoadingCallbacks = [];
		this._requiresRerunCallbacks = [];
		this._startsUsingSpecialCharsCallbacks = [];
		this.container.empty();
	};
	
	// Return the constructor
	return ParameterField;
});