/*
 * ParameterFieldRenderer: Renders a parameter field in the Parameter Panel.
 * 
 * Looks for implementations in a certain folder...
 */
define([
    "underscore",
    "./ParameterFieldWrapper",

    // For checking if we get the correct arguments in the constructor
    "../../dataobjects/DataProvider",
    "../../dataobjects/SearchSettings",
    "../../dataobjects/FieldProperties",

    // All the implemenations
    "../../implementations/parameterfields/all"
], function (_,
    ParameterFieldWrapper,

    DataProvider,
    SearchSettings,
    FieldProperties,
    parameterfieldfactories
) {
    "use strict";
    /**
     * Constructor
     * 
     * @param {DOMElement} div - the div to render in
     * @param {String} fieldEditMethod - the way to edit this value...
     * @param {String} fieldid - the id of the field to display
     * @param {SearchSettings} settings - some settings which are enabled for the whole search page.
     * @param {DataProvider} data - if the field requires some data to render itself, it can ask this interface.
     * @param {FieldProperties} fieldProperties - additional information about the field
     * @param {Object} overruleFactory - [Optional] a different factory to use to create the ParameterField
     */
    var ParameterFieldLoader = function (div, fieldEditMethod, fieldid, settings, data, fieldProperties, overruleFactory) {
        if (!(div && 1 === div.nodeType /* check if it is a HTML Element */ &&
                (typeof fieldEditMethod === "string") &&
                fieldid && (typeof fieldid === "string") &&
                settings && settings instanceof SearchSettings &&
                data && data instanceof DataProvider &&
                fieldProperties && fieldProperties instanceof FieldProperties)) {
            throw new Error("Illegal arguments passed to ParameterFieldLoader");
        }

        // Store div
        this.$div = $(div);

        // Some variables...
        this.fieldid = fieldid;
        this.settings = settings;
        this.data = data;
        this.fieldProperties = fieldProperties;

        // The implementations
        this.fieldfactory = null;
        this._fieldcomponentimpl = null;
        this.fieldcomponent = null;

        // Loading information
        this.doneloading = false;
        this.whenloadedcallbacks = [];
        this._errorstate = false;

        // Start loading...
        if (fieldEditMethod) {
            // Load ParameterField
            if (parameterfieldfactories.hasOwnProperty(fieldEditMethod)) {
                var mod = parameterfieldfactories[fieldEditMethod];
                if (overruleFactory) {
                    mod = overruleFactory;
                    fieldEditMethod = "???";
                }
                // Ok, loaded the render
                if (!_.isObject(mod) /* check if we get an object back... */ ) {
                    // Module did not return an object !???
                    this.fieldfactory = null;
                    this._showError("\"ParameterField_" + fieldEditMethod + "\" module does not return an object.");
                } else {
                    // Module returned correctly
                    this.fieldfactory = mod;

                    // Check if the factory has the correct methods...
                    if (!_.isFunction(this.fieldfactory.make)) {
                        this.fieldfactory = null;
                        this._showError("Edit Method does not provide make");
                    }
                }
            } else {
                // The parameterfield was not added to the parameterfields/all file...
                this.fieldfactory = null;
                this._showError("Cannot find parameter field type \"" + fieldEditMethod + "\".");
            }
        } else {
            // "fieldEditMethod" should never be null...
            // Must mean someone wrongly configured the available search fields... !?
            this.fieldfactory = null;
            this._showError("There is no way to edit this field...");
        }
        this._initialize();
    };

    //
    // Public methods...
    //

    /**
     * Get the field id
     */
    ParameterFieldLoader.prototype.getFieldId = function () {
        return this.fieldid;
    };

    /**
     * Check if the field is loaded...
     * 
     * @returns {Boolean} whether the field is loaded...
     */
    ParameterFieldLoader.prototype.isLoaded = function () {
        return this.doneloading;
    };

    /**
     * Check if the field is loaded correctly...
     * 
     * @returns {Boolean} whether the field could not be loaded correctly...
     */
    ParameterFieldLoader.prototype.inErrorState = function () {
        return this._errorstate;
    };

    /**
     * To execute something with the field... 
     * 
     * Note: will be executed with delay if the component is not loaded yet.
     * 
     * @param {Function} callback - callback has one argument: the parameter field...
     */
    ParameterFieldLoader.prototype.withField = function (callback) {
        if (!this.doneloading) {
            this.whenloadedcallbacks.push(callback);
        } else {
            callback(this.fieldcomponent);
        }
    };

    //
    // Private methods
    //

    /**
     * Make the parameter field component
     */
    ParameterFieldLoader.prototype._initialize = function () {
        if (this.doneloading || this.$div === null) {
            return; // We already initialized... !? This should never happen...
        }

        this._fieldcomponentimpl = null;

        if (this.fieldfactory) {
            // We have a fieldfactory, so no errors when loading... 
            this.$div.empty();

            // Make the component...
            try {
                this._fieldcomponentimpl = this.fieldfactory.make(this.$div.get(0), this.fieldid, this.settings, this.data, this.fieldProperties);
            } catch (e) {
                this._showError("Error thrown by implementation. See console.", e);
                console.log("Error thrown by implementation, logged below.");
                console.log(e);
            }
        }

        // Finally, make the wrapper component which handles error handling for the object... 
        // (error handling for the factory is done by this class)
        // 
        this.fieldcomponent = new ParameterFieldWrapper(this.fieldid, this._fieldcomponentimpl);

        // Done loading... Notify listeners...
        this.doneloading = true;
        for (var i = 0; i < this.whenloadedcallbacks.length; i++) {
            this.whenloadedcallbacks[i](this.fieldcomponent);
        }
        this.whenloadedcallbacks = [];
    };

    /**
     * Cannot load the component... Show an error message in the field
     * 
     * @param msg
     */
    ParameterFieldLoader.prototype._showError = function (msg) {
        this._errorstate = true;
        this.$div.empty();
        this.$div.append($('<i>', {
            "class": "icon icon-WCR_Msg-Warning",
            "title": msg
        }));
    };

    /**
     * Destroy method
     */
    ParameterFieldLoader.prototype.destroy = function () {
        if (this.doneloading && this.fieldfactory) {
            this.fieldfactory.destroy(this._fieldcomponentimpl);
        }

        this.$div = null;
        this._fieldcomponentimpl = null;
        this.fieldcomponent = null;
        this.fieldfactory = null;
    };

    // Return the constructor
    return ParameterFieldLoader;
});
