/**
 * Wrapper around a field in the Design Panel
 * 
 * The main task of this component is to implement error handling...
 */
define([
        "jquery",
        "underscore", 
        "../../dataobjects/ValueData",
        "../../dataobjects/ValueDataUtils",
        "../../dataobjects/ValidationFeedback"
        ], function($, _,
        		ValueData, 
        		ValueDataUtils,
        		ValidationFeedback
        ) {

    /**
     * @constructor
     * @param {String} fieldId - the fieldId
     * @param {Object} parameterfield - the parameterfield implementation... CAN BE NULL.
     */
    var ParameterFieldWrapper = function(fieldId, parameterfield) {
    	if(!(fieldId && typeof(fieldId) === "string")){
    		throw new Error("Incorrect parameter passed to ParameterFieldWrapper.");
    	}
    	
    	this.fieldId = fieldId;
    	this.parameterfield = parameterfield;
    	
    	// Used in case of a wrong parameterfield only
    	this.value = new ValueData();
    };
    
    //
    // Set & Get the value
    //

    /**
     * Get the value by filling the value object.
     * Also do the validation in the mean time and
     * use the feedback object to give the information back.
     * 
     * @param {ValueData} value - the object to put all the value information in
     * @param {ValidationFeedback} feedback - the object 
     */
    ParameterFieldWrapper.prototype.getValueData = function (value, feedback) {
    	if(!(value && value instanceof ValueData && feedback && feedback instanceof ValidationFeedback)){
    		throw new Error("Wrong information passed to ParameterFieldWrapper.getValueData...");
    	}
    	
    	this._safeCallMethod(
			"getValueData", 
			"getValueData(ValueData,ValidationFeedback):void",
			
			function(){
				// Normal call...
				this.parameterfield.getValueData(value, feedback);
			},
			
			function(){
				// Fallback...
				// Fallback implementation just copies the values which were set previously...
				ValueDataUtils.fromJSON(this.value, 
					ValueDataUtils.toJSON(this.value));
			}
    	);
    };

    
    /**
     * Set the value, update the field to represent this value.
     * 
     * @param {ValueData} value - the object which contains all the information about the value
     * @returns {Boolean} true if the component can display the criterion set in the value
     */
    ParameterFieldWrapper.prototype.setValueData = function (value) {
    	if(!(value && value instanceof ValueData)){
    		throw new Error("Wrong information passed to ParameterFieldWrapper.setValueData...");
    	}
    	
    	var retval = this._safeCallMethod(
    		"setValueData",
    		"setValueData(ValueData):void",
    		
    		function(){
    			// Normal call...
    			return this.parameterfield.setValueData(value);
    		}, 
    		
    		function(){
    			// Fallback...
    			this.value = value;
    			return true;
    		},
    		
    		// Value in case of error... 
    		// Don't rerun...
    		true
    	);
    	
    	if(retval === true || retval === false) {
    		return retval;
    	} else {
    		this._warnAboutError("setValueData(value:ValueData):bool", "Method should return true or false...");
    		return true;
    	}
    };

    /**
     * Clear the Field (called by "Clear this form" functionality) 
     */
    ParameterFieldWrapper.prototype.clear = function () {
    	this._safeCallMethod(
			"clear", 
			"clear():void",
			
			function(){
				// Normal call...
				this.parameterfield.clear();
			},
			
			function(){
				// Fallback...
			}
    	);
    };

    /**
     * Trigger a resize on the Field.
     * 
     * Fields do not have to implement this method and it might not be called!
     */
    ParameterFieldWrapper.prototype.resize = function () {
    	if(this._checkForMethod("resize")) {
    		// Ok, the implementation has a method
    		try{
    			return this.parameterfield.resize();
    		}catch(e){
    			// An error occured...
    			this._warnAboutError("resize", e);
    		}
    	}
    };
    
    /**
     * Set the focus on this field.
     */
    ParameterFieldWrapper.prototype.focus = function (){
    	if(this._checkForMethod("focus")) {
    		// Ok, the implementation has a method
    		try{
    			this.parameterfield.focus();
    		}catch(e){
    			// An error occured...
    			this._warnAboutError("focus", e);
    		}
    	} else {
    		console.log("WARNING: focus() not implemented in ParameterField for \""+this.fieldId+"\".");
    	}
    };

    /**
     * Does this field contain a SmartName?
     * 
     * @returns {bool} true when this field contains a SmartName
     */
    ParameterFieldWrapper.prototype.valueContainsSmartName = function() {
    	var retval = this._safeCallMethod(
    		"valueContainsSmartName",
    		"valueContainsSmartName():bool",
    		
    		function(){
    			// Normal call...
    			return this.parameterfield.valueContainsSmartName();
    		}, 
    		
    		function(){
    			// Fallback...
    			return true;
    		}, 
    		
    		// Value in case of error... 
    		// True... because we don't know if it contains SmartNames... (so assume it does)
    		true
    	);
    	
    	if(retval === true || retval === false) {
    		return retval;
    	} else {
    		this._warnAboutError("valueContainsSmartName():bool", "Method should return true or false...");
    		return true;
    	}
    };
    
    /**
     * Does this field contain a special search character?
     * 
     * @param specialCharacters the array of all special characters...
     */
    ParameterFieldWrapper.prototype.valueContainsSpecialSearchCharacters = function(specialCharacters) {
    	var retval = this._safeCallMethod(
    		"valueContainsSpecialSearchCharacters",
    		"valueContainsSpecialSearchCharacters():bool",
    		
    		function(){
    			// Normal call...
    			return this.parameterfield.valueContainsSpecialSearchCharacters(specialCharacters);
    		}, 
    		
    		function(){
    			// Fallback...
    			return false;
    		}, 
    		
    		// Value in case of error... 
    		// False... because we don't know if it contains special characters... (so assume it does not)
    		false
    	);
    	
    	if(retval === true || retval === false) {
    		return retval;
    	} else {
    		this._warnAboutError("valueContainsSpecialSearchCharacters():bool", "Method should return true or false...");
    		return false;
    	}
    };
    
    //
    // Change listeners
    //
    
    /**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes
     */
    ParameterFieldWrapper.prototype.onChange = function(callback) {
    	this._safeCallMethod(
    		"onChange",
    		"onChange(Function):void",
    		
    		function(){
    			// Normal call...
    			this.parameterfield.onChange(callback);
    		}, 
    		
    		function(){
    			// Fallback...
    		}
    	);
    };

    /**
     * Registers a validate function if this parameter field has validate method
     * 
     * @returns {Boolean}
     */
    ParameterFieldWrapper.prototype.validate = function() {
		if(this._checkForMethod("validate")) {
    		// Ok, the implementation has a method
    		try{
    			return this.parameterfield.validate();
    		}catch(e){
    			// An error occured...
    			this._warnAboutError("validate", e);
    		}
    	}
		// No validation method
		return true;
    };
    
    
	/**
     * Registers a new function to call when the field changes...
     * 
     * @param {Function} callback - a function to call when the field changes
     */
    ParameterFieldWrapper.prototype.doFacetUpdate = function(data) {
		if(this._checkForMethod("doFacetUpdate")) {
    		// Ok, the implementation has a method
    		try{
    			return this.parameterfield.doFacetUpdate(data);
    		}catch(e){
    			// An error occured...
    			this._warnAboutError("doFacetUpdate", e);
    		}
    	}
    };
    
    // 
    // Error Handling util methods
    //
    
    /**
     * Check if a method exists...
     * 
     * @param {String} methodName - the name of the method...
     * @returns {Boolean}
     */
    ParameterFieldWrapper.prototype._checkForMethod = function(methodName) {
    	return (
    		// We have a parameter field (could be that we did not find the implementation)
    		this.parameterfield && 
    		// the parameter field has the right property
    		this.parameterfield[methodName]!==undefined && 
    		// the right property is a function...
    		typeof(this.parameterfield[methodName]) === "function");
    };
    
    /**
     * Warn the user about missing methods...
     * 
     * @param {String} methodDescription - the name of the method + arguments...
     */
    ParameterFieldWrapper.prototype._warnAboutMissingMethod = function(methodDescription) {
    	if(this.parameterfield) {
    		console.log("[WARNING]: Missing method \""+methodDescription+"\" for the Design Panel Component for field \""+this.fieldId+"\".");
    		if(!window["warnedAboutParameterPanelMissingMethod"]) {
    			window["warnedAboutParameterPanelMissingMethod"] = true;
    			alert("Warning: At least one Design Panel Component is missing functionality... Some Search Fields might not work as expected... See Developer Console for more info.");
    		}
    	}
    };
    
    /**
     * Warn the user about missing methods...
     * 
     * @param {String} methodDescription - the name of the method + arguments...
     * @param {
     */
    ParameterFieldWrapper.prototype._warnAboutError = function(methodDescription, e) {
    	if(this.parameterfield) {
    		console.log("[ERROR]: Error occured in method \""+methodDescription+"\" for the Design Panel Component for field \""+this.fieldId+"\", logged below.");
    		console.log(e);
    		
    		if(!window["warnedAboutParameterPanelErrorInMethod"]) {
    			window["warnedAboutParameterPanelErrorInMethod"] = true;
    			alert("Error: An error occured in at least one Design Panel Component... Some Search Fields might not work as expected... See Developer Console for more info.");
    		}
    	}
    };
    
    /**
     * Tries to call a method in the parameter field.
     * 
     * Checks first if the method exists and provides a fallback if it does not...
     * 
     * @param {String} methodName - the name of the method...
     * @param {String} methodDescription - the name of the method + arguments...
     * @param {Function} execute - callback, execute the call to the method...
     * @param {Function} defaultimpl - callback, will be called if there is no method...
     * @param {Object} defaultret - default return value, used if the method exists but throws an exception
     */
    ParameterFieldWrapper.prototype._safeCallMethod = function(methodName, methodDescription, execute, defaultimpl, defaultret) {
    	if(this._checkForMethod(methodName)) {
    		// Ok, the implementation has a method
    		try{
    			return _.bind(execute, this)();
    		}catch(e){
    			// An error occured...
    			this._warnAboutError(methodDescription, e);
    			return defaultret;
    		}
    	} else {
    		// Fallback implementation
    		this._warnAboutMissingMethod(methodDescription);
    		return _.bind(defaultimpl, this)();
    	}
    };
    
    // Note...
    // No factory for the wrapper...
    return ParameterFieldWrapper;
});