/*
 * SearchLayoutLoader: Renders a parameter field in the Parameter Panel.
 * 
 * Looks for implementations in a certain folder...
 */
define([
        	"../FieldLayoutAPI",

            // Pre-load all the implemenations (will be bundled when optimized)
            "../../implementations/parameterfields/all"
        ], function(
        		FieldLayoutAPI
        ){
	/**
	 * Constructor
	 * 
	 * @param {DOMElement} div - the div to render in
	 * @param {String} layoutMethod - the way to layout the design panel components in this div...
	 * @param {FieldLayoutAPI} api - the api to use to layout the design panel components...
	 * @param {Object} additionalData - some additional data which can be passed to the layout.
	 */
	var SearchLayoutLoader = function(div, layoutMethod, api, additionalData) {
		if(!(	div && 1 === div.nodeType /* check if it is a HTML Element */ && 
				layoutMethod && (typeof layoutMethod === "string") && 
				api && api instanceof FieldLayoutAPI)) {
			throw new Error("Illegal arguments passed to SearchSetupLayout");
		}
		
		// Store div
		this.$div = $(div);
		
		// Some variables...
		this.api = api;
		this._additionalData = additionalData;
		
		// The implementation
		this._layoutimpl = null;
		
		// Loading information
		this.doneloading = false;
		this.whenloadedcallbacks = [];
		
		// Start loading...
		if(layoutMethod) {
			// Load Renderer
			require(["component/searchresults/implementations/searchsetuplayout/SearchSetupLayout_"+layoutMethod], 
					_.bind(function(mod){
				// Ok, loaded the layout implementation
				this._layoutimpl = mod;
				
				// Check if the factory has the correct methods...
				if(!(this._layoutimpl.hasOwnProperty("layout") && typeof(this._layoutimpl["layout"])==="function")) {
					this._layoutimpl = null;
					this._showError("Edit Method does not provide layout");
				}
				this._initialize();
			}, this), 
			_.bind(function(err) {
				// Oh, that file does not exist OR contains errors...
				// Show error...
				this._layoutimpl = null;
				if(err.requireType==="scripterror") {
					this._showError("Missing \"SearchSetupLayout_"+layoutMethod+".js\" file.", err);
				}else{
					this._showError("\"SearchSetupLayout_"+layoutMethod+"\" contains an error.", err);
				}
				this._initialize();
			}, this));
		} else {
			// "layoutMethod" should never be null...
			this._layoutimpl = null;
			this._showError("Null layout...");
			this._initialize();
		}
	};
	
	//
	// Public methods...
	//
	
	/**
	 * Check if the field is loaded...
	 * 
	 * @returns {Boolean} whether the field is loaded...
	 */
	SearchLayoutLoader.prototype.isLoaded = function(){
		return this.doneloading;
	};
	
	/**
	 * Function to be called when the layout is loaded...
	 * 
	 * @param {Function} callback - called when layout is loaded...
	 */
	SearchLayoutLoader.prototype.whenLoaded = function(callback) {
		this.whenloadedcallbacks.push(callback);
	};
	
	//
	// Private methods
	//
	
	/**
	 * Make the parameter field component
	 */
	SearchLayoutLoader.prototype._initialize = function() {
		if(this.doneloading){
			return;// We already initialized... !? This should never happen...
		}
		
		if(this._layoutimpl){
			this._layoutimpl.layout(this.$div.get(0), this.api, this._additionalData);
		}
		
		// Done loading... Notify listeners...
		this.doneloading = true;
		for(var i=0;i<this.whenloadedcallbacks.length;i++) {
			this.whenloadedcallbacks[i](this.fieldcomponent);
		}
		this.whenloadedcallbacks = [];
	};
	
	/**
	 * Cannot load the component... Show an error message in the field
	 * 
	 * @param msg
	 */
	SearchLayoutLoader.prototype._showError = function(msg, thrown) {
		
		if(thrown){
			console.log("[ERROR] "+msg+" -- See error below.");
			console.log(thrown);
		}else{
			console.log("[ERROR] "+msg);
		}
	};
	
	// Return the constructor
	return SearchLayoutLoader;
});