/**
 * ## A Search Results Table - uses renderers to display the contents & fields to load information about the columns... ##
 */
define([
	"utils/idUtils",
	"./CellRenderer",
	"component/contentoverflow/OverflowContentContainer",
	"../../dataobjects/TableDisplaySettings",
	"../../dataobjects/ValueData",
	"../../dataobjects/ValueDataUtils",
	"../../model/SearchModelFrontEndData",
	"../../model/SearchSetupInformation",
	"../expandmodes/ExpandModeListViewRow",
	"../expandmodes/ExpandModeModal",
	"component/loadingindicator/LoadingIndicator",

	"underscore"
], function (
	// Some util to generate new ids (used for Calendar View)
	idUtils,

	// Classes to actually display the results...
	CellRenderer,
	OverflowContentContainer,
	TableDisplaySettings,

	// Classes to represent result data
	ValueData,
	ValueDataUtils,

	// Get access to additional information passed from the server (only used for type checking)
	SearchModelFrontEndData,
	SearchSetupInformation,

	// Ways to show additional content outside of the table...
	ExpandModeListViewRow,
	ExpandModeModal,
	LoadingIndicator,

	_
) {

	/**
	 * Creates a new SearchTableRenderer.
	 * 
	 * A SearchTableRenderer is only responsible for rendering the table. (NOT for pagination, changing style, ...)
	 * 
	 * @param {SearchSetupInformation} setupInfo - information about the search setup (columns to show, configured labels...)
	 * @param {SearchModelFrontEndData} searchData - Data which is passed from the server...
	 */
	var SearchTableRenderer = function (searchData, setupInfo) {
		// Check input
		if (!(searchData && searchData instanceof SearchModelFrontEndData)) {
			throw new Error("Illegal argument passed: searchData");
		}
		if (!(setupInfo && setupInfo instanceof SearchSetupInformation)) {
			throw new Error("Illegal argument passed: setupInfo");
		}

		this._searchData = searchData;
		this._setupInfo = setupInfo;
	};


	/**
	 * Render the table.
	 * 
	 * @param {jQuery} div - the div in which to render the table.
	 */
	SearchTableRenderer.prototype.render = function (div) {
		this.div = div;
		this._initialize();
		var rows = $(".searchresultsrow", this.div);
		if (this.div.is(".list")) {
			rows = $(">tbody>tr.searchresultsrow", this.div);
		}
		this._structure = this._layout(rows, this.div);
		this._renderAllCells(this._structure.entries);
		this._initCalendarView();
	};

	/**
	 * Some general initialization
	 */
	SearchTableRenderer.prototype._initialize = function () {
		// Get the information about how the items are put in the table...
		this._tableDisplaySettings = new TableDisplaySettings();
		this._tableDisplaySettings.showThumbnails = this.div.attr("data-showThumbnails") === "1";
		this._tableDisplaySettings.resultsStyle = this.div.attr("data-resultsStyle");
		this._tableDisplaySettings.resultsStyleSubType = this.div.attr("data-resultsStyleSubType");
		this._tableDisplaySettings.hyperLinksAllowed = this.div.attr("data-noHyperLinks") !== "1";
		this._tableDisplaySettings.updateDerivedInfo();

		//Prepare general WCR search settings
		this._searchSettings = this._searchData.getSearchSettings();
	};

	/**
	 * Layout currently does not layout the table (that was already done by the xsl).
	 * 
	 * Instead it "parses" the layout.
	 */
	SearchTableRenderer.prototype._layout = function (rows, div) {
		var entries = this._getEntries(rows);
		var headers = this._getHeaders(div);

		// Store the structure information
		var structure = {
			headers: headers,
			headersByFieldId: _.indexBy(headers, "fieldid"),
			entries: entries
		};

		return structure;
	};

	/**
	 * gets the parses rows and gets all data for the entries
	 * @return {Array}
	 */
	SearchTableRenderer.prototype._getEntries = function (rows) {
		var entries = [];
		$.each(rows, _.bind(function (i, e) {
			var row = $(e);

			// Get the display data from the row
			var rowdisplaydatastring = row.attr("data-rowdisplaydata");
			if (!rowdisplaydatastring) {
				return;
			}
			var rowdisplaydata = new ValueData();
			ValueDataUtils.fromJSON(rowdisplaydata, $.parseJSON(rowdisplaydatastring));
			// Other data 
			// => data which is still generated by the xsl (FUTURE: REFACTOR => move to JS or Java instead - and put in the ValueData object)
			rowdisplaydata.setValue("thumbhref", row.attr("data-xslgen-thumbhref"));
			
			
			// All the cells for this row
			var cells = [];
			var cellsJQ = $(".datacell", row);
			$.each(cellsJQ, _.bind(function (i2, c) {
				var cell = $(c);
				// Get the name of the field to draw
				var fieldid = cell.attr("data-fieldid");

				// Look up the field
				/** @type SearchField */
				var searchfield = this._searchData.getFieldInformation(fieldid, this._setupInfo);
				
				// Check if this cell is in a table
				var parenttd = null;
				if(cell.parent().is("td")) {
					parenttd = cell.parent();
				}

				cells.push({
					elem: cell,
					parenttd: parenttd,
					fieldid: fieldid,
					searchfield: searchfield
				});
			}, this));

			// Does this row has an area where it can show additional information?
			var expandableRow = row.next(".expandablerow");
			if (expandableRow.length == 0) {
				expandableRow = null;
			}

			// Make the information for this entry
			entries.push({
				elem: row,
				expandsection: expandableRow,
				cells: cells,
				cellsByFieldId: _.indexBy(cells, "fieldid"),
				data: rowdisplaydata
			});
		}, this));
		return entries;
	};

	/**
	 * parses div to get the headers
	 */
	SearchTableRenderer.prototype._getHeaders = function (div) {
		// Find all header cells (only for list view)
		var headers = [];
		var headersJS = $(">thead>tr>th", div);
		$.each(headersJS, _.bind(function (i, e) {
			var header = $(e);
			var fieldid = header.attr("data-columnid");


			headers.push({
				elem: header,
				fieldid: fieldid
			});
		}, this));

		return headers;

	};

	/**
	 * Render cells in the table... (or divs)
	 * 
	 * The Data for each row in the table is currently attached to the row...
	 * When it will be build by JavaScript (in the future?), we will NOT access the information using data attributes!
	 * So, we might rewrite this method in the future (but it only affects this method)
	 */
	SearchTableRenderer.prototype._renderAllCells = function (entries, parentRenderer) {
		// Loop through the table and initialize all fields. 
		_.each(entries, function (entry) {
			var internalrowdata = {
				overflowcomponents: []
			};

			// Decide the "expand mode"... (see package "expandmodes")
			// Depending on the view, we show additional information in a different way...
			var expandMode = null;
			if (this._tableDisplaySettings.resultsStyle === "list") {
				// In list view we have an additional row which can expand.
				if (entry.expandsection) {
					expandMode = new ExpandModeListViewRow(entry.expandsection, $(" > td.contentcell", entry.expandsection));
				}
			}
			// In all other view, we use a popup...
			if (!expandMode) {
				expandMode = new ExpandModeModal();
			}

			_.each(entry.cells, function (cell) {
				// See what kind of display method we need to use...
				var displaymethod = cell.searchfield ? cell.searchfield["cellDisplayType"] : "";
				var customizedlabel = this._setupInfo.getDesignPanelItems().getCustomizedLabel(cell.searchfield);
				// Wrap the expandMode to set the custom title.
				var expandModeWrapper;

				expandModeWrapper = {
					show: function (generateContent, options) {
						expandMode.setDefaultTitle(customizedlabel);
						expandMode.show(generateContent, options);
					}
				};


				// Make field properties... (currently only the Unit information)
				var fieldProperties = this._searchData.makeFieldDataForField(
					cell.searchfield,
					expandModeWrapper,
					this._setupInfo.getLayoutOptions(),
					_.partial(this._doChildSavedSearch.bind(this), Date.now(), entry.expandsection, entry.elem, parentRenderer));

				// Draw the content of the cell...
				this._renderCell(cell.parenttd, cell.elem[0], cell.fieldid, displaymethod, entry.data, fieldProperties, internalrowdata);
			}, this);
		}, this);
	};

	/** 
	 * Does a child saved search, the results of the search will be added to the table with indentation
	 * @return {Object} an object containing two functions: close and show, which should be called to show or hide the child results
	 */
	SearchTableRenderer.prototype._doChildSavedSearch = function (timestamp, expandRow, row, parentRenderer, savedSearch, projectId) {

		var renderProperties = {}; // These render properties are used to 'talk' to the renderer, as the 'show' or 'hide' funtions may be called before the saved search has returned.
		renderProperties.showOnRender = true;
		renderProperties.isRendered = false;
		renderProperties.level = parentRenderer ? parentRenderer.level + 1 : 1; // level is the indenation level of the search

		var childRenderer; 

		var loadingIndicatorRow = $("<tr>").append($("<td>", {
			colspan: parseInt(expandRow.find(".contentcell").attr("colspan")) + 1
		}).append(new LoadingIndicator().start()));
		
		expandRow.after(loadingIndicatorRow);
		loadingIndicatorRow.addClass("child-search-loading-indicator-row");

		// Get the correct classes to render the search correctly inside the talble
		requirejs(["component/searchresults/internal/render/ChildSearchRenderer", "component/searchresults/internal/ChildSearchResultsControl"], function (ChildSearchRenderer, ChildSearchResultsControl) {
			childRenderer = new ChildSearchRenderer(renderProperties, row, expandRow, loadingIndicatorRow, this._searchData, this._setupInfo);
			if (parentRenderer) {
				parentRenderer.children.push(childRenderer);
			}

			// set ChildSearchResultsControl so that we can use it in the show function below
			var parentHpSectionId = row.closest(".wcr-search.mainsearchresultscontrol").attr("data-hpSectionId");

			// Do the search and then create a ChildSearchResultsControl
			ChildSearchResultsControl.doSearch(row, projectId, savedSearch, timestamp, this._setupInfo, this._searchSettings).then(function (html) {
				new ChildSearchResultsControl(html, childRenderer, parentHpSectionId);
			});
		}.bind(this));

		// return close and show functions that cache the return value.
		return {
			close: function () {
				if (renderProperties.isRendered && childRenderer) {
					childRenderer.hideChildren();
				} else {
					renderProperties.showOnRender = false;
				}
				loadingIndicatorRow.hide();
			},
			show: function () {
				if (renderProperties.isRendered) {
					childRenderer.showChildren();
				} else if (!renderProperties.showOnRender) {
					// the search was cancelled, put showOnRender back to true
					renderProperties.showOnRender = true;
					loadingIndicatorRow.show();
				}
			}

		};

	};



	/**
	 * Renders a cell in the table using a custom JavaScript component...
	 * 
	 * @param {jQuery} parenttd - the td around the div to render in (optional)
	 * @param {Element} div - the div to render in
	 * @param {String} fieldid - the name of the field to display
	 * @param {String} fieldDisplayMethod - the way to display this value...
	 * @param {ValueData} data - the available data for the row in the results
	 * @param {FieldProperties} fieldProperties - field specific information (currently only units)
	 * @param {Object} internalrowdata - Just an object (filled & used in this method only)
	 */
	SearchTableRenderer.prototype._renderCell = function (parenttd, div, fieldid, displaymethod, data, fieldProperties, internalrowdata) {
		var wrapperdiv = div;// <- div will be used by the OverflowContentContainer... (performance optimization!)
		// Make a new div inside the wrapperdiv (for the actual content)
		div = $("<div>").appendTo($(wrapperdiv))[0];
		
		// Make a new cell renderer for the cell... It will do the rest.
		var cellrenderer = new CellRenderer(div, displaymethod, fieldid, data, this._tableDisplaySettings, fieldProperties, this._searchSettings);

		cellrenderer.whenLoaded(_.bind(function () {
			// Can this cell contain lots of information?
			// If so, wrap it in a OverflowContentContainer object
			if (cellrenderer.getMightOverflow()) {
				// Make a new overflow component
				var overflow = new OverflowContentContainer(div, this._tableDisplaySettings.cellheight, wrapperdiv);

				// If we are in the LIST VIEW, the overflow components of different cells interact !!!
				if (this._tableDisplaySettings.overflowContainersLinked) {
					// When clicking one of the overflow components, expand all the others on the row too!
					internalrowdata.overflowcomponents.push(overflow);
					overflow.onChanged(function (expanded, cause) {
						if (cause === "CLICK") {
							$.each(internalrowdata.overflowcomponents, function (i, othercontainer) {
								if (expanded) {
									othercontainer.expand("CLICK_OTHER");
								} else {
									othercontainer.collapse("CLICK_OTHER");
								}
							});
						}
					});
				}
			}
			
			// The cellrenderer implementation might want some css to be set on the parent cell...
			if (parenttd) {
				parenttd.css(cellrenderer.getParentCss());
			}

		}, this));
	};

	/**
	 * Initialize the calendar view on these search results...
	 * 
	 * @param {jQuery} div - the div in which to render the table.
	 */
	SearchTableRenderer.prototype._initCalendarView = function () {
		// 1) Check if we can initialize the calendar view on this page
		var hpSectionId = this.div.closest(".wcr-search.mainsearchresultscontrol").attr("data-hpsectionid") || '';
		var paramFormElem = document.getElementById(hpSectionId + '_display_params_form');
		if (!(paramFormElem && paramFormElem.resultsStyle.value == "list")) {
			return;
		}

		var layoutOptions = this._setupInfo.getLayoutOptions();

		// 2) Check whether the user enabled the calendar view in this search
		if (layoutOptions.getOption("SelectProjectStatusStyle").substring(0, "Calendar".length) !== "Calendar") {
			return;
		}

		// 4) Generate a new unique id for this calendar view
		var calendar_id = idUtils.newClientSideId("calendar_view");

		// 3) Add additional columns in the search table for the calendar view
		var calendarViewNeedExtraColumn = layoutOptions.getOption("chkboxShowCalendarStatus");

		// Change the header cells
		var headerCell = this._structure.headersByFieldId["db_status"];
		if (!headerCell) {
			return;
		}
		headerCell = headerCell.elem;
		headerCell.empty();
		// no longer support sorting on this column. Note: the sort listener needs to check this!
		headerCell.removeClass("column-sort td-centered");
		if (calendarViewNeedExtraColumn) {
			headerCell.before($("<th>").css("padding", "0px").append($("<div>").attr({
				"id": "calendar_header_icons_" + calendar_id
			})));
		}
		headerCell.css({
			"padding": "0px"
		}).append($("<div>").attr({
			"id": "calendar_header_" + calendar_id
		}));

		// Get the list of projects and their info & change the table cells
		var projectInfoList = [];
		_.each(this._structure.entries, function (entry) {
			if (!entry.data.getSubList("db_status")) {
				console.log("Could not display project calendar because of missing status information.");
				return; // Cannot initialize calendar view since this project does not have "db_status" enabled.
			}

			var dataproject = entry.data.getSubList("projectinfo")[0];
			var datastatus = entry.data.getSubList("db_status")[0];

			var projectid = dataproject.getValue("id");

			//
			// Make the list of all the visible projects
			//
			var dueDate = dataproject.getValue("due_date");
			var projectinfo = {
				projId: projectid,
				dueDate: Number(dueDate),
				statusId: datastatus.getValue("id")
			};
			projectInfoList.push(projectinfo);


			//
			// Change the HTML
			//
			var cell = entry.cellsByFieldId["db_status"].elem;
			cell = cell.parents("td").first();

			// Set the correct id on the cell (for the calendar)
			cell.attr("id", "db_status_" + calendar_id + "_" + projectid);

			// Add an additional column if needed
			if (calendarViewNeedExtraColumn) {
				var newCell = $("<td>").css({
					"position": "relative"
				}).attr({
					"nowrap": "nowrap",
					"align": "center",
					"id": "db_status_icon_" + calendar_id + "_" + projectid,
					"iconurl": "images/statuses/" + datastatus.getValue("icon_url")
				});
				cell.before(newCell);
			}
		}, this);

		// Get the list of project statuses
		var valueDataList = this._searchData.getList("SVL_PROJECT_STATUS");
		var statusList = new Object();
		for (var i = 0; i < valueDataList.length; i++) {
			var valueData = valueDataList[i];
			var id = valueData.getValue("id");
			var name = valueData.getValue("name");
			var color = valueData.getValue("color");
			var type = Number(valueData.getValue("type"));

			statusList[id] = {
				id: id,
				name: name,
				color: color,
				type: type,
				order: i
			};
		}

		// This loads a bunch of old JavaScript files which we don't want to load on every page. (with global functions)
		require(["wcr/ProjectCalendar", "utils/webi18n!ProjCalendar"], _.bind(function (prjcalmod, calendarI18n) {
			// Get the calendar options
			var calendarOptions = new Object();
			calendarOptions.langArr = calendarI18n.asMap();
			calendarOptions.projectInfoList = projectInfoList;
			calendarOptions.statusList = statusList;
			calendarOptions.mainElemId = null;
			calendarOptions.calendarWidth = Number(layoutOptions.getOption("selectCalendarSize"));
			calendarOptions.calendarSectionId = calendar_id;
			calendarOptions.calendarHeaderId = 'calendar_header_' + calendar_id;
			if (this._setupInfo.getLayoutOptions().getOption("chkboxShowCalendarStatus")) {
				calendarOptions.calendarHeaderIconsId = 'calendar_header_icons_' + calendar_id;
			}
			calendarOptions.objName = 'gCalendar[\'' + calendar_id + '\']';
			calendarOptions.hasTaskMgmntLicense = this._searchData.getSearchSettings().getHasTaskManagementLicense();
			calendarOptions.calendartype = layoutOptions.getOption("SelectProjectStatusStyle");
			// type = DisplayOrder / OrderOfAppearance / NoLabel
			calendarOptions.statusBarLabelType = layoutOptions.getOption("SelectProjectStatusLabelStyle");
			calendarOptions.planForward = layoutOptions.getOption("chkboxApplyForwardPlanning");
			calendarOptions.searchTable = this.div;

			if (!window["gCalendar"]) window.gCalendar = [];
			window.gCalendar[calendar_id] = new projectCalendar(calendarOptions);
		}, this));
	};

	return SearchTableRenderer;
});
