/**
 * Information about the criteria
 */
define([
        "./SearchItemsBase",
        "./SearchCriterionInfo",
        "../dataobjects/ValueDataUtils"
        ], function(
        		SearchItemsBase,
        		SearchCriterionInfo,
        		ValueDataUtils
        		){
	/**
	 * @constructor
	 */
	var SearchCriteriaList = function(){
		SearchItemsBase.call(this, SearchCriterionInfo, false);
	};
	
	// Extend from SearchItemsBase to get listener functionality and such
	SearchCriteriaList.prototype = Object.create(SearchItemsBase.prototype);
	SearchCriteriaList.prototype.constructor = SearchCriteriaList;
	
	//
	// Functions available on this object:
	//
	// + each = Loop through all the criteria
	//          Implementation note: Currently a criterion can not contain other criteria (AND, OR criteria)
	// + add = add a criterion to the list
	// + findByFieldId = Find a criterion by its fieldid
	// 				Implementation noteE: Currently a criterion can not occur twice
	// 				What to do with this method when that is no longer the case???
	// + reorderItemsByIds = Reorder the list (or part of it) by a given list of ids. Does not have any effect on the values.
	// + tryToRemoveItem = Remove a criterion. 
	//				Note, will fail if there is something which is editing the criterion settings... 
	//				(see take() & release() in SearchCriterionInfo)
	//				
	// 				Implementation note: Currently a criterion can not contain other criteria (AND, OR criteria)
	// 				So a simple loop suffices to implement this method...
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + createItem =  Creates a new criterion object. (Just a method for ease of use.)
	// 				Note: this method DOES NOT add the criterion to this list
	// + watch = Watch for changes by a certain field id. Callback will be executed for the first time as soon as it is registered.
	//           The callback will be executed with all arguments undefined in case the object with that id does (no longer) exist.
	//           Watching an object should be used when there is no reason the object should (keep) exist(ing).
	//           If you want read/write access to the object you should keep track of the instance instead and "take()/release()" it.
	// + unwatch = Inverse of watch. Removes the listener.
	
	//
	// Other functions specific to this type of list:
	//
	
	/**
	 * Register a new SmartName change listener
	 * 
	 * @param callback {Function} called when a SmartName is added/removed in a criterion...
	 */
	SearchCriteriaList.prototype.onSmartNameChange = function(callback) {
		this._changeSmartNameCallbacks.push(callback);
	};
	
	//
	// -- end of public methods --
	//
	
	//
	// -- below: handling listeners --
	//  => Extend the list with another type of event: "SmartNameChange"
	//
	
	/**
	 * (private - override) Initialize the listeners...
	 * 
	 * Creates a single instance which is automatically registered & unregistered on all children...
	 */
	SearchCriteriaList.prototype._initListenerFunctions = function(){
		// Call super to initialize the "normal" change event
		SearchItemsBase.prototype._initListenerFunctions.call(this);
		
		// Make the listener function for "SmartNames" also
		this.internalSmartNameChangeCallback = _.bind(function(){
			this._fireSmartNameChange();
		}, this);
		
		// Initialize the list of callbacks for "SmartNames" also
		this._changeSmartNameCallbacks = [];
	};
	
	
	/**
	 * (private - override) Register some listeners on the child
	 * 
	 * @param {T} item - the item that just got added to the list
	 */
	SearchCriteriaList.prototype._registerCallbacksOnChild = function(item) {
		// Call super to initialize the "normal" change event
		SearchItemsBase.prototype._registerCallbacksOnChild.call(this, item);
		
		// Also add the listener for SmartNames
		item.onSmartNameChange(this.internalSmartNameChangeCallback);
	};
	
	/**
	 * (private) Unregister some listeners on the child
	 * 
	 * @param {T} item - the item that just got removed from the list
	 */
	SearchCriteriaList.prototype._unregisterCallbacksOnChild = function(item) {
		// Call super to initialize the "normal" change event
		SearchItemsBase.prototype._unregisterCallbacksOnChild.call(this, item);
		
		// Also remove the listener for SmartNames
		item.removeOnSmartNameChange(this.internalSmartNameChangeCallback);
	};
	
	/**
	 * (private) fire a SmartName change event
	 */
	SearchCriteriaList.prototype._fireSmartNameChange = function() {
		// Fire the change event
		$.each(this._changeSmartNameCallbacks, function(i, callback) {
			callback();
		});
	};
	
	/**
	 * (private) Implementation of executing a watcher callback
	 */
	SearchCriteriaList.prototype._executeWatcherCallback = function(fieldid, item, callback) {
		if(item) {
			callback(ValueDataUtils.clone(item.getValue()));
		} else {
			callback(null);
		}
	};
	
	return SearchCriteriaList;
});