define([
        "underscore",
        "jquery",
        "./SearchItemBase",
        "../dataobjects/ValueData",
        "../dataobjects/ValueDataUtils"
        ], function(_, $,
        		SearchItemBase,
        		ValueData,
        		ValueDataUtils
        		){
	
	/**
	 * @constructor
	 */
	var SearchCriterionInfo = function(fieldId) {
		SearchItemBase.call(this, fieldId);
		
		this._value = new ValueData();
		this._lastsetvalue = new ValueData();
		this._callbacksSmartName = [];
		
		// Calculated information... Whether the value contains any SmartNames.
		// Passed from Java side OR set by the ParameterField which is editing the criterion
		this._containsSmartName = false;
	};
	
	// Extend from SearchItemBase to get listener functionality and such
	SearchCriterionInfo.prototype = Object.create(SearchItemBase.prototype);
	SearchCriterionInfo.prototype.constructor = SearchCriterionInfo;
	
	// 
	// Usage Tracking
	//
	// IF YOU KEEP A REFERENCE to this object, YOU NEED to call the methods:
	//
	// + take = Call this when you start keeping a reference to this object!
	// + release = Call this when you no longer keep a reference to the object!
	//
	// Used by the parent list (to see if there are still references) when removing items:
	// + canBeRemovedFromModel => Check if this item can be removed from the model. (no-one is keeping a reference)
	//
	
	//
	// Other methods
	//
	// + getFieldId = Get the fieldId of the criterion
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + removeOnChange = Remove a previously registered change listener...
	//
	
	//
	// Actual information
	//
	
	/**
	 * Returns the value of this criterion, can be anything depending on the type of criterion...
	 * 
	 * @returns {ValueData} the value of this criterion
	 */
	SearchCriterionInfo.prototype.getValue = function() {
		return this._value;
	};
	
	/**
	 * Set the value of this criterion, can be anything depending on the type of criterion...
	 * 
	 * @param {ValueData} newValue the new value
	 * @param {Boolean} containsSmartName - whether the value contains a SmartName
	 */
	SearchCriterionInfo.prototype.setValue = function(newValue, containsSmartName) {
		if(!(newValue && newValue instanceof ValueData && (containsSmartName === true || containsSmartName === false))) {
			throw new Error("Illegal value is being set SearchCriterionInfo.setValue");
		}
		
		this._value = newValue;
		
		if(this._containsSmartName !== containsSmartName) {
			this._containsSmartName = containsSmartName;
			this._fireSmartNameChange();
		}
		
		// Fire change listeners
		// We check here if the value really changed...
		// For that we need to keep track of a duplicate read-only version of the data...
		// The value attribute can be changed...
		if(ValueDataUtils.equals(this._lastsetvalue, this._value)) {
			// Oh, seems that nothing changed...
			
		} else {
			// Something DID change
			// Store a read-only version of the new value in _lastsetvalue (note: the other value is not read-only)
			this._lastsetvalue = ValueDataUtils.clone(this._value);
			this._fireChangeDefer();
		}
		
	};
	
	/**
	 * Check whether the value contains a SmartName
	 */
	SearchCriterionInfo.prototype.getContainsSmartName = function(){
		return this._containsSmartName;
	};
	
	//
	// Listen to "contains SmartName" changes
	// 
	
	/**
	 * Register a new change listener
	 * @param {Function} callback - the callback to execute when a SmartName is added/removed in this criterion
	 */
	SearchCriterionInfo.prototype.onSmartNameChange = function(callback) {
		this._callbacksSmartName.push(callback);
	};
	
	/**
	 * Remove a previously registered change listener...
	 * @param {Function} callback - the callback to remove
	 */
	SearchCriterionInfo.prototype.removeOnSmartNameChange = function(callback) {
		this._callbacksSmartName = _.without(this._callbacks, callback);
	};
	
	/**
	 * (private) fire change listeners...
	 */
	SearchCriterionInfo.prototype._fireSmartNameChange = function() {
		$.each(this._callbacksSmartName, function(i, callback) {
			callback();
		});
	};
	
	//
	// Other util methods
	//
	/**
	 * (Blind) copy from a certain criterion.
	 * 
	 * @param {SearchCriterionInfo} srcCriterion - the criterion to copy from
	 */
	SearchCriterionInfo.prototype.copyFrom = function(srcCriterion) {
		this.setValue(ValueDataUtils.clone(srcCriterion.getValue()), srcCriterion.getContainsSmartName());
	};
	
	return SearchCriterionInfo;
});