define(["jquery", "underscore", "./SearchItemBase"], function($, _, SearchItemBase){
	
	/**
	 * @constructor
	 */
	var SearchDesignPanelItem = function(fieldId) {
		SearchItemBase.call(this, fieldId);
		
		this._row = "";
		this._column = "";
		this._label = "";
		this._originalVisibility = "Hidden";
		this._visibility = "Hidden";
	};
	
	// Extend from SearchItemBase to get listener functionality and such
	SearchDesignPanelItem.prototype = Object.create(SearchItemBase.prototype);
	SearchDesignPanelItem.prototype.constructor = SearchDesignPanelItem;
	
	// 
	// Usage Tracking
	//
	// IF YOU KEEP A REFERENCE to this object, YOU NEED to call the methods:
	//
	// + take = Call this when you start keeping a reference to this object!
	// + release = Call this when you no longer keep a reference to the object!
	//
	// Used by the parent list (to see if there are still references) when removing items:
	// + canBeRemovedFromModel => Check if this item can be removed from the model. (no-one is keeping a reference)
	//
	
	//
	// Other methods
	//
	// + getFieldId = Get the fieldId of the design panel item
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + removeOnChange = Remove a previously registered change listener...
	//
	
	//
	// Actual information
	//
	
	/**
	 * The row number of this design panel item
	 * 
	 * @returns {String} - a string because it can be "empty" (empty string)
	 */
	SearchDesignPanelItem.prototype.getRow = function() {
		return this._row;
	};
	
	/**
	 * The row number of this design panel item
	 * 
	 * @param {String} row
	 */
	SearchDesignPanelItem.prototype.setRow = function(row) {
		if(this._row != row) {
			this._row = row;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The column number of this design panel item
	 * 
	 * @returns {String} - a string because it can be "empty" (empty string)
	 */
	SearchDesignPanelItem.prototype.getColumn = function() {
		return this._column;
	};
	
	/**
	 * The column number of this design panel item
	 * 
	 * @param {String} column
	 */
	SearchDesignPanelItem.prototype.setColumn = function(column) {
		if(this._column != column) {
			this._column = column;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The custom label of this design panel item
	 * 
	 * @returns {String}
	 */
	SearchDesignPanelItem.prototype.getLabel = function() {
		return this._label;
	};
	
	/**
	 * The custom label of this design panel item
	 * 
	 * @param {String} label
	 */
	SearchDesignPanelItem.prototype.setLabel = function(label) {
		if(this._label != label) {
			this._label = label;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The original visibility of this design panel item
	 * 
	 * @returns {String}
	 */
	SearchDesignPanelItem.prototype.getOriginalVisibility = function() {
		return this._originalVisibility;
	};
	
	/**
	 * The original visibility of this design panel item
	 * 
	 * @param {String} originalVisibility
	 */
	SearchDesignPanelItem.prototype.setOriginalVisibility = function(originalVisibility) {
		if(this._originalVisibility != originalVisibility) {
			this._originalVisibility = originalVisibility;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The visibility of this design panel item
	 * 
	 * @returns {String}
	 */
	SearchDesignPanelItem.prototype.getVisibility = function() {
		return this._visibility;
	};
	
	/**
	 * The visibility of this design panel item
	 * 
	 * @param {String} visibility
	 */
	SearchDesignPanelItem.prototype.setVisibility = function(visibility) {
		if(this._visibility != visibility) {
			this._visibility = visibility;
			this._fireChangeDefer();
		}
	};
	
	//
	// Other util methods
	//
	/**
	 * (Blind) copy from a certain criterion.
	 * 
	 * @param {SearchDesignPanelItem} srcCriterion - the criterion to copy from
	 */
	SearchDesignPanelItem.prototype.copyFrom = function(srcCriterion) {
		this.setRow(srcCriterion.getRow());
		this.setColumn(srcCriterion.getColumn());
		this.setLabel(srcCriterion.getLabel());
		this.setOriginalVisibility(srcCriterion.getOriginalVisibility());
		this.setVisibility(srcCriterion.getVisibility());
	};
	
	/**
	 * Copy from a certain criterion.
	 * This method does not do a blind copy but first checks whether this field supports design panel.
	 * 
	 * @param {SearchField} searchfield - the SearchField containing information about the field for this criterion
	 * @param {SearchSortingCriterion} srcCriterion - the criterion to copy from
	 */
	SearchDesignPanelItem.prototype.safeCopyFrom = function(searchfield, srcCriterion) {
		if(searchfield.supportsDesignPanel) {
			this.copyFrom(srcCriterion);
		}
	};
	
	return SearchDesignPanelItem;
});