define(["jquery", "underscore", "./SearchItemBase"], function($, _, SearchItemBase){
	
	/**
	 * @constructor
	 */
	var SearchDisplayColumn = function(fieldId) {
		SearchItemBase.call(this, fieldId);
		
		this._enabled = false;
		this._editable = false;
		this._sequence = "-1";
	};
	
	// Extend from SearchItemBase to get listener functionality and such
	SearchDisplayColumn.prototype = Object.create(SearchItemBase.prototype);
	SearchDisplayColumn.prototype.constructor = SearchDisplayColumn;
	
	// 
	// Usage Tracking
	//
	// IF YOU KEEP A REFERENCE to this object, YOU NEED to call the methods:
	//
	// + take = Call this when you start keeping a reference to this object!
	// + release = Call this when you no longer keep a reference to the object!
	//
	// Used by the parent list (to see if there are still references) when removing items:
	// + canBeRemovedFromModel => Check if this item can be removed from the model. (no-one is keeping a reference)
	//
	
	//
	// Other methods
	//
	// + getFieldId = Get the fieldId of the display column criterion
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + removeOnChange = Remove a previously registered change listener...
	//
	
	
	//
	// Actual information
	//
	
	/**
	 * Whether displaying this column is enabled
	 * 
	 * @returns {bool}
	 */
	SearchDisplayColumn.prototype.getEnabled = function() {
		return this._enabled;
	};
	
	/**
	 * Whether displaying this column is enabled
	 * 
	 * @param {bool} enabled
	 */
	SearchDisplayColumn.prototype.setEnabled = function(enabled) {
		if(this._enabled != enabled) {
			this._enabled = enabled;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * Whether this column is set to be editable
	 * 
	 * @returns
	 */
	SearchDisplayColumn.prototype.isEditable = function() {
		return this._editable;
	};
	
	/**
	 * Whether this column is set to be editable
	 * 
	 * @param editable
	 */
	SearchDisplayColumn.prototype.setEditable = function(editable) {
		editable = (!!editable);// -- make sure it is a boolean
		if(this._editable != editable) {
			this._editable = editable;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The sequence of this column
	 * 
	 * @returns {String} sequence number. -1 if not enabled
	 */
	SearchDisplayColumn.prototype.getSequence = function() {
		return this._sequence;
	};
	
	/**
	 * The sequence of this column
	 * 
	 * @param {String} sequence - sequence number. -1 if not enabled
	 */
	SearchDisplayColumn.prototype.setSequence = function(sequence) {
		if(isNaN(sequence)) {
			sequence = "-1";
		}
		if(!_.isString(sequence)) {
			sequence = ""+sequence;
		}
		if(this._sequence != sequence) {
			this._sequence = sequence;
			this._fireChangeDefer();
		}
	};
	
	//
	// Other util methods
	//
	/**
	 * (Blind) copy from a certain criterion.
	 * 
	 * @param {SearchDisplayColumn} srcCriterion - the criterion to copy from
	 */
	SearchDisplayColumn.prototype.copyFrom = function(srcCriterion) {
		this.setEnabled(srcCriterion.getEnabled());
		this.setEditable(srcCriterion.isEditable());
		this.setSequence(srcCriterion.getSequence());
	};
	
	/**
	 * Copy from a certain criterion.
	 * This method does not do a blind copy but first checks whether this field supports viewing & editing.
	 * 
	 * @param {SearchField} searchfield - the SearchField containing information about the field for this criterion
	 * @param {SearchSortingCriterion} srcCriterion - the criterion to copy from
	 */
	SearchDisplayColumn.prototype.safeCopyFrom = function(searchfield, srcCriterion) {
		if(searchfield.supportsViewList) {
			this.setEnabled(srcCriterion.getEnabled());
			this.setSequence(srcCriterion.getSequence());
		}
		if(searchfield.supportsBatchActions) {
			this.setEditable(srcCriterion.isEditable());
		}
	};
	
	return SearchDisplayColumn;
});