/**
 * See Java implementation "SearchField"
 */
define([
        "../dataobjects/ValueData",
        "../dataobjects/ValueDataUtils",
		"underscore"
       ], function(ValueData, ValueDataUtils, _){
	"use strict";
	
	/**
	 * @constructor
	 */
	var SearchField = function(fieldId){
		/**
		 * Name of this field, like db_name or db_version or db_attr_0000...
		 */
		this.fieldId = fieldId;
		
		/**
		 * Can this field be instanced?
		 * Options:
		 *  - NONE
		 *  - MULTI_FIELD_SINGLE_SELECTION
		 *  - MULTI_FIELD_MULTI_SELECTION
		 *  */
		this.instancingType = "NONE";
		
		/**
		 * The field id from which the field instances
		 */
		this.instancingSource = "";
		
		/**
		 * The instance specific configuration
		 */
		this.instanceConfiguration = null;
		
		/**
		 * <p>
		 * The type which defines how the information in the cell is displayed on the search results page.
		 * </p>
		 * 
		 * <p>
		 * See javascript components in the /component/searchresults/implementations/datarenderers folder.
		 * </p>
		 * 
		 * Can be null if you cannot display this information.
		 */
		this.cellDisplayType = null;

		/**
		 * <p>
		 * The type which defines how the field is shown in the search design panel.
		 * </p>
		 * 
		 * <p>
		 * See javascript components in the /component/searchresults/implementations/parameterfields folder.
		 * </p>
		 * 
		 * Can be null if you cannot put this field in a design panel
		 */
		this.parameterFieldType = null;

		/**
		 * <p>
		 * The label to show in the column header of the table.
		 * </p>
		 */
		this.label = "";
		
		/**
		 * <p>
		 * The Metric Unit
		 * </p>
		 */
		this.unitMetric = null;
		
		/**
		 * <p>
		 * The Imperial Unit
		 * </p>
		 */
		this.unitImperial = null;
		
		/**
		 * <p>
		 * The Lucene Unit (what is in the index + stored in the data)
		 * </p>
		 */
		this.unitLucene = null;
		
		/**
		 * <p>
		 * The Data Type of this field. Can be either "string", "float" or "int".
		 * </p>
		 */
		this.dataType = "string";
		
		/**
		 * <p>
		 * Additional settings for the "DataRenderer" component or the "ParameterField" component. Can contain anything...
		 * </p>
		 * 
		 * @type {ValueData}
		 */
		this.displayConfig = new ValueData();
		
		/**
		 * <p>
		 * [OPTIONAL] If this field works with List Values (s.a. dropdown value selector) it could also define special static values
		 * </p>
		 * 
		 * @type {ValueData}
		 */
		this.staticListValues = null;
		
		/**
		 * <p>
		 * [OPTIONAL] If this field works with List Values (s.a. dropdown value selector) this field will hold a reference under which to look for the List Value items
		 * </p>
		 */
		this.listValueID = null;
		
		/*
		 * Settings
		 */
		this.supportsViewList = false;
		this.supportsBatchActions = false;
		this.supportsDesignPanel = false;
		this.supportsSorting = false;
		
	};
	
	SearchField.prototype.getDefaultLabel = function() {
		return this.label;
	};
	
	/**
	 * Create a new instance of the SearchField
	 * This will copy all properties (and clone the display configuration)
	 * 
	 * @param {STring}      fieldId - instanced fieldId
	 * @returns {SearchField}   
	 */
	SearchField.prototype.createInstance = function(instanceId, fieldId, instanceConfiguration){
		var instance = new SearchField(instanceId);
		_.extend(instance, this);
		instance.instancingSource = fieldId;
		instance.instanceConfiguration = instanceConfiguration;
		
		// make sure to clone the display config
		instance.displayConfig = ValueDataUtils.clone(this.displayConfig);
		return instance;
	};
	
	
	return SearchField;
});
