define(["../../searchresults/dataobjects/ValueData",
		"../../searchresults/dataobjects/ValueDataUtils",
		"underscore"
	   ], function (
	ValueData,
	ValueDataUtils,
	_
) {
	"use strict";

	/**
	 * Configuration of the fields with multiple instances.
	 * An instanced field is a field that can appear multiple times, but each instance needs to have a unique id.
	 * @class
	 */
	var SearchInstancedFieldConfiguration = function () {
		this._instances = {};
	};

	/**
	 * Get all the data stored in this configuration
	 * @returns {object} - example: { "db_facet~1": <ValueData>, "db_facet~2": <ValueData> }
	 */
	SearchInstancedFieldConfiguration.prototype.getDataClone = function () {
		var instances = {};
		_.each(this._instances, function (value, instanceId) {
			instances[instanceId] = ValueDataUtils.clone(value);
		});
		return instances;
	};

	/**
	 * Clone this object into a new SearchInstancedFieldConfiguration 
	 * @returns {SearchInstancedFieldConfiguration} 
	 */
	SearchInstancedFieldConfiguration.prototype.clone = function () {
		var dataClone = this.getDataClone();
		var clone = new SearchInstancedFieldConfiguration();
		clone.setData(dataClone);
		return clone;

	};
	/**
	 * Set the data stored in this configuration
	 * @param {object} data - example: { "db_facet~1": <ValueData>, "db_facet~2": <ValueData> }
	 */
	SearchInstancedFieldConfiguration.prototype.setData = function (data) {
		if (data == null) {
			this._instances = {};
		} else {
			this._instances = data;
		}
	};

	/**
	 * Create a new instance based on the fieldId.
	 * @param {string} fieldId      - For example "db_facet"
	 * @param {ValueData} data      - The configuration data
	 * @returns {string} instanceId - For example "db_facet~1"
	 */
	SearchInstancedFieldConfiguration.prototype.addInstance = function (fieldId, data) {
		var newInstanceId = this._generateNewInstanceId(fieldId);
		this._instances[newInstanceId] = data;
		return newInstanceId;
	};

	/**
	 * Get the instance based on its instanceId
	 * @param {string}      instanceId 
	 * @returns {ValueData} 
	 */
	SearchInstancedFieldConfiguration.prototype.getInstance = function (instanceId) {
		return this._instances[instanceId];
	};

	/**
	 * Set the instance based on its instanceId
	 * @param {string}      instanceId 
	 * @param {ValueData}      data 
	 */
	SearchInstancedFieldConfiguration.prototype.setInstance = function (instanceId, data) {
		this._instances[instanceId] = data;
	};

	/**
	 * Remove an instance
	 * @param {string} instanceId 
	 */
	SearchInstancedFieldConfiguration.prototype.removeInstance = function (instanceId) {
		delete this._instances[instanceId];
	};

	/**
	 * Generates a new unused instanceId for a given fieldId
	 * @private
	 * @param   {string} fieldId 
	 * @returns {string} - New instanceId
	 */
	SearchInstancedFieldConfiguration.prototype._generateNewInstanceId = function (fieldId) {
		var numbersInUse = [];
		_.each(this._instances, function (value, instanceId) {
			if (this._getFieldIdFromInstanceId(instanceId) === fieldId) {
				var numberstring = instanceId.substr(fieldId.length + 1); //Get the number from the instanceId. For example: "db_facet~1" -> "1"
				numbersInUse.push(parseInt(numberstring, 10));
			}
		}, this);
		var newInstanceIdNumber = 1;
		while (_.contains(numbersInUse, newInstanceIdNumber)) {
			newInstanceIdNumber++;
		}
		return fieldId + "~" + newInstanceIdNumber;
	};

	/**
	 * Extract the fieldId from the instanceId
	 * @private
	 * @param   {string} instanceId 
	 * @returns {string} 
	 */
	SearchInstancedFieldConfiguration.prototype._getFieldIdFromInstanceId = function (instanceId) {
		return instanceId.substr(0, instanceId.indexOf("~"));
	};
	
	/**
	 * Check whether we have instances of a certain fieldid
	 * 
	 * @param fieldid
	 * @returns {Boolean}
	 */
	SearchInstancedFieldConfiguration.prototype.hasInstancedOfType = function(fieldId) {
		var foundInstance = false;
		_.each(this._instances, function (value, instanceId) {
			if (this._getFieldIdFromInstanceId(instanceId) === fieldId) {
				foundInstance = true;
			}
		}, this);
		return foundInstance;
	};

	return SearchInstancedFieldConfiguration;
});
