define([
        "jquery",
        "underscore"
        ], function($, _){
	
	/**
	 * @constructor
	 */
	var SearchItemBase = function(fieldId) {
		if(!(fieldId && typeof(fieldId) === "string")) {
			throw new Error("Illegal fieldId value is passed in constructor of SearchItemBase");
		}
		
		this._fieldId = fieldId;
		this._callbacks = [];
		
		this._usagetacker = 0;
	};
	
	// 
	// Usage Tracking
	//
	// IF YOU KEEP A REFERENCE to this object, 
	// YOU NEED to call these methods!
	//
	
	/**
	 * Call this when you start keeping a reference to this object!
	 * 
	 * If you have a reference and someone else tries to remove it, it cannot be removed from the model.
	 */
	SearchItemBase.prototype.take = function() {
		this._usagetacker++;
	};
	
	/**
	 * Call this when you no longer keep a reference to the object!
	 * 
	 * Don't forget to call this when you are no longer keeping a reference to the object!
	 */
	SearchItemBase.prototype.release = function() {
		this._usagetacker--;
		if(this._usagetacker<0) {
			console.log("[ERR] SearchItemBase.release -> Illegal State: Too much release calls!");
		}
	};
	
	/**
	 * Check if this display column can be removed from the model.
	 * 
	 * It can only be removed if no one is still keeping a reference to it.
	 * 
	 * @returns {Boolean}
	 */
	SearchItemBase.prototype.canBeRemovedFromModel = function() {
		return this._usagetacker === 0;
	};
	
	//
	// Actual information
	//
	
	/**
	 * Get the fieldName of this criterion
	 * 
	 * @returns {String} fieldname
	 */
	SearchItemBase.prototype.getFieldId = function(){
		return this._fieldId;
	};
	
	//
	// Listen to changes
	// 
	
	/**
	 * Register a new change listener
	 * @param {Function} callback - the callback to execute when this item changes.
	 */
	SearchItemBase.prototype.onChange = function(callback) {
		this._callbacks.push(callback);
	};
	
	/**
	 * We don't want to invoke change listeners while the change we are doing is not complete.
	 * So, in general, we invoke the change listeners the frame after changes occured.
	 * 
	 * However, sometimes you want to do it sooner. That is when you call this method.
	 * E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	 * 
	 * 
	 * Note: you do not need to call this method! 
	 */
	SearchItemBase.prototype.commitChanges = function() {
		if(this._changed) {
			this._fireChange(true);
		}
	};
	
	/**
	 * Remove a previously registered change listener...
	 * @param {Function} callback - the callback to remove
	 */
	SearchItemBase.prototype.removeOnChange = function(callback) {
		this._callbacks = _.without(this._callbacks, callback);
	};
	
	/**
	 * (private) fire a change event, but a bit delayed and only once.
	 */
	SearchItemBase.prototype._fireChangeDefer = function() {
		this._changed = true;
		var currentChangeIndex = this._fireindex || 0;
		_.defer(_.bind(function(){
			if(currentChangeIndex == (this._fireindex || 0)) {
				this._fireChange(false);
			}
		}, this));
	};
	
	/**
	 * (private) fire change listeners...
	 */
	SearchItemBase.prototype._fireChange = function(byCommit) {
		this._changed = false;
		if(this._fireindex===undefined){
			this._fireindex = 0;
		}
		this._fireindex++;
		$.each(this._callbacks, function(i, callback) {
			callback(byCommit);
		});
	};
	
	return SearchItemBase;
});