define(["jquery", "underscore"], function($, _){
	//
	// Base class for all "lists" in the JavaScript Search Object Model
	// It contains "add"/"remove" and "listener" functionality
	//
	
	/**
	 * @constructor
	 * @param {Function} type - the type of objects that can be in this list (must extend SearchItemBase)
	 * @param {Boolean} addEmptyAllowed - Can we add an element we just created to the list without configuring anything?
	 */
	var SearchItemsBase = function(type, addEmptyAllowed){
		// The type of objects that can be in this list
		this._type = type;
		
		// Can we add an element we just created to the list without configuring anything?
		this._addEmptyAllowed = addEmptyAllowed;
		
		// The list of objects...
		this._list = [];
		
		// For handling listening to children
		this._initListenerFunctions();
		
		// All the ids for which the watcher callbacks still needs to be executed (in the next change-callback "frame")
		this._changedids = [];
		
		// All the callbacks that are watching a certain field by id (but without keeping a reference)
		this._watcherMap = {};
	};
	
	/**
	 * Loop through all the items
	 * 
	 * @param callback {Function} a function with one argument of type that is in this list
	 */
	SearchItemsBase.prototype.each = function(callback) {
		$.each(this._list, function(i, item) {
			callback(item);
		});
	};
	
	/**
	 * Add an item
	 * 
	 * @param {T} item - the item to add
	 */
	SearchItemsBase.prototype.add = function(item) {
		
		// NOTE: Currently an item can not occur twice
		// So, we check for an existing item first...
		// (And throw an error if there is one... You should delete it first!)
		// This check might be removed in the future...
		if(this.findByFieldId(item.getFieldId())){
			throw new Error("That item already exists... Delete it first.");
		}
		
		// Check if item is from the correct class
		if(!(item instanceof this._type)){
			throw new Error("An item of the incorrect type passed.");
		}
		
		// Check if this item object is not already in the list...
		if (_.indexOf(this._list, item)!==-1) {
			throw new Error("Cannot add the same item twice.");
		}
		
		// Do the add
		this._list.push(item);
		this._registerCallbacksOnChild(item);
		
		// Fire change event
		this._fireChangeDefer([item.getFieldId()]);
	};
	
	/**
	 * Find an item by its fieldid
	 * 
	 * @param {String} fieldId
	 */
	SearchItemsBase.prototype.findByFieldId = function(fieldId) {
		var retval = null;
		$.each(this._list, function(i, c) {
			if(c.getFieldId() === fieldId) {
				retval = c;
			}
		});
		return retval;
	};
	
	/**
	 * Call this method to make sure items in the list already exist...
	 * 
	 * Will add empty items when they do not exist.
	 * Only supported when empty items are allowed in the list.
	 * 
	 * Adding too many empty items is not a problem... 
	 * These will be removed when writing to xml on the server side...
	 * 
	 * @param {Array.<String>} fieldids - the ids to add if they are not there yet.
	 */
	SearchItemsBase.prototype.ensureExist = function(fieldids) {
		if(this._addEmptyAllowed) {
			var allitems = _.map(this._list, function(item) {return item.getFieldId();});
			_.each(_.difference(fieldids, allitems), function(fieldid) {
				this.add(this.createItem(fieldid));
			}, this);
		}
	};
	
	/**
	 * Reorder the list (or part of it) by a given list of ids. Does not have any effect on the values.
	 * 
	 * @param neworder
	 */
	SearchItemsBase.prototype.reorderItemsByIds = function(neworder) {
		if(this._addEmptyAllowed) {
			// It could be that the items we want to re-order are not in the list yet...
			// So: add empty items already so that when we start using them they are in the correct order.
			// (We don't care about empty items anyway... These will be removed when writing to xml on the server side...)
			this.ensureExist(neworder);
		}
		
		var indexbyid = _.invert(neworder);
		this._list.sort(function(a, b) {
			a=a.getFieldId();
		    b=b.getFieldId();
			var reorderA = _.has(indexbyid, a);
			var reorderB = _.has(indexbyid, b);
			if(reorderA && reorderB) {
				return (+indexbyid[a])<(+indexbyid[b])?-1:1;
			}else if(reorderA){
				return 1;
			}else if(reorderB){
				return -1;
			} else {
				return 0;
			}
		});
	};
	
	/**
	 * Remove an item. 
	 * Note, will fail if there is something which is editing the item... 
	 * (see take() & release() in SearchItemBase)
	 * 
	 * @param {T} item the item to remove
	 */
	SearchItemsBase.prototype.tryToRemoveItem = function(item) {
		var index = -1;
		$.each(this._list, function(i, c) {
			if(item === c) {
				index = i;
			}
		});
		if(index>=0) {
			this._unregisterCallbacksOnChild(this._list[index]);
			if(this._list[index].canBeRemovedFromModel()) {
				this._list.splice(index, 1);
				this._fireChangeDefer([item.getFieldId()]);
			}else{
				this._registerCallbacksOnChild(this._list[index]);
			}
		}
	};
	
	/**
	 * Register a new change listener
	 * 
	 * @param {Function} callback called when something changed...
	 */
	SearchItemsBase.prototype.onChange = function(callback) {
		this._changeCallbacks.push(callback);
	};
	
	/**
	 * We don't want to invoke change listeners while the change we are doing is not complete.
	 * So, in general, we invoke the change listeners the frame after changes occured.
	 * 
	 * However, sometimes you want to do it sooner. That is when you call this method.
	 * E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	 * 
	 * 
	 * Note: you do not need to call this method! 
	 */
	SearchItemsBase.prototype.commitChanges = function() {
		if(!this._commitInProgress) {
			this._commitInProgress = true;
			try{
				this.each(_.bind(function(item){
					item.commitChanges();
				}, this));
			} finally {
				this._commitInProgress = false;
			}
			if(this._changed) {
				this._fireChange();
			}
		}
	};
	
	/**
	 * Just a method for ease of use.
	 * 
	 * Creates a new item of the correct type. 
	 * Note: this method DOES NOT add the item to this list
	 * 
	 * @param {String} fieldid the fieldid linked to the item
	 */
	SearchItemsBase.prototype.createItem = function(fieldid){
		var T = this._type;
		return new T(fieldid);
	};
	
	/**
	 * Watch an object in the list by an id.
	 * This will not prevent the object from being removed from the list and should not be used if you need to "synchronize" with the information.
	 * 
	 * You will never get write access to an item in the list by calling the watch method.
	 * 
	 * The callback will be called also immediately when the watch function is called.
	 * 
	 * @param fieldid
	 * @param callback
	 */
	SearchItemsBase.prototype.watch= function(fieldid, callback) {
		if(!_.has(this._watcherMap, fieldid)) {
			this._watcherMap[fieldid] = [];
		}
		this._watcherMap[fieldid].push(callback);
		this._executeWatcherCallback(fieldid, this.findByFieldId(fieldid), callback);
	};
	
	/**
	 * Unwatch changes to an object in the list by an id. (inverse of watchWithoutReference)
	 * 
	 * @param fieldid
	 * @param callback
	 */
	SearchItemsBase.prototype.unwatch = function(fieldid, callback) {
		this._watcherMap[fieldid] = _.without(this._watcherMap[fieldid], callback);
	};
	
	//
	// -- end of public methods --
	//
	
	//
	// -- below: handling listeners --
	//
	
	/**
	 * (private) Initialize the listeners...
	 * 
	 * Creates a single instance which is automatically registered & unregistered on all children...
	 */
	SearchItemsBase.prototype._initListenerFunctions = function(){
		// Make a separate listener function for each criterion
		this.internalChangeCallbacks = {};
		
		// Initialize the list of callbacks
		this._changeCallbacks = [];
	};
	
	/**
	 * (private) Register some listeners on the child
	 * 
	 * @param {T} item - the item that just got added to the list
	 */
	SearchItemsBase.prototype._registerCallbacksOnChild = function(item) {
		var internalChangeCallback = null;
		if(_.has(this.internalChangeCallbacks, item.getFieldId())) {
			internalChangeCallback = this.internalChangeCallbacks[item.getFieldId()];
		} else {
			internalChangeCallback = _.bind(function(fieldId, byCommit){
				this._fireChangeDefer([fieldId]);
				if(byCommit) {
					this.commitChanges();
				}
			},this, item.getFieldId());
			this.internalChangeCallbacks[item.getFieldId()] = internalChangeCallback;
		}
		
		item.onChange(internalChangeCallback);
	};
	
	/**
	 * (private) Unregister some listeners on the child
	 * 
	 * @param {T} item - the item that just got removed from the list
	 */
	SearchItemsBase.prototype._unregisterCallbacksOnChild = function(item) {
		item.removeOnChange(this.internalChangeCallbacks[item.getFieldId()]);
	};
	
	/**
	 * (private) fire a change event, but a bit delayed and only once.
	 * 
	 * @param {String[]} updatedFieldIds - the list of fieldid's that are updated
	 */
	SearchItemsBase.prototype._fireChangeDefer = function(updatedFieldIds) {
		this._changed = true;
		this._changedids = this._changedids.concat(updatedFieldIds);
		var currentChangeIndex = this._fireindex || 0;
		_.defer(_.bind(function(){
			if(currentChangeIndex == (this._fireindex || 0)) {
				this._fireChange();
			}
		}, this));
	};
	
	/**
	 * (private) fire a change event
	 */
	SearchItemsBase.prototype._fireChange = function() {
		this._changed = false;
		var changewatchcallbackids = this._changedids;
		this._changedids = [];
		if(this._fireindex===undefined){
			this._fireindex = 0;
		}
		this._fireindex++;
		// Fire the change event
		$.each(this._changeCallbacks, function(i, callback) {
			callback();
		});
		// Fire change events for things that are watching by id
		_.each(changewatchcallbackids, function(fieldid) {
			if(_.has(this._watcherMap, fieldid)){
				var watchersForId = this._watcherMap[fieldid];
				if(watchersForId.length>0) {
					var item = this.findByFieldId(fieldid);
					_.each(watchersForId, function(callback) {
						this._executeWatcherCallback(fieldid, item, callback);
					}, this);
				}
			}
		}, this);
	};
	
	/**
	 * (private) execute the callback for watching an object with a certain id
	 * 
	 * To be overridden by each list to make sure the item is passed a read-only.
	 * 
	 * @param fieldid - the fieldid which the callback is watching
	 * @param item - the item with that fieldid (or null if it was removed)
	 * @param callback - the callback to execute when the item changed
	 */
	SearchItemsBase.prototype._executeWatcherCallback = function(fieldid, item, callback) {
		throw new Error("Watching is not implemented for this type of list");
	};
	
	return SearchItemsBase;
});