/**
 * Information about the display columns
 */
define([
        "underscore",
        "../dataobjects/ValueData"
        ], function(
        		_,
        		ValueData
        		){
	
	/**
	 * Represents an available setting in the SearchLayoutOptions.
	 * 
	 * NOTE: The list of available settings is loaded from the Java side (and only defined there)
	 *  => See SearchLayoutOptions.java
	 * 
	 * @param {String} id - the id of the setting
	 * @param {String} type - the type of the setting
	 * @param {String} defaultValue - the default value of the setting
	 */
	var Setting = function(id, type, defaultValue) {
		this.id = id;
		this.type = type;
		this.defaultValue = defaultValue;
	};
	
	
	/**
	 * @constructor
	 */
	var SearchLayoutOptions = function(){
		// List of all available settings
		this._availablesettings = null;
		this._availablesettingsmap = null;
		
		this._data = {};
		
		// A single instance which is automatically registered & unregistered on all children...
		this.internalChangeCallback = _.bind(function(byCommit){
			this._fireChangeDefer();
			if(byCommit) {
				this.commitChanges();
			}
		},this);
		
		// For grouping change events together
		this._changeCallbacks = [];
	};
	
	// Setting is "Inner class"
	SearchLayoutOptions.Setting = Setting;
	
	
	/**
	 * Set the list of available settings.
	 * (Can only be called once.)
	 * 
	 * @param {Array of Setting} availablesettings - the list of available settings
	 */
	SearchLayoutOptions.prototype.setAvailableSettings = function(availablesettings) {
		if(!availablesettings || !_.isArray(availablesettings)) {
			throw new Error("Invalid parameter passed to setAvailableSettings in SearchLayoutOptions");
		}
		if(this._availablesettings === null) {
			// Check the input and make a local copy.
			this._availablesettings = _.map(availablesettings, function(setting) {
				if(setting && setting instanceof Setting && setting.id && setting.type &&
					// Check defaultValue	
					(	(setting.type === "STRING" && _.isString(setting.defaultValue)) || 
						(setting.type === "BOOLEAN" && _.isBoolean(setting.defaultValue)) ||
						(setting.type === "COMPLEXVALUE" && setting.defaultValue === null) ||
						(setting.type === "ITEMSEQUENCE" && setting.defaultValue === null) )
					) {
					
					// Ok
					return setting;
				} else {
					throw new Error("Invalid parameter passed to setAvailableSettings in SearchLayoutOptions");
				}
			});
			this._availablesettingsmap = _.indexBy(this._availablesettings, "id");
		} else {
			throw new Error("List of available settings already set.");
		}
	};
	
	/**
	 * Get the list of available settings
	 * 
	 * @returns {Array of Setting}
	 */
	SearchLayoutOptions.prototype.getAvailableSettings = function() {
		// return a copy
		return _.map(this._availablesettings, _.identity);
	};
	
	/**
	 * Look up whether a setting exists and what it's type is.
	 *  - If the setting is of DataType STRING, "STRING" will be returned.
	 *  - If the setting is of DataType BOOLEAN, "BOOLEAN" will be returned.
	 *  - If the setting is of DataType COMPLEXVALUE, "COMPLEXVALUE" will be returned.
	 *  - If the setting is of DataType ITEMSEQUENCE, "ITEMSEQUENCE" will be returned.
	 *  - If an unknown setting is accessed, null will be returned.
	 * 
	 * @param settingid
	 */
	SearchLayoutOptions.prototype.getOptionType = function(settingid) {
		if(!_.has(this._availablesettingsmap, settingid)) {
			return null;
		}
		var setting = this._availablesettingsmap[settingid];
		return setting.type;
	};
	
	/**
	 * Look up the value of a certain setting.
	 * 
	 *  - If the setting is of DataType STRING, this function will return a string.
	 *  - If the setting is of DataType BOOLEAN, this function will return a boolean.
	 *  - If the setting is of DataType COMPLEXVALUE, this function will return a ValueData object.
	 *  - If the setting is of DataType ITEMSEQUENCE, this function will return an array of strings.
	 *  - If an unknown setting is accessed, an error will be thrown.
	 * 
	 * @param settingid
	 * @returns
	 */
	SearchLayoutOptions.prototype.getOption = function(settingid) {
		if(!_.has(this._availablesettingsmap, settingid)) {
			throw new Error("Unknown option accessed.");
		}
		var setting = this._availablesettingsmap[settingid];
		var value = this._data[settingid];
		var defaultValue = setting.defaultValue;
		
		if(setting.type === "STRING") {
			return _.isString(value)?value:defaultValue;
		} else if (setting.type === "BOOLEAN") {
			return _.isBoolean(value)?value:defaultValue;
		} else if (setting.type === "COMPLEXVALUE") {
			return (value && value instanceof ValueData)?value:null;
		} else if (setting.type === "ITEMSEQUENCE") {
			return _.isArray(value)?_.map(value, function(a) {return _.isString(a)?a:"";}):[];
		} else {
			throw new Error("Illegal state");// Can never happen
		}
	};
	
	/**
	 * Set the value of a certain setting.
	 * 
	 *  - If the setting is of DataType STRING, this function will expects a string as value.
	 *  - If the setting is of DataType BOOLEAN, this function will expects a boolean as value.
	 *  - If the setting is of DataType COMPLEXVALUE, this function will expects a ValueData object as value.
	 *  - If the setting is of DataType ITEMSEQUENCE, this function will expects an array of strings as value.
	 * 
	 * To reset to the default value, pass null.
	 * 
	 * If an unknown setting is accessed of an incorrect type is passed, an error will be thrown.
	 * 
	 * @param settingid
	 * @param value
	 */
	SearchLayoutOptions.prototype.setOption = function(settingid, value) {
		if(!_.has(this._availablesettingsmap, settingid)) {
			throw new Error("Unknown option accessed.");
		}
		var setting = this._availablesettingsmap[settingid];
		
		var checkArrayOfStrings = _.partial(_.reduce, _, function(memo, entry) {return memo && _.isString(entry);}, true);
		
		if(	(setting.type === "STRING" && _.isString(value)) ||
			(setting.type === "BOOLEAN" && _.isBoolean(value)) ||
			(setting.type === "COMPLEXVALUE" && (value && value instanceof ValueData)) ||
			(setting.type === "ITEMSEQUENCE" && _.isArray(value) && checkArrayOfStrings(value)) ) {
			var oldValue = this._data[settingid];
			if(  	(setting.type === "STRING" && oldValue!==value) ||
					(setting.type === "BOOLEAN" && oldValue!==value) ||
					(setting.type === "COMPLEXVALUE") ||
					(setting.type === "ITEMSEQUENCE")) {
				this._data[settingid] = value;
				this._fireChangeDefer();
			}
		} else {
			if(this._data.hasOwnProperty(settingid)) {
				delete this._data[settingid];
				this._fireChangeDefer();
			}
		}
	};
	
	/**
	 * Register a new change listener
	 * 
	 * @param {Function} callback called when something changed...
	 */
	SearchLayoutOptions.prototype.onChange = function(callback) {
		this._changeCallbacks.push(callback);
	};
	
	/**
	 * We don't want to invoke change listeners while the change we are doing is not complete.
	 * So, in general, we invoke the change listeners the frame after changes occured.
	 * 
	 * However, sometimes you want to do it sooner. That is when you call this method.
	 * E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	 * 
	 * 
	 * Note: you do not need to call this method! 
	 */
	SearchLayoutOptions.prototype.commitChanges = function() {
		if(this._changed) {
			this._fireChange();
		}
	};
	
	/**
	 * (private) fire a change event, but a bit delayed and only once.
	 */
	SearchLayoutOptions.prototype._fireChangeDefer = function() {
		this._changed = true;
		var currentChangeIndex = this._fireindex || 0;
		_.defer(_.bind(function(){
			if(currentChangeIndex == (this._fireindex || 0)) {
				this._fireChange();
			}
		}, this));
	};
	
	/**
	 * (private) fire a change event
	 */
	SearchLayoutOptions.prototype._fireChange = function() {
		this._changed = false;
		if(this._fireindex===undefined){
			this._fireindex = 0;
		}
		this._fireindex++;
		// Fire the change event
		$.each(this._changeCallbacks, function(i, callback) {
			callback();
		});
	};
	
	return SearchLayoutOptions;
});