/**
 * This class will contain all the "additional" data which is send from the TomCat server...
 */
define([
	"./SearchField",
	"../dataobjects/FieldProperties",
	"../dataobjects/ValueData",
	"../dataobjects/ValueDataUtils",
	"../dataobjects/ExternalSearchSettings",
	"utils/webi18n!SearchInputs",
	"underscore"
], function (
	SearchField,
	FieldProperties,
	ValueData,
	ValueDataUtils,
	ExternalSearchSettings,
	i18n,
	_
) {
	"use strict";
	/**
	 * @constructor
	 */
	var SearchModelFrontEndData = function (json) {

		// Which type of search is this?
		this.searchtype = null;
		
		// which information type are we dealing with
		this.informationtype = null;

		// Information about the fields which are used on the search page
		// Map of String -> SearchField
		this.fieldmap = {};
		
		// Information about the facets which are used on the search page
		// Map of String -> {isMultiValued:bool}
		this.facetmap = {};

		// Information about unit conversions which are used on the search page
		// Map of String -> String -> Float
		this.conversionmap = {};

		// Preferences (used to initialize ExternalSearchSettings object > see )
		// Map of String -> String
		this.preferences = {};

		//A map of DataValue objects that represent Static Values for individual fields (used to populate dropdown-type fields)
		//Map of String > DataValue[]
		this.listValuesMap = {};

		// Some information for the dynamic loading of field information
		this._fieldsloadedcallbacks = {};
		this._fieldsloading = {};
		this._fieldspending = {};
		this._fieldspendingarr = [];

		this._additionalFieldListeners = {};

		// Import all this information
		this._importJSON(json);

		// Make the Drill down Field
		this._makeDrilldownField();
	};

	/**
	 * Whether this is a task search
	 */
	SearchModelFrontEndData.prototype.isTaskSearch = function () {
		// TODO: we should probably simply pass the SearchInformationTypeEnum from the Java side instead !!
		return this.searchtype === "wctask_search";
	};

	/**
	 * Create the drilldown SearchField
	 */
	SearchModelFrontEndData.prototype._makeDrilldownField = function () {
		var field = new SearchField("drilldown");
		field.cellDisplayType = "drilldown";
		this.fieldmap["drilldown"] = field;
	};

	/**
	 * (private) import the fields JSON
	 * @param fields
	 */
	SearchModelFrontEndData.prototype._importFieldJSON = function (fields) {
		for (var key in fields) {
			if (fields.hasOwnProperty(key)) {
				var fieldId = key;
				var fieldJSON = fields[key];
				var field = new SearchField(fieldId);
				field.cellDisplayType = fieldJSON["cell"];
				field.parameterFieldType = fieldJSON["param"];
				field.label = fieldJSON["lbl"];

				field.supportsViewList = fieldJSON["sv"];
				field.supportsBatchActions = fieldJSON["sb"];
				field.supportsDesignPanel = fieldJSON["sd"];
				field.supportsSorting = fieldJSON["ss"];

				field.instancingType = fieldJSON["itype"];

				if (fieldJSON["unit"]) {
					field.unitMetric = fieldJSON["unit"]["met"];
					field.unitImperial = fieldJSON["unit"]["imp"];
					field.unitLucene = fieldJSON["unit"]["luc"];
				}
				field.dataType = fieldJSON["type"];

				ValueDataUtils.fromJSON(field.displayConfig, fieldJSON["cnf"]);

				if (fieldJSON["staticListValues"]) {
					field.staticListValues = [];
					for (var i = 0; i < fieldJSON["staticListValues"].length; i++) {
						var curStaticValData = new ValueData();
						ValueDataUtils.fromJSON(curStaticValData, fieldJSON["staticListValues"][i]);
						field.staticListValues.push(curStaticValData);
					}
				}

				if (fieldJSON["valueListID"]) {
					field.listValueID = fieldJSON["valueListID"];
				}

				this.fieldmap[fieldId] = field;
				
				// Read the information about all the supported facets also
				if(_.has(fieldJSON, "facetids")) {
					var isMultiValued = fieldJSON["facetmultiv"];
					
					var facetids = fieldJSON["facetids"];
					for(var i=0;i<facetids.length;i++) {
						var facetid = facetids[i];
						// Information about this facet...
						this.facetmap[facetid] = {
							label:field.label,
							isMultiValued:isMultiValued
						};
					}
				}

				// Call potential listeners
				if (this._fieldsloadedcallbacks.hasOwnProperty(fieldId)) {
					_.each(this._fieldsloadedcallbacks[fieldId], function (callback) {
						callback();
					});
					delete this._fieldsloadedcallbacks[fieldId];
				}
			}
		}
	};

	/**
	 * (private) import from JSON
	 * 
	 * SEE Java method @ SearchModelFrontEndData:write()
	 */
	SearchModelFrontEndData.prototype._importJSON = function (json) {
		if (!json) return;

		// Read the type of search
		this.searchtype = json["searchtype"];
		
		this.informationtype = json.informationtype;

		// Read the information about the used fields...
		var fields = json["fields"];
		this._importFieldJSON(fields);

		// Read the information about the unit conversions used
		var unitconversions = json["unitconv"];
		for (var i = 0; i < unitconversions.length; i++) {
			var unitconv = unitconversions[i];
			var from = unitconv["from"];
			var to = unitconv["to"];
			var factor = unitconv["factor"];
			if (!this.conversionmap.hasOwnProperty(from)) {
				this.conversionmap[from] = {};
			}
			if (!this.conversionmap[from].hasOwnProperty(to)) {
				this.conversionmap[from][to] = {};
			}
			this.conversionmap[from][to] = factor;
		}

		// Read the information about the preferences used
		this.preferences = json["prefs"] || {};

		//Read the information about Static List values
		var listValuesJSON = json["listvaluesmap"] || {};
		this.listValuesMap = this._readListValues(listValuesJSON);
	};

	//
	// Getters
	//
	/**
	 * Get the information about a field
	 * 
	 * @returns {SearchField} the information about the field
	 */
	SearchModelFrontEndData.prototype.getFieldInformation = function (fieldid, searchSetup) {
		if (!this.fieldmap[fieldid]){
			// we might be instancing another field
			var instancedFieldID = this._splitInstancedFieldId(fieldid);
			var searchField = this.fieldmap[instancedFieldID.fieldid];
			
			
			if (searchField && searchField.instancingType === "MULTI_FIELD_SINGLE_SELECTION"){						
				var iFC = searchSetup.getInstancedFieldConfiguration().getInstance(fieldid);
				
				// the only supported instance type currently
				var instance = searchField.createInstance(fieldid, instancedFieldID.fieldid, iFC);

				// create a clone with a copy of the instanced information
				instance.parameterFieldType = iFC.getValue("parameterFieldType")? iFC.getValue("parameterFieldType") : "";
				instance.numberOfFacets = iFC.getValue("numberOfFacets")? iFC.getValue("numberOfFacets") : "";
				
				// set isMultiValued property based on the corresponding property of the facet
				var facetids = iFC.getSubList("facetids");
				// currently this is an array with only one item (we don't support combined facets yet)
				if(facetids && iFC.getSubList("facetids")[0]){
					var facetConfig = iFC.getSubList("facetids")[0];
					var facetId = facetConfig.getValue("id");
					var facetIdData = this._splitFacetId(facetId);
					instance.isMultiValued = this._getFacetInfo(facetIdData.subfacetid).isMultiValued;		
				}

				this.fieldmap[fieldid] = instance;
			}
			
		}
		
		return this.fieldmap[fieldid];
	
	};
	
	/**
	 * Splits the facet id in a subfacetid and a "variation".
	 * 
	 * @private
	 * @param {string} facetid - the facetid to split
	 * @param {object} {subfacetid: <<fieldidofthefacet>>, variation: <<the variation of the facet>>}
	 */
	SearchModelFrontEndData.prototype._splitFacetId = function(facetId) {
		var splittedFacetId = facetId.split(':');
		var facettedFieldId = splittedFacetId[0];
		var variation = splittedFacetId[1] || null;
		return {subfacetid:facettedFieldId, variation:variation};
	};
	
	SearchModelFrontEndData.prototype._getFacetInfo = function(facetId) {
		if(_.has(this.facetmap, facetId)) {
			return this.facetmap[facetId];
		} else {
			return {
				label:"??",
				isMultiValued:false
			};
		}
	}

	/**
	 * Split the instanced field id in to fieldid and instancedid
	 * @private
	 * @param   {string} fieldid - instanced field id
	 * @returns {object} {fieldid: <<fieldidoftheinstancedfield>>,instanceid: <<instanceid>>}
	 */
	SearchModelFrontEndData.prototype._splitInstancedFieldId = function (fieldid) {
		var chunks = fieldid.split('~');

		return {
			fieldid: chunks[0],
			instanceid: chunks[1]
		};
	};

	/**
	 * Make sure the field information for this field is available!
	 * 
	 * If the information is available, the callback will be called immediately!
	 * Otherwise, it will have some delay while the fieldinformation is loaded...
	 */
	SearchModelFrontEndData.prototype.requireFieldInformationAvailable = function (fieldid, callback) {
		// handle instanced fields
		fieldid = this._splitInstancedFieldId(fieldid).fieldid;
		
		if (_.has(this.fieldmap, fieldid)) {
			// Ok, field was loaded already
			callback();
		} else {
			// This field was not used yet on this page... Need to load the information for this field...

			// Is this the first time we need this field?
			if (!(this._fieldsloading.hasOwnProperty(fieldid) || this._fieldspending.hasOwnProperty(fieldid))) {
				// First time we need this field...
				this._fieldsloadedcallbacks[fieldid] = [];
				this._fieldspending[fieldid] = "1";
				this._fieldspendingarr.push(fieldid);
			}
			// Call callback when done loading...
			this._fieldsloadedcallbacks[fieldid].push(callback);

			// Start the loading the next frame!
			_.defer(_.bind(this._doLoadExtraFieldInformation, this));
		}
	};

	/**
	 * Load the configuration and information of some additional fields...
	 */
	SearchModelFrontEndData.prototype._doLoadExtraFieldInformation = function () {
		if (this._fieldspendingarr.length > 0) {
			_.each(this._fieldspendingarr, _.bind(function (fieldid) {
				this._fieldsloading[fieldid] = "1";
			}, this));
			var pending = this._fieldspendingarr;
			this._fieldspendingarr = [];
			this._fieldspending = {};

			var opts = {
				action: "getfielddata",
				searchtype: this.searchtype,
				fieldids: pending,
				ajax: "1"
			};

			$.ajax("searchdataservice.wcr", {
				context: this,
				cache: false,
				dataType: "json",
				method: "POST",
				data: $.param(opts, true),
				success: function (data, textStatus, jqXHR) {
					if (data.hasOwnProperty("error")) {
						var errorData = data["error"];
						var errorMsg = "";
						if (errorData.hasOwnProperty("msg")) {
							errorMsg = errorData["msg"] + "\n";
						}
						if (errorData.hasOwnProperty("addonMsg")) {
							errorMsg = errorMsg + errorData["addonMsg"];
						}

						alert(errorMsg);
					} else if (data.hasOwnProperty("result")) {
						var JSONRoot = data["result"];

						//read the Value List data if needed
						//NOTE: have to run before reading the fields data as the List Value map 
						//has to be initialized before makeFieldDataForField is called 
						var listValuesJSON = JSONRoot["listvaluesmap"] || {};
						var listValuesMap = this._readListValues(listValuesJSON);
						
						// Update the this.listValuesMap using listValuesMap
						_.each(_.keys(listValuesMap), _.bind(function(key) {
							this.listValuesMap[key] = listValuesMap[key];
						}, this));

						//read the Field information
						this._importFieldJSON(JSONRoot["fields"]);
					}
				},
				error: function (jqXHR, textStatus, errorThrown) {
					console.log("Could not load the extra field information...");
				}
			});
		}

	};

	/**
	 * Get the conversion factor for units...
	 * 
	 * @returns {Number} the conversion factor
	 */
	SearchModelFrontEndData.prototype.getUnitConversionFactor = function (from, to) {
		var factorFromTo = (this.conversionmap[from] || {})[to];
		var factorToFrom = (this.conversionmap[to] || {})[from];
		if (factorFromTo != null && factorFromTo != undefined) {
			return factorFromTo;
		} else if (factorToFrom != null && factorToFrom != undefined) {
			return 1 / factorToFrom;
		} else {
			return 1;
		}
	};

	/**
	 * <p>Make a ExternalSearchSettings object...</br>
	 * This object contains information about general WebCenter settings that are not related to the currently processed Search Setup</p>
	 * 
	 * @param
	 * @returns {ExternalSearchSettings} a ExternalSearchSettings object initialized by internal preferences array that holds all the available options
	 */
	SearchModelFrontEndData.prototype.getSearchSettings = function () {
		// Initialize and return ExternalSearchSettings using the "preferences" field
		return new ExternalSearchSettings(this.preferences);
	};

	/**
	 * Reads the passed JSON data for List Values map and turns it into a map of DataValue objects per fields (which serves as a key in this map)
	 * 
	 * @param listValuesJSON
	 * @returns
	 */
	SearchModelFrontEndData.prototype._readListValues = function (listValuesJSON) {
		var listValuesMap = {};

		for (var key in listValuesJSON) {
			if (listValuesJSON.hasOwnProperty(key)) {
				var itemsList = [];

				//browse through the list of ValueData objects - each represents a static value item
				var fieldValuesList = listValuesJSON[key];
				if (fieldValuesList instanceof Array) {
					for (var i = 0; i < fieldValuesList.length; i++) {
						var curStaticListValueItem = fieldValuesList[i];
						if (curStaticListValueItem) {
							var listItemValueData = new ValueData();
							ValueDataUtils.fromJSON(listItemValueData, fieldValuesList[i]);
							itemsList.push(listItemValueData);
						}
					}
				}

				listValuesMap[key] = itemsList;
			}
		}

		return listValuesMap;
	};

	SearchModelFrontEndData.prototype.getList = function (globallistid) {
		return this.listValuesMap[globallistid];
	};

	/**
	 * <p>Make a FieldProperties object...</br>
	 * This object contains information about the field like units and will be exposed to the implementation of the field (a datarenderer or a parameterfield)</p>
	 * 
	 * @param {SearchField} searchfield - the SearchField
	 * @param {ExpandMode} expandMode - the expand mode to pass... Defines how additional information is shown...
	 * @param {SearchLayoutOptions} layoutOptions - the layout options of the current search setup. Could be used to configure some of the fields.
	 * @param {Function} childSearch - function to call to do a child search
	 * @returns {FieldProperties} a FieldProperties object
	 */
	SearchModelFrontEndData.prototype.makeFieldDataForField = function (searchfield, expandMode, layoutOptions, childSearch) {
		if (!(searchfield && searchfield instanceof SearchField)) {
			throw new Error("Illegal arguments passed to SearchModelFrontEndData.makeFieldDataForField");
		}
		if (!(expandMode && typeof (expandMode["show"]) === "function")) {
			throw new Error("Illegal arguments passed to SearchModelFrontEndData.makeFieldDataForField");
		}
		if (!(layoutOptions && typeof (layoutOptions["getOption"]) === "function")) {
			throw new Error("Illegal arguments passed to SearchModelFrontEndData.makeFieldDataForField");
		}

		// Look up the unit information
		var unitUserPreference = this.getSearchSettings().getUserUnitsPreference();

		/** @type String */
		var targetUnit = (unitUserPreference === "metric" ? searchfield.unitMetric : searchfield.unitImperial);

		// Make the object with all the information which is exposed to the components.
		var fieldProperties = new FieldProperties();
		fieldProperties.displayUnit = targetUnit;
		fieldProperties.unitConversionFactor = this.getUnitConversionFactor(searchfield.unitLucene, targetUnit);
		fieldProperties.dataType = searchfield.dataType;
		fieldProperties.displayConfig = ValueDataUtils.clone(searchfield.displayConfig);
		fieldProperties.expandMode = expandMode;
		fieldProperties.childSearch = childSearch;

		//read Static List Values for this field [OPTIONAL]
		fieldProperties.staticListValues = searchfield.staticListValues || [];

		//read List Values (stored in a map) [OPTIONAL]
		fieldProperties.listValues = [];
		if (searchfield.listValueID) {
			fieldProperties.listValues = this.listValuesMap[searchfield.listValueID] || [];
		}
		
		this._applyDynamicFieldDataConfiguration(searchfield, fieldProperties, layoutOptions);

		// Return it
		return fieldProperties;
	};

	/**
	 * Some fields have different configuration based on some preferences
	 */
	SearchModelFrontEndData.prototype._applyDynamicFieldDataConfiguration = function (searchfield, fieldProperties, layoutOptions) {
		//
		// Apply preference of "date time" for attributes...
		//
		var displayConfigDateRenderType = fieldProperties.displayConfig.getValue("date_render_type");
		if (displayConfigDateRenderType === "attribute_preference") {
			// retrieve the dedicated preference from ExternalSearchSettings
			fieldProperties.displayConfig.setValue("date_render_type", this.getSearchSettings().getDateAttrRenderType());
		}

		//
		// Apply the preference for "project status" for the project status field
		//
		if (searchfield.fieldId === "db_status") {
			var statusstyle = layoutOptions.getOption("SelectProjectStatusStyle");
			var showStatusIcon = "0";
			var showStatusLabel = "0";
			var showColorLabel = "0";

			if (statusstyle === "LabelAndIcon") {
				showStatusIcon = "1";
				showStatusLabel = "1";
			} else if (statusstyle === "OnlyLabel") {

			} else if (statusstyle === "ColorLabel") {
				showColorLabel = "1";
			} else if (statusstyle === "OnlyIcon") {
				showStatusIcon = "1";
			} else {
				// Automatic > either "auto" was the preference or this is a document/task search
				showStatusIcon = "auto"; //NOTE: this means > follow the "thumbnail setting" of the Search instead
				showStatusLabel = "1";
			}
			if (fieldProperties.displayConfig.getValue("show_status_icon") === "preference") {
				fieldProperties.displayConfig.setValue("show_status_icon", showStatusIcon);
			}
			if (fieldProperties.displayConfig.getValue("show_label") === "preference") {
				fieldProperties.displayConfig.setValue("show_label", showStatusLabel);
			}
			if (fieldProperties.displayConfig.getValue("show_color_label") === "preference") {
				fieldProperties.displayConfig.setValue("show_color_label", showColorLabel);
			}
		}

		//
		// Apply the preference for "drill down"
		//
		if (searchfield.fieldId === "drilldown") {
			try {
				fieldProperties.displayConfig.setValue("savedsearch", layoutOptions.getOption("selectDrillDownSavedSearchName"));
				fieldProperties.displayConfig.setValue("useParentLayout", layoutOptions.getOption("chkbox_mergeDrilldown") ? "1" : "0");
			} catch(e) {
				// savedsearch or useParentLayout is not set.
			}
		}
		
		if (searchfield.instancingType === "MULTI_FIELD_SINGLE_SELECTION"){
			// add instance information
			if (searchfield.instancingSource === "db_facet" && searchfield.instanceConfiguration){
				// set all extra properties for facets
				fieldProperties.facetIds = _.map(searchfield.instanceConfiguration.getSubList("facetids"), function(facet){
					// Get the facet id from the configuration...
					var facetid = facet.getValue("id", "db_facet");
					
					// To a parameterfield we do not want to pass the "variation" part of a facet id. 
					// (the JavaScript components do not need to know) 
					// The components only needs to know about the fieldid part of the facetid.
					var facetIdData = this._splitFacetId(facetid);
					
					return facetIdData.subfacetid;
				}, this);
				fieldProperties.getFacetName = _.bind(function(facetid) {
					// Get the label for the facet.
					var facetinfo = this._getFacetInfo(facetid);
					if(facetinfo) {
						// NOTE: need to customize with label from the design panel???
						return facetinfo.label;
					} else {
						return "??";
					}
				}, this);
				fieldProperties.getFacetValueData = _.bind(function(facetid, id) {
					return ((this._facetValueInfo || {})[facetid] || {})[id];
				}, this);
			}
			fieldProperties.instanceConfig = searchfield.instanceConfiguration;
		}	
		fieldProperties.isMultiValued = searchfield.isMultiValued;
	};


	/**
	 * Observe additional field (facet) data with a certain id
	 * @param {String}   fieldId
 	 * @param {Function} callback - called whenver facet is updated
	 */
	SearchModelFrontEndData.prototype.observeAdditionalFieldData = function (fieldId, callback) {
		if (!this._additionalFieldListeners[fieldId]) {
			this._additionalFieldListeners[fieldId] = [];
		}

		this._additionalFieldListeners[fieldId].push(callback);
	};

	/**
	 * Stop oberserving the additional field data
	 * @param {String}   fieldId
	 * @param {Function} callback - should be exactly the same as used for observeAdditionalFieldData()
	 */
	SearchModelFrontEndData.prototype.stopObserveAdditionalFieldData = function (fieldId, callback) {
		if (this._additionalFieldListeners[fieldId]) {
			this._additionalFieldListeners[fieldId] = _.without(this._additionalFieldListeners[fieldId], callback);
		}
	};


	/**
	 * Set the Additional Field Data (facet information)
	 * @param   {Object}   additionalFieldData {field_id: ValueData}
	 */
	SearchModelFrontEndData.prototype.setAdditionalFieldData = function(additionalFieldData){

		var additionalFieldData = _.mapObject(additionalFieldData, function(value, key){
			var aFData = new ValueData();
			ValueDataUtils.fromJSON(aFData, value);
			return aFData;
		});
		
		// Also extract the facet information for fast lookup
		this._facetValueInfo = {};
		
		// Get all the facet information
		var allFacetInformation = additionalFieldData["db_facet"];
		// ... and run through all the facets...
		if(allFacetInformation) {
			_.each(allFacetInformation.getSubListKeys(), function(id) {
				this._facetValueInfo[id] = {};
				
				var facetValues = allFacetInformation.getSubList(id);
				_.each(facetValues, function(facetValue) {
					this._facetValueInfo[id][facetValue.getValue("id")] = facetValue;
				}, this);
			}, this);
		}

		// Call listeners
		this._updateAdditionalFieldData(additionalFieldData);
	};

	/**
	 * Get the additional field data (ex. Facet Data) for a certain field ID
	 * @param   {String}    fieldId 
	 * @returns {ValueData} additional field information	
	 */
	SearchModelFrontEndData.prototype.getAdditionalFieldData = function (fieldId) {
		if (!this._additionalFieldData) {
			this._additionalFieldData = {};
		}

		if (!this._additionalFieldData[fieldId]) {
			this._additionalFieldData[fieldId] = new ValueData();
			this._additionalFieldData[fieldId].setSubList("facet", []);
		}

		return this._additionalFieldData[fieldId];
	};

	/**
	 * Called on search rerun, update all facet data
	 * @param   {Any}      data - new facet data
	 */
	SearchModelFrontEndData.prototype._updateAdditionalFieldData = function (additionalFieldData) {
		this._additionalFieldData = additionalFieldData;
		_.each(this._additionalFieldListeners, function (listeners, fieldId) {
			var aFData = this.getAdditionalFieldData(fieldId);

			_.each(listeners, function (callback) {
				callback(aFData);
			});
		}, this);
	};

	return SearchModelFrontEndData;
});
