/**
 * The SearchSetupInformation model
 * 
 * Note: Also available on the Java side !
 * 
 * Note: Only contains information about "new search architecture" fields
 */
define([
        "./SearchCriteriaList",
        "./SearchDisplayColumns",
        "./SearchDesignPanelItems",
        "./SearchSortingCriteriaList",
        "./SearchFieldFeaturesInfo",
        "./SearchLayoutOptions",
		"./SearchDashboardConfiguration",
		"./SearchInstancedFieldConfiguration",
        ], function(
        		SearchCriteriaList,
        		SearchDisplayColumns,
        		SearchDesignPanelItems,
        		SearchSortingCriteriaList,
        		SearchFieldFeaturesInfo,
        		SearchLayoutOptions,
				SearchDashboardConfiguration,
				SearchInstancedFieldConfiguration
				){
	"use strict";
	
	
	/**
	 * @constructor
	 */
	var SearchSetupInformation = function(){
        // Features
		this._features = new SearchFieldFeaturesInfo();
		// The criteria
		this._criteria = new SearchCriteriaList();
		// The display columns
		this._displayColumns = new SearchDisplayColumns();
		// The design panel settings
		this._designPanelItems = new SearchDesignPanelItems();
		// The sorting criteria settings
		this._sortingCriteria = new SearchSortingCriteriaList();
		// The layout options
		this._layoutoptions = new SearchLayoutOptions();
		// Dashboard configuration 
		this._searchDashboardConfiguration = new SearchDashboardConfiguration();
		// Instanced fields configuration
		this._searchInstancedFieldConfiguration = new SearchInstancedFieldConfiguration();
	};
	
	/**
	 * Get the display columns
	 * 
	 * @return {SearchDisplayColumns} list of display columns
	 */
	SearchSetupInformation.prototype.getDisplayColumns = function() {
		return this._displayColumns;
	};
	
	/**
	 * Get the list of Search Criterion
	 * 
	 * @return {SearchCriteriaList} list of criteria
	 */
	SearchSetupInformation.prototype.getCriteriaList = function() {
		return this._criteria;
	};
	
	/**
	 * Get the list of Design Panel Items
	 * 
	 * @return {SearchDesignPanelItems} list of design panel items
	 */
	SearchSetupInformation.prototype.getDesignPanelItems = function() {
		return this._designPanelItems;
	};
	
	/**
	 * Get the list of Search Criterion
	 * 
	 * @return {SearchSortingCriteriaList} list of criteria
	 */
	SearchSetupInformation.prototype.getSortingCriteriaList = function() {
		return this._sortingCriteria;
	};
	
	/**
	 * Get the "features" which are activated for all the criteria...
	 * 
	 * @return {SearchFieldFeaturesInfo}
	 */
	SearchSetupInformation.prototype.getFeatures = function() {
		return this._features;
	};
	
	/**
	 * Get the Layout Options (all kinds of settings)
	 * 
	 * @returns {SearchLayoutOptions}
	 */
	SearchSetupInformation.prototype.getLayoutOptions = function() {
		return this._layoutoptions;
	};
	
	/**
	 * Get the configuration object for the search dashboard
	 * @returns {SearchDashboardConfiguration} 
	 */
	SearchSetupInformation.prototype.getDashboardConfiguration = function(){
		return this._searchDashboardConfiguration;	
	};
	
	/**
	 * Get the configuration object for the instanced fields
	 * @returns {SearchInstancedFieldConfiguration} 
	 */
	SearchSetupInformation.prototype.getInstancedFieldConfiguration = function(){
		return this._searchInstancedFieldConfiguration;	
	};
	
	
	//
	// Additional methods for ease of use
	//
	
	/**
	 * Does this setup have an asset browser configured?
	 * 
	 * @returns {Boolean}
	 */
	SearchSetupInformation.prototype.hasAssetBrowser = function(){
		var cascadingdata = this.getLayoutOptions().getOption("cascading_attribute_trees");
		return cascadingdata!==null && cascadingdata.getSubList("treeElem")!=null && cascadingdata.getSubList("treeElem").length>0; 
	};
	
	/**
	 * Does this setup have facet graphs/lists/... configured?
	 * 
	 * @returns {Boolean}
	 */
	SearchSetupInformation.prototype.hasFacetComponents = function() {
		return this.getInstancedFieldConfiguration().hasInstancedOfType("db_facet");
	};
	
	/**
	 * Does this setup have design panel items configured? (either visible or available)
	 * 
	 * @returns {Boolean}
	 */
	SearchSetupInformation.prototype.hasDesignPanel = function() {
		var hasDesignPanelItems = false;
		this.getDesignPanelItems().each(function (item) {
			var visibility = item.getOriginalVisibility();
			if (visibility === "Available" || visibility === "Visible") {
				hasDesignPanelItems = true;
			}
		});
		return hasDesignPanelItems;
	};
	
	/**
	 * Does this search have anything configured that can trigger a rerun for which we need a search button?
	 * 
	 * @returns {Boolean}
	 */
	SearchSetupInformation.prototype.hasRerunableComponents = function() {
		// WATCH OUT: a similar check is still implemented on the Java side (Java side implementation to be removed in the future) => see SearchModelUtils.java:hasRerunableComponents
		return this.hasAssetBrowser() || this.hasFacetComponents() || this.hasDesignPanel();
	};
	
	/**
	 * Do we have a search button for this search?
	 * 
	 * @returns {Boolean}
	 */
	SearchSetupInformation.prototype.hasSearchButton = function() {
		var isshowsearchbuttonoption = this.getLayoutOptions().getOption("chkbox_ShowSearchButton");
		return isshowsearchbuttonoption && this.hasRerunableComponents();
	};
	
	return SearchSetupInformation;
});
