define(["jquery", "underscore", "./SearchItemBase"], function($, _, SearchItemBase){
	
	/**
	 * @constructor
	 */
	var SearchSortingCriterion = function(fieldId) {
		SearchItemBase.call(this, fieldId);
		
		this._enabled = false;
		this._order = "asc";
		this._number = "-1";
	};
	
	// Extend from SearchItemBase to get listener functionality and such
	SearchSortingCriterion.prototype = Object.create(SearchItemBase.prototype);
	SearchSortingCriterion.prototype.constructor = SearchSortingCriterion;
	
	// 
	// Usage Tracking
	//
	// IF YOU KEEP A REFERENCE to this object, YOU NEED to call the methods:
	//
	// + take = Call this when you start keeping a reference to this object!
	// + release = Call this when you no longer keep a reference to the object!
	//
	// Used by the parent list (to see if there are still references) when removing items:
	// + canBeRemovedFromModel => Check if this item can be removed from the model. (no-one is keeping a reference)
	//
	
	//
	// Other methods
	//
	// + getFieldId = Get the fieldId of the sorting criterion
	// + onChange = Register a new change listener
	// + commitChanges = We don't want to invoke change listeners while the change we are doing is not complete.
	// 				So, in general, we invoke the change listeners the frame after changes occured.
	// 
	// 				However, sometimes you want to do it sooner. That is when you call this method.
	// 				E.g.: you resorted all criteria and still want to update the UI (which listens to the changes) the same frame.
	// 
	// 				Note: you do not need to call this method! (but you probably should)
	// + removeOnChange = Remove a previously registered change listener...
	//
	
	
	//
	// Actual information
	//
	
	/**
	 * Whether sorting is enabled on this field
	 * 
	 * @returns {bool}
	 */
	SearchSortingCriterion.prototype.getEnabled = function() {
		return this._enabled;
	};
	
	/**
	 * Whether sorting is enabled on this field
	 * 
	 * @param {bool} enabled
	 */
	SearchSortingCriterion.prototype.setEnabled = function(enabled) {
		if(this._enabled != enabled) {
			this._enabled = enabled;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The order of this sorting criterion ("asc"/?)
	 * 
	 * @returns {String}
	 */
	SearchSortingCriterion.prototype.getOrder = function() {
		return this._order;
	};
	
	/**
	 * The order of this sorting criterion ("asc"/?)
	 * 
	 * @param {String} order
	 */
	SearchSortingCriterion.prototype.setOrder = function(order) {
		if(this._order != order) {
			this._order = order;
			this._fireChangeDefer();
		}
	};
	
	/**
	 * The number of this sorting criterion
	 * 
	 * @returns {String} - a string because it can be "empty" (empty string)
	 */
	SearchSortingCriterion.prototype.getNumber = function() {
		return this._number;
	};
	
	/**
	 * The number of this sorting criterion
	 * 
	 * @param {String} number
	 */
	SearchSortingCriterion.prototype.setNumber = function(number) {
		if(this._number != number) {
			this._number = number;
			this._fireChangeDefer();
		}
	};
	
	//
	// Other util methods
	//
	/**
	 * (Blind) copy from a certain criterion.
	 * 
	 * @param {SearchSortingCriterion} srcCriterion - the criterion to copy from
	 */
	SearchSortingCriterion.prototype.copyFrom = function(srcCriterion) {
		this.setEnabled(srcCriterion.getEnabled());
		this.setOrder(srcCriterion.getOrder());
		this.setNumber(srcCriterion.getNumber());
	};
	
	/**
	 * Copy from a certain criterion.
	 * This method does not do a blind copy but first checks whether this field supports sorting.
	 * 
	 * @param {SearchField} searchfield - the SearchField containing information about the field for this criterion
	 * @param {SearchSortingCriterion} srcCriterion - the criterion to copy from
	 */
	SearchSortingCriterion.prototype.safeCopyFrom = function(searchfield, srcCriterion) {
		if(searchfield.supportsSorting) {
			this.copyFrom(srcCriterion);
		}
	};
	
	return SearchSortingCriterion;
});