/**
 * Reads the information for the SearchSetupInformation object which is send from the server.
 * 
 * Note: See Java class with same name on server side!
 */
define([
        "../SearchSetupInformation",
        "../SearchCriterionInfo",
        "../SearchDesignPanelItem",
        "../SearchDisplayColumn",
        "../SearchSortingCriterion",
        "../SearchLayoutOptions",
        "../../dataobjects/ValueData",
        "../../dataobjects/ValueDataUtils",
		"underscore"
        ], function (
	// The model
	SearchSetupInformation,
	// A criterion
	SearchCriterionInfo,
	// A design panel item
	SearchDesignPanelItem,
	// A display column
	SearchDisplayColumn,
	// A sorting criterion
	SearchSortingCriterion,
	// The Layout Options section
	SearchLayoutOptions,

	ValueData,
	ValueDataUtils,
	_
) {

	/**
	 * Make a new SearchModelTransferedJsonReader to read the model
	 * @constructor
	 * @param {SearchSetupInformation} searchSetup - the model to fill
	 */
	var SearchModelTransferedJsonReader = function (searchSetup) {
		if (!(searchSetup && searchSetup instanceof SearchSetupInformation)) {
			throw new Error("Wrong parameter... No SearchSetupInformation object passed to SearchModelTransferedJsonReader.");
		}

		// The model we will fill
		this.model = searchSetup;
	};

	/**
	 * Read the JSON which was transfered from the server...
	 * 
	 * @param {Object} json - the json
	 */
	SearchModelTransferedJsonReader.prototype.read = function (json) {
		// Read the features !
		this._parseFeatures(json);

		// Read the actual criteria !
		this._parseCriteria(json);

		// Read the design panel items
		this._parseDesignPanelItems(json);

		// Read the design panel items
		this._parseDisplayColumns(json);

		// Read the design panel items
		this._parseSortingCriteria(json);

		// Read the layout options
		this._parseLayoutOptions(json);

		// parse the dashboard configuration
		this._parseDashboardConfiguration(json);

		// parse the instanced field configuration
		this._parseInstancedFieldConfiguration(json);
	};

	/**
	 * (private) Parse the search features
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseFeatures = function (json) {
		var featuresJson = json["features"] || {};
		var features = this.model.getFeatures();
		features.setUseSmartNames(featuresJson["smartnames"] === true);
		features.setUseAdvancedChars(featuresJson["advchars"] === true);
	};

	/**
	 * (private) Parse the layout options
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseLayoutOptions = function (json) {
		var options = json["options"] || [];

		var availableoptions = [];
		var values = [];

		_.each(options, function (option) {
			var id = option.id;
			var type = option.type;
			var defaultValue = null;
			var value = null;
			if (type === "BOOLEAN" || type === "STRING") {
				defaultValue = option.defaultValue;
				if (option["value"] !== null && option["value"] !== undefined) {
					value = option["value"];
				}
			} else if (type === "COMPLEXVALUE") {
				if (option["value"]) {
					value = new ValueData();
					ValueDataUtils.fromJSON(value, option["value"]);
				}
			} else if (type === "ITEMSEQUENCE") {
				if (option["value"]) {
					value = option["value"];
				}
			}

			availableoptions.push(new SearchLayoutOptions.Setting(id, type, defaultValue));
			if (value !== null) {
				values.push({
					id: id,
					value: value
				});
			}
		});

		this.model.getLayoutOptions().setAvailableSettings(availableoptions);

		_.each(values, function (val) {
			this.model.getLayoutOptions().setOption(val.id, val.value);
		}, this);
	};

	/**
	 * (private) Parse the criteria
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseCriteria = function (json) {
		var criteria = json["criteria"] || [];
		for (var i = 0; i < criteria.length; i++) {
			var criterion = criteria[i];
			var fieldId = criterion["id"];
			var containsSmartName = criterion["SN"];
			var valueData = criterion["valueData"] || {};

			// Import the valueData information
			var valueDataObj = new ValueData();
			ValueDataUtils.fromJSON(valueDataObj, valueData);

			// Make the criterion
			var criterionInfoObj = new SearchCriterionInfo(fieldId);
			criterionInfoObj.setValue(valueDataObj, containsSmartName);

			// Add the criterion
			this.model.getCriteriaList().add(criterionInfoObj);
		}
		
		// Commit changes => send all potential change events now
		this.model.getCriteriaList().commitChanges();
	};

	/**
	 * (private) Parse the search design panel
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseDesignPanelItems = function (json) {
		var designPanels = json["designPanelItems"] || [];
		for (var i = 0; i < designPanels.length; i++) {
			var designPanel = designPanels[i];

			var fieldId = designPanel["id"];
			var row = designPanel["row"];
			var column = designPanel["column"];
			var label = designPanel["label"];
			var originalVisibility = designPanel["originalVisibility"];
			var visibility = designPanel["visibility"];

			// Make the design panel
			var designPanelItemObj = new SearchDesignPanelItem(fieldId);
			designPanelItemObj.setRow(row);
			designPanelItemObj.setColumn(column);
			designPanelItemObj.setLabel(label);
			designPanelItemObj.setOriginalVisibility(originalVisibility);
			designPanelItemObj.setVisibility(visibility);

			// Add the criterion
			this.model.getDesignPanelItems().add(designPanelItemObj);
		}
		
		// Commit changes => send all potential change events now
		this.model.getDesignPanelItems().commitChanges();
	};

	/**
	 * (private) Parse the search display columns
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseDisplayColumns = function (json) {
		var displayColumns = json["displayColumns"] || [];
		for (var i = 0; i < displayColumns.length; i++) {
			var displayColumn = displayColumns[i];

			var fieldId = displayColumn["id"];
			var editable = displayColumn["editable"];
			var enabled = displayColumn["enabled"];
			var sequence = displayColumn["sequence"];

			// Make the design panel
			var displayColumnObj = new SearchDisplayColumn(fieldId);
			displayColumnObj.setEnabled(enabled);
			displayColumnObj.setEditable(editable);
			displayColumnObj.setSequence(""+sequence);

			// Add the criterion
			this.model.getDisplayColumns().add(displayColumnObj);
		}
		
		// Commit changes => send all potential change events now
		this.model.getDisplayColumns().commitChanges();
	};

	/**
	 * (private) Parse the search sorting criteria
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonReader.prototype._parseSortingCriteria = function (json) {
		var sortingCriteria = json["sortingCriteria"] || [];
		for (var i = 0; i < sortingCriteria.length; i++) {
			var sortingCriterion = sortingCriteria[i];

			var fieldId = sortingCriterion["id"];
			var order = sortingCriterion["order"];
			var number = sortingCriterion["number"];

			// Make the design panel
			var sortingCriterionObj = new SearchSortingCriterion(fieldId);
			sortingCriterionObj.setEnabled(true);
			sortingCriterionObj.setOrder(order);
			sortingCriterionObj.setNumber(number);

			// Add the criterion
			this.model.getSortingCriteriaList().add(sortingCriterionObj);
		}
		
		// Commit changes => send all potential change events now
		this.model.getSortingCriteriaList().commitChanges();
	};


	/**
	 * Parse the dashboard configuration
	 * @private
	 * @param {object} json 
	 */
	SearchModelTransferedJsonReader.prototype._parseDashboardConfiguration = function (json) {
		if (json.dashboard_configuration) {
			this.model.getDashboardConfiguration().setDashboard(json.dashboard_configuration);
		} else {
			this.model.getDashboardConfiguration().setDashboard(null);
		}
	};

	/**
	 * Parse the instanced field configuration
	 * @private
	 * @param {object} json 
	 */
	SearchModelTransferedJsonReader.prototype._parseInstancedFieldConfiguration = function (json) {
		var instancesJSON = json.instanced_field_configuration;
		var instances = {};
		if (instancesJSON) {
			for (var instanceId in instancesJSON) {
				if (instancesJSON.hasOwnProperty(instanceId)) {
					var vd = new ValueData();
					ValueDataUtils.fromJSON(vd, instancesJSON[instanceId])
					instances[instanceId] = vd;
				}
			}
		}
		this.model.getInstancedFieldConfiguration().setData(instances);
	};

	/**
	 * Static method... Read the json into the model.
	 * @param {SearchSetupInformation} searchSetup - the empty model
	 * @param {Object} json - the json
	 */
	SearchModelTransferedJsonReader.read = function (searchSetup, json) {
		var reader = new SearchModelTransferedJsonReader(searchSetup);
		reader.read(json);
	};

	return SearchModelTransferedJsonReader;
});
