/**
 * Writes the information for the SearchSetupInformation object which is send from the server.
 * 
 * Note: See Java class with same name on server side!
 */
define([
        "../SearchSetupInformation",
        "../SearchCriterionInfo",
        "../../dataobjects/ValueData",
        "../../dataobjects/ValueDataUtils",
		"underscore"
        ], function (
	// The model
	SearchSetupInformation,
	// A criterion
	SearchCriterionInfo,

	ValueData,
	ValueDataUtils,
	_
) {

	/**
	 * Make a new SearchModelTransferedJsonWriter to read the model
	 * @constructor
	 * @param {SearchSetupInformation} searchSetup - the model to fill
	 */
	var SearchModelTransferedJsonWriter = function (searchSetup) {
		if (!(searchSetup && searchSetup instanceof SearchSetupInformation)) {
			throw new Error("Wrong parameter... No SearchSetupInformation object passed to SearchModelTransferedJsonWriter.");
		}

		// The model we will fill
		this.model = searchSetup;
	};

	/**
	 * Write the JSON to transfer to the server...
	 * 
	 * @param {Object} json - the json object to write to
	 */
	SearchModelTransferedJsonWriter.prototype.write = function (json) {
		// Write the Features
		this._writeSearchFeatures(json);
		
		// Write the Layout options
		this._writeLayoutOptions(json);

		// Write the actual criteria !
		this._writeCriteria(json);

		// Write the design panel items
		this._writeDesignPanelItems(json);

		// Write the design panel items
		this._writeDisplayColumns(json);

		// Write the design panel items
		this._writeSortingCriteria(json);

		// write the dashboard configuration
		this._writeDashboardConfiguration(json);

		// write the instanced fields configuration
		this._writeInstancedFieldConfiguration(json);
	};
	
	/**
	 * (private) Write the features
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonWriter.prototype._writeSearchFeatures = function (json) {
		json["features"] = {
			smartnames: this.model.getFeatures().getUseSmartNames(),
			advchars: this.model.getFeatures().getUseAdvancedChars()
		};
	};

	/**
	 * (private) Write the layout options
	 * 
	 * @param json
	 */
	SearchModelTransferedJsonWriter.prototype._writeLayoutOptions = function (json) {
		var options = [];
		var availableoptions = this.model.getLayoutOptions().getAvailableSettings();
		_.each(availableoptions, function (option) {
			var id = option.id;
			var type = option.type;
			var entry = {
				id: id,
				type: type
			};

			var val = this.model.getLayoutOptions().getOption(id);

			if (type === "BOOLEAN" || type === "STRING") {
				entry["defaultValue"] = option.defaultValue;
				if (val !== option.defaultValue) {
					entry.value = val;
				}
			} else if (type === "COMPLEXVALUE") {
				if (val != null) {
					entry.value = ValueDataUtils.toJSON(val);
				}
			} else if (type === "ITEMSEQUENCE") {
				if (val != null) {
					entry.value = val;
				}
			}

			options.push(entry);
		}, this);
		json["options"] = options;
	};

	/**
	 * (private) Write the criteria
	 * @param json - an empty JavaScript object which will be filled
	 */
	SearchModelTransferedJsonWriter.prototype._writeCriteria = function (json) {
		var criteria = [];
		var criteriaListInModel = this.model.getCriteriaList();
		criteriaListInModel.each(function (criterionInfoObj) {

			// Export the valueData information
			var valueDataObj = criterionInfoObj.getValue();
			var valueData = ValueDataUtils.toJSON(valueDataObj);

			// Make the json representation of the criterion
			var criterion = {
				"id": criterionInfoObj.getFieldId(),
				"valueData": valueData,
				"SN": criterionInfoObj.getContainsSmartName()
			};

			// Add the criterion
			criteria.push(criterion);
		});
		json["criteria"] = criteria;
	};

	/**
	 * (private) Write the design panel items
	 * @param json - an empty JavaScript object which will be filled
	 */
	SearchModelTransferedJsonWriter.prototype._writeDesignPanelItems = function (json) {
		var designPanelItems = [];
		var designPanelItemsInModel = this.model.getDesignPanelItems();
		designPanelItemsInModel.each(function (designPanelItemObj) {

			// Make the json representation of the design panel item
			var designPanelItem = {
				"id": designPanelItemObj.getFieldId(),
				"row": designPanelItemObj.getRow(),
				"column": designPanelItemObj.getColumn(),
				"label": designPanelItemObj.getLabel(),
				"originalVisibility": designPanelItemObj.getOriginalVisibility(),
				"visibility": designPanelItemObj.getVisibility()
			};

			// Add the criterion
			designPanelItems.push(designPanelItem);
		});
		json["designPanelItems"] = designPanelItems;
	};

	/**
	 * (private) Write the display column information
	 * @param json - an empty JavaScript object which will be filled
	 */
	SearchModelTransferedJsonWriter.prototype._writeDisplayColumns = function (json) {
		var displayColumns = [];
		var displayColumnsInModel = this.model.getDisplayColumns();
		displayColumnsInModel.each(function (displayColumnObj) {
			if (displayColumnObj.getEnabled() || displayColumnObj.isEditable()) {

				// Make the json representation of the design panel item
				var displayColumn = {
					"id": displayColumnObj.getFieldId(),
					"editable": displayColumnObj.isEditable(),
					"sequence": (isNaN(displayColumnObj.getSequence())?-1:parseInt(displayColumnObj.getSequence(), 10)),
					"enabled": displayColumnObj.getEnabled()
				};

				// Add the criterion
				displayColumns.push(displayColumn);
			}
		});
		json["displayColumns"] = displayColumns;
	};

	/**
	 * (private) Write the sorting criterion
	 * @param json - an empty JavaScript object which will be filled
	 */
	SearchModelTransferedJsonWriter.prototype._writeSortingCriteria = function (json) {
		var sortingCriteria = [];
		var sortingCriteriaInModel = this.model.getSortingCriteriaList();
		sortingCriteriaInModel.each(function (sortingCriterionObj) {
			if (sortingCriterionObj.getEnabled()) {

				// Make the json representation of the design panel item
				var sortingCriterion = {
					"id": sortingCriterionObj.getFieldId(),
					"order": sortingCriterionObj.getOrder(),
					"number": sortingCriterionObj.getNumber()
				};

				// Add the criterion
				sortingCriteria.push(sortingCriterion);
			}
		});
		json["sortingCriteria"] = sortingCriteria;
	};


	/**
	 * Write the dashboard configuration to the JSON
	 * @private
	 * @param {object} json 
	 */
	SearchModelTransferedJsonWriter.prototype._writeDashboardConfiguration = function (json) {
		if (this.model.getDashboardConfiguration().hasDashboard()) {
			json.dashboard_configuration = this.model.getDashboardConfiguration().getDashboard();
		}
	};

	/**
	 * Write the instanced field configuration to the JSON
	 * @private
	 * @param {object} json 
	 */
	SearchModelTransferedJsonWriter.prototype._writeInstancedFieldConfiguration = function (json) {
		if (this.model.getInstancedFieldConfiguration()) {
			var map = this.model.getInstancedFieldConfiguration().getDataClone();
			var jsonMap={};
			if(map !== null){
				_.each(map, function(value, instanceId){
					jsonMap[instanceId] = ValueDataUtils.toJSON(value);
				});
			}
			json["instanced_field_configuration"] = jsonMap;
		}
	};

	/**
	 * Static method... Write the json into the model.
	 * @param {SearchSetupInformation} searchSetup - the empty model
	 * @param {Object} json - an empty JavaScript object
	 */
	SearchModelTransferedJsonWriter.write = function (searchSetup, json) {
		var writer = new SearchModelTransferedJsonWriter(searchSetup);
		writer.write(json);
	};

	return SearchModelTransferedJsonWriter;
});
