/**
 * SmartNames - Encoding of SmartNames & normal text in a string
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartName.java".
 *
 */

define(["../encoding/smartnameargumentencoding"], function (SmartNameArgumentEncoding) {
	"use strict";


	/**
	 * The implementation of this constructor depends on the fact whether the "arguments" parameter is 
	 * specified or not.
	 * 
	 * Situation I: "arguments" is undefined:
	 *  
	 * 		Create a SmartName from the string representation of the smart name.
	 * 
	 * 		The string representation of the smart name concatenates the arguments to the name but seperates
	 * 		them through the ":" character. If an argument contains this character, it needs to be escaped by
	 * 		a "\". Occurences of "\" need to be escaped by "\" themselves.
	 * 
	 * Situation II: "arguments" is not undefined.
	 * 
	 * 		Create a SmartName, from a name and arguments.
	 * 
	 * 		An example of a SmartName with arguments is "[ProjAttr:Artist_Name]", herein is "ProjAttr" the 
	 * 		name and "Artist_Name" is an argument of the SmartName.	
	 * 
	 * @param smartName
	 *          The string representations of the smart name or the name of the SmartName (depending on whether 
	 *          arguments is specified).
	 * @param arguments
	 * 			When specified, the arguments of the SmartName.
	 */
	var SmartName = function () {

		if (arguments.length === 1) {
			// Split the string on ":", the name is the first part, the rest are arguments.
			// When an argument contains ":" chars, they need to be escaped by a "\". Occurences 
			// of "\" need to be escaped by "\" themselves.
			var parts = SmartNameArgumentEncoding.DEFAULT.decodeSmartNameArguments(arguments[0]);

			// The name is always the first entry in the returned array.
			this._name = parts[0];

			// The other elements are arguments.
			if (parts.length > 1) {
				this._arguments = parts.slice(1, parts.length);
			} else {
				this._arguments = [];
			}

		} else if (arguments.length > 1) {

			// name and arguments are seperately provided, no need to parse.
			this._name = arguments[0];

			// If only 2 arguments are given and the second one is an array, these are the SmartName arguments.
			if (arguments.length == 2 && arguments[1].constructor == Array) {
				this._arguments = arguments[1];
			} else {
				// Otherwise all other arguments are the SmartName arguments.
				this._arguments = Array.prototype.slice.call(arguments);
				this._arguments = this._arguments.slice(1, this._arguments.length);
			}


		} else if (arguments.length < 1) {
			throw new Error("A SmartName requires at least one 'name' parameter.");
		}

	};

	(function () {

		/**
		 * Get the name of the SmartName
		 * 
		 * @return [String] the name of the SmartName
		 */
		this.getName = function () {
			return this._name;
		};

		/**
		 * Get the arguments of the SmartName
		 * 
		 * @return [Array] the arguments of the SmartName
		 */
		this.getArguments = function () {
			return this._arguments;
		};

		/**
		 * Return the full name of this string including the contacenated escaped arguments.
		 * 
		 * @return [String] The full name of this string including the contacenated escaped arguments.
		 */
		this.getFullName = function () {
			return SmartNameArgumentEncoding.DEFAULT.encodeSmartName(this._name, this._arguments);
		};

		/**
		 * Are these SmartNames the same?
		 * 
		 * @return [Boolean] Are these SmartNames the same?
		 */
		this.equals = function (other) {
			if (other instanceof SmartName) {
				var nameMatches = other.getName() === this.getName();
				var argumentsMatches = this._compareArguments(other);
				return nameMatches && argumentsMatches;
			} else {
				return false;
			}
		};

		/**
		 * A private helper function of the public "equals" method.
		 */
		this._compareArguments = function (other) {
			if (this._arguments.length === other.getArguments().length) {
				for (var i = 0; i < this._arguments.length; i++) {
					if (this._arguments[i] !== other.getArguments()[i]) {
						return false;
					}
				}
				return true;
			} else {
				return false;
			}
		};

	}).call(SmartName.prototype);

	return SmartName;
});