/**
 * Encodes/Decodes SmartName arguments and names from/to a string.
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartNameArgumentEncoding.java".
 *
 */

define(["utils/wcrerror", 
        "utils/extendedstring"],
        function(WCRError) {
	"use strict";
	var exports={};
	
	var SmartNameArgumentEncoding = function (argumentDelimeterChar, argumentEscapeChar) {
		
		this._delimeterChar = argumentDelimeterChar;
		this._escapeChar = argumentEscapeChar;
	};
	
	(function () {
		
		/**
		 * Encode the name and arguments of a SmartName.
		 * 
		 * Executing {@link #decodeSmartNameArguments(String)} on the result of this method should return exactly the same
		 * input arguments used for this method (name will be the first element in the returned array).
		 * 
		 * @param name [String] The name of the SmartName.
		 * @param arguments [Array] The arguments of the SmartName.
		 * @return [String] The encoded String.
		 */
		this.encodeSmartName = function (name, args) {
			
			// The result...
			var result = name;
			for (var i=0; i <args.length; i++) {
				// Escape every escape char.
				var arg1 = args[i].replaceAll(this._escapeChar, this._escapeChar + this._escapeChar);
				// Escape every delimeter char
				var arg2 = arg1.replaceAll(this._delimeterChar, this._escapeChar + this._delimeterChar);
				// Concatenate the argumente delimited by the argumentDelimeterChar.
				result += this._delimeterChar;
				result += arg2;
			}

			return result.toString();
			
		};
		
		
		/**
		 * Decode a String that represents a SmartName with its arguments.
		 * 
		 * The string representation of the smart name concatenates the arguments to the name but seperates them through the
		 * delimeter character of this encoding. If an argument contains this character, it needs to be escaped by the
		 * escape character of this encoding. Occurences of the escape character need to be escaped by themselves.
		 * 
		 * @param str [String] The String that needs to be decoded.
		 * @return The [Array] first element of this list is the name of the SmartName, followed by the seperate arguments.
		 */
		this.decodeSmartNameArguments = function (str) {
			var result = [];
			
			// This mini FSM parses the string.
			var escaped = false;
			var part = "";
			for (var i = 0; i < str.length; i++) {
                var c = str.charAt(i);
                if (c === this._escapeChar) {
                    if (escaped) {
                        part+=this._escapeChar;
                    }
                    escaped = !escaped;
                } else if (c === this._delimeterChar) {
    				// Check if we are escaped
    				if (escaped) {
    					part += this._delimeterChar;
    					escaped = false;
    				} else {
    					result.push(part);
    					part = "";
    				};
    			} else {
    				if (escaped) {
    					// A normal character was escaped ... throw error
    					throw new WCRError("SMARTNAME_ARGUMENT_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of arguments of SmartName failed. Found a single escape character in the string.");
    				} else {
    					// normal letter, add it.
    					part += c;
    				};
    			};
			}
			// Add the last part the results.
			result.push(part);
			
			// Parsing is done and we should not be escaped anymore ... throw error otherwise
			if (escaped) {
				throw new WCRError("SMARTNAME_ARGUMENT_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of arguments of SmartName failed. Found a single escape character in the string.");
			}

			// Return result
			return result;
		};
		
	}).call(SmartNameArgumentEncoding.prototype);
	
	
	/**
	 * The default SmartNameArgumentEncoding
	 */
	exports.DEFAULT = new SmartNameArgumentEncoding(":","\\");
	
	
	/**
	 * Export Constructor (for JUnit test)
	 */
	exports.constructor = SmartNameArgumentEncoding;
	
	return exports;
});