/**
 * Encodes SmartName information and normal String information in one string.
 */

/*
 *
 * !!! THE IMPLEMENTATION IN THIS FILE SHOULD MATCH THE IMPLEMENTATION IN THE JAVA FILE "SmartNameEncoding.java".
 *
 */

define(["./smartnamestringpart",
        "./smartname",
        "utils/wcrerror", "underscore", "utils/extendedstring"],
	function (
		SmartNameStringPart,
		SmartName,
		WCRError, _) {
		"use strict";
		var exports = {};

		/* 
		 * some utility methods
		 */

		function assert(test) {
			if (test === false) {
				throw new Error("Something is wrong in the implementation of the SmartNameEncoding. Assert failed.");
			}
		}

		/**
		 * Constructor
		 * 
		 * @param {String} startOfSmartName
		 *            string which marks the start of a SmartName (if it is not escaped)
		 * @param {String} endOfSmartName
		 *            string which marks the end of a SmartName (if it is not escaped)
		 * @param {char} escapeChar
		 *            escape character
		 *
		 * @param {boolean} fuzzyEncoding
		 *            when true, escape characters will not be used for "top level" smartnames. 
		 *            (when reading and even when writing!!)
		 *            Consequences: This means that you will not see the difference between something the user typed 
		 *            and accidently has a SmartName format and something that was really added as a SmartName.
		 *            Known limitation: there are certain situations in which "fuzzyEncoding" fails, 
		 *            but they are very unlikely if you choose a very unlikely escape character.
		 * @param {Function} smartnameLookupFunction
		 *            only used when fuzzyEncoding is used. A function which has one argument: a SmartName object and 
		 *            returns true or false depending on whether a SmartName with such a name exists.
		 */
		var SmartNameEncoding = function (startOfSmartName, endOfSmartName, escapeChar, fuzzyEncoding, smartnameLookupFunction) {

			this._startOfSmartName = startOfSmartName;
			this._endOfSmartName = endOfSmartName;
			this._escapeChar = escapeChar;
			this._fuzzyEncoding = fuzzyEncoding;
			if (this._fuzzyEncoding) {
				this._smartnameLookupFunction = smartnameLookupFunction;
			}

			this._charactersToEscape = [];
			var allCharacters = escapeChar + startOfSmartName + endOfSmartName;
			for (var i = 0; i < allCharacters.length; i++) {
				var c = allCharacters.charAt(i);
				if (!_.contains(this._charactersToEscape, c)) {
					this._charactersToEscape.push(c);
				}
			}
			// Special case... escapeChar, startOfSmartName & endOfSmartName have no two characters which are the same
			// In that case we can reduce the escape characters:
			this._charactersToEscape = [escapeChar, startOfSmartName.charAt(0), endOfSmartName.charAt(0)];

			// The escape character may not be part of the start of a SmartName or end of a SmartName strings...
			assert(startOfSmartName.indexOf(escapeChar) === -1);
			assert(endOfSmartName.indexOf(escapeChar) === -1);
		};

		(function () {

			/**
			 * Escapes a plain text string (text without SmartNames).
			 * 
			 * When using fuzzy encoding, this method does not have any effect. 
			 * (since fuzzy encoding does not support escape characters on the "top" level)
			 * 
			 * @param plainText
			 *            The plain text string which could contain something that looks like a SmartName
			 * @return the encoded plain text string
			 */
			this.escapeString = function (plainText) {
				if (this._fuzzyEncoding) {
					return plainText;
				} else {
					return this._escapeStringInternal(plainText);
				}
			};

			/**
			 * (private) Escapes a plain text string (text without SmartNames).
			 * 
			 * @param plainText
			 *            The plain text string which could contain something that looks like a SmartName
			 * @return the encoded plain text string
			 */
			this._escapeStringInternal = function (plainText) {
				for (var i = 0; i < this._charactersToEscape.length; i++) {
					plainText = plainText.replaceAll("" + this._charactersToEscape[i], "" + this._escapeChar + this._charactersToEscape[i]);
				}
				return plainText;
			};

			/**
			 * Unescapes an encoded plain text string (text without SmartNames).
			 * 
			 * When using fuzzy encoding, this method does not have any effect. 
			 * (since fuzzy encoding does not support escape characters on the "top" level)
			 * 
			 * @param encodedPlainText
			 *            The encoded string which only contains plain text (no SmartNames)
			 * @return the plain text string
			 */
			this.unescapeString = function (encodedPlainText) {
				if (this._fuzzyEncoding) {
					return encodedPlainText;
				} else {
					return this._unescapeStringInternal(encodedPlainText);
				}
			};

			/**
			 * (private) Unescapes an encoded plain text string (text without SmartNames)
			 * 
			 * @param encodedPlainText
			 *            The encoded string which only contains plain text (no SmartNames)
			 * @return the plain text string
			 */
			this._unescapeStringInternal = function (encodedPlainText) {
				for (var i = this._charactersToEscape.length - 1; i >= 0; i--) {
					encodedPlainText = encodedPlainText.replaceAll("" + this._escapeChar + this._charactersToEscape[i], "" + this._charactersToEscape[i]);
				}
				return encodedPlainText;
			};

			/**
			 * A string containing both SmartNames and normal text can be converted into one String with this method.
			 * 
			 * @param partsOfString
			 *            The parts that should be combined into one String
			 * @return the string containing all the information
			 */
			this.encodeSmartNameString = function (partsOfString) {
				// The result...
				var result = "";

				// Used to combine multiple text parts after each other before encoding (makes a difference)
				var constantData = "";

				// Loop through the items, encode them and add them to the result string...
				for (var i = 0; i < partsOfString.length; i++) {
					var part = partsOfString[i];
					if (part.isTextPart()) {
						constantData += part.getAsText();
					} else {
						result += this.escapeString(constantData);
						constantData = "";
						result += this._startOfSmartName;
						result += this._escapeStringInternal(part.getAsSmartName().getFullName());
						result += this._endOfSmartName;
					}
				}

				result += this.escapeString(constantData);

				return result;
			};

			/**
			 * Same as encodeSmartNameString, but for a single SmartName.<br/>
			 * encodeSmartNameString will return the same result if called for a single SmartName.
			 * 
			 * @param {SmartName} smartname
			 * @return {String}
			 */
			this.encodeSmartNameStringSingleSmartName = function (smartname) {
				return this._startOfSmartName + this._escapeStringInternal(smartname.getFullName()) + this._endOfSmartName;
			};

			/**
			 * (private) Util method used by decodeSmartNameString.
			 * 
			 * Corrects the internal decoding state when we find out we were wrong to assume we were parsing a SmartName. It
			 * will basically merge previous text parts together with the currentPart
			 * 
			 * @param currentPart
			 *            - The part of the original stringWithSmartNames which is decoded so far
			 * @param result
			 *            - The items which were already decoded and converted to SmartNameStringParts
			 * @returns -          the new value of part
			 */
			this._revertDecodedOpenSmartName = function (currentPart, result) {
				var newPart = "";
				if (result.length > 0 && result[result.length - 1].isTextPart()) {
					newPart += result.pop().getAsText();
				}
				newPart += currentPart;
				return newPart;
			};

			/**
			 * Decode a string that contains smart names to a list of seperate text and smart name parts.
			 * 
			 * @param   {String}   stringWithSmartNames The string that needs to be decoded.
			 * @returns {Array} The list that contains the seperate SmartNameStringPart objects that make 
			 *                  up the passed string with smart names.
			 */
			this.decodeSmartNameString = function (stringWithSmartNames) {
				var result = [];

				var startIndexSmartNameOpen = -1; // >=0 means open, -1 means not open
				var part = "";
				for (var i = 0; i < stringWithSmartNames.length; i++) {
					var c = stringWithSmartNames.charAt(i);
					if (c === this._escapeChar) {
						if (this._fuzzyEncoding && startIndexSmartNameOpen === -1) {
							// Fuzzy encoding only supports escape characters inside SmartNames.
							// Outside of the SmartNames it is just a normal character...
							part += c;
							continue;
						}

						if (i + 1 < stringWithSmartNames.length) {
							var escapedChar = stringWithSmartNames.charAt(i + 1);
							if (!_.contains(this._charactersToEscape, escapedChar)) {
								// we escaped some unknown character... (!) This string is not correctly encoded
								if (this._fuzzyEncoding) {
									// In fuzzy encoding this means we were wrong in assuming we were in a SmartName.
									// So, correct the state we are in...
									part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen, i), result);
									part += c;
									startIndexSmartNameOpen = -1;
								} else {
									throw new WCRError("SMARTNAME_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of string with SmartNames failed. Found a single escape character in the string.");
								}
							} else {
								part += escapedChar;
								i++; // skip an additional character...
							}
						} else {
							if (this._fuzzyEncoding) {
								// In fuzzy encoding this means we were wrong in assuming we were in a SmartName.
								// So, correct the state we are in...
								part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen, i), result);
								part += c;
								startIndexSmartNameOpen = -1;
							} else {
								throw new WCRError("SMARTNAME_PARSING_ILLEGAL_ESCAPE_CHARACTER", "Parsing of string with SmartNames failed. Found a single escape character in the string.");
							}
						}
					} else if (i + this._startOfSmartName.length <= stringWithSmartNames.length && this._startOfSmartName === stringWithSmartNames.substring(i, i + this._startOfSmartName.length)) {
						if (this._fuzzyEncoding && startIndexSmartNameOpen >= 0) {
							// In fuzzy encoding this means we were wrong in assuming we were in a SmartName.
							// So, correct the state we are in...
							part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen, i), result);
							startIndexSmartNameOpen = -1;
						}

						// found the start of a SmartName
						if (startIndexSmartNameOpen >= 0) {
							// there was already a SmartName open. (!) This string is not correctly encoded
							throw new WCRError("SMARTNAME_PARSING_SMARTNAME_ALREADY_OPEN", "Parsing of string with SmartNames failed. SmartName started within a SmartName.");
						} else {
							// ok :) (start of smartname)
							// add text part to the list
							if (part.length > 0) {
								result.push(SmartNameStringPart.makePlainTextPart(part));
							}
							part = "";
							// open SmartName
							startIndexSmartNameOpen = i;
						}
						i += (this._startOfSmartName.length - 1); // skip some additional indices...
					} else if (i + this._endOfSmartName.length <= stringWithSmartNames.length && this._endOfSmartName === stringWithSmartNames.substring(i, i + this._endOfSmartName.length)) {
						// found the end of a SmartName
						if (startIndexSmartNameOpen === -1) {
							if (this._fuzzyEncoding) {
								// Fuzzy encoding does not care for "end of SmartName" characters which do not end a SmartName.
								// Interprete as a normal character...
								part += c;
								continue;
							} else {
								// there was no SmartName open. (!) This string is not correctly encoded
								throw new WCRError("SMARTNAME_PARSING_SMARTNAME_NOT_OPEN", "Parsing of string with SmartNames failed. SmartName was closed before the SmartName was opened.");
							}
						} else {
							// ok :) (end of smartname)
							var closedSmartName = null;
							try {
								closedSmartName = new SmartName(part);
							} catch (e) {
								if (this._fuzzyEncoding) {
									// Oh, we have a random backslash somewhere in the smartname
									// (Incorrect arguments encoding)
									// So, were wrong in assuming we were in a SmartName.
									// So, correct the state we are in...
									part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen, i), result);
									part += c;
									startIndexSmartNameOpen = -1;
									continue;
								} else {
									throw e;
								}
							}

							//
							// Filter actual SmartNames by name...
							//
							// In fuzzy encoding, since we skipped a lot of escaping rules, we are not really sure if this is a SmartName (even though it looks like one)
							// So, we need to look it up...
							//
							if (this._fuzzyEncoding && !this._smartnameLookupFunction(closedSmartName)) {

								// Oh, not a SmartName after all...
								// We were wrong in assuming we were in a SmartName.
								// So, correct the state we are in...
								part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen, i), result);
								part += c;
								startIndexSmartNameOpen = -1;
								continue;
							}

							// add SmartName to result
							result.push(SmartNameStringPart.makeSmartNamePart(closedSmartName));
							part = "";
							// close SmartName
							startIndexSmartNameOpen = -1;
						}
						i += (this._endOfSmartName.length - 1); // skip some additional indices...
					} else {
						// Just a normal character...
						part += c;
					}
				}

				if (this._fuzzyEncoding && startIndexSmartNameOpen >= 0) {
					// We were wrong in assuming we were in a SmartName.
					// So, correct the state we are in...
					part = this._revertDecodedOpenSmartName(stringWithSmartNames.substring(startIndexSmartNameOpen), result);
					startIndexSmartNameOpen = -1;
				}

				// end of the string...
				if (startIndexSmartNameOpen >= 0) {
					// there is still a SmartName open. (!) This string is not correctly encoded
					throw new WCRError("SMARTNAME_PARSING_SMARTNAME_NEVER_CLOSED", "Parsing of string with SmartNames failed. SmartName was never closed.");
				} else {
					// ok :) (end of a text part...)
					// add text part to the list
					if (part.length > 0) {
						result.push(SmartNameStringPart.makePlainTextPart(part));
					}
				}

				return result;
			};

			/**
			 * Check if a string contains a smart name.
			 * 
			 * @param   {String} value The string to check.
			 * @returns {Boolean}  Whether the string contains a smart name or not.
			 */
			this.containsSmartName = function (value) {
				if (value) {
					var parts = this.decodeSmartNameString(value);
					for (var i = 0; i < parts.length; i++) {
						if (parts[i].isSmartNamePart()) {
							return true;
						}
					}
				}
				return false;
			};

			/**
			 * Method to easily modify a string with SmartNames
			 * 
			 * @param value The SmartNameEncoded string
			 * @param modifySmartName Function to modify the SmartNames --- func(SmartName):SmartName
			 * @param modifyText Function to modify the text between the SmartNames --- func(String):String
			 */
			this.modifySmartNameEncodedString = function (value, modifyText, modifySmartName) {
				// Defaults...
				if (modifyText === undefined || modifyText === null) {
					modifyText = function (a) {
						return a;
					};
				}
				if (modifySmartName === undefined || modifySmartName === null) {
					modifySmartName = function (a) {
						return a;
					};
				}

				// Decode the value with SmartNames...
				var splitInput = this.decodeSmartNameString(value);

				// Then loop through all the parts
				for (var i = 0; i < splitInput.length; i++) {
					var part = splitInput[i];

					if (part.isSmartNamePart()) {
						// Get current SmartName
						var smartnamepart = part.getAsSmartName();
						// Modify SmartName
						smartnamepart = modifySmartName(smartnamepart);
						// Set new SmartName
						splitInput[i] = SmartNameStringPart.makeSmartNamePart(smartnamepart);
					} else {
						// Get current value
						var textpart = part.getAsText();
						// Modify value
						textpart = modifyText(textpart);
						// Set new value
						splitInput[i] = SmartNameStringPart.makePlainTextPart(textpart);
					}
				}

				// re-encode the list of parts into a single string
				return this.encodeSmartNameString(splitInput);
			};


		}).call(SmartNameEncoding.prototype);

		/**
		 * In a Saved Search, SmartNames look like: "The project name is {Project Name}.". (Escape character is exclamation mark)
		 */
		exports.SAVEDSEARCH = new SmartNameEncoding("{", "}", '!', false);
		/**
		 * In Autonaming, SmartNames look like: "Project_[Company]". (Escape character is §)
		 */
		exports.AUTONAMING = new SmartNameEncoding("[", "]", '§', false);

		/**
		 * For Workflow Editor encoding<br/>
		 * 
		 * Note: uses fuzzyEncoding algorithm!
		 */
		exports.WORKFLOWEDITOR = new SmartNameEncoding("<", "/>", '§', true, function (sn) {
			var name = sn.getName();
			return true; // FOR NOW THIS IS IN LINE WITH THE BACKEND IMPORT/EXPORT
		});

		/**
		 * Universal copy paste encoding
		 * On the front-end, when copy pasting between fields, the following encoding is used.
		 */
		exports.UNIVERSALCOPYPASTEENCODING = new SmartNameEncoding("[", "]", '\\', false);

		/**
		 * Export Constructor (for JUnit test)
		 */
		exports.constructor = SmartNameEncoding;

		return exports;
	});