define([
    "jquery",
    "./smartnameinput",
    "component/smartnames/smartfield/smartfield-style"
], function (
	$,
	SmartNameInput
) {


	/**
	 * A SmartNameInput (can display & edit SmartNames, does not have a button to insert SmartNames)
	 * Constructor converts a div to a SmartName edit input
	 *
	 * SmartNameInput has the following options:
	 *  - readOnly: is this an input field or just a read only display
	 *  - displayFunction: a function func(SmartName):String which returns the way a SmartName should be displayed...
	 *  - encoding: the SmartNameEncoding instance to use. (default: SAVEDSEARCH)
	 *  - mark: add a smartname to the beginning of the text generated by this field if it does not contain SmartNames. (default: false)
	 *  - markid: the name (string) of the smartname to add (default: "@") (correctly encoded... see SmartNameArgumentEncoding)
	 *  - markaware: remove marks when using setValue on this field (default: true)
	 * 
	 * Both parameters "div" and "options" are optional.
	 *
	 * @param div {String or DOMElement} the div to convert into a SmartName input
	 * @param options {Object} the options
	 */
	var LazySmartNameInput = function (div, options) {

		this._div = div;
		this._options = options;

		this._disabled = false;
		this._shown = true;
		this._onValueChangedCallbacks = [];
		this._onBlurCallbacks = [];
		this._onFocusCallbacks = [];
		this._onSmartNamePopoverShowCallbacks = [];
		this._onSmartNamePopoverHideCallbacks = [];
		this._arrayOfSmartNames;

		this._smartNameInput = null;
		this._placeHolder = $("<input type='text' class='form-control'/>");

		// Whenever the _placeHolder gains focus, transform to a Non Lazy SmartNameInput.
		this._placeHolder.on("focus", (function () {
			this.toNonLazy();
		}).bind(this));

		this._placeHolder.on("dragenter", (function () {
			this.toNonLazy();
		}).bind(this));

	};

	/*
	 * Get or set the encoded SmartName string (for sending/receiving from the back-end)
	 */

	/**
	 * Get the encoded value in this SmartName input
	 * (for sending to the back-end)
	 *
	 * @returns {String}
	 */
	LazySmartNameInput.prototype.getValue = function () {
		// If a non-lazy SmartNameInput exists, return the value of that one ...
		if (this._smartNameInput) {
			return this._smartNameInput.getValue();
		} else {
			// .. else return the value of the placeholder.
			return this._placeHolder.val();
		}
	};

	/**
	 * Set the encoded value in this SmartName input
	 * (string recieved from the back-end)
	 *
	 * @param value {String} the encoded string
	 */
	LazySmartNameInput.prototype.setValue = function (value) {
		// If value is non empty and we're still lazy, we need to become non-lazy.
		if (value !== "" && this._smartNameInput === null) {
			this.toNonLazy();
		}

		// If a non-lazy SmartNameInput exists, set the value of that one ...
		if (this._smartNameInput !== null) {
			this._smartNameInput.setValue(value);
		} else {
			// .. else set the value of the placeholder (can only be ""). 
			this._placeHolder.val(value);
		}
	};

	/**
	 * Clears the SmartName input
	 */
	LazySmartNameInput.prototype.clear = function () {
		// If a non-lazy SmartNameInput exists, clear that one ...
		if (this._smartNameInput) {
			this._smartNameInput.clear();
		} else {
			// .. else clear the placeholder. 
			this._placeHolder.val("");
		}
	};


	/*
	 * Events...
	 */

	/**
	 * When the component loses focus
	 * 
	 * @param listener {Function} - a function with signature void func(void).
	 */
	LazySmartNameInput.prototype.onBlur = function (listener) {
		// If a non-lazy SmartNameInput exists, attach the listener directly to it ...
		if (this._smartNameInput) {
			this._smartNameInput.onBlur(listener);
		} else {
			// Otherwise do it on the placeholder and remember the callback to maybe attach it later to the non lazy one.
			this._onBlurCallbacks.push(listener);
			this._placeHolder.on("blur", listener);
		}
	};
	
	/**
	 * When the component gains focus
	 * 
	 * @param listener {Function} - a function with signature void func(void).
	 */
	LazySmartNameInput.prototype.onFocus = function (listener) {
		// If a non-lazy SmartNameInput exists, attach the listener directly to it ...
		if (this._smartNameInput) {
			this._smartNameInput.onFocus(listener);
		} else {
			// Otherwise do it on the placeholder and remember the callback to maybe attach it later to the non lazy one.
			this._onFocusCallbacks.push(listener);
			this._placeHolder.on("focus", listener);
		}
	};

	/**
	 * When the value in the component is changed
	 * The current value of the component is passed to the listener.
	 *
	 * @param listener {Function} - a function with signature void func(String).
	 */
	LazySmartNameInput.prototype.onValueChange = function (listener) {
		// If a non-lazy SmartNameInput exists, attach the listener directly to it ...
		if (this._smartNameInput) {
			this._smartNameInput.onValueChange(listener);
		} else {
			// Otherwise do it on the placeholder and remember the callback to maybe attach it later to the non lazy one.
			this._onValueChangedCallbacks.push(listener);
			this._placeHolder.on("change", listener);
		}
	};

	/**
	 * When the smart name popover is shown.
	 * 
	 * @param callback {Function} - a function with signature void func(void).
	 */
	LazySmartNameInput.prototype.onSmartNamePopoverShow = function (callback) {
		if (this._smartNameInput) {
			this._smartNameInput.onSmartNamePopoverShow(callback);
		} else {
			this._onSmartNamePopoverShowCallbacks.push(callback);
		}
	};
	
	/**
	 * When the smart name popover is hidden.
	 * 
	 * @param callback {Function} - a function with signature void func(void).
	 */
	LazySmartNameInput.prototype.onSmartNamePopoverHide = function (callback) {
		if (this._smartNameInput) {
			this._smartNameInput.onSmartNamePopoverHide(callback);
		} else {
			this._onSmartNamePopoverHideCallbacks.push(callback);
		}
	};

	/**
	 * Set the SmartNames (for the autocompletion functionality)
	 *
	 * @param arrayOfSmartNames {Array<String>} the SmartNames
	 */
	LazySmartNameInput.prototype.setSmartNames = function (arrayOfSmartNames) {

		// If a non-lazy SmartNameInput exists, set the sma directly to it ...
		if (this._smartNameInput) {
			this._smartNameInput.setSmartNames(arrayOfSmartNames);
		} else {
			// ... else remember the array to maybe set it later to the non lazy SmartNameInput.
			this._arrayOfSmartNames = arrayOfSmartNames;
		}
	};


	/*
	 * Inserting things into the SmartName input
	 */

	/**
	 * Inserting text
	 * 
	 * @param text {String} the text to insert
	 */
	LazySmartNameInput.prototype.insertText = function (text) {

		// Check that the text is not null or empty.
		if (text == null || text === undefined || text.length === 0) {
			return;
		}

		// This method can only be executed on a non lazy SmartNameInput.
		this.toNonLazy();
		// Execute it on the non lazy SmartNameInput.
		this._smartNameInput.insertText(text);
	};

	/**
	 * Inserting a SmartName...
	 *
	 * @param name {String} the name of the SmartName to insert
	 */
	LazySmartNameInput.prototype.insertSmartName = function (name) {
		// This method can only be executed on a non lazy SmartNameInput.
		this.toNonLazy();
		// Execute it on the non lazy SmartNameInput.
		this._smartNameInput.insertSmartName(name);
	};

	/*
	 * Other actions on this input field
	 */

	/**
	 * Show this SmartName input
	 */
	LazySmartNameInput.prototype.show = function () {
		// If a non-lazy SmartNameInput exists, show that one ...
		if (this._smartNameInput) {
			this._smartNameInput.show();
		} else {
			// Else blur the placeholder ...
			this._placeHolder.show();
		}
		this._shown = true;
	};

	/**
	 * Hide this SmartName input
	 */
	LazySmartNameInput.prototype.hide = function () {
		// If a non-lazy SmartNameInput exists, hide that one ...
		if (this._smartNameInput) {
			this._smartNameInput.hide();
		} else {
			// Else blur the placeholder ...
			this._placeHolder.hide();
		}
		this._shown = false;
	};

	/**
	 * Put the focus on this input field...
	 */
	LazySmartNameInput.prototype.focus = function () {
		// If a non-lazy SmartNameInput exists, focus that one ...
		if (this._smartNameInput) {
			this._smartNameInput.focus();
		} else {
			// Else blur the placeholder ...
			this._placeHolder.focus();
		}
	};

	/**
	 * Remove the focus from this input field...
	 */
	LazySmartNameInput.prototype.blur = function () {
		// If a non-lazy SmartNameInput exists, blur that one ...
		if (this._smartNameInput) {
			this._smartNameInput.blur();
		} else {
			// Else blur the placeholder ...
			this._placeHolder.blur();
		}
	};


	/**
	 * Tells this field to resize
	 */
	LazySmartNameInput.prototype.resize = function () {
		if (this._smartNameInput) {
			return this._smartNameInput.resize();
		}
	};

	/**
	 * Is the editor disabled?
	 * 
	 * @returns {Boolean} true if this editor is disabled...
	 */
	LazySmartNameInput.prototype.isDisabled = function () {

		// If a non-lazy SmartNameInput exists, do it on that one ...
		if (this._smartNameInput) {
			return this._smartNameInput.isDisabled();
		} else {
			return this._disabled;
		}
	};

	/**
	 * Disable the editor
	 */
	LazySmartNameInput.prototype.disable = function () {

		// If a non-lazy SmartNameInput exists, do it on that one ...
		if (this._smartNameInput) {
			this._smartNameInput.disable();
		} else {
			this._placeHolder.prop('disabled', true);
			this._placeHolder.addClass('LazySmartNameInput-Disabled');
		}
		this._disabled = true;
	};

	/**
	 * Enable the editor
	 */
	LazySmartNameInput.prototype.enable = function () {

		// If a non-lazy SmartNameInput exists, do it on that one ...
		if (this._smartNameInput) {
			this._smartNameInput.enable();
		} else {
			this._placeHolder.prop('disabled', false);
			this._placeHolder.removeClass('LazySmartNameInput-Disabled');
		}
		this._disabled = false;

	};

	/**
	 * Stop editing
	 */
	LazySmartNameInput.prototype.stopEditing = function () {
		if (this._smartNameInput) {
			this._smartNameInput.stopEditing();
		}
	};

	/**
	 * Get the jQuery wrapped HTML element of this SmartNameInput
	 *
	 * @returns {JQuery element} the jquery element matching this component
	 */
	LazySmartNameInput.prototype.getJQElement = function () {

		// If a non-lazy SmartNameInput exists, do it on that one ...
		if (this._smartNameInput) {
			this._smartNameInput.getJQElement();
		} else {
			// ... else do it on the placeholder
			return this._placeHolder;
		};
	};

	/**
	 * Create a non-lazy SmartNameInput version of this one and start using that one internally. 
	 */
	LazySmartNameInput.prototype.toNonLazy = function () {

		if (this._smartNameInput !== null) {
			// LazySmartNamInput is already no longer lazy, you can't call this twice now!
			return;
		}

		// Create a non-lazy SmartNameInput
		this._smartNameInput = new SmartNameInput(this._div, this._options);

		// Register all the "on value change" callbacks.
		for (var i = 0; i < this._onValueChangedCallbacks.length; i++) {
			this._smartNameInput.onValueChange(this._onValueChangedCallbacks[i]);
		}

		// Register all the "on blur" callbacks.
		for (var i = 0; i < this._onBlurCallbacks.length; i++) {
			this._smartNameInput.onBlur(this._onBlurCallbacks[i]);
		}
		// Register all the "on focus" callbacks.
		for (var i = 0; i < this._onFocusCallbacks.length; i++) {
			this._smartNameInput.onFocus(this._onFocusCallbacks[i]);
		}
		// Register all the "smartname popover shown" callbacks.
		for (var i = 0; i < this._onSmartNamePopoverShowCallbacks.length; i++) {
			this._smartNameInput.onSmartNamePopoverShow(this._onSmartNamePopoverShowCallbacks[i]);
		}
		// Register all the "smartname popover hidden" callbacks.
		for (var i = 0; i < this._onSmartNamePopoverHideCallbacks.length; i++) {
			this._smartNameInput.onSmartNamePopoverHide(this._onSmartNamePopoverHideCallbacks[i]);
		}
		
		
		// Remove the "on blur" listeners of the placeholder.
		this._placeHolder.off("blur");

		// Should we enable the SmartNameInput?
		if (this._disabled) {
			this._smartNameInput.disable();
		}

		// Should we show the SmartNameInput?
		if (!this._shown) {
			this._smartNameInput.hide();
		}

		// Should we already set an array of SmartNames?
		if (this._arrayOfSmartNames) {
			this._smartNameInput.setSmartNames(this._arrayOfSmartNames);
		}

		// Replace the placeholder with the non-lazy SmartNameInput
		this._placeHolder.replaceWith(this._smartNameInput.getJQElement());

		// Pass focus the newly create non-lazy SmartNameInput
		this._smartNameInput.focus();

		// Resize the SmartNameInput
		this._smartNameInput.resize();

		// Clean up the properties relating to the lazieness of this component, they're no longer needed.
		this._destroyLazyProp();
	};

	/**
	 * Destroy the properties relating to the lazieness of this component. 
	 */
	LazySmartNameInput.prototype._destroyLazyProp = function () {

		if (!this._smartNameInput) {
			window.console.warn("Destroying lazy properties of LazySmartNameInput while no non-lazy SmartNameInput exists.");
		}

		delete this._div;
		delete this._options;
		delete this._disabled;
		delete this._shown;
		delete this._onValueChangedCallbacks;
		delete this._onBlurCallbacks;
		delete this._onFocusCallbacks;
		delete this._placeHolder;

	};

	/**
	 * Destroy & cleanup this SmartName input
	 */
	LazySmartNameInput.prototype.destroy = function () {

		// If a non-lazy SmartNameInput exists, do it on that one.
		if (this._smartNameInput) {
			this._smartNameInput.destroy();
		}
	};

	return LazySmartNameInput;
});