define(["jquery",
		"utils/webi18n!SmartNameSelector",
        "utils/webcss!./smartnameeditpanel-style.css"],
	function ($, i18n) {
		"use strict";

		/**
		 * View component of an editable smartname
		 * @param {String} name - The smartname name
		 * @param {Array.<String>} args - The arguments of the smartname to edit
		 * @param {Object}    smartNameEditOptions - Options for the smartname to edit. Example:
		 *                                         {labels: ["label1", "label2"], // translated labels for the editable parameters
		 *                                         hasStrictParams: true, // true for strict set of parameters, false for dynamic amount of parameters
		 *                                         smartnameInputOptions: {Object} // see options of SmartNameInput. This is used to construct the correct/same smartfields for the parameters as its parent.
		 *                                         }
		 */
		var SmartNameEditPanelView = function (name, args, smartNameEditOptions) {

			// All the smartfields in the view.
			this._parameterSmartFields = [];

			// Check whether labels were specified for the to be configured parameters.
			if (smartNameEditOptions && smartNameEditOptions.labels) {
				this._labels = smartNameEditOptions.labels;
			} else {
				this._labels = [];
			}

			// Check whether the user can determine the number of parameters for the smart name.
			if (smartNameEditOptions && smartNameEditOptions.hasStrictParams == true) {
				this._hasStrictParams = true;
			} else {
				this._hasStrictParams = false;
			}

			// Check dictionary service
			if (smartNameEditOptions.smartnameInputOptions.dictionaryService) {
				this._dictionaryService = smartNameEditOptions.smartnameInputOptions.dictionaryService;
			}

			// Check encoding
			if (smartNameEditOptions.smartnameInputOptions.encoding) {
				this._encoding = smartNameEditOptions.smartnameInputOptions.encoding;
			}

			// Check display function
			if (smartNameEditOptions.smartnameInputOptions.displayFunction) {
				this._displayFunction = smartNameEditOptions.smartnameInputOptions.displayFunction;
			}

			// Check selector display function
			if (smartNameEditOptions.smartnameInputOptions.selectorDisplayFunction) {
				this._selectorDisplayFunction = smartNameEditOptions.smartnameInputOptions.selectorDisplayFunction;
			}


			// The callbacks that need to be called when the save or cancel button is pressed.
			this._onSave = [];
			this._onCancel = [];

			// Create the wrapping div.
			this._wrapper = $("<div>", {
				"class": "SmartNameParamConfig"
			});

			// Create the form holding all the parameter entries.
			this._paramForm = $("<div>", {
				"class": "parameter-panel form-horizontal form-compact notOnlyOffsetLabels"
			});
			// If some arguments are already specified ... add parameter entry for them
			if (args.length > 0 || this._labels.length > 0) {
				for (var i = 0; i < args.length; i++) {
					this.addParameter(args[i]);
				}
				// If more labels are specified ... add empty entries for them ...
				var emptyArgs = this._labels.length - args.length;
				if (emptyArgs > 0) {
					for (var i = 0; i < emptyArgs; i++) {
						this.addParameter("");
					}
				}
			} else {
				this.addParameter("");
			}

			// Create a div holding the buttons to add and remove parameter entries.
			var addRemoveButtonWrapper = $("<div>", {
				"class": "AddRemoveButtonWrapper"
			});

			// Add a button to add more parameters
			var addIcon = $("<i>", {
				"class": "icon icon-WCR_Plus"
			});
			this._addParameterButton = $("<button>", {
				"class": "btn btn-default btn-xs"
			}).append(addIcon);
			this._addParameterButton.on("click", function () {
				this.addParameter("");
			}.bind(this));

			// Add a button to remove parameters
			var removeIcon = $("<i>", {
				"class": "icon icon-WCR_Minus"
			});
			this._removeParameterButton = $("<button>", {
				"class": "btn btn-default btn-xs"
			}).append(removeIcon);
			this._removeParameterButton.on("click", function () {
				this.removeLastParameter();
			}.bind(this));

			// Create a div holding the buttons to save or cancel.
			var saveCancelButtonWrapper = $("<div>", {
				"class": "SaveCancelButtonWrapper"
			});

			// Add a button to save the configuration
			this._saveButton = $("<button>", {
				"class": "btn btn-primary btn-xs"
			}).text(i18n.t("SMARTNAME_EDIT_SAVE"));
			this._saveButton.on("click", function () {
				this.doSave();
			}.bind(this));

			// Add a button to cancel the configuration
			this._cancelButton = $("<button>", {
				"class": "btn btn-default btn-xs"
			}).append(i18n.t("SMARTNAME_EDIT_CANCEL"));
			this._cancelButton.on("click", function () {
				this.doCancel();
			}.bind(this));

			// Create a div holding all the button wrappers.
			var toolbarWrapper = $("<div>", {
				"class": "ToolbarWrapper clearfix"
			});

			addRemoveButtonWrapper.append(this._addParameterButton);
			addRemoveButtonWrapper.append(this._removeParameterButton);
			saveCancelButtonWrapper.append(this._cancelButton);
			saveCancelButtonWrapper.append(this._saveButton);
			if (!this._hasStrictParams) {
				// The "Add" and "Remove" buttons are removed when this setting is enabled.
				toolbarWrapper.append(addRemoveButtonWrapper);
			}
			toolbarWrapper.append(saveCancelButtonWrapper);
			this._wrapper.append(this._paramForm);
			this._wrapper.append(toolbarWrapper);

		};

		/**
		 * Add a parameter
		 * @param {String} value The correctly encoded value for the SmartField, this value 
		 *                       can contain correctly encoded smartnames (e.g. "Project_[[ProjAttr:Name1]]").
		 */
		SmartNameEditPanelView.prototype.addParameter = function (value) {

			var index = this._parameterSmartFields.length;

			var group = $("<div>", {
				"class": "form-group"
			});

			// Create an input label.
			var labelText = "Param " + (index + 1);
			if (index < this._labels.length && this._labels[index] && this._labels[index].length > 0) {
				labelText = this._labels[index];
			}
			var label = $("<label>", {
				"class": "col-sm-12 col-lg-4"
			}).text(labelText);
			group.append(label);

			// Create a SmartField input for the argument.
			var SmartField = require("component/smartnames/smartfield/smartfield");
			var field = new SmartField($("<input>", {
				"type": "text"
			}), false, this._dictionaryService, this._displayFunction, this._selectorDisplayFunction, this._encoding);
			field.setWidth("100%");
			var inputDiv = $("<div>", {
				"class": "col-sm-12 col-lg-8"
			}).append(field.getJQElement());
			group.append(inputDiv);
			field.setIsInline(true);

			// Set the value of the SmartField to the argument.
			field.setValue(value);

			// Add the smartfield to the array of argument smartfields.
			this._parameterSmartFields.push(field);

			// Add the row to the table.
			this._paramForm.append(group);


		};

		/**
		 * Remove the last added parameter
		 */
		SmartNameEditPanelView.prototype.removeLastParameter = function () {

			// Remove the last parameter entry
			if (this._parameterSmartFields.length === 0) {
				return;
			}

			// Remove the last smartfield from the array of smartfields.
			this._parameterSmartFields.splice(-1, 1);

			// Remove the last entry
			this._paramForm.children().last().remove();

		};


		/**
		 * Get the arguments (parameter values)
		 * @returns {Array.<String>} 
		 */
		SmartNameEditPanelView.prototype.getArguments = function () {
			var args = [];
			for (var i = 0; i < this._parameterSmartFields.length; i++) {
				var value = this._parameterSmartFields[i].getValue();
				if (value !== undefined) {
					args.push(value);
				}
			}
			return args;
		};

		/**
		 * Callback to execute on clicking save button
		 * @param {Function} callback
		 */
		SmartNameEditPanelView.prototype.onSave = function (callback) {
			this._onSave.push(callback);
		};

		/**
		 * Callback to execute on clicking cancel button
		 * @param {Function} callback
		 */
		SmartNameEditPanelView.prototype.onCancel = function (callback) {
			this._onCancel.push(callback);
		};

		/**
		 * Do the save. Executes all save callbacks.
		 */
		SmartNameEditPanelView.prototype.doSave = function () {
			for (var i = 0; i < this._onSave.length; i++) {
				this._onSave[i]();
			}
		};

		/**
		 * Do the cancel. Executes all cancel callbacks.
		 */
		SmartNameEditPanelView.prototype.doCancel = function () {
			for (var i = 0; i < this._onCancel.length; i++) {
				this._onCancel[i]();
			}
		};

		/**
		 * @returns {JQueryObj} The wrapping JQuery element
		 */
		SmartNameEditPanelView.prototype.getJQElement = function () {
			return this._wrapper;
		};

		/**
		 * Disable elements of the smartname edit panel view.
		 * This will hide the save button
		 */
		SmartNameEditPanelView.prototype.disable = function () {
			this._saveButton.hide();
			for (var i = 0; i < this._parameterSmartFields.length; i++) {
				this._parameterSmartFields[i].disable();
			}
		};

		/**
		 * Enable elements of the smartname edit panel view
		 * This will show the save button
		 */
		SmartNameEditPanelView.prototype.enable = function () {
			this._saveButton.show();
			for (var i = 0; i < this._parameterSmartFields.length; i++) {
				this._parameterSmartFields[i].enable();
			}
		};

		/**
		 * Destroy all elements of the smartname edit panel view
		 */
		SmartNameEditPanelView.prototype.destroy = function () {
			for (var i = 0; i < this._parameterSmartFields.length; i++) {
				this._parameterSmartFields[i].destroy();
			}
		};

		/**
		 * Called when the panel is resized, so resize its elements
		 */
		SmartNameEditPanelView.prototype.resize = function () {
			for (var i = 0; i < this._parameterSmartFields.length; i++) {
				this._parameterSmartFields[i].resize();
			}
		};

		return SmartNameEditPanelView;

	});