/**
 * SmartNamesInput - Interface to the outside
 *
 * Allows you to create a SmartName input (a simple textfield in which displaying SmartNames is supported.
 */

define([
    "component/smartnames/encoding/smartnameencoding",
    "component/smartnames/encoding/smartnamestringpart",
    "component/smartnames/encoding/smartname",
    "./smartnameeditpanel",
    "component/popover/Popover",
	"utils/webi18n!SmartNameSelector",
	"underscore",
    "jquery",
    "ace/ace", // load ace
    "ace/ext/language_tools", // load the language_tools extension
    "lib/ace-modifications/mod-ace-all", // load all the modifications to ace to support SmartNames !!!
    ], function (
	smartnameencoding,
	SmartNameStringPart,
	SmartName,
	SmartNameEditPanel,
	Popover,
	i18n,
	_,
	$,
	ace
) {
	"use strict";

	/**
	 * Commands to remove from the ace editor
	 */
	var commandsToDelete = [
        "showSettingsMenu", "goToNextError", "goToPreviousError", "centerselection", "gotoline",
        "fold", "unfold", "toggleFoldWidget", "toggleParentFoldWidget", "foldall", "foldOther",
        "unfoldall", "findnext", "findprevious", "selectOrFindNext", "selectOrFindPrevious",
        "find", "togglerecording", "replaymacro", "jumptomatching", "selecttomatching",
        "expandToMatching", "removeline", "duplicateSelection", "sortlines", "togglecomment",
        "toggleBlockComment", "modifyNumberUp", "modifyNumberDown", "replace", "copylinesup",
        "movelinesup", "copylinesdown", "movelinesdown", "cut_or_delete", "removetolinestart",
        "removetolineend", "outdent", "indent", "blockoutdent", "blockindent", "splitline",
        "transposeletters", "touppercase", "tolowercase", "expandtoline"];

	/**
	 * Method is called when rendering a SmartName in the SmartName input.
	 * 
	 * This method can be used to implement translation (displaying the SmartName in the language of the user) or show a more user-friendly SmartName.
	 * 
	 * @param {SmartName} smartName - the info to display
	 */
	function defaultDisplayFunction(smartName) {
		if (smartName.getName() == "List") {
			return "[List]";
		} else {
			var display = "[";
			if (smartName.getName() == "ProjAttr") {
				display += smartName.getArguments()[0] + " (Project)";
			} else if (smartName.getName() == "DocAttr") {
				display += smartName.getArguments()[0] + " (Document)";
			} else {
				display += smartName.getFullName();
			}
			display += "]";
			return display;
		}
	}

	/**
	 * A SmartNameInput (can display & edit SmartNames, does not have a button to insert SmartNames)
	 * Constructor converts a div to a SmartName edit input
	 *
	 * SmartNameInput has the following options:
	 *  - readOnly: is this an input field or just a read only display
	 *  - displayFunction: a function func(SmartName):String which returns the way a SmartName should be displayed...
	 *  - encoding: the SmartNameEncoding instance to use. (default: SAVEDSEARCH)
	 *  - mark: add a smartname to the beginning of the text generated by this field if it does not contain SmartNames. (default: false)
	 *  - markid: the name (string) of the smartname to add (default: "@") (correctly encoded... see SmartNameArgumentEncoding)
	 *  - markaware: remove marks when using setValue on this field (default: true)
	 *  - selectorDisplayFunction: Only used for editable smartnames. A function func(SmartName):String which returns the way a SmartName should be displayed in the smartname selector
	 *  - dictionaryService: Only used for editable smartnames. The dicationary service for loading smartnames and sections.
	 * 
	 * Both parameters "div" and "options" are optional.
	 *
	 * @param div {String or DOMElement} the div to convert into a SmartName input
	 * @param options {Object} the options
	 */
	var SmartNameInput = function (div, options) {

		// If no div is specified, create a div.
		if (div === undefined) {
			div = document.createElement("div");
		} else {
			div = $(div).get(0);
		}

		// process the options
		if (options === undefined) {
			options = {};
		}
		// readOnly, default is false
		if (options["readOnly"] === undefined) {
			options["readOnly"] = false;
		}
		if (options["encoding"] === undefined) {
			options["encoding"] = smartnameencoding.SAVEDSEARCH;
		}
		this._encoding = options["encoding"];

		// -> mark option... Mark text generated by this input with a special SmartName...
		this._markaware = (options["markaware"] === false ? false : true);
		this._mark = this._markaware && (options["mark"] === true ? true : false);

		// default: [@] SmartName... (!"argumentencoded")
		this._markid = (options["markid"] !== undefined ? options["markid"] : "@"); // "@"
		this._marksn = this._encoding.encodeSmartNameString([SmartNameStringPart.makeSmartNamePart(new SmartName(this._markid))]); // "{@}"

		// -> converts a div to a SmartName edit input
		var editor = ace.edit(div);
		editor.setBehavioursEnabled(false);
		// Save the stuff
		this._editor = editor;
		this._div = $(div);
		this._formcontrol = this._div.wrap($('<div/>')).parent();
		div.SmartNameInput = this;

		// -> disable warning message when selection changes (programmatically)
		editor.$blockScrolling = Infinity;

		// -> activate SmartNames functionality in editor
		var initText = editor.getValue();
		editor.getSession().enableSmartNames(this._encoding);
		this.setValue(initText);

		// -> set the function to display SmartNames
		// displayFunction, default is defaultDisplayFunction
		this._displayFunction = options["displayFunction"] || defaultDisplayFunction;
		editor.getSession().getDocument().displayFunction = this._displayFunction;

		// Set editor options
		editor.setOptions({
			highlightActiveLine: false,
			highlightSelectedWord: false
		});

		// Difference between a SmartNameDisplay & SmartNameInput...
		this.readOnly = options["readOnly"];
		this.disabled = false;
		if (this.readOnly) {
			editor.setReadOnly(true);
			editor.setTheme("component/smartnames/input/themes/smartnames-display");
		} else {
			this._formcontrol.addClass("form-control");
			editor.setTheme("component/smartnames/input/themes/smartnames-input");
		}

		//bugfix in ace (in case of combination of showGutter=false & autocompletion=on)
		editor.getSession().gutterRenderer = {
			getWidth: function () {
				return 0;
			},
			getText: function () {
				return "";
			}
		};

		// Set some editor options...
		editor.renderer.setOptions({
			showLineNumbers: false,
			maxLines: 1,
			printMargin: 0,
			printMarginColumn: 0,
			showPrintMargin: false,
			displayIndentGuides: false,
			showGutter: false
		}); //, fontFamily: "verdana"

		// Don't use padding on the form-control, use padding in the ace editor => so you can click on it.
		editor.renderer.setPadding(10);
		this._formcontrol.css({
			"padding-left": "0px",
			"padding-right": "0px"
		});

		// Remove all the not so usefull commands:
		editor.commands.removeCommands(commandsToDelete);

		// Disable the Ctrl-F when in a field:
		editor.commands.addCommand({
			name: 'myCommand',
			bindKey: {
				win: 'Ctrl-F',
				mac: 'Command-F'
			},
			exec: function (editor) {},
			readOnly: true
		});

		/**
		 * Show or hide the popover
		 * @param {Boolean} showOrNot - True to show, false to hide
		 */
		var showSmartNameEditPopover = _.bind(function (showOrNot) {
			if (showOrNot) {
				if (this._smartNameEditPopoverDiv) {
					this._smartNameEditPopoverDiv.show();
				}
				if (this._smartNameEditPopover) {
					this._smartNameEditPopover.show();
				}
				// Fire the popover show listeners.
				$.each(this._onSmartNamePopoverShowListeners, function (index, listener) {
					listener();
				});
			} else {
				this._destroySmartNameEditPanel();
				// Fire the popover hide listeners.
				$.each(this._onSmartNamePopoverHideListeners, function (index, listener) {
					listener();
				});
			}
		}, this);

		/**
		 * Recalculate the position of the smartname edit popover with the provided api
		 * @param {SmartNameAP} api - The api of the last edited smartname
		 */
		this._recalculateSmartNameEditPopover = _.bind(function (api) {
			if (api && this._smartNameEditPopover) {
				// Set new position
				var clientRect = api.getSmartNameBoundingClientRect(true);
				if (!this._smartNameEditPopoverDiv) {
					this._smartNameEditPopoverDiv = $("<div />");
					$("body").append(this._smartNameEditPopoverDiv);
				}
				this._smartNameEditPopoverDiv.css({
					position: 'absolute',
					width: clientRect.width,
					height: clientRect.height,
					top: clientRect.top + window.pageYOffset,
					left: clientRect.left + window.pageXOffset
				});
				this._smartNameEditPopover.setContext(this._smartNameEditPopoverDiv);

				// If shown, reshow to recalculate position
				if (this._smartNameEditPopover.isActive()) {
					// recalculate position
					this._smartNameEditPopover.show();
				}
			}
		}, this);

		// Initialize listening to SmartName edit events...
		this._lastAPI = null; // keep track of last edited smartname
		editor.initSmartNameEditListener(function (api) {

			if (this._lastAPI && this._smartNameEditPanel) {
				// if we have a lastAPI, save the changed values before opening new popup
				this._lastAPI.setSmartName(this._smartNameEditPanel.getSmartName().getFullName());
			}

			// Find out which smart name is being edited
			var smartName = new SmartName(api.getSmartName());

			// Detect what kind of smartname is being edited and generate the settings for the edit panel accordingly.
			var editPanelOptions = {};
			if (smartName.getName() == "List") {
				editPanelOptions.labels = [i18n.t("SMARTNAME_LIST_NAME"), i18n.t("SMARTNAME_LIST_RESULT_COLUMN"),
											   i18n.t("SMARTNAME_LIST_LOOKUP_COLUMN"), i18n.t("SMARTNAME_LIST_LOOKUP_VALUE")];
				editPanelOptions.hasStrictParams = true;
				editPanelOptions.smartnameInputOptions = options;
			} else {
				// This smartname does not support editing, do nothing ... only close the previous popover.
				showSmartNameEditPopover(false);
				return;
			}

			// Create a smartname edit panel.
			this._smartNameEditPanel = new SmartNameEditPanel(smartName, editPanelOptions);

			if (this.isDisabled()) {
				this._smartNameEditPanel.disable();
			} else {
				this._smartNameEditPanel.enable();
			}

			// Add this to a popover on the input.
			if (!this._smartNameEditPopover) {
				this._smartNameEditPopover = new Popover(this.getJQElement(), $("body"), $("body"), "bottom", "#FAFAFA", "SmartNameEditPopover", undefined, undefined, undefined, "#CCC");
			}
			this._smartNameEditPopover.setContent(this._smartNameEditPanel.getJQElement());

			// What needs to happend when the save button in the panel is pushed.
			this._smartNameEditPanel.onSave(function () {
				// Get the new smartname from the smartname parameter config panel.
				var smartName = this._smartNameEditPanel.getSmartName();
				// Update the smart name in the input.
				api.setSmartName(smartName.getFullName());
				// Hide the popover.
				showSmartNameEditPopover(false);
				// No longer highlight the smartname
				api.setActive(false);
			}.bind(this));

			// What needs to happend when the cancel button in the panel is pushed.
			this._smartNameEditPanel.onCancel(function () {
				// Hide the popover.
				showSmartNameEditPopover(false);
				// No longer highlight the smartname
				api.setActive(false);
			}.bind(this));

			// Highlight the smartname
			api.setActive(true);

			// Set correct position
			this._recalculateSmartNameEditPopover(api);

			// Show this in a popever.
			showSmartNameEditPopover(true);

			this._lastAPI = api;
		}.bind(this));

		this._editor.configureSmartNameTooltip(function (fullName, active, jqueryElement) {
			var smartName = new SmartName(fullName);

			if (smartName.getName() == "List" && !active) {
				jqueryElement.text(i18n.t("SMARTNAME_LIST_TOOLTIP"));
				return true;
			}
			return false;
		});

		this.onValueChange(_.bind(function () {
			if (this._lastAPI && !this._lastAPI.isValid()) {
				// SmartName is deleted, do not open the popup for this.
				showSmartNameEditPopover(false);
				// No longer highlight the smartname
				this._lastAPI.setActive(false);
				this._lastAPI = null;
			}
		}, this));
	};

	(function () {
		/*
		 * 
		 */

		/**
		 * (private) For "Mark" field functionality
		 * @param value The value to add the mark to
		 */
		this._addMark = function (value) {
			// Mark this value as being generated by a smartnameinput
			if (this._mark && !this._encoding.containsSmartName(value) && value.length > 0) {
				value = this._marksn + value;
			}

			return value;
		};

		/**
		 * (private) For "Mark" field functionality
		 * @param value The value to remove the mark from
		 */
		this._removeMark = function (value) {
			// Remove potential "mark"
			if (this._markaware && value.length >= this._marksn.length) {
				var decoded = this._encoding.decodeSmartNameString(value);
				for (var i = 0; i < decoded.length; i++) {
					var part = decoded[i];
					if (part.isSmartNamePart() && part.getAsSmartName().getName() === this._markid) {
						decoded.splice(i, 1);
						i--;
					}
				}
				value = this._encoding.encodeSmartNameString(decoded);
			}
			return value;
		};

		/*
		 * Get or set the encoded SmartName string (for sending/receiving from the back-end)
		 */

		/**
		 * Get the encoded value in this SmartName input
		 * (for sending to the back-end)
		 *
		 * @returns {String}
		 */
		this.getValue = function () {
			// return the encoded text... (with a mark if enabled)
			return this._addMark(this._editor.getSmartNameEncodedValue());
		};

		/**
		 * Set the encoded value in this SmartName input
		 * (string received from the back-end)
		 *
		 * @param value {String} the encoded string
		 */
		this.setValue = function (value) {
			// Validate the value that is passed to this method...
			// All values that enter should be correctly encoded.
			// If the error below is shown you did something wrong.
			try {
				this._encoding.decodeSmartNameString(value);
			} catch (e) {
				window.console.error("Error: Illegal value used in setValue on SmartField. Falling back to text...");
				// Escaping all special characters (to make sure it does not crash)
				value = this._encoding.escapeString(value);
			}

			// Set the value... (without the mark if markaware option enabled)
			this._editor.setSmartNameEncodedValue(this._removeMark(value));
		};

		/**
		 * Clears the SmartName input
		 */
		this.clear = function () {
			this._editor.setValue("");
		};

		/*
		 * Events...
		 */
		
		/**
		 * When the component gains focus
		 * 
		 * @param listener {Function} - a function with signature void func(void).
		 */
		this.onFocus = function (listener) {
			this._editor.on("focus", function (e) {
				try {
					listener();
				} catch (e) {
					console.error("An error was thrown in a focus listener: " + e);
				}
			});
		};

		/**
		 * When the component loses focus
		 * 
		 * @param listener {Function} - a function with signature void func(void).
		 */
		this.onBlur = function (listener) {
			this._editor.on("blur", function (e) {
				try {
					listener();
				} catch (e) {
					console.error("An error was thrown in a blur listener: " + e);
				}
			});
		};

		/**
		 * When the value in the component is changed
		 * The current value of the component is passed to the listener.
		 *
		 * @param listener {Function} - a function with signature void func(String).
		 */
		this.onValueChange = function (listener) {
			var self = this;
			this._editor.on("change", function (e) {
				try {
					if (self._editor.isChangeReady()) {
						listener(self.getValue());
					}
				} catch (e) {
					console.error("An error was thrown in a change listener: " + e);
				}
			});
		};
		
		/**
		 * When the smart name popover is shown.
		 * 
		 * @param listener {Function} - a function with signature void func(void).
		 */
		this._onSmartNamePopoverShowListeners = [];
		this.onSmartNamePopoverShow = function (listener) {
			this._onSmartNamePopoverShowListeners.push(listener);
		};
		
		/**
		 * When the smart name popover is hidden.
		 * 
		 * @param listener {Function} - a function with signature void func(void).
		 */
		this._onSmartNamePopoverHideListeners = [];
		this.onSmartNamePopoverHide = function (listener) {
			this._onSmartNamePopoverHideListeners.push(listener);
		};

		/**
		 * Set the SmartNames (for the autocompletion functionality)
		 *
		 * @param arrayOfSmartNames {Array<String>} the SmartNames
		 */
		this.setSmartNames = function (arrayOfSmartNames) {

			// convert the array to objects with extra information
			var smartNameMap = arrayOfSmartNames.map(_.bind(function (smartName) {
				var displayString = this._displayFunction(smartName);
				return {
					caption: displayString,
					value: displayString,
					score: 1,
					meta: "",
					smartname: smartName.getFullName()
				};
			}, this));

			// Enable SmartName completion
			this._editor.setOptions({
				enableLiveAutocompletion: [{
					getCompletions: function (editor, session, pos, prefix, callback) {
						// Filtering of the SmartNames is done automatically by ace/ext/language_tools
						// The popup is also created by ace/ext/language_tools
						callback(null, smartNameMap);
					}
                }]
			});

			// trigger a forced rerender after having smartnames (allows to make smartname translations dependent on asynchronous fetching)
			this._editor.resize(true);
		};

		/*
		 * Inserting things into the SmartName input
		 */

		/**
		 * Inserting text
		 * 
		 * @param text {String} the text to insert
		 */
		this.insertText = function (text) {
			this._editor.insert(text);
			this._editor.renderer.scrollCursorIntoView();
		};

		/**
		 * Inserting a SmartName...
		 *
		 * @param name {String} the name of the SmartName to insert
		 */
		this.insertSmartName = function (name) {
			this._editor.insertSmartName(name);
			this._editor.renderer.scrollCursorIntoView();
		};

		/*
		 * Other actions on this input field
		 */

		/**
		 * Show this SmartName input
		 */
		this.show = function () {
			this._formcontrol.show();
		};

		/**
		 * Hide this SmartName input
		 */
		this.hide = function () {
			this._formcontrol.hide();
		};

		/**
		 * Put the focus on this input field...
		 */
		this.focus = function () {
			this._editor.focus();
		};

		/**
		 * Remove the focus from this input field...
		 */
		this.blur = function () {
			this._editor.blur();
		};

		/**
		 * Tells this field to resize
		 */
		this.resize = function () {
			this._editor.resize();

			this._recalculateSmartNameEditPopover(this._lastAPI);

			if (this._smartNameEditPanel) {
				this._smartNameEditPanel.resize();
			}
		};

		/**
		 * Is the editor disabled?
		 * 
		 * @returns {Boolean} true if this editor is disabled...
		 */
		this.isDisabled = function () {
			return this.disabled;
		};

		/**
		 * Disable the editor
		 */
		this.disable = function () {
			if (!this.disabled) {
				this.disabled = true;
				this._editor.setReadOnly(true);
				this._div.addClass("ace_smartinput_disabled");

				if (this._smartNameEditPanel) {
					this._smartNameEditPanel.disable();
				}
			}
		};

		/**
		 * Enable the editor
		 */
		this.enable = function () {
			if (this.disabled) {
				this.disabled = false;
				this._editor.setReadOnly(false);
				this._div.removeClass("ace_smartinput_disabled");

				if (this._smartNameEditPanel) {
					this._smartNameEditPanel.enable();
				}
			}
		};

		/**
		 * Destroy smartname edit panel if it is available
		 */
		this._destroySmartNameEditPanel = function () {
			if (this._smartNameEditPopover) {
				this._smartNameEditPopoverDiv.remove();
				this._smartNameEditPopoverDiv = null;
				this._smartNameEditPopover.destroy();
				this._smartNameEditPopover = null;
			}

			if (this._smartNameEditPanel) {
				this._smartNameEditPanel.destroy();
				this._smartNameEditPanel = null;
			}
		};

		/**
		 * Destroy & cleanup this SmartName input
		 */
		this.destroy = function () {
			var value = this.getValue();

			this._destroySmartNameEditPanel();

			this.stopEditing();

			this._editor.destroy();
			this.getJQElement()
				.text(value)
				.removeClass("ace_editor")
				.removeClass("ace-smartnamefield-display")
				.removeClass("ace-smartnamefield-input");
			try { // try-catch for IE7
				delete this._div.get(0).SmartNameInput;
			} catch (e) {}
		};

		/**
		 * Stop editing. This will hide auto complete popup.
		 */
		this.stopEditing = function () {
			// make sure to hide the autocomplete popup
			if (this._editor.completer && this._editor.completer.getPopup()) {
				this._editor.completer.getPopup().hide();
			}
		};

		/**
		 * Get the jQuery wrapped HTML element of this SmartNameInput
		 *
		 * @returns {JQuery element} the jquery element matching this component
		 */
		this.getJQElement = function () {
			return this._formcontrol;
		};

	}).call(SmartNameInput.prototype);

	/**
	 * Method to automatically apply this module to a div...
	 * 
	 * @param div {DomElement} the DomElement to modify
	 */
	SmartNameInput.wcrinit = function (div) {
		new SmartNameInput(div);
	};

	/**
	 * Method to automatically destroy this component once initialized for a div...
	 * 
	 * @param div {DomElement} the DomElement to destroy again
	 */
	SmartNameInput.wcrdestroy = function (div) {
		div.SmartNameInput.destroy();
	};

	return SmartNameInput;
});