/**
 * Smart Name Selector UI module
 * - takes care of the user interface for Smart Name selection>insertion by the User.
 * 
 * @author JAMI, KADA
 * @since 19.1.2015
 */
define(["jquery",
		"underscore",
		"layout/InfiniteScrollingTemplate",
		"tpl!component/smartnames/selector/smartnameselectortable",
		"utils/webi18n!SmartNameSelector",
		"./smartnameselector-style"
	],
	function ($,
		_,
		InfiniteScrollingTemplate,
		tpl,
		webI18n) {

		"use strict";

		/**
		 * Smartname selector view
		 * @param {Function} [smartNameDisplayFunction] - Optional function to display the smartnames in the selector; argument is the {SmartName}.
		 */
		var SmartNameSelectorView = function (smartNameDisplayFunction) {

			// private variables
			this._onSectionSelectedCallbacks = [];
			this._onSmartnameSelectedCallbacks = [];
			this._activeSection = "";
			this._activeSectionSmartNames = [];

			//HTML elements
			this._wrapper = $("<div class='SmartNameSelectorView-wrapper'></div>");
			this._leftCol = $("<div class='SmartNameSelectorView-leftCol wcr-table'></div>");
			this._rightCol = $("<div class='SmartNameSelectorView-rightCol wcr-table'></div>");
			this._wrapper.append(this._leftCol);
			this._wrapper.append(this._rightCol);


			// Create the left section column and prefill with the passed sections.
			this._sectionTableHeader = $("<div class='SmartNameSelectorView-sectionTableHeader wcr-table-filter'/>").append(
				$("<div/>").addClass("add-spacing-bottom").append($("<label/>").text(webI18n.t("SECTIONS"))));
			this._sectionTable = $("<table class='SmartNameSelectorView-sectionTable'></table>");
			this._sectionTable.append("<tr><td>Loading Sections<br><img src='" + this._loaderGifUrl + "'/></td></tr>");
			var smartnameSectionTableWrapper = this._sectionTable.wrap("<div class='SmartNameSelectorView-smartnameMenuTableWrapper'></div>").parent();
			this._leftCol.append(this._sectionTableHeader);
			this._leftCol.append(smartnameSectionTableWrapper);

			//Create the right section column > stores Smart Names for a given section
			this._smartNameTableHeader = $("<div class='SmartNameSelectorView-smartNameTableHeader'></div>");
			this._smartNameTableHeaderSearchInput = $("<input class='form-control' type='text'></input>");
			this._smartNameTableHeader.append(
				$("<div/>").addClass("wcr-table-filter").append(
					$("<div/>").addClass("wcr-table-actions-row no-actions").append(
						$("<div/>").addClass("quick-filter").append(
							$("<div/>").addClass("input-group").append(
								this._smartNameTableHeaderSearchInput).append(
								$("<span/>").addClass("input-group-addon").append(
									$("<i/>").addClass("icon icon-WCR_Actions-Search12")
								)
							)
						)
					)
				)
			);
			this._smartnameMenuTable = $("<table class='SmartNameSelectorView-smartnameMenuTable'></table>");
			var smartnameMenuTableWrapper = this._smartnameMenuTable.wrap("<div class='SmartNameSelectorView-smartnameMenuTableWrapper'></div>").parent();
			this._rightCol.append(this._smartNameTableHeader);
			this._rightCol.append(smartnameMenuTableWrapper);

			// SmartNameTable template
			this.t = new InfiniteScrollingTemplate(webI18n, tpl, 400, 100);
			// Set the context for the template
			this.t.setContext(this._smartnameMenuTable);
			this.t.setScrollingContext(smartnameMenuTableWrapper);
			// Register click event
			this.t.bind("click", "td", this._smartNameClicked, this);
			// Error message function
			this.t.d.getErrorMsg = _.bind(function () {
				var labelStr = "NO_SMARTNAMES_FOUND_SECTION";
				if (this._smartNameTableHeaderSearchInput.val().length > 0) {
					labelStr = "NO_SMARTNAMES_FOUND_FILTER";
				}
				return webI18n.t(labelStr);
			}, this);


			// Register a listener on the search input
			var self = this;
			this._smartNameTableHeaderSearchInput.keyup(_.debounce(function () {
				var filterVal = self._smartNameTableHeaderSearchInput.val();

				// Only do this when there are smartNamesAvailable
				if (self._activeSectionSmartNames.length > 0) {
					// Make sure value is a correct filter value.
					if (filterVal === undefined || filterVal === null) {
						filterVal = "";
					}
					self._showAvailableSmartNames(filterVal);
				}
			}, 50));
			// Disable by default, will be enabled when smartnames will be set.
			this._smartNameTableHeaderSearchInput.prop('disabled', true);

			this._smartNameDisplayFunction = smartNameDisplayFunction || this._defaultSmartNameDisplayFunction;
		};

		/**
		 * Default display function for smartnames in the popup
		 * @param   {SmartName} smartName - The smartName to display
		 */
		SmartNameSelectorView.prototype._defaultSmartNameDisplayFunction = function (smartName) {
			if (smartName.getName() == "ProjAttr") {
				return smartName.getArguments()[0] + " (Project)";
			} else if (smartName.getName() == "DocAttr") {
				return smartName.getArguments()[0] + " (Document)";
			} else {
				return smartName.getFullName();
			}
		};

		/**
		 * Reference to the "Search Icon" image.
		 */
		SmartNameSelectorView.prototype._searchIconUrl = window.webcenterConfig.imageBaseUrl + "search.png";

		/**
		 * Reference to the animated "Loading" image.
		 */
		SmartNameSelectorView.prototype._loaderGifUrl = window.webcenterConfig.imageBaseUrl + "loaderTransparent.gif";


		/**
		 * Populates the left column with the pre-loaded Sections to choose from
		 */
		SmartNameSelectorView.prototype.setSections = function (sections) {
			if (sections === undefined) {
				return;
			}

			var self = this;
			this._sectionTable.html("");
			for (var i = 0; i < sections.length; i++) {
				var sectionRowElem = $("<tr/>")
					.append($("<td/>")
						.attr("id", sections[i].getSectionName())
						.text(webI18n.t(sections[i].getSectionLangTag()))
					);
				self.addSectionRowClickEvent(sectionRowElem, sections[i].getSectionName());
				this._sectionTable.append(sectionRowElem);
			}

			if (sections[0] !== null && sections[0] !== undefined) {
				this.selectSection(sections[0].getSectionName());
			}
		};

		/**
		 * Registers an "onClick" trigger handler to a specified SmartName Section row 
		 * 
		 * @param sectionRowElem
		 * @param sectionName
		 */
		SmartNameSelectorView.prototype.addSectionRowClickEvent = function (sectionRowElem, sectionName) {
			var self = this;
			sectionRowElem.click(function () {
				self.selectSection(sectionName);
			});
		};

		/**
		 * Retrieves a TR element that refers to a SmartName Section of a specified name
		 * 
		 * @param name
		 * @returns JQuery TR element
		 */
		SmartNameSelectorView.prototype._getSectionContainerRow = function (name) {
			var resultElem = null;
			this._sectionTable.find("td").each(function (i, e) {
				var eJq = $(e);
				if (eJq.attr("id") == name) {
					resultElem = eJq.parent();
				}
			});

			return resultElem;
		};

		/**
		 * Executed when a Section is selected by the User
		 * 
		 * @param name
		 */
		SmartNameSelectorView.prototype.selectSection = function (name) {

			// Remember the active section
			this._activeSection = name;
			// Clear the active smartName;
			this._activeSectionSmartNames = [];

			// Reset right column content
			this._smartnameMenuTable.html("<tr><td>Loading Smartnames<br><img src='" + this._loaderGifUrl + "'/></td></tr>");

			// Highlight the selected section
			var sectionRow = this._getSectionContainerRow(name);
			if (sectionRow !== null && sectionRow !== undefined) {
				//unhighlight all previously highlighted rows
				this._leftCol.find("tr").removeClass("activeCategory");

				//and highlight the processed one
				sectionRow.attr("class", "activeCategory");
			}

			// Execute any callbacks registered for this event
			for (var i = 0; i < this._onSectionSelectedCallbacks.length; i++) {
				this._onSectionSelectedCallbacks[i](name);
			}

		};

		/**
		 * Registers a callback function for "SmartName section selected" event
		 * 
		 * @param onSectionSelectedCallback
		 */
		SmartNameSelectorView.prototype.onSectionSelected = function (onSectionSelectedCallback) {
			this._onSectionSelectedCallbacks.push(onSectionSelectedCallback);
		};

		/**
		 * Populate the right column of the interface with the provided set of Smart Names
		 * 
		 * @param sectionName
		 * @param smartnames - [] of SmartName objects 
		 */
		SmartNameSelectorView.prototype.populateSmartNamesForSection = function (sectionName, smartnames) {

			// Check if the passed SmartNames belong to the active section.
			if (sectionName !== this._activeSection) {
				return;
			}

			// If this is the case, remember them.
			this._activeSectionSmartNames = smartnames;
			this._smartNameTableHeaderSearchInput.prop('disabled', false);
			this._smartNameTableHeaderSearchInput.val("");
			this._showAvailableSmartNames("");
		};

		/**
		 * Show the available SmartNames and highlight/hide them according to the "filter" value.
		 */
		SmartNameSelectorView.prototype._showAvailableSmartNames = function (filter) {
			// All the smart name objects who match the filter
			_.defer(_.bind(function () {
				var smartnames = [];

				var lowerCaseFilter = filter.toLowerCase();
				var firstPart, highLightPart, lastPart, smartName, displayName, startHighLight;
				// See how may of the SmartNames match the filter
				for (var i = 0; i < this._activeSectionSmartNames.length; i++) {
					smartName = this._activeSectionSmartNames[i];
					displayName = this._smartNameDisplayFunction(smartName);
					startHighLight = displayName.toLowerCase().indexOf(lowerCaseFilter);
					if (startHighLight !== -1) {
						firstPart = displayName.substr(0, startHighLight);
						highLightPart = displayName.substr(startHighLight, filter.length);
						lastPart = displayName.substr(startHighLight + filter.length, displayName.length);

						smartnames.push({
							'smartName': smartName,
							'firstPart': firstPart,
							'highLightPart': highLightPart,
							'lastPart': lastPart
						});
					}
				}
				this.t.d.smartNames = smartnames;
				this.t.render();
			}, this));

		};

		/**
		 * Executed when smart name is clicked in smartname table
		 * @param {DOMObject} el
		 * @param {Event} event
		 */
		SmartNameSelectorView.prototype._smartNameClicked = function (el, event) {
			var smartnameName = $(el).attr("data-bind-smartname");
			this.selectSmartname(smartnameName);
		};

		/**
		 * Executed when a Smart Name is selected by the User
		 * 
		 * @param smartnameName
		 */
		SmartNameSelectorView.prototype.selectSmartname = function (smartnameName) {
			//trigger "SmartName Selected" event callback functions
			for (var i = 0; i < this._onSmartnameSelectedCallbacks.length; i++) {
				this._onSmartnameSelectedCallbacks[i](smartnameName);
			}
		};


		/**
		 * Registers a callback function for "SmartName selected" event
		 * 
		 * @param onSmartnameSelectedCallback
		 */
		SmartNameSelectorView.prototype.onSmartNameSelected = function (onSmartnameSelectedCallback) {
			this._onSmartnameSelectedCallbacks.push(onSmartnameSelectedCallback);
		};


		/**
		 * Show the error message.
		 */
		SmartNameSelectorView.prototype.showErrorMsg = function (message) {

			this._sectionTable.html("");
			var errorMsgElem = $("<tr class='SmartNameSelectorView-smartnameMenuTable-errorRow'><td></td></tr>");
			errorMsgElem.children().first().text(message);
			this._sectionTable.append(errorMsgElem);
		};


		/**
		 * Returns a reference to the wrapper DIV element of this module
		 * 
		 * @returns JQuery object
		 */
		SmartNameSelectorView.prototype.getJQElement = function () {
			return this._wrapper;
		};


		return SmartNameSelectorView;
	});
