define(["jquery",
	"component/smartnames/smartfield/smartnamedictionarysection",
	"utils/Popup",
	"component/smartnames/selector/smartnameselector-view",
	"utils/webi18n!SmartNameSelector"
], function ($, SmartNameDictionarySection, PopUp, SmartNameSelectorView, webI18n) {

	/**
	 * @param {SmartNameDictionaryService} smartNameDictionaryService - The dictionary service for all smartnames
	 * @param {Function} [smartNameDisplayFunction] - Optional function to display a smartname in the selector; argument: {SmartName}
	 */
	var SmartNameSelector = function (smartNameDictionaryService, smartNameDisplayFunction) {

		this._onSmartnameSelectedCallbacks = [];
		this._onSelectorClosedCallbacks = [];
		this._onSelectorOpenedCallbacks = [];
		this._smartNameDictionaryService = smartNameDictionaryService;
		this._smartNameDisplayFunction = smartNameDisplayFunction;
		this._smartNameSelectorView = null;
		this._popUp = null;
		this._disabled = false;


		// Create the button.
		this._button = $("<div class='SmartNameSelector input-group-addon btn btn-default'></div>").append(this._getIcon());
		this._button.click(this.openSmartNameSelector.bind(this));
	};

	/**
	 * Initialization of the UI when the Selector Popup gets opened 
	 */
	SmartNameSelector.prototype.openSmartNameSelector = function () {

		// Do nothing when disabled.
		if (this._disabled) {
			return;
		}

		// Check if a view already exists ...
		if (this._smartNameSelectorView == null) {

			// View does not yet exist, create one ...
			this._smartNameSelectorView = new SmartNameSelectorView(this._smartNameDisplayFunction);


			// Define what should happen when a section is clicked in the view.
			this._smartNameSelectorView.onSectionSelected((function (name) {
				this._smartNameDictionaryService.getSmartNamesForSection(name, (function (sectionName, smartnames) {
					this._smartNameSelectorView.populateSmartNamesForSection(sectionName, smartnames);
				}).bind(this));
			}).bind(this), function (errorMsg) {
				// Do on error
			});

			// Define what should happen when a smartname is clicked in the view.
			this._smartNameSelectorView.onSmartNameSelected((function (smartnameName) {
				//close the popup, this will also destroy it
				this._popUp.close();

				//and trigger the registered callbacks
				for (var i = 0; i < this._onSmartnameSelectedCallbacks.length; i++) {
					this._onSmartnameSelectedCallbacks[i](smartnameName);
				}
			}).bind(this));
		}

		// Show the view in a popup, creat one if it does not yet exist.
		if (this._popUp == null) {
			this._popUp = new PopUp(this._smartNameSelectorView.getJQElement());
			this._popUp.onClose(_.bind(function () {
				
				// Execute the correct callbacks.
				$.each(	this._onSelectorClosedCallbacks, function (i, callback) {
					callback();
				});
				
				// Remove the element from the page.
				this._popUp.getJQElement().remove();
				this._smartNameSelectorView = null;
				this._popUp = null;
				
			}, this));
			this._popUp.setTitle(webI18n.t("INSERT_SMARTNAME"));
		}

		// Open the popup
		this._popUp.open();
		
		// Execute the correct callbacks.
		$.each(	this._onSelectorOpenedCallbacks, function (i, callback) {
			callback();
		});
		
		// Load the contents for the popup.
		var self = this;
		this._smartNameDictionaryService.getAllSections(function (sections) {
			self._smartNameSelectorView.setSections(sections);
		}, function (msg) {
			self._smartNameSelectorView.showErrorMsg(msg);
		});
	};

	/**
	 * Change to another icon.
	 */
	SmartNameSelector.prototype.setIcon = function(iconType) {
		this._button.empty().append(this._getIcon(iconType));
	};
	
	/**
	 * Get the icon for the given type, the following types are supported:
	 * 
	 * - normal (default).
	 * - resolved
	 * - warning
	 * 
	 * If another string different than these ones is passed, that one is used as the icon "class".
	 */
	SmartNameSelector.prototype._getIcon = function(iconType) {
		if (!iconType || iconType == "normal") {
			var icon = $("<i>", {"class" : "icon icon-WCR_SmartName icon-green"});
			return icon;
		} else if (iconType == "resolved") {
			var icon = $("<i>", {"class" : "icon containerIcon icon-WCR_Text-Field-SN-Resolved-cont"});
			var subIcon = $("<i>", {"class" : "icon subIcon icon-green icon-WCR_Text-Field-SN-Resolved-sub"});
			return icon.append(subIcon);
		} else if (iconType == "warning") {
			var icon = $("<i>", {"class" : "icon containerIcon icon-WCR_Text-Field-SN-Warning-cont"});
			var subIcon = $("<i>", {"class" : "icon subIcon icon-red icon icon-WCR_Text-Field-SN-Warning-sub"});
			return icon.append(subIcon);
		} else {
			var icon = $("<i>", {"class" : "icon "+iconType});
			return icon;
		}
	};
	
	/**
	 * Registers a callback function for "SmartName selected" event
	 * 
	 * @param onSmartNameSelectedFunction
	 */
	SmartNameSelector.prototype.onSmartNameSelected = function (onSmartNameSelectedFunction) {
		this._onSmartnameSelectedCallbacks.push(onSmartNameSelectedFunction);
	};
	
	/**
	 * Register a callback that should be called when the selector is opened.
	 * 
	 * @param callback
	 */
	SmartNameSelector.prototype.onSelectorOpened = function (callback) {
		this._onSelectorOpenedCallbacks.push(callback);
	};
	
	/**
	 * Register a callback that should be called when the selector is closed.
	 * 
	 * @param callback
	 */
	SmartNameSelector.prototype.onSelectorClosed = function (callback) {
		this._onSelectorClosedCallbacks.push(callback);
	};

	/**
	 * Disable the SmartNameSelector.
	 */
	SmartNameSelector.prototype.disable = function () {
		this._disabled = true;
		this._button.addClass("SmartNameSelector-Disabled");
		this._button.attr("disabled", "disabled");
	};

	/**
	 * Enable the SmartNameSelector.
	 */
	SmartNameSelector.prototype.enable = function () {
		this._disabled = false;
		this._button.removeClass("SmartNameSelector-Disabled");
		this._button.removeAttr("disabled");
	};

	/**
	 * Returns a reference to the wrapper DIV element of this module
	 * 
	 * @returns JQuery object
	 */
	SmartNameSelector.prototype.getJQElement = function () {
		return this._button;
	};

	return SmartNameSelector;


});
