define(["jquery"], function ($) {

    /**
     * This class is a wrapper around an HTML input or select element,it offers a homogeneous interface for both elements.
     * 
     * @param {object} inputElement The jQuery wrapped input or select element.
     * 
     * @author KADA
     */
    var NormalInput = function NormalInput(inputElement) {
        
        // If not wrapped by jQuery, do it.
        if (!(inputElement instanceof jQuery)) {
            inputElement = $(inputElement);
        }
        
        // Check if the inputElement is a valid element.
        this._checkValidNormalInput(inputElement);
        
        // Assign it to the private var.
        this._inputElement = inputElement;
        
        // Check if we should use the input to show non smart name values.
        this.useForNonSmartNameValue = this._canUseForNonSmartNameValue(inputElement);
    };


    /**
     * Check if the input element is an allowed element.
     */
    NormalInput.prototype._checkValidNormalInput = function (normalInput) {
        if (normalInput === undefined) {
            throw new Error("The normalInput was not defined.");    
        }
        if (!normalInput.is("input") && !normalInput.is("select")) {
            throw new Error("The normalInput should be an input or select element.");   
        }   
    };
    
    
    /**
     * This method returns whether the normal input should be used to display values that do not contain smartnames.
     * 
     * We do not do this for text type inputs since the conversion of the paste/copy/drop/insert logic of
     * the SmartNameInput is not compatible with these of normal HTML type text inputs. (E.g. copying smart names).
     * 
     * @returns {Boolean} Whether this normal input can be used to display values that do not contain smart names.
     */
    NormalInput.prototype._canUseForNonSmartNameValue = function(inputElement) {

        // Time to do some browser specific black magic!
        
        // If it's an input element, we need to check the type.
        if(inputElement.is("input")) {
            
            // Get the type of input.
            var type = this._inputElement.attr("type");
            
            // See if this element is allowed to use for non smart name valus.
            var blacklist = ["text","number","search","tel","url", "email"];
            if ($.inArray(type, blacklist) >= 0) {
                // This is a blacklisted element, that should always be represented with a smart name input.
                return false;
            } else {
                // Okay we got an element that is not blacklisted, now check if it's supported in this browser.
                // If it's not supported and falls back to text, we can not use it for non smart name values.
                var testInput = document.createElement("input");
                testInput.setAttribute("type", type);
                return testInput.type !== "text";  
            }
            
        }
        // If it's a select, it's allowed, except if the select should allow custom values
        if (inputElement.is("select")) {
        	if(inputElement.attr('data-allowcustom')){
        		return false;
        	}
        	return true;
        }
        return false;
        
    };
    
    /**
     * Has this input a finite set of allowed values?
     * 
     * @returns [Boolean] Whether this input has a finite set of allowed values.
     */
    NormalInput.prototype.hasSetRestrictedValue = function() {
        
        // If it's an input element, we need to check the type.
        if(this._inputElement.is("input")) {
            
            // Get the type of input.
            var type = this._inputElement.attr("type");
            
            // See if the type of the input belongs to a type restricted set
            var setRestrictedInputTypes = ["radio","color","range"];
            return $.inArray(type, setRestrictedInputTypes) >= 0;
        }
        // If it's a select, it's always restricted to a set.
        if (this._inputElement.is("select")) {
            return true;
        }
        return false;
    };
    
    /**
     * Get the value of the property indicating the initial value for the SmartNameField.
     * 
     * @returns The value of the property indicating the initial value for the SmartNameField.
     */
    NormalInput.prototype.getInitValProperty = function() {
    	return this._inputElement.attr("data-smartnameval");
    };
    
    /**
     * Set the value of the property indicating the initial value for the SmartNameField.
     */
    NormalInput.prototype.setInitValProperty = function(val) {
    	return this._inputElement.attr("data-smartnameval", val);
    };
    
    /**
     * Get the value of the input.
     * 
     * @returns [String] the value if the input. 
     */
    NormalInput.prototype.getValue = function() {
        return this._inputElement.val();
    };
    
    /**
     * Set the focus on the input.
     */
    NormalInput.prototype.focus = function() {
        this._inputElement.focus();
    };
    
    /**
     * Blur the focus on the input.
     */
    NormalInput.prototype.blur = function() {
        this._inputElement.blur();
    };
    
    /**
     * Register a callback to be called when the input focuses.
     */
    NormalInput.prototype.onFocus = function(callback) {
        this._inputElement.on("focus", callback);
    };
    
    /**
     * Register a callback to be called when the widget blurs.
     */
    NormalInput.prototype.onBlur = function(callback) {
        this._inputElement.on("blur", callback);
    };
    
    /**
     * Set the value of the input.
     * 
     * @param value [String] the value if the input. 
     */
    NormalInput.prototype.setValue = function(value) {
    	// Just set the value in case of input ...
    	if(this._inputElement.is("input")) {
    		this._inputElement.val(value);
    	}
    	// In case of a select we first need to check if it's a valid value.
    	if (this._inputElement.is("select")) {
    		var optionElem = $(this._inputElement.find('option').filter(function(i, elem) {
				return $(elem).attr("value") === value;
			})[0]);
    		if (optionElem.length !== 0) {
    			optionElem.prop('selected', true);
    		}
        }
    };
        
    /**
     * Register a function that needs to be called each time the value of the input changes.
     * 
     * @param functionOnValueChange A function that needs to be called each time the value of the input changes.
     */
    NormalInput.prototype.onValueChange = function(functionOnValueChange) {
        this._inputElement.change(function () {
            functionOnValueChange(this.value);
        });
    };
    
    /**
     * Reset the value of the input.
     */
    NormalInput.prototype.reset = function() {
        
        // If it's an input element, we need to check the type.
        if(this._inputElement.is("input")) {
            this._inputElement.val("");
        } else if (this._inputElement.is("select")) {
            this._inputElement.find('option:eq(0)').prop('selected', true);
        }
    };
    
    /**
     * Show the input.
     */
    NormalInput.prototype.show = function() {
        this._inputElement.show();
    };
    
    /**
     * Hide the input.
     */
    NormalInput.prototype.hide = function() {
        this._inputElement.hide();
    };
    
    /**
     * Disable the input.
     */
    NormalInput.prototype.disable = function() {
        this._inputElement.prop('disabled', true);
    };
    
    /**
     * Enable the input.
     */
    NormalInput.prototype.enable = function() {
        this._inputElement.prop('disabled', false);
    };
    
    /**
     * Get the jQuery wrapped input element. 
     * 
     * @returns [JQuery] The jQuery wrapped input element.
     */
    NormalInput.prototype.getJQElement = function() {
        return this._inputElement;
    };
    
    /**
     * Get dropdown options of the input element
     * 
     * @returns {Array.<Object>} 
     */
    NormalInput.prototype.getDropdownOptions = function() {
     // create dropdown for select smartnames
    	var options = [];
		if(this._inputElement.is("select") && this._inputElement.attr('data-allowcustom')){
			_.each(this._inputElement.find("option"), function(option){
				options.push({
					value: option.value,
					text: option.textContent
				});
			});
		}
        return options;
    };
    
    return NormalInput;
});