define(["jquery",
        "component/smartnames/input/smartnameinput",
        "component/smartnames/input/lazysmartnameinput",
        "component/smartnames/smartfield/smartfield-style",
        "component/smartnames/smartfield/normalinput",
        "component/smartnames/smartfield/smartnamedropdown",
        "component/smartnames/encoding/smartnameencoding",
        "component/smartnames/selector/smartnameselector",
        "component/smartnames/smartfield/smartnamedictionarydefaultservice",
        "component/smartnames/smartfield/smartfield-view"
        ],
	function ($,
		SmartNameinput,
		LazySmartNameinput,
		SmartFieldStyle,
		NormalInput,
		SmartNameDropdown,
		SmartNameEncoding,
		SmartNameSelector,
		SmartNameDictionaryService,
		SmartFieldView) {
		"use strict";

		/**
		 * A SmartField is a certain WebCenter component that is used to represent HTML inputs (or selects) 
		 * in which you can make use of smart names.
		 * 
		 * @param {Object} normalInput The reference to the input or select that we need to enhance to become 
		 *                             a SmartField.
		 * @param {Boolean} nonLazyInit - If false, it is lazily initialized (LazySmartNameinput), if true it is not lazy (SmartNameinput)
		 * @param {SmartNameDicationaryService} [smartNameDictionaryService] - The dicationary service for loading smartnames and sections
		 * @param {Function} [smartnameDisplayFunction] - Smartname display function (e.g. translated). The argument will be the SmartName object.
		 * @param {Function} [smartnameSelectorDisplayFunction] - Smartname display function for in the smartname selector. The arguemnt will be the SmartName object.
		 * @param {SmartNameEncoding} [smartNameEncoding = SAVEDSEARCH] - Component responsible for encoding/decoding smartnames in a string.                                                  
		 */
		var SmartField = function (normalInput, nonLazyInit, smartNameDictionaryService, smartnameDisplayFunction, smartnameSelectorDisplayFunction, smartNameEncoding) {

			// Create all the components we need. 
			this._normalInput = new NormalInput(normalInput);

			this._smartNameEncoding = smartNameEncoding || SmartNameEncoding.SAVEDSEARCH;

			this._smartNameDictionaryService = smartNameDictionaryService || SmartNameDictionaryService;

			this._smartNameSelector = new SmartNameSelector(this._smartNameDictionaryService, smartnameSelectorDisplayFunction);

			// SmartNames which support a alternative input method (e.g. dropdown) 
			// but are entered by the user in the "SmartNameInput mode" will contain a "marker" SmartName.
			var markSmartNames = this._normalInput.useForNonSmartNameValue || this._normalInput.getJQElement().attr('data-smartnamemarker') === "always";

			// Only create the SmartNameInput when we do not want lazy initialization.
			var smartNameInputOptions = {
				encoding: this._smartNameEncoding,
				mark: markSmartNames,
				displayFunction: smartnameDisplayFunction,
				selectorDisplayFunction: smartnameSelectorDisplayFunction,
				dictionaryService: this._smartNameDictionaryService
			};
			if (nonLazyInit) {
				this._smartNameInput = new SmartNameinput(undefined, smartNameInputOptions);
			} else {
				this._smartNameInput = new LazySmartNameinput(undefined, smartNameInputOptions);
			}

			// create dropdown for select smartnames
			var dropdownOptions = this._normalInput.getDropdownOptions();
			if (dropdownOptions.length > 0) {
				this._smartNameDropdown = new SmartNameDropdown(dropdownOptions, this._smartNameInput, this._normalInput.getJQElement().attr('data-delimiter'));
			}

			// Create the view.
			this._smartFieldView = new SmartFieldView(this._normalInput.getJQElement(), this._smartNameInput.getJQElement(),
				this._smartNameSelector.getJQElement(), this._smartNameDropdown && this._smartNameDropdown.getJQElement());

			// Additional properties of this component
			this._onValueChangedFunctions = []; // The functions that need to be updated when the value of this input changes.
			this._onFocus = []; // The function that need to be called when this field gains focus.
			this._onBlur = []; // The function that need to be called when this field loses focus.
			this._smartNameInputActive = false; // Keep track of which input we're showing.
			this._disabled = false; // Keep track of whether this input is enabled or not.

			// Start implementing all the callbacks and controlling the components.
			var self = this;
			this._smartNameSelector.onSmartNameSelected(function (smartnameName) {

				// If the SmartField is disabled, do not insert.
				if (self._disabled) {
					return;
				}

				if (!self._smartNameInputActive) {
					// activate the smartNameInput
					self._normalInput.hide();
					self._smartNameInput.show();
					self._smartNameInputActive = true;

					self._smartNameInput.clear();
					// Transfer the contents from the normal input to the smart name input
					// ... but only if it's not a set restricted value, in that case no content should be copied.
					if (!self._normalInput.hasSetRestrictedValue()) {
						self._smartNameInput.insertText(self._normalInput.getValue());
					}
				}

				// Focus on the smart name input.
				self._smartNameInput.focus();
				// Insert the smart name and hand focus over to the smart name input.
				self._smartNameInput.insertSmartName(smartnameName);
				// Focus on the smart name input.
				self._smartNameInput.focus();
			});

			// What needs to happen when the value of the smart name input changes ...
			var updateOnSmartNameInputChange = function (value) {
				// Only do this when the smart name input is active.
				if (self._smartNameInputActive) {
					// Update the value in the hidden input
					self._smartFieldView.setHiddenValue(value);

					// Execute the functions who are listening for a changed value.
					for (var i = 0; i < self._onValueChangedFunctions.length; i++) {
						self._onValueChangedFunctions[i](value);
					}

					// When the contents of the smart name input are empty, try to show the initial input again.
					if (value === "" && self._normalInput.useForNonSmartNameValue) {
						// activate the normalInput
						self._smartNameInput.hide();
						self._normalInput.show();
						self._smartNameInputActive = false;

						// transfer the contents from the normal input
						self._normalInput.reset();
						self._normalInput.focus();
					}

				} else {
					// When the contents of the smart name input are empty, don't show the 'smartname input field'.
					if (!(value === "" && self._normalInput.useForNonSmartNameValue)) {
						// activate the smartNameInput
						self._normalInput.hide();
						self._smartNameInput.show();
						self._smartNameInputActive = true;

						// update the hidden value. 
						self._smartFieldView.setHiddenValue(value);

						// Focus on the smart name input.
						self._smartNameInput.focus();
					}
				}
			};
			this._smartNameInput.onValueChange(updateOnSmartNameInputChange);

			// What needs to happen when the value of the normal input changes ...
			var updateOnNormalInputChange = function (value) {
				// Only do this when the normal input is active.
				if (!self._smartNameInputActive) {

					// Update the value in the hidden input
					self._smartFieldView.setHiddenValue(self.getValue());

					// Execute the functions who are listening for a changed value.
					for (var i = 0; i < self._onValueChangedFunctions.length; i++) {
						self._onValueChangedFunctions[i](value);
					}
				}
			};
			this._normalInput.onValueChange(updateOnNormalInputChange);
			
			// Blur/Focus listeners
			this._normalInput.onFocus(function () {
				// Only do this when the normal input is not active.
				if (!this._smartNameInputActive) {
					for (var i=0; i < this._onFocus.length; i++) {
						this._onFocus[i]();
					}
				}
			}.bind(this));
			this._normalInput.onBlur(function () {
				// Only do this when the normal input is not active.
				if (!this._smartNameInputActive) {
					for (var i=0; i < this._onBlur.length; i++) {
						this._onBlur[i]();
					}
				}
			}.bind(this));
			this._smartNameInput.onFocus(function () {
				// Only do this when the smartname input is not active.
				if (this._smartNameInputActive) {
					for (var i=0; i < this._onFocus.length; i++) {
						this._onFocus[i]();
					}
				}
			}.bind(this));
			this._smartNameInput.onBlur(function () {
				// Only do this when the smartname input is not active.
				if (this._smartNameInputActive) {
					for (var i=0; i < this._onBlur.length; i++) {
						this._onBlur[i]();
					}
				}
			}.bind(this));
			

			// Check if we should show the normalInput to start with.
			this._updateFields(this._normalInput.getValue());

			// Make a reference to this object in the element.
			this._normalInput.getJQElement()[0].SmartField = this;

			// Fire a "SmartFieldInitiated" (jQuery) event on the input to indicate that the SmartField is initiated.
			this._normalInput.getJQElement().triggerHandler("SmartFieldInit", [this]);

			// Check if we should use the value in attribute "data-smartnameval"
			var dataSmartNameValAttr = this._normalInput.getInitValProperty();
			if (dataSmartNameValAttr !== undefined && dataSmartNameValAttr.length > 0) {
				// Set the value found in this attribute.
				this.setValue(dataSmartNameValAttr);
				// remove it after we did this.
				this._normalInput.setInitValProperty("");
			} else {
				// Update the hidden field...
				updateOnNormalInputChange(this._normalInput.getValue());
			}

			// Start autoloading the SmartNames to insert in autocomplete.
			this._smartNameDictionaryService.autoLoadSectionAndSmartNames(function (smartNames) {
				self._smartNameInput.setSmartNames(smartNames);
			});

			// Trigger resize operation
			this._smartNameInput.resize();
		};

		/**
		 * Show right input fields based on 'new' value
		 * @param {String} value
		 */
		SmartField.prototype._updateFields = function (value) {
			// if it does not contain smartname and we want to use normal input for those, show normal input
			if (!this._smartNameEncoding.containsSmartName(value) && this._normalInput.useForNonSmartNameValue) {
				// show normal input
				this._normalInput.setValue(value);
				this._smartNameInput.hide();
				this._normalInput.show();
				this._smartNameInputActive = false;
			} else {
				// show smartname input
				this._smartNameInput.setValue(value);
				this._smartNameInput.show();
				this._normalInput.hide();
				this._smartNameInputActive = true;
			}
		};

		/**
		 * Show the field.
		 */
		SmartField.prototype.show = function () {
			this._smartFieldView.getJQElement().show();
		};
		
		/**
		 * Hide the field.
		 */
		SmartField.prototype.hide = function () {
			this._smartFieldView.getJQElement().hide();
		};
		
		/**
		 * Get the value of this SmartField.
		 */
		SmartField.prototype.setWidth = function (width) {
			this._smartFieldView.setWidth(width);
			this._smartNameInput.resize();
		};

		/**
		 * Get the value of this SmartField.
		 */
		SmartField.prototype.getValue = function () {
			if (this._smartNameInputActive) {
				return this._smartNameInput.getValue();
			} else {
				return this._normalInput.getValue();
			}
		};

		/**
		 * Set the value of this SmartField.
		 * @param {string} value The correctly encoded value of this SmartField, this value 
		 *                       can contain correctly encoded smartnames (e.g. "Project_[[ProjAttr:Name1]]").
		 */
		SmartField.prototype.setValue = function (value) {
			// Validate the value that is passed to this method...
			// All values that enter should be correctly encoded.
			// If the error below is shown you did something wrong.
			try {
				this._smartNameEncoding.decodeSmartNameString(value);
			} catch (e) {
				window.console.error("Error: Illegal value used in setValue on SmartField. Falling back to text...");
				// Escaping all special characters (to make sure it does not crash)
				value = this._smartNameEncoding.escapeString(value);
			}

			this._updateFields(value);

			// Update the value in the hidden input
			this._smartFieldView.setHiddenValue(value);
		};


		/**
		 * Disable the smart field.
		 */
		SmartField.prototype.disable = function () {
			this._smartNameInput.disable();
			this._normalInput.disable();
			this._smartNameSelector.disable();
			this._smartFieldView.disable();
			if(this._smartNameDropdown) {
				this._smartNameDropdown.disable();
			}
			this._disabled = true;
		};

		/**
		 * Enable the smart field.
		 */
		SmartField.prototype.enable = function () {
			this._smartNameInput.enable();
			this._normalInput.enable();
			this._smartNameSelector.enable();
			this._smartFieldView.enable();
			this._disabled = false;
		};

		/**
		 * Focus on the smart field.
		 */
		SmartField.prototype.focus = function () {
			if (this._smartNameInputActive) {
				this._smartNameInput.focus();
			} else {
				this._normalInput.focus();
			}
		};
		
		/**
		 * Blur the smart field.
		 */
		SmartField.prototype.blur = function () {
			if (this._smartNameInputActive) {
				this._smartNameInput.blur();
			} else {
				this._normalInput.blur();
			}
		};
		
		
		/**
		 * Stop Editing the field
		 */
		SmartField.prototype.stopEditing = function () {
			this._smartNameInput.stopEditing();
		};
		
		/**
		 * Switch between inline-block or block.
		 * Default is inline-block, which behaves the same as the transformed input.
		 * When using a percentage in the width field, the component should be set to block
		 */
		SmartField.prototype.setIsInline = function (inline) {
			this._smartFieldView.setIsInline(inline);
			this._smartNameInput.resize();
		};

		/**
		 * Returns whether the smart field is enabled or disabled.
		 * 
		 * @returns {Boolean} Whether the smart field is enabled or disabled.
		 */
		SmartField.prototype.isDisabled = function () {
			return this._disabled;
		};

		/**
		 * Clear the contents of this SmartField.
		 */
		SmartField.prototype.clear = function () {
			if (this._smartNameInputActive) {
				this._smartNameInput.clear();
			} else {
				this._normalInput.reset();
			}

			// Update the value in the hidden input
			this._smartFieldView.setHiddenValue("");
		};

		/**
		 * Register a function that should be called when the value of the SmartField changes.
		 * The new value will be passed to the function as the first argument.
		 * 
		 * @param {function} onValueChangedFunction A function that should be called whenever the value of the SmartField changes.
		 */
		SmartField.prototype.onValueChange = function (onValueChangedFunction) {
			this._onValueChangedFunctions.push(onValueChangedFunction);
		};
		
		/**
		 * Register a function that should be called when the field gains focus.
		 */
		SmartField.prototype.onFocus = function (callback) {
			this._onFocus.push(callback);
		};
		
		/**
		 * Register a function that should be called when the field loses focus.
		 */
		SmartField.prototype.onBlur = function (callback) {
			this._onBlur.push(callback);
		};

		/**
		 * Register a function that should be called when the "smart name popover" is shown.
		 */
		SmartField.prototype.onSmartNamePopoverShow = function (callback) {
			this._smartNameInput.onSmartNamePopoverShow(callback);
		};
		
		/**
		 * Register a function that should be called when the "smart name popover" is hidden.
		 */
		SmartField.prototype.onSmartNamePopoverHide = function (callback) {
			this._smartNameInput.onSmartNamePopoverHide(callback);
		};
		
		/**
		 * Register a callback that should be called when the smart name selector is opened.
		 */
		SmartField.prototype.onSmartNameSelectorOpened = function (callback) {
			this._smartNameSelector.onSelectorOpened(callback);
		};
		
		/**
		 * Register a callback that should be called when the smart name selector is closed.
		 */
		SmartField.prototype.onSmartNameSelectorClosed = function (callback) {
			this._smartNameSelector.onSelectorClosed(callback);
		};
		
		/**
		 * Transform this SmartField back to the original input.
		 * 
		 * (No way to go back unless you create a new SmartField from it).
		 */
		SmartField.prototype.toNormalInput = function () {

			// Fill the current value in the normal input
			this._normalInput.setValue(this.getValue());

			// Destroy the child components of this component that can be destroyed.
			this._smartNameInput.destroy();

			// Transform the view back to a normalInput.
			this._smartFieldView.toNormalInput();

			if (this._smartNameDropdown) {
				this._smartNameDropdown.destroy();
			}
		};
		
		/**
		 * Indicates whether the SmartField is switched to the actual "free-edit Smartfield input" mode. 
		 * If not it could be a dropdown-selector that is currently active instead.
		 * This mode is switched as smartnames/custom values are added to the field or removed (once the edit field is cleared out, the drop down selector is shown instead again)
		 * 
		 * @returns {Boolean}
		 */
		SmartField.prototype.isSmartFieldInputActive = function () {
			return this._smartNameInputActive;
		};

		/**
		 * Change the icon of the smartname dropdown.
		 */
		SmartField.prototype.setSelectorIcon = function (iconType) {
			this._smartNameSelector.setIcon(iconType);
		};

		/**
		 * Get a jQuery wrapped reference to the encapsulating div of this component.
		 * 
		 * @returns {jQuery} The jQuery wrapped reference to the encapsulating div of this component.
		 */
		SmartField.prototype.getJQElement = function () {
			return this._smartFieldView.getJQElement();
		};

		/**
		 * Get the original input/select field that was used to create the smartfield
		 * 
		 * @returns {jQuery} the original input/select field
		 */
		SmartField.prototype.getOriginalElement = function () {
			return this._normalInput.getJQElement();
		};

		/**
		 * Called when the smartfield is resized
		 */
		SmartField.prototype.resize = function () {
			this._smartNameInput.resize();
		};

		/**
		 * Called when the smartfield has to be destroyed
		 */
		SmartField.prototype.destroy = function () {
			this.toNormalInput();
		};

		/**
		 * Method to automatically apply this module to an input or select element ...
		 */
		SmartField.wcrinit = function (field) {
			new SmartField(field);
		};

		/**
		 * Method to automatically destroy this component once initialized for a div...
		 * 
		 * @param div {DomElement} the DomElement to destroy again
		 */
		SmartField.wcrdestroy = function (field) {
			field.SmartField.destroy();
		};

		return SmartField;
	});
