define(["jquery",
		"component/smartnames/smartfield/smartnamedictionaryservice",
        "component/smartnames/smartfield/smartnamedictionarysection",
        "component/smartnames/encoding/smartname",
	   "underscore"],
	function ($,
		SmartnameDictionaryService,
		SmartNameDictionarySection,
		SmartName,
		_) {
		"use strict";

		/**
		 * @class (singleton)
		 * @exports SmartNameDictionaryServiceDefault (singleton)
		 * @augments SmartnameDictionaryService
		 *             
		 * This component is the default smartname dictionary service for WebCenter.
		 * It uses smartnamedictionaryservice servlet to fetch sections and smartnames.
		 * It is returned as a singleton!
		 */
		var SmartNameDictionaryServiceDefault = function () {
			SmartnameDictionaryService.call(this);

			// The baseUrl of the service.
			this._jspBaseUrl = window.webcenterConfig.jspBaseUrl;

			// These variables will cache the returned sections and smartnames
			this._cachedSections = [];
			this._cachedSmartNamesForSection = {};

			// These variables take care of preventing duplicate requests.
			this._sectionsAreBeingRetrieved = false;
			this._onSectionsRetrieved = [];
			this._onSectionsRetrievedFail = [];
			this._smartNamesAreBeingRetrievedForSections = [];
			this._onSmartNamesRetrievedForSections = {};
			this._onSmartNamesRetrievedForSectionsFail = {};
		};

		SmartNameDictionaryServiceDefault.prototype = Object.create(SmartnameDictionaryService.prototype);
		SmartNameDictionaryServiceDefault.prototype.constructor = SmartNameDictionaryServiceDefault;

		/**
		 * Utility function used below
		 * @param   {Array} arr
		 * @returns {Array}
		 */
		SmartNameDictionaryServiceDefault.prototype._removeFrom = function (arr) {
			var what, a = arguments,
				L = a.length,
				ax;
			while (L > 1 && arr.length) {
				what = a[--L];
				while ((ax = arr.indexOf(what)) != -1) {
					arr.splice(ax, 1);
				}
			}
			return arr;
		};

		/**
		 * @override
		 */
		SmartNameDictionaryServiceDefault.prototype.getAllSections = function (onSectionsRetrievedCallbackSuccess, onSectionsRetrievedCallbackFail) {
			// Register the callbacks.
			this._onSectionsRetrieved.push(onSectionsRetrievedCallbackSuccess);
			if (onSectionsRetrievedCallbackFail !== undefined) {
				this._onSectionsRetrievedFail.push(onSectionsRetrievedCallbackFail);
			}

			// Check if we're not already retrieving the sections.
			if (!this._sectionsAreBeingRetrieved) {

				// First check if we already cached all the sections.
				if (this._cachedSections.length !== 0) {

					// Execute the callback.
					onSectionsRetrievedCallbackSuccess(this._cachedSections);

				} else {
					// Fetch them.

					// Indicate that they're being fetched
					this._sectionsAreBeingRetrieved = true;

					// Do the async call to the service
					$.getJSON(this._jspBaseUrl + "smartnamedictionaryservice.wcr?action=GET_SECTIONS", _.bind(function (data) {

						// Check if we have a data result.
						if (data.result) {

							this._cachedSections = [];
							for (var i = 0; i < data.result.sections.length; i++) {
								this._cachedSections.push(new SmartNameDictionarySection(data.result.sections[i].name, data.result.sections[i].langTag));
							}

							// Indicate that the sections are not being fetched anymore 
							this._sectionsAreBeingRetrieved = false;

							// Execute the callbacks.
							for (i = 0; i < this._onSectionsRetrieved.length; i++) {
								this._onSectionsRetrieved[i](this._cachedSections);
							}

							// Check if we have an error result.
						} else if (data.error) {

							// Parse the error message.
							var msg = data.error.addonMsg;

							// Indicate that the sections are not being fetched anymore 
							this._sectionsAreBeingRetrieved = false;

							// Execute the callbacks.
							for (var i = 0; i < this._onSectionsRetrievedFail.length; i++) {
								this._onSectionsRetrievedFail[i](msg);
							}

						}
						// clear them.
						this._onSectionsRetrieved = [];
						this._onSectionsRetrievedFail = [];


					}, this)).fail(_.bind(function (jqxhr, textStatus, error) {

						// Try to parse the error.
						var msg;
						try {
							msg = JSON.parse(jqxhr.responseText).error.msg;
						} catch (e) {
							msg = error;
						}

						// Indicate that the sections are not being fetched anymore 
						this._sectionsAreBeingRetrieved = false;

						// Execute the callbacks.
						for (var i = 0; i < this._onSectionsRetrievedFail.length; i++) {
							this._onSectionsRetrievedFail[i](msg);
						}
						// clear them.
						this._onSectionsRetrieved = [];
						this._onSectionsRetrievedFail = [];

					}, this));
				}
			}
		};

		/**
		 * @override
		 */
		SmartNameDictionaryServiceDefault.prototype.getSmartNamesForSection = function (sectionName, onSmartNamesRetrievedCallbackSuccess, onSmartNamesRetrievedCallbackFail) {

			// Register the callbacks.

			// Add them to an array that has the sectionName as a property of the onSmartNamesRetrievedForSections object
			if (this._onSmartNamesRetrievedForSections[sectionName] === undefined) {
				this._onSmartNamesRetrievedForSections[sectionName] = [];
			}
			this._onSmartNamesRetrievedForSections[sectionName].push(onSmartNamesRetrievedCallbackSuccess);

			// Add them to an array that has the sectionName as a property of the onSmartNamesRetrievedForSectionsFail object
			// But only if they are defined.
			if (this._onSmartNamesRetrievedForSectionsFail[sectionName] === undefined) {
				this._onSmartNamesRetrievedForSectionsFail[sectionName] = [];
			}
			if (onSmartNamesRetrievedCallbackFail !== undefined) {
				this._onSmartNamesRetrievedForSectionsFail[sectionName].push(onSmartNamesRetrievedCallbackFail);
			}


			// Check if we're not already retrieving the SmartNames for this section.
			if ($.inArray(sectionName, this._smartNamesAreBeingRetrievedForSections) < 0) {

				// First check if we already cached the smartnames for this section
				if (this._cachedSmartNamesForSection[sectionName] !== undefined && this._cachedSmartNamesForSection[sectionName].length !== 0) {
					// Return them from cache
					onSmartNamesRetrievedCallbackSuccess(sectionName, this._cachedSmartNamesForSection[sectionName]);
				} else {

					// Indicate that we start retrieving them.
					this._smartNamesAreBeingRetrievedForSections.push(sectionName);

					// Fetch them async.
					$.getJSON(this._jspBaseUrl + "smartnamedictionaryservice.wcr?action=GET_SMARTNAMES_FOR_SECTION&section=" + sectionName, _.bind(function (data) {

						this._cachedSmartNamesForSection[sectionName] = [];
						for (var i = 0; i < data.result.smartnames.length; i++) {
							this._cachedSmartNamesForSection[sectionName].push(new SmartName(data.result.smartnames[i].name, data.result.smartnames[i].arguments));
						}

						// Indicate that the smartNames are not being retrieved anymore.
						this._removeFrom(this._smartNamesAreBeingRetrievedForSections, sectionName);

						// execute the callbacks
						for (var i = 0; i < this._onSmartNamesRetrievedForSections[sectionName].length; i++) {
							this._onSmartNamesRetrievedForSections[sectionName][i](sectionName, this._cachedSmartNamesForSection[sectionName]);
						}


					}, this)).fail(_.bind(function (jqxhr, textStatus, error) {

						// Try to parse the error.
						var msg;
						try {
							msg = JSON.parse(jqxhr.responseText).error.msg;
						} catch (e) {
							msg = error;
						}

						// Indicate that the smartNames are not being retrieved anymore.
						this._removeFrom(this._smartNamesAreBeingRetrievedForSections, sectionName);

						// Check if there are onFail callBacks

						// Execute the onFail callbacks.
						for (var i = 0; i < this._onSmartNamesRetrievedForSectionsFail[sectionName].length; i++) {
							this._onSmartNamesRetrievedForSectionsFail[sectionName][i](msg);
						}
					}, this));
				}
			}
		};

		var singleton = new SmartNameDictionaryServiceDefault();

		return singleton;

	});