define([], function () {
	"use strict";

	/**
	 * The smart name dicationary service used by smartfields.
	 * You should extend this one and override the functions getAllSections and getSmartNamesForSection.
	 * The function autoLoadSectionAndSmartNames depends on these and has a default implementation.
	 */
	var SmartNameDictionaryService = function () {
		this._autoLoadInProgress = false;
		this._onAutoLoadCompleted = [];
	};

	/**
	 * This function retrieves all SmartName Sections from the SmartNameDictionary possibly asynchronously
	 * 
	 * @param {Function} onSectionsRetrievedCallbackSuccess - The callback to execute on succesfull retrieval.
	 *														The callback will be passed the array {Array.<SmartNameDictionarySection>} containing all the sections. 
	 * @param {Function} onSectionsRetrievedCallbackFail - The callback to execute on failure of the request.
	 * 													 The callback will be passed the errormessage.
	 */
	SmartNameDictionaryService.prototype.getAllSections = function (onSectionsRetrievedCallbackSuccess, onSectionsRetrievedCallbackFail) {
		// make your own implementation
	};

	/**
	 * This function retrieves all SmartNames for a Section from the SmartNameDictionary possibly asynchronously
	 * 
	 * @param {String} sectionName - The name of the section to retrieve the SmartNames for.
	 * @param {Function} onSmartNamesRetrievedCallbackSuccess - The callback to execute on succesfull retrieval.
	 *                                                        The callback will be passed the array {Array.<SmartName>} as second parameter containing all the sections. The first parameter is the section name.
	 * @param {Function} onSmartNamesRetrievedCallbackFail - The callback to execute on failure of the request.
	 * 														The callback will be passed the errormessage.
	 */
	SmartNameDictionaryService.prototype.getSmartNamesForSection = function (sectionName, onSmartNamesRetrievedCallbackSuccess, onSmartNamesRetrievedCallbackFail) {
		// make your own implementation
	};

	/**
	 * This function automatically loads all the sections and SmartNames and caches them.
	 * Used for autocompletion.
	 * 
	 * @param {Function} onAutoLoadCompletedCallback - This callback will be called when all the SmartNames for all the sections 
	 * 													are loaded. The smartNames will be passed as an argument {Array.<SmartName>} to this callback.
	 */
	SmartNameDictionaryService.prototype.autoLoadSectionAndSmartNames = function (onAutoLoadCompletedCallback) {

		// Register the callbacks.
		this._onAutoLoadCompleted.push(onAutoLoadCompletedCallback);

		// Check if we're not already autoloading.
		if (!this._autoLoadInProgress) {

			// Start autoLoading
			this._autoLoadInProgress = true;

			// Start by retrieving all the sections.
			var self = this;
			this.getAllSections(function (returnedSections) {
				// This function
				function retrieveNextSectionSmartNames(sectionsTodo, smartNames) {
					// Check if there is still a section to fetch the SmartNames for
					if (sectionsTodo.length === 0) {

						// No section left to fetch smartNames for, complete autoload.
						self._autoLoadInProgress = false;
						// Execute the callbacks.
						for (var i = 0; i < self._onAutoLoadCompleted.length; i++) {
							self._onAutoLoadCompleted[i](smartNames);
						}
						// clear
						self._onAutoLoadCompleted = [];
						// End recursion here.
						return;

					} else {

						// Get the current section to retrieve the SmartNames for ...
						var section = sectionsTodo.pop();
						// Retrieve the SmartNames for this section.
						self.getSmartNamesForSection(section.getSectionName(), function (sectionName, sectionSmartNames) {
							// Once the SmartNames are retrieved ...

							// Add them to the total array
							if (sectionSmartNames) {
								for (var i = 0; i < sectionSmartNames.length; i++) {
									smartNames.push(sectionSmartNames[i]);
								}
							}

							// Retrieve the SmartNames for the next section (recursively) ...
							retrieveNextSectionSmartNames(sectionsTodo, smartNames);
						}, function () {
							// If we get here something went wrong fetching the smartnames ... do nothing with this in autoload.
						});
					}
				}
				// iterate over all the sections and load their individual smartnames.
				if (returnedSections !== undefined && returnedSections.length > 0) {
					// Start sequential retrieval of all the SmartNames for all the sections. 
					// (Do this on a clone of the array, slice(0) will clone ...)
					retrieveNextSectionSmartNames(returnedSections.slice(0), []);
				}
			}, function () {
				// If we get here something went wrong fetching the sections ... do nothing with this in autoload.
			});
		}


	};

	return SmartNameDictionaryService;

});