define(["jquery", "require", "./DropdownOverflowUtil", "lib/jquery.floatingscroll"], function ($, require, DropdownOverflowUtil) {
	"use strict";

	/**
	 * Constructs a table component...
	 * Mainly used in combination with the wcr-table html structure.
	 * 
	 * @param {Element} context - the wcr-table element to work on
	 * @param {String} itemrowSelector - describes the selector for a row in the wcr-table-section. (Most of the time this is: "> table.ordtable > tbody > tr", but not if it is "grid" style or another representation)
	 * @param {String} checkboxSelector - describes the selector to find a checkbox in the "row"
	 * @param {Boolean} clickToSelectActive - whether the Table implements "click to select" 
	 */
	var Table = function (context, itemrowSelector, checkboxSelector, clickToSelectActive) {

		this._context = $(context);
		this._tablesection = this._context.find(".wcr-table-section");
		
		if(this._tablesection.length>0 && !this._tablesection.hasClass("noscrollbarfloating")) {
			
			// In 16.1 this will be ONLY applied to Search!
			// Way to risky to also apply to Attribute Category & DocTableX
			// (Other tables in WebCenter should be save though)
			// Remove this check as soon as we start on 16.2!
			if(this._context.hasClass("wcr161_apply_floatingscroll")) {
				
				// Add a floating scrollbar on the table section
				this._tablesection.floatingScroll();
				
				// Also listen to contentchange events on the table (custom events that need to be generated by the javascript code changing the content)
				this._tablesection.on("contentchange", _.bind(function() {
					// Update the floating scrollbar
					this._tablesection.floatingScroll("update");
				}, this));
				
			}
		}

		this._itemrowSelector = itemrowSelector;
		this._checkboxSelector = checkboxSelector;
		this._clickToSelectActive = clickToSelectActive;
		this._initializeRowSelection();
		this._initializeDropdownInteraction();

		if (this._context.find(".wcr-table-actions-row").not(".no-actions").not(".no-sticky-actions").length > 0 &&
			this._context.find(".wcr-table-actions-row .action").length > 0) {
			require(["stickykit"], function () {
				var element = $(context).find(".wcr-table-actions-row");
				element.stick_in_parent({
					parent: $(context)
				});

				//
				// Fix horizontal scrolling issue
				// Source: https://github.com/leafo/sticky-kit/issues/61
				//

				// side = 'left' or 'right'
				var fixHScroll = function ($el, side) {
					var $parent = $el.parent()[0];

					var fixscroll = function () {
						if ($el.css('position') === 'fixed') {
							$el.css(side, $parent.getBoundingClientRect()[side] + 10); // Note: parent has 10 px padding
						} else {
							$el.css(side, 'initial');
						}
					};

					$(window).on('scroll', fixscroll);
					element.on("sticky_kit:stick", fixscroll);
				};

				fixHScroll(element, 'left');
			});

		}




		// Add the object on the Dom Element
		// Note: use init.whenInitialized before trying to access this variable.
		this._context.get(0).Table = this;
	};

	Table.prototype._initializeRowSelection = function () {
		var self = this;

		if (this._checkboxSelector && this._itemrowSelector) {

			// listen for clicks on table rows to toggle the checkbox
			if (this._clickToSelectActive) {
				this._tablesection.on("click", this._itemrowSelector, function (e) {
					var excludeTargets = "a, .btn, .action, .dropdown, input, .radio";
					if (!$(e.target).is(excludeTargets) && !$(e.target).parentsUntil(this).is(excludeTargets)) {
						var checkbox = $(self._checkboxSelector, this);
						if (checkbox.length > 0 && !checkbox.is("[data-toggle-all]") && !checkbox.is(":disabled")) {
							if (!$(e.target).is(checkbox)) {
								self._doSelection(e, checkbox);
							}
						}
					}
				});
			}

			// when someone changes the selection (and calls the change event on the checkbox)
			// => update the "highlight" of the row + update the "selectAll" checkbox
			this._tablesection.on("change", this._checkboxSelector, function (e) {
				if (!$(this).is("[data-toggle-all]")) {
					// Set my row as "active"
					var myrow = $(self._itemrowSelector, self._tablesection).has(this);
					myrow.toggleClass("active", $(this).prop("checked"));

					// Update the "toggle-all" checkbox, but not the ones of child tables					
					var selectAllCheckboxes = self._tablesection.find("input[type='checkbox'][data-toggle-all]");
					var childTableSelectAllCheckboxes = self._tablesection.find(".wcr-table-section").find("input[type='checkbox'][data-toggle-all]");
					selectAllCheckboxes.not(childTableSelectAllCheckboxes).prop("checked", self.isEverythingSelected());
					
				}
			});

			// listen for the check all toggle
			this._tablesection.on("click", "input[type='checkbox'][data-toggle-all]", function (e) {
				var isChecked = $(this).prop("checked");

				// toggle all checkboxes
				self.selectAll(isChecked);
				e.stopPropagation();
			});
		}
	};

	/**
	 * User selected a row.
	 * Apply the selection (check all checkboxes which are applicable)
	 * 
	 * @param {Event} event - the event
	 * @param {jQuery} checkbox - the checkbox for the selected row
	 */
	Table.prototype._doSelection = function (event, checkbox) {
		checkbox.prop("checked", !checkbox.prop("checked")).trigger("change");

		if (checkbox.is("[type=radio]") && checkbox.prop("checked")) {
			this._fixAllActiveClasses();
		}
	};

	/**
	 * Whether everything in this table is selected
	 * @returns {boolean}
	 */
	Table.prototype.isEverythingSelected = function () {
		var countCheckedCheckbox = $(this._checkboxSelector + ":visible:not([data-toggle-all]):checked", this._tablesection).length;
		var countTableBodyRow = $(this._checkboxSelector + ":visible:not([data-toggle-all])", this._tablesection).length;

		return countCheckedCheckbox === countTableBodyRow;
	};

	/**
	 * Fix the active class for all selectable items
	 * This is needed when working with radios
	 */
	Table.prototype._fixAllActiveClasses = function () {
		var self = this;
		$(this._itemrowSelector, this._tablesection).each(function (i, el) {
			$(el).toggleClass("active", $(el).find(self._checkboxSelector).is(":checked"));
		});
	};

	/**
	 * Select all items in this table
	 * 
	 * @param {boolean} selectAll
	 */
	Table.prototype.selectAll = function (selectAll) {
		this._tablesection.find(this._checkboxSelector, this._tablesection).filter(":visible").prop("checked", selectAll).trigger("change");
	};

	/**
	 * Toggle the selection in the table...
	 */
	Table.prototype.toggleSelectAll = function () {
		this.selectAll(!this.isEverythingSelected());
	};

	/**
	 * Get the context as given to the table during construction
	 */
	Table.prototype.getContext = function () {
		return this._context;
	};

	/**
	 * Will move bootstrap dropdowns outside of the table when opening them.
	 * 
	 * Why? Because the table has an overflow defined for having scrollbars on mobile.
	 * 
	 * Problems?
	 *  - css hover => no longer hovering over the row (menu is outside of the table)
	 *  - click events => no longer bubble to the action menu since the dropdown that opened is no longer part of the action menu
	 */
	Table.prototype._initializeDropdownInteraction = function () {
		// too bad the Bootstrap dropdown menu doesn't really like overflow on the table...
		// this will solve the issue

		// and when you show it, move it to the body   
		DropdownOverflowUtil.repositionDropdowns($(".wcr-table-section", this._context));
	};

	/**
	 * Allow using data-init to make a Table component.
	 */
	Table.wcrinit = function (context) {
		if (typeof context !== $) {
			context = $(context);
		}

		// The jQuery selector to get a single row (or div)
		var itemrowSelector = context.attr("data-itemrow-selector");

		// The jQuery selector to get the checkbox inside that div
		var checkboxSelector = context.attr("data-checkbox-selector");

		//
		// OR you can just use the defaults in most cases...
		//
		if (!itemrowSelector) {
			itemrowSelector = " > table.ordtable > tbody > tr";
			if ($("> .wcr-table-section > table.ordtable", context).hasClass("gridview")) {
				itemrowSelector = " > table.ordtable > tbody > tr > td";
			}
		}
		if (!checkboxSelector) {
			checkboxSelector = "input[type='checkbox'][name^='select']";
		}

		// Whether click to select is active
		var clickToSelectActive = context.attr("data-click-to-select") === "0" ? false : true;

		new Table(context, itemrowSelector, checkboxSelector, clickToSelectActive);

	};


	return Table;
});