/**
 * TreeItem
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "utils/webi18n",
        "./treeitem-view"], function ($, WebI18n, TreeItemView) {
	
	/**
	 * Create a tree item
	 */
	var TreeItem = function () {
		this._treeItemView = new TreeItemView();
		this._children = [];
		this._isLeaf = false;
		this._isRoot = false;
	};
	

	/**
	 * Set the title element of this tree item.
	 * 
	 * @param titleElement
	 */
	TreeItem.prototype.setTitleElement = function (titleElement) {
		// If not wrapped by jQuery, do it.
        if (!(titleElement instanceof jQuery)) {
        	titleElement = $(titleElement);
        }
		this._treeItemView.setTitleElement(titleElement);
	};
	
	/**
	 * Set the content element of this tree item.
	 * @param contentElement
	 */
	TreeItem.prototype.setContentElement = function (contentElement) {
		// If not wrapped by jQuery, do it.
        if (!(contentElement instanceof jQuery)) {
        	contentElement = $(contentElement);
        }
		this._treeItemView.setContentElement(contentElement);
	};
	
	
	/**
	 * Add a TreeItem child from this TreeItem.
	 * 
	 * @param child
	 */
	TreeItem.prototype.addChild = function (child) {
		this._children.push(child);
		this._treeItemView.addChildView(child.getJQElement());
	};
	
	/**
	 * Remove a TreeItem child from this TreeItem.
	 * 
	 * @param child
	 */
	TreeItem.prototype.removeChild = function (child) {
		
		var index = this._children.indexOf(child);
		if (index >= 0) {
			this._children.splice(index, 1);
			this._treeItemView.removeChildView(child.getJQElement());
		} else {
			window.console.warn("Tried removing an unexisting child from treeitem.");
		}
	};
	
	/**
	 * Deep expand this tree item (also expand children from children from ...)
	 */
	TreeItem.prototype.deepExpand = function () {
		// Hide children.
		this.expand();
		// Iterate over all children.
		for (var i=0; i < this._children.length; i++) {
			// If this child can also collapse it's own children, do it.
	    	if (this._children[i].deepExpand) {
	    		this._children[i].deepExpand();
	    	}
		}
	};
	
	/**
	 * Deep collapse this tree item (also collapse children from children from ...)
	 */
	TreeItem.prototype.deepCollapse = function () {
		// Hide children.
		this.collapse();
		// Iterate over all children.
		for (var i=0; i < this._children.length; i++) {
			// If this child can also collapse it's own children, do it.
	    	if (this._children[i].deepCollapse) {
	    		this._children[i].deepCollapse();
	    	}
		}
	};
		
	/**
	 * Expand this tree item (show children).
	 */
	TreeItem.prototype.expand = function () {
		this._treeItemView.expand();
	};
	
	/**
	 * Collapse this tree item (hide children).
	 */
	TreeItem.prototype.collapse = function () {
		this._treeItemView.collapse();
	};
	
	/**
	 * Set the background color of this tree (best only call this method from the root).
	 * 
	 * @param color
	 */
	TreeItem.prototype.setBackgroundColor = function(color) {
		
		// If root ... set the wrapper background color.
		if (this._isRoot) {
			this._treeItemView.setWrapperBackgroundColor(color);
		}
		
		// Set own BackgrounColor
		this._treeItemView.setBackgroundColor(color);
		// Iterate over all children.
		for (var i=0; i < this._children.length; i++) {
			// If this child can also setBackgroundColor on it's own children, do it.
	    	if (this._children[i].setBackgroundColor) {
	    		this._children[i].setBackgroundColor(color);
	    	}
		}
	};
	
	/**
	 * Set the color of the actual item boxes of this tree item and it's children.
	 * 
	 * @param color
	 */
	TreeItem.prototype.setItemBoxColor = function(color) {
		// Set own ItemBoxColor
		this._treeItemView.setItemBoxColor(color);
		// Iterate over all children.
		for (var i=0; i < this._children.length; i++) {
			// If this child can also setItemBoxColor on it's own children, do it.
	    	if (this._children[i].setItemBoxColor) {
	    		this._children[i].setItemBoxColor(color);
	    	}
		}
	};
	
	/**
	 * Set this item as the root of the total tree.
	 */
	TreeItem.prototype.setRoot = function() {
		this._isRoot = true;
		this._treeItemView.setRoot();
	};
	
	/**
	 * Set this item as a leaf of the total tree.
	 */
	TreeItem.prototype.setLeaf = function() {
		this._isLeaf = true;
		this._treeItemView.setLeaf();
	};
	
	
	/**
	 * Return the view element of this TreeItem.
	 * 
	 * @returns {TreeItemView}
	 */
	TreeItem.prototype.getView = function() {
		return this._treeItemView;
	};
	
	
	/**
	 * Destroy this TreeItem
	 */
	TreeItem.prototype.destroy = function() {
		this._treeItemView.clearChildViews();
		delete this._children;
	};
	
	/**
	 * Return the jQuery element of this view.
	 * 
	 * @returns
	 */
	TreeItem.prototype.getJQElement = function() {
		return this._treeItemView.getJQElement();
	};
	
	return TreeItem;
	
});