define(["jquery",
		"underscore",
		"component/modal/Modal",
		"./CreateUserAndInviteInProject",
		"./RoleFieldComponent",
		"./UserCreationAndInviteMemberErrorUtil",
		"utils/webi18n!UserCreationStrings",
		"component/sdrop/SDrop",
		"component/overlaymessage/OverlayMessage",
		"wcr/sdrop_common",
		"wcr/webcenter_xml",
		"wcr/sso"
	],
	function ($,
		_,
		ModalPopup,
		CreateUserAndInviteInProject,
		RoleFieldComponent,
		ErrorUtil,
		i18n,
		SDrop,
		OverlayMessage,
		wcr_sdrop_common,
		WCXML,
		wcr_sso) {

		"use strict";

		/**
		 * User Creation Pop up With Role invitation
		 * This popup is destroyed when closed
		 * 
		 * @class
		 * @param {String} projectId
		 */
		var CreateNewUserPopupView = function (projectId, folderId) {

			this._projectId = projectId;
			this._folderId = folderId;

			this._hasLicenseToCreateUser = false;

			// callbacks for create and cancel buttons
			this._onCreateCallbacks = [];
			this._onCancelCallbacks = [];

			this._errorUtil = new ErrorUtil();

			// Handlers to execute respective callbacks when create or cancel button is clicked
			this._createClickHandler = _.bind(function (newUserObj) {
				_.each(this._onCreateCallbacks, _.bind(function (callback) {
					callback(newUserObj);
				}, this));
			}, this);

			this._cancelClickHandler = _.bind(function () {
				_.each(this._onCancelCallbacks, _.bind(function (callback) {
					callback();
				}, this));
			}, this);

			// Construct the popup
			this._constructPopup();
		};



		/**
		 * Create the Modal popup object and construct view
		 */
		CreateNewUserPopupView.prototype._constructPopup = function () {
			// Get the popup html content
			this._popupContent = $("<div>");
			this._popupContent.append(this._getPopupContent());

			var options = {};
			options.customClass = "create_newuser_modalpopup";
			options.onClose = _.bind(function () {
				// destroy the popup on close
				this._popup.destroy();
			}, this);

			// create the popup using model popup
			this._popup = new ModalPopup(this._popupContent, options);
			this._popup.setTitle(i18n.t("CREATE_AND_INVITE_NEW_USER"));
			this._popup.setFooter(this._getFooterContent());
		};

		/**
		 * Method to get the html structure of the popup
		 * 
		 * @returns
		 */
		CreateNewUserPopupView.prototype._getPopupContent = function () {
			// popup html content
			var popupContent = $('<div>', {
				"name": "create_newuser_popup",
				"class": "wcr-form form-labels-on-top add-padding"
			});

			/*
			 * html container to display error message
			 */
			this._errorMessageDiv = $("<div>", {
				"class": "error-message"
			});
			this._ssoErrorMessageDiv = $("<div>", {
				"class": "sso-error-message"
			});
			this._warningMessageDiv = $("<div>", {
				"class": "warning-message"
			});

			// E-mail input field
			this._emailInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"id": "userprofile-email"
			});
			this._emailInput.keyup(this._clearErrorMessage.bind(this));
			this._emailInput.change(function (event) {
				// parse and set username, firstname and lastname automatically
				var result = this._parseEmailAddress(event.target.value);
				if (!this._userNameInput.val() && result.username) {
					this._userNameInput.val(result.username);
				}
				if (!this._lastNameInput.val() && result.lastname) {
					this._lastNameInput.val(result.lastname);
				}
				if (!this._firstNameInput.val() && result.firstname) {
					this._firstNameInput.val(result.firstname);
				}
			}.bind(this));

			var emailField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>", {
					"class": "form-required",
					"for": "userprofile-email"
				}).text(i18n.t("EMAIL_ADDRESS")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._emailInput)
			);

			// Username input field
			this._userNameInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"id": "username"
			});
			this._userNameInput.keyup(function (event) {
				$(event.target).removeClass("validationHighlight");
				this._errorMessageDiv.empty();
			}.bind(this));

			var userNameField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>", {
					"class": "form-required",
					"for": "username"
				}).text(i18n.t("USER_NAME")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._userNameInput)
			);


			// Last name input field
			this._lastNameInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"id": "userprofile-lastname"
			});
			this._lastNameInput.keyup(function (event) {
				$(event.target).removeClass("validationHighlight");
				this._errorMessageDiv.empty();
			}.bind(this));

			var lastNameField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>", {
					"class": "form-required",
					"for": "userprofile-lastname"
				}).text(i18n.t("LAST_NAME")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._lastNameInput)
			);

			// First name input field
			this._firstNameInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"id": "userprofile-firstname"
			});

			var firstNameField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>", {
					"for": "userprofile-firstname"
				}).text(i18n.t("FIRST_NAME")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._firstNameInput)
			);

			// Company list dropdown
			this._companySelect = $("<select>", {
				"class": "form-control"
			});

			var companyField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>").text(i18n.t("COMPANY")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._companySelect)
			);

			// Location list dropdown
			this._locationSelect = $("<select>", {
				"class": "form-control"
			});

			var locationField = $("<div>", {
				"class": "form-group"
			}).append(
				$("<label>").text(i18n.t("LOCATION")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._locationSelect)
			);

			// get company and location details using ajax and populate dropdowns
			this._loadCompanyAndLocFields();

			this._errorSection = $("<div>", {
				"class": "form-section"
			});
			this._errorSection.append(this._errorMessageDiv, this._ssoErrorMessageDiv, this._warningMessageDiv);

			this._userDetailsSection = $("<div>", {
				"class": "form-section"
			});
			this._userDetailsSection.append(emailField, userNameField, firstNameField, lastNameField, companyField, locationField);

			// Role list dropdown field
			this._roleSection = $("<div>", {
				"class": "form-section"
			}).hide();

			if (this._getProjectId()) {
				var options = {};

				options.errorDiv = this._errorMessageDiv;
				options.warningDiv = this._warningMessageDiv;

				// Role list dropdown for current project
				this._roleField = new RoleFieldComponent(this._getProjectId(), this._folderId, options);
				var roleField = this._roleField.getJQElement();
				this._roleSection.append(roleField);
			}

			// div to append sso authentication sections
			this._ssoAuthSections = $("<div>", {
				"class": "sso_auth_sections"
			});

			// create sso instances for a new user
			this._createSSOAuthSections();

			// append all sections to the popup content
			popupContent.append(this._errorSection, this._userDetailsSection, this._ssoAuthSections, this._roleSection);

			return popupContent;
		};

		CreateNewUserPopupView.prototype._loadCompanyAndLocFields = function () {
			// remove the company options in dropdown
			this._companySelect.empty();

			this._companyDataProvider = new CompanyWithLocationsDataProvider();
			var sdropOptions = {
				'initialValue': "",
				'enableEmptySelection': true,
				'emptySelectionValue': "",
				'onChangedByUser': _.bind(this._loadLocationSelect, this),
				'manageStyleAndWidth': false
			};
			this._companySDrop = new SDrop(this._companySelect, "", this._companyDataProvider, sdropOptions);
			this._loadLocationSelect();

		};

		CreateNewUserPopupView.prototype._createSSOAuthSections = function () {
			// clear sso instances
			this._ssoAuthSections.empty();

			/*
			 * Get SSO Authentication XHTML using Ajax call
			 */
			var ssoXHTMLPromise = $.ajax({
				url: "GetSSOAuthUserConfigs.jsp",
				method: "POST",
				dataType: "text"
			});
			$.when(ssoXHTMLPromise).then(function (data, textStatus, jqXHR) {
				// the html snippets will be unescaped/decoded
				var xml = $(data.trim()).get(0);

				if (($(xml).prop("tagName").toLowerCase() === "sso") && ($("> userconfigs", xml).children().length > 0)) {
					var ssouuid = $("> uuid", xml).text();
					var ssotimestamp = $("> timestamp", xml).text();

					var ssoConfigHidden = $("<div>", {
						"id": "sso-userconfig-hidden"
					}).hide();

					ssoConfigHidden.append(
						$("<input>", {
							"type": "hidden",
							"name": "sso-uuid",
							"value": ssouuid
						}),
						$("<input>", {
							"type": "hidden",
							"name": "sso-timestamp",
							"value": ssotimestamp
						}),
						$("<input>", {
							"type": "hidden",
							"name": "isActiveUser",
							"value": "on"
						}),
						$("<input>", {
							"type": "hidden",
							"name": "deactivatedAtTimeStamp",
							"value": ""
						})
					);

					// append the hidden fields
					this._ssoAuthSections.append(ssoConfigHidden);

					// initialize the sso properties
					sso.timestamp = ssotimestamp;
					sso.uuid = ssouuid;
					sso.setUserInfo({
						eMail: "",
						firstName: "",
						lastName: "",
						userName: ""
					});

					// render the sso instance xhtml
					_.each($("> userconfigs", xml).children(), function (config) {
						var xhtml = $("<div>").html($("> xhtml", config).text());

						var ssoid = $("> id", config).text();
						var ssoname = $("> name", config).text();
						var show = $("> show", config).text() === "1";
						var allow = $("> allow", config).length > 0 ? $("> allow", config).text() === "1" : null;
						var defaultallow = $("> defaultallow", config).text() === "1";
						var hashtml = $("> hashtml", config).text() === "1";
						xhtml.attr({
							"id": ssoid,
							"name": ssoname
						});
						if (!show) {
							$(xhtml).hide();
						}
						if (ssoname === "Basic password authentication") {
							// add keyup events for password fields
							$("input[type='password']", xhtml).keyup(function (event) {
								if ($(event.target).hasClass("validationHighlight")) {
									var ssoid = $(event.target).attr("id").substring(4, 20);
									$("#" + ssoid + " input[type='password']").removeClass("validationHighlight");
									this._ssoErrorMessageDiv.children().empty();
								}
							}.bind(this));

							// enable the "Require the User to change the password at first login." checkbox
							$("input[type='checkbox']", xhtml).prop("checked", true);
						}

						// add div to render error messages for sso
						this._ssoErrorMessageDiv.append($("<div>", {
							"id": "sso-config-user-error-" + ssoid,
							"name": "sso-config-user-error-" + ssoid
						}));

						var ssoInstance = $("<div>", {
							"class": "sso form-section"
						});
						if (allow !== null) {
							ssoInstance.append(
								$("<input>", {
									"type": "hidden",
									"name": "sso-allow-" + ssoid,
									"id": "sso-allow-" + ssoid,
									"value": allow
								}));
						}
						if(show) {
							ssoInstance.append($("<div>", {"class": "section-title"}).append(
								$("<h4>", {text: ssoname}).prepend(
									$("<i>", {"class": "icon icon-normal icon-WCR_Cont-Circle12 icon-success"})
								)
							));
						}
						ssoInstance.append(
							$("<input>", {
								"type": "hidden",
								"name": "sso-defaultallow-" + ssoid,
								"id": "sso-defaultallow-" + ssoid,
								"value": defaultallow
							}),
							$("<input>", {
								"type": "hidden",
								"name": "sso-hashtml-" + ssoid,
								"id": "sso-hashtml-" + ssoid,
								"value": hashtml
							}),
							xhtml
						);

						// append the sso instance
						this._ssoAuthSections.append(ssoInstance);
					}.bind(this));

				} else if ($(xml).prop("tagName").toLowerCase() === "error") {
					var errorMsg = $("> message", xml).text();
					this._errorUtil.showErrorMessage(errorMsg, true, this._errorMessageDiv);
				}
			}.bind(this), function (jqXHR, textStatus, errorThrown) {
				this._errorUtil.showErrorMessage(errorMsg, false, this._errorMessageDiv);
			}.bind(this));
		};

		/**
		 * Get the projectID
		 */
		CreateNewUserPopupView.prototype._getProjectId = function () {
			return this._projectId;
		};

		/**
		 * Method to populate the location dropdown for selected company
		 */
		CreateNewUserPopupView.prototype._loadLocationSelect = function () {
			// Get the companyid
			var companyId = this._companySelect.val();

			// clear location list and disable by default
			this._locationSelect.empty().prop("disabled", true);

			if (companyId) {
				var companyData = this._getCompanyInfo(companyId);
				if (companyData) {
					var companyLocationsList = companyData.locations;
					if (companyLocationsList.length > 0) {
						// we have found the locations

						this._locationSelect.prop("disabled", false);
						_.each(companyLocationsList, function (location, index) {
							var option = $("<option>", {
								value: location.id,
								text: location.name
							});
							this._locationSelect.append(option);
						}, this);
					}
				}
			}
		};

		/**
		 * Create create and cancel buttons of the popup
		 * 
		 * @returns
		 */
		CreateNewUserPopupView.prototype._getFooterContent = function () {
			var footerButtons = $('<div>').addClass("btn-group").attr({
				'role': 'group',
				'name': 'FooterButtons'
			});

			// Create and Invite button
			this._createButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("CREATE_AND_INVITE")
			}).addClass("btn btn-primary");
			this._createButton.click(_.bind(function (event) {
				if (this._hasLicenseToCreateUser && this._validateFields()) {
					var userDetails = this._getUserDetails();
					var overlay = new OverlayMessage(this._popup.getJQElement());
					overlay.showLoading();


					// method to be called when user create and invite is successful
					var onSuccess = function (newUserObj) {
						this._createClickHandler(newUserObj);

						this._popup.close();
						overlay.hide();
					}.bind(this);

					// method to be called when user create or invite fails
					var onError = function (error, istag) {
						if (!istag) {
							if ($(error).prop("tagName").toLowerCase() === "error") {
								var errorMsg = $("> message", error).text();
								this._errorUtil.showErrorMessage(errorMsg, true, this._errorMessageDiv);
							}
						} else {
							this._errorUtil.showErrorMessage(error, false, this._errorMessageDiv);
						}
						overlay.hide();
					}.bind(this);

					// submit details to create user and invite to project
					var action = "create";
					userDetails.applydefaultprefs = "1";
					CreateUserAndInviteInProject.createOrModifyAndInvite(action, this._folderId, userDetails, onSuccess, onError);
				}
			}, this));

			// Cancel button
			this._cancelButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("CANCEL")
			}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function (event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));

			// Clear button
			this._clearButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("CLEAR")
			}).addClass("btn btn-default add-spacing-right");
			this._clearButton.click(_.bind(this._resetPopup, this));

			// Add the buttons to the footer
			footerButtons.append(this._clearButton, this._cancelButton, this._createButton);

			return footerButtons;
		};

		/**
		 * Method to validate the fields
		 * @returns {Boolean}
		 * 				- true if all required fields are filled and valid, false otherwise
		 */
		CreateNewUserPopupView.prototype._validateFields = function () {
			var isvalid = true;

			var errorTag = "REQUIRED_FIELDS_TAG";

			// validate mandatory fields
			var emailid = this._emailInput.val().trim();
			if (!emailid) {
				isvalid = false;
				this._emailInput.addClass("validationHighlight");
				errorTag = "BADEMAIL";

			} else if (emailid && !this._validateEmail(emailid)) {
				isvalid = false;
				this._emailInput.addClass("validationHighlight");
				errorTag = "EMAIL_BAD_EMAIL";
			} else if (!this._userNameInput.val().trim()) {
				isvalid = false;
				this._userNameInput.addClass("validationHighlight");
				errorTag = "INVALIDUSERNAMEMSG";
			} else if (!this._lastNameInput.val().trim()) {
				isvalid = false;
				this._lastNameInput.addClass("validationHighlight");
				errorTag = "BADLASTNAME";
			} else if (!this._roleField.validate()) {
				isvalid = false;
				this._roleField.highLightField();
				errorTag = "INVALIDROLE";
			} else if (!ssoprivate.validateConfigurations()) {
				isvalid = false;
				$("input[type='password']", this._ssoAuthSections).addClass("validationHighlight");
				errorTag = "";
			}

			// If validation fails, show an error message
			if (!isvalid) {
				this._errorUtil.showErrorMessage(errorTag, false, this._errorMessageDiv);
			}

			return isvalid;
		};

		/**
		 * Method to check if email address is of valid format <address@subdmain.domain>
		 * 
		 * @param value
		 * @returns {Boolean}
		 */
		CreateNewUserPopupView.prototype._validateEmail = function (value) {
			//does the email contain personal info ?
			if (value.indexOf('<') != -1) // process only the email part
			{ // find start index of email part
				var startIndex = value.indexOf('<') + 1;
				if (value.indexOf('>') != -1) { //find end index of email part
					var endIndex = value.indexOf('>');
					// strip the value only to the email part
					value = (value.substring(startIndex, endIndex)).trim();
				} else {
					return false;
				}
			}

			var pattern = "^[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?$";
			var filter = new RegExp(pattern);

			if (filter.test(value.toLowerCase())) {
				return true;
			}

			return false;
		};

		/**
		 * Method to parse email value and return an object to populate other fields automatically
		 */
		CreateNewUserPopupView.prototype._parseEmailAddress = function (email) {
			var result = {};

			// If email address is of format "<address@subdmain.domain>"
			if (email.indexOf('<') != -1 && email.indexOf('>') != -1) {
				// find start index of email part
				var startIndex = email.indexOf('<') + 1;
				//find end index of email part
				var endIndex = email.indexOf('>');
				// strip the value only to the email part
				email = (email.substring(startIndex, endIndex)).trim();
			}

			if (email.indexOf("@") > 0) {
				var identifier = email.substr(0, email.indexOf("@"));

				var parts = identifier.split(".");

				if (parts.length === 1) {
					result.lastname = identifier;
				} else {
					result.firstname = parts[0];
					result.lastname = _.rest(parts).join(" ");
				}

				result.username = identifier;
			}

			return result;
		};

		/**
		 * Method to clear error message
		 */
		CreateNewUserPopupView.prototype._clearErrorMessage = function (event) {
			if ($(event.target).hasClass("validationHighlight")) {
				$(event.target).removeClass("validationHighlight");
				this._errorMessageDiv.empty();
			}
		};
		/**
		 * Get the company info for a specific company
		 * @private
		 * @param   {String} companyId 
		 * @returns {Object} {id, name, locations: [{id, name}]}
		 */
		CreateNewUserPopupView.prototype._getCompanyInfo = function (companyId) {
			return this._companyDataProvider.getById(companyId);
		};

		/**
		 * Get the user details from fields
		 * @returns {Object}
		 */
		CreateNewUserPopupView.prototype._getUserDetails = function () {
			var userDetails = {};
			userDetails.newuseremail = this._emailInput.val();
			userDetails.newusername = this._userNameInput.val();
			userDetails.newuserlastname = this._lastNameInput.val();

			if (this._firstNameInput.val().trim()) {
				userDetails.newuserfirstname = this._firstNameInput.val().trim();
			}

			// company and location details
			var companyId = this._companySelect.val();
			if (companyId) {
				var company = this._getCompanyInfo(companyId);
				if (company) {
					userDetails.newusercompanycode = company.name;

					// location details
					var locationId = this._locationSelect.val();
					if (locationId) {
						// get location name from the location list
						var location = _.findWhere(company.locations, {
							id: locationId
						});
						userDetails.newusercompanylocation = location.name;
					}
				}
			}

			if (this._roleField.validate()) {
				// role id
				userDetails.roleid = this._roleField.getValue();
			}

			// project id
			userDetails.projectid = this._getProjectId();

			// sso settings
			var ssoSettings = _.reduce($("<form>").append(this._ssoAuthSections.clone()).serializeArray(), function (memo, value) {
				memo[value.name] = value.value;
				return memo;
			}, {});

			return _.extend(userDetails, ssoSettings);
		};

		/**
		 * Method open the popup
		 * @param {String} [input] - if provided, input is used to prefill e-mail adres or username field
		 */
		CreateNewUserPopupView.prototype.show = function (input) {
			this._resetPopup();

			this._checkLicense();

			// prefill
			if (input) {
				if (input.indexOf("@") > 0) {
					// e-mail prefill
					this._emailInput.val(input).trigger("change");

				} else {
					this._userNameInput.val(input).trigger("change");
				}
			}

			this._popup.open();
		};

		/**
		 * Reset the popup
		 */
		CreateNewUserPopupView.prototype._resetPopup = function () {
			this._errorMessageDiv.empty();
			this._ssoErrorMessageDiv.children().empty();

			this._emailInput.val("").removeClass("validationHighlight");
			this._userNameInput.val("").removeClass("validationHighlight");
			this._lastNameInput.val("").removeClass("validationHighlight");
			this._firstNameInput.val("");

			this._companySelect.val("");
			this._companySelect.trigger("change");
			this._loadLocationSelect();

			// Reset the role
			this._roleField.reset();

			$("input[type='password']", this._ssoAuthSections).val("");
			$("input[type='checkbox']", this._ssoAuthSections).prop("checked", true);

			// initialize the sso userInfo object
			sso.setUserInfo({
				eMail: "",
				firstName: "",
				lastName: "",
				userName: ""
			});

			this._emailInput.focus();
		};

		/**
		 * Check if license is present to create a regular user
		 */
		CreateNewUserPopupView.prototype._checkLicense = function () {
			// disable the create button initially
			this._createButton.prop("disabled", true);

			// check if license is available to create a regular user
			var licenseCheckPromise = $.ajax({
				url: "CreateNewUserLicenseCheck.jsp",
				method: "POST",
				dataType: "xml"
			});
			$.when(licenseCheckPromise).then(function (data, textStatus, jqXHR) {
				var xml = $(data).children().get(0);

				// enable the create user button if license is available
				if (xml && xml.tagName === "can_create_user") {
					var canCreateUser = ($(xml).text() === "true");
					if (canCreateUser) {
						this._hasLicenseToCreateUser = true;
						this._createButton.prop("disabled", false);
					} else {
						this._errorUtil.showWarningMessage("LIC_VIOLATION_REGULAR_USERCREATION", "alert-danger", this._warningMessageDiv);
					}
				} else if ($(xml).prop("tagName").toLowerCase() === "error") {
					var errorMsg = $("> message", xml).text();
					this._errorUtil.showErrorMessage(errorMsg, true, this._errorMessageDiv);
				}

			}.bind(this), function (jqXHR, textStatus, errorThrown) {
				this._errorUtil.showErrorMessage("UNKNOWNEXCEPTION_TAG", false, this._errorDiv);
			}.bind(this));
		};

		/**
		 * To add callback for create button
		 * @param callback
		 */
		CreateNewUserPopupView.prototype.onSubmit = function (callback) {
			this._onCreateCallbacks.push(callback);
		};

		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		CreateNewUserPopupView.prototype.onCancel = function (callback) {
			this._onCancelCallbacks.push(callback);
		};

		// create a singleton instance of the popup view
		return CreateNewUserPopupView;
	});
