define(["jquery",
		"underscore",
		"component/modal/Modal",
		"component/overlaymessage/OverlayMessage",
		"./CreateUserAndInviteInProject",
		"./RoleFieldComponent",
		"./UserCreationAndInviteMemberErrorUtil",
		"utils/webi18n!UserCreationStrings"
	],
	function ($,
		_,
		ModalPopup,
		OverlayMessage,
		CreateUserAndInviteInProject,
		RoleFieldComponent,
		ErrorUtil,
		i18n) {

		"use strict";

		/**
		 * Invite Member Pop up With Role invitation
		 * @class
		 * @param {object}  userInfo     - Object indicating User details
		 * @param {String}  projectId    - The Roles are loaded from this projectId
		 * @param {String}  folderId     - 
		 * @param {Boolean} userAlreadyInProject - If true there will be no role selection available (used for deactivated users who were already in the project)
		 *                            
		 */
		var InviteMemberPopupView = function (userInfo, projectId, folderId, userAlreadyInProject) {

			this._userInfo = userInfo;
			this._projectId = projectId;
			this._folderId = folderId;
			this._userAlreadyInProject = userAlreadyInProject;

			this._errorUtil = new ErrorUtil();
			
			// callbacks for create and cancel buttons
			this._onCreateCallbacks = [];
			this._onCancelCallbacks = [];

			// Handlers to execute respective callbacks when create or cancel button is clicked
			this._inviteClickHandler = _.bind(function (updatedUserObj) {
				_.each(this._onCreateCallbacks, _.bind(function (callback) {
					callback(updatedUserObj);
				}, this));
			}, this);

			this._cancelClickHandler = _.bind(function () {
				_.each(this._onCancelCallbacks, _.bind(function (callback) {
					callback();
				}, this));
			}, this);

		};

		
		/**
		 * Create the Modal popup object and construct view
		 */
		InviteMemberPopupView.prototype._constructPopup = function () {
			// Get the popup html content
			this._popupContent = $("<div>");
			this._popupContent.append(this._getPopupContent());

			// create the popup using model popup
			this._popup = new ModalPopup(this._popupContent, {
				"customClass": "invite_member_modalpopup"
			});
			this._popup.setTitle(this._userAlreadyInProject ? i18n.tFallback("ACTIVATE_USER", "Activate User") : i18n.t("INVITE_PROJECT_MEMBER"));
			this._popup.setFooter(this._getFooterContent());
			this._popup.onClose(this._cancelClickHandler);
		};

		
		
		/**
		 * Method to get the html structure of the popup
		 * 
		 * @returns
		 */
		InviteMemberPopupView.prototype._getPopupContent = function () {
			// popup html content
			var popupContent = $('<div>', {
				"name": "invite_member_popup",
				"class": "wcr-form form-labels-on-top add-padding"
			});

			/*
			 * html container to display error message
			 */
			this._errorMessageDiv = $("<div>", {
				"class": "error-message"
			});
			
			this._warningMessageDiv = $("<div>", {
				"class": "warning-message"
			});
		
			this._errorSection = $("<div>", {
				"class": "form-section"
			});
			this._errorSection.append(this._errorMessageDiv,this._warningMessageDiv);

			// append the section to the popup content
			popupContent.append(this._errorSection);
			
			//Only show role selector when needed
			if(!this._userAlreadyInProject){
				// Role list dropdown field
				this._roleSection = $("<div>", {
					"class": "form-section"
				}).hide();

				if (this._projectId) {
					var options = {};
					options.errorDiv = this._errorMessageDiv;
					options.warningDiv = this._warningMessageDiv;

					// Role list dropdown for current project
					this._roleField = new RoleFieldComponent(this._projectId, this._folderId, options);
					var roleField = this._roleField.getJQElement();
					this._roleSection.append(roleField);
				}

				// append the section to the popup content
				popupContent.append(this._roleSection);
			}
			
			// if user is not active show warning info
			if(!this._userInfo.isActiveUser()) {
				this._errorUtil.showWarningMessage("DEACTIVATED_USER_INFO","alert-warning", this._warningMessageDiv);
			}

			return popupContent;
		};
		

		/**
		 * Create create and cancel buttons of the popup
		 * 
		 * @returns
		 */
		InviteMemberPopupView.prototype._getFooterContent = function () {
			
			var footerButtons = $('<div>').addClass("btn-group").attr({
				'role': 'group',
				'name': 'FooterButtons'
			});

			var inviteButtonText = i18n.t("INVITE");
			
			if (this._userAlreadyInProject){
				// user is already in project, so we only need to activate the user
				inviteButtonText = i18n.t("ACTIVATE");
			} else if (!this._userInfo.isActiveUser()){
				// user is deactivated and not yet in project
				inviteButtonText = i18n.t("ACTIVATE_AND_INVITE");	
			}
			
			// Invite button
			this._inviteButton = $('<input>').attr({
				'type': 'button',
				'value': inviteButtonText
			}).addClass("btn btn-primary");
			this._inviteButton.click(_.bind(function (event) {
				if (this._validateFields()) {
					
					var overlay = new OverlayMessage(this._popup.getJQElement());
					overlay.showLoading();
					
					// method to be called when user invite is successful
					var onSuccess = function (updatedUserObj) {
						this._inviteClickHandler(updatedUserObj);						
						this._popup.close();
						overlay.hide();
					}.bind(this);

					// method to be called when user create or invite fails
					var onError = function (error, istag) {						
						if (!istag) {
							if ($(error).prop("tagName").toLowerCase() === "error") {
								var errorMsg = $("> message", error).text();
								this._errorUtil.showErrorMessage(errorMsg, true,this._errorMessageDiv);
							}
						} else {
							this._errorUtil.showErrorMessage(error, false,this._errorMessageDiv);
						}
						overlay.hide();
					}.bind(this);

					// Set user, project details in the request to invite member to the project
					var inviteRequestInfo = {};
					
					inviteRequestInfo.usermemberid = this._userInfo.getId();
					if(this._roleField && this._roleField.getValue() != "") {
						inviteRequestInfo.roleid = this._roleField.getValue();
					}
					
					inviteRequestInfo.projectid = this._projectId;
					
					if(this._userInfo.isActiveUser()) {
						// submit details to invite member to project
						CreateUserAndInviteInProject.inviteMemberToProject(inviteRequestInfo,this._userInfo, this._folderId, onSuccess, onError);
						
					} else{
						// user is not active so set the flag to activate the user
						inviteRequestInfo.activateuser = "1";
						inviteRequestInfo.userid = this._userInfo.getId();
						
						// modify user and invite member to project
						var action = "modify";
						CreateUserAndInviteInProject.createOrModifyAndInvite(action, this._folderId, inviteRequestInfo, onSuccess, onError, this._userAlreadyInProject);
					}
				}
			}, this));

			// Cancel button
			this._cancelButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("CANCEL")
			}).addClass("btn btn-default add-spacing-right");
			
			this._cancelButton.click(_.bind(function (event) {
				this._cancelClickHandler();
				this._popup.close();
			}, this));

			// Add the buttons to the footer
			footerButtons.append(this._cancelButton, this._inviteButton);

			return footerButtons;
		};

		/**
		 * Method to validate the fields
		 * @returns {Boolean}
		 * 				- true if required fields are filled and valid, false otherwise
		 */
		InviteMemberPopupView.prototype._validateFields = function () {
			
			var isvalid = true;
			var errorTag = "REQUIRED_FIELDS_TAG";

			if(this._roleField && !this._roleField.validate()) {
				isvalid = false;
				this._roleField.highLightField();
				errorTag = "INVALIDROLE";
			} 	
		 
			// If validation fails, show an error message
			if (!isvalid) {
				this._errorUtil.showErrorMessage(errorTag, false,this._errorMessageDiv);
			}

			return isvalid;
		};

		/**
		 * Method open the popup
		 * @param {String} [input] - if provided, input is used to prefill e-mail adres or username field
		 */
		InviteMemberPopupView.prototype.show = function (input) {
			// Create only one instance of popup.
			if (!this._popup) {
				// Construct the popup
				this._constructPopup();
			}

			this._resetPopup();
			this._popup.open();
		};

		/**
		 * Reset the popup
		 */
		InviteMemberPopupView.prototype._resetPopup = function () {
			this._errorMessageDiv.empty();
			if(this._roleField){
				this._roleField.reset();				
			}
		};

		/**
		 * To add callback for create button
		 * @param callback
		 */
		InviteMemberPopupView.prototype.onSubmit = function (callback) {
			this._onCreateCallbacks.push(callback);
		};

		/**
		 * To add callback for cancel button
		 * @param callback
		 */
		InviteMemberPopupView.prototype.onCancel = function (callback) {
			this._onCancelCallbacks.push(callback);
		};

		// create a singleton instance of the popup view
		return InviteMemberPopupView;
	});