define(["jquery",
		"underscore",
		"./UserCreationAndInviteMemberErrorUtil",
		"utils/webi18n!UserCreationStrings"
	],
	function ($,
		_,
		ErrorUtil,
		i18n
		) {

		"use strict";

		/**
		 * This class is Role field component with roles are loaded for given ProjectId
		 * 
		 * @class
		 * @param {String} projectId
		 * @param {String} options
		 */
		var RoleField = function (projectId, folderId, options) {

			this._projectId = projectId;
			this._folderId = folderId;
			this._options = options;
			
			this._errorDiv = options.errorDiv;
			this._warningDiv = options.warningDiv;
			
			this._errorUtil = new ErrorUtil();
			
			// are there roles available in the project?
			this._rolesAvailable = false;
			
			this._wrapper = $("<div>", {
				"class": "form-group"
			});
			
			this._constructView();
		};

		/**
		 * Construct Role field select option and load the roles which are available in the project 
		 */
		RoleField.prototype._constructView = function () {
			
			this._roleSelect = $("<select>", {
				"class": "form-control"
			});
			
			this._roleSelect.change(function (event) {
				$(event.target).removeClass("validationHighlight");
				this._errorDiv.empty(); 
			}.bind(this));

			this._wrapper.append(
				$("<label>", {
					"class": "form-required"
				}).text(i18n.t("ASSIGN_TO_ROLE")),
				$("<div>", {
					"class": "form-entry"
				}).append(this._roleSelect, $("<p>", {
					 "class": "help-block"
				}).text(i18n.t("INVITE_ROLE_TIP")))
			);

			if (this._projectId) {
				// get the project role(s) using ajax and populate the dropdown
				this._loadRoleField();
			}
		};
		
		
		/**
		 * Validate role field
		 * @returns {Boolean}
		 */
		RoleField.prototype.validate = function() {
			if(this._rolesAvailable) {
				return this.getValue() != "";
			}else{
				return true;
			}
		};
		
		
		/**
		 * Highlight the Role field
		 */		
		RoleField.prototype.highLightField = function() {
			this._roleSelect.addClass("validationHighlight");
		};
		
		
		/**
		 * Reset the Role field value
		 */
		RoleField.prototype.reset = function() {
			if (this._rolesAvailable) {
				this._roleSelect.val($("option:first", this._roleSelect).val());
			}
		};
		
		
		/**
		 * Get the value of selected role
		 * @returns {Boolean}
		 */
		RoleField.prototype.getValue = function() {
			var value = this._roleSelect.val() || "";
			return value;
		};
		
		
		/**
		 * Return the role field JQuery Element
		 * @returns
		 */
		RoleField.prototype.getJQElement = function() {
			return this._wrapper;
		};
		
		
		/**
		 * Load the Roles for given project
		 */
		RoleField.prototype._loadRoleField = function () {
			// remove the role options in dropdown
			this._roleSelect.empty();

			// Ajax call to get roles in current project
			var projectRolesXMLPromise = $.ajax({
				url: "GetRolesInProject.jsp",
				method: "POST",
				dataType: "xml",
				data: {
					"projectid": this._projectId,
					"folderid": this._folderId
				}
			});
			$.when(projectRolesXMLPromise).then(function (data, textStatus, jqXHR) {
				var xml = $(data).children().get(0);

				// Parse the xml and populate the role dropdown
				if (xml && xml.tagName === "role_list") {

					/*
					 * Object to store the role information
					 * [
					 * 		{ roleid: "", rolename: "" }, 
					 * 		... 
					 * ]
					 */
					this._roleList = [];

					// parse the role_list xml
					_.each($(xml).children(), function (role, index) {
						var roleId = $(role).attr("id");
						var roleName = $(role).text();

						this._roleList.push({
							"roleid": roleId,
							"rolename": roleName
						});
					}.bind(this));

					// populate the role dropdown
					if (this._roleList.length > 0) {
						this._rolesAvailable = true;
						this._wrapper.parent().show();
						
						var selectRoleOption = $("<option>", {
							"value": ""
						}).text("--- " + i18n.t("SELECT_ROLE") + " ---" );
						this._roleSelect.append(selectRoleOption);
						
						$.each(this._roleList, function (index, role) {
							var option = $("<option>", {
								"value": role.roleid
							}).text(role.rolename);
							this._roleSelect.append(option);
						}.bind(this));
					} else {
						this._rolesAvailable = false;
						this._errorUtil.showWarningMessage("NO_ROLE_CURRENT_PROJECT_INFO","alert-warning", this._warningDiv);
					}

				} else if ($(xml).prop("tagName").toLowerCase() === "error") {
					var errorMsg = $("> message", xml).text();
					this._errorUtil.showErrorMessage(errorMsg, true,this._errorDiv);
				}

			}.bind(this), function (jqXHR, textStatus, errorThrown) {
				this._errorUtil.showErrorMessage("UNKNOWNEXCEPTION_TAG", false,this._errorDiv);
			}.bind(this));
		};
	
		return RoleField;
	});