define(["./panel/XDropPanel",
		"./renderers/LabelRenderer",
		"jquery",
		"underscore",
		"component/modal/Modal",
		"utils/Observer",
		"utils/Observable",
		"utils/webi18n!XDrop"
	],
	function (XDropPanel,
		LabelRenderer,
		$,
		_,
		Modal,
		Observer,
		Observable,
		i18n) {
		"use strict";


		/**
		 * Xtendible Dropdown
		 * 
		 * @param {XDropDataProvider} dataProvider    
		 * @param {Object}            renderFactories - map of result type to factory
		 * @param {Obect}             options         extra options 
		 * {
		 *    absoluteXDropPanel: false (add results absolute in body if true)
		 * 
		 * }
		 */
		var XDrop = function (dataProvider, renderFactories, options) {
			Observer.mixin(this);
			Observable.mixin(this, "XDrop");


			this._options = _.extend({
				absoluteXDropPanel: false,
				placeholder: "",
				filterPlaceholder: i18n.t("FilterPlaceholder"),
				preferredMinPanelHeight: 300
			}, options);

			/**
			 * Used to create renderers for resultentries
			 *  we can provide some default renderers
			 */
			this._renderFactories = _.extend({
				label: LabelRenderer
			}, renderFactories);

			this._currentMaxHeight = null;

			this._isMobile = $("body").is(".mobile");
			this._updatePosition = this._updateRelativePosition;



			if (this._isMobile) {
				this._updatePosition = this._updateModalPosition;
			} else if (this._options.absoluteXDropPanel) {
				this._updatePosition = this._updateAbsolutePosition;
			}




			/**
			 * Main element of the XDrop
			 */
			this._element = $("<div>", {
				"class": "XDrop",
				"tabindex": 0
			});

			this._element[0].XDrop = this;


			this._value = $("<div>", {
				'class': "XDropResult"
			});


			/**
			 * Open/Close the xdrop panel if needed
			 */
			this._toggleButton = $("<a>", {
				'class': "dropdown-toggle"
			}).append($("<span>", {
				'class': "caret"
			}));
			this._clickOverlay = $("<div>", {
				'class': "click-overlay"
			});

			this._toggleButton.click(function () {
				this.toggle(!this.isOpen());
			}.bind(this));

			this._element.focus(this.toggle.bind(this, true));
			this._clickOverlay.click(this.toggle.bind(this, true));

			/**
			 * @member {Boolean} is the dropdown panel open
			 */
			this._isOpen = false;




			this._element.append(this._value).append(this._toggleButton).append(this._clickOverlay);

			this._panel = new XDropPanel(dataProvider, this._renderFactories, {
				focusFilterOnActivate: !this._isMobile
			});
			this._panel.getElement().addClass("limitedHeight");
			this._observe(this._panel, "selected", _.partial(this._emit, "selected").bind(this)); // forward the selected event
			this._panel.setVisible(false);

			this._panelWrapper = $("<div>", {
				"class": "XDropPanelWrapper" + (!this._isMobile ? " absolute" : "")
			});
			this._displaySelectedEntry();


			this._observe(this._panel, "changed", this._displaySelectedEntry);
			this._observe(this._panel, "close", function () {
				this.toggle(false);
			});

			this._observe(this._panel, "closeAndFocusNext", function () {
				this.toggle(false);
				var tabables = $(".form-control:visible,.xdrop:visible,.sdrop:visible");
				var index = tabables.index(this._element);
				tabables.eq(index + 1).focus();
			});



			this._boundOnDocumentClick = this._onDocumentClick.bind(this);
			this._boundOnResize = _.throttle(function () {
				this._updatePosition(this._isOpen);
			}.bind(this), 16, {
				leading: false
			});

			this._addPanelToDOM();
			this._displaySelectedEntry();

		};

		/**
		 * Internal method add the panel in the correct place 
		 * @private
		 */
		XDrop.prototype._addPanelToDOM = function () {
			this._panelWrapper.append(this._panel.getElement());

			if (this._isMobile) {
				// ignore				
			} else if (this._options.absoluteXDropPanel) {
				$(document.body).append(this._panelWrapper);
			} else {
				this._element.append(this._panelWrapper);

			}

		};

		/**
		 * Get the main element of the XDrop
		 * @returns {JQueryObj} 
		 */
		XDrop.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Is the XDrop opened?
		 * @returns {Boolean}
		 */
		XDrop.prototype.isOpen = function () {
			return this._isOpen;
		};

		/**
		 * Toggle the xdrop open/close
		 * @param {Boolean} isOpen 
		 */
		XDrop.prototype.toggle = function (isOpen) {
			if (this._isOpen !== isOpen) {

				if (isOpen) {
					if (!this._isMobile) {
						// when we click outside the xdrop, we want to deactivate
						$(document).on("click", this._boundOnDocumentClick);
					}
					$(window).on("resize", this._boundOnResize);
					$(window).on("scroll", this._boundOnResize);

				} else {
					if (!this._isMobile) {
						// we no longer need to deactivate on click
						$(document).off("click", this._boundOnDocumentClick);
					}
					$(window).off("resize", this._boundOnResize);
					$(window).off("resize", this._boundOnResize);
				}


				this._updatePosition(isOpen);



				this._isOpen = isOpen;
				this._element.toggleClass("open", isOpen);
				this._panelWrapper.toggleClass("hidden", !isOpen);

				this._panel.setVisible(isOpen);
				this._panel.setActive(isOpen);

				this._displaySelectedEntry();


			} else if (isOpen) {
				// if already open, focus the panel again
				this._panel.setActive(isOpen);
			}
		};


		/**
		 * Update the positioning when on mobile (show XDropPanel in modal)
		 * This will also open/close the modal
		 * @private
		 * @param {Boolean} isOpen 
		 */
		XDrop.prototype._updateModalPosition = function (isOpen) {
			if (isOpen && !this._modal) {
				this._modal = new Modal(this._panelWrapper, {
					animate: false
				});
				this._modal.setTitle(this._options.placeholder);
				var self = this;
				this._modal.onClose(function () {
					setTimeout(function () {
						self.toggle(false);
					}, 10);
				});
			}

			if (isOpen) {
				if (!this._modalOpen) {
					this._modal.open();
					this._modalOpen = true;
				}
				var headerHeight = this._modal.getJQElement().find(".modal-header").outerHeight();
				this._panelWrapper.css({
					"max-height": (window.innerHeight - headerHeight - 40) + "px"
				});
			} else if (this._modal) {
				if (this._modalOpen) {
					this._modal.close();
					this._modalOpen = false;
				}
			}

		};

		/**
		 * Update the modal positioning/size when using absolute position
		 * @private
		 * @param {Boolean} isOpen 
		 */
		XDrop.prototype._updateAbsolutePosition = function (isOpen) {
			if (isOpen) {
				// when doing absolute positioning, we need to set the correct position of the dropdown

				var position = this._getXDropPosition();
				var offset = this._element.offset();
				var maxHeight, height = "";
				var top, bottom;
				var above = false;


				if (position.spaceBelow < this._options.preferredMinPanelHeight && position.spaceAbove >= this._options.preferredMinPanelHeight) {
					// show above
					above = true;
					height = (position.spaceAbove - 15);

					if (position.clientRect.bottom > window.innerHeight) {
						// make sure we don't keep growing in the upwards direction
						height = this._currentMaxHeight;
					} else {
						this._currentMaxHeight = height;
					}

					maxHeight = height + "px";

					top = (offset.top - height) + "px";
					height = height + "px";
					bottom = "auto"; //offset.top; // should be top as well...
				} else {
					maxHeight = (position.spaceBelow - 15) + "px";
					top = (offset.top + position.clientRect.height - 1) + "px";
					bottom = "auto";
				}



				this._panelWrapper.css({
					top: top,
					left: offset.left,
					bottom: bottom,
					width: this._element.outerWidth(),
					height: height,
					"max-height": maxHeight
				}).toggleClass("above", above);
			}
		};

		/**
		 * Update the max height property when using relative/absolute position
		 * @private
		 * @param {Boolean} isOpen 
		 */
		XDrop.prototype._updateRelativePosition = function (isOpen) {
			if (isOpen) {
				var position = this._getXDropPosition();
				var maxHeight;
				var above = false;
				if (position.spaceBelow < this._options.preferredMinPanelHeight && position.spaceAbove >= this._options.preferredMinPanelHeight) {
					// show above
					above = true;


					maxHeight = (position.spaceAbove - 15);
					if (position.clientRect.bottom > window.innerHeight) {
						// make sure we don't keep growing in the upwards direction
						maxHeight = this._currentMaxHeight;
					} else {
						this._currentMaxHeight = maxHeight;
					}


				} else {
					maxHeight = (position.spaceBelow - 15);
				}

				this._panelWrapper.toggleClass("above", above).css({
					"max-height": maxHeight + "px"
				});
			}
		};

		/**
		 * Get the position of the XDrop element
		 * @private
		 * @returns {Object} {spaceAbove, spaceBelow and clientRect}
		 */
		XDrop.prototype._getXDropPosition = function () {
			var position = {};


			position.clientRect = this._element[0].getBoundingClientRect();
			var windowHeight = window.innerHeight;

			position.spaceBelow = windowHeight - position.clientRect.bottom;
			position.spaceAbove = position.clientRect.top;

			return position;
		};

		/**
		 * Display the correct value
		 * @private
		 */
		XDrop.prototype._displaySelectedEntry = function () {
			var oldValue;
			var resultEntry = this._panel.getValue();
			if (resultEntry) {
				var renderFactory = this._renderFactories[resultEntry.getType()];
				if (renderFactory) {
					var renderer = renderFactory.make($("<div>"), resultEntry, this._panel.getRenderAPI());
					oldValue = this._value;
					this._value = renderer.getElement();

					if (this._value != oldValue) {
						oldValue.replaceWith(this._value);
					}

					return;
				}
			}

			oldValue = this._value;

			this._value = $("<div>", {
				'class': "XDropResult placeholder",
				text: this.isOpen() ? "" : this._options.placeholder
			});
			oldValue.replaceWith(this._value);


		};

		/**
		 * Get the value of the XDrop
		 * @returns {ResultEntry} 
		 */
		XDrop.prototype.getValue = function () {
			return this._panel.getValue();
		};

		/**
		 * Set the value of the XDrop
		 * @param   {ResultEntry} resultEntry 
		 */
		XDrop.prototype.setValue = function (resultEntry) {
			return this._panel.setValue(resultEntry);
		};


		/**
		 * When the document is clicked, and the click didn't originate in the XDrop or XDrop panel, we need to deactivate
		 * @private
		 * @param {Event} event 
		 */
		XDrop.prototype._onDocumentClick = function (event) {
			var parentSet = $(event.target).parents().add(event.target);
			if (parentSet.filter([this._element[0], this._panel.getElement()[0]]).length === 0) {
				this.toggle(false);
			}
		};

		/**
		 * Set the query of the dropdown panel
		 * @param {String} query
		 */
		XDrop.prototype.setQuery = function (query) {
			this._panel.setQuery(query);
		};

		/**
		 * Get the DOM element of the results panel
		 * (this can be used for finding/clicking results in SAHI
		 * @returns {HTMLNode} 
		 */
		XDrop.prototype.getPanelElement = function () {
			return this._panel.getElement()[0];
		};

		return XDrop;
	});