define(["./CompositeDataProvider",
		"./ResultEntry",
		"underscore",
		"utils/webi18n!XDrop"
	],
	function (CompositeDataProvider,
		ResultEntry,
		_,
		i18n) {

		"use strict";

		/**
		 * A LimitResultsDataProvider merges the results from all sub dataproviders and limits the number of results
		 * If there are more results, a button is shown to show more results
		 * @class
		 * @augments CompositeDataProvider
		 * @mixes Observer
		 * @param {Object} options
		 * @param {XDropDataProvider...,Array.<XDropDataProvider>} dataproviders - list of data provider, this can be by providing a sequence of parameters, or arrays with datproviders
		 */
		var LimitResultsDataProvider = function (options, dataproviders) {
			CompositeDataProvider.apply(this, _.rest(arguments));

			this._options = _.extend({
				maxResults: 50,
				extraEntriesShownOnClick: 20,
				showMoreLabel: i18n.t("LoadMoreResults")
			}, options);

			this._showExtra = 0;


			var self = this;
			this._showMoreEntry = new ResultEntry("button", {
				getLabel: function () {
					return self._options.showMoreLabel;
				},
				getClassNames: function () {
					return "btn btn-showmore";
				},
				doAction: function (event, api) {
					self.setShowExtra(self._showExtra + self._options.extraEntriesShownOnClick);
				}
			});

			this._showMoreEntry.setKeepSelected(false);
			this._showMoreEntry.setDefaultSelectionIndex(70); // we don't want this to be selected as default
		};

		LimitResultsDataProvider.prototype = Object.create(CompositeDataProvider.prototype);
		LimitResultsDataProvider.prototype.constructor = LimitResultsDataProvider;


		/**
		 * @override reset the limit when query changes
		 * @param {String} query 
		 */
		LimitResultsDataProvider.prototype.setQuery = function (query) {
			this._showExtra = 0;

			CompositeDataProvider.prototype.setQuery.apply(this, arguments);

		};

		/**
		 * Update the amount of extra items to show
		 * @param {Number} showExtra 
		 */
		LimitResultsDataProvider.prototype.setShowExtra = function (showExtra) {

			if (this._showExtra != showExtra) {
				this._showExtra = showExtra;
				this._updateResultSet();
			}
		};

		/**
		 * Merge the resultsets of the sub dataproviders
		 * @protected
		 */
		LimitResultsDataProvider.prototype._updateResultSetImpl = function () {
			this._currentResultList = this._generateResultList();




			if (this._currentResultList.length > (this._options.maxResults + this._showExtra)) {
				var itemsToShow = this._options.maxResults + this._showExtra - 1;
				var itemsToRemove = this._currentResultList.length - itemsToShow;
				this._currentResultList.splice(itemsToShow, itemsToRemove, this._showMoreEntry);
			}

			this._resultSet.setEntries(this._currentResultList);
			this._resultSet.commitChanges();
		};

		return LimitResultsDataProvider;
	});