define(["underscore",
		"./XDropDataProvider",
		"./ResultEntry",
		"webcenterapi/data/WebCenterUser",
		"utils/FilterUtils",
		"utils/HTTPUtils",
		"wcr/webcenter_xml"
	],
	function (_,
		XDropDataProvider,
		ResultEntry,
		WebCenterUser,
		FilterUtils,
		HTTPUtils,
		WCXML) {

		"use strict";

		/**
		 * @class WCRUserDataProvider
		 * @augments XDropDataProvider
		 * 
		 * @constructor
		 * 
		 * @param {String} dataProviderType - type of data provider
		 * @param {Boolean} isActiveUsers - flag to know about status of the user needs to listed.
		 * @param {Object} options - Extra parameters as options.
		 */
		var WCRUserDataProvider = function (dataProviderType, isActiveUsers, options) {
			XDropDataProvider.call(this, options);

			this._data = [];
			this._dataProviderType = dataProviderType;
			this._isActiveUsers = isActiveUsers;
			this._query = '';

			this._throttledUpdate = _.throttle(this._getUsersList.bind(this), 500, {
				leading: false
			});
		};

		WCRUserDataProvider.prototype = Object.create(XDropDataProvider.prototype);
		WCRUserDataProvider.prototype.constructor = WCRUserDataProvider;

		/**
		 * Filter result set entries based on query provided by user.
		 * 
		 * @param queryString
		 */
		WCRUserDataProvider.prototype.setQuery = function (query) {
			this._query = query;
			this._throttledUpdate();
		};

		WCRUserDataProvider.prototype.setMaxResults = function (maxResults) {
			this._maxResults = maxResults;
			this._throttledUpdate();
		};

		/**
		 * Makes a param object to make AJAX call to fetch user list.
		 */
		WCRUserDataProvider.prototype._getUsersListParam = function () {
			var params = {
				"list": this._isActiveUsers ? "Active" : "Inactive",
				"filter": FilterUtils.preprocessPattern(this._query),
				"maxcount": this.getMaxResults(),
				"json": 0
			};

			return params;
		};

		/**
		 * Makes an AJAX call to fetch user list.
		 */
		WCRUserDataProvider.prototype._getUsersList = function () {
			if (this._maxResults !== 0) {
				var params = this._getUsersListParam();
				this._startLoading("LoadingUsers");
				$.ajax({
					'url': HTTPUtils.constructURL("GetUserList.jsp", {
						json: 0
					}),
					'type': "POST",
					'data': params,
					'success': this._loadCompleted.bind(this),
					'error': this._loadFailed.bind(this),
					'dataType': 'xml'
				});
			} else {
				this._resultSet.clear();
			}
		};

		/**
		 * Once AJAX call is completed successfully, then create user data entry for result set.
		 *  
		 * @param xml
		 */
		WCRUserDataProvider.prototype._loadCompleted = function (xml) {
			if (xml !== null) {
				// Check for error if any occurred
				var error = xml.getElementsByTagName("error");
				if (error.length > 0) {
					var messages = error[0].getElementsByTagName("message");
					error = "ERROR in response";
					if (messages.length > 0) {
						error = WCXML.GetElementSelfTextContent(messages[0]);
					}

					this._loadFailed(error);
				}

				var resultList = [];

				// Create user entry after parsing XML.
				var elements = xml.getElementsByTagName("entry");
				if (elements) {
					resultList = [];
					for (var i = 0; i < elements.length; i++) {

						var id = elements[i].getElementsByTagName("id")[0].childNodes[0].nodeValue;

						var name = elements[i].getElementsByTagName("name")[0].childNodes[0].nodeValue;
						var firstName = "";
						if (elements[i].getElementsByTagName("firstname")[0].childNodes[0]) {
							firstName = elements[i].getElementsByTagName("firstname")[0].childNodes[0].nodeValue;
						}
						var lastName = elements[i].getElementsByTagName("lastname")[0].childNodes[0].nodeValue;
						var email = "";
						if (elements[i].getElementsByTagName("email")[0].childNodes[0]) {
							email = elements[i].getElementsByTagName("email")[0].childNodes[0].nodeValue;
						}

						var status = elements[i].getElementsByTagName("status")[0].childNodes[0].nodeValue == 1;

						var user = new WebCenterUser(id, name, firstName, lastName, email, status);

						var userEntry = {
							type: this._dataProviderType,
							data: user,
							getData: function(){return this.data;}
						};
						resultList.push(userEntry);
					}
				}

				// Set data once result list is ready.
				this.setData(resultList);
				this._stopLoading("LoadingUsers");
			} else {
				this._loadFailed("Could not load list of users.");
			}
		};

		/**
		 * Something went wrong while loading the user list data
		 * 
		 */
		WCRUserDataProvider.prototype._loadFailed = function (error) {
			if (error !== 0) {
				if (!error) {
					error = "XHR error";
				}

				var errorMessage = error;

				if (error == "998") {
					errorMessage = "Session expired";
				}

				throw errorMessage;
			}
			this._stopLoading("LoadingUsers");
		};

		/**
		 * Set user data in to result set.
		 * 
		 * @param data
		 */
		WCRUserDataProvider.prototype.setData = function (data) {
			if (_.some(data, function (entry) {
					return !entry.type || !entry.data;
				})) {
				throw "Data entry should have a type, data and filter property";
			}

			this._data = data;
			this.setEntries();
		};


		/**
		 * Set result entries in to result set.
		 * 
		 */
		WCRUserDataProvider.prototype.setEntries = function () {
			this._resultSet.clear();
			var resultEntries = _.map(this._data, function (entry) {
				return new ResultEntry(entry.type, entry.data);
			});

			this._resultSet.appendAll(resultEntries);

			this._resultSet.commitChanges();
		};

		return WCRUserDataProvider;
	});
