define(["./ResultSet", 
		"utils/StringSet",
		"utils/Observable", 
		"underscore"], 
function(ResultSet,
		  StringSet,
		  Observable, 
		  _){	
	"use strict";
	
	/**
	 * An XDropData provider is responsible for providing data to the XDRop
	 * @class
	 */
	var XDropDataProvider = function XDropDataProvider(options){
		Observable.mixin(this, "dataprovider");	
		
		this._resultSet = new ResultSet();
		
		this._options = _.extend({
			maxResults: -1	// -1 means no limit of records.
		}, options);
		
		this._maxResults = this._options.maxResults;
		
		this._ongoingLoads = new StringSet();
	};
	
	/**
	 * Get the resultset of the dataprovider
     * @returns {ResultSet} 
	 */
	XDropDataProvider.prototype.getResultSet = function(){
		return this._resultSet;
	};
	
	/**
	 * Filter the results of the dataprovider
	 * !WARNING! This call is not throttled: to improve performance you should throttle the handling of changing the query 
	 * @param {String} queryString
	 */	
	XDropDataProvider.prototype.setQuery = function(queryString){
		//>>excludeStart("PExclude", pragmas.productionExclude);		
		/** PLEASE IMPLEMENT IN SUBCLASS **/
		console.warn("Not implemented");
		//>>excludeEnd("PExclude");
	};
	
	/**
	 * Limit the number of results returned by the dataprovider
	 * 
	 * @param {Number} maxResults - if maxResults = -1 --> no limit
	 */
	XDropDataProvider.prototype.setMaxResults = function(maxResults){
		this._maxResults = maxResults;
		//>>excludeStart("PExclude", pragmas.productionExclude);
		/** PLEASE IMPLEMENT IN SUBCLASS **/
		console.warn("Not implemented");
		//>>excludeEnd("PExclude");
	};
	
	XDropDataProvider.prototype.getMaxResults = function(maxResults){
		if (this._maxResults < 0) {
			return this._options.maxResults;
		} else if (this._options.maxResults >= 0) {
			return Math.min(this._maxResults, this._options.maxResults);
		} else {
			return -1;
		}
	};
	
	
	//// LOADING
	/**
	 * Mark the start of a load operation
	 * @private
	 * @param {String} key - unique identification of the load operation
	 */
	XDropDataProvider.prototype._startLoading = function(key){
		var wasLoading = this.isLoading();
		this._ongoingLoads.add(key);
		if (wasLoading != this.isLoading()){
			this._emit("loading", this.isLoading());	
		}
	};
	
	/**
	 * Is the dataprovider currently loading data?
	 * @returns {Boolean} 
	 */
	XDropDataProvider.prototype.isLoading = function(){
		return this._ongoingLoads.size() > 0;
	};
					   
	/**
	 * Mark the end of a load operation
	 * @private
	 * @param {String} key - unique identification of the load operation (same as used for _startLoading() call)
	 */
	XDropDataProvider.prototype._stopLoading = function(key){
		var wasLoading = this.isLoading();
		this._ongoingLoads.remove(key);
		if (wasLoading != this.isLoading()){
			this._emit("loading", this.isLoading());	
		}		
	};
	
	return XDropDataProvider;
});