define(["utils/AnimationFrameRenderer",
		"utils/Observable",
		"../dataproviders/ResultEntry",
		"../renderers/LabelRenderer",
		"jquery",
		"underscore",
		"utils/webi18n!XDrop"
	],
	function (AnimationFrameRenderer,
		Observable,
		ResultEntry,
		LabelRenderer,
		$,
		_,
		i18n) {

		"use strict";

		/**
		 * The XDropPanel acts as a ViewController for the XDrop functionality
		 * It contains the dataprovider and displays the ResultSet of the dataprovider using renderers
		 * 
		 * @class
		 * @param {XDropDataProvider} dataProvider    
		 * @param {Object}            renderFactories - map of result type to factory
		 */
		var XDropPanel = function (dataProvider, renderFactories, options) {
			AnimationFrameRenderer.mixin(this);
			Observable.mixin(this, "XDropPanel");

			this._options = _.extend({
				filterPlaceholder: i18n.t("FilterPlaceholder"),
				noMatchesLabel: i18n.t("NoMatches"),
				focusFilterOnActivate: true,
				showNoMatches: true
			}, options);
			
			// we provide some default renderers
			renderFactories = _.extend({
				label: LabelRenderer
			}, renderFactories);

			this._focusIndex = 0;

			this._selectedEntry = null;

			this._currentSearch = '';

			this._element = $("<div>", {
				'class': "XDropPanel"
			});


			// The filter field allows the user to filter down results
			// we need to handle keypresses and update dataproviders with new queries if the input is changed
			this._input = $("<input>", {
				'class': "form-control XDrop-filter",
				placeholder: this._options.filterPlaceholder
			}).keydown(function (event) {
				this._processKeyPress(event.keyCode, event);
			}.bind(this)).focus(function () {
				this.setActive(true);
			}.bind(this)).on("input", function () {
				this.setQuery(this._input.val());
			}.bind(this));
			this._element.append($("<div>", {
				'class': "XDropResult filter"
			}).append(this._input));

			/**
			 * All results will be added in this div
			 */
			this._results = $("<div>", {"class": "results"});
			this._element.append(this._results);



			/**
			 * Store the dataprovider and its resultset
			 */
			this._dataProvider = dataProvider;
			this._resultSet = dataProvider.getResultSet();

			// when the result set changes, we need to fix the focused result and render ourselves
			this._resultSet.on("changed", function () {
				// update focus index
				if (this._focusEntry && this._focusEntry.keepSelected()) {
					var newIndex = this._resultSet.indexOf(this._focusEntry);
					if (newIndex >= 0) {
						this.setFocusIndex(this.findFirstFocusableEntry(newIndex));
					} else {
						this.resetFocusIndex();
					}
				} else if (this._focusEntry) {
					this.setFocusIndex(this.findFirstFocusableEntry(this._focusIndex));
				} else {
					this.resetFocusIndex();
				}

				this._render();
			}.bind(this));
			
			this._dataProvider.on("loading", function(isLoading){
				this._element.toggleClass("is-loading", isLoading);
			}.bind(this));


			/**
			 * @member {ResultEntry} the currently focused result
			 */
			this._focusEntry = null;
			this.resetFocusIndex();


			this._isVisible = true;
			this._isActive = false;

			this._initRender(renderFactories);

			this._render();
		};


		/**
		 * Get the element of the XDropPanel
		 * @returns {JQueryObj} 
		 */
		XDropPanel.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Init the render engine for the XDropPanel
		 * @private
		 * @param {Object}            renderFactories - map of result type to factory
		 */
		XDropPanel.prototype._initRender = function (renderFactories) {
			this._renderFactories = renderFactories;
			this._renderMap = {};
			this._usedRenderMap = {};

			var self = this;

			/**
			 * The render API can be used by the renderers  to communicate with the XDrop
			 */
			this._renderAPI = {
				/**
				 * This method will request the XDrop to be closed (will only work if supported by the implementation)
				 */
				closeXDrop: function () {
					self._emit("close");
				},

				/**
				 * Change the currently selected result for the XDrop
				 * @param {ResultEntry} resultEntry 
				 */
				selectResultEntry: function (resultEntry) {
					self.setValue(resultEntry);
				},
				
				/**
				 * Set the focus to a specific result entry
				 * Only trigger this based on a user action
				 * @param {ResultEntry} resultEntry 
				 */
				focusResultEntry: function(resultEntry) {
					var focusIndex = self._resultSet.indexOf(resultEntry);
					if (focusIndex >= 0) {
						self.setFocusIndex(focusIndex);
					}						
				}
			};

		};

		/**
		 * Set the filter query of the XDropPanel
		 * @param {String} queryString 
		 */
		XDropPanel.prototype.setQuery = function (queryString) {
			if (queryString != this._currentSearch) {
				if (this._input.val() !== queryString){
					this._input.val(queryString);
				}
				
				this._currentSearch = queryString;

				// reset focus
				this.resetFocusIndex();

				// update the dataprovider
				this._dataProvider.setQuery(queryString);
				this._render();
			}
		};

		/**
		 * Handle keypresses in the filter field
		 * @private
		 * @param {Number} keyCode 
		 * @param {Event}  event   - browser event	
		 */
		XDropPanel.prototype._processKeyPress = function (keyCode, event) {
			if (this._isActive) {
				switch (keyCode) {
				case 13: // ENTER --> activate the entry
					var entry = this.getFocusedEntry();
					if (entry && this._renderMap[entry.getUniqueId()]) {
						this._renderMap[entry.getUniqueId()].activate();
					}
					break;
				case 38: // ARROW UP --> change focus
					this.setFocusIndex(this.findFirstFocusableEntry(this._focusIndex - 1, true, true));
					break;
				case 40: // ARROW DOWN --> change focus					
					this.setFocusIndex(this.findFirstFocusableEntry(this._focusIndex + 1, true));
					break;
				case 33: // PAGE UP	--> change focus				
					this.setFocusIndex(this.findFirstFocusableEntry(this._focusIndex - 10, true, true));
					event.stopPropagation();
					event.preventDefault();
					break;
				case 34: // PAGE DOWN --> change focus
					this.setFocusIndex(this.findFirstFocusableEntry(this._focusIndex + 10, true));
					event.stopPropagation();
					event.preventDefault();
					break;
				case 9: // TAB --> deactivate
					this.setActive(false);
					event.stopPropagation();
					event.preventDefault();
					this._emit("closeAndFocusNext");
					break;
				case 27: // ESCAPE --> deactivate
					this.setActive(false);
					this._emit("close");
					break;
				}
			}

		};

		/**
		 * Find the first entry that can be focused and selected
		 * @param   {Number}  [startPos=0]           - start position 
		 * @param 	{Boolean} forNonDefaultSelection - if true, also elements which shouldn't be selected as default are matched                                 
		 * @returns {Number}  focus index
		 */
		XDropPanel.prototype.findFirstFocusableEntry = function (startPos, forNonDefaultSelection, upwards) {
			if (!startPos || isNaN(startPos + 0)) {
				startPos = 0;
			}
			var resultList = this._resultSet.getEntriesImmutable();
			var i, pos, resultEntry;
			var bestDefaultSelectionIndex;
			var bestPosition = startPos;
			
			var step = 1;
			if (upwards){
				step = -1;
			}
			
			for (i = 0; i < resultList.length; i++) {
				pos = this._normalizeFocusIndex(startPos + (step*i));
				resultEntry = resultList[pos];
				if (resultEntry &&
					resultEntry.allowSelection()) {
					if (forNonDefaultSelection) {
						return pos;
					} else {
						if (resultEntry.getDefaultSelectionIndex() > bestDefaultSelectionIndex ||
							_.isUndefined(bestDefaultSelectionIndex)) {
							bestDefaultSelectionIndex = resultEntry.getDefaultSelectionIndex();
							bestPosition = pos;
						}
					}

				}
			}

			return bestPosition;
		};

		/**
		 * Normalize a focus Index (make sure it is in the range [0, resultSetSize.size()]
		 * @private
		 * @param   {Number} index 
		 * @returns {Number} 
		 */
		XDropPanel.prototype._normalizeFocusIndex = function (index) {
			var resultSetSize = this._resultSet.getSize();
			if (resultSetSize === 0) {
				return 0;
			} else {
				return ((index % resultSetSize) + resultSetSize) % resultSetSize;
			}

		};

		/**
		 * Reset the currently focused element
		 */
		XDropPanel.prototype.resetFocusIndex = function () {
			this.setFocusIndex(this.findFirstFocusableEntry());
		};

		/**
		 * Change the currently focused element
		 * @param {Number} index 
		 */
		XDropPanel.prototype.setFocusIndex = function (index) {
			this._focusIndex = this._normalizeFocusIndex(index);
			this._focusEntry = this._resultSet.getEntry(this._focusIndex);
			this._render();
		};

		/**
		 * Get the focused entry (based on the focusindex)
		 * @returns {ResultEntry} 
		 */
		XDropPanel.prototype.getFocusedEntry = function () {
			return this._resultSet.getEntry(this._focusIndex);
		};

		/**
		 * Get the render API of the XDropPanel
		 * @returns {Object} 
		 */
		XDropPanel.prototype.getRenderAPI = function () {
			return this._renderAPI;
		};

		/**
		 * Update the rendering of the XDropPanel	
		 * @private
		 */
		XDropPanel.prototype._doRender = function () {
			if (!this._isVisible) {
				return; // no render when invisible
			}
			
			var focusedRenderer = null;

			var domEl = this._results.get(0);
			var entries = this._resultSet.getEntriesImmutable();
			if (entries.length === 0 && this._options.showNoMatches){				
				// show no results indication
				var nomatches = new ResultEntry("label", this._options.noMatchesLabel);
				nomatches.setAllowSelection(false);				
				entries = [nomatches];				
			}
			for (var i = 0; i < entries.length; i++) {
				var renderer = this._getRenderer(entries[i]);
				var renderEl = renderer.getElement()[0];
				if (domEl.children[i] != renderEl) {
					domEl.insertBefore(renderEl, domEl.children[i] || null); // IE10 can't handle undefined --> || null 
				}

				// update the state of the renderers	
				var isFocused = i === this._focusIndex;
				
				renderer.setFocused(isFocused);
				if (isFocused){
					focusedRenderer = renderer;
				}
				renderer.setSelected(entries[i] === this._selectedEntry);

				renderer.render();
				renderer.setQuery(this._currentSearch);
			}

			this._cleanRenderers();
			
			if (focusedRenderer){
				// scroll the highlighted result in to the view
				var focusedElement = focusedRenderer.getElement()[0];
				var focusedClientRect = focusedElement.getBoundingClientRect();
				var panelClientRect = domEl.getBoundingClientRect();
				
				var diff;
				// +1 -1 is needed for borders
				if ((focusedClientRect.top+1) < panelClientRect.top){					
					diff = panelClientRect.top - focusedClientRect.top;					
					this._results.scrollTop(this._results.scrollTop()-diff);										
				} else if ((focusedClientRect.bottom-1) > panelClientRect.bottom){
					diff = panelClientRect.bottom - focusedClientRect.bottom;
					this._results.scrollTop(this._results.scrollTop()-diff);
				}
				
			}
		};


		/**
		 * Set the value of the XDrop
		 * Currently only single value selection is supported
		 * @param {ResultEntry} resultEntry - the result to be selected
		 */
		XDropPanel.prototype.setValue = function (resultEntry) {
			if (this._selectedEntry != resultEntry) {
				var oldEntry = this._selectedEntry;
				this._selectedEntry = resultEntry;

				if (oldEntry) {
					if (this._renderMap[oldEntry.getUniqueId()]) {
						this._renderMap[oldEntry.getUniqueId()].setSelected(false);
					}
				}
				this._emit("deselected", oldEntry);

				if (resultEntry) {
					if (this._renderMap[resultEntry.getUniqueId()]) {
						this._renderMap[resultEntry.getUniqueId()].setSelected(true);
					}
				}
				this._emit("selected", resultEntry);
				this._emit("changed");
			}
		};


		/**
		 * Get the value of the XDrop
		 * @returns {ResultEntry} 
		 */
		XDropPanel.prototype.getValue = function () {
			return this._selectedEntry;
		};

		/**
		 * Get the renderer for a resultentry
		 * @private
		 * @param   {ResultEntry} resultEntry 
		 * @returns {Renderer}    
		 */
		XDropPanel.prototype._getRenderer = function (resultEntry) {
			var renderer = this._renderMap[resultEntry.getUniqueId()];

			if (renderer) {
				delete this._renderMap[resultEntry.getUniqueId()];
			} else {
				var factory = this._renderFactories[resultEntry.getType()];
				if (!factory) {
					throw new Error("Trying to render item of unknown type");
					// future make fallback  renderer
				}

				renderer = factory.make($("<div>"), resultEntry, this._renderAPI);

				if (this._selectedEntry == resultEntry) {
					renderer.setSelected(true);
				}
			}

			this._usedRenderMap[resultEntry.getUniqueId()] = renderer;

			return renderer;
		};

		/**
		 * Clean out the no longer used renderers	
		 * @private
		 */
		XDropPanel.prototype._cleanRenderers = function () {
			_.each(this._renderMap, function (renderer) {
				renderer.destroy();
			});

			this._renderMap = this._usedRenderMap;
			this._usedRenderMap = {};
		};


		/**
		 * Change the visibility of the XDropPanel
		 * @param {Boolean} isVisible 
		 */
		XDropPanel.prototype.setVisible = function (isVisible) {
			if (this._isVisible != isVisible) {
				this._isVisible = isVisible;
				this._element.toggleClass("hidden", !isVisible);

				this._render();
			}
		};

		/**
		 * Activate or deactivate the component
		 * @param {Boolean} isActive 
		 */
		XDropPanel.prototype.setActive = function (isActive) {
			if (this._isActive != isActive) {
				this._isActive = isActive;
				if (this._options.focusFilterOnActivate){
					this._input.focus().select();
				}

				if (isActive) {
					if (this._selectedEntry) {
						var selectedIndex = this._resultSet.indexOf(this._selectedEntry);
						if (selectedIndex >= 0) {
							this.setFocusIndex(selectedIndex);
							this._render();
						}
					}

				} else {
					this._emit("close");
				}

			} else if (isActive) {
				if (this._options.focusFilterOnActivate){
					_.defer(function () {
						if (!this._input.is(":focus")) {
							this._input.focus().select();
						}
					}.bind(this));
				}
			}
		};
		return XDropPanel;
	});