define(["./Renderer",
		"underscore",
		"jquery",
		"webcenterapi/data/WebCenterUser",
		"component/xdrop/dataproviders/ResultEntry",
		"utils/webi18n!XDrop"
	],
	function (
		Renderer,
		_,
		$,
		WebCenterUser,
		ResultEntry, i18n) {

		"use strict";

		/**
		 * Base class for the Group and Role Renderers
		 * @class
		 * @augments Renderer
		 * @param {JQueryObj}   div         
		 * @param {ResultEntry} resultEntry 
		 * @param {Object}      api        -  render API
		 */
		var GroupRoleRenderer = function (div, resultEntry, api, maxMembers, iconclass) {
			Renderer.call(this, div, resultEntry, api);

			this._maxMembers = maxMembers;
			this._iconClass = iconclass;
			this._init();
		};



		GroupRoleRenderer.prototype = Object.create(Renderer.prototype);
		GroupRoleRenderer.prototype.constructor = GroupRoleRenderer;

		GroupRoleRenderer.prototype._init = function () {
			this._element.addClass("group-renderer");

			this._membersList = $("<ul class='member-list group-member-list'>");
			this._moreItem = $("<div class='more-item'>");
			this._moreItem.on("click", _.partial(function (renderer, e) {
				var start = parseInt(this.getAttribute("data-start"));

				var filtered = renderer._getFilteredMembers(this._previousQuery);
				renderer._addMembers(filtered, start, (filtered.length - renderer._maxMembers > 1) ? renderer._maxMembers : filtered.length);
				e.stopPropagation();
				e.preventDefault();
			}, this));

			var name = this._resultEntry.getData().getName();
			this._element.append($("<i>", {
					'class': "icon icon-normal " + this._iconClass + " add-spacing-right"
				}))
				.append($("<span>").text(name))
				.append(this._membersList)
				.append(this._moreItem);
			
			this._element.attr("data-sahi-id", this._resultEntry.getData().getSAHIMarker());
		};
	
	
		/**
		 * Get a Link to display a member (and add that member on click)
		 * @private
		 * @param   {String}                       type   
		 * @param   {WebCenterUser|WebCenterGroup} member - member
		 * @returns {JQueryObj}                    
		 */
		GroupRoleRenderer.prototype._getMemberLink = function(type, member){
			
			return $("<a>", {
				"data-sahi-id": this._resultEntry.getData().getSAHIMarker() + " " + member.getSAHIMarker(),
				"click": function(e){
					var resultEntry = new ResultEntry(type, member);
					this._selectResultEntry(resultEntry);
					this._closeXDrop();
					e.stopPropagation();
					e.preventDefault();
			  }.bind(this)}).append(member.getDisplayElement(false, true));
		};

		/**
		 * adds members to this._membersList
		 */
		GroupRoleRenderer.prototype._addMembers = function (members, startIndex, amount) {
			_.each(_.first(_.rest(members, startIndex), amount), function (member) {
				var type = member instanceof WebCenterUser ? "user" : "group";
				
				var li = $("<li>", {"class": "group-member "+type});
				
				if (type === "group" && member.getGroupAssignee()){
					// group with group assignee should be rendered differently
					li.append($("<i class='icon icon-WCR_Cont-Circle12 user'>"));
					li.append(this._getMemberLink("user", member.getGroupAssignee()));
					li.append(" @ ");
					li.append($("<i class='icon icon-WCR_Cont-Circle12 "+type+"'>"));
					li.append(this._getMemberLink(type, member));
				} else {
					li.append($("<i class='icon icon-WCR_Cont-Circle12 "+type+"'>"));
					li.append(this._getMemberLink(type, member));
				}

				this._membersList.append(li);
			}, this);
			if (_.rest(members, startIndex).length > amount) {
				this._moreItem.addClass("has-more-results").text(i18n.t("LoadMoreResults"));
				this._moreItem.attr("data-start", amount + startIndex);
			} else {
				this._moreItem.removeClass("has-more-results").text("");
			}
		};
	
		/**
		 * return all members that match the query, or if none match the query, returns all the members.
		 */
		GroupRoleRenderer.prototype._getFilteredMembers = function (query) {
			query = query || "";
			var filtered = _.filter(this._resultEntry.getData().getMembers(), function (member) {
				return member.match(query, false);
			});
			if (filtered.length === 0) {
				// if no members match the filter, it is the name of the group/role that matched it.
				// In this case we want to still show all members
				filtered = this._resultEntry.getData().getMembers();
			}
			return filtered;
		};

		/**
		 * Querying should filter the members and highlight the matched text
		 */
		GroupRoleRenderer.prototype.setQuery = function (query) {
			if (this._previousQuery !== query) {
				this._previousQuery = query;
				this._membersList.empty();

				var filtered = this._getFilteredMembers(query);
				this._addMembers(filtered, 0, (filtered.length - this._maxMembers > 1) ? this._maxMembers : filtered.length);

				if (this._resultEntry.allowSelection()) {
					this._element.removeHighlightText("xdrop-hightlightMatch");
					this._element.highlightText(query, "xdrop-hightlightMatch");
				} else {
					this._membersList.removeHighlightText("xdrop-hightlightMatch");
					this._membersList.highlightText(query, "xdrop-hightlightMatch");
				}
			}
		};

		return GroupRoleRenderer;

	});
