define(["jquery", "lib/jquery.highlightText"], function($){
	"use strict";
	
	
	/**
	 * Default renderer implementation
	 * @class
	 * @param {JQueryObj}   div         
	 * @param {ResultEntry} resultEntry 
	 * @param {Object}      api        -  render API
	 */
	var Renderer = function(div, resultEntry, api){
		this._element = div;
		
		this._element.addClass("XDropResult");
		this._resultEntry = resultEntry;
		
		this._hasFocus = false;
		this._isSelected = false;
		
		this._api = api;
		
		this._element.on("click", this.click.bind(this));

	};
	
	/**
	 * Get the element of the renderer
	 * @returns {JQueryObj} 
	 */
	Renderer.prototype.getElement = function(){
		return this._element;
	};
	
	/**
	 * Mark the element as focused
	 * @param {Boolean} hasFocus 
	 */
	Renderer.prototype.setFocused = function(hasFocus){
		if (this._hasFocus != hasFocus){
			this._element.toggleClass("has-focus", hasFocus);
			this._hasFocus = hasFocus;
		}
	};
	
	/**
	 * Mark the element as selected
	 * @param {Boolean} isSelected 
	 */
	Renderer.prototype.setSelected = function(isSelected){
		if (this._isSelected != isSelected){
			this._element.toggleClass("is-selected", isSelected);
			this._isSelected = isSelected;
		}
	};
	
	/**
	 * Higlight the query text in the element
	 * @param {String} query 
	 */
	Renderer.prototype.setQuery = function(query){
		if (this._previousQuery !== query && this._resultEntry.allowSelection()){
			this._element.removeHighlightText("xdrop-hightlightMatch");
			this._element.highlightText(query, "xdrop-hightlightMatch");
			
			this._previousQuery = query;			
		}
	};
	
	/**
	 * Called whenever ResultSet was updated
	 */
	Renderer.prototype.render = function(){
		// implement this method to update view when resultset was modified
	};
	
	/**
	 * A click event occured on the renderer, by default we will activate	
	 */
	Renderer.prototype.click = function(event){
		this.activate();		
		event.preventDefault();
		event.stopPropagation();
	};
	
	/**
	 * Default activate behavior is to select the current result entry and close the dropdown
	 */
	Renderer.prototype.activate = function(){
		if (this._resultEntry.allowSelection()){
			this._focusResultEntry(this._resultEntry);
			this._selectResultEntry(this._resultEntry);
			this._closeXDrop();
		}
	};
	
	/**
	 * Select a ResultEntry
	 * (wrapper around the XDrop Renderer API)
	 * @private
	 * @param {ResultEntry} resultEntry - the result entry that should be selected
	 */
	Renderer.prototype._selectResultEntry = function(resultEntry){
		this._api.selectResultEntry(resultEntry);
	};
	
	/**
	 * Focus a ResultEntry
	 * @private
	 * @param {ResultEntry} resultEntry 
	 */
	Renderer.prototype._focusResultEntry = function(resultEntry){
		this._api.focusResultEntry(resultEntry);	
	};
	
	/**
	 * Notify the XDrop that is can be closed
	 * (wrapper around the XDrop Renderer API)
	 * @private
	 */
	Renderer.prototype._closeXDrop = function(){
		this._api.closeXDrop();
	};
	
	/**
	 * Trigger a "data" event on the ResultEntry	
	 * @private
	 */
	Renderer.prototype._triggerEvent = function(){
		this._resultEntry.trigger.apply(this._resultEntry, arguments);
	};
	
	/**
	 * Destroy the element
	 */
	Renderer.prototype.destroy = function(){
		this._element.remove();	
	};
	
	return Renderer;
});