define(["jquery", "customization/PreviewController", "customization/VariableSelector", "utils/Observable"],
function($, PreviewController, VariableSelector, Observable){
	
	"use strict";
	
	/**
	 * @class
	 * @exports {wcrinit}
	 * @mixes {Observabe}
	 * This class is used to customize a certain component in webcenter, eg. the navbar.
	 * It can only be initiated through the use of the data-init="customization/CustomizationSection" attribute on a dom element.
	 * The dom element that has the data-init attribute should have the following properties:
	 * 		- It should have a data-lessfiles attribute. The value of this attribute should be a string containing all less-files (comma seperated) that need to be compiled to 
	 * 		  customize the component. eg. for the navbar in webcenter this is: data-lessfiles="style/less/bootstrap/mixins/all.less,style/less/wcr/topmenu.less"
	 *		- a variable amount of children that have the class ".less-variable". These dom elements indicate a certain less variable that can be customized. The prerequisites of thses
	 *		  dom elements are documented in VariableSelector.js
	 *	
	 *	The less files that will be built when customizing a variable will be namespaced with the 'w' class. If this shouldn't happen because the less file is already namespaced
	 *  An extra attribute "data-donotnamespace='true'" can be given to the dom element.
	 */
	var CustomizationSection = function () {
		Observable.mixin(this, "CustomizationSection");
		/**
		 * @member {VariableSelector}
		 *
		 */
		this._variables = [];
			
		
	};
	
	/**
	 * @return {Array.<VariableSelector>} list of variables for this customization
	 */
	CustomizationSection.prototype.getVariables = function () {
		return this._variables;
	};

	/**
	 * @param {VariableSelector} lessVariable - a less variable that can be customized in this section
	 */
	CustomizationSection.prototype.addVariable = function (lessVariable) {
		this._variables.push(lessVariable);
	};
	
	/**
	 * indicates that a variable has been changed, is called from the VariableSelector class
	 */
	CustomizationSection.prototype.fireVariableChanged = function () {
		this._emit("changed");
	};
	
	return {
		wcrinit: function(jsElement){
			var context = $(jsElement);
			var customizationSection = new CustomizationSection();
			var variableContexts = context.find(".less-variable");
			variableContexts.each(function(){
				var variable = new VariableSelector($(this), customizationSection);
				customizationSection.addVariable(variable);
			});
			
			var lessFiles = context.attr("data-lessfiles") && context.attr("data-lessfiles").split(",");
			if(lessFiles) {
				new PreviewController(customizationSection, lessFiles, context.attr("data-donotnamespace") === "true");
			}
		}
	};
	
});