define(["underscore",
        "utils/webi18n!MediaBeaconDocSourcePlugin",
        "damconnections/MediaBeacon/MBDataMapper"
    ],
    function (_,
        i18n,
        MBDataMapper) {

        "use strict";
        var MediaBeaconMetaDataPopupHandler = function () {
            /**
             * @member {Object} a JSON containing the information of all workspaces in MediaBeacon (as it is returned from MediaBeacon)
             */
            this.workspacesJSON = null;
            /**
             * @member {Object} containing the information of all namespaces and fields in MediaBeacon (as it is returned from MediaBeacon)
             */
            this.fieldJSON = null;
            /**
             * @member {Map} containing the information of all namespaces in MediaBeacon
             *                       mapped_namespace -> namespace
             */
            this.namespaceMap = null;
            /**
             * @member {Map} containing the fields per namespace
             *                  namespace -> {name, mapped_name, is_container}
             */
            this.fieldMap = null;
            /**
             * @member {node} the html element corresponding with the dialog
             */
            this.container = null;
            /**
             * @member {node} the body for the request part 
             */
            this.requestBody = null;
            /**
             * @member {node} the body for the attributemapping part 
             */
            this.metadataBody = null;
            /**
             * @member {node} the select element for selecting a workspace 
             */
            this.workspaceSelect = null;
            /**
             * @member {Object} the select element for selecting a workspace 
             */
            this.workspaceACLMap = {};
            /**
             * @member {Object} api 
             */
            this.api = null;
            /**
             * @member {MBDataMapper} MBDataMapper that manages the rows for the request
             */
            this.requestMapper = null;
            /**
             * @member {MBDataMapper} MBDataMapper that manages the rows for the metadata mapping
             */
            this.metadataMapper = null;
        };


        MediaBeaconMetaDataPopupHandler.prototype.init = function (container, data, api) {
            this.container = container;
            this.api = api;
        };

        /**
         * initializes some features of the dialog after loading the dialog
         * @param {object} metadata
         */
        MediaBeaconMetaDataPopupHandler.prototype.onLoad = function (data) {
            var errors = this._checkOnErrors();

            if (errors !== "") {
                alert(i18n.t('DocSrcHdlDamMetaError') + "\n" + errors);
                this.container.remove(); //remove the dialog
            } else {
                this.requestBody = this.container.find('#DAMRequest');
                this.metadataBody = this.container.find('#DAMMetadata');

                this._fillWorkspacesSDrop();

                this.requestMapper = new MBDataMapper(this.requestBody, MBDataMapper.types.SEARCH, this.fieldJSON);
                this.metadataMapper = new MBDataMapper(this.metadataBody, MBDataMapper.types.FETCH, this.fieldJSON);

                this.setMetadata(data);
            }
        };


        /**
         * deletes some element that are supposed to be unique
         */
        MediaBeaconMetaDataPopupHandler.prototype.onClose = function () {
            this.container.remove();
        };

        /**
         * Clears the popup
         */
        MediaBeaconMetaDataPopupHandler.prototype.clearMetadata = function () {
            this.workspaceSelect.val("");
            this.requestMapper.clearMetadata();
            this.metadataMapper.clearMetadata();
        };

        /**
         * Adds the new metadata to the popup
         * @param {object} data - metadata
         */
        MediaBeaconMetaDataPopupHandler.prototype.setMetadata = function (data) {
            //clear the popup
            this.clearMetadata();
            //fill popup with new data
            if (data && !$.isEmptyObject(data)) {
                if (data.workspace) {
                    this.workspaceSelect.val(data.workspace);
                }
                this.requestMapper.setMetadata(data.searchitems);
                this.metadataMapper.setMetadata(data.attributeMapping);
            } else {
                this.requestMapper.setMetadata(null);
                this.metadataMapper.setMetadata(null);
            }
        };

        /**
         * returns the data that was inputted in the popup
         */
        MediaBeaconMetaDataPopupHandler.prototype.getMetaData = function () {
            var metadata = {};
            var workspace = this.workspaceSelect.val();
            metadata.workspace = workspace;
            metadata.acl = this.workspaceACLMap[workspace];

            metadata.searchitems = this.requestMapper.getMetaData();
            metadata.attributeMapping = this.metadataMapper.getMetaData();

            return metadata;
        };

        /**
         * validates the input of the popup 
         * @return {Array.<String>}
         */
        MediaBeaconMetaDataPopupHandler.prototype.validateInput = function () {
            var errorMsg = "";
            // validate input
            errorMsg += this.requestMapper.validateInput();
            errorMsg += this.metadataMapper.validateInput();

            return errorMsg;
        };

        /**
         * Adds all workspaces to the sdrop
         * @private
         */
        MediaBeaconMetaDataPopupHandler.prototype._fillWorkspacesSDrop = function () {
            this.workspaceSelect = this.container.find("select[name='WorkspaceSelect']");
            this.workspaceSelect.append($("<option selected></option>")
                .attr("value", "")
                .text(i18n.t('DocSrcHdlDamRequestSelectWS')));
            if (this.workspacesJSON) {
                _.each(this.workspacesJSON.group_WS, _.bind(function (acl) {
                	var workspaces = _.filter(acl.workspaces, function (workspace) {
                        return workspace && !workspace.hidden;
                    });
                	
                	if(workspaces.length > 0){
                		this.workspaceSelect.append($("<option></option>").attr("disabled", true).text(acl.name));
                    	_.each(workspaces, _.bind(function (workspace) {
                    		this.workspaceACLMap[workspace.id] = acl.name;
                            this.workspaceSelect.append($("<option></option>").attr("value", workspace.id).text(workspace.name).prepend("&nbsp;&nbsp;"));
                        }, this));
                	}
                }, this));
            }
        };

        /**
         * Checks if an error occured on the back-end.
         * @private
         * @returns {String} error messages
         */
        MediaBeaconMetaDataPopupHandler.prototype._checkOnErrors = function () {
            this.workspacesJSON = JSON.parse($("input[name='workspaces']").val()||"{}");
            this.fieldJSON = JSON.parse($("input[name='fieldMap']").val()||"{}");

            var errorMsg = "";
            if (this.fieldJSON.error) {
                errorMsg += '\n' + i18n.t(this.fieldJSON.error);
            }
            if (this.workspacesJSON.error && this.workspacesJSON.error !== this.fieldJSON.error) {
                errorMsg += '\n' + i18n.t(this.workspacesJSON.error);
            }

            return errorMsg;
        };

        return MediaBeaconMetaDataPopupHandler;

    });
