define(["underscore",
        "component/smartnames/smartfield/smartfield",
        "utils/Map",
        "utils/webi18n!MediaBeaconDocSourcePlugin",
        "damconnections/DataMapper",
        "component/smartnames/encoding/smartnameencoding",
        "component/sdrop/SDrop"
    ],
    function (_,
        Smartfield,
        Map,
        i18n,
        DataMapper,
        SmartNameEncoding,
        SDrop) {
        "use strict";

        /**
         * Component that creates rows for mapping WebCenter data to MediaBeacon data 
         * 
         * @class
         * @augments DataMapper
         * @exports MBDataMapper
         *                  
         * @param {JQueryObj} container - Container in which the datamapper will reside
         * @param {DataMapper.types} type - determines which type of rows should be created
         * @param {JSONObj} fieldJSON - contains all namespaces and corresponding fields
         */
        var MBDataMapper = function (container, type, fieldJSON, smartNameDictionaryService) {
            DataMapper.call(this, container, type);
            /**
             * @member {Object} containing the information of all namespaces and fields in MediaBeacon (as it is returned from MediaBeacon)
             */
            this.fieldJSON = fieldJSON;
            /**
             * @member {Map} containing the information of all namespaces in MediaBeacon
             *                       mapped_namespace -> namespace
             */
            this.namespaceMap = null;
            /**
             * @member {Map} containing the fields per namespace
             *                  namespace -> {name, mapped_name, is_container}
             */
            this.fieldMap = null;
            /**
             * @member {WorkflowSmartNameDictionaryService} dictionary service for smartfields in Workflow Node
             */
            this.smartNameDictionaryService = smartNameDictionaryService;

            this.load();
        };

        MBDataMapper.prototype = Object.create(DataMapper.prototype);
        MBDataMapper.prototype.constructor = MBDataMapper;

        /**
         * @member {Object} Enum to indicate the possible rows
         */
        MBDataMapper.types = DataMapper.types;

        /**
         * initializes some features of the dialog after loading the dialog
         * @param {object} metadata
         */
        MBDataMapper.prototype.load = function () {
            this._parseFieldsJSON();

            //add listener to button (defined in template)
            this.addRowBtn = $(".addRowBtn", this.container);
            var self = this;
            this.addRowBtn.on("click", function (event) {
                if ($(this).is("[disabled]")) { //prevents clicking a disabled button for IE
                    event.preventDefault();
                } else {
                    self._addRow();
                }
            });
        };

        /**
         * Disables all elements.
         */
        MBDataMapper.prototype.disable = function () {
            this.elementsMap.each(function (id, elements) {
                elements.namespace.setDisabled(true);
                if(elements.field){
                    elements.field.setDisabled(true);
                }
                if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                    elements.value.disable();
                } else if (this.type === DataMapper.types.FETCH) {
                    elements.value.setDisabled(true);
                }
                elements.deleteBtn.attr('disabled', 'disabled');
            }.bind(this));
            if (this.addRowBtn) {
                this.addRowBtn.attr('disabled', 'disabled');
            }
        };

        /**
         * Enables all elements.
         */
        MBDataMapper.prototype.enable = function () {
            this.elementsMap.each(function (id, elements) {
                elements.namespace.setDisabled(false);
                elements.field.setDisabled(false);
                if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                    elements.value.enable();
                } else if (this.type === DataMapper.types.FETCH) {
                    elements.value.setDisabled(false);
                }
                elements.deleteBtn.removeAttr('disabled');
            }.bind(this));
            if (this.addRowBtn) {
                this.addRowBtn.removeAttr('disabled');
            }
        };

        /**
         * returns the data that was inputted in the popup
         */
        MBDataMapper.prototype.getMetaData = function () {
            var metadata = [];

            this.rows.each(function (id, value) {
                var item = {};
                var elements = this.elementsMap.get(id);
                item.namespace = elements.namespace.getSelectedItemID();
                item.field = (elements.field) ? elements.field.getSelectedItemID() : null;

                if (this.type === DataMapper.types.FETCH) {
                    item.value = elements.value.getSelectedItemID();
                } else if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                    var fieldObj = _.find(this.fieldMap.get(item.namespace), function (fo) {
                        return fo.name === item.field;
                    });
                    item.value = elements.value.getValue();
                    item.is_container = (fieldObj) ? fieldObj.is_container : false;
                }

                if (item.namespace || item.field || item.value) { //ignore empty entries
                    metadata.push(item);
                }
            }.bind(this));

            return metadata;
        };

        /**
         * Adds the new metadata to the popup
         * @param {object} data - metadata
         */
        MBDataMapper.prototype.setMetadata = function (data) {
            DataMapper.prototype.setMetadata.call(this, data);
            //add default an empty row
            this._addRow();
        };

        /**
         * validates the input of the popup 
         * @return {Array.<String>}
         */
        MBDataMapper.prototype.validateInput = function () {
            var counter = 0;
            var emptyRows = 0;
            var map = new Map();
            var errorMsg = "";

            if (this.rows) {
                this.rows.each(function (id, row) {
                    counter++;
                    var elements = this.elementsMap.get(id);
                    var namespace = elements.namespace.getSelectedItemID();
                    var field = (elements.field) ? elements.field.getSelectedItemID() : null;
                    var value = null;
                    if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                        value = elements.value.getValue();
                        if (!(namespace && field) && value) {
                            errorMsg += '\n' + i18n.t('DocSrcHdlDamRequestEmptyKey') + counter;
                        }
                    } else if (this.type === DataMapper.types.FETCH) {
                        value = elements.value.getSelectedItemID();
                        if (!value && (namespace || field)) {
                            errorMsg += '\n' + i18n.t('DocSrcHdlDamMetaEmptyKey') + counter;
                        }
                    }

                    var sDropElements = [row.find('.sdrop').find('input')[0], row.find('.sdrop').find('p')[0]];
                    //setup map for checking uniqueness of parameters
                    if (!(value === null && namespace === null && field === null)) {
                        var key = namespace + ":" + field;
                        var highlightElements;
                        if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                            highlightElements = (namespace === null) ? [sDropElements[0]] : [sDropElements[1]];
                        } else if (this.type === DataMapper.types.FETCH) {
                            highlightElements = [sDropElements[0]];
                        }
                        if (map.get(key)) {
                            var elements = map.get(key);
                            elements = elements.concat(highlightElements);
                            map.add(key, elements);
                        } else {
                            map.add(key, highlightElements);
                        }
                    } else {
                        emptyRows++;
                    }
                }.bind(this));
            }

            //check if all parameters are unique
            if (map.size() < this.rows.size() - emptyRows) {
                errorMsg += (this.type === DataMapper.types.SEARCH) ? '\n' + i18n.t('DocSrcHdlDamRequestDuplicateKey') : '\n' + i18n.t('DocSrcHdlDamMetaDuplicateKey');
                map.each(function (key, value) {
                    $(value).removeClass('validationHighlight'); //could be added in previous validation phase
                    if (value.length > 1) {
                        $.each(value, function (index, value) {
                            $(value).addClass('validationHighlight');
                        });
                    }
                });
            }
            return errorMsg;
        };

        /**
         * Adds a row to the body. The UI of the row depends on the type of row that is asked
         * @private
         * @param {Object} item 
         */
        MBDataMapper.prototype._addRow = function (item) {
            if (!item) {
                item = {
                    namespace: "",
                    field: "",
                    value: ""
                };
            }
            var elements = this.elementsMap.get(this.rowId) || {};
            var row = $("<div>").addClass("row add-padding-bottom");

            var attributeSdrop = null;
            if (this.type === DataMapper.types.FETCH) { //this rows starts with a select
                var col = $("<div>").addClass("col-sm-5");
                var select = $("<select class='form-control'>");
                row.append(col.append(select));
                var options = {
                    manageStyleAndWidth: false,
					tabbable: false
                };
                if (item.value) {
                    options.initialValue = item.value;
                }
                attributeSdrop = new SDrop(select, null, new SDropDataProvider({
                    'dataSource': "GetNameIDList.jsp?list=Attributes"
                }), options);
            }

            //Sdrops with MB fields        
            var namespaceSDrop = this._addMBComponent(row, item, elements);
            var smartfield = null;

            if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) { //this row ends with a smartname input field
                //Column 3        
                var inputfield = $('<input type="text" class="form-control"/>');
                var col2 = $("<div>").addClass("col-sm-5");
                row.append(col2.append(inputfield));

                //convert input to smartfield
                if (this.type === DataMapper.types.PUSH) { //workflow context
                    smartfield = new Smartfield(inputfield,
                        false, // lazy init
                        this.smartNameDictionaryService, // our dictionary
                        _.bind(this.smartNameDictionaryService.displaySmartName, this.smartNameDictionaryService), // display in input field
                        _.bind(this.smartNameDictionaryService.displaySmartNameSelectorSmartName, this.smartNameDictionaryService), // display in selector
                        SmartNameEncoding.WORKFLOWEDITOR // encoding
                    );
                } else {
                    smartfield = new Smartfield(inputfield);
                }
                smartfield.setWidth("100%");
                if (item.value) {
                    smartfield.setValue(item.value);
                }
            }

            // delete button
            var self = this;
            var id = this.rowId;
            var btn = $("<a/>")
                .addClass("btn btn-default btn-sm deleteRowBtn")
                .on("click", function(event){
                    if ($(this).is("[disabled]")) { //prevents clicking a disabled button for IE
                        event.preventDefault();
                    }else{
                        self._deleteRow(id);
                    }
                });
            
            btn.append($("<i/>", {
                'class': "icon icon-WCR_Actions-Delete12 icon-normal"
            }));
            row.append(btn);

            this.body.append(row);

            //update the mapping with the elements of the row
            this.rows.add(this.rowId, row);
            elements.namespace = namespaceSDrop;
            if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                elements.value = smartfield;
            } else if (this.type === DataMapper.types.FETCH) {
                elements.value = attributeSdrop;
            }
            elements.deleteBtn = btn;
            this.elementsMap.add(this.rowId, elements);
            this.rowId++;
        };

        /**
         * Adds 2 sdrops to the row
         * @private
         * @param   {node}   row       
         * @param   {String} namespace 
         * @param   {String} field     
         * @returns {node}   row
         */
        MBDataMapper.prototype._addMBComponent = function (row, item, elements) {
            var self = this;
            //Column 1
            var sdrop1 = $("<select class='form-control'>");
            var sdrop1Container = $("<div class='from-group'>").append($("<div class='form-entry'>"));
            row.append($("<div class='col-sm-3'/>").append(sdrop1Container.append(sdrop1)));
            //Column 2
            var sdrop2 = $("<select class='form-control' disabled>");
            var sdrop2Container = $("<div class='form-entry'>");
            row.append($("<div class='col-sm-3'/>").append($("<div class='from-group'>").append(sdrop2Container.append($("<div>").append(sdrop2)))));

            //convert selects to sdrop + fill them
            //make sdrop for namespaces and fill it with namespaces
            var keys = self.namespaceMap.entries();
            sdrop1.append($("<option value disabled selected></option>").text(i18n.t('SDROP_START_TYPING')));
            for (var i = 0; i < keys.length; i++) {
                sdrop1.append($("<option></option>")
                    .attr("value", self.namespaceMap.get(keys[i]))
                    .text(keys[i]));
            }

            //ChangeListener: fill second sdrop with corresponding values for the selected namespace
            var rowId = self.rowId;
            sdrop1.change(function (value) {
                var selectedNamespace = $(value.target).val();
                var names = self.fieldMap.get(selectedNamespace);

                var toRemove = sdrop2Container.find('div');
                if (toRemove) {
                    toRemove.remove();
                }

                sdrop2 = $('<select>');
                sdrop2.append($("<option value disabled selected></option>").text(i18n.t('SDROP_START_TYPING')));
                $.each(names, function (i, item) {
                    sdrop2.append($("<option></option>")
                        .attr("value", item.name)
                        .text(item.mapped_name));
                });
                sdrop2Container.append(sdrop2);

                //initialise SDrop
                var options = {
                    manageStyleAndWidth: false,
					tabbable: false
                };
                if (item.field) {
                    options.initialValue = item.field;
                }
                var fieldSDrop = new SDrop(sdrop2, null, new StaticSDropDataProvider(), options);

                //add the sdrop to the map with the elements of that row
                if (elements) {
                    elements["field"] = fieldSDrop;
                    self.elementsMap.add(rowId, elements);
                }
            });

            //initialise SDrop
            var options = {
                manageStyleAndWidth: false,
				tabbable: false
            };
            if (item.namespace) {
                options.initialValue = item.namespace;
                sdrop1.val(item.namespace);
                sdrop1.trigger("change");
            }
            var namespaceSDrop = new SDrop(sdrop1, null, new StaticSDropDataProvider(), options);

            return namespaceSDrop;
        };

        /**
         * Parses the json that is covered in the template (by a hidden input field)
         * and stores it 2 maps:
         *  - fieldMap : namespace -> array van fields
         *  - namespaceMap : namespace -> mapped_namespace
         * @private
         */
        MBDataMapper.prototype._parseFieldsJSON = function () {
            if (this.fieldJSON) {
                var fields = this.fieldJSON.fields;
                this.fieldMap = new Map();
                this.namespaceMap = new Map();

                for (var i in fields) {
                    if (fields.hasOwnProperty(i)) {
                        var field = fields[i];
                        //setup map
                        if (this.fieldMap.get(field.namespace)) {
                            var elements = this.fieldMap.get(field.namespace);
                            elements.push({
                                name: field.name,
                                mapped_name: field.mapped_name,
                                is_container: field.is_container
                            });
                            this.fieldMap.add(field.namespace, elements);
                        } else {
                            this.fieldMap.add(field.namespace, [{
                                name: field.name,
                                mapped_name: field.mapped_name,
                                is_container: field.is_container
                            }]);
                            this.namespaceMap.add(field.mapped_namespace, field.namespace);
                        }
                    }
                }
            }
        };

        return MBDataMapper;

    });
