define(["underscore",
        "utils/webi18n!WebcenterDocSourcePlugin",
        "component/smartnames/smartfield/smartfield",
        "utils/Map",
        "wcr/webcenter_xml",
        "damconnections/Webcenter/WCRDataMapper",
        "component/sdrop/SDrop"
    ], 
    function (_, i18n,
        Smartfield,
        Map,
        WCXML,
        WCRDataMapper,
        SDrop) {

        "use strict";
        var WebcenterMetaDataPopupHandler = function () {
            /**
             * @member {node} the html element corresponding with the dialog
             */
            this.container = null;
            /**
             * @member {Object} API given by the calling element (e.g. i18n of the parent)
             */
            this.api = null;
            /**
             * @member {node} the body for the request part 
             */
            this.requestBody = null;
            /**
             * @member {node} the body for the attributemapping part 
             */
            this.metadataBody = null;
            /**
             * @member {node} the element containing the SDrop 
             */
            this.sDropContainer = null;
            /**
             * @member {SDrop} sdrop that holds the saved searches 
             */
            this.savedSearchSDrop = null;
            /**
             * @member {Map} mapping that holds all information about the saved searches 
             *                       id -> {name, smartnames {Array}}
             */
            this.savedSearchMap = new Map();
            /**
             * @member {Array} mapping that holds all information about the saved searches to populate the SDrop
             */
            this.savedSearchList = [];
            /**
             * @member {node} element (checkbox) that defines if autonaming will be enable/disabled
             */
            this.autonamingInput = null;
            /**
             * @member {WCRDataMapper} WCRDataMapper that manages the rows for the request
             */
            this.requestMapper = null;
            /**
             * @member {WCRDataMapper} WCRDataMapper that manages the rows for the metadata mapping
             */
            this.metadataMapper = null;
        };

        /**
         * Initialisation function
         * @param {node}     container - the dialog
         * @param {Object}   data      - the metadata
         * @param {Object} api
         */
        WebcenterMetaDataPopupHandler.prototype.init = function (container, data, api) {
            this.container = container;
            this.api = api;
        };

        /**
         * Initializes some features of the dialog after loading the dialog
         * @param {object} metadata
         */
        WebcenterMetaDataPopupHandler.prototype.onLoad = function (data) {
            var errors = this._checkOnErrors($("input[name='attributesXML']").val()) + this._checkOnErrors($("input[name='savedsearchesXML']").val());

            if (errors) {
                this.container.remove(); //remove the dialog
                alert(errors);
            } else {
                this.requestBody = this.container.find('#DAMRequest');
                this.metadataBody = this.container.find('#DAMMetadata');
                this.sDropContainer = this.container.find(".SDropContainer");
                this.autonamingInput = $("input[name='autoNaming']");

                var attributesXML = $("input[name='attributesXML']").val();
                this._parseSavedSearchXML($("input[name='savedsearchesXML']").val());
                this._configureSavedSearchSelect();

                this.requestMapper = new WCRDataMapper(this.requestBody, WCRDataMapper.types.SEARCH, attributesXML);
                this.metadataMapper = new WCRDataMapper(this.metadataBody, WCRDataMapper.types.FETCH, attributesXML);


                this.setMetadata(data);
            }
        };

        /**
         * Deletes the dialog from the DOM
         */
        WebcenterMetaDataPopupHandler.prototype.onClose = function () {
            this.savedSearchSDrop.destroy();
            this.metadataMapper.close();
            this.requestMapper.close();
            this.container.remove();
        };

        /**
         * Returns the data that is inputted in the popup
         * @returns {object} metadata
         */
        WebcenterMetaDataPopupHandler.prototype.getMetaData = function () {
            var metadata = {};
            metadata.savedSearch = this.savedSearchSDrop.getSelectedItemID();
            metadata.matchAttributeByName = this.autonamingInput.prop('checked');

            metadata.parameters = this.requestMapper.getMetaData();
            metadata.attributeMapping = this.metadataMapper.getMetaData();

            return metadata;
        };

        /**
         * Adds the new metadata to the popup
         * @param {object} data - metadata
         */
        WebcenterMetaDataPopupHandler.prototype.setMetadata = function (data) {
            //clear the popup
            this.clearMetadata();

            //fill popup with new data
            if (data && !$.isEmptyObject(data)) {
                if (data.savedSearch) {
                    var value = data.savedSearch;
                    this._configureSavedSearchSelect(value);
                }
                if (data.matchAttributeByName) {
                    this.autonamingInput.prop('checked', data.matchAttributeByName);
                }
                this.requestMapper.setMetadata(data.parameters);
                this.metadataMapper.setMetadata(data.attributeMapping);
            } else {
                this.requestMapper.setMetadata();
                this.metadataMapper.setMetadata();
            }
        };

        /**
         * Clears the popup
         */
        WebcenterMetaDataPopupHandler.prototype.clearMetadata = function () {
            this._configureSavedSearchSelect();
            this.autonamingInput.prop('checked', false);

            this.requestMapper.clearMetadata();
            this.metadataMapper.clearMetadata();
        };

        /**
         * Validates the input of the popup 
         * @return {Array.<String>} - error messages
         */
        WebcenterMetaDataPopupHandler.prototype.validateInput = function () {
            var errorMsg = "";

            var savedsearch = this.savedSearchSDrop.getSelectedItemID();
            if (!savedsearch) {
                errorMsg += '\n' + i18n.t('DocSrcHdlDamMetaEmptySavedSearch');
            }

            errorMsg += this.requestMapper.validateInput();
            errorMsg += this.metadataMapper.validateInput();

            return errorMsg;
        };


        /**
         * Checks if the template received from the backend contains errors
         * @private
         */
        WebcenterMetaDataPopupHandler.prototype._checkOnErrors = function (data) {
            var errorMsg = "";
            if(data){
	            try {
	                var xml = $.parseXML(data);
	                var elements = xml.getElementsByTagName("error");
	                if (elements.length > 0) {
	                    errorMsg += "\n" + i18n.t("DocSrcHdlDamMetaExternalWCRError");
	                    if (elements[0].getElementsByTagName("message").length > 0) {
	                        errorMsg += "\n" + elements[0].getElementsByTagName("message")[0].textContent;
	                    }
	                }
	            } catch (exception) {
	                var containsDAMError = data.indexOf("damconnection.wcr");
	                if (containsDAMError !== -1) {
	                    errorMsg = "\n" + i18n.t(data);
	                } else {
	                    errorMsg = "\n" + i18n.t("DocSrcHdlDamXMLParseError");
	                }
	            }
            }
            return errorMsg;
        };

        /**
         * Parses the XML with the global saved searches and their smartnames to a Map 
         * and a list to populate the SDrop.
         * @private
         * @param   {String} string - the xml returned from the external WCR
         * @returns {String} errors - the errros messages
         */
        WebcenterMetaDataPopupHandler.prototype._parseSavedSearchXML = function (string) {
            this.savedSearchMap = new Map();
            this.savedSearchList = [];

            var xml = $.parseXML(string);
            var elements = xml.getElementsByTagName("saved_search");
            if (elements) {
                for (var i = 0; i < elements.length; i++) {
                    var name = WCXML.GetElementSelfTextContent(elements[i].getElementsByTagName("name")[0]);
                    var smartnames = [];
                    var smartnameElements = elements[i].getElementsByTagName("smartname");
                    for (var j = 0; j < smartnameElements.length; j++) {
                        smartnames.push({
                            key: WCXML.GetElementSelfTextContent(smartnameElements[j])
                        }); //this is the format the WCRDataMapper expects to receive
                    }
                    this.savedSearchMap.add(name, {
                        name: name,
                        smartnames: smartnames
                    });
                    this.savedSearchList.push({
                        name: name,
                        id: name
                    });
                }
            }
        };

        /**
         * Removes the previous SDrop and creates a new one. Also a change listener is registered
         * @private
         */
        WebcenterMetaDataPopupHandler.prototype._configureSavedSearchSelect = function (value) {
            //remove previous sdrop
            var toRemove = this.sDropContainer.find('div');
            if (toRemove) {
                toRemove.remove();
            }
            var select = $('<select class="SavedSearchSDrop">');
            this.sDropContainer.append(select);

            //parse data and setup Map        
            var dataProvider = new StaticSDropDataProvider();
            dataProvider.setData(this.savedSearchList);
			var options = {
                manageStyleAndWidth: false,
				tabbable: false
            };
			
			if(value){
				options.initialValue = value;
			}
            this.savedSearchSDrop = new SDrop(select, null, dataProvider, options);

            //add listener
            select.change(_.bind(function (event) {
                var value = this.savedSearchSDrop.getSelectedItemID();
                var savedSearch = this.savedSearchMap.get(value);
                if (savedSearch) {
                    this.requestMapper.setMetadata(savedSearch.smartnames);
                }
            }, this));
        };

        return WebcenterMetaDataPopupHandler;
    });
