define(["underscore",
        "component/smartnames/smartfield/smartfield",
        "utils/Map",
        "utils/webi18n!MediaBeaconDocSourcePlugin",
        "damconnections/DataMapper",
        "component/smartnames/encoding/smartnameencoding",
        "component/sdrop/SDrop"
    ],
    function (_,
        Smartfield,
        Map,
        i18n,
        DataMapper,
        SmartNameEncoding,
        SDrop) {
        "use strict";

        /**
         * Component that creates rows for mapping WebCenter data to WebCenter data 
         * 
         * @class
         * @augments DataMapper
         * @exports WCRDataMapper
         *                  
         * @param {JQueryObj} container - Container in which the datamapper will reside
         * @param {DataMapper.types} type - determines which type of rows should be created
         * @param {JSONObj} fieldJSON - contains all namespaces and corresponding fields
         */
        var WCRDataMapper = function (container, type, attributesXML, smartNameDictionaryService) {
            DataMapper.call(this, container, type);
            /**
             * @member {String} the attributes of the external WCR 
             */
            this.attributesXML = attributesXML;
            /**
             * @member {Array} list of the attributes of the external WCR
             */
            this.externalAttributes = [];
            /**
             * @member {WorkflowSmartNameDictionaryService} dictionary service for smartfields in Workflow Node
             */
            this.smartNameDictionaryService = smartNameDictionaryService;

            this.load();
        };

        WCRDataMapper.prototype = Object.create(DataMapper.prototype);
        WCRDataMapper.prototype.constructor = WCRDataMapper;

        /**
         * @member {Object} Enum to indicate the possible rows
         */
        WCRDataMapper.types = DataMapper.types;

        /**
         * initializes some features of the dialog after loading the dialog
         * @param {object} metadata
         */
        WCRDataMapper.prototype.load = function () {
            if (this.attributesXML) {
                this._parseAttributesXML(this.attributesXML);
            }

            if (this.type === DataMapper.types.FETCH || this.type === DataMapper.types.PUSH) {
                //add listener to button (defined in template)
                this.addRowBtn = $(".addRowBtn", this.container);
                var self = this;
                this.addRowBtn.on("click",function(event){
                    if ($(this).is("[disabled]")) { //prevents clicking a disabled button for IE
                        event.preventDefault();
                    }else{
                        self._addRow();
                    }
                });
            }
        };

        /**
         * Have to be called when the popup closes.
         * It will destroy all sdrops.
         */
        WCRDataMapper.prototype.close = function () {
            if (this.type === DataMapper.types.FETCH || this.type === DataMapper.types.PUSH) {
                //destroy all sdrops
                this.elementsMap.each(function (id, row) {
                    row.key.destroy();
                    if (this.type === DataMapper.types.FETCH) {
                        row.value.destroy();
                    }
                }.bind(this));
            }
        };

        /**
         * Disables all elements.
         */
        WCRDataMapper.prototype.disable = function () {
            this.elementsMap.each(function (id, elements) {
                if (this.type === DataMapper.types.SEARCH) {
                    elements.key.prop('disabled', true);
                    elements.value.disable();
                } else if (this.type === DataMapper.types.FETCH) {
                    elements.internalAttribute.setDisabled(true);
                    elements.externalAttribute.setDisabled(true);
                    elements.deleteBtn.attr('disabled', 'disabled');
                }
                if (this.type === DataMapper.types.PUSH) {
                    elements.key.setDisabled(true);
                    elements.value.disable();
                    elements.deleteBtn.attr('disabled', 'disabled');
                }
            }.bind(this));
            if(this.addRowBtn){
                this.addRowBtn.attr('disabled', 'disabled');
            }
        };

        /**
         * Enables all elements.
         */
        WCRDataMapper.prototype.enable = function () {
            this.elementsMap.each(function (id, elements) {
                if (this.type === DataMapper.types.SEARCH) {
                    elements.key.prop('disabled', false);
                    elements.value.enable();
                } else if (this.type === DataMapper.types.FETCH) {
                    elements.internalAttribute.setDisabled(false);
                    elements.externalAttribute.setDisabled(false);
                    elements.deleteBtn.removeAttr('disabled');
                }
                if (this.type === DataMapper.types.PUSH) {
                    elements.key.setDisabled(false);
                    elements.value.enable();
                    elements.deleteBtn.removeAttr('disabled');
                }
            }.bind(this));
            if(this.addRowBtn){
                this.addRowBtn.removeAttr('disabled');
            }
        };

        /**
         * Returns the data that is inputted in the popup
         * @returns {object} metadata
         */
        WCRDataMapper.prototype.getMetaData = function () {
            var metadata = [];
            this.rows.each(function (id, value) {
                var item = {};
                var elements = this.elementsMap.get(id);

                if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                    item.key = (this.type === DataMapper.types.SEARCH) ? elements.key.val() : elements.key.getSelectedItemID();
                    item.value = elements.value.getValue();
                    if (item.key || item.value) { //ignore empty entries
                        metadata.push(item);
                    }
                } else if (this.type === DataMapper.types.FETCH) {
                    item.internalAttribute = elements.key.getSelectedItemID();
                    item.externalAttribute = elements.value.getSelectedItemID();
                    if (item.internalAttribute || item.externalAttribute) { //ignore empty entries
                        metadata.push(item);
                    }
                }
            }.bind(this));

            return metadata;
        };

        /**
         * Adds the new metadata to the popup
         * @param {object} data - metadata
         */
        WCRDataMapper.prototype.setMetadata = function (data) {
            DataMapper.prototype.setMetadata.call(this, data);
            //fill popup with new data
            if (data && !$.isEmptyObject(data) && data.length > 0) {
                if (this.type === DataMapper.types.SEARCH) {
                    $(".smartnameheader", this.body).show();
                }
            } else if (this.type === DataMapper.types.FETCH || this.type === DataMapper.types.PUSH) {
                //add default an empty row
                this._addRow();
            } else if (this.type === DataMapper.types.SEARCH) {
                $(".smartnameheader", this.body).hide();
            }
        };

        /**
         * Validates the input of the popup 
         * @return {Array.<String>} - error messages
         */
        WCRDataMapper.prototype.validateInput = function () {
            var counter = 0;
            var emptyRows = 0;
            var map = new Map();
            var errorMsg = "";

            if (this.rows) {
                this.rows.each(function (id, row) {
                    counter++;
                    var rowElements = this.elementsMap.get(id);

                    if (this.type === DataMapper.types.SEARCH || this.type === DataMapper.types.PUSH) {
                        var key = (this.type === DataMapper.types.SEARCH) ? rowElements.key.val() : rowElements.key.getSelectedItemID();
                        var value = rowElements.value.getValue();
                        if (!key && value) {
                            errorMsg += '\n' + i18n.t('DocSrcHdlDamRequestEmptyKey') + id;
                        }
                        //setup map for checking uniqueness of parameters
                        if (key && value) {
                            if (map.get(key)) {
                                var elements = map.get(key);
                                elements.push(rowElements.key);
                                map.add(key, elements);
                            } else {
                                map.add(key, [rowElements.key]);
                            }
                        } else {
                            emptyRows++;
                        }

                    } else if (this.type === DataMapper.types.FETCH) {
                        var internalAttribute = rowElements.key.getSelectedItemID();
                        var externalAttribute = rowElements.value.getSelectedItemID();
                        if ((!internalAttribute && externalAttribute) || (internalAttribute && !externalAttribute)) {
                            errorMsg += '\n' + i18n.t('DocSrcHdlDamMetaEmptyKey') + id;
                        }
                        //setup map for checking uniqueness of parameters
                        if (internalAttribute && externalAttribute) { //if not an empty row
                            var key = internalAttribute;
                            var highlightElements = [rowElements.key];
                            if (map.get(key)) {
                                var elements = map.get(key);
                                elements = elements.concat(highlightElements);
                                map.add(key, elements);
                            } else {
                                map.add(key, highlightElements);
                            }
                        } else {
                            emptyRows++;
                        }
                    }
                }.bind(this));

                //check if all parameters are unique
                if (map.size() < this.rows.size() - emptyRows) {
                    errorMsg += (this.type === DataMapper.types.SEARCH) ? '\n' + i18n.t('DocSrcHdlDamRequestDuplicateKey') : '\n' + i18n.t('DocSrcHdlDamMetaDuplicateKey');
                    map.each(function (key, value) {
                        $(value).removeClass('validationHighlight'); //could be added in previous validation phase
                        if (value.length > 1) {
                            $.each(value, function (index, value) {
                                $(value).addClass('validationHighlight');
                            });
                        }
                    });
                }
            }

            return errorMsg;
        };

        /**
         * Adds a row to the popup for metadata of a DAM connection
         * @private
         * @param {node} body  - the container
         * @param {Object}   type  
         * @param {String} key   
         * @param {String} value 
         */
        WCRDataMapper.prototype._addRow = function (item) {
            var row = $("<div>").addClass("row add-padding-bottom");

            if (this.type === DataMapper.types.SEARCH) {
                var key = (item && item.key) ? item.key : null;
                var inputfield = this._addInputfield(row, false, key);

                var value = (item && item.value) ? item.value : null;
                var smartfield = this._addInputfield(row, true, value);

                this.body.append(row);
                this.rows.add(this.rowId, row);
                this.elementsMap.add(this.rowId, {
                    key: inputfield,
                    value: smartfield
                });
                this.rowId++;

            } else if (this.type === DataMapper.types.FETCH) {
                var internalAttribute = (item && item.internalAttribute) ? item.internalAttribute : null;
                var interalAttributeSDrop = this._addSDropWithAttributes(row, true, internalAttribute);
                var externalAttribute = (item && item.externalAttribute) ? item.externalAttribute : null;
                var externalAttributeSDrop = this._addSDropWithAttributes(row, false, externalAttribute);

                var deleteBtn = this._addDeleteButton(row);

                this.body.append(row);
                this.rows.add(this.rowId, row);
                this.elementsMap.add(this.rowId, {
                    key: interalAttributeSDrop,
                    value: externalAttributeSDrop,
                    deleteBtn: deleteBtn
                });
                this.rowId++;
            } else if (this.type === DataMapper.types.PUSH) {
                var externalAttribute = (item && item.key) ? item.key : null;
                var externalAttributeSDrop = this._addSDropWithAttributes(row, false, externalAttribute);
                var value = (item && item.value) ? item.value : null;
                var smartfield = this._addInputfield(row, true, value);

                var deleteBtn = this._addDeleteButton(row);

                this.body.append(row);
                this.rows.add(this.rowId, row);
                this.elementsMap.add(this.rowId, {
                    key: externalAttributeSDrop,
                    value: smartfield,
                    deleteBtn: deleteBtn
                });
                this.rowId++;
            }
        };

        /**
         * Adds an inputfield to the row
         * @private
         * @param {node} row  - the container
         * @param {Boolean} asSmartfield - indicating if it has to be a smartfield or not   
         * @param {String} value 
         */
        WCRDataMapper.prototype._addInputfield = function (row, asSmartfield, value) {
            var column = $("<div>").addClass("col-sm-6");
            var inputfield = $("<input/>", {
                'class': "form-control",
                type: "text"
            })

            if (this.type === DataMapper.types.SEARCH && !smartfield) {
                inputfield.val(value);
                inputfield.prop('disabled', true);
            }

            row.append(column.append(inputfield));

            if (asSmartfield) {
                var smartfield = null;
                if (this.type === DataMapper.types.PUSH) { // Workflow context
                    smartfield = new Smartfield(inputfield,
                        false, // lazy init
                        this.smartNameDictionaryService, // our dictionary
                        _.bind(this.smartNameDictionaryService.displaySmartName, this.smartNameDictionaryService), // display in input field
                        _.bind(this.smartNameDictionaryService.displaySmartNameSelectorSmartName, this.smartNameDictionaryService), // display in selector
                        SmartNameEncoding.WORKFLOWEDITOR // encoding
                    );
                } else {
                    smartfield = new Smartfield(inputfield);
                }
                smartfield.setWidth("100%");
                if (value) {
                    smartfield.setValue(value);
                }
                return smartfield;
            } else {
                return inputfield;
            }
        };

        /**
         * Adds an sdrop to the row
         * @private
         * @param {node} row  - the container
         * @param {Boolean} withInternalAttributes - a boolean that defines what the content of the sdrop have to be   
         * @param {String} value 
         */
        WCRDataMapper.prototype._addSDropWithAttributes = function (row, withInternalAttributes, value) {
            var column = (this.type === DataMapper.types.FETCH && !withInternalAttributes) ? $("<div>").addClass("col-sm-6") : $("<div>").addClass("col-sm-5");
            var select = $("<select class='form-control'>");
            row.append(column.append(select));

            //configure SDrop
            var options = {
                manageStyleAndWidth: false,
				tabbable: false
            };
            if (value) {
                options.initialValue = value;
            }

            var dataProvider = null;
            if (withInternalAttributes) {
                dataProvider = new SDropDataProvider({
                    'dataSource': "GetNameIDList.jsp?list=Attributes"
                });
            } else {
                dataProvider = new StaticSDropDataProvider();
                dataProvider.setData(this.externalAttributes);
            }

            return new SDrop(select, null, dataProvider, options);

        };

        /**
         * Adds a delete button to the row
         * @private
         * @param {node} row 
         */
        WCRDataMapper.prototype._addDeleteButton = function (row) {
            //column 3
            var self = this;
            var id = this.rowId;
            var btn = $("<a/>")
                .addClass("btn btn-default btn-sm")
                .attr('title', i18n.t("DocSrcHdlDamBtnDeleteTooltip"))
                .on("click", function(event){
                    if ($(this).is("[disabled]")) { //prevents clicking a disabled button for IE
                        event.preventDefault();
                    }else{
                        self._deleteRow(id);
                    }
                });
            
            btn.append($("<i/>", {
                'class': "icon icon-WCR_Actions-Delete12 icon-normal"
            }));

            row.append(btn);
            return btn;
        };

        /**
         * Parses the XML with attributes from the external WCR to a list 
         * @private
         * @param   {String} string - xml
         * @returns {Array} - parsed xml
         */
        WCRDataMapper.prototype._parseAttributesXML = function (string) {
            var xml = $.parseXML(string);
            var elements = xml.getElementsByTagName("entry");
            if (elements) {
                this.externalAttributes = [];
                for (var i = 0; i < elements.length; i++) {
                    this.externalAttributes
                        .push({
                            name: WCXML
                                .GetElementSelfTextContent(elements[i]),
                            id: elements[i].getAttribute("id")
                        });
                }
            }
        };

        return WCRDataMapper;
    });
