define(["underscore",
	"jquery",
	"jqueryUI",
	"jqueryUITouchPunch" // enable touch interaction
], function (_, $, JQueryUI, JQueryUITouchPunch) {
	"use strict";

	/**
	 * Handler for DragAndDrop in the Builder 
	 * @constructor
	 * @class
	 * 
	 **/
	var DragDropHandler = function DragDropHandler() {
		/**
		 * @member {Object} the currently active dropzone (needed to implement "greedy" behaviour in a correct way
		 */
		this._activeDropzone = null;

		/**
		 * List of available dropzones
		 */
		this._dropzones = [];
	};


	/**
	 * Enable dragging for an element
	 * @param {Boolean}	                 enable  	       - enable/disable dragging
	 * @param {Context}	                 context
	 * @param {BlockDefinition,Function} blockDefinition - definition of the block that is being dragged, if function a function to generate the BlockDefintion (called with the ui.draggeble element as context)
	 * @param {Block}                    currentParent	  - block that is currently the parent of the block will be dragged
	 * @param {JQueryObj}                element         - element that should be draggable
	 * @param {JQueryObj,selector}       [handle]        - draggable handle  selector or jqueryObj
	 * @param {Function}                 [start]	        - change the start behavior of the draggable (you can i.e. modify the block definition on start of drag or do other things)
	 * @param {Function}                 [stop]	         - change the stop behavior of the draggable                        
	 */
	DragDropHandler.prototype.toggleDragging = function (enable, context, blockDefinition, currentParent, element, handle, start, stop) {
		if (enable) {

			// we inject the context in the element so we can check in the dropzone whether the element being dragged is of the same context
			element.data("DragDropHandlerContext", context);

			var self = this;
			element.draggable({
				handle: handle,
				refreshPositions: true,
				helper: function () {
					if (_.isFunction(blockDefinition)) {
						blockDefinition = blockDefinition(this);
					}
                    
                    var blockTitle = (blockDefinition.block) ? blockDefinition.block.getTitle() : blockDefinition.getDisplayName();
                    
					var helper = $("<h4>", {
						text: blockTitle
					}).css({
						background: '#fefefe',
						border: "1px solid #ccc",
						padding: "10px",
						margin: 0
					}); // TODO REMOVE INLINE STYLING				

					//to center the handler evenly around the cursor 
					var clone = helper.clone().appendTo(document.body).css('position', "absolute");
					element.draggable("option", "cursorAt", {
						right: Math.floor(clone.outerWidth() / 2),
						bottom: Math.floor(clone.outerHeight() / 2)
					});
					clone.remove();

					helper.data('CurrentParent', currentParent);
					helper.data('BlockDefinition', blockDefinition);
					return helper;

				},
				drag: function (event, ui) {
					if (self._activeDropzone && self._activeDropzone.handlers.drag) {
						self._activeDropzone.handlers.drag.apply(self._activeDropzone.binding || this, arguments);
					}
				},
				start: function (event, ui) {
					if (!start) {
						$(this).addClass("hidden");
					} else {
						start.apply(this, arguments);
					}
				},
				stop: function (event, ui) {
					if (!stop) {
						var draggable = $(this);
						if (ui.helper.data("isValidDrop")) {
							// wait till the drop is completed
							ui.helper.data("isValidDrop").then(function () {
								draggable.removeClass("hidden");
							});
						} else {
							draggable.removeClass("hidden");
						}

					} else {
						stop.apply(this, arguments);
					}
				},
				scrollSensitivity: 50

			});
		} else {
			element.filter(".ui-draggable").draggable("destroy").data("DragDropHandlerContext", null);
		}
	};

	/**
	 * Registers an element as dropzone
	 * @param   {Block} block - The block calling for the creation of a dropzone, used to get the context
	 * @param   {JQueryObject} dropzoneElement - The dropzone element
	 * @param   {object}   handlers 
	 * @param   {[[Type]]} binding 
	 */
	DragDropHandler.prototype.registerDropzone = function (block, dropzoneElement, handlers, binding) {
		var dropzone = {
			block: block,
			element: dropzoneElement.get(0),
			handlers: handlers,
			binding: binding,
			over: false
		};
		var self = this;
		this._dropzones.push(dropzone);

		dropzoneElement.droppable({
			accept: function (element) {
				if (block.getContext() === element.data("DragDropHandlerContext")) {
					if (handlers.accept) {
						return handlers.accept.apply(binding || this, arguments);
					} else {
						return true;
					}
				} else {
					return false;
				}
			},
			over: function (event, ui) {
				// mark the dropzone to be "hovered"
				dropzone.over = true;

				if (event.target === dropzone.element) {
					// only handle over event when the target is really the element of this dropzone

					if (self._activeDropzone && self._activeDropzone != dropzone) {
						// send the OUT event to the previous active dropzone

						if (self._activeDropzone.handlers.out) {
							self._activeDropzone.handlers.out.apply(self._activeDropzone.binding || this, arguments);
						}
					}
					// set this dropzone as the currently active dropzone 
					self._activeDropzone = dropzone;

					if (handlers.over) {
						// give the dropzone the OVER event
						handlers.over.apply(binding || this, arguments);
					}
				}
			},
			out: function (event, ui) {
				// the dropzone is no longer hovered
				dropzone.over = false;

				// a new dropzone might become active, for now we don't know which zone it will be
				var newActiveDropzone = null;

				if (self._activeDropzone === dropzone) {
					// handle the OUT event only if we are the active dropzone					
					self._activeDropzone = null;
					if (handlers.out) {
						// pass the OUT Event
						handlers.out.apply(binding || this, arguments);
					}

					// find the closest parent that still might be hovered
					var parents = $(event.target).parents(".ui-droppable");
					var activeDropzones = _.where(self._dropzones, {
						over: true
					});
					var i = 0;
					while (i < parents.length && !newActiveDropzone) {
						newActiveDropzone = _.findWhere(activeDropzones, {
							element: parents[i]
						});
						i++;
					}
				}

				if (newActiveDropzone) {
					//a new active dropzone was found!
					self._activeDropzone = newActiveDropzone;

					if (self._activeDropzone.handlers.over) {
						// make sure to call the OVER event on the new active dropzone
						self._activeDropzone.handlers.over.apply(self._activeDropzone.binding || this, arguments);
					}
				}
			},
			drop: function (event, ui) {
				// whenever a drop event occurs, no dropzone is hovered any more
				dropzone.over = false;

				if (self._activeDropzone === dropzone) {
					// pass the DROP event only on the active dropzone	
					var dropResult;
					if (handlers.drop) {
						dropResult = handlers.drop.apply(binding || this, arguments);
					}

					ui.helper.data("isValidDrop", Promise.resolve(dropResult));


					self._activeDropzone = null;
				}

			},
			activate: function (event, ui) {

				if (handlers.activate) {
					handlers.activate.apply(binding || this, arguments);
				}

			},
			// since there is a bug in JQueryUI.droppable greedy functionality, we implemented it ourselves with the activedropzone mechanism
			greedy: false,
			tolerance: "touch"
		});

	};

	/**
	 * Remove the droppable functionality from the dropzoneElement
	 * @param {JQueryObject} dropzoneElement 
	 */
	DragDropHandler.prototype.removeDropzone = function (dropzoneElement) {
		if (dropzoneElement.is(".ui-droppable")){
			dropzoneElement.droppable("destroy");
		}
		
		this._dropzones = _.filter(this._dropzones, function (dropzone) {
			return dropzone.element !== dropzoneElement.get(0);
		});
			
	};

	return DragDropHandler;
});
