define(["../../core/Block",
		"jquery",
		"Template",
		"tpl!./GalleryItemList",
		"underscore",
		"utils/webcss!./Gallery.css",
	],
	function (Block,
		$,
		Template,
		itemlistTpl, _, css) {

		"use strict";

		/**
		 * A Gallery block is used to display a (searchable) list of available items 
		 * @augments Block
		 * @class
		 * @constructor
		 * @param {String} id
		 * @param {Context} context
		 */
		var Gallery = function (id, context) {
			Block.call(this, id, context);
			
			this._element.addClass("dshBlockGallery");
		};

		Gallery.prototype = Object.create(Block.prototype);
		Gallery.prototype.constructor = Gallery;

		/**
		 * @Override
		 */
		Gallery.prototype.getType = function () {
			return "dashboard/builder/gallery/Gallery";
		};

		/**
		 * @Override
		 */
		Gallery.prototype._doInit = function () {
			var self = this;
			return Promise.resolve()
				.then(function () {
					return self._context.getLanguageBundle("DashboardGallery");
				})
				.then(function (bundle) {
					self._i18n = bundle;
					self.t = new Template(self._i18n, itemlistTpl);


					self._filteredItems = [];
					self.t.d.getItems = function () {
						return self._filteredItems;
					};

					self.t.d.getActiveClass = function (blockDefinition) {
						return "";
					};

					self.$filtericon = $("<i>", {
						'class': "icon icon-WCR_Search"
					});


					self.$filter = $("<input>", {
						'class': "form-control",
						type: "text"
					}).on("change keyup", _.debounce(_.bind(self._onFilterChanged, self), 100));

					self.$category = $("<select>", {
						'class': "form-control"
					}).addClass("hiddenElement").on("change", function () {
						self._updateResultList();
						self.$filter.val("").trigger("change");
					});

					self.$filterbutton = $("<button>", {
						type: "button",
						'class': "btn"
					}).append(self.$filtericon);


					var inputGroupAddon = $("<span>", {
						'class': "input-group-btn"
					}).append(self.$filterbutton).click(function () {
						self.$filter.val("").focus().trigger("change");
					});

					self._element.append(self.$category).append($("<div>", {
						'class': "input-group"
					}).append(self.$filter).append(inputGroupAddon));

					self.$list = $("<div>").addClass("results").appendTo(self._element);
					self.t.setContext(self.$list);

					self._context.getBlockDefinitionDataSource().then(function (datasource) {
						self._observe(datasource, "changed.blockDefinitions", function () {
							self._updateCategoryDropdown();
							self._updateResultList();
						});
						self._updateCategoryDropdown();
						self._updateResultList();
					});

					self._render();


				});



		};
		/**
		 * React on changes to the filter state
		 */
		Gallery.prototype._onFilterChanged = function () {
			var hasCharacters = this.$filter.val().length > 0;
			this.$filtericon.toggleClass("icon-WCR_Search", !hasCharacters).toggleClass("icon-WCR_Remove", hasCharacters);
			this.$filterbutton.attr("title", hasCharacters ? this._i18n.t("GALLERY_CLEAR_FILTER") : this._i18n.t("GALLERY_FILTER"));

			this._updateResultList();
		};

		/**
		 * Update the dropdown with category information
		 */
		Gallery.prototype._updateCategoryDropdown = function () {
			// update the categories dropdown
			this._context.getBlockDefinitionDataSource().then(function (datasource) {
				return datasource.getCategories();
			}).then(function (categories) {
				this.$category.toggleClass("hiddenElement", categories.length <= 1);
				var currentValue = this.$category.val();

				this.$category.empty();
				this.$category.append(
					$("<option>", {
						value: "",
						text: this._i18n.t("GALLERY_ALL_CATEGORIES")
					})
				);
				_.each(categories, function (category) {

					this.$category.append(
						$("<option>", {
							value: category,
							text: this._i18n.tFallback("CATEGORY_"+category, category)
						})
					);
				}, this);
				this.$category.val(currentValue);

			}.bind(this));
		};


		/**
		 * Update the result lists based on the current filter
		 */
		Gallery.prototype._updateResultList = function () {
			if (this._cancelPendingUpdate) {
				this._cancelPendingUpdate();
			}
			var isCanceled = false;
			var cancelError = new Error("CANCEL ERROR");
			this._cancelPendingUpdate = function () {
				isCanceled = true;
			};

			this._context.getBlockDefinitionDataSource().then(function (datasource) {
				return datasource.filterBlocks(this.$category.val(), this.$filter.val());
			}.bind(this)).then(function (definitions) {
				if (isCanceled) {
					throw cancelError;
				}

				return _.sortBy(definitions, function (definition) {
					return definition.getDisplayName();
				});
			}).then(function (items) {
				if (isCanceled) {
					throw cancelError;
				}

				// display
				this._filteredItems = items;
				this._render();
			}.bind(this)).catch(function (error) {
				if (error !== cancelError) {
					throw error;
				}
			});
		};



		/**
		 * @Override
		 * @private
		 */
		Gallery.prototype._doRender = function () {
			if (!this._blockRendering){
				// we only want to re render when no dragging operation is ongoing... 
				
				var self = this;
				Promise.resolve().then(function () {
					if (self._draggables) {
						// if there where draggables, make sure to destroy the draggable behavior
						return self._context.getDragDropHandler().then(function (dragDropHandler) {
							dragDropHandler.toggleDragging(false, null, null, null, self._draggables);
							self._draggables = null;
						});

					}

				}).then(function () {
					// render the template
					self.t.render();

					if (self._context.isEditModeActive()) {
						// we need to enable drag/drop
						var list = self._filteredItems.slice(0);
						
						// we want to create only one block definition
						var newBlockDefinition = null;
						var getBlockDefinition = function(element){
							if (!newBlockDefinition){
								var definition = list[parseInt($(element).attr("data-index"), 10)];
								var newID = self._context.registerBlockFromDefinition(definition);
								newBlockDefinition = self._context.getBlockDefinition(newID);
							} 
							return newBlockDefinition;
						};

						self._context.getDragDropHandler().then(function (dragDropHandler) {
							self._draggables = self._element.find("[data-index]");
							dragDropHandler.toggleDragging(self._context.isEditModeActive(),
								self._context,
								function (element) {
									return getBlockDefinition(element);
								}, null, self._draggables, null,
								function (event, ui) {
									self._blockRendering = true;
									// block rendering, so that the dragging can be completed without any problems
								
									// on start of drag, make the sure to inject the correct block definition in the ui.helper
									var blockDef = getBlockDefinition(this);
									ui.helper.text(blockDef.getDisplayName());
									ui.helper.data("BlockDefinition", blockDef);
								},
								function (event, ui) {
									// if the draggable was dropped on an invalid location, remove the definition from the context
									if (!ui.helper.data("isValidDrop")) {
										// we need to remove the definition from the context
										self._context.removeBlockDefinition(ui.helper.data("BlockDefinition"));
										
									}
								
									// make sure we don't reuse this block definition anymore
									newBlockDefinition = null;
								
									
								
									// the widget can be rendered again, trigger a render
									self._blockRendering = false;
									self._render();
								});
						});
					}
				});
			}


		};

		return Gallery;
	});