define(["jquery", "underscore"], function ($, _) {
	"use strict";


	/**
	 * Flex Item, data object to represent a Column in a Flex Layout
	 * @class
	 * 
	 * @param {FlexLayout} layout
	 * @param {Number} ID - number of the column
	 * @param {Object} columnConfig                   
	 */
	var FlexItem = function (layout, ID, columnConfig) {
		this._layout = layout;
		this._columnID = ID;

		this._layout._observeParam("CssSizeParameter", function () {
			this._applyColumnStyles(this._element);
			this._layout._render();
		}.bind(this));

		this._blockID = null;
		this._blockPromise = null;
		this._block = null;
		this._element = null;
		this._isPlaceholder = null;

		this.configure(columnConfig);
		this._layout._initializePlaceholder(this);
	};

	/**
	 * Set the ID of the FlexItem (sequence number)
	 * @param {Number} ID 
	 */
	FlexItem.prototype.setID = function (ID) {
		if (this._columnID != ID) {
			this._columnID = ID;
			if (this._element) {
				this._applyColumnStyles(this._element);
			}
		}
	};

	/**
	 * Get the ID of the FlexItem (sequence number)
	 * @returns {Number} ID 
	 */
	FlexItem.prototype.getID = function () {
		return this._columnID;
	};

	/**
	 * Update the configuration of the FlexItem
	 * @param {object} columnConfig 
	 */
	FlexItem.prototype.configure = function (columnConfig) {
		// we are only interested in a subset of configuration keys
		this._config = _.pick(columnConfig, 'fitContent', 'grow', 'shrink', 'width', 'height', 'maxWidth', 'minWidth', 'lockedColumn');

		if (columnConfig.blockid) {
			this.setItem(columnConfig.blockid);
		} else {
			this.setItem(null);
		}

		if (this._element) {
			this._applyColumnStyles(this._element);
		}
	};

	/**
	 * Set the item for this column
	 * @param {String}  id           - ID of the item
	 * @param {Promise.<Block>} blockPromise - promise for the item
	 */
	FlexItem.prototype.setItem = function (id) {
		if (this._blockID !== id) {
			// detach current item
			if (this._block) {
				var oldBlock = this._block;
				this._block = null;
				this._layout.detachChild(oldBlock);
			}

			if (!id) {
				this._blockID = null;
				this._blockPromise = null;
				this._block = null;
				// this is an empty column, fill with placeholder
				this._layout._initializePlaceholder(this);

			} else {
				this._layout._context.getDragDropHandler().then(
					function (dragDropHandler) {
						//remove the droppable functionality when a placeholder is replaced
						if (this._isPlaceholder) {
							dragDropHandler.removeDropzone(this.getElement());
							this._isPlaceholder = null;
						}
					}.bind(this));

				this._blockID = id;
				
				var	whenFutureBlockAvailable =	this._layout._registerFutureChild(this._blockID);
 
				if (this._config.lockedColumn) {
					this._blockPromise = this._layout.getContext().loadBlock(this._blockID);
				} else {
					this._blockPromise = this._layout.loadBlockWrapper(this._blockID);
				}

				this._blockPromise.then(function (block) {
					if (this._blockID === id) {
						// we still want this block to be in the column, do the replace
						this._replaceElement(block.getElement());

						this._layout._registerChild(block);
						this._block = block;
					}
					
					whenFutureBlockAvailable();
				}.bind(this));
			}

		}

	};


	/**
	 * Get the promise of the block that is being displayed in the column
	 * @returns {Promise.<Block>} 
	 */
	FlexItem.prototype.getBlockPromise = function () {
		return this._blockPromise;
	};

	/**
	 * Replaces the current element with a paceholder element.
	 * @param {JQueryObject} ghost - the placeholder element
	 */
	FlexItem.prototype.makePlaceholder = function (ghost) {
		this._replaceElement(ghost);
		this._isPlaceholder = true;
	};

	/**
	 * Replace the current element with another element
	 * @private
	 * @param {JQueryObject} newElement - element
	 */
	FlexItem.prototype._replaceElement = function (newElement) {
		if (this._element) {
			// in the dom already, replace it
			this._element.before(newElement).detach();
		}


		this._element = newElement;
		this._applyColumnStyles(this._element);

	};

	/**
	 * Get the configuration of the Column
	 * @returns {Object} configuration
	 */
	FlexItem.prototype.getConfiguration = function () {
		var config = _.pick(this._config, 'fitContent', 'grow', 'shrink', 'width', 'height', 'maxWidth', 'minWidth', 'lockedColumn');
		if (this._blockID) {
			config.blockid = this._blockID;
		}
		return config;
	};

	/**
	 * Get the element of the Column
	 * 
	 * @returns {JQueryObj} 
	 */
	FlexItem.prototype.getElement = function () {
		if (!this._element) {
			// no element set, we just create a dummy element
			this._replaceElement($("<div>"));
		}
		return this._element;
	};

	/**
	 * Get the ID of the block in the column
	 * @returns {String} 
	 */
	FlexItem.prototype.getBlockID = function () {
		return this._blockID;
	};


	/**
	 * Apply column styles
	 */
	FlexItem.prototype._applyColumnStyles = function (element) {

		element.attr("data-column", this._columnID);

		var elemclasswidthdecided = true;
		var elemclassfittocontent = false;

		var cssStyles = {
			'-webkit-flex': this._config.grow + " " + this._config.shrink + " " + "auto",
			'-ms-flex': this._config.grow + " " + this._config.shrink + " " + "auto",
			'flex': this._config.grow + " " + this._config.shrink + " " + "auto"
		};
		
		var isEmpty = !this._blockID && this._layout._context.isEditModeActive();

		//indicate placeholder if the column is empty
		if (isEmpty) {
			cssStyles.border = "1px dotted #ccc";
			cssStyles.padding = "10px";
			cssStyles.margin = "5px";
		}

		if (this._config.width) {
			cssStyles.width = this._config.width;
		} else if (this._config.grow > 0 && this._config.shrink > 0 && !this._config.fitContent) {
			// IE fix ...
			cssStyles.width = "100%";
			elemclasswidthdecided = false;
		} else if (this._config.fitContent) {
			cssStyles.width = "auto";
			elemclassfittocontent = true;
			elemclasswidthdecided = false;
		}

		if (this._config.height) {
			cssStyles.height = this._config.height;
		}

		if (this._config.maxWidth) {
			cssStyles['max-width'] = this._config.maxWidth;
		}

		if (this._config.minWidth) {
			cssStyles['min-width'] = this._config.minWidth;
		} else {
			cssStyles['min-width'] = 0;
		}


		var margin = this._layout._parameters.getParameter("CssSizeParameter").getValue();
		if (this._columnID > 0) {
			cssStyles['margin-left'] = margin;
		} else {
			if (isEmpty){
				cssStyles['margin-left'] = "5px";
			} else {
				cssStyles['margin-left'] = "";
			}
			
			
		}

		element.css(cssStyles);
		
		// Tell the items in this column that the column width has been decided
		// They need to adapt to fit
		// Mainly used for the design panel & the results table
		element.toggleClass("columnwidthdecided", elemclasswidthdecided);
		element.toggleClass("columnwidthfittocontent", elemclassfittocontent);

	};

	/**
	 * Get the block which is assigned to the column
	 * @returns {Block} 
	 */
	FlexItem.prototype.getBlock = function () {
		return this._block;
	};

	/**
	 * get the layout to which this flex items belongs
	 * @return {FlexLayout}
	 */
	FlexItem.prototype.getLayout = function () {
		return this._layout;
	};

	return FlexItem;
});