define(["./FlexItem",
		"utils/Promisify",
		"../../core/Layout",
		"underscore",
	   	"./FlexLayoutBlockWrapper",
        "dashboard/core/parameters/CssSizeParameter",
        "dashboard/layout/flex/FlexColumnParameter"],
	function (FlexItem,
		Promisify,
		Layout,
		_,
		FlexLayoutBlockWrapper,
        CssSizeParameter,
        FlexColumnParameter) {
		"use strict";

		/**
		 * Flexbox layout
		 * @augments Layout
		 * @param {String}  id      
		 * @param {Context} context 
		 */
		var FlexLayout = function (id, context) {
			Layout.call(this, id, context);

			this._element.attr('style', 'display: -ms-flexbox; display: flex');

			this._observe(this, "child.detach", this._onChildDetached);
		};

		FlexLayout.prototype = Object.create(Layout.prototype);
		FlexLayout.prototype.constructor = FlexLayout;

		/**
		 * @Override
		 */
		FlexLayout.prototype.getType = function () {
			return "dashboard/layout/flex/FlexLayout";
		};

		/**
		 * @override
		 */
		FlexLayout.prototype._doConfigure = function (config, IDMap) {
			// in builder mode, we want horizontal scrolling if not enough space
			// however, in view mode, it should be disabled since this is causing issues with some widgets (i.e. facet diagram)
			this._element.css("overflow-x", this._context.isEditModeActive()?"auto":"");
            
            var flexItems = this._columnParameter.getItemList();
            for(var newSetupPos = 0; newSetupPos < flexItems.length; newSetupPos++){
                var flexItem = flexItems[newSetupPos];
                this._element.append(flexItem.getElement());
            }
            
            var toBeDeletedItems = this._columnParameter.getToBeDeletedItems();

			for(var i = 0; i < toBeDeletedItems.length; i++){
                var item = toBeDeletedItems[i];

				item.getElement().detach();
				if (item.getBlockPromise()) {
					item.getBlockPromise().then(function (block) {
						block.setParent(null);
					});
				}
            }

		};

		/**
		 * @override
		 */
		FlexLayout.prototype._loadBlockWrapper = function (itemID) {
			var promise = this._context.registerAndLoadBlock({
				type: "dashboard/layout/flex/FlexLayoutBlockWrapper",
				config: {
					blockID: itemID
				}
			});		
			return promise;
		};

		/**
		 * @override
		 */
		FlexLayout.prototype._doInit = function () {
			var self = this;
			self._observe(self._context, "editModeActive", self._initializeDropzone);
			self._initializeDropzone();
            self._observeParam("FlexColumnParameter", function(){
                self._columnParameter.getConfiguration().then(function(configuration){
                    self._doConfigure(configuration.config);
                    self._render();
                });
            });
		};


		/**
		 * Replace a certain column with a Block
		 * @param {Number} columnNR - column number which should be replaced (starts at 0)
		 * @param {String} blockID  - id of the block which should be placed in the column	
		 */
		FlexLayout.prototype.replaceColumnWithBlock = function (columnNR, blockID) {
			this._columnParameter.setItem(columnNR, blockID);
		};

		/**
		 * React on a detached child
		 * @Override
		 */
		FlexLayout.prototype._onChildDetached = function (child) {
			_.each(this._columnParameter.getItemList(), function (column) {
				if (column.getBlock() === child) {
					column.setItem(null);
				}
			});
            this._columnParameter.update();
		};

		/**
		 * Publicly accessible loadblockwrapper function
		 * @param   {String}                 blockID - id of the block for which a wrapper should be loaded
		 *                                           @returns {Promise.<BlockWrapper>} 
		 */
		FlexLayout.prototype.loadBlockWrapper = function (blockID) {
			return this._loadBlockWrapper(blockID);
		};

		/**
		 * Initialize the dropzone
		 * @private
		 */
		FlexLayout.prototype._initializeDropzone = function () {
			if (this._context.isEditModeActive()) {

				Promisify.require("dashboard/layout/flex/FlexLayoutDragDrop").then(function () {
					_.each(this._columnParameter.getItemList(), function (column) {
						this._initializePlaceholder(column);
					}.bind(this));
				}.bind(this));

			}
		};


		/**
		 * Get the FlexItem corresponding to the block id
		 * @param {String} id 
		 */
		FlexLayout.prototype.getFlexItemForID = function (id) {
			return _.find(this._columnParameter.getItemList(), function (column) {
				return column.getBlockID() === id;
			});
		};

		/**
		 * Function that will be overridden by _initializePlaceholder from FlexLayoutDragDrop.js once the file is loaded.
		 * @private
		 * @param {FlexItem} column 
		 */
		FlexLayout.prototype._initializePlaceholder = function (column) {};
    
    
        /**
		 * Configure the default parameters of the block
		 * @override
		 */
		FlexLayout.prototype._initializeDefaultParameters = function () {
            return this._context.getLanguageBundle("FlexLayout").then(function(i18n){
                var spacingParameter = new CssSizeParameter("CssSizeParameter", this._context, i18n.t("SPACING"));
                spacingParameter.setValue("0px");
                this._addParameter(spacingParameter);

                this._columnParameter = new FlexColumnParameter("FlexColumnParameter", this._context, this, "");
                this._addParameter(this._columnParameter);
            }.bind(this));
		};

		return FlexLayout;


	});
