define(["doctablex/DocTableXUtils",
        "doctablex/DocumentTypeUtils",
        "attributecategory/Attribute",
        "wcr/webcenter_xml",
        "wcr/webcenter_html",
        "wcr/ObjectArrayCloneUtil",
		"utils/Cache",
        "jquery",
        "underscore"],
	function (DocTableXUtils,
		DocumentTypeUtils,
		Attribute,
		WCXML,
		WCHTML,
		ObjectHandler,
		Cache,
		$,
		_) {
		/**
		 * AJAXDataCache.js
		 * 
		 * Global singleton object providing lazy loading via AJAX and cache support for AJAX enabled pages 
		 * e.g. document reference attributes, upload multiple docs., attribute category layout, etc.
		 * Should be used (extended) for data that can be considered constant during the life of a particular
		 * page (e.g. list of projects, templates, folders, document saved searches, etc.).
		 *
		 * There is no support for aging of cache data implemented yet. It will be added if needed. 
		 * 
		 * @author JANH - September 2014
		 */

		"use strict";

		var AJAXDataCache = (function () {
			/**
			 *  private variable to hold the only instance of the AJAXDataCache class
			 */
			var _instance = null;

			/** 
			 * A simple singleton "empty object" for the AJAXDataCache purposes. It's used when the cache 
			 * entry was loaded successfully, but the cached object doesn't exist.
			 * Never change its content!
			 */
			var cEmptyObject = {
				IAmEmptyObject : true
			};


			//////////////////////////////////////////////////////////////////////
			// DATA THAT CAN BE STORED IN THE CACHE
			//////////////////////////////////////////////////////////////////////

			/** @member {Promise, Object} _projCreationSysDefaults - Object holding system-level document type 
			 * 		settings (including autonaming and default attribute category).
			 */
			//JANH TODO add the detailed description of "Object" here as the structure is known!
			var _projCreationSysDefaults = {};


			/** @member {Array[projectID].{Promise, Object}} _docTypeSettingsByProjID - Associative array of 
			 * 		objects holding project-level document type settings (including autonaming and default 
			 * 		attribute category) indexed by corresponding project ID.
			 */
			var _docTypeSettingsByProjID = [];


			var _docTemplatesByProjID = [];


			/** @member {Array[projectID].{Promise, Object}} _folderByProjectID - Array of 
			 * 		folders where the logged in user can upload documents to.  		
			 *  	Each element has the folder id and name. 
			 */
			var _folderByProjectID = [];

			/** @member {Array[projectID].{Promise, Object}} _docversionsByProjectID - Array of 
			 * 		docversions that are in the current project 
			 */
			var _docversionsByProjectID = [];


			/** @member {Array[docVersionID].{Promise, Object}} _docInfoBydocVerID - Associative array of 
			 * 		objects holding document version info indexed by IDs of the corresponding doc. versions.
			 */
			var _docInfoBydocVerID = [];


			/** @member {Array[docVersionID].{Promise, Object}} _attrCatInfoByDocVerID - Associative 
			 * 		array of objects holding basic info about attribute category of a document. It's indexed by 
			 * 		the document version IDs. If result === cEmptyObject, the document has no attribute
			 * 		category assigned, if result == null, the result is not loaded yet.
			 */
			var _attrCatInfoByDocVerID = [];

			/** @member {Promise} _globalDocumentSettings - object containing global upload settings (site defaults for upload)
			 */
			var _globalDocumentSettings = null;

			/** @member {Array[docVersionID].{Promise, Object}} _downloadPerspectivesByDocVerID - Associative 
			 * 		array of objects holding download perspectives of a document. It's indexed by 
			 * 		the document version IDs. If result === [], the document has no perspectives, 
			 * 		if result == null, the result could not be loaded.
			 */
			var _downloadPerspectivesByDocVerID = [];

			/** @member {Array[AttributeName].{Promise, Object}} _attributesByName - Associative 
			 * 		array of objects holding Attributes. It's indexed by the document attribute IDs. 
			 * 		If result === [], the attribute was not found, if result == null, the result could not be loaded.
			 */
			var _attributesByName = [];
			var _requestedAttNames = [];

			/** @member {Promise.Object} _projectStatusesPromise - A promise holding all project statuses mapped id -> object with project status info
			 */
			var _projectStatusesPromise = null;

			/**
			 * @member {Cache} promised cache for project information based on the project id
			 * @private
			 */
			var _projectInfo = new Cache(30, false);


			/** @member {Promise.Array.<AttObj>} _attributesPromise - A promise holding all system attributes in an array.
			 */
			var _attributesPromise = null;

			/**
			 * Constructor of the singleton AJAXDataCache object. 
			 */
			var createInstance = function () {
				return {



					//////////////////////////////////////////////////////////////////////
					// PUBLIC CACHE RELATED METHODS
					//////////////////////////////////////////////////////////////////////


					/**
					 * Method to reset all data related to passed document version ID cached in AJAXDataCache.
					 * 
					 * IMPORTANT: Every added cacheable info containing a document data hashed by document version ID 
					 * has to be added to this method to be correctly reset!
					 * 
					 */
					resetDocDataByDocVerID: function (docVerID) {
						if (docVerID) {
							if (_docInfoBydocVerID[docVerID]) {
								_docInfoBydocVerID[docVerID] = null;
							}
							if (_attrCatInfoByDocVerID[docVerID]) {
								_attrCatInfoByDocVerID[docVerID] = null;
							}
						}
					},


					/**
					 * Method to load system defaults for project creation info.
					 * It contains:
					 * - project creation system defaults for a blank project,
					 * - supported graphic file extensions,
					 * - supported document types with their configuration,
					 * - default autonaming options per document type.
					 * 
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 */
					getProjCreationSysDefaults: function (forceUpdate) {

						// force the update of the cache
						if (forceUpdate) {
							_projCreationSysDefaults = null;
						}

						var cacheItem = _projCreationSysDefaults;

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_projCreationSysDefaults = cacheItem;
						}

						if (!cacheItem.request) {

							cacheItem.request = $.Deferred();

						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						// if the cacheItem was just created initialize and fire the AJAX call and hook the cacheItem.request
						// Deferred object to it.
						try {
							// URL Parameters for fetching 'Project Creation Info' --> only Document Templates feedback is 
							// interesting in this case
							var url = window.webcenterConfig.jspBaseUrl + 'GetProjectCreationInfo.jsp?action=LoadDefaultSettings&ajax=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _projCreationSysDefaults;

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load project creation system defaults.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_projCreationSysDefaults = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _projCreationSysDefaults;

								// in case of SDK error display the details in alert box and leave error message as a result
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_projCreationSysDefaults = null;

									// nothing loaded when an SDK error occurs
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								// object to temporarily store loaded info (note just needed parts are stored)
								var sysDefs = {};

								var sysDefaultsElem = WCXML.GetFirstFirstLevelChild(xml, "system_defaults");
								if (sysDefaultsElem) {
									// load project creation system defaults
									sysDefs.blankProjectAutoname = WCXML.GetElementText(sysDefaultsElem, "blank_project_autoname") == '1';
									sysDefs.useDefAutoname4BlankProj = WCXML.GetElementText(sysDefaultsElem, "use_default_autoname_for_blank_project") == '1';
									sysDefs.useDefCategory4BlankProj = WCXML.GetElementText(sysDefaultsElem, "use_default_category_for_blank_project") == '1';

									// supported graphic file extensions
									var graphFileExtListElem = WCXML.GetFirstFirstLevelChild(sysDefaultsElem, "graphic_files_extensions");
									if (graphFileExtListElem) {
										var extElems = WCXML.GetFirstLevelChilds(graphFileExtListElem, "extension");
										if (extElems) {
											sysDefs.graphicExtList = [];
											for (var extIdx = 0; extIdx < extElems.length; extIdx++) {
												var extStr = WCXML.GetElementSelfTextContent(extElems[extIdx]);
												extStr = extStr.substring(extStr.indexOf(".") + 1); // remove leading dot
												sysDefs.graphicExtList.push(extStr);
											}
										}
									}

									// supported document types with their configuration
									var docTypeIDs = [];
									var docTypeListElem = WCXML.GetFirstFirstLevelChild(sysDefaultsElem, "doc_type_list");
									if (docTypeListElem) {
										var docTypeElems = WCXML.GetFirstLevelChilds(docTypeListElem, "doc_type");
										if (docTypeElems) {
											sysDefs.docTypeList = [];
											for (var dtIdx = 0; dtIdx < docTypeElems.length; dtIdx++) {
												var docTypeElem = docTypeElems[dtIdx];
												var docType = {};
												docType.id = WCXML.GetElementAttribute(docTypeElem, "id");
												docTypeIDs.push(docType.id);
												docType.name = WCXML.GetElementText(docTypeElem, "name");
												docType.labelTag = WCXML.GetElementText(docTypeElem, "label_tag");
												docType.enabled = WCXML.GetElementText(docTypeElem, "enabled");
												docType.uploadJSP = WCXML.GetElementText(docTypeElem, "Upload_JSP");
												docType.typicalExt = WCXML.GetElementText(docTypeElem, "upload_hint");
												// note not all the values in XML are parsed here!

												// store the loaded info to the document types list
												sysDefs.docTypeList.push(docType);
											}
										}
									}

									// default autonaming options per document type
									if (sysDefs.docTypeList && sysDefs.docTypeList.length > 0) {
										var autonameListElem = WCXML.GetFirstFirstLevelChild(sysDefaultsElem, "doctype_autonaming_list");
										if (autonameListElem) {
											var autonameElems = WCXML.GetFirstLevelChilds(autonameListElem, "doctype_autonaming");
											if (autonameElems) {
												for (var anIdx = 0; anIdx < autonameElems.length; anIdx++) {
													var anElem = autonameElems[anIdx];
													var anDocTypeID = WCXML.GetElementText(anElem, "doctypeid");
													if (anDocTypeID) {
														// there is an document type ID referenced. make sure the ID is valid and if so
														// find back the referred document type and add there the autonaming info.
														// note the default attribute category and task(s) to be started can be found here too!
														var docTypeIdx = docTypeIDs.indexOf(anDocTypeID);
														if (docTypeIdx < 0) {
															continue; // skip document types with not defined doc type ID
														}
														var docType = sysDefs.docTypeList[docTypeIdx];
														if (docType) {
															// add more info to "docType" object already created during loading supported document types
															docType.anPrefix = WCXML.GetElementText(anElem, "autoname_prefix");
															docType.anSuffix = WCXML.GetElementText(anElem, "autoname_suffix");
															docType.anCounterLength = WCXML.GetElementText(anElem, "autoname_counter_length");
															docType.anOption = WCXML.GetElementText(anElem, "autoname_option");
															docType.categoryID = WCXML.GetElementText(anElem, "category_id");
															if (!docType.categoryID) {
																docType.categoryID = cEmptyObject; // unified way how to pass "no attr. cat."
															}
															docType.categoryOption = WCXML.GetElementText(anElem, "category_option");
															// note not all the values in XML are parsed here!
														}
													}
													// else ignore autonaming setup for deleted / not existing document types
												}
											} // end of: if (autonameElems)
										}
									} // end of: if (sysDefs.docTypeList && sysDefs.docTypeList.length > 0)
								} //end of: if (sysDefaultsElem)

								// resolve the Deferred object governing the AJAX loading and parsing process.
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(sysDefs);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getProjCreationSysDefaults: " + err);
						}

						if (DocumentTypeUtils) {
							// register the DocumentTypeUtils initialization method to get the DocumentTypeUtils object initialized
							// as soon as the configuration is known (loaded from the server and cached)
							cacheItem.request.done(function (sysDefs) {
								DocumentTypeUtils.initDocumentTypes(sysDefs, forceUpdate);
							});
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},


					/**
					 * Method to load project (creation info) effective settings. The system defaults (loaded by
					 * getProjCreationSysDefaults() are merged with the project specific settings (if any).
					 * It contains:
					 * - project creation defaults,
					 * - supported graphic file extensions,
					 * - supported document types with their configuration,
					 * - default autonaming options per document type.
					 * 
					 * @param {String} projectID - ID of project, which effective document types etc. should be loaded.
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 */
					getEffectiveDocTypesSettingsByProjID: function (projectID, forceUpdate) {

						if (!projectID || projectID.length === 0) {
							var errorMsg = 'Project ID has to be specified in AJAXDataCache.getEffectiveDocTypesSettingsByProjID() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_docTypeSettingsByProjID[projectID] = null;
						}

						var cacheItem = _docTypeSettingsByProjID[projectID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_docTypeSettingsByProjID[projectID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// local Deferred object to govern the AJAX loading and parsing process
							var projDocTypesReq = $.Deferred();

							// URL Parameters for fetching 'Project Creation Info' --> only Document Templates feedback is 
							// interesting in this case
							var url = window.webcenterConfig.jspBaseUrl + 'GetDocUploadInfo.jsp?action=GetAutonamingSettingsForProject&projectID=' + projectID + '&ajax=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load project creation system defaults.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								projDocTypesReq.reject(errorMsg);

								// the final handling is done only after the merge operation!
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									projDocTypesReq.reject(errorMsg);

									// the final handling is done only after the merge operation!

								} else {

									// object to temporarily store loaded info (note just needed parts are stored)
									var docTypeList = [];

									var anInfoElem = WCXML.GetFirstFirstLevelChild(xml, "autonaming_info");
									if (anInfoElem) {
										var projANElem = WCXML.GetFirstFirstLevelChild(anInfoElem, "project_autoname");
										if (projANElem) {
											var docTypeANListElem = WCXML.GetFirstFirstLevelChild(projANElem, "doctype_autonaming_list");
											if (docTypeANListElem) {
												var dtanElems = WCXML.GetFirstLevelChilds(docTypeANListElem, "doctype_autonaming");
												if (dtanElems) {
													for (var dtIdx = 0; dtIdx < dtanElems.length; dtIdx++) {
														var docTypeANElem = dtanElems[dtIdx];
														var curProjID = WCXML.GetElementText(docTypeANElem, "projectid");
														if (curProjID != projectID) {
															window.alert("Project ID of processed <doctype_autonaming> tag is not the same as 'projectID'!");
															continue; // skip such a record...
														}
														var docType = {};
														docType.projectID = curProjID;
														docType.id = WCXML.GetElementText(docTypeANElem, "doctypeid");
														docType.anPrefix = WCXML.GetElementText(docTypeANElem, "autoname_prefix");
														docType.anSuffix = WCXML.GetElementText(docTypeANElem, "autoname_suffix");
														docType.anCounterLength = WCXML.GetElementText(docTypeANElem, "autoname_counter_length");
														docType.anOption = WCXML.GetElementText(docTypeANElem, "autoname_option");
														docType.categoryID = WCXML.GetElementText(docTypeANElem, "category_id");
														if (!docType.categoryID) {
															docType.categoryID = cEmptyObject; // unified way how to pass "no attr. cat."
														}
														docType.categoryOption = WCXML.GetElementText(docTypeANElem, "category_option");
														// note not all the values in XML are parsed here!

														docTypeList.push(docType);
													}
												}
											}
										}
									} //end of: if (anInfoElem)

									// resolve the intermediate Deferred object governing the AJAX loading and parsing process
									// return the parsed value for the request.done() method.
									projDocTypesReq.resolve(docTypeList);
								}
								// don't resolve the cacheItem.request yet as the postprocessing of loaded document types 
								// settings is mandatory here (and dependent on system doc. types settings loading too)!
							});

							// initiate the request to get the system doc. types settings for further processing
							var sysDefsReq = this.getProjCreationSysDefaults(forceUpdate);

							// register done() callback that is run after the project and system document types settings are loaded
							// this is sort of post processing of loaded project document setup to be sure it's effective
							// document types setup
							$.when(projDocTypesReq, sysDefsReq).done(function (projDocTypes, sysDefs) {
									// deep copy of the loaded system defaults ('sysDefs') to not modify the cached object
									var mergedDefs = ObjectHandler.getCloneOfObject(sysDefs);
									var sysDocTypes = mergedDefs.docTypeList;
									if (projDocTypes && sysDocTypes) {
										// merge all the project and system-level settings based on read options
										for (var s = 0; s < sysDocTypes.length; s++) {
											var sysDocType = sysDocTypes[s];
											var sysDocTypeID = sysDocType.id;
											if (!sysDocTypeID) {
												continue; // (safety, but should not happen normally)
											}
											for (var p = 0; p < projDocTypes.length; p++) {
												var projDocType = projDocTypes[p];
												// find the corresponding system level doc. type settings for the current 'projDocType'
												if (sysDocTypeID != projDocType.id) {
													continue;
												}

												// find the current doc. type in the "merged" (result) object
												var mergedDocType = mergedDefs.docTypeList[s];

												// note the options have the same meaning for both autonaming and def. attr. cat. 
												// anOption = '0' is for no autonaming defined ("none" option)
												// anOption = '1' is for project specific settings that overrules the corresponding system setup.
												// anOption = '3' is for system default settings ("default" option)
												if ((projDocType.anOption == '0') || (projDocType.anOption == '1')) {
													mergedDocType.anPrefix = projDocType.anPrefix;
													mergedDocType.anSuffix = projDocType.anSuffix;
													mergedDocType.anCounterLength = projDocType.anCounterLength;
													mergedDocType.anOption = projDocType.anOption;
												}

												if ((projDocType.categoryOption == '0') || (projDocType.categoryOption == '1')) {
													// the same logic as for document types above
													mergedDocType.categoryID = projDocType.categoryID;
													mergedDocType.categoryOption = projDocType.categoryOption;
												}
												// there is one-to-one matching between project and system-wide document type settings
												break;
											}
										}
									}
									// resolve the deferred object only after the postprocessing of the loaded document types
									// is done. return (and store in the cache) the loaded and postprocessed value.
									cacheItem.request.resolve(mergedDefs);
								})
								.fail(function (errorMsg) {
									if (jqXHR.status == 0) {
										return;
									}

									// reject the main request as at least one of the "subrequests" failed
									cacheItem.request.reject(errorMsg);
								});

						} catch (err) {
							window.alert("Problem in AJAXDataCache.getDocTypeSettingsByProjID: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},




					/**
					 * Method to load list of document templates (with details) for a specified project.
					 * 
					 * @param {String} projectID - ID of project for which to load document templates.
					 * @param {boolean} loadThumbnails - If true the thumbnail URLs will be also loaded, otherwise not.
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 */
					getDocTemplatesByProjID: function (projectID, loadThumbnails, forceUpdate) {

						if (!projectID || projectID.length === 0) {
							var errorMsg = 'Project ID has to be specified in AJAXDataCache.getDocTemplatesByProjID() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_docTemplatesByProjID[projectID] = null;
						}

						var cacheItem = _docTemplatesByProjID[projectID];

						if (cacheItem && !cacheItem.isThumbnailsLoaded && loadThumbnails) {
							// if the cache alraedy contains document templates for specified project, but that was loaded
							// without thumbnails, it must be reloaded to get teh version with thumbnails.
							_docTemplatesByProjID[projectID] = null;
							cacheItem = null;
						}

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_docTemplatesByProjID[projectID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
							cacheItem.isThumbnailsLoaded = loadThumbnails;
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// URL Parameters for fetching 'Project Creation Info' --> only Document Templates feedback is 
							// interesting in this case
							var url = window.webcenterConfig.jspBaseUrl + 'GetProjectCreationInfo.jsp?action=LoadTemplateDocuments&projectID=' + projectID + '&ajax=1';
							if (loadThumbnails) {
								url += '&loadThumbnails=1';
							}

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _docTemplatesByProjID[projectID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load project creation system defaults.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_docTemplatesByProjID[projectID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _docTemplatesByProjID[projectID];

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_docTemplatesByProjID[projectID] = null;

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								// inner function to load document details
								var parseTempDocElem = function (dvElem, folder) {
									var tempDoc = {};
									if (dvElem) {
										tempDoc.id = WCXML.GetElementText(dvElem, "document_id");
										tempDoc.docVerID = WCXML.GetElementAttribute(dvElem, "id");
										tempDoc.docVerIdx = WCXML.GetElementText(dvElem, "version_number");
										tempDoc.categoryID = WCXML.GetElementText(dvElem, "category_id");
										if (!tempDoc.categoryID) {
											tempDoc.categoryID = cEmptyObject; // unified way how to pass "no attr. cat."
										}
										tempDoc.docTypeID = WCXML.GetElementText(dvElem, "doctype_id");

										var docElem = WCXML.GetFirstFirstLevelChild(dvElem, "document");
										tempDoc.name = WCXML.GetElementText(docElem, "name");
										tempDoc.thumbnailURL = WCXML.GetElementText(docElem, "thumbnail_url");

										if (folder) {
											// folder info is available only for document is not in the root of document template project
											tempDoc.folderID = (folder.id ? folder.id : "");
											tempDoc.folderName = (folder.name ? folder.name : "");
										}
									}
									return tempDoc;
								};

								// object to temporarily store loaded info (note just needed parts are stored)
								var docTempList = [];

								var rootElem = WCXML.GetFirstFirstLevelChild(xml, "template_documents");
								if (rootElem) {
									docTempList.tempProjID = WCXML.GetElementText(rootElem, "template_project_id");
									var tempDocsElem = WCXML.GetFirstFirstLevelChild(rootElem, "template_docs");
									if (tempDocsElem) {
										var tempFoldersElem = WCXML.GetFirstFirstLevelChild(tempDocsElem, "folder_doc_list");
										if (tempFoldersElem) {
											var folderElems = WCXML.GetFirstLevelChilds(tempFoldersElem, "folder");
											if (folderElems) {
												// load documents sitting in folders of document template's project
												for (var fIdx = 0; fIdx < folderElems.length; fIdx++) {
													var folderElem = folderElems[fIdx];
													// folder details sits on this level - load it here
													var folder = {};
													folder.id = WCXML.GetElementAttribute(folderElem, "id");
													folder.name = WCXML.GetElementText(folderElem, "name");

													// load documents sitting in the particular folder of document template's project
													var tempDocRootElem = WCXML.GetFirstFirstLevelChild(folderElem, "doc_version_checkout_list");
													if (tempDocRootElem) {
														var tempDocVerElems = WCXML.GetFirstLevelChilds(tempDocRootElem, "doc_version_checkout");
														if (tempDocVerElems) {
															for (var dvIdx = 0; dvIdx < tempDocVerElems.length; dvIdx++) {
																var dvElem = WCXML.GetFirstFirstLevelChild(tempDocVerElems[dvIdx], "doc_version");
																docTempList.push(parseTempDocElem(dvElem, folder));
															}
														}
													}
												}
											}
											// load documents sitting in the root of document template's project
											var tempDocRootElem = WCXML.GetFirstFirstLevelChild(tempFoldersElem, "doc_version_checkout_list");
											if (tempDocRootElem) {
												var tempDocVerElems = WCXML.GetFirstLevelChilds(tempDocRootElem, "doc_version_checkout");
												if (tempDocVerElems) {
													for (var dvIdx = 0; dvIdx < tempDocVerElems.length; dvIdx++) {
														var dvElem = WCXML.GetFirstFirstLevelChild(tempDocVerElems[dvIdx], "doc_version");
														docTempList.push(parseTempDocElem(dvElem));
													}
												}
											}
										}
									}

									if (loadThumbnails) {
										// load the thumbnails URLs from the '<template_docs_thumbnail>' (optional) section 
										// of the returned XML
										var thumbListElem = WCXML.GetFirstFirstLevelChild(rootElem, "template_docs_thumbnail");
										if (thumbListElem) {
											var thumbElems = WCXML.GetFirstLevelChilds(thumbListElem, "template_doc_thumbnail");
											if (thumbElems) {
												for (var i = 0; i < thumbElems.length; i++) {
													var thumbElem = thumbElems[i];
													var docID = WCXML.GetElementText(thumbElem, "id");
													// having a document thumbnail record find the corresponding document template
													// in already loaded list and overwrite the 'thumbnailURL' property by
													// the "real" thumbnail URL
													for (var dtli = 0; dtli < docTempList.length; dtli++) {
														var docTemp = docTempList[dtli];
														if (docID == docTemp.id) {
															docTemp.thumbnailURL = WCXML.GetElementText(thumbElem, "thumbnail");
															break;
														}
													}
												}
											}
										}
									}

								} //end of: if (rootElem)

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(docTempList);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getDocTemplatesByProjID: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},



					/**
					 * Method to load list of folders for a specified project where the user has upload rights to.
					 * The additional permissions of the folders are also loaded. The purpose is to know the 'Change Property' permission value.
					 * 
					 * @param {String} projectID - ID of project for which to load the folders .
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - 
					 */
					getFoldersByProjID: function (projectID, forceUpdate) {

						if (!projectID || projectID.length === 0) {
							var errorMsg = 'Project ID has to be specified in AJAXDataCache.getFoldersByProjID() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_folderByProjectID[projectID] = null;
						}

						var cacheItem = _folderByProjectID[projectID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_folderByProjectID[projectID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// Make a call to a jsp page to load the list of folders this user can upload to
							var url = 'GetDocUploadInfo.jsp?action=GetSingleProjectFolders&projectID=' + projectID + '&ajax=1&loadfolderpermissions=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _folderByProjectID[projectID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load project upload folders.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_folderByProjectID[projectID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _folderByProjectID[projectID];

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_folderByProjectID[projectID] = null;

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								// object to temporarily store loaded info (note just needed parts are stored)
								var folderTempList = [];

								var rootElem = WCXML.GetFirstFirstLevelChild(xml, "upload_projects");
								if (rootElem) {
									// get the project node
									var projectElem = WCXML.GetFirstFirstLevelChild(rootElem, "project");
									if (projectElem) {

										// check whether you can upload to the root folder
										var rootUpload = WCXML.GetElementText(projectElem, "root_upload_enabled");
										if (rootUpload != null && rootUpload == '1') {
											var folder = {};
											folder.id = '0'; // root folder is recognized by the ID == '0'
											folder.name = '';
											// if you can upload to the root, you may or may not have change propery permissions to the root 
											var rootChangeProp = WCXML.GetElementText(projectElem, "root_change_prop_enabled");
											folder.changePropPerm = (rootChangeProp && (rootChangeProp == '1'));
											folderTempList.push(folder);
										}

										// iterate over the other folder elements
										var folderList = WCXML.GetFirstFirstLevelChild(projectElem, "folder_list");
										if (folderList) {
											var folderElems = WCXML.GetFirstLevelChilds(folderList, "folder");
											if (folderElems) {
												for (var fIdx = 0; fIdx < folderElems.length; fIdx++) {
													var folderElem = folderElems[fIdx];
													var folder = {};
													folder.id = WCXML.GetElementAttribute(folderElem, "id");
													folder.name = WCXML.GetElementText(folderElem, "name");

													// get the permission
													var folderChangePropPermission = false;
													var permElems = WCXML.GetFirstLevelChilds(folderElem, "permission");
													if (permElems && permElems.length > 0) {
														var folderPropPerm = WCXML.GetElementText(permElems[0], "update");
														folderChangePropPermission = (folderPropPerm && (folderPropPerm == '1'));
														folder.checkoutPerm = (WCXML.GetElementText(permElems[0], "checkout") == '1');
													}
													folder.changePropPerm = folderChangePropPermission;
													folderTempList.push(folder);
												}
											}
										}
									}

								} //end of: if (rootElem)

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(folderTempList);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getFoldersByProjID: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},


					/**
					 * Method to load list of document versions for a specified project.
					 * 
					 * @param {String} projectID - ID of project for which to load the documents.
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - 
					 */
					getDocVersionsByProjID: function (projectID, forceUpdate) {

						if (!projectID || projectID.length === 0) {
							var errorMsg = 'Project ID has to be specified in AJAXDataCache.getDocVersionsByProjID() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_docversionsByProjectID[projectID] = null;
						}

						var cacheItem = _docversionsByProjectID[projectID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_docversionsByProjectID[projectID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// Make a call to a jsp page to load the list of folders this user can upload to
							var url = 'GetDocumentList.jsp?projectID=' + projectID + '&ajax=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _docversionsByProjectID[projectID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load project documents.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_docversionsByProjectID[projectID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _docversionsByProjectID[projectID];

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_docversionsByProjectID[projectID] = null;

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								// object to temporarily store loaded info (note just needed parts are stored)
								var docVersionList = [];

								var rootElem = WCXML.GetFirstFirstLevelChild(xml, "document_list");
								if (rootElem) {

									var docElems = WCXML.GetFirstLevelChilds(rootElem, "document");
									if (docElems) {
										for (var i = 0; i < docElems.length; i++) {
											var docElem = docElems[i];
											var docversion = {};
											docversion.id = WCXML.GetElementText(docElem, "document_version_id");
											docversion.docID = WCXML.GetElementText(docElem, "document_id");
											docversion.name = WCXML.GetElementText(docElem, "document_name");
											docversion.versionIndex = WCXML.GetElementText(docElem, "document_version");
											docversion.revisionIndex = WCXML.GetElementText(docElem, "document_revision");
											docVersionList.push(docversion);
										}
									}

								} //end of: if (rootElem)

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(docVersionList);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getdocByProjID: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},

					/**
					 * Method to load info of a specified document.
					 * 
					 * @param {String} docVersionID - ID of document version which info should be loaded.
					 * @param {String} projectID - ID of project where the document is stored (useful for documents
					 * 		linked to multiple projects).
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 * 		In case the document doesn't exist or current user has no permissions to view the document,
					 * 		the returned value is 'emptyObject'.
					 */
					getDocInfoWithThumbnail: function (docVersionID, projectID, forceUpdate) {

						if (!docVersionID || docVersionID.length === 0) {
							var errorMsg = 'Document version ID has to be specified in AJAXDataCache.getDocInfoWithThumbnail() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) _docInfoBydocVerID[docVersionID] = null;

						var cacheItem = _docInfoBydocVerID[docVersionID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_docInfoBydocVerID[docVersionID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// URL Parameters for fetching 'document details info'
							var url = window.webcenterConfig.jspBaseUrl + 'GetDocumentHistory.jsp?docversionid=' + docVersionID;
							if (projectID) {
								url += '&projectid=' + projectID;
							}
							url += '&projectinfo=1&includetemplates=1&approvalinfo=1&limitdocversions=1&thumbnailurl=1&dateformat=2&acatinfo=1&folderinfo=1&ajax=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _docInfoBydocVerID[docVersionID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load document details (version ' + docVersionID + ').\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_docInfoBydocVerID[docVersionID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});

							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _docInfoBydocVerID[docVersionID];

								// object to temporarily store loaded document info (note just needed info is stored)
								var docInfo = null;

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									if ((sdkErrorStr.indexOf('021') == 0) || (sdkErrorStr.indexOf('005') == 0)) {
										// user doesn't have permission to view the document
										// return the empty object to signalize it to the caller
										docInfo = cEmptyObject;
										// resolve the intermediate Deferred object governing the AJAX loading and parsing process
										// return the set value for the request.done() method.
										cacheItem.request.resolve(docInfo);
									} else {
										var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
										DocTableXUtils.log(errorMsg);

										// this way the cache can recover from one-time errors 
										_docInfoBydocVerID[docVersionID] = null;

										// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
										cacheItem.request.reject(errorMsg);
									}
									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								var docInfoElem = WCXML.GetFirstFirstLevelChild(xml, "document_info");
								if (docInfoElem) {
									docInfo = {};
									docInfo.docID = WCXML.GetElementText(docInfoElem, "docID");
									docInfo.latestDocVerIdx = WCXML.GetElementText(docInfoElem, "last_version_number");
									docInfo.docName = WCXML.GetElementText(docInfoElem, "document_name");
									docInfo.docDesc = WCXML.GetElementText(docInfoElem, "description");
									docInfo.modDate = WCXML.GetElementText(docInfoElem, "modification_date");
									docInfo.docIsUrl = WCXML.GetElementBoolean(docInfoElem, "is_url");
									docInfo.docCheckedOutOwner = WCXML.GetElementText(docInfoElem, "checked_out_owner_id");

									// load permissions for 'docInfo' document of the currently logged in user 
									var docPermsElem = WCXML.GetFirstFirstLevelChild(docInfoElem, "permission");
									if (docPermsElem) {
										var perms = {};
										perms.view = (WCXML.GetElementText(docPermsElem, "view") == '1');
										perms.add = (WCXML.GetElementText(docPermsElem, "add") == '1');
										perms.change = (WCXML.GetElementText(docPermsElem, "update") == '1');
										perms.remove = (WCXML.GetElementText(docPermsElem, "delete") == '1');
										perms.checkout = (WCXML.GetElementText(docPermsElem, "checkout") == '1');
										perms.download = (WCXML.GetElementText(docPermsElem, "download") == '1');
										perms.changeApp = (WCXML.GetElementText(docPermsElem, "change_approval") == '1');
										perms.changeTask = (WCXML.GetElementText(docPermsElem, "change_task") == '1');
										perms.changeBOM = (WCXML.GetElementText(docPermsElem, "change_bom") == '1');
										perms.isProjMgr = (WCXML.GetElementText(docPermsElem, "can_manage_this_project") == '1');
										docInfo.userPerms = perms;
									}

									var docVerElem = WCXML.GetFirstFirstLevelChild(docInfoElem, "document_version");
									if (docVerElem) {
										docInfo.docVerID = WCXML.GetElementText(docVerElem, "docVersionID");
										docInfo.docVerIdx = WCXML.GetElementText(docVerElem, "version_number");
										docInfo.docVerDesc = WCXML.GetElementText(docVerElem, "version_comments");
										docInfo.docRevIdx = WCXML.GetElementText(docVerElem, "revision");
										docInfo.docRevLetter = WCXML.GetElementText(docVerElem, "revision_letter");
										docInfo.docTypeID = WCXML.GetElementText(docVerElem, "doc_type_id");
										docInfo.thumbnailURL = WCXML.GetElementText(docVerElem, "thumb_url");
										docInfo.docSourceId = WCXML.GetElementText(docVerElem, "source_id");
										docInfo.has3dView = WCXML.GetElementText(docVerElem, "has_3d_view");

										// read the category Info
										var aCatInfo;
										var category_id = WCXML.GetElementText(docVerElem, "category_id");
										if (category_id) {
											var categoryElem = WCXML.GetFirstFirstLevelChild(docVerElem, "category");
											if (categoryElem) {
												aCatInfo = {};
												aCatInfo.id = category_id;
												aCatInfo.name = WCXML.GetElementText(categoryElem, "name");
												aCatInfo.modTime = WCXML.GetElementText(categoryElem, "mod_time_stamp");
												aCatInfo.userUnits = WCXML.GetElementText(categoryElem, "user_units");
											} else {
												aCatInfo = cEmptyObject; // unified way how to pass "no attr. cat."
											}
										} else {
											aCatInfo = cEmptyObject;
										}
										docInfo.catInfo = aCatInfo;

										var folderInfo = {};
										var folderElem = WCXML.GetFirstFirstLevelChild(docVerElem, "folder");
										if (folderElem) {
											// load the ID and name of the folder this document is in
											folderInfo.id = WCXML.GetElementText(folderElem, "id");
											folderInfo.name = WCXML.GetElementText(folderElem, "name");
										} else {
											// the document is in the Root folder
											folderInfo.id = "";
											folderInfo.name = "";
										}
										docInfo.folderInfo = folderInfo;
										
										var authorElem = WCXML.GetFirstFirstLevelChild(docVerElem, "author");
										if (authorElem) {
											var author = {};
											// don't expect a group or role to be a document author
											var userElem = WCXML.GetFirstFirstLevelChild(authorElem, "user");
											if (userElem) {
												author.isUser = true;
												author.id = WCXML.GetElementText(userElem, "userID");
												author.lastName = WCXML.GetElementText(userElem, "lastname");
												author.firstName = WCXML.GetElementText(userElem, "firstname");
												author.username = WCXML.GetElementText(userElem, "login");
												author.isActive = WCXML.GetElementBoolean(userElem, "is_active");
											}
											docInfo.docVerAuthor = author;
										}

										// load approval status details
										var appStatus = {};
										appStatus.cycleStatusCode = WCXML.GetElementText(docVerElem, "document_approval_cycle_code");
										appStatus.cycleStatus = WCXML.GetElementText(docVerElem, "document_approval_cycle_status");
										appStatus.status = WCXML.GetElementText(docVerElem, "document_approval_status");
										appStatus.statusID = WCXML.GetElementText(docVerElem, "document_approval_code");
										var cycleStatusStringCodeList = {
											'-1': 'no_approval_rule',
											'0': 'cycle_started',
											'1': 'cycle_stopped',
											'2': 'cycle_finished'
										};
										appStatus.cycleStatusStringCode = cycleStatusStringCodeList[appStatus.cycleStatusCode];

										appStatus.stageIdx = WCXML.GetElementText(docVerElem, "stage_index");
										// load stage ID and name of the active stage too!
										var stagesElem = WCXML.GetFirstFirstLevelChild(docVerElem, "stages");
										if (stagesElem) {
											var stageElems = WCXML.GetFirstLevelChilds(stagesElem, "stage");
											if (stageElems) {
												for (var si = 0; si < stageElems.length; si++) {
													var stageElem = stageElems[si];
													var index = WCXML.GetElementText(stageElem, "index");
													if (index == appStatus.stageIdx) {
														// we found the currently active stage - store its ID and name
														appStatus.stageID = WCXML.GetElementText(stageElem, "stage_id");
														appStatus.stageName = WCXML.GetElementText(stageElem, "name");
														break;
													}
												}
											}
										}
										docInfo.appStatus = appStatus;
									}

									// load list of approval types (with details) applicable for this document
									docInfo.appTypeList = [];
									var appTypeListElem = WCXML.GetFirstFirstLevelChild(docInfoElem, "approvaltypes");
									if (appTypeListElem) {
										var appTypeElems = WCXML.GetFirstLevelChilds(appTypeListElem, "approvaltype");
										if (appTypeElems) {
											for (var i = 0; i < appTypeElems.length; i++) {
												var appTypeElem = appTypeElems[i];
												var appType = {};
												appType.id = WCXML.GetElementText(appTypeElem, "approvalid");
												appType.name = WCXML.GetElementText(appTypeElem, "type_name");
												appType.translName = WCXML.GetElementText(appTypeElem, "approvalname");
												appType.level = WCXML.GetElementText(appTypeElem, "approval_level");
												appType.dispOrder = WCXML.GetElementText(appTypeElem, "display_order");
												appType.icon = WCXML.GetElementText(appTypeElem, "icon");
												docInfo.appTypeList.push(appType);
											}
										}
									}

									// load (list of) project(s) where the document is present (linked).
									docInfo.projList = [];
									var projListRootElem = WCXML.GetFirstFirstLevelChild(docInfoElem, "project_list");
									if (projListRootElem) {
										var projElems = WCXML.GetFirstLevelChilds(projListRootElem, "project");
										if (projElems) {
											for (var i = 0; i < projElems.length; i++) {
												var projElem = projElems[i];
												var project = {};
												project.id = WCXML.GetElementAttribute(projElem, "id");
												project.name = WCXML.GetElementText(projElem, "name");
												var isTemplate = WCXML.GetElementText(projElem, "is_template");
												project.isTemplate = (!!isTemplate && isTemplate == '1');
												project.statusID = WCXML.GetElementText(projElem, "status_id");
												project.dueDate = WCXML.GetElementText(projElem, "due_date");
												project.categoryID = WCXML.GetElementText(projElem, "category_id");
												// store the loaded project details into docInfo result
												docInfo.projList.push(project);
											}
										}
									}
								}

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(docInfo);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getDocInfoWithThumbnail: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},





					/**
					 * Method to load basic attribute category info of the specified document.
					 * 
					 * @param {String} docVersionID - ID of document version which attribute category basic info should be loaded.
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 * 		In case the document has no attributes or current user has no permission to view the attributes,
					 * 		the returned value is 'emptyObject'.
					 */
					getDocAttrCatInfo: function (docVersionID, forceUpdate) {

						if (!docVersionID || docVersionID.length == 0) {
							var errorMsg = 'Document version ID has to be specified in AJAXDataCache.getDocAttrCatInfo() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_attrCatInfoByDocVerID[docVersionID] = null;
						}

						var cacheItem = _attrCatInfoByDocVerID[docVersionID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_attrCatInfoByDocVerID[docVersionID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// URL Parameters for fetching document's attribute category info
							var url = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?docverid=' + docVersionID + '&ajax=1';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _attrCatInfoByDocVerID[docVersionID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load attribute category of document version with ID: ' + docVersionID + ').\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_attrCatInfoByDocVerID[docVersionID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});


							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _attrCatInfoByDocVerID[docVersionID];

								var aCatInfo = null;

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									if ((sdkErrorStr.indexOf('043') == 0) || (sdkErrorStr.indexOf('005') == 0)) {
										// error code 043: there is no attribute category assigned to the given document 
										//				(a valid situation)
										// error code 005: user doesn't have permission to view the document's attributes
										// return the empty object to signalize it to the caller
										aCatInfo = cEmptyObject;
									} else {
										// any other SDK error means something went wrong during the load
										var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
										DocTableXUtils.log(errorMsg);

										// this way the cache can recover from one-time errors 
										_attrCatInfoByDocVerID[docVersionID] = null;

										// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
										cacheItem.request.reject(errorMsg);

										// don't continue in parsing the response when an SDK error is detected
										return cacheItem.request.promise();
									}
								}

								if (aCatInfo == null) {
									// aCatInfo not set yet - process the loaded XML
									var attrCatElem = WCXML.GetFirstFirstLevelChild(xml, "attribute_category");
									if (attrCatElem) {
										aCatInfo = {};
										aCatInfo.id = WCXML.GetElementText(attrCatElem, "id");
										if (aCatInfo.id) {
											aCatInfo.name = WCXML.GetElementText(attrCatElem, "name");
											aCatInfo.userUnits = WCXML.GetElementText(attrCatElem, "user_units");
										} else {
											aCatInfo = cEmptyObject; // unified way how to pass "no attr. cat."
										}
									} else {
										aCatInfo = cEmptyObject;
									}
								}

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(aCatInfo);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getDocAttrCatInfo: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},

					/**
					 * Method to load global upload settings.
					 * 
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 * 		The result will always be an object with the settings.
					 */
					getGlobalDocumentSettings: function (forceUpdate) {
						// force the update of the cache
						if (forceUpdate) {
							_globalDocumentSettings = null;
						}

						var cacheItem = _globalDocumentSettings;

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_globalDocumentSettings = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// Make a call to a jsp page to load the profile defaults settings for upload
							var url = window.webcenterConfig.jspBaseUrl + 'GetDocUploadInfo.jsp?action=LoadGlobalDocumentSettings';

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _globalDocumentSettings;

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load global settings.\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_globalDocumentSettings = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});


							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _globalDocumentSettings;

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									// any other SDK error means something went wrong during the load
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_globalDocumentSettings = null;

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								var settings = {};
								var siteDefs = WCXML.GetFirstFirstLevelChild(xml, "site_defaults");
								if (siteDefs) {
									settings.approvalRestartUploaderCanOverrule = WCXML.GetElementText(siteDefs, "approval_restart_enable_checkbox");
									settings.approvalRestartOption = WCXML.GetElementText(siteDefs, "approval_restart_option");
									settings.allowRaiseRevision = WCXML.GetElementText(siteDefs, "allow_raise_revision");
									settings.openCadType = WCXML.GetElementText(siteDefs, "open_cad_type");
								} else {
									settings = cEmptyObject;
								}

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(settings);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getGlobalDocumentSettings: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},

					/**
					 * Method to load download perspectives of the specified document.
					 * Note: parameters are docVersionID and docTypeID, we want to cache on docTypeID, but the Ajax call requires a docVersionId
					 * 
					 * @param {String} docVersionID - ID of document version 
					 * @param {String} docTypeID - type of document version 
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 * 		In case the document has no perspectives, the resolved value is an empty array.
					 */
					getDocDownloadPerspectives: function (docVersionID, docTypeID, forceUpdate) {

						if (!docVersionID || docVersionID.length === 0) {
							var errorMsg = 'Document version ID has to be specified in AJAXDataCache.getDocDownloadPerspectives() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						if (!docTypeID || docTypeID.length === 0) {
							var errorMsg = 'Document type ID has to be specified in AJAXDataCache.getDocDownloadPerspectives() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_downloadPerspectivesByDocVerID[docTypeID] = null;
						}

						var cacheItem = _downloadPerspectivesByDocVerID[docTypeID];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_downloadPerspectivesByDocVerID[docTypeID] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						try {
							// URL Parameters for fetching document's perspectives
							var url = window.webcenterConfig.jspBaseUrl + "GetDocumentPerspectives.jsp?docVerID=" + docVersionID;

							// make the AJAX request
							var ajaxRequest = $.ajax({
								type: 'GET',
								url: url,
								dataType: 'xml',
								context: this
							});

							// define "on error" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.fail(function (jqXHR, status, errorString) {
								if (jqXHR.status == 0) {
									return;
								}

								// make a local and updated! cacheItem variable
								var cacheItem = _downloadPerspectivesByDocVerID[docTypeID];

								var errorMsg = 'Request failed.\n';
								if (errorString == 'SESSION_EXP') {
									errorMsg += 'Session expired, please try again.';
								} else {
									errorMsg += 'Unable to load download perspectives for document version with ID: ' + docVersionID + ').\nServer status: ' + status;
								}
								DocTableXUtils.log(errorMsg);

								// this way the cache can recover from one-time errors 
								_downloadPerspectivesByDocVerID[docTypeID] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							});


							// define "on success" callback. Even two or more callbacks for a single request are 
							// supported! (due to the jQuery's Promise object).
							ajaxRequest.done(function (xml, status, jqXHR) {
								// make a local and updated! cacheItem variable
								var cacheItem = _downloadPerspectivesByDocVerID[docTypeID];

								// in case of SDK error display the details in alert box and return nothing
								var sdkErrorStr = findAndGetSDKError(xml);
								if (sdkErrorStr) {
									// any other SDK error means something went wrong during the load
									var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
									DocTableXUtils.log(errorMsg);

									// this way the cache can recover from one-time errors 
									_downloadPerspectivesByDocVerID[docTypeID] = null;

									// nothing loaded when an SDK error occurs - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);

									// don't continue in parsing the response when an SDK error is detected
									return cacheItem.request.promise();
								}

								var perspectivesList = null;
								var perpectivesListElem = WCXML.GetFirstFirstLevelChild(xml, "perspective_name_list");
								if (perpectivesListElem) {
									perspectivesList = [];
									var perspectiveElemList = WCXML.GetFirstLevelChilds(perpectivesListElem, "perspective_name");
									for (var i = 0; i < perspectiveElemList.length; i++) {
										var perspectiveElem = perspectiveElemList[i];
										var perspective = {};
										perspective.name = WCXML.GetElementText(perspectiveElem, "name");
										perspective.longName = WCXML.GetElementText(perspectiveElem, "long_name");
										perspectivesList.push(perspective);
									}
								}

								// resolve the intermediate Deferred object governing the AJAX loading and parsing process
								// return the parsed value for the request.done() method.
								cacheItem.request.resolve(perspectivesList);
							});
						} catch (err) {
							window.alert("Problem in AJAXDataCache.getDocDownloadPerspectives: " + err);
						}

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},

					/**
					 * Method to load project status info
					 * @returns {Promise.<Object>} A promise with an object with mapping project status id -> object with info
					 */
					getProjectStatuses: function () {
						var promise = _projectStatusesPromise;
						if (!promise) {
							var projectStatusListDeferred = $.Deferred();

							try {
								// URL Parameters for fetching project status info. We take the "Active" status as request parameter, but we get all back
								var url = window.webcenterConfig.jspBaseUrl + 'GetGenericEntitiesByID.jsp?project_status=00001_0000000001&compress_option=0';

								// make the AJAX request
								var ajaxRequest = $.ajax({
									type: 'GET',
									url: url,
									dataType: 'xml',
									context: this
								});

								// define "on error" callback
								ajaxRequest.fail(function (jqXHR, status, errorString) {
									if (jqXHR.status === 0) {
										return;
									}

									var errorMsg = 'Request failed.\n';
									if (errorString == 'SESSION_EXP') {
										errorMsg += 'Session expired, please try again.';
									} else {
										errorMsg += 'Unable to load project statuses details.\nServer status: ' + status;
									}
									DocTableXUtils.log(errorMsg);

									// nothing loaded in case of an error
									projectStatusListDeferred.reject(errorMsg);
								});

								// define "on success" callback
								ajaxRequest.done(function (xml, status, jqXHR) {
									// in case of SDK error display the details in alert box and return nothing
									var sdkErrorStr = findAndGetSDKError(xml);
									if (sdkErrorStr) {
										var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
										DocTableXUtils.log(errorMsg);

										// nothing loaded when an SDK error occurs
										projectStatusListDeferred.reject(errorMsg);

										// don't continue in parsing the response when an SDK error is detected
										return;
									}

									var genericEntitiesElem = WCXML.GetFirstFirstLevelChild(xml, "get_generic_entities");
									if (genericEntitiesElem) {
										var projectStatusListElem = WCXML.GetFirstFirstLevelChild(genericEntitiesElem, "project_status_list");
										if (projectStatusListElem) {
											var customProjecStatusElem = WCXML.GetFirstLevelChilds(projectStatusListElem, "custom_project_status");
											var projectStatuses = {};
											_.each(customProjecStatusElem, function (customProjectStatus) {
												var projectStatusInfo = {};
												projectStatusInfo.id = WCXML.GetElementAttribute(customProjectStatus, "id");
												projectStatusInfo.statusName = WCXML.GetElementText(customProjectStatus, "status_name");
												projectStatusInfo.iconURL = WCXML.GetElementText(customProjectStatus, "icon_url");
												projectStatusInfo.translationTag = WCXML.GetElementText(customProjectStatus, "translation_tag");
												projectStatusInfo.translatedName = WCXML.GetElementText(customProjectStatus, "translated_name");
												projectStatusInfo.colorRGB = WCXML.GetElementText(customProjectStatus, "color_rgb");

												projectStatuses[projectStatusInfo.id] = projectStatusInfo;
											});
											projectStatusListDeferred.resolve(projectStatuses);
										} else {
											projectStatusListDeferred.reject("Project Status element not found");
										}
									} else {
										projectStatusListDeferred.reject("Generic Entities element not found");
									}
								});
							} catch (err) {
								window.alert("Problem in AJAXDataCache.getProjectStatus: " + err);
								projectStatusListDeferred.reject("Problem in AJAXDataCache.getProjectStatuses: " + err);
							}

							promise = projectStatusListDeferred.promise();
							_projectStatusesPromise = promise;

							projectStatusListDeferred.fail(function () {
								_projectStatusesPromise = null;
							});
						}

						return promise;
					},

					/**
					 * Load project info
					 * @param {String} projectID - Id of the project
					 * @returns {$.Deferred.<Object>} Promise with the project info object
					 */
					getProjectInfo: function (projectID) {
						if (!projectID || projectID.length == 0) {
							var errorMsg = 'Project ID has to be specified in AJAXDataCache.getProjectInfo() method!';
							return $.Deferred().reject(errorMsg).promise();
						}
						return _projectInfo.get(projectID);
					},

					/**
					 * Method to load attributes by name.
					 * 
					 * @param {String} attributeName - name of attribute 
					 * @param {boolean} forceUpdate - If true the data stored in the cache will be reloaded, otherwise 
					 * 		the cached data is used.
					 * @returns - Promise object representing the request made with all registered callbacks. The loaded 
					 * 		value is returned via the first	parameter of registered function(s) of Promise.done() and/or 
					 * 		Promise.then() method. (See jQuery Promise and Deferred objects documentation for details.)
					 */
					getAttribute: function (attributeName, forceUpdate) {
						if (!attributeName || attributeName.length == 0) {
							var errorMsg = 'Attribute Name has to be specified in AJAXDataCache.getAttribute() method!';
							return $.Deferred().reject(errorMsg).promise();
						}

						// force the update of the cache
						if (forceUpdate) {
							_attributesByName[attributeName] = null;
						}

						var cacheItem = _attributesByName[attributeName];

						if (!cacheItem) {
							// create a new cache item for the data to be loaded
							cacheItem = {};
							_attributesByName[attributeName] = cacheItem;
						}

						if (!cacheItem.request) {
							// create a new Deferred object representing the result of this method (not only the AJAX call!)
							cacheItem.request = $.Deferred();
						} else {
							// return the Promise object in the cache (representing the request), which already contains 
							// the result or is waiting for the executed AJAX call to finish
							return cacheItem.request.promise();
						}

						// fetch attributes
						_requestedAttNames.push(attributeName);
						this.fetchAttributes();

						// value not yet prepared in the cache (the loading is in progress) - return a Promise object
						// to be able to register callbacks in the caller(s)
						return cacheItem.request.promise();
					},

					/**
					 * Get the system attributes in an array as provided by the Attribute Category.
					 * @returns {Promise.<Array.<Object>>} - Promise which resolves to the attributes in an array.
					 */
					getAttributes: function () {
						var promise = _attributesPromise;
						if (!promise) {
							var attributesDeferred = $.Deferred();

							try {
								// URL Parameters for fetching project status info. We take the "Active" status as request parameter, but we get all back
								var url = window.webcenterConfig.jspBaseUrl + 'GetAttributeListXML.jsp';

								// make the AJAX request
								var ajaxRequest = $.ajax({
									type: 'GET',
									url: url,
									dataType: 'xml',
									context: this
								});

								// define "on error" callback
								ajaxRequest.fail(function (jqXHR, status, errorString) {
									if (jqXHR.status === 0) {
										return;
									}

									var errorMsg = 'Request failed.\n';
									if (errorString == 'SESSION_EXP') {
										errorMsg += 'Session expired, please try again.';
									} else {
										errorMsg += 'Unable to load attributes from system.\nServer status: ' + status;
									}
									DocTableXUtils.log(errorMsg);

									// nothing loaded in case of an error
									attributesDeferred.reject(errorMsg);
								});

								// define "on success" callback
								ajaxRequest.done(function (xml, status, jqXHR) {
									// in case of SDK error display the details in alert box and return nothing
									var sdkErrorStr = findAndGetSDKError(xml);
									if (sdkErrorStr) {
										var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
										DocTableXUtils.log(errorMsg);

										// nothing loaded when an SDK error occurs
										attributesDeferred.reject(errorMsg);

										// don't continue in parsing the response when an SDK error is detected
										return;
									}

									var attributesElem = WCXML.GetFirstFirstLevelChild(xml, "attributes");
									if (attributesElem) {
										var attributesListElem = WCXML.GetFirstFirstLevelChild(attributesElem, "attribute_list");
										if (attributesListElem) {
											var attributeArr = WCXML.GetFirstLevelChilds(attributesListElem, "attribute");
											if (attributeArr) {
												var attributes = [];
												for (var att = 0; att < attributeArr.length; att++) {
													var attElem = attributeArr[att];
													var deleted = WCXML.GetElementText(attElem, "deleted");
													if (!deleted || deleted == "0") {
														var attObj = new Attribute({
															name: WCXML.GetElementText(attElem, "name"),
															id: WCXML.GetElementAttribute(attElem, "id"),
															description: WCXML.GetElementAttribute(attElem, "description"),
															unitId: WCXML.GetElementText(attElem, "attribute_unit_id"),
															rsId: WCXML.GetElementText(attElem, "restricted_set_id"),
															value: "",
															placedArr: []
														});
														attributes.push(attObj);
													}
												}
												attributesDeferred.resolve(attributes);
											}
										}
									}
								});
							} catch (err) {
								window.alert("Problem in AJAXDataCache.getAttributes: " + err);
								attributesDeferred.reject("Problem in AJAXDataCache.getAttributes: " + err);
							}

							promise = attributesDeferred.promise();
							_attributesPromise = promise;

							attributesDeferred.fail(function () {
								_attributesPromise = null;
							});
						}

						return promise;
					},

					/**
					 * Debounced Ajax, so multiple attributes can be requested at once
					 */
					fetchAttributes: _.debounce(function () {
						// URL Parameters for fetching document's perspectives
						var url = window.webcenterConfig.jspBaseUrl + "GetAttributeListXML.jsp?";

						var inProgress = _requestedAttNames;
						for (var i = 0; i < _requestedAttNames.length; i++) {
							if (i > 0) {
								url += "&";
							}
							url += "attrname=" + encodeURIComponent(_requestedAttNames[i]);
						}


						// make the AJAX request
						var ajaxRequest = $.ajax({
							type: 'GET',
							url: url,
							dataType: 'xml',
							context: this
						});

						// define "on error" callback. Even two or more callbacks for a single request are 
						// supported! (due to the jQuery's Promise object).
						ajaxRequest.fail(function (jqXHR, status, errorString) {
							if (jqXHR.status == 0) {
								return;
							}

							var errorMsg = 'Request failed.\n';
							if (errorString == 'SESSION_EXP') {
								errorMsg += 'Session expired, please try again.';
							} else {
								errorMsg += 'Unable to load attributes with names: ' + inProgress + ').\nServer status: ' + status;
							}
							DocTableXUtils.log(errorMsg);

							for (var i = 0; i < inProgress.length; i++) {
								var attributeName = inProgress[i];

								// make a local and updated! cacheItem variable
								var cacheItem = _attributesByName[attributeName];

								// this way the cache can recover from one-time errors 
								_attributesByName[attributeName] = null;

								// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
								cacheItem.request.reject(errorMsg);
							}
						});


						// define "on success" callback. Even two or more callbacks for a single request are 
						// supported! (due to the jQuery's Promise object).
						ajaxRequest.done(function (xml, status, jqXHR) {
							// in case of SDK error display the details in alert box and return nothing
							var sdkErrorStr = findAndGetSDKError(xml);
							if (sdkErrorStr) {
								// any other SDK error means something went wrong during the load
								var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
								DocTableXUtils.log(errorMsg);

								for (var i = 0; i < inProgress.length; i++) {
									var attributeName = inProgress[i];
									// make a local and updated! cacheItem variable
									var cacheItem = _attributesByName[attributeName];

									// this way the cache can recover from one-time errors 
									_attributesByName[attributeName] = null;

									// nothing loaded in case of an error - propagate it and empty the corresponding cache entry
									cacheItem.request.reject(errorMsg);
								}
							}

							var attributesElem = WCXML.GetFirstFirstLevelChild(xml, "attributes");
							if (attributesElem) {
								var attributeListElem = WCXML.GetFirstFirstLevelChild(attributesElem, "attribute_list");
								if (attributeListElem) {
									var attributeElemList = WCXML.GetFirstLevelChilds(attributeListElem, "attribute");
									for (var i = 0; i < attributeElemList.length; i++) {
										var attributeElem = attributeElemList[i];
										var attribute = new Attribute({
											name: WCXML.GetElementText(attributeElem, "name"),
											id: WCXML.GetElementAttribute(attributeElem, "id"),
											description: WCXML.GetElementAttribute(attributeElem, "description"),
											unitId: WCXML.GetElementText(attributeElem, "attribute_unit_id"),
											rsId: WCXML.GetElementText(attributeElem, "restricted_set_id"),
											placedArr: [],
											value: ""
										});

										// remove from inProgress...
										var ind = inProgress.indexOf(attribute.name);
										if (ind != -1) {
											inProgress.splice(ind, 1);
										}

										// make a local and updated! cacheItem variable
										var cacheItem = _attributesByName[attribute.name];
										if (cacheItem) {
											cacheItem.request.resolve(attribute);
										}
									}
								}
							}

							// for attributes that were not returned, resolve with null
							for (var i = 0; i < inProgress.length; i++) {
								var attributeName = inProgress[i];
								var cacheItem = _attributesByName[attributeName];
								cacheItem.request.reject("Attribute " + attributeName + " not found"); // TODO: localize?
							}
						});

						_requestedAttNames = [];
					}, 50)


					//////////////////////////////////////////////////////////////////////
					// END OF PUBLIC METHODS DEFINITION
					//////////////////////////////////////////////////////////////////////

				};
			};

			/**
			 * Method to find and parse SDK error if exists.
			 * 
			 * @param {Object} xml - Parsed XML to be searched for an SDK error.
			 * @returns - The error code with error message if there is an SDK error, empty string otherwise.
			 */
			var findAndGetSDKError = function (xml) {
				try {
					var errorElem = WCXML.GetFirstFirstLevelChild(xml, "error");
					if (errorElem) {
						var errorCode = WCXML.GetElementText(errorElem, "code");
						var errorMsg = WCXML.GetElementText(errorElem, "message");
						DocTableXUtils.log("SDK error " + errorCode + ": " + errorMsg);
						return errorCode + ": " + errorMsg; // SDK error occurred
					}
				} catch (err) {
					DocTableXUtils.log("Problem in findAndGetSDKError() private method.");
					return "000: Undefined error"; // something goes really wrong
				}
				return ""; // no SDK error in the passed XML - good to go
			};


			/**
			 * Load project info
			 * @param {String} projectID - Id of the project
			 */
			var _loadProjectInfo = function (projectID) {
				var deferred = $.Deferred();

				try {
					// URL Parameters for fetching project status info. We take the "Active" status as request parameter, but we get all back
					var url = window.webcenterConfig.jspBaseUrl + 'GetProjects.jsp?type=6&projectID=' + projectID;

					// make the AJAX request
					var ajaxRequest = $.ajax({
						type: 'GET',
						url: url,
						dataType: 'xml',
						context: this
					});

					// define "on error" callback
					ajaxRequest.fail(function (jqXHR, status, errorString) {
						if (jqXHR.status === 0) {
							return;
						}

						var errorMsg = 'Request failed.\n';
						if (errorString == 'SESSION_EXP') {
							errorMsg += 'Session expired, please try again.';
						} else {
							errorMsg += 'Unable to load project details.\nServer status: ' + status;
						}
						DocTableXUtils.log(errorMsg);

						// nothing loaded in case of an error
						deferred.reject(errorMsg);
					});

					// define "on success" callback
					ajaxRequest.done(function (xml, status, jqXHR) {
						// in case of SDK error display the details in alert box and return nothing
						var sdkErrorStr = findAndGetSDKError(xml);
						if (sdkErrorStr) {
							var errorMsg = 'Request failed with SDK error:\n' + sdkErrorStr;
							DocTableXUtils.log(errorMsg);

							// nothing loaded when an SDK error occurs
							deferred.reject(errorMsg);

							// don't continue in parsing the response when an SDK error is detected
							return;
						}

						var getProjectList = WCXML.GetFirstFirstLevelChild(xml, "get_project_list");
						if (getProjectList) {
							var projectList = WCXML.GetFirstFirstLevelChild(getProjectList, "project_list");
							if (projectList) {
								var projectElem = WCXML.GetFirstFirstLevelChild(projectList, "project");
								if (projectElem) {
									var project = {};
									project.id = WCXML.GetElementAttribute(projectElem, "id");
									project.name = WCXML.GetElementText(projectElem, "name");
									var isTemplate = WCXML.GetElementText(projectElem, "is_template");
									project.isTemplate = (!!isTemplate && isTemplate == '1');
									project.statusID = WCXML.GetElementText(projectElem, "status_id");
									project.dueDate = WCXML.GetElementText(projectElem, "due_date");
									project.categoryID = WCXML.GetElementText(projectElem, "category_id");
									deferred.resolve(project);
								} else {
									deferred.reject("Project Element not found");
								}
							} else {
								deferred.reject("Project List element not found");
							}
						} else {
							deferred.reject("Get Project List element not found");
						}
					});
				} catch (err) {
					window.alert("Problem in AJAXDataCache._loadProjectInfo: " + err);
					deferred.reject("Problem in AJAXDataCache._loadProjectInfo: " + err);
				}

				var promise = deferred.promise();

				_projectInfo._registerItem(projectID, promise);
			};
			_projectInfo._loadItem = _loadProjectInfo;


			// JavaScript singleton pattern - return object exposing nothing else than getInstance() method
			// and some "static" class level getters.
			return {
				getInstance: function () {
					// make sure the object is singleton
					if (!_instance) {
						_instance = createInstance();
					}
					return _instance;
				},
				isEmptyValue: function (result) {
					return result && (result.IAmEmptyObject === true);
				},
				getEmptyValue: function () {
					return cEmptyObject;
				}
			};
		})();

		return AJAXDataCache;
	});