define(["wcr/webcenter_xml",
        "doctablex/DocumentRowFactory",
        "doctablex/DocTableXView",
        "doctablex/DocTableXUtils",
        "doctablex/AJAXDataCache",
        "doctablex/StaticDocumentRow",
        "doctablex/DocumentRow",
        "doctablex/DocumentRowSearch",
        "doctablex/DocumentRowBrowse",
        "doctablex/DocumentRowFile",
        "doctablex/DocumentRowNew",
        "doctablex/DocumentRowLink",
        "doctablex/DocumentRowCopy",
        "doctablex/DocumentRowExternalFile",
        "utils/webi18n!DocRefStrings",
        "underscore",
        "jquery",
        "wcr/search",
        "wcr/extpopup",
        "lib/PromisePolyfill"], 
function(WCXML,
		documentRowFactory,
        DocTableXView,
        DocTableXUtils,
        AJAXDataCache,
        StaticDocumentRow,
        DocumentRow,
        DocumentRowSearch,
        DocumentRowBrowse,
        DocumentRowFile,
        DocumentRowNew,
        DocumentRowLink,
        DocumentRowCopy,
        DocumentRowExternalFile,
        DocRefStrings,
        _,
        $) {
    /**
     * DocTableX.js
     * 
     * Object representing table of documents in all WebCenter relevant cases.
     * The main goal is to handle adding, removing and order of individual rows (documents / document sources)
     * in the documents table.
     * Examples: 
     * - document reference attribute values, 
     * - documents linked to a task on TEP, 
     * - upload multiple documents page, ...
     * 
     * @author JANH - September 2014
     */

    "use strict";

    var DocTableX = function(options) {
        if (!options) {
            // handle the inheritance definition in JavaScript: childClass.prototype = new parentClass();
            // the childClass constructor is responsible for calling the constructor of superclass too!!!
            return this;
        }

        /** @member {Boolean} isInConfigPreview - Flag to tweak functionality if the documents table is 
         * 		displayed in the attribute category layout configuration preview == without a project 
         * 		context (value true) or it's for real (value false). Default is false. */
        this.isInConfigPreview = options.isInConfigPreview || false;
        
        /** member {Object} currentUser - Object containing significant info about currently logged in user. */
        this.currentUser = options.currentUser || null;

        /** @member {function} onChangeCallback - Function to be called when the document table is modified. */
        this.onChangeCallback = options.onChangeCallback || null;

        /** @member {function} onBeforeDownloadAction - Function to be called before a download action
         * 		is submitted. It's to prevent "Do you want to leave page" kind of messages and potentially
         * 		other "preparation" needed to execute download on the particular page. */
        this.onBeforeDownloadAction = options.onBeforeDownloadAction || null;

        /** @member {DOM element} pageActionForm - Action form under which this documents table exists.
         * 		Default value is in WebCenter the most commonly used 'document.actionform'. */
        this.pageActionForm = options.pageActionForm;
        if (!this.pageActionForm) this.pageActionForm = document.actionform;
        
        /** member {Boolean} isModifiedElsewhere - This flag says if another instance of the very same documents 
         * 		table has been modified on the actual page. If so this documents table is switched to read-only 
         * 		mode and acts like a slave to the modified (master) documents table. */
        this.isModifiedElsewhere = false;
        
        /** 
         * @member {Array.<StaticDocumentRow (or a subclass)>} docRows - Array of document rows. 
         * 		Each element represents 1 document (a row/item in the documents table).
         */
        this.docRows = [];
        
        /** member {int} _rowCreationIdx - Next "creation index" of the row to be created (used for SAHI testing). */
        this._rowCreationIdx = 0;
        
        /** member {Object} keepCloneFileInfoMap - Keep the file names which is cloned */
        this.keepCloneFileInfoMap = {};

        /** @member {Boolean} isChanged - Flag whether the document table content was changed by the user or not */
        this.isChanged = false;

        /** @member {Array.<String>} parentDocVerIDs - list of all parent document version IDs. 
         * 		This is needed for cycle detection algorithm. DocTableX initialization code is responsible
         * 		for passing the correct value in options.parentDocVerIDs. */
        this.parentDocVerIDs = options.parentDocVerIDs || [];

        /** @member {StaticDocumentRow or subclass} _parentDocRow - Document row object that is the closest parent of this 
         * 		documents table (or null if there is none). */
        this._parentDocRow = options.parentDocRow || null;

        
        /** @member {String} phElemID - ID of a place holder HTML element, where the documents table will be rendered. */
        this.phElemID = options.phElemID;

        /** @member {String} saveXMLRootTag - Name of the root XML tag where all values of this document 
         * 		table are put for saving purposes. Default name is 'dtval'. */
        this.saveXMLRootTag = 'dtval';
        if (options.saveXMLRootTag) this.saveXMLRootTag = WCXML.xmlencode(options.saveXMLRootTag);

        /** @member {String} thisInDOM - JavaScript statement to find the reference of this object 
         * 		stored in the dedicated HTML element (with ID this.phElemID). 
         * 		It exist mainly to support action handlers in generated HTML. */
        this.thisInDOM = 'document.getElementById(\'' + this.phElemID + '\').docTable';
        
        /** @member {String} extraActions - An array of jQuery objects, the elements of these objects should 
         * be valid entries of the "ActionMenu" component. All extra actions on the DocTablex (like download) 
         * will be contained in this. */
        this._extraActions = [];
        
        /** @member {String} greenLinksPrefix - Unique prefix for green links - derived from this.prefix. */
        this.greenLinksPrefix;   // don't set it directly! use this._setPrefix() private method instead (as below)!

        /** @member {String} prefix - Unique prefix for this DocTableX instance only. 
         * 		It's mainly used to generate unique IDs and names of HTML elements. */
        this._setPrefix(options.prefix);


        /** @member {documentSourceHandler} dshObj - Reference to the defined document sources (green links). */
        this.dshObj = options.dshObj;
        if (!this.dshObj) {
            alert("Document table cannot be created without DocumentSourceHandler reference!");
            return null;
        }


        /** @member {Boolean} isEditable - True if the options says so and the current user can edit this 
         * 		documents table, false otherwise. By default the documents table is read-only. */
        this.isEditable = options.isEditable || false;
        if (this.currentUser && (this.currentUser.canEditAttributes == false)) {
        	// this particular user cannot edit attributes => cannot edit documents table as well
            this.isEditable = false;
        }

        /** @member {Boolean} isInactive - If true, this documents table is not rendered, validated and
         * 		no save XML is generated (values are not saved).
         * 		Note this field is set dynamically just before each render operation, if it has changed. */
        this.isInactive = options.isInactive || false;

        /** @member {Boolean} isInjectedValue - True if the value of this documents table was changed
         * by application logic (without intervention of a user), false otherwise.
         * Currently only task specification values that are synchronized with project attributes
         * can have their value "injected" by application logic even if they are read-only for the user. 
         * This is used to make sure <readonly>1</readonly> XML tag is NOT generated on save
         * in case the value of this documents table was changed by application logic. */
        this.isInjectedValue = options.isInjectedValue || false;
        
        /** @member {Boolean} parentACatNotExpandedYet - If true, the descendant attribute and property
         * 		categories won't be initialized (and loaded)! It's to optimize speed and memory 
         * 		consumption + it is also a fundamental part of validation logic that skips descendants 
         * 		not seen by the user! */
        this.parentACatNotExpandedYet = options.parentACatNotExpandedYet || false;

        /** @member {Boolean} autoExpandAllRows - If this is true, all document rows within this documents
         * 		table will be automatically expanded at documents table initialization time. */
        this.autoExpandAllRows = options.autoExpandAllRows || false;
        
        /** @member {Boolean} clearValues - Set this to true to clear all documents from this documents table 
         * 		on submit. Leave it on false (default value) if the documents should be saved/preserved. 
         * 		This is to support attribute visibility type "hidden"... */
        this.clearValues = options.clearValues || false;

        /** @member {Boolean} allowMultiple - True if the doc table can accept 2 or more documents, false otherwise. */
        this.allowMultiple = options.allowMultiple || false;

        /** @member {int} docReferenceMode - Mode of the corresponding (parent) document reference attribute.
         * 		Supported values are 0..Latest Version; 1..Selected Version; 2..Selected Revision. 
         * 		Default value for DocTableX context is 0 (Latest Version) */
        this.docReferenceMode = options.docReferenceMode || 0;

        /** @member {String} projectID - ID of the project, where documents in this doc. table reside
         * 		and/or should be uploaded to. Use-case where documents in a single doc. table are from
         * 		different projects is not considered! */
        this.projectID = options.projectID || null;
        
        /** @member {String} taskID - ID of the task, where documents in this doc. table are attached to. */
        this.taskID = options.taskID || null;

        /** @member {Boolean} hasDocumentTemplates - True if the current project has document template library 
         * 		configured and there are any document in there visible for current user, false otherwise. 
         * 		In case we are in the configuration preview the default value is true, otherwise false. */
        this.hasDocumentTemplates = this.isInConfigPreview;

        /** @member {Boolean} canUserAddDocuments - True if the current user has "Add" permission to at
         * 		least one folder (root counted as a folder too) = it can add documents to the project
         * 		In case we are in the configuration preview the default value is true, otherwise false. */
        this.canUserAddDocuments = this.isInConfigPreview;

        /** @member {String} aCatView - Name of the parent attribute category view. It is used as a default 
         * 		view for descendant category(s) if the particular document row doesn't have category view 
         * 		defined via table layout rendering configuration. */
        this.aCatView = options.aCatView;

        /** @member {boolean} projectCreationContext - True if the table is in project creation context */
        this.projectCreationContext = options.projectCreationContext || false;

		//JANH TODO remove this!!!
        /** @member {boolean} replace new operation in document template - True if the doc row is added during replace operation. */
        this.replaceDocRowAdded = false;

        /** @member {Array<Object>} targetFolders - Array of available folders pushed from outside (of DocTableX). 
         * 		The outside rutine has to make sure the provided folders are correct and logged in user has view and
         * 		upload rights to them. Currently this is used only in ProjectCreation.js when copying of folders from
         * 		selected project template is disabled by user.
         * 		Note the meaning/use of DocTableX.targetFolders is different than DocumentRow.targetFolders, which are 
         * 		loaded and managed by document row itself...
         * 		Each member of the array looks like: {
         * 			id,
         * 			name,
         * 			changePropPerm,
         * 			isDefaultFromDocTempl (only in a selected item)
         * 		}
         * 		Root (if in the array) is always the first element.
         * 		'this.targetFolders == []' (empty array) means that the target folders were loaded, 
         * 		but the user has no upload rights to the current project.
         * 		'this.targetFolders == null' on the other hand means the target folders were not loaded yet!
         */
        this.targetFolders = options.targetFolders || null;
        
        // rendering options ========================================

        // default (minimal) set of rendering options
        var defaultRenderOptions = {
                renderMethod : 'list',
                showGreenLinksOnTop : true,
                showGreenLinksAtBottom : false,
                showThumbnails : true
        };

        /** @member {Object} renderOptions - Collection of rendering options controlling how the documents
         * 		table header looks like. By default it's applied to document rows within the table too!
         * 		The reason is to control document rows rendering in order to be in sync with table header. */
        this.renderOptions = options.renderOptions || defaultRenderOptions;
        
        // make sure the 'showThumbnails' render options has a boolean value
        if (this.renderOptions.showThumbnails === undefined) this.showThumbnails = true;

        /** @member {String} propsACatID - ID of the document reference properties attribute category. 
         * 		If it's null, this document reference has no properties. */
        this.propsACatID = options.propsACatID || null;

        
        // Task (Execution page) specific ================================
        
        /** @member {String} defTaskDocAttrCatID - ID of default attribute category assigned to documents uploaded 
         * 		on Task Execution page (if configured in corresponding task type). */
        this.defTaskDocAttrCatID = options.defTaskDocAttrCatID || '';

        /** @member {String} defTaskDocFolderName - Name of default target folder, where to put all documents 
         * 		uploaded on Task Execution page (if configured in corresponding task type). 
         * 		Note '' (empty string) is for root folder so it's better to initialize to null 
         * 		in case of no value... */
        this.defTaskDocFolderName = options.defTaskDocFolderName || null;

        
        /** @member {function} _docRowsInsertedPromiseResolve - externalized 'this.docRowsInsertedPromise' 
         * 		resolve method */
        this._docRowsInsertedPromiseResolve;
        /** @member {function} _docRowsInsertedPromiseReject - externalized 'this.docRowsInsertedPromise' 
         * 		reject method*/
        this._docRowsInsertedPromiseReject;
        
        /** @member {Promise} docRowsInsertedPromise - Promise that is resolved only after all document rows
         * 		of this documents table are inserted at initialization time. 
         * 		Be aware that document rows must be added to this documents row, but not necessarily fully 
         * 		initialized! (which is often the case) */
        this.docRowsInsertedPromise = new Promise(function(resolve, reject) {
        	this._docRowsInsertedPromiseResolve = resolve;
        	this._docRowsInsertedPromiseReject = reject;
        }.bind(this));
        
        
        // private initialization ========================================

        if (this.constructor == DocTableX.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass


            /** @member {Object} view - View of this data model to perform rendering operations. */
            this.view = new DocTableXView(
                    this,   // data model reference
                    this.phElemID,   // place holder HTML element, where the view will render the content
                    this.renderOptions   // rendering options (render method, etc.)
            );


            // load additional required info from the server (like if there are any available document templates)
            // via AJAX (if not known)
            // note: this should be called only after the 'this.view' is properly initialized!
            this._initializeFromServer();

        }

        return this;
    };

    /**
     * Method to be called when basic initialization/creation of documents table is done. 
     * It resolves the DocTableX.docRowsInsertedPromise, which can be used outside documents table to
     * synchronize further operations on top of the documents table.
     * It includes:
     * - Initialization of the DocTableX object,
     * - Basic initialization/loading of all document rows to be added to the documents table is done,
     * - All the "initial" document rows (from above) are added to the documents table.
     * 
     * Note the loading of document rows children (their attribute categories...) and other info 
     * might be still ongoing, so the document row cannot be considered FULLY loaded/initialized!
     */
    DocTableX.prototype.dataLoadingFinished = function() {
    	this._docRowsInsertedPromiseResolve();
    	return this.docRowsInsertedPromise;
    };
    
    /**
     * The same as DocTableX.dataLoadingFinished(), but for the case something went wrong during the
     * initialization/loading/adding of document rows.
     */
    DocTableX.prototype.dataLoadingFailed = function() {
    	this._docRowsInsertedPromiseReject();
    	return this.docRowsInsertedPromise;
    };
    
    
    /**
     * Private initialization method using AJAX to load asynchronously required info that is not passed by 
     * 'options' (constructor parameter) from the server.
     */
    DocTableX.prototype._initializeFromServer = function() {
        if (this.isInConfigPreview) {
        	// skip the template documents and target folders loading if in a preview/configuration mode 
        	// without a project context
        	return;
        }
        
        if (this.projectID) {
	        // initialize document source links (green links) based on whether a document template is defined for
	        // 'this.projectID' project or not
	        var docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.projectID, true);
	        // get the project folders, where the user has upload permission
			var folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.projectID);
			
			$.when(docTemplatesReq, folderListReq).done(function(docTempList, folderList) {
				var origHasDocTempsVal = this.hasDocumentTemplates;
	            var origCanAddDocsVal = this.canUserAddDocuments;
	            // remember whether there are document templates available for current project or not
				// (based on this info the "New" document source is shown or not)
	            this.hasDocumentTemplates = !!docTempList && (docTempList.length > 0);
	            // remember whether the logged in user can add documents to at least root or one folder in this project
				this.canUserAddDocuments = !!folderList && (folderList.length > 0);
	            
				if ((this.hasDocumentTemplates != origHasDocTempsVal) || (this.canUserAddDocuments != origCanAddDocsVal)) {
	                // if one of the values changed, repaint the document sources to show/hide all the links
	            	// and/or the "New" document source link
	                this.view.updateActions();
	        		
	                if (this.canUserAddDocuments != origCanAddDocsVal) {
	                	// repaint document rows that might be affected by the change of 
	                	// 'this.canUserAddDocuments' value. the affected ones are identified by
	                	// the StaticDocumentRow.isRemoteDocument() method
	                	this.view.repaintRemoteDocRows();
	                }
	            }
	        }.bind(this))
	        .fail(function(errorMsg) {   // on error
	            alert("Failed to load data needed to initialize DocTableX instance from the server.\n" + errorMsg);
	        });
        } else {
	        // initialize document source links (green links) for blank project

        	// there are no document templates available for brand new project created 
        	// from blank - "New" document source is not shown
            this.hasDocumentTemplates = false;
            // logged in user can add documents as he/she is creating the project
			this.canUserAddDocuments = true;
        }
    };


    DocTableX.prototype.getNextRowCreationIndex = function() {
        return this._rowCreationIdx++;
    };
    
    /**
     * Sets the 'isChanged' flag to true and executes the 'onChangeCallback' if it's defined.
     */
    DocTableX.prototype.setIsChanged = function() {
        if (this.onChangeCallback) {
        	this.onChangeCallback();
        }
        this.isChanged = true;
    };

    /**
     * Returns true if document table or one of its rows/items was changed, otherwise false.
     * @returns {Boolean}
     */
    DocTableX.prototype.getIsChanged = function() {
        // a document row was added/removed or some rows were moved around
        if (this.isChanged) return true;

        // ask each document row if something was changed in it
        for (var dr = 0; dr < this.docRows.length; dr++) {
            var docRow = this.docRows[dr];
            if (docRow && docRow.getIsChanged()) return true;
        }
        return false;
    };


    /**
     * Sets the prefix of this DocTableX instance. It's used to generate unique IDs and names of HTML elements.
     * 
     * @param {String} newPrefix - The full new prefix to be used with this particular object.
     */
    DocTableX.prototype._setPrefix = function(newPrefix) {
        if (!newPrefix) newPrefix = '';   // null or undefined can break the code later...
        this.prefix = newPrefix;
        // update dependent prefixes
        this.greenLinksPrefix = newPrefix + '_greenLinks';
    };


    DocTableX.prototype.getTablePrefix = function() {
        return this.prefix;
    };

    /**
     * See description of 'this.thisInDOM' member in the constructor.
     * @returns {String} JavaScript statement to get reference to this on generated HTML page.
     */
    DocTableX.prototype.getThisInDOM = function() {
        return this.thisInDOM;
    };

    DocTableX.prototype.isBlankProjectCreationContext = function() {
    	return this.projectCreationContext && !this.projectID;
    };
    
    /**
	 * Is the doctableX located in a project creation context?
	 * @returns {Boolean} True if project creation context, false otherwise
	 */
	DocTableX.prototype.isProjectCreationContext = function () {
		return this.projectCreationContext;
	};

	/**
	 * Get IDs of all existing documents within this documents table (temporary IDs of new documents are
	 * not included!)
	 */
    DocTableX.prototype.getAllDocumentsIDs = function() {
    	var docIDs = [];
        for (var dr = 0; dr < this.docRows.length; dr++) {
            var docRow = this.docRows[dr];
            if (docRow && docRow.docID) {
            	// add the document ID only if it is known (not the case for uploaded, copied, etc. documents)
            	docIDs.push(docRow.docID);
            }
        }
        return docIDs;
    };


	/**
     * This method returns a document row based on its "row ID" - a numerical constant value unique for each document row.
     * 
     * @param {int} rowID - ID of the searched document row
     * @returns {StaticDocumentRow (or a subclass)} - It returns the found document row instance or null, if nothing found.
     */
    DocTableX.prototype.getRowByID = function(rowID) {
        for (var dr = 0; dr < this.docRows.length; dr++) {
            var docRow = this.docRows[dr];
            if (docRow.rowID == rowID) return docRow;
        }
        return null;
    };

    /**
     * This method searches for a document row with specified row ID 'rowID' in this documents table
     * and all its descendants. It returns as soon as something is found (as there cannot be 2 rows
     * with the same row ID!)
     * 
     * @param {int} rowID - ID of the searched document row
     * @param {Array.<StaticDocumentRow (or a subclass)>} - Result is carried in this array throughout
     * 		recursion. At the end it contains the result (should be at max one doc. row). It's empty
     * 		array if nothing is found.
     */
    DocTableX.prototype.getRowByIDRecursive = function(rowID, foundDocRows) {
    	if (foundDocRows && (foundDocRows.length > 0)) {
    		// found something - return as fast as possible
    		return;
    	}
        for (var dr = 0; dr < this.docRows.length; dr++) {
            var docRow = this.docRows[dr];
            if (docRow) {
            	if (docRow.rowID == rowID) {
            		foundDocRows.push(docRow);
            		break;   // return as fast as possible
            	} else if (docRow.aCatObj) {
		    		docRow.aCatObj.executeOnAllDocReferences(function (docTableObj) {
						// ('docTableObj' is always the "current" documents table object)
		    			// recursive call of this method
		    			docTableObj.getRowByIDRecursive(rowID, foundDocRows);
					}, this);
            	}
            }
        }
    };
    
    /**
     * Recursively called method to gather all document rows of given documents table that were
     * copied from specified document row (identified by document row ID 'rowID').
     * 
     * @param {Array.<String>} rowIDs - Unique identifiers of all document rows, which copies interest us
     * @param {Array.<StaticDocumentRow or its subclass>} foundDocRows - Array of results found so far.
     */
    DocTableX.prototype.getRowsCopiedFromRowsRecursive = function (rows, foundDocRows) {
    	if (!rows || !foundDocRows) {
    		return;   // nothing to do
    	}
    	for (var dr = 0; dr < this.docRows.length; dr++) {
    		var docRow = this.docRows[dr];
    		if (docRow) {
    			_.each(rows, function (aRow) {
    				if ((aRow.rowID == docRow.copyOfExistingRowID) && (foundDocRows.indexOf(aRow) == -1)) {
    					// found another row copied from one of document row in 'rows' array
   						foundDocRows.push(docRow);
    				}
    			});
		    	if (docRow.aCatObj) {
		    		docRow.aCatObj.executeOnAllDocReferences(function (docTableObj) {
						// ('docTableObj' is always the "current" documents table object)
		    			// recursive call of this method
						docTableObj.getRowsCopiedFromRowsRecursive(rows, foundDocRows);
					}, this);
				}
    		}
    	}
	};

	
    /**
     * Get all document rows from this documents table that were created by "duplicate" operation
     * from specified "original" document row.
     * 
     * @param {StaticDocumentRow or its subclass} origDocRow - Original ("master") document row.
     * @returns {Array.<StaticDocumentRow or its subclass.} - Found duplicated rows.
     */
    DocTableX.prototype.getAllRowsClonedFrom = function(origDocRow) {
    	var clonedRows = [];
        for (var dr = 0; dr < this.docRows.length; dr++) {
            var docRow = this.docRows[dr];
            if (docRow && docRow.clonedFrom && (docRow.clonedFrom.rowID == origDocRow.rowID)) {
            	// document row cloned from the 'origDocRow' found - add it to the result 
            	clonedRows.push(docRow);
            }
        }
        return clonedRows;
    };

    
    /**
     * This method returns configuration for document sources of this documents table.
     * 
     * @returns {Object} - Configuration for the Document Link Source object ('dlsObject') needed for proper
     * 		initialization of document source actions.
     */
    DocTableX.prototype.getDocumentSourceOptions = function(phElemID) {
		// remember itself in each green link to have direct access to the document table during
		// processing of green links actions
    	var result = {
			docTable : this,
			linksId : phElemID,
			hasDocumentTemplates : this.hasDocumentTemplates,
			canUserAddDocuments : this.canUserAddDocuments,
			blankProjectCreationContext : this.isBlankProjectCreationContext()
    	};
        return result;
    };
    
    /**
     * This method will add an extra action button to the table header. This action needs to be compatible with the "ActionMenu" component.
     */
    DocTableX.prototype.addExtraAction = function(actionButton) {
		
    	// What is the name of the action.
    	var name = actionButton.attr("data-name");
    	
    	// What are all the already added actions.
    	var addedActionNames = [];
    	for (var i=0; i < this._extraActions.length; i++) {
    		addedActionNames.push(this._extraActions[i].attr("data-name"));
    	}
    	
    	// Check if this action wasn't added before yet.
    	if ($.inArray(name, addedActionNames) < 0) {
    		
    		// Add it to the list of extra actions.
        	this._extraActions.push(actionButton);
    		
    		// Let the view update the actions.
    		this.view.updateActions();
    	}
    };
    
    /**
     * This method returns all the extra actions that should be available in this documents table.
     * 
     * @returns {Array<jQuery>} - An array of jQuery objects, the elements of these objects should be valid entries of the "ActionMenu" component.
     */
    DocTableX.prototype.getExtraActions = function() {
    	return this._extraActions;
    };

    /**
     * Method to check whether this documents table is configured to show "local/remote" button next
     * to each referenced document that is remote (not stored in the documents table project).
     * 
     * @returns {Boolean} - 'true' if the documents table is configured to show "local/remote" button, 
     * 		'false' otherwise.
     */
    DocTableX.prototype.hasLocalRemoteButton = function() {
        return (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'LocalRemote') != null);
    };

	/**
	 * Resets the documents table so it contains no documents (document rows) and set its "isChanged" flag to false.
	 * @param {Boolean} full - when true, will cleanly delete the rows and trigger a change event and redraw
	 */
	DocTableX.prototype.reset = function(full) {
		if (full) {
			for (var dr = 0; dr < this.docRows.length; dr++) {
				this.docRows[dr].deleteItself();
			};
		}
		
		// reset the document rows array and related to make the documents table empty
		this.docRows = [];
		this.parentDocVerIDs = [];

		this.keepCloneFileInfoMap = {};

		this._extraActions = [];
		
		if(full){
			this.setIsChanged();
		} else {
		this.isChanged = false;
		}
	};

    /**
    * Method to specify a custom view object for this DocTableX instance. 
    * The view is responsible for rendering (visual representation) of a documents table it is assigned to.
    * 
    * @param {Object} customView - The new custom view instance to render this data model.
    * @param {Boolean} repaint - Flag whether the document row should be repainted using the new view (true) or not (false).
    */
    DocTableX.prototype.setCustomView = function(customView, repaint) {
        if (!customView) return;

        // set the basic properties of the custom view, if the calling side hasn't done it yet
        customView.dm = this;
        if (!customView.phElemID) customView.phElemID = this.phElemID;
        // by default the rendering options of all document rows are the same and inherited from the parent
        // documents table. It's mainly to have the table's header and row's columns in sync!
        if (!customView.renderOptions) customView.renderOptions = this.renderOptions;

        // from now on the view is switched to the provided view class instance
        this.view = customView;

        if (repaint) {
            this.displayTable();
        }
    };

    
    /**
     * Method to switch the whole documents table (including document source links) from read-only 
     * to editable (or vice versa) state and propagate the change to the document rows without reloading 
     * the whole page.
     * 
     * Note we need to propagate this change deeper into the document references tree-like structure as 
     * the validation of unique document names does not rely on the doc. ref. structure and thus can go 
     * past a read-only documents table to their children too!
     * 
     * @param {Boolean} newValue - New value for the this.isEditable member.
     * @param {Boolean} repaint - True if repaint of the whole doc. table is required, false otherwise.
     */
    DocTableX.prototype.setIsEditable = function(newValue, repaint) {
        if ((newValue !== undefined) && (newValue != this.isEditable)) {
        	this.isEditable = newValue;
        	// update children only if parent was actually changed!
	        for (var dr = 0; dr < this.docRows.length; dr++) {
	            this.docRows[dr].setIsEditable(newValue);
	        }
        }
        if (repaint) {
            this.displayTable();
        }
    };


    
    /**
     * Returns an StaticDocumentRow or one of its subclasses (and/or mixins) object which is the closest parent 
     * of this documents table.
     * if it's the root/main one (not in a hierarchy) null is returned.
     * 
     * @returns {StaticDocumentRow or subclass instance}
     */
    DocTableX.prototype.getParentDocRow = function() {
    	return this._parentDocRow;
    };

    /**
     * Returns an DocTableX object which is the closest parent of this documents table.
     * if it's the root/main one (not in a hierarchy) null is returned.
     * 
     * @returns {DocTableX instance}
     */
    DocTableX.prototype.getParentDocTable = function() {
    	if (this.getParentDocRow()) {
    		return this.getParentDocRow().getParentDocTable();
    	} else {
    		return null;
    	}
    };

    
    /**
     * @returns {Boolean} - True, if this documents table is (one of) the main/root documents table(s) on the page,
     * 		otherwise false is returned.
     */
    DocTableX.prototype.isRootLevelTable = function() {
   		return !this.getParentDocTable();
    };

    
    /** 
     * Returns the root documents table of this document reference tree-like structure.
     * If 'this' is the root documents table it returns itself.
     * 
     * @returns {DocTableX}.
     */
    DocTableX.prototype.getRootLevelTable = function() {
    	var rootDocTable = this;
    	while (rootDocTable && rootDocTable.getParentDocTable()) {
    		rootDocTable = rootDocTable.getParentDocTable();
    	}
    	return rootDocTable;
    };
    
    /** 
     * Returns prefix of the root/main documents table in the document references tree-like structure 
     * this documents table is part of.
     * 
     * @returns {String} Prefix of the root documents table.
     */
    DocTableX.prototype.getRootLevelPrefix = function() {
    	var rootDocTable = this.getRootLevelTable();
    	return rootDocTable ? rootDocTable.prefix : '';
    };
    
    
    /**
     * Method to set the current documents table to be inactive. Inactive documents table is not rendered
     * (any of its parts), validated or submitted, but the data model behind is kept untouched as long as 
     * the page is opened.
     * 
     * @param {Boolean} newValue - New value for the this.isInactive member.
     */
    DocTableX.prototype.setIsInactive = function(newValue) {
        if ((newValue !== undefined) && (newValue != this.isInactive)) {
        	this.isInactive = newValue;
        }
    };

    /**
     * Method to set how the new value of the corresponding document reference attribute/specification
     * has been added. By default and 99% of cases this.isInjectedValue === false. Currently the
     * only exception is specification values that are synchronized with project attributes.
     * Their values are "injected" by the application logic (without user intervention) when the
     * task is displayed on TEP or Task details -> Specifications tab.
     * 
     * @param {Boolean} newValue - New value for this.isInjectedValue member.
     */
    DocTableX.prototype.setInjectedValue = function(newValue) {
   		this.isInjectedValue = !!newValue;
    };


	/**
     * Method to get the overall status of document rows selection within the documents table.
     * 
     * @returns {Boolean} True if all document rows within this table are selected, false otherwise.
     */
    DocTableX.prototype.isAllRowsSelected = function() {
	   	for (var dr = 0; dr < this.docRows.length; dr++) {
	   		if (!this.docRows[dr].isSelected) return false;
	   	}
	   	return true;
    };

	/**
     * Method to update the state of "all document rows" selector in the documents table header.
     * It should be called whenever a change in document rows selection is made/detected.
     */
    DocTableX.prototype.updateAllRowsSelector = function() {
   		$('#' + this.phElemID + '_allRowsSelector').prop('checked', this.isAllRowsSelected());
    };

    
	/**
	 * Method to assess whether this documents table has the same document rows as provided by the parameter.
	 * The document rows are compared based on their IDs. The attributes of the documents are NOT taken into account!
	 * The properties of the corresponding references are not taken into account too.
	 * 
	 * @param {Array<StaticDocumentRow or subclasses>} otherDocRows - The array of document rows to be compared
	 * 		with 'this.docRows'.
	 * @returns {Boolean} True if the provided documents (document rows) are the same as those stored in this 
	 * 		documents table, false otherwise.
	 */
	DocTableX.prototype.hasSameContentAs = function(otherDocRows) {
	   	if ((this.docRows && !otherDocRows) || (!this.docRows && otherDocRows)) return false;
	   	if (this.docRows && (this.docRows.length != otherDocRows.length)) return false;
	   	
	   	// now go over referenced documents (document rows) and compare them one by one
	   	for (var dr = 0; dr < this.docRows.length; dr++) {
	   		var docRow = this.docRows[dr];
	   		var otherDocRow = otherDocRows[dr];
	   		if ((docRow && !otherDocRow) || (!docRow && otherDocRow)) return false;
	   		if (docRow && (docRow.docVerID != otherDocRow.docVerID)) return false;

	   		// compare properties as those can be different for the same documents too
	   		var props = (docRow.props) ? docRow.props : [];
	   		var otherProps = (otherDocRow.props) ? otherDocRow.props : [];
	   		
	   		if (props.length != otherProps.length) return false;
	   		// check the individual property values
	    	for (var pi = 0; pi < props.length; pi++) {
                var prop = props[pi];
                if (prop && prop.id) {
	                // find the property's counterpart in the otherDocRows
		         	var otherProp = otherDocRow.findPropertyByID(prop.id);
		         	if (!otherProp) return false;
		         	
		           	if (docRow.propsACatObj && docRow.propsACatObj.loaded) {
		                // find the property's corresponding attribute in the reference category
			           	// (only that one contains the most up-to-date value!)
			         	var propAttrObj = docRow.propsACatObj.findAttribute(prop.id, prop.name);
			         	if (propAttrObj) {
				         	// it is found - check the current property value against the otherDocRow
				         	if (propAttrObj.value != otherProp.value) return false;
			         	}
		           	} else {
		           		// the propsACatObj is not initialized (the UI is not ready) yet - compare the property 
		           		// values from DB (as they cannot be changed in UI anyway yet)
			         	if (prop.value != otherProp.value) return false;
		           	}
                }
        	}   // end of "for (var pi = 0..."
	   	}
	   	return true;
	};

	
	/**
	 * This method does a deep copy of all document rows from 'this' documents table to 'dstDocTable'.
	 * The document rows are not just blindly cloned, because the copied rows are not exact copies
	 * (they still have some unique properties like prefix), but rather recreated based on the information
	 * derived from the original document row (see StaticDocumentRow.getDocInfo() for more details).
	 * 
	 * @param {DocTableX} dstDocTable - Destination documents table, where all copied document rows will
	 * 		be put. Note this method doesn't empty the destination documents table, so it needs to be
	 * 		"prepared" in the caller.
	 * @param {Boolean} repaint - If true the whole destination documents table will be repainted after
	 * 		the copy operation.
	 * 
     * @returns {Promise} - Object that is resolved when all document rows of this doc table are copied.
	 */
	DocTableX.prototype.copyDocumentRowsTo = function (dstDocTable, repaint) {
		var docRowPromises = [];
		for (var dr = 0; dr < this.docRows.length; dr++) {
			var docRow = this.docRows[dr];
			docRowPromises.push(docRow.duplicateItselfTo(dstDocTable));
		}
		return Promise.all(docRowPromises).then(function() {
			if (repaint) {
				// repaint the modified documents table, if needed
				dstDocTable.displayTable();
			}
		}.bind(this));
	};


   /**
    * Method to initialize this documents table with already loaded/prepared set of document rows.
    * 
    * IMPORTANT: The document row instances needs to be already fully initialized with the correct (== this)
    * documents table as their "parent documents table"!
    * 
    * @param {Array} docRowsArr - Array of fully initialized document rows to be added to this doc. table.
    * @param {Boolean} repaint - True if repaint of the whole doc. table is required, false otherwise.
    * @returns {Boolean} True if the all went OK, false in case a document row could not be added.
    */
	DocTableX.prototype.initWithDocRows = function(docRowsArr, repaint) {
	   	if (!docRowsArr) return false;
	   	
	   	for (var dr = 0; dr < docRowsArr.length; dr++) {
	   		var docRow = docRowsArr[dr];
	   		if (docRow) {
	   			// the 'docRow' has to be already initialized with the correct (this) parent documents table!
	   			if (this !== docRow.getParentDocTable()) return false;
	   			
	   	    	// check if the added document doesn't create a cyclic dependency in the doc. ref. structure
	   	    	if (this.isDocDependentOnItself(docRow)) {
	   		        // if so, mark the added document row as being dependent on itself (in order to render 
	   	    		// it read-only with just a basic info to prevent confusion and problems with the cycle itself)
	   		        docRow.setDependsOnItself(true);
	   	        }

	   			// add the actual document row to the documents table
	   			this.docRows.push(docRow);
	   		}
	   	}
	   	// repaint if needed
	   	if (repaint) {
	   		this.displayTable();
	   	}
	   	return true;
	};


    /**
     * Private method to insert given document row instance to the 'docs' array of this documents table
     * at specified index.
     * 
     * @param {Object} docRow - Document row instance to be added to the document table.
     * @param {int} index - Index in DocTableX.docs array where to put the 'docRow' instance. 
     * 		If null, it is put at the end of the current 'docRows' array.
     * @param {Boolean} repaint - True if repaint of the whole doc. table is required, false otherwise.
     * @param {Boolean} isInternalUpdate - True if this is not adding of a new document reference value,
     * 		but just populating the document reference attribute with existing values!
     * 
     * @returns - This method returns 'true' to indicate the specified document row was added, otherwise
     * 		it returns 'false'.
     */
    DocTableX.prototype._insertRow = function(docRow, index, repaint, isInternalUpdate) {
    	// first check if the added document doesn't create a cyclic dependency in the doc. ref. structure
    	if (this.isDocDependentOnItself(docRow)) {
	        // if so, mark the added document row as being dependent on itself (in order to render 
    		// it read-only with just a basic info to prevent confusion and problems with the cycle itself)
	        docRow.setDependsOnItself(true);
        }
    	
    	var wasDocRowInserted = true;
    	
    	// add a document row instance to the document table at specified index
        // don't allow empty gaps in the DocTableX.docs array!
        if (this.allowMoreToAdd() || isInternalUpdate) {
            if ((index != null) && (index < this.getNumberOfRows())) {
                if (index < 0) index = 0;
                this.docRows.splice(index, 0, docRow);
            } else {
                // put the new row at the end of the documents table by default
                this.docRows.push(docRow);
            }
        } else {
            // handle document reference attribute with single/limited number of values with already
            // assigned value - ask the user if he wants to replace it or not...
            if (confirm(DocRefStrings.t('DocTableReplaceSingleValueQ'))) {
                // replace current document row(s) with the new one if user confirmed it
            	// 1. delete all existing document rows (from the end as it's more efficient)
            	var dr = this.docRows.length;
            	while (dr > 0) {
            		dr--;
            		this.deleteDocumentRow(dr);
            	}
            	// 2. add the new one
            	wasDocRowInserted = this._insertRow(docRow, index, false);
            } else {
                // do nothing if the user doesn't want to replace the original value
            	wasDocRowInserted = false;
            }
        }

        // repaint if needed
        if (repaint) {
            this.displayTable();
        }
        return wasDocRowInserted;
    };


    /**
     * Method to append document row at the end of the documents table
     * This method doesn't alter the 'isChanged' state and doesn't check on cycle dependencies.
     * It is used only internally to populate documents table during its initialization, not when a new 
     * document row is added by user action!
     * 
     * @param {Object} docInfo - Collection of options to initialize the created document row with. 
     * 		See DocumentRow constructor for details about supported options.
     * @param {function} rowClass - The constructor function of document row class of the document row to be
     * 		created and added to the documents table.
     * @param {Boolean} repaint - If true the documents table is repainted after the row is added,
     * 			no repaint otherwise.
     * @returns {Promise} - Object that is resolved when the specified document row is appended.
     */
    DocTableX.prototype.appendDocRow = function(docInfo, rowClass, repaint) {
        // make sure the correct parent documents table reference is set
        docInfo.parentDocTable = this;
        if (this.targetFolders) {
        	docInfo.targetFolders = this.targetFolders;
        }

        if (this.autoExpandAllRows) {
        	// auto-expand all documents rows if the documents table is set to do so...
        	docInfo.aCatExpanded = true;
        }
        
        // create a new instance of appropriate document row and add it to this documents table
        return documentRowFactory.createDocumentRow(rowClass, docInfo).then(function (docRow) {
        	// sending null to 'index' parameter adds the new document row at the bottom of the documents table
	        return this._insertRow(docRow, null, repaint, true);
        }.bind(this));
        
        // no 'setIsChanged()', because this method is used only internally to fill the documents table with 
        // already existing document rows...
    };

    /**
     * Helper method to simplify appending of StaticDocumentRow instances to documents table. 
     * Internally it calls the appendDocRow() method.
     * 
     * @param {Object} docInfo - Collection of options to initialize the created document row with.
     * @param {Boolean} repaint - If true the documents table is repainted after the row is added,
     * 			no repaint otherwise.
     * @returns {Promise} - Object that is resolved when the specified document row is appended.
     */
    DocTableX.prototype.appendStaticDocRow = function(docInfo, repaint) {
    	return this.appendDocRow(docInfo, StaticDocumentRow, repaint);
    };

    
    /**
     * Method to add a new document row to the document table. This method is called when user adds a new
     * document row.
     * 
     * @param {Object} docInfo - Collection of options to initialize the created document row with. 
     * 		See DocumentRow constructor for details about supported options.
     * @param {function} rowClass - The constructor function of document row class of the document row to be
     * 		created and added to the documents table.
     * @param {Boolean} repaint - Value true enables repaint of the documents table right after the row is inserted. 
     * @param {Boolean} isAddedByUser - True if the added document was added directly by user, false if 
     * 		it's during a render/load operation.
     * @param {int} index - Index specifying place where to insert specified document row in doc. table.
     * 		If its undefined (or null) the new document row is added at the bottom of the documents table.
     * 
     * @returns {Promise} - Object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addDocRow = function(docInfo, rowClass, repaint, isAddedByUser, index) {
        // make sure the correct parent documents table reference is set
        docInfo.parentDocTable = this;
        if (this.targetFolders) {
        	docInfo.targetFolders = this.targetFolders;
        }
        
        // create a new instance of appropriate document row and add it to this documents table
        return documentRowFactory.createDocumentRow(rowClass, docInfo).then(function (docRow) {
	        if (isAddedByUser && this.isDocDependentOnItself(docRow)) {
	            // the same document version ID found in the list of parent documents => a cycle detected!
	        	// as the user adds the document, give him/her chance to decide whether to really do that.
	            if (!confirm(DocRefStrings.t('DocTableCycleDependencyWarning'))) {
	                // don't add the document if the user doesn't want to create a cyclic dependency!
	            	if (docRow) {
		            	// destroy the already created document row object (with all its data)
	            		docRow.destroy(true);
	            	}
	                return null;
	            }
	        }
	        // put the new document row at its place in the documents table
	        if (!index) index = 0;   // by default the user added document row is prepended to the existing rows.
	        if(this._insertRow(docRow, index, repaint)){
	        	return docRow;
	        } else {
	        	// row was not inserted
            	if (docRow) {
	            	// destroy the already created document row object (with all its data)
            		docRow.destroy(true);
            	}
	        	return null;
	        }
        }.bind(this));
    };

    
    /**
     * Method to clone a new document row and add it to this documents table. 
     * This method is called when user clicks clone button in the document row.
     * 
     * @param {int} index - Index specifying place where to insert specified document row in doc. table.
     * 		If its undefined (or null) the new document row is added at the bottom of the documents table.
     */
    DocTableX.prototype.addCloneRowAction = function(index) {
    	// clone current (clicked) row
    	this.addCloneRow(this.docRows[index], index);
    };

    
    /**
     * Method to clone specified document row 'origDocRow' and add it to this documents table at place
     * defined by 'index'. 
     * 
     * @param {StaticDocumentRow or subclass} origDocRow - document row to be cloned.
     * @param {int} index - Index specifying place where to insert specified document row in doc. table.
     * 		If its undefined (or null) the new document row is added at the bottom of the documents table.
     * 
     * @returns {Promise} - Object that is resolved when the cloned document row is added.
     */
    DocTableX.prototype.addCloneRow = function(origDocRow, index) {
    	// make sure the copied reference properties have the most up-to-date values (covers the case
    	// when user already changed a property value and only then clicked on the "Duplicate" button...)
    	origDocRow.updatePropertyValues();
    	
    	// pass the current Document & get cloned new doc Row 
    	return this.createDocRowObj(origDocRow, 'Clone').then(function (docRow) {
	    	// put the new document row at its place in the documents table
	        if ((index !== undefined) && (index !== null)) {
	        	if (index < 0) index = 0;   // by default the user added document row is prepended to the existing rows.
	        	else index++;
	        } else {
	        	index = null;
	        }
	        if (this._insertRow(docRow, index, true)) {
	        	this.setIsChanged();
	        }
    	}.bind(this));
    };

    /**
     * This method is used to rename the file while doing clone operation
     * 
     *@param {Object} docInfo - Document (its properties) . See DocumentRow constructor for details 
     * 		about supported properties.
     */
    DocTableX.prototype.getClonedDocName = function(docRow) {
    	var docName =  docRow.docName;
    	if (docName) {
	    	var docNameSequence  = this.keepCloneFileInfoMap[docName];
	    	if (!docNameSequence) {
	    		docNameSequence = 1;
			} else {
				docNameSequence += 1;
			}
	    	// remember the value of the sequence counter for the particular document (original) name
	    	this.keepCloneFileInfoMap[docName] = docNameSequence;
    	
    		var fileExt  = docName.indexOf(".") < 0 ? "" : docName.slice(docName.lastIndexOf(".") + 1);
        	var fileName = docName.indexOf(".") < 0 ? docName : docName.slice(0, docName.lastIndexOf("."));
        	docName = fileName + '-Duplicate-'+ docNameSequence  + '.' + fileExt;
    	}
    	return docName;
    };
    
    /**
     * Method to add multiple new documents to the document table picked from a search dialog. 
     * It supports adding a group of documents to non-empty documents table at given position maintaining
     * order in the group.
     * 
     * @param {Array<Object>} docs - List of documents (their properties) to be added. 
     * 		See DocumentRow and DocumentRowSearch constructors for details about supported properties.
     * @param {function} rowClass - The constructor function of document row class of the document row to be
     * 		created and added to the documents table.
     * @param {Boolean} isAddedByUser - True if the added document was added directly by user, false if 
     * 		it's during a render/load operation.
     * 
     * @returns {Promise} - Promise object that is resolved when all specified document rows are created and added.
     */
    DocTableX.prototype.addDocRows = function(docInfos, rowClass, isAddedByUser) {
        var anyDocRowAdded = false;
        var docRowPromises = [];
        // go over each "document" object and add it to the documents table at its right place
        for (var idx = 0; idx < docInfos.length; idx++) {
            var isLastItem = (idx == docInfos.length-1);
            docRowPromises.push(this.addDocRow(docInfos[idx], rowClass, isLastItem, isAddedByUser, idx).then(function (newRow) {
            	if (newRow) {
	                anyDocRowAdded = true;
	            }
            }.bind(this)));
        }

        return Promise.all(docRowPromises).then(function() {
	        if (anyDocRowAdded && isAddedByUser) {
	            // at least one new row was added to the doc. table by user
	            this.setIsChanged();
	        }
        }.bind(this));
    };
 
    /**
     * Specialized variation of this.addDocRows() method adding/creating just DocumentRowSearch instances.
     * 
     * @returns {Promise} - Promise object that is resolved when all specified document rows are processed.
     */
    DocTableX.prototype.addSearchDocRows = function(docInfos, isAddedByUser){
        return this.addDocRows(docInfos, DocumentRowSearch, isAddedByUser);  
    };

    /**
     * Add a new document based on "Upload" (browse) document source.
     * 
     * @param {DOM Object} dlsElem - DOM object of the clicked document link source. It contains 
     * 		a reference to corresponding Document Link Source instance with all needed data/setup. 
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addBrowseRow = function(dlsElem) {
        var dlsObj = dlsElem.dlsObj;

        // create a new document row and add it to the master (this document table)
        var docInfo = {
            parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            targetFolderName : dlsObj.targetFolder,  // foldername as specified in the configuration
            allowedFileExts : DocTableXUtils._getFileExtensionsList(dlsObj.documentType),
            docTempType : dlsObj.docTemplateType,
            docTempNames : DocTableXUtils._getDocTemplateNameList(dlsObj.docTemplateName),
			defaultTargetFolderName : dlsObj.targetFolderOriginal
        };        

        // put the new document row at the top of the documents table and repaint it
        return this.addDocRow(docInfo, DocumentRowBrowse, true, true).then(function (newRow) {
        	if (newRow) {
	        	// a new row was added to the doc. table by user
	            this.setIsChanged();
	        }
        }.bind(this));
    };

    /**
     * Add a new document based on "Upload" (browse) document source.
     * 
     * @param {DOM Object} dlsObj - object representing the document link source instance with all 
     * 		needed data/setup. 
     * @param {File} file - file to be attached to the row
     * @param [Boolean] repaint - repaint or not? - Defaults to true
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addFileRow = function(dlsObj, file, repaint) {
        // create a new document row and add it to the master (this document table)
        var docInfo = {
        	file: file,
    		parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            targetFolderName : dlsObj.targetFolder,  // foldername as specified in the configuration
            allowedFileExts : DocTableXUtils._getFileExtensionsList(dlsObj.documentType),
            docTempType : dlsObj.docTemplateType,
            docTempNames : DocTableXUtils._getDocTemplateNameList(dlsObj.docTemplateName),
			defaultTargetFolderName : dlsObj.targetFolderOriginal
        };

        // put the new document row at the top of the documents table
        return this.addDocRow(docInfo, DocumentRowFile, repaint!==false, true).then(function (newRow) {
        	if (newRow) {
	        	// a new row was added to the doc. table by user
	            this.setIsChanged();
	        }
        }.bind(this));
    };
    
    
    /**
     * Add a new document based on a DAM document source. This row has a reference to an external file.
     * 
     * @param {DOM Object} dlsElem - DOM object of the clicked document link source. It contains 
     * 		a reference to corresponding Document Link Source instance with all needed data/setup. 
     * @param {Object} options - contains extra fields about the external file
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addExternalFileRow = function(dlsElem, options) {
        var dlsObj = dlsElem.dlsObj;
        
        var docInfo = {
            parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            targetFolderName : dlsObj.targetFolder,  // foldername as specified in the configuration
            allowedFileExts : DocTableXUtils._getFileExtensionsList(dlsObj.documentType),
            docTempType : dlsObj.docTemplateType,
            docTempNames : DocTableXUtils._getDocTemplateNameList(dlsObj.docTemplateName),
            DAM: options.DAM,
            DAMJS: options.DAMJS,
            DAMData: dlsObj.DAMMetadata,
            docName: options.docName,
            docDesc: options.docDesc,
            externalId: options.externalId,
            thumbnailURL: options.thumbnailUrl,
            externalDocumentMetadata: options.documentMetadata,
			defaultTargetFolderName : dlsObj.targetFolderOriginal
        };

        // put the new document row at the top of the documents table and repaint it
        return this.addDocRow(docInfo, DocumentRowExternalFile, true, true).then(function (newRow) {
        	if (newRow) {
	            // a new row was added to the doc table by user
	            this.setIsChanged();
	        }
        }.bind(this));
    };

    
    /**
     * Add a new document based on "Copy" (Select from Project) document source.
     * 
     * @param {DOM Object} dlsElem - DOM object of the clicked document link source. It contains a 
     * 		reference to corresponding Select from Project document source instance with all needed data/setup.
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addCopyRow = function(dlsElem) {
        var dlsObj = dlsElem.dlsObj;

        // create a new document row and add it to the master (this document table)
        var docInfo = {
            parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            docOperation : dlsObj.operation,
            targetFolderName : dlsObj.targetFolder,  // foldername as specified in the configuration
            docTempType : dlsObj.docTemplateType,
            docTempNames : DocTableXUtils._getDocTemplateNameList(dlsObj.docTemplateName),
			defaultTargetFolderName : dlsObj.targetFolderOriginal
        };        
    	
        // put the new document row at the top of the documents table and repaint it
        return this.addDocRow(docInfo, DocumentRowCopy, true, true).then(function (newRow) {
        	if (newRow) {
	            // a new row was added to the doc. table by user
	            this.setIsChanged();
	        }
    	}.bind(this));
    };
    

    /**
     * Add a new document based on "Link" document source.
     * This means that we offer a list of all the documents in the current project, 
     * and the user can choose to add the document to the attribute valie 'ByReference' !
     * 
     * @param {DOM Object} dlsElem - DOM object of the clicked document link source. It contains 
     * 		a reference to corresponding Document Link Source instance with all needed data/setup. 
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.addLinkRow = function(dlsElem) {
        var dlsObj = dlsElem.dlsObj;

        // create a new document row and add it to the master (this document table)
        var docInfo = {
            parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            docOperation : dlsObj.operation,
			defaultTargetFolderName : dlsObj.targetFolderOriginal
        };

        // put the new document row at the top of the documents table and repaint it
        return this.addDocRow(docInfo, DocumentRowLink, true, true).then(function (newRow) {
        	if (newRow) {
	            // a new row was added to the doc. table by user
	            this.setIsChanged();
	        }
    	}.bind(this));
    };

    /**
     * Add a new document based on "New" (Document Template) document source.
     * 
     * @param {DocumentRowNew} docRow - Preinitialized document row based on the "New" document source.
     */
    DocTableX.prototype.addDocTemplateRow = function(docRow) {
        if (!docRow) return;
        
        if (this.replaceDocRowAdded){
        	this.displayTable();
        } else {
        	// put the new document row at the top of the documents table and repaint it
	        if (this._insertRow(docRow, 0, true)) {
	            // a new row was added to the doc. table by user
	            this.setIsChanged();
	        }
        }
    };


    /**
     * This method initiates the creation of DocumentRowNew instance. It's added to the documents table
     * only after user picks a valid document template (from doc. template selection dialog that is
     * opened first).
     * @param {DOM Object} dlsElem - DOM object of the clicked document link source. It contains 
     * 		a reference to corresponding Document Source New instance with all needed data/setup. 
     * 
     * @returns {Promise} - Promise object that is resolved when the specified document row is added.
     */
    DocTableX.prototype.openDocTemplateDialog = function(dlsElem) {
        var dlsObj = dlsElem.dlsObj;

        // specify info needed to create a new document row (of type DocumentRowNew) and create it
        var docInfo = {
            parentDocTable : this,
            dlsName : dlsObj.label.getLabel(),
            targetFolderName : dlsObj.targetFolder,
            docTempType : dlsObj.docTemplateType,
            docTempNames : DocTableXUtils._getDocTemplateNameList(dlsObj.docTemplateName),
			defaultTargetFolderName : dlsObj.targetFolderOriginal,
			targetFolders: this.targetFolders
        };

        // on its creation, the DocumentRowNew instance first opens the document template selection dialog
        // and let the user select a document template (if not a fixed one). only as a consequence of selecting
        // the document template the DocumentRowNew instance is updated and added to the documents table.
        // check the selection dialog's "on OK" handler method.
        return documentRowFactory.createDocumentRow(DocumentRowNew, docInfo);
    };

    
    /**
     * Method returning an array of document template names extracted from the string passed via 
     * 'docTempNamesStr' parameter.
     * 
     * @param {String} docTempNamesStr - A string containing list of user specified doc. template names.
     * @returns {Array<String>} - Array of processed doc. template names from the input parameter.
     */
    DocTableX.prototype.getDocTemplateNameList = function(docTempNamesStr) {
        return DocTableXUtils._getDocTemplateNameList(docTempNamesStr);
    };



    /**
     * Method to identify whether the specified document row is responsible for a cyclic dependency, because it 
     * contains a document that is already present in an upper level of a tree-like hierarchy defined by 
     * doc. references. See also description of 'this.parentDocVerIDs' for more details.
     * 
     * @param {DocumentRow (or a subclass)} aDocRow - Document row for which to check the cyclic dependency existence.
     * @returns {Boolean} - Returns 'true' if the document referenced in the specified document row creates a
     * 		cyclic dependency, 'false' otherwise.
     */
    DocTableX.prototype.isDocDependentOnItself = function(aDocRow) {
        var pdvIDs = this.parentDocVerIDs;
        return (aDocRow.docVerID && pdvIDs && (pdvIDs.indexOf(aDocRow.docVerID) != -1));
    };


    /**
     * @returns {int} How many documents are in the doc. table.
     */
    DocTableX.prototype.getNumberOfRows = function() {
        return this.docRows.length;
    };

    /**
     * @returns {Boolean} True if it's possible to add more documents in the doc. table, false otherwise.
     */
    DocTableX.prototype.allowMoreToAdd = function() {
        return this.allowMultiple || (this.getNumberOfRows() < 1);
    };

    /**
     * Controller method to start rendering of the documents table.
     */
    DocTableX.prototype.displayTable = function() {
    	
    	if (this.isModifiedElsewhere) {
    		// make sure documents table with the 'isModifiedElsewhere' flag is always rendered read-only
    		// (the original "editable" state is stored in 'this.storedState.isEditable')
    		this.setIsEditable(false, false);
    	}
    	
    	// render the main structure of the table (up to individual <tr/> elements with correct IDs)
        var phElem = this.view.render();

        if (phElem && (phElem.length == 1)) {
            // if rendering of the doucments table succeeded store the current DocTableX instance itself 
            // in DOM to be accessible by action handlers in generated HTML
            phElem[0].docTable = this;
        }
    };

    /**
     * Method generating XML to save changes done in document table. 
     * Currently optimized for doc. reference attributes!
     * 
     * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
     */
    DocTableX.prototype.convertToXML = function() {
        var xmlValue = '';
        try {
            // the root XML tag has to be there even if the document table is empty!
            // (especially if it is made empty by deleting all the document rows)
            xmlValue += '<' + this.saveXMLRootTag + '>';
            if (!this.isInjectedValue && ((!this.isModifiedElsewhere && !this.isEditable) || (this.isModifiedElsewhere && !this.storedState.isEditable))) {
            	// for read-only document rows which are not changed (by application logic) generate 
            	// the special <readonly> tag to prevent back-end from attempts to (re)save read-only 
            	// document reference values!
            	xmlValue += WCXML.ElementEncode('readonly', '1');
            }
            for (var dr = 0; dr < this.docRows.length; dr++) {
                xmlValue += this.docRows[dr].convertToXML();
            }
            xmlValue += '</' + this.saveXMLRootTag + '>';
        } catch(err) {
            DocTableXUtils.log('Problem in DocTableX.convertToXML:\n' + err);
            xmlValue = '';
        }
        return xmlValue;
    };
    
    /**
     * This method generates a String of pipe ('|') separated document names from the document
     * rows of this documents table.
     * It's primarily used as "plain" document reference value used in project header view etc.
     * 
     * @returns {String}
     */
    DocTableX.prototype.getDocumentNames = function() {
    	var result = '';
        for (var dr = 0; dr < this.docRows.length; dr++) {
        	var docRow = this.docRows[dr];
        	if (docRow && docRow.docName) {
                result += docRow.docName + ' | ';
        	}
        }
        return result.slice(0, result.length-3);   // (to remove the last pipe)
    };
    
    /**
     * This method generates a String of pipe ('|') separated hyperlinks to WebCenter documents 
     * from the document rows of this documents table.
     * It's primarily used as "plain" document reference value used in project header view etc.
     * 
     * @returns {String}
     */
    DocTableX.prototype.getHyperlinks = function() {
    	var result = '';
        for (var dr = 0; dr < this.docRows.length; dr++) {
        	var docRow = this.docRows[dr];
        	if (docRow && docRow.docVerID && docRow.docName) {
        		result += '<a class="'+window.webcenterConfig.jspBaseUrl+'docrefdoclink darktext_darklink_11" href="docdetails.jsp?menu_file=myprojects&amp;docVerID=' + docRow.docVerID + '" target="_blank">' + WCXML.xmlencode(docRow.docName) + '</a>' + ' | ';
        	}
        }
        return result.slice(0, result.length-3);   // (to remove the last pipe)
    };
    
    
    /**
     * Validation of the documents table content versus required constraints.
     * This recursively called function goes as deep as the document reference attribute values expansion
     * went in the UI. In other words. Referenced documents and their attributes not touched and visible
     * by the user are not validated (as the user could not do any change there).
     * The validation is done regardless the documents table is editable or not, because any descendant
     * can be editable even if this table is read-only...
     * 
     * @param {Object} validObj - Validation object holding all found validation errors together with the related 
     * 		data needed to process them and notify user. 
     * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
     * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
     * 		starting from 1.
     */
    DocTableX.prototype.validateDocTable = function(validObj, depth) {
    	if (this.isInactive || this.isModifiedElsewhere) {
    		// don't validate hidden or "slave" (modified from elsewhere) fields in any case
    		return;
    	}
    	
	    for (var dr = 0; dr < this.docRows.length; dr++) {
        	var docRow = this.docRows[dr];
        	// run validation on each document row in the documents table
      		docRow.validateDocRow(validObj, depth);
        }
    };


    /**
     * Method to be called when the changes in document table are submitted to the back-end.
     * It invokes generation of XML to be passed to back-end to process/save all the changes and
     * put the XML to dedicated hidden input field.
     * If and only if this.clearValues == true, all existing (and newly added) document rows are removed,
     * no matter if the documents table is hidden, read-only or editable!
     */
    DocTableX.prototype.onSubmit = function() {
    	// Firefox has problems with submitting forms without clicking a submit button.
    	// It only seems to go wrong when using document references as it uses removing of DOM elements heavily and it seems to trigger the problem.
    	// Because forms are consistently being used wrongly in WebCenter, we cannot fix it everywhere, so we add a hack on affected places.
    	$("form").each(function(i, form) {
    		if ($(form).find(".forceSubmit").length === 0) {
    			$(form).append('<input type="submit" class="forceSubmit displayNone" />');
    			
    			var originalSubmit = form.submit;
    			form.submit = function() {
    				form.submit = originalSubmit;
    				$(form).find(".forceSubmit").click();
        		};
    		}
    	});
    	
    	if (this.clearValues) {
    		// wipe out all the document rows! This has to be (and is) done only after a successful validation
    		// and just before a submit. Otherwise it will produce weird results!
    		this.docRows = [];
    	}
    	if (!this.isInactive || this.clearValues) {
	        // find the hidden element with the XML to save the changes
	        var saveXMLElemID = this.getTablePrefix();
	        var saveXMLElem = $('#' + saveXMLElemID);
	        if (saveXMLElem.length > 0) {
	        	var xml = this.convertToXML();
	            saveXMLElem.val(xml);
	        }
    	}
    	if (!this.isInactive) {
        	// propagate "on submit" event to the descendants even if this documents table is read-only, 
        	// because the descendants might be editable! (based on folder permissions / setup).
    		// however if the documents table is hidden, all changes are ignored (not saved)!
	        for (var dr = 0; dr < this.docRows.length; dr++) {
	        	this.docRows[dr]._onSubmit();
	        }
    	}
    };




    // Action handler methods =====================================


    
//JANH CR remark: Why this action handle has so many (and unrealated) parameters?! => because it's used in
// 	"replace document row" scenario as well!
    /**
     * Handles request to make given remote document local and replace document to 'this.projectID' project.
     * 
     * @param {int} rowIdx - Index of the document row, where is the document to be made local.
     * @param {String} flowType - This param is required for localRemote Flow. Based on this param docrow object initiated.
	 * @param {String} operation - Operation which is clicked during replace.
	 * @param {String} templateName - TemplateName will be passed only in case of 'DocumentTemplate' (NEW) operation.
	 * @param {DocumentRowNew} docRow - Preinitialized document row based on the "SavedSearch" document source.            
     */
    DocTableX.prototype.localRemoteButtonClicked = function(rowIdx, flowType, operation, templateName, searchDocRow) {
        if (rowIdx < 0) return;   // do nothing if the input is not OK
        
        // replace the original document row with the newly created DocumentRowSearch row with the original document preselected
        // to be copied to this project
        var docRow = this.docRows[rowIdx];

    	// remove the original document row HTML from the page leaving there the row's place holder element
    	docRow.deleteItself(true);

        if (docRow.origDocRow) {
        	// there is an original document row remembered that was at this place 
        	// before - replace this row by the original one
        	this.docRows[rowIdx] = docRow.origDocRow;
            // render the new document row HTML on the page
            this.docRows[rowIdx].displayRow();
        } else {
	        // initialize the new document row (representing the new local document) from the original one
        	this.createDocRowObj(docRow, operation, templateName, searchDocRow, flowType).then(function (docRow) {
        		this.docRows[rowIdx] = docRow;
        	this.setIsChanged();
        // render the new document row HTML on the page
        this.docRows[rowIdx].displayRow();
        	}.bind(this));
        }
    };
    
    

    /**
     * Create document row object based on replace Type and return a Promise object handling the 
     * asynchronicity of document row creation. 
     * 
     * @param {docrow} docRow - Document row of replaced item 
	 * @param {String} operation - Operation which is clicked during replace.
	 * @param {String} templateName - This parameter is passed only 'New' DocumentTemplate operation
	 * @param {DocumentRowNew} docSearchRow - Preinitialized document row based on the "SavedSearch" document source.   
	 * @param {String} flowType - This parameter is passed only in case of 'LocalRemote'           
	 * 
	 * @returns {Promise} - Promise object representing the initialized/created document row 
     */
    DocTableX.prototype.createDocRowObj = function(docRow, operation, templateName, docSearchRow, flowType) {
        
    	var initDocRowPromise = null;
    	
    	var docInfo = {
			parentDocTable : this,
			docVerID : docRow.docVerID,
			docID : docRow.docID,
			targetFolderName : docRow.targetFolderName,
			projectID : this.projectID,
		    docTempType : docRow.docTemplateType,
		    docTempNames : docRow.docTemplateName,
		    operation : operation,
			properties : docRow.props,   
			origDocRow : docRow,
			flowType: flowType,
			targetFolders: this.targetFolders
		};
    	
    	if (docSearchRow) {
    		docSearchRow.parentDocTable = this;
    		docSearchRow.dlsName = DocRefStrings.t('DocRowReplaceSearchAction');
    		docSearchRow.origDocRow = docRow;   
    		docSearchRow.properties = docRow.props;
            if (this.targetFolders) {
            	docSearchRow.targetFolders = this.targetFolders;
            }
    		initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowSearch, docSearchRow);
	    } else if (flowType == 'LocalRemote') {
			 // DocRow creation for Local Remote operation
			 docInfo.dlsName = DocRefStrings.t('DocRowMakeLocalAction');
			 docInfo.docName = docRow.docName;
			 docInfo.operation = 'Copy';
			initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowSearch, docInfo);
		} else if (operation == 'Upload') {
			 // DocRow creation for Upload operation
			 docInfo.dlsName = DocRefStrings.t('DocRowReplaceUploadAction');
			initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowBrowse, docInfo);
		} else if (operation == 'Reference' || operation == 'LinkToTask'){
			 // DocRow creation for Link operation
			 docInfo.dlsName = DocRefStrings.t('DocRowReplaceLinkAction');
			initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowLink, docInfo);
		} else if (operation == 'Copy') {
			docInfo.dlsName = DocRefStrings.t('DocRowReplaceNewAction');
			//JANH TODO review this "templateName" logic!!!
			docInfo.docTempNames = templateName;
			initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowNew, docInfo);
			this.replaceDocRowAdded = true;
	  	} else if (operation == 'Clone') {
	  		//  DocRow creation for Clone 
	  		docInfo.dlsName = DocRefStrings.t('DocRowCloneAction');
	  		docInfo.docName = docRow.docName;   //JANH TODO this doesn't work as the document name is overwritten by info coming from server!
	  		docInfo.drID = null;
	  		docInfo.origDocRow = null;
	  		docInfo.clonedFrom = docRow;
	  		docInfo.operation = 'Copy';
	  		
	  		if (docRow.docOperation === 'New' && docRow.selDocTemplate) {
	  			docInfo.docVerID = docRow.selDocTemplate.docVerID;
	  			docInfo.docTypeID = docRow.selDocTemplate.docTypeID;
	  			docInfo.docName = docRow.selDocTemplate.name;
	  			docInfo.selDocName = null;
	  		}
	  		
	  		initDocRowPromise = documentRowFactory.createDocumentRow(DocumentRowSearch, docInfo);
        }
		return initDocRowPromise;
    };
    

    /**
     * Action handler handling user click on "Delete/Remove" button.
     * 
     * @param {int} rowIdx - Index of the document row to be removed.
     */
    DocTableX.prototype.deleteButtonClicked = function(rowIdx) {
    	this.deleteDocumentRow(rowIdx);
    };
    
    /**
     * Handles removal of a document row from documents table.
     * 
     * @param {int} rowIdx - Index of the document row to be removed.
     */
    DocTableX.prototype.deleteDocumentRow = function(rowIdx) {
        if (rowIdx < 0) return;   // the row probably doesn't exist already

        var delRow = this.docRows[rowIdx];

    	// remove the document row at given index from the table (first the HTML)
    	delRow.deleteItself();

    	// normal delete operation - now delete data model (element of this.docRows array)
        this.docRows.splice(rowIdx, 1);

        this.setIsChanged();

        
        if (this.getNumberOfRows() > 0) {
            // go over all the potentially effected document rows and repaint them to update their visual status
            var firstEffectedRowIdx = (rowIdx - 1 < 0) ? 0 : rowIdx - 1;
            for (var dr = firstEffectedRowIdx; dr < this.getNumberOfRows(); dr++) {
                this.docRows[dr].view.repaintRow();
            }
        } else {
            // in case the documents table remains empty, repaint it as header may change too!
            this.displayTable();
        }
        
    };


    /**
     * Handles manual (re)ordering of document rows in the doc. table. This one is called when 
     * the particular doc. row should be moved up.
     * 
     * @param {int} rowIdx - Index of the document row to be moved.
     */
    DocTableX.prototype.moveUpButtonClicked = function(rowIdx) {
        if (rowIdx > 0) {
            // swap the two adjacent rows so the current one (with rowIdx) goes up (gets lower index).
            // first the data model
            var tmpRow = this.docRows[rowIdx-1];
            this.docRows[rowIdx-1] = this.docRows[rowIdx];
            this.docRows[rowIdx] = tmpRow;

            // and then the HTML (to reflect the change in data model)
            this.docRows[rowIdx].swapWith(this.docRows[rowIdx-1]);

            this.setIsChanged();
        }
    };


    /**
     * Handles manual (re)ordering of document rows in the doc. table. This one is called when 
     * the particular doc. row should be moved down.
     * 
     * @param {int} rowIdx - Index of the document row to be moved.
     */
    DocTableX.prototype.moveDownButtonClicked = function(rowIdx) {
        if (rowIdx < this.getNumberOfRows()-1) {
            // swap the two adjacent rows so the current one (with rowIdx) goes down (gets higher index).
            // first the data model
            var tmpRow = this.docRows[rowIdx+1];
            this.docRows[rowIdx+1] = this.docRows[rowIdx];
            this.docRows[rowIdx] = tmpRow;

            // and then the HTML (to reflect the change in data model)
            this.docRows[rowIdx+1].swapWith(this.docRows[rowIdx]);

            this.setIsChanged();
        }
    };


    /**
     * Handles expanding and collapsing of all descendants.
     */
    DocTableX.prototype.expandCollapseAllClicked = function(elem) {
        var elemObj = $(elem);
        if (elemObj.hasClass('expandAll')) {
            // expand all rows of this documents table (but not the descendants!)
            this.expandAllRows();
        } else if (elemObj.hasClass('collapseAll')) {
            // initiate a recursion to collapse all descendants
            this.collapseAllRows();
        }
    };

    /**
     * Method to expand all rows of this documents table. It cooperates with DocumentRow and 
     * AttributeCategory objects to achieve that.
     * Note this does not expand the descendants in the doc. ref. tree-like structure (just the first level).
     */
    DocTableX.prototype.expandAllRows = function() {
        for (var dr = 0; dr < this.docRows.length; dr++) {
            this.docRows[dr].showDocAttributeCategory(false, true);
        }
    };

    /**
     * Method to collapse all rows of this documents table. It cooperates with DocumentRow and 
     * AttributeCategory objects to achieve that.
     * The collapsion is done from the leafs to the root (of the doc. ref. tree-like structure).
     */
    DocTableX.prototype.collapseAllRows = function() {
        for (var dr = 0; dr < this.docRows.length; dr++) {
            this.docRows[dr].collapseAllDescendants();
        }
    };
    
    
    /**
     * Selects all document rows within the documents table. It's called when "select all" checkbox in the table header 
     * is checked.
     * 
     * @param {DOM element} elem - The "select all" checkbox that was clicked (caused this action).
     */
    DocTableX.prototype.onSelectAllRowsClicked = function(elem) {
    	this.setSelectAllRows(elem && elem.checked);
    };

    /**
     * Selects all document rows within the documents table. It's called when "select all" checkbox 
     * in the table header is checked.
     * 
     * @param {Boolean} isSelected - Flag defining whether the document rows should be selected or deselected.
     */
    DocTableX.prototype.setSelectAllRows = function(isSelected) {
    	// (de)select all document rows in this table based on the 'isSelected' parameter value
        for (var dr = 0; dr < this.docRows.length; dr++) {
        	var docRow = this.docRows[dr];
        	docRow.selectAllDescendants(isSelected);
        }
        // don't forget to update the "all rows" selector in the header of this documents table
   		this.updateAllRowsSelector();
    };

    
    /**
     * Handles "download selected documents" action including the validation of selected document(s)
     * and the current user rights to download those document(s).
     */
    DocTableX.prototype.downloadActionClicked = function() {
    	// validate selected documents eligibility to be downloaded by the current user
    	var validationObj = this.validateDocDownloadAction();
    	
    	if (validationObj.rowsToDownload.length > 0) {
		if (validationObj.lockedDocFound) {
				// if there is a locked document(s) selected, show a warning/info message
			if (!confirm(DocRefStrings.t('DocTableLockedDocDownloadWarning'))) {
				return;
			}
		}
		if (validationObj.noPermDocNames.length > 0) {
				// show a warning message that some of the selected documents cannot be downloaded
				var msg = DocRefStrings.t('DocTableNoDownloadPermInfo') + ':\n';
				alert(msg + validationObj.noPermDocNames.join('\n'));
			}
    	} else {
			// if there is a serious validation issue the request to download documents is not made
			if (validationObj.noPermDocNames.length > 0) {
				// the user cannot download any of the selected documents
				alert(DocRefStrings.t('DocTableNoDownloadPermError'));
	     		return;
			} else {
				// there is no document selected
				alert(DocRefStrings.t('DocTableSelectDoc'));
	     		return;
			}
		}

		// validation OK
		
		// find the closest <form> element related to this documents table and submit it with 
		// the "dowload documents" request.
		var tableForm = $('#' + this.phElemID).closest("form");
		if (tableForm.length > 0) {
			// make sure the "actionVal" hidden input is defined on the page and set its value before submit
			var actionValInput = tableForm.find('input[name="actionVal"]');
			if (actionValInput.length == 0) {
				actionValInput = $('<input type="hidden" name="actionVal"/>');
				tableForm.append(actionValInput);
			}
			actionValInput.val('DocTableXDownloadDocuments');

			// make sure the "rootPrefixVal" hidden input is defined on the page and set its value before submit
			var rootPrefixValInput = tableForm.find('input[name="rootPrefixVal"]');
			if (rootPrefixValInput.length == 0) {
				rootPrefixValInput = $('<input type="hidden" name="rootPrefixVal"/>');
				tableForm.append(rootPrefixValInput);
			}
			// the "rootPrefixVal" input holds the value of the root documents table prefix, which is used
			// to prefix the name of all row selectors in the documents table and all its children doc. tables
			rootPrefixValInput.val(this.getTablePrefix());

			if (this.onBeforeDownloadAction) {
				// execute the "preparation" for download. It's mainly to prevent "Do you want to leave page" 
				// message on various pages, where the DocTableX may be...
				this.onBeforeDownloadAction();
			}
			
			// submit the request to download selected documents (note the page is not really reloaded 
			// in this case even if it's a submit!)
			this.pageActionForm.submit();
		} else {
			window.alert('The corresponding form is not defined / cannot be found!');   //JANH debug - then DEL!!!
		}
    };


    DocTableX.prototype.validateDocDownloadAction = function(validObj) {
    	if (!validObj) {
    		// create the validation object holding data gathered during the validation process
    		validObj = {
				rowsToDownload : [],
				lockedDocFound : false,
				noPermDocNames : []   // names of documents which the user cannot download
    		};
    	}
    	
    	// find all selected rows in this table and remember them to be able to download corresponding
    	// documents later.
        for (var dr = 0; dr < this.docRows.length; dr++) {
        	var docRow = this.docRows[dr];
        	docRow.validateDocToDownload(validObj);
        }
        // provide the validation object for customized error handling...
        return validObj;
    };

    
    /**
     * Properly destroy the DocTableX elements that need to be destroyes before it is thrown out of the DOM
     * 
	 * @param {boolean} destroyACatObj - See detailed description in StaticDocumentRow.destroy() comment.
     */
    DocTableX.prototype.destroy = function(destroyACatObj) {
        for (var dr = 0; dr < this.docRows.length; dr++) {
            this.docRows[dr].destroy(destroyACatObj);
        }
        this._extraActions = [];
    };
    
    /**
     * Reconstruct the DocTableX elements
     */
    DocTableX.prototype.reconstruct = function() {
        for (var dr = 0; dr < this.docRows.length; dr++) {
            this.docRows[dr].reconstruct();
        }
    };
    
    /**
     * Replace a given row with a new staticDocumentRow, based on some documentInfo
     * @param {StaticDocumentRow} - Document row to be replaced in this documents table.
     * 
     * @returns {Promise} - Promise object that resolves when the new document row replacing the original
     * 		document row is initialized and added to this documents table. The resolve value is the newly
     * 		created StaticDocumentRow instance itself.
     */
    DocTableX.prototype.replaceRowWithNewStaticRow = function(row, docInfo) {
    	// first find out where the row to be replaced is in this documents table
    	var rowIdx = row.getIndexInTable();
    	// now create new row and insert it to the right place
    	return this.addDocRow(docInfo, StaticDocumentRow, false, false, rowIdx).then(function (newRow) {
            // now delete the original row (passed by parameter 'row'), which is pushed one index "down"
            // in the document rows array (by the newly added/inserted row 'newRow')
            if (this.allowMultiple) {
            	this.deleteDocumentRow(row.getIndexInTable());
            } else {
            	// note "single-value" document tables (applicable for document references only) already
            	// handles the deletion of the original document row in its _insertDocRow() method!
            }
            return newRow;
    	}.bind(this));
    };

    /**
	 * Method invoked on selection of document source links in replace dropdown and render the required fields in docTableX.
	 * 
	 * @param {dlsObj}
	 *            inputDlsObj - dlsObj is passed to decide the operation invoked for replace           
	 * @param {Integer}
	 *            rowIdx - Index corresponding to clicked item.
	 */
	
    DocTableX.prototype.processReplaceRequest = function (inputDlsObj, rowIdx) {
		
    	try {
    		var docSrcLen =  this.dshObj.docSources.length;
	    	for (var dls=0; dls<docSrcLen; dls++) {
				var dlsObj = this.dshObj.docSources[dls];
				if (dlsObj) {
					if((inputDlsObj.operation===dlsObj.operation) && (inputDlsObj.sourceType===dlsObj.sourceType)){
					var	sourceType = dlsObj.sourceType;
						if(sourceType === 'Browse') {
								var uploadOperation = dlsObj.operation;
								this.renderReplaceContent(uploadOperation, null, null, rowIdx);
								break;
						} else if(sourceType === 'CurrentProject') {
								var linkOperation = dlsObj.operation;
								this.renderReplaceContent(linkOperation, null, null,rowIdx);
								break;
						} else if(sourceType === 'DocumentTemplates') {
								var newOperation = dlsObj.operation;
								var template  = dlsObj.docTemplateName;
								this.renderReplaceContent(newOperation, template, null, rowIdx);
								break;
						} else if(sourceType === 'SavedSearch') {
								var searchOperation = dlsObj.operation;
								var searchDlsObj  = dlsObj;
								this.renderReplaceContent(searchOperation, null, searchDlsObj, rowIdx);
								break;
						} else {
								DocTableXUtils.log("Unknown Source Type (" + sourceType +")");
						}
					}
				}
			}
    	}catch(err) {
			alert('Problem in DocTableX.processReplaceRequest at line ' + err.lineNumber + ' : ' + err.message);
		}
	};
    
	
	/**
	 * This method is invoked by 'processReplaceRequest' method to render the fields corresponding to replaceType dropdown value in docTableX.
	 * 
	 * @param {String}
	 *            Operation - Operation which is clicked during replace.
	 * @param {String}
	 *            templateName - This parameter will be passed only in case of 'DocumentTemplate' (NEW) operation
	 * @param {dlsObj}
	 *            searchDlsObj - This parameter will be passed only in case of 'Saved Search' operation
	 * @param {Integer}
	 *            rowIdx - Index corresponding to clicked item.
	 */
	
    DocTableX.prototype.renderReplaceContent = function (operation, templateName, searchDlsObj, rowIdx) {
		if(searchDlsObj){
				var dlsElem = {
					'dlsObj': searchDlsObj,
					'docTableObj': this
				};
			this.dshObj.loadSavedSearchDialog(dlsElem, 'DocReference', true, rowIdx);
		}else{
			this.localRemoteButtonClicked(rowIdx, 'Replace', operation, templateName);
		}
	};
	
	
	/**
     * This utillity method is invoked to get all document source links to render in replace dropdown.
     * 
     * JANH TODO: This is (almost) identical copy of documentSourceHandler.displayLinks() method -> refactor the code!!!
     * 
     */
	DocTableX.prototype.getDocSrcLinks = function() {

			var docSrcLinksObj = [];
			var canUserAddDocuments = true;
			
			var options = this.getDocumentSourceOptions(this.greenLinksPrefix);
    		if (options.canUserAddDocuments === false) canUserAddDocuments = false;
    		
			var docSources = this.dshObj.docSources;
			for (var dls=0; dls<docSources.length; dls++) {
				var visible = true;
				var dlsObj = docSources[dls];
				if (dlsObj) {
					
					if (dlsObj.sourceType == 'DocumentTemplates' && options.hasDocumentTemplates === false) visible = false;
					if (dlsObj.sourceType == 'CurrentProject' && options.blankProjectCreationContext === true) visible = false;
					
					if (!canUserAddDocuments) {
						switch (dlsObj.sourceType) {
							case 'Browse' :
							case 'DocumentTemplates' :
							case 'ProjectFolder' :    // now "Select from Project" document source
								visible = false;
								break;
							case 'SavedSearch' :
								if (dlsObj.operation.indexOf('Reference') == -1) {
									visible = false;
								} else {
									if (!dlsObj.origOperations) {
										// prevent 'dlsObj.origOperations' to be set more than once during initializtion process
										dlsObj.origOperations = dlsObj.operation;
									}
									dlsObj.operation = 'Reference';
								}
								break;
						}
					} else {
						if (dlsObj.sourceType == 'SavedSearch' && dlsObj.origOperations) {
							dlsObj.operation = dlsObj.origOperations;
							dlsObj.origOperations = null;
						}
					}
					
					if (visible) {
						docSrcLinksObj.push(dlsObj);
					}
				}
			}
    		return docSrcLinksObj;
	};
    
	/**
     * Recursive function to check if any action is performed in any document that requires password validation.
     * 
     * (Get's overloaded by PCM)
     * 
     * @returns {Boolean} true if password is required, otherwise false.
     */
	DocTableX.prototype.checkIfPasswordRequired = function () {
		return false;
	};
    

	/**
	 * This method effectively switches the documents table to a "modified elsewhere" mode that makes 
	 * the documents table read-only and following the changes done in the "master" documents table.
	 * 
	 * @param {Boolean} repaint - If true this documents table will be repainted if its status changed
	 * 		from read-only to editable or vice versa.
	 */
	DocTableX.prototype.modifiedElsewhere = function(repaint) {
		if (this.isModifiedElsewhere) {
			return;
		}
		
		this.isModifiedElsewhere = true;
		// the only thing that can be "out of sync" are document rows. DocumentRow and its subclasses can be 
		// added, StaticDocumentRows can be deleted, moved (reordered) or replaced.
		// (note as far as property values are not synchronized as well, those can be "out of sync" too...)
	
		this.storedState = {
			isEditable : this.isEditable
		};

		if (this.isEditable != false) {
			// set the mode and repaint the whole documents table, if needed
			this.setIsEditable(false, repaint);
		}			
	};
	
	/**
	 * Update the project to which the documents have to be uploaded
	 * @param {string} projectId The id of the project
	 */
	DocTableX.prototype.updateTargetProject = function(projectId){
		this.projectID = projectId;
		this._initializeFromServer();
	};
	
	/**
	 * Update all the target folders used in the docTableX this is needed when the project id changed to support showing the new possible folders
	 */
	DocTableX.prototype.updateTargetFolders = function(){
		$.when(AJAXDataCache.getInstance().getFoldersByProjID(this.projectID)).done(function (folderList) {
			// update the this.targetFolders and preselect a folder (if applicable) for all the rows of the table
			for(var i = 0; i < this.docRows.length; i++){
				this.docRows[i].updateTargetFolders(folderList, true);
				this.docRows[i].selTargetFolder = null;
			}
			//visually show the updated values
			this.view.repaintDocRows();
			
		}.bind(this))
		.fail(function (errorMsg) {
			this.setStatus('error');
			alert("Problem with retrieving project folders:\n" + errorMsg);
		}.bind(this));
	};
	
	/**
	 * Set the specified folder as the target folder for all the rows in the doctablex
	 * @param {string} folderName The name of the target folder 
	 *                           -> When undefined the default target folder of the row will be restored
	 */
	DocTableX.prototype.updateSelectedTargetFolder = function(folderName){
		
		for(var i = 0; i < this.docRows.length; i++){
			var docRow = this.docRows[i];
			
			if(typeof folderName === "undefined"){
				docRow.setTargetFolderByName(docRow.defaultTargetFolderName);
			}else{
				if(folderName === ''){
					docRow.setSelectedTargetFolder(null,false);
				}else{
					docRow.setTargetFolderByName(folderName, false);
				}
			}
		}
		//visually show the updated values
		this.view.repaintDocRows();
	};
	
	/**
	 * Set the specified folder as the target folder for all subsequent documents
	 * @param {string} folderName The name of the folder
	 *                            when undefined the default folders will be restored
	 */
	DocTableX.prototype.updateDefaultTargetFolder = function(folderName){
		
		for(var i = 0; i < this.dshObj.docSources.length; i++){
			var docSource = this.dshObj.docSources[i];
			
			if(typeof folderName === "undefined"){
				//restore the original target folders
				docSource.targetFolder = docSource.targetFolderOriginal;
			}
			
			docSource.targetFolder = folderName;
			
		}
	};
	
	
	
	
    return DocTableX;
});
