define(["doctablex/DocTableXUtils",
        "doctablex/DocTableX",
        "doctablex/DocTableXView",
        "require",
        "lib/PromisePolyfill"], 
function(DocTableXUtils,
         DocTableX,
         DocTableXView,
         require) {
    /**
     * DocTableXFactory.js
     * 
     * This class is responsible for creating new documents table objects based on particular context/needs.
     * It currently supports creation of DocTableX class and its PCM (text management) counterpart.
     * The PCM counterpart is created using mixins - PCM functionality is mixed (injected) into base 
     * document row objects.
     * 
     * @author JANH - January 2016
     */

    "use strict";


    /**
     * Constructor of DocTableXFactory class. Never use "new DocTableXFactory()" - use the instance
     * created at the end of this class/file.
     * 
     * @param {Boolean} isPCMEnabled - True if the PCM license is active on this system, false otherwise.
     */
    var DocTableXFactory = function(isPCMEnabled) {
    	this.isPCMEnabled = false;
    	if (isPCMEnabled === undefined) {
            if (window.webcenterConfig && window.webcenterConfig.isPCMEnabled) {
            	this.isPCMEnabled = !!window.webcenterConfig.isPCMEnabled;
            }
    	} else {
    		this.isPCMEnabled = !!isPCMEnabled;
    	}

    	this.initialization = new Promise(function(resolve, reject) {
	        // this part must be executed only once!!! (to have "mixin".__overridden() correct!)
	    	if (this.isPCMEnabled) {
	    		require(["doctablex/TextContentTableX", "doctablex/TextContentTableXView"], 
	    				function (TextContentTableX, TextContentTableXView) {
			    	// mixin the PCM functionality to the base class
		    		// extend the model class
		    		TextContentTableX.call(DocTableX.prototype);
			    	// extend the corresponding view class
		    		TextContentTableXView.call(DocTableXView.prototype);
		    		// resolve this.initialization promise when "require" is done
		    		resolve(TextContentTableX);
	    		}.bind(this));
	    	} else {
	    		// resolve this.initialization promise immediately
	    		resolve();
	    	}
    	}.bind(this));
    	
    	return this;
    };
    
    
    /**
     * This method returns a Promise resolving into the new documents table created by this method.
     * 
     * @param {object} options - Collection of options to initialize the created documents table with.
     * @returns {Promise} - Promise object resolving into the created documents table instance.
     */
    DocTableXFactory.prototype.createDocTableX = function(options) {
    	return this.initialization.then(function (TextContentTableX) {
	    	// create a new instance of the DocTableX class
	    	var obj = new DocTableX(options);
	    	
	    	// Check whether the DocTableX contains PCM specific collumns.
    		var hasPCMCols = (DocTableXUtils.getRenderItem(options.renderOptions.columns, 'MasterText') != null) 
  				   || (DocTableXUtils.getRenderItem(options.renderOptions.columns, 'TranslationText') != null)
  				   || (DocTableXUtils.getRenderItem(options.renderOptions.columns, 'TextLanguages') != null);
    		
    		// If PCM is enabled and the table has PCM collumns ... activate the TextContentTableX mixin.
    		if (this.isPCMEnabled && hasPCMCols) {
    			// Add the TextContentTableX mixin to the DocTableX.
	    		TextContentTableX.__initMixin.call(obj, options);
    		}
	    	
	    	return obj;
    	}.bind(this));
    };

    
    return new DocTableXFactory();
});
