define(["jquery",
        "utils/webi18n!DocRefStrings"],
	function ($,
		DocRefStrings) {
		/**
		 * DocTableXUtils.js
		 * 
		 * Global singleton object providing utility-like functionality for DocTableX.js, DocumentRow.js, 
		 * AJAXDataCache.js and related classes.
		 * Never instantiate this class!
		 * 
		 * @author JANH - December 2014
		 */

		"use strict";

		var DocTableXUtils = (function () {
			//////////////////////////////////////////////////////////////////////
			// PRIVATE MEMBERS
			//////////////////////////////////////////////////////////////////////

			/** @member {integer} _uniquePrefix - Private counter used to generate unique numbers (for unique indexes). */
			var _uniquePrefix = 0;


			/** @member {String} fileInputReposElemID - ID of a container/repository of file inputs that 
			 * 		have to "survive" document table repaints. All DocTableX instances uses the 
			 * 		same repository for now.
			 * 		Note that the algorithm supports also the case when the repository is different for
			 * 		each DocTableX instance or even a combination of these. */
			var fileInputReposElemID = 'DocTableFileInputsReposElemID';


			var getParameterByName = function (name) {
				name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
				var regex = new RegExp("[\\?&]" + name + "=([^&#]*)");
				var results = regex.exec(location.search);
				return (results === null) ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
			};



			// JavaScript singleton pattern - return object exposing nothing else that the getInstance() function
			return {
				//////////////////////////////////////////////////////////////////////
				// PUBLIC MEMBERS
				//////////////////////////////////////////////////////////////////////

				maxBOMCount: 2147483647.0,


				//////////////////////////////////////////////////////////////////////
				// PUBLIC METHODS
				//////////////////////////////////////////////////////////////////////

				/**
				 * Returns next unique number and increment the private counter.
				 */
				getNextUniquePrefix: function () {
					return _uniquePrefix++;
				},

				/**
				 * General logging method that writes to the JavaScript console if it exists.
				 * (It doesn't in IE8 unless the developer tools are open there.)
				 * 
				 * @param {String} msg - Message to be logged to the JavaScript console.
				 */
				log: function (msg) {
					if (console && console.log) {
						console.log(msg);
					}
				},

				/**
				 * General error logging method that writes to the JavaScript console if it exists (it doesn't 
				 * in IE8 unless the developer tools are opened there), otherwise it alerts the same message.
				 * IMPORTANT: Use this just for real errors! For logging debug info use DocTableXUtils.log().
				 * 
				 * @param {String} msg - Message to be logged to the JavaScript console.
				 */
				error: function (msg) {
					if (console && console.error) {
						console.error(msg);
					} else {
						alert(msg);
					}
				},

				/**
				 * Method to print out stack trace in the javascript console.
				 * BE CAREFULL with this method and use it only for debugging! It might not work in all
				 * browsers!
				 */
				printStackTrace: function() {
					  var obj = {};
					  Error.captureStackTrace(obj, DocTableXUtils.printStackTrace);
					  DocTableXUtils.log(obj.stack);
				},

				/**
				 * Method to find out whether the specified ID is a valid WebCenter ID.
				 * 
				 * @param {String} id - String to be checked if it's a valid WebCenter ID.
				 * @returns True if the passed ID is a valid WebCenter ID string.
				 */
				isValidID: function (id) {
					return id && (typeof id == 'string') && (id.trim().length == 16);
				},

				/**
				 * Method returning extension of specified (full or short) document name.
				 * 
				 * @param {String} docName - Full or short name of the document which extension should be returned. 
				 * @param {boolean} removeDot - Flag whether to strip of the leading dot ('true') or not ('false').
				 * 
				 * @returns Extension of the specified file without the dot ('.') or null if the document doesn't have any extension.
				 */
				getFileExtension: function (docName, removeDot) {
					if (docName == null || docName.length == null || docName.length < 3)
						return null; // not a name long enough to have an extension
					if (docName.lastIndexOf('.') < 0 || docName.lastIndexOf('.') >= (docName.length - 1)) // no extension
						return null; // no extension
					var ext = docName.substr(docName.lastIndexOf('.'));
					// return the extension with no leading and trailing spaces and converted to lowercase
					ext = ext.replace(/^\s+|\s+$/g, '').toLowerCase();
					if (removeDot) ext = ext.substring(ext.indexOf(".") + 1);
					return ext;
				},
				
				/**
				 * Some files will change extension after upload.
				 * So, to check whether we are uploading the same extension, we need to know how it will change.
				 * 
				 * @param {String} extension - the extension which will be uploaded
				 * 
				 * @returns the actual extension the document will have
				 */
				getResolvedExtension: function(extension) {
					// remove (potential) dot first
					var hasDot = (extension && extension[0]==='.');
					if(hasDot) {
						extension = extension.substring(1);
					}
					// list of extensions that will change
					if(extension==="zcp") {
						extension = "clf";
					}
					// add (potential) dot again
					return hasDot?"."+extension:extension;
				},


				/**
				 * Method that search for given 'askedItemName' (by "full item name") in the list of render options 
				 * and returns the first one it finds.
				 * 
				 * @param {Array.<Object>} items - Array of items/columns from Render Options Configuration to be searched.
				 * @param {String} askedItemName - Name of the item/column we are looking for.
 				 * @param {String} askedItemDetail - Other info (often ID) of the item/column that defines it more closely.
 				 * 			For example an ID of a property in 'ReferenceProperty' column.
				 * @returns {Object} Returns the 'askedItemName' item (column) if finds one, 'null' otherwise.
				 */
				getRenderItem: function (items, askedItemName, askedItemDetail) {
					if (!items) return null;
					for (var idx = 0; idx < items.length; idx++) {
						var item = items[idx];
						if (item && item.type) {
							var fullItemName = item.type;
							if (fullItemName === 'DocumentProperty' || fullItemName === 'ProjectProperty') {
								fullItemName += '_' + item.typeDetail;
							}
							if (askedItemDetail) {
								if ((fullItemName == askedItemName) && (item.typeDetail == askedItemDetail)) return item;
							} else {
								if (fullItemName == askedItemName) return item;
							}
						}
					}
					return null;
				},


				/**
				 * Private method returning an array of file extensions (in lowercase without leading comma) extracted 
				 * from the string passed via 'fileExtensionsStr' parameter.
				 * 
				 * @param {String} fileExtensionsStr - A string containing list of user specified file extensions.
				 * @returns {Array<String>} - Array of processed file extensions from the input parameter.
				 */
				_getFileExtensionsList: function (fileExtensionsStr) {
					var fileExtList = [];
					if (fileExtensionsStr && (fileExtensionsStr.length > 0)) {

						var extList = fileExtensionsStr.split(",");
						if (extList.length == 1) extList = fileExtensionsStr.split(";"); // allow both , and ; as a separator but not mixed

						for (var i = 0; i < extList.length; i++) {
							var ext = extList[i];
							if (!ext) continue; // empty
							ext = ext.replace(/^\s+|\s+$/g, '').toLowerCase();
							if (ext.substr(0, 1) == '.') ext = ext.substr(1);
							fileExtList.push(ext);
						}
					}
					return fileExtList;
				},

				/**
				 * Private method returning an array of document template names extracted from the string passed via 
				 * 'docTempNamesStr' parameter.
				 * 
				 * @param {String} docTempNamesStr - A string containing list of user specified doc. template names.
				 * @returns {Array<String>} - Array of processed doc. template names from the input parameter.
				 */
				_getDocTemplateNameList: function (docTempNamesStr) {
					var docTempList = [];
					if (docTempNamesStr && (docTempNamesStr.length > 0)) {

						var dtnList = docTempNamesStr.split(","); // Document Template Name List
						if (dtnList.length == 1) dtnList = docTempNamesStr.split(";"); // allow both , and ; as a separator but not mixed

						for (var i = 0; i < dtnList.length; i++) {
							var name = dtnList[i];
							if (!name) continue; // empty
							name = name.replace(/^\s+|\s+$/g, '').toLowerCase();
							docTempList.push(name);
						}
					}
					return docTempList;
				},

				/**
				 * Utility method to get current "menu_file" URL parameter value.
				 */
				getMenuFileParamValue: function () {
					return getParameterByName('menu_file');
				},

				/**
				 * Utility method to get URL of the current page designed to be used as "source page parameter"
				 * value in generated HTML links (often 'sourcepage' in WebCenter).
				 */
				getSourcePageURLEncoded: function () {
					var pageName = location.pathname.replace(/\/[^\/]*/i, "");
					return encodeURIComponent(pageName + location.search);
				},

				/**
				 * Utility method to get translated label if possible. Otherwise the label itself is returned.
				 */
				getEncodedTranslatedLabel: function (label) {
					var labelStr = '';
					var labelKey = 'DocRefColLabel_' + label;
					if (label.indexOf('DocRefColLabel_') === 0) {
						// 'label' contains a translation tag - use it
						labelStr = DocRefStrings.tFallback(label, label);
					} else if (DocRefStrings.tFallback(labelKey, label) != labelKey) {
						// for back-compatibility: try if the label with translation tag prefix exists
						labelStr = DocRefStrings.tFallback(labelKey, label);
					}

					return WCXML.xmlencode(labelStr);
				},

				/**
				 * Method that search for any fields related to text content in the list of render options 
				 * and returns boolean value
				 * 
				 * @returns {Boolean} Returns 'true' if text content item is found, 'false' otherwise.
				 */
				hasTextContentItem: function (items) {
					var tcFound = false;
					if (!items) {
						return false;
					}
					for (var idx = 0; idx < items.length; idx++) {
						var item = items[idx];
						if (item && item.type) {
							var fullItemName = item.type;
							if (fullItemName == 'MasterText' || fullItemName == 'TranslationText') {
								tcFound = true;
								break;
							}
						}
					}
					return tcFound;
				}

				//////////////////////////////////////////////////////////////////////
				// END OF PUBLIC METHODS DEFINITION
				//////////////////////////////////////////////////////////////////////
			};
		})();


		return DocTableXUtils;
	});