define(["doctablex/DocTableXUtils",
		"jquery",
        "wcr/webcenter_xml",
        "component/actionmenu/ActionMenu",
        "component/table/DropdownOverflowUtil",
		"utils/webi18n!DocRefStrings",
		"utils/webcss!./style/doctablex-style.css"],
	function (DocTableXUtils,
		$,
		WCXML,
		ActionMenu,
		DropdownOverflowUtil,
		DocRefStrings) {

		/**
		 * DocTableXView.js
		 * 
		 * This class implements rendering methods for documents table
		 * (DocTableX class).
		 * 
		 * @author JANH - January 2015
		 */

		"use strict";

		var DocTableXView = function (dataModel, phElementID, options) {
			if (!dataModel && !phElementID && !options) {
				return this;
			}

			/**
			 * @member {DocTableX} dm - Data model to be rendered: Instance of DocTableX (or the interface compliant class).
			 */
			this.dm = dataModel;

			/**
			 * @member {jQery object} phElem - jQuery object representing DOM element, where the the content should be rendered.
			 */
			this.phElemID = phElementID;

			/**
			 * {Associative Array} renderMethodMap - Mapping between labels in configurable document table options 
			 * 		('docLinkTableOptions' in AttributeCategory.js) and rendering methods that draw the particular column/item.
			 */
			this.renderMethodMap = {
				'AttributeFlapout': this.renderExpandCollapseButtonHeader,
				'UpDownArrows': this.renderSortingArrowsHeader,
				'SequenceNumber': this.renderSequenceNumberHeader,
				'RowSelector': this.renderRowSelectorHeader,
				'DocumentName': this.renderDocumentSpecHeader,
				'DocumentActions': this.renderDocumentActionsHeader,
				'DocumentProperty_Thumbnail': this.renderThumbnailHeader,
				'DocumentProperty_Description': this.renderDocumentDescriptionHeader,
				'DocumentProperty_ApprovalStatus': this.renderApprovalStatusHeader,
				'DocumentProperty_ApprovalStage': this.renderApprovalStageHeader,
				'DocumentProperty_Project': this.renderDocProjectNameHeader,
				'DocumentProperty_VersionRevision': this.renderDocumentVersionHeader,
				'DocumentProperty_VersionAuthor': this.renderDocumentAuthorHeader,
				'DocumentProperty_ModifiedDate': this.renderDocumentModifiedDateHeader,
				'ReferenceProperty': this.renderReferencePropertyHeader,
				'DocumentAttribute': this.renderDocAttributeHeader,
				'ProjectAttribute': this.renderProjectAttributeHeader,
				'ProjectProperty_ProjectName': this.renderProjectNameHeader,
				'ProjectProperty_ProjectStatus': this.renderProjectStatusHeader,
				'ProjectProperty_ProjectDueDate': this.renderProjectDueDateHeader,
				'LocalRemote': this.renderLocalRemoteButtonHeader,
				'ReplaceButton': this.renderReplaceButtonHeader,
				'CloneButton': this.renderCloneButtonHeader,
				'DeleteButton': this.renderDeleteButtonHeader
			};
			// Note 'LocalRemote' option is handled via DocTableX.hasLocalRemoteButton() method. It has no special
			// render method as it doesn't have a special column, but is put next to the 'DocumentName' column.

			/**
			 * @member {Object} renderOptions - This object holds properties controlling the rendering.
			 */
			this.renderOptions = options;

			/**
			 * @member {boolean} showDocActions - Flag controlling whether the "Actions" menu will be shown in 
			 * 		the documents table header. It's shown only if configured in rendering options and this is 
			 * 		the main (root) documents table!
			 */
			this.showDocActions = (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'RowSelector') != null) && this.dm.isRootLevelTable();

			return this;
		};

		/**
		 * Main method rendering the whole document table including the document source links (green links). 
		 * Green links can be on top, at bottom of the documents table or on both places.
		 * 
		 * @param {Object} options - A collection of properties and values controlling the rendering. 
		 * 		The supported options are: 
		 * 		- {boolean} options.simple 
		 * 		- list (true) or table (false) display of the doc table 
		 * 		- ...
		 * @returns {Object} - jQuery object representing the rendered documents table (its skeleton).
		 */
		DocTableXView.prototype.render = function (options) {
			var phElem = $('#' + this.phElemID);

			if (phElem.length === 0) {
				// no placeholder to render to yet - bail out
				return phElem;
			}
			if (this.dm.isInactive) {
				// if the documents table is hidden, hide the placeholder ==
				// the whole docs. table.
				// note the documents table has to be rendered anyhow to
				// support re-showing the documents table
				// again without loss of any data (specifically documents
				// selected to be uploaded).
				phElem.hide();
			} else {
				// make sure the root HTML element of documents table is visible if it's not inactive
				phElem.show();
			}

			// if the 'options' parameter is valid use it instead of default
			// rendering options, but only for
			// this time so remember the default options too!
			var origRenderOptions = this.renderOptions;
			if (options) {
				this.renderOptions = options;
			}

			// render the main structure of the table (up to individual
			// <tr/> elements with correct IDs)
			if (this.renderOptions.renderMethod == 'list') {
				this.renderSimple(phElem);
			} else if (this.renderOptions.renderMethod == 'table') {
				this.renderComplex(phElem);
			} else {
				window.alert("Not supported render method: " + this.renderOptions.renderMethod + ".");
			}

			// go over document rows and call their "render" methods
			for (var dri = 0; dri < this.dm.docRows.length; dri++) {
				var docRow = this.dm.docRows[dri];
				// render the document row with hidden (collapsed) attribute
				// category (if any)
				docRow.displayRow(false);
			}

			// restore the original (default) rendering options
			this.renderOptions = origRenderOptions;

			return phElem;
		};

		/**
		 * This method is used to repaint just remote document rows
		 * (referenced documents from a different project) It's handy
		 * especially when a data loaded (e.g. on documents table
		 * (DocTableX) level) affects remote document rows within the
		 * documents table. This is to make the documents table rows update
		 * (in this case) more efficient.
		 */
		DocTableXView.prototype.repaintRemoteDocRows = function () {
			if (!this.dm.docRows) {
				return; // nothing to repaint
			}
			for (var dri = 0; dri < this.dm.docRows.length; dri++) {
				var docRow = this.dm.docRows[dri];
				if (docRow.isRemoteDocument()) {
					// if the actual document is remote, repaint this
					// document row
					docRow.view.repaintRow();
				}
			}
		};

		/**
		 * This method is used to repaint the document rows It's handy
		 * especially when a data loaded (e.g. on documents table
		 * (DocTableX) level) affects the document rows within the documents
		 * table.
		 */
		DocTableXView.prototype.repaintDocRows = function () {
			if (!this.dm.docRows) {
				return; // nothing to repaint
			}
			for (var dri = 0; dri < this.dm.docRows.length; dri++) {
				var docRow = this.dm.docRows[dri];
				docRow.view.repaintRow();
			}
		};


		/**
		 * The simplest implementation of documents table render method.
		 *
		 */
		DocTableXView.prototype.renderSimple = function (phElem) {
			try {
				this.renderOptions.hideHeader = true;
				this.renderComplex(phElem);
			} catch (err) {
				DocTableXUtils.log('Problem in DocTableXView.renderSimple:\n' + err);
				return null;
			}

			return phElem;
		};

		/**
		 * This method renders full featured documents table main structure. -
		 * document table with blank rows prepared as placeholders for the
		 * document rows rendering, - a placeholder for green links in case
		 * of editable document table.
		 * 
		 * @param {jQuery object} phElem - Place holder HTML DOM element, where to render the content 
		 * 		generated by this method.
		 * @returns - jQuery object representing the rendered documents table (its skeleton).
		 */
		DocTableXView.prototype.renderComplex = function (phElem) {
			// render what we have for now:
			// - document table with blank rows prepared as placeholders for the document rows rendering,
			// - a placeholder for green links in case of editable document table.
			try {
				// destroy rich text editors etc., BUT KEEP the attribute category objects holding attribute
				// values including changes already done by the user!
				this.dm.destroy();
				phElem.html(''); // make the rendering placeholder empty (removing potential old HTML while repainting)

				// Make sure that all menus on the page are cleaned up.
				DropdownOverflowUtil.cleanupDropdowns();


				/*
				 * Generate the core structure.
				 */
				this._root = phElem;
				this._tableWrapper = $("<div>", {
					"class": "wcr-table doctablex-wrapper"
				});
				
				var headerSectionClass = "wcr-table-filter";
				if (this.dm.isModifiedElsewhere) {
					headerSectionClass = "alert alert-info";
				}
				this._headerSection = $("<div>", {
					"class": headerSectionClass
				});
				this._tableSection = $("<div>", {
					"class": "wcr-table-section"
				});
				this._root.append(this._tableWrapper);
				this._tableWrapper.append(this._headerSection);
				this._tableWrapper.append(this._tableSection);


				/*
				 * Generate the action header section.
				 */

				var isHeaderVisible = true;
				
				if (this.dm.isModifiedElsewhere) {
					var infoMsgElem = $("<span>").text(DocRefStrings.t('DocTableEditedElsewhere'));
					this._headerSection.append(infoMsgElem);
				} else {
					this._headerActionsHolder = $("<div>", {
						"class": "wcr-table-actions-row no-filter"
					});
					// Get the actions from the document source handler (if we have edit permissions).
					var actions = $("<div>", {
						"class": "action-menu-component"
					});
					if (this.dm.isEditable) {
						var dlsOptions = this.dm.getDocumentSourceOptions(this.phElemID);
						actions = this.dm.dshObj.getAsActions(dlsOptions);
					}
					actions.attr("data-checkbox-selector", "input[name='" + this.dm.getRootLevelPrefix() + "_selectedDocs']");
	
					// Add potential extra actions registered to the document table.
					var extraActions = this.dm.getExtraActions(this.phElemID);
					actions.append(extraActions);
	
					// Add them to the holder.
					this._headerSection.append(this._headerActionsHolder);
					this._headerActionsHolder.append(actions);
	
					// show/hide the "bar" on top of Documents table containing document sources and other actions if there are/are not DSs/actions defined
					if (actions.children().length === 0) {
						this._headerSection.hide();
						isHeaderVisible = false;
					} else {
						this._headerSection.show();
					}
				
					// bootstrap the action menu.
					ActionMenu.wcrinit(actions);
				}

				/*
				 * Generate the table section.
				 */

				// The actual DocTableX
				this._tableElem = $('<table id="' + this.phElemID + '_mainTable" class="table doctablex"></table>');
				this._tableSection.append(this._tableElem);

				// Documents table header
				this.renderTableHeader(this._tableElem);
				if (!this.renderOptions.hideHeader) {
					this._tableElem.find("thead").show();
				} else {
					this._tableElem.find("thead").hide();
				}

				// set table background color if styleObject is defined
				if (this.renderOptions.styleObj && this.renderOptions.styleObj['background-color']) {
					this._tableElem.css("background-color", this.renderOptions.styleObj['background-color']);
				}

				// Render existing and new document row placeholder(s)
				for (var dri = 0; dri < this.dm.docRows.length; dri++) {
					var docRow = this.dm.docRows[dri];
					var docEntryWrapper = $("<tbody>", {
						"class": "DocEntry",
						"data-docrow-id": docRow.view.phElemID
					});
					var docEntry = $("<tr>", {
						"id": docRow.view.phElemID
					});
					docEntryWrapper.append(docEntry);
					this._tableElem.append(docEntryWrapper);
				}

				if (!this.dm.isInactive || this.dm.clearValues) {
					// render hidden element with the XML to save the changes
					var saveXMLElemID = this.dm.getTablePrefix();
					this._tableWrapper.append('<input type="hidden" id="' + saveXMLElemID + '" name="' + saveXMLElemID + '"/>');
				}

				// Make sure that dropdowns in DocTableX are never hidden due to overflow.
				// and when you show it, move it to the body   
				DropdownOverflowUtil.repositionDropdowns(this._tableWrapper);

				if ((this.dm.getNumberOfRows() == 0) && !isHeaderVisible) {
					this._tableSection.hide();
				} else {
					this._tableSection.show();
				}
			} catch (err) {
				DocTableXUtils.log('Problem in DocTableXView.renderComplex:\n' + err);
				return null;
			}

			return phElem;
		};

		/**
		 * Rerender the actions
		 */
		DocTableXView.prototype.updateActions = function () {

			if (this._headerActionsHolder) {
				// Clear the current actions out of the menu.
				this._headerActionsHolder.children().detach();

				// Get the actions from the document source handler (if we have edit permissions).
				var actions = $("<div>", {
					"class": "action-menu-component"
				});
				if (this.dm.isEditable) {
					var dlsOptions = this.dm.getDocumentSourceOptions(this.phElemID);
					actions = this.dm.dshObj.getAsActions(dlsOptions);
				}
				actions.attr("data-checkbox-selector", "input[name='" + this.dm.getRootLevelPrefix() + "_selectedDocs']");

				// Add potential extra actions registered to the document table.
				var extraActions = this.dm.getExtraActions(this.phElemID);
				actions.append(extraActions);

				// Add it to the holder again.
				this._headerActionsHolder.append(actions);

				// show/hide the "bar" on top of Documents table containing document sources and other actions if there are/are not DSs/actions defined
				if (actions.children().length === 0) {
					this._headerSection.hide();
				} else {
					this._headerSection.show();
				}

				// bootstrap the action menu.
				ActionMenu.wcrinit(actions);
			}
		};

		/**
		 * Method to render documents table header (if any).
		 */
		DocTableXView.prototype.renderTableHeader = function (tableElem) {
			if (!tableElem || (tableElem.length === 0)) {
				return;
			}
			try {
				// html() jQuery method is used to overwrite any potential old content of the table 
				// (in case this method is called in doc. table header "repaint" context...)
				tableElem.find('> thead').html(''); // make the rendering placeholder empty (removing potential old HTML while repainting)

				// Make sure that all menus on the page are cleaned up.
				DropdownOverflowUtil.cleanupDropdowns();

				if (this.dm.getNumberOfRows() > 0) {
					// draw documents table header only if there are documents in the table
					var theadElem = tableElem.find('> thead');
					if (theadElem.length === 0) {
						// if there is no <thead> element yet (complete redraw of doc. table case), create it
						theadElem = $('<thead/>');
					}
					tableElem.append(theadElem);
					var trElem = $('<tr/>');
					theadElem.append(trElem);

					// render documents table header items (columns) based on configuration stored in
					// 'this.renderOptions.columns'
					var columns = this.renderOptions.columns;
					if (columns) {
						for (var col = 0; col < columns.length; col++) {
							var column = this.renderOptions.columns[col];
							if (column) {
								var renderMethodID = column.type;
								if (renderMethodID === 'DocumentProperty' || renderMethodID === 'ProjectProperty') {
									renderMethodID += '_' + column.typeDetail;
								}
								var renderMethod = this.renderMethodMap[renderMethodID];
								if (renderMethod) {
									renderMethod.call(this, trElem, column);
								} else {
									DocTableXUtils.log('Not supported table header rendering method: ' + renderMethodID);
								}
							}
						} // end of: for (var col = ...
					}
				}
			} catch (err) {
				DocTableXUtils.log('Problem in DocTableXView.renderTableHeader:\n' + err);
			}
		};

		/**
		 * Method to repaint documents table header (if any).
		 */
		DocTableXView.prototype.repaintTableHeader = function () {
			var tableElem = $('#' + this.phElemID + '_mainTable');
			this.renderTableHeader(tableElem);
		};


		/* vvv Column rendering methods vvv */

		DocTableXView.prototype.renderExpandCollapseButtonHeader = function (phElem) {
			var actionMethod = this.dm.getThisInDOM() + '.expandCollapseAllClicked(this);return false;';
			var htmlcode = '<th class="docrefexpandcol td-narrow">';
			htmlcode += '<i class="icon icon-normal activIcon icon-WCR_Plus-square12 expandAll" onclick="' + actionMethod + '"/>';
			htmlcode += '<i class="icon icon-normal activIcon icon-WCR_Minus-square12 collapseAll" onclick="' + actionMethod + '"/>';
			htmlcode += '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderSortingArrowsHeader = function (phElem) {
			if (this.dm.isEditable) {
				var htmlcode = '<th class="docrefsortcol td-narrow"></th>';
				phElem.append(htmlcode);
			}
		};

		DocTableXView.prototype.renderSequenceNumberHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefsortcol">' + WCXML.xmlencode(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		/**
		 * Render "all document rows" selector checkbox in the header.
		 */
		DocTableXView.prototype.renderRowSelectorHeader = function (phElem) {
			var htmlcode = '<th class="docrefselectcol td-narrow">';
			var selectAllRowsMethod = this.dm.getThisInDOM() + '.onSelectAllRowsClicked(this);';
			htmlcode += '<input type="checkbox" id="' + this.phElemID + '_allRowsSelector" onclick="' + selectAllRowsMethod + '"';
			if (this.dm.isAllRowsSelected()) {
				htmlcode += ' checked="checked"';
			}
			htmlcode += '>';
			htmlcode += '</th>';
			// render the basic table header selector (selector of all rows)
			var thElem = $(htmlcode);

			if (this.showDocActions && (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'DocumentName') == null)) {
				// Create action button.
				var action = $("<a>", {
					"class": "action",
					"data-name": "DocActionsDownload",
					"data-icon": "icon icon-WCR_Actions-Download12",
					"data-require-selection-min": "1"
				});
				// Define what to happen on click.
				action.on("click", function () {
					this.dm.downloadActionClicked();
				}.bind(this));
				// Set the label
				action.text(DocRefStrings.t('DocActionsDownload'));

				// add the 'Actions' menu to the actual column of the header
				this.dm.addExtraAction(action);
			}
			// render the prepared HTML code
			phElem.append(thElem);
		};

		DocTableXView.prototype.renderThumbnailHeader = function (phElem, config) {
			if (this.renderOptions.showThumbnails) {
				var htmlcode = '<th class="docrefthumbcol td-narrow">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
				phElem.append(htmlcode);
			}
		};

		DocTableXView.prototype.renderDocumentSpecHeader = function (phElem, config) {

			var thElem = $('<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>');

			if (this.showDocActions) {
				// Create action button.
				var action = $("<a>", {
					"class": "action",
					"data-name": "DocActionsDownload",
					"data-icon": "icon icon-WCR_Actions-Download12",
					"data-require-selection-min": "1"
				});
				// Define what to happen on click.
				action.on("click", function () {
					this.dm.downloadActionClicked();
				}.bind(this));
				// Set the label
				action.text(DocRefStrings.t('DocActionsDownload'));

				// add the 'Actions' menu to the actual column of the header
				this.dm.addExtraAction(action);
			}
			phElem.append(thElem);
		};

		DocTableXView.prototype.renderDocumentActionsHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefactionscol td-narrow">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderDocumentDescriptionHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderApprovalStatusHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefApprovalStatus td-status">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderApprovalStageHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderDocProjectNameHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		/**
		 * Render the project name header
		 * @param {JQueryElem} phElem
		 * @param {Object} config
		 */
		DocTableXView.prototype.renderProjectNameHeader = function (phElem, config) {
			// we use the same
			this.renderDocProjectNameHeader(phElem, config);
		};

		/**
		 * Render the project status header
		 * @param {JQueryElem} phElem
		 * @param {Object} config
		 */
		DocTableXView.prototype.renderProjectStatusHeader = function (phElem, config) {
			// we use the same
			this.renderDocProjectNameHeader(phElem, config);
		};

		/**
		 * Render the project due date header
		 * @param {JQueryElem} phElem
		 * @param {Object} config
		 */
		DocTableXView.prototype.renderProjectDueDateHeader = function (phElem, config) {
			// we use the same
			this.renderDocProjectNameHeader(phElem, config);
		};

		/**
		 * Render the project attribute header
		 * @param {JQueryElem} phElem
		 * @param {Object} config
		 */
		DocTableXView.prototype.renderProjectAttributeHeader = function (phElem, config) {
			// we use the same as documents
			this.renderDocAttributeHeader(phElem, config);
		};

		DocTableXView.prototype.renderDocumentVersionHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefversioncol td-centered">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderDocumentAuthorHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderDocumentModifiedDateHeader = function (phElem, config) {
			var htmlcode = '<th class="docrefdoccol">' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</th>';
			phElem.append(htmlcode);
		};

		DocTableXView.prototype.renderLocalRemoteButtonHeader = function (phElem) {
			if (this.dm.isEditable) {
				var htmlcode = '<th class="docrefsortcol td-narrow"></th>';
				phElem.append(htmlcode);
			}
		};


		DocTableXView.prototype.renderReplaceButtonHeader = function (phElem) {
			if (this.dm.isEditable) {
				var htmlcode = '<th class="docrefreplacecol td-narrow"></th>';
				phElem.append(htmlcode);
			}
		};

		DocTableXView.prototype.renderCloneButtonHeader = function (phElem) {
			if (this.dm.isEditable) {
				var htmlcode = '<th class="docrefclonecol td-narrow"></th>';
				phElem.append(htmlcode);
			}
		};


		DocTableXView.prototype.renderDeleteButtonHeader = function (phElem) {
			if (this.dm.isEditable) {
				var htmlcode = '<th class="docrefremovecol td-narrow"></th>';
				// render the prepared HTML code
				phElem.append(htmlcode);
			}
		};

		DocTableXView.prototype.renderReferencePropertyHeader = function (phElem, config) {
			if (config.visibility != 'Hidden') {
				var htmlcode = '';
				htmlcode += '<th class="docrefdoccol">' + WCXML.xmlencode(config.label);
				if (config.visibility == 'Required') {
					htmlcode += '&nbsp;<font color="red">*</font>';
				}
				htmlcode += '</th>';
				// render the prepared HTML code
				phElem.append(htmlcode);
			}
		};

		DocTableXView.prototype.renderDocAttributeHeader = function (phElem, config) {
			var htmlcode = '';
			htmlcode += '<th class="docrefdoccol">' + WCXML.xmlencode(config.label);
			if (config.visibility == 'Required') {
				htmlcode += '&nbsp;<font color="red">*</font>';
			}
			htmlcode += '</th>';
			// render the prepared HTML code
			phElem.append(htmlcode);
		};

		return DocTableXView;
	});