define(["doctablex/StaticDocumentRow",
       "doctablex/DocumentRowView",
       "doctablex/DocTableXUtils",
       "doctablex/AJAXDataCache",
       "wcr/webcenter_xml",
       "utils/EventBasedValidator",
       "utils/webi18n!DocRefStrings",
       "underscore",
       "jquery"], 
function(StaticDocumentRow,
         DocumentRowView,
         DocTableXUtils,
         AJAXDataCache,
         WCXML,
         EventBasedValidator,
         DocRefStrings,
         _,
         $){
    /**
     * DocumentRow.js
     * 
     * Object representing a single document, either new or already uploaded to be used in a given
     * documents table (DocTableX instance). It holds required data and implements logic to submit and 
     * render the represented document (row).
     * It has subclasses for different document sources (= new document "types").
     * 
     * @author JANH - October 2014
     */

    "use strict";

    var DocumentRow = function(options) {
        if (!options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        // set the view class of this data model class before calling superclass constructor to be created in the base class
        if (!options.viewClass) options.viewClass = DocumentRowView;
        StaticDocumentRow.call(this, options);   // calling superclass constructor!


		/** @member {String} newDocTmpID - the "new document's temporary document ID" the unique identifier
		 * 		of a newly created document represented by this document row. It's null/undefined for
		 * 		all existing documents (documents with doc. version ID stored in WebCenter database).
		 * 		See DocumentRow.setNewDocTmpID() method for details how it's generated.
		 */ 
        this.newDocTmpID = null;
        
        /** @member {String} docOperation - operation performed on the document based on the document source
         * 		definition (like Move, Copy, Upload, Reference, etc.). DocumentRow class has no default
         * 		operation - that should be provided by each of its subclasses supporting the various 
         * 		document source types. */
        this.docOperation = options.operation || null;
        
        /** @member {String} docFlowType - Type of operation such as LocalRemote or Replace
         * that created this document row. */
        this.docFlowType = options.flowType || null;

        /** @member {Boolean} useDocTypeDefaultACat - Flag whether to use document type attribute category 
         * 		as the default attr. category for this document (row) if none is imposed (via document 
         * 		template etc.). Default value ('false') is meant for documents already uploaded to WCR, where
         * 		the selection of their attr. category was already done. 'true' is for new documents... */
        this.useDocTypeDefaultACat = false;
        
        
        /** @member {StaticDocumentRow (or a subclass)} origDocRow - The whole initialized document row object holding the
         * 		previous/original document row, which was replaced by this. Currently used for "make local" functionality. */
    	this.origDocRow = options.origDocRow || null;

        /** @member {Boolean} hasOrigDocACat - This member is used to remember if the original/source document 
         * 		(that is copied/moved/linked/referenced) has an attribute category or not. It's initialized
         * 		in this.updateDocAttrCategoryFromValuer() method when the required data is loaded from server. */
        this.hasOrigDocACat = false;


        // new documents have by default version index '1' and no revision, the index of the latest document version 
    	// is '1' by default too
        this.docVerIdx = '1';
        this.docRevLetter = '';
        this.latestDocVerIdx = '1';
        
        if (!this.project || !this.project.id) {
            // base class initializes this.projectID like this: "projectID = options.projectID || null;"
            // the new document always ends up in the actual project. the only exception is 
        	// doc. operation 'Reference' (in saved search), where the project is already set.
            this.project = {
               	id : this.getParentDocTable().projectID
            };
        }
        // see StaticDocumentRow constructor for details about 'this.projects'
        if (!this.projects) {
        	this.projects = [this.project];
        }

        /** @member {String} sourceProjID - ID of the project, where the source document is stored
         * 		or null if not applicable (e.g. Browse operation). */
		this.sourceProjID = options.sourceProjID || null;
        
        /** @member {Boolean} isChangedByUser - Flag set to true if and only if this document row was changed by direct user action. */
        this.isChangedByUser = false;
        
        //JANH TODO not used right now. If not needed remove it together with all associated code!!!
        /** @member {Array.<String>} fieldsModByUser - Collection of input/select GUI field names modified by user.
         * 		It's used to track what can be update automatically and what values needs to be preserved. */
        this.fieldsModByUser = [];
        

        // autonaming related info ==================================

        // values of autonaming setup loaded from the server. this.anCounterLength has currently no use here, but
        // is kept together with the rest for consistency reasons
        /** @member {String} docNameSuffix - Suffix added to the generated document name at [EditField] smartname position. */
        this.docNameSuffix = null;

        /** @member {String} anPrefix - Autonaming prefix defined for the currently selected file/document. 
         * 		This also determines if the autonaming is on or off (for the particular document). */
        this.anPrefix = options.anPrefix || null;
        /** @member {String} anSuffix - Autonaming suffix defined for the currently selected file/document. 
         * 		If it contains [EditField] smartname, special doc. name suffix field is rendered and value entered
         * 		there is stored in this.docNameSuffix. */
        this.anSuffix = options.anSuffix || null;
        /** @member {int} anCounterLength - Autonaming counter length. It has no special meaning or use for now. */
        this.anCounterLength = options.anCounterLength || 0;

        
        // document templates related info ==================================

        /** @member {String} docTempType - The type/mode of document template selection. Supported values are:
         * 		AnyDocTemplate, LimitedDocTemplate, FixedDocTemplate and NoDocTemplate. */
        this.docTempType = options.docTempType;
        if (!this.docTempType) this.docTempType = 'AnyDocTemplate';

        /** @member {Array<String>} docTempNames - List of allowed document templates in case 
         * 		'this.docTempType' != 'AnyDocTemplate' */
        this.docTempNames = options.docTempNames || '';

        /** @member {Array<Object>} docTemplates - Array of document templates applicable for this document row. 
         * 		It consists of related document template project documents (if any) filtered by the corresponding
         * 		document source configuration of "Document Template Selector". 
         * 		'this.docTemplates == []' (empty array) means that the doc. templates were loaded, 
         * 		but there is none of them applicable.
         * 		'this.docTemplates == null' on the other hand means the doc. templates were not loaded yet! 
         */
        this.docTemplates = options.docTemplates || null;

        /** @member {Object} selDocTemplate - Object representing currently selected document template.
         * 		It's null if there is no document template selected. */
        this.selDocTemplate = options.selDocTemplate || null;

        /** @member {String} defTaskDocAttrCatID - see description in DocTableX constructor */
        this.defTaskDocAttrCatID = this.getParentDocTable().defTaskDocAttrCatID;

        
        // Target folder related info ==================================

        /** @member {String} targetFolderName - Name of the destination folder as configured in the
    	 * corresponding document source object (there it is 'dlsObj.targetFolder'). 
    	 * Note: This is only used to initialize the selected folder. */
        this.targetFolderName = options.targetFolderName || '';  
		
		/**
		* @member {String} defaultTargetFolderName - The Name of the default destination folder as 
		*                   configured in the corresponding document source object
		*                   if the option is empty then we assume that the target folder wasn't changed yet
		*/
		if(options.defaultTargetFolderName == null){
			this.defaultTargetFolderName = this.targetFolderName;
		}else{
			this.defaultTargetFolderName = options.defaultTargetFolderName
		}
    	
        /** @member {Array<Object>} targetFolders - Array of folders the logged in user has upload rights to
         * 		(applicable for this document row). Root (if in the array) is always the first element.
         * 		Each member of the array looks like: {
         * 			id,
         * 			name,
         * 			changePropPerm,
         * 			isDefaultFromDocTempl (only in a selected item)
         * 		}
         * 		'this.targetFolders == []' (empty array) means that the target folders were loaded, 
         * 		but the user has no upload rights to the current project.
         * 		'this.targetFolders == null' on the other hand means the target folders were not loaded yet!
         */
        this.targetFolders = options.targetFolders || null;

        /** @member {Object} selTargetFolder - Object representing currently selected target folder.
         * 		It's null if there is no target folder selected. */
        this.selTargetFolder = options.selTargetFolder || null;
		
		

        /** @member {String} defTaskDocAttrCatID - see description in DocTableX constructor */
        this.defTaskDocFolderName = this.getParentDocTable().defTaskDocFolderName;


        // private initialization ========================================

        if (this.constructor == DocumentRow.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass

            this.setStatus('ready');
        }
        
        this._validationCallback = _.bind(function(validObj){
        	this.validateDocRowUniqueName(validObj);
        },this);
        EventBasedValidator.getInstance().onValidateUniqueDocName(this._validationCallback);

        return this;
    };

    //define DocumentRow as a subclass of StaticDocumentRow
    DocumentRow.prototype = new StaticDocumentRow();
    DocumentRow.prototype.constructor = DocumentRow;
    DocumentRow.prototype.parent = StaticDocumentRow.prototype;

    
	/**
	 * This method derives all necessary information/data needed to create a new copy of the original 
	 * (== this) document row.
	 * It extends the basic implementation in StaticDocumentRow.
	 * 
	 * @override
	 * @returns {Object} - Document info object == options for the constructor of any document row object.
	 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
	 * 		any "chattering" between the original and new document row.
	 */
    DocumentRow.prototype.getDocRowInfo = function () {
		var docInfo = DocumentRow.prototype.parent.getDocRowInfo.call(this);

		_.extend(docInfo, {
			docOperation : this.docOperation,
			docFlowType : this.docFlowType,
			origDocRow : this.origDocRow,   // this is not deep copy - read-only value!
			hasOrigDocACat : this.hasOrigDocACat,
			anPrefix : this.anPrefix,
			anSuffix : this.anSuffix,
			anCounterLength : this.anCounterLength,
			docTempType : this.docTempType,
			docTempNames : this.docTempNames,
			targetFolderName : this.targetFolderName,
			docTemplates : this.docTemplates,   // this is not deep copy - read-only value!
			selDocTemplate : this.selDocTemplate,   // this is not deep copy - read-only value!
			targetFolders : this.targetFolders,   // this is not deep copy - read-only value!
			selTargetFolder : this.selTargetFolder,   // this is not deep copy - read-only value!
			sourceProjID : this.sourceProjID
		});

		return docInfo;
	};
	

    /**
     * Method to reset document related data. Supposed to be called whenever a document selection is changed
     * while adding new documents. It will revert all document related values to default (empty) or those
     * already stored in the database.
     */
    DocumentRow.prototype.resetDocument = function(propagate) {
        // reset document autonaming related data
        this.anPrefix = null;
        this.anSuffix = null;
        this.anCounterLength = 0;

        // don't reset document template related members as the doc. template selection should preserve!
        // document operation 'this.docOperation' cannot be changed too

        if (propagate) {
            // call the super class resetDocument() too!
            DocumentRow.prototype.parent.resetDocument.call(this);
            // reset the fields edited by user only if this is a full reset operation.
            this.isChangedByUser = false;
            this.fieldsModByUser = [];
        }
    };

    
	/**
	 * Updates attribute category info in this document row from the 'aCatInfo' object loaded via AJAX.
	 * This method extends the basic implementation in StaticDocumentRow class by adding logic needed for
	 * new document rows.
	 * Currently the only addition is initialization of this.hasOrigDocACat flag.
	 * 
	 * @override
	 * @param {Object} aCatInfo - Info about attribute category of this document row loaded from the server
	 * 
	 * @returns {Boolean} - Flag whether the attribute category was changed or not.
	 */
    DocumentRow.prototype.updateDocAttrCategoryFromValue = function (aCatInfo) {
		var firstInitOfCategoryID = !this.aCatID;
		// execute the parent implementation
		var modified = DocumentRow.prototype.parent.updateDocAttrCategoryFromValue.call(this, aCatInfo);
		
		// if this is the first time attribute category initialization, remember if the original/source 
		// document has an attribute category before further calculation that might change it...
		if (firstInitOfCategoryID) {
            this.hasOrigDocACat = this.hasAttrCat();
		}
		return modified;
	};


    /**
     * Common stripped down version of method updating document related properties based on info loaded 
     * from server.
     * Should be utilized (or even overwritten) by subclasses that require a different behaviour of 
     * 'StaticDocumentRow.updateDocInfoFromServer()' method.
     * 
     * @returns {boolean} - 'true' if document was changed, 'false' otherwise.
     */
    DocumentRow.prototype._strippedUpdateDocInfo = function(docInfo) {
    	var modified = false;

        if (this.docOperation == 'Copy') {
        	// special handling of "copy" operation == creation of a new document based on existing one

           	// description of the source document should not be passed over
       		this.docDesc = '';
        	
        	if (docInfo && docInfo.docVerID && (docInfo.docVerID == this.docVerID)) {
                if (!this.docName && (this.docName != docInfo.docName)) {
					// this can be edited on page by user
                    modified = true;
                    this.docName = docInfo.docName;
                }
                if (this.thumbnailURL != docInfo.thumbnailURL) {
                    modified = true;
                    this.thumbnailURL = docInfo.thumbnailURL;
                }
                if (this.docTypeID != docInfo.docTypeID) {
                    modified = true;
                    this.docTypeID = docInfo.docTypeID;
                }
                // In project creation view
                // this.projectCreationContext should be set to false for existing documents in other projects
                if (this.projectCreationContext && this.isRemoteDocument()){
                	this.projectCreationContext = false;
                }
				if (docInfo.folderInfo) {
					// (folder of the copied document cannot be changed)
					this.folder.id = docInfo.folderInfo.id;
					this.folder.name = docInfo.folderInfo.name;
				}
                if (docInfo.appTypeList) {
                    this.appTypeList = docInfo.appTypeList;
                    // no operation within documents table can change the 'appTypeList' - no change to 'modified' flag here
                }
            	// update the this.aCatID and other members related to attribute category of current document
            	// based on the response returned from the server (and internal logic)
               	if (this.updateDocAttrCategoryFromValue(docInfo.catInfo)) {
               		modified = true;
               	}
            } else if (AJAXDataCache.isEmptyValue(docInfo)) {
                // when 'getDocInfoWithThumbnail' returns 'emptyObject' it means the document info cannot be loaded. If it's
                // not an AJAX error, the reason is no permissions or that the document doesn't exist (which is in the context
                // of the SDK call the same thing).
                this.setStatus('noperm');
            }
        } else {
        	// for all document operations except 'Copy' call the standard update of doc. info
        	modified = DocumentRow.prototype.parent._updateDocInfo.call(this, docInfo);
        }
        
        return modified;
    };

    
    /**
     * Method returning 'true', if attributes of this document can be edited in "general" - meaning
     * in the columns of documents table. Otherwise 'false' is returned.
     * 
     * This is the general-purpose version for documents added to the documents table.
     * 
     * @override
     * @returns {Boolean}
     */
    DocumentRow.prototype.canUserEditAttributes = function() {
    	// applicable when a new document is created (copied, moved or uploaded)
    	var newDocChangePropPerm = false;
    	if (this.selTargetFolder) {
    		// this works both in case of existing project and new project created from template/existing
    		// project. In the second case the 'selTargetFolder' represents the folder in the 
    		// template/existing project
    		newDocChangePropPerm = this.selTargetFolder.changePropPerm;
    	} else if (this.getParentDocTable().isBlankProjectCreationContext()) {
    		// user creating a new blank project has rights to change attributes of newly uploaded/copied
    		// documents to the new project!
    		newDocChangePropPerm = true;
    	}
    	
    	return !this.isInactive()
    			// no view right and/or error loading doc. details
    			&& !this.isInErrorStatus()
    			&& this.isLatestDocVersion()
    			&& newDocChangePropPerm;
    };

    
    /**
     * Returns 'true' if the currently logged in user can download document represented by this row. 
     * In case of any new document user cannot immediately download the document that was just uploaded - the
     * method returns always false (for new rows/documents).
     * 
     * @override
     * @returns {Boolean}
     */
    DocumentRow.prototype.canUserDownloadDocument = function() {
    	return false;
    };

    
    /**
     * Method generating URL to retrieve attributes (and their values) from the server.
     * This is overridden version of the basic method defined in DocumentRow class.
     * @override
     */
    DocumentRow.prototype.generateDocAttributeValuesAJAXURL = function() {
        var url =  window.webcenterConfig.jspBaseUrl;
        if (this.selDocTemplate && this.selDocTemplate.docVerID 
        		&& DocTableXUtils.isValidID(this.selDocTemplate.categoryID)) {
            url += 'GetAttributes.jsp?doctemplate=1';   // a flag that the document template is specified
            url += '&projectid=' + this.project.id;   // ID of project where the new document is about to be uploaded
            url += '&documentid=' + this.selDocTemplate.id;   // document ID of selected document template
            url += '&docverid=' + this.selDocTemplate.docVerID;   // doc. version ID of selected document template
            url += '&categoryid=' + this.selDocTemplate.categoryID;   // ID of selected document template's attr. category
        } else if (DocTableXUtils.isValidID(this.aCatID)) {
            // just load attribute types, restricted sets etc. to correctly render them (along with 
            // empty attribute values)
            url += 'GetAttributes.jsp?categoryid=' + this.aCatID;
        } else {
            DocTableXUtils.log("Cannot generate URL in DocumentRowNew.generateDocAttributeValuesAJAXURL().");
        }
        return url;
    };

    
    /**
     * This method is called whenever a document row owned property (document name, description, 
     * attribute category, folder etc.) is changed by direct action of the user. It's different from
     * this.setIsChanged() which is called whenever anything (including attributes) are changed in
     * document row and it could be done programmatically as well.
     * 
     * @param {Boolean} needRepaint - If true the document row is going to be repainted, if false no
     * 		repaint happens in this method, otherwise (if the parameter is omitted or null) this row 
     * 		will be repainted only if 'this.isChangedByUser' is changed to true in this method. 
     */
    DocumentRow.prototype.setIsChangedByUser = function (needRepaint) {
		if (!this.isChangedByUser) {
			this.isChangedByUser = true;

			if (this.newDocTmpID) {
				// It effectively decouple the particular first level children documents of a cloned and master document 
				// row, which are by default treated as the same documents == just one document is created and cloned 
				// rows reference it!
				if (!this.storedState || !this.storedState.newDocTmpID) {
					this.storedState = {
						newDocTmpID : this.newDocTmpID
					};
				}
				// update the "new document's temporary document ID" as it was changed and thus "differentiated" from the 
				// document it was copied from ("master" document)
				// (it's how it would be if it won't be direct child of cloned document row)
				this.setNewDocTmpID('newID_' + DocTableXUtils.getNextUniquePrefix());
			}

			// once a vital property of a document row is changed by user, it cannot be considered 
			// "copied from" anymore as it becomes an independent object.
			this.copyOfExistingRowID = null;
			
			if (needRepaint !== false) {
				// repaint the document row (note even setting the this.copyOfExistingRowID to 'null' can
				// have influence on how the document row looks like!)
				this.view.repaintRow();
				needRepaint = false;
			}
		}
		if (needRepaint) {
			// repaint this document row if asked for and not done already
			this.view.repaintRow();
		}
	};
    
	
	/**
	 * Method to set a new temporary new document ID. It's needed in cases user does a change in
	 * "dependent" new document row that breaks the link the the original (master) document row.
	 * (Currently it can happen only among children of duplicated row.)
	 * 
	 * @param {String} newValue - New value of the temporary new document ID for this document row.
	 */
    DocumentRow.prototype.setNewDocTmpID = function(newValue) {
    	this.newDocTmpID = newValue;
    	
    	//JANH TODO update all 'this.getParentDocTable().parentDocVerIDs' array from here to the leafs
    	// of the hierarchy as 'newDocTmpID' is part of it!!! Valid only when new document rows will be 
    	// selectable directly via a document source/some implicit functionality (as proposed by Kim).
    };
	
    
    /**
     * Method returning true if the specified input/select field in GUI was already modified by user,
     * false otherwise.
     * 
     * @param {String} fieldName - Name of the asked GUI field. Supported values (for now):
     * 		docName, docNameSuffix, docDescription, targetFolder, docTemplate
     * @returns {Boolean}
     */
    DocumentRow.prototype.isModifiedByUser = function(fieldName) {
    	return (this.fieldsModByUser.indexOf(fieldName) > -1);
    };

    /**
     * Method to update the this.fieldsModByUser member based on specified field name and new value.
     * Note empty new value resets the "modified" flag of the particular field.
     * 
     * @param {String} fieldName - Name of the modified GUI field. Supported values (for now):
     * 		docName, docNameSuffix, docDescription, targetFolder, docTemplate
     * @param {Boolean} isValEmpty - Flag specifying if the new value is empty (true) or not (false).
     */
    DocumentRow.prototype.setFieldModByUser = function(fieldName, isValEmpty) {
    	if (isValEmpty) {
    		// deleting/reseting the value to "empty" value resets the field modification flag - remove
    		// the record about the modification of the 'fieldName'
    		var idx = this.fieldsModByUser.indexOf(fieldName);
    		if (idx > -1) this.fieldsModByUser.splice(idx, 1);
    	} else {
    		// any non-empty value means (in the context of this method) the user changed the value manually
    		this.fieldsModByUser.push(fieldName);
    	}
    };
    
    /**
     * @returns {Boolean} - true if autonaming is preconfigured for the document type of the currently selected file.
     */
    DocumentRow.prototype.isAutonamingEnabled = function() {
        // based on the original implementation the autonaming is turned on if and only if the autoname prefix
        // is a non-empty value (see LoadProjectAutonaming() in AddDocument.js)
        return (this.anPrefix != null) && (this.anPrefix.length > 0);
    };


    /**
     * @returns {Boolean} - true if autonaming settings contains configurable document name suffix.
     */
    DocumentRow.prototype.hasAutonameSuffix = function() {
        return (this.anSuffix && (this.anSuffix.toLowerCase().indexOf('[editfield]') > -1));
    };


    /**
     * Method to set the row to be selected by the user.
     * None of new document rows can be selected!
     * 
     * @override
     * @param {boolean} selected - A boolean value representing if the row is selected (value true) or not.
     */
    DocumentRow.prototype.setIsSelected = function(selected, repaint) {
    	this.isSelected = false;   // a new document row cannot be selected (for now - because there is just "download" operation supported)
    };

    /**
     * Method to select all descendants of the current document (document row).
     * It cooperates with DocTableX and AttributeCategory objects to achieve that.
     * The selection is done from the leafs to the root (of the doc. ref. tree-like structure).
     * 
     * A new document row including its potential descendants cannot be selected!
     *  
     * @override
     * @param {Boolean} isSelected - Flag defining whether the document rows should be selected or deselected.
     */
     DocumentRow.prototype.selectAllDescendants = function(isSelected) {
    	 // do nothing for new document rows.
     };

    
    /**
     * Method to set the document type and autonaming settings based on project configuration.
     * Default attribute category is the attr. category of the source document here, so the default 
     * attr. category of the document type is not considered.
     * This method assumes the this.docTypeID is already set correctly and docTypesConfig is initialized too!
     * 
     * @param {Object} docTypesConfig - Effective configuration of all the document types defined in the system
     * 		and (potentially) overruled by the project specific settings.
     * @param {boolean} isSetACatIDEnabled - Whether attempt to set the attribute category based on selected
     * 		document template or document type should be done here.
     * @param {boolean} repaint - Whether to repaint the document row after the changes.
     */
    DocumentRow.prototype.updateDocumentType = function(docTypesConfig, repaint) {
        var docTypes = docTypesConfig.docTypeList;
        if (docTypes && this.docTypeID) {

        	var isBlankProjectCreationContext = this.getParentDocTable().isBlankProjectCreationContext();
            var defDocAutonamingDisabled = isBlankProjectCreationContext && !docTypesConfig.useDefAutoname4BlankProj;
            var defAttrCategoryDisabled = isBlankProjectCreationContext && !docTypesConfig.useDefCategory4BlankProj;
        	
            for (var i = 0; i < docTypes.length; i++) {
                var docType = docTypes[i];
                if (this.docTypeID == docType.id) {
                    // document type of the currently uploaded file found in the configuration
                    this.docType = docType;

                    if (!defDocAutonamingDisabled) {
	                    // set the default autonaming setup defined in the doc. type, but only if it's not disabled
                    this.anPrefix = docType.anPrefix;
                    this.anSuffix = docType.anSuffix;
                    this.anCounterLength = docType.anCounterLength;
                    }

                    // set the default attribute category defined in the doc. type unless there is 
                    // already a document template selected. in that case the attribute category settings
                    // of the document template always wins! For blank projects this is controlled by the
                    // global settings and can be disabled.
                    if (this.selDocTemplate) {
                        // document template defined attribute category
                        this.aCatID = this.selDocTemplate.categoryID;
                    }
                    if (!this.hasAttrCat() && this.useDocTypeDefaultACat && !defAttrCategoryDisabled) {
                        // document type default attribute category
                        this.aCatID = docType.categoryID;
                    }

                    if (repaint) {
                        // repainting the document row with the new values
                        this.view.repaintRow();
                        // repaint and force expand the (potentially) new attribute category
                        this.showDocAttributeCategory(false, true);
                    }
                    break;
                }
            }
        }
    };


    
    // Document templates related methods =====================================



    /**
     * Method that updates the list of document templates applicable for this document row.
     * Note: 'this.docTemplates == []' (empty array) means that the doc. templates were loaded, 
     * but there is none of them applicable.
     * 'this.docTemplates == null' on the other hand means the doc. templates were not loaded yet! 
     * 
     * @param {Array<Object>} docTempList - List of all loaded document templates from the associated library.
     */
    DocumentRow.prototype.updateDocTemplates = function(docTempList) {
        this.docTemplates = [];   // reset the current document template list

        if (!docTempList || (docTempList.length == 0) || this.isInConfigPreview()) {
        	// nothing to put to the doc. template list or in a preview mode without needed context
        	return;
        }

        if (!this.docTempType || (this.docTempType == 'AnyDocTemplate')) {
            this.docTemplates = docTempList;   // no filtering of doc. templates needed
        } else if (this.docTempType == 'NoDocTemplate') {
            // nothing needs to be done (the doc. template list is already empty)
        } else if (this.docTempType == 'FixedDocTemplate') {
            var filterDocTempName = this.docTempNames[0];
            if (filterDocTempName) {
                for (var j = 0; j < docTempList.length; j++) {
                    var docTemp = docTempList[j];
                    if (docTemp && docTemp.name && (docTemp.name.toLowerCase() == filterDocTempName)) {
                        // set the first and only document template
                        this.docTemplates.push(docTemp);
                        break;
                    }
                }
                if (this.docTemplates.length == 1) {
                    // automatically select the only document template if there is a fixed template configured
                    // first reset the attribute category
                    this.updateDocTemplateSelection(0);
                }
            }
        } else if (this.docTempType == 'ListDocTemplate') {
            for (var j = 0; j < docTempList.length; j++) {
                var docTemp = docTempList[j];
                if (docTemp && docTemp.name && (this.docTempNames.indexOf(docTemp.name.toLowerCase()) >= 0)) {
                    // another filter-complying doc. template found - add it to the result list
                    this.docTemplates.push(docTemp);
                }
            }
        }
    };


    /**
     * This method automatically preselects a document template 'default.<extension>' (or 'default') after 
     * a new file 'filename.<extension>' is selected to be uploaded.
     * It is implementation of function "automatically assign attribute category of default.<extension> document template".
     */
    DocumentRow.prototype.preselectDocTemplate = function() {
        if (this.docTempType && ((this.docTempType == 'NoDocTemplate') || (this.docTempType == 'FixedDocTemplate'))) {
            // there is no preselection possible as there is no options...
            return;
        }

        // automatically pre-select default template if the document templates project is configured so
        // (note this.origFileExt and this.docTemplates has to be valid at this point!)
        var fileExt = this.origFileExt;   // this.origFileExt when called from DocumentRowBrowse
        if (!fileExt) fileExt = this.fileExt;   // this.fileExt when called from other classes

        // go through document templates available for this document row and try to
        // match "default.<extension>" pattern
        var idx = this._matchDocTemplate('default.' + fileExt);
        if (idx < 0) {
            // if not found, try to match "default" pattern
            idx = this._matchDocTemplate('default');
        }
        // preselect the document template as default template for the selected document based on 'idx' value.
        // note 'idx == -1' is a special case when no document template is preselected
        this.updateDocTemplateSelection(idx);
    };

    /**
     * Private method to find document template by specified name.
     * 
     * @param {String} defaultName - Name (including extension) of the default doc. template to be found.
     */
    DocumentRow.prototype._matchDocTemplate = function(defaultName) {
        if (this.docTemplates) {
            for (var i = 0; i < this.docTemplates.length; i++) {
                var docTemp = this.docTemplates[i];
                if (docTemp.name.toLowerCase() == defaultName) {
                    return i;
                }
            }
        }
        return -1;
    };


    /**
     * Method to select document template based on its index in 'this.docTemplates' array. It also 
     * updates all dependent properties like attribute category etc.
     * 
     * @param selIdx - Index of the selected document template. It corresponds to the index of the doc.
     * 		template in 'this.docTemplates' array. 
     */
    DocumentRow.prototype.updateDocTemplateSelection = function(selIdx) {
        if (selIdx == -1) {
            // no document template selected (or previous selection reset)
            this.selDocTemplate = null;
            // revert to default attribute category (if any)
            if (this.defTaskDocAttrCatID) {
            	// in case of task documents context (on TEP) select attribute category defined in 
            	// corresponding task type (if defined)
				this.aCatID = this.defTaskDocAttrCatID;
			} else if (this.docType && this.useDocTypeDefaultACat) {
				// else set attribute category from document type configuration (if any)
				this.aCatID = this.docType.categoryID;
			}
        } else if (this.docTemplates && (selIdx >= 0) && (selIdx < this.docTemplates.length)) {
            // a new document template selected
            var docTemp = this.docTemplates[selIdx];
            this.selDocTemplate = docTemp;
            this.aCatID = docTemp.categoryID;
            // if document template has no attribute category defined, revert back to
            // the default attr. category based on document type configuration (if any)
            if (!this.hasAttrCat() && this.docType && this.useDocTypeDefaultACat) {
                this.aCatID = this.docType.categoryID;
            }
        }
        // the "active" target folder has to be recalculated as the document template changed
       	this.preselectTargetFolder();
    };

    

    // Folder selector related methods =====================================


    
    /**
     * Method that initializes the list of available target folders - folders where the currently logged in
     * user has upload permissions. 
     *
     * @param {Array<Object>} folderList - List of folders where the user can upload to
     */
    DocumentRow.prototype.updateTargetFolders = function(folderList, force) {
    	if(this.targetFolders && !force){
    		// if folders were already initialized, don't overwrite, unless forced
    		return;
    	}
    	
    	if (folderList && (folderList.length > 0) && !this.isInConfigPreview()) {
    		this.targetFolders = folderList;
    	} else {
        	// nothing to put to the folder list or in a preview mode without needed context.
    		// The first reason means that the user cannot upload a document 
    		// to any folder - reset the target folders list and selected target folder. 
        	// (this still can happen in case of 'By Reference', so let this pass...)
    		this.targetFolders = [];
    		this.selTargetFolder = null;
    	}
    };

    
    /**
     * This method automatically preselects the default target folder based on configuration.
     * The selection is done based on the following rules:
     * 1. If the document source has a defined target folder ('this.targetFolderName'), use that one
     * 	  (if exists in this project).
     * 2. Else check the document templates. If a document template is selected preselect the same
	 *	  folder in which the document template doc. is sitting (if exists in this project).
     */
    DocumentRow.prototype.preselectTargetFolder = function() {
    	if (!this.targetFolders || (this.targetFolders.length == 0)) {
    		// nothing to preselect - bail out
    		this.selTargetFolder = null;
    		return;
    	}
    	
        if (!this.isModifiedByUser('targetFolder')) {
        	// first check if default target folder of a task type is defined (only if in task documents context)
        	var defTaskDocTargetFolder = this._findTargetFolderByName(this.defTaskDocFolderName);
        	if (defTaskDocTargetFolder) {
        		this.setSelectedTargetFolder(defTaskDocTargetFolder, false);
        	} else {
		    	// check the target folder in the particular document source configuration. 
		    	// if there's one, make it default folder if also in this project
		    	var docSourceTargetFolder = this._findTargetFolderByName(this.defaultTargetFolderName);
				// if not (docSourceTargetFolder == null) check the default target folder coming from selected 
		    	// document template (if any) or just pick the first item in the list 
		    	// (not guaranteed to be the root!)
		    	this.setSelectedTargetFolder(docSourceTargetFolder, !docSourceTargetFolder);
        	}
        }
    };
    
    /**
     * Set target folder by name
     * @param name
     */
    DocumentRow.prototype.setTargetFolderByName = function(name) {
    	var docSourceTargetFolder = this._findTargetFolderByName(name, true);
    	this.setSelectedTargetFolder(docSourceTargetFolder, !docSourceTargetFolder);
    	this.displayRow();
    };


    /**
     * Private method to find an element of 'this.targetFolders' array based on its name (the folder name).
     * 
     * @param {String} folderName - Name of the folder we are looking for
     * @param {Boolean} allowRoot - When true, emty String will return root folder
     */
    DocumentRow.prototype._findTargetFolderByName = function(folderName, allowRoot) {
    	if (this.targetFolders && (folderName || allowRoot) && (typeof folderName === "string")) {
    		var lcFolderName = folderName.toLowerCase();
	    	for (var i = 0; i < this.targetFolders.length; i++) {
	    		var folderObj = this.targetFolders[i];
	    		if (folderObj && (folderObj.name.toLowerCase() == lcFolderName)) return folderObj;
	    	}
    	}
    	return null;
    };
    

    /**
     * Method to set the selected target folder including the handling of currently selected 
     * document template ("Default folder (template)" case).
     * Should be used to set the this.selTargetFolder member in all cases except reset to null!
     * 
     * @param {Object} newFolder - target folder object to be set to this.selTargetFolder member.
     * 		It is applied only if isDefaultFromDocTempl == false.
     * @param {Boolean} isDefaultFromDocTempl - Switching the mode of this call. 
     * 		If 'true', the target folder is set based on selected document template's folder 
     * 		(if applicable in this project) and to keep track of it the 
     * 		this.selTargetFolder.isDefaultFromDocTempl is set to 'true'. 'newFolder' parameter is ignored
     * 		in this case.
     * 		Otherwise (if 'false') the provided 'newFolder' is simply set as the selected target folder.
     */
    DocumentRow.prototype.setSelectedTargetFolder = function(newFolder, isDefaultFromDocTempl) {
    	if (this.selTargetFolder) {
    		// remove the 'isDefaultFromDocTempl' flag from the originally selected target folder
    		// (note we are working directly with this.targetFolders items - no cloning!)
    		this.selTargetFolder.isDefaultFromDocTempl = false;
    	}
    	if (!this.targetFolders || (this.targetFolders.length == 0)) {
    		this.selTargetFolder = null;
        	// nothing more to do here...
    		return;
    	}
    	if (isDefaultFromDocTempl) {
    		if (this.selDocTemplate) {
	    		// there is a document template selected - look up the doc. template folder name 
	    		// in 'this.targetFolders' and select the match (if any)
	    		this.selTargetFolder = this._findTargetFolderByName(this.selDocTemplate.folderName);
	    		if (this.selTargetFolder) {
		    		// remember this target folder was selected based on selected document template's folder
		    		this.selTargetFolder.isDefaultFromDocTempl = true;
	    		}
    		}
    	} else {
    		// not a default document template folder case - just set the provided folder ('newFolder')
    		this.selTargetFolder = newFolder;
    	}
    	
		if (!this.selTargetFolder) {
    		// make sure something is selected (if there is a non-empty list of target folders)
			this.selTargetFolder = this.targetFolders[0];
			if (isDefaultFromDocTempl) {
	    		// remember this target folder was selected based on selected document template's folder
	    		this.selTargetFolder.isDefaultFromDocTempl = true;
			}
		}
		
		if(this.selTargetFolder){
			this.targetFolderName = this.selTargetFolder.name;
		}
    };
    
    
    /**
     * Method generating XML to save changes in this document table row. It overwrites the base class version of this method
     * to not include unknown properties of a new document like document ID, doc. version ID and document name (due to autonaming).
     * This method is dedicated for document reference attributes only!
     * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
     * 
     * @override
     * @param {String> dsXML - Document Source XML == content of <ds> tag (if any).
     * @returns {String} - XML complying to JavaScript -> JSP interface to save changes done in document table.
     */
    DocumentRow.prototype.convertToXML = function(dsXML) {
        var xmlValue = '';
        try {
        	// if there is no permission / an error related to a new document row - don't save the new row
            if (this.isInErrorStatus()) return xmlValue;
        	
            xmlValue += '<dr>';
            
            if (this.origDocRow) {
            	if (this.origDocRow.drID) xmlValue += WCXML.ElementEncode('id', this.origDocRow.drID);
            	if (this.origDocRow.docVerID) xmlValue += WCXML.ElementEncode('dv', this.origDocRow.docVerID);
            } else if (this.drID) {
            	xmlValue += WCXML.ElementEncode('id', this.drID);
            }
            
            if (dsXML && (dsXML.length > 0)) {
               	// XML describing a new document row
                xmlValue += dsXML;
            }
            
            // render XML of reference properties (if any)
            xmlValue += this.convertRefPropertiesToXML();

            // prefix of the hidden input field, where the XML of all attributes of the referenced document
            // attribute category is stored.
            // this is to allow server to find the input fields of the descendant document references
            if (this.canUserEditAttributes()) {
            	// the prefix is however generated only if the user has permissions to actually change 
            	// (and also view) the attributes, otherwise it'd confuse the Java action handler 
            	// processing the XML, which would attempt to save the read-only attributes leading
            	// to "permission denied" kind of errors...
            	xmlValue += WCXML.ElementEncode('daprefix', this.aCatPrefix);
            }
            xmlValue += '</dr>';
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRow.convertToXML:\n' + err);
            xmlValue = '';
        }
        return xmlValue;
    };


    
    /**
     * Validation of the name in a new document row (when uploading, copying etc. a new document).
     * 
     * @override
     * @param {Object} validObj - Validation object holding all found validation errors together with the related 
     * 		data needed to process them and notify user. 
     * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
     * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
     * 		starting from 1.
     * 
     * @returns {Boolean} - Returns true if no problem found in this particular document row and 
     * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
     */
    DocumentRow.prototype.validateDocRow = function(validObj, depth) {
    	var isOK = true;
    	
    	// check the document name field
    	var docNameInput = $('#' + this.getRowPrefix() + ' div.DocumentNameRow > :input:first');
    	
   		var errorObj = null;
   		if (docNameInput.length == 0) {
   			// there is no "document name" input field (can happen due to configuration) - check the
   			// this.docName for a value - it should be prefilled in this case.
   			if (!this.docName && !this.isAutonamingEnabled()) {
   				// compose the errorObj object referring to this situation
   				errorObj = {
   					isBlocking : true,
   	   				level : depth,
   					msg : DocRefStrings.t('DocRowDocNameRequired'),
   					element : $('#' + this.getRowPrefix()),   // no doc. name field - select the whole row
   					docRowObj : this
   				};
   			}
   		} else if ((docNameInput.length == 1) && !docNameInput.val() && !this.isAutonamingEnabled()) {
			// if the document name field exists and it's empty and the auto-naming for the document 
    		// is not on, compose the errorObj object referring to missing document name
			errorObj = {
				isBlocking : true,
   				level : depth,
				msg : DocRefStrings.t('DocRowDocNameRequired'),
				element : docNameInput,
				docRowObj : this
			};
    	}
   		
   		if (errorObj != null) {
			validObj.errors.push(errorObj);
			isOK = false;
   		}
    	
    	// call the super validation implementation that takes care of required properties and documents
    	// (both in documents table column and associated attr. category).
    	if (!DocumentRow.prototype.parent.validateDocRow.call(this, validObj, depth)) {
    		isOK = false;
    	}
    	return isOK;
    };
    
    
    /**
     * Add document to validation object for checking unique document names
	 * @param validObj - object that gets passed to collect all document names
     */
    DocumentRow.prototype.validateDocRowUniqueName = function(validObj) {
    	if (!this.isAutonamingEnabled() && this.isEditable && !this.isInactive() && !this.isModifiedElsewhere()) {
    		// check the document name field if autonaming is off, it's editable and not hidden
        	var docNameInput = $('#' + this.getRowPrefix() + ' div.DocumentNameRow > input:first');
        	var elemToHighlight = docNameInput;
        	if (docNameInput.length == 0) {
        		// special handling for cases where the document name input field is not displayed/configured
        		docNameInput = $('<input type="text" class="fake-docname-input docname-input"></input>').val(this.docName);
        		// highlight the whole row if no document input available
        		elemToHighlight = $('#' + this.getRowPrefix());
        	}
       		validObj.addDoc(docNameInput, this.project.id, {row: this}, elemToHighlight);
    	}
    };
    
    
    /**
     * Extend behavior of deletion: unscribe from validation
     * @override
     */
    DocumentRow.prototype.deleteItself = function(leaveRowPlaceHolder) {
    	StaticDocumentRow.prototype.deleteItself.call(this, leaveRowPlaceHolder);

    	EventBasedValidator.getInstance().removeListener(this._validationCallback);
    };

	/**
	 * @override
	 */
    DocumentRow.prototype.destroy = function (destroyACatObj) {
    	StaticDocumentRow.prototype.destroy.call(this, destroyACatObj);
    	if(destroyACatObj){
    		EventBasedValidator.getInstance().removeListener(this._validationCallback);
    	}
    };


    //Action handlers =====================================


    /**
     * Action handler called when user changes "document name" field.
     * 
     * @param {String} newValue - The new document name as entered by user.
     */
    DocumentRow.prototype.docNameChanged = function(newValue) {
        if (this.docName != newValue) {
            // reflect the document name change in the data model
            this.docName = newValue;
            this.setIsChanged();
            this.setIsChangedByUser();
            this.setFieldModByUser('docName', !newValue);
        }
    };

    /**
     * Action handler called when user changes "document name" field.
     * 
     * @param {String} newValue - The new document name as entered by user.
     */
    DocumentRow.prototype.docNameSuffixChanged = function(newValue) {
        if (this.docNameSuffix != newValue) {
            // reflect the document name suffix change in the data model
            this.docNameSuffix = newValue;
            this.setIsChanged();
            this.setIsChangedByUser();
            this.setFieldModByUser('docNameSuffix', !newValue);
        }
    };


    /**
     * Action handler called when user changes "document description" field.
     * 
     * @param {String} newValue - The new document description as entered by user.
     */
    DocumentRow.prototype.docDescChanged = function(newValue) {
        if (this.docDesc != newValue) {
            // reflect the document name change in the data model
            this.docDesc = newValue;
            this.setIsChanged();
            this.setIsChangedByUser();
            this.setFieldModByUser('docDescription', !newValue);
        }
    };

    
    /**
     * Action handler called when user changes selection of document template.
     * 
     * @param {int} selIdx - Index of the newly selected document template.
     */
    DocumentRow.prototype.docTemplateSelected = function(selIdx) {
        // reset the attribute category to make sure the change in selection is propagated correctly
        //JANH note once this line is removed the category is re-created only if really changed! (when selecting a document template)
        this.resetDocumentAttrCategory();

        this.updateDocTemplateSelection(selIdx);

    	// this method is called only if a real change is done in the document templates drop down field
        this.setIsChanged();
        this.setIsChangedByUser(false);
        this.setFieldModByUser('docTemplate', selIdx == -1);

        // repainting the document row as the content of the folder selector might change
        this.view.repaintRow();
        // repaint with the new attribute category and force expand
        this.showDocAttributeCategory(false, true);
    };


    /**
     * Action handler called when user changes "folder selection" field.
     * 
     * @param {int} selIdx - Index of the newly selected targt folder. The value is representing 
     * 		the new target folder ('-1' for "default folder" or index of the folder in this.targetFolders 
     * 		array for others).
     */
    DocumentRow.prototype.targetFolderSelected = function(selIdx) {
	    if (selIdx == -1) {
    		// default folder from the selected template case
	    	this.setSelectedTargetFolder(null, true);
	    } else {
	    	this.setSelectedTargetFolder(this.targetFolders[selIdx]);
	    }

        //UTGO: Text content permission will change upon folder selection
        //hence repaint the row for New / Copy doc. operation
    	if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns)
    			&& (this.docOperation === 'Copy' || this.docOperation === 'New' || this.docOperation === 'Move')){
    		this.view.repaintRow();
    	}
    	
    	// this method is called only if a real change is done in the document templates drop down field
        this.setIsChanged();
        this.setIsChangedByUser();
        this.setFieldModByUser('targetFolder', selIdx == -1);
    	
        // repaint the attribute category as it can change from read-only to editable or vice versa
	    // (force update, but don't force expand)
        this.showDocAttributeCategory(false, false);
    };


    return DocumentRow;
});
