define(["doctablex/DocumentRow",
        "doctablex/DocumentRowBrowseView",
        "doctablex/AJAXDataCache",
        "doctablex/DocTableXUtils",
        "doctablex/DocumentTypeUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
        "jquery"],
	function (DocumentRow,
		DocumentRowBrowseView,
		AJAXDataCache,
		DocTableXUtils,
		DocumentTypeUtils,
		DocRefStrings,
		WCXML,
		$) {
		/**
		 * DocumentRowBrowse.js
		 * 
		 * This is a subclass of DocumentRow designed to handle adding of documents via "Browse" document 
		 * source (upload operation).
		 * 
		 * @author JANH - December 2014
		 */

		"use strict";

		var DocumentRowBrowse = function (options) {
			if (!options) {
				// when there is nothing to initialize from, return just itself (not initialized)
				return this;
			}
			// set the view class of this data model class before calling superclass constructor to be created in the base class
			if (!options.viewClass) options.viewClass = DocumentRowBrowseView;
			DocumentRow.call(this, options); // calling superclass constructor!

			/** @member {String} fileInputName - Unique name for the file input field. */
			this.fileInputName = this.getRowPrefix() + '_fileForm';

			/** @member {String} dlsName - Document Source Link (green link) name/label that creates this row. */
			this.dlsName = options.dlsName || 'Upload';

			//JANH as the docOperation is fixed for "Browse" doc. source is it really needed here???
			/**
			 * @member {String} docOperation - operation performed on the document for "Browse" document source
			 * 		is always Upload.
			 */
			this.docOperation = 'Upload';

	    	/** @member {String} newDocTmpID - Temporary document identifier used for new documents that doesn't
	    	 * 		have WebCenter document version ID assigned yet. It's guaranteed to be unique for each new
	    	 * 		document added within documents table. */
	    	this.newDocTmpID = options.newDocTmpID ? options.newDocTmpID : 'newID_' + this.rowID;
	    	
			/** @member {Array<String>} allowedFileExts - List of file extensions allowed by the corresponding document source. */
			this.allowedFileExts = options.allowedFileExts || [];

			/** @member {String} origFileExt - Member to store the document's original file extension. */
			this.origFileExt = null;

			/** @member {String} _masterFileFormFieldName - Private member to store the name of the master's
			 * 		file selector in case this row is copied from a master row (when this.copyOfExistingRowID is set).
			 * 		It's the name of file input field, where the document to uploaded to the WebCenter 
			 * 		and referenced by this row is kept. 
			 * 		The main purpose of this member is to support copying of doc. ref. attribute values 
			 * 		(synchronization of attribute values specifically for DocumentRowBrowse). */
			this._masterFileFormFieldName = options.fileFormFieldName || '';
			
			// an uploaded document will always end up in the project of the parent (actual) documents table 
			// (description in DocumentRow constructor)
			this.project = {
				id: this.getParentDocTable().projectID
			};

			/** @member {boolean} useDocTypeDefaultACat - Overwritten default value from DocumentRow class.
			 * 		It's a flag determining whether to use document type attribute category as the default 
			 * 		attr. category for this document (row) if none is imposed (via document template etc.). */
			this.useDocTypeDefaultACat = true;

			
			// private initialization ========================================

			if (this.constructor == DocumentRowBrowse.prototype.constructor) {
				// this part of initialization is private for the instances of this class only!
				// it won't be executed for any subclass

				// load missing document information (like available document templates) via AJAX (if not known)
				// note: this should be called only after the 'this.view' is properly initialized!
				this._initializeFromServer();

			} // end of private initialization

			return this;
		};

		// define DocumentRowBrowse as a subclass of DocumentRow
		DocumentRowBrowse.prototype = new DocumentRow();
		DocumentRowBrowse.prototype.constructor = DocumentRowBrowse;
		DocumentRowBrowse.prototype.parent = DocumentRow.prototype;


		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by 
		 * 'options' (constructor parameter) from the server.
		 * Note there is no this.updateDocInfoFromServer() call is the represented document is not yet a WebCenter
		 * document, so no info can be retrieved about it from the server...
		 * 
		 * @override
		 */
		DocumentRowBrowse.prototype._initializeFromServer = function () {
			this.setStatus('loading');

	    	if (this.copyOfExistingRowID) {
	            this.setStatus('ready');
	    		return;
	    	}

			// initialize document template selector and folder selector (if any)
			if (!this.docTemplates && this.project.id) {
				// asynchronous load of document templates (using the AJAX cache object)
				// it's performed only if the doc. templates are not loaded yet and if a project/template
				// to load the configuration from is known! (not the case e.g. in task type definition)
				var docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.project.id);

				// Get the project folders 
				var folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.project.id);


				$.when(docTemplatesReq, folderListReq).done(function (docTempList, folderList) {
					// get the selected file's document templates and and preselect the default one (if any)
					this.updateDocTemplates(docTempList);
					this.preselectDocTemplate();

					if (this.defTaskDocAttrCatID) {
						// overwrite attribute category by default attribute category of a task type (if defined)
						// this can happen only in task documents context (on TEP)!
						this.aCatID = this.defTaskDocAttrCatID;
					}

					// update the this.targetFolders and preselect a folder (if applicable)
					this.updateTargetFolders(folderList);
					this.preselectTargetFolder();

					this.setStatus('ready');

					this.updateOtherInfoFromServer(true, false);

					// repainting the document row with the loaded values/info
					this.view.repaintRow();
					// the attribute category might change too - update it's values and force expand
					this.showDocAttributeCategory(true, true);
				}.bind(this))
				.fail(function (errorMsg) {
					this.setStatus('error');
					alert("Problem with retrieving document templates:\n" + errorMsg);
				}.bind(this));
				
			} else if (this.docTemplates || this.getParentDocTable().isBlankProjectCreationContext()) {
				this.setStatus('ready');
			} else {
				this.setStatus('error');
			}
		};


		/**
		 * This method derives all necessary information/data needed to create a new copy of the original 
		 * (== this) document row.
		 * It extends the basic implementation in superclasses.
		 * 
		 * @override
		 * @returns {Object} - Document info object == options for the constructor of any document row object.
		 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
		 * 		any "chattering" between the original and new document row.
		 */
		DocumentRowBrowse.prototype.getDocRowInfo = function () {
			var docInfo = DocumentRowBrowse.prototype.parent.getDocRowInfo.call(this);
			_.extend(docInfo, {
				newDocTmpID : this.newDocTmpID,   // for documents not existing in the WebCenter yet
				allowedFileExts : this.allowedFileExts,
				fileFormFieldName : ''
			});
			var fileInputObj = this.getEffectiveFileInputObj();
			if (fileInputObj) {
				docInfo.fileFormFieldName = fileInputObj.attr('name');
			}
			return docInfo;
		};

		
		/**
		 * Method to reset document related data. Supposed to be called whenever a document selection is changed!
		 */
		DocumentRowBrowse.prototype.resetDocument = function (fileInput) {
			// don't preserve this file input as it is about to be reset
			$(fileInput).removeClass('docTableFileInputToPreserve');

			this._masterFileFormFieldName = '';
			
			// document type reset
			this.docTypeID = null;
			this.docType = null;
			
			this.resetDocumentAttrCategory();

			// call the super class resetDocument(), but don't propagate the call further!
			DocumentRowBrowse.prototype.parent.resetDocument.call(this, false);

			// don't reset members related to user entered data as those should preserve!
			// e.g. selected document template...
		};

		
		/**
		 * Method that returns an object representing file input field in this document row.
		 * 
		 * @returns {jQuery object} jQuery object representing the file input field of this document row.
		 */
		DocumentRowBrowse.prototype.getFileInputObj = function () {
			if (this.view.uploadInput) {
				return this.view.uploadInput;
			} else {
				return $();
			}
		};

		/**
		 * This method returns the file input field object of this document row or its "master"
		 * row if this document row is indirectly cloned from the "master" row (and the link is still
		 * maintained).
		 * 
		 * @returns {jQuery object} - jQuery object representing file input field of this doucment row
		 * 		or its master (in case this row is indirectly cloned).
		 */
		DocumentRowBrowse.prototype.getEffectiveFileInputObj = function () {
			var fileInput = null;
			if (this.isIndirectlyClonedRow()) {
				var masterRow = this.getMasterOfIndirectlyClonedRow();
				if (masterRow) {
					fileInput = masterRow.getFileInputObj();
				}
			} else if (this.copyOfExistingRowID && this._masterFileFormFieldName) {
				// if there is no file input field, but this document row is copied from another Browse
				// document row, return file input from the master row
				fileInput = $("input[name='" + this._masterFileFormFieldName + "']");
			}
			else {
				fileInput = this.getFileInputObj();
			}
			return fileInput;
		};

		
		/**
		 * This method returns whether this document row has selected a valid file to be uploaded.
		 * 
		 * @returns {boolean} - True if user has already selected a file using the file input field, 
		 * 		false otherwise. It supports an indirectly cloned document rows (still linked to master)
		 * 		as well - in that case it returns true if the master row has a valid file selected in
		 * 		its file input field, else otherwise.
		 */
		DocumentRowBrowse.prototype.isValidFileSelected = function () {
			var fileInput = this.getEffectiveFileInputObj();
			return !!fileInput && !!fileInput.val();
		};

		
		/**
		 * This method is used to get the name of file selected in the file input field of this Browse
		 * document row or its "master". It makes the work with copied/synchronized document rows easier.
		 * 
		 * @returns {String} - Name of a file that is selected to be uploaded/referenced by this document row.
		 */
		DocumentRowBrowse.prototype.getSelectedFileName = function () {
			var selFileName = '';
			var fileInputObj = this.getEffectiveFileInputObj();
			if ((fileInputObj.length > 0) && fileInputObj.val()) {
				selFileName = this.getCurrentDocName(fileInputObj[0]);
			}
	        return selFileName;
		};

		
		/**
		 * Method to "salvage" file input field with selected file in case this document row is
		 * being deleted and there are clones of this row around.
		 * 
		 * @override
		 * @param {StaticDocumentRow or its subclass} srcRow - Row from where to move the "salvaged"
		 * 		file input field.
		 */
		DocumentRowBrowse.prototype.salvageValuablesFromDocRow = function (srcRow) {
			if (srcRow && srcRow.isValidFileSelected()) {
				var salvagedFileInput = srcRow.getFileInputObj();
				var salvagedFileInputPH = salvagedFileInput.parent();
				// get the destination HTML element
				var destination = this.getFileInputObj();
				// apply the new properties on the salvaged item to make it integral part of the 
				// 'destination' document row
				salvagedFileInput.attr('name', destination.attr('name'));
				salvagedFileInput.attr('onchange', destination.attr('onchange'));
				
				destination.replaceWith(salvagedFileInput);
				
				// as the file input was moved from 'srcRow' to destination, it's kind to return
				// something back to 'srcRow' to not end up there with an empty "hole"...
				salvagedFileInputPH.append(srcRow.view.renderFileInputField());
			}
		};

		/**
		 * This method takes care of special behavior of file input field on top of parent implementation 
		 * (See DocumentRow.setIsChangedByUser() for more details).
		 * 
		 * @override
		 * @param {Boolean} isDependenciesAlreadyHandled - True if the special handling of file input field
		 * 		should be skipped (because it was already done), false otherwise.
		 */
		DocumentRowBrowse.prototype.setIsChangedByUser = function (isDependenciesAlreadyHandled) {
			// specific for this class is the file input field handling:
			if (!isDependenciesAlreadyHandled) {
				// check if there are any dependencies for this row and handle them (if any)
				this._handleDependenciesOnChange();
			}

			// don't forget to execute the parent implementation of this method too
			DocumentRowBrowse.prototype.parent.setIsChangedByUser.call(this, true);
		};
		
		
		/**
		 * Private method to find dependent document rows and handle their dependencies accordingly.
		 * The process is very similar to when the document row is deleted (see StaticDocumentRow.deleteItself()
		 * method). The difference is that Browse document row doesn't support cloning so only document
		 * rows copied directly from this row are handled.
		 */
		DocumentRowBrowse.prototype._handleDependenciesOnChange = function() {
			// check if there are any document rows copied out of this one...
			var copiedRows = this.getAllRowsCopiedFromRows([this]);
			if (copiedRows.length > 0) {

				//JANH TODO this code is copy&pasted from StaticDocumentRow.deleteItself() method!!!

				// we are going to change a master (document row from which others were cloned).
				// make a new "master" and point the rest of cloned rows to the new "master".
				var newMaster = copiedRows[0];
				// be careful that the master itself can be copied of another document row!
				newMaster.copyOfExistingRowID = this.copyOfExistingRowID;
				
				// in case there are items not possible to recreate in the "master" row, they need
				// to be "salvaged" (moved to the new "master" row)
				newMaster.salvageValuablesFromDocRow(this);

				for (var i = 1; i < copiedRows.length; i++) {
					copiedRows[i].copyOfExistingRowID = newMaster.rowID;
				}
			}
		};
		
		
		/**
		 * Method generating XML to save info about newly uploaded document.
		 * It calls the superclass convertToXML() method to add the common part of the XML.
		 * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
		 * 
		 * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
		 */
		DocumentRowBrowse.prototype.convertToXML = function () {
			var xmlValue = '';
			try {
				if (this.isInErrorStatus() || !this.isValidFileSelected()) {
					// don't generate any XML if there is an error or no document selected 
					// (output would be just <av></av>).
					return xmlValue;
				}

				// generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
				var dsXML = '<ds>';
				dsXML += WCXML.ElementEncode('action', this.docOperation);
            	if (this.newDocTmpID) dsXML += WCXML.ElementEncode('new_doc_temp_id', this.newDocTmpID);
       			// the use of getEffectiveFileInputObj() method also covers the case when this doc. row 
            	// is indirectly cloned (DocumentRowBrowse cannot be cloned directly). In this case
        		// we need to provide the name of the original ("master") file input field as that is 
        		// the only one containing the selected file and that's what the getEffectiveFileInputObj()
            	// method exactly does!
            	var fileInputObj = this.getEffectiveFileInputObj();
           		if (fileInputObj) {
           			dsXML += WCXML.ElementEncode('file_form_field', fileInputObj.attr('name'));
           		}
				if (this.project.id) dsXML += WCXML.ElementEncode('project_id', this.project.id);
				if (this.selTargetFolder) {
					// get the selected target folder name in XML (if any)
					dsXML += WCXML.ElementEncode('target_folder_name', this.selTargetFolder.name);
				}
				if (this.isAutonamingEnabled()) {
					var suffixStr = (this.docNameSuffix ? this.docNameSuffix : '');
					dsXML += WCXML.ElementEncode('doc_name_suffix', suffixStr);
				} else {
					if (this.docName) dsXML += WCXML.ElementEncode('doc_name', this.docName);
				}
				if (this.docDesc) dsXML += WCXML.ElementEncode('doc_description', this.docDesc);

				if (this.hasAttrCat()) dsXML += WCXML.ElementEncode('doc_attr_cat_id', this.aCatID);

				if (this.selDocTemplate) {
					dsXML += WCXML.ElementEncode('doc_temp_id', this.selDocTemplate.id);
					dsXML += WCXML.ElementEncode('doc_temp_folder_name', this.selDocTemplate.folderName);
				}
				dsXML += '</ds>';

				// call the super method to generate the common part of XML
				xmlValue = DocumentRowBrowse.prototype.parent.convertToXML.call(this, dsXML);

			} catch (err) {
				DocTableXUtils.log('Problem in DocumentRow.convertToXML:\n' + err);
				xmlValue = '';
			}
			return xmlValue;
		};



		/**
		 * Validation of the document row content specific to document source browse (uploading of a new document).
		 * 
		 * @override
		 * @param {Object} validObj - Validation object holding all found validation errors together with the related 
		 * 		data needed to process them and notify user. 
		 * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
		 * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
		 * 		starting from 1.
		 * 
		 * @returns {Boolean} - Returns true if no problem found in this particular document row and 
		 * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
		 */
		DocumentRowBrowse.prototype.validateDocRow = function (validObj, depth) {
			var isOK = true;

			// check the file input field - it has to have a non-empty value
			var fileInputObj = this.getEffectiveFileInputObj();
			if ((fileInputObj.length == 1) && !fileInputObj.val()) {
				// compose the errorObj object referring to this situation
				var errorObj = {
					isBlocking: true,
					level: depth,
					msg: DocRefStrings.t('DocRowFileNotChosen'),
					element: fileInputObj.parent(),
					docRowObj: this
				};
				validObj.errors.push(errorObj);
				isOK = false;
			}

			// call the super validation implementation that takes care of required properties and documents
			// (both in documents table column and associated attr. category).
			if (!DocumentRowBrowse.prototype.parent.validateDocRow.call(this, validObj, depth)) {
				isOK = false;
			}
			return isOK;
		};


		
		/**
		 * Action handler called when user selects a new file in the file input field (of this row).
		 * 
		 * @param {DOM object} fileInput - The file input HTML field itself
		 */
		DocumentRowBrowse.prototype.fileChangedByUser = function (fileInput) {
			// dependencies were already handled in DocumentRowBrowse.fakeBrowseButtonOnClickMethod().
			// It's the special case as file input fields needs a special treatment!
			var isDependenciesAlreadyHandled = true;
			this.fileChanged(fileInput).done(function() {
				this.setIsChangedByUser(isDependenciesAlreadyHandled);
			}.bind(this))
			.fail(function(origDocName) {
				if (origDocName && !this.docName) {
					// in case of cancel or other error set the row to be changed (by user) only if the actual
					// file input field value is changed.
					this.setIsChangedByUser(isDependenciesAlreadyHandled);
				}
			}.bind(this));
		};
		
		
		/**
		 * Method to handle change of selected file in the file input field (of this row).
		 * 
		 * @param {DOM object} fileInput - The file input HTML field itself
		 * @returns {Promise} - Promise that resolves when the selected file is successfully changed, rejects otherwise.
		 */
		DocumentRowBrowse.prototype.fileChanged = function (fileInput) {
			var fileChangeStatus = $.Deferred();
			
			if (this.isInConfigPreview()) {
				alert(DocRefStrings.t('DocSrcHdlNoProjectInConfig'));
				return fileChangeStatus.reject().promise();
			}

			this.setStatus('loading');

			var origDocName = this.docName;
			
			var docName = this.getCurrentDocName(fileInput);

			// reset the selected file all together
			this.resetDocument(fileInput);

			if (!docName) {
				// there is no document selected - bail out
				this.displayRow();
				return fileChangeStatus.reject(origDocName).promise();
			}
			
			// get the file extension and remove the leading dot ('.') from it.
			this.origFileExt = DocTableXUtils.getFileExtension(docName, true);

			// check if the extension of the file is among allowed extensions
			// (if no extensions specified, all extensions are allowed)
			if (this.allowedFileExts && (this.allowedFileExts.length > 0) && (this.allowedFileExts.indexOf(this.origFileExt) == -1)) {
				// the extension is not allowed - alert user about that and reset the file selection
				alert(DocRefStrings.t('notAllowedDocExtension') + this.allowedFileExts.toString());

				// reset the file input field to empty if the selected file is not allowed
				this.getFileInputObj().val(null);

				this.view.repaintRow();
				return fileChangeStatus.reject(origDocName).promise();
			}

			// set the document name based on the selected file (always!)
			this.docName = docName;

			// preserve this file input as there is a file selected by user (which cannot be set by JavaScript)
			$(fileInput).addClass('docTableFileInputToPreserve');

			var cacheInst = AJAXDataCache.getInstance();

			// process document types, default document attribute category, autonaming setup and document templates 
			// based on data loaded from the server
			var effDocTypeSetupReq = null;
			var docTemplatesReq = null;
			if (this.project.id) {
				// if we are in already created project or a new project created based on existing template/project
				effDocTypeSetupReq = cacheInst.getEffectiveDocTypesSettingsByProjID(this.project.id);
				docTemplatesReq = cacheInst.getDocTemplatesByProjID(this.project.id);
			} else {
				// case of project created from blank
				effDocTypeSetupReq = cacheInst.getProjCreationSysDefaults();
			}

			$.when(effDocTypeSetupReq, docTemplatesReq).done(function (docTypesConfig, docTempList) {
				// get the selected file's document type based on its extension
				this.docTypeID = DocumentTypeUtils.getDocTypeIDByExtension(this.origFileExt);
				// get the selected file's document type and all the derived info like default attribute category and 
				// autonaming configuration
				this.updateDocumentType(docTypesConfig, false);
				// get the selected file's document templates and and preselect the default one (if any)
				this.updateDocTemplates(docTempList);

				if (!this.isModifiedByUser('docTemplate')) {
					// run the document template pre-selection calculation
					this.preselectDocTemplate();

					if (this.defTaskDocAttrCatID) {
						// overwrite attribute category by default attribute category of a task type (if defined)
						// this can happen only in task documents context (on TEP)!
						this.aCatID = this.defTaskDocAttrCatID;
					}
				}

				// also adapt target folder selection if needed
				this.preselectTargetFolder();

				this.updateOtherInfoFromServer(true, false);

				this.setStatus('ready');
				fileChangeStatus.resolve();

				// repainting the document row with the updated values/info
				this.view.repaintRow();
				// the attribute category might change too, but that's recognized by 
				// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
				// the decision on the logic within "show attribute category" logic; 
				// force expand the attribute category
				this.showDocAttributeCategory(false, true);
			}.bind(this))
			.fail(function (errorMsg) { // on error
				this.setStatus('error');
				fileChangeStatus.reject(origDocName);
				alert("Failed to process document configuration from the server.\n" + errorMsg);
			}.bind(this))
			.always(function() {
				// mark this document row as changed (dirty). It can be done only after the call to this.resetDocument() 
				// as it resets the 'isChanged' flag too!
				this.setIsChanged();
			}.bind(this));
			
			return fileChangeStatus.promise();
		};

		/**
		 * Get the name of current selected document
		 * @returns {String}
		 */
		DocumentRowBrowse.prototype.getCurrentDocName = function (fileInput) {
			var docName = fileInput.value;
			if (!docName) return;

			// find the file name without folders and drive letters
			if (docName.lastIndexOf("\\") != -1) {
				docName = docName.substring(docName.lastIndexOf("\\") + 1);
			} else if (docName.lastIndexOf("/") != -1) {
				docName = docName.substring(docName.lastIndexOf("/") + 1);
			} else if (docName.lastIndexOf(":") != -1) {
				docName = docName.substring(docName.lastIndexOf(":") + 1);
			}

			return docName;
		};

		/**
		 * Get the view of the AttributeCategory that should be shown
		 * @returns {String}
		 * @override
		 */
		DocumentRowBrowse.prototype.getAttCatView = function () {
			var expandAttrCatConfig = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout');
			if (expandAttrCatConfig && expandAttrCatConfig.useCreationView === false) {
				return this.aCatView || 'Default';
			}

			return 'Creation';
		};

		/**
		 * Get file for this row.
		 * This method is designed to support multiple upload and HTML5 Viewer. 
		 * 
		 * @returns {File} file in file input
		 */
		DocumentRowBrowse.prototype.getFile = function () {
			var ctx = $("#" + this.phElemID);
			// it's OK to not use this.getEffectiveFileInputObj() method here as the use of this method
			// is limited to places where the "duplicate document row" operation and creation of a complex 
			// on-the-fly document reference hierarchies is not supported...
			var fileElem = this.getFileInputObj();
			if (fileElem.length == 1) {
				var fileList = fileElem[0].files;
				if (fileList && fileList.length === 1) {
					return fileList[0];
				}
			}
			return null;
		};
		
		/**
		 * Method to handle click on the fake Browse button. It handles all potential dependencies of this row
		 * (which can change the file input field of this Browse document row) and then passes the action to the 
		 * real (and actual) file input field/element.
		 */
		DocumentRowBrowse.prototype.fakeBrowseButtonOnClickMethod = function() {
			// check if there are any dependencies for this row and if so handle them before the click 
			// event is passed to the real file input field, because the file input field can be moved
			// to another document row during the handling of dependencies. This way we can be sure the 
			// file selection dialog that opens is opened from the right file input field!
			this._handleDependenciesOnChange();
			
			// forward the click event to the real file input field
			this.getFileInputObj().click();
		};
		
		
		return DocumentRowBrowse;
	});