define(["doctablex/DocumentRowView",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(DocumentRowView, 
          DocTableXUtils, 
          DocRefStrings,
          WCXML){
    /**
	 * DocumentRowBrowseView.js
	 * 
	 * This class implements rendering methods for an upload document row ("browse" document source).
	 * 
	 * @author JANH - January 2015
	 */

    "use strict";

    var DocumentRowBrowseView = function(dataModel, phElementID, options) {
		if (!dataModel && !phElementID && !options) {
			// when there is nothing to initialize from, return just itself (not initialized)
			return this;
		}
		DocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

		// note this.dataModel and phElemID are initialized in the super class...)

		/** @member {jQuery object} _fakeBrowseButton - Private member used to store and reference "Browse" button displayed in UI. */
		this._fakeBrowseButton = $();
    	
    	
		// private initialization ========================================
	
		if (this.constructor == DocumentRowBrowseView.prototype.constructor) {
			// this part of initialization is private for the instances of this class only!
			// it won't be executed for any subclass
		}
		
		return this;
    };

    // define DocumentRowBrowseView as a subclass of DocumentRowView
    DocumentRowBrowseView.prototype = new DocumentRowView();
    DocumentRowBrowseView.prototype.constructor = DocumentRowBrowseView;
    DocumentRowBrowseView.prototype.parent = DocumentRowView.prototype;


    /**
	 * Base class overridden method to redraw already existing document row. 
	 * It uses the ID of document row <tr> element as returned by DocumentRow.getRowPrefix() to find 
	 * DOM element to be updated.
	 * This DocumentRowBrowseView's variation supports preserving of the contained file input field.
	 * 
	 * @override
	 */
    DocumentRowBrowseView.prototype.render = function(options) {
		
		// render the parent class render() method
		DocumentRowBrowseView.prototype.parent.render.call(this, options);
	
		// Apply edit capability
		if (this.dm.isEditable) {
			this.enableFileInputField(true);
		}
    };

    // it's not necessary to override repaintRow() method here as it always calls internally the render() method above...


    /**
	 * Method rendering the thumbnail or its placeholder (if it's not loaded yet).
	 * This override variation of the method always draws the thumbnail representing a new, unknown document.
	 * 
	 * @override
	 * @returns {String} - The generated HTML.
	 */
    DocumentRowBrowseView.prototype.renderThumbnail = function(phElem, config) {
		try {
			var htmlcode = '';
			htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
			htmlcode += this.customThumbnailHTML('thumbnails/other.gif', config.thumbnailSize);
			htmlcode += '</' + this.itemRootElemStr + '>';

			// render the prepared HTML code
            phElem.append(htmlcode);
		} catch (err) {
			DocTableXUtils.log('Problem in DocumentRowBrowseView.renderThumbnail:\n' + err);
		}
    };


    /**
     * Method that returns the "on change" JavaScript method string prepared to be placed in HTML.
     * The "on change" method handles the change of selected file to be uploaded to WebCenter.
     * 
     * @returns {String} String representing the "on change" method for HTML.
     */
    DocumentRowBrowseView.prototype.getFileOnChangeMethod = function () {
    	return 'javascript: ' + this.dm.getThisInDOM() + '.fileChangedByUser(this);';
    };

    
    /**
     * @returns {jQuery object} jQuery object representing a new file input field for Browse document row.
     */
    DocumentRowBrowseView.prototype.renderFileInputField = function () {
    	if (!this.uploadInput) {
    		this.uploadInput = $("<input>", {
    	    	"type" : "file", 
    	    	"name" : this.dm.fileInputName, 
    	    	"onchange" : this.getFileOnChangeMethod(), 
    	    	"class" : "hiddenElement"});
    	}
    	return this.uploadInput;
    };

    /**
     * @returns {jQuery object} jQuery object representing a new file input field for Browse document row.
     */
    DocumentRowBrowseView.prototype.enableFileInputField = function (enable) {
    	if (enable) {
	    	this._fakeBrowseButton.prop('disabled', false);
	    	this.dm.getFileInputObj().prop('disabled', false);
    	} else {
	    	this._fakeBrowseButton.prop('disabled', true);
	    	this.dm.getFileInputObj().prop('disabled', true);
    	}
    };

    
    /**
	 * The render method generating HTML for "Browse" (Upload) document source to be able to select a new document
	 * for upload to WebCenter including name, description and document template (if any) definition fields.
	 * 
	 * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
	 * @returns {String} - The generated HTML.
	 */
    DocumentRowBrowseView.prototype.renderDocumentSpec = function(phElem, config) {
		try {
			// Unregister the old resizer for this collumn.
			if (this._nameColResizer) {
				$(window).off("resize", this._nameColResizer);
			}
			
			// Create the collumn element.
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));

            
            // Create a styled file input.
            var fakeFileInputField = $('<div>', {'class' : 'docname-upload'});
            
            var fakeBrowseButtonClickMethod = 'javascript: ' + this.dm.getThisInDOM() + '.fakeBrowseButtonOnClickMethod(); return false;';
            this._fakeBrowseButton = $("<a>", {
            	"href" : "#", 
            	"class" : "btn btn-default btn-sm", 
            	"onclick" : fakeBrowseButtonClickMethod, 
            	"title" : DocRefStrings.t('DocSrcHdlOpenFileBrowser')}).text(DocRefStrings.t('DocSrcHdlOpenFileBrowser'));
            this._fakeBrowseButton.append($("<i>"), {
            	"class" : "icon icon-WCR_Actions-Upload12"});
            var fakeBrowseButtonSpan = $("<span>");
            fakeBrowseButtonSpan.append(this._fakeBrowseButton);
            
        	// be aware the 'selectedFileName' value might not be coming from this file input field!
            // (in case of copied/synchronized Browse document row)
        	var selectedFileName = this.dm.getSelectedFileName();
        	if (!selectedFileName) {
        		selectedFileName = DocRefStrings.t('DocSrcHdlNoFileSelected');
        	}
            var fakeFileNameSpan = $("<span>", {'class' : 'docname-upload-text'}).text(selectedFileName);
            
            fakeFileInputField.append(fakeBrowseButtonSpan);
            fakeFileInputField.append(fakeFileNameSpan);
            
        	// add the hidden "real" file input field in the page
            fakeFileInputField.append(this.renderFileInputField());
            
            // document name
            var docNameInput;
			if (this.dm.isAutonamingEnabled()) {
				// when autonaming is on the behavior of "document name" field differs
				if (this.dm.hasAutonameSuffix()) {
					
					var docSuffixOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameSuffixChanged(this.value);';
					var docSuffix = (this.dm.docNameSuffix ? this.dm.docNameSuffix : '');
					
					docNameInput = $("<input>", {"type" : "text", 
						"class" : this.getSAHIHelperStr('docnamesuffix')+" docname-input", 
						"placeholder" : DocRefStrings.t('DocRowDocumentSuffix'),
						"value" : docSuffix,
						"onchange" : docSuffixOnChangeMethod});
				} else {
					// Generate a placeholder to represent the counter.
					docNameInput = $("<input>", {"type" : "text", 
						"class" : this.getSAHIHelperStr('docname')+" docname-input", 
						"placeholder" : this.getAutoNamingPreview(),
						"disabled": "disabled"});
				}
			} else {
				var docNameOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameChanged(this.value);';
				var docName = (this.dm.docName ? this.dm.docName : '');
				docNameInput = $("<input>", {"type" : "text",
					"class" : this.getSAHIHelperStr('docname')+" docname-input", 
					"placeholder" : DocRefStrings.t('DocRowDocumentName'),
					"value" : docName,
					"onchange" : docNameOnChangeMethod});
			}
			
			// document description
			var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
			var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
			var docDescInput = $("<input>", {"type" : "text", 
				"class" : this.getSAHIHelperStr('docdescription')+" docname-input", 
				"placeholder" : DocRefStrings.t('DocRowDocumentDescription'),
				"value" : docDesc,
				"onchange" : docDescOnChangeMethod});

			// template selector
			var templateSelector = this.renderDocTemplateSelector(config.hideDocTemplate != "true");
			
			// folder selector
			var folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");

			
			// Disable fields when not editable
            if(!this.dm.isEditable){
            	docNameInput.prop('disabled', true);
            	docDescInput.prop('disabled', true);
            	this.enableFileInputField(false);
            }
			
            // Wrapper
            var fileNameWrapper = $("<div>", {"class" : ""});
            fileNameWrapper.append(fakeFileInputField);
            
            // Responsive doc description/name
            var responsiveDocNameDescWrapper = $("<div>", {"style" : "width: 100%"});
			var responsiveDocNameWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
			var responsiveDocDescWrapper = $("<div>", {"style" : "float: left; width: 100%", "class" : "DocumentNameRow"});
			responsiveDocNameWrapper.append(docNameInput);
			responsiveDocDescWrapper.append(docDescInput);
			responsiveDocNameDescWrapper.append(responsiveDocNameWrapper, responsiveDocDescWrapper);
			
			// Responsive template/ folder selectors
			var responsiveTemplateFolderWrapper = $("<div>", {"style" : "width: 100%", "id" : this.phElemID + "_docTempFolderSelPHID"});
			var responsiveTemplateWrapper = $("<div>", {"style" : "float: left; width: 100%;"});
			var responsiveFolderWrapper = $("<div>", {"style" : "float: left; width: 100%"});
			if (templateSelector) {
				responsiveTemplateWrapper.append(templateSelector);
			} else {
				responsiveTemplateWrapper.hide();
			}
			responsiveFolderWrapper.append(folderSelector);
			responsiveTemplateFolderWrapper.append(responsiveTemplateWrapper, responsiveFolderWrapper);
			
			// Apply visibility.
			if (config.hideDescription  == "true") {
				responsiveDocDescWrapper.hide();
			}
			if (config.hideDocTemplate == "true") {
				responsiveTemplateWrapper.hide();
			}
			if (config.hideFolder == "true") {
				responsiveFolderWrapper.hide();
			}
			
			// Create a resizer for this collumn.
			this._nameColResizer = function () {
				var breakPoint = 360;
				if (responsiveDocNameDescWrapper && responsiveDocNameWrapper.is(':visible') && responsiveDocDescWrapper.is(':visible')) {
					var width = responsiveDocNameDescWrapper.width();
					if (width < breakPoint) {
						responsiveDocNameWrapper.css("width","100%");
						responsiveDocNameWrapper.css("padding-right", "0px");
						responsiveDocDescWrapper.css("width", "100%");
					} else {
						responsiveDocNameWrapper.css("width","50%");
						responsiveDocNameWrapper.css("padding-right", "2px");
						responsiveDocDescWrapper.css("width", "50%");
					}
				}
				if (responsiveTemplateFolderWrapper && responsiveTemplateWrapper.is(':visible') && responsiveFolderWrapper.is(':visible')) {
					var width = responsiveTemplateFolderWrapper.width();
					if (width < breakPoint) {
						responsiveTemplateWrapper.css("width","100%");
						responsiveTemplateWrapper.css("padding-right", "0px");
						responsiveFolderWrapper.css("width", "100%");
					} else {
						responsiveTemplateWrapper.css("width","50%");
						responsiveTemplateWrapper.css("padding-right", "2px");
						responsiveFolderWrapper.css("width", "50%");
					}
				}
			};
			$(window).resize(this._nameColResizer);
			// Also try to execute the resizer after the table has rendered, so defer the execution of the resizer.
			window.setTimeout(function () {
				this._nameColResizer();
			}.bind(this), 1);
			
			// Add the elements.
			container.append(fileNameWrapper, responsiveDocNameDescWrapper, responsiveTemplateFolderWrapper);
            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);
            
		} catch (err) {
			DocTableXUtils.log('Problem in DocumentRowBrowseView.renderDocumentSpec:\n' + err);
		}
    };
    
    /**
	 * Method called before the DocTableRow elements are thrown out of the DOM.
	 * 
	 * @param {boolean} destroyACatObj - 'true' if the "full destroy" should be done. It means
	 * 		that even attributeCategory object should be removed from the memory (all changes user 
	 * 		might have already done will be lost);
	 * 		'false'/<no value> when the attributeCategory object should be kept
	 */
    DocumentRowBrowseView.prototype.destroy = function(destroyACatObj) {
    	
    	if (!destroyACatObj) {
    		// if this is not the full destroy (e.g. when the document row is deleted)
    		// detach the file input HTML elements to "preserve" it and be able to append 
    		// it back after repaint.
    		if (this.uploadInput) {
    			this.uploadInput.detach();
    		}
    	}
	};

    return DocumentRowBrowseView;
});