define(["doctablex/StaticDocumentRow", 
        "doctablex/DocumentRow", 
        "doctablex/DocumentRowCopyView",
        "doctablex/AJAXDataCache",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
		"underscore",
        "jquery",
		"wcr/sdrop"], 
function(StaticDocumentRow,
		 DocumentRow,
         DocumentRowCopyView,
         AJAXDataCache,
         DocTableXUtils,
         DocRefStrings,
         WCXML,
         _,
         $){
    /**
     * DocumentRowCopy.js
     * 
     * This is a subclass of DocumentRow designed to handle copying of documents from selected project.
     * It's an implementation of "Select from Project" document source.
     * 
     * @author JANH, 2016
     */

    "use strict";

    var DocumentRowCopy = function(options) {
        if (!options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        // set the view class of this data model class before calling superclass constructor to be created in the base class
        if (!options.viewClass) options.viewClass = DocumentRowCopyView;
        DocumentRow.call(this, options);   // calling superclass constructor!     

        /** @member {String} dlsName - Document Source Select from Project/Copy (green link) name/label 
         * 		that creates this row. 'Copy' by default. */
        this.dlsName = options.dlsName || 'Copy';

        /** @member {String} docOperation - operation performed on the document for "Select from Project" 
         * 		document source can be Copy, Move or Link. Default is Copy. */
        this.docOperation = options.docOperation || 'Copy';

    	/** @member {String} newDocTmpID - Temporary document identifier used for new documents that doesn't
    	 * 		have WebCenter document version ID assigned yet. It's guaranteed to be unique for each new
    	 * 		document added within documents table. */
        this.newDocTmpID = options.newDocTmpID || '';
    	if ((this.docOperation == 'Copy') && !options.newDocTmpID) {
    		this.newDocTmpID = 'newID_' + this.rowID;
    	}

        /** @member {Object} selProject - Object holding ID 'id' and 'name' of the currently selected project.
         * 		It's never null or undefined. For internal structure of the object see this._emptySelProjectObj(). */
        this.selProject = options.selProject || this._emptySelProjectObj();
        
        /** @member {Object} selDocument - Object holding document 'versionID' and 'name' of the currently 
         * 		selected document. It's never null or undefined. For internal structure of the object 
         * 		see this._emptySelDocumentObj(). */
        this.selDocument = options.selDocument || this._emptySelDocumentObj();
        
        
        // private initialization ========================================

        if (this.constructor == DocumentRowCopy.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass

            // load missing document information (like available document templates) via AJAX (if not known)
            // note: this should be called only after the 'this.view' is properly initialized!
            this._initializeFromServer();

        }   // end of private initialization

        return this;
    };

    // define DocumentRowCopy as a subclass of DocumentRow
    DocumentRowCopy.prototype = new DocumentRow();
    DocumentRowCopy.prototype.constructor = DocumentRowCopy;
    DocumentRowCopy.prototype.parent = DocumentRow.prototype;


    /**
     * Private initialization method using AJAX to load asynchronously required info that is not passed by 
     * 'options' (constructor parameter) from the server.
     * 
     * @override
     */
    DocumentRowCopy.prototype._initializeFromServer = function() {
        this.setStatus('loading');

        // deferred object tracking the state of loading data from the server - it's resolved when all is loaded
    	var initFromServerState = $.Deferred();
    	
    	if (this.copyOfExistingRowID) {
            this.setStatus('ready');
    		return initFromServerState.resolve().promise();
    	}
    	
		// initialize document template selector and folder selector (if any)

    	// asynchronous load of document templates (using the AJAX cache object)
		// it's performed only if the doc. templates are not loaded yet and if a project/template
		// to load the configuration from is known! (not the case e.g. in task type definition)
		var docTemplatesReq = null;
		if (this.docOperation == 'Copy') {
            // in case of copy operation load also document templates (if any)
        	if (this.project.id) {
        		docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.project.id);
        	}
        	// else: when creating project from blank, there are no document templates to be loaded...
		}
		
		// Get the project folders 
		var folderListReq = null;
		if (this.docOperation != 'Reference'){
        	if (this.project.id) {
        		folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.project.id);
        	}
        	// else: when creating project from blank, there are no folders to be loaded...
		}

		$.when(docTemplatesReq, folderListReq).done(function (docTempList, folderList) {
			// member of this class (see definition in DocumentRow class) - flag controlling whether 
            // to use attribute category configured as default for this document type or not. 
            // Do it (value 'true') if there is no attribute category attached to the original
            // (copied/moved/linked) document, otherwise don't (value 'false')
            this.useDocTypeDefaultACat = (this.hasNoAttrCat() && (this.docOperation == 'Copy'));
            
			// get the selected file's document templates and and preselect the default one (if any)
			if (docTempList) {
				this.updateDocTemplates(docTempList);
				this.preselectDocTemplate();
			}
			// update the this.targetFolders and preselect a folder (if applicable)
			this.updateTargetFolders(folderList);
			this.preselectTargetFolder();

			// loading from the server finished
			initFromServerState.resolve();

			this.setStatus('ready');

			// repainting the document row with the loaded values/info
			this.view.repaintRow();
			// the attribute category might change too - update it's values and force expand
			this.showDocAttributeCategory(false, true);
		}.bind(this))
		.fail(function (errorMsg) {
			// loading from the server finished (with an error)
        	initFromServerState.reject();
			this.setStatus('error');
			window.alert("Problem with retrieving document templates:\n" + errorMsg);
		}.bind(this));
        
        return initFromServerState.promise();
    };
    
    
    /**
     * Private method to update document related properties of this class instance based on info loaded 
     * from server. It's overriding the StaticDocumentRow._updateDocInfo() using the the stripped down 
     * DocumentRow version to handle (mainly) "Copy" operation in a different way...
     * 
     * @override
     * @returns {boolean} - 'true' if document was changed, 'false' otherwise.
     */
    DocumentRowCopy.prototype._updateDocInfo = function(docInfo) {
    	return this._strippedUpdateDocInfo(docInfo);
    };

    
	/**
	 * This method derives all necessary information/data needed to create a new copy of the original 
	 * (== this) document row.
	 * It extends the basic implementation in superclasses.
	 * 
	 * @override
	 * @returns {Object} - Document info object == options for the constructor of any document row object.
	 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
	 * 		any "chattering" between the original and new document row.
	 */
    DocumentRowCopy.prototype.getDocRowInfo = function () {
		var docInfo = DocumentRowCopy.prototype.parent.getDocRowInfo.call(this);

		_.extend(docInfo, {
			dlsName : this.dlsName,
			selProject : this.selProject,
			selDocument : this.selDocument,
			newDocTmpID : this.newDocTmpID   // for documents not existing in the WebCenter yet
		});
		// the projects and documents SDrop(s) are reinitialized in the new/copied instance independently
		return docInfo;
	};


	/** 
	 * Private method to return an object representing an empty project selection.
	 */
    DocumentRowCopy.prototype._emptySelProjectObj = function() {
		return {
			id : '',
			name : ''
		};
    };

	/** 
	 * Private method to return an object representing an empty document selection.
	 */
    DocumentRowCopy.prototype._emptySelDocumentObj = function() {
		return {
			versionID : '',
			name : ''
		};
    };

	
    /**
     * Method to reset document related data. Supposed to be called whenever a document selection is changed!
     */
    DocumentRowCopy.prototype.resetDocument = function(resetSelection) {
    	if (resetSelection) {
    		// this is executed only when selected project is deselected (not when document selection is changed!)
			this.selProject = this._emptySelProjectObj();
			this.selDocument = this._emptySelDocumentObj();

			this.sourceProjID = null;
	        this.folder = {};
    	}
    	
		// call the super class resetDocument(), but don't propagate the call further!
        DocumentRowCopy.prototype.parent.resetDocument.call(this, false);
    };


    /**
     * Overrides the base class isAutonamingEnabled() method. 
     * 
     * @override
     * @returns {boolean} - True if autonaming is preconfigured for the document type of the currently selected file and
     * 		the document operation is 'Copy'. All other doc. operations doesn't support autonaming!
     */
    DocumentRowCopy.prototype.isAutonamingEnabled = function() {
        return (this.anPrefix != null) && (this.anPrefix.length > 0) && (this.docOperation == 'Copy');
    };

    
    /**
     * Method returning 'true', if attributes of this document can be edited in "general" - meaning
     * in the columns of documents table. Otherwise 'false' is returned.
     * 
     * Special case is the Link operation that combines permissions from copy and move operations.
     * 
     * @override
     * @returns {Boolean}
     */
    DocumentRowCopy.prototype.canUserEditAttributes = function() {
    	// the calculated "change property" permission
    	var docChangePropPerm = false;
    	
    	// applicable when an existing document is referenced or moved
    	var referencedDocChangePropPerm = (this.userPerms) ? this.userPerms.change : false;
    	// applicable when a new document is created (copied, moved or uploaded)
    	var newDocChangePropPerm = false;
    	if (this.selTargetFolder) {
    		newDocChangePropPerm = this.selTargetFolder.changePropPerm;
    	} else if (this.getParentDocTable().isBlankProjectCreationContext()) {
    		// user creating a new blank project has rights to change attributes of newly uploaded/copied
    		// documents to the new project!
    		newDocChangePropPerm = true;
    	}
    	
    	if (this.docOperation == 'Link') {
    		docChangePropPerm = referencedDocChangePropPerm || newDocChangePropPerm;
    	} else {
    		docChangePropPerm = newDocChangePropPerm;
    	}
    	
    	return !this.isInactive()
    			// no view right and/or error loading doc. details
    			&& !this.isInErrorStatus()
    			&& this.isLatestDocVersion()
    			&& !this.dependsOnItself
    			&& docChangePropPerm;
    };
    
    
    /**
     * Method generating URL to retrieve attributes (and their values) from the server.
     * This is overridden version of the basic method defined in DocumentRow class.
     * @override
     */
    DocumentRowCopy.prototype.generateDocAttributeValuesAJAXURL = function() {
        var url = window.webcenterConfig.jspBaseUrl;
        if (this.selDocTemplate && this.selDocTemplate.docVerID 
        		&& DocTableXUtils.isValidID(this.selDocTemplate.categoryID)) {
            url += 'GetAttributes.jsp?doctemplate=1';   // a flag that the document template is specified
            url += '&projectid=' + this.project.id;   // ID of project where the new document is about to be uploaded
            url += '&documentid=' + this.selDocTemplate.id;   // document ID of selected document template
            url += '&docverid=' + this.selDocTemplate.docVerID;   // doc. version ID of selected document template
            url += '&categoryid=' + this.selDocTemplate.categoryID;   // ID of selected document template's attr. category
        } else if (this.hasOrigDocACat && this.docVerID) {
            // no document template selected - the original document's attribute category should be loaded (if any)
            url += 'GetAttributes.jsp?docverid=' + this.docVerID;   // a flag that the document template is specified
        } else if (DocTableXUtils.isValidID(this.aCatID)) {
            // just load attribute types, restricted sets etc. to correctly render them (along with 
            // empty attribute values)
            url += 'GetAttributes.jsp?categoryid=' + this.aCatID;
        } else {
            DocTableXUtils.log("Cannot generate URL in DocumentRowCopy.generateDocAttributeValuesAJAXURL().");
        }
        return url;
    };
    

    /**
	 * Get the view of the AttributeCategory that should be shown
	 * @override
	 * @returns {String}
     */
    DocumentRowCopy.prototype.getAttCatView = function(){
    	var override = false;
    	
    	var expandAttrCatConfig = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout');
    	if (expandAttrCatConfig && expandAttrCatConfig.useCreationView === false) {
    		// set flag that we want to override the creation view
        	override = true;
        }
    	
    	if(!override){
    		// only for new documents return creation view
    		if (this.docOperation == 'Copy') {
    			return 'Creation';
    		}
    	}
    	
    	return this.aCatView || 'Default';
    };

    
    /**
     * Method setting the passed 'project' value (object) as currently selected project.
     * As a side effect it also clears any previous selection of document as it's reset by the new
     * project selection anyway.
     * 
     * @param {Object} project - Object representing the selected project. The structure is the same
     * 		with this.selProject (see this._emptySelProjectObj() for details).
     */
    DocumentRowCopy.prototype.setSelectedProject = function(project) {
    	if (!project || !project.id) {
    		this.selProject = this._emptySelProjectObj();
        	this.sourceProjID = null;
    	} else {
    		if (this.selProject && (this.selProject.id != project.id)) {
            	this.sourceProjID = null;   // reset document's source project if a different project was selected
    		} else {
    			// store the ID of the selected project as "source project ID"
            	this.sourceProjID = project.id;
    		}
    		this.selProject = project;
    	}
    	// don't forget to clear (potentially) already selected document (from the previous project)
    	this.setSelectedDocument(null);
    };

    /**
     * Method setting the passed 'document' value (object) as currently selected document.
     * 
     * @param {Object} document - Object representing the selected document. The structure is the same
     * 		with this.selDocument (see this._emptySelDocumentObj() for details).
     */
    DocumentRowCopy.prototype.setSelectedDocument = function(document) {
    	if (!document || !document.versionID) {
            this.selDocument = this._emptySelDocumentObj();
            this.folder = {};
    	} else {
    		if (this.docVerID != document.versionID) {
    			this.folder = {};   // reset document's source folder if a different document was selected
    		}
	    	this.selDocument = document;
    	}
		this.docVerID = this.selDocument.versionID;
		this.docName = this.selDocument.name;
    };


    /**
     * Method to update project selector (SDrop) and perform related logic when the project selector
     * is/must be updated.
     */
    DocumentRowCopy.prototype._updateProjectSelector = function() {
    	if (this.view.projectSelector && this.selProject && this.selProject.id) {
    		// there is already preselected project - select it in the SDrop as well.
    		this.view.projectSelector.setSelectedItemByID(this.selProject.id);
    	}
    };

    
    /**
     * Method to update document selector (SDrop) and perform related logic when the document selector
     * is/must be updated.
     */
    DocumentRowCopy.prototype._updateDocumentSelector = function() {
    	if (!this.view.documentSelector || !this.selProject || !this.selProject.id) {
    		return null;
    	}
	    var docListReq = AJAXDataCache.getInstance().getDocVersionsByProjID(this.selProject.id);
		docListReq.done(function(docVersionList) {
			// preprocess the loaded data to be suitable for use in SDrop data provider
			var docNameVerIDPairs = [];
			for (var i = 0; i < docVersionList.length; i++) {
				var docVer = docVersionList[i];
				if (docVer && docVer.id) {
					docNameVerIDPairs.push({
						name : docVer.name,
						id : docVer.id
					});
				}
			}
	        // initialize the new data provider based on loaded data
	        var docSelectorDP = new StaticSDropDataProvider();
	        docSelectorDP.setData(docNameVerIDPairs);
	        
	        // replace the previous data provider and show the document selector SDrop again
	    	this.view.documentSelector.setDataProvider(docSelectorDP, true);
	    	
	    	if (this.selDocument && this.selDocument.versionID) {
	    		// there is already preselected document - select it in the SDrop as well.
	    		this.view.documentSelector.setSelectedItemByID(this.selDocument.versionID);
	    	}
	    	// show the document selector
    		this.view.documentSelector.show();
    		this.view.documentSelector.enable();
	
			this.setStatus('ready');
	    }.bind(this));
		docListReq.fail(function(errorMsg) {
	        this.setStatus('error');
	        window.alert(DocRefStrings.t('DocRowLoadingDocsError') + ":\n" + errorMsg);
			// repainting the document row with the updated values/info and hide attribute category 
			// as no document is selected
			this.view.repaintRow();
			this.hideDocAttributeCategory();
	    }.bind(this));
		
		return docListReq.promise();
    };
    
    
    
    /**
     * Method generating XML to save info about newly uploaded document.
     * It calls the superclass convertToXML() method to add the common part of the XML.
     * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
     * 
     * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
     */
    DocumentRowCopy.prototype.convertToXML = function() {
        var xmlValue = '';
        try {
            if (this.isInErrorStatus() || !this.docVerID) {
                // don't generate any XML if there is an error or no document version selected 
            	return xmlValue;
            }

            // generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
            var dsXML = '<ds>';
            dsXML += WCXML.ElementEncode('action', this.docOperation);
        	if (this.newDocTmpID) dsXML += WCXML.ElementEncode('new_doc_temp_id', this.newDocTmpID);
            
            if (this.docVerID) dsXML += WCXML.ElementEncode('source_dv_id', this.docVerID);
            if (this.project.id) dsXML += WCXML.ElementEncode('project_id', this.project.id);
            if (this.selTargetFolder) {
                // get the selected target folder name in XML (if any)
            	dsXML += WCXML.ElementEncode('target_folder_name', this.selTargetFolder.name);
            }
        	if (this.isAutonamingEnabled()) {
                var suffixStr = (this.docNameSuffix ? this.docNameSuffix : '');
                dsXML += WCXML.ElementEncode('doc_name_suffix', suffixStr);
            } else {
                if (this.docName) dsXML += WCXML.ElementEncode('doc_name', this.docName);
            }
            if (this.docDesc) dsXML += WCXML.ElementEncode('doc_description', this.docDesc);

            if (this.hasAttrCat()) dsXML += WCXML.ElementEncode('doc_attr_cat_id', this.aCatID);

            if (this.selDocTemplate) {
                dsXML += WCXML.ElementEncode('doc_temp_id', this.selDocTemplate.id);
                dsXML += WCXML.ElementEncode('doc_temp_folder_name', this.selDocTemplate.folderName);
            }
            dsXML += '</ds>';

            // call the super method to generate the common part of XML
            xmlValue = DocumentRowCopy.prototype.parent.convertToXML.call(this, dsXML);

        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowCopy.convertToXML:\n' + err);
            xmlValue = '';
        }
        return xmlValue;
    };

    
    /**
     * Validation of the document selected in the drop-down list of available documents.
     * 
     * @override
     * @param {Object} validObj - Validation object holding all found validation errors together with the related 
     * 		data needed to process them and notify user. 
     * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
     * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
     * 		starting from 1.
     * 
     * @returns {Boolean} - Returns true if no problem found in this particular document row and 
     * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
     */
    DocumentRowCopy.prototype.validateDocRow = function(validObj, depth) {
    	var isOK = true;
    	var errorObj = null;
    	
    	// first check if a document is selected
    	if (!this.docVerID) {
			// compose the errorObj object referring to this situation
    		var elemToHighlight = $('#' + this.getRowPrefix());
    		if (this.view.documentSelector && this.view.documentSelector.wrapper) {
    			// highligh the document selector itself (if exists)
    			elemToHighlight = this.view.documentSelector.wrapper.parent();
    		}
			errorObj = {
				isBlocking: true,
				level: depth,
				msg: DocRefStrings.t('DocRowFileNotChosen'),
				element: elemToHighlight,
				docRowObj: this
			};
			validObj.errors.push(errorObj);
			isOK = false;
    	}
    	
    	if (this.docOperation == 'Link') {
    		// Link document operation doesn't let user to change the document name - still
    		// check there is a document name coming from the user's selection!
   			if (!this.docName) {
   				// compose the errorObj object referring to this situation
   				errorObj = {
   					isBlocking : true,
   	   				level : depth,
   					msg : DocRefStrings.t('DocRowDocNameNotSelectable'),
   					element : $('#' + this.getRowPrefix()),   // no doc. name field - select the whole row
   					docRowObj : this
   				};
   				validObj.errors.push(errorObj);
   				isOK = false;
   			}
    		
    		// Call the StaticDocumentRow validation implementation that takes care of required properties and documents (but not naming).
        	// (both in documents table column and associated attr. category).
        	if (!StaticDocumentRow.prototype.validateDocRow.call(this, validObj, depth)) {
        		isOK = false;
        	}
    	} else {
    		// Call the StaticDocumentRow validation implementation that takes care of required properties and documents and naming.
        	// (both in documents table column and associated attr. category).
        	if (!DocumentRow.prototype.validateDocRow.call(this, validObj, depth)) {
        		isOK = false;
        	}
    	}
    	
    	return isOK;
    };
    

    /**
     * Add document to validation object for checking unique document names
	 * @override
	 * @param validObj - object that gets passed to collect all document names
     */
    DocumentRowCopy.prototype.validateDocRowUniqueName = function(validObj) {
    	if (this.docOperation == 'Link') {
    		// do nothing as no new document is created by these operations...
    	} else {
    		// copy and move both add "new" document to particular project
    		DocumentRowCopy.prototype.parent.validateDocRowUniqueName.call(this, validObj);
    	}
    };

    
    
    //Action handlers =====================================

    
    /**
     * Action handler when the selected project changes.
     * We need to update the document selector based on selected project.
     * 
     * @param {Object} project - Object that contains ID and name of the selected project
     */
    DocumentRowCopy.prototype.projectSelected = function(project) {
    	
    	if ((!this.selProject.id && !project) || (this.selProject.id == project.id)) {
    		// no project was selected and it is still not selected OR the new selected project is 
    		// the same as before - do nothing
    		return;
    	}
		this.setStatus('loading');
		
    	this.setSelectedProject(project);
    	
    	// flag this document row as modified by user
		this.setIsChanged();
        this.setIsChangedByUser(false);

    	// project got deselected
    	if (!this.selProject.id) {
    		// reset the selected document as no project is currently selected
    		this.resetDocument(true);
    		
			// repainting the document row with the updated values/info and hide attribute category 
			// as no document can be selected
			this.view.repaintRow();
			this.hideDocAttributeCategory();

			this.setStatus('ready');
			return;
    	}
    	
    	// a valid project was selected - initialize project documents selector
    	if (this.view.documentSelector) {
    		// update the document selector based on the current state
    		this._updateDocumentSelector(true);
        }
    };
    

    /**
     * Action handler when the selected document changes.
     * We need to ask the server information about the selected document
     * 
     * @param {Object} document - Object that contains document name and version ID of the selected document.
     * @param {Boolean} internalUpdate - true if this method was called internally, false (or undefined)
     * 		if a user action triggers this method call.
     * @returns 
     * 		- null if nothing has to be done, 
     * 		- a Promise object telling when the update operation finishes, when the update has to be done.
     */
    DocumentRowCopy.prototype.documentSelected = function(document, internalUpdate) {
    	
    	if ((!this.docVerID && !document) || (this.docVerID == document.versionID)) {
    		// no document was selected and it is still not selected OR the new selected document is 
    		// the same as before - do nothing
    		return null;
    	}
		this.setStatus('loading');

		this.resetDocument(false);
		
		// update internal members to reflect the selected document change!
		this.setSelectedDocument(document);

		if (!internalUpdate) {
			this.setIsChanged();
	        this.setIsChangedByUser(false);
		}

		// document got deselected
		if (!this.docVerID) {
			// repainting the document row with the updated values/info and hide attribute category 
			// as no document is selected
			this.view.repaintRow();
			this.hideDocAttributeCategory();

			this.setStatus('ready');
			return null;
		} 

		// get info about the newly selected document (repaint is called explicitly later)
		var docInfoReq = this.updateDocInfoFromServer(false);

        // load effective document type configuration to get autonaming setup out of that
        var effDocTypeSetupReq = null;
        if (this.project.id) {
        	effDocTypeSetupReq = AJAXDataCache.getInstance().getEffectiveDocTypesSettingsByProjID(this.project.id);
        } else {
        	// case of project created from blank
        	effDocTypeSetupReq = AJAXDataCache.getInstance().getProjCreationSysDefaults();
        }

		$.when(effDocTypeSetupReq, docInfoReq).done(function (docTypesConfig, docInfo) {
	        if (this.docOperation == 'Copy') {
	           	// description of the source document should not be passed over in case of copy
	        	// (the rest of document sources cannot change description so it's there for info reasons)
	       		this.docDesc = '';
	       		
				// in case of copy make sure autonaming configuration applicable to the copied document is 
	       		// initialized  correctly (note the selected file's document type is already known from
	       		// this.updateDocInfoFromServer() method)
				this.updateDocumentType(docTypesConfig, false);
	        }
	        
			if (!this.isModifiedByUser('docTemplate')) {
				// run the document template pre-selection calculation
				this.preselectDocTemplate();

				if (this.defTaskDocAttrCatID) {
					// overwrite attribute category by default attribute category of a task type (if defined)
					// this can happen only in task documents context (on TEP)!
					this.aCatID = this.defTaskDocAttrCatID;
				}
			} else {
				// a document template was selected by user - look for attribute category of the doc. template
				if (this.selDocTemplate) {
					this.aCatID = this.selDocTemplate.categoryID;
				}
			}

			// also adapt target folder selection if needed
			this.preselectTargetFolder();

			this.setStatus('ready');

			// repainting the document row with the updated values/info
			this.view.repaintRow();
			// the attribute category might change too, but that's recognized by 
			// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
			// the decision on the logic within "show attribute category" logic; 
			// force expand the attribute category
			this.showDocAttributeCategory(true, true);
		}.bind(this));
		docInfoReq.fail(function(errorMsg) {   // on error
			this.setStatus('error');
			DocTableXUtils.error('Failed to load document information from the server:\n' + errorMsg);
		}.bind(this));
		docInfoReq.always(function() {
			if (!internalUpdate) {
				this.setFieldModByUser('docName', !this.docName);
			}
		}.bind(this));

		return docInfoReq.promise();
    };
    
    
    return DocumentRowCopy;
});