define(["doctablex/DocumentRowView",
        "doctablex/DocTableXUtils",
        "component/sdrop/SDrop",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(DocumentRowView, 
          DocTableXUtils,
          SDrop,
          DocRefStrings,
          WCXML){
    /**
     * DocumentRowCopyView.js
     * 
     * This class implements rendering methods for "Copy" document row ("Select from Project" document source).
     * 
     * @author JANH, 2016
     */

    "use strict";

    var DocumentRowCopyView = function(dataModel, phElementID, options) {
        if (!dataModel && !phElementID && !options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        DocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

        // note this.dataModel and phElemID are initialized in the super class...)

        /** @member {jQuery object} - projectSelectorElem and documentSelectorElem are HTML selects that
         * 		are used to initialize corresponding SDrops. They are created just once and then reused.
         * 		They (as well as SDrops) survive repaints! */
    	this.projectSelectorElem = null;
    	this.documentSelectorElem = null;

        /** @member {SDrop object} - projectSelector and documentSelector are SDrop instances used to 
         * 		select projects and documents resp. They are created just once and then reused 
         * 		(they survive repaints)! */
    	this.projectSelector = null;
    	this.documentSelector = null;
    	
        // private initialization ========================================

        if (this.constructor == DocumentRowCopyView.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass

        }

        return this;
    };

    // define DocumentRowCopyView as a subclass of DocumentRowView
    DocumentRowCopyView.prototype = new DocumentRowView();
    DocumentRowCopyView.prototype.constructor = DocumentRowCopyView;
    DocumentRowCopyView.prototype.parent = DocumentRowView.prototype;


    // the main render method is inherited from the base class



    /**
     * The render method generating HTML for "Current Project" (Reference) document source to be able to 
     * select a document from the current project and add it to this documents table (value of actual
     * document reference attribute)
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     * @param {Object} config - Holds the documents table rendering configuration for this column/item.
     * @returns {String} - The generated HTML.
     */
    DocumentRowCopyView.prototype.renderDocumentSpec = function(phElem, config) {
    	try {
			// unregister the old resizer for this column
			if (this._nameColResizer) {
				$(window).off("resize", this._nameColResizer);
			}

			// create the collumn element
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));
            
            // start building the real content of the document specification column
            
            if (this.dm.isInErrorStatus()) {
            	if (this.projectSelector) {
            		this.projectSelector.destroy();
            	}
            	if (this.documentSelector) {
            		this.documentSelector.destroy();
            	}
           		var message = $('<span>').text(DocRefStrings.t('DocRowLoadingDocsError'));
           		container.append(message);
            } else {

                // document row "label" with selected doc. template name
            	var sourceLabel = $('<div>', {'style' : 'float: left;', 'class' : 'docname-label inline-element'}).text(this.dm.dlsName + ':');
            	
				var responsiveProjSelectorWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentSelectorRow"});
				var responsiveDocSelectorWrapper = $("<div>", {"style" : "float: left; width: 100%", "class" : "DocumentSelectorRow"});


            	// one time operation to create and initialize project and document selector SDrops
            	if (!this.projectSelectorElem) {
	            	this.projectSelectorElem = $('<select>', {
	            		'class' : 'project-selector-SDrop'
	            	});
					responsiveProjSelectorWrapper.append(sourceLabel, this.projectSelectorElem);

	                // SDrop initialization can be done only after the placeholder(s) are added to the DOM!
                	this.projectSelector = this.initializeProjectSelectorSDrop();
    	            if (this.projectSelector) {
    	            	// update the project selector based on the current state
    	            	this.dm._updateProjectSelector();
    	            }
            	} else if (this.projectSelector) {
            		// SDrop already initialized - add it to the DOM again
            		responsiveProjSelectorWrapper.append(sourceLabel, this.projectSelector.wrapper);
            	} 

            	
            	if (!this.documentSelectorElem) {
	            	this.documentSelectorElem = $('<select>', {
	            		'class' : 'document-selector-SDrop'
	            	});
					responsiveDocSelectorWrapper.append(this.documentSelectorElem);

	                // SDrop initialization can be done only after the placeholder(s) are added to the DOM!
    	            this.documentSelector = this.initializeDocumentSelectorSDrop();
    	            if (this.documentSelector) {
    	            	// update the document selector based on the current state
    	            	this.dm._updateDocumentSelector();
    	            }
            	} else if (this.documentSelector) {
            		// SDrop already initialized - add it to the DOM again
            		responsiveDocSelectorWrapper.append(this.documentSelector.wrapper);
            	}
            	
            	// SDrop selector are not recreated so they need to be re-enabled if disabled once...
            	this.projectSelector.setDisabled(!this.dm.isEditable);
            	this.documentSelector.setDisabled(!this.dm.isEditable);

            	
            	// document name
            	var docNameInput = $("<input>").hide();
            	if (this.dm.docOperation != 'Link') {
	                if (this.dm.isAutonamingEnabled()) {
	                    // when autonaming is on the behavior of "document name" field differs
	                    if (this.dm.hasAutonameSuffix()) {
							// replace the document name field by document suffix field
	                        var docSuffixOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameSuffixChanged(this.value);';
	                        var docSuffix = (this.dm.docNameSuffix ? this.dm.docNameSuffix : '');
	                        docNameInput = $("<input>", {"type" : "text", 
	    						"class" : this.getSAHIHelperStr('docnamesuffix')+" docname-input", 
	    						"placeholder" : DocRefStrings.t('DocRowDocumentSuffix'),
	    						"value" : docSuffix,
	    						"onchange" : docSuffixOnChangeMethod});
						} else {
							docNameInput = $("<input>", {"type" : "text", 
								"class" : this.getSAHIHelperStr('docname')+" docname-input", 
								"placeholder" : this.getAutoNamingPreview(),
								"disabled": "disabled"});
	                    }
	                } else {
	                	// autonaming is off user can change name of copied and/or moved documents only!
	                	var docName = (this.dm.docName ? this.dm.docName : '');
	                    var docNameOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameChanged(this.value);';
	                    docNameInput = $("<input>", {"type" : "text",
	    					"class" : this.getSAHIHelperStr('docname')+" docname-input", 
	    					"placeholder" : DocRefStrings.t('DocRowDocumentName'),
	    					"value" : docName,
	    					"onchange" : docNameOnChangeMethod});
	                }
                } else {
                	docNameInput = $("<span>", {"class" : "docname-text"}).text(this.dm.docName);
                }

				// document description
                var docDescInput = $("<div>").hide();
				var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
				var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
	            if (this.dm.docOperation == 'Copy') {
					docDescInput = $("<input>", {"type" : "text", 
						"class" : this.getSAHIHelperStr('docdescription')+" docname-input", 
						"placeholder" : DocRefStrings.t('DocRowDocumentDescription'),
						"value" : docDesc,
						"onchange" : docDescOnChangeMethod});
	            } else if (docDesc) {
	            	docDescInput.show();
		        	var label = $("<span>", {"class" : "docname-label"}).text(DocRefStrings.t('DocRowDocumentDescription') + ':');
		        	var description = $("<span>", {"class" : "docname-text"}).text(docDesc);
		        	docDescInput.append(label, description);
	            }
	
	            // document template selector (if any)
				var templateSelector = $("<div>").hide();
	            if (this.dm.docOperation == 'Copy') {
					// document template selector available only for "Copy" document operation
					templateSelector = this.renderDocTemplateSelector(config.hideDocTemplate != "true");
	            }	
	            
				// folder selector (is visible for all supported document operations)
				var folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");
	
				
				// Disable fields when not editable
	            if (!this.dm.isEditable) {
	            	docNameInput.prop('disabled', true);
	            	docDescInput.prop('disabled', true);
	            }

	            // Responsive project and document selector
	            var responsiveProjDocSelectorWrapper = $("<div>", {"style" : "width: 100%"});
				responsiveProjDocSelectorWrapper.append(responsiveProjSelectorWrapper, responsiveDocSelectorWrapper);

	            // Responsive doc description/name
	            var responsiveDocNameDescWrapper = $("<div>", {"style" : "width: 100%"});
				var responsiveDocNameWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
				var responsiveDocDescWrapper = $("<div>", {"style" : "float: left; width: 100%", "class" : "DocumentNameRow"});
				responsiveDocNameWrapper.append(docNameInput);
				responsiveDocDescWrapper.append(docDescInput);
				responsiveDocNameDescWrapper.append(responsiveDocNameWrapper, responsiveDocDescWrapper);
				
				// Responsive template/folder selectors
				var responsiveTemplateFolderWrapper = $("<div>", {"style" : "width: 100%", "id" : this.phElemID + "_docTempFolderSelPHID"});
				var responsiveTemplateWrapper = $("<div>", {"style" : "float: left; width: 100%;"});
				var responsiveFolderWrapper = $("<div>", {"style" : "float: left; width: 100%"});
				if (templateSelector) {
					responsiveTemplateWrapper.append(templateSelector);
				} else {
					responsiveTemplateWrapper.hide();
				}
				responsiveFolderWrapper.append(folderSelector);
				responsiveTemplateFolderWrapper.append(responsiveTemplateWrapper, responsiveFolderWrapper);
				
				// Apply visibility.
				if (config.hideDescription  == "true") {
					responsiveDocDescWrapper.hide();
				}
				if (config.hideDocTemplate == "true") {
					responsiveTemplateWrapper.hide();
				}
				if (config.hideFolder == "true") {
					responsiveFolderWrapper.hide();
				}
				
				var labelWidth = null;
				
				// Create a resizer for this column.
				this._nameColResizer = function () {
					var breakPoint = 360;
					var width = 0;
					if (responsiveProjDocSelectorWrapper && responsiveProjSelectorWrapper.is(':visible') && responsiveDocSelectorWrapper.is(':visible')) {
						width = responsiveProjDocSelectorWrapper.width();
						if (width < breakPoint) {
							responsiveProjSelectorWrapper.css("width","100%");
							responsiveProjSelectorWrapper.css("padding-right", "0px");
							responsiveDocSelectorWrapper.css("width", "100%");
						} else {
							responsiveProjSelectorWrapper.css("width","50%");
							responsiveProjSelectorWrapper.css("padding-right", "2px");
							responsiveDocSelectorWrapper.css("width", "50%");
						}
						if (labelWidth == null) {
							labelWidth = sourceLabel.width() + 10;
						}
						var projSelWidth = responsiveProjSelectorWrapper.width();

						var labelPerc = Math.ceil(labelWidth / projSelWidth * 100);
						var projSelPerc = 100 - labelPerc;
						
						this.projectSelector.wrapper.addClass("inline-element");

						sourceLabel.css({"width": labelPerc + "%", "padding-top": "5px"});
						this.projectSelector.wrapper.css("width", projSelPerc + "%");
					}
					if (responsiveDocNameDescWrapper && responsiveDocNameWrapper.is(':visible') && responsiveDocDescWrapper.is(':visible')) {
						width = responsiveDocNameDescWrapper.width();
						if (width < breakPoint) {
							responsiveDocNameWrapper.css("width","100%");
							responsiveDocNameWrapper.css("padding-right", "0px");
							responsiveDocDescWrapper.css("width", "100%");
						} else {
							responsiveDocNameWrapper.css("width","50%");
							responsiveDocNameWrapper.css("padding-right", "2px");
							responsiveDocDescWrapper.css("width", "50%");
						}
						if (responsiveDocNameWrapper.height() > responsiveDocDescWrapper.height()) {
							responsiveDocDescWrapper.css("height", responsiveDocNameWrapper.height() + "px");
						} else {
							responsiveDocNameWrapper.css("height", responsiveDocDescWrapper.height() + "px");
						}
					}
					if (responsiveTemplateFolderWrapper && responsiveTemplateWrapper.is(':visible') && responsiveFolderWrapper.is(':visible')) {
						width = responsiveTemplateFolderWrapper.width();
						if (width < breakPoint) {
							responsiveTemplateWrapper.css("width","100%");
							responsiveTemplateWrapper.css("padding-right", "0px");
							responsiveFolderWrapper.css("width", "100%");
						} else {
							responsiveTemplateWrapper.css("width","50%");
							responsiveTemplateWrapper.css("padding-right", "2px");
							responsiveFolderWrapper.css("width", "50%");
						}
					}
				}.bind(this);
				$(window).resize(this._nameColResizer);
				// Also try to execute the resizer after the table has rendered, so defer the execution of the resizer.
				window.setTimeout(function () {
					this._nameColResizer();
				}.bind(this), 1);
				
				// Add the elements.
				container.append(responsiveProjDocSelectorWrapper, responsiveDocNameDescWrapper, responsiveTemplateFolderWrapper);
            }

            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);

    	} catch (err) {
            DocTableXUtils.log('Problem in DocumentRowCopyView.renderDocumentSpec:\n' + err);
        }
    };
    

    /** 
     * Method to create and initialize an SDrop instance for project selector. 
     * 
     * @returns {SDrop object} - Created SDrop instance.
     */
    DocumentRowCopyView.prototype.initializeProjectSelectorSDrop = function() {
    	if (!this.projectSelectorElem) {
    		// placeholder element for project selector SDrop not created yet
    		return null;
    	} 
    	if (this.projectSelectorElem[0] && this.projectSelectorElem[0].SDrop) {
    		// the project selector SDrop is already initialized
    		return this.projectSelectorElem[0].SDrop;
    	}

    	// first initialize data provider
        var projectSelectorDP =  new SDropDataProvider({
        	dataSource : 'GetNameIDList.jsp?list=VisibleProjects'
        });
        
        // SDrop initialization
        var projSelector = new SDrop(this.projectSelectorElem, null, projectSelectorDP, {
			manageStyleAndWidth: false,
			minWidth : 400,
			maxWidth : 600,
			autoload : true
        });
        projSelector.options.i18n.startTyping = DocRefStrings.t('DocRowCopySelectProject');

        // add listener
        this.projectSelectorElem.change(function (event) {
        	// gather data
        	var projID = projSelector.getSelectedItemID();
        	// call the internal logic with selected project data
        	this.dm.projectSelected(projSelector.dataProvider.getById(projID));
        }.bind(this));

        return projSelector;
    };

    
    /** 
     * Method to create and initialize an SDrop instance for document selector. 
     * 
     * @returns {SDrop object} - Created SDrop instance.
     */
    DocumentRowCopyView.prototype.initializeDocumentSelectorSDrop = function() {
    	if (!this.documentSelectorElem) {
    		// placeholder element for document selector SDrop not created yet
    		return null;
    	}
    	if (this.documentSelectorElem[0] && this.documentSelectorElem[0].SDrop) {
    		// the document selector SDrop is already initialized
    		return this.documentSelectorElem[0].SDrop;
    	}    		
    	
    	// document selector SDrop not initialized yet - perform the one time creation
        var tmpDataProvider = new StaticSDropDataProvider();
        var docSelector = new SDrop(this.documentSelectorElem, null, tmpDataProvider, {
   			manageStyleAndWidth: false,
   			minWidth : 400,
   			maxWidth : 600,
   			autoload : true
        });
        docSelector.options.i18n.startTyping = DocRefStrings.t('DocRowCopySelectDocument');

        // add listener
        this.documentSelectorElem.change(function (event) {
        	// gather data
        	var docVerID = docSelector.getSelectedItemID();
        	var name = docSelector.dataProvider.getById(docVerID).name;
        	var aDocument = (docVerID) ? {
        		versionID : docVerID,
        		name : name
        	} : null;
        	// call the internal logic 
        	var internalUpdate = true;
        	this.dm.documentSelected(aDocument, internalUpdate);
        }.bind(this));

    	return docSelector;
    };
    
    
    /**
	 * Method rendering the thumbnail or its placeholder (if it's not loaded yet).
	 * This override variation of the method always draws the thumbnail representing a new, unknown document.
	 * 
	 * @override
	 * @returns {String} - The generated HTML.
	 */
    DocumentRowCopyView.prototype.renderThumbnail = function(phElem, config) {
        try {
            if (this.dm.docVerID) {
            	DocumentRowCopyView.prototype.parent.renderThumbnail.call(this, phElem, config);
            } else {
                var htmlcode = '<' + this.itemRootElemStr + '>';
    			htmlcode += this.customThumbnailHTML('thumbnails/other.gif', config.thumbnailSize);
            	htmlcode += '</' + this.itemRootElemStr + '>';
                phElem.append(htmlcode);
            }
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowCopyView.renderThumbnail:\n' + err);
        }
    };

    
    /**
	 * Method called before the DocTableRow elements are thrown out of the DOM.
	 * 
	 * @param {boolean} destroyACatObj - 'true' if the "full destroy" should be done. It means
	 * 		that even attributeCategory object should be removed from the memory (all changes user 
	 * 		might have already done will be lost);
	 * 		'false'/<no value> when the attributeCategory object should be kept
	 */
    DocumentRowCopyView.prototype.destroy = function(destroyACatObj) {
    	if (destroyACatObj) {
    		// full destroy including data objects
    		if (this.projectSelector) {
    			this.projectSelector.destroy();
    		}
    		if (this.documentSelector) {
    			this.documentSelector.destroy();
    		}
    	} else {
    		// if this is not the full destroy (e.g. when the document row is deleted)
    		// detach the project and document selector HTML elements to "preserve" them
    		// and be able to append them back after repaint.
    		if (this.projectSelector && this.projectSelector.wrapper) {
    			this.projectSelector.wrapper.detach();
    		}
    		if (this.documentSelector && this.documentSelector.wrapper) {
    			this.documentSelector.wrapper.detach();
    		}
    	}
	};

	
    return DocumentRowCopyView;
});