define(["doctablex/DocumentRow",
        "doctablex/DocumentRowExternalFileView",
        "doctablex/AJAXDataCache",
        "doctablex/DocTableXUtils",
        "doctablex/DocumentTypeUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
        "jquery",
        "underscore"
    ],
    function (DocumentRow,
        DocumentRowExternalFileView,
        AJAXDataCache,
        DocTableXUtils,
        DocumentTypeUtils,
        DocRefStrings,
        WCXML,
        $,
        _) {

        /**
         * DocumentRowExternalFile.js
         * 
         * This is a subclass of DocumentRow designed to handle adding of documents via external id (upload operation).
         * 
         * @author BDGE - October 2016
         */

        "use strict";

        var DocumentRowExternalFile = function (options) {
            if (!options) {
                // when there is nothing to initialize from, return just itself (not initialized)
                return this;
            }
            // set the view class of this data model class before calling superclass constructor to be created in the base class
            if (!options.viewClass) {
                options.viewClass = DocumentRowExternalFileView;
            }

            DocumentRow.call(this, options); // calling superclass constructor!

            /** @member {String} dlsName - Document Source Link (green link) name/label that creates this row. */
            this.dlsName = options.dlsName || 'Upload';

            /**
             * @member {String} docOperation - operation performed on the document for "Browse" document source
             * 		is always Upload.
             */
            this.docOperation = 'DAM';

            /** @member {String} newDocTmpID - Temporary document identifier used for new documents that doesn't
             * 		have WebCenter document version ID assigned yet. It's guaranteed to be unique for each new
             * 		document added within documents table. */
            this.newDocTmpID = options.newDocTmpID ? options.newDocTmpID : 'newID_' + this.rowID;

            /** @member {String} externalId - This id points towards a document on an other system. */
            this.externalId = options.externalId;

            /** @member {Array<String>} allowedFileExts - List of file extensions allowed by the corresponding document source. */
            this.allowedFileExts = options.allowedFileExts || [];

            // an uploaded document will always end up in the project of the parent (actual) documents table 
            // (description in DocumentRow constructor)
            this.project = {
                id: this.getParentDocTable().projectID
            };

            // A reference to the current DAM connection
            this.DAM = options.DAM;

            // get the plugin-specific Javascript object containing funtions to handle data
            this.DAMJS = options.DAMJS;

            // Contains configuration settings for the current DAM connection
            this.DAMData = options.DAMData;

            // Contains the attributes(WCR)/metadata(MB) of the external document
            this.externalDocumentMetadata = options.externalDocumentMetadata;

            // True if the attributes of the document have been parsed and set based on metadata of external document
            this.areAttributesReplaced = false;

            /** @member {boolean} useDocTypeDefaultACat - Overwritten default value from DocumentRow class.
             * 		It's a flag determining whether to use document type attribute category as the default 
             * 		attr. category for this document (row) if none is imposed (via document template etc.). */
            this.useDocTypeDefaultACat = true;

            this.setStatus('loading');

            // get the file extension and remove the leading dot ('.') from it.
            this.origFileExt = DocTableXUtils.getFileExtension(this.docName, true);

            // check if the extension of the file is among allowed extensions
            // (if no extensions specified, all extensions are allowed)
            if (this.allowedFileExts && (this.allowedFileExts.length > 0) && (this.allowedFileExts.indexOf(this.origFileExt) == -1)) {
                // the extension is not allowed - alert user about that and reset the file selection
                window.alert(DocRefStrings.t('notAllowedDocExtension') + this.allowedFileExts.toString());
                this.setStatus('error');
                return this;
            }

            if (this.constructor == DocumentRowExternalFile.prototype.constructor) {
                // this part of initialization is private for the instances of this class only!
                // it won't be executed for any subclass

                // load missing document information (like available document templates) via AJAX (if not known)
                // note: this should be called only after the 'this.view' is properly initialized!
                this._initializeFromServer();

            }

            return this;
        };

        // define DocumentRowExternalFile as a subclass of DocumentRow
        DocumentRowExternalFile.prototype = new DocumentRow();
        DocumentRowExternalFile.prototype.constructor = DocumentRowExternalFile;


        /**
         * Private initialization method using AJAX to load asynchronously required info that is not passed by 
         * 'options' (constructor parameter) from the server.
         * Note there is no this.updateDocInfoFromServer() call is the represented document is not yet a WebCenter
         * document, so no info can be retrieved about it from the server...
         * 
         * @override
         */
        DocumentRowExternalFile.prototype._initializeFromServer = function () {
            if (this.copyOfExistingRowID) {
                this.setStatus('ready');
                return;
            }

            // initialize document template selector (if any)
            if (!this.docTemplates && this.project.id) {
                this.setStatus('loading');

                // asynchronous load of document templates (using the AJAX cache object)
                // it's performed only if the doc. templates are not loaded yet and if a project/template
                // to load the configuration from is known! (not the case e.g. in task type definition)
                var docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.project.id);

                // Get the project folders 
                var folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.project.id);

                // Get the individual settings for all doctypes
                var effDocTypeSetupReq = AJAXDataCache.getInstance().getEffectiveDocTypesSettingsByProjID(this.project.id);

                $.when(effDocTypeSetupReq, docTemplatesReq, folderListReq).done(function (docTypesConfig, docTempList, folderList) {

                        // get the selected file's document type based on its extension
                        this.docTypeID = DocumentTypeUtils.getDocTypeIDByExtension(this.origFileExt);
                        // get the selected file's document type and all the derived info like default attribute category and 
                        // autonaming configuration
                        this.updateDocumentType(docTypesConfig, false);

                        // get the selected file's document templates and and preselect the default one (if any)
                        this.updateDocTemplates(docTempList);
                        this.preselectDocTemplate();

                        if (this.defTaskDocAttrCatID) {
                            // overwrite attribute category by default attribute category of a task type (if defined)
                            // this can happen only in task documents context (on TEP)!
                            this.aCatID = this.defTaskDocAttrCatID;
                        }

                        // update the this.targetFolders and preselect a folder (if applicable)
                        this.updateTargetFolders(folderList);
                        this.preselectTargetFolder();

                        this.setStatus('ready');

                        this.updateOtherInfoFromServer(true, false);

                        // repainting the document row with the loaded values/info
                        this.view.repaintRow();

                        // the attribute category might change too - update it's values and force expand
                        this.showDocAttributeCategory(false, true);
                    }.bind(this))
                    .fail(function (errorMsg) {
                        this.setStatus('error');
                        window.alert("Problem with retrieving document templates:\n" + errorMsg);
                    }.bind(this))
                    .always(function () {
                        // mark this document row as changed (dirty). It can be done only after the call to this.resetDocument() 
                        // as it resets the 'isChanged' flag too!
                        this.setIsChanged();
                    }.bind(this));

            } else if (this.docTemplates || this.getParentDocTable().isBlankProjectCreationContext()) {
                this.setStatus('ready');
            } else {
                this.setStatus('error');
            }
        };

        /**
         * This method derives all necessary information/data needed to create a new copy of the original 
         * (== this) document row.
         * It extends the basic implementation in superclasses.
         * 
         * @override
         * @returns {Object} - Document info object == options for the constructor of any document row object.
         * 		It contains all needed data derived from the original document row and deeply cloned to prevent
         * 		any "chattering" between the original and new document row.
         */
        DocumentRowExternalFile.prototype.getDocRowInfo = function () {
            var docInfo = DocumentRow.prototype.getDocRowInfo.call(this);
            _.extend(docInfo, {
                newDocTmpID: this.newDocTmpID, // for documents not existing in the WebCenter yet
                allowedFileExts: this.allowedFileExts,
                externalId: this.externalId,
                DAM: this.DAM,
                DAMJS: this.DAMJS,
                DAMData: this.DAMData,
                externalDocumentMetadata: this.externalDocumentMetadata
            });

            return docInfo;
        };

        /**
         * Callback method to perform required steps after an attribute category is loaded and rendered on the page
         * (but not necessarily shown).
         * For reference category it also updates the attribute values by corresponding property values.
         * 
         * @param {String} category - Value defining type/use of just loaded attribute category.
         * 		It supports the following values: 
         * 		- 'docCat' for attribute category assigned to the referenced document, 
         * 		- 'refPropsCat' for reference properties attribute category,
         * 		- 'projInfoCat' for project attribute category used to display project info inline in document row.
         * @override
         */
        DocumentRowExternalFile.prototype._onAttrCategoryReady = function (category) {
            if (!this.areAttributesReplaced) {
                this.areAttributesReplaced = true;

                if (this.externalDocumentMetadata && this.DAMJS.getAttributeValueMapping) {
                    // fetch attribute map from plugin
                    var mapping = this.DAMJS.getAttributeValueMapping(this.externalDocumentMetadata, this.DAMData);

                    // run over attribute map and replace values
                    _.each(mapping, function (map) {
                        var attr = this.aCatObj.findAttribute(map.id || -1, map.name);
                        if (attr) {
                            attr.setValue(map.value);
                        }
                    }.bind(this));

                    // Redisplay to show changes
                    this.aCatObj.displayAttributeCategory();
                }
            }

            DocumentRow.prototype._onAttrCategoryReady.call(this, category);
        };


        /**
         * Method generating XML to save info about newly uploaded document.
         * It calls the superclass convertToXML() method to add the common part of the XML.
         * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
         * 
         * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
         */
        DocumentRowExternalFile.prototype.convertToXML = function () {
            var xmlValue = '';
            try {
                if (this.isInErrorStatus()) {
                    // don't generate any XML if there is an error or no document selected 
                    // (output would be just <av></av>).
                    return xmlValue;
                }

                // generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
                var dsXML = '<ds>';
                if (this.docOperation) {
                    dsXML += WCXML.ElementEncode('action', this.docOperation);
                }
                if (this.newDocTmpID) {
                    dsXML += WCXML.ElementEncode('new_doc_temp_id', this.newDocTmpID);
                }
                if (this.selTargetFolder) {
                    dsXML += WCXML.ElementEncode('target_folder_name', this.selTargetFolder.name);
                }
                if (this.isAutonamingEnabled()) {
                    var suffixStr = (this.docNameSuffix ? this.docNameSuffix : '');
                    dsXML += WCXML.ElementEncode('doc_name_suffix', suffixStr);
                }
                if (this.project.id) {
                    dsXML += WCXML.ElementEncode('project_id', this.project.id);
                }
                if (this.docName) {
                    dsXML += WCXML.ElementEncode('doc_name', this.docName);
                }
                if (this.docDesc) {
                    dsXML += WCXML.ElementEncode('doc_description', this.docDesc);
                }
                if (this.externalId) {
                    dsXML += WCXML.ElementEncode('doc_external_id', this.externalId);
                }
                if (this.DAM) {
                    dsXML += WCXML.ElementEncode('doc_dam_provider', this.DAM.provider);
                    dsXML += WCXML.ElementEncode('doc_dam_id', this.DAM.id);
                }
                dsXML += '</ds>';

                // call the super method to generate the common part of XML
                xmlValue = DocumentRow.prototype.convertToXML.call(this, dsXML);

            } catch (err) {
                DocTableXUtils.log('Problem in DocumentRowExternalFile.convertToXML:\n' + err);
                xmlValue = '';
            }
            return xmlValue;
        };


        return DocumentRowExternalFile;
    });
