define(["doctablex/DocumentRowView",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings"
    ],
    function (DocumentRowView,
        DocTableXUtils,
        DocRefStrings) {
        /**
         * DocumentRowExternalFileView.js
         * 
         * This class implements rendering methods for uploading an external document
         * 
         * @author BDGE - November 2016
         */

        "use strict";

        var DocumentRowExternalFileView = function (dataModel, phElementID, options) {
            if (!dataModel && !phElementID && !options) {
                return this;
            }
            DocumentRowView.call(this, dataModel, phElementID, options); // calling superclass constructor!

            return this;
        };

        // define DocumentRowExternalFileView as a subclass of DocumentRowView
        DocumentRowExternalFileView.prototype = new DocumentRowView();
        DocumentRowExternalFileView.prototype.constructor = DocumentRowExternalFileView;

        /**
         * Method rendering the thumbnail or its placeholder (if it's not loaded yet).
         * This override variation of the method always draws the thumbnail representing a new, unknown document.
         * 
         * @override
         * @returns {String} - The generated HTML.
         */
        DocumentRowExternalFileView.prototype.renderThumbnail = function (phElem, config) {
            var thumbSize = config.thumbnailSize || 50;
            phElem.append($('<td>', {
                'class': "td-narrow"
            }).append(
                $('<img>', {
                    "src": this.dm.thumbnailURL || 'thumbnails/other.gif'
                }).css("max-width", thumbSize + "px").css("max-height", thumbSize + "px")
            ));
        };


        /**
         * The render method generating HTML for external file (Upload) document source to be able to select a new document
         * for upload to WebCenter including name, description and document template (if any) definition fields.
         * 
         * @override
         * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
         * @returns {String} - The generated HTML.
         */
        DocumentRowExternalFileView.prototype.renderDocumentSpec = function (phElem, config) {
            try {
                // document name
                var docNameInput;
                if (this.dm.isAutonamingEnabled()) {
                    // when autonaming is on the behavior of "document name" field differs
                    if (this.dm.hasAutonameSuffix()) {

                        var docSuffixOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameSuffixChanged(this.value);';
                        var docSuffix = (this.dm.docNameSuffix ? this.dm.docNameSuffix : '');

                        docNameInput = $("<input>", {
                            "type": "text",
                            "class": this.getSAHIHelperStr('docnamesuffix') + " docname-input",
                            "placeholder": DocRefStrings.t('DocRowDocumentSuffix'),
                            "value": docSuffix,
                            "onchange": docSuffixOnChangeMethod
                        });
                    } else {
                        // Generate a placeholder to represent the counter.
                        docNameInput = $("<input>", {
                            "type": "text",
                            "class": this.getSAHIHelperStr('docname') + " docname-input",
                            "placeholder": this.getAutoNamingPreview(),
                            "disabled": "disabled"
                        });
                    }
                } else {
                    var docNameOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameChanged(this.value);';
                    var docName = (this.dm.docName ? this.dm.docName : '');
                    docNameInput = $("<input>", {
                        "type": "text",
                        "class": this.getSAHIHelperStr('docname') + " docname-input",
                        "placeholder": DocRefStrings.t('DocRowDocumentName'),
                        "value": docName,
                        "onchange": docNameOnChangeMethod
                    });
                }

                var responsiveDocNameWrapper = $("<div>", {
                    "style": "float: left; width: 100%;",
                    "class": "DocumentNameRow"
                }).append(docNameInput);

                // document description
                var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
                var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
                var docDescInput = $("<input>", {
                    "type": "text",
                    "class": this.getSAHIHelperStr('docdescription') + " docname-input",
                    "placeholder": DocRefStrings.t('DocRowDocumentDescription'),
                    "value": docDesc,
                    "onchange": docDescOnChangeMethod
                });

                var responsiveDocDescWrapper = $("<div>", {
                    "style": "float: left; width: 100%",
                    "class": "DocumentNameRow"
                }).append(docDescInput);

                if (!this.dm.isEditable) {
                    docNameInput.attr("disabled", "disabled");
                    docDescInput.attr("disabled", "disabled");
                }

                // template selector
                var templateSelector = this.renderDocTemplateSelector(config.hideDocTemplate != "true");

                // folder selector
                var folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");

                // Responsive template/ folder selectors
                var responsiveTemplateFolderWrapper = $("<div>", {
                    "style": "width: 100%",
                    "id": this.phElemID + "_docTempFolderSelPHID"
                });
                var responsiveTemplateWrapper = $("<div>", {
                    "style": "float: left; width: 100%;"
                });
                var responsiveFolderWrapper = $("<div>", {
                    "style": "float: left; width: 100%"
                });
                if (templateSelector) {
                    responsiveTemplateWrapper.append(templateSelector);
                } else {
                    responsiveTemplateWrapper.hide();
                }
                responsiveFolderWrapper.append(folderSelector);
                responsiveTemplateFolderWrapper.append(responsiveTemplateWrapper, responsiveFolderWrapper);

                // Apply visibility.
                if (config.hideDescription == "true") {
                    responsiveDocDescWrapper.hide();
                }
                if (config.hideDocTemplate == "true") {
                    responsiveTemplateWrapper.hide();
                }
                if (config.hideFolder == "true") {
                    responsiveFolderWrapper.hide();
                }

                // Add the cell to the row.
                phElem.append($("<td>").append(responsiveDocNameWrapper, responsiveDocDescWrapper, responsiveTemplateFolderWrapper));

            } catch (err) {
                DocTableXUtils.log('Problem in DocumentRowExternalFileView.renderDocumentSpec:\n' + err);
            }
        };

        return DocumentRowExternalFileView;
    });
