define(["doctablex/DocTableXUtils",
        "doctablex/StaticDocumentRow",
        "doctablex/StaticDocumentRowView",
        "doctablex/DocumentRow",
        "doctablex/DocumentRowBrowse",
        "doctablex/DocumentRowFile",
        "doctablex/DocumentRowExternalFile",
        "doctablex/DocumentRowSearch",
        "doctablex/DocumentRowSearchView",
        "doctablex/DocumentRowNew",
        "doctablex/DocumentRowNewView",
        "doctablex/DocumentRowLink",
        "doctablex/DocumentRowLinkView",
        "doctablex/DocumentRowCopy",
        "require",
        "lib/PromisePolyfill"],
function(DocTableXUtils,
		StaticDocumentRow,
		StaticDocumentRowView,
		DocumentRow,
        DocumentRowBrowse,
        DocumentRowFile,
        DocumentRowExternalFile,
        DocumentRowSearch,
        DocumentRowSearchView,
        DocumentRowNew,
        DocumentRowNewView,
        DocumentRowLink,
        DocumentRowLinkView,
        DocumentRowCopy,
        require) {
	
    /**
     * DocumentRowFactory.js
     * 
     * This class is responsible for creating new document row objects based on particular context/needs.
     * It supports creation of all core document row classes (StaticDocumentRow, DocumentRow, DocumentRowBrowse, etc.)
     * and their PCM (text management) counterparts.
     * The PCM counterparts are created using mixins - PCM functionality is mixed (injected) into base 
     * document row objects.
     * 
     * 
     * @author JANH - January 2016
     */

    "use strict";
    
    
    /**
     * Constructor of DocumentRowFactory class. Never use "new DocumentRowFactory()" - use the instance
     * created at the end of this class/file.
     * 
     * @param {Boolean} isPCMEnabled - True if the PCM license is active on this system, false otherwise.
     */
    var DocumentRowFactory = function(isPCMEnabled) {
    	this.isPCMEnabled = false;
    	if (isPCMEnabled === undefined) {
            if (window.webcenterConfig) {
            	this.isPCMEnabled = !!window.webcenterConfig.isPCMEnabled;
            }
    	} else {
    		this.isPCMEnabled = !!isPCMEnabled;
    	}

    	this.initialization = new Promise(function(resolve, reject) {
	        // this part must be executed only once!!! (to have "mixin".__overridden() correct!)
	    	if (this.isPCMEnabled) {
	    		require(["doctablex/TextContentRow",
	    	        "doctablex/TextContentRowView",
	    	        "doctablex/AddedTextContentRow",
	    	        "doctablex/TextContentRowSearch",
	    	        "doctablex/TextContentRowSearchView",
	    	        "doctablex/TextContentRowNew",
	    	        "doctablex/TextContentRowNewView",
	    	        "doctablex/TextContentRowLink",
	    	        "doctablex/TextContentRowLinkView"], 
   				function (TextContentRow,
   						TextContentRowView,
   						AddedTextContentRow,
   						TextContentRowSearch,
   						TextContentRowSearchView,
   						TextContentRowNew,
   						TextContentRowNewView,
   						TextContentRowLink,
   						TextContentRowLinkView) {
	    			
		    		// mixin the PCM functionality to the base class
		    		// extend the model classes
		    		TextContentRow.call(StaticDocumentRow.prototype);
		    		AddedTextContentRow.call(DocumentRow.prototype);
// 			   		TextContentRowBrowse.call(DocumentRowBrowse.prototype);   // doesn't exist (yet)
		    		TextContentRowSearch.call(DocumentRowSearch.prototype);
		    		TextContentRowNew.call(DocumentRowNew.prototype);
		    		TextContentRowLink.call(DocumentRowLink.prototype);
// 			   		TextContentRowCopy.call(DocumentRowCopy.prototype);   // doesn't exist (yet)
		    		
		    		// extend the corresponding view classes
		    		TextContentRowView.call(StaticDocumentRowView.prototype);
//	    			AddedTextContentRowView.call(DocumentRowView.prototype);   // doesn't exist (yet)
// 			   		TextContentRowBrowseView.call(DocumentRowBrowseView.prototype);   // doesn't exist (yet)
		    		TextContentRowNewView.call(DocumentRowNewView.prototype);
		    		TextContentRowSearchView.call(DocumentRowSearchView.prototype);
		    		TextContentRowLinkView.call(DocumentRowLinkView.prototype);
//	    			TextContentRowCopyView.call(DocumentRowCopyView.prototype);   // doesn't exist (yet)
	
		    		//TODO add more mixins as they become available (implemented)
		    		
		    		// make a container object
		    		var pcmClasses = {
	    				TextContentRow : TextContentRow,
   						TextContentRowView : TextContentRowView,
   						AddedTextContentRow : AddedTextContentRow,
   						TextContentRowSearch : TextContentRowSearch,
   						TextContentRowSearchView : TextContentRowSearchView,
   						TextContentRowNew : TextContentRowNew,
   						TextContentRowNewView : TextContentRowNewView,
   						TextContentRowLink : TextContentRowLink,
   						TextContentRowLinkView : TextContentRowLinkView
		    		};
		    		// resolve this.initialization promise when "require" is done
		    		resolve(pcmClasses);
		    		
	    		}.bind(this));
	    	} else {
	    		// resolve this.initialization promise immediately
	    		resolve();
	    	}
    	}.bind(this));
    	return this;
    };
    
    /**
     * This method returns a Promise resolving into the new document row of specified type (docRowClass).
     * 
     * @param {class} docRowClass - Class (type) of the document row to be created.
     * @param {object} options - Collection of options to initialize the created document row with.
     * @returns {Promise} - Promise object resolving into the created document row instance.
     */
    DocumentRowFactory.prototype.createDocumentRow = function(docRowClass, options) {
    	return this.initialization.then(function (pcmClasses) {
	    	// create a new instance of the DocTableX class
	    	var obj = new docRowClass(options);
	
	    	if (this.isPCMEnabled) {
	    		// prepare the list of mixins to extend given base class. note the mixins have to be 
	    		// initialized in the order (sequence) of the inheritance hierarchy of the base classes.
	    		// 'null' in the sequence means there is no text content specific mixin at that level of the hierarchy.
	    		var PCMRowClassSequence = [];
	 		    if (docRowClass == StaticDocumentRow) {
					PCMRowClassSequence = [pcmClasses.TextContentRow];
				} else if (docRowClass == DocumentRow) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow];
				} else if (docRowClass == DocumentRowBrowse) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow];   // currently there is no specific text content functionality for browse/upload operation
				} else if (docRowClass == DocumentRowFile) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow];   // currently there is no specific text content functionality for browse/upload operation
				} else if (docRowClass == DocumentRowSearch) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow, pcmClasses.TextContentRowSearch];
				} else if (docRowClass == DocumentRowNew) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow, pcmClasses.TextContentRowNew];
				} else if (docRowClass == DocumentRowLink) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow, pcmClasses.TextContentRowLink];
				} else if (docRowClass == DocumentRowCopy) {
					PCMRowClassSequence = [pcmClasses.TextContentRow, pcmClasses.AddedTextContentRow];   // currently there is no specific text content functionality for Select from Project (Copy) operation
				} else if (docRowClass == DocumentRowExternalFile) {
					PCMRowClassSequence = [];
				} else {
					DocTableXUtils.error("Class '" + docRowClass + "' is not supported in PCM context!");
				}
		    	
				// call initializators of the mixin added functionality respecting the inheritance hierarchy of the base classes
				for (var i = 0; i < PCMRowClassSequence.length; i++) {
		    		var PCMRowClass = PCMRowClassSequence[i];
					if (PCMRowClass.__initMixin) {
				    	// initialization of the mixin (added functionality) within the 'obj' object
						PCMRowClass.__initMixin.call(obj, options);
						// (note the PCMRowClassView.__initMixin() method has to be called from within the 
						// PCMRowClass.__initMixin() method at the right time, the same way as in case of base classes!)
					}
		    	}
	    	}
	    	return obj;
    	}.bind(this));
    };

    
    /**
     * Helper method to simplify creation of StaticDocumentRow instances. Internally it calls the 
     * createDocumentRow(StaticDocumentRow, options) method.
     * 
     * @param {object} options - Collection of options to initialize the created document row with.
     * @returns {Promise} - Promise object resolving into the created document row instance.
     */
    DocumentRowFactory.prototype.createStaticDocumentRow = function(options) {
    	return this.createDocumentRow(StaticDocumentRow, options);
    };
    
    return new DocumentRowFactory();
});