define(["doctablex/StaticDocumentRow", 
        "doctablex/DocumentRow", 
        "doctablex/DocumentRowLinkView",
        "doctablex/AJAXDataCache",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
        "underscore",
        "jquery"], 
function(StaticDocumentRow,
		 DocumentRow,
         DocumentRowLinkView,
         AJAXDataCache,
         DocTableXUtils,
         DocRefStrings,
         WCXML,
         _,
         $){
    /**
     * DocumentRowLink.js
     * 
     * This is a subclass of DocumentRow designed to handle adding of documents via "Current Project" document 
     * source (aka link in existing document from current project to attribute value).
     * 
     * @author GRBO - March 2015
     */

    "use strict";

    var DocumentRowLink = function(options) {
        if (!options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        // set the view class of this data model class before calling superclass constructor to be created in the base class
        if (!options.viewClass) options.viewClass = DocumentRowLinkView;
        DocumentRow.call(this, options);   // calling superclass constructor!     

        /** @member {String} dlsName - Document Source Link (green link) name/label that creates this row. */
        this.dlsName = options.dlsName || 'Link';

        /**
         * @member {String} docOperation - operation performed on the document for "Link" document source
         * 		is by default "(by) reference".
         */
        this.docOperation = options.docOperation;
        if (!this.docOperation) this.docOperation = 'Reference';
        
        // the linked document is always from the project of the parent (actual) documents table 
        // (description in DocumentRow constructor)
        this.project = {
            	id : this.getParentDocTable().projectID
        };
        
        /** @member {Array<Object>} availableDocuments - Array of objects holding needed info about
         * 		documents from current project available to be referenced by this document reference.
         * 		Value 'null' means uninitialized;  */
        this.availableDocuments = options.availableDocuments || null;


        // private initialization ========================================

        if (this.constructor == DocumentRowLink.prototype.constructor) {
            // this part of initialization is private for the instances of this class only!
            // it won't be executed for any subclass

            // load missing document information (like available document templates) via AJAX (if not known)
            // note: this should be called only after the 'this.view' is properly initialized!
            this._initializeFromServer();

        }   // end of private initialization

        return this;
    };

    // define DocumentRowLink as a subclass of DocumentRow
    DocumentRowLink.prototype = new DocumentRow();
    DocumentRowLink.prototype.constructor = DocumentRowLink;
    DocumentRowLink.prototype.parent = DocumentRow.prototype;


    /**
     * Private initialization method using AJAX to load asynchronously required info that is not passed by 
     * 'options' (constructor parameter) from the server.
     *
     * @override
     */
    DocumentRowLink.prototype._initializeFromServer = function() {
    	if (this.copyOfExistingRowID) {
            this.setStatus('ready');
    		return;
    	}

    	// initialize documents selector if project ID is known
        if (this.project.id) {
            this.setStatus('loading');

            // store the ID of the project as "source project ID" for a document that will be selected
            // (this document source allows just documents from this.project to be selected)
            this.sourceProjID = this.project.id;
            
            // asynchronous load of documents in this project (using the AJAX cache object)
            var docListReq = AJAXDataCache.getInstance().getDocVersionsByProjID(this.project.id);
            
    		docListReq.done(function(docVersionList) {
    			
    			this.initDocsInCurrentProject(docVersionList);
    			this.setStatus('ready');

                // repainting the document row with the loaded values/info
                this.view.repaintRow();
                this.showDocAttributeCategory(true, true);
            }.bind(this));
    		docListReq.fail(function(errorMsg) {
                this.setStatus('error');
                alert(DocRefStrings.t('DocRowLoadingDocsError') + ":\n" + errorMsg);
            }.bind(this));
        } else {
            this.setStatus('error');
        }
    };
    
    
	/**
	 * This method derives all necessary information/data needed to create a new copy of the original 
	 * (== this) document row.
	 * It extends the basic implementation in superclasses.
	 * 
	 * @override
	 * @returns {Object} - Document info object == options for the constructor of any document row object.
	 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
	 * 		any "chattering" between the original and new document row.
	 */
    DocumentRowLink.prototype.getDocRowInfo = function () {
		var docInfo = DocumentRowLink.prototype.parent.getDocRowInfo.call(this);

		_.extend(docInfo, {
			dlsName : this.dlsName,
			availableDocuments : []
		});
		// copy of 'this.availableDocuments' array
		_.each(this.availableDocuments, function (doc) {
			docInfo.availableDocuments.push(_.extend({}, doc));
		}, this);
			
		return docInfo;
	};

	
    //JANH TODO is this really needed. Can't we just rely on the super implementation?!?!?
    /**
     * Method to reset document related data. Supposed to be called whenever a document selection is changed!
     */
    DocumentRowLink.prototype.resetDocument = function(fileInput) {
    	// preserve the project object as it cannot change in case of this document source!
    	var origProject = this.project;
    	// call the super class resetDocument() with enabled propagation!
        DocumentRowLink.prototype.parent.resetDocument.call(this, true);
        this.project = origProject;
    };

    
    /**
     * Initialize variables to support the document list (Document source is 'Current Project').
     * Supports two distinct document operations: "Link a document to a task" and "Reference a document".
     * 
     * @param {Array} docList - List of current documents in the project which are viewable to this user.
     */
    DocumentRowLink.prototype.initDocsInCurrentProject = function(docList) {
		this.availableDocuments = [];
    	if (docList && (docList.length > 0)) {
    		if (this.docOperation == 'LinkToTask') {
    			// get IDs of all documents currently in the documents table -> in this context they are
    			// the documents already linked to the task
    			var alreadyLinkedDocIDs = this.getParentDocTable().getAllDocumentsIDs();
    			// make sure the already linked documents are excluded from the list of available documents
    			for (var i = 0; i < docList.length; i++) {
    				var idx = alreadyLinkedDocIDs.indexOf(docList[i].docID);
    				if (idx == -1) {
    					// this document is not yet linked to the task - add it to the final list!
    					this.availableDocuments.push(docList[i]);
    				}
    			}
    		} else {
    			// in case of (docOperation == 'Reference') use all the available documents from the project 
    			// as each document can be referenced multiple times
    			this.availableDocuments = docList;
    		}
    	}
		// else documents in project loaded/initialized, but no documents available
    };
    

    /**
     * Method returning 'true', if attributes of this document can be edited in "general" - meaning
     * in the columns of documents table. Otherwise 'false' is returned.
     * 
     * This is the version for referencing already existing documents.
     * 
     * @override
     * @returns {Boolean}
     */
    DocumentRowLink.prototype.canUserEditAttributes = function() {
    	// the user permissions to change attributes of the original document (as no new document
    	// is created in this case!)
    	var referencedDocChangePropPerm = (this.userPerms) ? this.userPerms.change : false;
    	
    	return !this.isInactive()
    			// no view right and/or error loading doc. details
    			&& !this.isInErrorStatus()
    			&& this.isLatestDocVersion()
    			&& !this.dependsOnItself
    			&& referencedDocChangePropPerm;
    };
    
    
    /**
     * Method generating URL to retrieve attributes (and their values) from the server.
     * This is overridden version of the basic method defined in DocumentRow class.
     * @override
     */
    DocumentRowLink.prototype.generateDocAttributeValuesAJAXURL = function() {
    	return DocumentRowLink.prototype.parent.parent.generateDocAttributeValuesAJAXURL.call(this);
    };
    


    /**
     * Method generating XML to save info about newly uploaded document.
     * It calls the superclass convertToXML() method to add the common part of the XML.
     * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
     * 
     * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
     */
    DocumentRowLink.prototype.convertToXML = function() {
        var xmlValue = '';
        try {
            if (this.isInErrorStatus() || !this.docVerID) {
                // don't generate any XML if there is an error or no document version selected 
            	return xmlValue;
            }

            // generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
            var dsXML = '<ds>';
            dsXML += WCXML.ElementEncode('action', this.docOperation);
            
            if (this.docVerID) dsXML += WCXML.ElementEncode('source_dv_id', this.docVerID);
           
            dsXML += '</ds>';

            // call the super method to generate the common part of XML
            xmlValue = DocumentRowLink.prototype.parent.convertToXML.call(this, dsXML);

        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowLink.convertToXML:\n' + err);
            xmlValue = '';
        }
        return xmlValue;
    };

    
    /**
     * Validation of the document selected in the drop-down list of available documents.
     * 
     * @override
     * @param {Object} validObj - Validation object holding all found validation errors together with the related 
     * 		data needed to process them and notify user. 
     * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
     * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
     * 		starting from 1.
     * 
     * @returns {Boolean} - Returns true if no problem found in this particular document row and 
     * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
     */
    DocumentRowLink.prototype.validateDocRow = function(validObj, depth) {
    	var isOK = true;
    	
    	// check the document name field
   		var docNameSelect = $('#' + this.getRowPrefix() + ' div.DocumentNameRow > select:first');
   		
   		var errorObj = null;
   		if (docNameSelect.length == 0) {
   			// there is no "document name" input field (can happen due to configuration) - check the
   			// this.docName for a value - it should be prefilled in this case.
   			if (!this.docName) {
   				// compose the errorObj object referring to this situation
   				errorObj = {
   					isBlocking : true,
   	   				level : depth,
   					msg : DocRefStrings.t('DocRowDocNameNotSelectable'),
   					element : $('#' + this.getRowPrefix()),   // no doc. name field - select the whole row
   					docRowObj : this
   				};
   			}
   		} else if ((docNameSelect.length == 1) && !docNameSelect.val()) {
			// compose the errorObj object referring to this situation
			errorObj = {
				isBlocking : true,
   				level : depth,
				msg : DocRefStrings.t('DocRowDocNotSelected'),
				element : docNameSelect,
				docRowObj : this
			};
    	}
   		
   		if (errorObj != null) {
			validObj.errors.push(errorObj);
			isOK = false;
   		}
    	
    	// call immediately the StaticDocumentRow implementation of validateDocRow() method skipping
    	// the direct parent (DocumentRow class) validation implementation, which is not suitable for
    	// this case...
    	if (!DocumentRowLink.prototype.parent.parent.validateDocRow.call(this, validObj, depth)) {
    		isOK = false;
    	}
    	return isOK;
    };
    
    /**
     * Add document to validation object for checking unique document names
	 * @param validObj - object that gets passed to collect all document names
	 * @override
     */
    DocumentRowLink.prototype.validateDocRowUniqueName = function(validObj) {
    	// this document operation doesn't create new documents - do nothing
    };


    
    //Action handlers =====================================

    
    
    /**
     * Action handler when the selected document changes.
     * We need to ask the server information about the selected document
     * 
     * @param {String} docVersionID - Version ID of the newly selected document.
     * @returns 
     * 		- null if nothing has to be done, 
     * 		- a Promise object telling when the update operation finishes, when the update has to be done.
     */
    DocumentRowLink.prototype.docVersionSelected = function(docVersionID) {
    	
    	if (this.docVerID == docVersionID) {
    		return null;   // nothing to do
    	}
    	// put the selected doc version into the data model
		// the user changed the document selection - reset the info related to the previous doc.
		this.resetDocument();

		this.setStatus('loading');    		
		this.docVerID = docVersionID;

		// check if the added document doesn't create a cyclic dependency in the doc. ref. structure
		if (this.getParentDocTable().isDocDependentOnItself(this)) {
			// as this is an action done by user, confirms this is what the user really wants
			if (confirm(DocRefStrings.t('DocRowDocDependentOnItself'))) {
				// if yes, mark the added document row as being dependent on itself (in order to render 
				// it read-only with just a basic info to prevent confusion and problems with the cycle itself)
				this.setDependsOnItself(true);
			} else {
				// if not, reset the document selection
				this.docVerID = '';
			}
		}

		// document got deselected
		if (this.docVerID == '') {
			this.setStatus('ready');
			// repainting the document row with the updated values/info and hide attribute category 
			// as no document is selected
			this.view.repaintRow();
			this.hideDocAttributeCategory();
			return null;
		} 

		// get info about the newly selected document (repaint is called explicitly later)
		var docInfoReq = this.updateDocInfoFromServer(false);

		docInfoReq.done(function() {
			this.setStatus('ready');

			// repainting the document row with the updated values/info
			this.view.repaintRow();
			// the attribute category might change too, but that's recognized by 
			// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
			// the decision on the logic within "show attribute category" logic; 
			// force expand the attribute category
			this.showDocAttributeCategory(false, true);
		}.bind(this));
		docInfoReq.fail(function(errorMsg) {   // on error
			this.setStatus('error');
			DocTableXUtils.error('Failed to load document information from the server:\n' + errorMsg);
		}.bind(this));
		docInfoReq.always(function() {
			this.setIsChanged();
            this.setIsChangedByUser();
            this.setFieldModByUser('docName', !this.docName);
		}.bind(this));

		return docInfoReq.promise();
    };
    
    
    return DocumentRowLink;
});