define(["doctablex/DocumentRow",
        "doctablex/DocumentRowNewView",
        "doctablex/AJAXDataCache",
        "doctablex/DocTableXUtils",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
        "underscore",
        "jquery"],
	function (DocumentRow,
		DocumentRowNewView,
		AJAXDataCache,
		DocTableXUtils,
		DocRefStrings,
		WCXML,
		_,
		$) {
		/**
		 * DocumentRowNew.js
		 * 
		 * This is a subclass of DocumentRow designed to handle adding of documents via "New" document 
		 * source (adding documents from a library / doc.template).
		 * 
		 * @author JANH - February 2015
		 */

		"use strict";


		var DocumentRowNew = function (options) {
			if (!options) {
				// when there is nothing to initialize from, return just itself (not initialized)
				return this;
			}
			// set the view class of this data model class before calling superclass constructor to be created in the base class
			if (!options.viewClass) options.viewClass = DocumentRowNewView;
			DocumentRow.call(this, options); // calling superclass constructor!

			/** @member {String} dlsName - Document Source Link (green link) name/label that creates this row. */
			this.dlsName = options.dlsName || 'New';

			/** @member {String} docOperation - Operation performed on the document from a library is always "New". */
			this.docOperation = 'New';

	    	/** @member {String} newDocTmpID - Temporary document identifier used for new documents that doesn't
	    	 * 		have WebCenter document version ID assigned yet. It's guaranteed to be unique for each new
	    	 * 		document added within documents table. */
	    	this.newDocTmpID = options.newDocTmpID ? options.newDocTmpID : 'newID_' + this.rowID;

			if (!this.project || !this.project.id) {
				// base class initializes this.project.id like this: "projectID = options.projectID || null;"
				// as DocumentSourceHandler returns valid project ID only for doc. operation 'Reference',
				// the this.projectID has to be set for the remaining doc. operations manually here - it will
				// always end up in the actual project!
				this.project = {
					id: this.getParentDocTable().projectID
				};
			}

			/** @member {boolean} isDocTempDialogInitialized - A flag to remember whether the document template 
			 * 		selection dialog was already initialized (value 'true') or not (value 'false'). */
			this.isDocTempDialogInitialized = false;


			// private initialization ========================================
			// this part of initialization is private for the instances of this class only!
			// it won't be executed for any subclass
			if (this.constructor == DocumentRowNew.prototype.constructor) {
				// load missing document information like attribute category via AJAX (if not known).
				// note: this should be called only after the 'this.view' is properly initialized!
				this._initializeFromServer();

			} // end of private initialization

			return this;
		};

		//define DocumentRowNew as a subclass of DocumentRow
		DocumentRowNew.prototype = new DocumentRow();
		DocumentRowNew.prototype.constructor = DocumentRowNew;
		DocumentRowNew.prototype.parent = DocumentRow.prototype;



		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by 
		 * 'options' (constructor parameter) from the server.
		 * This variant automatically opens the document template selection dialog as soon as all required 
		 * data is loaded.
		 * 
		 * @override
		 */
		DocumentRowNew.prototype._initializeFromServer = function () {
	    	if (this.copyOfExistingRowID) {
	            this.setStatus('ready');
	    		return;
	    	}
			
			this.setStatus('loading');

			// load document templates (if any) including their thumbnails 
			var docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.project.id, true);

			// initialize project folders 
			var folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.project.id);

			$.when(docTemplatesReq, folderListReq).done(function (docTempList, folderList) {
				// store the ID of the document template project as "source project ID"
				this.sourceProjID = docTempList.tempProjID;
				
				// initialize document templates selection options
				this.updateDocTemplates(docTempList);

				// update the this.targetFolders and preselect a folder (if applicable)
				this.updateTargetFolders(folderList);

				// show the document template selection dialog. This method also initializes it if this is the
				// first time it is shown
				this.showDocTemplateDialog();

				this.updateOtherInfoFromServer(true, false);

				// no repaint of the document row itself here as the dialog is rendered first and the row only
				// after the user selects a doc. template within the dialog!

			}.bind(this))
			.fail(function (errorMsg) { // on error
				if (this.isInConfigPreview()) {
					alert(DocRefStrings.t('DocSrcHdlNoDocTempInConfig'));
				} else {
					this.setStatus('error');
					alert("Failed to load document templates from the server.\n" + errorMsg);
				}
			}.bind(this));
		};

		
		/**
		 * This method derives all necessary information/data needed to create a new copy of the original 
		 * (== this) document row.
		 * It extends the basic implementation in superclasses.
		 * 
		 * @override
		 * @returns {Object} - Document info object == options for the constructor of any document row object.
		 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
		 * 		any "chattering" between the original and new document row.
		 */
		DocumentRowNew.prototype.getDocRowInfo = function () {
			var docInfo = DocumentRowNew.prototype.parent.getDocRowInfo.call(this);

			_.extend(docInfo, {
				dlsName : this.dlsName,
				newDocTmpID : this.newDocTmpID   // for documents not existing in the WebCenter yet
			});

			return docInfo;
		};

		
		//Document template selection dialog related methods =======================================

		/**
		 * This method is called to select a document template for this row.
		 * 
		 * @param {int} {int} docTemplateIdx - Index of the selected document template. It corresponds to the index of the doc.
		 * 		template in this.docTemplates array.
		 */
		DocumentRowNew.prototype.addDocTemplate = function (docTemplateIdx) {
			// back to "loading" status as the user selection was made.
			this.setStatus('loading');

			// perform all operations needed when a doc. template is set. the index of the selected doc. template
			// is the value of selected HTML element (option).
			this.updateDocTemplateSelection(docTemplateIdx);

			// based on the selected template adapt target folder selection (if needed)
			this.preselectTargetFolder();

			// as the last step add itself (this document row) to the parent documents table
			// (this also triggers full render of this row)
			this.parentDocTable.addDocTemplateRow(this);

			this.setStatus('ready');

			// make sure the attribute category is expanded as soon as the doc. row is added to the documents table
			// the attribute category might change too, but that's recognized by 
			// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
			// the decision on the logic within "show attribute category" logic; 
			this.showDocAttributeCategory(false, true);
		};

		/**
		 * Show the document template dialog for this row.
		 */
		DocumentRowNew.prototype.showDocTemplateDialog = function () {
			if (this.isModifiedElsewhere()) {
				return; // no dialog selection if this is driven from elsewhere
			} else if (!this.docTemplates) {
				return; // no document row is added to documents table - it's forgotten 
			} else if (this.docTemplates.length == 0) {
				// covering a possible case (not just for debugging!)
				alert(DocRefStrings.t('DocSrcHdlNoDocTempComplyFilter'));
				return; // no document row is added to documents table - it's forgotten 
			} else if (this.docTemplates.length == 1) {
				// exactly one template available - skip the selection dialog as there is only one option anyway.
				// preselect the only document template available and quit
				this.updateDocTemplateSelection(0);

				// based on the selected template adapt target folder selection (if needed)
				this.preselectTargetFolder();

				// as the last step add itself (this document row) to the parent documents table
				// (this also triggers full render of this row)
				this.parentDocTable.addDocTemplateRow(this);

				this.setStatus('ready');

				// make sure the attribute category is expanded as soon as the doc. row is added to the documents table
				// the attribute category might change too, but that's recognized by 
				// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
				// the decision on the logic within "show attribute category" logic; 
				this.showDocAttributeCategory(false, true);
				return;
			}

			// get the dialog place-holder element (it creates one if it doesn't exist yet)
			var dialogElem = this.view.getDocTemplateDialogPlaceholder();

			// render the internals of the dialog just before it's opened
			this.view.renderDocTemplateDialog();

			// finally open the prepared dialog
			dialogElem.open();

			this.setStatus('wait4user');
		};



		/**
		 * Method overriding the DocumentRowNew.updateDocTemplateSelection() method. 
		 * This version does in addition copy document name, thumbnail and document type ID from the selected 
		 * doc. template to this doc. row as default values for the newly created document.
		 * 
		 * @override
		 * @param {int} selIdx - Index of the selected document template. It corresponds to the index of the doc.
		 * 		template in this.docTemplates array.
		 * @returns Promise object telling when the update operation is done.
		 */
		DocumentRowNew.prototype.updateDocTemplateSelection = function (selIdx) {
			// call do super class implementation first
			DocumentRowNew.prototype.parent.updateDocTemplateSelection.call(this, selIdx);

			// copy the interesting properties as defaults from the selected document template to this document row
			if (this.selDocTemplate) {
				this.docName = this.selDocTemplate.name;
				this.thumbnailURL = this.selDocTemplate.thumbnailURL;
				this.docTypeID = this.selDocTemplate.docTypeID;
			}

			// load effective document types configuration (for autonaming) and
			var effDocTypeSetupReq = AJAXDataCache.getInstance().getEffectiveDocTypesSettingsByProjID(this.project.id);
			var isDataReady = (effDocTypeSetupReq.state() == 'resolved');

			$.when(effDocTypeSetupReq).done(function (docTypesConfig) {
				// set document type and related autonaming settings of the selected template
				// force repaint only if the data are not ready yet (must be loaded asynchronously from the server)
				this.updateDocumentType(docTypesConfig, !isDataReady);
			}.bind(this))
			.fail(function (errorMsg) { // on error
				DocTableXUtils.log("Failed to load document templates from the server.\n" + errorMsg);
			}.bind(this))
			.always(function() {
				this.setIsChanged();
			}.bind(this));

			return effDocTypeSetupReq.promise();
		};



		//Save (XML generation) and validation methods =====================================


		/**
		 * Method generating XML to save info about newly uploaded document.
		 * It calls the superclass convertToXML() method to add the common part of the XML.
		 * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
		 * 
		 * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
		 */
		DocumentRowNew.prototype.convertToXML = function () {
			var xmlValue = '';
			try {
				if (this.isInErrorStatus()) return xmlValue;

				// generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
				var dsXML = '<ds>';
				dsXML += WCXML.ElementEncode('action', this.docOperation);
            	if (this.newDocTmpID) dsXML += WCXML.ElementEncode('new_doc_temp_id', this.newDocTmpID);
				dsXML += WCXML.ElementEncode('template_dv_id', this.selDocTemplate.docVerID);
				if (this.project.id) dsXML += WCXML.ElementEncode('project_id', this.project.id);
				if (this.selTargetFolder) {
					// get the selected target folder name in XML (if any)
					dsXML += WCXML.ElementEncode('target_folder_name', this.selTargetFolder.name);
				}
				if (this.isAutonamingEnabled()) {
					var suffixStr = (this.docNameSuffix ? this.docNameSuffix : '');
					dsXML += WCXML.ElementEncode('doc_name_suffix', suffixStr);
				} else {
					if (this.docName) dsXML += WCXML.ElementEncode('doc_name', this.docName);
				}
				if (this.docDesc) dsXML += WCXML.ElementEncode('doc_description', this.docDesc);

				if (this.hasAttrCat()) dsXML += WCXML.ElementEncode('doc_attr_cat_id', this.aCatID);

				dsXML += WCXML.ElementEncode('doc_temp_id', this.selDocTemplate.id);
				dsXML += WCXML.ElementEncode('doc_temp_folder_name', this.selDocTemplate.folderName);
				dsXML += '</ds>';

				// call the super method to generate the common part of XML
				xmlValue = DocumentRowNew.prototype.parent.convertToXML.call(this, dsXML);

			} catch (err) {
				DocTableXUtils.log('Problem in DocumentRowNew.convertToXML:\n' + err);
				xmlValue = '';
			}
			return xmlValue;
		};

		/**
		 * Get the view of the AttributeCategory that should be shown
		 * @returns {String}
		 * @override
		 */
		DocumentRowNew.prototype.getAttCatView = function () {
			var expandAttrCatConfig = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout');
			if (expandAttrCatConfig && expandAttrCatConfig.useCreationView === false) {
				return this.aCatView || 'Default';
			}

			return 'Creation';
		};

		return DocumentRowNew;
	});