define(["doctablex/StaticDocumentRow",
        "doctablex/DocumentRow",
        "doctablex/DocTableXUtils",
        "doctablex/DocumentRowSearchView",
        "doctablex/AJAXDataCache",
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml",
        "underscore",
        "jquery"], 
function(StaticDocumentRow,
		 DocumentRow,
         DocTableXUtils,
         DocumentRowSearchView,
         AJAXDataCache,
         DocRefStrings,
         WCXML,
         _,
         $){
    /**
     * DocumentRowSearch.js
     * 
     * This is a subclass of DocumentRow designed to handle adding of documents via "Search" document 
     * source (upload operation).
     * 
     * @author JANH - January 2015
     */

    "use strict";


    var DocumentRowSearch = function(options) {
        if (!options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        // set the view class of this data model class before calling superclass constructor to be created in the base class
        if (!options.viewClass) options.viewClass = DocumentRowSearchView;
        DocumentRow.call(this, options);   // calling superclass constructor!

        /** @member {String} dlsName - Document Source Link (green link) name/label that creates this row. */
        this.dlsName = options.dlsName || 'Search';

        /** @member {String} docOperation - operation performed on the document based on the user choice. 
         * 		Supported values are: Copy, Move, Link, Reference. */
        this.docOperation = options.docOperation ? options.docOperation : 'Copy';
        
        /** @member {StaticDocumentRow} clonedFrom - Instance of the original document row cloned in this document row. */
        this.clonedFrom = options.clonedFrom;
        this.isCloneInitOngoing = this.isClone();
   
    	/** @member {String} newDocTmpID - Temporary document identifier used for new documents that doesn't
    	 * 		have WebCenter document version ID assigned yet. It's guaranteed to be unique for each new
    	 * 		document added within documents table. */
        this.newDocTmpID = options.newDocTmpID || '';
    	if ((this.docOperation == 'Copy') && !options.newDocTmpID) {
    		this.newDocTmpID = 'newID_' + this.rowID;
    	}
    	
        /** @member {String} selDocName - the name of the originally selected document in case of copy/duplicate.
         * 		It's however not used so far... */
        this.selDocName = '';
        
        /** @member {String} fileExt - Extension of the document selected in the saved search results. */
        // get the file extension and remove the leading dot ('.') from it.
        this.fileExt = DocTableXUtils.getFileExtension(this.docName, true);

        // private initialization ========================================
        // this part of initialization is private for the instances of this class only!
        // it won't be executed for any subclass
        if (this.constructor == DocumentRowSearch.prototype.constructor) {

            // load missing document information (like available document templates and document types 
        	// configuration) via AJAX (if not known)
            // note: this should be called only after the 'this.view' is properly initialized!
        	this._initFromServerState = this._initializeFromServer();

        }   // end of private initialization

        return this;
    };

    //define DocumentRowSearch as a subclass of DocumentRow
    DocumentRowSearch.prototype = new DocumentRow();
    DocumentRowSearch.prototype.constructor = DocumentRowSearch;
    DocumentRowSearch.prototype.parent = DocumentRow.prototype;


    /**
     * Private initialization method using AJAX to load asynchronously required info that is not passed by 
     * 'options' (constructor parameter) from the server.
     */
    DocumentRowSearch.prototype._initializeFromServer = function() {
        this.setStatus('loading');

        // deferred object tracking the state of loading data from the server - it's resolved when all is loaded
    	var initFromServerState = $.Deferred();
    	
    	if (this.copyOfExistingRowID) {
            this.setStatus('ready');
    		return initFromServerState.resolve().promise();
    	}
    	
        // load basic document info from the server (including this.docTypeID)
        var docInfoReq = this.updateDocInfoFromServer(false);
        docInfoReq.done(function() {

        	// (check DocumentRowSearch._updateDocInfo() for special handling of different document operations!)
            
            // after the basic document info is loaded perform dependent AJAX calls:
            // - load effective document type configuration to get autonaming setup out of that
            var effDocTypeSetupReq = null;
            if (this.project.id) {
            	effDocTypeSetupReq = AJAXDataCache.getInstance().getEffectiveDocTypesSettingsByProjID(this.project.id);
            } else {
            	// case of project created from blank
            	effDocTypeSetupReq = AJAXDataCache.getInstance().getProjCreationSysDefaults();
            }

            // special processing of 'copy' operation
            var docTemplatesReq = null;
            if (this.docOperation == 'Copy') {
                // in case of copy operation load also document templates (if any)
            	if (this.project.id) {
            		docTemplatesReq = AJAXDataCache.getInstance().getDocTemplatesByProjID(this.project.id);
            	}
            	// else: when creating project from blank, there are no document templates to be loaded...
            }

        	// initialize project folders 
    		var folderListReq = null;
    		if (this.docOperation != 'Reference'){
            	if (this.project.id) {
            		folderListReq = AJAXDataCache.getInstance().getFoldersByProjID(this.project.id);
            	}
            	// else: when creating project from blank, there are no folders to be loaded...
    		}

    		$.when(effDocTypeSetupReq, docTemplatesReq, folderListReq)
    				.done(function(docTypesConfig, docTempList, folderList) {
    			// member of this class (see definition in DocumentRow class) - flag controlling whether 
                // to use attribute category configured as default for this document type or not. 
                // Do it (value 'true') if there is no attribute category attached to the original
                // (copied/moved/linked/referenced) document, otherwise don't (value 'false')
                this.useDocTypeDefaultACat = (this.hasNoAttrCat() && (this.docOperation == 'Copy'));

                // set the autonaming setup based on effective document type settings of the document's project
                // (note default attr. category is not used here)
                this.updateDocumentType(docTypesConfig, false);
                
                // initialize document templates selection (if any and it is a copy operation)
                if (docTempList) {
                    this.updateDocTemplates(docTempList);
                    this.preselectDocTemplate();
                }
                
            	// update the this.targetFolders and preselect a folder (if applicable)
    			this.updateTargetFolders(folderList);
    			if (this.isClone()) {
    				// when duplicating a document, we want the copy to end up in the same folder as original
    				if (this.clonedFrom && this.clonedFrom.selTargetFolder) {
    					// when there is a selected target folder, embrace it! (duplication of a newly added row)
    					var selTargetFolderCopy = _.extend({}, this.clonedFrom.selTargetFolder);
    					this.setSelectedTargetFolder(selTargetFolderCopy, false);
    				} else if (this.folder && (this.folder.name != null)) {
    					// when there is no selected target folder, default to the folder of the duplicated
    					// document (case of a StaticDocumentRow duplication)
    					this.setTargetFolderByName(this.folder.name);
	    				// (note if the "setTargetFolderByName" fails because permissions or something
	    				// the duplicated document gets copied to the root folder == expected behaviour)
    				}
    			} else {
    				// if this is not a "duplicate" operation, the configured default folder should be (pre)selected
    				this.preselectTargetFolder();
    			}
    			
    			// loading from the server finished
    			initFromServerState.resolve();
    			
                this.setStatus('ready');

                // repainting the document row with the updated values/info
                this.view.repaintRow();
				// the attribute category might change too, but that's recognized by 
				// StaticDocumentRow.isAttrCatUpdateNeeded() method so we can pass false here and leave
				// the decision on the logic within "show attribute category" logic; 
				// force expand the attribute category
                this.showDocAttributeCategory(false, true);
            }.bind(this))
            .fail(function(errorMsg) {
    			// loading from the server finished (with an error)
            	initFromServerState.reject();
                this.setStatus('error');
                DocTableXUtils.log("Failed to load document configuration from the server.\n" + errorMsg);
            }.bind(this));
        }.bind(this));

        docInfoReq.fail(function(errorMsg) {
			// loading from the server finished (with an error)
        	initFromServerState.reject();
            this.setStatus('error');
    		// no need to alert the error message as the document row rendering is altered based on the error status!
    		DocTableXUtils.log("Failed to process basic info about document loaded from the server.\n" + errorMsg);
        }.bind(this));
        
        return initFromServerState.promise();
    };

    
    /**
     * Private method to update document related properties of this class instance based on info loaded 
     * from server. It's overriding the StaticDocumentRow._updateDocInfo() using the the stripped down 
     * DocumentRow version to handle (mainly) "Copy" operation in a different way...
     * 
     * @override
     * @returns {boolean} - 'true' if document was changed, 'false' otherwise.
     */
    DocumentRowSearch.prototype._updateDocInfo = function(docInfo) {
    	var modified = this._strippedUpdateDocInfo(docInfo);
    	
    	// handling of clonned rows is specific to DocumentRowSearch as the clone operation is simulated
    	// by the DocumentRowSearch -> Copy operation.
    	if (this.isClone()) {
        	// make sure the cloned document name is generated (and not overwritten by the original 
        	// document name coming from the server).
	  		this.docName = this.getParentDocTable().getClonedDocName(this);
	  		modified = true;
        }
        
    	return modified;
    };
    
    
	/**
	 * This method derives all necessary information/data needed to create a new copy of the original 
	 * (== this) document row.
	 * It extends the basic implementation in superclasses.
	 * 
	 * @override
	 * @returns {Object} - Document info object == options for the constructor of any document row object.
	 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
	 * 		any "chattering" between the original and new document row.
	 */
    DocumentRowSearch.prototype.getDocRowInfo = function () {
		var docInfo = DocumentRowSearch.prototype.parent.getDocRowInfo.call(this);

		_.extend(docInfo, {
			dlsName : this.dlsName,
			clonedFrom : this.clonedFrom,   // this is not deep copy - read-only value!
			newDocTmpID : this.newDocTmpID   // for documents not existing in the WebCenter yet
		});

		return docInfo;
	};

    
    /** 
     * This method returns 'true' if this document row is created as a clone of other existing document row
     * (duplicate action), 'false' if the 'this.clonedFrom' is not initialized or contains something else 
     * than a document row.
     */
    DocumentRowSearch.prototype.isClone = function() {
    	if (this.clonedFrom && (this.clonedFrom.rowID || (this.clonedFrom.rowID === 0))) {
    		return true;
    	}
    	return false;
    };
    
    /** 
     * Clone operation must be done only once while the cloned document row is created. Subsequent initialization
     * of parent attribute category should not trigger the clone row initialization again is it would overwrite
     * (potentially) already changed values...
     * This method have to be called after the first clone initialization is done to prevent any further attempts.
     */
    DocumentRowSearch.prototype.finishCloneInit = function() {
    	this.isCloneInitOngoing = false;
    };

    
    /**
     * Callback method to perform specialized actions after an attribute category is loaded and rendered on
     * the page (but not necessarily shown).
     * This method extends the basic version defined in StaticDocumentRow with the support for 
     * "duplicate/clone" operation.
     * 
     * @override
     * @param {String} categoryUse - Value defining type/use of just loaded attribute category.
     * 		It supports the following values: 'docCat', 'refPropsCat' and 'projInfoCat'.
     */
    DocumentRowSearch.prototype._onAttrCategoryReady = function(categoryUse) {
    	if ((categoryUse == 'docCat') && this.isAttrCatLoaded(this.aCatObj)) {
        	// special handling of document attribute category in case of cloning
        	if (this.isCloneInitOngoing) {
        		var sourceACatObj = this.clonedFrom.aCatObj;
        		if (this.isAttrCatLoaded(sourceACatObj)) {
        			this.aCatObj.copyAttrValuesFromCategory(sourceACatObj);
        		} else if (this.clonedFrom.aCatNotExpandedYet) {
        			// if here, the source document row attribute category was not expanded yet and 
        			// there are no inline document attributes configured in the document row (rendering config.)
        			// => what we get from DB is what is in the source document attribute category
        		} else {
        			// if the attribute category was already expanded, this should not happen!
        			window.alert("Cloning document row with attribute category not loaded even if the cloned doc row already loaded its own category!?!?");   //JANH debug - then DEL!!!
        		}
        		this.finishCloneInit();
        	}
        }
    	// call the parent implementation to handle the common behaviour...
    	DocumentRowSearch.prototype.parent._onAttrCategoryReady.call(this, categoryUse);
    };
    
    
    /**
     * Validation of the search document row.
     * 
     * @override
     * @param {Object} validObj - Validation object holding all found validation errors together with the related 
     * 		data needed to process them and notify user. 
     * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
     * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
     * 		starting from 1.
     * 
     * @returns {Boolean} - Returns true if no problem found in this particular document row and 
     * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
     */
    DocumentRowSearch.prototype.validateDocRow = function(validObj, depth) {
    	var isOK = true;

    	if (this.docOperation == 'Reference' || this.docOperation == 'Link') {
    		// By Reference and Link operations doesn't let user to change the document name - still
    		// check there is a document name coming from the user's selection!
   			if (!this.docName) {
   				// compose the errorObj object referring to this situation
   				var errorObj = {
   					isBlocking : true,
   	   				level : depth,
   					msg : DocRefStrings.t('DocRowDocNameNotSelectable'),
   					element : $('#' + this.getRowPrefix()),   // no doc. name field - select the whole row
   					docRowObj : this
   				};
   				validObj.errors.push(errorObj);
   				isOK = false;
   			}
    		
    		// Call the StaticDocumentRow validation implementation that takes care of required properties and documents (but not naming).
        	// (both in documents table column and associated attr. category).
        	if (!StaticDocumentRow.prototype.validateDocRow.call(this, validObj, depth)) {
        		isOK = false;
        	}
    	} else {
    		// Call the StaticDocumentRow validation implementation that takes care of required properties and documents and naming.
        	// (both in documents table column and associated attr. category).
        	if (!DocumentRow.prototype.validateDocRow.call(this, validObj, depth)) {
        		isOK = false;
        	}
    	}
    	
    	return isOK;
    };

    /**
     * Add document to validation object for checking unique document names
	 * @param validObj - object that gets passed to collect all document names
	 * @override
     */
    DocumentRowSearch.prototype.validateDocRowUniqueName = function(validObj) {
    	if ((this.docOperation == 'Link') || (this.docOperation == 'Reference')) {
    		// do nothing as no new document is created by these operations...
    	} else {
    		// copy and move both add "new" document to particular project
    		DocumentRowSearch.prototype.parent.validateDocRowUniqueName.call(this, validObj);
    	}
    };


    /**
     * Overrides the base class isAutonamingEnabled() method. 
     * 
     * @override
     * @returns {boolean} - True if autonaming is preconfigured for the document type of the currently selected file and
     * 		the document operation is 'Copy'. All other doc. operations doesn't support autonaming!
     */
    DocumentRowSearch.prototype.isAutonamingEnabled = function() {
        return (this.anPrefix != null) && (this.anPrefix.length > 0) && (this.docOperation == 'Copy');
    };
    
    /**
     * Method returning 'true', if attributes of this document can be edited in "general" - meaning
     * in the columns of documents table. Otherwise 'false' is returned.
     * 
     * This version cover both cases:
     * - Newly added reference to already existing document(s).
     * - Newly created (copied, moved or uploaded) document in this project.
     * 
     * Special case is the Link operation that combines permissions from both above cases.
     * 
     * @override
     * @returns {Boolean}
     */
    DocumentRowSearch.prototype.canUserEditAttributes = function() {
    	// the calculated "change property" permission
    	var docChangePropPerm = false;
    	
    	// applicable when an existing document is referenced or moved
    	var referencedDocChangePropPerm = (this.userPerms) ? this.userPerms.change : false;
    	// applicable when a new document is created (copied, moved or uploaded)
    	var newDocChangePropPerm = false;
    	if (this.selTargetFolder) {
    		newDocChangePropPerm = this.selTargetFolder.changePropPerm;
    	} else if (this.getParentDocTable().isBlankProjectCreationContext()) {
    		// user creating a new blank project has rights to change attributes of newly uploaded/copied
    		// documents to the new project!
    		newDocChangePropPerm = true;
    	}
    	
    	if (this.docOperation == 'Reference') {
    		docChangePropPerm = referencedDocChangePropPerm;
    	} else if (this.docOperation == 'Link') {
    		docChangePropPerm = referencedDocChangePropPerm || newDocChangePropPerm;
    	} else {
    		docChangePropPerm = newDocChangePropPerm;
    	}
    	
    	return !this.isInactive()
    			// no view right and/or error loading doc. details
    			&& !this.isInErrorStatus()
    			&& this.isLatestDocVersion()
    			&& !this.dependsOnItself
    			&& docChangePropPerm;
    };


    /**
     * Method generating URL to retrieve attributes (and their values) from the server.
     * This is overridden version of the basic method defined in DocumentRow class.
     * @override
     */
    DocumentRowSearch.prototype.generateDocAttributeValuesAJAXURL = function() {
        var url = window.webcenterConfig.jspBaseUrl;
        if (this.selDocTemplate && this.selDocTemplate.docVerID 
        		&& DocTableXUtils.isValidID(this.selDocTemplate.categoryID)) {
            url += 'GetAttributes.jsp?doctemplate=1';   // a flag that the document template is specified
            url += '&projectid=' + this.project.id;   // ID of project where the new document is about to be uploaded
            url += '&documentid=' + this.selDocTemplate.id;   // document ID of selected document template
            url += '&docverid=' + this.selDocTemplate.docVerID;   // doc. version ID of selected document template
            url += '&categoryid=' + this.selDocTemplate.categoryID;   // ID of selected document template's attr. category
        } else if (this.hasOrigDocACat && this.docVerID) {
            // no document template selected - the original document's attribute category should be loaded (if any)
            url += 'GetAttributes.jsp?docverid=' + this.docVerID;   // a flag that the document template is specified
        } else if (DocTableXUtils.isValidID(this.aCatID)) {
            // just load attribute types, restricted sets etc. to correctly render them (along with 
            // empty attribute values)
            url += 'GetAttributes.jsp?categoryid=' + this.aCatID;
        } else {
            DocTableXUtils.log("Cannot generate URL in DocumentRowSearch.generateDocAttributeValuesAJAXURL().");
        }
        return url;
    };

    
    /**
	 * Get the view of the AttributeCategory that should be shown
	 * @returns {String}
	 * @override
     */
    DocumentRowSearch.prototype.getAttCatView = function(){
    	var override = false;
    	
    	var expandAttrCatConfig = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout');
    	if (expandAttrCatConfig && expandAttrCatConfig.useCreationView === false) {
    		// set flag that we want to override the creation view
        	override = true;
        }
    	
    	if(!override){
    		// only for new documents return creation view
    		if (this.docOperation == 'Copy') {
    			return 'Creation';
    		}
    	}
    	
    	return this.aCatView || 'Default';
    };
    

    //Save (XML generation) and validation methods =====================================


    /**
     * Method generating XML to save info about newly uploaded document.
     * It calls the superclass convertToXML() method to add the common part of the XML.
     * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
     * 
     * @returns {String} XML complying to JavaScript -> JSP interface to save changes done in document table
     */
    DocumentRowSearch.prototype.convertToXML = function() {
        var xmlValue = '';
        try {
            if (this.isInErrorStatus()) return xmlValue;
        	
            // generate <ds> (Document Source) part of XML here. the common part is left for the superclass method...
            var dsXML = '<ds>';
            dsXML += WCXML.ElementEncode('action', this.docOperation);
        	if (this.newDocTmpID) dsXML += WCXML.ElementEncode('new_doc_temp_id', this.newDocTmpID);
            if (this.docVerID) dsXML += WCXML.ElementEncode('source_dv_id', this.docVerID);
            if (this.project.id) dsXML += WCXML.ElementEncode('project_id', this.project.id);
            if (this.selTargetFolder) {
                // get the selected target folder name in XML (if any)
            	dsXML += WCXML.ElementEncode('target_folder_name', this.selTargetFolder.name);
            }
        	if (this.isAutonamingEnabled()) {
                var suffixStr = (this.docNameSuffix ? this.docNameSuffix : '');
                dsXML += WCXML.ElementEncode('doc_name_suffix', suffixStr);
            } else {
                if (this.docName) dsXML += WCXML.ElementEncode('doc_name', this.docName);
            }
            if (this.docDesc) dsXML += WCXML.ElementEncode('doc_description', this.docDesc);

            if (this.hasAttrCat()) dsXML += WCXML.ElementEncode('doc_attr_cat_id', this.aCatID);

            if (this.selDocTemplate) {
                dsXML += WCXML.ElementEncode('doc_temp_id', this.selDocTemplate.id);
                dsXML += WCXML.ElementEncode('doc_temp_folder_name', this.selDocTemplate.folderName);
            }
            dsXML += '</ds>';

            // call the super method to generate the common part of XML
            xmlValue = DocumentRowSearch.prototype.parent.convertToXML.call(this, dsXML);

        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowSearch.convertToXML:\n' + err);
            xmlValue = '';
        }
        return xmlValue;
    };

    
    return DocumentRowSearch;
});