define(["doctablex/DocumentRowView", 
        "doctablex/DocTableXUtils", 
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(DocumentRowView,
         DocTableXUtils,
         DocRefStrings,
         WCXML){
    /**
     * DocumentRowSearchView.js
     * 
     * This class implements rendering methods for an upload document row ("Search" document source).
     * 
     * @author JANH - January 2015
     */

    "use strict";

    var DocumentRowSearchView = function(dataModel, phElementID, options) {
        if (!dataModel && !phElementID && !options) {
            // when there is nothing to initialize from, return just itself (not initialized)
            return this;
        }
        DocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

        // note this.dataModel and phElemID are initialized in the super class...)

        return this;
    };

    // define DocumentRowSearchView as a subclass of DocumentRowView
    DocumentRowSearchView.prototype = new DocumentRowView();
    DocumentRowSearchView.prototype.constructor = DocumentRowSearchView;
    DocumentRowSearchView.prototype.parent = DocumentRowView.prototype;


    // the main render and repaint methods are inherited from the base class


    /**
     * The render method generating HTML for "Search" document source to be able to create a new document based
     * on existing document in WebCenter including name and description definition fields (if applicable).
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     * @returns {String} - The generated HTML.
     */
    DocumentRowSearchView.prototype.renderDocumentSpec = function(phElem, config) {
        try {
        	// Unregister the old resizer for this collumn.
			if (this._nameColResizer) {
				$(window).off("resize", this._nameColResizer);
			}
			
			// Create the collumn element.
        	var colElem = $('<' + this.itemRootElemStr +'>');
        	
        	// Set a custom width if this was specified.
        	if (config.width && config.width != "auto") {
        		colElem.width(config.width);
        	}
        	
        	// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
            var container = $(this.addStartOfCustomWidthDiv(config, '150px'));

            var isCopy = this.dm.docOperation == 'Copy';
            var isMove = this.dm.docOperation == 'Move';
            var isLink = this.dm.docOperation == 'Link';
                                    
            
            // document row "label" with selected doc. template name
            var sourceFile = this.renderDocDetails();
    		container.append(sourceFile);
    		
            // document name
        	var docNameInput = $("<input>").hide();
            if (isCopy || isMove) {
                // document name is editable only for copy and move doc. operations
                if (this.dm.isAutonamingEnabled()) {
                    // when autonaming is on the behavior of "document name" field differs
                    if (this.dm.hasAutonameSuffix()) {
						// replace the document name field by document suffix field
                        var docSuffixOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameSuffixChanged(this.value);';
                        var docSuffix = (this.dm.docNameSuffix ? this.dm.docNameSuffix : '');
                        docNameInput = $("<input>", {"type" : "text", 
    						"class" : this.getSAHIHelperStr('docnamesuffix')+" docname-input", 
    						"placeholder" : DocRefStrings.t('DocRowDocumentSuffix'),
    						"value" : docSuffix,
    						"onchange" : docSuffixOnChangeMethod});
					} else {
						docNameInput = $("<input>", {"type" : "text", 
							"class" : this.getSAHIHelperStr('docname')+" docname-input", 
							"placeholder" : this.getAutoNamingPreview(),
							"disabled": "disabled"});
                    }
                } else {
                	// autonaming is off user can change name of copied and/or moved documents only!
                	var docName = (this.dm.docName ? this.dm.docName : '');
                    var docNameOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docNameChanged(this.value);';
                    docNameInput = $("<input>", {"type" : "text",
    					"class" : this.getSAHIHelperStr('docname')+" docname-input", 
    					"placeholder" : DocRefStrings.t('DocRowDocumentName'),
    					"value" : docName,
    					"onchange" : docNameOnChangeMethod});
                }
            } else {
            	docNameInput = $("<span>", {"class" : "docname-text"}).text(this.dm.docName);
            }

	        // document description
            var docDescInput = $("<div>").hide();
	        var docDescOnChangeMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docDescChanged(this.value);';
			var docDesc = (this.dm.docDesc ? this.dm.docDesc : '');
	        if (isCopy) {
	        	docDescInput = $("<input>", {"type" : "text", 
					"class" : this.getSAHIHelperStr('docdescription')+" docname-input", 
					"placeholder" : DocRefStrings.t('DocRowDocumentDescription'),
					"value" : docDesc,
					"onchange" : docDescOnChangeMethod});
	        } else if (docDesc) {
	        	docDescInput.show();
	        	var label = $("<span>", {"class" : "docname-label"}).text(DocRefStrings.t('DocRowDocumentDescription') + ':');
	        	var description = $("<span>", {"class" : "docname-text"}).text(docDesc);
	        	docDescInput.append(label, description);
	        }

            // document template selector (if any)
			var templateSelector = $("<div>").hide();
            if (isCopy) {
            	// template selector
    			templateSelector = this.renderDocTemplateSelector(config.hideDocTemplate != "true");
            }
            // folder selector
            var folderSelector = $("<div>").hide();
			if (isCopy || isMove || isLink) { 
				folderSelector = this.renderDocFolderSelector(config.hideFolder != "true");
			}
            
			// Disable fields when not editable
            if (!this.dm.isEditable){
            	docNameInput.attr("disabled", "disabled");
            	docDescInput.attr("disabled", "disabled");
            }

			// Responsive doc description/name
            var responsiveDocNameDescWrapper = $("<div>", {"style" : "width: 100%"});
			var responsiveDocNameWrapper = $("<div>", {"style" : "float: left; width: 100%;", "class" : "DocumentNameRow"});
			var responsiveDocDescWrapper = $("<div>", {"style" : "float: left; width: 100%", "class" : "DocumentNameRow"});
			responsiveDocNameWrapper.append(docNameInput);
			responsiveDocDescWrapper.append(docDescInput);
			responsiveDocNameDescWrapper.append(responsiveDocNameWrapper, responsiveDocDescWrapper);
			
			// Responsive template/ folder selectors
            var responsiveTemplateFolderWrapper = $("<div>", {"style" : "width: 100%", "id" : this.phElemID + "_docTempFolderSelPHID"});
            var responsiveTemplateWrapper = $("<div>", {"style" : "float: left; width: 100%;"});
			var responsiveFolderWrapper = $("<div>", {"style" : "float: left; width: 100%"});
			if (templateSelector) {
			responsiveTemplateWrapper.append(templateSelector);
			} else {
				responsiveTemplateWrapper.hide();
			}
			responsiveFolderWrapper.append(folderSelector);
			responsiveTemplateFolderWrapper.append(responsiveTemplateWrapper, responsiveFolderWrapper);
            
			// Apply visibility.
			if (config.hideSource == "true") {
				sourceFile.hide();
			}
			if (config.hideDescription  == "true" 
				|| (docDescInput.prop("tagName").toUpperCase() == "DIV" && docDesc.length == 0)) {
				responsiveDocDescWrapper.hide();
			}
			if (config.hideDocTemplate == "true") {
				responsiveTemplateWrapper.hide();
			}
			if (config.hideFolder == "true") {
				responsiveFolderWrapper.hide();
			}
			
			// Create a resizer for this column.
			this._nameColResizer = function () {
				var breakPoint = 360;
				if (responsiveDocNameDescWrapper && responsiveDocNameWrapper.is(':visible') && responsiveDocDescWrapper.is(':visible')) {
					var width = responsiveDocNameDescWrapper.width();
					if (width < breakPoint) {
						responsiveDocNameWrapper.css("width","100%");
						responsiveDocNameWrapper.css("padding-right", "0px");
						responsiveDocDescWrapper.css("width", "100%");
					} else {
						responsiveDocNameWrapper.css("width","50%");
						responsiveDocNameWrapper.css("padding-right", "2px");
						responsiveDocDescWrapper.css("width", "50%");
					}
					if (responsiveDocNameWrapper.height() > responsiveDocDescWrapper.height()) {
						responsiveDocDescWrapper.css("height", responsiveDocNameWrapper.height() + "px");
					} else {
						responsiveDocNameWrapper.css("height", responsiveDocDescWrapper.height() + "px");
					}
				}
				if (responsiveTemplateFolderWrapper && responsiveTemplateWrapper.is(':visible') && responsiveFolderWrapper.is(':visible')) {
					var width = responsiveTemplateFolderWrapper.width();
					if (width < breakPoint) {
						responsiveTemplateWrapper.css("width","100%");
						responsiveTemplateWrapper.css("padding-right", "0px");
						responsiveFolderWrapper.css("width", "100%");
					} else {
						responsiveTemplateWrapper.css("width","50%");
						responsiveTemplateWrapper.css("padding-right", "2px");
						responsiveFolderWrapper.css("width", "50%");
					}
				}
			};
			$(window).resize(this._nameColResizer);
			// Also try to execute the resizer after the table has rendered, so defer the execution of the resizer.
			window.setTimeout(function () {
				this._nameColResizer();
			}.bind(this), 1);
			
			// Add all the elements to the container.
			container.append(sourceFile, responsiveDocNameDescWrapper, responsiveTemplateFolderWrapper);
            
            // Add the html to the collumn element.
            colElem.append(container);
            
            // Add the cell to the row.
            phElem.append(colElem);
            
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowSearchView.renderDocumentSpec:\n' + err);
        }
    };
    
    
    /**
     * Method to render column/item with a button to cancel the "make local" operation on the given document
     * (in this document row).
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     */
    DocumentRowSearchView.prototype.renderLocalRemoteButton = function(phElem, config) {
        try {
        	if (this.dm.origDocRow) {
            	// this is the cancel button for "make local" operation (local/remote button). It's shown 
        		// if and only if there is 'origDocRow' stored in this document row object. 
        		// That means this document row was created as a response to "make local" action on 
        		// a remote document, which can be put back by clicking on this "cancel" button.
	            var htmlcode = '';
	            htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
                var clickMethod = 'javascript: ' + this.dm.getParentInDOM() + '.localRemoteButtonClicked(' + this.rowIndex + ');';
                var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteResetButtonTooltip'));
                htmlcode += '<button type="button" class="btn btn-xs btn-default actionBtn" title="'+ translMsg +'" onclick="'+clickMethod+'">';
            	htmlcode += '<i class="icon icon-normal activIcon icon icon icon-WCR_Revert"/>';
                htmlcode += '</button>';
		        htmlcode += '</' + this.itemRootElemStr + '>';
		        // render the prepared HTML code
		        phElem.append(htmlcode);
        	} else {
        		DocumentRowSearchView.prototype.parent.renderLocalRemoteButton.call(this, phElem, config);
        	}
        } catch (err) {
            DocTableXUtils.log('Problem in StaticDocumentRowView.renderLocalRemoteButton:\n' + err);
        }
    };
    
    
    /**
     * Method to render Clone button in the row based on render configuration in Attribute Category 
     * 
     * Clone Button rendered only operation type is Copy or New
     * 
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     */
    DocumentRowSearchView.prototype.renderCloneButton = function(phElem) {
    	// call the StaticDocumentRow.renderCloneButton() directly skipping the default implementation
    	// for all new document rows from DocumentRowView class
    	DocumentRowSearchView.prototype.parent.parent.renderCloneButton.call(this, phElem);
    };
    
    
    /**
     * Render the element indicating which is the documents source. 
     * 
     * @returns {jQuery} - The element indicating the document source.
     */
    DocumentRowSearchView.prototype.renderDocDetails = function(isClone) {
    	
    	// document row "label" with selected doc. template name
    	// ... but first make sure if the config allows to render this:
        var sourceFile = $("<div>").hide();
		if ((this.dm.origDocRow) && this.dm.dlsName.indexOf('Replace') == -1) {
			var label = $("<span>", {"class" : "docname-label"}).text(DocRefStrings.t('DocRowLocalRemoteMakeLocal'));
			sourceFile.append(label);
		} else {
			if(isClone){
				var label = $("<span>", {"class" : "docname-label"}).text(this.dm.dlsName);
				sourceFile.append(label);
    		}else{
    			var label = $("<span>", {"class" : "docname-label"}).text(this.dm.dlsName+" ("+this.dm.docOperation+")");
    			sourceFile.append(label);
    		}
		}
		var text = $("<span>", {"class" : "docname-text"}).text(this.dm.selDocName);
		sourceFile.append(text);
		sourceFile.show();
    	
		return sourceFile;
    };
    		
    
    return DocumentRowSearchView;
});