define(["doctablex/StaticDocumentRowView", 
        "doctablex/DocTableXUtils", 
        "utils/webi18n!DocRefStrings",
        "wcr/webcenter_xml"], 
function(StaticDocumentRowView,
         DocTableXUtils,
         DocRefStrings,
         WCXML){
    /**
     * DocumentRowView.js
     * 
     * This class implements reusable rendering methods for dynamic document rows.
     * 
     * @author JANH - January 2015
     */

    "use strict";

    var DocumentRowView = function(dataModel, phElementID, options) {
        if (!dataModel && !phElementID && !options) {
            // handle the inheritance definition in JavaScript: childClass.prototype = new parentClass();
            // the childClass constructor is responsible for calling the constructor of superclass too!!!
            return this;
        }
        StaticDocumentRowView.call(this, dataModel, phElementID, options);   // calling superclass constructor!

        // note this.dataModel and phElemID are initialized in the super class...)

        return this;
    };

    //define DocumentRowView as a subclass of StaticDocumentRowView
    DocumentRowView.prototype = new StaticDocumentRowView();
    DocumentRowView.prototype.constructor = DocumentRowView;
    DocumentRowView.prototype.parent = StaticDocumentRowView.prototype;

    
   /**
     * Method to render document template selector in form of a dropdown box. It uses 
     * this.dm.docTemplates as a source of document templates to be displayed.
     * 
     * @param {boolean} visible - Wether the selector should be visible.
     * 
     * @returns {jQuery} - The generated document template selector element.
     */
    DocumentRowView.prototype.renderDocTemplateSelector = function(visible) {
    	var templateSelectorWrapper = $("<div>");
        try {
        	var docTemplateList = this.dm.docTemplates;
            if (!docTemplateList || (docTemplateList.length == 0)) {
                // no document template selector at all
                return undefined;
            }
            
            if (this.dm.docTempType == 'FixedDocTemplate') {
                // no document template selector as there is just one doc. template to be selected
            	var templateLabel = $("<div>").text(DocRefStrings.t('PROJ_CREATION_DOCUMENT_TEMPLATE'));
            	var fixedTemplate = $("<div>", {"class" : this.getSAHIHelperStr('doctemplate')}).text(docTemplateList[0].name);
            	templateSelectorWrapper.append(templateLabel, fixedTemplate);
            } else {
                // standard document template selector (dropdown) as there is more doc. templates to select from
                var docTempOnSelectMethod = 'javascript: ' + this.dm.getThisInDOM() + '.docTemplateSelected(this.value);';
                var templateSelector = $("<select>", {"class" : "docname-input "+this.getSAHIHelperStr('doctemplate'), "onChange" : docTempOnSelectMethod});
                var helpOption = $("<option>", {"value" : -1}).text(DocRefStrings.t('SELECTDOCTEMPLATE'));
                if (!this.dm.selDocTemplate) {
                	helpOption.attr("selected","selected");
                }
                templateSelector.append(helpOption);
                if (docTemplateList) {
                    for (var idx = 0; idx < docTemplateList.length; idx++) {
                    	var docTemp = docTemplateList[idx];
                    	var option = $("<option>", {"value" : idx}).text(docTemp.name);
                    	if (this.dm.selDocTemplate == docTemp) {
                    		option.attr("selected","selected");
                    	}
                    	templateSelector.append(option);
                    }
                }
                
                if(!this.dm.isEditable){
                	templateSelector.prop("disabled", true);
                }
                
                templateSelectorWrapper.append(templateSelector);
            }
            
            // Determine the style attribute of the selector.
            if (visible === false) {
            	templateSelectorWrapper.hide();
            }
            
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowView.renderDocTemplateSelector:\n' + err);
        }
        return templateSelectorWrapper;
    };
    

    /**
     * Method to render folder selector in form of a dropdown list. It uses this.dm.targetFolders
     * as a source of folders to be displayed.
     * 
     * @param {boolean} visible - Wether the selector should be visible.
     * 
     * @returns {jQuery} - The generated folder selector element.
     */
    DocumentRowView.prototype.renderDocFolderSelector = function(visible) {
    	var docFolderWrapper = $("<div>");
        try {
        	var folderList = this.dm.targetFolders;
            if (!folderList || (folderList.length == 0)) {
                // no folder selection to render 
                return docFolderWrapper;
            }
            
            if (folderList.length == 1) {
                // Only one folder available - just add the text
            	var folderName = folderList[0].name;
            	if (folderList[0].id == '0') {
                	// special handling of the root folder
            		folderName = DocRefStrings.t('DEFAULT_FOLDER_ROOT');
            	}
            	var defaultFolder = $("<span>", {"class" : this.getSAHIHelperStr('docfolder')}).text(DocRefStrings.t('FOLDER')+": "+folderName);
            	docFolderWrapper.append(defaultFolder);
            } else {
                // standard document folder selector (dropdown) as there is more folders to select from
                var folderSelectedMethod = 'javascript: ' + this.dm.getThisInDOM() + '.targetFolderSelected(this.value);';
                var folderSelector = $("<select>", {
                	"class" : "docname-input "+this.getSAHIHelperStr('docfolder'),
                	"onChange" : folderSelectedMethod
                });
                
                if(!this.dm.isEditable){
                	folderSelector.prop("disabled", true);
                }
                
                // by default the first folder in the list is selected
                var selFolderID = folderList[0].id;
                if (this.dm.selTargetFolder) {
                	selFolderID = this.dm.selTargetFolder.id;
                }

                // if there is a selected document template, add the "Default from template" as first option
        		var isDefaultFromDocTempl = false;
                if (this.dm.selDocTemplate) { 
                	if (this.dm.selTargetFolder.isDefaultFromDocTempl) {
                		// if the folder is "Default from template", the very first element will be selected
                		// no matter what folder that option actually means!
                		isDefaultFromDocTempl = true;
                	}
                	var option = new $("<option>", {"value" : -1}).text(DocRefStrings.t('DEFAULT_FOLDER_TEMPLATE'));
                	if (isDefaultFromDocTempl) {
                		option.attr("selected", "selected");
                	}
                	folderSelector.append(option);
                }
                
                // render all target folders as options
                for (var fIdx = 0; fIdx < folderList.length; fIdx++) {
                	var folder = folderList[fIdx];
                	var isSelected = (selFolderID == folder.id) && !isDefaultFromDocTempl;
                	var option = new $("<option>", {"value" : fIdx});
                	if (isSelected) {
                		option.attr("selected", "selected");
                	}
                	if (folder.id == '0') {
                    	// root folder - render the translated label
                		option.text(DocRefStrings.t('DEFAULT_FOLDER_ROOT'));
                	} else {
                		option.text(folder.name);
                	}
                	folderSelector.append(option);
                } 
                docFolderWrapper.append(folderSelector);
            }
            
            if (visible === false) {
            	docFolderWrapper.hide();
            }
            
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowView.renderDocFolderSelector:\n' + err);
        }
        return docFolderWrapper;
    };

    /**
     * Check whether advanced document actions are allowed in this row (e.g. upload new version) 
	 *
     * @override
     * @returns whether or not advanced document actions are allowed
     */
    DocumentRowView.prototype.showAdvancedDocumentActions = function() {
        return false;
    };
    
    /**
     * Overrides the document selector rendering method to hide the check box selector. Newly added
     * documents cannot be downloaded before submitted into WebCenter!
	 *
     * @override
     */
    DocumentRowView.prototype.renderRowSelector = function(phElem) {
    	var htmlcode = '<' + this.itemRootElemStr + ' class="center">';
    	phElem.append(htmlcode);
    };

    
    /**
     * Method to render column/item with a button to cancel the "Replace" operation (in LocalRemoteMethod) on the given document
     * (in this document row).
     * 
     * @override
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     */
    DocumentRowView.prototype.renderReplaceButton = function(phElem) {
        
       try {
    	  // Create the cell that should hold this button.
       	  var cell = $("<" + this.itemRootElemStr + ">", {'class':'center'});
       	 
       	  if (this.dm.origDocRow && this.dm.docFlowType == "Replace") {
       		    var button =  $("<button/>", {
					'type': 'button',
       		    	'class': 'btn btn-xs btn-default actionBtn',
       		    	'title': WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteResetButtonTooltip'))
       		    });
       		    var icon = $("<i>", {'class':'icon icon-normal activIcon icon icon icon-WCR_Revert'});
       		    button.append(icon);
            	button.on("click", function () {
            		this.dm.getParentDocTable().localRemoteButtonClicked(this.rowIndex); 
            		return false;
            	}.bind(this));
            	cell.append(button);
            }
        	// render the prepared HTML code
            phElem.append(cell);
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowView.renderReplaceButton:\n' + err);
        }
    };
    
    
    /**
     * Get a string that will indicate how the document will be autonamed ...
     * 
     * @return {String} A string that will indicate what the final name of this autonamed document could be.
     */
    DocumentRowView.prototype.getAutoNamingPreview = function() {
    	
    	// Generate a placeholder to represent the counter.
    	var counterLength = 0;
    	if (this.dm.anCounterLength) {
    		counterLength = this.dm.anCounterLength;
    	}
		var counterPlaceHolder = "";
        for (var i=0; i < counterLength; i++) {
        	counterPlaceHolder += "X";
        }
        
        // Define the autonaming prefix and suffix. 
        var prefix = '';
        if (this.dm.anPrefix) {
        	prefix = this.dm.anPrefix;
        }
        var suffix = '';
        if (this.dm.anSuffix) {
        	suffix = this.dm.anSuffix;
        }
        
        // For some weird reason the prefix and suffix come encapsulated in '' quotes. Remove them if they are ...
        var prefixMatch = /^'(.*)'$/.exec(prefix);
        if (prefixMatch != null && prefixMatch.length > 1) {
        	prefix = prefixMatch[1];
        }
        var suffixMatch = /^'(.*)'$/.exec(suffix);
        if (suffixMatch != null && suffixMatch.length > 1) {
        	suffix = suffixMatch[1];
        }
        
        return prefix+counterPlaceHolder+suffix;
    };

    /**
     * Override method from DocumentRowSearchView - Not to show clone button while operation type is other than Copy
     * 
     * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
     */
    DocumentRowView.prototype.renderCloneButton = function(phElem) {
        try {
            var htmlcode = '';
           
             htmlcode += '<' + this.itemRootElemStr + '/>';
            
            // render the prepared HTML code
            phElem.append(htmlcode);
        } catch (err) {
            DocTableXUtils.log('Problem in DocumentRowView.renderCloneButton:\n' + err);
        }
    };
    
    return DocumentRowView;
});