define([], function(){
    /**
     * DocumentTypeUtils.js
     * 
     * Global singleton object providing utility-like functionality for document types and their configuration.
     * Never instantiate this class!
     * 
     * @author JANH - December 2014
     */

    "use strict";

    var DocumentTypeUtils = (function() {
        //////////////////////////////////////////////////////////////////////
        // PRIVATE MEMBERS
        //////////////////////////////////////////////////////////////////////

        var isInitialized = false;

        var _otherDocTypeID = '00001_0000000005';
        var _cadDesignDocTypeID = '00001_0000000001';
        var _cadManufacturingID = '00001_0000000002';
        var _cad3DDocTypeID = '00001_0000000003';
        var _cadCanvasDocTypeID = '00001_0000000020';
        var _graphicsDocTypeID = '00001_0000000004';
        var _capeDocDocTypeID = '00001_0000000008';
        var _pageListDocTypeID = '00001_0000000014';
        var _zippedColladaDocTypeID = '00001_0000000016';
        var _colladaDocTypeID = '00001_0000000017';
        var _chiliDocTypeID = '00001_0000000018';
        var _textContentDocTypeID = '00001_0000000019';

        //JANH no better option than a hardcoded list for now!
        // note *.a3d, *.acd and *.mfg files are handled separately
        var _cadDesignFileExts = ['ard', 'cf2', 'cff2', 'ds2', 'dd3', 'dxf', 'dwg', 'scb', 'vlm'];

        // note the _graphicsFileExt and _enabledDocTypes loading is triggered as soon as this JS file is imported.
        // it means that the AJAXDataCache.js has to precede this file import + the values are not available
        // at the very first moments the page is loaded!
        var _graphicsFileExts = [];
        var _enabledDocTypes = [];

        //////////////////////////////////////////////////////////////////////
        // PRIVATE METHODS
        //////////////////////////////////////////////////////////////////////

        var isCADDesignFile = function(ext) {
            for (var i = 0; i < _cadDesignFileExts.length; i++) {
                if ((ext == _cadDesignFileExts[i]) && isEnabled(_cadDesignDocTypeID)) return true;
            }
            return false;
        };

        var isGraphicsFile = function(ext) {
            for (var i = 0; i < _graphicsFileExts.length; i++) {
                if ((ext == _graphicsFileExts[i]) && isEnabled(_graphicsDocTypeID)) return true;
            }
            return false;
        };

        /**
         * Returns true if the 'docTypeID' is among enabled document types (currently in the system)
         */
        var isEnabled = function(docTypeID) {
            for (var i = 0; i < _enabledDocTypes.length; i++) {
                if (docTypeID == _enabledDocTypes[i].id) return true;
            }
            return false;
        };


        // JavaScript singleton pattern - return object exposing nothing else that the getInstance() function
        return {

    //////////////////////////////////////////////////////////////////////
    // PUBLIC MEMBERS
    //////////////////////////////////////////////////////////////////////


    //////////////////////////////////////////////////////////////////////
    // PUBLIC METHODS
    //////////////////////////////////////////////////////////////////////


    /**
     * method to initialize the current configuration of document types on the server.
     * It requires sysDefs to be loaded. The expected sysDefs object structure is the same is returned by
     * AJAXDataCache.getProjCreationSysDefaults() method.
     * Note this method is executed automatically on the successful execution of the
     * AJAXDataCache.getProjCreationSysDefaults() method.
     * 
     * @param sysDefs - Data object containing (at least) system configuration of document types as returned by
     * 		 AJAXDataCache.getProjCreationSysDefaults() method. 
     * @param forceUpdate - Force the replacement of the doc. types configuration if it has been already initialized.
     */
    initDocumentTypes : function(sysDefs, forceUpdate) {
        if (sysDefs && (!isInitialized || forceUpdate)) {
            // get list of supported graphics files extensions
            if (sysDefs.graphicExtList) _graphicsFileExts = sysDefs.graphicExtList;
            if (sysDefs.docTypeList) {
                // get list of enabled document types
                for (var i = 0; i < sysDefs.docTypeList.length; i++) {
                    var docType = sysDefs.docTypeList[i];
                    if (docType.enabled == '1') _enabledDocTypes.push(docType);
                }
            }
            isInitialized = true;
        }
    },


    getIsInitialized : function() {
        return isInitialized;
    },


    /**
     * Method to assess document type based on file extension and current WebCenter doc. type configuration.
     * It supports document type enable/disable feature.
     * 
     * IMPORTANT: This method doesn't work correctly without prior initialization of the DocumentTypeUtils object!
     * 		Make sure the initDocumentTypes() method is executed prior to any call of getDocTypeIDByExtension().
     * 		Note AJAXDataCache.getProjCreationSysDefaults() method is the preferred way to do the initialization.
     * 
     * @param fileExt - Extension of the file to be assessed for a document type.
     * @param selDocType - Hint made by user (or implicit from the context) on the preferred/applicable document type.
     * @returns ID of assessed document type based the given file (extension).
     */
    getDocTypeIDByExtension : function(fileExt, selDocType) {
        if (!isInitialized) {
            // if this class is not initialized, no document type assessments can be done
            return _otherDocTypeID;
        }

        var ext = (fileExt ? fileExt.toLowerCase() : '');

        // recognize the basic ArtiosCAD files (a3d and mfg are handled later)
        if (isCADDesignFile(ext)) return _cadDesignDocTypeID;	

        // recognize graphic files
        if (isGraphicsFile(ext)) return _graphicsDocTypeID;	

        // pagelist cannot be recognized without a hint from 'selDocType'
        if ((selDocType == 'pagelist') && isEnabled(_pageListDocTypeID)) return _pageListDocTypeID;

        // user defined document types also need to be passed via the 'selDocType' - the doc. type ID 
        // is already stored in the selDocType
        if (selDocType && selDocType.indexOf && selDocType.indexOf('00002_0000000') > -1) return selDocType; 

        // determine the doc type ID based on unique file extension
        switch (ext) {
	        case 'a3d' : if (isEnabled(_cad3DDocTypeID)) return _cad3DDocTypeID; break;
	        case 'acd' : if (isEnabled(_cadCanvasDocTypeID)) return _cadCanvasDocTypeID; break;
            case 'mfg' : if (isEnabled(_cadManufacturingID)) return _cadManufacturingID; break;
            case 'clf' : if (isEnabled(_capeDocDocTypeID)) return _capeDocDocTypeID; break;
            case 'zcp' : if (isEnabled(_capeDocDocTypeID)) return _capeDocDocTypeID; break;
            case 'dae' : if (isEnabled(_colladaDocTypeID)) return _colladaDocTypeID; break;
            case 'zae' : if (isEnabled(_zippedColladaDocTypeID)) return _zippedColladaDocTypeID; break;
            case 'chp' : if (isEnabled(_chiliDocTypeID)) return _chiliDocTypeID; break;
            case 'wcc' : if (isEnabled(_textContentDocTypeID)) return _textContentDocTypeID; break;
            case 'wtc' : if (isEnabled(_textContentDocTypeID)) return _textContentDocTypeID; break;
        }
        // if nothing is recognized up until now set the default option - "other" document type
        return _otherDocTypeID;
    }

    //////////////////////////////////////////////////////////////////////
    // END OF PUBLIC METHODS DEFINITION
    //////////////////////////////////////////////////////////////////////
        };
    })();




    return DocumentTypeUtils;
});