define(["doctablex/DocTableXUtils",
        "doctablex/StaticDocumentRowView",
        "doctablex/AJAXDataCache",
        "utils/webi18n!DocRefStrings",
        "wcr/ObjectArrayCloneUtil",
        "wcr/webcenter_xml",
        "underscore",
        "jquery",
		"lib/PromisePolyfill"],
	function (DocTableXUtils,
		StaticDocumentRowView,
		AJAXDataCache,
		DocRefStrings,
		ObjectHandler,
		WCXML,
		_,
		$) {


		/**
		 * StaticDocumentRow.js
		 * 
		 * Object representing a single document already uploaded in WebCenter (so it cannot change), which
		 * is in a documents table (DocTableX instance). It holds required data and implements logic to 
		 * show attribute category (if any) and basic manipulation of the document row and document's properties.
		 * This class is meant to be a base class of all other document (row) types (classes) in DocTableX.
		 * 
		 * @author JANH - January 2015
		 */

		"use strict";

		/**
		 * Constructor for the StaticDocumentRow class.
		 * 
		 * @param {Object} options - Object containing property-value pairs needed for initialization of this object.
		 * @returns {StaticDocumentRow object} - Returns the created (and initialized) instance of this class.
		 */
		var StaticDocumentRow = function (options) {
			if (!options) {
				// handle the inheritance definition in JavaScript: childClass.prototype = new parentClass();
				// the childClass constructor is responsible for calling the constructor of superclass too!!!
				return this;
			}

			this.setStatus('init');

			/** @member {boolean} isChanged - Flag whether this document row was changed by the user or not */
			this.isChanged = false;

			/** @member {int} rowID - Numerical constant value unique for each document row. It doesn't change the whole life of 
			 * 		this row instance. The options.origDocRow is to support reverting operations like "make local" */
			this.rowID = null;
			if (options.origDocRow) {
				this.rowID = options.origDocRow.rowID;
			} else if (options.rowID) {
				this.rowID = options.rowID;
			} else {
				this.rowID = DocTableXUtils.getNextUniquePrefix();
			}

			/** @member {DocTableX} parentDocTable - Reference to the parent document table instance. */
			this.parentDocTable = options.parentDocTable;
			if (!this.parentDocTable) {
				alert("StaticDocumentRow cannot be initialized without parent table reference!");
				return;
			}

			/** @member {String} prefix - A unique prefix associated with this document row instance only.
			 * 		It's mainly used to generate unique IDs and names of HTML elements. 
			 * 		The number at the end is NOT related to documents table row in which this document (row)
			 * 		is currently put! It survives full repaints and never changes! */
			this.prefix = this.getParentDocTable().getTablePrefix() + '_' + this.rowID;

			/** @member {int} _creationIdx - Private member used only to enable better SAHI automated testing.
			 * 		It's index representing the order in which the particular document row was created 
			 * 		in the particular documents table. */
			this._creationIdx = this.getParentDocTable().getNextRowCreationIndex();


			/** @member {boolean} isEditable - True if current user can edit this document row, false otherwise.
			 * 		By default the document row is editable when documents table is editable and vice versa. */
			this.isEditable = this.getParentDocTable().isEditable;
			if (typeof options.isEditable != 'undefined') {
				// default value can be overridden by specific option
				this.isEditable = options.isEditable === true;
			}

			/** @member {Boolean} aCatNotExpandedYet - This is to remember whether the document's attribute
			 * 		category was already expanded at least once (value 'false') or not (value 'true').
			 * 		If true, the descendant attribute and property categories won't be initialized (and loaded)! 
			 * 		It's to optimize speed and memory consumption + it is also a fundamental part of validation 
			 * 		logic that skips descendants not seen by the user! */
			this.aCatNotExpandedYet = true;

			/** @member {boolean} dependsOnItself - True if this document row is cyclic dependent on (one of)
			 * 		its parents, false otherwise. This is calculated per documents table when the document
			 * 		row is added to particular documents table (can be different in each docs. table). */
			this.dependsOnItself = false;


			/** @member {boolean} isSelected - True if this document row is selected for multi-row action(s).
			 * 		By default no row is selected. A row can be selected only if there is selection column
			 * 		configured to be displayed. By default the selection column is displayed on TEP. */
			this.isSelected = false;


			/** @member {Object} renderOptions - Collection of properties defining how this document row
			 * 		should be rendered. */
			this.renderOptions = options.renderOptions;
			// If render options are not provided via 'options', use the documents table options
			if (!this.renderOptions) this.renderOptions = this.getParentDocTable().renderOptions;
			if (!this.renderOptions) this.renderOptions = {};

			/** @member {jQuery Deferred object} docInfoRequest - This object holds all the additional
			 * 		document info loaded via AJAX call and also indicates when the AJAX call is completed.
			 * 		This instance member is used to synchronize all the processing/logic that requires 
			 * 		the additional document info to work properly. */
			this.docInfoRequest = $.Deferred();

			/**
			 * Document related info
			 */
			this.drID = options.drID; // document reference attribute ID (might not be relevant for all DocTableX usecases!)
			this.docVerID = options.docVerID;
			this.docID = options.docID;
			this.docName = options.docName;
			this.docDesc = options.docDesc; // it's document description, not version comment!
			this.docVerIdx = options.docVerIdx; // a numeric document version index
			this.latestDocVerIdx = options.latestDocVerIdx; // the index of the latest document version
			this.docRevLetter = options.docRevLetter; // a numeric representation of document revision
			this.appStatus = options.appStatus;
			this.thumbnailURL = options.thumbnailURL;
			this.docAuthor = options.docVerAuthor;
			this.modDate = options.modDate;
			// document's folder ID and name. this is also the "source folder for the duplicated document rows!
			this.folder = {
				id : null,
				name : null
			};
			if (options.folder) {
				this.folder.id = options.folder.id || null;
				this.folder.name = options.folder.name || null;
			}
			
			/** @member {Array} appTypeList - All approval types applicable for this document. */
			this.appTypeList = options.appTypeList;

			/** @member {Object} userPerms - List of permissions the currently logged in user has to this document. */
			this.userPerms = options.userPerms || {};


			/** @member {String} docTypeID - ID of a document type assigned to the current document. 
			 * 		In case of upload this is the assessed document type ID of the selected file based on
			 * 		its extension (and document type configuration). */
			this.docTypeID = null;

			/** @member {Object} docType - Object containing all relevant info about the document type assigned
			 * 		for this document. */
			this.docType = null;

			/** @member {String} docCheckedOutOwner - contains user ID of user locking the document 
			 * 		(if not locked, it remains null) */
			this.docCheckedOutOwner = null;

			/** @member {Object} project - Object representing the project, where this document is / should be 
			 * 		uploaded to. The object contains {String} id, {String} name and {Boolean} isTemplate members.
			 * 		For documents already uploaded to WebCenter the project ID is loaded via AJAX.
			 * 		For new ones it's expected the correct project ID is sent in 'options'.
			 *		If the document is "used by reference" (not moved/copied/uploaded/linked to the current 
			 *		project), it stays in its original project. In such a case 'this.project.id' is not equal 
			 *		to project ID of the parent documents table! */
			this.project = {
				id: options.projectID || null,
				name: (options.projectID) ? options.projectName : null,
				isTemplate: options.isInTemplate || null
			};

			/** @member {Array.<Object>} projects - List of projects, where the document is linked to.
			 *		The object describing the project consist of project ID and name. This member cannot
			 *		be set to null! */
			this.projects = options.projects || []; //JANH TODO clone this!?

	        // add all properties defined by the reference category of the document reference attribute 
	        // (of this documents table) not in the this.renderOptions to the Render Options Configuration for 
	        // this document row and make them "hidden" in order to preserve their values. 
			// Properties with empty values are skipped, because there is no value to preserve...
	        var rocList = this.renderOptions.columns;
			if (rocList && options.properties) {
		        for (var i = 0; i < options.properties.length; i++) {
					var prop = options.properties[i];
					if (prop && !DocTableXUtils.getRenderItem(rocList, 'ReferenceProperty', prop.id)) {
						// this property is not defined in rendering configuration - add it implicitly as 
						// hidden property, but only if it has a value to be preserved!
						if (prop.value || (prop.value === 0)) {
							// create a new Render Options Configuration record with the "hidden" property 
							var roc = {
									type : 'ReferenceProperty',
									typeDetail : prop.id,
									label : prop.name,
									visibility : 'Hidden'
							};
							rocList.push(roc);
						}
					}
				}
			}        
			
			// sync reference properties from doc. reference rendering configuration and values from 
			// options.properties, which comes from DB...
			var syncedProps = [];
			if (rocList) {
				for (var i = 0; i < rocList.length; i++) {
					var roc = rocList[i]; // Render Option Configuration
					if (roc && roc.type && (roc.type == 'ReferenceProperty')) {
						var propID = roc.typeDetail;
						var prop = this.findPropertyByID(propID, options.properties);
						if (prop) {
							// the property attribute was found, now update the name etc. based on the 
							// property definition in rendering options (can differ from the attribute
							// definition in the property category)
							prop.name = roc.label;
							prop.isRequired = (roc.visibility == 'Required');
							prop.isVisible = (roc.visibility != 'Hidden');
							// value and ID are left from the property attribute!
						} else {
							// create the missing property (a minimal object)
							prop = {
								id: propID,
								name: roc.label,
								isRequired: (roc.visibility == 'Required'),
								isVisible: (roc.visibility != 'Hidden'),
								value: ''
							};
						}
						syncedProps.push(prop);
					}
				}
			}
			
			/** @member {Array.<Object>} props - Array of document reference properties with their values
			 * 		defined for this particular referenced document. */
			this.props = syncedProps;

			/** @member {Boolean} propsValuesInitialized - This member is used to remember whether the 
			 * 		values of properties were already initialized in the reference properties attribute 
			 * 		category (more precisely its attributes == properties) or not. */
			this.propsValuesInitialized = false;

			// document's attribute category related info ==================================

			var expandAttrCatConfig = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout');

			/** @member {boolean} aCatExpandDisabled - If there is no 'expandAttrCatConfig' configuration,
			 * 		the expand/collapse button is not shown in UI and expansion (show) of the document attr.
			 * 		category as whole is effectively disabled. This member holds that info for easy access.
			 * 		It's 'true' if the document's attr. category CANNOT be expanded (shown), 'false' otherwise. */
			this.aCatExpandDisabled = !expandAttrCatConfig;

			/** @member {boolean} aCatExpanded - True if the document's attr. category is expanded (shown),
			 * false otherwise. This member can be used to auto-expand all rows of the documents table. */
			this.aCatExpanded = options.aCatExpanded || false;

			/** @member {boolean} projectCreationContext - True if the referenced doc has still project creation context
			 * only if reference doc. and all its parents are newly created docs., it will be considered as in project creation context */
			this.projectCreationContext = this.parentDocTable.projectCreationContext;

			/** @member {String} aCatView - Attribute category view that should be used when this document's 
			 * 		attribute category is rendered (expanded). Default value is taken from the parent documents
			 * 		table. If even that doesn't exist we fall back on 'Default' view. */
			this.aCatView = this.getParentDocTable().aCatView;
			if (expandAttrCatConfig && expandAttrCatConfig.typeDetail) {
				// use the view from document reference attribute configuration if specified
				this.aCatView = expandAttrCatConfig.typeDetail;
			}

			/** @member {String} aCatPrefix - Prefix of all (input) fields in the associated attr. category. */
			this.aCatPrefix = this.getRowPrefix() + '_ACat_';
			/** @member {String} aCatRowID - ID of attribute category HTML placeholder element. */
			this.aCatRowID = this.getRowPrefix() + '_ACatRow';
			/** @member {String} aCatElemID - ID of HTML element holding reference to the particular 
			 * 		attributeCategory object. It also contains the attr. category HTML. */
			this.aCatElemID = this.getRowPrefix() + '_ACatPanel';

			/** @member {String} aCatID - ID of the document's attribute category. 
			 * 		If it's AJAXDataCache.isEmptyValue(value) == true, this document has no attr. category assigned.
			 *  	If it's value == null, the attr. category ID is not known yet, but (very probably) the 
			 *  		AJAX loading is in progress. */
			this.aCatID = options.aCatID || null;
			this.aCatModDate = null;

			/** @member {attributeCategory} aCatObj - A reference to attributeCategory object created based on 'this.aCatID'. */
			this.aCatObj = null;

			/** @member {Object} renderedAttrCat - An object holding all necessary info needed to 
			 * 		checks whether the rendered attribute category needs to be re-created or not based
			 * 		on the currently expected category and its state (editability). */
			this.renderedAttrCat = {
				id: '',
				isEditable: false
			};

			/** @member {boolean} isExpandedAttrsEditable - True if the configuration allows to edit attributes of this 
			 * 		document row (if any), false otherwise. By default the document's attributes are editable. */
			this.isExpandedAttrsEditable = true;
			if (expandAttrCatConfig) {
				// the editability of document's attributes (attr. category) depends on 'visibility' settings of
				// "expand attribute category" button
				this.isExpandedAttrsEditable = (expandAttrCatConfig.visibility == 'Editable');
			}


			// document reference properties attribute category related info ==================================

			/** @member {String} propsACatPrefix - Prefix of all (input) fields in the associated doc. ref.
			 * 		properties attr. category. */
			this.propsACatPrefix = this.getRowPrefix() + '_PropsACat_';
			/** @member {String} propsACatElemID - ID of HTML element holding reference to the doc. ref. properties 
			 * 		attributeCategory object. It does NOT contain the attr. category HTML! */
			this.propsACatElemID = this.getRowPrefix() + '_PropsACatID';
			/** @member {String} hiddenPropsElemID - ID of placeholder for hidden document reference properties */
			this.hiddenPropValsElemID = this.getRowPrefix() + '_HiddenPropValuesContainerID';

			/** @member {String} propsACatID - ID of the document reference properties attribute category. 
			 * 		If it's value == null, this document has no such an attribute category assigned. 
			 * 		All documents row within a single documents table has the same properties attr. cat. ID,
			 * 		but the attr. cat. object has to be different for each of the document row! */
			this.propsACatID = this.getParentDocTable().propsACatID;

			/** @member {attributeCategory} propsACatObj - A reference to attributeCategory object created 
			 * 		based on 'this.propsACatID'. Must be unique for each document row! */
			this.propsACatObj = null;


			/** @member {Boolean} hasDocRowSelector - True if this document row can be selected in the UI, false otherwise. */
			this.hasDocRowSelector = (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'RowSelector') != null);


			/** @member {String} phElemID - place holder HTML element, where the view will render the content. */
			this.phElemID = this.getRowPrefix();

			/** @member {Object} view - View of this data model to perform rendering operations. */
			var viewClass = options.viewClass || StaticDocumentRowView;
			this.view = new viewClass(
				this, // data model reference
				this.phElemID, // place holder HTML element, where the view will render the content
				this.renderOptions // rendering options (render method, etc.)
			);


			// project attribute category related info ========================================
			this.projACatPrefix = this.getRowPrefix() + '_ProjACat_'; // these will be based on the projectid as well
			this.projACatElemIDPrefix = this.getRowPrefix() + '_ProjACatID_'; // these will be based on the projectid as well


			/** @member {String} copyOfExistingRowID - this is to identify document rows that have been
			 * 		copied from another document row and which one. The value is the row ID of the 
			 * 		original/master document row. It's either null or undefined if this row is not
			 * 		copied from anything. It's independent from clonedFrom property! */
			this.copyOfExistingRowID = options.copyOfExistingRowID || null;


			// private initialization ========================================

			if (this.constructor == StaticDocumentRow.prototype.constructor) {
				// this part of initialization is private for the instances of this class only!
				// it won't be executed for any subclass

				// load basic document info from the server (including this.docTypeID)
				// note: this should be called only after the 'this.view' is properly initialized!
				this._initializeFromServer();
			}

			return this;
		};


		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by 
		 * 'options' (constructor parameter) from the server.
		 */
		StaticDocumentRow.prototype._initializeFromServer = function () {
			this.updateDocInfoFromServer(true);
		};


		/**
		 * This method derives all necessary information/data needed to create a new copy of the original 
		 * (== this) document row.
		 * It's supposed to be extended in subclasses by overriding this basic implementation.
		 * 
		 * @returns {Object} - Document info object == options for the constructor of any document row object.
		 * 		It contains all needed data derived from the original document row and deeply cloned to prevent
		 * 		any "chattering" between the original and new document row.
		 */
		StaticDocumentRow.prototype.getDocRowInfo = function () {
			var docInfo = {
				copyOfExistingRowID: this.rowID,
				parentDocTable: this.getParentDocTable(),
				drID: this.drID, // document reference attribute ID (might not be relevant for all DocTableX usecases!)
				docVerID: this.docVerID,
				docID: this.docID,
				docName: this.docName,
				docDesc: this.docDesc, // it's document description, not version comment!
				docVerIdx: this.docVerIdx, // a numeric document version index
				latestDocVerIdx: this.latestDocVerIdx, // the index of the latest document version
				docRevLetter: this.docRevLetter, // a numeric representation of document revision
				appStatus: this.appStatus,
				thumbnailURL: this.thumbnailURL,
				docAuthor: this.docAuthor,
				modDate: this.modDate,
				appTypeList: this.appTypeList,
				userPerms: this.userPerms,
				project: _.extend({}, this.project),
				projects: [], // deep copy is done below
				properties: [], // deep copy is done below
				aCatID: this.aCatID // note propsACatID is taken from parentDocTable

//				isEditable : this.isEditable,   // will be inherited from the destination parent documents table
//				renderOptions : this.renderOptions,   // will be inherited from the destination parent documents table
//				aCatExpanded : this.aCatExpanded,   // depends on what's best from UX point of view
			};
			// copy of 'this.folder'
			docInfo.folder = {
				id : this.folder.id,
				name : this.folder.name
			};
			
			// copy of 'this.projects'
			_.each(this.projects, function (proj) {
				docInfo.projects.push(_.extend({}, proj));
			}, this);
			// copy of 'this.properties'
			_.each(this.properties, function (prop) {
				docInfo.properties.push(_.extend({}, prop));
			}, this);

			return docInfo;
		};


		/**
		 * Method to execute duplication of this document row to 'dstDocTable'.
		 * @param {DocTableX} dstDocTable - Defines where to duplicate this document row. It must be fully
		 * 		initialized and rendered documents table.
		 * 
		 * @returns {Promise} - Object that is resolved when the specified document row is duplicated.
		 */
		StaticDocumentRow.prototype.duplicateItselfTo = function (dstDocTable) {
			var docInfo = this.getDocRowInfo();

			return dstDocTable.appendDocRow(docInfo, this.constructor);
		};


		/**
		 * @returns {StaticDocumentRow or a subclass instance} The direct parent of this document row.
		 */
		StaticDocumentRow.prototype.getParentDocRow = function () {
			return this.getParentDocTable().getParentDocRow();
		};


		/**
		 * This method returns true if this document row is a direct descendant of cloned (duplicated) row.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.isDirectChildOfClonedRow = function () {
			var parentDocRow = this.getParentDocRow();
			return !!(parentDocRow && parentDocRow.clonedFrom);
		};

		/**
		 * This method returns true if this document row is a direct descendant of cloned (duplicated) 
		 * row and it's copied from one of the direct descendants of the master row.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.isIndirectlyClonedRow = function () {
			return this.isDirectChildOfClonedRow() && !!this.copyOfExistingRowID;
		};


		/**
		 * Method returning all document rows that are descendants of this row (including this row).
		 * It utilizes this.getAllDescendantRowsRecursive() to perform the recursive search.
		 * 
		 * @returns {Array.<StaticDocumentRow or its subclass>}
		 */
		StaticDocumentRow.prototype.getAllDescendantRows = function () {
			var foundDocRows = [];
			// root level is this document row
			
			// recursion call to gather all descendant document rows.
			this.getAllDescendantRowsRecursive(foundDocRows);

	    	return foundDocRows;
		};
		
		/**
		 * Recursively searches for descendants of this document row.
		 * 
		 * @param {Array.<StaticDocumentRow or its subclass>} foundDocRows - Intermediate list of found
		 * 		document rows. At the end of the recursion it contains the result.
		 */
		StaticDocumentRow.prototype.getAllDescendantRowsRecursive = function (foundDocRows) {
			// first add its own rowID (from root to the leafs)
			foundDocRows.push(this);
	    	if (this.aCatObj) {
	    		this.aCatObj.executeOnAllDocReferences(function (docTableObj) {
					// ('docTableObj' is always the "current" documents table object)
	    	    	for (var dr = 0; dr < docTableObj.docRows.length; dr++) {
	    	    		var docRow = docTableObj.docRows[dr];
	    	    		if (docRow) {
	    	    			// recursive call of this method
	    	    			docRow.getAllDescendantRowsRecursive(foundDocRows);
	    	    		}
	    	    	}
				}, this);
			}
	    };        

	    
	    /**
	     * This method returns all document rows that are copied from a set of document rows specified
	     * by 'rows' parameter.
	     * 
	     * @param {Array.<StaticDocumentRow or its subclass>} rows - We are searching for all copies
	     * 		of document rows specified in this array.
	     * @returns {Array.<StaticDocumentRow or its subclass>} - Array of all copied rows. This array
	     * 		doesn't contain duplicates and document rows from the 'rows' input parameter.
	     */
		StaticDocumentRow.prototype.getAllRowsCopiedFromRows = function (rows) {
			var foundDocRows = [];
			if (!rows || (rows.length == 0)) {
				return foundDocRows;
			}
			// get root level
	    	var rootDocTable = this.getParentDocTable().getRootLevelTable();
	    	if (!rootDocTable) {
	    		return foundDocRows;
	    	}
	    	// recursion call to gather all document rows with given row ID
	    	rootDocTable.getRowsCopiedFromRowsRecursive(rows, foundDocRows);
	    	
	    	// filter out rows that are part of 'rowIDs' input parameter
	    	var foundDocRowsFinal = _.filter(foundDocRows, function (foundRow) {
	    		return rows.indexOf(foundRow) == -1;
	    	});
	    	
	    	return foundDocRowsFinal;
	    };        


		/**
		 * This method returns document row from which this document row was copied as a result of
		 * cloning of the parent document row (== 2nd level/indirect clone).
		 * the original counterpart document row.
		 * 
		 * @returns {StaticDocumentRow or its subclass} - Returns null if this document row is not involved 
		 * 		in cloning or is the original document row.
		 */
		StaticDocumentRow.prototype.getMasterOfIndirectlyClonedRow = function () {
			if (this.isDirectChildOfClonedRow()) {
				// 2nd level - try to find the original counterpart document row
				if (this.getParentDocRow()) {
					var foundDocRows = [];
					var parentparentDocTable = this.getParentDocRow().getParentDocTable();
					parentparentDocTable.getRowByIDRecursive(this.copyOfExistingRowID, foundDocRows);
					// there should be no more than one item in the result array always
					if (foundDocRows.length == 1) {
						return foundDocRows[0];
					} else if (foundDocRows.length > 1) {   //JANH debug - then DEL!!!
						window.alert('More than one document row with the very same "rowID"!!!');
					}
				}
			}
			// nothing found - return nothing
			return null;
		};

		/**
		 * Get all rows duplicated from this row.
		 * Note duplicated rows can be only within the parent documents table, not elsewhere.
		 * 
		 * @returns {Array.<StaticDocumentRow or its subclass.} - Found duplicated rows.
		 */
		StaticDocumentRow.prototype.getAllClonesOfThisRow = function () {
			return this.getParentDocTable().getAllRowsClonedFrom(this);
		};

		
		/**
		 * Method to specify a custom view for this StaticDocumentRow (or a subclass) instance. 
		 * The view is responsible for rendering (visual representation) of a document row it is assigned to.
		 * 
		 * @param {Object} customView - The new custom view instance to render this data model.
		 * @param {boolean} repaint - Flag whether the document row should be repainted using the new view (true) or not (false).
		 */
		StaticDocumentRow.prototype.setCustomView = function (customView, repaint) {
			if (!customView) return;

			this.view = customView;
			this.view.dm = this;
			if (!this.view.phElemID) this.view.phElemID = this.getRowPrefix();

			if (repaint) {
				this.displayRow();
			}
		};


		/**
		 * Method to switch the document row (including reference properties) from read-only to editable 
		 * (or vice versa) state and propagate the change to the document's attribute category (if any)
		 * without reloading the whole page.
		 * 
		 * @param {Boolean} newValue - New value for the this.isEditable member.
		 */
		StaticDocumentRow.prototype.setIsEditable = function (newValue) {
			if ((newValue !== undefined) && (newValue != this.isEditable)) {
				this.isEditable = newValue;
			}
		};


		/**
		 * Getter method returning whether this document row is hidden or not == if it sits in a hidden
		 * documents table or not.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.isInactive = function () {
			return this.getParentDocTable().isInactive;
		};


		/**
		 * Method to set the row to be selected by the user.
		 * 
		 * @param {boolean} selected - A boolean value representing if the row is selected (value true) or not.
		 * @param {boolean} repaint - Flag whether to update the corresponding UI selector or not.
		 */
		StaticDocumentRow.prototype.setIsSelected = function (selected, repaint) {

			// See if we have to fire a change event.
			var changed = this.isSelected != (selected == true);

			if (this.hasDocRowSelector) {
				// select the document row only if it's configured to have the selector!
				this.isSelected = (selected == true);
				if (repaint) {
					this.updateRowSelector();
				}
			}

			// Fire a changed event.
			if (changed) {
				$('#' + this.phElemID + '_rowSelector').trigger("change");
			}
		};

		/**
		 * Method to update the state of document row selector (the checkbox).
		 * It should be called whenever a change in document row selection is made/detected.
		 */
		StaticDocumentRow.prototype.updateRowSelector = function () {
			$('#' + this.phElemID + '_rowSelector').prop('checked', this.isSelected);
		};


		/**
		 * Method to validate eligibility of the currently logged in user to download the document
		 * represented by this document row.
		 * 
		 * @param {Object} validObj - Validation object holding data gathered during the validation process
		 * 		that are necessary to handle validation outcome.
		 * 		Its structure is described in DocTableX.validateDocDownloadAction() method, where it's created.
		 */
		StaticDocumentRow.prototype.validateDocToDownload = function (validObj) {
			// validate only selected documents (if if they are not visible right now)
			if (this.isSelected) {
				// remember if any of the selected documents were locked (for further confirmation)
				if (!validObj.lockedDocFound) {
					validObj.lockedDocFound = !!this.docCheckedOutOwner;
				}
				// check also user permission to download each of the selected files!
				if (this.canUserDownloadDocument()) {
					validObj.rowsToDownload.push(this);
				} else {
					validObj.noPermDocNames.push(this.docName);
				}
			}
			// go over descendants to validate other selected documents (no matter if this document can be downloaded!)
			if (this.aCatObj) {
				// if there is a child attribute category, go over all its document reference 
				// attributes and select their document rows as well!
				this.aCatObj.executeOnAllDocReferences(function (docTableObj) {
					// 'docTableObj' is always the "current" documents table object!
					docTableObj.validateDocDownloadAction(validObj);
				}, this);
			}
		};


		/**
		 * Method governing display/show of this document row. It renders document row itself based on provided rendering 
		 * options and optionally also show the attribute category associated to the document row (if any).
		 * 
		 * @param {Object} options - A set of rendering option(s). This controls how the document row will be displayed.
		 * @param {boolean} expandAttrCat - Controls the display state of attribute category (see description of the 
		 * 		showDocAttributeCategory(update, expand) method for more details).
		 */
		StaticDocumentRow.prototype.displayRow = function (expandAttrCat) {
			// first render what can be rendered + placeholders for what needs to be first loaded from the server
			this.view.render();

			// prepare document reference properties attribute category (if any) to be used for reference properties
			// rendering within documents table columns
			if (this.propsACatID && (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'ReferenceProperty') != null)) {
				this._prepareReferenceCategoryObj(false);
			}

			// prepare project attribute categories to be shown
			var showProjectAttributes = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'ProjectAttribute');
			if (showProjectAttributes) {
				this._doForEachProject(function (project) {
					if (project.categoryID) {
						this._prepareProjectCategoryObj(project, false);
					}
				});
			}

			// prepare attribute category to be shown (based on 'expandAttrCat' value), but don't 
			// force-update its data.
			if (this.isAttrCatNeeded()) {
				this.showDocAttributeCategory(false, expandAttrCat);
			}
		};


		/**
		 * Method to reset document related data. Supposed to be called whenever a document selection is changed
		 * while adding new documents. It will revert all document related values to default (empty) or those
		 * already stored in the database.
		 */
		StaticDocumentRow.prototype.resetDocument = function () {
			// reset all data related to this document cached in AJAXDataCache 
			// (it has to precede the reset of doc. version ID!)
			AJAXDataCache.getInstance().resetDocDataByDocVerID(this.docVerID);

			this.dependsOnItself = false;

			// local document data reset
			this.docVerID = '';
			this.docID = '';
			this.docName = '';
			this.appStatus = '';
			this.docVerIdx = '';
			this.thumbnailURL = '';
			this.docDesc = '';
			this.modDate = '';
			this.appTypeList = [];
			this.userPerms = {};

			this.project = {};
			this.projects = [];

			this.folder = {};

			// document type reset
			this.docTypeID = null;
			this.docType = null;

			// document's attribute category related info reset
			this.resetDocumentAttrCategory();

			this.isChanged = false;
			this.isSelected = false;

			// note this.props are not reset here as they cannot change for given document row in any
			// possible situation / use case (known so far)
		};

		/**
		 * Method to reset document's attribute category. 
		 * This is useful e.g. during upload when the attr. category of selected document template competes with default
		 * attr. category of the document type (of the uploaded document).
		 */
		StaticDocumentRow.prototype.resetDocumentAttrCategory = function () {
			this.aCatID = null;
			this.renderedAttrCat = {
				id: '',
				isEditable: false
			};
			this.destroyACatObj(true);
		};



		StaticDocumentRow.prototype.setIsChanged = function () {
			this.isChanged = true;
			if (this.getParentDocTable()) this.getParentDocTable().setIsChanged();
		};

		StaticDocumentRow.prototype.getIsChanged = function () {
			return this.isChanged;
		};


		StaticDocumentRow.prototype.setIsNewDocVersionUploaded = function (value) {
			this.isNewDocVersionUploaded = !!value;
		};

		StaticDocumentRow.prototype.getIsNewDocVersionUploaded = function () {
			return this.isNewDocVersionUploaded;
		};


		/**
		 * Method to set the current internal status of the document row processing.
		 * Possible values are:
		 * - init : initialization (constructor)
		 * - loading : loading data from the server
		 * - wait4user : user has to make a selection/action before the document row is added to the documents table
		 * - noperm : no permission to view the referenced document (document row)
		 * - error : error occurred either during loading from the server or required data for correct initialization is not known
		 * 
		 * Note 'noperm' or 'error' triggers automatic repaint of the row to reflect that state.
		 * 
		 * @param newStatus - One of the above statuses to be set as current document row status.
		 */
		StaticDocumentRow.prototype.setStatus = function (newStatus) {
			if (!this.isInErrorStatus()) {
				// the status can be freely changed only if not set to one of the "error" values already
				this.status = newStatus;
				// new status value set - check whether it's not an "error" status again
				if (this.isInErrorStatus()) {
					// repaint the row and force-hide the attribute category if in "error" status
					this.view.repaintRow();
					this.hideDocAttributeCategory();
					if (this.view.newVersionBtn) {
						// also hide the "upload new document version" row (if shown)
						this.view.newVersionBtn.clearRow();
					}
				}
			}
		};


		StaticDocumentRow.prototype.isInErrorStatus = function () {
			return (this.status == 'noperm') || (this.status == 'error');
		};


		/**
		 * Getter for parent document table.
		 * 
		 * @returns {DocTableX} Documents table instance to which this document row is assigned.
		 */
		StaticDocumentRow.prototype.getParentDocTable = function () {
			return this.parentDocTable;
		};


		/**
		 * Find out the index of this document row in its documents table.
		 * 
		 * @returns {int} - Index of this document row (first row has index 0).
		 */
		StaticDocumentRow.prototype.getIndexInTable = function () {
			if (this.getParentDocTable()) {
				return this.getParentDocTable().docRows.indexOf(this);
			} else {
				return -1;
			}
		};


		/**
		 * Unique prefix of this document row. It never changes. 
		 * It's used to generate unique IDs and names of HTML elements.
		 * 
		 * @returns {String} - A string representing the full document row prefix.
		 */
		StaticDocumentRow.prototype.getRowPrefix = function () {
			return this.prefix;
		};


		/**
		 * Method that generates a JavaScript code to get this instance of StaticDocumentRow (sub)class 
		 * in action handler context in a HTML element (e.g. like button onclick function).
		 * In order this method works the following must be true:
		 * - this.parentDocTable is set correctly,
		 * - a valid reference to the document table must be stored as a custom property 'docTable' in HTML element
		 * 		with ID parentDocTable.phElemID.
		 * 
		 * @returns {String} - A JavaScript code like:
		 * 		'document.getElementById('<parentDocTable.phElemID>').docTable.getRowByID(<row unique ID>)'
		 */
		StaticDocumentRow.prototype.getThisInDOM = function () {
			return this.getParentInDOM() + '.getRowByID(' + this.rowID + ')';
		};


		/**
		 * A "shortcut" to call 'this.getParentDocTable().getThisInDOM()'
		 * It's similar to StaticDocumentRow.getThisInDOM(), but for the documents table.
		 * 
		 * @returns {String} - A JavaScript code like:
		 * 		'document.getElementById('<parentDocTable.phElemID>').docTable
		 */
		StaticDocumentRow.prototype.getParentInDOM = function () {
			if (this.getParentDocTable()) {
				return this.getParentDocTable().getThisInDOM();
			} else {
				// cannot find itself in document table - this should not happen
				DocTableXUtils.error("Error: Cannot find itself in parent documents table!");
				return 'window';
			}
		};



		/**
		 * Method to find property by ID in 'this.props' array.
		 * 
		 * @param {String} propertyID - ID of the document reference property (attribute) to be found.
		 * @param {Array.<Object>} propertyList - List (array) of properties where the particular one should 
		 * 		be searched for. This parameter is optional. If not specified (or null) the value of 
		 * 		'this.props' member is taken instead.
		 * @returns {Object} Returns the found property object or null, if the property was not found.
		 */
		StaticDocumentRow.prototype.findPropertyByID = function (propertyID, propertyList) {
			if (!propertyList) propertyList = this.props; // 'propertyList' is an optional parameter
			if (propertyList) {
				for (var i = 0; i < propertyList.length; i++) {
					var aProp = propertyList[i];
					if (aProp.id == propertyID) return aProp;
				}
			}
			return null;
		};


		/**
		 * Method that updates values of doc. reference properties of this row (stored in this.props) by
		 * values from attributes of property attribute category.
		 * 
		 * @param {Array.<Object>} propertyList - List (array) of properties which should be updated instead
		 * 		of the default 'this.props'. This parameter is optional.
		 */
		StaticDocumentRow.prototype.updatePropertyValues = function (propertyList) {
			if (!this.isAttrCatLoaded(this.propsACatObj)) {
				return; // properties cannot be updated if their attribute category is not loaded
			}
			if (!propertyList) propertyList = this.props; // 'propertyList' is an optional parameter
			if (propertyList) {
				for (var i = 0; i < propertyList.length; i++) {
					var aProp = propertyList[i];
					if (aProp && aProp.id) {
						var propAttObj = this.propsACatObj.findAttribute(aProp.id, aProp.name);
						if (propAttObj) {
							// update the value in the propertyList by the reference attribute's value
							aProp.value = propAttObj.value;
						}
					}
				}
			}
		};



		// Basic document info related code =========================



		/**
		 * Private method to update document related properties of this class instance based on info loaded from server.
		 * Should be overwritten by subclasses that require a different behaviour of 'updateDocInfoFromServer()' method.
		 * 
		 * @returns {boolean} - 'true' if document was changed, 'false' otherwise.
		 */
		StaticDocumentRow.prototype._updateDocInfo = function (docInfo) {
			var modified = false;
			if (docInfo && docInfo.docVerID && (docInfo.docVerID == this.docVerID)) {
				if (this.docID != docInfo.docID) {
					modified = true;
					this.docID = docInfo.docID;
				}
				if (!this.docName && (this.docName != docInfo.docName)) {
					// this can be edited on page by user
					modified = true;
					this.docName = docInfo.docName;
				}
				if (!this.docDesc && (this.docDesc != docInfo.docDesc)) {
					// this can be edited on page by user
					modified = true;
					this.docDesc = docInfo.docDesc;
				}
				if (this.docVerIdx != docInfo.docVerIdx) {
					modified = true;
					this.docVerIdx = docInfo.docVerIdx;
				}
				if (this.latestDocVerIdx != docInfo.latestDocVerIdx) {
					modified = true;
					this.latestDocVerIdx = docInfo.latestDocVerIdx;
				}
				if (this.docRevLetter != docInfo.docRevLetter) {
					modified = true;
					this.docRevLetter = docInfo.docRevLetter;
				}
				if (this.appStatus != docInfo.appStatus) {
					modified = true;
					this.appStatus = docInfo.appStatus;
				}
				if (this.thumbnailURL != docInfo.thumbnailURL) {
					modified = true;
					this.thumbnailURL = docInfo.thumbnailURL;
				}
				if (this.docTypeID != docInfo.docTypeID) {
					modified = true;
					this.docTypeID = docInfo.docTypeID;
				}
				if (this.docOperation != 'Move') {
					// the project ID defines what default attr. category, autonaming configuration and doc. type 
					// settings are taken for this document.
					// it is updated only if empty (not set yet or reset) and when the document was not moved!
					// (note copying is handled by overridden version of this method in DocumentRowSearch.js)
					if (!this.project.id && docInfo.projList[0] && (this.project.id != docInfo.projList[0].id)) {
						modified = true;
						this.project = _.extend(this.project, docInfo.projList[0]);
					}
					if (docInfo.projList && ('' + this.projects != '' + docInfo.projList)) {
						modified = true;
						this.projects = [];
						_.each(docInfo.projList, function (proj) {
							this.projects.push(_.extend({}, proj));
						}, this);
					}
				}

				// in project creation view
				// this.projectCreationContext should be set to false for existing documents in other projects
				if (this.isRemoteDocument()) {
					this.projectCreationContext = false;
				}
				
				if (docInfo.folderInfo) {
					this.folder.id = docInfo.folderInfo.id;
					this.folder.name = docInfo.folderInfo.name;
				}

				if (docInfo.docVerAuthor) {
					if (!this.docAuthor || (this.docAuthor.id != docInfo.docVerAuthor.id)) {
						modified = true;
						this.docAuthor = docInfo.docVerAuthor;
					}
				}
				if (this.modDate != docInfo.modDate) {
					try {
						modified = true;
						if (!docInfo.modDate) this.modDate = '';
						else this.modDate = new Date(parseInt(docInfo.modDate));
					} catch (err) {
						DocTableXUtils.log("Failed to parse '" + docInfo.modDate + "' to a Date object.");
						this.modDate = '';
					}
				}
				if (docInfo.userPerms) {
					this.userPerms = docInfo.userPerms;
					// no operation within documents table can change the 'userPerms' - no change to 'modified' flag here
				}
				if (docInfo.appTypeList) {
					this.appTypeList = docInfo.appTypeList;
					// no operation within documents table can change the 'appTypeList' - no change to 'modified' flag here
				}

				if (docInfo.docCheckedOutOwner) {
					this.docCheckedOutOwner = docInfo.docCheckedOutOwner;
				}

				// update the this.aCatID and other members related to attribute category of current document
				// based on the response returned from the server (and internal logic)
				if (this.updateDocAttrCategoryFromValue(docInfo.catInfo)) {
					modified = true;
				}

			} else if (AJAXDataCache.isEmptyValue(docInfo)) {
				// when 'getDocInfoWithThumbnail' returns 'emptyObject' it means the document info cannot be loaded. If it's
				// not an AJAX error, the reason is no permissions or that the document doesn't exist (which is in the context
				// of the SDK call the same thing).
				this.setStatus('noperm');
			}
			return modified;
		};


		/**
		 * Load and update document info related to this document row and repaint it if needed.
		 * The info is loaded using AJAXDataCache object.
		 * 
		 * @param repaint - Flag whether to repaint this document row when doc info is loaded ('true') or not (value 'false').
		 * @param forceUpdate - Flag whether to force the cache to be refreshed with the most actual data.
		 * 
		 * @returns - If this.docVerID is a valid ID this method returns a promise object representing the AJAX request 
		 * 		to get info about attribute category assigned to this document (the primary is attribute category ID).
		 * 		Otherwise it returns rejected promise object immediately.
		 */
		StaticDocumentRow.prototype.updateDocInfoFromServer = function (repaint, forceUpdate) {
			this.setStatus('loading');

			// nothing to do without a document version ID
			if (!this.docVerID) {
				var errorMsg = "Cannot update document info from server, this.docVerID is not known!";
				return $.Deferred().reject(errorMsg).promise();
			}

			this.docInfoRequest = AJAXDataCache.getInstance().getDocInfoWithThumbnail(this.docVerID, this.getDocSourceProjectID(), forceUpdate);
			$.when(this.docInfoRequest).done(function (docInfo) { // on success
				// update of basic document info from the server, which is stored in this document row object
				var docRowModified = this._updateDocInfo(docInfo);

				// extended update (if needed)
				var moreChanges = this.updateOtherInfoFromServer(repaint, forceUpdate);

				if ((docRowModified || moreChanges) && repaint) {
					// update/repaint the row with the loaded data, but only if something was changed
					this.view.repaintRow();
				}
			}.bind(this))
			.fail(function (errorMsg) { // on error
				this.setStatus('error');
				// no need to alert the error message as the document row rendering is altered based on the error status!
				DocTableXUtils.log(errorMsg);
			}.bind(this));

			// return the promise object representing the request of data to allow further processing
			return this.docInfoRequest;
		};
		
		
		/**
		 * Can be implemented in subclasses/mixins
		 * @param repaint - Flag whether to repaint this document row when doc info is loaded ('true') or not (value 'false').
		 * @param forceUpdate - Flag whether to force the cache to be refreshed with the most actual data.
		 */
		StaticDocumentRow.prototype.updateOtherInfoFromServer = function (repaint, forceUpdate) {
			this._loadProjectStatusAndCategories().then(_.bind(function () {
				this.displayRow();
			}, this)).catch(function (err) {
				if (err) {
					DocTableXUtils.log("StaticDocumentRow.updateOtherInfoFromServer() has an error:" + err);
				}
			});

			return false;
		};

		/**
		 * Load the project status and categories info from server
		 * @private
		 * @returns {Promise.<Array.<Object>>} - Resolved when all information is fetched for all projects. The extended projects are returned in an array.
		 */
		StaticDocumentRow.prototype._loadProjectStatusAndCategories = function () {
			var showProjectStatus = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'ProjectProperty_ProjectStatus');
			var showProjectAttributes = DocTableXUtils.getRenderItem(this.renderOptions.columns, 'ProjectAttribute');

			if (this.getParentDocTable().isProjectCreationContext() &&
				this.docOperation &&
				!(this.docOperation == 'Link' || this.docOperation == 'Reference')) {
				// project creation context and an operation that is NOT link or reference
				// in this case we show nothing on the project creation page instead of information of the template (project.id is template in that case)
				showProjectStatus = false;
				showProjectAttributes = false;
			}

			if (showProjectStatus || showProjectAttributes) {
				return Promise.all(_.map(this._getProjects(), _.bind(function (project) {
					return Promise.resolve(project).then(_.bind(function (project) {
						if (!project.statusID || !project.categoryID) {
							// we need to fetch extra info
							return this._extendWithProjectInfo(project);
						} else {
							// we already have the info
							return project;
						}
					}, this)).then(function (project) {
						// project has correct ids
						if (showProjectStatus) {
							var statusID = project.statusID;
							if (statusID) {
								return Promise.resolve(AJAXDataCache.getInstance().getProjectStatuses()).then(function (projectStatuses) {
									var statusObj = projectStatuses[statusID];
									if (statusObj) {
										project.statusObj = statusObj;
									}
									return project;
								});
							}
						}
						return project;
					});
				}, this))).catch(function (err) {
					DocTableXUtils.log("StaticDocumentRow._loadProjectStatusAndCategories() has an error:" + err);
				});
			}
			return Promise.reject();
		};


		/**
		 * Method that finds out the project in which the document (source document in case of a new document row) is stored in.
		 * This method can be overridden in subclasses if special behaviour is needed there.
		 * 
		 * @returns {Object/undefined} Object with project ID and name if the (source) document's project is found, 
		 * 		otherwise undefined is returned.
		 */
		StaticDocumentRow.prototype.getDocSourceProjectID = function () {
			if (this.sourceProjID) {
				// subclasses representing a new document can have this member set
				return this.sourceProjID;
			}
			if (!this.isRemoteDocument()) {
				// not a remote file means the DOcTableX.projectID is the project where this document is stored
				return this.getParentDocTable().projectID;
			}
			if (this.projects && (this.projects.length > 0)) {
				if (this.project && this.project.id) {
					// when there are this.projects and this.project both defined, try to find this.project among this.projects.
					// if successful this.project is the project of the source document, otherwise this.project is the target
					// project (common for new document rows, but those often sets the 'sourceProjID').
					var foundProject = _.find(this.projects, function (proj) {
						return proj && proj.id == this.project.id;
					}, this);
					if (foundProject) {
						return foundProject.id;
					}
				}
				// when we cannot find the project-s "correlation" the first project in this.projects 
				// list is the best guess...
				return this.projects[0];
			}
			// if this.projects are not loaded the document row is not initialized enough yet.
			return null;
		};
		

		/**
		 * Execute the given function for each project. This is both done on this.projects and this.project
		 * @param {Function} func - The function to execute with project as parameter
		 */
		StaticDocumentRow.prototype._doForEachProject = function (func) {
			var projList = this.projects;
			if (projList && (projList.length > 0)) {
				for (var i = 0; i < projList.length; i++) {
					var proj = projList[i];
					func.call(this, proj);
				}
			} else if (this.project && this.project.id) {
				func.call(this, this.project);
			}
		};

		/**
		 * Get projects in an array to do an action on: it will represent this.projects and this.project as one array.
		 * @private
		 * @returns {Array.<Object>} Array of project objects
		 */
		StaticDocumentRow.prototype._getProjects = function () {
			var result = [];
			var projList = this.projects;
			if (projList && (projList.length > 0)) {
				for (var i = 0; i < projList.length; i++) {
					var proj = projList[i];
					result.push(proj);
				}
			} else if (this.project && this.project.id) {
				result.push(this.project);
			}
			return result;
		};

		/**
		 * Extend the given project with extra project info
		 * @private
		 * @throws {Error} If the project is undefined or it doesn't have an id
		 * @param   {Object}   project - The project to extend
		 * @returns {Promise.<Object>} Promise with extended project as resolved parameter
		 */
		StaticDocumentRow.prototype._extendWithProjectInfo = function (project) {
			return Promise.resolve().then(function () {
				if (project && project.id) {
					return Promise.resolve(AJAXDataCache.getInstance().getProjectInfo(project.id)).then(function (projectInfo) {
						project = _.extend(project, projectInfo);
						return project;
					});
				} else {
					throw new Error("Project or Project ID is undefined");
				}
			});
		};


		/**
		 * Just an encapsulation/shortcut for calling 'this.getParentDocTable().isInConfigPreview'
		 * 
		 * @returns {Boolean} Returns true if the documents table is displayed in the attribute category
		 * 		layout context, false otherwise.
		 */
		StaticDocumentRow.prototype.isInConfigPreview = function () {
			return (this.getParentDocTable().isInConfigPreview);
		};


		/**
		 * Find out whether this document is local or remote (= referenced from different project).
		 * 
		 * @returns {Boolean} Returns 'true' if document represented by this document row is not from the same
		 * 		project as the documents table is set to.
		 */
		StaticDocumentRow.prototype.isRemoteDocument = function () {
			var isRemote = true;
			if (this.projects && this.projects.length > 1) {
				isRemote = !_.find(this.projects, function (project) {
					return project.id == this.getParentDocTable().projectID;
				}, this);
			} else {
				isRemote = (this.project.id && (this.project.id != this.getParentDocTable().projectID));
			}

			return isRemote;
		};


		/**
		 * Find out whether the referenced document version is the latest version of the document in the system.
		 * 
		 * @returns {Boolean} Returns 'true' if the latest document version is referenced, 'false' otherwise.
		 */
		StaticDocumentRow.prototype.isLatestDocVersion = function () {
			return this.latestDocVerIdx && (this.docVerIdx == this.latestDocVerIdx);
		};



		// Attribute category related methods =====================================


		/**
		 * Find out if this document (row) has attribute category.
		 * 
		 * @returns {Boolean} Returns true if this document has an attribute category assigned, otherwise false.
		 * 		It returns false even if the this.aCatID is not set/known yet, too!
		 */
		StaticDocumentRow.prototype.hasAttrCat = function () {
			return !!this.aCatID && !AJAXDataCache.isEmptyValue(this.aCatID) && DocTableXUtils.isValidID(this.aCatID);
		};

		/**
		 * Finds out whether this document (row) has certainly NO attribute category.
		 * 
		 * @returns {Boolean} Returns true only if it's certain that this document has no attribute category assigned,
		 * 		otherwise false. (Note It returns false even if the this.aCatID is not set/known yet!)
		 */
		StaticDocumentRow.prototype.hasNoAttrCat = function () {
			return !!this.aCatID && AJAXDataCache.isEmptyValue(this.aCatID);
		};


		/**
		 * Set document row to have no attribute category as the document has no attribute category defined.
		 */
		StaticDocumentRow.prototype.setDocumentHasNoAttrCategory = function () {
			this.aCatID = AJAXDataCache.getEmptyValue();
			this.destroyACatObj(true);
		};


		/**
		 * Method returning true if at least one attribute from the attr. category is used/displayed
		 * in this document row taking into account current rendering configuration. 
		 * Otherwise it returns false.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.isAttrCatNeeded = function () {
			if (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'AttributeFlapout')) {
				// this document row is configured to show attribute category "flapout" - attr. cat. needed!
				return true;
			}
			if (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'DocumentAttribute')) {
				// at least one document attribute configured to be displayed in a column - attr. cat. needed!
				return true;
			}
			// none of the above conditions met - the document row is not configured to show attribute category
			return false;
		};


		/**
		 * Method returning true if the actual attribute category associated to this document row has
		 * changed or if the category is the same, but its editability has changed (both ways: from 
		 * editable to read-only and vice versa). False otherwise.
		 * The current attribute category (by ID) and the users rights+configuration to edit the category
		 * is checked against the state when the attribute category object (this.aCatObj) was created
		 * the last time.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.isAttrCatUpdateNeeded = function () {
			return (this.renderedAttrCat.id != this.aCatID) || (this.renderedAttrCat.isEditable != this.canUserEditExpandedAttributes());
		};


		/**
		 * Is the given attrCatObj loaded?
		 * @param   {object}   attrCatObj
		 * @returns {Boolean} True if loaded, false otherwise.
		 */
		StaticDocumentRow.prototype.isAttrCatLoaded = function (attrCatObj) {
			return (attrCatObj && attrCatObj.loaded);
		};


		/**
		 * Method returning 'true', if attributes of this document can be edited in "general" - meaning
		 * in the columns of documents table. Otherwise 'false' is returned.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.canUserEditAttributes = function () {
			return !this.isInactive()
				// no view right and/or error loading doc. details
				&& !this.isInErrorStatus() && this.isLatestDocVersion() && !this.dependsOnItself
				// the right to change attributes (among other things)
				&& this.userPerms.change;
		};

		/**
		 * Method returning 'true', if attributes of this document can be edited in its attribute category 
		 * "flapout". Otherwise 'false' is returned.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.canUserEditExpandedAttributes = function () {
			// the normal attribute editability summed up with the '+' sign visibility value in the doc. 
			// table rendering configuration 
			return this.canUserEditAttributes() && this.isExpandedAttrsEditable;
		};


		/**
		 * Method controlling whether the currently logged in user can modify reference properties.
		 * This method signals that user can edit properties in general, but each particular property still
		 * needs to be set editable (or required) in doc. table rendering configuration!
		 * 
		 * @returns {Boolean} 'true' if the user can edit properties (in general), 'false' otherwise.
		 */
		StaticDocumentRow.prototype.canUserEditProperties = function () {
			return !this.isInactive() && this.isEditable && !this.dependsOnItself;
			// (user can edit reference properties whenever the he/she can edit values of the documents table
			// with the exception of the second appearance of cyclic dependent document)
		};


		/**
		 * Returns 'true' if the currently logged in user can add documents to at least one folder 
		 * (including root folder) of the current project. Otherwise it returns 'false'.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.canUserAddDocuments = function () {
			return this.getParentDocTable().canUserAddDocuments;
		};


		/**
		 * Returns 'true' if the currently logged in user can download document represented by this row. 
		 * Otherwise it returns 'false'.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.canUserDownloadDocument = function () {
			return this.userPerms.download;
		};


		/**
		 * Returns 'true' if the currently logged in user can edit the document row and has "checkout" 
		 * permission on given document/folder, which means the user can upload new versions, 
		 * lock documents and/or start/stop approval cycle on them in the given folder. 
		 * Otherwise it returns 'false'.
		 * 
		 * @returns {Boolean}
		 */
		StaticDocumentRow.prototype.canUserUploadNewVersion = function () {
			return !this.isInactive()
				// no view right and/or error loading doc. details
				&& !this.isInErrorStatus() && !!this.isLatestDocVersion() && !this.dependsOnItself
				// doc.ref.mode 1 ("Selected Version") doesn't allow uploading new versions
				&& (this.getParentDocTable().docReferenceMode != 1)
				// the right to upload new versions, lock documents and start/stop approval on them
				&& !!this.userPerms.checkout && !this.isModifiedElsewhere();
		};


		/**
		 * @returns {Boolean} True if the document row belongs to a documents table that is in
		 * 		"modifed elsewhere" mode, false otherwise.
		 */
		StaticDocumentRow.prototype.isModifiedElsewhere = function () {
			return this.getParentDocTable().isModifiedElsewhere === true;
		};


		/**
		 * Updates attribute category info in this document row from the 'aCatInfo' object loaded via AJAX.
		 * Currently it updates 'this.aCatID' and 'this.aCatModDate' (used to control server side cache).
		 * 
		 * @param {Object} aCatInfo - Info about attribute category of this document row loaded from the server
		 * 
		 * @returns {Boolean} - Flag whether the attribute category was changed or not.
		 */
		StaticDocumentRow.prototype.updateDocAttrCategoryFromValue = function (aCatInfo) {
			var modified = false;
			if (aCatInfo) {
				if (aCatInfo.id) {
					if (this.aCatID != aCatInfo.id) modified = true;
					this.aCatID = aCatInfo.id;
				} else {
					// there is no attribute category assigned to this document
					if (!AJAXDataCache.isEmptyValue(this.aCatID)) modified = true;
					this.setDocumentHasNoAttrCategory();
				}
				this.aCatModDate = aCatInfo.modTime;
//	        	this.aCatUserUnits = aCatInfo.userUnits;   //JANH TODO not used for now!!!
			} else {
				// an error occurred during the AJAX call - pretend the document has no attribute category
				if (!AJAXDataCache.isEmptyValue(this.aCatID)) modified = true;
				this.setDocumentHasNoAttrCategory();
			}
			// return whether the attribute category was changed or not
			return modified;
		};



		/**
		 * Private (low level) method to setup and initialize attributeCategory object of this document's
		 * attribute category.
		 * All required callbacks/listeners related to the attribute category are registered here too.
		 * 
		 * @param {function} onReadyCallback - Function to be called when the attribute category object is ready 
		 * 		(it finishes initialization, loading data and it's rendered).
		 * @param {function} onFailCallback - Function to be called when an error occurs during initialization, 
		 * 		data loading or rendering.
		 */
		StaticDocumentRow.prototype._initializeAndRenderAttrCategoryObj = function (onReadyCallback, onFailCallback) {

			if (this.getParentDocTable().parentACatNotExpandedYet) {
				// no need to initialize/show attribute category of this row as user cannot see it anyway!
				return;
			}

			if ($('#' + this.aCatElemID).length == 0) {
				// no place to render attribute category - postponing its initialization
				return;
			}

			if (!this.hasAttrCat()) {
				DocTableXUtils.log("Attempt to render attribute category when its ID is not known!");
				return;
			}

			this.setStatus('loading');

			// current user object for the attribute cat.
			var currentUser = ObjectHandler.getCloneOfObject(this.getParentDocTable().currentUser);
			currentUser.canEditAttributes = this.canUserEditExpandedAttributes();

			// support for nesting level counting and cycle detection
			var childParentDocVerIDs = [];
			var pdvIDs = this.getParentDocTable().parentDocVerIDs;
			if (pdvIDs && (pdvIDs.length > 0)) {
				// deep copy of the parent document version IDs array to not influence (potential) other 
				// document tables with the same parent document table
				childParentDocVerIDs = ObjectHandler.getCloneOfArray(pdvIDs);
			}

			// add the ID into the array of parent doc. version IDs in all cases - then the length of 
			// the array indicates the nesting level...
			if (this.docVerID) {
				childParentDocVerIDs.push(this.docVerID);
			} else if (this.newDocTmpID) {
				// this is the case for newly added documents that has no document version ID yet assigned
				//JANH TODO - ideally this should be in DocumentRow.js in the overriden variant of this method, but for now it's acceptable...
				childParentDocVerIDs.push(this.newDocTmpID);
			} else {
				window.alert('Problems in composing inheritance path!!!'); //JANH debug - then DEL!!!
			}

			// setup all for the attribute category
			var aCatOptions = {};
			aCatOptions.prefix = this.aCatPrefix; // rowPrefix + '_ACat_'
			aCatOptions.ACatObj = this.aCatElemID; // rowPrefix + '_ACatRowID'
			aCatOptions.ACatPanel = this.aCatElemID; // rowPrefix + '_ACatPanelID'
			aCatOptions.readURL = window.webcenterConfig.jspBaseUrl + 'GetAttributeCategoryXML.jsp?categoryid=' + this.aCatID + '&ajax=1';
			if (this.aCatModDate) {
				aCatOptions.readURL += '&lastmodified=' + this.aCatModDate;
			}
			aCatOptions.valuesURL = this.generateDocAttributeValuesAJAXURL();
			if (!aCatOptions.valuesURL) {
				// just load attribute types, restricted sets etc. to correctly render them (along with 
				// empty attribute values)
				aCatOptions.valuesURL = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?categoryid=' + this.aCatID + '&ajax=1';
			}

			aCatOptions.currentUser = currentUser;
			aCatOptions.view = this.getAttCatView();
			aCatOptions.type = "document";
			aCatOptions.projectID = this.project.id;
			aCatOptions.documentID = this.docID;
			if (this.docVerID) {
				aCatOptions.docVersionID = this.docVerID;
			}

			// success and fail callbacks registration
			if (onReadyCallback) aCatOptions.callbackOnRender = onReadyCallback;
			if (onFailCallback) aCatOptions.callbackOnFail = onFailCallback;
			aCatOptions.callbackOnLoad = function() {
				// this callback takes care of hiding attribute category if after it's loaded we realize
				// there is nothing to show (the requested view is not defined). 
				// this is however limited to auto-expanded attribute categories only! (currently only
				// in TEP -> Documents section)
				if (this.aCatObj.noCurrentViewDisplay && this.getParentDocTable().autoExpandAllRows) {
					this.aCatExpandDisabled = true;
					this.showDocAttributeCategory();   // here it actually hides the category in a consistent way
				}
			}.bind(this);

			// supply the "parent doc. version ID list" of current doc table + the actual document version ID
			// to the constructor of the assigned attribute category in order to be able to initialize 
			// all potential document tables within that attr. category (== the descendant document tables)!
			aCatOptions.docTableParentDocVerIDs = childParentDocVerIDs;

			// level of current doc. version ID in the document tables hierarchy
			aCatOptions.level = childParentDocVerIDs.length;

			// in the project creation page, Project creation context decided based on the document operation and level
			aCatOptions.projectCreationContext = this.projectCreationContext;
			if ((aCatOptions.level > 0) && (this.docOperation == 'Link' || this.docOperation == 'Reference')) {
				//If it is an existing doc. make project creation context as false
				aCatOptions.projectCreationContext = false;
			}

			// supply the reference to the Document row object that is the closest parent of the attribute category
			// (documents table(s) within the category).
			aCatOptions.parentDocRow = this;

			// let the new attribute category know that its parent (documents table) is hidden
			// (this flag is used to handle visibility and edit-ability of attribute values correctly)
			aCatOptions.isInactive = this.isInactive();

			// if the attribute category is not expanded yet, don't initialize any potential descendants of 
			// this attribute category until they are really need (when the attribute category is expanded)
			aCatOptions.aCatNotExpandedYet = this.aCatNotExpandedYet;

			// if parent attribute has a style defined, make sure it propagates trough to the child category
			if (this.renderOptions.styleObj) {
				aCatOptions.defaultStyleObject = this.renderOptions.styleObj.inlineStyleObject;
			}

			require(["attributecategory/AttributeCategory"], function (AttributeCategory) {
				// create the attribute category object here and store it!
				var newACatObj = new AttributeCategory(aCatOptions);
				if (this.aCatObj) {
					// destroy the previous attribute category (including its data model in memory with event listeners etc.)
					// this can happen as loading of AttributeCategory via required.js is asynchronous...
					this.aCatObj.destroy(true);
				}
				this.aCatObj = newACatObj;
				// (note attributeCategory constructor itself stores the aCatObj in the dedicated DOM object 
				// with id="<aCatOptions.ACatPanel>")

        		// register "on change" listener to propagate changes done in this attribute category
        		// to the root of the tree-like doc. references structure (if any)
				var rootDocTable = this.getParentDocTable().getRootLevelTable();
	        	if (rootDocTable && rootDocTable.onChangeCallback) {
	        		this.aCatObj.valueChangedListeners.push(function () {
	                	// call also corresponding root level "on change" callback function as it can have special 
	    				// behaviour related to page where this document reference hierarchy is used/placed.
		        		rootDocTable.onChangeCallback();
					}.bind(this));
	        	}
			}.bind(this));

			// remember the initialized (created) attribute category details for further checks if the 
			// category needs to be re-created or not.
			this.renderedAttrCat.id = this.aCatID;
			this.renderedAttrCat.isEditable = currentUser.canEditAttributes;
		};



		/**
		 * Private (low level) method to setup and initialize attributeCategory object governing rendering of 
		 * reference properties associated with this document row.
		 * All required callbacks/listeners related to the attribute category are registered here too.
		 * 
		 * @param {function} onReadyCallback - Function to be called when the attribute category object is ready 
		 * 		(it finishes initialization, loading data and it's rendered).
		 * @param {function} onFailCallback - Function to be called when an error occurs during initialization, 
		 * 		data loading or rendering.
		 */
		StaticDocumentRow.prototype._initializeAndRenderReferenceCategoryObj = function (onReadyCallback, onFailCallback) {

			if (this.getParentDocTable().parentACatNotExpandedYet) {
				// no need to initialize this as user cannot see it anyway!
				return;
			}

			if ($('#' + this.propsACatElemID).length == 0) {
				// no place to render reference category (yet) - postponing its initialization
				return;
			}

			if (!this.propsACatID) {
				DocTableXUtils.log("Attempt to initialize reference properties when reference category ID is not known!");
				return;
			}

			this.setStatus('loading');

			// remember that we are re-creating the reference properties attribute category and thus
			// the property values has to be copied from 'this.props'. 
			// this is however one-time operation only!
			this.propsValuesInitialized = false;

			// current user object for the attribute cat.
			var currentUser = ObjectHandler.getCloneOfObject(this.currentUser);
			// set whether the attribute category should be in editable or readonly mode
			currentUser.canEditAttributes = this.canUserEditProperties();

			// setup all for the attribute category
			var aCatOptions = {};
			aCatOptions.prefix = this.propsACatPrefix; // rowPrefix + '_PropsACat_'
			aCatOptions.ACatObj = this.propsACatElemID; // rowPrefix + '_PropsACatID'
			aCatOptions.ACatPanel = this.propsACatElemID; // rowPrefix + '_ACatPanelID' - THIS MAKES THE ACAT RENDER!!!
			aCatOptions.readURL = window.webcenterConfig.jspBaseUrl + 'GetAttributeCategoryXML.jsp?categoryid=' + this.propsACatID + '&ajax=1';
			// just load attribute types, restricted sets etc. to correctly render them (along with 
			// empty attribute values)
			aCatOptions.valuesURL = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?categoryid=' + this.propsACatID + '&ajax=1';

			aCatOptions.currentUser = currentUser;
			//		aCatOptions.defaultHourAsHalfHours = 34; 

			// to distinguish this attribute category from document/project attr. categories in the attribute category itself!
			aCatOptions.isDocRefPropertiesACat = true;

			// success and fail callbacks registration
			if (onReadyCallback) aCatOptions.callbackOnRender = onReadyCallback;
			if (onFailCallback) aCatOptions.callbackOnFail = onFailCallback;


			require(["attributecategory/AttributeCategory"], function (AttributeCategory) {
				// create the reference attribute category object here and store it!
				var newACatObj = new AttributeCategory(aCatOptions);
				if (this.propsACatObj) {
					// destroy the previous attribute category (including its data model in memory with event listeners etc.)
					// this can happen as loading of AttributeCategory via required.js is asynchronous...
					this.propsACatObj.destroy(true);
				}
				this.propsACatObj = newACatObj;
				// (note attributeCategory constructor itself stores the aCatObj in the dedicated DOM object 
				// with id="<aCatOptions.ACatPanel>")

        		// register "on change" listener to propagate changes done in this attribute category
        		// to the root of the tree-like doc. references structure (if any)
				var rootDocTable = this.getParentDocTable().getRootLevelTable();
	        	if (rootDocTable && rootDocTable.onChangeCallback) {
					this.propsACatObj.valueChangedListeners.push(function () {
	                	// call also corresponding root level "on change" callback function as it can have special 
	    				// behaviour related to page where this document reference hierarchy is used/placed.
		        		rootDocTable.onChangeCallback();
					}.bind(this));
	        	}
			}.bind(this));
		};


		/**
		 * Private (low level) method to setup and initialize attributeCategory object governing rendering of 
		 * project attributes associated with this document row.
		 * All required callbacks/listeners related to the attribute category are registered here too.
		 * 
		 * @param {Object} project - The project for which we are inititializing the attribute category.
		 * @param {function} onReadyCallback - Function to be called when the attribute category object is ready 
		 * 		(it finishes initialization, loading data and it's rendered).
		 * @param {function} onFailCallback - Function to be called when an error occurs during initialization, 
		 * 		data loading or rendering.
		 */
		StaticDocumentRow.prototype._initializeAndRenderProjectCategoryObj = function (project, onReadyCallback, onFailCallback) {
			if (this.getParentDocTable().parentACatNotExpandedYet) {
				// no need to initialize this as user cannot see it anyway!
				return;
			}

			if (!project || !project.id || !project.categoryID) {
				DocTableXUtils.log("Attempt to initialize project attributes when project or attribute category ID is not known!");
				return;
			}

			var projACatElemID = this.projACatElemIDPrefix + project.id;

			if ($('#' + projACatElemID).length == 0) {
				// no place to render reference category (yet) - postponing its initialization
				return;
			}

			this.setStatus('loading');

			// current user object for the attribute cat.
			var currentUser = ObjectHandler.getCloneOfObject(this.currentUser);
			// set whether the attribute category should be in editable or readonly mode
			currentUser.canEditAttributes = this.canUserEditProperties();

			// setup all for the attribute category
			var aCatOptions = {};
			aCatOptions.prefix = this.projACatPrefix + project.id; // rowPrefix + '_ProjACat_' + projectid
			aCatOptions.ACatObj = this.projACatElemIDPrefix + project.id; // rowPrefix + '_ProjACatID' + projectid
			aCatOptions.ACatPanel = this.projACatElemIDPrefix + project.id; // rowPrefix + '_ACatPanelID' + projectid - THIS MAKES THE ACAT RENDER!!!
			aCatOptions.readURL = window.webcenterConfig.jspBaseUrl + 'GetAttributeCategoryXML.jsp?categoryid=' + project.categoryID + '&ajax=1';
			// just load attribute types, restricted sets etc. to correctly render them (along with 
			// empty attribute values)
			aCatOptions.valuesURL = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?projectid=' + project.id;
			aCatOptions.type = 'project';
			aCatOptions.projectID = project.id;

			aCatOptions.currentUser = currentUser;

			aCatOptions.aCatNotExpandedYet = this.aCatNotExpandedYet;
			
			// to distinguish this attribute category from document/project attr. categories in the attribute category itself!
			aCatOptions.isProjectInfoACat = true;

			// success and fail callbacks registration
			if (onReadyCallback) aCatOptions.callbackOnRender = onReadyCallback;
			if (onFailCallback) aCatOptions.callbackOnFail = onFailCallback;


			require(["attributecategory/AttributeCategory"], function (AttributeCategory) {
				// create the reference attribute category object here and store it!
				var newACatObj = new AttributeCategory(aCatOptions);
				if (this.aCatObj) {
					// destroy the previous attribute category (including its data model in memory with event listeners etc.)
					// this can happen as loading of AttributeCategory via required.js is asynchronous...
					this.aCatObj.destroy(true);
				}
				this.aCatObj = newACatObj;
				// (note attributeCategory constructor itself stores the aCatObj in the dedicated DOM object 
				// with id="<aCatOptions.ACatPanel>");

				// no onChange listeners as they are readonly
			}.bind(project));
		};

		/**
		 * Method generating URL to retrieve attributes (and their values) from the server.
		 * 
		 * To be overwritten in subclasses.
		 */
		StaticDocumentRow.prototype.generateDocAttributeValuesAJAXURL = function () {
			var url = null;
			if (this.docVerID) {
				url = window.webcenterConfig.jspBaseUrl + 'GetAttributes.jsp?docverid=' + this.docVerID;
			}
			return url;
		};



		/////////////////////////////////////////////////////////////////////////////////////
		// Controller methods =====================================


		/**
		 * Main method to show the attribute category of the document in this doc. table row.
		 * 
		 * @param {boolean} update - If true the attributeCategory object is recreated from scratch, otherwise
		 * 		the existing object is used. If no attr. cat object has been created yet, the new one is created 
		 * 		and stored in this instance (into 'this.aCatObj').
		 * @param {Boolean} expand - A three-value flag controlling the attribute category state (expanded/collapsed):
		 * 		- null or undefined: don't change the state of the attribute category (let it expanded if it's 
		 * 		  expanded and vice versa).
		 * 		- true: expand the attr. category with all its attributes in the predefined view
		 * 		- false: collapse the attr. category.
		 */
		StaticDocumentRow.prototype.showDocAttributeCategory = function (update, expand) {
			if (this.aCatExpandDisabled) {
				// 'expand' parameter is overruled by this.aCatExpandDisabled, which value comes from
				// rendering configuration of the expand/collapse (flapout) button
				expand = false;
			}
			if (this.docVerID && !this.aCatID) {
				// we have a doc. version ID (== already uploaded document), but the attribute category ID is 
				// not known - get it from the "docInfo" returned from the server
				this.docInfoRequest.done(function (docInfo) {
					var isACatChanged = false;
					if (!this.aCatID) {
						// update the this.aCatID and other members related to attribute category of current document
						// based on the response returned from the server (and internal logic), but only if it was not 
						// already set within a handler of a parallel asynchronous request
						isACatChanged = this.updateDocAttrCategoryFromValue(docInfo.catInfo);
					}
					// now we can render the attribute category, but only if it's changed or it's update/refresh is requested
					this._showDocAttributeCategory(update || isACatChanged, expand);
				}.bind(this));
				this.docInfoRequest.fail(function () {
					// something went wrong - hide the attribute category as it cannot be probably initialized correctly
					this.hideDocAttributeCategory();
				}.bind(this));

			} else if (this.hasAttrCat()) {
				// attribute category assigned to this document is known already - show it
				this._showDocAttributeCategory(update, expand);
			} else {
				// there is no attribute category currently assigned to the document 
				// (needed e.g. for DocumentRowBrowse functionality!)
				this.hideDocAttributeCategory();
			}
		};


		/**
		 * This method finds the expand/collapse (expColButton) button of this row in document table and return it.
		 * @returns {DOM object} - If the button is not found null is returned, otherwise the button.
		 */
		StaticDocumentRow.prototype.getExpColButton = function () {
			var expColButton = $('#' + this.getRowPrefix() + ' > ' + this.view.itemRootElemStr + ' > i.expandCollapseButton');
			return expColButton;
		};


		/**
		 * Private method that takes care of expanding the attribute category of this document in doc. table.
		 * The attribute category HTML needs to be prepared already!
		 * Nothing happens if the attr. category is already expanded.
		 * Use the public StaticDocumentRow.showDocAttributeCategory(false, true) to show the attr. category, 
		 * not this method directly.
		 * 
		 * @param {DOM object} expColButton - HTML button element that was clicked (caused the expand action)
		 */
		StaticDocumentRow.prototype._expandDocAttrCat = function (expColButton) {
			if (this.aCatExpandDisabled) {
				// don't allow expanding of attribute category if that function is disabled
				return;
			}
			if (!expColButton) {
				// find the expand/collapse button if it's not specified as parameter
				expColButton = this.getExpColButton();
			}
			if (this.dependsOnItself) {
				// if the document is dependent on itself, change the collapse button to "cyclic dependency" icon
				expColButton.removeClass('icon-WCR_Plus-square expandDepButton')
					.addClass('icon-WCR_Refresh cyclicDepButton')
					.attr('title', WCXML.xmlencode(DocRefStrings.t('DocRowCyclicDepTooltip')));
			} else {
				// toggle expand/collapse state of the button
				expColButton.removeClass('icon-WCR_Plus-square expandDepButton icon-WCR_Refresh cyclicDepButton')
					.addClass('icon-WCR_Minus-square collapseDepButton')
					.attr('title', WCXML.xmlencode(DocRefStrings.t('DocRowCollapseTooltip')));
			}
			// show the attribute category row
			$('#' + this.aCatRowID).removeClass('hiddenElement');

			// remember the state of the button in the data model
			this.aCatExpanded = true;

			if (this.aCatNotExpandedYet) {
				// remember the attribute category was expanded at least once already
				this.setACatIsExpanded();
			}
		};


		StaticDocumentRow.prototype.setACatIsExpanded = function () {
			this.aCatNotExpandedYet = false;
			if (this.aCatObj) {
				// update the info about expansion in the particular attribute category
				this.aCatObj.setACatIsExpanded();

				// the whole document row needs to be redrawn in this case (including the attribute cat. row)!
				this.displayRow();
			}
		};


		/**
		 * Private method that takes care of collapsing the attribute category of this document in doc. table.
		 * It does nothing if the attribute category doesn't exist or is already collapse.
		 * Use the public StaticDocumentRow.hideDocAttributeCategory() to hide the attr. category, 
		 * not this method directly.
		 * 
		 * @param {DOM object} expColButton - HTML button element that was clicked (caused the collapse action)
		 */
		StaticDocumentRow.prototype._collapseDocAttrCat = function (expColButton) {
			if (!expColButton) {
				// find the expand/collapse button if it's not specified as parameter
				expColButton = this.getExpColButton();
			}
			if (this.dependsOnItself) {
				// if the document is dependent on itself, change the expand button to "cyclic dependency" icon
				expColButton.removeClass('icon-WCR_Minus-square collapseDepButton')
					.addClass('icon-WCR_Refresh cyclicDepButton')
					.attr('title', WCXML.xmlencode(DocRefStrings.t('DocRowCyclicDepTooltip')));
			} else {
				// toggle expand/collapse state of the button 
				expColButton.removeClass('icon-WCR_Minus-square collapseDepButton icon-WCR_Refresh cyclicDepButton')
					.addClass('icon-WCR_Plus-square expandDepButton')
					.attr('title', WCXML.xmlencode(DocRefStrings.t('DocRowExpandTooltip')));
			}
			// hide the attribute category row
			$('#' + this.aCatRowID).addClass('hiddenElement');

			// remember the state of the button in the data model
			this.aCatExpanded = false;
		};


		/**
		 * Private method containing main logic governing initialization, rendering and expanding/collapsing of
		 * document's attribute category.
		 * 
		 * @param {Boolean} forceUpdate - Parameter controlling whether the attribute category will be
		 * 		force-updated (value 'true') no matter other circumstances or the decision will be left on
		 * 		the built in logic (value 'false'). Update an attribute category in this context means 
		 * 		to re-created it from scratch.
		 * @param {Boolean} expand - If this param. is 'true', the attribute category will be expanded (shown),
		 * 		otherwise it will be collapsed (not shown).
		 */
		StaticDocumentRow.prototype._showDocAttributeCategory = function (forceUpdate, expand) {
			if (forceUpdate || this.isAttrCatUpdateNeeded()) {
				// update means the full reinitialization of the child attribute category
				// (all the not saved changes in this category will be lost!)
				this.destroyACatObj(true);
			}

			// attribute category should be shown
			// first check whether there is anything to show:
			// - if document has an attribute category assigned and
			// - there is at least one attribute from that attr. cat. used/displayed taking into consideration
			//   actual rendering configuration
			if (!this.hasAttrCat() || !this.isAttrCatNeeded()) {
				// this document has no attribute category - hide the attribute category flap-out
				this.hideDocAttributeCategory();
				return;
			}

			// find the expand/collapse (expColButton) button
			var expColButton = this.getExpColButton();

			if (this.aCatExpandDisabled) {
				// make sure the expand/collapse (expColButton) button is not visible 
				expColButton.addClass('hiddenElement');
				if (this.aCatExpanded) {
					this.hideDocAttributeCategory();
				}
			} else {
				// make sure the expand/collapse (expColButton) button is visible as this document 
				// has attr. category assigned and the expanding of attribute category is not disabled
				expColButton.removeClass('hiddenElement');
			}
			
			if (this.dependsOnItself) {
				// second appearance of a cyclic dependent document - don't initialize and/or expand 
				// the attribute category! This prevents infinite loops when the same document and 
				// attr. cat. is used twice in a single branch!
				this._collapseDocAttrCat(expColButton);
				return;
			}

			// there is no cyclic dependency so far - it's safe to expand if needed
			if (expand || this.aCatExpanded) {
				this._expandDocAttrCat(expColButton);
			} else {
				this._collapseDocAttrCat(expColButton);
			}

			var isAttrRenderedInColumn = (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'DocumentAttribute') != null);

			if (this.aCatNotExpandedYet && !isAttrRenderedInColumn) {
				// if the attribute category has not been expanded yet and there are no inline document 
				// attributes (rendered in a column of the document row), 
				// there is no need to initialize or show the attribute category
				return;
			}

			if (this.aCatObj) {
				// we already have this.aCatObj object initialized, there were no change in attr. cat. ID
				// and there is no request of force update - make sure the 'aCatObj' is stored in 
				// the dedicated DOM object as expected by the attributeCategory implementation!
				var wasMoved = false;
				var aCatPanelObj = $('#' + this.aCatElemID);
				if (aCatPanelObj.length == 1) {
					if (aCatPanelObj[0].ACatObj != this.aCatObj) {
						aCatPanelObj[0].ACatObj = this.aCatObj;
						wasMoved = true;
					}
				}

				// redraw the whole attribute category only for complete redraws (when isMoved == true), 
				// not for expands and other operations that leaves attr. category HTML intact!
				if (wasMoved) {
					// make sure the 'isInactive' flag is updated!
					// (this flag is used to handle visibility and edit-ability of attribute values correctly)
					this.aCatObj.isInactive = this.isInactive();
					// render attribute category
					this.aCatObj.displayAttributeCategory();
				}
			} else {
				// new attribute category selected or no attr. cat. initialization done yet - fully 
				// (re)initialize the attribute category object

				// the constructor of the attribute category also renders the attr. category HTML, 
				// so do it only if there is already DOM object to render the attribute category to!
				// (the immediate rendering is also the reason why it's in here and not in 
				// updateDocAttrCategoryFromValue() or similar method.)
				this._initializeAndRenderAttrCategoryObj(
					this._onAttrCategoryReady.bind(this, 'docCat'),
					this._onAttrCategoryLoadFailed.bind(this));
			}
		};


		/**
		 * Private method to load the document reference properties attribute category with defined callbacks, but 
		 * without attribute (property) values! The values are supplied by 'this.props' member.
		 * This attribute category is never shown as a whole, so the expand/collapse logic is not needed here.
		 * 
		 * @param {Boolean} update - Param. controlling whether the attribute category should be re-created 
		 * 		from scratch (value 'true') or the already initialized attr. category should be used (value 'false').
		 */
		StaticDocumentRow.prototype._prepareReferenceCategoryObj = function (update) {
			if (!this.propsACatID) {
				DocTableXUtils.log("StaticDocumentRow._prepareReferenceCategoryObj() called when no reference category ID known (or it was nullified)"); //JANH debug - then DEL!!!
				return;
			}

			if (update) {
				// update means the full reinitialization of the properties attribute category
				// (all the not saved changes in this category will be lost!)
				this.destroyPropsACatObj(true);
			}
			if (this.propsACatObj) {
				// we already have this.propsACatObj object initialized, there were no change in attr. 
				// cat. ID and there is no request of force update - make sure the 'propsACatObj' is 
				// stored in the dedicated DOM object as expected by the attributeCategory implementation!
				var wasMoved = false; //JANH TODO - potential fix of WCR-13177 - then DEL!!!
				var aCatPanelObj = $('#' + this.propsACatElemID);
				if (aCatPanelObj.length == 1) {
					if (aCatPanelObj[0].ACatObj != this.propsACatObj) {
						aCatPanelObj[0].ACatObj = this.propsACatObj;
						wasMoved = true; //JANH TODO - potential fix of WCR-13177 - then DEL!!!
					}
				}
				/*//JANH TODO - potential fix of WCR-13177 (and erratic behavior of reference properties rendering) - then DEL!!!								
								// the reference category should not be displayed (rendered) here as it's not used for anything.
								// just the fully initialized attr. cat. object has to stay where the attr. cat. code expects it...
				*/
				// redraw the whole attribute category only for complete redraws (when isMoved == true), 
				// not for expands and other operations that leaves attr. category HTML intact!
				if (wasMoved) {
					// render attribute category
					this.propsACatObj.displayAttributeCategory(); //JANH TODO - potential fix of WCR-13177 - then DEL!!!
				}
			} else {
				// new attribute category selected or no attr. cat. initialization done yet - fully 
				// (re)initialize the attribute category object.

				// the constructor of the attribute category also renders the attr. category HTML, 
				// so do it only if there is already DOM object to render the attribute category to!
				// (the immediate rendering is also the reason why it's in here and not in 
				// updateDocAttrCategoryFromValue() or similar method.)
				this._initializeAndRenderReferenceCategoryObj(
					this._onAttrCategoryReady.bind(this, 'refPropsCat'),
					this._onAttrCategoryLoadFailed.bind(this));
			}
		};

		/**
		 * Private method to load the project attribute category with defined callbacks, but 
		 * without attributevalues!
		 * This attribute category is never shown as a whole, so the expand/collapse logic is not needed here.
		 * 
		 * @param {Object} project - The project to prepare the attribute category of.
		 * @param {Boolean} update - Param. controlling whether the attribute category should be re-created 
		 * 		from scratch (value 'true') or the already initialized attr. category should be used (value 'false').
		 */
		StaticDocumentRow.prototype._prepareProjectCategoryObj = function (project, update) {
			if (!project.categoryID) {
				DocTableXUtils.log("StaticDocumentRow._prepareProjectCategoryObj() called when no attribute category ID known (or it was nullified)"); //JANH debug - then DEL!!!
				return;
			}

			if (update) {
				// update means the full reinitialization of the properties attribute category
				// (all the not saved changes in this category will be lost!)
				this.destroyProjACatObj(project, true);
			}
			if (project.aCatObj) {
				// we already have aCatObj object initialized, there were no change in attr. 
				// cat. ID and there is no request of force update - make sure the 'aCatObj' is 
				// stored in the dedicated DOM object as expected by the attributeCategory implementation!
				var wasMoved = false; //JANH TODO - potential fix of WCR-13177 - then DEL!!!
				var aCatPanelObj = $('#' + this.projACatElemIDPrefix + project.id);
				if (aCatPanelObj.length == 1) {
					if (aCatPanelObj[0].ACatObj != project.aCatObj) {
						aCatPanelObj[0].ACatObj = project.aCatObj;
						wasMoved = true; //JANH TODO - potential fix of WCR-13177 - then DEL!!!
					}
				}
				/*//JANH TODO - potential fix of WCR-13177 (and erratic behavior of reference properties rendering) - then DEL!!!								
								// the project category should not be displayed (rendered) here as it's not used for anything.
								// just the fully initialized attr. cat. object has to stay where the attr. cat. code expects it...
				*/
				// redraw the whole attribute category only for complete redraws (when isMoved == true), 
				// not for expands and other operations that leaves attr. category HTML intact!
				if (wasMoved) {
					// render attribute category
					project.aCatObj.displayAttributeCategory(); //JANH TODO - potential fix of WCR-13177 - then DEL!!!
				}
			} else {
				// new attribute category selected or no attr. cat. initialization done yet - fully 
				// (re)initialize the attribute category object.

				// the constructor of the attribute category also renders the attr. category HTML, 
				// so do it only if there is already DOM object to render the attribute category to!
				// (the immediate rendering is also the reason why it's in here and not in 
				// updateDocAttrCategoryFromValue() or similar method.)
				this._initializeAndRenderProjectCategoryObj(project,
					this._onAttrCategoryReady.bind(this, 'projInfoCat'),
					this._onAttrCategoryLoadFailed.bind(this));
			}
		};

		
		/**
		 * Callback method to perform required steps after an attribute category is loaded and rendered on the page
		 * (but not necessarily shown).
		 * For reference category it also updates the attribute values by corresponding property values.
		 * 
		 * @param {String} categoryUse - Value defining type/use of just loaded attribute category.
		 * 		It supports the following values: 
		 * 		- 'docCat' for attribute category assigned to the referenced document, 
		 * 		- 'refPropsCat' for reference properties attribute category,
		 * 		- 'projInfoCat' for project attribute category used to display project info inline in document row.
		 */
		StaticDocumentRow.prototype._onAttrCategoryReady = function (categoryUse) {
			var repaintNeeded = true;
			if ((categoryUse == 'refPropsCat') && this.isAttrCatLoaded(this.propsACatObj)) {
				// reference properties attribute category
				if (!this.propsValuesInitialized) {
					// for reference properties attribute category set values of all its attributes 
					// representing a property of this document to values stored in 'this.props' member 
					// (values of the properties)
					// note: reference category is initialized without loading its attribute values as they 
					// are stored on dedicated place
					for (var i = 0; i < this.props.length; i++) {
						var prop = this.props[i];
						if (prop && prop.id) {
							// having a valid property - find its corresponding attribute in the reference category
							var propAttObj = this.propsACatObj.findAttribute(prop.id, prop.name);
							if (propAttObj) {
								// it is found - copy the value to be same as the property value
								propAttObj.copyValueFromDocRefProp(prop);
							}
						}
					}
					// make sure the "values initialization" is done only once and not during subsequent
					// repaints of the reference attribute category!
					this.propsValuesInitialized = true;
				}
				/*//JANH TODO - commented out for now to test if this is causing WCR-13177 and erratic behavior of reference properties rendering!
								// delete all HTML generated by reference category as it's never shown and not important in any other way.
								// this is also a big optimization of the page HTML size as the reference category is there for each row!
								// the 'this.propsACatElemID' element (attribute category's aCatPanel) together with its 'aCatObj' property
								// has to stay on the page to not break any attr. cat. functionality...
								this.destroyPropsACatObj();
								$('#' + this.propsACatElemID).html('');
				*/
			} else if ((categoryUse == 'docCat') && this.isAttrCatLoaded(this.aCatObj)) {
				if (!this.aCatNotExpandedYet && this.aCatObj.aCatNotExpandedYet) {
					// the 'this.aCatNotExpandedYet' flag was probably set when this.aCatObj didn't 
					// exist yet - make sure the attribute category object is updated to be in sync.
					this.setACatIsExpanded();
					// this.setACatIsExpanded() contains call to display the whole row
					repaintNeeded = false;
				}
			}

			// set status to ready only for this class, not subclasses as they can have more init work to do!
			this.setStatus('ready');

			// update/repaint the row with the loaded data, but only if needed
			if (repaintNeeded) {
				this.view.repaintRow();
			}
		};


		/**
		 * Callback method handling errors that might occur during attribute category initialization or rendering.
		 */
		StaticDocumentRow.prototype._onAttrCategoryLoadFailed = function () {
			this.setStatus('error');
			// no need to alert the error message as the document row rendering is altered based on the error status!
			DocTableXUtils.log("Failed to load attribute category.");
		};



		/**
		 * Method to hide the attribute category of this document (in this document row).
		 * If there is no attribute category currently assigned to the document, the HTML in placeholder for
		 * attr. category is deleted too!
		 */
		StaticDocumentRow.prototype.hideDocAttributeCategory = function () {
			var expColButton = this.getExpColButton();
			// collapse the document's attribute category (if any)
			this._collapseDocAttrCat(expColButton);

			if (!this.hasAttrCat()) {
				// hide the expand/collapse (expColButton) button too as this document has no attr. category
				// either loaded yet or assigned at all
				expColButton.addClass('hiddenElement');
				var aCatPanel = $('#' + this.aCatElemID);
				// delete 'ACatObj' custom property, where the old attributeCategory object was stored (if any) and also
				// all HTML remnants of the old attribute category 
				if (this.aCatObj && (this.aCatObj !== aCatPanel.ACatObj)) {
					// be sure to delete all instances of attribute category that might exist
					// (this.aCatObj and aCatPanel.ACatObj might not be in sync all the time!)
					this.aCatObj.destroy(true);
				}
				if (aCatPanel.ACatObj) {
					aCatPanel.ACatObj.destroy(true);
				}
				this.aCatID = null;
				this.aCatObj = null;
				aCatPanel.ACatObj = null;
				aCatPanel.html('');
			}
		};


		/**
		 *  Method to collapse all descendants of the current document (document row).
		 *  It cooperates with DocTableX and AttributeCategory objects to achieve that.
		 *  The collapsion is done from the leafs to the root (of the doc. ref. tree-like structure).
		 */
		StaticDocumentRow.prototype.collapseAllDescendants = function () {
			if (this.aCatObj) {
				this.aCatObj.executeOnAllDocReferences(function (docTableObj) {
					// 'docTableObj' is always the "current" documents table object!
					docTableObj.collapseAllRows();
				}, this);
			}
			// collapse the document's attribute category (if any)
			this._collapseDocAttrCat();
		};


		/**
		 * Method to select all descendants of the current document (document row).
		 * It cooperates with DocTableX and AttributeCategory objects to achieve that.
		 * The selection is done from the leafs to the root (of the doc. ref. tree-like structure).
		 *  
		 * @param {Boolean} isSelected - Flag defining whether the document rows should be selected or deselected.
		 */
		StaticDocumentRow.prototype.selectAllDescendants = function (isSelected) {
			if (this.aCatObj && this.aCatExpanded) {
				// if there is an expanded child attribute category, go over all its document reference 
				// attributes and select their document rows as well!
				this.aCatObj.executeOnAllDocReferences(function (docTableObj) {
					// 'docTableObj' is always the "current" documents table object!
					docTableObj.setSelectAllRows(isSelected);
				}, this);
			}
			// don't forget to (de)select this document row
			this.setIsSelected(isSelected, true);
		};


		/**
		 * Method to remove the HTML related to this document row optionally also removing its placeholder(s).
		 * It does NOT update the save XML, so that needs to be handled by the caller of this method.
		 */
		StaticDocumentRow.prototype.deleteItself = function (leaveRowPlaceHolder) {

			// first get all deleted rows == the row itself and all its descendants
			var allDeletedRows = this.getAllDescendantRows();

			// deal with dependencies
			var copiedRows = this.getAllRowsCopiedFromRows(allDeletedRows);
			for (var ri = 0; ri < allDeletedRows.length; ri++) {
				var actDelRow = allDeletedRows[ri];

				// handle dependencies of rows copied from this row
				var actCopiedRows = _.filter(copiedRows, function (row) {
					return row.copyOfExistingRowID == actDelRow.rowID;
				});

				if (actCopiedRows.length > 0) {
					// we are going to delete a master (document row from which others were copied).
					// make a new "master" and point the rest of copied rows to the new "master".
					var newMaster = actCopiedRows[0];
					// be careful that the master itself can be copied from another document row!
					newMaster.copyOfExistingRowID = actDelRow.copyOfExistingRowID;
	
					// in case there are items not possible to recreate in the "master" row, they need
					// to be "salvaged" (moved to the new "master" row)
					newMaster.salvageValuablesFromDocRow(actDelRow);
					
					for (var i = 1; i < actCopiedRows.length; i++) {
						actCopiedRows[i].copyOfExistingRowID = newMaster.rowID;
					}
				}
	
				// handle dependencies of rows cloned from this row
				var actClonedRows = actDelRow.getAllClonesOfThisRow();
				if (actClonedRows.length > 0) {
					// we are going to delete a master (document row from which others were cloned).
					// make a new "master" and point the rest of cloned rows to the new "master".
					var newMaster = actClonedRows[0];
					// be careful that the master itself can be clone of another document row!
					newMaster.clonedFrom = actDelRow.clonedFrom;
					
					// in case there are items not possible to recreate in the "master" row, they need
					// to be "salvaged" (moved to the new "master" row)
					newMaster.salvageValuablesFromDocRow(actDelRow);
	
					for (var i = 1; i < actClonedRows.length; i++) {
						actClonedRows[i].clonedFrom = newMaster;
					}
				}
			}
			
			// destroy all associated data structures (like categories, rich text editors etc.) recursively
			this.destroy(true);

			// remove HTML code of document row and associated attribute category row (if any)
			if (leaveRowPlaceHolder) {
				$('#' + this.view.phElemID).empty();
			} else {
				$('#' + this.view.phElemID).remove();
			}
			$('#' + this.aCatRowID).remove();
			if (this.view._newVersionRow) {
				this.view._newVersionRow.remove();
			}
		};

		/**
		 * Generic method to "salvage" any items in the specified 'srcRow' that cannot be recreated
		 * (as file input field with a file selected by user) and transfer them to 'this' document row.
		 * It's meant to be overridden by subclass of StaticDocumentRow, where necessary.
		 * 
		 * @param {StaticDocumentRow or its subclass} srcRow - Row from where to move the "salvaged" items.
		 */
		StaticDocumentRow.prototype.salvageValuablesFromDocRow = function (srcRow) {
			// poor StaticDocument row has no valuables
		};

		
		/**
		 * Method to swap two document rows. On of them is this row and the second is specified via 'otherRow' 
		 * parameter. 
		 * It swaps the HTML related the two rows (including their placeholders), repaints the rows to
		 * make sure their visual representation is updated and finally it updates 
		 * the save XML too!
		 * 
		 * @param {StaticDocumentRow or subclass} otherRow - Object representing the second document row
		 * 		to swap this with.
		 */
		StaticDocumentRow.prototype.swapWith = function (otherRow) {
			// special handling of rich text attributes (TinyMCE cannot be just moved in the DOM without consequences)
			if (this.aCatObj) {
				this.aCatObj.beforeMove();
			}
			
			// swap this document row and associated attribute category row with the 'otherRow' and its
			// attribute category row
			// note at least the placeholder element for attribute category is always there 
			// (even if the document has no attr. category assigned)
			var thisEntry = $('#' + this.view.phElemID).parent();
			var otherEntry = $('#' + otherRow.view.phElemID).parent();
			thisEntry.insertAfter(otherEntry);

			// repaint both effected rows and update the save XML too! (note both rows are from the same doc. table)
			// 'this' document row has to be fully redrawn as it was moved!
			this.displayRow();
			otherRow.view.repaintRow();

			// special handling of rich text attributes (TinyMCE cannot be just moved in the DOM without consequences)
			if (this.aCatObj) {
				this.aCatObj.afterMove();
			}
		};


		/**
		 * This method updates this.dependsOnItself member, which determines whether this document row
		 * is cyclic dependent on one of its parents or not.
		 * If this is the case and it's dependent a lot of document row functionality is limited or disabled.
		 * 
		 * @param {Boolean} isDependent - True if this document should be set to cyclic dependency mode,
		 * 		false if it's not the case.
		 */
		StaticDocumentRow.prototype.setDependsOnItself = function (isDependent) {
			this.dependsOnItself = (isDependent == true);
		};



		// Save (XML generation) and validation methods =====================================


		/**
		 * Method generating XML to save changes in properties of the document's reference (document row).
		 * This method has to be called for every document row that can potentially contain reference properties!
		 * 
		 * @returns {String} - XML of reference properties and its data prefix ('refprefix').
		 */
		StaticDocumentRow.prototype.convertRefPropertiesToXML = function () {
			var xmlValue = '';
			try {
				if (this.props && (this.props.length > 0)) {
					// generate XML for properties even in read-only case as back-end currently requires it!
					xmlValue += '<refatts>';
					for (var propIdx = 0; propIdx < this.props.length; propIdx++) {
						var prop = this.props[propIdx]; // actual "property"
						if (prop && prop.id) {
							xmlValue += '<att>';
							xmlValue += WCXML.ElementEncode('id', prop.id);
							xmlValue += '</att>';
						}
					}
					xmlValue += '</refatts>';

					// prefix of the hidden input field, where the XML of all document reference properties is stored.
					// this is to allow server to find the input fields of the properties of given document reference
					xmlValue += WCXML.ElementEncode('refprefix', this.propsACatPrefix);
				}
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRow.convertRefPropertiesToXML:\n' + err);
				xmlValue = '';
			}
			return xmlValue;
		};

		/**
		 * Method generating XML to save uploading a new version
		 * This method has to be called for every document row in which a new version can be uploaded
		 * 
		 * @returns {String} - XML of reference properties and its data prefix ('refprefix').
		 */
		StaticDocumentRow.prototype.convertUploadNewVersionToXML = function () {
			var xmlValue = '';
			if (this.view.newVersionBtn && this.view.newVersionBtn.isExpanded && this.view._newVersionCell) {
				var root = this.view._newVersionCell;
				xmlValue += '<ds>';
				xmlValue += WCXML.ElementEncode('action', 'UploadNewVersion');
				xmlValue += WCXML.ElementEncode('file_form_field', 'newVersionDocFile_' + this.docVerID);
				xmlValue += WCXML.ElementEncode('version_comment', root.find("input[name=newVersionDocDesc_" + this.docVerID + "]").val());
				xmlValue += WCXML.ElementEncode('raise_revision', (root.find("input[name=incrementRevision_" + this.docVerID + "]").attr("checked") == "checked") ? '1' : '0');
				xmlValue += WCXML.ElementEncode('update_all_projects', (root.find("input[name=updateAllProjects_" + this.docVerID + "]").attr("checked") == "checked") ? '1' : '0');
				if ($("input[name=AppCycleFlag" + this.docVerID + "]").length > 0) {
					xmlValue += WCXML.ElementEncode('start_approval', (root.find("input[name=AppCycleFlag" + this.docVerID + "]").attr("checked") == "checked") ? '1' : '0');
				} else {
					xmlValue += WCXML.ElementEncode('start_approval', (root.find("input[name=startApprovalDefaultValue" + this.docVerID + "]").val()));
				}
				// load the due date controls
				if (root.find("input[name=due_date" + this.docVerID + "]").val() != undefined ){
					xmlValue += WCXML.ElementEncode('approval_due_date', root.find("input[name=due_date" + this.docVerID + "]").val());					
				}
				if (root.find("select[name=due_periodOfDay" + this.docVerID + "]").val() != undefined){
					xmlValue += WCXML.ElementEncode('approval_due_period', root.find("select[name=due_periodOfDay" + this.docVerID + "]").val());					
				}
			
				xmlValue += '</ds>';
			}
			return xmlValue;
		};


		/**
		 * Method generating XML to save changes in this document table row.
		 * This method is dedicated for document reference attributes only!
		 * Note this method assumes the inactive state (this.isInactive()) is handled correctly by the caller!
		 * 
		 * @param {String} dsXML - Document Source XML == content of <ds> tag (if any).
		 * @returns {String} - XML complying to JavaScript -> JSP interface to save changes done in document table.
		 */
		StaticDocumentRow.prototype.convertToXML = function (dsXML) {
			var xmlValue = '';
			try {
				// generate the <dr> with basic info even if there is no permission / an error related 
				// to this document row! Otherwise the row will be deleted during save operation on back-end!
				// moreover the properties can be changed and saved even if the user has no permissions to
				// the referenced document itself...

				xmlValue += '<dr>';
				if (this.drID) xmlValue += WCXML.ElementEncode('id', this.drID);
				if (dsXML && (dsXML.length > 0)) {
					xmlValue += dsXML;
				}
				if (this.docVerID) {
					xmlValue += WCXML.ElementEncode('dv', this.docVerID);
					if (this.docVerIdx) xmlValue += WCXML.ElementEncode('dvidx', this.docVerIdx);
					if (this.docID) xmlValue += WCXML.ElementEncode('doc_id', this.docID);
					if (this.docName) xmlValue += WCXML.ElementEncode('dn', this.docName);
				}

				if (this.appStatus) {
					if (this.appStatus.cycleStatus) xmlValue += WCXML.ElementEncode('astat', this.appStatus.cycleStatus);
				}

				// for editable document rows generate new document version and properties XML too (if applicable)
				if (this.isEditable || this.getIsNewDocVersionUploaded()) {
					// note a new document version can be uploaded even if the documents table is not editable as long as
					// the user has "Update/Lock" permissions, "Upload new version" button is configured and document reference
					// attribute is set to "specified revision" or "latest version"
					// render XML of upload new version form
					xmlValue += this.convertUploadNewVersionToXML();
				}
				if (this.isEditable) {
					// render XML of reference properties (if any)
					xmlValue += this.convertRefPropertiesToXML();
				}

				// prefix of the hidden input field, where the XML of all attributes of the referenced document
				// attribute category is stored.
				// this is to allow server to find the input fields of the descendant document references
				if (!this.isInErrorStatus()) {
					// generate <daprefix> even if read-only as the descendants of this document row
					// can be editable and this is the way how the back-end can find them!
					xmlValue += WCXML.ElementEncode('daprefix', this.aCatPrefix);
				}
				xmlValue += '</dr>';
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRow.convertToXML:\n' + err);
				xmlValue = '';
			}
			return xmlValue;
		};



		/**
		 * Validation of the document row content versus required constraints.
		 * 
		 * @param {Object} validObj - Validation object holding all found validation errors together with the related 
		 * 		data needed to process them and notify user. 
		 * 		Its structure is described in attributeCategory.validateAttributes() method, where it's created.
		 * @param {int} depth - Depth of validation recursion within the document reference tree-like structure
		 * 		starting from 1.
		 * 
		 * @returns {Boolean} - Returns true if no problem found in this particular document row and 
		 * 		all its descendants (if an attribute category is assigned to this document), false otherwise.
		 */
		StaticDocumentRow.prototype.validateDocRow = function (validObj, depth) {
			var isOK = true;

			if (this.isInactive()) {
				// hidden document rows are not validated
				return isOK;
			}

			if (this.isAttrCatLoaded(this.aCatObj)) {
				// array of attribute IDs to be skipped in the attributeCategory.validateAttributes() method
				var ignoreList = [];

				if (this.canUserEditAttributes() && !validObj.isTemplate) {
					// check document attributes defined in table rendering (in dedicated columns) with required flag
					var rocList = this.renderOptions.columns;
					if (rocList) {
						for (var i = 0; i < rocList.length; i++) {
							var roc = rocList[i]; // Render Option Configuration
							if (roc && roc.type && (roc.type == 'DocumentAttribute')) {
								// found a document attribute displayed in table column
								var attrID = roc.typeDetail;
								// add it immediately to 'ignoreList' as it should be skipped in the validation
								// of the document's attribute category in any case now
								ignoreList.push(attrID);
								if (roc.visibility == 'Required') {
									// it's required in the column so no matter what is its settings in the
									// document's attribute category itself, check it as required
									var attObj = this.aCatObj.findAttribute(attrID, roc.attributeName);
									if (attObj && !attObj.value) {
										// required attribute has empty value - compose the errorObj object
										// referring to that situation
										var errorObj = {
											level: depth,
											attrLabel: this.aCatObj.getAttributeLabel(attObj),
											element: $('#' + this.aCatObj.prefix + attObj.id).closest("td"),
											docRowObj: this
										};
										validObj.errors.push(errorObj);
										isOK = false;
									} else {
										// if the attribute is in documents table configuration, but it's not
										// in the document's attribute category, it's a configuration issue
										// and it's OK to skip such attributes here in validation
									}
								}
							} // end of: if (roc && roc.type && ...
						}
					}
				} // end of: if (this.canUserEditAttributes()) {

				// validate assigned attribute category even if it's read-only! There is a chance it contains
				// document references that points to documents with editable attribute categories...
				// attributeCategory.validateAttributes() method has built in optimization to process
				// these cases:
				// execute the full validation of the descendant attribute category of this document (row)
				// attributes already checked in 'if (this.canUserEditAttributes())' block are skipped
				if (!this.aCatObj.validateAttributes(false, validObj, ignoreList, this, depth + 1)) {
					isOK = false;
				}
			}

			// check reference properties with required flag (if any)
			if (this.canUserEditProperties() && !validObj.isTemplate) {
				if (this.props && this.isAttrCatLoaded(this.propsACatObj)) {
					for (var i = 0; i < this.props.length; i++) {
						var prop = this.props[i];
						if (prop && prop.id && prop.isRequired) {
							// a required property found - check its actual value (in the reference category)
							var propAttObj = this.propsACatObj.findAttribute(prop.id, prop.name);
							if (propAttObj && !propAttObj.value) {
								// required BOM count has empty value - compose the errorObj object referring to that situation
								var errorObj = {
									level: depth,
									msg: DocRefStrings.t('DocRowPropertyRequiredButNull', prop.name),
									element: $('#' + this.propsACatObj.prefix + propAttObj.id).closest('td'),
									docRowObj: this
								};
								validObj.errors.push(errorObj);
								isOK = false;
							} else {
								// missing and/or not loaded required properties are not considered blocking
							}
						}
					}
				}
			}

			// check new version fields (if they exist)
			if (this.view.newVersionBtn) {
				var errors = this.view.newVersionBtn.validate();
				if (errors && errors.length > 0) {
					// add docrow specific information to errors and set isOK to false
					for (var i = 0; i < errors.length; i++) {
						var errorObj = errors[i];
						errorObj.level = depth;
						errorObj.docRowObj = this;
						validObj.errors.push(errorObj);
					}
					isOK = false;
				}
			}

			return isOK;
		};


		/**
		 * This method auto-expands the document reference tree-like structure so this document row 
		 * (and associated attribute category if any) together with all its parents are visible.
		 * It's particularly handy for validation where all the found problems are shown to the user
		 * even in the collapsed attribute categories.
		 * 
		 * @param {int} maxDepth - The maximal number of levels from the root the expansion should be 
		 * 		performed on. It's usefull in cases you need to limit the depth of expansion. For example
		 * 		when the validation founds a problem in a documents table column, but not in the child 
		 * 		attribute category, then the attr. cat needn't to be expanded.
		 */
		StaticDocumentRow.prototype.expandBranchTillThis = function (maxDepth) {
			var phElemID = this.getRowPrefix();
			var items = phElemID.split('_');

			// the generated IDs follows the logic of the ID generation in attr. categories employed by documents table(s). 
			// An example for the 1st, 2nd and 3rd level deep:
			// 1. docTableElemID: '00002_0000000931_docTable'
			// 1. parentID: ''
			// 2. docTableElemID: '00002_0000000931_6_ACat_00002_0000000717_docTable'
			// 2. parentID: '00002_0000000931_6_ACat_'
			// 3. docTableElemID: '00002_0000000931_6_ACat_00002_0000000717_16_ACat_00002_0000000713_docTable'
			// 3. parentID: '00002_0000000931_6_ACat_00002_0000000717_16_ACat_'

			var level = 1;
			var idx = 0;
			var parentID = '';
			while ((idx + 2 < items.length) && (level < maxDepth)) {
				var elemID = items[idx] + '_' + items[++idx];
				var docTableElemID = parentID + elemID + '_docTable'; // 00002_0000000931_docTable
				var rowID = items[++idx];

				var docTableElem = $('#' + docTableElemID);
				if (docTableElem.length == 1) {
					var docTableObj = docTableElem[0].docTable;
					if (docTableObj) {
						var rowObj = docTableObj.getRowByID(rowID);
						if (rowObj) {
							// perform the expansion of the attribute category assigned to the 'rowObj' document row
							rowObj.showDocAttributeCategory(false, true);
						}
					}
				}
				parentID += elemID + '_' + rowID + '_ACat_';
				idx += 2; // skip the '_ACat_' and similar...
				level++;
			}
		};


		/**
		 * Private method propagating "on submit" event to the descendant attribute categories.
		 */
		StaticDocumentRow.prototype._onSubmit = function () {
			if (this.aCatObj && !this.isInactive()) {
				this.aCatObj.convertDocReferencesToXML();
			}
		};



		// Action handler methods =====================================


		/**
		 * Action handler for expand/collapse button click.
		 * Based on the actual button (it's class determining also it's appearance) the method decides if the 
		 * attribute category row should be expanded or callapsed. 
		 * (Note there is no such a button visible in UI if this document row has no attr. category.)
		 * 
		 * @param {DOM object} elem - The HTML element representing the expand/collapse button.
		 */
		StaticDocumentRow.prototype.expandClicked = function (elem) {
			if (!elem) return;
			var buttonElem = $(elem);

			// find out whether the click action means "expand" or "collapse" (or nothing)
			if (buttonElem.hasClass('cyclicDepButton')) {
				// ignore actions on a cyclic dependent document.
				return;
			} else if (buttonElem.hasClass('expandDepButton')) {
				// we have an expand button so we are going to expand, but not repaint (optimization)
				this.showDocAttributeCategory(false, true);
			} else if (buttonElem.hasClass('collapseDepButton')) {
				// we have a collapse button so we are going to expand
				this.hideDocAttributeCategory();
			}
		};

		/**
		 * Action handler for checkbox in the selection column.
		 * 
		 * @param {boolean} value - A value representing boolean state of the checkbox.
		 */
		StaticDocumentRow.prototype.selectRowClicked = function (elem) {
			this.setIsSelected(elem && elem.checked);
			this.getParentDocTable().updateAllRowsSelector();
		};

		/**
		 * Method called before the DocTableRow elements are thrown out of the DOM.
		 * 
		 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 * 
		 */
		StaticDocumentRow.prototype.destroy = function (destroyACatObj) {
			this.destroyACatObj(destroyACatObj);
			this.destroyPropsACatObj(destroyACatObj);
			this.destroyProjACatObjAll(destroyACatObj);
			this.view.destroy(destroyACatObj);
		};

		/**
		 * Properly destroys the attribute category object associated with this document row before 
		 * it is thrown out of the DOM.
		 * 
		 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
		StaticDocumentRow.prototype.destroyACatObj = function (destroyACatObj) {
			if (this.aCatObj) {
				this.aCatObj.destroy(destroyACatObj);
			}
			if (destroyACatObj) {
				this.aCatObj = null;
			}
		};

		/**
		 * Properly destroys the doc. reference properties attribute category object associated 
		 * with this document row before it is thrown out of the DOM.
		 * 
		 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
		StaticDocumentRow.prototype.destroyPropsACatObj = function (destroyACatObj) {
			if (this.propsACatObj) {
				this.propsACatObj.destroy(destroyACatObj);
			}
			if (destroyACatObj) {
				this.propsACatObj = null;
			}
		};

		/**
		 * Properly destroys the project attribute category object associated 
		 * with this document row before it is thrown out of the DOM.
		 * 
		 * @param {Object} project - The project to destroy the acat obj of.
		 * @param {Boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
		StaticDocumentRow.prototype.destroyProjACatObjAll = function (destroyACatObj) {
			var projList = this.projects;
			if (projList && (projList.length > 0)) {
				for (var i = 0; i < projList.length; i++) {
					var proj = projList[i];
					if (!proj) {
						continue;
					}
					this.destroyProjACatObj(proj, destroyACatObj);
				}
			} else if (this.project) {
				this.destroyProjACatObj(this.project, destroyACatObj);
			}
		};

		/**
		 * Properly destroys the project attribute category object associated 
		 * with this document row before it is thrown out of the DOM.
		 * 
		 * @param {Object} project - The project to destroy the acat obj of.
		 * @param {Boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
		StaticDocumentRow.prototype.destroyProjACatObj = function (project, destroyACatObj) {
			if (project.aCatObj) {
				project.aCatObj.destroy(destroyACatObj);
			}
			if (destroyACatObj) {
				project.aCatObj = null;
			}
		};

		/**
		 * Reconstruct the Document Row elements
		 */
		StaticDocumentRow.prototype.reconstruct = function () {
			if (this.aCatObj) {
				this.aCatObj.reconstruct();
			}
		};

		/**
		 * Get the view of the AttributeCategory that should be shown
		 * @returns {String}
		 */
		StaticDocumentRow.prototype.getAttCatView = function () {
			return this.aCatView || 'Default';
		};

		return StaticDocumentRow;
	});