define(["doctablex/DocTableXUtils",
		"utils/WCEvent",
		"wcr/webcenter_xml",
		"wcr/webcenter_html",
		"jquery",
		"underscore",
		"component/actionbuttons/UploadNewVersionButton",
		"component/actionbuttons/ViewButton",
		"component/actionbuttons/DownloadButton",
		"component/actionbuttons/Open3DButton",
		"component/actionbuttons/OpenACADButton",
		"component/contentoverflow/OverflowContentContainer",
		"component/table/DropdownOverflowUtil",
		"utils/ColorUtils",
		"utils/webi18n!DocRefStrings",
		"wcr/extpopup",
		"wcr/approvalInfoPopup"],
	function (DocTableXUtils,
		WCEvent,
		WCXML,
		WCHTML,
		$,
		_,
		UploadNewVersionButton,
		ViewButton,
		DownloadButton,
		Open3DButton,
		OpenACADButton,
		OverflowContentContainer,
		DropdownOverflowUtil,
		ColorUtils,
		DocRefStrings) {

		/**
		 * StaticDocumentRowView.js
		 * 
		 * This class implements reusable rendering methods for all document rows.
		 * 
		 * @author JANH - January 2015
		 */

		"use strict";

		/**
		 * Constructor for the StaticDocumentRowView class.
		 * 
		 * @param {StaticDocumentRow object} dataModel - Object representing data model for this View class.
		 * @param {String} phElementID - ID of HTML DOM element, where to render this View.
		 * @param {Object} options - Object containing property-value pairs needed for initialization of this object.
		 * @returns {StaticDocumentRowView object} - Returns the created (and initialized) instance of this class.
		 */
		var StaticDocumentRowView = function (dataModel, phElementID, options) {
			if (!dataModel && !phElementID && !options) {
				// handle the inheritance definition in JavaScript: childClass.prototype = new parentClass();
				// the childClass constructor is responsible for calling the constructor of superclass too!!!
				return this;
			}

			/** @member {StaticDocumentRowView (or a subclass)} dm - Data model to be rendered. */
			this.dm = dataModel;
			if (!this.dm) {
				window.alert("Cannot initialized view without data model.");
				return null;
			}

			/** @member {jQery object} phElem - jQuery object representing DOM element, where the the content 
			 * 		should be rendered. */
			this.phElemID = phElementID;


			/** {int} rowIndex - Calculated index of this row in the parent documents table. It's to optimize
			 * 		rendering by precalculating this index. The calculation is done at the start of each rendering. */
			this.rowIndex = -1;

			/** @member {Object} renderOptions - This object holds properties controlling the rendering. */
			this.renderOptions = options;
			if (!this.renderOptions) {
				// if no (rendering) options specified, get the rendering options from data model, which are 
				// by default documents table rendering options. This is to make sure the rendering of this 
				// document row is in line with the documents table header etc...
				this.renderOptions = this.dm.renderOptions;
			}

			/** @member {Boolean} supportAdvancedThumb - Controls features supported by the document thumbnail.
			 * 		If 'true' the thumbnail supports on mouse over zoom function and opening the Viewer by
			 * 		clicking on the thumbnail. Otherwise none of these functions is supported. */
			this.supportAdvancedThumb = options.supportAdvancedThumb;
			if (!this.supportAdvancedThumb && (this.supportAdvancedThumb !== false)) {
				// the 'supportAdvancedThumb' rendering option is not defined or null - set it here
				// the advanced thumbnail is supported only for instances of StaticDocumentRow (by default)
				this.supportAdvancedThumb = false;
				if (this.constructor == StaticDocumentRowView.prototype.constructor) {
					this.supportAdvancedThumb = true;
				}
			}

			/** {String} itemRootElemStr - Name of the HTML tag that is the root for each column/item. */
			this.itemRootElemStr = this.renderOptions.itemRootElemStr || 'td';


			/** {Associative Array} renderMethodMap - Mapping between labels in configurable document table 
			 * 		options ('docLinkTableOptions' in AttributeCategory.js) and rendering methods that draw 
			 * 		the particular column/item. */
			this.renderMethodMap = {
				'AttributeFlapout': this.renderExpandCollapseButton,
				'UpDownArrows': this.renderSortingArrows,
				'SequenceNumber': this.renderSequenceNumber,
				'RowSelector': this.renderRowSelector,
				'DocumentName': this.renderDocumentSpec,
				'DocumentActions': this.renderDocumentActions,
				'DocumentProperty_Thumbnail': this.renderThumbnail,
				'DocumentProperty_Description': this.renderDocumentDescription,
				'DocumentProperty_ApprovalStatus': this.renderApprovalStatus,
				'DocumentProperty_ApprovalStage': this.renderApprovalStage,
				'DocumentProperty_Project': this.renderDocProjectName,
				'DocumentProperty_VersionRevision': this.renderDocumentVersion,
				'DocumentProperty_VersionAuthor': this.renderDocumentAuthor,
				'DocumentProperty_ModifiedDate': this.renderDocumentModifiedDate,
				'ReferenceProperty': this.renderReferenceProperty,
				'DocumentAttribute': this.renderDocAttribute,
				'ProjectAttribute': this.renderProjectAttribute,
				'ProjectProperty_ProjectName': this.renderDocProjectName,
				'ProjectProperty_ProjectStatus': this.renderProjectStatus,
				'ProjectProperty_ProjectDueDate': this.renderProjectDueDate,
				'LocalRemote': this.renderLocalRemoteButton,
				'DeleteButton': this.renderDeleteButton,
				'ReplaceButton': this.renderReplaceButton,
				'CloneButton': this.renderCloneButton
			};


			// fields for action buttons (initialized on demand)
			this.viewBtn = null;
			this.downloadBtn = null;
			this.open3dBtn = null;
			this.openAcadBtn = null;
			this.newVersionBtn = null; // new version button needs context row to render inline form in
			this._newVersionRow = null;
			this._newVersionCell = null;

			return this;
		};


		/**
		 * Method returning a unique string for each document row in a single documents table to be used as
		 * a class, ID or name of HTML elements to enable easier SAHI automatic testing development.
		 * 
		 * Note the returned string is not unique across multiple document tables (even nested ones)!!!
		 */
		StaticDocumentRowView.prototype.getSAHIHelperStr = function (elemName) {
			return 'jira_' + elemName + '-' + this.dm.getParentDocTable().getTablePrefix() + '-' + this.dm._creationIdx;
		};

		/**
		 * Main render method controlling the rendering process from "helicopter view".
		 * It handles synchronization of multiple rendering requests that come at (almost) the same time.
		 * 
		 * @param options - Optional parameter defining a different set of one-time rendering options that are 
		 * 		applied for this (and only this) rendering pass.
		 */
		StaticDocumentRowView.prototype.render = function (options) {
			var phElem = $('#' + this.phElemID);
			if (phElem.length === 0) {
				// nowhere to render yet - bail out
				return phElem;
			}

			// if the 'options' parameter is valid use it instead of default rendering options, but only for 
			// this time so remember the default options too!
			var origRenderOptions = this.renderOptions;
			if (options) {
				this.renderOptions = options;
			}

			// get the actual index of this document (row) in the parent documents table and save it to be used by
			// individual rendering methods (where needed)
			this.rowIndex = this.dm.getIndexInTable();

			// to enable SAHI automated testing
			phElem.addClass(this.getSAHIHelperStr('docrow'));

			if (this.renderOptions.renderMethod == 'list') {
				this.renderSimple(phElem);
			} else if (this.renderOptions.renderMethod == 'table') {
				this.renderComplex(phElem);
			} else {
				window.alert("Not supported render method: " + this.renderOptions.renderMethod + ".");
			}

			// restore the original (default) rendering options
			this.renderOptions = origRenderOptions;

			// apply custom style style
			this.applyCustomStyle(phElem);

			return phElem;
		};


		/**
		 * Method to repaint already existing document row. It uses the ID of document row <tr> element as store
		 * in this.phElemID member to find DOM element to be updated.
		 */
		StaticDocumentRowView.prototype.repaintRow = function () {
			this.renderOptions.isRepaintRow = true;

			// render the document row itself
			this.render();

			this.renderOptions.isRepaintRow = false;
		};



		/**
		 * The simplest implementation of document table row render method.
		 * 
		 * Not used now.
		 * 
		 * @param {jQuery object} phElem - Place holder HTML DOM element, where to render the content generated
		 * 		by this method.
		 * @returns - jQuery object representing HTML DOM element, where the new HTML was rendered.
		 */
		StaticDocumentRowView.prototype.renderSimple = function (phElem) {
			if (!phElem || (phElem.length === 0)) {
				// nowhere to render - bail out
				return phElem;
			}
			try {
				// make the rendering placeholder empty (removing potential old HTML while repainting)
				phElem.html('');
				// make also the rendering placeholder of hidden properties empty
				$('#' + this.dm.hiddenPropValsElemID).html('');

				// render the placeholder row for attribute category and properties attr. category of this 
				// document (if any).
				// note it's here just to unify the structure of the document row across different rendering methods
				this.renderACatPlaceholders(phElem);

				// render placeholder for upload new version (also just for HTML structure unification)
				this.renderNewVersionPlaceholder(phElem);

				// render fixed set of row items (columns)
				this.renderSortingArrows(phElem);
				this.renderThumbnail(phElem, {
					thumbnailSize: 50
				});
				this.renderDocumentSpec(phElem, {});
				this.renderDeleteButton(phElem);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderSimple:\n' + err);
				return null;
			}

			return phElem;
		};


		/**
		 * Method to generate HTML representing the row in documents table.
		 * It generates the "table view" (full-featured as far as current state of implementation allows).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @returns - jQuery object representing the rendered document row.
		 */
		StaticDocumentRowView.prototype.renderComplex = function (phElem) {
			if (!phElem || (phElem.length === 0)) {
				// nowhere to render - bail out
				return phElem;
			}

			// make the rendering placeholder empty (removing potential old HTML while repainting)
			this.destroy(false);
			phElem.html('');
			
			// make also the rendering placeholder of hidden properties empty
			$('#' + this.dm.hiddenPropValsElemID).html('');

			// Make sure that all menus on the page are cleaned up.
			DropdownOverflowUtil.cleanupDropdowns();

			// render the placeholder row for attribute category and properties attr. category of this 
			// document (if any).
			// note the 'attrCatRowColspan' is calculated only after all the columns are rendered. 
			var attrCatRowPH = this.renderACatPlaceholders(phElem);

			// render placeholder for upload new version
			// note the 'uploadNewVersionRowColspan' is calculated only after all the columns are rendered. 
			var newVersionRowPH = this.renderNewVersionPlaceholder(phElem);

			// render row items (columns) based on configuration stored in 'this.renderOptions.columns'
			var columns = this.renderOptions.columns;
			if (columns) {
				for (var col = 0; col < columns.length; col++) {
					var column = this.renderOptions.columns[col];
					if (column) {
						var renderMethodID = column.type;
						if (renderMethodID === 'DocumentProperty' || renderMethodID === 'ProjectProperty') {
							renderMethodID += '_' + column.typeDetail;
						}
						var renderMethod = this.renderMethodMap[renderMethodID];
						if (renderMethod) {
							renderMethod.call(this, phElem, column);
						}
					}
				} // end of: for (var col = ...
			}
			// set the colspan of the "attribute category column to spread over all available columns 
			// except the first one (which is used to indent the child attribute category).
			// the colspan value == "number of rendered columns - 1" due to that
			var slaveRowsColspan = phElem.find('>td').size() - 1;
			attrCatRowPH.find(' > td.attrCategoryColumn').attr('colspan', slaveRowsColspan);
			if (newVersionRowPH) {
				newVersionRowPH.find(' > td.uploadNewVersionColumn').attr('colspan', slaveRowsColspan);
			}
			
			return phElem;
		};


		/**
		 * Adds expand/collapse button to this document row.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderExpandCollapseButton = function (phElem) {
			try {
				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow td-centered">';
				if (!this.dm.isInErrorStatus()) {
					// it's initially hidden, shown only if there is anything to expand
					var clickExpandMethod = this.dm.getThisInDOM() + '.expandClicked(this); return false;';
					// show expand/collapse button only if the document has assigned attribute category (and we know it for sure)
					var hiddenElemClassStr = (this.dm.hasAttrCat() ? '' : ' hiddenElement');

					var iconClass = "icon icon-normal";
					var iconTitle = "";
					if (this.dm.dependsOnItself) {
						iconClass += " icon-WCR_Refresh";
						iconTitle = WCXML.xmlencode(DocRefStrings.t('DocRowCyclicDepTooltip'));
					} else if (this.dm.aCatExpanded) {
						iconClass += " activIcon icon-WCR_Minus-square collapseDepButton";
						iconTitle = WCXML.xmlencode(DocRefStrings.t('DocRowCollapseTooltip'));
					} else {
						iconClass += " activIcon icon-WCR_Plus-square expandDepButton";
						iconTitle = WCXML.xmlencode(DocRefStrings.t('DocRowExpandTooltip'));
					}

					htmlcode += '<i class="' + iconClass + ' expandCollapseButton ' + hiddenElemClassStr + '" onclick="' + clickExpandMethod + '" title="' + iconTitle + '"/>';
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderExpandCollapseButton:\n' + err);
			}
		};


		/**
		 * Method that renders the manual sorting/ordering arrows.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderSortingArrows = function (phElem) {
			try {
				if (!this.dm.parentDocTable.isEditable) {
					// don't render this column at all if the documents table is in read-only or static mode
					return;
				}

				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
				// action handler methods for both arrows
				var moveUpMethod = this.dm.getParentInDOM() + '.moveUpButtonClicked(' + this.rowIndex + ');return false;';
				var moveDownMethod = this.dm.getParentInDOM() + '.moveDownButtonClicked(' + this.rowIndex + ');return false;';
				// logic what arrow to display (first and last rows in a table has only down / up arrow)
				var isFirstRow = (this.rowIndex === 0);
				var isLastRow = (this.rowIndex == this.dm.getParentDocTable().getNumberOfRows() - 1);
				if (isFirstRow && !isLastRow) {
					htmlcode += '<a class="btn btn9x14 float-right" href="#" onclick="' + moveDownMethod + '"><i class="icon icon-WCR_Sort-Down"></i></a>';
				} else if (isLastRow && !isFirstRow) {
					htmlcode += '<a class="btn btn9x14 float-right" href="#" onclick="' + moveUpMethod + '"><i class="icon icon-WCR_Sort-Up"></i></a>';
				} else if (!isFirstRow && !isLastRow) {
					htmlcode += '<div class="float-right">';
					htmlcode += '<a class="btn btn9x14 float-right" href="#" onclick="' + moveUpMethod + '"><i class="icon icon-WCR_Sort-Up"></i></a><br/>';
					htmlcode += '<a class="btn btn9x14 float-right" href="#" onclick="' + moveDownMethod + '"><i class="icon icon-WCR_Sort-Down"></i></a>';
					htmlcode += '</div>';
				}
				htmlcode += '</' + this.itemRootElemStr + '>';
				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderSortingArrows:\n' + err);
			}
		};


		/**
		 * Method to render column/item with a number representing order.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderSequenceNumber = function (phElem) {
			try {
				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="center td-narrow">';
				htmlcode += '<span>' + (this.rowIndex + 1) + '</span>';
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderSequenceNumber:\n' + err);
			}
		};


		/**
		 * Adds row selector checkbox to this document row.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderRowSelector = function (phElem) {
			try {
				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
				if (!this.dm.isInErrorStatus() && this.dm.docVerID) {
					var selectRowMethod = this.dm.getThisInDOM() + '.selectRowClicked(this);';
					htmlcode += '<input type="checkbox" id="' + this.phElemID + '_rowSelector"';
					htmlcode += ' name="' + this.dm.getParentDocTable().getRootLevelPrefix() + '_selectedDocs"';
					htmlcode += ' value="' + this.dm.docVerID + '" onclick="' + selectRowMethod + '"';
					if (this.dm.isSelected) {
						htmlcode += ' checked="checked"';
					}
					htmlcode += '>';
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderRowSelector:\n' + err);
			}
		};


		/**
		 * Method rendering the thumbnail or its placeholder if it's not loaded yet.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderThumbnail = function (phElem, config) {
			try {
				var htmlcode = '';
				if (this.renderOptions.showThumbnails) {
					htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
					if (!this.dm.isInErrorStatus()) {
						var thumbnailURL = this.dm.thumbnailURL;
						if (thumbnailURL && (thumbnailURL.length > 0)) {
							htmlcode += this.customThumbnailHTML(thumbnailURL, config.thumbnailSize);
						} else {
							// thumbnail not loaded - place a placeholder instead of it
							htmlcode += '<div class="thumb"></div>';
						}
					}
					htmlcode += '</' + this.itemRootElemStr + '>';
				}
				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderThumbnail:\n' + err);
			}
		};


		/**
		 * Basic version that renders the document name with hyper link.
		 * More advanced versions defined in subclasses can render fields to specify a new document instead of
		 * just showing existing document name...
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentSpec = function (phElem, config) {
			try {

				// Create the collumn element.
				var colElem = $('<' + this.itemRootElemStr + '>');

				// Set a custom width if this was specified.
				if (config.width && config.width != "auto") {
					colElem.width(config.width);
				}

				// Make sure that the contents will obey this width. They may choose to ignore the width on the cell in order to fit so force them!
				var container = $(this.addStartOfCustomWidthDiv(config, '150px'));

				// When no header is displayed, display it here ...
				container.append($(this.renderInlineLabel(config)));

				// Add the name + link to the document here ... 
				container.append(this.renderDocNameLink());

				// Add the container to the table.
				colElem.append(container);
				phElem.append(colElem);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentSpec:\n' + err);
			}
		};

		/**
		 * Method to render the link to the document name.
		 * 
		 * @returns {jQuery} - The generated document name link element.
		 */
		StaticDocumentRowView.prototype.renderDocNameLink = function () {
			// The element to be rendered.
			var docNameLink = $("<div>");

			// If an error occured while loading the document ...
			if (this.dm.isInErrorStatus()) {
				// Create the error msg.
				var errorMsg = $("<span>");
				var msg = "";
				if (this.dm.docName) {
					msg += this.dm.docName + ": ";
				}
				msg += DocRefStrings.t('DocRowCannotLoadDoc');
				errorMsg.text(msg);

				// Instead of showing the link, we will show a warning icon followed by the error msg.
				var icon = $("<i>", {
					"class": "icon icon-WCR_Msg-Warning",
					"title": DocRefStrings.t('DocRowCannotLoadDoc')
				});
				docNameLink.append(icon, errorMsg);
			}
			// No error, generate the name and URL.
			else {
				// Generate the document name URL to the document.
				var documentUrl = window.webcenterConfig.jspBaseUrl + 'docdetails.jsp?docVerID=' + this.dm.docVerID + '&amp;menu_file=' + DocTableXUtils.getMenuFileParamValue();
				var documentLink = $("<a>", {
					"target" : "_blank",
					"class" : "docrefdoclink",
					"href" : documentUrl
				}).text(this.dm.docName);
				docNameLink.append(documentLink);

				// If the document is currently checked out ...
				if (this.dm.docCheckedOutOwner) {
					var lockicon = $("<i>", {
						"class": "icon icon-normal icon-WCR_Actions-Lock12 add-spacing-left",
						"title": DocRefStrings.t('DOC_CHECKED_OUT'),
						"alt": DocRefStrings.t('DOC_CHECKED_OUT')
					});
					docNameLink.append(lockicon);
				}
			}

			return docNameLink;
		};
		/**
		 * Main method to render document action buttons based on rendering configuration.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentActions = function (phElem, config) {
			var htmlcode = '';
			htmlcode += '<' + this.itemRootElemStr + ' class="docrefactionscol td-narrow">';

			htmlcode += '<div class="row-actions row-actions-always-visible">';

			if (!this.dm.isInErrorStatus()) {
				var actions = config.typeDetail.split("|");
				for (var i = 0; i < actions.length; i++) {
					var action = actions[i];

					// add placeholder and create context for button
					var elemId = this.phElemID + '_actionBtn_' + DocTableXUtils.getNextUniquePrefix();
					htmlcode += '<div id="' + elemId + '" class="hiddenElement"/>';
					var btn = null;
					switch (action) {
					case "View":
						btn = this.getViewBtn();
						break;
					case "Open3d":
						btn = this.getOpen3DBtn();
						break;
					case "OpenAcad":
						btn = this.getOpenACADBtn();
						break;
					case "Download":
						btn = this.getDownloadBtn();
						break;
					case "UploadNewVersion":
						btn = this.getUploadNewVersionBtn();
						break;
					}
					// defer setting the context, as string concatenation is used which prevents dynamic element passing
					if (btn) {
						_.defer(_.bind(function (elemId, btn) {
							// call for render button only. It's possible to have multiple buttons
							btn.renderBtn($("#" + elemId));
						}, this, elemId, btn));
					}
				}
			}

			htmlcode += '</div>';
			htmlcode += '</' + this.itemRootElemStr + '>';

			// render the prepared HTML code
			phElem.append(htmlcode);
		};

		/**
		 * Method to render column/item with document description.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentDescription = function (phElem, config) {
			try {

				var colElem = $('<' + this.itemRootElemStr + '>');
				// Set a custom width if this was specified.
				if (config.width && config.width != "auto") {
					colElem.width(config.width);
				}

				var container = $(this.addStartOfCustomWidthDiv(config, '150px'));
				var textContainer = $("<div>");
				var overFlowContainer = new OverflowContentContainer(textContainer, 44);
				container.append(overFlowContainer.getJQElement());

				if (!this.dm.isInErrorStatus()) {
					if (this.dm.docDesc) {
						// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
						textContainer.append(this.renderInlineLabel(config));
						textContainer.append($('<span>').text(this.dm.docDesc));
					}
				}

				// render the prepared HTML code
				colElem.append(container);
				phElem.append(colElem);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentDescription:\n' + err);
			}
		};



		/**
		 * Method rendering the the approval status info (image and link to the most obvious action).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderApprovalStatus = function (phElem, config) {
			try {
				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-status">';
				htmlcode += this.renderInlineLabel(config);
				if (!this.dm.isInErrorStatus()) {
					var appStat = this.dm.appStatus;
					if (appStat && appStat.cycleStatus) {
						// there is an approval rule defined for this document (row)
						// find out the details of approval type in which the document currently is
						var appType = null;
						var appTypeList = this.dm.appTypeList;
						for (var i = 0; i < appTypeList.length; i++) {
							var aType = appTypeList[i];
							if (aType.id == appStat.statusID) {
								appType = aType;
								break;
							}
						}
						// render the approval status icon
						// Approval pop-up loc
						var approvalPopupHTML = 'onclick="javascript:showapprovaldetails(this, event,\'' + this.dm.docVerID + '\', \'Approval Setup ' + WCXML.xmlencode(WCHTML.encodeForJScript(this.dm.docName)) + '\')"';
						approvalPopupHTML += ' onmouseout="javascript:delayhidemenu();"';

						htmlcode += '<div class="docrefApprovalStatus">';

						var translLabel;
						var iconClass;
						if (appStat.cycleStatus == 'Not Started') {
							translLabel = WCXML.xmlencode(DocRefStrings.t('APPROVAL_STATUS_HOLD'));
							iconClass = "icon icon-WCR_Stopped icon-blue";
						} else if (appType) {
							// other than stopped
							translLabel = WCXML.xmlencode(appType.translName);

							if (appType.icon == "rejected.gif") {
								iconClass = "icon-WCR_Rejected icon-red";
							} else if (appType.icon == "approved.gif") {
								iconClass = "icon-WCR_Approved icon-green";
							} else if (appType.icon == "pending.gif") {
								iconClass = "icon-WCR_Pending icon-blue";
							} else if (appType.icon == "rejectforce.gif") {
								iconClass = "icon-WCR_ForceReject icon-red";
							} else if (appType.icon == "approveforce.gif") {
								iconClass = "icon-WCR_ForceApproved icon-green";
							} else if (appType.icon == "approvedcond.gif") {
								iconClass = "icon-WCR_ApprovedConditional icon-green";
							} else if (appType.icon == "fappcond.gif") {
								iconClass = "icon-WCR_ApprovedConditional icon-green";
							}
						}
						
						htmlcode += '<i class="' + iconClass + '" style="cursor: pointer;" title="' + translLabel + '" ' + approvalPopupHTML + ' />';
						htmlcode += '</div>';

						htmlcode += '</div>';

					}
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderApprovalStatus:\n' + err);
			}
		};


		/**
		 * Method to render column/item with current approval stage of the document.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderApprovalStage = function (phElem, config) {
			try {
				var htmlcode = '';
				// Set a custom width if this was specified.
				if (config.width && config.width != "auto") {
					htmlcode += '<' + this.itemRootElemStr + ' class="docrefApprovalStage"' + ' style="width:' + config.width + '">';
				} else {
					htmlcode += '<' + this.itemRootElemStr + ' class="docrefApprovalStage" >';
				}
				htmlcode += this.renderInlineLabel(config);
				if (!this.dm.isInErrorStatus()) {
					var appStat = this.dm.appStatus;
					if (appStat && appStat.stageName) {
						// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
						htmlcode += this.addStartOfCustomWidthDiv(config, '50px');
						htmlcode += WCXML.xmlencode(appStat.stageName);
						htmlcode += this.addEndOfCustomWidthDiv();
					}
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentAuthor:\n' + err);
			}
		};


		/**
		 * Method to render column/item with project name (where the document sits).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocProjectName = function (phElem, config) {
			try {
				this._renderProjectProperty(phElem, config, function (project) {
					var htmlcode = '<a href="' + window.webcenterConfig.jspBaseUrl + 'projdetails.jsp?projectID=' + project.id + '&amp;menu_file=' + DocTableXUtils.getMenuFileParamValue() + '" target="_blank">';
					htmlcode += WCXML.xmlencode(project.name);
					htmlcode += '</a>';
					return htmlcode;
				});
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocProjectName:\n' + err);
			}
		};

		/**
		 * Private function to render a project property
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 * @param {Function} getPropertyHTMLFunction - A Function with project obj as parameter that will return custom HTML for the property
		 * @param {Boolean} [ignoreBR]- True if you want to ignore BR if multiple projects
		 */
		StaticDocumentRowView.prototype._renderProjectProperty = function (phElem, config, getPropertyHTMLFunction, ignoreBR) {
			var htmlcode = '';
			// Set a custom width if this was specified.
			if (config.width && config.width != "auto") {
				htmlcode += '<' + this.itemRootElemStr + ' style="width:' + config.width + '">';
			} else {
				htmlcode += '<' + this.itemRootElemStr + '>';
			}
			htmlcode += this.renderInlineLabel(config);
			if (!this.dm.isInErrorStatus()) {
				// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
				htmlcode += this.addStartOfCustomWidthDiv(config, '50px');

				var projList = this.dm.projects;
				if (projList && (projList.length > 0)) {
					for (var i = 0; i < projList.length; i++) {
						var proj = projList[i];
						if (!proj || !proj.id) {
							continue;
						}
						if (i > 0 && !ignoreBR) {
							htmlcode += '<br/>';
						}
						htmlcode += getPropertyHTMLFunction(proj);
					}
				} else if (this.dm.project.id) {
					// render project link out of the this.project.id/name only if this.projects is 
					// not initialized or empty, otherwise one project would be rendered twice!
					htmlcode += getPropertyHTMLFunction(this.dm.project);
				}
				htmlcode += this.addEndOfCustomWidthDiv();
			}
			htmlcode += '</' + this.itemRootElemStr + '>';
			// render the prepared HTML code
			phElem.append(htmlcode);
		};

		/**
		 * Method to render column/item with project due date (of project where the document sits).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderProjectDueDate = function (phElem, config) {
			try {
				this._renderProjectProperty(phElem, config, function (project) {
					var dueDate = project.dueDate;
					try {
						if (!dueDate || dueDate == 0) { // intentional ==
							dueDate = '';
						} else {
							dueDate = new Date(parseInt(dueDate));
						}
					} catch (err) {
						DocTableXUtils.log("Failed to parse '" + project.dueDate + "' to a Date object.");
						dueDate = '';
					}

					var htmlcode = WCXML.xmlencode(WCHTML.getDateHourString(dueDate));
					return htmlcode;
				});
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderProjectDueDate:\n' + err);
			}
		};

		/**
		 * Method to render column/item with project status (of project where the document sits).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderProjectStatus = function (phElem, config) {
			try {
				this._renderProjectProperty(phElem, config, function (project) {
					var htmlcode = '';
					if (project && project.statusObj && project.statusObj.statusName) {
						switch (config.projectStatusLayout) {
						case 'LabelAndIcon':
							if (project.statusObj.iconURL) {
								htmlcode += '<img alt="" border="0" title="' + WCXML.xmlencode(project.statusObj.statusName) + '" src="images/statuses/' + project.statusObj.iconURL + '" />';
							}
							htmlcode += '<br />';
							htmlcode += WCXML.xmlencode(project.statusObj.statusName);
							break;
						case 'OnlyLabel':
							htmlcode += WCXML.xmlencode(project.statusObj.statusName);
							break;
						case 'ColorLabel':
							if (project.statusObj.colorRGB) {
								var foreGroundColor = ColorUtils.calculateForegroundColor(project.statusObj.colorRGB);
								htmlcode += '<div class="projectStatus" style="background-color: ' + project.statusObj.colorRGB + '; color: ' + foreGroundColor + ';">';
								htmlcode += '<a class="btn btn-no-action">' + WCXML.xmlencode(project.statusObj.statusName) + '</a></div>';
							} else {
								// fallback on label -> should not happen
								htmlcode += WCXML.xmlencode(project.statusObj.statusName);
							}
							break;
						case 'OnlyIcon':
							if (project.statusObj.iconURL) {
								htmlcode += '<img alt="" border="0" title="' + WCXML.xmlencode(project.statusObj.statusName) + '" src="images/statuses/' + project.statusObj.iconURL + '" />';
							}
							break;
						}
					}
					return htmlcode;
				}, config.projectStatusLayout === 'ColorLabel');
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderProjectStatus:\n' + err);
			}
		};


		/**
		 * Method to render column/item with document version/revision.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentVersion = function (phElem, config) {
			try {
				var col = $('<' + this.itemRootElemStr + '>', {
					'class': 'td-centered td-narrow'
				});

				col.html(this.renderInlineLabel(config));

				if (!this.dm.isInErrorStatus()) {
					if (this.dm.docVerIdx) {
						var versionStr = this.dm.docVerIdx;
						if (this.dm.docRevLetter) {
							versionStr += '(' + this.dm.docRevLetter + ')';
						}
						col.append($('<span>', {
							'class': 'row-action-text'
						}).text(versionStr));

						// If this is not the latest version and the mode is to display the latest, indicate this with a symbol.
						if (this.dm.docVerIdx != this.dm.latestDocVerIdx && this.dm.parentDocTable.docReferenceMode === 0) {
							var notLatestSymbol = $("<i>", {
								"class": "icon icon-WCR_Msg-Warning",
								"title": DocRefStrings.t('DocTableRefNotLatestVersion')
							});
							col.append(notLatestSymbol);
						}
					}

					if (config.uploadNewVersion) {
						var rowActionDiv = $('<div>', {
							'class': 'row-actions row-actions-always-visible'
						});

						// add placeholder for upload new version button
						var uplBtn = $('<div>', {
							'id': this.phElemID + '_uploadNewVersionBtn'
						}); // , 'style': 'padding: 4px'
						rowActionDiv.append(uplBtn);
						col.append(rowActionDiv);

						// render button in the placeholder (but defer it, so it only renders after the element is added to the DOM)
						_.defer(_.bind(function () {
							var btn = this.getUploadNewVersionBtn();
							if (btn) {
								btn.setBtnContext($('#' + this.phElemID + '_uploadNewVersionBtn'));
							}
						}, this));
					}
				}

				// render the prepared HTML code
				phElem.append(col);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentVersion:\n' + err);
			}
		};

		/**
		 * Method to render column/item with document author full name.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentAuthor = function (phElem, config) {
			try {
				var htmlcode = '';
				// Set a custom width if this was specified.
				if (config.width && config.width != "auto") {
					htmlcode += '<' + this.itemRootElemStr + ' class="Black_11"' + ' style="width:' + config.width + '">';
				} else {
					htmlcode += '<' + this.itemRootElemStr + ' class="Black_11">';
				}
				htmlcode += this.renderInlineLabel(config);
				if (!this.dm.isInErrorStatus()) {
					if (this.dm.docAuthor) {
						// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
						htmlcode += this.addStartOfCustomWidthDiv(config, '50px');

						htmlcode += WCHTML.getUserNameStringActiveHTML(this.dm.docAuthor.id, 
								this.dm.docAuthor.firstName, this.dm.docAuthor.lastName, this.dm.docAuthor.username, 
								this.dm.docAuthor.isActive,  DocRefStrings.t("DocRefUserDeactivated"), true);

						htmlcode += this.addEndOfCustomWidthDiv();
					}
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentAuthor:\n' + err);
			}
		};


		/**
		 * Method to render column/item with document last modification date (and time).
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocumentModifiedDate = function (phElem, config) {
			try {
				var htmlcode = '';
				// Set a custom width if this was specified.
				if (config.width && config.width != "auto") {
					htmlcode += '<' + this.itemRootElemStr + ' class="center"' + ' style="width:' + config.width + '">';
				} else {
					htmlcode += '<' + this.itemRootElemStr + ' class="center">';
				}
				htmlcode += this.renderInlineLabel(config);
				if (!this.dm.isInErrorStatus()) {
					if (this.dm.modDate) {
						// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
						htmlcode += this.addStartOfCustomWidthDiv(config, '50px');
						htmlcode += WCXML.xmlencode(WCHTML.getDateHourString(this.dm.modDate));
						htmlcode += this.addEndOfCustomWidthDiv();
					}
				}
				htmlcode += '</' + this.itemRootElemStr + '>';

				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDocumentAuthor:\n' + err);
			}
		};


		/**
		 * Method to render column/item with a button to make the document in the corresponding row local to the current project.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderLocalRemoteButton = function (phElem, config) {
			try {
				if (!this.dm.parentDocTable.isEditable) {
					// don't render this column at all if the documents table is in read-only or static mode
					return;
				}

				var htmlcode = '';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
				if (!this.dm.isInErrorStatus() && this.dm.getParentDocTable().hasLocalRemoteButton() && this.dm.isRemoteDocument()) {

					if (this.dm.isEditable && this.dm.canUserAddDocuments() && (config.visibility != 'ReadOnly') && !this.dm.dependsOnItself) {
						// the local/remote button has an action associated only when the row/table is 
						// editable, the user has "Add" permission (can add documents in this project),
						// the Local/Remote button is configured to be "editable" (not just an info) and 
						// this is not a cyclic dependent document
						var clickMethod = 'javascript: ' + this.dm.getParentInDOM() + '.localRemoteButtonClicked(' + this.rowIndex + ',' + '\'LocalRemote\'' + ');';
						var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteButtonTooltip'));

						htmlcode += '<button type="button" class="btn btn-xs btn-default actionBtn" title="' + translMsg + '" onclick="' + clickMethod + '">';
						htmlcode += '<i class="icon icon-WCR_Actions-MakeLocal12"/>';
						htmlcode += '</button>';
					} else {
						var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowLocalRemoteInfoTooltip'));
						htmlcode += '<i class="icon icon-WCR_Actions-MakeLocal12" title="' + translMsg + '" />';
					}
				}
				htmlcode += '</' + this.itemRootElemStr + '>';
				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderLocalRemoteButton:\n' + err);
			}
		};

		/**
		 * Method to render replace button in the row based on render configuration in Attribute Category
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderReplaceButton = function (phElem) {
			try {

				if (!this.dm.parentDocTable.isEditable) {
					// don't render this column at all if the documents table is in read-only or static mode
					return;
				}

				// Cell to add replace button
				var cell = $("<" + this.itemRootElemStr + ">", {
					'class': 'td-narrow'
				}).css('min-width', '70px');
				
				// ReplaceHandler invocation to get greenlinks
				var greenLinksObj = this.dm.getParentDocTable().getDocSrcLinks();
				var docSrcLength = greenLinksObj.length;
				
				// Having a replace button only makes sense when you have an actual document sources configured.
				if (docSrcLength > 0) {		
					// Replace icon
					var icon = $("<i>", {
						'class': 'icon icon-normal icon-WCR_Actions-Replace12'
					});
					// Replace dropdown div
					var dropDownParentDiv = $('<div/>').attr({
						'class': 'btn-group'
					});
					var dropDownIcon = $('<button/>').attr({
						'type': 'button',
						'class': 'btn btn-xs btn-default',
						'id': 'replace_greenlinks' + this.phElemID,
						'title': DocRefStrings.t('DocRowReplaceButtonTag')
					}).append(icon);
					dropDownParentDiv.append(dropDownIcon);
		
					// Creating dropdown only if more than one document source configured.
					if (docSrcLength > 1) {
		
						// Read dlsOptions
						var docTableObj = this.dm.getParentDocTable();
						var dlsOptions = docTableObj.getDocumentSourceOptions(docTableObj.greenLinksPrefix + '_inline');
		
						var dropDownButton = $('<button/>').attr({
							'type': 'button',
							'class': 'btn btn-xs btn-default dropdown-toggle',
							'data-toggle': 'dropdown'
						});
						var dropDownArrow = $('<span/>').attr({
							'class': 'caret'
						});
						var dropDownSrOnly = $('<span/>').attr({
							'class': 'sr-only'
						}).text(DocRefStrings.t('DocRowReplaceButtonTag'));
						dropDownButton.append(dropDownArrow);
						dropDownButton.append(dropDownSrOnly);
		
						var optionsDiv = $('<ul/>').attr({
							'class': 'dropdown-menu dropdown-menu-right',
							'role': 'menu'
						});
						dropDownParentDiv.append(dropDownButton);
						dropDownParentDiv.append(optionsDiv);
		
						// Create option for replace dropdown
						$('<li/>').addClass('dropdown-header').text(DocRefStrings.t('DocRowReplaceBy')).appendTo(optionsDiv);
						for (var i = 0; i < docSrcLength; i++) {
							if (greenLinksObj[i]) {
								var anchorDiv = $('<a/>').attr({
									'href': '#',
									'class': 'optionClass',
									'id': this.phElemID + "_" + 'option_' + i
								}).text(greenLinksObj[i].label.getLabel(dlsOptions.currentLanguage)).click(_.bind(function (event) {
									var optionId = $(event.target).attr('id');
									var dlsIndex = optionId.substring(optionId.lastIndexOf('_') + 1, optionId.length);
									var dlsObj = greenLinksObj[dlsIndex];
									this.dm.getParentDocTable().processReplaceRequest(dlsObj, this.rowIndex);
								}, this));
								$('<li/>').append(anchorDiv).appendTo(optionsDiv);
							}
						}
					}
		
					//OnClick Event handling for replace
					dropDownIcon.on("click", function () {
						if (greenLinksObj.length > 0) {
							this.dm.getParentDocTable().processReplaceRequest(greenLinksObj[0], this.rowIndex);
						}
						return false;
					}.bind(this));
		
					// Adding dropdown div to cell
					cell.append(dropDownParentDiv);
				}
				phElem.append(cell);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderReplaceButton:\n' + err);
			}
		};

		/**
		 * Method to render Clone button in the row based on render configuration in Attribute Category
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderCloneButton = function (phElem) {
			try {
				if (!this.dm.parentDocTable.isEditable) {
					// don't render this column at all if the documents table is in read-only or static mode
					return;
				}
				var htmlcode = '';
				if ((this.dm.docOperation === "Copy" || this.dm.docOperation === "New") || (!this.dm.isRemoteDocument() && this.dm.docOperation !== "Link" && this.dm.canUserDownloadDocument())) {
					var cloneMethod = 'javascript: ' + this.dm.getParentInDOM() + '.addCloneRowAction(' + this.rowIndex + ');return false;';
					htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
					htmlcode += '<button type="button" class="btn btn-xs btn-default actionBtn" title="' + DocRefStrings.t('DocRowDuplicateButtonTag') + '" onclick="' + cloneMethod + '">';
					htmlcode += '<i class="icon icon-normal icon-WCR_Duplicate"/>';
					htmlcode += '</button>';
					htmlcode += '</' + this.itemRootElemStr + '>';
				} else {
					htmlcode += '<' + this.itemRootElemStr + '/>';
				}
				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderCloneButton:\n' + err);
			}
		};

		/**
		 * Method to render remove button.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 */
		StaticDocumentRowView.prototype.renderDeleteButton = function (phElem) {
			try {
				if (!this.dm.parentDocTable.isEditable) {
					// don't render this column at all if the documents table is in read-only or static mode
					return;
				}
				var htmlcode = '';
				var removeMethod = 'javascript: ' + this.dm.getParentInDOM() + '.deleteButtonClicked(' + this.rowIndex + ');return false;';
				htmlcode += '<' + this.itemRootElemStr + ' class="td-narrow">';
				htmlcode += '<button type="button" class="btn btn-xs btn-default actionBtn" title="' + DocRefStrings.t('DocRowRemoveButtonTag') + '" onclick="' + removeMethod + '">';
				htmlcode += '<i class="icon icon-normal icon-WCR_Actions-Delete12"/>';
				htmlcode += '</button>';
				htmlcode += '</' + this.itemRootElemStr + '>';
				// render the prepared HTML code
				phElem.append(htmlcode);
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView.renderDeleteButton:\n' + err);
			}
		};


		/**
		 * Method to render column/item with a document's reference property.
		 * What should be rendered is determined by configuration ('config' parameter), the value to display is
		 * taken from this.dm.props array of document reference properties and how it is displayed is governed
		 * by Reference category of the "parent" doc. reference attribute.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderReferenceProperty = function (phElem, config) {
			var options = {
				isProperty: true,
				aCatObj: this.dm.propsACatObj,
				canEdit: this.dm.canUserEditProperties(),
				aCatPrefix: this.dm.propsACatPrefix,
				rootElement: this.itemRootElemStr,
				className: 'referencePropertyCell',
				tooltipKey: 'DocRowCannotFindProperty',
				messageKey: 'DocRowCannotFindPropertyShort'
			};

			if (config.visibility == 'Hidden') {
				var hiddenPropsElem = $('#' + this.dm.hiddenPropValsElemID);
				if (hiddenPropsElem.length == 1) {
					// change the root tag of rendered HTML to <div> (as hiddenPropsElem is a <div> as well)
					var origItemRootElemStr = this.itemRootElemStr;
					this.itemRootElemStr = 'div';

					this._renderAttrValue(hiddenPropsElem, config, options);

					this.itemRootElemStr = origItemRootElemStr;
				}
			} else {
				this._renderAttrValue(phElem, config, options);
			}
		};

		/**
		 * Method to render column/item with a referenced document attribute.
		 * What should be rendered is determined by configuration ('config' parameter), the value to display is
		 * taken from this.dm.props array of document reference properties and how it is displayed is governed
		 * by Reference category of the "parent" doc. reference attribute.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderDocAttribute = function (phElem, config) {
			if (this.dm.isInErrorStatus() || this.dm.dependsOnItself) {
				// don't render the document attributes in a column if there is an error, missing permissions 
				// or there is a cyclic dependency detected
				phElem.append('<' + this.itemRootElemStr + '/>');
			} else {
				var options = {
					isProperty: false,
					aCatObj: this.dm.aCatObj,
					canEdit: this.dm.canUserEditAttributes(),
					aCatPrefix: this.dm.aCatPrefix,
					rootElement: this.itemRootElemStr,
					className: 'documentAttributeCell',
					tooltipKey: 'DocRowCannotFindAttribute',
					messageKey: 'DocRowCannotFindAttributeShort'
				};
				this._renderAttrValue(phElem, config, options);
			}
		};


		/**
		 * Method to render column/item with a project attribute
		 * What should be rendered is determined by configuration ('config' parameter)
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 */
		StaticDocumentRowView.prototype.renderProjectAttribute = function (phElem, config) {
			var projList = this.dm.projects;
			if (projList && (projList.length > 0)) {
				var rootElem = $('<' + this.itemRootElemStr + '>');
				for (var i = 0; i < projList.length; i++) {
					var proj = projList[i];
					if (!proj || !proj.id || !proj.aCatObj) {
						continue;
					}
					var options = {
						isProperty: false,
						aCatObj: proj.aCatObj,
						canEdit: false,
						aCatPrefix: this.dm.projACatPrefix + proj.id,
						rootElement: 'div',
						className: 'projectAttributeCell',
						tooltipKey: 'DocRowCannotFindProjectAttribute',
						messageKey: 'DocRowCannotFindProjectAttributeShort'
					};
					this._renderAttrValue(rootElem, config, options);
				}
				phElem.append(rootElem);
			} else if (this.dm.project && this.dm.project.id && this.dm.project.aCatObj) {
				var options = {
					isProperty: false,
					aCatObj: this.dm.project.aCatObj,
					canEdit: false,
					aCatPrefix: this.dm.projACatPrefix + this.dm.project.id,
					rootElement: this.itemRootElemStr,
					className: 'projectAttributeCell',
					tooltipKey: 'DocRowCannotFindProjectAttribute',
					messageKey: 'DocRowCannotFindProjectAttributeShort'
				};
				this._renderAttrValue(phElem, config, options);
			}
		};

		/**
		 * Private method implementing renderReferenceProperty(), renderDocAttribute(), renderProjectAttribute() functionality.
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 * @param {Object} options - Options for rendering attribute values
		 *		{Boolean} isProperty: is it a reference property
		 * 		{Object} aCatObj: the aCatObj
		 * 		{Boolean} canEdit: can the attribute/property be edited
		 * 		{String} aCatPrefix: the prefix of the attribute category
		 * 		{String} rootElement: the root element of the value
		 * 		{String} className: class name for the element
		 * 		{String} tooltipKey: Tooltip translation key for wrong configuration
		 * 		{String} messageKey: Message translation key for wrong configuration
		 */
		StaticDocumentRowView.prototype._renderAttrValue = function (phElem, config, options) {
			try {
				var isItemFound = false;

				var isHiddenProperty = options.isProperty && (config.visibility == 'Hidden');
				var visibility = config.visibility;
				if (isHiddenProperty) {
					// hidden properties have to be rendered as editable - it's in fact the same as 
					// "hidden keep values" visibility type of standard attributes in a category...
					visibility = 'Editable';
				}
				// ID of the ref. property (attribute) is stored in typeDetail member of the rendering configuration
				var attrID = config.typeDetail;
				// this is the unique reference properties / document attributes category of this doc. row
				var aCatObj = options.aCatObj;
				if (attrID && this.dm.isAttrCatLoaded(aCatObj)) {
					// make sure the reference property stored in configuration is still in the doc. reference properties
					// attribute category ('aCatObj'). The 'aCatObj' is used for rendering only here!
					var attObj = aCatObj.findAttribute(attrID, config.attributeName);
					if (attObj && !attObj.isRichText()) {
						// property found in doc. ref. property attribute category - render it using the 'aCatObj'
						// first find the cellObj and other required info about the property in 'aCatObj' attr. category
						var cellInfo = aCatObj.findCell(attObj);
						if (cellInfo) {
							var cellObj = cellInfo.cellObj;
							var row = cellInfo.row;
							var col = cellInfo.col;

							// decide on editability of properties and/or attributes rendered in a column
							var isEditable = (visibility != 'ReadOnly') && options.canEdit;

							// compose the ID of the cell, where value of the 'attObj' is rendered by attribute category
							var itemACatPrefix = options.aCatPrefix;
							var itemCellID = itemACatPrefix + 'td_' + row + '_' + col;

							// for all cases make sure there is no HTML element with the same ID as the attribute elem. we
							// are going to render
							var itemCells = $('#' + itemCellID);
							if (itemCells.length == 1) {
								var infoMsgHTML = '<span class="docRefInfo">';
								var translMsg = WCXML.xmlencode(DocRefStrings.t('DocRowAttributeInColumnInfo') + ': ' + config.label);
								infoMsgHTML += '<img border="0" src="images/doc_ref_info.png" title="' + translMsg + '"/>';
								infoMsgHTML += '</span>';
								// remove ID of the element rendered within the original attribute category to prevent
								// having two HTML elements with the same ID on the page. The content of the element
								// is replaced by an info icon with description
								itemCells.removeAttr('id');
								itemCells.html(infoMsgHTML);
							} else if (itemCells.length > 1) {
								// something went wrong. this should not happen!
								window.alert("More than one HTML element with ID '" + itemCellID + "' on the page!"); //JANH debug - then DEL!!! (make a log from the alert)
							}

							// generate HTML of the item value
							var classNames = options.className;
							var htmlcode = '<' + options.rootElement + ' id="' + itemCellID + '" class="' + classNames + '"';

							// set a custom width if this was specified (and not hidden)
							if (!isHiddenProperty && config.width && config.width != "auto") {
								htmlcode += ' style="width:' + config.width + '">';
							} else {
								htmlcode += '>';
							}

							if (!isHiddenProperty) {
								// render inline label if configured so (and not hidden)
								htmlcode += this.renderInlineLabel(config);
							}

							if (attObj.isDocReference()) {
								// special handling for document reference attributes - these are rendered as list of hyperlinks
								// when placed in a column!
								if (attObj.valueHyperlink) { //JANH TODO what if the logged in user has no rights to view some (all the) documents!?!?! (it's a reported bug)
									// just break a single line of hyper-links delimited by ' | ' to a list of hyper-links
									// for rendering purposes...
									htmlcode += attObj.valueHyperlink.replace(/ \| /g, '<br/>');
								}
							} else {
								// not a document reference attribute - render it as in a real attribute category
								htmlcode += aCatObj.renderAttributeValueInternal(cellObj, attObj, row, col, visibility, isEditable, false);
							}
							htmlcode += '</' + options.rootElement + '>';

							var attrValueElem = $(htmlcode);

							var formEntry = attrValueElem.find('div.form-entry');
							if (config && config.width && (config.width != 'auto') && (formEntry.length > 0)) {
								// NOTE:: endsWith() is not supported in pre IE 12(edge), hence used equivalent RegExp
								if (config.width.trim().match(/%$/)) {
									// percentage width is handled a bit different than absolute width in pixels
									formEntry.addClass('LimitWidth').css('width', '100%'); // = $("<div>", {"class" : "form-entry LimitWidth", "style" : "width:100%"});
								} else {
									formEntry.addClass('LimitWidth').css('width', config.width);
									//formEntry = $("<div>", {"class" : "form-entry LimitWidth", "style" : "width: "+config.width});
								}
							}

							// render the prepared HTML code in the corresponding documents table column and row
							phElem.append(attrValueElem);

							if (isEditable) {
								// "editable" (also "required") mode requires some special treatment for more advanced 
								// attribute/property types
								if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
									// for date+time create/register calendar popup
									var dateID = "#" + itemACatPrefix + "longFormat_adate" + attObj.id;
									var hourID = "#" + itemACatPrefix + "date_period_of_day_" + attObj.id;
									aCatObj.createCalendar(dateID, hourID);
								}
							}

							// don't care if and how the item was rendered/moved to the particular column of
							// this document row, just report back the item was found!
							isItemFound = true;
						}
					}
				}
				if (!isItemFound && !isHiddenProperty) {
					// item was not found. it can be either because of error or still loading...
					var htmlcode = '<' + options.rootElement + ' class="' + options.className + '">';
					htmlcode += this.renderInlineLabel(config);
					if (this.dm.isAttrCatLoaded(aCatObj)) {
						// if document reference attribute configuration (which is the boss) is not in sync with 
						// the 'aCatObj', render exclamation mark instead of the property to signalize wrong configuration...
						var tooltip = WCXML.xmlencode(DocRefStrings.t(options.tooltipKey));
						var message = WCXML.xmlencode(DocRefStrings.t(options.messageKey));
						htmlcode += '<span style="color: #cccccc" title="' + tooltip + '">';
						htmlcode += message;
						htmlcode += '</span>';
					} else if (!options.isProperty && this.dm.hasNoAttrCat()) {
						var tooltip = WCXML.xmlencode(DocRefStrings.t('DocRowNoAttrCategoryInfo'));
						var message = WCXML.xmlencode(DocRefStrings.t('DocRowNoAttrCategoryInfoShort'));
						htmlcode += '<span style="color: #cccccc"  title="' + tooltip + '">';
						htmlcode += message;
						htmlcode += '</span>';
					}
					// else the properties attribute category (or other vital info) was not loaded yet
					// render placeholder and wait until the required info is loaded and this method is 
					// called again.
					htmlcode += '</' + options.rootElement + '>';

					// render the prepared HTML code
					phElem.append(htmlcode);
				}
			} catch (err) {
				DocTableXUtils.log('Problem in StaticDocumentRowView._renderAttrValue ' + config.label + ':\n' + err);
			}
		};


		/**
		 * Adds hidden row in documents table as a placeholder for attribute category of this document 
		 * and attribute category of the reference (properties of the reference) and attribute categories of the projects
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @returns {jQuery object} representing the rendered placeholder element.
		 */
		StaticDocumentRowView.prototype.renderACatPlaceholders = function (phElem) {
			if (this.renderOptions.isRepaintRow) {
				// don't render the placeholder if only the document row is repainted
				return phElem.siblings('#' + this.dm.aCatRowID);
			}

			// delete the original attribute category row including the placeholder (if any), but first
			// destroy rich text editors etc. 
			// The destroy operation here will KEEP the attribute category object(s) holding attribute values 
			// including changes already done by the user though!
			this.dm.destroy();
			$('#' + this.dm.aCatRowID).remove();

			// note the place holder for attribute and/or property category is rendered even in case of
			// an error or no perm. status! This is to not introduce diversity in the rows structure,
			// where not necessary...
			var htmlcode = '';
			var renderConfig = this.renderOptions.columns;
			// if this is full render operation (not just a repaint), no error occurred and there is expand/collapse 
			// button to show/hide attribute category of the document (if any), prepare placeholder for the actual 
			// document's attribute category
			htmlcode += '<tr id="' + this.dm.aCatRowID + '" class="rowNoBorder hiddenElement">'; // the attributes row is initially hidded
			htmlcode += '<td></td>'; // the first column is empty (it serves as indentation)
			htmlcode += '<td class="attrCategoryColumn">';
			// always hidden placeholder for reference category (reference properties)
			// the created attribute category should not be displayed as a whole.
			// such a placeholder element is however needed in order to work properly (action handlers, etc.)
			if (DocTableXUtils.getRenderItem(renderConfig, 'ReferenceProperty') != null) {
				htmlcode += '<div id="' + this.dm.propsACatElemID + '" class="hiddenElement"></div>';
				// placeholder for hidden (visibility == hidden) document reference properties values
				htmlcode += '<div id="' + this.dm.hiddenPropValsElemID + '" class="hiddenElement"></div>';
			}
			// placeholder for attribute category of the referenced document
			if (!this.aCatExpandDisabled || (DocTableXUtils.getRenderItem(renderConfig, 'DocumentAttribute') != null)) {
				// needs to be put on page when there is expand/collapse button or inline
				// document attribute in the table column configured (in rendering configuration)
				htmlcode += '<div id="' + this.dm.aCatElemID + '"></div>';
			}
			// placeholder for attribute category of projects
			if (DocTableXUtils.getRenderItem(renderConfig, 'ProjectAttribute') != null) {
				var projList = this.dm.projects;
				if (projList && (projList.length > 0)) {
					for (var i = 0; i < projList.length; i++) {
						var proj = projList[i];
						if (!proj || !proj.id) {
							continue;
						}
						htmlcode += '<div id="' + this.dm.projACatElemIDPrefix + proj.id + '" class="hiddenElement"></div>';
					}
				} else if (this.dm.project && this.dm.project.id) {
					htmlcode += '<div id="' + this.dm.projACatElemIDPrefix + this.dm.project.id + '" class="hiddenElement"></div>';
				}
			}
			htmlcode += '</td>';
			htmlcode += '</tr>';

			var htmlElem = $(htmlcode);
			// apply custom style style
			this.applyCustomStyle(htmlElem);

			// render the prepared HTML code
			phElem.after(htmlElem);

			return htmlElem;
		};

		/**
		 * Adds placeholder for form fields for uploading a new version of a document
		 * 
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @returns {jQuery object} representing the rendered placeholder element.
		 */
		StaticDocumentRowView.prototype.renderNewVersionPlaceholder = function (phElem) {
			if (this.renderOptions.isRepaintRow) {
				// don't render the placeholder if only the document row is repainted
				return this._newVersionRow;
			}
			// delete the original new version row including the placeholder (if any)
			if (this._newVersionRow) {
				this._newVersionRow.remove();
			}

			this._newVersionRow = $('<tr id="' + this.phElemID + '_newVersionRow" class="newVersionRow"></tr>');

			// render the prepared HTML code
			phElem.after(this._newVersionRow);

			this._newVersionCell = $('<td class="uploadNewVersionColumn">');
			this._newVersionRow.append($('<td>'));
			this._newVersionRow.append(this._newVersionCell);

			if (this.newVersionBtn) {
				this.newVersionBtn.setRowContext(this._newVersionCell);
			}

			// apply custom style style
			this.applyCustomStyle(this._newVersionRow);

			return this._newVersionRow;
		};


		/**
		 * Method to generate HTML code of a thumbnail.
		 * 
		 * @param {String} thumbURL - URL of the thumbnail image.
		 * @param {String} thumbSize - How big the thumbnail should be rendered according to the configuration.
		 * @returns {String} - The generated HTML.
		 */
		StaticDocumentRowView.prototype.customThumbnailHTML = function (thumbURL, thumbSize) {
			var htmlcode = '';
			if (!thumbSize || (thumbSize == '0')) {
				thumbSize = '50';
			}
			if (this.supportAdvancedThumb && this.dm.docVerID) {
				// advanced thumbnail mode requires the document's version ID to work properly. if both these
				// prerequisities are met, generate the advanced thumbnail.
				var zoomURL = window.webcenterConfig.jspBaseUrl + 'DownloadViewerThumbnail.jsp?docversionid=' + this.dm.docVerID + '&amp;pix=500&amp;ajax=1';
				var docDescription = (this.dm.docDesc ? this.dm.docDesc : '');
				// default thumbnail size is 50x50px.
				// cunstruct a clickable thumbnail with a bigger preview on mouse over + open the Viewer on click
				var hrefStr = window.webcenterConfig.jspBaseUrl + 'viewdocument.jsp?docVerID=' + this.dm.docVerID + '&amp;menu_file=' + DocTableXUtils.getMenuFileParamValue() + '&amp;sourcePage=' + DocTableXUtils.getSourcePageURLEncoded();
				htmlcode += '<a href="' + hrefStr + '" target="_blank">';
				htmlcode += '<img onmouseout="delayhidethumb();"';
				htmlcode += ' onmouseover="showthumb(event, \'' + thumbURL + '\', \'' + WCXML.xmlencode(WCHTML.encodeForJScript(this.dm.docName)) + '\', 202, null, \'' + zoomURL + '\', 500);"';
				htmlcode += ' src="' + window.webcenterConfig.jspBaseUrl + thumbURL + '"';
				htmlcode += ' alt="' + WCXML.xmlencode(docDescription) + '" width="' + thumbSize + 'px" height="' + thumbSize + 'px" border="0" align="middle">';
				htmlcode += '</a>';
			} else {
				// document version ID not known (probably upload of a new document) - generate just a basic thumbnail
				// based on the 'thumbURL' and 'thumbSize' parameters
				htmlcode += '<img src="' + window.webcenterConfig.jspBaseUrl + thumbURL + '"';
				htmlcode += ' width="' + thumbSize + 'px" height="' + thumbSize + 'px" border="0" align="middle">';
			}
			return htmlcode;
		};


		/**
		 * Method to render "inline labels" in each documents table cell if the table's header is hidden.
		 * 
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 * @returns {String} Generated HTML code containing the corresponding label.
		 */
		StaticDocumentRowView.prototype.renderInlineLabel = function (config) {
			var htmlcode = '';
			if (this.renderOptions.hideHeader && config && config.label) {
				// in case the header of documents table is not displayed, render the header label inside each documents table cell
				htmlcode += '<div class="docrefinlinelabel">' + DocTableXUtils.getEncodedTranslatedLabel(config.label);
				if (config.visibility == 'Required') {
					htmlcode += '&nbsp;<font color="red">*</font>';
				}
				htmlcode += '</div>';
			}
			return htmlcode;
		};


		/**
		 * Method to insert a special <div> tag to force width of the content of given table column (<td> element).
		 * 
		 * @param {Object} config - Holds the documents table rendering configuration for this column/item.
		 * @param {String} defaultMinWidth - Value representing the minimal width of given column if custom
		 * 		width is not specified. Format of an expected value is "100px".
		 * @returns {String} Generated HTML code containing the begining <div> tag.
		 */
		StaticDocumentRowView.prototype.addStartOfCustomWidthDiv = function (config, defaultMinWidth) {
			var htmlcode = '<div>';
			if (config.width && config.width != "auto") {
				// if a width was set, really enforce it! The table can still ignore the width on the cell but not this!
				// NOTE:: endsWith() is not supported in pre IE 12, hence used equivalent RegExp
				if (config.width.trim().match(/%$/)) {
					// percentage width is handled a bit different than absolute width in pixels
					htmlcode = '<div class="LimitWidth" style="width:100%">';
				} else {
					htmlcode = '<div class="LimitWidth" style="width:' + config.width + '">';
				}
			} else if (defaultMinWidth) {
				htmlcode = '<div style="min-width:' + defaultMinWidth + '">';
			}
			return htmlcode;
		};

		/**
		 * Method to insert an ending tag of the special <div> to force width of the content of given table 
		 * column (<td> element). 
		 * It returns ending part of the HTML element started in StaticDocumentRow.addStartOfCustomWidthDiv() method.
		 * 
		 * @returns {String} 
		 */
		StaticDocumentRowView.prototype.addEndOfCustomWidthDiv = function () {
			return '</div>';
		};


		/**
		 * Check if the component exists for upload new version button. If not, create it.
		 */
		StaticDocumentRowView.prototype.getUploadNewVersionBtn = function () {
			if (!this.newVersionBtn && this.showAdvancedDocumentActions() && this.dm.canUserUploadNewVersion()) {
				var options = {
					useInlineForm: true,
					docReferenceMode: this.dm.getParentDocTable().docReferenceMode,
					alwaysUpdateAllProjects: true,
					uniqueIdPrefix: DocTableXUtils.getNextUniquePrefix() + '_',
					onCancelCallback: _.bind(function () {
						this.dm.setIsNewDocVersionUploaded(false);
					}, this)
				};
				this.newVersionBtn = new UploadNewVersionButton(this.dm.docVerID, null, this.dm.parentDocTable.currentUser, options);

				// mark as changes when upload button is clicked
				WCEvent.on("btnClicked.UploadNewVersion." + this.dm.docVerID, _.bind(function () {
					this.dm.setIsChanged();
					this.dm.setIsNewDocVersionUploaded(true);

					// show the "Save" button on the page (if exists) when at least one "upload new version" button is on the page
					// THis is used for DocTableX in project/document attribute details pages, where the user can upload a new version 
					// of a referenced document even if he/she has no "change properties" rights (the whole attribute category is 
					// read-only for him/her).
					var pageSaveButton = $("#SaveButtonRowID");
					if (pageSaveButton.length == 1) {
						pageSaveButton.show();
					}

				}, this));

				// if this happens after placeholder for form was drawn, register it
				if (this._newVersionCell) {
					this.newVersionBtn.setRowContext(this._newVersionCell);
				}
			}
			return this.newVersionBtn;
		};

		/**
		 * Check if the component exists for upload new version button. If not, create it.
		 */
		StaticDocumentRowView.prototype.getViewBtn = function () {
			if (!this.viewBtn) {
				this.viewBtn = new ViewButton(this.dm.docVerID);
			}
			return this.viewBtn;
		};

		/**
		 * Check if the component exists for upload new version button. If not, create it.
		 */
		StaticDocumentRowView.prototype.getDownloadBtn = function () {
			if (!this.downloadBtn) {
				this.downloadBtn = new DownloadButton(this.dm.docVerID);
			}
			return this.downloadBtn;
		};

		/**
		 * Check if the component exists for Open 3d button. If not, create it.
		 */
		StaticDocumentRowView.prototype.getOpen3DBtn = function () {
			if (!this.open3dBtn) {
				this.open3dBtn = new Open3DButton(this.dm.docVerID);
			}
			return this.open3dBtn;
		};

		/**
		 * Check if the component exists for open in ACAD button. If not, create it.
		 */
		StaticDocumentRowView.prototype.getOpenACADBtn = function () {
			if (!this.openAcadBtn) {
				this.openAcadBtn = new OpenACADButton(this.dm.docVerID);
			}
			return this.openAcadBtn;
		};

		/**
		 * Check whether advanced document actions are allowed in this row (e.g. upload new version) 
		 *
		 * @returns whether or not advanced document actions are allowed
		 */
		StaticDocumentRowView.prototype.showAdvancedDocumentActions = function () {
			return true;
		};

		/**
		 * Method called before the DocTableRow elements are thrown out of the DOM.
		 * 
		 * @param {boolean} destroyACatObj - 'true' if the attributeCategory object should be removed from 
		 * 		the memory (all changes user might have already done will be lost);
		 * 		'false'/<no value> when the attributeCategory object should be kept
		 */
		StaticDocumentRowView.prototype.destroy = function (destroyACatObj) {
			
		};

		/**
		 * Apply custom style to given element
		 * @param {jQuery object} phElem - HTML DOM element, where to render the content generated by this method.
		 * @param {String} translMsg - Message to to shown
		 */
		StaticDocumentRowView.prototype.applyCustomStyle = function (elem) {
			// apply style to the whole row (but don't touch greenlinks, as these are css customizable)
			if (this.renderOptions.styleObj) {
				elem.find("*").not(".darktext_greenlink_12_bold, .darktext_greenlink_11, .icon, button, .TextContentElementWidget *, .input-group-addon, .form-div, .form-div *, .elementypecolor")
					.css(this.renderOptions.styleObj);
				elem.find("input, select, option").not(".greenlink_button")
					.css("background-color", "white");
			}
		};


		return StaticDocumentRowView;
	});