define(["doctablex/TextContentRowView",
		"doctablex/DocTableXUtils",
        "packaging-content-management/text-content-services/ContentService",
        "packaging-content-management/text-content-services/languageService",
        "packaging-content-management/PCMEventBus",
        "services/passwordValidationService",
		"wcr/webcenter_xml"],
	function (TextContentRowView,
		DocTableXUtils,
		ContentService,
		languageService,
		pcmEventBus,
		passwordValidationService,
		WCXML) {

		/**
		 * TextContentRow.js
		 *
		 * Object representing a single text content already uploaded in a given documents table
		 * (DocTableX instance). It holds required data and implements controller logic of the represented
		 * text content (row).
		 *
		 * @author JANH, HUBU - February 2015
		 */

		"use strict";

		var TextContentRow = function () {

			this.__isMixin = true; //JANH debug then DEL!!!

			/**
			 * List of overridden methods with their original code.
			 */
			TextContentRow.__overridden = {};

			/**
			 * Load and update document info related to this document row and repaint it if needed.
			 * The info is loaded using AJAXDataCache object.
			 * 
			 * @override mixin
			 * @param forceUpdate - Flag whether to force the cache to be refreshed with the most actual data.
			 * 			(not used for now!)
			 * 
			 * @returns - 'true' if the text content was updated and the repaint is needed, 'false' otherwise.
			 */
			if (this.updateOtherInfoFromServer) {
				TextContentRow.__overridden.updateOtherInfoFromServer = this.updateOtherInfoFromServer;
			}
			this.updateOtherInfoFromServer = function (repaint, forceUpdate) {
				// Call super
				TextContentRow.__overridden.updateOtherInfoFromServer.apply(this, arguments);
				// if table has any text content column, fetch the data required for rendering text content view  
				if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns) && this.docTypeID == '00001_0000000019') {
					// load text content data for this doc. if not available yet
					if (!this.textContent) {
						this.updateTextContentDataFromServer(this.docVerID, false, forceUpdate);
					}
				}
				return false; // the need for repaint is handled internally by updateTextContentDataFromServer() method
			};

			/**
			 * Load and update text content info related to this document row and repaint it if needed.
			 * The info is loaded using AJAXDataCache object.
			 * 
			 * @param repaint - Flag whether to repaint this document row after loading text content info
			 * @param forceUpdate - Flag whether to force the cache to be refreshed with the most actual data.
			 * @returns - If this.docVerID is a valid ID this method returns a promise object representing the AJAX request 
			 * 		to get info about attribute category assigned to this document (the primary is attribute category ID).
			 * 		Otherwise it returns rejected promise object immediately.
			 */
			this.updateTextContentDataFromServer = function (requestDocVerID, repaint, forceUpdate) {
				this.setStatus('loading');

				if (!requestDocVerID) {
					requestDocVerID = this.docVerID;
				}

				// nothing to do without a document version ID
				if (!requestDocVerID) {
					DocTableXUtils.error("Cannot load Text Content information from server, this.docVerID is not known!");
					return;
				}

				//Load text content info only if not available in the text content model
				if (DocTableXUtils.hasTextContentItem(this.renderOptions.columns) && this.docTypeID == '00001_0000000019') {

					var dfd = $.Deferred();

					// What should happen when we get the content back from the back end ...
					var callback = function (textContent) {

						// Remember a reference to this content
						this.textContent = textContent;

						// If this is a New/Copy operation, add the languages currently in the project.
						if (this.textContent.canManage().allowed && (this.docOperation == 'Copy' || this.docOperation == 'New')) {

							var masterLangs = this.parentDocTable.getMasterLanguages();
							var transLangs = this.parentDocTable.getTranslationLanguages();

							// Make sure that the content contains all the project languages.
							for (var i = 0; i < masterLangs.length; i++) {
								this.textContent.addLanguage(masterLangs[i], true, _.bind(function (element) {
									if (element) {
										pcmEventBus.emit("PCM.TextContent.langAdded", this.textContent, element);
									}
								}, this));
								
							}
							for (var i = 0; i < transLangs.length; i++) {
								this.textContent.addLanguage(transLangs[i], false, _.bind(function (element) {
									if (element) {
										pcmEventBus.emit("PCM.TextContent.langAdded", this.textContent, element);
									}
								}, this));
							}

							// Repaint rows 
							dfd.resolve(this.textContent);
							this.view.repaintRow();

						} else {
							// First time when the document is pulled into project
							// Resolve the returned promise of the 'updateTextContentDataFromServer' method.
							dfd.resolve(this.textContent);

							// Repaint this row.
							this.view.repaintRow();
						}
					}.bind(this);

					var errorCallback = function (message) {
						DocTableXUtils.log("TextContent error : " + message);
					};
					
					
					// Create the content request object:
					var contentRequest = {
						docVersionId: requestDocVerID,
						retainStatus: this.isRetainingTextContentStatus,
						fromProjectCreation: this.getParentDocTable().projectCreationContext,
						willBeNew : this.docOperation == "New" || this.docOperation == "Copy" || this.docOperation == "Upload",
						needEffectiveElementPermissions: true
					};

					// Will this be a new document?
					if (contentRequest.willBeNew && !contentRequest.fromProjectCreation) {
						// In that case the target project id will be:
						contentRequest.targetProjectId = this.parentDocTable.projectID;
					}
					
					// Request the content!
					ContentService.getContent(contentRequest, callback, errorCallback);

					return dfd.promise();
				}

				// return the promise object representing the request of data to allow further processing
				return textContentRequest;
			};


			/**
			 * Method generating XML to save changes in the text content row.
			 * This method has to be called for every document row that can potentially contain text content info!
			 * 
			 * @returns {String} - XML of text content changes
			 */
			this.convertTextContentToXML = function () {
				var xmlValue = '';
				try {
					if (this.getIsChanged() && this.textContent) {
						xmlValue += '<text_content>';

						// Get the validated password from the password validation service
						var password = passwordValidationService.getValidatedPassword();
						
						// Create the save JSON that contains all the save actions and the content request.
						var saveJson = {
								saveActions: this.textContent.getSaveActions(password),
								contentRequest: {
									docVersionId: this.textContent.docVersionId,
									targetProjectId: this.textContent.projectId,
									retainStatus: this.isRetainingTextContentStatus,
									fromProjectCreation: this.getParentDocTable().projectCreationContext,
									willBeNew: this.docOperation == "New" || this.docOperation == "Copy" || this.docOperation == "Upload",
							}
						};

						//JSON String requires encoding
						xmlValue += WCXML.xmlencode(JSON.stringify(saveJson, function (key, value) {
							// We have to be carefull to not have circular references in our SaveActions, therefore we will need to handle specific fields carefully.
							if (key == 'affectedTextContent' || key == 'affectedTextContentElement') {
								return value.getContentId();
							} else {
								return value;
							}
						}));
						xmlValue += '</text_content>';
					}
				} catch (err) {
					DocTableXUtils.log('Problem in TextContentRow.convertTextContentToXML:\n' + err);
					xmlValue = '';
				}
				return xmlValue;
			};

			/**
			 * Method generating XML to save changes in this text content row.
			 * This method is dedicated for text content document reference attributes only!
			 * Text content updates are inside <text_content> tag as JSON String
			 * 
			 * @override mixin
			 * @returns {String} - XML complying to JavaScript -> JSP interface to save changes done in document table.
			 */
			if (this.convertToXML) {
				TextContentRow.__overridden.convertToXML = this.convertToXML;
			}
			this.convertToXML = function () {
				var xmlValue = '';
				try {
					if (this.getIsChanged() && this.textContent && this.textContent.isChanged()) {
						xmlValue += '<ds>';
						xmlValue += WCXML.ElementEncode('action', 'UpdateDocContent');
						xmlValue += this.convertTextContentToXML();
						xmlValue += '</ds>';
					}
					// call the super method to generate the common part of XML
					xmlValue = TextContentRow.__overridden.convertToXML.call(this, xmlValue);
				} catch (err) {
					DocTableXUtils.log('Problem in TextContentRow.convertToXML:\n' + err);
					xmlValue = '';
				}
				return xmlValue;
			};

		}; // end of mixin body


		/**
		 * Mixin constructor/initializer.
		 * (class method)
		 */
		TextContentRow.__initMixin = function (options) {

			TextContentRowView.__initMixin.call(this.view, this.renderOptions); // initializer of the added view mixin functionality

			// Check if we have to retain the text content status. 
			if (options.origDocRow && options.origDocRow.retainTextContentStatus) {
				this.isRetainingTextContentStatus = options.origDocRow.retainTextContentStatus == true || options.origDocRow.retainTextContentStatus == 'true';
			} else {
				this.isRetainingTextContentStatus = false;
			}
			
			// When language is added from language indicator.
			pcmEventBus.on("PCM.TextContent.langAdded", _.bind(function(){
				this.setIsChanged();
			}, this));
			
			// When language is removed from language indicator.
			pcmEventBus.on("PCM.TextContent.langRemoved", _.bind(function(){
				this.setIsChanged();
			}, this));

			return this;
		};

		return TextContentRow;
	});