define(["jquery",
		"doctablex/DocTableXUtils",
		"doctablex/TextContentTableXView",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"utils/AttributeEventBus",
		"packaging-content-management/PCMEventBus"
		],
function ($,
	DocTableXUtils,
	TextContentTableXView,
	languageService,
	LanguageUtils,
	AttributeEventBus,
	pcmEventBus) {
	
	/**
	 * TextContentTableX.js
	 *
	 * Class (mixin) designed to extend DocTableX to support PCM (text content) specific logic and features of text statements. 
	 * Use DocTableXFactory to create instances of DocTableX enhanced by methods and members defined here.
	 *
	 * @author JANH, KADA - February 2015
	 */

	"use strict";

	var TextContentTableX = function() {

		this.__isMixin = true;   //JANH debug then DEL!!!

		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by
		 * 'options' (constructor parameter) from the server.
		 */
		this._initializeMixinFromServer = function() {
//			DocTableX.prototype._initializeFromServer.call(this);   //JANH !!!!!
		   
			// Retrieve the languages.
			languageService.getLanguages(this.projectID, function (projMasterLangs, projTransLangs){
				
				// Remember a reference to these project languages.
				this._projMasterLangs = projMasterLangs;
				this._projTransLangs = projTransLangs;
				
				// If no master or translation languages have been set yet ...
				if (!this._masterLangsSet) {
					// Set the available master language to those of the project.
					this._masterLangs = this._projMasterLangs;
					// Update the language dropdowns with the project master languages.
					this.view.updateLanguageDropDowns();
				}
				if (!this._transLangsSet) {
					// Set the available translation language to those of the project.
					this._transLangs = this._projTransLangs;
					// Update the language dropdowns with the project translation languages.
					this.view.updateLanguageDropDowns();
				}
				
			}.bind(this));
			
		    // Listen to attributes that are changed.
		    AttributeEventBus.getInstance().onChangeAttrValueListener(this.onAttributeChange.bind(this));
			
			// Listen to categories that are loaded.
		    AttributeEventBus.getInstance().onCategoryLoadedListener(this.onCategoryLoaded.bind(this));
			
			// Get all the already loaded attribute categories.
			var loadedCategories = AttributeEventBus.getInstance().getLoadedCategories();
			$.each(loadedCategories, function (index, loadedCategory) {
				// Process them.
				this.onCategoryLoaded(loadedCategory);
			}.bind(this));
			
			
			
			
		};
		
		/**
		 * When attributes are loaded.
		 */
		this.onCategoryLoaded = function(srcACatObj) {
			
			// Only continue if this attribute category can affect the languages of this table.
			if (!this._canAffectLanguages(srcACatObj)) {
				return;
			}
			
			// If there are attributes in this category.
			if (srcACatObj && srcACatObj.attributes && srcACatObj.attributes.length > 0) {
				// iterate over the attributes.
				for (var i=0; i < srcACatObj.attributes.length; i++) {
					
					var attObj = srcACatObj.attributes[i];
					
					// Only listen to changes on the "MASTER_TEXT_LANGUAGE" and
					// "TRANSLATION_LANGUAGE" attributes.
					var masterLangChange = false;
					if ('MASTER_TEXT_LANGUAGE' == attObj.name) {
						masterLangChange = true;
					} else if ('TRANSLATION_LANGUAGE' == attObj.name) {
						masterLangChange = false;
					} else {
						continue;
					}
					
					// Figure out which languages were set.
					var currentLangDesc = attObj.value.split(" | ");
					
					// Map the language descriptions to actual language objects.
					// We need all the languages in WebCenter for this.
					(function (isMasterLangChange, currentLanguages) {
						languageService.getAllLanguages(this.projectID,	isMasterLangChange, function(allLangs) {
							var currentLang = [];
							for ( var i = 0; i < currentLanguages.length; i++) {
								var lang = LanguageUtils.getLangForDescription(allLangs, currentLanguages[i]);
								if (lang) {
									currentLang.push(lang);
								}
							}
							
							// Update the dropdowns.
							if (isMasterLangChange) {
								this._masterLangs = currentLang;
								this._masterLangsSet = true;
								this.view.updateLanguageDropDowns();
							} else {
								this._transLangs = currentLang;
								this._transLangsSet = true;
								this.view.updateLanguageDropDowns();
							}
							
							// Add these languages from the documents.
							this.addLanguageToAllDocs(currentLang, isMasterLangChange);
							
						}.bind(this));
					}.bind(this))(masterLangChange, currentLangDesc);
				}
			}
		};
		
		/**
		 * When attributes are changed, this table has to react on it (e.g. when the "MASTER_TEXT_LANGUAGE" 
		 * and "TRANSLATION_LANGUAGE" attributes change ...)
		 * 
		 * @param srcID {String} - The ID of the attribute.
		 * @param attObj {Object} - The changed attribute object.
		 */
		this.onAttributeChange = function(srcID, attObj) {

			// Let us check wether this attribute can affect this table.
			if ("projectAttrUID:" + this.projectID + "_attr:" + attObj.id != srcID &&
					"taskAttrUID:" + this.taskID + "_attr:" + attObj.id != srcID) {
				return;
			}

			// Only listen to changes on the "MASTER_TEXT_LANGUAGE" and
			// "TRANSLATION_LANGUAGE" attributes.
			var masterLangChange = false;
			var currentLang = [];
			if ('MASTER_TEXT_LANGUAGE' == attObj.name) {
				masterLangChange = true;
				currentLang = this._masterLangs;
			} else if ('TRANSLATION_LANGUAGE' == attObj.name) {
				currentLang = this._transLangs;
			} else {
				return;
			}


			// Start figuring out with languages were added or removed.

			// Get the descriptions of the new languages from the attribute.
			var newLangDesc = attObj.value.split(" | ");

			// Figure out which languages were removed.
			var removedLang = [];
			$.each(currentLang, function(index, lang) {
				for ( var i = 0; i < newLangDesc.length; i++) {
					if (lang.langDesc == newLangDesc[i]) {
						return;
					}
				}
				removedLang.push(lang);
			}.bind(this));

			// Figure out which language descriptions were added.
			var addedLangDesc = [];
			for ( var i = 0; i < newLangDesc.length; i++) {
				var langFound = false;
				$.each(currentLang, function(index, lang) {
					if (lang.langDesc == newLangDesc[i]) {
						langFound = true;
						return;
					}
				});
				if (!langFound) {
					addedLangDesc.push(newLangDesc[i]);
				}
			}

			// When nothing really changed ...
			if (addedLangDesc.length == 0 && removedLang.length == 0) {
				return; // Just return.
			}

			// Map the language descriptions to actual language objects.
			// We need all the languages in WebCenter for this.
			languageService.getAllLanguages(this.projectID,	masterLangChange, function(allLangs) {

				// Start searching for the passed language description in the "All Languages" array.
				var addedLang = [];
				for ( var i = 0; i < addedLangDesc.length; i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, addedLangDesc[i]);
					if (lang) {
						addedLang.push(lang);
					}
				}

				// Now that we have the passed language objects for the languages that need to be added/removed ...

				var masterLangToSelect = null;
				var transLangToSelect = null;
				// Add/remove these languages from our
				// internal project language variables
				if (addedLang.length > 0) {
					if (masterLangChange) {
						this._masterLangs = currentLang.concat(addedLang);
						masterLangToSelect = addedLang[addedLang.length - 1].langCode;
					} else {
						this._transLangs = currentLang.concat(addedLang);
						transLangToSelect = addedLang[addedLang.length - 1].langCode;
					}
				}
				if (removedLang.length > 0) {
					if (masterLangChange) {
						LanguageUtils.removeFrom(this._masterLangs, removedLang);
					} else {
						LanguageUtils.removeFrom(this._transLangs, removedLang);
					}
				}

				// Add/Remove these languages from the documents.
				this.addLanguageToAllDocs(addedLang, masterLangChange);
				this.removeEmptyLanguageFromAllDocs(removedLang, masterLangChange);

				// Update the dropdowns.
				this.view.updateLanguageDropDowns(masterLangToSelect, transLangToSelect);

			}.bind(this));
		};
	
	    /**
		 * Handling added languages from project attributes
		 */
		this.addLanguageToAllDocs = function(addedLangs, isMaster) {
	
			// Add the language to all document rows.
			for (var dr = 0; dr < this.docRows.length; dr++) {
				
				// Only add if the current row has a text content document.
				if (!this.docRows[dr].textContent) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (this.docRows[dr].isRemoteDocument()) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (!this.docRows[dr].textContent.canManage().allowed) {
					continue;
				}
				
				// Start adding the languages.
				var docRow = this.docRows[dr];
				var textContent = docRow.textContent;
				for (var i=0; i < addedLangs.length; i++) {
					textContent.addLanguage(addedLangs[i], isMaster, function (element) {
						if (element) {
							pcmEventBus.emit("PCM.TextContent.langAdded", textContent, element);
							docRow.setIsChanged();
						}
					});
				}
			}
	    };
	    
	
		/**
		 * Handling removed languages from project attributes
		 */
	    this.removeEmptyLanguageFromAllDocs = function(removedLangs, isMaster) {
	    	
	    	// Remove the language to all document rows.
			for (var dr = 0; dr < this.docRows.length; dr++) {
				
				// Only remove if the current row has a text content document.
				if (!this.docRows[dr].textContent) {
					continue;
				}
				
				// Only remove if the current row is not in a remote project.
				if (this.docRows[dr].isRemoteDocument()) {
					continue;
				}
				
				// Only remove if we can manage the document.
				if (!this.docRows[dr].textContent.canManage().allowed) {
					continue;
				}
				
				// Start removing the languages.
				var textContent = this.docRows[dr].textContent;
				for (var i=0; i < removedLangs.length; i++) {
					var element = textContent.removeEmptyLanguage(removedLangs[i], isMaster);
					if (element) {
						pcmEventBus.emit("PCM.TextContent.langRemoved", textContent, element);
					}
					this.docRows[dr].setIsChanged();
				}
			}
	    };
	    
	    /**
	     * Return whether this given attribute category could influence the languages of this table.
	     */
	    this._canAffectLanguages = function (srcAcat) {
    		if (srcAcat.aCatOptions.isDocRefPropertiesACat) {
    			return false;
    		} else if (this.parentDocRow && this.parentDocRow.newDocTmpID) {
	    		return false;
    		} else if (this.docVersionID) {
    			return false;
	    	} else if (this.taskID) {
	    		return srcAcat.taskID == this.taskID;
	    	} else if (this.projectID) {
	    		return srcAcat.projectID == this.projectID;
	    	} else {
	    		return false;
	    	}
	    };
	    
	    /**
	     * Recursive function to check if any action is performed in any document that requires password validation.
	     * 
	     * @returns {Boolean} true if password is required, otherwise false.
	     */
	    this.checkIfPasswordRequired = function() {
	    	var isPasswordRequired = false;
	    	
	    	// Iterate through each docrow
			_.each(this.docRows, function(docRow) {
				// check if textContent is present and password is required for approval
				if(!isPasswordRequired && docRow && docRow.textContent) {
					isPasswordRequired = docRow.textContent.isPasswordRequiredForApproval();
				}
				// If password is required do not proceed [skip iteration]
				if(!isPasswordRequired && docRow.aCatObj) {
					_.each(docRow.aCatObj.attributes, function(attribute) {
		    			// If password is not required at current category, check  in attribute category of each of its docRow
		    			if(!isPasswordRequired && attribute.docTable) {
		    				isPasswordRequired = attribute.docTable.checkIfPasswordRequired();
		    			}
		    		});
				}
			});
			
			return isPasswordRequired;
	    };
	    
	    /**
		 * Get all the master languages of this table.
		 */
		 this.getMasterLanguages = function() {
			 return this._masterLangs;
		 };
		 
		 /**
		 * Get all the translation languages of this table.
		 */
		 this.getTranslationLanguages = function() {
			 return this._transLangs;
		 };
	    
		/**
		 * Get all the project translation languages.
		 */
		 this.getProjTranslationLanguages = function() {
			 return this._projTransLangs;
		 };
	    
		/**
		 * Get all the project master languages.
		 */
		 this.getProjMasterLanguages = function() {
			 return this._projMasterLangs;
		 };
	    
		 
		 /**
		  * 
		  * Handles request to make given remote document local and replace document to 'this.projectID' project.
		  * 
		  * @param {int} rowIdx - Index of the document row, where is the document to be made local.
		  * @param {String} flowType - This param is required for localRemote Flow. Based on this param docrow object initiated.
		  * @param {String} operation - Operation which is clicked during replace.
		  * @param {String} templateName - TemplateName will be passed only in case of 'DocumentTemplate' (NEW) operation.
		  * @param {DocumentRowNew} docRow - Preinitialized document row based on the "SavedSearch" document source.
		  * @param {Boolean} retainTextContentStatus - When a new TextContent object is requested, should the status of the original be retained? 
		  */
		  if (this.localRemoteButtonClicked) { TextContentTableX.__overridden.localRemoteButtonClicked = this.localRemoteButtonClicked; }
		  this.localRemoteButtonClicked = function (rowIdx, flowType, operation, templateName, searchDocRow, retainTextContentStatus) {
			  this.retainTextContentStatus = retainTextContentStatus;
			  
			  	// Get the affected row.
		        var docRow = this.docRows[rowIdx];
			  
		        // Indicate that we should retain the text content status.
		        if (retainTextContentStatus) {
		        	docRow.retainTextContentStatus = true;
		        } else {
		        	docRow.retainTextContentStatus = false;
		        }
		        
		        // Call the supper implementation.
		        TextContentTableX.__overridden.localRemoteButtonClicked.call(this, rowIdx, flowType, operation, templateName, searchDocRow);
		  };
		 
		 return this;
	};

	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentTableX.__initMixin = function(options) {
        if (!options) {
            // nothing to initialize from
            return;
        }

		// This variable holds the default language code for each collumn.
		this._defaultSelectedLanguageCodes = {};
		
		// These variables contain the master and translation languages of the project.
		this._projMasterLangs = [];
		this._projTransLangs = [];
		
		// These variables contain the master and translation languages of this table.
		this._masterLangs = [];
		this._transLangs = [];
		
		// These variables contain a specific master and translation languages of this table was set.
		this._masterLangsSet = false;
		this._transLangsSet = false;
		
		// TODO: Make this work for not only document versions but also projects......
		//Get the immediate parent doc. ver. Id.
		this.parentDocVerId = this.parentDocVerIDs[this.parentDocVerIDs.length-1];
	
		TextContentTableXView.__initMixin.call(this.view, this.renderOptions);   // initializer of the added view mixin functionality

		// load additional required info from the server via AJAX (if needed)
		// note: this should be called only after the 'this.view' is properly initialized!
		this._initializeMixinFromServer();
	};
	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContentTableX.__overridden = {
	};

	
	return TextContentTableX;
});