define(["underscore", "jquery", "Template"], function (_, $, Template) {

	"use strict";

	/**
	 * A class that can be used to template inifinte scrolling lists. To achieve this you can use d.eachPaged in your template to run over all elements on the current page.
	 * The template will be rerendered and appended to the context every time the scrollhandle of the scrollingContext 
	 * (can be set with setScrollingContext) reaches the bottom of the scrollbar.
	 * 
	 * @class
	 * @augments Template
	 * @exports InfiniteScrollingTemplate
	 * 
	 * @param {I18n} i18n          
	 * @param {String} template      
	 * @param {Number} [initialAmount = 200]           - the amount of entries on the first page
	 * @param {Number} [stepCount = initialAmount]     - the amount of entries that should be added when the scroll bar is at the bottom
	 */
	var InfiniteScrollingTemplate = function (i18n, template, initialAmount, stepCount) {
		Template.call(this, i18n, template);

		/**
		 * @member {Number}
		 * @private
		 */
		this._initialAmount = initialAmount || 200;

		/**
		 * @member {Number}
		 * @private
		 */
		this._stepCount = stepCount || this._initialAmount;
		
		/**
		 * @member {Number} the current page
		 * @private
		 */
		this._currentPage = 1;
		
		/**
		 * @member {JQueryObject} - the context that will trigger the renderNext method on scroll
		 * @private
		 */
		this._scrollingContext = null;
		
		/**
		 * This function can be used in template to get all values of an array on the current page
		 */
		this.d.eachPaged = _.bind(this._eachPageEntries, this);

	};

	InfiniteScrollingTemplate.prototype = Object.create(Template.prototype);
	InfiniteScrollingTemplate.prototype.constructor = InfiniteScrollingTemplate;

	/**
	 * @override
	 */
	InfiniteScrollingTemplate.prototype.render = function () {
		this._currentPage = 1;
		if (this._context) {
			this._context.html(this._parsedTemplate(this.d));
		}
	};

	/**
	 * Sets the scrolling context for the template - ie. binds the 'scroll' event
	 * @param {JQueryObject} scrollingContext 
	 */
	InfiniteScrollingTemplate.prototype.setScrollingContext = function (scrollingContext) {
		scrollingContext.on("scroll", _.partial(function (infiniteScrollingTemplate, e) {
			if (this.scrollHeight - this.offsetHeight - this.scrollTop < 5) {
				infiniteScrollingTemplate._renderNext();
			}
		}, this));
	};

	/**
	 * Appends the rendering of the next page to the context
	 */
	InfiniteScrollingTemplate.prototype._renderNext = function () {
			this._currentPage++;
			this._context.append(this._parsedTemplate(this.d));
	};
	
	/**
	 * Calls callback for each object in array that is on the current page
	 * @param {Array} array  
	 * @param {Function} callback
	 */
	InfiniteScrollingTemplate.prototype._eachPageEntries = function (array, callback) {
		var from = this._currentPage === 1 ? 0 : this._initialAmount + (this._currentPage - 1) * this._stepCount;
		var amount = this._currentPage === 1 ? this._initialAmount : this._stepCount;
		for (var i = from; i < from + amount && i < array.length; i++) {
			callback(array[i]);
		}
	};

	return InfiniteScrollingTemplate;

});
