/**
 *
 * For SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies the EditSession (control)
 *   - to make the cursor appear in the correct place and
 *   - make it possible to select a SmartName by double clicking it
 *   - allow the user to drag text with SmartNames from one place in the field to another...
 * 
 * Also adds methods to enable the SmartName functionality and check if it is enabled...
 *
 * @author JEPE
 */
define(["ace/edit_session", "ace/range", "./smartnameanchorcontrol", "./smartnameextensionutils"], function(edit_session, range, smartnameanchorcontrol, smartnameextensionutils) {
    (function(){
        var Range = range.Range;
        var SMARTNAMECHAR = smartnameextensionutils.SMARTNAMECHAR;

        /*
         * New API
         */

        /**
         * Are SmartNames enabled in this input field?
         *
         * @returns {Boolean} are SmartNames enabled in this edit_session?
         */
        this.isSmartNamesEnabled = function() {
            if(this._smartNamesEnabled===undefined){
                return false;
            }else{
                return this._smartNamesEnabled;
            }
        };

        /**
         * Enable the SmartNames in this editor/edit_session
         * 
         * @param {SmartNameEncoding} the encoding to use
         */
        this.enableSmartNames = function (encoding){
            if(encoding!==undefined){
                this._smartNamesEnabled = true;
				this.getDocument().enableSmartNames();
                this._smartNameEncoding = encoding;
            }
        };

        /**
         * Gets the current SmartName encoding
         *
         * @returns {SmartNameEncoding} the encoding of SmartNames to use
         */
        this.getSmartNameEncoding = function (){
            return this._smartNameEncoding;
        };

        /*
         * Override stuff
         */

        /** private function copied from edit_session */
        // For every keystroke this gets called once per char in the whole doc!!
        // Wouldn't hurt to make it a bit faster for c >= 0x1100
        function isFullWidth(c) {
            if (c < 0x1100)
                return false;
            return c >= 0x1100 && c <= 0x115F ||
                   c >= 0x11A3 && c <= 0x11A7 ||
                   c >= 0x11FA && c <= 0x11FF ||
                   c >= 0x2329 && c <= 0x232A ||
                   c >= 0x2E80 && c <= 0x2E99 ||
                   c >= 0x2E9B && c <= 0x2EF3 ||
                   c >= 0x2F00 && c <= 0x2FD5 ||
                   c >= 0x2FF0 && c <= 0x2FFB ||
                   c >= 0x3000 && c <= 0x303E ||
                   c >= 0x3041 && c <= 0x3096 ||
                   c >= 0x3099 && c <= 0x30FF ||
                   c >= 0x3105 && c <= 0x312D ||
                   c >= 0x3131 && c <= 0x318E ||
                   c >= 0x3190 && c <= 0x31BA ||
                   c >= 0x31C0 && c <= 0x31E3 ||
                   c >= 0x31F0 && c <= 0x321E ||
                   c >= 0x3220 && c <= 0x3247 ||
                   c >= 0x3250 && c <= 0x32FE ||
                   c >= 0x3300 && c <= 0x4DBF ||
                   c >= 0x4E00 && c <= 0xA48C ||
                   c >= 0xA490 && c <= 0xA4C6 ||
                   c >= 0xA960 && c <= 0xA97C ||
                   c >= 0xAC00 && c <= 0xD7A3 ||
                   c >= 0xD7B0 && c <= 0xD7C6 ||
                   c >= 0xD7CB && c <= 0xD7FB ||
                   c >= 0xF900 && c <= 0xFAFF ||
                   c >= 0xFE10 && c <= 0xFE19 ||
                   c >= 0xFE30 && c <= 0xFE52 ||
                   c >= 0xFE54 && c <= 0xFE66 ||
                   c >= 0xFE68 && c <= 0xFE6B ||
                   c >= 0xFF01 && c <= 0xFF60 ||
                   c >= 0xFFE0 && c <= 0xFFE6;
        };

        /**
         * Overrides the function $getStringScreenWidth in EditSession to make the cursor & selection appear in the correct place.
         * (A SmartName is actually just one character in the document...)
         */
        this.$getStringScreenWidth = function(str, maxScreenColumn, screenColumn) {
            if (maxScreenColumn == 0)
                return [0, 0];
            if (maxScreenColumn == null)
                maxScreenColumn = Infinity;
            screenColumn = screenColumn || 0;
            
            var self = this;
            function updateScreenColumn(c) {
            	if (c == 9) {
                    screenColumn += self.getScreenTabSize(screenColumn);
                } else if (c >= 0x1100 && isFullWidth(c)) {
                    screenColumn += 2;
                } else {
                    screenColumn += 1;
                }
            }

            var c, column;
            for (column = 0; column < str.length; column++) {
                c = str.charCodeAt(column);
                if (c == SMARTNAMECHAR.charCodeAt(0) && this.isSmartNamesEnabled()) {// CODE MOD
                    // NOTE: not correct in case we allow multiple lines in a SmartField edit field or if wrapping is enabled... 
                    // In that case: row!=0 OR column != column in document.
                    // (!) Will probably need to modify ace framework if we want to support that (three or more small changes to pass the row to this method)
                    var info = smartnameanchorcontrol.getSmartNameInfoAt(this.getDocument(), 0, column);
                    var display = smartnameextensionutils.getSmartNameDisplayString(this.getDocument(), info);
                    for (var subcolumn = 0;subcolumn < display.length; subcolumn++) {
                    	var c2 = display.charCodeAt(subcolumn);
                    	updateScreenColumn(c2);
                    }
                } else {
                	updateScreenColumn(c);
                }
                if (screenColumn > maxScreenColumn) {
                    break;
                }
            }

            return [screenColumn, column];
        };

        /*
         * Override the function getWordRange in EditSession to select SmartNames when double clicking...
         */
        this.getWordRangeBase = this.getWordRange;
        this.getWordRange = function(row, column) {
            var line = this.getLine(row);
            if(line.charAt(column)==SMARTNAMECHAR && this.isSmartNamesEnabled()){
                return new Range(row, column, row, column+1);
            }

            return this.getWordRangeBase(row, column);
        };

        /*
         * Override the moveText function in EditSession to drag text in the SmartNameField 
         * (and drag the SmartName information with them)
         */
        this.moveText = function(fromRange, toPosition, copy) {
            var text = this.getTextRange(fromRange);
            var completeText = smartnameextensionutils.recompileText(this, this.getDocument(), fromRange.start.column, text, true);// CODE MOD
            //console.log("The text: "+completeText);
            var folds = this.getFoldsInRange(fromRange);

            var toRange = Range.fromPoints(toPosition, toPosition);
            if (!copy) {
                this.remove(fromRange);
                var rowDiff = fromRange.start.row - fromRange.end.row;
                var collDiff = rowDiff ? -fromRange.end.column : fromRange.start.column - fromRange.end.column;
                if (collDiff) {
                    if (toRange.start.row == fromRange.end.row && toRange.start.column > fromRange.end.column)
                        toRange.start.column += collDiff;
                    if (toRange.end.row == fromRange.end.row && toRange.end.column > fromRange.end.column)
                        toRange.end.column += collDiff;
                }
                if (rowDiff && toRange.start.row >= fromRange.end.row) {
                    toRange.start.row += rowDiff;
                    toRange.end.row += rowDiff;
                }
            }

            var result = smartnameextensionutils.parseText(this, completeText, true);// CODE MOD
            smartnameextensionutils.startSmartNameChangeGroup(this.getDocument());// CODE MOD
            text = result.text;// CODE MOD: should be the same already (but just in case)
            toRange.end = this.insert(toRange.start, text);
            //console.log(result.smartnames);
            var rangeAnchors = new Range(0, toRange.start.column, 0, toRange.start.column+result.text.length);// CODE MOD
            smartnameextensionutils.addSmartNameAnchors(this.getDocument(), rangeAnchors, result.smartnames);// CODE MOD
            smartnameextensionutils.stopSmartNameChangeGroup(this.getDocument(), rangeAnchors);// CODE MOD
            if (folds.length) {
                var oldStart = fromRange.start;
                var newStart = toRange.start;
                var rowDiff = newStart.row - oldStart.row;
                var collDiff = newStart.column - oldStart.column;
                this.addFolds(folds.map(function(x) {
                    x = x.clone();
                    if (x.start.row == oldStart.row)
                        x.start.column += collDiff;
                    if (x.end.row == oldStart.row)
                        x.end.column += collDiff;
                    x.start.row += rowDiff;
                    x.end.row += rowDiff;
                    return x;
                }));
            }

            return toRange;
        };

    }).call(edit_session.EditSession.prototype);
});