/**
 *
 * For SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies the Editor (main class)
 *
 * @author JEPE
 */
define(["ace/editor", "ace/range", "./smartnameinfo", "./smartnameextensionutils", "./smartnametooltip", "utils/extendedstring"], function(editor, range, SmartNameInfo, smartnameextensionutils, smartnametooltip) {
    var Range = range.Range;
    var SmartNameTooltip = smartnametooltip.SmartNameTooltip;

    (function(){

        /*
         *
         * NEW API
         *
         */

        /**
         * New method to add SmartNames.
         */
        this.insertSmartName = function(smartname){
            smartnameextensionutils.startSmartNameChangeGroup(this.getSession().getDocument());
            this.insert(smartnameextensionutils.SMARTNAMECHAR);
            var pos = this.getCursorPosition().column-1;

            var rangeAnchors = new Range(0, pos, 0, pos+1);
            smartnameextensionutils.addSmartNameAnchors(this.getSession().getDocument(), rangeAnchors, [{index: 0, info: new SmartNameInfo(smartname)}]);
            smartnameextensionutils.stopSmartNameChangeGroup(this.getSession().getDocument(), rangeAnchors);
            
            this._triggerSmartNameEditing({row:0, column:pos});
        };

        /**
         * Get the content of this SmartName input...
         */
        this.getSmartNameEncodedValue = function(){
            return smartnameextensionutils.recompileText(this.getSession(),this.getSession().getDocument(), 0, this.getValue());
        };

        /**
         * Pasting etc with SmartNames will cause multiple "change" events.
         * While "isChangeReady" is false, we ignore these events and don't pass them to the user of the SmartNameInput.
         */
        this.isChangeReady = function(){
            return smartnameextensionutils.isInSmartNameChangeReady(this.getSession().getDocument());
        };

        /**
         * Set the content of this SmartName input...
         */
        this.setSmartNameEncodedValue = function(text){
            var result = smartnameextensionutils.parseText(this.getSession(), text);
            smartnameextensionutils.startSmartNameChangeGroup(this.getSession().getDocument());
            this.setValue(result.text);
            var rangeAnchors = new Range(0, 0, 0, result.text.length);
            smartnameextensionutils.addSmartNameAnchors(this.getSession().getDocument(), rangeAnchors, result.smartnames);
            smartnameextensionutils.stopSmartNameChangeGroup(this.getSession().getDocument(), rangeAnchors);
            this.navigateFileEnd();
        };
        
        /**
         * Enable the SmartName tooltip.
         * 
         * Callback is a function that takes two arguments:
         *  - argument 1: {String} the full name of the SmartName
         *  - argument 2: {Boolean} whether this is the active SmartName
         *  - argument 3: {jQuery} the jQuery element to fill
         * 
         * The callback needs to return true if there is content to be shown
         * The callback needs to return false if no tooltip should be shown
         * 
         * @param {Function} callback - a function (see description above)
         */
        this.configureSmartNameTooltip = function(callback){
        	this.smartnameTooltipFunction = callback;
        	
        	// Make a tooltip for the SmartNames
        	new SmartNameTooltip(this);
        };

        /**
         * Initialize listening to SmartName clicks and set the SmartName Edit Listener.
         * Callback will be executed when the user clicks a SmartName
         */
        this.initSmartNameEditListener = function(callback) {
        	this.onEditSmartNameCallback = callback;
        	
        	// Listen to click for Editing SmartNames
        	this.on("click", function(mouseEvent){
        		var pos = mouseEvent.getDocumentPosition();
        		this._triggerSmartNameEditing(pos);
        	}.bind(this));
        };
        
        /**
         * Trigger SmartName editing for the SmartName which is at a certain position.
         * 
         * @param {Object} pos - object with row and column.
         */
        this._triggerSmartNameEditing = function(pos){
        	if(this.onEditSmartNameCallback) {
        		var api = smartnameextensionutils.getSmartNameEditAPI(this, pos.row, pos.column);
        		if(api!=null) {
        			this.onEditSmartNameCallback(api);
        		}
        	}
        };

        /*
         *
         * Override stuff
         *
         */


        /**
         * Modify editor, add & modify insert function
         *
         * Why? We don't allow multiple lines + new method to insert a SmartName.
         */

        this.insertBase = this.insert;

        this.insert = function(text, pasted) {
            if(this.getSession().isSmartNamesEnabled()){
                // Multiple lines is not supported (so we make sure it never happens)
                // Note: if already parsed as some smartnamestring these characters are already removed... 
                // This is only in the case you type it...
                text = text.replaceAll("\r", "").replaceAll("\n", "");
            }
            this.insertBase(text, pasted);
        };



        /**
         * Modify editor: onPaste & getCopyText
         *
         * Why? Because we want to copy the SmartNames also...
         */

        var onPasteBase = this.onPaste;

        this.onPaste = function(text) {
            var result = smartnameextensionutils.tryParseText(this.session, text, true);
            smartnameextensionutils.startSmartNameChangeGroup(this.session.getDocument());

            onPasteBase.call(this, result.text);

            // get the startindex on the line...
            var pasteIndex = this.getCursorPosition().column-result.text.length;

            // add anchors...
            var rangeAnchors = new Range(0, pasteIndex, 0, pasteIndex+result.text.length);
            smartnameextensionutils.addSmartNameAnchors(this.session.getDocument(), rangeAnchors, result.smartnames);
            smartnameextensionutils.stopSmartNameChangeGroup(this.session.getDocument(), rangeAnchors);
		};

        this.getCopyText = function() {
            var startIndex = this.getSelectionRange().start.column;
            var text = this.getSelectedText();

            text = smartnameextensionutils.recompileText(this.getSession(), this.getSession().getDocument(), startIndex, text, true);// CODE MOD

            this._signal("copy", text);
            return text;
        };
    }).call(editor.Editor.prototype);
});