/**
 *
 * For SmartName support in ace editor
 * 
 * Modification to the ace framework.
 *
 * Modifies the dragdrop_handler to make it possible to drag & drop text with SmartNames into the field.
 *
 * @author JEPE
 */
define(["ace/config", "ace/lib/event", "ace/range", "./smartnameanchorcontrol", "./smartnameextensionutils"], function(config, event, range, smartnameanchorcontrol, smartnameextensionutils) {
    var Range = range.Range;
    
    /*
     * BECAUSE THE dragdrop_handler is written with private functions (which I cannot override),
     * I attach these events also to the editor.
     * These will be called AFTER the drop has been handled.
     *
     * CONSEQUENCE: KNOWN ISSUE => A change event will be thrown twice. Once with the wrong information, once with the SmartNames converted...
     */
    config.on("editor", function(editor) {

        var mouseTarget = editor.container;
        var isInternal;

        /**
         * When starting to drag, put the encoded text into the drag event.
         * This will allow you to drag the field content with SmartName to a text editor (like notepad)
         */
        var onDragStart = function(e) {
            // webkit workaround, see this.onMouseDown
            if (this.cancelDrag || !mouseTarget.draggable) {
                return;
            }
            var startIndex = editor.session.selection.getRange().start.column;

            var dataTransfer = e.dataTransfer;
            dataTransfer.clearData();

            var text = editor.session.getTextRange();
            text = smartnameextensionutils.recompileText(editor.session, editor.session.getDocument(), startIndex, text, true);

            dataTransfer.setData("Text", text);

            //note: check if dragging into it works correct in webkit browsers 
            // (otherwise need to mirror code in dragdrop_handler better)
            isInternal = true;
        }

        /**
         * Keep the same state as dragdrop_handler
         */
        var onDragEnd = function(e) {
            isInternal = false;
        }

        /**
         * When dropping text from another field or a text editor,
         *   parse it to get the SmartNames in it... Then add those SmartNames to the editor...
         */
        var onDrop = function(e) {
            if(!isInternal){
                var droppedText = editor.getSelectedText();

                //console.log("Dropped from outside... "+droppedText);

                var result = smartnameextensionutils.tryParseText(editor.session, droppedText, true);
                if(result.text === droppedText){
                    //console.log("No valid SmartNames in the dropped text...");
                    return;// do not change anything...
                }
                //console.log("Replace with text: "+result.text);

                smartnameextensionutils.startSmartNameChangeGroup(editor.session.getDocument());
                // get the startindex on the line...
                var range = editor.getSelectionRange();

                // remove the (already dropped) value in the editor
                if (!range.isEmpty())
                    editor.session.remove(range);

                // insert the new text
                editor.session.insert(range.start, result.text);

                // add anchors...
                var rangeAnchors = new Range(0, range.start.column, 0, range.start.column+result.text.length);
                smartnameextensionutils.addSmartNameAnchors(editor.session.getDocument(), rangeAnchors, result.smartnames);
                smartnameextensionutils.stopSmartNameChangeGroup(editor.session.getDocument(), rangeAnchors);
            }
        }

        event.addListener(mouseTarget, "dragstart", onDragStart.bind(this));
        event.addListener(mouseTarget, "dragend", onDragEnd.bind(this));
        event.addListener(mouseTarget, "drop", onDrop.bind(this));
    });
    
});