/**
 *
 * For SmartName support in ace editor
 * 
 * This module defines a special kind of anchor. (it extends Anchor)
 * An anchor in ace can be put on a certain position in the text and will follow that character.
 * So when text is inserted or removed, the anchor will try to stay at the same character.
 *
 * This special anchor will contain the SmartName information.
 * A SmartName is actually just one character in the text .
 * (which makes sure you cannot select half of it or edit the content)
 *
 * This Anchor will thus follow the SmartName character.
 *
 * This Anchor is different from normal Anchors because 
 *   - it can be deleted when the character is was following is deleted.
 *   - it is managed by "SmartNameControl"
 *   - it does not listen to events but gets the information about 
 *       what has changed in the editor from the SmartNameControl class.
 * 
 * @author JEPE
 */

define(["ace/anchor", "ace/lib/oop", "./mod-ace-anchor"],function(anchor, oop) {
"use strict";
    
    // We extend Anchor (which allows us to follow a certain character)
    var Anchor = anchor.Anchor;
    
    /**
     * Constructor for a SmartNameAnchor, 
     *   contains the information of a SmartName in the editor
     * 
     * These Anchors will be managed by "SmartNameControl".
     * 
     * @param doc {Document} The document to attach this SmartNameAnchor to
     * @param row {Number} The row of the SmartName character we need to follow
     * @param column {Number} The column of the SmartName character we need to follow
     * @param info {SmartNameInfo} The information about the SmartName
     */
    var SmartNameAnchor = function(doc, row, column, info) {
        this._info = info;
        this._isvalidanchor = true;
        Anchor.call(this, doc, row, column);
    };
    // As explained, we inherit from Anchor
    oop.inherits(SmartNameAnchor, anchor.Anchor);
    
    // New/overriden functions
    (function() {
        
        /**
         * Get the SmartName information from the SmartNameAnchor
         * 
         * @returns {SmartNameInfo} the information
         */
        this.getSmartNameInfo = function(){
            return this._info;
        };
        
        /**
         * Set the SmartName information in the SmartNameAnchor
         * 
         * @param {SmartNameInfo} newInfo - the information
         */
        this.setSmartNameInfo = function(newInfo){
            this._info = newInfo;
        };
        
        /**
         * This method is called by the class the manages this Anchor.
         * The delta argument contains the information about what changed in the text.
         * 
         * This argument is used to update the position of this SmartNameAnchor.
         * 
         * @returns {Boolean} false if the text on which this SmartNameAnchor was linked is removed.
         */
        this.applyDeltas = function(delta){
            // Check if the character to which this Anchor is linked is deleted by the change.
            if( (delta.action === "remove") &&
                delta.start.column<=this.column && delta.end.column>this.column ){
            	this._isvalidanchor = false;
                // In that case, the SmartName information needs to be deleted too...
                return false;
            }
            
            // Update the position
            Anchor.prototype.onChange.call(this, delta);
            
            // This is still valid SmartName information.
            return true;
        };
        
        /**
         * Whether this anchor is still valid...
         */
        this.isValid = function(){
        	return this._isvalidanchor;
        };
        
        /**
         * We don't update our position anymore on events, 
         *  but on explicit calls from the class that manages this Anchor.
         * 
         * That allows the managing class to detect when an anchor needs to be deleted.
         */
        this.onChange = function(e) {}
        
    }).call(SmartNameAnchor.prototype);
    
    /*
     * return the constructor.
     */
    return SmartNameAnchor;
});