/**
 *
 * For SmartName support in ace editor
 * 
 * This module defines some functions to manage the SmartName anchors in a document.
 * SmartNameAnchors follow a character in the text and link SmartNameInfo to it.
 *
 * @author JEPE
 */

define(["./smartnameanchor", "./smartnameinfo", "ace/range"], function (SmartNameAnchor, SmartNameInfo, range) {
	"use strict";

	// We need the ace Range class.
	var Range = range.Range;

	// The functions to export...
	var exports = {};

	// An unresolved SmartName. 
	// This special SmartName will be returned when something went wrong.
	// (When we are asked to get the SmartName information for a position where there is no SmartName.)
	var illegalSmartNameInfo = new SmartNameInfo("?");
	illegalSmartNameInfo.unresolved = true;

	/**
	 * private method, gets the array which contains the anchors.
	 * The anchors will be saved in the Document object.
	 */
	function getSmartNameList(document) {
		if (document.smartnameanchors === undefined) {
			document.smartnameanchors = [];
		}
		return document.smartnameanchors;
	}

	/**
	 * Get the SmartName information linked to position (row, column) in the document.
	 * Using this information we can then correctly display the SmartName at that position.
	 * (Only the SMARTNAMECHAR will be in the text, which is not enough to display the SmartName)
	 * 
	 * @param document {Document} the document in which the SmartName is put
	 * @param row {Number} the row on which the SmartName is put
	 * @param column {Number} the column on which the SmartName is put
	 * @returns {SmartNameInfo} the information about the SmartName
	 */
	var _getSmartNameAnchorAt = function (document, row, column) {
		// get all the SmartNameAnchors in the document
		var smartnameanchors = getSmartNameList(document);
		// ... and search for the one at the correct position
		for (var i = 0; i < smartnameanchors.length; i++) {
			var anchor = smartnameanchors[i];
			var pos = anchor.getPosition();
			if (pos.row === row && pos.column === column) {
				// return the information from the anchor at that position
				return anchor;
			}
		}

		// Hmmm... We lost the SmartName information. There is no Anchor at that position.
		//console.log("Problem: SmartName at "+column+" is no longer valid.");
		return null;
	};

	/**
	 * Get the SmartName information linked to position (row, column) in the document.
	 * Using this information we can then correctly display the SmartName at that position.
	 * (Only the SMARTNAMECHAR will be in the text, which is not enough to display the SmartName)
	 * 
	 * @param document {Document} the document in which the SmartName is put
	 * @param row {Number} the row on which the SmartName is put
	 * @param column {Number} the column on which the SmartName is put
	 * @returns {SmartNameInfo} the information about the SmartName
	 */
	exports.getSmartNameInfoAt = function (document, row, column) {
		var anchor = _getSmartNameAnchorAt(document, row, column);
		if (anchor !== null) {
			return anchor.getSmartNameInfo();
		} else {
			// Hmmm... We lost the SmartName information. There is no Anchor at that position.
			//console.log("Problem: SmartName at "+column+" is no longer valid.");
			return illegalSmartNameInfo;
		}
	};

	/**
	 * Set the SmartName information linked to position (row, column) in the document.
	 * Using this information we can then correctly display the SmartName at that position.
	 * (Only the SMARTNAMECHAR will be in the text, which is not enough to display the SmartName)
	 * 
	 * @param document {Document} the document in which the SmartName is put
	 * @param row {Number} the row on which the SmartName is put
	 * @param column {Number} the column on which the SmartName is put
	 * @param info {SmartNameInfo} the information to save
	 */
	exports.setSmartNameInfoAt = function (document, row, column, info) {
		// we create a new SmartNameAnchor
		var anchor = new SmartNameAnchor(document, row, column, info);

		// get the list of SmartNameAnchors
		var smartnameanchors = getSmartNameList(document);

		// and add it to the list
		smartnameanchors.push(anchor);

		var range = new Range(row, column, row, column + 1);
		// for this new anchor, signal a change event.
		// (this is required for undo/redo support)
		document._signal("change", {
			action:"addSmartName",
			start: range.start,
			end: range.end,
			//data : deltaSignal,
			smartname: {
				row: row,
				column: column,
				info: info
			}
		});
	};

	/**
	 * Called when something in the document changes.
	 * This information will then be used to also update the SmartNamesAnchors.
	 * (And keep the SmartName information linked to the correct character)
	 * 
	 * @param document {Document} the document on which the change happened
	 * @param delta {Object} the change
	 */
	exports.applyDelta = function (document, delta) {
		// get all the SmartNameAnchors in the document
		var smartnameanchors = getSmartNameList(document);
		// ... and update their positions in the document.
		//     (also detect if the SmartNames are deleted...)
		var deletedSmartNameAnchors = [];
		for (var i = 0; i < smartnameanchors.length; i++) {
			var anchor = smartnameanchors[i];
			var isvalid = anchor.applyDeltas(delta);
			if (!isvalid) {
				// That SmartName was removed...
				// add the information to the deletedSmartNameAnchors array...
				deletedSmartNameAnchors.push(anchor);

				// delete the anchor
				smartnameanchors.splice(i, 1);
				// and continue (from the same index in the array)
				i--;
			}
		}

		// loop through all deleted anchors and signal change events for them.
		// (this is required for undo/redo support)
		for (var i = 0; i < deletedSmartNameAnchors.length; i++) {
			var anchor = deletedSmartNameAnchors[i];
			var row = anchor.row;
			var column = anchor.column;
			var info = anchor._info;

			var range = new Range(row, column, row, column + 1);
			document._signal("change", {
				start : range.start,
				end: range.end,
				//data: deltaSignal
				action: "removeSmartName",
				smartname: {
					row: row,
					column: column,
					info: info
				}
			});
		}
	};

	/**
	 * Rename ("reconfigure") a SmartName.
	 * 
	 * @param session
	 * @param row
	 * @param column
	 * @param newName
	 * @returns {Boolean}
	 */
	exports.renameSmartName = function (session, row, column, newName) {
		var anchor = _getSmartNameAnchorAt(session.doc, row, column);
		if (anchor != null) {
			var oldinfo = anchor.getSmartNameInfo();
			var newinfo = new SmartNameInfo(newName);
			if (newName === oldinfo.getFullName()) {
				return false;
			}
			anchor.setSmartNameInfo(newinfo);
			var range = new Range(row, column, row, column + 1);
			session.doc._signal("change", {
				start: range.start,
				end: range.end,
				action: "renameSmartName",
				smartname: {
					row: row,
					column: column,
					oldinfo: oldinfo,
					newinfo: newinfo
				}
			});
			return true;
		}
		return false;
	};

	/**
	 * Sets the active SmartName.
	 * 
	 * @param document
	 * @param row
	 * @param column
	 */
	exports.setActiveSmartName = function (document, row, column) {
		exports.clearActiveSmartName(document);
		var anchor = _getSmartNameAnchorAt(document, row, column);
		if (anchor != null) {
			var range =new Range(row, column, row, column + 1); 
			document.activeAnchor = anchor;
			document._signal("change", {
				start: range.start,
				end: range.end,
				action: "changeActiveSmartName",
				// no undo/redo
				ignore: true
			});
		}
	};

	/**
	 * Clear the active SmartName.
	 * 
	 * @param document
	 */
	exports.clearActiveSmartName = function (document) {
		if (document.activeAnchor && document.activeAnchor.isValid()) {
			var pos = document.activeAnchor.getPosition();
			var range = new Range(pos.row, pos.column, pos.row, pos.column + 1);
			document.activeAnchor = null;
			document._signal("change", {
				start : range.start,
				end: range.end,
				action: "changeActiveSmartName",
				//range: new Range(pos.row, pos.column, pos.row, pos.column + 1),
				// no undo/redo
				ignore: true
			});
		}
	};

	/**
	 * Returns the position of the active SmartName (or null if there is no active SmartName)
	 * 
	 * @param document
	 * @returns {Object} with row & column or null...
	 */
	exports.getActiveSmartNamePosition = function (document) {
		if (document.activeAnchor && document.activeAnchor.isValid()) {
			return document.activeAnchor.getPosition();
		}
		return null;
	};

	/**
	 * When we click on a SmartName => provide API to edit it!
	 * 
	 * NOTE: the SmartName can be changed because of undo/redo while you have this API reference!!!
	 * User of the API needs to listen to changes and check => then update itself!
	 * 
	 * NOTE: the SmartName might be deleted from the document
	 * In that case isValid() becomes false
	 * 
	 * @param editor
	 * @param document
	 * @param column
	 */
	exports.provideSmartNameAPI = function (editor, row, column) {
		var session = editor.getSession();
		var anchor = _getSmartNameAnchorAt(session.doc, row, column);
		if (anchor === null) {
			return null;
		}

		// What can you do with this SmartName?
		return {
			/**
			 * Whether the SmartName this API is editing still exists...
			 * It might have been removed from the input!
			 * 
			 * @returns {Boolean} whether this API can still be used...
			 */
			isValid: function () {
				return anchor.isValid();
			},

			/**
			 * Make the SmartName active
			 * 
			 * @param {Boolean} active - whether the SN should be active
			 */
			setActive: function (active) {
				if (anchor.isValid()) {
					if (active) {
						var pos = anchor.getPosition();
						exports.setActiveSmartName(session.doc, pos.row, pos.column);
					} else {
						exports.clearActiveSmartName(session.doc);
					}
				}
			},

			/**
			 * Check if this SmartName is active
			 * @returns {Boolean} - true if the SmartName is active
			 */
			isActive: function () {
				return anchor.isValid() && (anchor === session.doc.activeAnchor);
			},

			/**
			 * Get the SmartName
			 * 
			 * @returns {String} the smartname
			 */
			getSmartName: function () {
				return anchor.getSmartNameInfo().getFullName();
			},

			/**
			 * Set the SmartName
			 * 
			 * @param {String} smartname - the smartname to set 
			 * @returns
			 */
			setSmartName: function (smartname) {
				if (anchor.isValid()) {
					var pos = anchor.getPosition();
					exports.renameSmartName(session, pos.row, pos.column, smartname);
				}
			},

			/**
			 * Get the position of the SmartName.<br/>
			 * Returns {top:..., left:..., height:..., width:...} in pixels, relative to the viewport origin.
			 * 
			 * @param clipToView - whether to return only the rect for the visible part of the SmartName or not.
			 * @returns
			 */
			getSmartNameBoundingClientRect: function (clipToView) {
				if (anchor.isValid()) {
					// 1. Get the "document" position of the SmartName (row, col)
					var pos = anchor.getPosition();
					var docrange = new Range(pos.row, pos.column, pos.row, pos.column + 1);

					// 2. Get the "screen" position of the SmartName (row, col)
					var screenrange = docrange.toScreenRange(session);

					// 3. Calculate the absolute position in pixels from the (row & col)
					var r = editor.renderer;
					// Make sure the character width/height is already calculated (otherwise it would still give 0)
					r.$textLayer.checkForSizeChanges();

					// 3.a. Get the bound of the scroller (NOTE: we should not call this method often)
					var scrollerBounds = r.scroller.getBoundingClientRect();

					// 3.b. Calculate the positions...
					// Based on inverse code in "smartnametooltip.js" and lot's of other places...
					var xstart = screenrange.start.column * r.characterWidth + r.$padding + scrollerBounds.left - r.scrollLeft;
					var xend = screenrange.end.column * r.characterWidth + r.$padding + scrollerBounds.left - r.scrollLeft;
					var ystart = screenrange.start.row * r.lineHeight + scrollerBounds.top - r.scrollTop;
					var yend = (screenrange.end.row + 1) * r.lineHeight + scrollerBounds.top - r.scrollTop;

					// 3.c. Optional - clip to view...
					if (clipToView) {
						if (xstart < scrollerBounds.left) {
							xstart = scrollerBounds.left;
						}
						if (xstart > scrollerBounds.right) {
							xstart = scrollerBounds.right;
						}
						if (xend < scrollerBounds.left) {
							xend = scrollerBounds.left;
						}
						if (xend > scrollerBounds.right) {
							xend = scrollerBounds.right;
						}
						if (ystart < scrollerBounds.top) {
							ystart = scrollerBounds.top;
						}
						if (ystart > scrollerBounds.bottom) {
							ystart = scrollerBounds.bottom;
						}
						if (yend < scrollerBounds.top) {
							yend = scrollerBounds.top;
						}
						if (yend > scrollerBounds.bottom) {
							yend = scrollerBounds.bottom;
						}
					}

					// return the values.
					return {
						left: xstart,
						top: ystart,
						width: xend - xstart,
						height: yend - ystart
					};
				} else {
					return null;
				}
			}
		};
	};

	/**
	 * Called when the user does undo or redo.
	 * 
	 * @param document {Document} the document on which the undo/redo is done
	 * @param delta {Object} the change that needs to be undone/redone
	 * @param undo {Boolean} true if this is an undo operation, false in case of redo
	 */
	exports.undoRedo = function (document, delta, undo) {
		var action = delta.action;

		if (action === "addSmartName" || action === "removeSmartName") {
			if (undo === (action === "addSmartName")) {
				// when undo & addSmartName
				// when redo & removeSmartName
				//  -> remove the SmartName
				//     -> Should already be ok... (text is also removed again...)
			} else {
				// when redo & addSmartName
				// when undo & removeSmartName
				//  -> add the SmartName
				exports.setSmartNameInfoAt(document, delta.smartname.row, delta.smartname.column, delta.smartname.info);
			}
		}

		// Undo/redo "renaming" of SmartNames...
		if (action === "renameSmartName") {
			var toInfo = undo ? delta.smartname.oldinfo : delta.smartname.newinfo;
			var anchor = _getSmartNameAnchorAt(document, delta.smartname.row, delta.smartname.column);
			if (anchor != null) {
				anchor.setSmartNameInfo(toInfo);
				
				var range =new Range(delta.smartname.row, delta.smartname.column, delta.smartname.row, delta.smartname.column + 1); 
				// Need to trigger a repaint...
				// That is the only reason for this change event...
				document._signal("change", {
					start: range.start,
					end: range.end,
					action: "renameSmartName",
					//range: new Range(delta.smartname.row, delta.smartname.column, delta.smartname.row, delta.smartname.column + 1),
					// add smartname information to the event to allow for undo/redo
					smartname: {
						row: delta.smartname.row,
						column: delta.smartname.column,
						oldinfo: undo ? delta.smartname.newinfo : delta.smartname.oldinfo,
						newinfo: undo ? delta.smartname.oldinfo : delta.smartname.newinfo
					}
				});
			}
		}
	};

	return exports;
});