/**
 *
 * For SmartName support in ace editor
 * 
 * This module defines some functions to manage the SmartName anchors in a document.
 * SmartNameAnchors follow a character in the text and link SmartNameInfo to it.
 *
 * @author JEPE
 */

define(["component/smartnames/encoding/smartname",
        "component/smartnames/encoding/smartnamestringpart",
        "component/smartnames/encoding/smartnameencoding",
        "./smartnameinfo",
        "./smartnameanchorcontrol"],
	function (SmartName, SmartNameStringPart, SmartNameEncoding, SmartNameInfo, smartnameanchorcontrol) {
		"use strict";
		var exports = {};

		// Export the SMARTNAMECHAR. 
		// This character will be inserted into the text where there is a SmartName.
		// The information of this SmartName will then be attached by using an Anchor
		//   (which is the purpose of this module)
		exports.SMARTNAMECHAR = "\x10";

		// Export the getSmartNameDisplayString function
		// Called from a number of locations to get the Display String for a SmartName
		// Uses the function on document if available...
		exports.getSmartNameDisplayString = function (document, info) {
			if (document.displayFunction !== undefined) {
				var smartname = new SmartName(info.getFullName());
				return document.displayFunction(smartname);
			} else {
				return "[" + info.getFullName() + "]";
			}
		};

		//
		// Export util methods to parse & recode the text with SmartNames
		//

		/**
		 * Get the encoding to encode the SmartNames...
		 * 
		 * @param useuniversalencoding {Boolean} 
		 * 					Whether to use the universal encoding (copying & pasting between fields) 
		 * 					or the encoding used to send the value to the server.
		 * 
		 * @return {SmartNameEncoding} the correct encoding
		 */
		function getEncodingToUse(useuniversalencoding, session) {
			return useuniversalencoding === true ? SmartNameEncoding.UNIVERSALCOPYPASTEENCODING : session.getSmartNameEncoding();
		}


		/**
		 * Parses the encoded SmartName String
		 *
		 * @param session {EditSession} The ace EditSession object
		 * @param text {String} The encoded SmartName string
		 * @param universalencoding {Boolean} 
		 * 					Whether to use the universal encoding (copying & pasting between fields) 
		 * 					or the encoding used to send the value to the server.
		 * @return {Object} with "text" String and "smartname" Array<SmartNameInfo>
		 */
		exports.parseText = function (session, text, useuniversalencoding) {
			if (session.isSmartNamesEnabled()) {
				var result = {
					text: "",
					smartnames: []
				};
				text = text.replaceAll("\n", "").replaceAll("\r", "");
				var decodedtext = getEncodingToUse(useuniversalencoding, session).decodeSmartNameString(text);

				for (var i = 0; i < decodedtext.length; i++) {
					var part = decodedtext[i];
					if (part.isTextPart()) {
						result.text += part.getAsText();
					} else {
						result.smartnames.push({
							index: result.text.length,
							info: new SmartNameInfo(part.getAsSmartName().getFullName())
						});
						result.text += exports.SMARTNAMECHAR;
					}
				}
				return result;
			} else {
				return {
					text: text,
					smartnames: []
				};
			}
		};

		/**
		 * Tries to parse the encoded SmartName String
		 *
		 * If it is not an encoded SmartName string, just paste it as normal text...
		 * 
		 * @param text {String} The encoded SmartName string
		 * @return {Object} with "text" String and "smartname" Array<SmartNameInfo>
		 */
		exports.tryParseText = function (session, text, useuniversalencoding) {
			if (session.isSmartNamesEnabled()) {
				try {
					// parse the result we get as something with SmartNames in it...
					return exports.parseText(session, text, useuniversalencoding);
				} catch (e) {
					// when it fails, we just paste it as text...
					return {
						text: text.replaceAll("\n", "").replaceAll("\r", "").replaceAll("" + exports.SMARTNAMECHAR, "?"),
						smartnames: []
					};
				}
			} else {
				return {
					text: text,
					smartnames: []
				};
			}
		};

		/**
		 * Collects the text from the editor & reencodes it...
		 * 
		 * @param session {EditSession} the edit session
		 * @param document {Document} the document
		 * @param startIndex {Number} the column index on the row we are processing. (Only one line is supported for the moment)
		 * @param text {String} the text we want to recode (contains SMARTNAMECHAR's)
		 * @param universalencoding {Boolean} 
		 * 					Whether to use the universal encoding (copying & pasting between fields) 
		 * 					or the encoding used to send the value to the server.
		 * @returns {String} the recoded text
		 */
		exports.recompileText = function (session, document, startIndex, text, useuniversalencoding) {
			if (session.isSmartNamesEnabled()) {
				var data = [];
				for (var i = 0; i < text.length; i++) {
					if (text.charAt(i) === exports.SMARTNAMECHAR) {
						var info = smartnameanchorcontrol.getSmartNameInfoAt(document, 0, startIndex + i);
						data.push(SmartNameStringPart.makeSmartNamePart(new SmartName(info.getFullName())));
					} else {
						data.push(SmartNameStringPart.makePlainTextPart("" + text.charAt(i)));
					}
				}
				return getEncodingToUse(useuniversalencoding, session).encodeSmartNameString(data);
			} else {
				return text;
			}
		};

		/**
		 * Adds SmartNameAnchors to the document (use after parseText...)
		 * 
		 * @param document {Document}
		 * @param startPosition {Number}
		 * @param smartnames {SmartNameInfo}
		 */
		exports.addSmartNameAnchors = function (document, range, smartnames) {
			for (var i = 0; i < smartnames.length; i++) {
				var index = smartnames[i].index;
				var info = smartnames[i].info;
				smartnameanchorcontrol.setSmartNameInfoAt(document, 0, index + range.start.column, info);
			}
		};

		/**
		 * When we add text with SmartNames, multiple change events will be thrown.
		 * However, for uses outside of this editor, you probably only want to see 
		 *   the ones when all SmartNames are correctly initialized.
		 * This is done by keeping state "smartnamechangeready" and using this state in SmartNameInput.
		 *
		 * Calling this method will disable external change events.
		 *
		 * @param document {Document} The document on which the change events are processed.
		 */
		exports.startSmartNameChangeGroup = function (document) {
			if (document.smartnamechangeready == undefined) {
				document.smartnamechangeready = 0;
			}
			document.smartnamechangeready++;
		};

		/**
		 * When we add text with SmartNames, multiple change events will be thrown.
		 * However, for uses outside of this editor, you probably only want to see 
		 *   the ones when all SmartNames are correctly initialized.
		 * This is done by keeping state "smartnamechangeready" and using this state in SmartNameInput.
		 * 
		 * Calling this method will enable external change events again
		 * 
		 * @param document {Document} The document on which the change events are processed.
		 * @param range {Range} The total modified range during the time change events were disabled.
		 */
		exports.stopSmartNameChangeGroup = function (document, range) {
			document.smartnamechangeready--;

			// this change event gets passed to the user...
			document._signal("change", {
				action:"changeAfterSmartNames",
				start: range.start,
				end: range.end
			});
		};

		/**
		 * When we add text with SmartNames, multiple change events will be thrown.
		 * However, for uses outside of this editor, you probably only want to see 
		 *   the ones when all SmartNames are correctly initialized.
		 * This is done by keeping state "smartnamechangeready" and using this state in SmartNameInput.
		 * 
		 * Are external change events disabled?
		 *
		 * @param document {EditSession} The document on which the change events are processed.
		 */
		exports.isInSmartNameChangeReady = function (document) {

			if (document.smartnamechangeready == undefined) {
				return true;
			} else {
				return document.smartnamechangeready === 0;
			}
		};
		
		/**
		 * When we click on a SmartName => provide API to edit it!
		 * 
		 * @param editor
		 * @param document
		 * @param column
		 */
		exports.getSmartNameEditAPI = function(editor, row, column){
			if(editor.getSession().isSmartNamesEnabled()) {
				var char = editor.getSession().getLine(row).charAt(column);
				if(char===exports.SMARTNAMECHAR) {
					return smartnameanchorcontrol.provideSmartNameAPI(editor, row, column);
				}
			}
			return null;
		};

		return exports;
	});