/* ***** BEGIN LICENSE BLOCK *****
 * Distributed under the BSD license:
 *
 * Copyright (c) 2010, Ajax.org B.V.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of Ajax.org B.V. nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL AJAX.ORG B.V. BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * NOTE: this code is based on the "token_tooltip"
 * Source: https://raw.githubusercontent.com/ajaxorg/ace/master/demo/kitchen-sink/token_tooltip.js
 */

define(function (require, exports, module) {
	"use strict";

	var oop = require("ace/lib/oop");
	var event = require("ace/lib/event");
	var Tooltip = require("ace/tooltip").Tooltip;
	var smartnameextensionutils = require("./smartnameextensionutils");
	var smartnameanchorcontrol = require("./smartnameanchorcontrol");

	function SmartNameTooltip(editor) {
		if (editor.smartnameTooltip)
			return;
		Tooltip.call(this, editor.container);
		editor.smartnameTooltip = this;
		this.editor = editor;

		this.update = this.update.bind(this);
		this.onMouseMove = this.onMouseMove.bind(this);
		this.onMouseOut = this.onMouseOut.bind(this);
		event.addListener(editor.renderer.scroller, "mousemove", this.onMouseMove);
		event.addListener(editor.renderer.content, "mouseout", this.onMouseOut);
		event.addListener(editor, "change", function (e) {
			var data = e.data;
			if(typeof data === "undefined"){
				data = e;
			}
			
			if (data.action === "changeActiveSmartName") {
				if (this.lastHoverSmartName) {
					this.update();
				}
			}
		}.bind(this));
	}

	oop.inherits(SmartNameTooltip, Tooltip);

	(function () {

		this.update = function () {
			this.$timer = null;

			var r = this.editor.renderer;
			if (this.lastT - (r.timeStamp || 0) > 1000) {
				r.rect = null;
				r.timeStamp = this.lastT;
				this.maxHeight = window.innerHeight;
				this.maxWidth = window.innerWidth;
			}

			var canvasPos = r.rect || (r.rect = r.scroller.getBoundingClientRect());
			var offset = (this.x + r.scrollLeft - canvasPos.left - r.$padding) / r.characterWidth;
			var row = Math.floor((this.y + r.scrollTop - canvasPos.top) / r.lineHeight);
			var col = Math.round(offset);

			var screenPos = {
				row: row,
				column: col,
				side: offset - col > 0 ? 1 : -1
			};
			var session = this.editor.session;
			var docPos = session.screenToDocumentPosition(screenPos.row, screenPos.column);

			var updateSize = false;
			var needToDisplay = false;

			// Get the SmartName at that position
			if (session.getLine(docPos.row).charAt(docPos.column) === smartnameextensionutils.SMARTNAMECHAR) {
				// There is a SmartName at that position!
				var info = smartnameanchorcontrol.getSmartNameInfoAt(session.getDocument(), docPos.row, docPos.column);
				var activePos = smartnameanchorcontrol.getActiveSmartNamePosition(session.getDocument());
				var active = (activePos !== null && activePos.row === docPos.row && activePos.column === docPos.column);
				if (info === this.lastHoverSmartName && active === this.lastSmartNameActive) {
					// nothing changed, we are still hovering over the same SmartName
					updateSize = false;
					needToDisplay = this.isOpen; // keep it open if it was open for this SmartName
				} else {
					var parent = $(this.getElement());
					parent.empty();

					// ok, ask the user what to display
					this.lastHoverSmartName = info;
					this.lastSmartNameActive = active;
					if (this.editor.smartnameTooltipFunction) {
						needToDisplay = this.editor.smartnameTooltipFunction(info.getFullName(), active, parent);
						updateSize = needToDisplay;
					}
				}
			} else {
				// no smartname...
				this.lastHoverSmartName = null;
				needToDisplay = false;
				updateSize = false;
			}


			// hide if no content
			if (!needToDisplay) {
				this.hide();
				return;
			}

			// update size if the content changed
			if (updateSize) {
				this.width = this.getWidth();
				this.height = this.getHeight();
			}

			// update position (follow mouse)
			this.show(null, this.x, this.y);
		};

		this.onMouseMove = function (e) {
			this.x = e.clientX;
			this.y = e.clientY;
			if (this.isOpen) {
				this.lastT = e.timeStamp;
				this.setPosition(this.x, this.y);
			}
			if (!this.$timer)
				this.$timer = setTimeout(this.update, 100);
		};

		this.onMouseOut = function (e) {
			if (e && e.currentTarget.contains(e.relatedTarget))
				return;
			this.hide();
			this.$timer = clearTimeout(this.$timer);
			this.lastHoverSmartName = null;
		};

		this.setPosition = function (x, y) {
			if (x + 10 + this.width > this.maxWidth)
				x = window.innerWidth - this.width - 10;
			if (y > window.innerHeight * 0.75 || y + 20 + this.height > this.maxHeight)
				y = y - this.height - 30;

			Tooltip.prototype.setPosition.call(this, x + 10, y + 20);
		};

		this.destroy = function () {
			this.onMouseOut();
			event.removeListener(this.editor.renderer.scroller, "mousemove", this.onMouseMove);
			event.removeListener(this.editor.renderer.content, "mouseout", this.onMouseOut);
			delete this.editor.smartnameTooltip;
		};

	}).call(SmartNameTooltip.prototype);

	exports.SmartNameTooltip = SmartNameTooltip;

});