define(["jquery",
        "underscore",
        "utils/webi18n!TextContentStrings",
        "component/message/MessageComponent",
        "component/passwordpopup/PasswordValidationPopup",
        "./content/SingleTextContent",
        "./content/TableContent",
        "./text-content-services/ContentService",
        "./table-content-page/controller/TableContentPage",
        "./text-information-view/TextInformationView",
        "./text-content-utils/ContentUtils"],
		function($,	_,
				TextContentStrings,
				MessageComponent,
				PasswordValidationPopup,
				SingleTextContent,
				TableContent,
				contentService,
				TableContentPage,
				TextInformationView,
				ContentUtils) {
	"use strict";
	
	/**
	 * Create a basic content page.
	 */
	var ContentBasicPage = function (docVersionId, projectId, parent) {
		
		this._docVersionId = docVersionId;
		this._projectId = projectId;
		
		this._parentElement = parent;
		this._wrapper = $("<div>");
		this._messageBar = $("<div>",{"style" : "width: 100%"});
		this._wrapper.append(this._messageBar);
		
		this._messageModel = new MessageComponent.MessageModel();
        this._messageWidget = new MessageComponent.MessageWidget(this._messageModel, this._messageBar);
        
        this.displayInfoMessage("Loading content ...", "LOADING"); //LOADING_TEXTCONTENT_DATA
        
        // Load the content from the server.
        var contentRequest = {
				docVersionId: docVersionId,
				retainStatus: false,
				fromProjectCreation: false,
				willBeNew: false,
				needEffectiveElementPermissions: true
		};
        contentService.getContent(contentRequest, function (content) {
        	this._messageModel.markObsolete("LOADING");
        
        	if (content instanceof SingleTextContent) {
				this.displayTextContentPage(content);
			} else if (content instanceof TableContent) {
				// Create a Container to hold the top buttons of the page.
				this._topWarningContainer = $("<div/>", {'class':'WarningContainer'}).hide();
				this._topWarningContainer.css("max-width", "none");
				this._wrapper.append(this._topWarningContainer);
				
				this._topSaveWarning = this._createSaveWaring(content);
				this._topWarningContainer.append(this._topSaveWarning.saveWarningDiv);
				
				this.displayTablePage(content);
				
				// Create a Container to hold the bottom buttons of the page.
				this._bottomWarningContainer = $("<div/>", {'class':'WarningContainer'}).hide();
				this._bottomWarningContainer.css("max-width", "none");
				this._wrapper.append(this._bottomWarningContainer);
				
				this._bottomSaveWarning = this._createSaveWaring(content);
				this._bottomWarningContainer.append(this._bottomSaveWarning.saveWarningDiv);
				
				this.onPageChange(content);
			} else {
				this.displayErrorMessage("Could not display a page for the loaded content object.");
			}
			
        }.bind(this),
        function (errorMsg) {
        	if (this._topWarningContainer) {
        		this._topWarningContainer.hide();
			}
        	
        	if (this._bottomWarningContainer) {
        		this._bottomWarningContainer.hide();
			}
        	
        	this.displayErrorMessage(errorMsg);
        }.bind(this));
        
		// It could be that an error message was passed ... in that case we should show it.
		var errorMsg = ContentUtils.getParameterByName("error");
		if (errorMsg && errorMsg.length > 0) {
			if (this._topWarningContainer) {
        		this._topWarningContainer.hide();
			}
        	
        	if (this._bottomWarningContainer) {
        		this._bottomWarningContainer.hide();
			}
			this.displayErrorMessage(errorMsg);
		}
	};
	
	/**
	 * Display the page for a table content page.
	 * 
	 * @param {TableContent} tableContent - The TableContent object to display.
	 */
	ContentBasicPage.prototype.displayTablePage = function (tableContent) {
		this._page = new TableContentPage(this, tableContent, this);
		this._wrapper.append(this._page.getJQElement());
	};
	
	/**
	 * Display the page for a single text content page.
	 * 
	 * @param {SingleTextContent} singleTextContent - The SingleTextContent object to display.
	 */
	ContentBasicPage.prototype.displayTextContentPage = function (textContent) {
		this._page = new TextInformationView(textContent, this._projectId);
		this._wrapper.append(this._page.getJQElement());
	};
	
	/**
	 * Display an error message.
	 * 
	 * @param {String} errorKey - Language key for the error message to display.
	 */
	ContentBasicPage.prototype.displayErrorMessage = function (errorKey, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.ERROR, errorKey, null, false, tag);
        this._messageModel.addToMessageQueue(message);
	};
	
	/**
	 * Display a success message.
	 * 
	 * @param {String} successKey - Language key for the success message to display.
	 */
	ContentBasicPage.prototype.displaySuccessMessage = function (successKey, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.SUCCESS, successKey, null, false, tag);
        this._messageModel.addToMessageQueue(message);
	};
	
	/**
	 * Display a warning message.
	 * 
	 * @param {String} warnKey - Language key for the warning message to display.
	 */
	ContentBasicPage.prototype.displayWarnMessage = function (warnKey, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.WARN, warnKey, null, false, tag);
        this._messageModel.addToMessageQueue(message);
	};
	
	/**
	 * Display an info message.
	 * 
	 * @param {String} warnKey - Language key for the warning message to display.
	 */
	ContentBasicPage.prototype.displayInfoMessage = function (infoKey, tag) {
		var message = new MessageComponent.Message(MessageComponent.MessageType.INFO, infoKey, null, false, tag);
        this._messageModel.addToMessageQueue(message);
	};
	
	/**
	 * Execute when page is having some changes.
	 * 
	 */
	ContentBasicPage.prototype.onPageChange = function (content) {
		this._page.onChange(_.bind(function () {
			this.displaySaveWarning(TextContentStrings.t("PCM_CONTENT_CHANGED"), true, content);
		}, this));
	};
	
	/**
	 *  Display warning with save button.
	 *  
	 * @param message
	 * @param isDismissable
	 * @param content
	 */
	ContentBasicPage.prototype.displaySaveWarning = function (message, isDismissable, content) {
		if (isDismissable) {
			this._topSaveWarning.warningDismiss.show();
			this._bottomSaveWarning.warningDismiss.show();
			this._topSaveWarning.saveWarningDiv.addClass("alert-dismissible");
			this._bottomSaveWarning.saveWarningDiv.addClass("alert-dismissible");
		} else {
			this._topSaveWarning.warningDismiss.hide();
			this._bottomSaveWarning.warningDismiss.hide();
			this._topSaveWarning.saveWarningDiv.removeClass("alert-dismissible");
			this._bottomSaveWarning.saveWarningDiv.removeClass("alert-dismissible");
		}
		
		this._topSaveWarning.warningMessageHolder.text(message);
		this._bottomSaveWarning.warningMessageHolder.text(message);
		
		if (content.getSaveActions().length > 0) {
			this._topWarningContainer.show();
			this._bottomWarningContainer.show();
		} else {
			this._topWarningContainer.hide();
			this._bottomWarningContainer.hide();
		}
	};
	
	/**
	 * Create the save option for a table content page.
	 * 
	 */
	ContentBasicPage.prototype._createSaveWaring = function (content) {
		var saveWarningObj = {};
		
		// Fields of the object which can be accessible.
		saveWarningObj.saveWarningDiv = $("<div>", {'class':'alert alert-warning'});
		saveWarningObj.warningMessageHolder = $("<span>");
		saveWarningObj.warningDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});;
		saveWarningObj.saveButton = $("<button>", {'class':'btn btn-primary btn-sm SaveButton'}).text(TextContentStrings.t("PCM_SAVE"));
		saveWarningObj.saveCell = $("<td>",{'class' : 'SaveCell'}).append(saveWarningObj.saveButton);
	
		// Table layout of the warning DIV.
		var warningInnerTable = $("<table>");
		var warningInnerRow = $("<tr>");
		var messageCell = $("<td>").append(saveWarningObj.warningMessageHolder);
		
		warningInnerRow.append(messageCell, saveWarningObj.saveCell);
		warningInnerTable.append(warningInnerRow);
		
		saveWarningObj.saveWarningDiv.append(warningInnerTable);
		
		// Prepare warning dismiss option.
		saveWarningObj.warningDismiss.append($("<span>").html("&times;"));
		
		saveWarningObj.saveWarningDiv.append(saveWarningObj.WarningDismiss);
		saveWarningObj.saveWarningDiv.addClass("alert-dismissible");
		
		// Function to be executed on click of save button.
		var saveButtonHandler = _.bind(function() {
			this._topSaveWarning.saveButton.hide();
			this._bottomSaveWarning.saveButton.hide();
			this._topSaveWarning.saveCell.append($("<img/>",{'src':'./images/loaderTransparent.gif'}));
			this._bottomSaveWarning.saveCell.append($("<img/>",{'src':'./images/loaderTransparent.gif'}));

			// Execute save for function for the respective page.
			this._page.saveContent(_.bind(function (message, isDismissable) {
				// When error occurred, display error message.
				this._topWarningContainer.hide();
	        	this._bottomWarningContainer.hide();
				this.displayErrorMessage(message);
			}, this));
		}, this);
		
		// Event for save button click.
		saveWarningObj.saveButton.on("click", function () {
			if(content.isPasswordRequiredForApproval()) {
				var passwordPopup = new PasswordValidationPopup();
				passwordPopup.onSubmit(saveButtonHandler);
				passwordPopup.open();
			} else {
				saveButtonHandler();
			}
		}.bind(this));
		
		return saveWarningObj;
	};
	
	/**
	 * Return the parent jQuery element.
	 * 
	 *  @return {jQuery} The wrapping parent jQuery element of the page.
	 */
	ContentBasicPage.prototype.getParentJQElement = function () {
		return this._parentElement;
	};

	/**
	 * Return the wrapping jQuery element of the page.
	 * 
	 * @return {jQuery} The wrapping jQuery element of the page.
	 */
	ContentBasicPage.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return ContentBasicPage;
});