define(["underscore",
        "jquery",
        "utils/Map",
        "utils/webi18n!TextContentStrings",
        "packaging-content-management/LanguagePanel",
        "component/modal/Modal",
        "component/tooltip/Tooltip",
        "../../language-indicator/TableLanguageIndicator",
        "../../text-content-services/languageService",
        "../../content/SingleTextContent",
        "../../content/TableContent",
        "../../table-content-page/view/TableContentSettingsView",
        "../../ContentConstants"],
	function (_, 
			$,
			Map,
			TextContentStrings,
			LanguagePanel,
			Modal,
			Tooltip,
			TableLanguageIndicator,
			languageService,
			SingleTextContent,
			TableContent,
			TableContentSettingsView,
			ContentConstants) {
		
	"use strict";
	
	/**
	 * @class
	 * @exports ContentSettings
	 * 
	 * @param {Content} content
	 * @param {String} projectId
	 */
	var ContentSettings = function (content, projectId) {
		this._content = content;
		this._projectId = projectId;
		this._contentManagerView = content.canManage().allowed;
		
		this._onChangeCallbacks = [];
		this._onLanguageAddCallbacks = [];
		this._onLanguageRemoveCallbacks = [];
		this._onSmartNameResolveClick = [];
		this._smartNameResolveFeedbacks = 0;
		
		//Use the settingMap to send the fields and values to the view
		this._settingsMap = new Map();
		this._settingsMap.add(ContentConstants.UpdateField.INVARIANT, content.isInvariant());
		this._settingsMap.add(ContentConstants.UpdateField.ELEMENT_TYPE, content.getElementType());
		this._settingsMap.add(ContentConstants.UpdateField.APPROVAL_TYPE, content.getApprovalType());
		this._settingsMap.add(ContentConstants.UpdateField.TEXTTYPE, content.getTextType());
		this._settingsMap.add("projectId", projectId);
		this._settingsMap.add("docVerId", content.docVersionId);

		//For languages field, add indicators for the view
		var options = {showLabels:false};
		if (!this._contentManagerView) {
			options.readOnly = true;
		}

		// Create language indicators for master and translation languages.
		var langIndMaster = new TableLanguageIndicator(content, true, options);
		var langIndTrans = new TableLanguageIndicator(content, false, options);
		
		// Register call backs for master language indicators.
		langIndMaster.onLanguageAdd(_.bind(function (language, isMaster) {
			$.each(this._onLanguageAddCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));
		
		langIndMaster.onLanguageRemove(_.bind(function (language, isMaster) {
			$.each(this._onLanguageRemoveCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));

		// Register call backs for translation language indicators.
		langIndTrans.onLanguageAdd(_.bind(function (language, isMaster) {
			$.each(this._onLanguageAddCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));
		
		langIndTrans.onLanguageRemove(_.bind(function (language, isMaster) {
			$.each(this._onLanguageRemoveCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));
		
		
		this._settingsMap.add("LangIndMaster", langIndMaster);
		this._settingsMap.add("LangIndTrans", langIndTrans);
		
		// Create view for text content settings or table content settings.
		if (content instanceof SingleTextContent) {
			// Text content settings are being managed in InitTextInformationView itself.
		} else if (content instanceof TableContent) {
			var nextStates = [];	// Variable to hold next available states.
			
			// Calculate the current state.
			var currentState = this._content.getCurrentState();
		
			// When smart nameable content's elements are approved and text data is resolved.
			if (this._content.isElementsApproved() && this._content.isTextDataResolved() && this._content.getTextType() == ContentConstants.TextType.SMARTNAME) {
				
				// Show next action to approve table.
				if (ContentConstants.State.APPROVED != currentState) {
					// Set next available states in the variable.
					nextStates = [ContentConstants.State.APPROVED];
				}
			}
			
			// Create view
			this._view = new TableContentSettingsView(this._contentManagerView, this._settingsMap, this._content.getCurrentState(), nextStates);
			
			this._view.onNewState(_.bind(function (newState) {
				this._content.setNewState(newState);
				
				// Hide resolve button from the header, if available.
				if (this._resolveSmartNameButton) {
					this._resolveSmartNameButton.hide();
				}
				
				// Execute all the call backs
				$.each(this._onChangeCallbacks, _.bind(function (index, callBack) {
					callBack(ContentConstants.SaveAction.TABLECONTENTAPPROVAL, true);
				}, this));
			}, this));
			
			this._view.onRevertState(_.bind(function (wasNewState) {
				this._content.setNewState(this._view.getCurrentState());
				
				// Hide resolve button from the header, if available.
				if (this._resolveSmartNameButton) {
					this._resolveSmartNameButton.show();
				}
				
				// Execute all the call backs
				$.each(this._onChangeCallbacks, _.bind(function (index, callBack) {
					callBack(ContentConstants.SaveAction.TABLECONTENTAPPROVAL, false);
				}, this));
			}, this));
			
			// Register callback for header click of the widget.
			this._onHeaderClick();

			// Add admin button to the header.
			this._addAdminButton();
			
			// Add GS1 file export button to the header.
			this._addGS1ExportButton();
			
			// We might also want to add a button to resolve the smart names of this table.
			this._addSmartNameResolveButton();
		}
		
		// Using language find the languages, that will be displayed on the header
		languageService.getLanguages(this._projectId, function(projMastLangs, projTransLangs, allMasterLangs, allTransLangs) {
			this._settingsMap.add(ContentConstants.ContentField.ALL_LANGS, allMasterLangs.concat(allTransLangs));
			this._settingsMap.add(ContentConstants.ContentField.PROJ_MASTER_LANGS, projMastLangs);
			this._settingsMap.add(ContentConstants.ContentField.PROJ_TRANS_LANGS, projTransLangs);
			this._settingsMap.add(ContentConstants.ContentField.DOC_MASTER_LANGS, this._content.getLangCodesOfElements(true));
			this._settingsMap.add(ContentConstants.ContentField.DOC_TRANS_LANGS, this._content.getLangCodesOfElements(false));
			
			// Now language list is available, set this list to indicators.
			langIndMaster.setLanguageList(projMastLangs);
			langIndTrans.setLanguageList(projTransLangs);
			
			this._view.constructHeader();
			this._view.constructMainContent();
		}.bind(this));
		
		
		// Listen to changes on content element widgets, they could cause an update of the state of this widget.
		pcmEventBus.on("PCM.TextContentElementWidget.update", function () {
			this._manageStateAndAction();
		}.bind(this));
		
		this.onInvariantChange();
		this.onTextTypeChange();
		this.onApprovalTypeChange();
	};
	
	/**
	 * Manage current content state display and next available action.
	 */
	ContentSettings.prototype._manageStateAndAction = function () {
		// Calculate the current state.
		var currentState = this._content.getCurrentState();
		
		// Check whether status of content is changed or not.
		var isStateChanged = this._content.isStatusChanged();
		
		// When smart nameable content's elements are approved and text data is resolved.
		if (this._content.getTextType() == ContentConstants.TextType.SMARTNAME) {
			// Show tool tip is elements are not approved and text data is not resolved also.
			if (!this._content.isElementsApproved() && !this._content.isTextDataResolved()) {
				// Set the state with next available states in the view.
				this._view.setNextStates(currentState, []);
			} 
			// Show tool tip is elements are not approved.
			else if (!this._content.isElementsApproved()) {
				// Set the state with next available states in the view.
				this._view.setNextStates(currentState, []);
			}
			// Show tool tip if text data is not resolved.
			else if (!this._content.isTextDataResolved()) {
				// Set the state with next available states in the view.
				this._view.setNextStates(currentState, []);
			}
			// Show next action to approve table.
			else if (ContentConstants.State.APPROVED != currentState) {
				// Set the state with next available states in the view.
				this._view.setNextStates(currentState, [ContentConstants.State.APPROVED]);
			} else {
				// Set the state with next available states in the view.
				this._view.setNextStates(currentState, []);
			}
		}
		
		// Set current state of the content.
		if (isStateChanged) {
			this.setCurrentState(currentState);
		} else {
			// if state is same as original.
			this.setCurrentState(undefined);
		}
	};
	
	/**
	 * Add a button to resolve smart name string available in the elements. 
	 */
	ContentSettings.prototype._addSmartNameResolveButton = function () {
		// Only add the resolve smart name button when we have smart names as content AND the table is not signed off yet.
		if (this._content.getTextType() == ContentConstants.TextType.SMARTNAME &&
				this._content.getCurrentState() != ContentConstants.State.APPROVED) {
			
			// Create a custom button.
			this._resolveSmartNameIcon = $("<i>", {"class" : "icon icon-WCR_Refresh"});
			var resolveSmartNameLoader = $("<img/>",{'src':'./images/loaderTransparent.gif'});

			this._resolveSmartNameButton = $("<div>", {"class": "CustomButton"}).append(this._resolveSmartNameIcon);
			
			// Create the tooltip for this button.
			new Tooltip(undefined, 
					this._resolveSmartNameButton, 
					this._view.getJQElement(), 
					$("body"), 
					false, 
					TextContentStrings.t("RESOLVE_SMARTNAMES"), 
					undefined, undefined, {});
			
			// What should happen when the resolve button is clicked.
			this._resolveSmartNameButton.on("click", function (e) {
				// Create array to keep all the deferred objects.
				var deferredObjects = [];

				// Show loading symbol.
				this._resolveSmartNameButton.empty();
				this._resolveSmartNameButton.append(resolveSmartNameLoader);
				
				$.each(this._onSmartNameResolveClick, function (index, callback) {
					var deferred = $.Deferred();

					// Execute call back with deferred object.
					callback(deferred);
					
					deferredObjects.push(deferred);
				}.bind(this));
				
				e.stopPropagation();
				
				// When all deferred objects are resolved.
				$.when.apply($, deferredObjects).then(function () {
					// Resolving is completed, show resolve button again.
					this._resolveSmartNameButton.empty();
					this._resolveSmartNameButton.append(this._resolveSmartNameIcon);
					
					// Manage state and action after resolving text data.
					this._manageStateAndAction();
				}.bind(this));
			}.bind(this));
			
			// Add the custom button
			this._view.addCustomButton(this._resolveSmartNameButton);
		}
	};
	
	/**
	 * Add a button for administrative work. 
	 */
	ContentSettings.prototype._addAdminButton = function () {
		// Create button for administration.
		this._adminIcon = $("<i>", {"class" : "icon icon-WCR_Menu-Admin"});
		var adminButton = $("<div>", {"class": "CustomButton"}).append(this._adminIcon);
		
		// Create the tooltip for this button.
		new Tooltip(undefined, 
				adminButton, 
				this._view.getJQElement(), 
				$("body"), 
				false, 
				TextContentStrings.t("CONFIGURE_TABLE_TT"), 
				undefined, undefined, {});
		
		// Collapse widget.
		this._view.collapse();
		
		// Add event to this button.
		adminButton.on("click", function (event) {
			this._toggleVisibility();
		}.bind(this));
		
		// Add the this button to widget.
		this._view.addCustomButton(adminButton);
	};
	
	/**
	 * Add a button for exporting GS1 file. 
	 */
	ContentSettings.prototype._addGS1ExportButton = function () {
		var exportIcon = $("<i>", {"class" : "icon icon-WCR_WF-Export"});
		var exportGS1Button = $("<div>", {"class": "CustomButton"}).append(exportIcon);
		
		// Create the tooltip for this button.
		new Tooltip(undefined, 
				exportGS1Button, 
				this._view.getJQElement(), 
				$("body"), 
				false, 
				TextContentStrings.t("PCM_EXPORT_TO_GS1"), 
				undefined, undefined, {});
		
		
		exportGS1Button.on("click", function () {
			
			var saveActions = this._content.getSaveActions();
			if (saveActions && saveActions.length > 0) {
				
				var message = $("<span>").text(TextContentStrings.t("PCM_EXPORT_TO_GS1_SAVE_WARNING"));
				var exportGS1Popup = new Modal(message);
				exportGS1Popup.setTitle(TextContentStrings.t("PCM_EXPORT_TO_GS1"));
				exportGS1Popup.open();
				
			} else {
				var languagePanel = new LanguagePanel(this._settingsMap.get("projectId"));
				var exportGS1Popup = new Modal(languagePanel.getJQElement());
		        var exportBtn = $("<button>");
		        exportBtn.addClass('btn btn-primary');
		        exportBtn.text(TextContentStrings.t("PCM_GS1_EXPORT"));
		        
		        exportBtn.on("click", function () {
		        	//Construct the url for GS1 Export
		        	var target_lang = languagePanel.getTargetLanguagesJson();
		        	var url = window.webcenterConfig.jspBaseUrl + 'doprojectdocsactions.jsp?actionVal=DoTextGS1Export';
		        	url += '&projectID=' + this._settingsMap.get("projectId") + '&selectedDocs='+this._settingsMap.get("docVerId")+'&isdoc=false';
		        	url +=  '&target_lang=' + target_lang+ ',,';
		        	//Load the url
		        	window.location = url;
		        	exportGS1Popup.close();
		        }.bind(this));
		        
				exportGS1Popup.setTitle(TextContentStrings.t("PCM_EXPORT_TO_GS1"));
				exportGS1Popup.setFooter(exportBtn);
				exportGS1Popup.open();
			}
			
			
			
			event.stopPropagation();
		}.bind(this));
	
		this._view.addCustomButton(exportGS1Button);
	};
	
	/**
	 * What to do when header of widget is clicked.
	 */
	ContentSettings.prototype._onHeaderClick = function () {
		this._view.onHeaderClick(_.bind(function () {
			this._toggleVisibility();
		}, this));
	};
	
	/**
	 * Method is to manage visibility of state full widget.
	 */
	ContentSettings.prototype._toggleVisibility = function () {
		// Remove all the classes and add relevant class again.
		this._adminIcon.removeClass();
		
		if (this._view.isExpanded()) {
			this._view.collapse();
			this._adminIcon.addClass("icon icon-WCR_Menu-Admin");
		} else {
			this._view.expand();
			this._adminIcon.addClass("icon icon-WCR_Menu-AdminExit2");
		}
		
		// Stop triggering event on parent DIV.
		event.stopPropagation();
	};
	
	/**
	 * Register a call back to be executed when resolve button is clicked.
	 * 
	 * @param callback
	 */
	ContentSettings.prototype.onSmartNameResolveClick = function (callback) {
		this._onSmartNameResolveClick.push(callback);
	};
	
	/**
	 * Keep call back to be executed on adding language.
	 * @param callback
	 */
	ContentSettings.prototype.onLanguageAdd = function (callback) {
		this._onLanguageAddCallbacks.push(callback);
	};
	
	/**
	 * Keep call back to be executed on removing language.
	 * @param callback
	 */
	ContentSettings.prototype.onLanguageRemove = function (callback) {
		this._onLanguageRemoveCallbacks.push(callback);
	};
	
	/**
	 * Update status of the content setting widget.
	 */
	ContentSettings.prototype.setCurrentState = function (newState) {
		this._view.setCurrentState(newState);
	};
	
	/**
	 * Call back for invariant change.
	 */
	ContentSettings.prototype.onInvariantChange = function () {
		this._view.onInvariantChange(_.bind(function (isInvariant) {
			this._content.setInvariant(isInvariant);
			
			// Execute all the call backs
			$.each(this._onChangeCallbacks, _.bind(function (index, callBack) {
				callBack(ContentConstants.SaveAction.INVARIANT, isInvariant);
			}, this));
			
			// Hide/show translation DIVs based on invariant flag
			if (isInvariant) {
				$(".translation-flags").hide();
			} else {
				$(".translation-flags").show();
			}
			
		}, this));
	};
	
	/**
	 * Call back for text type change.
	 */
	ContentSettings.prototype.onTextTypeChange = function () {
		this._view.onTextTypeChange(_.bind(function (newTextType) {
			var isTextTypeUpdated = this._content.setTextType(newTextType);
			
			// If resolve smart name button is created.
			if (this._resolveSmartNameButton) {
				// When value is available that means change is done otherwise, user have reverted back change done for text type.
				if (isTextTypeUpdated) {
					// Hide resolve button because text type is changed and no longer applicable for current text type. 
					this._resolveSmartNameButton.hide();
				} else if (ContentConstants.TextType.SMARTNAME == this._content.getTextType()) {
					// Text type is reverted, so resolve button can be shown again.
					this._resolveSmartNameButton.show();
				}
			}

			// Execute all the call backs
			$.each(this._onChangeCallbacks, _.bind(function (index, callBack) {
				callBack(ContentConstants.SaveAction.TEXTTYPE, isTextTypeUpdated);
			}, this));
			
		}, this));
	};
	
	/**
	 * Call back for approval type change.
	 */
	ContentSettings.prototype.onApprovalTypeChange = function () {
		this._view.onApprovalTypeChange(_.bind(function (newApprovalType) {
			this._content.setApprovalType(newApprovalType);

			// Execute all the call backs
			$.each(this._onChangeCallbacks, _.bind(function (index, callBack) {
				callBack(ContentConstants.SaveAction.APPROVALTYPE, newApprovalType);
			}, this));
			
		}, this));
	};
	
	/**
	 * Keep call back function to execute on change.
	 * @param callback
	 */
	ContentSettings.prototype.onChange = function (callback) {
		this._onChangeCallbacks.push(callback);
	};
	
	/**
	 * Get the wrapping jQuery element of the content
	 */
	ContentSettings.prototype.getJQElement = function () {
		return this._view.getJQElement();
	};
	
	return ContentSettings;
});