define(["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "packaging-content-management/table-content-page/view/StateFullContentWidget",
        "packaging-content-management/language-indicator/LanguageIndicator",
        "../../ContentConstants",
        "utils/webcss!./ContentSettingsView.css"],
        function (_,
        		$,
        		i18n,
        		StateFullContentWidget,
        		LanguageIndicator,
        		ContentConstants) {

	"use strict";

	/**
	 * @class
	 * @exports ContentSettingsView
	 * 
	 * @param {Boolean} contentManagerView - True if content manager permission is available for Content
	 * @param {Map.<String><Object>} settingsMap - Key value pair of content settings fields
	 * @param {String} currentState - Current state of the content
	 * @param {Array.<String>} nextStates - Possible states that can be set for the content
	 */
	var ContentSettingsView = function (contentManagerView, settingsMap, currentState, nextStates) {

		this._contentManagerView = contentManagerView;
		this._settingsMap = settingsMap;
		this._currentState = currentState;
		this._nextStates = nextStates;

		// Create the wrapping DIV of this widget.
		this._wrapper = $("<div>").addClass("ContentSettingsWidget");
		this._stateFullContentWidget = new StateFullContentWidget(this._currentState, this._nextStates);
		this._stateFullContentWidget.setMainContent(this._wrapper);
		
		// Change call backs array.
		this._onApprovalTypeChange = [];
		this._onTextTypeChange = [];
		this._onInvariantChange = [];
		this._onLangChange = [];
		
	};
	
	/**
	 * Set new status of the content setting widget.
	 */
	ContentSettingsView.prototype.setCurrentState = function (currentState) {
		this._stateFullContentWidget.setCurrentState(currentState);
	};
	
	/**
	 * Get current status of the content setting widget.
	 */
	ContentSettingsView.prototype.getCurrentState = function () {
		return this._stateFullContentWidget.getCurrentState();
	};
	
	/**
	 * Set the state of state full content widget
	 * @param currentState
	 * @param nextStates
	 */
	ContentSettingsView.prototype.setNextStates = function (currentState, nextStates) {
		this._currentState = currentState;
		this._nextStates = nextStates;
		this._stateFullContentWidget.update(currentState, nextStates);
	};
	
	/**
	 * Register a callback to be called when a new state is selected.
	 */
	ContentSettingsView.prototype.onNewState = function (callback) {
		this._stateFullContentWidget.onNewState(callback);
	};
	
	/**
	 * Register a callback to be called when the new state is reverted.
	 */
	ContentSettingsView.prototype.onRevertState = function (callback) {
		this._stateFullContentWidget.onRevertState(callback);
	};
	
	/**
	 * Expand the main content.
	 */
	ContentSettingsView.prototype.expand = function () {
		this._stateFullContentWidget.expand();
	};
	
	/**
	 * Register a callback to be called when the header is clicked.
	 */
	ContentSettingsView.prototype.isExpanded = function () {
		return this._stateFullContentWidget.isExpanded();
	};
	
	/**
	 * Hide the main content.
	 */
	ContentSettingsView.prototype.collapse = function () {
		this._stateFullContentWidget.collapse();
	};
	
	/**
	 * What to do when header of state full widget is clicked.
	 */
	ContentSettingsView.prototype.onHeaderClick = function (callback) {
		this._stateFullContentWidget.onHeaderClick(callback);
	};
	
	/**
	 * Register the callbacks for this view.
	 */
	ContentSettingsView.prototype.onApprovalTypeChange = function(callback) {
		this._onApprovalTypeChange.push(callback);
	};
	
	ContentSettingsView.prototype.onTextTypeChange = function(callback) {
		this._onTextTypeChange.push(callback);
	};
	
	ContentSettingsView.prototype.onInvariantChange = function(callback) {
		this._onInvariantChange.push(callback);
	};
	
	ContentSettingsView.prototype.onLangChange = function(callback) {
		this._onLangChange.push(callback);
	};

	/**
	 * Creates invariant field panel
	 * 
	 * @param {String} label - Label to be set for Invariant field
	 * @returns {JQueryObj}
	 */
	ContentSettingsView.prototype._createFieldInvariant = function(label){
		
		// Create this field only for user with content manager permission.
		if(this._contentManagerView){
			var invariantFormGroup = $("<div/>", {'class':'form-group'});
			var checkboxOffSet = $("<div/>", {'class':'col-sm-offset-2 col-sm-10'});
			var invariantDiv = $("<div/>", {'class':'checkbox'});
			checkboxOffSet.append(invariantDiv);
		
			var invariantLabel = $("<label>").addClass("col-sm-10");
			var invariantInput = $("<input>", {'type':'checkbox', 'value':''});
			if (this._settingsMap.get(ContentConstants.UpdateField.INVARIANT)) {
				invariantInput.attr('checked','checked');
			}
			
			invariantLabel.append(invariantInput).append(label);
			invariantDiv.append(invariantLabel);
			
			// When somebody changes the value of the input.
			invariantInput.on("change", function() {
				// Execute the necessary call backs.
				$.each(this._onInvariantChange, _.bind(function (index, callback) {
					callback(invariantInput.is(":checked"));
				}, this));
			}.bind(this));
			
			invariantFormGroup.append(checkboxOffSet);
			return invariantFormGroup;
		}
	};

	/**
	 * Creates Approval Setup field panel
	 * @returns {JQueryObj}
	 */
	ContentSettingsView.prototype._createFieldApprovalType = function(){
		var approvalTypeString=null;
		var approvalTypeCode=null;
		var approvalType = this._settingsMap.get(ContentConstants.UpdateField.APPROVAL_TYPE); 

		if (!approvalType || approvalType == ContentConstants.ApprovalSetup.DEFAULT) {
			approvalTypeString = i18n.t("PCM_SETUP_DEFAULT");
			approvalTypeCode = ContentConstants.ApprovalSetup.DEFAULT;
		} else if (approvalType == ContentConstants.ApprovalSetup.ONE_OF_GROUP) {
			approvalTypeString = i18n.t("PCM_ALLOFROLE_ONEPERGROUP");
			approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_GROUP;
		} else if (approvalType == ContentConstants.ApprovalSetup.ONE_OF_ROLE) {
			approvalTypeString = i18n.t("PCM_ONE_OF_ROLE");
			approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_ROLE;
		}

		var approvalTypeDiv = $("<div>").addClass("form-group");
		var approvalTypeLabelCell = $("<label>").addClass("col-sm-2 control-label").attr({'for': 'approvalType'}).text(i18n.t("PCM_APPROVAL_SETUP"));
		var approvalTypeSelectCell = $("<div>").addClass("col-sm-10");
		
		var approvalTypeField = $("<div>").text(approvalTypeString);
		
		if (this._contentManagerView) {
			approvalTypeField = $("<select>", {'class':'form-control', 'name': 'approvalType'});
			approvalTypeField.append(new Option(i18n.t("PCM_SETUP_DEFAULT"), ContentConstants.ApprovalSetup.DEFAULT));
			approvalTypeField.append(new Option(i18n.t("PCM_ALLOFROLE_ONEPERGROUP"), ContentConstants.ApprovalSetup.ONE_OF_GROUP));
			approvalTypeField.append(new Option(i18n.t("PCM_ONE_OF_ROLE"), ContentConstants.ApprovalSetup.ONE_OF_ROLE));
			approvalTypeField.val(approvalTypeCode);
			
			// When somebody changes the value of the input.
			approvalTypeField.on("change", function() {
				
				// Execute the necessary call backs.
				$.each(this._onApprovalTypeChange, _.bind(function (index, callback) {
					callback(approvalTypeField.val());
				}, this));
				
			}.bind(this));
		}
		approvalTypeSelectCell.append(approvalTypeField);
		
		approvalTypeDiv.append(approvalTypeLabelCell).append(approvalTypeSelectCell);

		return approvalTypeDiv;
	};

	/**
	 * Creates Approval Setup field panel
	 * @returns {JQueryObj}
	 */
	ContentSettingsView.prototype._createFieldTextType = function(){
		var textTypeDiv = $("<div>").addClass("form-group");

		var textTypeLabelCell = $("<label>", {'class' : 'col-sm-2 control-label'}).attr({"for" : "textType"}).text(i18n.t("PCM_TEXT_TYPE"));

		var textType =  this._settingsMap.get(ContentConstants.UpdateField.TEXTTYPE);

		// Depending on our permissions, add the input or the read only value.
		var textTypeInputValue = $("<div>", {'class' : 'col-sm-10'});
		var textTypeInput = $("<div>").text(textType);
		
		if (this._contentManagerView) {
			textTypeInput = $("<select>", {'class':'form-control','name': 'textType'});
			textTypeInput.append(new Option(i18n.t("PCM_SIMPLE_TEXT"), ContentConstants.TextType.SIMPLE));
			textTypeInput.append(new Option(i18n.t("PCM_RICH_TEXT"), ContentConstants.TextType.RICHTEXT));
			textTypeInput.append(new Option(i18n.t("PCM_SMART_NAME"), ContentConstants.TextType.SMARTNAME));
			textTypeInput.val(textType);
			
			textTypeInput.on("change", _.bind(function () {
				$.each(this._onTextTypeChange, _.bind(function (index, callback) {
					callback(textTypeInput.val());
				}, this));
			}, this));
		}
		textTypeInputValue.append(textTypeInput);
		
		textTypeDiv.append(textTypeLabelCell).append(textTypeInputValue);

		return textTypeDiv;
	};

	/**
	 * Creates language section that contains language indicators for master
	 * @returns {JQueryObj}
	 */
	ContentSettingsView.prototype._createMasterLanguagesList = function(){
		
		var languageMasterIndicator = this._settingsMap.get("LangIndMaster");
		
		var jqMasterElement = languageMasterIndicator.getJQElement();
		
		var masterLanguageFormGroupDiv = $("<div>").addClass("form-group");
		var masterLanguageLabelCell = $("<label>", {'class' : 'col-sm-2 control-label'}).attr({"for" : "textType"}).text(i18n.t("PCM_MASTER_LANGUAGE"));
		
		var masterLanguagesValues = $("<div>", {'class' : 'col-sm-10'});
		masterLanguagesValues.append(jqMasterElement);
		
		masterLanguageFormGroupDiv.append(masterLanguageLabelCell).append(masterLanguagesValues);
		
		// If master languages are not visible, then hide language DIV.
		if (languageMasterIndicator.getNumOfVisibleLanguages() == 0) {
			masterLanguageFormGroupDiv.hide();
		}
		
		return masterLanguageFormGroupDiv;
	};
	
	ContentSettingsView.prototype._createTranslationLanguagesList = function(){
		
		var languageTranslationIndicator = this._settingsMap.get("LangIndTrans");
		
		var jqTranslationElement = languageTranslationIndicator.getJQElement();				

		var translationLanguageFormGroupDiv = $("<div>").addClass("form-group translation-flags");
		var translationLanguageLabelCell = $("<label>", {'class' : 'col-sm-2 control-label'}).attr({"for" : "textType"}).text(i18n.t("PCM_TRANSLATION_LANGUAGE"));
		
		var translationLanguagesValues = $("<div>", {'class' : 'col-sm-10'});
		translationLanguagesValues.append(jqTranslationElement);
		
		translationLanguageFormGroupDiv.append(translationLanguageLabelCell).append(translationLanguagesValues);
		
		// If document is marked as invariant then don't create flags for translation languages.
		if (this._settingsMap.get(ContentConstants.UpdateField.INVARIANT)) {
			translationLanguageFormGroupDiv.hide();
		}
		
		// If translation languages are not visible, then hide language DIV.
		if (languageTranslationIndicator.getNumOfVisibleLanguages() == 0) {
			translationLanguageFormGroupDiv.hide();
		}
		
		return translationLanguageFormGroupDiv;
	};
	
	/**
	 * Add a custom button to the header of this view.
	 */
	ContentSettingsView.prototype.addCustomButton = function (button) {
		this._stateFullContentWidget.addCustomButton(button);
	};

	/**
	 * Get the wrapping jQuery element of the content setting.
	 */
	ContentSettingsView.prototype.getJQElement = function () {
		return this._stateFullContentWidget.getJQElement();
	};

	return ContentSettingsView;
});