/**
 * Content object
 * 
 * @author PASH
 */
define(["../text-content-utils/ContentUtils",
        "utils/webi18n!TextContentStrings"],
	function (ContentUtils,
			TextContentStrings) {
		"use strict";
		
		/**
		 * @class Represents a single text content object.
		 * @exports Content
		 * @constructor
		 */
		var Content = function (json) {
			if (json) {
				this.state = json.state;
				this.updated = json.updated;
				this.docVersionId = json.docVersionId;
				this.projectId = json.projectId;
				this.isApprovalPasswordRequired = ContentUtils.getBooleanValue(json.isApprovalPasswordRequired);
				this.isLatestDocVersion = ContentUtils.getBooleanValue(json.isLatestDocVersion);
				this.effectiveContentUserPermissions = json.effectiveContentUserPermissions;
			} else {
				this.state = null;
				this.updated = null;
				this.docVersionId = null;
				this.projectId = null;
				this.isApprovalPasswordRequired = false;
				this.isLatestDocVersion = false;
				this.effectiveContentUserPermissions = null;
			}
			this._saveActions = [];
		};
		
		/**
		 * Return the (GS1) element type for this element.
		 */
		Content.prototype.getElementType = function () {
			return this.elementType;
		};
		
		/**
		 * Return the state of the content document.
		 */
		Content.prototype.getState = function () {
			return this.state;
		};
		
		/**
		 * Clone this Content.
		 * 
		 * @returns {Content} A Content that is exactly the same.
		 */
		Content.prototype.clone = function () {
			var clone = new Content();
			
			// Return cloned object after copying all the content.
			return this.copyIn(clone);
		};
		
		/**
		 * Copy all information from this object to desired object.
		 * 
		 * @param copyInObj Object, where values needs to be updated from this. 
		 * @returns {Content} Object with all values of this object.
		 */
		Content.prototype.copyIn = function (copyInObj) {
			// Nothing to do if object is not of this type.
			if (!copyInObj instanceof Content) {
				return copyInObj;
			}
			
			// Update all the values from this object.
			copyInObj.updated = this.updated;
			copyInObj.docVersionId = this.docVersionId;
			copyInObj.projectId = this.projectId;
			copyInObj.isApprovalPasswordRequired = this.isApprovalPasswordRequired;
			copyInObj.isLatestDocVersion = this.isLatestDocVersion;
			copyInObj.effectiveContentUserPermissions = this.effectiveContentUserPermissions;
			
			// Return updated object.
			return copyInObj;
		};
		
		/**
		 * Return whether the current user can manage the content of this object.
		 */
		Content.prototype.canManage = function() {
			var result = {};
			
			// Make sure that this is the latest version of the document.
			if (!this.isLatestDocumentVersion()) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_DOC_NOT_LATEST");
				return result;
			}

			var wcrPerm = this.hasProperWcrPermissions();
			// Make sure that permissions are calculated
			if (!wcrPerm.allowed) {
				result.allowed = wcrPerm.allowed;
				result.message = wcrPerm.message;
				return result;
			}

			// Content manager permission is not assigned to the user.
			if (!this.effectiveContentUserPermissions.hasContentManagerPermissions) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NO_CONTENT_MANAGER_ROLE");
				return result;
			}

			result.allowed = true;
			result.message = undefined;
			return result;
		};
		
		/**
		 * Method provide information of sufficient permissions from WebCenter for current user.
		 * 
		 * @return
		 */
		Content.prototype.hasProperWcrPermissions = function() {
			var result = {};
			
			if (!this.effectiveContentUserPermissions) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NOT_CALCULATED");
				return result;
			}
			
			if (!this.effectiveContentUserPermissions.hasProperWcrPermissions) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_DOC_NO_WCR_CHECKIN");
				return result;
			}

			result.allowed = true;
			result.message = undefined;
			return result;
		};
		
		Content.prototype.isLatestDocumentVersion = function() {
			return this.isLatestDocVersion;
		};
		
		return Content;
});