/**
 * SingleTextContent object
 * 
 * @author PASH
 */
define(["underscore",
        "./TextContent",
		"./TextContentPermission",
		"../text-content-utils/ContentUtils",
		"../ContentConstants"
	],
	function (_,
		TextContent,
		TextContentPermission,
		ContentUtils,
		ContentConstants) {
		"use strict";
		
		/**
		 * @class Represents a single text content object.
		 * @exports SingleTextContent
		 * @constructor
		 */
		var SingleTextContent = function (json) {
			
			// Call the super
			TextContent.call(this, json);
			
			if (json) {
				this.permission = [];
				_.each(json.permission, _.bind(function (perm) {
					this.permission.push(new TextContentPermission(perm));
				}, this));
			} else {
				this.permission = [];
			}
		};
		
		// inherit
		SingleTextContent.prototype = Object.create(TextContent.prototype);
		// correct the constructor pointer 
		SingleTextContent.prototype.constructor = SingleTextContent;
		
		/**
		 * Clone this single text content object.
		 * 
		 * @returns {SingleTextContent} Cloned object of this. 
		 */
		SingleTextContent.prototype.clone = function () {
			// Create new object to make clone of this object.
			var clone = new SingleTextContent();
			
			// Update permissions
			clone.permission = [];
			_.each(this.permission, _.bind(function (perm) {
				clone.permission.push(perm.clone());
			}, this));
			
			// Return cloned object after copying all the content.
			return TextContent.prototype.copyIn.call(this, clone);
		};
		
		/**
		 * This will update the permission changes done in role mapping page
		 * 
		 * @param permissionsArray
		 */
		SingleTextContent.prototype.updatePermissions = function (permissionsArray) {
			if (permissionsArray.length > 0) {
				this.permission = permissionsArray ;
				this._saveActions.push({
					type: ContentConstants.SaveAction.PERMISSIONS,
					timeStamp : (new Date()).getTime(),
					textContentPermissionsSet: permissionsArray
				});
				
			}
		};

		/**
		 * Change the element type of the TextContent object.
		 */
		SingleTextContent.prototype.setElementType = function (elementType) {
			if (this.elementType != elementType) {
				// Get current element type value as old and set element type with updated element type value.
				var oldElementType = this.elementType;
				this.elementType = elementType;
				
				// Get last save action.
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				
				// If latest save action is for element type change
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.ELEMENTTYPE) {
					// If element type is changed to new value.
					if (this.isElementTypeChanged()) {
						lastSaveAction.timeStamp = (new Date()).getTime();
						lastSaveAction.newElementType = elementType;
					} else {
						// In case element type became as previous value which is already saved in document.
						this._saveActions.splice(_.indexOf(this._saveActions, lastSaveAction), 1);
					}
				} else {
					// Create a save action for element type update.
					this._saveActions.push({
						affectedTextContent : this,
						type: ContentConstants.SaveAction.ELEMENTTYPE,
						timeStamp : (new Date()).getTime(),
						newElementType: elementType,
						oldElementType: oldElementType
					});
				}
			}
		};
		
		/**
		 * Return true if the Element type of the text Content document is changed.
		 */
		SingleTextContent.prototype.isElementTypeChanged = function () {
			// Find previous save action for text content element type change.
			var saveAction = this._getSaveActionByType(ContentConstants.SaveAction.ELEMENTTYPE);
			
			// Save action is not available, that means there is no change happened in element type.
			if (!saveAction) {
				return false;
			}
			
			// Element type is not same as old actual element type, so element type is changed.
			if (this.elementType != saveAction.oldElementType) {
				return true;
			}
			
			// Reached here, means element type of content is not changed.
			return false;
		};
		
		return SingleTextContent;
});