/**
 * TableContent object
 * @author PASH
 */
define(["underscore",
        "jquery",
        "./Content",
		"./TextContent",
		"./TextContentPermission",
		"./TableTextContent",
		"../ContentConstants",
		"../text-content-utils/ContentUtils",
		"./TextContentElement"],
	function (_, $,
		Content,
		TextContent,
		TextContentPermission,
		TableTextContent,
		ContentConstants,
		ContentUtils,
		TextContentElement) {
		"use strict";
		
		/**
		 * @class Represents a content statement.
		 * @exports TextContent
		 * @constructor
		 */
		var	TableContent = function (json) {
			// Call the super
			Content.call(this, json);
			
			if (json) {
				this.textContents = [];
				_.each(json.tableTextContents, _.bind(function (textContentJson) {
					this.textContents.push(new TableTextContent(textContentJson, this));
				}, this));
				this.invariant = ContentUtils.getBooleanValue(json.invariant);
				this.structuredElementType = json.structuredElementType;
				this.approvalType = json.approvalType;
				this.preview = json.preview;
				this.state = json.state;
				this.textType = ContentUtils.getValidTextType(json.type);
				this.effectiveUserPermissionLanguageCache = json.effectiveUserPermissionLanguageCache;
				this.isApprovalPasswordRequired = ContentUtils.getBooleanValue(json.isApprovalPasswordRequired);
				this.instanceId = json.instanceId ? json.instanceId : 1;
				this.locationList = json.locationList;
				this.permission = [];
				_.each(json.permission, _.bind(function (perm) {
					this.permission.push(new TextContentPermission(perm));
				}, this));
			} else {
				this.textContents = [];
				this.invariant = false;
				this.structuredElementType = null;
				this.approvalType = null;
				this.preview = null;
				this.state = null;
				this.textType = ContentUtils.getValidTextType();
				this.effectiveUserPermissionLanguageCache = null;
				this.isApprovalPasswordRequired = false;
				this.instanceId = 1;
				this.locationList = [];
				this.permission = [];
			}
		};

		// inherit
		TableContent.prototype = Object.create(Content.prototype);
		// correct the constructor pointer 
		TableContent.prototype.constructor = TableContent;
		
		/**
		 * Clone this TableContent.
		 * 
		 * @returns {TableContent} A TableContent that is exactly the same.
		 */
		TableContent.prototype.clone = function () {
			var clone = new TableContent();
			
			// Return cloned object after copying all the content.
			return this.copyIn(clone);
		};
		
		/**
		 * Copy all information from this object to desired object.
		 * 
		 * @param copyInObj Object, where values needs to be updated from this. 
		 * @returns {TableContent} Object with all values of this object.
		 */
		TableContent.prototype.copyIn = function (copyInObj) {
			// Nothing to do if object is not of this type.
			if (!copyInObj instanceof TableContent) {
				return copyInObj;
			}
			
			// Copy all the text contents
			copyInObj.textContents = [];
			_.each(this.textContents, function (textContent) {
				var clonedTextContent = textContent.clone();
				clonedTextContent._tableContent = copyInObj;
				copyInObj.textContents.push(clonedTextContent);
			});

			// Copy permission
			copyInObj.permission = [];
			_.each(this.permission, _.bind(function (perm) {
				copyInObj.permission.push(perm.clone());
			}, this));
			
			// Update other information from this object.
			copyInObj.invariant = this.invariant;
			copyInObj.structuredElementType = this.structuredElementType;
			copyInObj.approvalType = this.approvalType;
			copyInObj.state = this.state;
			copyInObj.textType = this.textType;
			copyInObj.preview = this.preview;
			copyInObj.effectiveUserPermissionLanguageCache = this.effectiveUserPermissionLanguageCache;
			copyInObj.isApprovalPasswordRequired = this.isApprovalPasswordRequired;
			copyInObj.instanceId = this.instanceId;
			copyInObj.locationList = this.locationList;
			
			// Return updated object after copying all the content.
			return Content.prototype.copyIn.call(this, copyInObj);
		};
		
		/**
		 * This will update the permission changes done in role mapping page
		 * 
		 * @param permissionsArray
		 */
		TableContent.prototype.updatePermissions = function (permissionsArray) {
			if (permissionsArray.length > 0) {
				this.permission = permissionsArray ;
				this._saveActions.push({
					type: ContentConstants.SaveAction.PERMISSIONS,
					timeStamp : (new Date()).getTime(),
					textContentPermissionsSet: permissionsArray
				});
			}
		};
		
		/**
		 * This method removed request element and its dependent translations.
		 * 
		 * @param tableContentElement
		 * @param elementType
		 * @param instanceId
		 * @returns {Boolean}
		 */
		TableContent.prototype.removeTableTextContent = function (tableContentElement, projMasterLanguages) {
			var removedOtherElements = false;
			
			var textContent = tableContentElement.parentStatement;
			
			// Check if the element selected is the last master to be deleted, in that case we have to remove the whole wcc
			if((tableContentElement.isMaster() && textContent.isLastMasterToBeRemoved(tableContentElement.getLangCode(), projMasterLanguages) ||	
					textContent.checkDeleteAction(tableContentElement))){
				removedOtherElements = true;
				
				// Iterate over all the elements of this text content and mark them for delete.
				$.each(textContent.getElementsList(), _.bind(function (index, contentElement) {
					if (contentElement.getState() != ContentConstants.State.EMPTY) {
						contentElement.deleted = true;
						contentElement.newState = ContentConstants.State.DELETED;
					}
				}, this));
				
				// Since all the elements are removed for this text content, so mark this to remove from table content.
				this._saveActions.push({
					type: ContentConstants.SaveAction.REMOVETABLETEXTCONTENT,
					timeStamp : (new Date()).getTime(),
					elementTypeToBeRemoved : textContent.elementType,
					instanceIdToBeRemoved : textContent.instanceId,
					affectedTextContent : textContent
				});
			}
			
			// Mark current element also as deleted.
			tableContentElement.deleted = true;
			tableContentElement.newState = ContentConstants.State.DELETED;
			
			if (!removedOtherElements) {
				// Add a save action to remove the language, if other elements are not affected.
				this._saveActions.push({
					type: ContentConstants.SaveAction.REMOVEELEMENT,
					affectedTextContent: textContent,
					language: tableContentElement.language,
					timeStamp : (new Date()).getTime()
				});
			}
			
			// Check if the element selected was the last approved master to be deleted, in that case we have to revert status of the translation elements.
			if(tableContentElement.isMaster() && !textContent.isMasterApproved()) {
				// Iterate over all the elements of this text content and revert their state to draft.
				$.each(textContent.getElementsList(), _.bind(function (index, contentElement) {
					// Continue if element is not translation.
					if (contentElement.isMaster()) {
						return true;
					}
					
					// Nothing to do if element is deleted.
					if (contentElement.isDeleted()) {
						return true;
					}
					
					// Element is not even added, continue the loop.
					if (contentElement.getState() == ContentConstants.State.EMPTY) {
						return true;
					}
					
					// If translation element is not in draft state then revert state to draft. 
					if (ContentConstants.State.DRAFT != contentElement.getState()) {
						contentElement.setNewState(ContentConstants.State.DRAFT);
					}
				}, this));
			}
			
			return removedOtherElements;
		};
		
		/**
		 * Create JSON request object from TableContent
		 * 
		 * @param {String} password
		 * 
		 * @returns {Array} The list of all save actions
		 */
		TableContent.prototype.getSaveActions = function (password) {
			var allSaveActions = [];
			
			// Get the save actions of this content object.
			allSaveActions = allSaveActions.concat(this._saveActions);
			
			// Get the save actions of the text content elements.
			for (var i=0; i < this.textContents.length; i++) {
				allSaveActions = allSaveActions.concat(this.textContents[i].getSaveActions(password));
			}

			return allSaveActions;
		};
		
		/**
		 * Return whether this table content is invariant.
		 */
		TableContent.prototype.isInvariant = function() {
			return this.invariant;
		};
		
		/**
		 * Return the (GS1) element type for this table.
		 */
		TableContent.prototype.getElementType = function () {
			return this.structuredElementType;
		};
		
		/**
		 * Get the approval type of the TableContent object.
		 */
		TableContent.prototype.getApprovalType = function () {
			return this.approvalType;
		};
		
		/**
		 * Return the text type (rich or simple)
		 */
		TableContent.prototype.getTextType = function () {
			return this.textType;
		};
		
		/**
		 * Get the text contents for this table.
		 */
		TableContent.prototype.getTextContents = function () {
			return this.textContents;
		};
		
		/**
		 * Get the text content elements for this table.
		 */
		TableContent.prototype.getTextContentElements = function () {
			var allElements = [];
			$.each(this.textContents, function (i, textContent) {
				allElements = allElements.concat(textContent.getElementsList());
			});
			return allElements;
		};
		
		/**
		 * Get the unique collection of all language codes in the contained text content documents.
		 */
		TableContent.prototype.getLangCodes = function () {
			var langCodes = [];
			$.each(this.textContents, function (index, textContent) {
				var textContentlangCodes = textContent.getLangCodes();
				$.each(textContentlangCodes, function (index, langCode) {
					if ($.inArray(langCode, langCodes) < 0) {
						langCodes.push(langCode);
					}
				});
			});
			return langCodes;
		};
		
		/**
		 * Get the collection of master or translation language codes in the contained text content documents.
		 * 
		 * @param {Boolean} isMaster -  true will return master languages otherwise translation languages
		 * @returns {Array.<String>} langCodes - Array of language codes
		 */
		TableContent.prototype.getLangCodesOfElements = function (isMaster) {
			var langCodes = [];
			$.each(this.textContents, function (index, textContent) {
				var textContentlangCodes = textContent.getLangCodesOfElements(isMaster);
				$.each(textContentlangCodes, function (index, langCode) {
					if ($.inArray(langCode, langCodes) < 0) {
						langCodes.push(langCode);
					}
				});
			});
			return langCodes;
		};
		
		/**
		 * A method to get the master languages which are Signed Off
		 * 
		 * @param availableMasterLangs - Array of language objects
		 * @returns {Array}
		 */
		TableContent.prototype.getSignedOffMasterLangs = function(availableMasterLangs) {
			var signedOffMasterLangs = [];
			$.each(this.textContents, function (index, textContent) {
				var signedOffMasterLangsFromTextContent = textContent.getSignedOffMasterLangs(availableMasterLangs);
				$.each(signedOffMasterLangsFromTextContent, function (index, lang) {
					if ($.inArray(lang, signedOffMasterLangs) < 0) {
						signedOffMasterLangs.push(lang);
					}
				});
				
				//Remove a lang already added in the array but is not signed off in the new TextContent
				signedOffMasterLangs = $.grep(signedOffMasterLangs, function( lang, index ) {
					return ($.inArray(lang, signedOffMasterLangsFromTextContent) > 0);
				});
				
			});
			
			return signedOffMasterLangs;
		};
		
		/**
		 * Set state of the table content.
		 * 
		 * @param {ContentConstants.State} Content state as approved or as draft.
		 */
		TableContent.prototype.setNewState = function(newState) {
			// If new state is already updated.
			if (newState == this.state) {
				return;
			}
			
			// If all the text content elements are not signed off or not having resolved text data, then content is not allowed to be approved.
			if (!(this.isElementsApproved() && this.isTextDataResolved()) && ContentConstants.State.APPROVED == newState) {
				// Unfortunately this code is getting executed. Don't allow to approve.
				return;
			}
			
			// Create save action for table approval.
			this._processTableApproval(newState);
		};
		
		/**
		 * Update current state of the table content based on text contents available in it.
		 */
		TableContent.prototype.updateCurrentState = function () {
			var currentState = ContentConstants.State.APPROVED;
			
			// If all elements not are approved and text data for all elements is not resolved.
			if (!this.isElementsApproved() || !this.isTextDataResolved()) {
				// Keep current state of the table content as draft.
				currentState = ContentConstants.State.DRAFT;
			}
			
			// State and current state is not same. Create save action and update state.
			if (this.state != currentState) {
				
				// If content's text type is smart name and current state is approved.
				if (ContentConstants.State.APPROVED == currentState && ContentConstants.TextType.SMARTNAME == this.getTextType()) {
					// Don't update the state as approved, this should be done by setNewState method.
					return;
				}
				
				// Create save action for table approval.
				this._processTableApproval(currentState);
			}
		};
		
		/**
		 * Method is to get current state of this object.
		 * 
		 * @return {ContentConstants.State} Content state as approved or as draft.
		 */
		TableContent.prototype.getCurrentState = function () {
			return this.state;
		};

		/**
		 * 
		 */
		TableContent.prototype._processTableApproval = function (newState) {
			// Find previous reusable save action for table content approval.
			var saveAction = this._getReusableSaveAction(ContentConstants.SaveAction.TABLECONTENTAPPROVAL);
			
			// Create save action for updating state of table content, if not available.
			if (!saveAction) {
				// Create status save action for table content.
				this._saveActions.push({
					type: ContentConstants.SaveAction.TABLECONTENTAPPROVAL,
					timeStamp : new Date().getTime(),
					newState: newState,
					oldNewState: this.state
				});
			} else
			// If new state is became old state.
			if (newState == saveAction.oldNewState) {
				// Remove save action.
				this._saveActions.splice(_.indexOf(this._saveActions, saveAction), 1);
			} else
			// Update already available save action for new state.
			if (saveAction.newState != newState) {
				saveAction.timeStamp = new Date().getTime();
				saveAction.newState = newState;
			}
			
			// Update new state in state of this content.
			this.state = newState;
		};
		
		/**
		 * Find save action from available save actions based on type.
		 * 
		 * @param {ContentConstants.SaveAction} type: action type which needs to be found.
		 * 
		 * @return saveAction: if found then first occurrence of action, otherwise undefined.
		 */
		TableContent.prototype._getSaveActionByType = function (type) {
			// Find save action for requested type.
			var saveAction = undefined;
			
			if (type) {
				$.each(this._saveActions, _.bind(function (index, action) {
					if (type == action.type) {
						saveAction = action;
						return false;
					}
				}, this));
			}
			return saveAction;
		};
		
		
		/**
		 * Get reusable save action for the desired type of save action.
		 * 
		 * @param {ContentConstants.SaveAction} type: action type which needs to be found as last save action.
		 * 
		 * @return saveAction: if found as last action, otherwise undefined.
		 */
		TableContent.prototype._getReusableSaveAction = function (type) {
			// Get last save action.
			var lastSaveAction = this._saveActions[this._saveActions.length-1];
			
			// If last save action doesn't exist then return undefined.
			if (!lastSaveAction) {
				return undefined;
			}
			
			// If type is save as type of last save action, then return last save action.
			if (type && type == lastSaveAction.type) {
				return lastSaveAction;
			}
			
			// If last save action is not type of desired type, then return undefined.
			return undefined;
		};
		
		/**
		 * Return true if the current state of the Table Content document is changed.
		 */
		TableContent.prototype.isStatusChanged = function () {
			// Find previous save action for table content approval.
			var saveAction = this._getSaveActionByType(ContentConstants.SaveAction.TABLECONTENTAPPROVAL);
			
			// Save action is not available, that means there is no change happened in status.
			if (!saveAction) {
				return false;
			}
			
			// State is not same as old actual state, so state of table content is changed.
			if (this.state != saveAction.oldNewState) {
				return true;
			}
			
			// Reached here, means status is not changed.
			return false;
		};
		
		/**
		 * Return true if the current state of the Table Content document is approved.
		 */
		TableContent.prototype.isElementsApproved = function () {
			var isElementsApproved = true;	// Flag to check whether text data is approved or not.
			
			// Iterate over all the text contents to check status.
			$.each(this.getTextContents(), _.bind(function (index, textContent) {
				// In case 1 or more than 1 element is not in approved state.
				if (!textContent.isElementsApproved()) {
					// Text data is not signed off. Set this flag to false, so content is not signed off.
					isElementsApproved = false;
					
					// Break the loop.
					return false;
				}
			}, this));
			
			return isElementsApproved;
		};
		
		/**
		 * Returns true if current status is resolved for all the text data. <br>
		 * 
		 * For rich text or simple text this flag is always considered as true.
		 * 
		 * @return {boolean} current resolved status for all available elements.
		 */
		TableContent.prototype.isTextDataResolved = function() {
			var isTextDataResolved = true;	// Flag to check whether text data is resolved or not.
			
			// Return true is text type of the document is not smart name type.
			if (ContentConstants.TextType.SMARTNAME != this.getTextType()) {
				return isTextDataResolved;
			}
			
			// Iterate over all the text contents to check resolved status.
			_.each(this.textContents, _.bind(function (textContent) {
				var elements = textContent.getElementsList();	// Get elements from this text content.
				
				// Iterate over all the elements to find status.
				_.each(elements, _.bind(function (element) {
				
					// If element does not actually exists in document.
					if (ContentConstants.State.DELETED == element.getCurrentState() || 
							ContentConstants.State.EMPTY == element.getCurrentState()) {
						return true;
					}
					
					// If status of smart named text data is not resolved.
					if (ContentConstants.ResolvedStatus.RESOLVED != element.getTextData().status) {
						// Text data is not resolved. Set this flag to false, so content is not signed off.
						isTextDataResolved = false;
					}
					
				}, this));
			}, this));
			
			return isTextDataResolved;
		};
		
		/**
		 * Check if any approval action (Approve/Reject) is done and password required
		 * 
		 * @returns isPasswordRequired {boolean}
		 */
		TableContent.prototype.isPasswordRequiredForApproval = function() {
			var isPasswordRequired = false;
			
			// If password is required for approval as set by Admin in preferences
			if(this.isApprovalPasswordRequired) {
				
				// then, check whether any approval action is done on any language
				_.each(this.textContents, _.bind(function (textContent) {
					if (!isPasswordRequired &&  textContent.isPasswordRequiredForApproval()){
						isPasswordRequired = true;
						return false;
					}
				}));
			}
			
			return isPasswordRequired;
		};
		
		/**
		 * Marking invariant flag as true in table content.
		 * 
		 * @param invariant
		 *            the invariant to set
		 */
		TableContent.prototype.setInvariant = function (invariant) {
			// In case invariant flag is same, no need to do anything.
			if (this.invariant == invariant) {
				return;
			}
			
			// Variable to create save action.
			var needToAdd = true;
			
			$.each(this._saveActions, function (index, saveAction) {
				// Continue loop if save action is not for invariant flag change. 
				if (ContentConstants.SaveAction.INVARIANT != saveAction.type) {
					return true;
				}
				
				// remove save action for invariant change because flag is same as original.
				if (invariant == saveAction.oldInvariant) {
					this._saveActions.splice(index, 1);
					needToAdd = false;
					return false;
				}
				
				// Update new value for invariant flag to save action.
				saveAction.timeStamp = (new Date()).getTime();
				saveAction.shouldBecomeInvariant = invariant;
				needToAdd = false;
			}.bind(this));

			if (needToAdd) {
				// Create save action and keep it with all save actions.
				this._saveActions.push({
					type: ContentConstants.SaveAction.INVARIANT,
					timeStamp : (new Date()).getTime(),
					shouldBecomeInvariant: invariant,
					oldInvariant: this.invariant
				});
			}
			
			// Assign latest invariant value.
			this.invariant = invariant;
			
			_.each(this.textContents, function (textContent) {
				textContent.invariant = invariant;
				// We couldn't find any matching element type, return undefined foundTextContent				
			}.bind(this));			
			
		};
		
		/**
		 * Change text type of the table content document. 
		 * 
		 * @param textType
		 * 
		 * @return {boolean} isTextTypeUpdated : If text type is updated then it will return true otherwise false.
		 */
		TableContent.prototype.setTextType = function (textType) {
			var isTextTypeUpdated = false;
			// In case text type is same, no need to do anything.
			if (this.textType == textType) {
				return isTextTypeUpdated;
			}
			
			// Variable to create save action.
			var needToAdd = true;
			
			$.each(this._saveActions, function (index, saveAction) {
				// Continue loop if save action is not for text type. 
				if (ContentConstants.SaveAction.TEXTTYPE != saveAction.type) {
					return true;
				}
				
				// remove save action for text type because text type is became same as original.
				if (textType == saveAction.oldTextType) {
					this._saveActions.splice(index, 1);
					needToAdd = false;
					isTextTypeUpdated = false;
					return false;
				}
				
				// Update new text type to save action of text type change.
				saveAction.timeStamp = (new Date()).getTime();
				saveAction.newTextType = textType;
				isTextTypeUpdated = true;
				needToAdd = false;
			}.bind(this));

			if (needToAdd) {
				// Create save action and keep it with all save actions.
				this._saveActions.push({
					type: ContentConstants.SaveAction.TEXTTYPE,
					timeStamp : (new Date()).getTime(),
					newTextType: textType,
					oldTextType: this.textType
				});
				
				isTextTypeUpdated = true;
			}
			
			// Assign latest text type.
			this.textType = textType;
			
			return isTextTypeUpdated;
		};
		
		/**
		 * Change the approval type of the TextContent object.
		 */
		TableContent.prototype.setApprovalType = function (approvalType) {
			// In case approval type is same, no need to do anything.
			if (this.approvalType == approvalType) {
				return;
			}
			
			// Variable to create save action.
			var needToAdd = true;
			
			$.each(this._saveActions, function (index, saveAction) {
				// Continue loop if save action is not for approval type change. 
				if (ContentConstants.SaveAction.APPROVALTYPE != saveAction.type) {
					return true;
				}
				
				// remove save action for approval type because approval type is became same as original.
				if (approvalType == saveAction.oldApprovalType) {
					this._saveActions.splice(index, 1);
					needToAdd = false;
					return false;
				}
				
				// Update new approval type to save action of approval type change.
				saveAction.timeStamp = (new Date()).getTime();
				saveAction.newApprovalType = approvalType;
				needToAdd = false;
			}.bind(this));

			if (needToAdd) {
				// Create save action and keep it with all save actions.
				this._saveActions.push({
					type: ContentConstants.SaveAction.APPROVALTYPE,
					timeStamp : (new Date()).getTime(),
					newApprovalType: approvalType,
					oldApprovalType: this.approvalType
				});
			}
			
			// Assign latest approval type.
			this.approvalType = approvalType;
		};
		
		/**
		 * Set the new preview.
		 */
		TableContent.prototype.setNewPreview = function (preview) {
			// In case the preview not really changed, no need to do anything.
			var falseDefaultChange = ( preview.type == "Default" && (!this.preview));
			if (falseDefaultChange || ( this.preview && this.preview.name == preview.name && this.preview.type == preview.type)) {
				return;
			}
			
			// Set the new preview.
			this.preview = preview;
			
			// Check if last save action is available as a new preview ID.
			var lastSaveAction = this._saveActions[this._saveActions.length-1];
			if (lastSaveAction && ContentConstants.SaveAction.NEWPREVIEW == lastSaveAction.type) {
				lastSaveAction.timeStamp = (new Date()).getTime();
				lastSaveAction.newPreview = preview;
			} else {
				// Create save action and keep it with all save actions.
				this._saveActions.push({
					type: ContentConstants.SaveAction.NEWPREVIEW,
					timeStamp : (new Date()).getTime(),
					newPreview: preview
				});
			}
		};
		
		/**
		 * Checks if the param elementType and param instanceId is already added in the TableContent. If yes, return the TableTextContent.
		 * Else, creates copy
		 * 
		 * @param elementType
		 * @param instanceId
		 * @returns TableTextContent
		 */
		TableContent.prototype.checkAndReturnTableTextContent = function(elementType, instanceId){
			var foundTextContent;
			_.each(this.textContents, function (textContent) {
				if((textContent.elementType).toLowerCase() == (elementType).toLowerCase() && textContent.instanceId == instanceId){
					foundTextContent = textContent;
					return false;
				}
				// We couldn't find any matching element type, return undefined foundTextContent				
			}.bind(this));			
			
			return foundTextContent;
		};
		
		/**
		 * Add new TableTextContent to Table Content
		 */
		TableContent.prototype.addTableTextContent = function (elementType, instanceId, languageCode, isMaster) {
						
			var newTextContent = new TableTextContent(null, this);
			newTextContent.effectiveContentUserPermissions = this.effectiveContentUserPermissions;
			newTextContent.instanceId = instanceId;
			newTextContent.elementType = elementType;
			newTextContent.textType = this.getTextType();
							
			var textContentElement = new TextContentElement(null, newTextContent);
			
			textContentElement.state = ContentConstants.State.DRAFT;
			textContentElement.content = {};
			textContentElement.pastApprovals = [];
			textContentElement.language = languageCode;
			textContentElement.master = isMaster;
			textContentElement.effectiveElementUserPermissions = this.effectiveUserPermissionLanguageCache[languageCode];
			
			newTextContent.element.push(textContentElement);
			this.textContents.push(newTextContent);
			
			var lastSaveAction = this._saveActions[this._saveActions.length-1];
			if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.ADDTABLETEXTCONTENT
					&& lastSaveAction.newElementType == elementType && lastSaveAction.newInstanceId == instanceId) {
				lastSaveAction.timeStamp = (new Date()).getTime();
				lastSaveAction.newElementType = elementType;
				lastSaveAction.newInstanceId = instanceId;
				lastSaveAction.newElementLanguage = languageCode;
				lastSaveAction.isElementMaster = isMaster;
			} else {
				this._saveActions.push({
					type: ContentConstants.SaveAction.ADDTABLETEXTCONTENT,
					timeStamp : (new Date()).getTime(),
					newElementType: elementType,
					newInstanceId: instanceId,
					newElementLanguage: languageCode,
					isElementMaster: isMaster
				});
			}
			
			return textContentElement;
		};
		
		/**
		 * This method is used to identify if the element/parent passed was marked for delete. In that case, remove that entry from saveactions
		 * 
		 * @param csElement
		 */
		TableContent.prototype.removeMatchingSaveActions = function(csElem){
			for(var i=0; i<this._saveActions.length; i++){
				if(csElem.language == this._saveActions[i].language && 
						this._saveActions[i].type == ContentConstants.SaveAction.REMOVEELEMENT){
					this._saveActions.splice(i, 1);
					break;
				}
				
				if(csElem.parentStatement.instanceId == this._saveActions[i].instanceIdToBeRemoved && 
						csElem.parentStatement.elementType ==  this._saveActions[i].elementTypeToBeRemoved &&
						this._saveActions[i].type == ContentConstants.SaveAction.REMOVETABLETEXTCONTENT){
					this._saveActions.splice(i, 1);
					break;
				}
			}
		};
		
		/**
		 * This method will parse through all the tableTextContents and find the element types for which master language is added,
		 * and return the element types (element type code)
		 */
		TableContent.prototype.getMasterElementTypes = function(){
			var elementTypesArray = [];
			for(var i=0; i<this.textContents.length; i++){
				var elements = this.textContents[i].getMasterElements();
				if(elements.length > 0) 
					elementTypesArray.push(elements[0]);
			}
			return elementTypesArray;
		};
		

		/**
		 * Get a content ID for this object. (It's only unique within a content object).
		 */
		TableContent.prototype.getContentId = function () {
			return this.structuredElementType;
		};
		
		return TableContent;
});