/**
 * TableTextContent object
 * 
 * @author PASH
 */
define(["underscore",
        "packaging-content-management/content/TextContent",
		"packaging-content-management/content/TextContentPermission",
		"packaging-content-management/text-content-utils/ContentUtils",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"packaging-content-management/ContentConstants",
		"packaging-content-management/content/TextContentElement"
	],
	function (_,
		TextContent,
		TextContentPermission,
		ContentUtils,
		LanguageUtils,
		ContentConstants,
		TextContentElement) {
		"use strict";
		
		/**
		 * @class Represents a table text content object.
		 * @exports TableTextContent
		 * @constructor
		 */
		var TableTextContent = function (json, tableContent) {
			this._tableContent = tableContent;
			
			// Call the super
			TextContent.call(this, json);
		};

		// inherit
		TableTextContent.prototype = Object.create(TextContent.prototype);
		// correct the constructor pointer 
		TableTextContent.prototype.constructor = TableTextContent;
		
		/**
		 * Clone this TableContent.
		 * 
		 * @returns {TableTextContent} A TableTextContent that is exactly the same.
		 */
		TableTextContent.prototype.clone = function () {
			var clone = new TableTextContent();
			
			// Return cloned object after copying all the content.
			return TextContent.prototype.copyIn.call(this, clone);
		};
		
		/**
		 * Get a content ID for this object. (It's only unique within a content object).
		 */
		TableTextContent.prototype.getContentId = function () {
			return this.elementType + "_" + this.instanceId;
		};
		
		/**
		 * Find out whether document is latest version or not.
		 * @returns
		 */
		TableTextContent.prototype.isLatestDocumentVersion = function () {
			return this._tableContent.isLatestDocumentVersion();
		};
		
		/**
		 * super{@link #getPermission()}
		 */
		TableTextContent.prototype.getPermission = function () {
			return this._tableContent.getPermission();
		};
		
		
		/**
		 * super{@link #isLatestDocumentVersion()}
		 */
		TableTextContent.prototype.isLatestDocumentVersion = function () {
			return this._tableContent.isLatestDocumentVersion();
		};
		
		/**
		 * Return the parent table content.
		 */
		TableTextContent.prototype.getParent = function () {
			return this._tableContent;
		};
		
		/**
		 * Checks if there are any translations in the document for given element type
		 * 
		 * @param csElem
		 * @returns {Boolean}
		 */
		TableTextContent.prototype.hasTranslations = function(csElem) {			
			var totalTranslations = $.grep(csElem.parentStatement.element, function(element, index){
				return !element.isMaster() && element.state !== 'EMPTY' && !element.deleted;
			});
			
			return (totalTranslations.length > 0);
		};
		
		/**
		 * Return cached user permissions for an element with the passed langcode.
		 * 
		 * @param langCode
		 * @returns
		 */
		TableTextContent.prototype.getCachedUserPermission = function(langCode) {
			if (this._tableContent.effectiveUserPermissionLanguageCache) {
				return this._tableContent.effectiveUserPermissionLanguageCache[langCode];
			} else {
				return undefined;
			}
		};
				
		/**
		 * Checks if the element passed is the last element to be deleted. Also checks if other elements are already marked for delete
		 * 
		 * @param element
		 */
		TableTextContent.prototype.checkDeleteAction = function(csElement) {
			// There is only one element in the array, which means we will have to delete full text content (wcc)
			if(this.element.length == 1){
				return true;
			}
			
			// Leaving out the csElement passed, check for rest of the elements in the TextContent if they are marked for delete.
			// if yes, add increment the counter
			var deleteCounter = 0;
			for(var i=0; i<this.element.length; i++){
				if(this.element[i].lang !== csElement.lang && this.element[i].deleted){
					deleteCounter++;
				}else{
					// Found an element that is not marked for delete, return false - breaks the loop
					return false;
				}
			}
			
			if(deleteCounter.length == this.element.length-1)
				return false;
		};
		
		/**
		 * This method is to set new state with calculation done by it's elements state.
		 * 
		 * Setting up current state of this object doesn't required to be saved on document.
		 * So save action will not be created for this operation.
		 */
		TableTextContent.prototype.updateCurrentState = function () {
			TextContent.prototype.updateCurrentState.call(this);
			
			// Update current state of the parent object.
			this.getParent().updateCurrentState();
		};
		
		/**
		 * Return true if the language passed is the last master to be removed
		 * else false
		 */
		TableTextContent.prototype.isLastMasterToBeRemoved = function (language, projectMasterLanguages) {
			// Iterate over all elements.
			for(var i=0; i < this.element.length ;i++) {
				// If element is not for master language then continue.
				if (!this.element[i].isMaster()) {
					continue;
				}
				
				// If element is in same which is being removed then continue.
				if (this.element[i].language == language) {
					continue;
				}
				
				// If element is deleted then continue.
				if (this.element[i].isDeleted()) {
					continue;
				}
				
				// If element is not even added.
				if (ContentConstants.State.EMPTY == this.element[i].getCurrentState()) {
					continue;
				}
				
				 if (!LanguageUtils.isAProjectAttrLang(projectMasterLanguages, this.element[i].language)) {
                     continue;
				 }
				 
				// If execution reaches here, that means deleted element was not last master.
				return false;
			}
			
			// The element which is removed was last master.
			return true;
		};
		
		
		return TableTextContent;
});




