/**
 * TextContent object
 * @author PASH
 */
define(["underscore",
        "./Content",
		"./TextContentElement",
		"./SingleTextContent",
		"../ContentConstants",
		"../text-content-services/languageService",
		"../text-content-utils/LanguageUtils",
		"../text-content-utils/ContentUtils"],
	function (_,
		Content,
		TextContentElement,
		SingleTextContent,
		ContentConstants,
		languageService,
		LanguageUtils,
		ContentUtils) {
		"use strict";

		/**
		 * @class Represents a content statement.
		 * @exports TextContent
		 * @constructor
		 */
		var	TextContent = function (json) {
			// Call the super
			Content.call(this, json);
			
			if (json) {
				this.element = [];
				_.each(json.element, _.bind(function (elem) {
					this.element.push(new TextContentElement(elem, this));
				}, this));
				this.elementType = json.elementType;
				this.invariant = ContentUtils.getBooleanValue(json.invariant);
				this.approvalType = json.approvalType;
				this.state = json.state;
				this.textType = ContentUtils.getValidTextType(json.type);
				this.maxChars = json.maxChars;
				this.attributeId = json.attributeId;
				this.isApprovalPasswordRequired = ContentUtils.getBooleanValue(json.isApprovalPasswordRequired);
				this.instanceId = json.instanceId ? json.instanceId : 1;
				this.locationList = json.locationList;
				this.effectiveUserPermissionLanguageCache = json.effectiveUserPermissionLanguageCache;
			} else {
				this.element = [];
				this.elementType = null;
				this.invariant = false;
				this.approvalType = null;
				this.state = null;
				this.textType = ContentUtils.getValidTextType();
				this.maxChars = null;
				this.attributeId = null;
				this.isApprovalPasswordRequired = false;
				this.instanceId = 1;
				this.locationList = [];
			}
		};
		
		// inherit
		TextContent.prototype = Object.create(Content.prototype);
		// correct the constructor pointer 
		TextContent.prototype.constructor = TextContent;
		
		/**
		 * Get a content ID for this object. (It's only unique within a content object).
		 */
		TextContent.prototype.getContentId = function () {
			return this.elementType+"_"+this.instanceId;
		};
		
		
		/**
		 * Return the text type (rich or simple)
		 */
		TextContent.prototype.getTextType = function () {
			return this.textType;
		};
		
		/**
		 * Return the instance number.
		 */
		TextContent.prototype.getInstance = function () {
			return this.instanceId;
		};
		
		/**
		 * Get all the master elements.
		 */
		TextContent.prototype.getMasterElements = function() {
			return $.grep(this.element, function(element, index){
				return element.master && element.state !== ContentConstants.State.EMPTY && !element.deleted;
			});
		};
		
		/**
		 * Get all the translation elements.
		 */
		TextContent.prototype.getTranslationElements = function() {
			return $.grep(this.element, function(element, index){
				return !element.master && element.state !== ContentConstants.State.EMPTY && !element.deleted;
			});
		};
		
		/**
		 * Return the list of TextContentElement, regardless of
		 * whether it's deleted or not.
		 */
		TextContent.prototype.getElementsList = function() {
			return this.element;
		};
		
		/**
		 * A method to get the TextContentElement languages of this TextContent.
		 * 
		 * @param {boolean} isMaster - true will return master languages otherwise translation languages
		 */
		TextContent.prototype.hasOnlyOneMaster = function() {
			return this.getMasterElements().length == 1;
		};
		
		/**
		 * Get all the language codes of the elements in this TextContent.
		 */
		TextContent.prototype.getLangCodes = function() {
			var masterLangCodes = this.getLangCodesOfElements(true);
			var transLangCodes = this.getLangCodesOfElements(false);
			var langCodes =  masterLangCodes.concat(transLangCodes);
			return langCodes;
		};
		
		/**
		 * A method to get the TextContentElement languages of this TextContent.
		 * 
		 * @param {boolean} isMaster - true will return master languages otherwise translation languages
		 */
		TextContent.prototype.getLangCodesOfElements = function(isMaster) {
			var langs = [];
			_.each(this.element, _.bind(function (element) {
				if (isMaster == element.master) {
					if (!element.deleted && element.state !== ContentConstants.State.EMPTY) {
						langs.push(element.language);
					}
				}
			}, this));
			return langs;
		};
		
		/**
		 * A method to get the master languages which are SignedOff/Approved
		 * 
		 * @param availableMasterLangs - Array of language objects
		 * @returns {Array}
		 */
		TextContent.prototype.getSignedOffMasterLangs = function(availableMasterLangs) {
			var signedOffMasterLangs = [];
			var signedOffMasterLangCodes = [];
			
			_.each(this.getMasterElements(), _.bind(function (element) {
				if (ContentConstants.State.APPROVED === element.state) {
					signedOffMasterLangCodes.push(element.language);
				}
			}, this));
			
			if (!availableMasterLangs) {
				return signedOffMasterLangCodes;
			}
			
			for ( var i = 0; i < availableMasterLangs.length; i++) {
				for ( var j = 0; j < signedOffMasterLangCodes.length; j++) {
					if (signedOffMasterLangCodes[j] == availableMasterLangs[i].langCode) {
						signedOffMasterLangs.push(availableMasterLangs[i]);
					}
				}
			}
			
			return signedOffMasterLangs;
		};
		
		
		/**
		 * This will return an array of status for Master/Translation
		 * 
		 * @param isMaster
		 */
		TextContent.prototype.getLangStatusOfElements = function(isMaster) {
			var status = [];
			_.each(this.element, _.bind(function (element) {
				if (isMaster == element.master) {
					status.push(element.state);
				}
			}, this));
			return status;
		};
		
		/**
		 * A method to get the TextContentElement of this ContentStatement for a given language.
		 * 
		 * @param language
		 * @param isMaster
		 */
		TextContent.prototype.addLanguage = function(language, isMaster, callback) {
			
			// language can be an object or a code as string.
			var langCode = (typeof language === "object") ? language.langCode : language;
			
			// If this is an invariant statement and we're adding a translation language, this operation should do nothing.
			if (!isMaster && this.invariant) {
				window.console.log("An attempt was made to add a translation language ("+langCode+") to an invariant statement.");
				return;
			}
			
			// We need to create a new TextContentElement for this language.
			// First check whether we do not already a TextContentElement for this language.
			var csElem = this.getElement(langCode, isMaster);
			
			if (csElem && !csElem.deleted) {
				// The textContentElement is already added.
				return;
			} else if (csElem && csElem.deleted) {				
				// The TextContentElement was added and deleted, undelete it.
				csElem.deleted = false;
				
				// after resetting the deleted flag, check in save actions if this element was added to delete, 
				// if yes remove it from save actions
				for(var i=0; i<this._saveActions.length; i++){
					if(csElem.language == this._saveActions[i].language && 
							this._saveActions[i].type == ContentConstants.SaveAction.REMOVEELEMENT){
						this._saveActions.splice(i, 1);
						break;
					}						
				}
				
				// Return the newly added csElem with callback.
				callback(csElem);
			
			} else {
				
				// No such elements exists, create a new text content element.
				csElem = new TextContentElement();			
				csElem.master = isMaster;
				csElem.state = ContentConstants.State.DRAFT;
				csElem.language = langCode;
				csElem.created = new Date();
				csElem.updated = new Date();
				csElem.pastApprovals = [];
				csElem.approval = [];
				csElem.comment = [];
				csElem.deleted = false;
				csElem.commentChanged = false;
				csElem.approvalChanged = false;
				csElem.parentStatement = this;
				csElem.isApprovalPasswordRequired = false;
				
				// Get the effective user permissions from the cache.
				if (this.getCachedUserPermission(langCode)) {
					csElem.effectiveElementUserPermissions = this.getCachedUserPermission(langCode);
				}
				
				// Create the empty content for this element.
				var textData = {};
				if (this.getTextType() == ContentConstants.TextType.SMARTNAME) {
					textData.content = "";
					textData.smartName = "";
					textData.status = ContentConstants.ResolvedStatus.NOT_RESOLVED;
				} else if (this.getTextType() == ContentConstants.TextType.RICHTEXT) {
					textData.content = "<p><br/></p>";
				} else {
					textData.content = "";
				}
				csElem.content = textData;
				
				// And add it to the list of elements.
				this.element.push(csElem);
				
				// Add a Save Action
				this._saveActions.push({
					type: ContentConstants.SaveAction.ADDELEMENT,
					timeStamp : (new Date()).getTime(),
					textDataString: csElem.content.content,
					language: langCode,
					isMaster: isMaster,
					affectedTextContent: this 
				});
				
				// Return the newly added csElem with callback.
				callback(csElem);
			}
		};
		
		
		/**
		 * Remove the specified Text Content Element specified by the "language" and "isMaster" parameters 
		 * but only if its text data is "empty".
		 * 
		 * @param language
		 * @param isMaster
		 */
		TextContent.prototype.removeEmptyLanguage = function(language, isMaster) {
			
			// Variable whether we should removing the language.
			var remove = false;
			
			// parse input
			var langCode = (typeof language === "object") ? language.langCode : language;
			
			// Get the element.
			var csElem = this.getElement(langCode, isMaster);
			
			// Check if an element could be found.
			if (!csElem) {
				return;
			}
			
			// Check if the state of the element is in DRAFT mode.
			if (csElem.getCurrentState() != ContentConstants.State.DRAFT) {
				return;
			}
			
			// Check if the element has text data.
			var textData = csElem.getTextData();
			
			// If the element has text data.
			if (textData) {
				// Get the encoded text content of the element and measure if there's any actual real content.
				var textLength = 0;
				if (this.textType == ContentConstants.TextType.RICHTEXT ||this.textType == ContentConstants.TextType.SIMPLE) {
					var encodedContent = csElem.getTextData().content;
					textLength = $('<div>').html(encodedContent).text().trim().length;
				} else if (this.textType == ContentConstants.TextType.SMARTNAME) {
					var encodedContent = csElem.getTextData().smartname;
					textLength = encodedContent.trim().length;
				}
				remove = textLength == 0;
			} else {
				remove = true;
			}
			
			// If there's no real actual content, remove the language.
			if (remove) {
				return this.removeLanguage(language, isMaster);
			};
		};
		
		/**
		 * Remove the specified Text Content Element specified by the "language" and "isMaster" parameters.
		 * 
		 * @param language
		 * @param isMaster
		 */
		TextContent.prototype.removeLanguage = function(language, isMaster) {
			
			// parse input
			var langCode = (typeof language === "object") ? language.langCode : language;
			
			// Get the element.
			var csElem = this.getElement(langCode, isMaster);
			
			// Make sure you're not removing the only master.
			if (csElem.isMaster() && this.hasOnlyOneMaster()) {
				return;
			}
			
			// Make sure that you're not removing the inly signedoff master.
			var signedOffMasterLangs = this.getSignedOffMasterLangs();
			if (csElem.isMaster() && signedOffMasterLangs.length == 1 && csElem.language == signedOffMasterLangs[0]) {
				return;
			}
			
			// Check if it can be found.
			if (csElem) {
				
				// Set the deleted mark
				csElem.deleted = true;
				
				// Add a save action to remove the language.
				this._saveActions.push({
					type: ContentConstants.SaveAction.REMOVEELEMENT,
					affectedTextContent: this,
					language: langCode,
					timeStamp : (new Date()).getTime()
				});
				
				// Return the removed TextContentElement
				return csElem;
			};
		};
		
		/**
		 * Return cached user permissions for a language.
		 * 
		 * @param langCode
		 */
		TextContent.prototype.getCachedUserPermission = function(langCode) {
			if (this.effectiveUserPermissionLanguageCache) {
				return this.effectiveUserPermissionLanguageCache[langCode];
			} else {
				return undefined;
			}
		};
		
		/**
		 * Return the TextContentElement of this ContentStatement for a given language, regardless of
		 * whether it's deleted or not.
		 * 
		 * @param langCode
		 * @param isMaster
		 */
		TextContent.prototype.getElement = function(langCode, isMaster) {
			// Get it from the existing language elements.
			var element = undefined;
			_.each(this.element, _.bind(function (csElem) {
				if (csElem.language === langCode && csElem.master == isMaster) {
					element = csElem;
				}
			}, this));
			
			return element;
		};

		/**
		 * Get the approval type of the TextContent object.
		 */
		TextContent.prototype.getApprovalType = function () {
			return this.approvalType;
		};
		
		/**
		 * A method to get the TextContentElement of this ContentStatement for a given language. It will
		 * only return non deleted elements. If no such element exists for the langcode, an "unadded" one will
		 * be returned.
		 * 
		 * @param langCode
		 * @param isMaster
		 */
		TextContent.prototype.provideElement = function(langCode, isMaster) {
			// Get it from the existing language elements.
			var element = undefined;
			_.each(this.element, _.bind(function (csElem) {
				if (csElem.language === langCode && csElem.master == isMaster && !csElem.deleted) {
					element = csElem;
				}
			}, this));
			
			// Create an unAdded one we could not find an added one.
			if (!element && langCode) {
				element = new TextContentElement();
				element.master = isMaster;
				element.content = {};
				element.state = ContentConstants.State.EMPTY;
				element.language = langCode;
				element.parentStatement = this;
			}
			
			return element;
		};

		/**
		 * Change the approval type of the TextContent object.
		 */
		TextContent.prototype.setApprovalType = function (approvalType) {
			if (this.approvalType != approvalType) {
				this.approvalType = approvalType;
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.APPROVALTYPE) {
					lastSaveAction.timeStamp = (new Date()).getTime();
					lastSaveAction.newApprovalType = approvalType;
				} else {
					this._saveActions.push({
						type: ContentConstants.SaveAction.APPROVALTYPE,
						timeStamp : (new Date()).getTime(),
						newApprovalType: approvalType
					});
				}
			}
		};
		
		/**
		 * Set whether the TextContent object is invariant or not.
		 */
		TextContent.prototype.setInvariant = function (invariant) {
			if (this.invariant != invariant) {
				this.invariant = invariant;
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.INVARIANT) {
					lastSaveAction.timeStamp = (new Date()).getTime();
					lastSaveAction.shouldBecomeInvariant = invariant;
				} else {
					this._saveActions.push({
						type: ContentConstants.SaveAction.INVARIANT,
						timeStamp : (new Date()).getTime(),
						shouldBecomeInvariant: invariant
					});
				}
			}
		};
		
		/**
		 * Set text type of the TextContent object.
		 */
		TextContent.prototype.setTextType = function (textType) {
			// If text type is updated.
			if (this.textType != textType) {
				// Get current text type value as old and set text type with updated text type value.
				var oldTextType = this.textType;
				this.textType = textType;
				
				// Get last save action.
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				
				// If latest save action is for text type change
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.TEXTTYPE) {
					// If text type is changed to new value.
					if (this.isTextTypeChanged()) {
						lastSaveAction.timeStamp = (new Date()).getTime();
						lastSaveAction.newTextType = textType;
					} else {
						// In case text type became as previous value which is already saved in document.
						this._saveActions.splice(_.indexOf(this._saveActions, lastSaveAction), 1);
					}
				} else {
					// Create a save action for text type update.
					this._saveActions.push({
						type: ContentConstants.SaveAction.TEXTTYPE,
						timeStamp : (new Date()).getTime(),
						newTextType: textType,
						oldTextType: oldTextType
					});
				}
			}
		};
		
		/**
		 * Return true if the text type of the text Content document is changed.
		 */
		TextContent.prototype.isTextTypeChanged = function () {
			// Find previous save action for text content text type change.
			var saveAction = this._getSaveActionByType(ContentConstants.SaveAction.TEXTTYPE);
			
			// Save action is not available, that means there is no change happened in text type.
			if (!saveAction) {
				return false;
			}
			
			// Text type is not same as old actual text type, so text type is changed.
			if (this.textType != saveAction.oldTextType) {
				return true;
			}
			
			// Reached here, means text type of content is not changed.
			return false;
		};
		
		/**
		 * Get the max chars of the TextContent object.
		 */
		TextContent.prototype.getMaxChars = function () {
			return this.maxChars;
		};
		
		/**
		 * Change the max char TextContent object.
		 */
		TextContent.prototype.setMaxChars = function (maxChars) {
			if (this.maxChars != maxChars) {
				this.maxChars = maxChars;
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.MAXCHARS) {
					lastSaveAction.timeStamp = (new Date()).getTime();
					lastSaveAction.newMaxChars = maxChars;
				} else {
					this._saveActions.push({
						type: ContentConstants.SaveAction.MAXCHARS,
						timeStamp : (new Date()).getTime(),
						newMaxChars: maxChars
					});
				}
			}
		};
		

		/**
		 * Method is to remove all the HTML tags from text content string
		 * @param textContentWithHTML {String} - text content string with HTML tags.
		 * @returns {String} - text content string without HTML tags.
		 */
		TextContent.prototype._removeHTMLTags = function (textContentWithHTML) {
			return $('<p>').html(textContentWithHTML).text();
		};
		
		/**
		 * Create JSON request object from ContentStatement
		 * 
		 * @param {String} password
		 * 
		 * @returns {Array} The list of all save actions
		 */
		TextContent.prototype.getSaveActions = function (password) {
			
			// Example of a saveActions:
			// [{
			//		type: "ContentInvariantSaveAction",
			//		timeStamp : "1471341651001",
			//		shouldBecomeInvariant: true
			// },{
			//		type: "TextContentElementTextDataSaveAction",
			//		timeStamp: "1471341651001",
			//		textDataString: "Some Text",
			//		affectedTextContent: "ENERGY_VALUE_1",
			//		affectedTextContentElement: "ENERGY_VALUE_1_en-US"
			//}]
			
			var allSaveActions = [];
			
			// Get the save actions of this content object.
			allSaveActions = allSaveActions.concat(this._saveActions);
			
			// Get the save actions of the text content elements.
			for (var i=0; i < this.element.length; i++) {
				allSaveActions = allSaveActions.concat(this.element[i].getSaveActions(password));
			}

			return allSaveActions;
		};
		
		/**
		 * Clone this ContentStatement.
		 * 
		 * @returns {TextContent} A TextContent that is exactly the same.
		 */
		TextContent.prototype.clone = function () {
			var clone = new TextContent();
			
			// Return cloned object after copying all the content.
			return this.copyIn(clone);
		};
		
		/**
		 * Copy all information from this object to desired object.
		 * 
		 * @param copyInObj Object, where values needs to be updated from this. 
		 * @returns {TextContent} Object with all values of this object.
		 */
		TextContent.prototype.copyIn = function (copyInObj) {
			// Nothing to do if object is not of this type.
			if (!copyInObj instanceof TextContent) {
				return copyInObj;
			}
			
			// Update all elements in desired object.
			_.each(this.element, _.bind(function (elem) {
				var clonedElem = elem.clone();
				clonedElem.parentStatement = copyInObj;
				copyInObj.element.push(clonedElem);
			}, this));

			// Update other information from this object.
			copyInObj.elementType = this.elementType;
			copyInObj.invariant = this.invariant;
			copyInObj.textType = this.textType;
			copyInObj.maxChars = this.maxChars;
			copyInObj.state = this.state;
			copyInObj.element = [];
			copyInObj.attributeId = this.attributeId;
			copyInObj.approvalType = this.approvalType;
			copyInObj.isApprovalPasswordRequired = this.isApprovalPasswordRequired;
			copyInObj.effectiveUserPermissionLanguageCache = this.effectiveUserPermissionLanguageCache;
			
			// Return updated object after copying all the content.
			return Content.prototype.copyIn.call(this, copyInObj);
		};

		/**
		 * Indicate whether this statement changed state so far.
		 * 
		 * @returns {Boolean}
		 */
		TextContent.prototype.isChanged = function(){
			for (var i=0; i < this.element.length; i++) {
				var element = this.element[i];
				if (element.isChanged()) {
					return true;
				}
			}
			return this.getSaveActions("").length > 0;
		};
		
		/**
		 * Indicate whether this statement changed state so far.
		 * 
		 * @returns {Boolean}
		 */
		TextContent.prototype.isContentChanged = function(){
			for (var i=0; i < this.element.length; i++) {
				var element = this.element[i];
				if (element.isContentChanged()) {
					return true;
				}
			}
			return false;
		};
		
		
		/**
		 * Check if any approval action (Approve/Reject) is done and password required
		 * 
		 * @returns isPasswordRequired {boolean}
		 */
		TextContent.prototype.isPasswordRequiredForApproval = function() {
			var isPasswordRequired = false;
			
			// If password is required for approval as set by Admin in preferences
			if(this.isApprovalPasswordRequired) {
				
				// then, check whether any approval action is done on any language
				_.each(this.element, _.bind(function (csElement) {
					if (!isPasswordRequired &&  csElement.isPasswordRequiredForApproval()){
						isPasswordRequired = true;
						return false;
					}
				}));
			}
			
			return isPasswordRequired;
		};
		
		/**
		 * Check whether element is exists in this text content.
		 */
		TextContent.prototype.contains = function(element) {
			return ($.inArray(element, this.element) != -1);
		};
		
		/**
		 * Return whether at least a master text of this content is approved for the passed TextContentElement.
		 * 
		 * @param textContent
		 */
		TextContent.prototype.isMasterApprovedFor = function (elementToCheck) {
			// Make sure that this element is part of this SingleTextContent.
			if (!this.contains(elementToCheck)) {
				return false;
			}
			
			// Make sure that the master of this SingleTextContent is approved.
			// Extract the master elements
			var approvedMasterElements = $.grep(this.element, function(element, index){
				return element.master && !element.deleted && element.getCurrentState() == ContentConstants.State.APPROVED;
			});
			
			if (approvedMasterElements.length > 0) {
				return true;
			} else {
				return false;
			}
		};
		
		/**
		 * Return whether any other master statements is signed off.
		 * 
		 * @param masterElement
		 * 
		 */
		TextContent.prototype.isOtherMasterApproved = function(masterElement) {
			// Iterate over all elements.
			for(var i=0; i < this.element.length ;i++) {
				// If element is not for master language then continue.
				if (!this.element[i].isMaster()) {
					continue;
				}
				
				// Element should not be same, which came as parameter.
				if (this.element[i].getLangCode() == masterElement.getLangCode()) {
					continue;
				}
				
				// If element is deleted then continue.
				if (this.element[i].isDeleted()) {
					continue;
				}
				
				// If element is not even added.
				if (ContentConstants.State.EMPTY == this.element[i].getCurrentState()) {
					continue;
				}
				
				// Found approved master for this text content.
				if (ContentConstants.State.APPROVED == this.element[i].getCurrentState()) {
					return true;
				}
			}
			
			// There is no approved master available.
			return false;
		};
		
		/**
		 * Return whether one of the master statements is signed off.
		 */
		TextContent.prototype.isMasterApproved = function() {
			// Iterate over all elements.
			for(var i=0; i < this.element.length ;i++) {
				// If element is not for master language then continue.
				if (!this.element[i].isMaster()) {
					continue;
				}
				
				// If element is deleted then continue.
				if (this.element[i].isDeleted()) {
					continue;
				}
				
				// If element is not even added.
				if (ContentConstants.State.EMPTY == this.element[i].getCurrentState()) {
					continue;
				}
				
				// Found approved master for this text content.
				if (ContentConstants.State.APPROVED == this.element[i].getCurrentState()) {
					return true;
				}
			}
			
			// There is no approved master available.
			return false;
		};
		
		/**
		 * Return whether one of the master statements is signed off in document without worrying about new state.
		 * 
		 * @param {Array} projMastLangs - array of project master languages.
		 */
		TextContent.prototype.isExistsApprovedMasterFor = function(elementToCheck, projMastLangs) {
			// Make sure that this element is part of this SingleTextContent.
			if (!this.contains(elementToCheck)) {
				return false;
			}
			
			// Make sure that the master of this SingleTextContent is actually approved and persists in document.
			// Extract the master elements
			var approvedMasterElements = $.grep(this.element, function(element, index) {
				// check whether element is master or not.
				if (!element.isMaster()) {
					return false;
				}
				
				// check whether element is not marked to delete.
				if (element.isDeleted()) {
					return false;
				}
				
				// Check whether language is part of project language or not.
				if (!LanguageUtils.isAProjectAttrLang(projMastLangs, element.language)) {
					return false;
				}
			
				// Include approved statements, which status is already saved in document.
				return element.getState() == ContentConstants.State.APPROVED;
			});
			
			return approvedMasterElements.length > 0;
		};
		
		/**
		 * Return whether this content is invariant.
		 */
		TextContent.prototype.isInvariant = function() {
			return this.invariant;
		};
		
		/**
		 * Return true if the language passed is the last master to be removed
		 * else false
		 */
		TextContent.prototype.isLastMasterToBeRemoved = function (language) {
			// Iterate over all elements.
			for(var i=0; i < this.element.length ;i++) {
				// If element is not for master language then continue.
				if (!this.element[i].isMaster()) {
					continue;
				}
				
				// If element is in same which is being removed then continue.
				if (this.element[i].language == language) {
					continue;
				}
				
				// If element is deleted then continue.
				if (this.element[i].isDeleted()) {
					continue;
				}
				
				// If element is not even added.
				if (ContentConstants.State.EMPTY == this.element[i].getCurrentState()) {
					continue;
				}
				
				// If execution reaches here, that means deleted element was not last master.
				return false;
			}
			
			// The element which is removed was last master.
			return true;
		};
		
		/**
		 * Return the current state of the Text Content.
		 */
		TextContent.prototype.getCurrentState = function() {
			return this.state;
		};
		
		/**
		 * This method is to set new state with calculation done by it's elements state.
		 * 
		 * Setting up current state of this object doesn't required to be saved on document.
		 * So save action will not be created for this operation.
		 */
		TextContent.prototype.updateCurrentState = function () {
			// Check whether all its elements are approved or not.
			if (this.isElementsApproved()) {
				// Keep current state of the text content as approved.
				this.state = ContentConstants.State.APPROVED;
			} else {
				// Keep current state of the text content as draft.
				this.state = ContentConstants.State.DRAFT;
			}
		};
		
		/**
		 * Return true if the current state of the Text Content document is approved.
		 */
		TextContent.prototype.isElementsApproved = function () {
			var isElementsApproved = true;	// Flag to check whether text data is approved or not.
			
			// Iterate over all the text content elements to check status.
			$.each(this.getElementsList(), _.bind(function (index, element) {
				// If element is deleted then continue for the next element.
				if (element.isDeleted()) {
					return true;
				}
				
				// If element doesn't exists then check for next element.
				if (ContentConstants.State.EMPTY == element.getCurrentState()) {
					return true;
				}
				
				// If element is not approved then whole text content is not approved.
				if (ContentConstants.State.APPROVED != element.getCurrentState()) {
					// Keep current state of the text content as draft.
					isElementsApproved = false;
					
					// Break the loop.
					return false;
				}
			}, this));
			
			return isElementsApproved;
		};
		
		/**
		 * Find save action from available save actions based on type.
		 * 
		 * @param {ContentConstants.SaveAction} type: action type which needs to be found.
		 * 
		 * @return saveAction: if found then first occurrence of action, otherwise undefined.
		 */
		TextContent.prototype._getSaveActionByType = function (type) {
			// Find save action for requested type.
			var saveAction = undefined;
			
			if (type) {
				$.each(this._saveActions, _.bind(function (index, action) {
					if (type == action.type) {
						saveAction = action;
						return false;
					}
				}, this));
			}
			return saveAction;
		};
		
		return TextContent;
	});