/**
 * TextContentElement object
 * @author PASH
 */
define(["../text-content-utils/ContentUtils",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (ContentUtils,
		ContentConstants,
		TextContentStrings) {
		"use strict";

		/**
		 * @class Represents a text content element (part of a content).
		 * @exports TextContentElement
		 * @constructor
		 */
		var TextContentElement = function (json, parentStatement) {
			if (json) {
				this.parentStatement = parentStatement;
				this.master = ContentUtils.getBooleanValue(json.master);
				this.content = json.content;
				this.state = json.state;
				this.newState = null;
				this.language = json.language;
				this.created = json.created;
				this.updated = json.updated;
				this.pastApprovals = json.approvals;
				this.comment = json.comments;
				this.effectiveElementUserPermissions = json.effectiveElementUserPermissions;
				this.deleted = false;
				this.locationList = json.locationList;
			} else {
				this.parentStatement = parentStatement;
				this.master = false;
				this.content = {};
				this.state = null;
				this.newState = null;
				this.language = null;
				this.created = null;
				this.updated = null;
				this.pastApprovals = [];
				this.comment = [];
				this.effectiveElementUserPermissions = null;
				this.deleted = false;
				this.locationList = [];
			}
			this._saveActions = [];
		};
		
		/**
		 * Clone this TextContentElement.
		 * 
		 * @returns {TextContentElement} A perfect clone of this TextContentElement.
		 */
		TextContentElement.prototype.clone = function () {
			var clone = new TextContentElement();
			
			clone.parentStatement = this.parentStatement;
			clone.master = this.master;
			clone.content = this.content;
			clone.state = this.state;
			clone.newState = this.newState;
			clone.language = this.language;
			clone.created = this.created;
			clone.updated = this.updated;
			clone.pastApprovals = this.pastApprovals;
			clone.comment = this.comment;
			clone.effectiveElementUserPermissions = this.effectiveElementUserPermissions;
			clone.deleted = this.deleted;
			clone.locationList = this.locationList;
			
			return clone;
		};
		
		/**
		 * Get a content ID for this object. (It's only unique within a content object).
		 */
		TextContentElement.prototype.getContentId = function () {
			if (this.parentStatement.getContentId().length > 0) {
				return this.parentStatement.getContentId() + "_" + this.language;
			} else {
				return this.language;
			}
		};
		
		/**
		 * Revert back to the original approval change.
		 */
		TextContentElement.prototype.cancelLastApprovalAction = function () {
			
			// We need to find the most recent approval action.
			var approvalAction = undefined;
			
			// Loop over all the actions.
			for (var i = this._saveActions.length-1; i >= 0; i--) {
				var action = this._saveActions[i];
				if (action.type == ContentConstants.SaveAction.ELEMENTAPPROVAL) {
					approvalAction = action;
					break;
				}
			}
			
			// If an approval action to revert could be found ...
			if (approvalAction) {
				// Undo it together with all actions that came after it.
				return this.undoSaveAction(approvalAction);
			} else {
				return false;
			}
		};
		
		/**
		 * Revert recent action by undoing save action.
		 */
		TextContentElement.prototype.undoSaveAction = function (saveAction) {
			
			// We need to find the action and undo it together with all the actions that came after it.
			var actionsToUndo = [];
			
			// Whether this undo will affect other elements.
			var affectsOtherElements = false;
			
			// Loop over all the actions.
			for (var i = this._saveActions.length-1; i >= 0; i--) {
				var action = this._saveActions[i];
				actionsToUndo.push(action);
				if (saveAction == action) {
					break;
				}
			}
			
			// Undo it together with all actions that came after it.
			for (var i = 0; i < actionsToUndo.length; i++) {
				var action = actionsToUndo[i];
				
				// Undo the action.
				if (action.type == ContentConstants.SaveAction.ELEMENTAPPROVAL) {
					this.newState = action.oldNewState;
				} else if (action.type == ContentConstants.SaveAction.ELEMENTTEXTDATA) {
					this.content = action.oldTextData;
				} else if (action.type == ContentConstants.SaveAction.ELEMENTSMARTNAMERESOLVE) {
					this.content = action.oldTextData;
				}
				
				// Remove it from the save actions.
				this._saveActions.splice(this._saveActions.indexOf(action), 1);
			}
			
			// If this saveAction is a critical  ...
			if (saveAction.critical) {
				// We need to undo the save actions on other elements that came after it.
				var otherActionsToUndo = [];
				// Loop over all the other elements and revert to actions done later than this one ...
				var elements = this.parentStatement.getElementsList();
				for (var i=0; i < elements.length; i++) {
					// If this is an other element ...
					if (elements[i] != this) {
						// Iterate over all save actions ...
						var saveActions = elements[i].getSaveActions();
						for (var j=0; j < saveActions.length; j++) {
							var otherSaveAction = saveActions[j];
							if (otherSaveAction.timeStamp >= saveAction.timeStamp) {
								// We now found the first save action of the element that came after the critical one on this element.
								otherActionsToUndo.push(otherSaveAction);
								break;
							}
						}
					}
				}
				// Undo those actions on their respective elements.
				for (var i=0; i < otherActionsToUndo.length; i++) {
					otherActionsToUndo[i].affectedTextContentElement.undoSaveAction(otherActionsToUndo);
					affectsOtherElements = true;
				}
			}
			
			// Update current state of the parent object.
			this.getParent().updateCurrentState();
			
			return affectsOtherElements;
		};
		
		/**
		 * 
		 */
		TextContentElement.prototype.undoAllSaveActions = function () {
			if (this._saveActions.length > 0) {
				this.undoSaveAction(this._saveActions[0]);
			}
		};
		
		
		/**
		 * Bring the text content element to a new state. It will return whether updating this state affects other elements.
		 * (If this affects other elements and this update is executed by a widget, it should notify other widgets for this 
		 * TextContent object to also update themselves).
		 * 
		 * @return boolean - Whether going to the new state affects more TextContentElements than this one. 
		 * 
		 */
		TextContentElement.prototype.setNewState = function (state) {
			
			// Only continue if it makes sense ...
			if (!state || state == this.newState) {
				return;
			}
						
			// Whether this state change will affect other elements.
			var affectsOtherElements = false;
		
			// Create a save action for this element.
			var time = (new Date()).getTime();
			var saveAction = {
				affectedTextContent: this.parentStatement,
				affectedTextContentElement: this,
				onMaster: this.isMaster(),
				type: ContentConstants.SaveAction.ELEMENTAPPROVAL,
				timeStamp : time,
				newState: state,
				oldNewState: this.newState
			};
			
			// Check for the special case when a master text is reverted to draft from signed off ...
			var revertedToDraft = (state === ContentConstants.State.DRAFT && this.state === ContentConstants.State.APPROVED);
			if (this.isMaster() && revertedToDraft && !this.parentStatement.isInvariant()) {
				
				// We need to find out whether no other master exists which is still signed off.
				var otherMasterSignedOff = this.parentStatement.isOtherMasterApproved(this);
				
				// If no other master text is signed off ...
				if (!otherMasterSignedOff) {
					
					// Make this saveAction critical 
					// (When it's undone it will revert all other saveActions on this textContent element that came after it).
					saveAction.critical = true;
					
					// Make sure that all translations are also reverted to draft ...
					// Iterate over all the elements
					for (var i=0; i < this.parentStatement.element.length; i++) {
						var element = this.parentStatement.element[i];
						if (!element.isMaster() && element !== this) {
							
							// undoAllSaveActions
							element.undoAllSaveActions();
							
							if (element.getState() != ContentConstants.State.DRAFT && element.getState() != ContentConstants.State.EMPTY && !element.isDeleted()) {
								// Update the state of the other element.
								var translationSaveAction = {
										affectedTextContent: element.parentStatement,
										affectedTextContentElement: element,
										onMaster: element.isMaster(),
										type: ContentConstants.SaveAction.ELEMENTAPPROVAL,
										timeStamp : time,
										newState: ContentConstants.State.DRAFT,
										oldNewState: element.newState,
										critical: true
								};
								// Update the new state of the translation
								element.newState = ContentConstants.State.DRAFT;
								// Add a save action for the translation
								element._saveActions.push(translationSaveAction);
							}
							
							// At this point we have affected other elements.
							affectsOtherElements = true;
						}
					}
				}
			} 
			
			// When you approve a master, this might affect other elements (translations become editable).
			if (state == ContentConstants.State.APPROVED && this.isMaster()) {
				affectsOtherElements = true;
				
				// Make this saveAction critical 
				// (When it's undone it will revert all other saveActions on this textContent element that came after it).
				saveAction.critical = true;
			}
			
			// Set the new state (while still remembering the new one).
			this.newState = state;
			
			//first reset approval array to blank. This will make sure there is only one approval entry for every element.
			this._saveActions.push(saveAction);
			
			// Update current state of the parent object.
			this.getParent().updateCurrentState();
			
			return affectsOtherElements;
		};
		
		/**
		 * Return true if current element is master.
		 */
		TextContentElement.prototype.isMaster = function () {
			return this.master;
		};
		
		/**
		 * Return true if current element is marked as deleted.
		 */
		TextContentElement.prototype.isDeleted = function () {
			return this.deleted;
		};
		
		/**
		 * Return the language code.
		 */
		TextContentElement.prototype.getLangCode = function () {
			return this.language;
		};
		
		/**
		 * Return the (GS1) element type for this element.
		 */
		TextContentElement.prototype.getElementType = function () {
			return this.parentStatement.getElementType();
		};
		
		/**
		 * Return the instance number.
		 */
		TextContentElement.prototype.getInstance = function () {
			return this.parentStatement.getInstance();
		};
		
		/**
		 * Return the text data type (rich or simple)
		 */
		TextContentElement.prototype.getTextType = function () {
			return this.parentStatement.getTextDataType();
		};
		
		/**
		 * Return the parent text content object
		 */
		TextContentElement.prototype.getParent = function () {
			return this.parentStatement;
		};
		
		/**
		 * Based on status of text content element password needs to be asked.
		 * @returns {Boolean}
		 */
		TextContentElement.prototype.isPasswordRequiredForApproval = function () {
			if(this.newState == ContentConstants.State.APPROVED || this.newState == ContentConstants.State.REJECTED)
				return true;
			else
				return false;
		};

		/**
		 * Return the current state of the element.
		 */
		TextContentElement.prototype.getCurrentState = function() {
			if(this.newState) {
				return this.newState;
			} else {
				return this.state;
			}
		};
		
		/**
		 * Return the new state of the element.
		 */
		TextContentElement.prototype.getNewState = function() {
			return this.newState;
		};
		
		
		/**
		 * Return the state of the element.
		 */
		TextContentElement.prototype.getState = function() {
			return this.state;
		};
		
		/**
		 * Revert back to the original approval change.
		 */
		TextContentElement.prototype.isChanged = function () {
			return this._saveActions.length > 0;
		};
		
		/**
		 * Indicate whether the current content has changed.
		 */
		TextContentElement.prototype.isContentChanged = function () {
			for (var i=0; i < this._saveActions.length; i++) {
				var saveAction = this._saveActions[i];
				if (saveAction.type == ContentConstants.SaveAction.ELEMENTTEXTDATA) {
					return true;
				}
			}
			return false;
		};
		
		
		/**
		 * Change the text data of this TextContentElement.
		 * 
		 * It will return whether updating the text data caused a state change.
		 * (e.g. editing a rejected document will cause it to revert back to draft.)
		 */
		TextContentElement.prototype.setTextData = function (newTextData) {
			
			// Whether this edit affected a state change (rejected document back to draft).
			var stateChange = false;
			
			// Findout whether the content was changed.
			var contentChanged = false;
			if (ContentConstants.TextType.SMARTNAME === this.parentStatement.getTextType()) {
				contentChanged = newTextData.smartName != this.content.smartName;
			} else {
				contentChanged = newTextData.content != this.content.content;
			}
			
			// Make sure that new Text Data is being set.
			if (contentChanged) {
				
				// A new string is given ...
				
				// If this was done in REJECTED state, go back to draft ...
				if (this.getCurrentState() == ContentConstants.State.REJECTED) {
					this.setNewState(ContentConstants.State.DRAFT);
					stateChange = true;
				}
				
				// Create a content edit ...
				var oldTextData = this.content;
				this.content = newTextData;
				// First check if the last action was also a text change.
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.ELEMENTTEXTDATA) {
					// Last action was also a text change, update that one.
					lastSaveAction.timeStamp = (new Date()).getTime();
					lastSaveAction.newTextData = newTextData;
				} else {
					// Last action was not a text change, create a new one.
					this._saveActions.push({
						affectedTextContent: this.parentStatement,
						affectedTextContentElement: this,
						onMaster: this.isMaster(),
						type: ContentConstants.SaveAction.ELEMENTTEXTDATA,
						timeStamp : (new Date()).getTime(),
						newTextData: newTextData,
						oldTextData: oldTextData,
						textType: this.getParent().getTextType()
					});
				}
				
				return stateChange;
			}
		};
		
		/**
		 * If the text data of this text content element contains smartnames, resolve the smartnamed value with 
		 * the given resolved value.
		 */
		TextContentElement.prototype.resolveSmartNamedTextData = function (resolvedTextData) {
			
			// Only do this when the text data has smartNamed values.
			if (this.content.smartName != undefined && this.content.smartName != null) {
				// Create a smartname resolve ...
				var oldTextData = this.content;
				this.content = resolvedTextData;
				// First check if the last action was also a smartname resolve.
				var lastSaveAction = this._saveActions[this._saveActions.length-1];
				if (lastSaveAction && lastSaveAction.type == ContentConstants.SaveAction.ELEMENTSMARTNAMERESOLVE) {
					// Last action was also a smartname resolve, update that one.
					lastSaveAction.timeStamp = (new Date()).getTime();
					lastSaveAction.newTextData = resolvedTextData;
				} else {
					// Last action was not a text change, create a new one.
					this._saveActions.push({
						affectedTextContent: this.parentStatement,
						affectedTextContentElement: this,
						onMaster: this.isMaster(),
						type: ContentConstants.SaveAction.ELEMENTSMARTNAMERESOLVE,
						timeStamp : (new Date()).getTime(),
						newTextData: this.content,
						oldTextData: oldTextData,
						textType: this.getParent().getTextType()
					});
				}
			}
		};
		
		/**
		 * Add comment to TextContentElement. If user adds and deletes the comment, then remove it from saveActions array.
		 * 
		 * @param comment
		 */
		TextContentElement.prototype.addTextContentElementComment = function(comment){
			var commentExistsAndUpdated = false;		
			for(var i=0; i<this._saveActions.length; i++){
				if(this._saveActions[i].type == ContentConstants.SaveAction.COMMENTS){
					//If comment was deleted, then remove this entry from saveActions array
					commentExistsAndUpdated = true;
					
					//If the added comment is removed before saving, them remove it from saveActions
					if(comment.length <= 0){
						var revertAction = this._saveActions[i];
						this._saveActions.splice(this._saveActions.indexOf(revertAction), 1);
					}else{
						this._saveActions[i].languageComment = comment;						
					}
					break;
				}
			}
		
			if(commentExistsAndUpdated) {
				return;
			} else {
				//Couldn't find existing comments to update, so lets add new one
				this._saveActions.push({
					affectedTextContent: this.parentStatement,
					affectedTextContentElement: this,
					type: ContentConstants.SaveAction.COMMENTS,
					timeStamp : (new Date()).getTime(),
					languageComment: comment
				});
			}
		};
		
		/**
		 * This method will return whether we can effectively edit the element at this point in time.
		 * (It will depend on "hasEdit" but it will also take the current status into account) and 
		 * Whether or not a master is approved.
		 * 
		 * @param {Array} array of project master languages.
		 * @returns {Object} {"allowed" : false, "message" : "Reason for not allowing edit permission"}
		 */
		TextContentElement.prototype.canEdit = function (projMastLangs) {
			var result = {};
			
			// Make sure that we have the ability to edit (e.g. permissions, latest document, proper WCR rights, ...)
			result = this.hasEdit();
			if (!result.allowed) {
				return result;
			}
			
			// We need to check that whether at least one master text is approved before we can edit translations.
			if (!this.isMaster() && !this.parentStatement.isMasterApprovedFor(this)) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_MASTER_NOT_SIGNEDOFF");
				return result;
			}
			
			// You can not edit an element when it's pending for approval.
			if (this.getCurrentState() == ContentConstants.State.READY_FOR_APPROVAL) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_EDIT_READY_FOR_APPROVAL");
				return result;
			}

			// You can not edit an element when it's approved.
			if (this.getCurrentState() == ContentConstants.State.APPROVED) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_EDIT_SIGNEDOFF");
				return result;
			}
			
			// You can not edit an element when it's rejected.
			if (this.getCurrentState() == ContentConstants.State.REJECTED) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_EDIT_SIGNOFF_REJECTED");
				return result;
			}
			
			// Determine whether in this state we can edit.
			if (this.getCurrentState() != ContentConstants.State.DRAFT) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_EDIT_STATE");
				return result;
			}
		
			// To edit translation when approval type is not default, at-least 1 approved masters should be available in document already.
			if (!this.isMaster() && ContentConstants.ApprovalSetup.DEFAULT != this.parentStatement.getApprovalType() && !this.parentStatement.isExistsApprovedMasterFor(this, projMastLangs)) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_REVIEW_NEEDED_OTHER");
				return result;
			}
			
			// User is allowed to edit.
			result.allowed = true;
			result.message = undefined;
			return result;
		};
		
		/**
		 * This method will return whether we have the ability to edit this element, it will take into account:
		 * - Whether it's the latest document version.
		 * - Whether we have calculated user permissions from the back end.
		 * - Whether the proper permissions are set in WebCenter.
		 * - Whether or not this user is assigned edit permissions.
		 * 
		 *  It will take NOT into account:
		 *  - The current status of the element.
		 *  
		 *  @returns {Object} {"allowed" : false, "message" : "Reason for not allowing edit permission"}
		 */
		TextContentElement.prototype.hasEdit = function () {
			
			var result = {};
			
			// Make sure that permissions are calculated
			if (!this.effectiveElementUserPermissions) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NOT_CALCULATED");
				return result;
			}
			
			// User is not allowed to edit because permission is not assigned.
			if (!this.effectiveElementUserPermissions.hasEditPermission) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NO_EDITOR_ROLE");
				return result;
			}

			// Make sure that the correct WebCenter permissions are set.
			result = this.parentStatement.hasProperWcrPermissions();
			if (!result.allowed) {
				return result;
			}
			
			// Make sure that this is the latest version of the document.
			if (!this.parentStatement.isLatestDocumentVersion()) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_DOC_NOT_LATEST");
				return result;
			}		

			// We can edit when we have the right permissions.
			result.allowed = true;
			result.message = undefined;
			return result;
		};		

		/**
		 * This method will return whether we can effectively review the element at this point in time.
		 * (It will depend on "hasReview" but it will also take the current status into account).
		 * - Whether or not a master is approved.
		 * - Whether or not it still makes sense for this user to review (did somebody else review in his place?)
		 * 
		 * @returns {Object} {"allowed" : false, "message" : "Reason for not allowing review permission"}
		 */
		TextContentElement.prototype.canReview = function (projMastLangs) {
			var result = {};
			
			// Make sure that we have the ability to review (e.g. permissions, latest document, proper WCR rights, ...)
			result = this.hasReview();
			if (!result.allowed) {
				return result;
			}

			// We need to check that whether at least one master text is approved before we can review translations.
			if (!this.isMaster() && !this.parentStatement.isMasterApprovedFor(this)) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_MASTER_NOT_SIGNEDOFF");
				return result;
			}
			
			// You can not review a statement that is already approved.
			if (this.getCurrentState() == ContentConstants.State.APPROVED) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_REVIEW_SIGNEDOFF");
				return result;
			}

			// You can not review a statement that is already rejected.
			if (this.getCurrentState() == ContentConstants.State.REJECTED) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_REVIEW_REJECTED");
				return result;
			}
			
			// Determine whether in this state we can review.
			if (this.getCurrentState() != ContentConstants.State.READY_FOR_APPROVAL
					&& this.getCurrentState() != ContentConstants.State.DRAFT) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_REVIEW_STATE");
				return result;
			}
			
			// To review translation when approval type is not default, at-least 1 approved masters should be available in document already.
			if (!this.isMaster() && this.parentStatement.getApprovalType() != ContentConstants.ApprovalSetup.DEFAULT && 
					!this.parentStatement.isExistsApprovedMasterFor(this, projMastLangs)) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_REVIEW_NEEDED_OTHER");
				return result;
			}

			// Check if user still needs to approve or reject.
			if (this.effectiveElementUserPermissions.isReviewalStillNeeded) {
				result.allowed = true;
				result.message = undefined;
				return result;
			} else {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_ALREADY_REVIEWED_ROLE");
				return result;
			}
			
		};
		
		/**
		 * This method will return whether we have the ability to review this element, it will take into account:
		 * - Whether it's the latest document version.
		 * - Whether we have calculated user permissions from the backend.
		 * - Whether the proper permissions are set in WebCenter.
		 * - Whether or not this user is assigned review permissions.
		 * 
		 *  It will take NOT into account:
		 *  - The current status of the element.
		 *  
		 *  @returns {Object} {"allowed" : false, "message" : "Reason for not allowing review permission"}
		 */
		TextContentElement.prototype.hasReview = function () {
			
			var result = {};

			// Make sure that permissions are calculated
			if (!this.effectiveElementUserPermissions) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NOT_CALCULATED");
				return result;
			}
			
			// Check whether is having permission to approve.
			if (!this.effectiveElementUserPermissions.hasReviewPermission) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_NO_APPROVAL_ROLE");
				return result;
			}

			// Make sure that the correct WebCenter permissions are set.
			result = this.parentStatement.hasProperWcrPermissions();
			if (!result.allowed) {
				return result;
			}

			// Make sure that this is the latest version of the document.
			if (!this.parentStatement.isLatestDocumentVersion()) {
				result.allowed = false;
				result.message = TextContentStrings.t("CONTENT_PERM_DOC_NOT_LATEST");
				return result;
			}
			
			result.allowed = true;
			result.message = undefined;
			return result;
		};
		
		/**
		 * Get the save actions for this object.
		 */
		TextContentElement.prototype.getSaveActions = function (password) {
			// Make sure to add password to saveActions if we have done any approval
			for(var i=0; i<this._saveActions.length; i++){
				if(this._saveActions[i].type == ContentConstants.SaveAction.ELEMENTAPPROVAL &&
						this.parentStatement.isApprovalPasswordRequired &&
						(this._saveActions[i].newState == ContentConstants.State.APPROVED || 
								this._saveActions[i].newState == ContentConstants.State.REJECTED)){
					this._saveActions[i].encodedPassword = password;
					break;
				}
			}
			return this._saveActions;
		};

		/**
		 * Get the text data, e.g. the real content. 
		 */
		TextContentElement.prototype.getTextData = function () {
			return this.content;
		};
		
		/**
		 * Get the parent of this element. 
		 */
		TextContentElement.prototype.parent = function () {
			return this.parentStatement;
		};
		
		/**
		 * Get user's past approval status for the current element.
		 * 
		 * @param projMastLangs
		 * @returns
		 */
		TextContentElement.prototype.getMyApprovalStatus = function (projMastLangs) {
			var myApprovalStatus = undefined;
			
			var user = ContentUtils.getLoggedInUserName();
			
			// Iterate over approvals and calculate my approval status.
			$.each(this.pastApprovals, _.bind(function (index, pastApproval) {
				if (pastApproval.user == user && pastApproval.state == ContentConstants.State.APPROVED) {
					myApprovalStatus = ContentConstants.State.APPROVED;
				} else if (pastApproval.user == user && pastApproval.state == ContentConstants.State.REJECTED) {
					myApprovalStatus = ContentConstants.State.REJECTED;
				} else if (this.canReview(projMastLangs).allowed) {
					// In case, past approval appears as approve or reject before ready for approval.
					// Approve or reject are considered higher state then ready for approval.
					if (!myApprovalStatus) {
						myApprovalStatus = ContentConstants.State.READY_FOR_APPROVAL;
					}
				}
			}, this));
			
			return myApprovalStatus;
		};
		
		return TextContentElement;
	});