/**
 * @author KADA
 * 
 * 
 */

define(["jquery",
        "underscore",
        "./LanguageIndicatorView",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/PCMEventBus",
        "../text-content-services/languageService",
        "../text-content-utils/LanguageUtils"],
		function($,
				_,
				LanguageIndicatorView,
				ContentConstants,
				pcmEventBus,
				languageService,
				LanguageUtils){
	
	"use strict";
	
	/**
	 * Create a language indicator
	 */
	var LanguageIndicator = function(contentStatement, isMasterLanguageList, options) {
		
		// Make a reference to the ContentStatement
		this._contentStatement = contentStatement;
		this._isMasterLanguageList = isMasterLanguageList;
		this.textInfo = null;
		

		// Quickly make some default options for the view, in future use elements of "options" for this.
		var defOptions = {
				showMaster: true,
				showTranslation: true,
				showCompact: false, 
				showLabels: true,
				width: '250px'
		};
		
		if (options !== undefined) {
			// Merge the default options with the passed options. 
			$.extend(true, defOptions, options);
		}
		this._options = options;
				
		// Create a view for this component.
		this._view = new LanguageIndicatorView(defOptions, this._isMasterLanguageList);
		
		// What needs to happen when the language add functionality of this indicator is enabled.
		this._view.onLanguageAddAction(_.bind(function (language, isMaster){
			// Add the language to the content statement.
			this._contentStatement.addLanguage(language, isMaster, _.bind(function (element) {
				if (element) {
					pcmEventBus.emit("PCM.TextContent.langAdded", this._contentStatement, element);
				}
			}, this));
		}, this));
		
		// What needs to happen when the language add functionality of this indicator is enabled.
		this._view.onLanguageRemoveAction(_.bind(function (language, isMaster){
			if (isMaster) {
				// Check that we don't remove the last master language ...
				var masterLangCodes = this._contentStatement.getLangCodesOfElements(true);
				if (masterLangCodes.length == 1 && masterLangCodes[0] == language.langCode) {
					return;
				}
			}
			var csElem = this._contentStatement.removeLanguage(language.langCode, isMaster);
			if (csElem) {
				pcmEventBus.emit("PCM.TextContent.langRemoved", this._contentStatement, csElem);
			}
		}, this));
		
		// Listen to newly added languages and update view.
		pcmEventBus.on("PCM.TextContent.langAdded", function (textContent, textElement) {
			if (textContent ==  this._contentStatement) {
				this.update();
			}
		}.bind(this));
		
		// Listen to removed languages and update view
		pcmEventBus.on("PCM.TextContent.langRemoved", function (textContent, textElement) {
			if (textContent ==  this._contentStatement) {
				this.update();
			}
		}.bind(this));
		
		// Show which languages are available and added.
		this.update();
		
	};
	
	/**
	 * Set the ContentStatement that corresponds with this language indicator.
	 * 
	 * @param contentStatement
	 */
	LanguageIndicator.prototype.setContentStatement = function (contentStatement) {
		
		this._contentStatement = contentStatement;
	};
	
	/**
	 * Update this language indicator.
	 */
	LanguageIndicator.prototype.update = function () {
		
		if (this._contentStatement) {
			
			// Find out the project for which we have to show the languages ...
			var projectId = this._contentStatement.projectId;
			// If the project in which the contentStatement will be created does not yet exist ... fall back on the projectId in the options.
			if (!projectId) {
				projectId = this._options.projectId;
			}
			
			// Check if there are languages specified in the options.
			if (this._options.getLanguages) {
				// Update the widget based on the specified languages.
				this._updateLanguages(this._options.getLanguages());
			} else {
				// Fall back on the project languages to update the widget.
				languageService.getProjectLanguages(projectId, this._isMasterLanguageList, function (projectLangs) {
					this._updateLanguages(projectLangs);
				}.bind(this));
			}
		}
	};
	
	/**
	 * Update the languages of the indicator.
	 */
	LanguageIndicator.prototype._updateLanguages = function (languages) {
		
		// Find out which languages are added to the Text Content and which are not (based on the passed languages).
		var availableLangCodes = this._contentStatement.getLangCodesOfElements(this._isMasterLanguageList);
		var availableLangs = LanguageUtils.getLangsForCodes(languages, availableLangCodes);
		var unAvailableLangs = this._getUnavailableLangs(availableLangs, languages);
		
		// Which are the disabled languages? (The ones who can not be removed)
		var disabledLang = undefined;
		if(this._isMasterLanguageList && availableLangs.length == 1) {
			disabledLang = availableLangs[0];
		} else if(this._isMasterLanguageList) {
			var signedOffMasterLangs = this._contentStatement.getSignedOffMasterLangs(availableLangs);
			if(signedOffMasterLangs.length === 1) {
				disabledLang = signedOffMasterLangs[0];
			}
		}
		this._view.setLanguages(availableLangs, unAvailableLangs, disabledLang);
	};
	
	/**
	 * 
	 * @param availableLangs
	 * @param allLangsMap
	 */
	LanguageIndicator.prototype._getUnavailableLangs = function (availableLangs, allLangs) {
		
		var unAvailableLangs = [];
		for (var i=0; i < allLangs.length; i++) {
			
			var lang = allLangs[i];
			var isLangAvailable = false;
			
			for (var j=0; j < availableLangs.length; j++) {
				var availableLang = availableLangs[j];
				if (lang.langCode == availableLang.langCode) {
					isLangAvailable = true;
				}
			}
			
			if (!isLangAvailable) {
				unAvailableLangs.push(lang);
			}
		}
		return unAvailableLangs;
	};
	
	LanguageIndicator.prototype.getJQElement = function () {
		return this._view.getJQElement();
	};
	
	
	return LanguageIndicator;
});