/**
 * @author KADA
 * 
 * 
 */

define(["jquery",
        "../text-content-services/languageService",
        "../text-content-utils/LanguageUtils"],
		function($,
				languageService,
				LanguageUtils){
	
	"use strict";
	
	/**
	 * Create a language selector.
	 * 
	 * @param showMaster {boolean} - Whether master languages should be displayed.
	 * @param sowTrans {boolean} - Whether translation languages should be displayed.
	 * @param projectId {string} - The ID of the project for which to display the languages.
	 * @param showOnlyProjLang {boolean} - Whether only project languages should be displayed.
	 * @param selectedLangCode {string} - The language code of the language that should be selected.
	 * @param select {jQuery} - A jQuery wrapped select input to transform into a language selector (optional).
	 */
	var LanguageSelector = function(showMaster, showTrans, projectId, onlyProjLang, selectedLangCode, select) {
		
		this._showMaster = showMaster;
		this._showTrans = showTrans;
		this._projectId = projectId;
		this._onlyProjLang = onlyProjLang;
		this._selectedLangCode = selectedLangCode;
		
		// The callbacks that should be triggered on change.
		this._changeCallbacks = [];
		
		// Set the selector to use.
		this._selector = select;
		// Check if such a selector was passed.
		if (!this._selector) {
			// Create the select.
			this._selector = $("<select>");
		}
		
		// What needs to happen on change of the select.
		this._selector.on("change", function () {
			// Update the ccurrently selected language code.
			this._selectedLangCode = this._selector.val();
			// Execute all the callbacks.
			for (var i=0; i < this._changeCallbacks.length; i++) {
				this._changeCallbacks[i](this._selectedLangCode);
			};
		}.bind(this));
		
		// Check if only project languages need to be fetched.
		if (this._onlyProjLang) {
			// Get project languages.
			languageService.getLanguages(this._projectId, function (projMasterLang, projTransLang) {
				if (this._showMaster && this._showTrans) {
					this._displayLangMap(projMasterLang, projTransLang);
				} else if (this._showMaster) {
					this._displayLangMap(projMasterLang, undefined);
				} else if (this._showTrans) {
					this._displayLangMap(undefined, projTransLang);
				}
			}.bind(this));
		} else {
			// All languages need to be displayed.
			languageService.getBothAllLanguages(function (masterLang, transLang) {
				if (this._showMaster && this._showTrans) {
					this._displayLangMap(masterLang, transLang);
				} else if (this._showMaster) {
					this._displayLangMap(masterLang, undefined);
				} else if (this._showTrans) {
					this._displayLangMap(undefined, transLang);
				}
			}.bind(this));
		};
	};
	
	/**
	 * Display the passed languages in the selector.
	 */
	LanguageSelector.prototype._displayLangMap = function(masterLangs, transLangs) {
		
		// Empty the current options of the selector.
		this._selector.empty();
		
		// Whether we need to show labels of what a master and what a translation language is.
		var showLabels = false;
		if (masterLangs && transLangs && (masterLangs.length > 0 && transLangs.length > 0)) {
			showLabels = true;
		}
		
		// Add an "Select a language ..." suggestion
		var option1 = $("<option>", {"disabled": "disabled",}).text("Select a language ...");
		this._selector.append(option1);
		
		// Keep track whether an option is selected or not ...
		var selected = false;

		// Find out whether an option group needs to be created for the master languages.
		var optionGroup = this._selector;
		if (showLabels) {
			optionGroup = $("<optgroup>", {"label" : "Master Languages",});
			this._selector.append(optionGroup);
		}
		
		// Add all the master languages
		$.each(masterLangs, function (index, lang) {
			var option = $("<option>", {"value": lang.langCode,}).text(lang.langDesc);
			// Check whether it should be selected ...
			if (this._selectedLangCode && this._selectedLangCode == lang.langCode) {
				option.attr("selected", "selected");
				selected = true;
			}
			optionGroup.append(option);
		}.bind(this));
		
		
		
		// Find out whether an option group needs to be created for the translation languages.
		if (showLabels) {
			optionGroup = $("<optgroup>", {"label" : "Translation Languages",});
			this._selector.append(optionGroup);
		}
		
		// Add all the translation languages
		$.each(transLangs, function (index, lang) {
			var option = $("<option>", {"value": lang.langCode,}).text(lang.langDesc);
			// Check whether it should be selected ...
			if (this._selectedLangCode && this._selectedLangCode == lang.langCode) {
				option.attr("selected", "selected");
				selected = true;
			}
			optionGroup.append(option);
		}.bind(this));
		
		// If no option was selected, select the first placeholder option ...
		if (!selected) {
			option1.attr("selected","selected");
		}
	};
	
	/**
	 * Register a callback that should be triggered on change of the selector ...
	 * 
	 * (The newly selected language code will be passed as the first argument)
	 */
	LanguageSelector.prototype.onChange = function (callback) {
		this._changeCallbacks.push(callback);
	};
	
	/**
	 * Get the currently selected language code.
	 */
	LanguageSelector.prototype.getSelectedLangCode = function() {
		return this._selectedLangCode;
	};
	
	/**
	 * Get the wrapping jQuery element (select element).
	 */
	LanguageSelector.prototype.getJQElment = function() {
		return this._select;
	};
	
	/**
	 * Already initiate the language selector based on an existing element.
	 */
	LanguageSelector.wcrinit = function(select) {
		var $select = $(select);
		var showMaster = $select.attr("data-showmaster");
		var showTrans = $select.attr("data-showtrans");
		var projectId = $select.attr("data-projectid");
		var onlyProjLang = $select.attr("data-onlyprojlang");
		var selectedLangCode = $select.attr("data-selected");
		return new LanguageSelector(showMaster, showTrans, projectId, onlyProjLang, selectedLangCode, $select);
	};
	
	return LanguageSelector;
});