/**
 * @author PASH
 * 
 * 
 */

define(["jquery",
        "underscore",
        "./LanguageIndicatorView",
        "../text-content-utils/LanguageUtils"],
		function($,
				_,
				LanguageIndicatorView,
				LanguageUtils) {
	
	"use strict";
	
	/**
	 * Create a language indicator
	 */
	var TableLanguageIndicator = function(content, isMasterLanguageList, options) {
		this._content = content;
		this._languageList = undefined;
		this._isMasterLanguageList = isMasterLanguageList;
		
		this._onLanguageAddCallbacks = [];
		this._onLanguageRemoveCallbacks = [];

		// Quickly make some default options for the view, in future use elements of "options" for this.
		var defOptions = {
				showMaster: true,
				showTranslation: true,
				showCompact: false,
				showLabels: true,
				width: '250px'
		};
		
		if (options !== undefined) {
			// Merge the default options with the passed options. 
			$.extend(true, defOptions, options);
		}
		this._options = options;
				
		// Create a view for this component.
		this._view = new LanguageIndicatorView(defOptions, this._isMasterLanguageList);
		
		// What needs to happen when the language add functionality of this indicator is enabled.
		this._view.onLanguageAddAction(_.bind(function (language, isMaster){
			this._addLanguage(language);
			
			$.each(this._onLanguageAddCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));
		
		// What needs to happen when the language add functionality of this indicator is enabled.
		this._view.onLanguageRemoveAction(_.bind(function (language, isMaster){
			this._removeLanguage(language);
			
			$.each(this._onLanguageRemoveCallbacks, _.bind(function (index, callback) {
				callback(language, isMaster);
			}, this));
		}, this));
	};
	
	TableLanguageIndicator.prototype.onLanguageAdd = function (callback) {
		this._onLanguageAddCallbacks.push(callback);
	};
	
	TableLanguageIndicator.prototype.onLanguageRemove = function (callback) {
		this._onLanguageRemoveCallbacks.push(callback);
	};
	
	/**
	 * Set languages to the language list and update the view.
	 * 
	 * @param languageList
	 */
	TableLanguageIndicator.prototype.setLanguageList = function (languageList) {
		this._languageList = languageList;
		
		this._availableLangCodes = this._content.getLangCodesOfElements(this._isMasterLanguageList);
		this._availableLangs = LanguageUtils.getLangsForCodes(this._languageList, this._availableLangCodes);
		this._updateUnavailableLanguages();
		
		this.update();
	};
	
	/**
	 * Manage available and unavailable languages and update the view after adding language.
	 * 
	 * @param language
	 */
	TableLanguageIndicator.prototype._addLanguage = function (language) {
		if (!_.contains(this._availableLangs,language)) {
			this._availableLangs.push(language);
			this._updateUnavailableLanguages();
			this.update();
		}
	};
	
	/**
	 * Manage available and unavailable languages and update the view after removings language.
	 * 
	 * @param language
	 */
	TableLanguageIndicator.prototype._removeLanguage = function (language) {
		$.each(this._availableLangs, _.bind(function (index, availableLanguage) {
			if (availableLanguage == language) {
				this._availableLangs.splice(index, 1);
				this._updateUnavailableLanguages();
				this.update();
			}
		}, this));
	};
	
	/**
	 * Update this language indicators.
	 */
	TableLanguageIndicator.prototype.update = function () {
		// Update view with available and unavailable languages. 
		// Last parameter for "disabledLang" is passed as undefined to allow deleting last available language also. 
		this._view.setLanguages(this._availableLangs, this._unAvailableLangs, undefined);
	};
	
	/**
	 * Filter out languages which are not added yet.
	 * 
	 * @param availableLangs
	 * @param allLangsMap
	 */
	TableLanguageIndicator.prototype._updateUnavailableLanguages = function () {
		this._unAvailableLangs = [];
		
		$.each(this._languageList, _.bind(function (index, language) {
			if (!_.contains(this._availableLangs,language)) {
				this._unAvailableLangs.push(language);
			}
			
		}, this));
	};
	
	/**
	 * Return the number of visible languages.
	 */
	TableLanguageIndicator.prototype.getNumOfVisibleLanguages = function () {
		return this._view.getNumOfVisibleLanguages();
	};
	
	TableLanguageIndicator.prototype.getJQElement = function () {
		return this._view.getJQElement();
	};
	
	return TableLanguageIndicator;
});