define(["underscore",
        "jquery",
        "packaging-content-management/ContentConstants",
        "utils/webi18n!TextContentStrings",
        "./TableContentLanguageWidget",
        "./TableContentPreviewSelector",
        "../view/TableContentPageView",
        "../view/TableContentPreviewWidget",
        "../../content-page/controller/ContentSettings",
        "../../text-content-services/ContentService",
        "services/smartNameResolverService",
        "packaging-content-management/text-content-services/languageService",
        "packaging-content-management/text-content-services/tableElementTypeService",
        "packaging-content-management/PCMEventBus",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "../../text-content-utils/ContentUtils"],
	function (_,
			$,
			ContentConstants,
			TextContentStrings,
			TableContentLanguageWidget,
			TableContentPreviewSelector,
			TableContentPageView,
			TableContentPreviewWidget,
			ContentSettings,
			ContentService,
			smartNameResolverService,
			languageService,
			tableElementTypeService,
			pcmEventBus,
			LanguageUtils,
			ContentUtils) {
		
		
	var TableContentPage = function (contentBasic, tableContent, parentPage) {
		var projectId = contentBasic._projectId;
		
		this._tableContent = tableContent;
		
		this._onChangeCallBacks = [];
		
		// Create the preview selector and widgets.
		this._previewSelector = new TableContentPreviewSelector(tableContent, projectId);
		this._previewWidget = new TableContentPreviewWidget(tableContent);
		
		this._previewSelector.onPreviewSelected(function (previewHTML, previewSettings) {
			this._previewWidget.setPreview(previewHTML, previewSettings, this.getPreviewName());
			this._previewWidget.fillForTableContent(this._tableContent);
			// Execute all the call backs
			$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
				callBack();
			}, this));
		}.bind(this));
		
		// Default locale selection in the preview widget.
		this._previewWidget.onNewDefaultLocaleMapping(function(localeMapping) {
			this._previewSelector.setDefaultLocaleMapping(localeMapping);
		}.bind(this));
		
		
		//Create table content setting widget
		this._contentSettings = new ContentSettings(tableContent, projectId);
		
		this._contentSettings.onChange(_.bind(function (changeType, changedValue) {
			this._contentSettingsChangeHandler(changeType, changedValue);
			
			// Execute all the call backs
			$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
				callBack();
			}, this));
		}, this));

		// What should happen when resolve button is clicked to resolve smart named strings.
		this._contentSettings.onSmartNameResolveClick(_.bind(function (deferred) {
			// Collect all the smart named string, which needs to be resolved.
			var toBeResolved = {};
			$.each(this._tableContent.getTextContentElements(), function (i, element) {
				// Keep content id as key, so after resolved it will be easy to identify.
				toBeResolved[element.getContentId()] = element.getTextData().smartName;
			});
			
			// Resolve all the strings with the help of smart name resolver service.
			smartNameResolverService.resolve(projectId, this._tableContent.docVersionId, undefined, toBeResolved, false, function (resolved) {
				
				// Iterate over all the language widgets to update each element available in it.
				$.each(this._tableContentLangWidgets, _.bind(function (index, langWidget) {
					var elemWidgets = langWidget.getElementWidgets();
					// Iterate over all available element widgets and resolve them.
					$.each(elemWidgets, _.bind(function (index, elemWidget) {
						var element = elemWidget.getTextContentElement();
						elemWidget.resolve(resolved[element.getContentId()]);
					}, this));
					
				}, this));
				
				// Execute all the call backs
				$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
					callBack();
				}, this));
				
				// Notify content setting that, smart name is resolved for this request.
				deferred.resolve();
			}.bind(this), function (msg) {
				// Display error message.
				parentPage.displayErrorMessage(msg);
				// Notify content setting that, smart name is resolved for this request.
				deferred.resolve();
			}.bind(this));
			
		}, this));
		
		// Register event for adding language from language indicators.
		this._contentSettings.onLanguageAdd(_.bind(function (language, isMaster) {
			var availableLangWidget = $.grep(this._tableContentLangWidgets, _.bind(function (langWidget, index) {
				if (language.langCode == langWidget.getLanguageCode()) {
					return true;
				}
			}, this));
			
			// Check and create language widget, if widget is not created.
			if (availableLangWidget.length == 0) {
				// Create the language widget.
				var langWidgetParam = {"language":language,
						"isMaster":isMaster,
						"isProjectLang": true,
						 "projMastLangs":this._projMastLangs,
						 "projTransLangs":this._projTransLangs,
						"fullTableElementTypesList":this._tableElementTypesFullList
						};
				
				var langWidget = this._createLanguageWidget([], language.langCode, langWidgetParam);
				
				// Add this newly created widget to the view.
				this._view.addLanguageWidget(langWidget);
			} else {
				// Length of the available language widget suppose to be 1 always.
				$.each(availableLangWidget, _.bind(function (index, langWidget) {
					var elementWidgets = langWidget.getElementWidgets();
					
					$.each(elementWidgets, _.bind(function (index, widget) {
						var element = widget.getTextContentElement();
						element.parentStatement.addLanguage(element.language, element.isMaster(), _.bind(function (element) {
							element.newState = null;
							widget.update(element);
						}, this));
					}, this));
					
					// Set the state of language widget as deleted.
					langWidget.calculateCurrentAndNextStatesForLanguageWidget();
				
					// Execute all the call backs
					$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
						callBack();
					}, this));
				}, this));
			}
		}, this));
		
		// Register event for removing language from language indicators.
		this._contentSettings.onLanguageRemove(_.bind(function (language, isMaster) {
			// Get the widget for the removed language.
			var availableLangWidget = $.grep(this._tableContentLangWidgets, _.bind(function (langWidget, index) {
				if (language.langCode == langWidget.getLanguageCode()) {
					return true;
				}
			}, this));
			
			// Length of the available language widget suppose to be 1 always.
			$.each(availableLangWidget, _.bind(function (index, langWidget) {
				var elementWidgets = langWidget.getElementWidgets();
				
				$.each(elementWidgets, _.bind(function (index, widget) {
					var element = widget.getTextContentElement();
					
					// Whole language is going to be removed after this.
					this.getTableContent().removeTableTextContent(element, this._projMastLangs);
					
					// Update element widget for deleted elements.
					widget.update(element);
				}, this));
				
				// Set the state of language widget as deleted.
				langWidget.setCurrentState(ContentConstants.State.DELETED);
			
				// Execute all the call backs
				$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
					callBack();
				}, this));
			}, this));
			
			// If master language is removed.
			if (isMaster) {
				// This could effect status of the translations.
				$.each(this._tableContentLangWidgets, _.bind(function (index, langWidget) {
					// Calculate status only for translations.
					if (langWidget.isMaster()) {
						return true;
					}
					
					var elementWidgets = langWidget.getElementWidgets();
					
					// Iterate over all the elements.
					$.each(elementWidgets, _.bind(function (index, widget) {
						var element = widget.getTextContentElement();
						
						// Update element widget for deleted elements.
						widget.update(element);
					}, this));
					
					// Calculate and update current status with next available states.
					langWidget.calculateCurrentAndNextStatesForLanguageWidget();
				}, this));
			}
		}, this));
		
		// Create all the language widgets.
		var langCodes = tableContent.getLangCodes();
		this._tableContentLangWidgets = [];
		
		var deferred = $.Deferred();
		
		tableElementTypeService.getTableElementTypes(_.bind(function(tableElementTypesFullList){
			deferred.resolve(tableElementTypesFullList);
		}, this));
		
		
		$.when(deferred.promise()).done(_.bind(function(tableElementTypesFullList) {
			this._tableElementTypesFullList = tableElementTypesFullList;
			// Using language find the language description, that will be displayed on the header
			languageService.getLanguages(projectId, function(projMastLangs, projTransLangs, allMasterLangs, allTransLangs){
				this._projMastLangs = projMastLangs;
				this._projTransLangs = projTransLangs;
				$.each(langCodes, function (index, langCode) {
					// Get the elements for this language.
					var elements = [];
					$.each(tableContent.getTextContents(), function (index, textContent) {
						$.each(textContent.getElementsList(), function (index, element) {
							if (element.language == langCode && element.state != ContentConstants.State.EMPTY) {
								elements.push(element);
							}
						});
					});
					
					var isProjectLang = false;
					
					// Search for the language in master language list
					var language = LanguageUtils.getLangForCode(projMastLangs, langCode);	
					var isMaster = true;
					// If language was not found in master language list, search in translation language list
					if(!language){
						language = LanguageUtils.getLangForCode(projTransLangs, langCode);
						isMaster = false;
					}
					 var allLanguages = allMasterLangs.concat(allTransLangs);
					// If language was not found in project master and translation list, we will fall back on all allLanguages list to find the language
					if(language){
						isProjectLang = true;
					} else {
						language = LanguageUtils.getLangForCode(allLanguages, langCode);
						isMaster = false; // set master flag to false
					}
					
					//If not a project language fetch 
					if(!isProjectLang){
						isMaster = elements[0].master;
					}
					
					var langWidgetParam = {"language":language,
							"isMaster":isMaster,
							"isProjectLang": isProjectLang,
							 "projMastLangs":projMastLangs,
							 "projTransLangs":projTransLangs,
							"fullTableElementTypesList":tableElementTypesFullList
							};
					
					this._createLanguageWidget(elements, langCode, langWidgetParam);
				}.bind(this));
				
				// Add the widgets to the page view
				this._view.addAllLanguageWidgets(this._tableContentLangWidgets);
				
				// When a placeholder is clicked.
				this._previewWidget.onPlaceholderClicked(function (textContentElementId) {
					$.each(this._tableContentLangWidgets, function (index, langWidget) {
						langWidget.focus(textContentElementId);
					}.bind(this));
				}.bind(this));
				
			}.bind(this));
		}, this));			
		
		// Create a view.
		this._view = new TableContentPageView(this._contentSettings, this._previewSelector, this._previewWidget, parentPage.getParentJQElement());
	};
	
	/**
	 * Keep all call backs in array to execute then on change.
	 * 
	 * @param callBack
	 */
	TableContentPage.prototype.onChange = function (callBack) {
		this._onChangeCallBacks.push(callBack);
	};
	
	/**
	 * Create language widget for specific language code.
	 * 
	 * @param elementsArray
	 * @param languageCode
	 * @param langWidgetParam
	 * @returns {TableContentLanguageWidget}
	 */
	TableContentPage.prototype._createLanguageWidget = function (elementsArray, languageCode, langWidgetParam) {
		var canManage = this._tableContent.canManage().allowed;
		
		// Create the language widget.
		var langWidget = new TableContentLanguageWidget(canManage, elementsArray, languageCode, langWidgetParam, this);
		
		// When the content of one of the elements in the language widget changes.
		langWidget.onChange(function (textContentElement, changeActionType) {
			
			// Update the text in the preview.
			this._previewWidget.setText(textContentElement);
			
			// Check whether any element was added or removed from the language widget
			if (changeActionType == ContentConstants.OnChangeCallbackType.ADDED ||
					changeActionType == ContentConstants.OnChangeCallbackType.DELETED) {
					// The visibility of elements in the preview needs to be updated.
					this._previewWidget.updateVisibility();
			}
			
			// Update overall state of the document.
			$.each(this._tableContentLangWidgets, _.bind(function (index, langWidget) {				
				// Manage language widget when change action type if delete
				// TODO: Move this to a lower level, should sit on language widget level not table content page.
				if (ContentConstants.OnChangeCallbackType.DELETED == changeActionType) {
					var elementWidgets = langWidget.getElementWidgets();
					$.each(elementWidgets, _.bind(function (index, widget) {
						if (widget.getTextContentElement() == textContentElement) {
							langWidget.manageDeleteAction(textContentElement, widget);
						}
					}, this));
				}
			}, this));
			
			// Execute all the call backs
			$.each(this._onChangeCallBacks, _.bind(function (index, callBack) {
				callBack();
			}, this));
			
		}.bind(this));
		
		// When the user hovers over one of the elements in the language widget.
		langWidget.onHover(function (textContentElement) {
			this._previewWidget.highlightPlaceholder(textContentElement);
		}.bind(this), function (textContentElement) {
			this._previewWidget.unHighlightPlaceholder(textContentElement);
		}.bind(this));
		
		// When the user focuses on one of the elements in the language widget.
		langWidget.onFocus(function (textContentElement) {
			this._previewWidget.focusPlaceholder(textContentElement);
		}.bind(this));
		
		// When the user no longer focuses on one of the elements in the language widget.
		langWidget.onBlur(function (textContentElement) {
			this._previewWidget.blurPlaceholder(textContentElement);
		}.bind(this));
		
		// Create the table content widget.
		this._tableContentLangWidgets.push(langWidget);
		
		return langWidget;
	};
	
	/**
	 * Executed for special handling when content settings are changed.
	 */
	TableContentPage.prototype._contentSettingsChangeHandler = function (changeType, changedValue) {
		// When invariant flag is changed for document.
		if (ContentConstants.SaveAction.INVARIANT == changeType) {
			// Iterate over all the language widget.
			$.each(this._tableContentLangWidgets, function (index, langWidget) {
				if (changedValue) {
					if (!langWidget.isMaster()) {
						// Hide all the available translation.
						langWidget.getJQElement().hide();
					}
				} else {
					if (!langWidget.isMaster()) {
						// Show all the available translation.
						langWidget.getJQElement().show();
					}
				}
			}.bind(this));
		}
		
		// When text type is changed flag is changed for document.
		if (ContentConstants.SaveAction.TEXTTYPE == changeType) {
			// Iterate over all the language widget.
			$.each(this._tableContentLangWidgets, function (index, langWidget) {
				// When value is available that means change is done otherwise, user have reverted back change done for text type.
				if (changedValue) {
					langWidget.disable(TextContentStrings.t("CONTENT_CANNOT_EDIT_SAVE_FIRST"));
				} else {
					langWidget.enable();
				}
			}.bind(this));
		}
		
	};
	
	/**
	 * Method is to save all the changes with the help of content service.
	 * 
	 * @param callBack
	 */
	TableContentPage.prototype.saveContent = function (errorCallback) {
		// Save changes with the help of content service.
		ContentService.saveContents([this._tableContent], function (updatedContents, errorMsgs) {
			var updatedContentIds = Object.keys(updatedContents);
			var errorMsgIds = Object.keys(errorMsgs);
			if (updatedContentIds.length > 0) {
				// Get the updated content.
				var updatedContent = updatedContents[updatedContentIds[0]];
				// We need to refresh the page but without the docversionId parameter since we want the latest version.
				// Therefore we need the following parameters
				var docID = ContentUtils.getParameterByName("docID");
				var menu = ContentUtils.getParameterByName("menu_file");
				
				// Generate the new query string and redirect to it.
				var query = '';
				if (errorMsgs[errorMsgIds[0]]) {
					query = jQuery.param({'docID': docID, 'menu_file': menu, 'docVerID': updatedContent.docVersionId, 'error' : errorMsgs[errorMsgIds[0]]});
				} else {
					query = jQuery.param({'docID': docID, 'menu_file': menu, 'docVerID': updatedContent.docVersionId});
				}
				// Generate the new query string and redirect to it.
				window.location.replace("./doctypetextview.jsp?"+query);
			} else if (errorMsgIds.length > 0) {
				errorCallback(errorMsgs[errorMsgIds[0]], true);
			}
			
		}.bind(this));
	};
	
	/**
	 * Get the table content elements that are currently supported by the selected preview.
	 */
	TableContentPage.prototype.getPreviewElements = function () {
		return this._previewWidget.getPreviewElements();
	};
	
	/**
	 * Get the name of the currently selected preview.
	 */
	TableContentPage.prototype.getPreviewName = function () {
		return this._previewSelector.getPreviewName();
	};
	
	/**
	 * Get the wrapping jQuery element of the table page.
	 */
	TableContentPage.prototype.getJQElement = function () {
		return this._view.getJQElement();
	};
	
	/**
	 * Method is to check whether at-least 1 language widget for master languages is approved.
	 * 
	 * @returns {boolean} True if any one of the master language is having status as approved.
	 */
	TableContentPage.prototype.isMasterLanguageApproved = function () {
		var isMasterApproved = false;
		
		$.each(this._tableContentLangWidgets, function (index, langWidget) {
			// Check language widget is for master language.
			if (!langWidget.isMaster()) {
				return true;
			}
			
			// If any one of the master language is approved, then set the flag as approved and break the loop.
			if (ContentConstants.State.APPROVED == langWidget.getState()) {
				isMasterApproved = true;
				return false;
			}
		}.bind(this));
		
		return isMasterApproved;
		
	};
	
	/**
	 * Get the TableContent private object
	 * @returns
	 */
	TableContentPage.prototype.getTableContent = function () {
		return this._tableContent;
	};
	
	return TableContentPage;
});
