define(["underscore",
        "jquery",
        "../view/TableContentPreviewSelectorView",
        "../../table-content-preview/PreviewLoader",
        "../../table-content-preview/PreviewProcessor"],
	function (_, $, TableContentPreviewSelectorView, PreviewLoader, PreviewProcessor) {
	
	/**
	 * 
	 */
	var TableContentPreviewSelector = function (tableContent, projectId) {
		
		
		this._previewLoader = new PreviewLoader();
		this._previewEntriesMap = {};
		this._tableContent = tableContent;
		
		this._previewLoader.loadPreviews(function (previewEntries) {
			// Only continue if we actually got some previews.
			if (previewEntries && previewEntries.length > 0) {
				
				// Appoint a default selected preview.
				this._selectedPreviewEntry = previewEntries[0];
				
				// Make sure that all the previews have a type.
				$.each(previewEntries, function(index, previewEntry) {
					if(previewEntry && !previewEntry.type){ 
						previewEntry.type = "Custom";
					}
					// Select the selected table preview entry.
					if (tableContent.preview){
						if (tableContent.preview.name == previewEntry.name && tableContent.preview.type == previewEntry.type) {
							this._selectedPreviewEntry = previewEntry;
						}
					}
				}.bind(this));
				
				// Group them on type.
				$.each(previewEntries, function(index, previewEntry) {
					if (this._previewEntriesMap[previewEntry.type]) {
						this._previewEntriesMap[previewEntry.type].push(previewEntry);
					} else {
						this._previewEntriesMap[previewEntry.type] = [previewEntry];
					}
				}.bind(this));
				
				// Load the settings of the currently selected preview and make it the current one.
				this.loadSelectedPreview(function () {
					this.selectPreview(false);
				}.bind(this));
			}
		}.bind(this));
		
		// The current and selected preview (same at the start).
		this._currentPreviewEntry = undefined; 
		this._selectedPreviewEntry = undefined; 
		this._currentPreviewEntryName = undefined;
		
		// Callbacks to execute when a preview is selected.
		this._onPreviewSelectedCallbacks = [];
	
		// Create the view.
		this._view = new TableContentPreviewSelectorView(this._previewEntriesMap, this._currentPreviewEntry, projectId);
		
		// When the user switches to a new preview in the dropdown.
		this._view.onNewPreviewTypeSelected(function (previewType){
			// Get the correct preview entry.
			try {
				this._selectedPreviewEntry = this._previewEntriesMap[previewType][0];
			} catch (e) {
				this._selectedPreviewEntry = undefined;
			}
			this.loadSelectedPreview();
		}.bind(this));
		
		// When a different preview entry standard/name is selected: Get correct path and set it.
		this._view.onNewPreviewNameSelected(function (previewType, previewName){
			try {
				$.each(this._previewEntriesMap[previewType], function(index, preview) {
					if (preview.name == previewName) {
						this._selectedPreviewEntry = preview;
						return;
					}
				}.bind(this));
			} catch (e) {
				this._selectedPreviewEntry = undefined;
			}
			this.loadSelectedPreview();
		}.bind(this));
		
		// When a different preview is selected (user click on "apply").
		this._view.onNewPreviewSettings(function (settings){
			this._selectedPreviewSettings = settings;
			this.selectPreview(true);
		}.bind(this));
	};
	
	/**
	 * Return the name of the current preview.
	 */
	TableContentPreviewSelector.prototype.getPreviewName = function () {
		if (this._currentPreviewEntry) {
			return this._currentPreviewEntry.name;
		} else {
			return undefined;
		}
	};
	
	/**
	 * Make the currently selected preview the current preview.
	 * 
	 * @param {Boolean} savePreview - If true preview setting will be updated in TableContent
	 */
	TableContentPreviewSelector.prototype.selectPreview = function (savePreview) {
		
		// Set it to the current.
		this._currentPreviewEntry = this._selectedPreviewEntry;
		this._currentPreviewHTML =  this._selectedPreviewHTML;
		this._currentPreviewSettings = this._selectedPreviewSettings;
		if(this._selectedPreviewEntryName){
			this._currentPreviewEntryName = this._selectedPreviewEntryName;
		}

		// Select it in the view.
		this._view.setSelectedPreviewSettings(this._currentPreviewEntry, this._currentPreviewSettings, this._currentPreviewEntryName);
		
		// Update the name of the preview in the selector.
		this._view.setPreviewName(this._currentPreviewEntry.name);
		
		// Set the new preview in the table content
		if (savePreview && this._tableContent.canManage().allowed) {
			this._tableContent.setNewPreview(this._currentPreviewEntry);
		}
		
		// Execute the callbacks.
		$.each(this._onPreviewSelectedCallbacks, function(index, callback) {
			callback(this._currentPreviewHTML, this._currentPreviewSettings);
		}.bind(this));
	};
	
	/**
	 * Load the HTML and the settings of the currently selected preview
	 * 
	 * @param callback {Function} - The callback to be executed when the selected preview is loaded (optional).
	 */
	TableContentPreviewSelector.prototype.loadSelectedPreview = function (callback) {
		if (this._selectedPreviewEntry) {
			// Get the html and settings of the newly selected one.
			var previewProcessor = new PreviewProcessor(this._selectedPreviewEntry);
			
			// Parse the preview and get the settings.
			previewProcessor.getPreview(function (html, settings) {
				// Set
				this._selectedPreviewHTML = html;
				this._selectedPreviewSettings = settings;
				
				// Display the settings in the view.
				this._view.setSelectedPreviewSettings(this._selectedPreviewEntry, this._selectedPreviewSettings, this._selectedPreviewEntry.name);
			
				// Execute the callback if one was given.
				if (callback) {
					callback();
				}
			}.bind(this));
		}
	};
	
	/**
	 * Register a callback that should be executed when a preview is selected.
	 */
	TableContentPreviewSelector.prototype.onPreviewSelected = function (callback) {
		this._onPreviewSelectedCallbacks.push(callback);
	};
	
	
	/**
	 * Set the default locale mapping in the preview widget.
	 */
	TableContentPreviewSelector.prototype.setDefaultLocaleMapping = function (defaultLocaleMapping) {
		this._view.setDefaultPreviewLocaleMapping(defaultLocaleMapping);
	};
	
	/**
	 * Get the wrapping jQuery element.
	 */ 
	TableContentPreviewSelector.prototype.getJQElement = function () {
		return this._view.getJQElement();
	};
	
	return TableContentPreviewSelector;
});
