define(["underscore",
        "jquery",
        "../../ContentConstants",
        "utils/webcss!./StateFullContentWidget.css"],
	function (_, 
			$,
			ContentConstants) {
		
		
		/**
		 * Create a StateFullContentWidget.
		 */
		var StateFullContentWidget = function (state, nextStates) {

			// Skeleton.
			this._wrapper = $("<div>", {"class": "StateFullContentWidget"});
			this._header = $("<div>", {"class": "Header"});
			this._body = $("<div>", {"class": "MainContent"});

			this._wrapper.append(this._header, this._body);
			
			this._headerContentWrapper = $("<div>", {"class" : "HeaderContent"});
			this._mainContentWrapper = $("<div>");
			this._customButtonWrapper = $("<div>", {"class" : "CustomButtons"});
			
			this._state = state;
			this._newState = undefined;
			this._nextStates = nextStates;
			
			this._onNewStateCallbacks = [];
			this._onRevertStateCallbacks = [];
			this._onHeaderClickCallbacks = [];
			this._customButtons = [];
			
			this._constructView();
			this._activateApprovalActions();
			this._registerHeaderClick();
			
		};
		
		/**
		 * Update state full content widget for new current state and its next states.
		 * 
		 * @param {String} currentState : current state of the state full widget.
		 * @param {Array} nextStates: Next available state.
		 */ 
		StateFullContentWidget.prototype.update = function (state, nextStates) {
			this._state = state;
			this._nextStates = nextStates;
			this._newState = undefined;
			
			this.clearCustomButtons();
			
			this._mainContentWrapper.detach();
			
			this._constructView();
			
			this._customButtonWrapper.empty();
			$.each(this._customButtons, _.bind(function (index, button) {
				this._customButtonWrapper.append(button);
			}, this));
			
			this._activateApprovalActions();
		};
		
		/**
		 * Set the new state of the widget.
		 */
		StateFullContentWidget.prototype.setCurrentState = function (currentState) {
			
			// Check if the new current state is the same as the inital state ...
			if (this._state && this._state == currentState) {
				// Check if a new state is already given ...
				if (!this._newState) {
					// No new state given, so no need to return or do a thing.
					return;
				} else {
					// A new state was given, so we need to "revert" back to the initial state.
					this._newState = undefined;
				}
			}
			
			// If we have a new state but the current state is the same as this new state, don't do a thing.
			if (this._newState && this._newState == currentState) {
				return;
			}
			
			// Stop the interactive approval actions.
			this._deactivateApprovalActions();
			
			// Set the new state.
			this._newState = currentState;
			
			// Remove the old approval collumn.
			this._approvalCol.remove();
			
			// Construct an approval collumn.
			this._approvalCol = this._constructApprovalCol();
			
			// Add it to the header.
			this._header.prepend(this._approvalCol);
			
			// Make it interactive
			window.setTimeout(function () {
				this._activateApprovalActions();
			}.bind(this), 200);
		};
		
		/**
		 * Disable header actions and bulk approval.
		 */ 
		StateFullContentWidget.prototype.disable = function (msgTag) {
			this._nextStates = [];
			
			this.clearCustomButtons();
			
			this._constructView();
			
			this._customButtonWrapper.empty();
		};
		
		/**
		 * Construct the view.
		 */
		StateFullContentWidget.prototype._constructView = function () {
			
			// Construct an approval column.
			this._approvalCol = this._constructApprovalCol();
			
			// Empty the header DIV.
			this._header.empty();
			
			// Create the state actions in the header.
			this._header.append(this._approvalCol);
			
			// Add the wrappers to the skeleton.
			this._header.append(this._headerContentWrapper);
			
			// Add the custom button wrapper to the header.
			this._header.append(this._customButtonWrapper);
			
			// Empty body DIV.
			this._body.empty();
			
			// Add to the body
			this._body.append(this._mainContentWrapper);
		};
		
		/**
		 * Construct the approval actions collumn.
		 */
		StateFullContentWidget.prototype._constructApprovalCol = function () {
						
			var approvalCol = $('<div>').addClass("ApprovalCol");
			
			// Create an icon for the current state ...
			var curStateCol = $('<div>', {"class" : "CurrentState"}).css({'opacity': '1'});
			
			if(this._nextStates.length == 0) {
				curStateCol.css({'cursor': 'default'});
			} else {
				curStateCol.css({'cursor': 'pointer'});
			}
			
			var statusIconClass = null;
			var statusClass = null;
			if(!this._newState) { // If no new state is defined, render the current state.
				switch(this._state) {
				case ContentConstants.State.DRAFT:
					statusIconClass = "icon icon-WCR_Text-Status-Draft StatusIconAndLabel";
					statusClass = "Status StatusDraft";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					statusIconClass = "icon icon-WCR_Text-Status-Pending StatusIconAndLabel";
					statusClass = "Status StatusPending";
					break;
				case ContentConstants.State.APPROVED:
					statusIconClass = "icon icon-WCR_Text-Status-Approved StatusIconAndLabel";
					statusClass = "Status StatusApproved";
					break;
				case ContentConstants.State.REJECTED:
					statusIconClass = "icon icon-WCR_Text-Status-Rejected StatusIconAndLabel";
					statusClass = "Status StatusRejected";
					break;
				default :
					statusIconClass = "icon icon-WCR_Text-Status-NA StatusIconAndLabel";
					statusClass = "Status";
					break;	
				}
			} else { // If a new state is defined, render the current state.
				switch(this._newState) {
				case ContentConstants.State.DRAFT:
					statusIconClass = "icon icon-WCR_Text-Action-Draft RevertTextToDraftIconAndLabel";
					statusClass = "Status RevertTextToDraft";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					statusIconClass = "icon icon-WCR_Text-Action-Start StartTextApprovalIconAndLabel";
					statusClass = "Status StartTextApproval";
					break;
				case ContentConstants.State.APPROVED:
					statusIconClass = "icon icon-WCR_Text-Action-Approve ApproveTextIconAndLabel";
					statusClass = "Status ApproveText";
					break;
				case ContentConstants.State.REJECTED:
					statusIconClass = "icon icon-WCR_Text-Action-Reject RejectTextIconAndLabel";
					statusClass = "Status RejectText";
					break;
				default :
					statusIconClass = "icon icon-WCR_Text-Status-NA StatusIconAndLabel";
					statusClass = "Status";
					break;
				}
			}			
			
			// Display the language status
			curStateCol.removeClass().addClass(statusClass).append(
					$('<div>').addClass("ApprovalStatus").append(
							$('<div>').addClass("StatusIcon").append($('<i>').addClass(statusIconClass)))
			);
			
			// Add the collumn.
			approvalCol.append(curStateCol);
			
			// Create actions to go to the next state (or revert) ...
			var actionsCol = $('<div>').addClass("Actions").css({'opacity': '0'}).hide();
			
			// If a new states was specified ...
			if (!this._newState) {
				// Add all the possible actions ...
				var actionsWidth = 40;
				for (var i=0; i < this._nextStates.length; i++) {
					switch(this._nextStates[i]) {
					case ContentConstants.State.DRAFT:
						statusIconClass = "icon icon-WCR_Text-Action-Draft RevertTextToDraftIconAndLabel";
						statusClass = "Status RevertTextToDraft";
						break;
					case ContentConstants.State.READY_FOR_APPROVAL:
						statusIconClass = "icon icon-WCR_Text-Action-Start StartTextApprovalIconAndLabel";
						statusClass = "Status StartTextApproval";
						break;
					case ContentConstants.State.APPROVED:
						statusIconClass = "icon icon-WCR_Text-Action-Approve ApproveTextIconAndLabel";
						statusClass = "Status ApproveText";
						break;
					case ContentConstants.State.REJECTED:
						statusIconClass = "icon icon-WCR_Text-Action-Reject RejectTextIconAndLabel";
						statusClass = "Status RejectText";
						break;
					default :
						// Exectution should not reach here.
						break;
					}
					
					// Create the action.
					var action = $('<div>').addClass(statusClass).append(
							$('<div>').addClass("ApprovalStatus").append(
									$('<div>').addClass("StatusIcon").append($('<i>').addClass(statusIconClass)))
					);
					
					// When the action is clicked, move to the new state.
					(function (that, state) {
						action.on("click", function (event) {
							// Set the new state
							this.setCurrentState(state);
							// Iterate over all the callbacks ...
							for (var j=0; j < this._onNewStateCallbacks.length; j++) {
								 this._onNewStateCallbacks[j](state);
							}
							
							// Stop triggering event for its parent.
							event.stopPropagation();
						}.bind(that));
					})(this, this._nextStates[i]);
					
					// Add the action, to the action collumn.
					actionsCol.append(action);
					actionsWidth += 40;
				}
				
				// Set the width of the actions collumn.
				actionsCol.width(actionsWidth);
			
			} else {
				// Create a revert button.
				var undoAction = $('<div>').addClass("Status RevertApprovalAction").append(
						$('<div>').addClass("ApprovalStatus").append(
								$('<div>').addClass("StatusIcon").append($('<i>').addClass("icon icon-WCR_Revert RevertApprovalActionIconAndLabel")))
				);
				
				// When the undo button is clicked.
				undoAction.on("click", function (event) {
					
					// Remember what was the new state.
					var wasNewState = this._newState;
					
					// Revert the state
					this.setCurrentState(undefined);
					
					// Iterate over all the callbacks ...
					for (var j=0; j < this._onRevertStateCallbacks.length; j++) {
						 this._onRevertStateCallbacks[j](wasNewState);
					}
					
					// Stop triggering event for its parent.
					event.stopPropagation();
				}.bind(this));
				
				// Add the action.
				actionsCol.append(undoAction);
			}
			
			// Add the collumn.
			approvalCol.append(actionsCol);
			
			// Prevent event to be trigger on parent DIV.
			approvalCol.on("click", function (event) {
				event.stopPropagation();
			});
			
			return approvalCol;
		};
		
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		StateFullContentWidget.prototype._deactivateApprovalActions = function() {
			
			// Only do this when there are approval actions available to the element ...
			if (this._nextStates.length == 0) {
				return;
			}
			
			var curState = this._approvalCol.find(".CurrentState");
			var actions = this._approvalCol.find(".Actions");
			
			// Stop the animations
			curState.finish();
			actions.finish();
			
			// Make sure that the header content is back in place ...
			this._headerContentWrapper.stop();
			this._headerContentWrapper.animate({'marginLeft': '0px'}, 500);
			this._headerContentWrapper.finish();
			
			// Make sure the 
			this._approvalCol.unbind('mouseenter mouseleave');
		};
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		StateFullContentWidget.prototype._activateApprovalActions = function() {
			
			// Only do this when there are approval actions available to the element ...
			if (this._nextStates.length == 0 && !this._newState) {
				return;
			}
			
			// Create a handler when mouseover element status to view approval actions
			if (!this._actionsDivHandlerIn) {
				this._actionsDivHandlerIn = _.bind(function () {
					
					// Get the actions and curState elements.
					var curState = this._approvalCol.find(".CurrentState");
					var actions = this._approvalCol.find(".Actions");
					
					// Don't do anything when there are no actions.
					if (this._nextStates.length == 0 && !this._newState) {
						return;
					}
					
					// Fade-in the actions.
					curState.finish();
					actions.finish();
					curState.animate({opacity: 0}, 400, 'swing');
					actions.animate({opacity: 1}, 400, 'swing');
					actions.show();
					
					// Figure out how far the actions will span.
					var offset = 0;
					if (actions.children().length == 2) {
						offset = 40;
					} else if (actions.children().length == 3) {
						offset = 80;
					}
					this._headerContentWrapper.finish();
					if (offset > 0) {
						this._headerContentWrapper.stop();
						this._headerContentWrapper.animate({'marginLeft': offset+'px'}, 500);
					}
				}, this);
			}
						
			// Create a handler when mouseout element status to hide approval actions
			if (!this._actionsDivHandlerOut) {
				this._actionsDivHandlerOut = _.bind(function () {
					
					// Get the actions and curState elements.
					var curState = this._approvalCol.find(".CurrentState");
					var actions = this._approvalCol.find(".Actions");
					
					// Don't do anything when there are no actions.
					if (this._nextStates.length == 0 && !this._newState) {
						return;
					}
					
					// Fade-out the actions.
					curState.finish();
					actions.finish();
					curState.animate({opacity: 1}, 400, 'swing');
					actions.animate({opacity: 0}, 400, 'swing', function () {
						actions.hide();
					});
					
					// Make sure the header content is back in it's place.
					this._headerContentWrapper.stop();
					this._headerContentWrapper.animate({marginLeft: '0px'}, 500, 'swing');
					
				}, this);
			}
			
			// add the hover handler for approval actions
			this._approvalCol.unbind('mouseenter mouseleave');
			this._approvalCol.on("mouseenter", this._actionsDivHandlerIn);
			this._approvalCol.on("mouseleave", this._actionsDivHandlerOut);
		};
		
		/**
		 * Add a button to the widget.
		 */
		StateFullContentWidget.prototype.addCustomButton = function (button) {
			this._customButtons.push(button);
			this._customButtonWrapper.append(button);
		};
		
		/**
		 * Clear the custom buttons.
		 */
		StateFullContentWidget.prototype.clearCustomButtons = function () {
			// Detach custom buttons so that they keep their bindings.
			$.each(this._customButtons, function (index, button) {
				button.detach();
			});
		};
		
		/**
		 * Set the jQuery element to be put in the main body.
		 */
		StateFullContentWidget.prototype.setMainContent = function (mainContent) {
			this._mainContent = mainContent;
			this._mainContentWrapper.empty();
			this._mainContentWrapper.append(this._mainContent);
		};
		
		/**
		 * Set the jQuery element to be put in the header.
		 */
		StateFullContentWidget.prototype.setHeaderContent = function (headerContent) {
			this._headerContent = headerContent;
			this._headerContentWrapper.empty();
			this._headerContentWrapper.append(this._headerContent);
		};
		
		/**
		 * Register event for click on header.
		 */
		StateFullContentWidget.prototype._registerHeaderClick = function () {
			this._header.on("click", _.bind(function () {
				$.each(this._onHeaderClickCallbacks, _.bind(function (index, callback) {
					callback();
				}, this));
			}, this));
		};
		
		/**
		 * Get the original state of the widget.
		 */
		StateFullContentWidget.prototype.getState = function () {
			return this._state;
		};
		
		/**
		 * Set the original state of the widget.
		 */
		StateFullContentWidget.prototype.setState = function (state) {
			this._state = state;
		};
		
		/**
		 * Get the possible next states of the widget.
		 */
		StateFullContentWidget.prototype.getNextStates = function () {
			return this._nextStates;
		};
		
		/**
		 * Set the possible next states of the widget.
		 */
		StateFullContentWidget.prototype.setNextStates = function (nextStates) {
			this._nextStates = nextStates;
		};
		
		/**
		 * Get the new state of the widget.
		 */
		StateFullContentWidget.prototype.getNewState = function () {
			return this._newState;
		};
		
		/**
		 * Expand the main content.
		 */
		StateFullContentWidget.prototype.expand = function () {
			this._body.show();
		};
		
		/**
		 * Register a callback to be called when the header is clicked.
		 */
		StateFullContentWidget.prototype.isExpanded = function () {
			return this._body.is(':visible');
		};
		
		/**
		 * Hide the main content.
		 */
		StateFullContentWidget.prototype.collapse = function () {
			this._body.hide();
		};
		
		/**
		 * Get the current state of the widget.
		 */
		StateFullContentWidget.prototype.getCurrentState = function () {
			if (this._newState) {
				return this._newState;
			}
			return this._state;
		};
		
		/**
		 * Register a callback to be called when a new state is selected.
		 */
		StateFullContentWidget.prototype.onNewState = function (callback) {
			this._onNewStateCallbacks.push(callback);
		};
		
		/**
		 * Register a callback to be called when the new state is reverted.
		 */
		StateFullContentWidget.prototype.onRevertState = function (callback) {
			this._onRevertStateCallbacks.push(callback);
		};
		
		/**
		 * Register a callback to be called when the header is clicked.
		 */
		StateFullContentWidget.prototype.onHeaderClick = function (callback) {
			this._onHeaderClickCallbacks.push(callback);
		};
		
		/**
		 * Get the wrapping jQuery Element of the widget.
		 */
		StateFullContentWidget.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		return StateFullContentWidget;
});