define(["jquery",
        "../../ContentConstants",
        "utils/webcss!./TableContentPreview.css"],
	function ($, ContentConstants) {
	
	/**
	 * Create a TableContentPreviewWidget.
	 */
	var TableContentPreviewWidget = function (tableContent, parent) {
		
		// The tableContent.
		this._tableContent = tableContent;
		
		// The preview wrapper.
		this._parent = parent;
		this._wrapper = $("<div>",{"class" : "TableContentPreview"});
		this._innerWrapper = $("<div>");
		this._wrapper.append(this._innerWrapper);
		
		// When a placeholder clicked.
		this._onPlaceHolderClicked = [];
		this._onPlaceHolderMouseOut = [];
		this._onPlaceHolderMouseEnter = [];
		
		// When preview loaded, default locale selection.
		this._onNewDefaultLocaleMapping = [];
		
		// Contain elements and their value to hide or show.
		this._placeHolderMap = {};	
		this._constructPreview = $.Deferred();
		this._promise = this._constructPreview.promise();
	};
	
	/**
	 * Set the preview in the widget.
	 * 
	 * @param previewHTML
	 * @param previewSettings
	 */
	TableContentPreviewWidget.prototype.setPreview = function (previewHTML, previewSettings, previewName) {
						
		this._previewSettings = previewSettings;
		this._previewName = previewName;
		this._innerWrapper.html(previewHTML);
		
		// Bind the callbacks.
		var placeholders = this._wrapper.find(".Placeholder");
		placeholders.on("click", function (e) {
			var textContentElementId = this._getContentId($(e.delegateTarget));
			for (var i=0; i < this._onPlaceHolderClicked.length; i++) {
				this._onPlaceHolderClicked[i](textContentElementId);
			}
		}.bind(this));
		placeholders.on("mouseenter", function (e) {
			var textContentElementId = this._getContentId($(e.delegateTarget));
			for (var i=0; i < this._onPlaceHolderMouseOut.length; i++) {
				this._onPlaceHolderMouseOut[i](textContentElementId);
			}
		}.bind(this));
		placeholders.on("mouseleave", function (e) {
			var textContentElementId = this._getContentId($(e.delegateTarget));
			for (var i=0; i < this._onPlaceHolderMouseEnter.length; i++) {
				this._onPlaceHolderMouseEnter[i](textContentElementId);
			}
		}.bind(this));
		
		// Scale the preview
		this.scalePreview();
		
		// Scale the preview when the window size changes.
		$(window).resize(function () {
			// Reset scaling.
			this.scalePreview();
		}.bind(this));
		
		this._constructPreview.resolve(placeholders);
		
		return this._promise;		
	};
		
	/**
	 * Get a list of the table content elements that would be supported by the selected preview.
	 */
	TableContentPreviewWidget.prototype.getPreviewElements = function () {
		var elements = [];
		var elementsDeferred = $.Deferred();

		this._promise.done(_.bind(function(){
			var placeholders = this._wrapper.find(".Placeholder");
			
			$.each(placeholders, function (i, placeholder) {
			
				var type = placeholder.getAttribute("data-elemType");
				// Only continue if we could find an element type.
				if (!type || type.length == 0) {
					return;
				}
				
				var instance = parseInt(placeholder.getAttribute("data-instance"));
				// Fall back on "1" if none is specified.
				if (!instance || instance == undefined || instance == null || instance.length == 0) {
					instance = 1;
				}
				
				var locale =  parseInt(placeholder.getAttribute("data-locale"));
				// Fall back on "1" if none is specified.
				if (!locale || locale == undefined || locale == null || locale.length == 0) {
					locale = 1;
				}
				
				// Create an element.
				var newElement = {
					type : type,
					locale : locale,
					instance : instance
				};
				
				// Make sure no such element is already in the array.
				var duplicate = false;
				$.each(elements, function(i, element) {
					if (element.type == newElement.type &&
							element.locale == newElement.locale &&
							element.instance == newElement.instance) {
						duplicate = true;
						return;
					}
				}.bind(this));
				
				// If the element is not a duplicate.
				if (!duplicate) {
					elements.push(newElement);
				}
			
			}.bind(this));
			
			elementsDeferred.resolve(elements);
			
		}, this));
		
		return elementsDeferred.promise();
	};
	
	/**
	 * Scale the preview.
	 */
	TableContentPreviewWidget.prototype.scalePreview = function () {
		// We only can do this if a parent is set.
		if (this._parent) {
			
			this._innerWrapper.css("transform", "scale(1)");
			this._innerWrapper.css("visibility", "hidden");
			
			// Give the preview a small delay in order to settle ...
			setTimeout(function () {
				
				
				// Create the maximum width.
				var maxWidth = this._innerWrapper[0].scrollWidth;
				var maxHeight = this._innerWrapper[0].scrollHeight;
				if (maxWidth == 0 || maxHeight == 0) {
					return; // Only continue if a width or height is set.
				}
				
				// Width of the parent.
				var widthParent = this._parent.width();
				var heightWindow = $(window).height()-20;
				
				// Calculate the scale.
				var scaleH = widthParent/maxWidth;
				var scaleV = heightWindow/maxHeight;
				var scale = 1;
				if (scaleH < scaleV) {
					scale = scaleH;
					this._innerWrapper.css("height", "auto");
				} else {
					scale = scaleV;
					this._innerWrapper.css("height", heightWindow+"px");
				}
				
				// Find the origin.
				var origin;
				if (scaleH < 1) {
					origin = "top left";
				} else {
					origin = "top center";
				}
				
				// Calculate the scale.
				this._innerWrapper.css("transform", "scale("+scale+")");
				this._innerWrapper.css("transform-origin", origin);
				this._innerWrapper.css("visibility", "visible");
				
			}.bind(this), 10);
			
		}
	};
	
	/**
	 * Set the parent element of the preview.
	 */
	TableContentPreviewWidget.prototype.setParent = function (parent) {
		this._parent = parent;
		this.scalePreview();
	};
	
	/**
	 * Set the text of the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.fillForTableContent = function (tableContent) {
		
		this._tableContent = tableContent;
		
		// Add the correct "Empty" classes.
		var placeholders = this._wrapper.find(".Placeholder");
		for (var i=0; i < placeholders.length; i++) {
			
			var placeholder = $(placeholders[i]);
			var content = placeholder.text();
			if (!content || content.length == 0) {
				placeholder.addClass("Empty");
			}
		}
		
		// Iterate over all the text contents.
		$.each(tableContent.getTextContents(), function (index, textContent) {
			$.each(textContent.getMasterElements(), function (index, element) {
				this.setText(element);
			}.bind(this));
			$.each(textContent.getTranslationElements(), function (index, element) {
				this.setText(element);
			}.bind(this));
		}.bind(this));
		
		// Update the visibility of the preview.
		this.updateVisibility();
	};
	
	
	/**
	 * Show and hide the element type in the preview based on deletion or addition.
	 */
	TableContentPreviewWidget.prototype.updateVisibility = function () {
		
		// Take the locale mapping from the preview settings.
		var localeMapping = this._previewSettings.localeMapping;
		// If no locale mapping is provided in the preview settings ... make a "default" one.
		if (!localeMapping) {
			localeMapping = this._makeLocaleMapping(this._previewSettings.locales);
		}
		
		// Extract the language codes in the locale mapping.
		var localeLangCodes = [];
		for (var locale in localeMapping) {
		    if (localeMapping.hasOwnProperty(locale)) {
		    	localeLangCodes.push(localeMapping[locale]);
		    }
		}
		
		// Get all the elements in the table
		var allElements = this._tableContent.getTextContentElements();
		
		// Filter all the elements down to only the one who are used in the current locale mapping.
		var allElementsforLocales = [];
		_.each(allElements, function(element) {
			if (_.contains(localeLangCodes, element.language)) {
				allElementsforLocales.push(element);
			}
		}.bind(this));
		
		// Optional field handling
		var optionalContainers = this._wrapper.find(".Optional");
		
		$.each(optionalContainers, function (i, optionalContainer) {
			
			var hasElementInTable = false;
			var hasMandatoryPlaceholder = false;
			
			var optionalPlaceHolders = $(optionalContainer).find(".Placeholder");
			
				$.each(optionalPlaceHolders, function (i, optionalPlaceholder) {
						
					// Check if the placeholder is a mandatory one.
					if($(optionalPlaceholder).hasClass("Mandatory")){
						hasMandatoryPlaceholder =  true;
					}
					
					// Check whether the placeholder is for an element in the table. 
					// If that is the case "hasElementInTable" is true and you can end the
					// loop because it will be shown anyway regardless of whether a mandatory
					// is present or not.
					var contentId = this._getContentId($(optionalPlaceholder));
					
					_.each(allElementsforLocales, _.bind(function (tableContentElement) {
						if (!tableContentElement.deleted) {
							var elementContentId = tableContentElement.getContentId();
							if(contentId == elementContentId){
								hasElementInTable = true;
								return true;
							}
						}
					}, this));
					
				}.bind(this));
			
			if (!hasElementInTable && !hasMandatoryPlaceholder) {
				$(optionalContainer).hide();
			} else {
				$(optionalContainer).show();
			}
		}.bind(this));
		
		this.scalePreview();
	};
	
	
	/**
	 * Set the text of the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype._getPlaceholder = function (textContentElement) {
		
		// Check whether we have some preview settings.
		if (!this._previewSettings) {
			return;
		}
		
		// If an array is passed, call recursively.
		if (typeof textContentElement == "array") {
			var placeholders = [];
			$.each(textContentElements, function (index, element) {
				var placeholder = this._getPlaceholder(element);
				if (placeholder) {
					placeholders.push(placeholder);
				}
			}.bind(this));
			return $(placeholders);
		}
		
		// Extract the necesary information.
		var elemType = textContentElement.getParent().getElementType();
		var instance = textContentElement.getParent().getInstance();
		var lang = textContentElement.getLangCode();
		
		// Take the locale mapping from the preview settings.
		var localeMapping = this._previewSettings.localeMapping;
		// If no locale mapping is provided in the preview settings ... make a "default" one.
		if (!localeMapping) {
			localeMapping = this._makeLocaleMapping(this._previewSettings.locales);
			// Default locale selection in the preview.
			for (var i=0; i < this._onNewDefaultLocaleMapping.length; i++) {
				this._onNewDefaultLocaleMapping[i](localeMapping);
			}
		}
		
		// Filter on Element Type
		var filteredOnElemType = this._wrapper.find(".Placeholder[data-elemType='"+elemType+"']");
		
		// Filter on
		var filteredOnInstance = filteredOnElemType.filter("[data-instance='"+instance+"']");
		
		// If the instance is "1" or undefined allow to also fall back on place-holders without an instance attribute.
		if (instance == "1" || instance == 1 || !instance) {
			filteredOnInstance = $.merge(filteredOnInstance, filteredOnElemType.filter(":not([data-instance])"));
		}

		// Filter on language
		var locale = undefined;
		if (lang) {
			locale = this._localeForLang(lang, localeMapping);
			if (!locale) {
				console.log("Could not find the correct locale for the given language.");
				return;
			}
		}
		var filteredOnLanguage = filteredOnInstance.filter("[data-locale='"+locale+"']");
		if (locale == "1" || locale == 1 || !locale) {
			filteredOnLanguage = $.merge(filteredOnLanguage, filteredOnInstance.filter(":not([data-locale])"));
		}
		
		// The ones that made the cut are the valid placeholders ...
		return filteredOnLanguage;
	};
	
	/**
	 * Set the text of the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype._getContentId = function (placeholder) {
		var elemType = placeholder.attr("data-elemtype");
		var instance = placeholder.attr("data-instance");
		var locale = placeholder.attr("data-locale");
		
		if (!instance) {
			// Fall back on "1"
			instance = "1";
		}
		if (!locale) {
			// Fall back on "1"
			locale = "1";
		}
		if (!elemType) {
			// We can not fall back on a default element type, just return nothing.
			console.log("No element type found for the passed placeholder.");
			return;
		}
		
		// Take the locale mapping from the preview settings.
		var localeMapping = this._previewSettings.localeMapping;
		// If no locale mapping is provided in the preview settings ... make a "default" one.
		if (!localeMapping) {
			localeMapping = this._makeLocaleMapping(this._previewSettings.locales);
		}
		
		// We need to know what actual language corresponds to the locale ...
		var langCode = this._langForLocale(locale, localeMapping);
		if (langCode) {
			return elemType+"_"+instance+"_"+langCode;
		} else {
			console.log("No such language found in the locale mapping.");
			return;
		}	
	};
	
	/**
	 * Get the local for the given langCode.
	 */
	TableContentPreviewWidget.prototype._localeForLang = function (langCode, localeMapping) {
		// When no specific locale mapping is given, fall back on the one in the settings.
		if (!localeMapping && this._previewSettings.localeMapping) {
			localeMapping = this._previewSettings.localeMapping;
		}
		// Check if there's a locale mapping to use.
		if (localeMapping) {
			for (var locale in localeMapping) {
				if (localeMapping.hasOwnProperty(locale)) {
					if (langCode == localeMapping[locale]) {
						return locale;
					}
				}
			}
			console.log("No locale for the requested language.");
		} else {
			console.log("No locale mapping given!");
		}
	};
	
	/**
	 * Get the langcode for the given locale.
	 */
	TableContentPreviewWidget.prototype._langForLocale = function (locale, localeMapping) {
		// When no specific locale mapping is given, fall back on the one in the settings.
		if (!localeMapping && this._previewSettings.localeMapping) {
			localeMapping = this._previewSettings.localeMapping;
		}
		// Check if there's a locale mapping to use.
		if (localeMapping && localeMapping[locale]) {
			return localeMapping[locale];
		} else {
			console.log("Invalid locale mapping given!");
		}
	};
	
	
	/**
	 * Set the text of the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.setText = function (textContentElement) {

		// Get the placeholders.
		var placeholders = this._getPlaceholder(textContentElement);
		
		
		// Only continue if placeholders could be found.
		if (!placeholders || placeholders.length == 0) {
			return;
		}
		
		// Set the text content.
		var textType = textContentElement.getParent().getParent().getTextType();
		var textString = textContentElement.getTextData().content;

		if (textType ==  ContentConstants.TextType.SIMPLE) {
			placeholders.text(textString);
		} else if (textType ==  ContentConstants.TextType.RICHTEXT) {
			placeholders.html(textString);
		} else if (textType ==  ContentConstants.TextType.SMARTNAME) {
			placeholders.text(textString);
		}
		
		var textData = placeholders.text();
		// Check if the textstring contains characters
		if (textData && textData.length > 0) {
			placeholders.removeClass("Empty");
		} else {
			placeholders.addClass("Empty");
		}
		
	};
	
	/**
	 * Select the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype._makeLocaleMapping = function (locales) {
		// Get the languages in the table.
		var langCodes = this._tableContent.getLangCodes();
		// Create a locale map.
		var localeMap = {};
		// Iterate over all the locales.
		for (var i=1; i <= locales.length; i++) {
			// Assign the languages (master lang will come first).
			if (langCodes[i-1]) {
				localeMap[i] = langCodes[i-1];
			} else {
				break;
			};
		}
		return localeMap;
	};

	
	/**
	 * Register a callback that should be called when a placeholder is clicked.
	 */
	TableContentPreviewWidget.prototype.onPlaceholderClicked = function (callback) {
		this._onPlaceHolderClicked.push(callback);
	};
	
	/**
	 * Register a callback that should be called when a placeholder is left with the mouse.
	 */
	TableContentPreviewWidget.prototype.onPlaceholderMouseOut = function (callback) {
		this._onPlaceHolderMouseOut.push(callback);
	};
	
	/**
	 * Register a callback that should be called when a placeholder is entered with the mouse.
	 */
	TableContentPreviewWidget.prototype.onPlaceholderMouseEnter = function (callback) {
		this._onPlaceHolderMouseEnter.push(callback);
	};
	
	/**
	 * Select the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.focusPlaceholder = function (textContentElement) {
		// Get the placeholders.
		var placeholders = this._getPlaceholder(textContentElement);
		
		// Add the selected class to the placeholders.
		if (placeholders) {
			placeholders.addClass("Selected");
		}
	};
	
	/**
	 * Deselect the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.blurPlaceholder = function (textContentElement) {
		// Get the placeholders.
		var placeholders = this._getPlaceholder(textContentElement);
		
		// Remove the selected class to the placeholders.
		if (placeholders) {
			placeholders.removeClass("Selected");
		}
	};
	
	/**
	 * Highlight the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.highlightPlaceholder = function (textContentElement) {
		// Get the placeholders.
		var placeholders = this._getPlaceholder(textContentElement);
		
		// Unselect all selected placeholders.
		this._wrapper.find(".Placeholder").removeClass("HighLight");
		
		// Only continue of placeholders could be found.
		if (!placeholders || placeholders.length == 0) {
			return;
		}
		
		// Add the selected class to the placeholders.
		placeholders.addClass("HighLight");
	};
	
	/**
	 * Highlight the placeholder in the preview for the passed text content element.
	 */
	TableContentPreviewWidget.prototype.unHighlightPlaceholder = function (textContentElement) {
		// Get the placeholders.
		var placeholders = this._getPlaceholder(textContentElement);
		
		// Only continue of placeholders could be found.
		if (placeholders) {
			placeholders.removeClass("HighLight");
		}
	};
	

	/**
	 *  Register a callback to be executed when preview is loaded first time. Default locale mapping will be set in preview.
	 */
	TableContentPreviewWidget.prototype.onNewDefaultLocaleMapping = function (callback) {
		this._onNewDefaultLocaleMapping.push(callback);
	};
	
	/**
	 * Get the wrapping jQuery element.
	 */
	TableContentPreviewWidget.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return TableContentPreviewWidget;
});