define(["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "../../ContentConstants",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "packaging-content-management/content-page/view/ContentSettingsView",
        "packaging-content-management/text-content-services/elementTypeService",
        "utils/webcss!./TableContentSettingsView.css"],
	function (_, 
			$,
			i18n,
			ContentConstants,
			LanguageUtils,
			ContentSettingsView,
			elementTypeService) {
		
	"use strict";
	
	/**
	 * @class
	 * @exports TableContentSettingsView
	 * @augments ContentSettingsView
	 * 
	 * @param {Boolean} contentManagerView - True if content manager permission is available for TableContent
	 * @param {Map.<String><Object>} settingsMap - Key value pair of table content settings fields
	 * @param {String} currentState - Current state of the table content
	 * @param {Array.<String>} nextStates - Possible states that can be set for the table content
	 */
	var TableContentSettingsView = function (contentManagerView, settingsMap, currentState, nextStates) {
		ContentSettingsView.call(this, contentManagerView, settingsMap, currentState, nextStates);
	};
	
	// Inherit
	TableContentSettingsView.prototype = Object.create(ContentSettingsView.prototype);
	TableContentSettingsView.prototype.constructor  = TableContentSettingsView;
	
	/**
	 * Constructs the header for the widget
	 */
	TableContentSettingsView.prototype.constructHeader = function(){
		this._headerDiv = $('<div/>', {'class':'TableContentHeader', 'id':'tableSettingsDiv' });
		
		// Create the div for the element name.
		var elementNameDiv = $('<div>', {"class" : "TableContentHeaderElementType"});
		this._headerDiv.append(elementNameDiv);
		
		// Load the display string for the element type.
		elementTypeService.getReadableElementType(this._settingsMap.get(ContentConstants.UpdateField.ELEMENT_TYPE), false, function (elementDisplayString) {
			elementNameDiv.text(elementDisplayString);
		});
		
		var addMasterPrefix = true;
		
		// DIV should be created only if master flags are going to be displayed.
		var masterLangDiv = undefined; 
		
		var projMasterLangs = this._settingsMap.get(ContentConstants.ContentField.PROJ_MASTER_LANGS);
		var docMasterLangCodes = this._settingsMap.get(ContentConstants.ContentField.DOC_MASTER_LANGS);
		for(var i = 0; i < projMasterLangs.length; i++){
			for(var j = 0; j < docMasterLangCodes.length; j++){
				if(projMasterLangs[i].langCode == docMasterLangCodes[j]){
					// Add master prefix only when master flags are available to show.
					if (addMasterPrefix) {
						// Create language DIV and append to header. 
						masterLangDiv = $('<div/>', {'class':'masterTableContentHeaderLang' });
						this._headerDiv.append(masterLangDiv);
						
						// Prefix for the master language flags.
						masterLangDiv.append(i18n.t("MASTER_LANGUAGE_PREFIX"));
						addMasterPrefix = false;
					}
					masterLangDiv.append($('<img>', {"class" : "Flag"}).attr({"src": "images/country_flags/" + projMasterLangs[i].langFlag, "title": projMasterLangs[i].langDesc}));
				}
			}
		}
		
		// If document is marked as invariant then don't show translation language flags.
		if (!this._settingsMap.get(ContentConstants.UpdateField.INVARIANT)) {
			var addTransPrefix = true;
			
			// DIV should be created only if translation flags are going to be displayed.
			var translationLangDiv = undefined; 
				
			var projTransLangs = this._settingsMap.get(ContentConstants.ContentField.PROJ_TRANS_LANGS);
			var docTransLangCodes = this._settingsMap.get(ContentConstants.ContentField.DOC_TRANS_LANGS);
			for(var i = 0; i < projTransLangs.length; i++){
				for(var j = 0; j < docTransLangCodes.length; j++){
					if(projTransLangs[i].langCode == docTransLangCodes[j]){
						// Add translation prefix only when translation flags are available to show.
						if (addTransPrefix) {
							// Create language DIV and append to header. 
							translationLangDiv = $('<div/>', {'class':'translationTableContentHeaderLang translation-flags' });
							this._headerDiv.append(translationLangDiv);
							
							// Prefix for the translation language flags.
							translationLangDiv.append(i18n.t("TRANSLATION_LANGUAGE_PREFIX"));
							addTransPrefix = false;
						}
						
						// Create flag and append to translation DIV to show.
						translationLangDiv.append($('<img>', {"class" : "Flag"}).attr({"src": "images/country_flags/" + projTransLangs[i].langFlag, "title": projTransLangs[i].langDesc}));
					}
				}
			}
		}
		
		// Create an info link.
		var infoDiv = $('<div/>', {'class':'tableContentHeaderInfo' });
		var infoLink = $('<a/>', {
			'class':'inline_help-nopadding', 
			'href': 'help.jsp?topic=common/wc/task/ta_wc_configuring_a_nutrition_facts_table&lang=en',
			'onclick' : "var pop = window.open('help.jsp?topic=common/wc/task/ta_wc_configuring_a_nutrition_facts_table&lang=en', 'help', 'scrollbars=1, width=800 ,height=500, toolbar=yes, location=yes, menu=yes'); pop.focus(); return false;"});
		var infoSymbol = $("<i>", {
			'class' : 'icon icon-WCR_Info-Balloon icon-normal',
			'title': 'Help'
		});
		infoDiv.append(infoLink.append(infoSymbol));
		this._headerDiv.append(infoDiv);
		
		this._stateFullContentWidget.setHeaderContent(this._headerDiv);
		
		this._headerDiv.css({"cursor":"pointer"});
		this._headerDiv.on("click", function(event){
			 if(this._stateFullContentWidget.isExpanded()){
				 this._stateFullContentWidget.collapse();
			 }else{
				 this._stateFullContentWidget.expand();
			 }
			 
			// Stop triggering event for its parent.
			event.stopPropagation();
		 }.bind(this));		 
	};
	
	/**
	 * Constructs main body content of the widget
	 */
	TableContentSettingsView.prototype.constructMainContent = function(){
		// Create a Container to hold the properties table content setting
		var contentForm= $("<div>").addClass("form-horizontal");
		
		//Create approval setup field.
		var approvalType = this._createFieldApprovalType();
		contentForm.append(approvalType);

		// Create text type field.
		var textType = this._createFieldTextType();
		contentForm.append(textType);

		// Create invariant flag field.
		var invariantField = this._createFieldInvariant(i18n.t("PCM_TABLE_MAKE_INVARIANT"));
		contentForm.append(invariantField);

		// Create master language flags field.
		var masterLangField = this._createMasterLanguagesList();
		contentForm.append(masterLangField);
		
		// Create translation language flags field.
		var translationLangField = this._createTranslationLanguagesList();
		contentForm.append(translationLangField);
		
		this._wrapper.append(contentForm);
	};

	
	return TableContentSettingsView;;
});