define(["jquery",
        "utils/webi18n!TextContentStrings",
        "component/tooltip/Tooltip",
        "utils/Promisify",
        "utils/webcss!lib/summernote/summernote.css",
        "utils/webcss!./RichTextEditor.css"],
	function ($,
			TextContentStrings,
			Tooltip,
			Promisify) {
	
	
	var RichTextEditor = function (content, height) {
		
		// The array holding all the "onChange" callbacks.
		this._onChange = [];
		this._onFocus = [];
		this._onBlur = [];
		
		// Save the height.
		this._height = height;
		
		// An actual Rich Text editor is created, already start requiring summernote.
		this._summerNotePromise = Promisify.makeQueryable(Promisify.require("lib/summernote/summernote"));
		
		// The wrapping div of the editor.
		this._wrapper = $("<div>");
		
		this._editorWrapper = $("<div>");
		this._editorWrapper.addClass("RichTextEditor");
		this._wrapper.append(this._editorWrapper);

		// Return whether the editor is still in "lazy" mode, e.g. whether a real summernote editor was constructed.
		this._isLazy = true;
		
		// Whether we're disabled or not.
		this._disabled = false;
		
		// This will be showed until the user clicks on the editor.
		this._lazyPlaceholder =  $("<div>", {"contenteditable" : "true", "class" : "RichTextEditorPlaceholder"});
		this._lazyPlaceholder.css({
			"height: ": (this._height-16)+"px"
		});
		
		// Parse the content.
		var parsedContent = undefined;
		try {
			parsedContent = $(content.content);
		} catch (exception) {
			// This is not valid html ... ugh ... just insert in the editor and hope for the best.
			this._lazyPlaceholder.html(content.content);
		}
		// Check if the content could be parsed.
		if (parsedContent && parsedContent.length == 0) {
			// Parsed content could not be parsed, just dump it in the editor and hope for the best.
			this._lazyPlaceholder.html(content.content);
		}
		// Check if the root element of content is "<body>" (we can't insert that in the editor).
		else if (parsedContent && parsedContent.length == 1 && parsedContent.is("body")) {
			// We need to insert all of the parsed content except the body tag itself.
			this._lazyPlaceholder.html(parsedContent.html());
		} 
		// If we get here, then we have perfectly fine rich text. 
		else {
			this._lazyPlaceholder.html(content.content);
		}
		
		// Add the lazyPlaceholder
		this._editorWrapper.append(this._lazyPlaceholder);
		
		// As soon as the lazyPlaceholder on hover, make the widget not lazy anymore.
		this._lazyPlaceholder.hover(function () {
			this._makeNotLazy();
		}.bind(this));
		
		// As soon as the lazyPlaceholder gains focus, make the widget not lazy anymore (to be safe).
		this._lazyPlaceholder.focus(function () {
			this._makeNotLazy();
			// Call the callbacks (to be sure)
			$.each(this._onFocus, function (i, callback) {
				callback();
			});
		}.bind(this));
		
		// As soon as the lazyPlaceholder has click, make the widget not lazy anymore (to be safe).
		this._lazyPlaceholder.click(function () {
			this._makeNotLazy();
			// Call the callbacks (to be sure)
			$.each(this._onFocus, function (i, callback) {
				callback();
			});
		}.bind(this));
	};
	
	/**
	 * Make the widget no longer "lazy", e.g. create a real summernote editor that can be used to change the content.
	 * 
	 * @param cursorRange - The position of the cursor in the real editor.
	 */
	RichTextEditor.prototype._makeNotLazy = function (cursorRange) {
		
		// Only do this if we're still lazy and not disabled.
		if (this._isLazy && !this._disabled) {
			
			// Check whether summernote is loaded first ...
			if (this._summerNotePromise.isResolved()) {
				
				// Create the div that will be hidden and function as the handle for summernote.
				this._handle = $("<div>");
						
				// Transfer the contents from the placeholder to the editor.
				this._handle.html(this._lazyPlaceholder.html());
				
				// Remove the placeholder from the wrapper and add the handle.
				this._editorWrapper.empty();
				this._editorWrapper.append(this._handle);
				
				// Create the editor.
				this._handle.summernote({
					airMode: true,
					disableDragAndDrop: true,
					popover: {
						air: [['style', ['bold', 'italic', 'underline', 'clear']],
						      ['font', ['strikethrough', 'superscript', 'subscript']]]
					},
					keyMap: {
						  pc: {
						    'ENTER': 'insertParagraph',
						    'CTRL+Z': 'undo',
						    'CTRL+Y': 'redo',
						    'TAB': 'tab',
						    'SHIFT+TAB': 'untab',
						    'CTRL+B': 'bold',
						    'CTRL+I': 'italic',
						    'CTRL+U': 'underline',
						    'CTRL+SHIFT+S': 'strikethrough',
						    'CTRL+BACKSLASH': 'removeFormat'
						  },
						  mac: {
						    'ENTER': 'insertParagraph',
						    'CMD+Z': 'undo',
						    'CMD+SHIFT+Z': 'redo',
						    'TAB': 'tab',
						    'SHIFT+TAB': 'untab',
						    'CMD+B': 'bold',
						    'CMD+I': 'italic',
						    'CMD+U': 'underline',
						    'CMD+SHIFT+S': 'strikethrough',
						    'CMD+BACKSLASH': 'removeFormat',
						    'CMD+SHIFT+NUM7': 'insertUnorderedList',
						    'CMD+SHIFT+NUM8': 'insertOrderedList',
						    'CMD+LEFTBRACKET': 'outdent',
						    'CMD+RIGHTBRACKET': 'indent'
						  }
					},
					callbacks: {
						onPaste: this._handlePaste.bind(this),
						onFocus: function () {
							$.each(this._onFocus, function (i, callback) {
								callback();
							});
						}.bind(this),
						onBlur: function () {
							$.each(this._onBlur, function (i, callback) {
								callback();
							});
						}.bind(this)
					}
				});
				this._handle.summernote("setStyle", "word-wrap", "break-word");
				
				// On click of the wrapper.
				this._editorWrapper.on("mousedown", function (e) {
					// If the wrapper itself is clicked ... 
					if (this._editorWrapper[0] == e.target) {
						// Check if the editor has focus ...
						if (!this.hasFocus()) {
							// The editor has no focus ... give it focus.
							this._handle.summernote("focus");
						}
						// Prevent the real "click" from happening as this will cause the editor to lose focus.
						e.preventDefault();
					}
				}.bind(this));
				
				// Execute change call-backs when content is changed in editor.
				this._handle.on('summernote.change', function () {
					$.each(this._onChange, function (index, changeCallback) {
						changeCallback();
					}.bind(this));
				}.bind(this));
				
				// We're not lazy anymore.
				this._isLazy = false;
				
				// Clean out the rich text html.
				this.cleanRichText();
				
			}
			// Summernote is not loaded yet ... postpone the not lazy making.
			else {
				// Make this editor not lazy as soon as summernote is loaded.
				this._summerNotePromise.then(this._makeNotLazy.bind(this));
			}
		}
	};
	
	/**
	 * Filter the god damn frigging paste event.
	 */
	RichTextEditor.prototype._handlePaste = function(e) {
		// Defer the handling of this event.
		window.setTimeout(function (e) {
			this.cleanRichText();
		}.bind(this, e), 0);
	};

	/**
	 * Clean out the HTML of the rich text. 
	 * (Has only effect when not lazy)
	 */
	RichTextEditor.prototype.cleanRichText = function() {
		if (!this._isLazy) {
			var allowedTags = ["p", "b", "i", "u", "sup", "br", "sub", "strike", "s"];
			var allowedEmptyTags = ["br"];
			var allowedAttributes = [];
			var changed = this._handle.summernote('cleanEditable', allowedTags, allowedEmptyTags, allowedAttributes);
			if (changed) {
				$.each(this._onChange, function (index, changeCallback) {
					changeCallback();
				}.bind(this));
			}
			
			// Update max character warning with cleaned content.
			this.setMaxChar(this._setMaxChar);
		}
	};
	
	/**
	 * Set the height.
	 */
	RichTextEditor.prototype.setHeight = function (height) {
		if (this._isLazy) {
			this._lazyPlaceholder.height((height-16)+"px");
		}
		this._height = height;
	};
	
	/**
	 * Disable editing.
	 */
	RichTextEditor.prototype.disable = function () {
		this._disabled = true;
		this._editorWrapper.addClass("Disabled");
		if (this._isLazy) {
			this._lazyPlaceholder.attr("contenteditable", "false");
		} else {
			this._editorWrapper.find('.note-editable').attr('contenteditable', false);
		}
	};
	
	/**
	 * Enable editing.
	 */
	RichTextEditor.prototype.enable = function () {
		this._disabled = false;
		this._editorWrapper.removeClass("Disabled");
		if (this._isLazy) {
			this._lazyPlaceholder.attr("contenteditable", "true");
		} else {
			this._editorWrapper.find('.note-editable').attr('contenteditable', true);
		}
	};

	
	/**
	 * Focus the rich text editor.
	 */
	RichTextEditor.prototype.focus = function () {
		this._makeNotLazy();
		if (!this._isLazy) {
			this._handle.summernote('focus');
			this._handle.summernote('restoreRange');
		}
	};
	
	/**
	 * Blur the editor.
	 */
	RichTextEditor.prototype.blur = function () {
		if (!this._isLazy) {
			this._handle.summernote('saveRange');
			this._handle.summernote('blur');
		}
		// Also clear the current selection.
		if (window.getSelection()) {
			window.getSelection().removeAllRanges();
		} else if (document.selection) {
			document.selection.empty();
		}
	};
	
	/**
	 * Return whether the editor has focus.
	 * 
	 * @returns {Boolean}
	 */
	RichTextEditor.prototype.hasFocus = function () {
		// Return whether the focussed element is inside this "editor wrapper".
		return $.contains(this._editorWrapper[0], document.activeElement);
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.getContent = function () {
		var newContent;
		if (this._isLazy) {
			newContent = this._lazyPlaceholder[0].innerHTML;
		} else {
			newContent = this._handle.summernote('code');
		}
		
		var parsedNewContent = $.parseHTML(newContent);
		// Check if newCotent has one single root tag.
		if (parsedNewContent && parsedNewContent.length > 1) {
			// Create a "body" root tag to contain all the elements.
			var root = $("<body>");
			root.append(parsedNewContent);
			return {
				content: root[0].outerHTML
			};
		} else {
			return {
				content: newContent
			};
		}
	};
	
	/**
	 * Return whether the RichTextEditor is currently displaying a scrollbar or not.
	 */
	RichTextEditor.prototype.hasVerticalScroll = function () {
		if (this._editorWrapper[0].scrollHeight > this._editorWrapper[0].clientHeight) {
			return true;
		} else {
			return false;
		}
	};
	
	/**
	 * Update the content of the editor.
	 * 
	 * @param textData
	 */
	RichTextEditor.prototype.setContent = function (textData) {
		if (this._isLazy) {
			this._lazyPlaceholder.html(textData.content);
		} else {
			this._handle.summernote('code', textData.content);
			this.cleanRichText();
		};
	};
	
	/**
	 * This is going to be called when the editor is shown in the page.
	 */
	RichTextEditor.prototype.shownInPage = function () {
		if (this._setMaxChar) {
			this.setMaxChar(this._setMaxChar);
		}
	};
	
	/**
	 * Set the maximum number of chars
	 * 
	 * @param {Number} - maxChar
	 * @param {Boolean} - isSingleLineMode
	 */
	RichTextEditor.prototype.setMaxChar = function (maxChar) {
		this._setMaxChar = maxChar;
		var actualText = $('<div>').html(this.getContent().content).text();
		var maxCharInt = parseInt(this._setMaxChar);

		if(! this._maxCharDiv){
			this._maxCharDiv = $('<div>', {"class" : "MaxCharIndicator"});
			this._wrapper.append(this._maxCharDiv);
		}

		if (parseInt(this._setMaxChar) < actualText.length) {

			if(!this._maxCharWarningIcon){
				this._maxCharWarningIcon = $("<i>", {"class": "icon icon-normal icon-WCR_Msg-Warning icon-orange", "name": "maxcharwarning"});
				this._maxCharDiv.append(this._maxCharWarningIcon);		
			}
			
			var msg = TextContentStrings.t("PCM_MAX_CHARS")+": " + actualText.length+"/"+maxCharInt;
			if (!this._maxCharWarningTT) {
				this._maxCharWarningTT = new Tooltip(undefined, this._maxCharWarningIcon, $("body"), $("body"), false, msg, undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
			} else {
				this._maxCharWarningTT.setTooltipContent(msg);
			}

			//Avoid overlap if scroll bar is displayed
			if(this.hasVerticalScroll()){
				this._maxCharDiv.css({"right": "25px"});
			} else {
				this._maxCharDiv.css({"right": "5px"});
			}

			this._maxCharWarningIcon.show();
		} else {
			//Hide the max warning icon if is shown already
			if(this._maxCharWarningIcon)
				this._maxCharWarningIcon.hide();
		}
	};
	
	/**
	 * Register an "onFocus" callback.
	 * 
	 * @param callback
	 */
	RichTextEditor.prototype.onFocus = function (callback) {
		this._onFocus.push(callback);
	};
	
	/**
	 * Register an "onBlur" callback.
	 * 
	 * @param callback
	 */
	RichTextEditor.prototype.onBlur = function (callback) {
		this._onBlur.push(callback);
	};
	
	/**
	 * Register an "onChange" callback.
	 * 
	 * @param callback
	 */
	RichTextEditor.prototype.onChange = function (callback) {
		this._onChange.push(callback);
	};
	
	/**
	 * Return the wrapping jQuery element.
	 * @returns
	 */
	RichTextEditor.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return RichTextEditor;
});
