define(["jquery",
        "packaging-content-management/ContentConstants",
        "component/smartnames/smartfield/smartnamedictionaryservice",
        "utils/Promisify"], function ($, ContentConstants, SmartNameDictionaryService, Promisify) {
	
	var SmartNameEditor = function (textData) {
		
		this._onFocus = [];
		this._onBlur = [];
		this._onChange = [];
		
		// Variable keeping track of whether the editor is disabled.
		this._isDisabled = false;
		
		// Variable keeping track of whether the editor has focus.
		this._hasFocus = false;
		
		// Variable keeping track of whether the cursor is hovering the editor.
		this._hasHover = false;
			
		// Variable keeping track of whether the smart name popover panel is shown.
		this._isSmartNamePopoverShown = false;
		
		// Variable keeping track of whether the smart name selector popup is shown.
		this._isSmartNameSelectorShown = false;
		
		// Remember the status/smartname/resolved of the text data.
		this._status = textData.status;
		this._smartNameString = textData.smartName;
		this._resolved = textData.content;
		
		// Indicate whether we're showing the smartname or resolved content.
		this._smartNameMode = true;
		
		// The wrapper.
		this._wrapper = $("<div>", {"class" : "SmartNameEditor"});
		
		
		// When an editor is really created, load the smart field.
		var promise = Promisify.require("component/smartnames/smartfield/smartfield").then(function (SmartField) {
			
			// Create the smart field
			var input = $("<input>", {"type" : "text"});
			this._smartField = new SmartField(input, false);
			this._smartField.setWidth("100%");
			
			// Add the smart field
			this._wrapper.append(this._smartField.getJQElement());
			
			// Set the correct content
			if (textData.smartName) {
				this._smartField.setValue(textData.smartName);
			}
			
			// On Focus ...
			this._smartField.onFocus(function () {
				if (this._smartField.isDisabled()) {
					return;
				}
				// Set the variable.
				this._hasFocus = true;
				// Execute the registered callbacks.
				for (var i=0; i < this._onFocus.length; i++) {
					this._onFocus[i]();
				}
				// Update the editor
				this.update();
			}.bind(this));
			
			// On Blur ...
			this._smartField.onBlur(function () {
				// Set the variable.
				this._hasFocus = false;
				// Execute the registered callbacks.
				for (var i=0; i < this._onBlur.length; i++) {
					this._onBlur[i]();
				}
				// Update the editor
				this.update();
			}.bind(this));
			
			// On Change ...
			this._smartField.onValueChange(function (value) {
				if (!this._smartNameMode) {
					// This should never happen.
					return; // Ignore changes when not in smartname mode.
				}
				
				if (value == this._smartNameString) {
					// There is no change happen in smart name string.
					return;
				}
				// Save the smartnamed value internally.
				this._smartNameString = value;
				this._resolved = "";
				this._status = ContentConstants.ResolvedStatus.NOT_RESOLVED;
				this._smartField.setSelectorIcon("normal");
				
				// Execute the registered callbacks.
				for (var i=0; i < this._onChange.length; i++) {
					this._onChange[i]();
				}
			}.bind(this));
			
			// When the smart name button is pressed and the smart name dictionary selector opens ...
			this._smartField.onSmartNameSelectorOpened(function () {
				this._isSmartNameSelectorShown = true;
			}.bind(this));
			
			// When the smart name dictionary selector is closed ...
			this._smartField.onSmartNameSelectorClosed(function () {
				this._isSmartNameSelectorShown = false;
			}.bind(this));
			
			// On Smart Name Popover Shown ...
			this._smartField.onSmartNamePopoverShow(function () {
				this._isSmartNamePopoverShown = true;
			}.bind(this));
			
			// On Smart Name Popover Hide ...
			this._smartField.onSmartNamePopoverHide(function () {
				this._isSmartNamePopoverShown = false;
				this.update();
			}.bind(this));
			
			// Update the smartfield.
			this.update();
			
		}.bind(this));
		
		// Keep track of the loading of the smartfield in this promise object.
		// We make the promise queryable so we can "query" whether the promise is resolved or not.
		this._smartFieldPromise = Promisify.makeQueryable(promise);
		
		// On mouse enter, the editor should show the smartname value.
		this._wrapper.on("mouseenter", function () {
			this._hasHover = true;
			// Update the editor
			this.update();
		}.bind(this));
		
		// On mouse out, the editor should show the resolved value.
		this._wrapper.on("mouseleave", function () {
			// Set the variable.
			this._hasHover = false;
			// Update the editor
			this.update();
		}.bind(this));
	};
	
	/**
	 * Show the smartnamed content in the editor.
	 */
	SmartNameEditor.prototype.update = function () {
	
		// Check whether the smart field was already fetched and created.
		if (this._smartFieldPromise.isResolved()) {
			
			// Check whether the value of the text data is resolved or not ...
			var hasResolvedValue = this._status == ContentConstants.ResolvedStatus.RESOLVED;
			
			// When the input has hover or focus or does not  have a resolved value, switch to smartname mode.
			if (this._hasHover || this._hasFocus || !hasResolvedValue || this._isSmartNamePopoverShown || this._isSmartNameSelectorShown) {
				this._switchToSmartName();
			} else {
				this._switchToResolved();
			}
			
		} else {
			// Only update once the smart field is loaded and created
			this._smartFieldPromise.then(this.update.bind(this));
		};
		
	};
	
	/**
	 * Show the smartnamed content in the editor.
	 */
	SmartNameEditor.prototype._switchToSmartName = function () {
		
		if (this._status == ContentConstants.ResolvedStatus.NOT_RESOLVED) {
			this._smartField.setSelectorIcon("normal");
		} else if (this._status == ContentConstants.ResolvedStatus.FAILED_RESOLVE) {
			this._smartField.setSelectorIcon("warning");
		} else if (this._status == ContentConstants.ResolvedStatus.RESOLVED) {
			this._smartField.setSelectorIcon("resolved");
		}
		
		if (this._smartNameMode) {
			return; // Already in smartname mode.
		}
		
		// Check whether we have a smartname value.
		if (!this._smartNameString) {
			this._smartNameString = "";
		}
		this._smartNameMode = true;
		this._smartField.setValue(this._smartNameString);
		if (!this._isDisabled) {
			this._smartField.enable();
		}
		
		this._wrapper.addClass("SmartNameMode");
	};
	
	SmartNameEditor.prototype._switchToResolved = function () {
		
		if (!this._smartNameMode) {
			return; // Already in resolved mode.
		}
		
		// Check whether we have a resolved value.
		if (!this._resolved) {
			this._resolved = "";
		}
		this._smartNameMode = false;
		this._smartField.blur();
		this._smartField.setValue(this._resolved);
		this._smartField.disable();
		this._smartField.setSelectorIcon("resolved");
		this._wrapper.addClass("ResolvedMode");
	};
	
	SmartNameEditor.prototype.focus = function () {
		// Check whether the smart field was already fetched and created.
		if (this._smartFieldPromise.isResolved()) {
			// The smart field already exists so just focus on it.
			this._smartField.focus();
		} else {
			// Only focus once the smart field is loaded and created
			this._smartFieldPromise.then(this.focus.bind(this));
		};
	};
	
	SmartNameEditor.prototype.blur = function () {
		// Check whether the smart field was already fetched and created.
		if (this._smartFieldPromise.isResolved()) {
			// The smart field already exists so just blur it.
			this._smartField.blur();
		} else {
			// Only blur once the smart field is loaded and created
			this._smartFieldPromise.then(this.blur.bind(this));
		};
	};
	
	SmartNameEditor.prototype.onFocus = function (callback) {
		this._onFocus.push(callback);
	};
	
	SmartNameEditor.prototype.onBlur= function (callback) {
		this._onBlur.push(callback);
	};
	
	SmartNameEditor.prototype.onChange = function (callback) {
		this._onChange.push(callback);
	};
	
	SmartNameEditor.prototype.enable = function () {
		
		// Set the disabled flag on false.
		this._isDisabled = false;
		
		// Check whether the smart field was already fetched and created.
		if (this._smartFieldPromise.isResolved()) {
			// The smart field already exists so just enable it.
			this._smartField.enable();
		} else {
			// Only enable once the smart field is loaded and created
			this._smartFieldPromise.then(this.enable.bind(this));
		};
	};
	
	SmartNameEditor.prototype.disable= function () {
		
		// Set the disabled flag on true.
		this._isDisabled = true;
		
		// Check whether the smart field was already fetched and created.
		if (this._smartFieldPromise.isResolved()) {
			// The smart field already exists so just disable it.
			this._smartField.disable();
		} else {
			// Only disable once the smart field is loaded and created
			this._smartFieldPromise.then(this.disable.bind(this));
		};
	};
	
	SmartNameEditor.prototype.setHeight = function (height) {
		// Don't do anything.
	};
	
	SmartNameEditor.prototype.hasVerticalScroll = function () {
		return false;
	};
	
	SmartNameEditor.prototype.setContent = function (textData) {
		// Remember the status/smartname/resolved of the text data.
		this._status = textData.status;
		this._smartNameString = textData.smartName;
		this._resolved = textData.content;
		// Update the widget.
		this.update();
	};
	
	SmartNameEditor.prototype.getContent = function () {
		return {
			status: this._status,
			smartName: this._smartNameString,
			content: this._resolved
		};
	};
	
	SmartNameEditor.prototype.setMaxChar = function (maxChar) {
		//Not yet supported
	};
	
	/**
	 * This is going to be called when the editor is shown in the page.
	 */
	SmartNameEditor.prototype.shownInPage = function () {
		//Not yet supported
	};

	SmartNameEditor.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return SmartNameEditor;
});