define(["underscore",
        "jquery",
        "./TextContentElementWidgetView",
        "./TextContentElementWidgetDocXView",
        "./TextContentElementWidgetViewerView",
        "./TextContentElementWidgetTableView",
        "../text-content-services/languageService",
        "../text-content-services/elementTypeService",
        "../text-content-utils/LanguageUtils",
        "./ApprovalOverviewWidget",
        "utils/webi18n!TextContentStrings",
        "../ContentConstants",
        "../PCMEventBus"],
	function (_, 
			$, 
			TextContentElementWidgetView,
			TextContentElementWidgetDocXView,
			TextContentElementWidgetViewerView,
			TextContentElementWidgetTableView,
			languageService,
			elementTypeService,
			LanguageUtils,
			ApprovalOverviewWidget,
			TextContentStrings,
			ContentConstants,
			pcmEventBus) {
		
		"use strict";

		/**
		 * Create a Widget in order to display and potentially edit the contents of a 'TextContentElement'.
		 * 
		 * @param {TextContentElement} textContentElement - The textcontent for this widget.
		 * @param {Object} options - An object containing some variables specifying how the widget should behave.
		 * 
		 */
		var TextContentElementWidget = function (textElement, options) {
			
			this._textElement = textElement;
			this._elementTypeCode = textElement.parentStatement.elementType;
			this._textType = textElement.parentStatement.textType;
			this._options = options;
			
			// Extract the fields from the TextContentElementWidget that are needed for the view.
			var content = this._textElement.content;
			var languageCode = this._textElement.language;
			var comments = this._textElement.comment;
			var state = this._textElement.state;
			var newState = this._textElement.newState;
			var isMaster = this._textElement.master;
			var editPermission = this._textElement.canEdit(this._projMasterLanguages);
			var canEdit = editPermission.allowed; 
			var whyCanNotEdit = editPermission.message;
			var hasEdit = this._textElement.hasEdit().allowed;
			var reviewPermission = this._textElement.canReview(this._projMasterLanguages);
			var canReview = reviewPermission.allowed;
			var whyCanNotReview = reviewPermission.message;
			var hasReview = this._textElement.hasReview().allowed;
			var isContentManagerView = this._textElement.parentStatement.canManage().allowed;

			// Extract the 'viewOptions' from the passed options.
			var viewOptions = {};
			if (this._options && typeof this._options.viewOptions === 'object') {
				viewOptions = this._options.viewOptions;
			}
			
			// callbacks to be executed when any changes happen in the widget
			this._onChangeCallbacks = [];
			
			// callbacks to be executed when the view toggles between the normal or single line mode.
			this._onSingleLineModeToggleCallbacks = [];
			
			// create the view of the widget based on the viewOptions
			switch(viewOptions.widgetview) {
			case 'default':
				this._view = new TextContentElementWidgetView(content, isMaster, comments, state, newState, languageCode, 
						canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
						viewOptions, this._textType);
				this._setLanguageHeader(isMaster, isContentManagerView);
				break;
			case 'doctablex':
				this._view = new TextContentElementWidgetDocXView(content, isMaster, comments, state, newState, languageCode, 
						canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
						viewOptions, this._textType);
				break;
			case 'viewer':
				this._view = new TextContentElementWidgetViewerView(content, isMaster, comments, state, newState, languageCode, 
						canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
						viewOptions, this._textType);
				break;
			case 'table':
				this._view = new TextContentElementWidgetTableView(content, isMaster, comments, state, newState, languageCode, 
						canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
						viewOptions, this._textType);
				break;
			default:
				break;
			}

			// Specify what should happen on Approval actions
			this._view.onApprove(_.bind(function () {
				this.approve();
			}, this));
			
			this._view.onReject(_.bind(function () {
				this.reject();
			}, this));
			
			this._view.onStartApproval(_.bind(function () {
				this.startApproval();
			}, this));
			
			this._view.onRevertToDraft(_.bind(function () {
				this.revertToDraft();
			}, this));
			
			// Specify what should happen when the approval action is reverted
			this._view.onUndoApproval(_.bind(function() {
				this.undoApproval();
			}, this));
			
			// Specify what should happen when a comment is added/edited
			this._view.onComment(_.bind(function(commentText) {
				this.comment(commentText);			
			}, this));
			
			// What needs to happen when the approval info button is clicked
			this._view.onApprovalInfo(_.bind(function () {
				// Create a new approval overview widget.
				this._aprovalOverviewWidget = new ApprovalOverviewWidget(this._textElement);
				this._view.showApprovalInfoDialog(this._aprovalOverviewWidget.getJQElement(), this._options);
			}, this));
			
			// What needs to happen when content is changed in editor.
			this._view.onMainContent(_.bind(function (content) {
				this.updateCurrentContent(content);
			}, this));
			
			// When the view toggled between single line or normal mode ... execute these callbacks.
			this._view.onSingleLineModeToggle(_.bind(function(isSingleLineMode) {
				for (var i=0; i < this._onSingleLineModeToggleCallbacks.length; i++) {
					this._onSingleLineModeToggleCallbacks[i](isSingleLineMode);
				}
			}, this));
			
			// Also listen on the PCMEventBus for potential changes of the current textcontent.
			pcmEventBus.on("PCM.TextContentElementWidget.update", function(textContent){
				if (this._textElement.parentStatement === textContent) {
					this.update(this._textElement);
				}
			}.bind(this));
			
			pcmEventBus.on("PCM.TextContent.langRemoved", function (textContent, textElement) {
				if (textContent == this._textElement.parentStatement && this._textElement.language === textElement.language) {
					this._textElement = this._textElement.parentStatement.provideElement(textElement.language, textElement.isMaster());
					this.update(this._textElement);
				}
			}.bind(this));
			
			pcmEventBus.on("PCM.TextContent.langAdded", function (textContent, textElement) {
				if (textContent == this._textElement.parentStatement && this._textElement.language === textElement.language) {
					this._textElement = textElement;
					this.update(this._textElement);
				}
			}.bind(this));
			
			// Get master and translation project languages and then only proceed. These languages are static for current view, so it's better to keep them ready in the beginning.
			languageService.getLanguages(textElement.parentStatement.projectId, _.bind(function (projMastLangs, projTransLangs) {
				
				this._projMasterLanguages = projMastLangs;
				this._pronTranslationLanguages = projTransLangs;
			
				this.update(this._textElement);
				
			}, this));
			
		};
		
		/**
		 * Needs to be called when the widget is added to the page.
		 */
		TextContentElementWidget.prototype.shownInPage = function () {
			return this._view.shownInPage();
		};
		
		/**
		 * Try to display this widget in single line mode.
		 */
		TextContentElementWidget.prototype.trySingleLine = function () {
			return this._view.trySingleLine();
		};
		
		/**
		 * Approve this element.
		 */
		TextContentElementWidget.prototype.approve = function () {	
			this._textElement.setNewState(ContentConstants.State.APPROVED);
			this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVED, true);
		};
		
		/**
		 * Reject this element.
		 */
		TextContentElementWidget.prototype.reject = function () {	
			this._textElement.setNewState(ContentConstants.State.REJECTED);
			this._executeOnChange(ContentConstants.OnChangeCallbackType.STARTED_APPROVAL, true);
		};
		
		/**
		 * Start the approval for this element.
		 */
		TextContentElementWidget.prototype.startApproval = function () {	
			this._textElement.setNewState(ContentConstants.State.READY_FOR_APPROVAL);
			this._executeOnChange(ContentConstants.OnChangeCallbackType.STARTED_APPROVAL, true);
		};
		
		/**
		 * Revert the element back to draft.
		 */
		TextContentElementWidget.prototype.revertToDraft = function () {	
			this._textElement.setNewState(ContentConstants.State.DRAFT);
			this._executeOnChange(ContentConstants.OnChangeCallbackType.REVERTED_TO_DRAFT, true);
		};
		
		/**
		 * Undo the last approval action of the element.
		 */
		TextContentElementWidget.prototype.undoApproval = function () {	
			// We assume that an undo action will try to undo the last approval action.
			this._textElement.cancelLastApprovalAction();
			this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVAL_REVERTED, true);
			return this._textElement;
		};
		
		/**
		 * Add a comment through the widget.
		 */
		TextContentElementWidget.prototype.comment = function (commentText) {	
			// Action to add/update comment
			this._textElement.addTextContentElementComment(commentText);				
			this._executeOnChange(ContentConstants.OnChangeCallbackType.COMMENTED);
		};
		
		/**
		 * Update the content currently in the widget.
		 */
		TextContentElementWidget.prototype.updateCurrentContent = function (currentContent) {
			
			// Findout whether the content was changed.
			var contentChanged = false;
			if (ContentConstants.TextType.SMARTNAME === this._textType) {
				contentChanged = currentContent.smartName != this._textElement.content.smartName;
			} else {
				contentChanged = currentContent.content != this._textElement.content.content;
			}
			
			// Update should be done only if content is actually changed
			if (contentChanged) {
				var notifyOtherWidgets = this._textElement.setTextData(currentContent);
				this._executeOnChange(ContentConstants.OnChangeCallbackType.TEXT_UPDATED, notifyOtherWidgets);
			}			
		};
		
		/**
		 * Resolve the text data in this widget.
		 */
		TextContentElementWidget.prototype.resolve = function (resolvedText) {
			
			if (resolvedText != null) {
				// Create a new resolved textData object for the element and resolve it.
				this._textElement.resolveSmartNamedTextData({
					smartName : this._textElement.getTextData().smartName,
					content : resolvedText,
					status : ContentConstants.ResolvedStatus.RESOLVED
				});
			} else {
				// Create a new resolved textData object for the element and resolve it.
				this._textElement.resolveSmartNamedTextData({
					smartName : this._textElement.getTextData().smartName,
					content : "",
					status : ContentConstants.ResolvedStatus.FAILED_RESOLVE
				});
			}
			
			
			this.focus(false);
			this._view.updateTextData(this._textElement.getTextData());
			this._executeOnChange(ContentConstants.OnChangeCallbackType.TEXT_UPDATED, false);
		};
		
		/**
		 * Make sure the widget is updated with the latest state of the passed TextContentElement.
		 */
		TextContentElementWidget.prototype.update = function (textContentElement) {
			this._textElement = textContentElement;
			
			// Extract the fields from the TextContentElementWidget that are needed for the view.
			var content = textContentElement.content;
			var languageCode = textContentElement.language;
			var comments = textContentElement.comment;
			var state = textContentElement.state;
			var newState = this._textElement.getNewState();
			var myApprovalState = this._textElement.getMyApprovalStatus(this._projMasterLanguages);
			
			var editPermission = textContentElement.canEdit(this._projMasterLanguages);
			var canEdit = editPermission.allowed; 
			var whyCanNotEdit = editPermission.message;
			var hasEdit = textContentElement.hasEdit().allowed;
			
			var reviewPermission = textContentElement.canReview(this._projMasterLanguages);
			var canReview = reviewPermission.allowed;
			var whyCanNotReview = reviewPermission.message;
			var hasReview = textContentElement.hasReview().allowed;
			
			var isMaster = textContentElement.master;
			var isContentManagerView = textContentElement.parentStatement.canManage().allowed;
			var textType = textContentElement.parentStatement.textType;
			
			this._view.updateWidgetView(content, isMaster, comments, state, newState, myApprovalState, languageCode, canEdit, whyCanNotEdit, hasEdit, 
					canReview, whyCanNotReview, hasReview, isContentManagerView, textType);
		};
		
		/**
		 * Update the language header of the widget
		 * 
		 * @param isMaster
		 * @param isContentManagerView
		 */
		TextContentElementWidget.prototype._setLanguageHeader = function(isMaster, isContentManagerView) {
			// Retrieve the languages for this text element.
			var projId = this._textElement.parentStatement.projectId;
			languageService.getAllLanguages(projId, isMaster, function (allLangs) {
				this._view.setLanguage(LanguageUtils.getLangForCode(allLangs, this._textElement.language));
			}.bind(this));
		};
		
		/**
		 * Execute the onchange callback when anything changes in the widget
		 */
		TextContentElementWidget.prototype._executeOnChange = function (changetype, notifyOtherWidgets) {
			
			if (notifyOtherWidgets) {
				// Also listen on the PCMEventBus for potential changes of the current element.
				pcmEventBus.emit("PCM.TextContentElementWidget.update", this._textElement.parentStatement);
			}
			
			_.each(this._onChangeCallbacks, _.bind(function(changetype, callback) {
				callback(changetype, this._textElement);
			}, this, changetype));
		};
		
		/**
		 * Allow other widgets to add custom buttons to the widget.
		 */
		TextContentElementWidget.prototype.addCustomActionButton = function(button) {
			this._view.addCustomActionButton(button);
		};
		
		
		TextContentElementWidget.prototype.removeCustomActionButton = function(button) {
			this._view.removeCustomActionButton(button);
		};
		
		/**
		 * Add callback to be executed when any changes is done in the widget
		 */
		TextContentElementWidget.prototype.onChange = function (callback) {
			this._onChangeCallbacks.push(callback);
		};
		
		/**
		 * Add callback to be executed when the widget is focused.
		 */
		TextContentElementWidget.prototype.onFocus = function (callback) {
			this._view.onFocus(callback);
		};
		
		/**
		 * Add callback to be executed when the widget is blurred.
		 */
		TextContentElementWidget.prototype.onBlur = function (callback) {
			this._view.onBlur(callback);
		};
	
		/**
		 * Add callback for when the view toggles single line mode or not.
		 */
		TextContentElementWidget.prototype.onSingleLineModeToggle = function (callback) {
			return this._onSingleLineModeToggleCallbacks.push(callback);
		};
		
		/**
		 * Return whether or not the widget is in single line mode.
		 */
		TextContentElementWidget.prototype.isSingleLineMode = function () {
			return this._view.isSingleLineMode();
		};
		
		/**
		 * Set whether or not the widget is in single line mode.
		 */
		TextContentElementWidget.prototype.setSingleLineMode = function (isSingleLineMode) {
			return this._view.setSingleLineMode(isSingleLineMode);
		};
		
		/**
		 * Select/Unselect the widget.
		 */
		TextContentElementWidget.prototype.select = function (selected) {
			this._view.select(selected);
		};
		
		/**
		 * Focus/Blur the widget.
		 */
		TextContentElementWidget.prototype.focus = function (focus) {
			this._view.focus(focus);
		};
		
		/**
		 * Disable the widget.
		 */
		TextContentElementWidget.prototype.disable = function (msgTag) {
			this._view.enable(false, msgTag);
		};
		
		/**
		 * Enable the widget.
		 */
		TextContentElementWidget.prototype.enable = function () {
			this._view.enable(true);
		};
		
		/**
		 * Get the current text element
		 */
		TextContentElementWidget.prototype.getTextContentElement = function () {
			return this._textElement;
		};
		
		/**
		 * Set maximum number of chars for this widget
		 */
		TextContentElementWidget.prototype.setMaxChar = function(maxChar) {
			this._view.setMaxChar(maxChar);
		};
		
		/**
		 * Return the jQuery HTML element of the widget
		 * @returns
		 */
		TextContentElementWidget.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};
		
		// Return the constructor.
		return TextContentElementWidget;
		
});
