define(["underscore",
        "jquery",
        "./TextContentElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			TextContentElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the default TextContentElementWidgetDocXView.
		 * 
		 * @param {Object} text - The content object from the TextContentElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the TextContentElementWidget represented by this widget.
		 * @param {String} state - The state of the TextContentElementWidget represented by this widget.
		 * @param {String} language - The language object of the TextContentElementWidget represented by this widget.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} textType - text type as "richtext" or "simple".
		 */
		var TextContentElementWidgetDocXView = function (text, isMaster, comments, state, newState, languageCode, 
				canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
				viewOptions, textType) {
			
			// This view always supports single line mode.
			viewOptions.supportSingleLineMode = true;
			
			// By default, the height of the widget is 76px.
			viewOptions.defaultContentHeight  = 76;
			
			// call the parent prototype object to inherit properties
			TextContentElementWidgetView.call(this, text, isMaster, comments, state, newState, languageCode, 
					canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
					viewOptions, textType);
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		TextContentElementWidgetDocXView.prototype = Object.create(TextContentElementWidgetView.prototype);
		TextContentElementWidgetDocXView.prototype.constructor = TextContentElementWidgetDocXView;
		TextContentElementWidgetDocXView.prototype.parent = TextContentElementWidgetView.prototype;
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		TextContentElementWidgetDocXView.prototype._constructView = function () {
			this._wrapper.empty();
			this._wrapper.addClass("DocTableXView");
			
			// Init the elements.
			this._initializeDOMElements();
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol TextContent-ContentColDocX").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructTextDiv()
							)
					),
					this._constructRightColDiv(),
					this._constructCommentsDiv()
			);
			
			// Update css for TextContent textarea div
			$("div[name='Content-ContentAreaDiv']", this._wrapper).addClass("Content-ContentRowDocX");
			
			// Update css for comments div
			$(".TextComment", this._wrapper).addClass("TextCommentDocX");
		};
		
		/**
		 * Construct the right column with comment, approval info, mystatus buttons
		 * 
		 * @returns rightPanel - jQuery element for the right side actions column of the widget
		 */
		TextContentElementWidgetDocXView.prototype._constructRightColDiv = function() {
			var rightPanel = $('<div>').addClass("TextContent-RightCol");
			
			// If the element status is empty - don't add comment, approvalinfo, mystatus buttons
			if(!this.isEmptyElement()) {
				this._createCommentButton();
				rightPanel.append(this._commentButton.removeClass().addClass("TextContent-ActionDocX TextContent-ActionBorder"));
				rightPanel.append(this._approvalInfoButton.removeClass().addClass("TextContent-ActionDocX"));
			}
			
			return rightPanel;
		};
		
		/**
		 * Show the comments section
		 */
		TextContentElementWidgetDocXView.prototype._showCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row. 
			// (While not ideal, it's the best place for this at the moment).
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Set the bottom padding to the height of the comments div to make place for it.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : this._commentsDiv.height()+6+"px"
					});	
				}
			}
			
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
		};

		/**
		 * Hide the comments section
		 */
		TextContentElementWidgetDocXView.prototype._hideCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row.
			// When hiding the comment section we need to remove that space again.
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Reset the bottom padding of 6px.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : ""
					});	
				}
			}
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};
		
		
		return TextContentElementWidgetDocXView;	
});