define(["underscore",
        "jquery",
        "./TextContentElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			TextContentElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the default TextContentElementWidgetTableView.
		 * 
		 * @param {Object} text - The content object from the TextContentElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the TextContentElementWidget represented by this widget.
		 * @param {String} state - The state of the TextContentElementWidget represented by this widget.
		 * @param {String} language - The language object of the TextContentElementWidget represented by this widget.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} textType - text type as "richtext" or "simple".
		 */
		var TextContentElementWidgetTableView = function (text, isMaster, comments, state, newState, languageCode, 
				canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
				viewOptions, textType) {
			
			// This view always supports single line mode.
			viewOptions.supportSingleLineMode = true;
			
			// By default, the height of the widget is 76px.
			viewOptions.defaultContentHeight  = 76;
			
			// call the parent prototype object to inherit properties
			TextContentElementWidgetView.call(this, text, isMaster, comments, state, newState, languageCode, 
					canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
					viewOptions, textType);
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
			
			// Add some extra styling on focus and blur.
			this.onFocus(function () {
				this.select(true);
			}.bind(this));
			this.onBlur(function () {
				this.select(false);
			}.bind(this));
		};
		
		TextContentElementWidgetTableView.prototype = Object.create(TextContentElementWidgetView.prototype);
		TextContentElementWidgetTableView.prototype.constructor = TextContentElementWidgetTableView;
		TextContentElementWidgetTableView.prototype.parent = TextContentElementWidgetView.prototype;
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		TextContentElementWidgetTableView.prototype._constructView = function () {
			
			// Detach custom buttons so that they keep their bindings.
			$.each(this._customActionButtons, function (index, button) {
				button.detach();
			});
			
			// Empty the wrapper.
			this._wrapper.empty();
			this._wrapper.addClass("TableView");
			
			// Init the elements.
			this._initializeDOMElements();
			
			// Create widget elements and append to the wrapper
			this._approvalDiv = this._constructApprovalDiv();
			this._textDiv = this._constructTextDiv();
			this._contentColDiv = $('<div>').addClass("TextContent-ContentCol");
			this._customActionDiv = this._constructCustomActionDiv();
			this._wrapper.append(
				this._approvalDiv, this._contentColDiv.append(
						$('<div>').addClass("TextContent-InnerContentCol").append(this._textDiv)
				),
				this._customActionDiv
			);
			
			// Figure out the style for the actions div.
			if (this._customActionButtons.length > 0) {
				this._contentColDiv.css("right", "33px");
				this._customActionDiv.show();
			} else {
				this._contentColDiv.css("right", "0px");
				this._customActionDiv.hide();
			}
			
			// Update css for TextContent textarea div
			$("div[name='Content-ContentAreaDiv']", this._wrapper).addClass("Content-ContentRowDocX");
			
			// Update css for comments div
			$(".TextComment", this._wrapper).addClass("TextCommentDocX");
		};
		
		/**
		 * Add a custom action button a part for the view that will hold custom actions.
		 */
		TextContentElementWidgetTableView.prototype.addCustomActionButton = function(button) {
			this._customActionButtons =[button];
			this._contentColDiv.css("right", "33px");
			this._customActionDiv.empty();
			this._customActionDiv.append(button);
			this._customActionDiv.show();
		};
		
		/**
		 * Removes all a custom action buttons.
		 */
		TextContentElementWidgetTableView.prototype.removeCustomActionButton = function() {			
			//Remove the button from the array of buttons
			this._customActionButtons.length = 0;
			this._contentColDiv.css("right", "0px");
			this._customActionDiv.hide();
			this._customActionDiv.empty();			
		};
		
		/**
		 * Create a part for the view that will hold custom actions.
		 */
		TextContentElementWidgetTableView.prototype._constructCustomActionDiv = function() {
			var customActionWrapper = $('<div>').addClass("TextContent-RightCol");
			for (var i=0; i < this._customActionButtons.length; i++) {
				customActionWrapper.append(this._customActionButtons[i]);
			}
			return customActionWrapper;
		};
		
		return TextContentElementWidgetTableView;	
});