define(["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "utils/ConvertUtils",
        "../ContentConstants",
        "component/modal/Modal",
        "component/tooltip/Tooltip",
        "./RichTextEditor",
        "./SimpleEditor",
        "./SmartNameEditor",
        "utils/webcss!./TextContentElementWidgetView.css"],
	function (_,
			$,
			TextContentStrings,
			ConvertUtils,
			ContentConstants,
			Modal,
			Tooltip,
			RichTextEditor,
			SimpleEditor,
			SmartNameEditor) {
	
		"use strict";

		/**
		 * Create the default TextContentElementWidgetView.
		 * 
		 * @param {Object} content - The content object from the TextContentElementWidget represented by this widget.
		 * @param {Boolean} isMaster - Flag which specifies whether the language is a Master or Translation
		 * @param {Array} comments - The array of Comment objects from the TextContentElementWidget represented by this widget.
		 * @param {String} state - The state of the TextContentElementWidget represented by this widget.
		 * @param {String} newState - The potential new state of the TextContentElementWidget represented by this widget.
		 * @param {String} languageCode - The language object of the TextContentElementWidget represented by this widget.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} textType - text type as "richtext" or "simple".
		 */
		var TextContentElementWidgetView = function (content, isMaster, comments, state, newState, languageCode, 
				canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, 
				viewOptions, textType) {
			
			this._content = content;
			if (!this._content.content) {
				this._content.content = "";
			}
			this._isMaster = isMaster;
			this._comments = comments;
			this._state = state;
			this._newState = newState;
			this._langaugeCode = languageCode;
			this._textType = textType;
			this._canEdit = canEdit;
			this._whyCanNotEdit = whyCanNotEdit;
			this._hasEdit = hasEdit;
			this._canReview = canReview;
			this._whyCanNotReview = whyCanNotReview;
			this._hasReview = hasReview;
			this._isContentManagerView = isContentManagerView;
			
			// Let the viewOptions override the default, extend one these default values.
			this._viewOptions = {
				// Some configuration might override this variable to disable the editing capabilities of this widget.
				editable: true,
				// Some configuration might override this variable to disable the review capabilities of this widget.
				reviewable: true,
				// Whether or not the view supports "single line mode".
				supportSingleLineMode: false,
				// The default height of the content.
				defaultContentHeight: 85
			};
			$.extend(this._viewOptions, viewOptions);
			
			// Variable holding wether we're in single line mode or not.
			this._isSingleLineMode = false;
			
			// If the widgets supports single line mode.
			if (this._viewOptions.supportSingleLineMode) {
				// Listen to the window resize to make sure it's still possible to display it in a single line.
				$( window ).resize(_.debounce(function() {
					this.trySingleLine();
				}.bind(this)).bind(this));
			}
			
			// Define the current height.
			this._contentHeight = this._viewOptions.defaultContentHeight;
			
			//max chars
			this._maxChars = this._viewOptions.maxChars;
			
			// Create the callback arrays
			this._onApproveCallbacks = [];
			this._onRejectCallbacks = [];
			this._onStartApprovalCallbacks = [];
			this._onRevertToDraftCallbacks = [];
			this._onCommentCallbacks = [];
			this._onUndoCallbacks = [];
			this._onApprovalInfoCallbacks = [];
			this._onMainContentCallbacks = [];
			this._onFocus = [];
			this._onBlur = [];			
			
			// callbacks to be executed when the view toggles between the normal or single line mode.
			this._onSingleLineModeToggleCallbacks = [];
			
			// Create the common HTML elements for the view.
			this._approveButton = null;
			this._rejectButton = null;
			this._startApprovalButton = null;
			this._revertToDraftButton = null;
			this._editor = null;
			this._commentTextArea = null;
			this._commentButton = null;
			this._approvalInfoButton = null;
			this._undoButton = null;
			this._customActionButtons = [];
			

			// Wether to show popovers on why the user does not have certain rights. 
			this._showTooltips = true;
			
			// Create the wrapping div of this widget.
			this._wrapper = $("<div>").addClass("TextContentElementWidget");
			
			if(this._viewOptions.widgetview === 'default') {
				// Start constructing the default view
				this._constructView();
			} else {
				return;
			}
		};
		
		/**
		 * Needs to be called when the widget is added to the page.
		 */
		TextContentElementWidgetView.prototype.shownInPage = function () {
			
			// Notify the editor that it was added to the page.
			this._editor.shownInPage();
			
			// When this widget is added to the page, it should try displaying itself in single line mode.
			this.trySingleLine();
			
		};
		
		/**
		 * Try to display this widget as a single line.
		 * 
		 * @param content
		 */
		TextContentElementWidgetView.prototype.trySingleLine = function () {
			
			var wasSingleLineMode = this._isSingleLineMode;
			
			// Only allow if this view does support the single line mode.
			if (this._viewOptions.supportSingleLineMode) {
								
				// Set it on single line mode.
				this._setSingleLineMode(true);
				
				// Check if a scroll bar is showing.
				var showingScroll = this._editor.hasVerticalScroll();
				
				// If a scroll bar is showing, do not show in single line mode.
				if (showingScroll) {
					this._setSingleLineMode(false);
				}
			}
			
			// When we finally from single line mode to normal or vice versa ... execute callbacks for this.
			if (this._isSingleLineMode != wasSingleLineMode) {
				for (var i=0; i < this._onSingleLineModeToggleCallbacks.length; i++) {
					// Call the callback and indicate wether we switched to single line mode or not.
					this._onSingleLineModeToggleCallbacks[i](this._isSingleLineMode);
				}
			}
			
			// Return whether or not we succeeded to go to single line mode.
			return this._isSingleLineMode;
		};
		
		/**
		 * Toggle wether we're in single line mode or not.
		 */
		TextContentElementWidgetView.prototype.setSingleLineMode = function (singleLineMode) {
			
			var wasSingleLineMode = this._isSingleLineMode;
			
			this._setSingleLineMode(singleLineMode);
			
			if (this._isSingleLineMode != wasSingleLineMode) {
				for (var i=0; i < this._onSingleLineModeToggleCallbacks.length; i++) {
					// Call the callback and indicate wether we switched to single line mode or not.
					this._onSingleLineModeToggleCallbacks[i](this._isSingleLineMode);
				}
			}
		};
		
		/**
		 * Toggle wether we're in single line mode or not (without potential callback).
		 */
		TextContentElementWidgetView.prototype._setSingleLineMode = function (singleLineMode) {
			// Only allow if this view does support the single line mode.
			if (this._viewOptions.supportSingleLineMode) {
				
				// Do nothing if we're already in single line mode or not.
				if (singleLineMode == this._isSingleLineMode) {
					return;
				}
				
				// Set wether we're in single line mode or not
				if (singleLineMode) {
					this._wrapper.addClass("SingleLineMode");
					this._contentHeight = 32;
					this._isSingleLineMode = true;
				} else {
					this._wrapper.removeClass("SingleLineMode");
					this._contentHeight = 78;
					this._isSingleLineMode = false;
				}
				this._editor.setHeight(this._contentHeight);
			}
		};
		
		/**
		 * Construct the HTML elements of this view.
		 */
		TextContentElementWidgetView.prototype._constructView = function () {
			
			// Detach custom buttons so that they keep their bindings.
			$.each(this._customActionButtons, function (index, button) {
				button.detach();
			});
			
			// Empty the wrapper.
			this._wrapper.empty();
						
			// Init the elements.
			this._initializeDOMElements();
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructHeaderDiv(),
									this._constructTextDiv()
							)
					),
					this._constructCommentsDiv()
			);
		};
		
		/**
		 * Provides a dialog for Default view to show approval info
		 * @param approvalInfoContent {JQuery} - Approval information as jquery elements
		 */
		TextContentElementWidgetView.prototype.showApprovalInfoDialog = function (approvalInfoContent) {
			// Condition check to avoid creating multiple approval info modal
			if (!this._approvalInfoModal) {
				this._approvalInfoModal = new Modal(approvalInfoContent, {"customClass" : "ApprovalOverviewWidgetModal"});

				this._approvalInfoModal.setTitle(TextContentStrings.t("PCM_APPROVAL_INFO_POPUP_TITLE"));
			} else {
				this._approvalInfoModal.setContent(approvalInfoContent);
			}
			this._approvalInfoModal.open();
		};
		
		/**
		 * Method to update the widget and repaint
		 * 
		 * @param content
		 * @param isMaster
		 * @param comments
		 * @param state
		 * @param newState
		 * @param {String} myApprovalState - The state of the TextContentElementWidget represented by this widget for the current logged in user.
		 * @param languageCode
		 * @param canEdit
		 * @param whyCanNotEdit
		 * @param hasEdit
		 * @param canReview
		 * @param whyCanNotReview
		 * @param hasReview
		 * @param isContentManagerView
		 * @param textType
		 */
		TextContentElementWidgetView.prototype.updateWidgetView = function(textData, isMaster, comments, state, newState, myApprovalState, languageCode, canEdit, whyCanNotEdit, hasEdit, canReview, whyCanNotReview, hasReview, isContentManagerView, textType) {
			
			// These booleans will keep track on when we actually need to reconstruct the view.
			// We want to avoid doing this on "false" updates as this operation is expensive (especially creating the editors).
			
			var reconstructView = false;
			var updateTextData = false;
			
			// Update the content and check if something changed.
			updateTextData = (this._content.content != textData.content) || (this._content.smartName != textData.smartName) || (this._content.status != textData.status);
			this._content = textData;
			
			// Update whether this is a master document and check if something changed.
			reconstructView = reconstructView || this._isMaster != isMaster;
			this._isMaster = isMaster;
				
			// Update the comments and check if something changed.
			reconstructView = reconstructView || this._comments != comments;
			this._comments = comments;
			
			// Update the state of the widget and check if something changed.
			reconstructView = reconstructView || this._state != state;
			this._state = state;
			
			// Update the new state of the widget and check if something changed.
			reconstructView = reconstructView || this._newState != newState;
			this._newState = newState;
			
			// Update the language code of the widget and check if something changed.
			reconstructView = reconstructView || this._langaugeCode != languageCode;			
			this._langaugeCode = languageCode;
			
			// Update the can edit permission of the widget and check if something changed.
			reconstructView = reconstructView || this._canEdit != canEdit;			
			this._canEdit = canEdit;
			
			// Update the has edit permission of the widget and check if something changed.
			reconstructView = reconstructView || this._hasEdit != hasEdit;			
			this._hasEdit = hasEdit;
			
			// Update the no edit reason of the widget and check if something changed.
			reconstructView = reconstructView || this._whyCanNotEdit != whyCanNotEdit;			
			this._whyCanNotEdit = whyCanNotEdit;
			
			// Update the can review permission of the widget and check if something changed.
			reconstructView = reconstructView || this._canReview != canReview;			
			this._canReview = canReview;
			
			// Update the has review permission of the widget and check if something changed.
			reconstructView = reconstructView || this._hasReview != hasReview;			
			this._hasReview = hasReview;
			
			// Update the no review reason of the widget and check if something changed.
			reconstructView = reconstructView || this._whyCanNotReview != whyCanNotReview;			
			this._whyCanNotReview = whyCanNotReview;

			// Update the content manager permission of the widget and check if something changed.
			reconstructView = reconstructView || this._isContentManagerView != isContentManagerView;			
			this._isContentManagerView = isContentManagerView;

			// Update the my approval status of the widget and check if something changed.
			reconstructView = reconstructView || this._myApprovalStatus != myApprovalState;			
			this._myApprovalStatus = myApprovalState;
			
			// Update the text data type of the widget and check if something changed.
			reconstructView = reconstructView || this._textType != textType;			
			this._textType = textType;
			
			// If something critical was actually changed ...
			if (reconstructView) {
				// Construct the view again.
				this._constructView();
			}
			
			// If just the content was updated ...
			if (updateTextData) {
				// Just update the content of the editor.
				this._editor.setContent(this._content);
			}
			
		};
		
		/**
		 * Just update the text data of the widget.
		 */
		TextContentElementWidgetView.prototype.updateTextData = function(textData) {
			this._content = textData;
			this._editor.setContent(this._content);
		};
		
		
		/**
		 * Initialize the DOM elements to repaint the widget
		 */
		TextContentElementWidgetView.prototype._initializeDOMElements = function() {
			
			// Create StartApproval button
			this._startApprovalButton = $('<div>').attr({"name": "startApprovalButton"}).addClass("TextContent-ApprovalAction StartTextApproval").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Start").addClass("StartTextApprovalIconAndLabel")))
			);
			
			// Create RevertToDraft button
			this._revertToDraftButton = $('<div>').attr({"name": "revertToDraftButton"}).addClass("TextContent-ApprovalAction RevertTextToDraft").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Draft").addClass("RevertTextToDraftIconAndLabel")))
			);
			
			// Create Approve button
			this._approveButton = $('<div>').attr({"name": "approveButton"}).addClass("TextContent-ApprovalAction ApproveText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Approve").addClass("ApproveTextIconAndLabel")))
			);
			
			// Create Reject button
			this._rejectButton = $('<div>').attr({"name": "rejectButton"}).addClass("TextContent-ApprovalAction RejectText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Reject").addClass("RejectTextIconAndLabel")))
			);
			
			// Undo your latest approval
			this._undoButton = $('<div>').attr({"name": "revertButton"}).addClass("RevertApprovalAction TextContent-ApprovalAction TextContent-1ApprovalAction").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol ActionIcon-1ApprovalAction").append($('<i>').addClass("icon icon-WCR_Revert").addClass("RevertApprovalActionIconAndLabel")))
			);
			
			// Bind events to execute call backs for each action
			
			// Define the callback that should be executed every time a state button get's clicked.
			var onStateClicked = function (state, callbacks) {
				
				// If we're in single line mode, it could be that the editor is moved in order to not get overlapped
				// with the next state options.
				if (this._isSingleLineMode) {
					var editorElement = this._editor.getJQElement();
					editorElement.finish();
					editorElement.animate({marginLeft: '0px'}, 500, 'swing');
				}
				
				// Update the approval column.
				this._deactivateApprovalActions();
				window.setTimeout(function () {
					this._activateApprovalActions();
				}.bind(this), 200);

				// Update the header
				this.updateHeader();
				
				// Run all the necessary call backs
				_.each(callbacks, _.bind(function(callback) {
					callback();
				}, this));
				
			}.bind(this);
			
			// What happens when the approval button is pressed?
			this._approveButton.click(function () {
				onStateClicked(ContentConstants.State.APPROVED, this._onApproveCallbacks);
			}.bind(this));
			
			// What happens when the reject button is pressed?
			this._rejectButton.click(function () {
				onStateClicked(ContentConstants.State.REJECTED, this._onRejectCallbacks);
			}.bind(this));
			
			// What happens when the start approval button is pressed?
			this._startApprovalButton.click(function () {
				onStateClicked(ContentConstants.State.READY_FOR_APPROVAL, this._onStartApprovalCallbacks);
			}.bind(this));
			
			// What happens when the revert to draft button is pressed?
			this._revertToDraftButton.click(function () {
				// Check whether we are reverting from an approved statement ....
				var proceed = true;
				if(this._state === ContentConstants.State.APPROVED){
					var confirmStr = this._isMaster ? TextContentStrings.t("PCM_MASTER_STATE_CHANGED_FROM_SIGNEDOFF_TO_DRAFT") : TextContentStrings.t("PCM_SIGNEDOFF_TO_DRAFT");
					proceed = confirm(confirmStr);
				}
				// Only continue if the user allows to proceed.
				if (proceed) {
					onStateClicked(ContentConstants.State.DRAFT, this._onRevertToDraftCallbacks);
				}
			}.bind(this));
			
			// What happens when somebody clicked undo ...
			this._undoButton.click(function () {
				
				// First run the "undo" call backs.
				_.each(this._onUndoCallbacks, _.bind(function(callback) {
					callback();
				}, this));
				
				// Then the widget should be updated from the controller (e.g. TextContentElementWidgetView will call the update method after it undid the last change).
			}.bind(this));
			
			// Create the comment button
			this._commentButton = $('<div>').attr({"name": "commentButton"}).attr({"title": TextContentStrings.t("PCM_COMMENT")}).addClass("ToggleComment TextContent-Action");
			// On click etc. will be set later as it depends on permissions and whether there are comments.
			
			// Create the comment area
			this._commentTextArea = $('<textarea>').attr({"name": "addCommentTextArea"});
			this._commentTextArea.on("change keyup", _.bind(function(event) {
				var commentText = $.trim($(event.target).val());
				_.each(this._onCommentCallbacks, _.bind(function(commentText, callback) {
					callback(commentText);
				}, this, commentText));
			}, this));
			
			// Create the approval info button
			this._approvalInfoButton = $('<div>').attr({"name": "approvalInfoButton"}).addClass("TextContent-Action").attr({"title": TextContentStrings.t("PCM_APPROVAL_INFO")});
			this._approvalInfoButton.append($('<i>').addClass("icon icon-WCR_Info"));
			
			// What needs to happen when the approval info button is clicked.
			this._approvalInfoButton.on("click", function (){
				// Execute all the call backs ...
				for (var i=0; i < this._onApprovalInfoCallbacks.length; i++) {
					this._onApprovalInfoCallbacks[i]();
				}
			}.bind(this));
			
			// Create the editor.
			if (ContentConstants.TextType.RICHTEXT === this._textType) {
				// Create a rich text editor.
				this._editor = new RichTextEditor(this._content, this._contentHeight);
			} else if (ContentConstants.TextType.SIMPLE === this._textType) {
				// Create a simple text editor.
				this._editor = new SimpleEditor(this._content);
			} else if (ContentConstants.TextType.SMARTNAME === this._textType) {
				// Create a smartname text editor.
				this._editor = new SmartNameEditor(this._content);
			}
			
			// Set the height of the editor.
			this._editor.setHeight(this._contentHeight);
			
			if(this._maxChars && this._maxChars.length > 0){
				this._editor.setMaxChar(this._maxChars);
			}
			
			// Add the class
			this._editor.getJQElement().addClass("ContentArea");
			// Register content change event in rich text editor
			this._editor.onChange(_.bind(function () {
				var content = this._editor.getContent();
				this._executeContentChange(content);
			}, this));
			// Register Focus and Blur event in the rich text editor.
			this._editor.onFocus(_.bind(function () {
				if (this._canEdit) {
					$.each(this._onFocus, function (i, callback) {
						callback();
					});
				}
			}, this));
			this._editor.onBlur(_.bind(function () {
				$.each(this._onBlur, function (i, callback) {
					callback();
				});
			}, this));
		};
		
		
		
		/**
		 * Method is to update content if its actually changed.
		 * @param currentContent {String} - content from content area
		 */
		TextContentElementWidgetView.prototype._executeContentChange = function(currentContent) {
			
			// Findout whether the content was changed.
			var contentChanged = false;
			if (ContentConstants.TextType.SMARTNAME === this._textType) {
				contentChanged = currentContent.smartName != this._content.smartName;
			} else {
				contentChanged = currentContent.content != this._content.content;
			}
			
			// Avoid update if content is not modified
			if (contentChanged) {
				_.each(this._onMainContentCallbacks, _.bind(function(currentContent, callback) {
					callback(currentContent);
				}, this, currentContent));
				
				// Update current text data value to this object also. 
				this._content = currentContent;
				
				// If single line mode is supported ... try to show it in a single line.
				if (this._viewOptions.supportSingleLineMode) {
					this.trySingleLine();
				}
				
				if(this._maxChars && this._maxChars.length > 0){
					this._editor.setMaxChar(this._maxChars, this._isSingleLineMode);
				}
			}
		};
		
		/**
		 * Set the language indicator of the view.
		 * 
		 * @param language - language object {langCode, langDesc, langFlag}
		 */
		TextContentElementWidgetView.prototype.setLanguage = function (language) {
			
			// Remember the language object for this textContentElementWidget
			if (language) {
				this._language = language;
			}
			
			// Update the header.
			this.updateHeader();
		};
		
		/**
		 * Set the maximum number of chars
		 */
		TextContentElementWidgetView.prototype.setMaxChar = function (maxChar) {
			this._maxChars = maxChar;
			this._editor.setMaxChar(this._maxChars);
		};
		
		/**
		 * Allow other widget to add custom buttons to this widget.
		 * 
		 * @param button
		 */
		TextContentElementWidgetView.prototype.addCustomActionButton = function(button) {
			this._customActionButtons.push(button);
			button.insertBefore(this._headerDiv.find(".ActionCloser"));
		};
		
		/**
		 * Allow other widgets to remove custom buttons from this widget
		 * 
		 * @param button
		 */
		TextContentElementWidgetView.prototype.removeCustomActionButton = function(button) {
			//TODO: to remove the buttons one by one
		};		
		
		/**
		 * Construct the approval column of the widget
		 * @returns approvalDiv - approval column container
		 */
		TextContentElementWidgetView.prototype._constructApprovalDiv = function () {
			
			// Create the approval column and passing true to allow creation of approval actions.
			this._approvalCol = this._constructApprovalColumn(true);
			
			// Activate the approval column (put a little delay).
			window.setTimeout(function () {
				this._activateApprovalActions();
			}.bind(this), 200);
			
			
			return this._approvalCol;
		};
		
		
		/**
		 * Construct the approval column of the widget
		 * 
		 * @param constructApprovalAction {boolean} - Flag to identify whether approval action needs to be created.
		 * @returns approvalDiv - approval column container
		 */
		TextContentElementWidgetView.prototype._constructApprovalColumn = function (constructApprovalAction) {
			
			// A wrapper column containing two additional overlapping columns.
			this._approvalCol = $('<div>').addClass("TextContent-ApprovalCol");
			this._approvalStatusCol = this._constructElementStatus(constructApprovalAction);
			this._approvalActionCol = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates");

			// Add the current status
			this._approvalCol.append(this._approvalStatusCol);
			
			// If true, then create next approval action or undo action for approval column.
			if (constructApprovalAction) {
				// If no approval action is taken add approval buttons, add revert button otherwise
				if(!this._newState) {
					this._approvalActionCol = this._constructApprovalActions();
					this._approvalCol.append(this._approvalActionCol);
				} else {
					// If new state is not equal to deleted then create undo action.
					if (ContentConstants.State.DELETED != this._newState) {
						this._approvalActionCol = this._constructUndoAction();
						this._approvalCol.append(this._approvalActionCol);
					}
				}
			}
			
			return this._approvalCol;
		};
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		TextContentElementWidgetView.prototype._deactivateApprovalActions = function() {
			
			// Only do this when there are approval actions available to the element ...
			if (!this._hasAction) {
				return;
			}
			
			var curState = this._approvalCol.find("div[name='TextContent-StatusDiv']");
			var actions = this._approvalCol.find("div[name='TextContent-ActionsDiv']");
			
			// Stop the animations
			curState.finish();
			actions.finish();
			this._editor.getJQElement().finish();
			this._editor.getJQElement().css("margin-left","0px");
			
			// Make sure the 
			this._approvalCol.unbind('mouseenter mouseleave');
			
			this._approvalCol.remove();
			
			// Since approval column is not going to be activated, so pass false to avoid creation of approval action.
			this._constructApprovalColumn(false);
			this._wrapper.prepend(this._approvalCol);
		};
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		TextContentElementWidgetView.prototype._activateApprovalActions = function() {
			
			// Only do this when there are approval actions available to the element ...
			if (!this._hasAction) {
				return;
			}
			
			// Create a handler when mouseover element status to view approval actions
			if (!this._actionsDivHandlerIn) {
				this._actionsDivHandlerIn = _.bind(function () {
					
					// Get the actions and curState elements.
					var curState = this._approvalCol.find("div[name='TextContent-StatusDiv']");
					var actions = this._approvalCol.find("div[name='TextContent-ActionsDiv']");
					
					// Don't do anything when there are no actions.
					if (actions.length == 0) {
						return;
					}
					
					// There are actions ...
					
					// Fade-in the actions.
					curState.finish();
					actions.finish();
					curState.animate({opacity: 0}, 400, 'swing');
					actions.animate({opacity: 1}, 400, 'swing');
					actions.show();
					
					// If we're in single line mode, we need to move the editor so it does not get overlapped
					// with the actions.
					if (this._isSingleLineMode) {
						// Figure out how far the actions will span.
						var offset = 0;
						if (actions.hasClass("TextContent-Approval-Next2States")) {
							offset = 40;
						} else if (actions.hasClass("TextContent-Approval-Next3States")) {
							offset = 80;
						}
						var editorElement = this._editor.getJQElement();
						editorElement.finish();
						if (offset > 0) {
							editorElement.animate({'marginLeft': offset+'px'}, 500);
						}
					}
				}, this);
			}
						
			// Create a handler when mouseout element status to hide approval actions
			if (!this._actionsDivHandlerOut) {
				this._actionsDivHandlerOut = _.bind(function () {
					
					// Get the actions and curState elements.
					var curState = this._approvalCol.find("div[name='TextContent-StatusDiv']");
					var actions = this._approvalCol.find("div[name='TextContent-ActionsDiv']");
					
					// Don't do anything when there are no actions.
					if (actions.length == 0) {
						return;
					}
					
					// There are actions ...
					
					// Fade-out the actions.
					curState.finish();
					actions.finish();
					curState.animate({opacity: 1}, 400, 'swing');
					actions.animate({opacity: 0}, 400, 'swing', function () {
						actions.hide();
					});
					
					// If we're in single line mode, it could be that the editor is moved in order to not get overlapped
					// with the next state options.
					if (this._isSingleLineMode) {
						var nextStates = this._wrapper.find(".TextContent-Approval-NextStates");
						// Make sure the editors are in place again.
						var editorElement = this._editor.getJQElement();
						editorElement.finish();
						nextStates.show();
						editorElement.animate({marginLeft: '0px'}, 500, 'swing');
						
					}
				}, this);
			}
			
			// add the hover handler for approval actions
			this._approvalCol.unbind('mouseenter mouseleave');
			this._approvalCol.on("mouseenter", this._actionsDivHandlerIn);
			this._approvalCol.on("mouseleave", this._actionsDivHandlerOut);
		};
		
		/**
		 * Create TextContent status
		 * 
		 * @param constructApprovalAction {boolean} - Flag to identify whether approval action needs to be created.
		 * @returns elementStatusDiv - jQuery element for TextContent status
		 */
		TextContentElementWidgetView.prototype._constructElementStatus = function (constructApprovalAction) {
			var elementStatusDiv = $('<div>').attr({"name": "TextContent-StatusDiv"}).css({'opacity': '1'});
			
			// User is not an editor or reviewer and if approval actions are being created then provide default cursor.
			if(!(this._canEdit || this._canReview) || !constructApprovalAction) {
				elementStatusDiv.css({'cursor': 'default'});
			} else {
				elementStatusDiv.css({'cursor': 'pointer'});
			}
			
			var elementStatusIconClass = null;
			var elementStatusClass = null;
			if(!this._newState) { // If no new state is defined, render the current state.
				switch(this._state) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Draft ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusDraft";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					if(this._canReview && this._myApprovalStatus === ContentConstants.State.READY_FOR_APPROVAL) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-You ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.APPROVED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouApproved ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.REJECTED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouRejected ElementStatusIconAndLabel";
					} else {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending ElementStatusIconAndLabel";
					}
					elementStatusClass = "TextContentElement-Status ElementStatusPending";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Approved ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusApproved";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Rejected ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusRejected";
					break;
				default :
					elementStatusIconClass = "icon icon-WCR_Text-Status-NA ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status";
					break;	
				}
			} else { // If a new state is defined, render the current state.
				switch(this._newState) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Draft RevertTextToDraftIconAndLabel";
					elementStatusClass = "TextContentElement-Status RevertTextToDraft";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Start StartTextApprovalIconAndLabel";
					elementStatusClass = "TextContentElement-Status StartTextApproval";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Approve ApproveTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status ApproveText";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Reject RejectTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status RejectText";
					break;
				case ContentConstants.State.DELETED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-NA ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status";
					break;
				default :
					// Exectution should not reach here.
					break;
				}
			}			
			
			// Display the language status
			elementStatusDiv.removeClass().addClass(elementStatusClass).append(
					$('<div>').addClass("TextContent-ApprovalStatus").append(
							$('<div>').addClass("TextContent-StatusIcon").append($('<i>').addClass(elementStatusIconClass)))
			);
			
			return elementStatusDiv;
		};
		
		/**
		 * Create Approval action buttons
		 * 
		 * @returns approvalActionsDiv - jQuery element for approval action buttons
		 */
		TextContentElementWidgetView.prototype._constructApprovalActions = function () {
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'display': 'none', 'opacity': '0'});
			
			// Variable keeping track of whether we have actions available.
			this._hasAction = true;
			
			// If the language is ready for approval, show Approve, Reject and RevertToDraft buttons based on the permissions
			if(this._state === ContentConstants.State.READY_FOR_APPROVAL) {
				// If User is have both EDITOR and APPROVER permissions and TextContent is in Pending-Signoff - add Draft, Approve and Reject buttons
				if(this._canReview && this._hasEdit) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton, this._revertToDraftButton);
					approvalActionsDiv.addClass("TextContent-Approval-Next3States");
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-3ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-3ApprovalActions");
				}
				// If User can approve - add Approve and Reject buttons
				else if(this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton);
					approvalActionsDiv.addClass("TextContent-Approval-Next2States");
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
				}
				// If the User has EDITOR permission - add the RevertToDraft button
				else if (this._hasEdit) {
					approvalActionsDiv.append(this._revertToDraftButton);
					approvalActionsDiv.addClass("TextContent-Approval-Next1State");
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
				} else {
					this._hasAction = false;
				}
			}
			else if (this._state === ContentConstants.State.DRAFT) {
				// If the language is in Draft and user has both EDITOR and APPROVER permissions, then add Start Approval & Approve button (User can skip start approval)
				if(this._canEdit && this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._startApprovalButton);
					approvalActionsDiv.addClass("TextContent-Approval-Next2States");
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
				} 
				// If the language is in Draft and user has EDITOR permission, then add StartApproval button
				else if(this._canEdit) {
					approvalActionsDiv.append(this._startApprovalButton);
					approvalActionsDiv.addClass("TextContent-Approval-Next1State");
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
				} else {
					this._hasAction = false;
				}
			}
			// If the language is rejected, add a explict move to draft button
			// if the language is approved, and user has content manager permission, then revert draft is possible
			else if((this._hasEdit && this._state === ContentConstants.State.REJECTED) ||
					(this._isContentManagerView && this._state === ContentConstants.State.APPROVED)) {
				approvalActionsDiv.append(this._revertToDraftButton);
				approvalActionsDiv.addClass("TextContent-Approval-Next1State");
				approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
				approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
			} else {
				this._hasAction = false;
			}
			
			return approvalActionsDiv;
		};
		
		/**
		 * Create the Revert Approval button
		 * @returns approvalActionsDiv - jQuery element for revert approval action button
		 */
		TextContentElementWidgetView.prototype._constructUndoAction = function () {
			this._hasAction = true;
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'opacity': '0'});
			approvalActionsDiv.append(this._undoButton);
			return approvalActionsDiv;
		};
		
		
		/**
		 * Construct the header of the widget
		 * 
		 * @returns headerDiv - jQuery element for Language header and other actions
		 */
		TextContentElementWidgetView.prototype._constructHeaderDiv = function () {
			
			// create the header div
			this._headerDiv = $('<div>').addClass("TextContent-TitleRow");
			
			// Add language description div to header
			this._headerDiv.append($('<div>').attr({'name': 'languageDescDiv'}));
			
			// create and append comment button to header
			this._createCommentButton();
			this._headerDiv.append(this._commentButton);
			
			// create and append approval info button to header
			this._headerDiv.append(this._approvalInfoButton);
			
			// Add a custom header div
			for (var i=0; i < this._customActionButtons.length; i++) {
				this._headerDiv.append(this._customActionButtons[i]);
			}
			
			// Clear
			this._headerDiv.append($('<div>').css({'clear': 'both'}).addClass("ActionCloser"));
			
			// Update the header (This will display the language).
			this.updateHeader();
			
			return this._headerDiv;
		};
		
		/**
		 * Update the header
		 */
		TextContentElementWidgetView.prototype.updateHeader = function() {
			
			// Check if we have a header div.
			if (!this._headerDiv) {
				return;
			}
			
			// Update the style of the header based on the state.
			// Get the current state ...
			var state = this._state;
			if (this._newState) {
				state = this._newState;
			}
			var headerClass = null;
			if(ContentConstants.State.DRAFT == state) {
				headerClass = "TextContent-TitleRow TextHeaderDraft";
			} else if(ContentConstants.State.READY_FOR_APPROVAL == state) {
				headerClass = "TextContent-TitleRow TextHeaderPending";
			} else if(ContentConstants.State.APPROVED == state) {
				headerClass = "TextContent-TitleRow TextHeaderApproved";
			} else if(ContentConstants.State.REJECTED == state) {
				headerClass = "TextContent-TitleRow TextHeaderRejected";
			} else {
				headerClass = "TextContent-TitleRow";
			}
			this._headerDiv.removeClass();
			this._headerDiv.addClass(headerClass);
			
			// Update the language.
			if (this._language) {

				// Find the language header and update it with the right label and flag.
				var languageDescDiv = this._headerDiv.find("div[name='languageDescDiv']");
				languageDescDiv.addClass("TextContent-Title");
				
				var languageLabel = null;
				if (this._language.langDesc) {
					languageLabel = this._language.langDesc;
				} else if (this._language.langCode) {
					languageLabel = this._language.langCode;
				} else {
					languageLabel = this._langaugeCode;
				}
				
				languageDescDiv.empty();
				if(this._isMaster) {
					languageDescDiv.append($('<b>').text(languageLabel));
				} else {
					languageDescDiv.append($('<span>').text(languageLabel));
				}
				
				if(this._language.langFlag) {
					languageDescDiv.prepend($('<img>').attr({"src": "images/country_flags/" + this._language.langFlag}));
				}
			}
			
		};
		
		/**
		 * Create comment button
		 */
		TextContentElementWidgetView.prototype._createCommentButton = function() {
			// Add the Comment button based on the permissions
			var commentIcon = null;
			if(this._comments && this._comments.length > 0) {
				commentIcon = "icon icon-WCR_Text-Comment-full";
			} else {
				commentIcon = "icon icon-WCR_Text-Comment-outline";
			}
			this._commentButton.append($('<i>').addClass(commentIcon));
			
			
			if((this._hasEdit || this._hasReview || this._isContentManagerView) || (this._comments && this._comments.length > 0)) {
				// click event handler for comment button
				this._commentButton.click(_.bind(function(event) {
					if(this._commentsDiv.hasClass("commentshidden")) {
						this._showCommentSection();
					} else {
						this._hideCommentSection();
					}
				}, this));
				
				// hide the comments when clicked anywhere on the page except the comment button
				$(document).mouseup(_.bind(function (e) {
					if (!this._commentsDiv.is(e.target) 
							&& this._commentsDiv.has(e.target).length === 0 && !this._commentButton.is(e.target) && !this._commentButton.find("i:first").is(e.target)) {
						// Check if we should hide the comments section.
						if(!this._commentsDiv.hasClass("commentshidden")) {
							this._hideCommentSection();
						}
					}
					e.stopPropagation();
				}, this));
			} else {
				this._commentButton.prop({'disabled': 'disabled'}).css({'cursor': 'not-allowed'});
			}
		};
		
		/**
		 * Show the comments section
		 */
		TextContentElementWidgetView.prototype._showCommentSection = function() {
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
			// Focus on comment text area to allow user to type immediately.
			this._commentTextArea.focus();
		};

		/**
		 * Hide the comments section
		 */
		TextContentElementWidgetView.prototype._hideCommentSection = function() {
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};

		
		/**
		 * Construct the TextContent area
		 * @returns contentDiv
		 */
		TextContentElementWidgetView.prototype._constructTextDiv = function () {

			// Create the div that will be holding the content.
			this._contentDiv = $('<div>').addClass("Content-ContentRow").attr({"name": "Content-ContentAreaDiv"});

			// Add the text editor to the content div.
			this._contentDiv.append(this._editor.getJQElement());

			// Reset the "editability" tooltip.
			this._nonEditableTooltip = undefined;
			this.applyEditPermission();

			return this._contentDiv;

		};
		
		/**
		 * Apply edit permission if allowed or provide tool tip message on disabled field.
		 */
		TextContentElementWidgetView.prototype.applyEditPermission = function () {
			// We can only edit when you have the edit permission and the state is "DRAFT" or "REJECTED".
			if (!this._canEdit) {
				// Disable content editing.
				this._editor.disable();
				this._contentDiv.addClass("NotEditable");

				// If we can't edit because we lack permissions, say why ...
				var msgTag = undefined;
				if (!this._canEdit) {
					// Get the message tag.
					msgTag = this._whyCanNotEdit;
					// If the state is "EMPTY", make sure the "NO_PCM_PERM_NOT_ADDED" message is shown.
					if (this._state === ContentConstants.State.EMPTY) {
						msgTag = "NO_PCM_PERM_NOT_ADDED";
					}
				} else if (this._state === ContentConstants.State.READY_FOR_APPROVAL) {
					msgTag = "NO_PCM_PERM_EDIT_READY_FOR_APPROVAL";
				} else if (this._canEdit && this._state === ContentConstants.State.EMPTY) {
					msgTag = "NO_PCM_PERM_NOT_ADDED";
				}
				
				// If we have a reason, display it. 
				if (msgTag && this._showTooltips) {
					if (!this._nonEditableTooltip ) {
						// Create the tooltip
						this._nonEditableTooltip = new Tooltip(undefined, this._contentDiv, this._wrapper, $("body"), false, TextContentStrings.t(msgTag), undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
					} else {
						this._nonEditableTooltip.setTooltipContent(TextContentStrings.t(msgTag));
					}
				}
			} else {
				// Enable content editing.
				this._editor.enable();
				this._contentDiv.removeClass("NotEditable");
			}
		};	
		
		/**
		 * update the TextContent area
		 * @param content {"content": content} - An object for the text content statement
		 * @returns textDiv - jQuery element for TextContent textarea
		 */
		TextContentElementWidgetView.prototype.updateContent = function (content) {
			
			// Create an empty content object if none was passed.
			if (!content) {
				content = {content : ""};
			}
			this._content = content;
			
			// Set the content.
			this._editor.setContent(content);
			
			// Make an attempt to show the widget in a single line (if configured so).
			this.trySingleLine();
			
			// Apply the edit permissions again.
			this.applyEditPermission();
		};
		
		
		/**
		 * Construct the comments column of the widget
		 * 
		 * @returns commentDiv - jQuery element for comments column
		 */
		TextContentElementWidgetView.prototype._constructCommentsDiv = function () {
			// If user does not have permissions and there are no existing comments, then do not create comments div
			if((!this._hasEdit && !this._hasReview && !this._isContentManagerView) && (this._comments && this._comments.length === 0)) {
				return;
			}
			
			this._commentsDiv = $('<div>').attr({"name": "commentsDiv"});
			
			// hide the comments initially
			this._commentsDiv.addClass("TextComment commentshidden").hide();
			this._commentsDiv.append(
					$('<table>').addClass("TextComments").append(
							$('<tbody>')
					)
			);
			
			// Show the Add comment content area if the user has permissions to add comment
			if(this._hasEdit || this._hasReview || this._isContentManagerView) {
				this._commentsDiv.find("tbody:first").append($('<tr>').addClass("Comment").append(
						$('<td>').addClass("CommentContent").append(
								$('<div>').addClass("CommentAuthor").text(TextContentStrings.t("PCM_COMMENT_CURRENT_USER")),
								this._commentTextArea.addClass("CommentField").attr({"placeholder": TextContentStrings.t("PCM_ADD_COMMENT_PLACEHOLDER")})
						)
				));
			}
			
			// Add the existing comments to the comments div
			if(this._comments && this._comments.length > 0) {
				var textcomments = $.extend(true, [], this._comments);
				textcomments.reverse();
				
				var newComment = null;
				_.each(textcomments, _.bind(function(comment) {
					if(!comment.isNew) {
						this._commentsDiv.find("tbody:first").append(
								$('<tr>').addClass("Comment").append(
										$('<td>').addClass("CommentContent").append(
												$('<div>').addClass("CommentAuthor").text(comment.user),
												$('<div>').addClass("CommentText").text(comment.content),
												$('<div>').addClass("CommentDate").text(ConvertUtils.getLangFormattedDateAndTime(TextContentStrings, new Date(WCXML.xmlencode(comment.created))))
										)
								)
						);
					} else {
						newComment = comment;
					}
				}, this));
				
				if(newComment) {
					this._commentTextArea.text(newComment.content);
				}
			}
			
			return this._commentsDiv;
		};
		
		/**
		 * Get the localized value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedState]
		 */
		TextContentElementWidgetView.prototype._getLocalizedContentState = function(state, myapprovalstatus) {
			var localizedState = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_APPROVED");
				break;
			case ContentConstants.State.DRAFT:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				if(myapprovalstatus === ContentConstants.State.APPROVED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUAPPROVED");
				} else if(myapprovalstatus === ContentConstants.State.REJECTED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUREJECTED");
				} else if(myapprovalstatus === ContentConstants.State.READY_FOR_APPROVAL) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUPENDING");
				} else {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING");
				}
				break;
			case ContentConstants.State.EMPTY:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_EMPTY");
				break;
			case ContentConstants.State.REJECTED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_REJECTED");
				break;
			default:
				break;
			}
			
			return localizedState;
		};
		
		/**
		 * Get the localized action value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedAction]
		 */
		TextContentElementWidgetView.prototype._getLocalizedContentAction = function(state) {
			var localizedAction = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedAction = TextContentStrings.t("PCM_ACTION_APPROVE");
				break;
			case ContentConstants.State.DRAFT:
				localizedAction = TextContentStrings.t("PCM_ACTION_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				localizedAction = TextContentStrings.t("PCM_ACTION_START_APPROVAL");
				break;
			case ContentConstants.State.REJECTED:
				localizedAction = TextContentStrings.t("PCM_ACTION_REJECT");
				break;
			default:
				break;
			}
			
			return localizedAction;
		};
		
		/**
		 * Select the widget.
		 */
		TextContentElementWidgetView.prototype.select = function (selected) {
			if (selected) {
				this._wrapper.addClass("Selected");
			} else {
				this._wrapper.removeClass("Selected");
			}
		};
		
		/**
		 * Focus on the widget.
		 */
		TextContentElementWidgetView.prototype.focus = function (focus) {
			if (focus) {
				// Focus on the text editor
				this._editor.focus();
			} else {
				// Blur the text editor
				this._editor.blur();
			}
		};
		
		/**
		 * Register a callback to be called on widget focus.
		 */
		TextContentElementWidgetView.prototype.onFocus = function (callback) {
			// Remember the callback
			this._onFocus.push(callback);
		};
		
		/**
		 * Register a callback to be called on widget blur.
		 */
		TextContentElementWidgetView.prototype.onBlur = function (callback) {
			// Remember the callback
			this._onBlur.push(callback);
		};
		
		/**
		 * Enable/Disable the widget.
		 */
		TextContentElementWidgetView.prototype.enable = function (enable, msgTag) {
			// Enable widget.
			if (enable) {
				// Construct the view again to enable widget based on user permissions.
				this._constructView();
			} 
			// Disable widget.
			else {
				// Check which type of editor to disable/enable.
				this._editor.disable();
				this._contentDiv.addClass("NotEditable");
				if(this._customActionDiv)
					this._customActionDiv.hide();
				
				
				// If we have a reason, display it. 
				if (msgTag && this._showTooltips) {
					if (!this._nonEditableTooltip) {
						// Create the tooltip
						this._nonEditableTooltip = new Tooltip(undefined, this._contentDiv, this._wrapper, $("body"), false, TextContentStrings.t(msgTag), undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
					} else {
						this._nonEditableTooltip.setTooltipContent(TextContentStrings.t(msgTag));
					}
				}
				
				// Remove the hover handler for approval actions
				this._deactivateApprovalActions();
				
			}
		};
		
		/**
		 * Return wether or not the widget is in single line mode or not.
		 */
		TextContentElementWidgetView.prototype.isSingleLineMode = function () {
			return this._isSingleLineMode;
		};
		
		/**
		 * Add callback for Start Approval action
		 * @param callback - callback function to perform the start approval action
		 * @returns
		 */
		TextContentElementWidgetView.prototype.onStartApproval = function (callback) {
			return this._onStartApprovalCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Reject action
		 * @param callback - callback function to perform the reject action
		 * @returns
		 */
		TextContentElementWidgetView.prototype.onReject = function (callback) {
			return this._onRejectCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Approve action
		 * @param callback - callback function to perform the approve action
		 * @returns
		 */
		TextContentElementWidgetView.prototype.onApprove = function (callback) {
			return this._onApproveCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Revert to Draft action
		 * @param callback - callback function to perform the revert to draft action
		 * @returns
		 */
		TextContentElementWidgetView.prototype.onRevertToDraft = function (callback) {
			return this._onRevertToDraftCallbacks.push(callback);
		};
		
		/**
		 * Add callback for adding comment
		 * @param - callback function to add comment using comment textarea
		 */
		TextContentElementWidgetView.prototype.onComment = function (callback) {
			return this._onCommentCallbacks.push(callback);
		};
		
		/**
		 * Add callback for when the view toggles single line mode or not.
		 */
		TextContentElementWidgetView.prototype.onSingleLineModeToggle = function (callback) {
			return this._onSingleLineModeToggleCallbacks.push(callback);
		};
		
		/**
		 * Add callback for reverting the approval
		 * @param callback - callback function to revert the approval action (approve/reject/revert-to-draft/start-approval)
		 */
		TextContentElementWidgetView.prototype.onUndoApproval = function (callback) {
			return this._onUndoCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the approval info button is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		TextContentElementWidgetView.prototype.onApprovalInfo = function (callback) {
			return this._onApprovalInfoCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the main content is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		TextContentElementWidgetView.prototype.onMainContent = function (callback) {
			return this._onMainContentCallbacks.push(callback);
		};
		
		/**
		 * Method to return the wrapper jQuery element of the widget
		 * 
		 * @returns wrapper - Return the wrapping jquery element.
		 */
		TextContentElementWidgetView.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		/**
		 * Method to check if the element state is EMPTY
		 * 
		 * @returns {Boolean}
		 */
		TextContentElementWidgetView.prototype.isEmptyElement = function () {
			return this._state === ContentConstants.State.EMPTY;
		};
		
		// Return the constructor.
		return TextContentElementWidgetView;
});
