/**
 * Common service class to load or save content as table or single text.
 * @PASH
 */
define(["jquery",
        "packaging-content-management/content/SingleTextContent",
        "packaging-content-management/content/TableContent",
        "packaging-content-management/content/TextContentElement",
        "services/passwordValidationService",
        "utils/webi18n!TextContentStrings"], 
        function ($, 
        		SingleTextContent,
        		TableContent,
        		TextContentElement,
        		passwordValidationService,
        		TextContentStrings) {
	
	var ContentService = function () {
        
	};

	/**
	 * @param {String} contentRequest - Object indicating the circumstances under which the content should be requested.
	 * @param {Function} callback - Will be executed on successful return from the back end (and be passed the content).
	 * @param {Function} errorCallback - Will be executed on error from back end.
	 */
	ContentService.prototype.getContent = function (contentRequest, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"contentRequest": JSON.stringify(contentRequest)
		};
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
			var content;
			if (jsonData.objType == "SingleTextContent") {
				content = new SingleTextContent(jsonData);
			} else if (jsonData.objType == "TableContent") {
				content = new TableContent(jsonData);
			} else {
				content = new SingleTextContent(jsonData);
			}
        	callback(content);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	/**
	 * Save the changes in a Content object.
	 * 
	 * The Content should already exist!
	 * 
	 * @param {Content} content - The Content object to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated content or
	 * 								the content for which the update failed and a possible error message).
	 */
	ContentService.prototype.saveContent = function (content, callback) {
		this.saveContents([content], callback);
	};
	
	/**
	 * Save the changes in multiple Content objects.
	 * 
	 * These Contents should already exist!
	 * 
	 * @param {Content} contents - The Contents objects to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated contents, 
	 * 								the contents for which the update failed and error messages).
	 */
	ContentService.prototype.saveContents = function (contents, callback) {
		
		var password = passwordValidationService.getValidatedPassword();
		
		var saveActionJSONs = [];
		var docVersionIds = [];
		var docVersionIdMap = [];
		for (var i=0; i < contents.length; i++) {
			// Create an array of SaveAction JSON's.
			var saveActionJSON = JSON.stringify(contents[i].getSaveActions(password), function (key, value) {
				// We have to be careful to not have circular references in our SaveActions, therefore we will need to handle specific fields carefully.
				if (key == 'affectedTableContent' || key == 'affectedTextContent' || key == 'affectedTextContentElement') {
					return value.getContentId();
				} else {
					return value;
				}
			});
			
			saveActionJSONs.push(saveActionJSON);
			docVersionIds.push(contents[i].docVersionId);
			docVersionIdMap[contents[i].docVersionId] = contents[i];
		}
		
		var url = window.webcenterConfig.jspBaseUrl + 'SaveContent.jsp';
		$.ajax(url, {
			dataType: "json",
			cache: false,
			noAjaxParam: false,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			type: "POST",
			data: {
				saveactions: saveActionJSONs,
				docverids : docVersionIds
			},
			success: function (json) {
				
				// Get all the error messages.
				var errorMsgs = {};
				// Get all the updated contents;
				var updatedContents = {};
				
				// If a general error occurred.
				if (json.error) {
					// Add the general error occurred.
					errorMsgs[0] = json.error;
					// All contents failed.
					updatedContents = contents.slice(0, contents.length);
				}
				// Find out which updates succeeded and get the new updated contents.
				else if (json["updates"]) {
					// Iterate over all update entries ...
					for (var i=0; i < json.updates.length; i++) {
						var update = json.updates[i];
						// Check if an error occurred during the update ...
						if (update["error"]) {
							errorMsgs[update["id"]] = update["error"];
						}
						// Check if we have a newly updated version of the content.
						if (update["new"]) {
							if (update["new"].objType == "TableContent") {
								updatedContents[update["id"]] = new TableContent(update["new"]);
							} else {
								updatedContents[update["id"]] = new SingleTextContent(update["new"]);
							}
						}
					}
				}
				
				// If a callback was specified.
				if (callback) {
					callback(updatedContents, errorMsgs);
				}
				
			},
			error: function (jqXHR, textStatus, errorThrown) {
				var errorXml = $.parseXML(jqXHR.responseText);
				$xmlErr = $(errorXml);
				$message = $xmlErr.find( "message" );
				// If user login failed, the redirect him to login page
				if($message.text() == "User login failed"){
					var urlwanted = window.location.href;
					window.location.replace("home.jsp?urlwanted="+urlwanted);
					return;
				}
				
				if(callback) {
					callback([], contents.slice(0, contents.length), [TextContentStrings.t("TEXTCONTENT_SAVE_FAILED")]);
				}
			}
		});
	};
	
	return new ContentService();
	
});