/**
 * ContentUtils object
 * Utility methods
 * 
 * @author PASH
 */
define(["jquery",
        "../ContentConstants"],
	function ($,
			ContentConstants) {
		"use strict";
		
		var ContentUtils = function () {
			
		};

		/**
		 * Returns boolean value for any variable.
		 * 
		 * @param {Object} variable an object of any type to get its boolean value.
		 */
		ContentUtils.prototype.getBooleanValue = function (variable) {
			if (!variable) {
				return false;
			} else if (typeof variable == "boolean") {
				return variable;
			} else if (typeof variable == "string") {
				return "true" == variable ? true : false;
			} else {
				return true;
			}
		};
		
		/**
		 * Returns logged in user name in format of "LastName, FirstName (UserName)".
		 */
		ContentUtils.prototype.getLoggedInUserName = function () {
			var loginUser = gCurrentUser.userName;
			var loginUserFirstName = gCurrentUser.firstName;
			var loginUserLastName = gCurrentUser.lastName;
			
			return loginUserLastName + ", " + loginUserFirstName + " (" + loginUser + ")";
		};
		
		/**
		 * Get a query parameter by name;
		 * 
		 * @param name
		 * @returns
		 */
		ContentUtils.prototype.getParameterByName = function (name){
		    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
		    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
		        results = regex.exec(location.search);
		    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
		};
		
		/**
		 * Calculates what should be the current state based on inner elements.
		 * CurrentState = lowest state of any element. This means that, if there is any one element that is in Rejected state,
		 * then the language widget state will be Rejected.
		 */
		ContentUtils.prototype.calculateCurrentState = function(elementsStateArray){
			var currentState = undefined;

			$.each(elementsStateArray, _.bind(function (index, elementORState) {
				var innerState = elementORState;
				
				// There are possibilities to get elements array in place of states array.
				if (elementORState && elementORState.getCurrentState) {
					innerState = elementORState.getCurrentState();
				}
				
				switch(innerState) {
				case ContentConstants.State.REJECTED:
					// Found 1 element with status as rejected. So overall state is rejected.
					currentState = ContentConstants.State.REJECTED;
					
					return false;
				case ContentConstants.State.DRAFT:
					// if current state is not rejected
					if (currentState != ContentConstants.State.REJECTED) {
						currentState = ContentConstants.State.DRAFT;
					}
					
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					if (currentState !== ContentConstants.State.DRAFT && 
						currentState != ContentConstants.State.REJECTED) {
						currentState = ContentConstants.State.READY_FOR_APPROVAL;
					}
					
					break;
				case ContentConstants.State.APPROVED:
					// Current state is not yet calculated.
					if (!currentState || ContentConstants.State.EMPTY == currentState) {
						currentState = ContentConstants.State.APPROVED;
					}
					
					break;
				default:
					currentState = ContentConstants.State.EMPTY;
				}
				
			}, this));
			
			return currentState;
		};
		
		/**
		 * Get next states based on the permission and current state.
		 */
		ContentUtils.prototype.getNextStatesForLanguage = function(canManage, currentState, elementsArray, projMasterLangs) {
			var nextStates = [];
			
			var hasEdit = undefined;
			var canEdit = undefined;
			var hasReview = undefined;
			var canReview = undefined;
			
			if (elementsArray.length > 0) {
				hasEdit = elementsArray[0].hasEdit().allowed;
				hasReview = elementsArray[0].hasReview().allowed;
				
				// There are possibilities that edit or review is not allowed due to state of first element, so calculate for all until allowed for at-least 1 of them.
				$.each(elementsArray, _.bind(function (index, element) {
					// Calculate edit permission if not calculated yet.
					if (!canEdit) {
						canEdit = element.canEdit(projMasterLangs).allowed;
					}
					
					// Calculate review permission if not calculated yet.
					if (!canReview) {
						canReview = element.canReview(projMasterLangs).allowed;
					}
					
					// If both permissions are calculated then break the loop.
					if (canEdit && canReview) {
						return false;
					}
				}, this));
			}
			
			switch(currentState) {
			case ContentConstants.State.REJECTED:
				nextStates = hasEdit ? [ContentConstants.State.DRAFT] : [];
				
				break;
			case ContentConstants.State.DRAFT:
				nextStates = canEdit ? [ContentConstants.State.READY_FOR_APPROVAL] : [];
				nextStates = canEdit && hasReview ? [ContentConstants.State.READY_FOR_APPROVAL, ContentConstants.State.APPROVED] : nextStates;
				
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				nextStates = hasEdit ? [ContentConstants.State.DRAFT] : [];
				nextStates = canReview ? [ContentConstants.State.APPROVED, ContentConstants.State.REJECTED] : nextStates;
				nextStates = hasEdit && canReview ? [ContentConstants.State.DRAFT, ContentConstants.State.APPROVED, ContentConstants.State.REJECTED] : nextStates;
				
				break;
			case ContentConstants.State.APPROVED:
				nextStates = canManage ? [ContentConstants.State.DRAFT] : [];
				
				break;
			default:
				break;
			}
			
			return nextStates;
		};
		
		/**
		 * This method is to check value of input parameter for the text type and returns actual value of text type. 
		 * In case input string is not matching to any of the available text type, then it returns as rich text.
		 * 
		 * @param textType {String} - text type available 
		 * @returns Valid textType {String} for content document.
		 */
		ContentUtils.prototype.getValidTextType = function(textType) {
			
			switch (textType) {
			case ContentConstants.TextType.RICHTEXT:
				return ContentConstants.TextType.RICHTEXT;
				
			case ContentConstants.TextType.SIMPLE:
				return ContentConstants.TextType.SIMPLE;
				
			case ContentConstants.TextType.SMARTNAME:
				return ContentConstants.TextType.SMARTNAME;
				
			default:
				return ContentConstants.TextType.RICHTEXT;
			}
		};
		
		return new ContentUtils();
});