/**
 * Text Information view generation.
 * @author PASH
 */
define(["jquery",
        "underscore",
        "utils/webi18n!TextContentStrings",
        "packaging-content-management/text-information-view/InitTextInformationView",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/text-content-services/ContentService"], 
	function($,
			_,
			webi18n,
			InitTextInformationView,
			ContentConstants,
			ContentService) {
		
		"use strict";
		
		
		/**
		 * Constructor for TextInformationView
		 * @param textContent {TextContent} The Text content to display.
		 * @param projectId {projectId} a reference to the projectId
		 * @param i18nPCM {i18nPCM} array of localized strings from wcstrings.
		 */
		var TextInformationView = function(textContent, projectId) {
			
			this.projectId = projectId;
			this.webi18n = webi18n;
			this._content = textContent;
			
			// Create the view
			this._view = new InitTextInformationView(webi18n);
			
			//For showing unsaved warning upon leaving the page
			this._view.onBeforeUnload();
			
			// Save the content.
			this._view.onSaveClicked(function () {
				ContentService.saveContents([this._content], function (updatedContents, errorMsgs) {
					var updatedContentIds = Object.keys(updatedContents);
					var errorMsgIds = Object.keys(errorMsgs);
					if (updatedContentIds.length > 0) {
						// Get the updated content.
						var updatedContent = updatedContents[updatedContentIds[0]];
						// We need to refresh the page but without the docversionId parameter since we want the latest version.
						// Therefore we need the following parameters
						var docID = this._getParameterByName("docID");
						var menu = this._getParameterByName("menu_file");
						// Generate the new query string and redirect to it.
						var query = jQuery.param({'docID': docID, 'menu_file': menu, 'docVerID': updatedContent.docVersionId, 'error' : errorMsgs[errorMsgIds[0]]});
						window.location.replace("./doctypetextview.jsp?"+query);
					}
				}.bind(this));
			}.bind(this));
			
			
			this._view.onInvariantChange(function (val) {
				var isInvariant = val;
				this._content.setInvariant(isInvariant);
				if(isInvariant) {
					this._view._propertiesContainer.find(".TranslationLangRow:first").hide();
					this._view._translationElementsContainer.hide();
				} else {
					this._view._propertiesContainer.find(".TranslationLangRow:first").show();
					this._view._translationElementsContainer.show();
				}
			}.bind(this));
			
			this._view.onElementTypeChange(function (val) {
				this._content.setElementType(val);
				
				if (this._content.isElementTypeChanged()) {
					// Disable all the widget until document is saved.
					this._view.disableWidgets(true, true);
				} else {
					// Enable all the widgets.
					this._view.enableWidgets(true, true);
				}
				
			}.bind(this));
			
			this._view.onApprovalTypeChange(function (val) {
				this._content.setApprovalType(val);
			}.bind(this));
			
			this._view.onMaxCharsChange(function (val) {
				this._content.setMaxChars(val);
				return true;
			}.bind(this));
			
			
			// Execute, when text type changes for content object 
			this._view.onTextTypeChange(_.bind(function (val) {
				var isTextTypeUpdated = true;
				
				var elementsList = this._content.getElementsList();
				
				// Find out how many elements needs to be reverted back to draft status.
				var goingToBeDraftElements = $.grep(elementsList, function(element, index){
					// do not worry about deleted content element objects.
					if (element.deleted) {
						return false;
					}
					
					// Based on status of the element return elements of the content.
					switch(element.state) {
						case ContentConstants.State.READY_FOR_APPROVAL:
						case ContentConstants.State.APPROVED:
						case ContentConstants.State.REJECTED:
							return true;
						case ContentConstants.State.EMPTY:
						case ContentConstants.State.DELETED:
						case ContentConstants.State.DRAFT:
		                default: 
		                	return false;
					}
				});
				
				// If all the text content elements are not in draft state, then ask for the confirmation.
				if (goingToBeDraftElements.length > 0) {
					var result = confirm(this.webi18n.t("PCM_TEXT_TYPE_CHANGE"));
					if (!result) {
						isTextTypeUpdated = false;
						// If text type is changed to simple, then confirm again for the action because approval process will start again.
						if (ContentConstants.TextType.SIMPLE === val) {
							this._view.setTextType(ContentConstants.TextType.RICHTEXT);
						} else {
							this._view.setTextType(ContentConstants.TextType.SIMPLE);
						}
					}
				}
				
				// check whether text type is modified or change rolled back by user.
				if (isTextTypeUpdated) {
					this._content.setTextType(val);
					
					if (this._content.isTextTypeChanged()) {
						// Disable all the widget until document is saved.
						this._view.disableWidgets(true, true);
					} else {
						// Enable all the widgets.
						this._view.enableWidgets(true, true);
					}
				}
				
				// Text type is modified or not.
				return isTextTypeUpdated;
			}, this));
			
			this._view.onLangChange(function () {
				
				var elements = this._content.element;
				if (elements) {
					this._view.getTextContentRows(elements, this._content.invariant);
				}
			}.bind(this));
			
			// Update the view.
			this.updateView();
		};
		
		/** 
		 * Initializing Text Information view
		 * @returns
		 */
		TextInformationView.prototype.updateView = function() {
			
			//set ContentStatement
			this._view.setContentObj(this._content);

			// renders the last updated date at top of the document
			this._view.getTextUpdatedDate();
			
			// renders the element type, invariant option 
			this._view.getMetaDataRow();
			
			// renders the approval dropdown option
			this._view.getApprovalSetupRow();
			
			// renders the type of the document drop down option
			this._view.getTextTypeRow();
			
			// renders the max chars row
			this._view.getMaxCharsRow();
			
			// Renders the language indicators
			this._view.getProjectLanguagesRow();
			
			// renders the text content elements
			var elements = this._content.element;
			if (elements) {
				this._view.getTextContentRows();
			}
		};
		
		/**
		 * Get a query parameter by name;
		 * 
		 * @param name
		 * @returns
		 */
		TextInformationView.prototype._getParameterByName = function (name){
		    name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
		    var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
		        results = regex.exec(location.search);
		    return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
		};
		
		/**
		 * Get the wrapping jQuery element for this page.
		 */
		TextInformationView.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};
		
		return TextInformationView;
	}
);
