define(["underscore", "utils/webi18n!DamConnectionsErrors"], function (_, i18n) {
    /**
     * Constructor
     */
    var DamConnection = function (form) {
        this._names = [];
        this._ids = [];
        this._duplicate = false;
        
        this._nameStatusDiv = $("div[name=nameStatus]", form);
        this._connectionStatusDiv = $("div[name=connectionStatus]", form);

        try {
            //get names from database
            $.ajax({
                url: "DamConnectionServlet.wcr?action=info",
                type: 'GET',
                cache: false,
                responseType: 'json',
                success: _.bind(function (responseJson) {
                    this._names = _.pluck(responseJson.result.dams, "name");
                    this._ids = _.pluck(responseJson.result.dams, "id");
                }, this)
            });

            // listen to input events
            $("input[name=dam_name]", this.div).on("change keyup", _.bind(this.checkDuplicate, this));
            $("button[name=urlcheckConnection]", this.div).on("click", _.bind(this.testConnection, this));
            $("button[name=returnBtn]", this.div).on("click", this.goBack);
            
            $(form).on("submit", (_.bind(function(e){
                this.validate(e);
            }, this)));
        } catch (err) {
            console.log(err);
        }
    };

    /**
     * Shows an error when the name already exists in the database
     */
    DamConnection.prototype.checkDuplicate = function () {
        var id = $('input[name=connectionId]').val();
        var name = $('input[name=dam_name]', this.div).val();
        if (_.indexOf(this._names, name) > -1) {
            if (id) { //in edit mode
                var index = this._names.indexOf(name);
                if (this._ids[index] !== id) {
                    this._duplicate = true;
                    this.showMsgError(this._nameStatusDiv, i18n.t("DAMCONNECTION_DUPLICATE_NAME"));
                }
            } else {
                this._duplicate = true;
                this.showMsgError(this._nameStatusDiv, i18n.t("DAMCONNECTION_DUPLICATE_NAME"));
            }
        } else {
            this._duplicate = false;
            this.hideMsg(this._nameStatusDiv);
        }
    };

    /**
     * Checks whether the connection is reachable
     */
    DamConnection.prototype.testConnection = function () {
        this._connectionStatusDiv.addClass('hidden');
        this._connectionStatusDiv.removeClass('alert-success');
        this._connectionStatusDiv.removeClass('alert-danger');
        this._connectionStatusDiv.empty();

        try {
            // get info from form
            var inputs = $(':input');
            var fd = new FormData();
            inputs.each(function () {
                var val = $(this).val();
                var name = $(this).prop("name");
                if (name && val) {
                    fd.append(name, val);
                }
            });

            // test the connection
            $.ajax({
                url: "DamConnectionServlet.wcr?action=test",
                type: 'POST',
                responseType: 'json',
                enctype: 'multipart/form-data',
                processData: false,
                contentType: false,
                data: fd,
                success: _.bind(function (response) {
                    if (response.result.status == "OK") {
                    	this.showMsgOk(this._connectionStatusDiv, i18n.t("DAMCONNECTION_URL_TEST_SUCCESS"));
                    } else {
                        this.showMsgError(this._connectionStatusDiv, response.result.message);
                    }
                }, this),
                error: _.bind(function (response) {
                    this.showMsgError(this._connectionStatusDiv, i18n.t("DAMCONNECTION_URL_TEST_ERROR"));
                }, this)
            });
        } catch (err) {
            console.log(err);
        }
    };


    /**
     * Validates the form on submit
     */
    DamConnection.prototype.validate = function (e) {
        var errors = [];

        // DAMName should be set
        if ($("input[name=dam_name]", this.div).val().trim().length === 0) {
            errors.push(i18n.t("DAMCONNECTION_DAMNAMEISNULL_TAG"));
        }
        // DAMName should be unique
        this.checkDuplicate();
        if (this._duplicate) {
            errors.push(i18n.t("DAMCONNECTION_DAMNAMEISNOTUNIQUE_TAG"));
        }
        // DAMUrl should be set
        if ($("input[name=dam_url]").val().trim().length === 0) {
            errors.push(i18n.t("DAMCONNECTION_DAMURLISNULL_TAG"));
        }
        // Do plugin checking
        if (window.validateDAM) {
            var res = window.validateDAM();
            for (var i = 0; i < res.length; i++) {
                errors.push(res[i]);
            }
        }

        if (errors.length > 0) {
            e.preventDefault();
            alert(errors.join("\r\n"));
        }
    };


    /**
     * Goes back to the overview page
     */
    DamConnection.prototype.goBack = function () {
        window.location = "damconnection.jsp";
    };
    
    /**
     * Show error message
     * @param div - container
     * @param msg - error message
     */
    DamConnection.prototype.showMsgError = function (div, msg) {
        div.html($("<span />").text(msg));
        div.addClass('alert-danger');
        div.removeClass('alert-success').removeClass('hidden');
    };
    
    /**
     * Show success message
     * @param div - container
     * @param msg - message
     */
    DamConnection.prototype.showMsgOk = function (div, msg) {
    	div.html($("<span />").text(msg));
    	div.removeClass('alert-danger');
    	div.addClass('alert-success');
    	div.removeClass('hidden');
    };
    
    /**
     * Hide error / success message
     * @param div - container
     */
    DamConnection.prototype.hideMsg = function (div) {
    	div.addClass('hidden');
    	div.removeClass('alert-danger');
    	div.empty();
    };
    
    
    /**
	 * Init method 
	 * Apply to form element
	 * 
	 * @param form
	 */
    DamConnection.wcrinit = function(form) {
    	new DamConnection(form);
    };
        
    return DamConnection;

});
