define(["underscore", "utils/init", "utils/XMLUtils"], function (_, init, XMLUtils) {

    /**
     * This class will listen for cross-frame requests and answer them. 
     *      
     * @returns {Constructor} the constructor of CrossFrameListener
     */
    "use strict";
    var CrossFrameListener = function () {

        // Contains all registered handlers that listen to iframe messages from parent window
        this.handlers = {};
        
        // The ID of the DOM element containing search results
        this.searchResultsTableSelector = ".mainsearchresultscontrol";

        // This handler is responsible for returning the selected documents
        this._register("get_selected_files", function (e) {
            var selectedDocumentIds = _.map(this._getSelectedDocumentsOnPage(), function (selectedDocument) {
                return selectedDocument.id;
            });

            var deferreds = []; // contains a list of XHR requests that have to be executed
            var response = []; // will contain a list of selected documents and extra information (e.g. downloadurl, thumbnailurl, description, name)

            _.each(selectedDocumentIds, function (docVerId, index) {
                response[index] = {
                    thumbnailUrl: this._relPathToAbs("DocumentGetThumbnailData.jsp?docverid=" + docVerId + "&forceThumbnail=true"),
                    externalId: docVerId
                };

                // Fetch name and description
                deferreds.push(
                    this._fetchDocumentInfo(docVerId, function (result) {
                        result = XMLUtils.xml2obj(result);
                        response[index].docName = result.doc_version.document.name;
                        response[index].docDesc = result.doc_version.document.description;
                    })
                );

                // Fetch attributes
                deferreds.push(
                    this._fetchAttributeCategory(docVerId, function (result) {
                        result = XMLUtils.xml2obj(result);
                        response[index].documentMetadata = result.attribute_category;
                    })
                );
            }.bind(this));

            // Wait for all XHR requests to finish and send response back to requester
            $.when.apply($, deferreds).done(function () {
                this._sendMessage(e.source, "return_selected_files", response);
            }.bind(this));

        });
    };

    /**
     * Starts a request to fetch extra document info based on document version id
     * @private
     * @param   {String} docVerID
     * @param   {Function} callback executed when the request finishes
     * @returns {jQuery.Deferred}
     */
    CrossFrameListener.prototype._fetchDocumentInfo = function (docVerID, callback) {
        return $.get('GetDocumentVersion.jsp', {
            docversionid: docVerID
        }, callback);
    };

    CrossFrameListener.prototype._fetchAttributeCategory = function (docVerID, callback) {
        return $.get('GetAttributes.jsp', {
            docverid: docVerID
        }, callback);
    };

    /**
     * This handler is responsible for triggering the appropiate registered handlers
     * @private
     * @param {object} e postMessage event
     */
    CrossFrameListener.prototype._handleMessage = function (e) {
        if (this.handlers[e.data.command]) {
            _.each(this.handlers[e.data.command], function (handler) {
                handler(e);
            });
        }
    };

    /**
     * Sends a message to a given target with a specified command and options (optional)
     * @private
     * @param {Window} target  The target window that will receive the message
     * @param {String} command the command for this message
     * @param {Object | String} options Extra data that will be included in this message
     */
    CrossFrameListener.prototype._sendMessage = function (target, command, options) {
        target.postMessage({
            command: command,
            options: options
        }, "*");
    };


    /**
     * Registers a new handler that will be executed when a new message with the given command arrives
     * 
     * @private
     * @param {String} command
     * @param {Function} handler
     */
    CrossFrameListener.prototype._register = function (command, handler) {
        if (!this.handlers[command]) {
            this.handlers[command] = [];
        }
        this.handlers[command].push(handler.bind(this));
    };

    /**
     * Will access the Table of documents and get the selected documents. (Only works in the context of search results for documents 'docsearchresults.jsp')
     * @private
     * @returns {Array<String>} The list of selected documents on the current page
     */
    CrossFrameListener.prototype._getSelectedDocumentsOnPage = function () {
        var documents = $(this.searchResultsTableSelector)[0].SearchResultsControl.rerunsectioncontrol._actionhandler.context.getDocumentsOnPage();
        return _.filter(documents, function (doc) {
            return doc.isselected;
        });
    };

    /**
     * Converts a relative path to an absolute path
     * See: https://developer.mozilla.org/en-US/docs/Web/API/document/cookie#Using_relative_URLs_in_the_path_parameter
     * @param   {string}   sRelPath the relative path
     * @returns {string} absolute path
     */
    CrossFrameListener.prototype._relPathToAbs = function (sRelPath) {
        var nUpLn, sDir = "",
            sPath = location.pathname.replace(/[^\/]*$/, sRelPath.replace(/(\/|^)(?:\.?\/+)+/g, "$1"));
        for (var nEnd, nStart = 0; nEnd = sPath.indexOf("/../", nStart), nEnd > -1; nStart = nEnd + nUpLn) {
            nUpLn = /^\/(?:\.\.\/)*/.exec(sPath.slice(nEnd))[0].length;
            sDir = (sDir + sPath.substring(nStart, nEnd)).replace(new RegExp("(?:\\\/+[^\\\/]*){0," + ((nUpLn - 1) / 3) + "}$"), "/");
        }
        return window.location.origin + sDir + sPath.substr(nStart);
    };

    /**
     * Calling this function allows the CrossFrameListener to receive messages from his parent window. 
     */
    CrossFrameListener.prototype.attach = function () {
        window.addEventListener("message", this._handleMessage.bind(this), false);

        // wait for search results to be initialized before notifying parent window
        init.whenInitialized($(this.searchResultsTableSelector), function () {
            this._sendMessage(parent, "notify_parent");
        }.bind(this));
    };

    return CrossFrameListener;
});
