define(["component/table/Table",
		"./LandingPageRowView",
		"utils/webi18n!ContextAwareLandingPage",
		"underscore",
		"jquery",
	   "component/pagemessagecomponent/PageMessageComponent"],
	function (
		Table,
		LandingPageRowView,
		i18n,
		_,
		$,
		PageMessageComponent) {
		"use strict";

		/**
		 * Constructs a Landing Page table component
		 * @augments Table
		 * 
		 * @param {Element} context             - the wcr-table element to work on
		 * @param {String}  itemrowSelector     - describes the selector for a row in the wcr-table-section. (Most of the time this is: "> table.ordtable > tbody > tr", but not if it is "grid" style or another representation)
		 * @param {String}  checkboxSelector    - describes the selector to find a checkbox in the "row"
		 * @param {Boolean} clickToSelectActive - whether the Table implements "click to select" 
		 * @param {Boolean} isTemplate - True if the projectID is a template
		 */
		var LandingPageTable = function (context, itemrowSelector, checkboxSelector, clickToSelectActive, isTemplate) {
			Table.call(this, context, itemrowSelector, checkboxSelector, clickToSelectActive);
			
			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isTemplate = isTemplate;

			/**
			 * @member {Array.<LandingPageRowView>}
			 * @private
			 */
			this._views = [];
			
			/**
			 * @member {JQueryObj} To show where the landing pages are coming from, displayed above the table
			 * @private
			 */
			this._ruleSourceLink = $("<a>").addClass("text-info actionlink");
			this._ruleSource = $("<div>").addClass("alert alert-info").text(i18n.t("LANDING_PAGE_INFO") + " ").append(this._ruleSourceLink);

			// Do a render
			this.render();
		};

		LandingPageTable.prototype = Object.create(Table.prototype);
		LandingPageTable.prototype.constructor = LandingPageTable;

		/**
		 * Is the table for a template?
		 * @returns {Boolean} True if template, false otherwise
		 */
		LandingPageTable.prototype.isTemplate = function () {
			return this._isTemplate;
		};

		/**
		 * Render the landing page table
		 */
		LandingPageTable.prototype.render = function () {
			_.each(this._views, function (view) {
				view.destroy();
			});

			this._views = [];

			var rowdata = $("#LandingPageTable").data("landingpagerowdata");
			var bom_enabled = $("#LandingPageTable").data("bom_enabled");
			var areTasksEnabled = $("#LandingPageTable").data("areTasksEnabled");
			var templateName = $("#LandingPageTable").data("templateName");
			var templateID = $("#LandingPageTable").data("templateID");
			

			// Update the table
			var table = $("<table>").addClass("ordtable");
			if(!this.isTemplate() && templateName !== '' && templateID !== '' ){
				this._ruleSourceLink.attr("href", "templatelandingpage.jsp?menu_file=projmgmnttmplts&projectID=" + templateID).text(templateName);
				this._ruleSource.insertBefore(this._context);
			}
			
			this._tablesection.empty().append(table);

			

			var thead = $("<thead>");
			var selectAllInput = $("<input>", {
				"type": "checkbox",
				"name": "selectall",
				"value": "",
				"data-toggle-all": "input[name='selectitem']",
				"alt": i18n.t("SELECTALL")
			});
			selectAllInput.prop('disabled', !this._isTemplate);

			var selectAll = $("<th>").addClass("td-checkbox").append(selectAllInput);
			var role = $("<th>").text(i18n.t("ROLE"));
			var landingPage = $("<th>").text(i18n.t("LANDINGPAGE"));
			var landingPageDetails = $("<th>").text(i18n.t("DETAILS"));
			thead.append($("<tr>").append(selectAll).append(role).append(landingPage).append(landingPageDetails));
			table.append(thead);

			this._tbody = $("<tbody>");
			table.append(this._tbody);
			if(rowdata.length > 0){
				_.each(rowdata, function (row) {
					var landingPageView = new LandingPageRowView(row, bom_enabled, areTasksEnabled);
					landingPageView.setEnabled(this._isTemplate); // only enable for templates
					this._views.push(landingPageView);
					this._tbody.append(landingPageView.getElement());
				}, this);
			}else{
				var noResultsDiv = $("<div>").addClass("no-results").text(i18n.t("NO_LANDING_PAGES_FOUND"));
				this._tablesection.append(noResultsDiv);
			}
			

			// Handle 'leaving page with unsaved edits' message
			$('#LandingPageForm').data('serialize', $('#LandingPageForm').serialize()); // On load save form current state
			this._formSubmitting = false;
			window.removeEventListener("beforeunload", _.bind(this._informOfUnsavedChanges,this), false);
			window.addEventListener("beforeunload", _.bind(this._informOfUnsavedChanges,this), false);

			// Handle submitting the form
			$("#LandingPageForm").off("submit");
			$("#LandingPageForm").submit(_.bind(this._onSubmitting,this));

			$("#LandingPageForm").find("select.form-control").on("click", function (event) {
				event.stopPropagation(); //Prevent the row from being selected when clicking on the landing page dropdown
			});

			if ($("#LandingPageTable").data("readOnlyMode")) {
				$("#LandingPageForm").find("input.form-control,select.form-control").attr("disabled", "disabled");
			}
			
			// to make sure the action menu is also updated after re-order
			this._context.find('input[name=selectitem]').trigger('change');

			if (this._isTemplate) {
				// Initialize Sortable rows
				this._context.find('.ordtable tbody').sortable({
					items: "tr",
					placeholder: "PlaceHolderRow",
					forcePlaceholderSize: true,
					start: function (event, ui) {
						// When dragging, set the height of the helper to better indicate where the row will be placed.
						var height = (ui.item.height() + 4) + "px";
						ui.placeholder.height(height);
					}.bind(this),
					stop: function (event, ui) {
						// fire action event
						this.updateTable("reorder", null);
						
					}.bind(this),
					helper : function(e, ui) {  
						ui.children().each(function() {  
							$(this).width($(this).width());  
						});  
						return ui;  
					}
				});
			}
			
		};

		/**
		 * Determine if there are unsaved changes in the form
		 * @private
		 * @returns {boolean} 
		 */
		LandingPageTable.prototype._isDirty = function () {
			return ($('#LandingPageForm').serialize() !== $('#LandingPageForm').data('serialize')) || $("#LandingPageTable").data("isFormDirty");
		};

		/**
		 * Inform the user of the unsaved changes using a browser popup
		 * @private
		 * @param {JQuery.Event} e the event
		 * @returns {string} - In recent browsers the confirmationMessage is negelected and overwritten by the browser
		 */
		LandingPageTable.prototype._informOfUnsavedChanges = function (e) {
			//a non template page is by definition unchangeable => no warning message is needed
			if(!this.isTemplate()){
				return undefined;
			}
			
			if (this._formSubmitting || !this._isDirty()) {
				return undefined;
			}
			var confirmationMessage = i18n.t("LEAVE_PAGE_MESSAGE");
			(e || window.event).returnValue = confirmationMessage; //Gecko + IE
			return confirmationMessage; //Gecko + Webkit, Safari, Chrome etc.	
		};

		/**
		 * Handle submitting of the form
		 * @private
		 * @param {JQuery.Event} e the event
		 */
		LandingPageTable.prototype._onSubmitting = function (e) {
			this._formSubmitting = true;
			// remove old warning
			PageMessageComponent.markObsolete("SSERROR");
			PageMessageComponent.markObsolete("IGNOREAFTERDEFAULTERROR");
			// trim all fields
			$("#LandingPageForm").find("input.form-control").each(function (field) {
				$(this).val($(this).val().trim());
			});

			var rowdata = this._context.data("landingpagerowdata");
			for (var i = 0; i < rowdata.length; i++) {
				// saved search without search name
				if ($("#landingpage" + i).val() === "dosavedsearch.jsp") {
					if ($("#details" + i).val() === "") {
						$("#details" + i).css('border-color', 'red');
						PageMessageComponent.sendErrorMessage(i18n.t("SSERROR"), null, "SSERROR");
						this._formSubmitting = false;
						e.preventDefault();
					}
				}
				// empty role and not last
				if ($("#roles" + i).val() === "" && i != (rowdata.length - 1)) {
					if ($("#roles" + i).val() === "") {
						$("#roles" + i).css('border-color', 'red');
						PageMessageComponent.sendErrorMessage(i18n.t("IGNOREAFTERDEFAULTERROR"), null, "IGNOREAFTERDEFAULTERROR");
						this._formSubmitting = false;
						e.preventDefault();
					}
				}
			}
			e.stopPropagation();
		};
		
		LandingPageTable.prototype.updateTable = function (actionName, actionInfo) {
			var rowdata = this._context.data("landingpagerowdata");
			// update from form fields
			for (var j = 0; j < rowdata.length; j++) {
				var rowid = rowdata[j].id;
				var landingpage = this._context.find("#landingpage" + rowid)[0].value;
				var details = this._context.find("#details" + rowid)[0].value;
				rowdata[j].url = landingpage + "?folderName=" + encodeURIComponent(details); // hack: parameter name is not important - will get split again in LandingPageRowView
				var roles = this._context.find("#roles" + rowid)[0].value;
				rowdata[j].roles = [roles];
			}
			var newRowdata = rowdata;
			switch (actionName) {
				case "reorder":
					newRowdata = _.map(this._context.find('tr[data-row-id]'), function(element) {
						var id = parseInt($(element).attr('data-row-id'), 10);
						return _.findWhere(rowdata, {id: id});
					});
					break;
				case "delete_landing_pages":
					var rowids = actionInfo.getActionItemsAttribute("value");
					for (var k = rowids.length - 1; k >= 0; k--) {
						newRowdata.splice(rowids[k], 1);
					}
					break;
				case "new_landing_page":
					newRowdata.push({
						"roles": [""],
						"url": "projdetailsdocs.jsp",
					});
					break;
			}
			// update/set id to match current order
			_.each(newRowdata, function(row, index){
				row.id = index;
			});
			this._context.data("landingpagerowdata", newRowdata);
			this._context.data("isFormDirty", true); //to enable warning on page if leaving without saving
			this.render();
			if (actionInfo)
				actionInfo.actionmenu.updateSelectionAttributes();
		};

		/**
		 * Allow using data-init to make a LandingPageTable component.
		 */
		LandingPageTable.wcrinit = function (context) {
			if (typeof context !== $) {
				context = $(context);
			}

			// The jQuery selector to get a single row (or div)
			var itemrowSelector = context.attr("data-itemrow-selector");

			// The jQuery selector to get the checkbox inside that div
			var checkboxSelector = context.attr("data-checkbox-selector");

			// Is template?
			var isTemplate = context.attr("data-template") === "true";

			if (!itemrowSelector) {
				itemrowSelector = " > table.ordtable > tbody > tr";
			}

			if (!checkboxSelector) {
				checkboxSelector = "input[type='checkbox'][name^='select']";
			}

			// Whether click to select is active
			var clickToSelectActive = context.attr("data-click-to-select") === "0" ? false : true;

			new LandingPageTable(context, itemrowSelector, checkboxSelector, clickToSelectActive, isTemplate);
		};


		return LandingPageTable;
	});
