/**
 * This is the javascript code needed to make the fields in the table in the 
 *    Project Creation => Create From Project => Popup => Table 
 * work...
 * 
 * Why do I do it this way?
 *  - This component can contain multiple times on a page (I don't use id's)
 *  - I don't create any globals... (not polluting the global namespace)
 *    (I have a lot of internal functions)
 *  - Not possible to forget including the javascript as the javascript is attached to the html using utils/init.
 *    (The page does not need to know if the table is used.) 
 *    (The javascript is loaded automatically.)
 * 
 * @author JEPE
 */
define(["jquery", "underscore", "./CopyProjectOptionsTableTaskCopyRows"], function ($, _, CopyProjectOptionsTableTaskCopyRows) {
	/**
	 * Constructor of the component managing the project copy settings table
	 */
	function CopyProjectOptionsTable(table){
		// the <table> in which all the checkboxes and other html is located
		this.$table = $(table);
		
		// store this object
		this.$table.get(0).CopyProjectOptionsTable = this;
		
		// Find the task copy rows... (those rows have more logic)
		var $taskCopyRows = $(".taskCopyRow", this.$table);
		
		// Make a separate controller for those rows
		// But first check if those rows exist (could be that we don't have task license...)
		if($taskCopyRows.length>0) {
			this._taksrowController = new CopyProjectOptionsTableTaskCopyRows($taskCopyRows);
		} else {
			this._taksrowController = null;
		}
		
		this._initializeDependencyListeners();
	}
	
	/**
	 * (public!)
	 * When a project id is selected, the table get's initialized correctly...
	 * @param projectId The projectId we are copying from
	 */
	CopyProjectOptionsTable.prototype.setProjectId = function(projectId) {
		// if we selected a different project, reset all values...
		if(!this._projectId || this._projectId!==projectId) {
			this._projectId=projectId;
			this._resetToDefaults();
			this.validateAndSave();
			if(this._taksrowController!==null){
				this._taksrowController.setProjectId(projectId);
			}
		}
	};
	
	/**
	 * (public!)
	 * Collects and returns the options selected in the dialog as a string name=value&name=value
	 * 
	 * !!! MAKE SURE validateAndSave was called before calling this method.
	 */
	CopyProjectOptionsTable.prototype.getSelectedOptions = function() {
		return $.param(this.lastValidatedValues);
	};
	
	/**
	 * (public!)
	 * If a template is chosen, send some default values...
	 */
	CopyProjectOptionsTable.prototype.getTemplateCopyOptions = function() {
		var data = {};
		
		// Send all values for a template
		$(":input[data-valuefortemplate]", this.$table).each(function() {
			data[$(this).attr("name")] = $(this).attr("data-valuefortemplate");
		});
		
		return $.param(data);
	};
	
	/**
	 * (public!)
	 * Validate the values the user entered. (Do this before closing the popup)
	 * After calling this method you can call "getSelectedOptions" to get the settings
	 * @returns true if the user entered correct values. False if validation (and saving failed)
	 */
	CopyProjectOptionsTable.prototype.validateAndSave = function() {
		var data = {};
		var validation = this._verifyAndCollectValues(data);
		if(validation) {
			this.lastValidatedValues = data;
		}
		return validation;
	};
	
	/**
	 * (public!)
	 * If the user cancels, go back to the last valid state.
	 */
	CopyProjectOptionsTable.prototype.cancelAndRestore = function(){
		if(this.lastValidatedValues) {
			this._restoreValues(this.lastValidatedValues);
		} else {
			this.resetToDefaults();
			this.validateAndSave();
		}
	};
	
	/**
	 * Create all listeners for checkboxes which depend on another checkbox
	 */
	CopyProjectOptionsTable.prototype._initializeDependencyListeners = function() {
		// Put all checkboxes in the form in a map of name->$obj
		var allCheckboxesByName = _.reduce($("input[type='checkbox']"), function(obj, item) {
			var $item = $(item);
			obj[$item.prop("name")] = $item;
			return obj;
		}, {});
		var self = this;
		
		// Loop through all checkboxes which depend on another checkbox and add the required listeners...
		$("input[type='checkbox'][data-copyoptiondependson]", this.$table).each(function() {
			var $checkbox = $(this);
			var nameOfDependency = $checkbox.attr("data-copyoptiondependson");
			var dependency = allCheckboxesByName[nameOfDependency];
			if(dependency && dependency instanceof jQuery) {
				var callback = self._checkboxWithDependingCheckboxClicked.bind(self, dependency, $checkbox);
				dependency.click(callback);// for IE
				dependency.change(callback);
			}
		});
	};
	
	/**
	 * A listener which is invoked when a checkbox is clicked which has a depending checkbox.
	 * 
	 * When the original checkbox is not checked, also uncheck the depending checkbox and disable it.
	 * 
	 * @param $clickedCheckbox the checkbox on which the other checkbox depends
	 * @param $dependingCheckbox the checkbox with the dependency
	 */
	CopyProjectOptionsTable.prototype._checkboxWithDependingCheckboxClicked = function($clickedCheckbox, $dependingCheckbox) {
		var checked = $clickedCheckbox.prop("checked");
		$dependingCheckbox.prop("disabled", !checked);
		if(!checked) {
			$dependingCheckbox.prop("checked", false);
		}
		// also update dependencies of this checkbox
		$($dependingCheckbox).triggerHandler("change");
	};
	
	/**
	 * Reset all checkboxes to their original state...
	 */
	CopyProjectOptionsTable.prototype._resetToDefaults = function() {
		$("input[type='checkbox']", this.$table).each(function() {
			$(this).prop("checked", $(this).prop("defaultChecked"));
		});
		
		// Update all disable/enable params...
		$(":input[type='checkbox']", this.$table).each(function() {
			$(this).triggerHandler("change");
		});
	};
	
	/**
	 * Collect all the values of the inputs...
	 * 
	 * @param {Object} data the object in which the values will be put
	 * @returns {Boolean} true if the form validates correctly
	 */
	CopyProjectOptionsTable.prototype._verifyAndCollectValues = function(data) {
		var validation = true;
		
		// Collect all checkboxes...
		$(":checked[type='checkbox']", this.$table).each(function() {
			data[$(this).attr("name")] = "on";
		});
		
		// Collect the information from the task copy rows
		if(this._taksrowController){
			validation = validation & this._taksrowController.verifyAndCollectData(data);
		}
		return validation;
	};
	
	/**
	 * Restore values...
	 * 
	 * @param data the values to restore
	 */
	CopyProjectOptionsTable.prototype._restoreValues = function(data) {
		// Set all checkboxes...
		$(":input[type='checkbox']", this.$table).each(function() {
			if(data[$(this).attr("name")]==="on"){
				$(this).prop("checked", true);
			}
		});
		
		// Update all disable/enable params...
		$(":input[type='checkbox']", this.$table).each(function() {
			$(this).triggerHandler("change");
		});
		
		// Set the information from the task copy rows
		if(this._taksrowController){
			this._taksrowController.restoreData(data);
		}
	};
	
	/**
	 * Implemented as a component...
	 * 
	 * Called when initializing the component
	 * 
	 * @param table the table containing the copy table content.
	 */
	CopyProjectOptionsTable.wcrinit = function (table){
		new CopyProjectOptionsTable(table);
	};
	
	/**
	 * Implemented as a component...
	 * 
	 * Called when destroying the component
	 * 
	 * @param table the table containing the copy table content.
	 */
	CopyProjectOptionsTable.wcrdestroy = function (table){};
    
    return CopyProjectOptionsTable;
});