define(["underscore",
	   "utils/Observable"],
	function (_,
		Observable) {

		"use strict";

		/**
		 * This reprents a Rule
		 *
		 * @exports Rule
		 * @class
		 *
		 * @param {String} id
		 * @param {String} name
		 * @param {String} description
		 * @param {String} order
		 * @param {Array.<RuleTrigger>} triggers
		 * @param {Array.<RuleAction>} actions                    
		 */
		var Rule = function (id, name, description, order, enabled, triggers, actions) {
			Observable.mixin(this, "rule");

			/**
			 *  @member {String}
			 *  @private
			 */
			this._id = id;
			/**
			 *  @member {String}
			 *  @private
			 */
			this._name = name;
			/**
			 *  @member {String}
			 *  @private
			 */
			this._description = description;
			/**
			 *  @member {String}
			 *  @private
			 */
			this._order = order;
			/**
			 *  @member {Array.<RuleTrigger>}
			 *  @private
			 */
			this._triggers = triggers || [];
			/**
			 *  @member {Array.<RuleAction>}
			 *  @private
			 */
			this._actions = actions || [];

			/**
			 *  @member {Boolean}
			 *  @private
			 */
			this._enabled = enabled;
		};

		/**
		 * Get the id of the rule
		 * @returns {String}
		 */
		Rule.prototype.getID = function () {
			return this._id;
		};


		/**
		 * Get the name of the rule
		 * @returns {String}
		 */
		Rule.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Set a new name of the rule
		 * @param {String} newName
		 */
		Rule.prototype.setName = function (newName) {
			this._name = newName;
		};

		/**
		 * Get the rule description
		 * @returns {String}
		 */
		Rule.prototype.getDescription = function () {
			return this._description;
		};

		/**
		 * Set a new description of the rule
		 * @param {String} newDescription
		 */
		Rule.prototype.setDescription = function (newDescription) {
			this._description = newDescription;
		};

		/**
		 * Get the rule order
		 * @returns {String}
		 */
		Rule.prototype.getOrder = function () {
			return this._order;
		};

		/**
		 * Set a new order of the rule
		 * @param {String} newOrder
		 */
		Rule.prototype.setOrder = function (newOrder) {
			this._order = newOrder;
		};

		/**
		 * Is the rule enabled?
		 * @returns {Boolean}
		 */
		Rule.prototype.isEnabled = function () {
			return this._enabled;
		};

		/**
		 * Set a rule's enabled state
		 * @param {Boolean} newEnabled
		 */
		Rule.prototype.setEnabled = function (newEnabled) {
			this._enabled = newEnabled;
		};

		/**
		 * Get the triggers of this rule
		 * @returns {Array.<RuleTrigger>}
		 */
		Rule.prototype.getTriggers = function () {
			return this._triggers;
		};

		/**
		 * Add a new trigger to the rule
		 * @param {RuleTrigger} trigger
		 * @param {Number} [i] - At this index or at the end if not specified
		 */
		Rule.prototype.addTrigger = function (trigger, i) {
			if (i === undefined) {
				// at the end
				i = this._triggers.length;
			}
			this._triggers.splice(i, 0, trigger);
			this._emit("trigger.added", trigger, i);
		};

		/**
		 * Remove a trigger from the rule
		 * @param {RuleTrigger} trigger to remove
		 * @returns {Number} The (old) index of the removed trigger
		 */
		Rule.prototype.removeTrigger = function (trigger) {
			var i = _.indexOf(this._triggers, trigger);
			if (i >= 0) {
				this._triggers.splice(i, 1);
				this._emit("trigger.removed", trigger, i);
			}
			return i;
		};

		/**
		 * Replace a trigger with another one
		 * @param {RuleTrigger} oldTrigger - The one to remove
		 * @param {RuleTrigger} newTrigger - The one to add
		 */
		Rule.prototype.replaceTrigger = function (oldTrigger, newTrigger) {
			var i = this.removeTrigger(oldTrigger);
			this.addTrigger(newTrigger, i);
		};

		/**
		 * Get the actions of this rule
		 * @returns {Array.<RuleAction>}
		 */
		Rule.prototype.getActions = function () {
			return this._actions;
		};

		/**
		 * Add a new action to the rule
		 * @param {RuleAction} action
		 * @param {Number} [i] - At this index or at the end if not specified
		 */
		Rule.prototype.addAction = function (action, i) {
			if (i === undefined) {
				// at the end
				i = this._actions.length;
			}
			this._actions.splice(i, 0, action);
			this._emit("action.added", action, i);
		};

		/**
		 * Remove an action from the rule
		 * @param {RuleAction} action - The action to remove
		 * @returns {Number} The (old) index of the removed action
		 */
		Rule.prototype.removeAction = function (action) {
			var i = _.indexOf(this._actions, action);
			if (i >= 0) {
				this._actions.splice(i, 1);
				this._emit("action.removed", action, i);
			}
			return i;
		};

		/**
		 * Replace an action with another one
		 * @param {RuleAction} oldAction - The one to remove
		 * @param {RuleAction} newAction - The one to add
		 */
		Rule.prototype.replaceAction = function (oldAction, newAction) {
			var i = this.removeAction(oldAction);
			this.addAction(newAction, i);
		};

		/**
		 * Deep Copy this rule
		 * @returns {Rule} A deep copy of the rule
		 */
		Rule.prototype.copy = function () {
			var copyTriggers = [];
			var copyActions = [];
			_.each(this._triggers, function (trigger) {
				copyTriggers.push(trigger.copy());
			});
			_.each(this._actions, function (action) {
				copyActions.push(action.copy());
			});
			return new Rule(this._id, this._name, this._description, this._order, this._enabled, copyTriggers, copyActions);
		};

		return Rule;
	}
);