define([
	"./RuleActionView",
	"webcenterapi/TaskTypeAPI",
	"component/errorhandler/ErrorHandler",
	"component/sdrop/dataproviders/MapPromiseDataProvider",
	"component/sdrop/SDrop",
	"underscore",
	"jquery",
	"utils/webi18n!RulesTable"],
	function (
		RuleActionView,
		TaskTypeAPI,
		ErrorHandler,
		MapPromiseDataProvider,
		SDrop,
		_,
		$,
		i18n) {
		"use strict";

		/**
		 * @member {ErrorHandler}
		 * @private
		 */
		var errorHandler = new ErrorHandler();

		/**
		 * @member {TaskTypeAPI}
		 * @private
		 */
		var taskTypeAPI = new TaskTypeAPI(window.webcenterConfig, errorHandler);


		/**
		 * This represents a rule action view for starting tasks
		 *
		 * @exports RuleActionStartTaskView
		 * @class
		 * @augments RuleActionView
		 *             
		 * @param {RuleActionStartTask} ruleActionStartTask - The rule action for starting a task
		 * @param {Rule} rule - The rule where the action is part of
		 */
		var RuleActionStartTaskView = function (ruleActionStartTask, rule) {
			RuleActionView.call(this, ruleActionStartTask, rule);
			/**
			 * @member {JQueryObject} attribute select used for the sDrop
			 * @private
			 */
			this._taskTypeSelectWrapper = $("<div>").addClass("col-sm-6");
			this._taskTypeSelect = $('<select />');
			this._taskTypeSelectWrapper.append(this._taskTypeSelect);
			this._selectorWrapper.after(this._taskTypeSelectWrapper);

			/**
			 * @member {$.Deferred<Map.<WebCenterTaskType>>}
			 * @private
			 */
			this._taskTypesPromise = taskTypeAPI.getTaskTypes();

			/**
			 * @member {MapPromiseDataProvider} DataProvider for the sDrop
			 * @private
			 */
			this._dataProvider = new MapPromiseDataProvider({
				'promise': this._taskTypesPromise
			});

			/**
			 * @member {Object} options for the sDrop
			 * @private
			 */
			this._options = {
				autoload: false,
				autoloadOnFirstUsage: true,
				manageStyleAndWidth: false,
				onChangedByUser: _.bind(this._taskTypeSelected, this)
			};

			// Init
			this._taskTypesPromise.done(_.bind(function (taskTypes) {
				var currentTaskTypeID = this._action.getTaskTypeID();
				if (currentTaskTypeID && taskTypes.has(currentTaskTypeID)) {
					// only set the initial value if it is still in the task types map, it could have been deleted 
					this._options.initialValue = currentTaskTypeID;
				} else if (currentTaskTypeID) {
					// It doesn't exist in the task types map -> it has been deleted, don't save it
					this._action.setTaskTypeID(null);
				}
				this._sDrop = new SDrop(this._taskTypeSelect, "", this._dataProvider, this._options);
				this._updateEnabled();
			}, this)).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._taskTypeSelectWrapper.empty().text(errorThrown);
			}, this));
		};

		RuleActionStartTaskView.prototype = Object.create(RuleActionView.prototype);
		RuleActionStartTaskView.prototype.constructor = RuleActionStartTaskView;

		/**
		 * Executed when an attribute is selected
		 * @private
		 */
		RuleActionStartTaskView.prototype._taskTypeSelected = function () {
			var val = this._taskTypeSelect.val();
			var selected = this._sDrop.dataProvider.getById(val);
			if (selected && selected.item) {
				var taskType = selected.item;
				this._action.setTaskTypeID(taskType.getId());
			}
		};

		/**
		 * @override
		 */
		RuleActionStartTaskView.prototype._updateEnabled = function () {
			RuleActionView.prototype._updateEnabled.call(this);
			if (this._sDrop && !this._enabled) {
				this._sDrop.setDisabled(true); // there is a bug in sdrop if you set it to false, so not supported at this moment
			}
		};

		/**
		 * @override
		 */
		RuleActionStartTaskView.prototype.destroy = function () {
			RuleActionView.prototype.destroy.call(this);
			if (this._sDrop) {
				this._sDrop.destroy();
			}
		};

		/**
		 * @override
		 */
		RuleActionStartTaskView.prototype.validate = function () {
			if (!this._action.getTaskTypeID()) {
				window.alert(i18n.t("RULE_ACTION_STARTTASK_REQUIRED"));
				return false;
			}
			return RuleActionView.prototype.validate.call(this);
		};

		return RuleActionStartTaskView;
	});