define([
	"./RuleFactory",
	"./RuleConstants",
	"utils/webi18n!RulesTable",
	"underscore"],
	function (
		RuleFactory,
		RuleConstants,
		i18n,
		_) {
		"use strict";

		/**
		 * This represents a rule action view (a single row)
		 *
		 * @exports RuleActionView
		 * @class
		 *             
		 * @param {RuleAction} action - The rule action
		 * @param {Rule} rule - The rule where the action is part of
		 */
		var RuleActionView = function (action, rule) {
			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {RuleAction}
			 * @private 
			 */
			this._action = action;

			/**
			 * @member {RuleFactory}
			 * @private 
			 */
			this._ruleFactory = new RuleFactory();

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<div>").addClass("row ruleActionView");

			/**
			 * @member {JQueryObj}
			 * @protected 
			 */
			this._selectorWrapper = $("<div>").addClass("col-md-5 select");
			this._selector = $("<select>").addClass("form-control");
			_.each(RuleConstants.ActionType, function (actionType) {
					var option = $("<option>", {
						'value': actionType
					}).text(i18n.t(("RULE_ACTION_" + actionType).toUpperCase()));
					if (actionType === this._action.getType()) {
						option.prop('selected', true);
					}
					this._selector.append(option);
				},
				this);
			this._selector.on("change", _.bind(function (event) {
				this._changeActionType();
			}, this));
			this._selectorWrapper.append(this._selector);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._removeButton = $("<div>").addClass("col-md-1 remove").append(
				$("<i>").addClass("icon icon-normal icon-WCR_Filter-Remove12").click(
					_.bind(function () {
						this._removeAction();
					}, this)));


			this._element.append(this._removeButton).append(this._selectorWrapper);
		};

		/**
		 * Called when the action is changed to another type
		 * @private
		 */
		RuleActionView.prototype._changeActionType = function () {
			var newAction = this._ruleFactory.createAction(this._selector.val());
			this._rule.replaceAction(this._action, newAction);
		};

		/**
		 * Called when remove action button is clicked
		 * @private
		 */
		RuleActionView.prototype._removeAction = function () {
			this._rule.removeAction(this._action);
		};

		/**
		 * Get the element that represents this view
		 * @returns {JQueryObj}
		 */
		RuleActionView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the trigger view needs to be destroyed
		 */
		RuleActionView.prototype.destroy = function () {
			// override if needed
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RuleActionView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @protected
		 */
		RuleActionView.prototype._updateEnabled = function () {
			this._removeButton.toggle(this._enabled);
			this._selector.prop("disabled", !this._enabled);
		};

		/**
		 * Return true if all input is validated and show alert messages if needed
		 * @returns {Boolean} True if ok (default), false otherwise
		 */
		RuleActionView.prototype.validate = function () {
			return true;
		};

		return RuleActionView;
	}
);