define(["./RuleFactory",
		"./RuleViewFactory",
		"./RuleConstants",
		"utils/webi18n!RulesTable",
		"utils/Observer",
		"underscore",
	   	"jquery"],
	function (
		RuleFactory,
		RuleViewFactory,
		RuleConstants,
		i18n,
		Observer,
		_,
		$) {
		"use strict";

		/**
		 * This represents a rule actions view (for adding multiple actions)
		 *
		 * @exports RuleActionsView
		 * @class
		 *             
		 * @param {Rule} rule - The rule to get the actions from
		 */
		var RuleActionsView = function (rule) {
			Observer.mixin(this);

			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {RuleFactory}
			 * @private 
			 */
			this._ruleFactory = new RuleFactory();

			/**
			 * @member {RuleViewFactory}
			 * @private 
			 */
			this._ruleViewFactory = new RuleViewFactory();

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<div>").addClass("form-entry ruleActions");

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._addButton = $("<div>").addClass("form-group triggerAdd").append(
				$("<a>").addClass("AddBtn btn btn-sm btn-primary").append(
					$("<i>").addClass("icon icon-WCR_Actions-Add icon-small"))).click(_.bind(function () {
				this._addAction();
			}, this));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._actionViewsElement = $("<div>").addClass("form-group ruleActionViews");

			// Add the elements
			this._element.append(this._actionViewsElement);
			this._element.append(this._addButton);

			// Listen
			this._observe(rule, "action.added", this._actionAdded);
			this._observe(rule, "action.removed", this._actionRemoved);

			/**
			 * @member {Array.<RuleActionView>}
			 * @private 
			 */
			this._actionViews = [];
			if (_.size(this._rule.getActions()) > 0) {
				_.each(this._rule.getActions(), function (action, i) {
					this._actionAdded(action, i);
				}, this);
			} else {
				// add at least 1
				this._addAction();
			}
		};

		/**
		 * Get the element
		 * @returns {JQueryObj}
		 */
		RuleActionsView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the view needs to be destroyed
		 */
		RuleActionsView.prototype.destroy = function () {
			_.each(this._actionViews, function (actionView) {
				actionView.destroy();
			});
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RuleActionsView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @private
		 */
		RuleActionsView.prototype._updateEnabled = function () {
			this._addButton.toggle(this._enabled);

			_.each(this._actionViews, function (actionView) {
				return actionView.setEnabled(this._enabled);
			}, this);
		};


		/**
		 * Called when an action is added
		 * @private
		 */
		RuleActionsView.prototype._actionAdded = function (action, i) {
			var newActionView = this._ruleViewFactory.createActionViewFor(action, this._rule);
			this._actionViews.splice(i, 0, newActionView);

			if (i === 0) {
				this._actionViewsElement.prepend(newActionView.getElement());
			} else {
				this._actionViewsElement.children().eq(i - 1).after(newActionView.getElement());
			}
		};

		/**
		 * Called when an action is removed
		 * @private
		 */
		RuleActionsView.prototype._actionRemoved = function (action, i) {
			var oldActionView = this._actionViews[i];
			if (oldActionView) {
				oldActionView.destroy();
				oldActionView.getElement().remove();
			}
			this._actionViews.splice(i, 1);
		};

		/**
		 * Add a new (empty) action
		 * @private
		 */
		RuleActionsView.prototype._addAction = function () {
			this._rule.addAction(this._ruleFactory.createAction(RuleConstants.ActionType.STARTTASK));
		};

		/**
		 * Validate the entered data
		 */
		RuleActionsView.prototype.validate = function () {
			if (_.size(this._rule.getActions()) <= 0) {
				window.alert(i18n.t("RULEACTIONREQUIRED"));
				return false;
			}
			return _.every(this._actionViews, function (actionView) {
				return actionView.validate();
			});
		};

		return RuleActionsView;
	}
);