define(["jquery",
        "underscore",
		"./RuleView",
        "component/modal/Modal",
	   	"utils/webi18n!RulesTable",
	   	"lib/PromisePolyfill"],
	function ($,
		_,
		RuleView,
		ModalPopup,
		i18n) {

		"use strict";


		/**
		 * This reprents the popup in which a rule is shown.
		 *
		 * @exports RulePopupView
		 * @class
		 */
		var RulePopupView = function () {
			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._addButton = null;
			this._cancelButton = null;

			/**
			 * @member {Function} - Returns promises
			 * @private 
			 */
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];

			// Handlers to execute respective callbacks when add or cancel button is clicked
			this._addClickHandler = _.bind(function (event, data) {
				return Promise.all(_.map(this._onAddCallbacks, function (callback) {
					return callback(data);
				}));
			}, this);

			this._cancelClickHandler = _.bind(function (event) {
				return Promise.all(_.map(this._onCancelCallbacks, function (callback) {
					return callback();
				}));
			}, this);

			// Create the popup
			this._constructPopup();
		};

		/**
		 * Create the modal popup
		 */
		RulePopupView.prototype._constructPopup = function () {
			// Create only one instance of popup.
			if (!this._popup) {
				// Get the popup html content
				this._popupContent = $("<div>").text(i18n.t("RULELOADING"));

				// create the popup using model popup
				this._popup = new ModalPopup(this._popupContent, this._options);
				this._popup.getJQElement().addClass("rulePopupView");
				this._popup.setFooter(this._getFooterContent());
			}
		};

		/**
		 * Create add and cancel buttons of the popup
		 * @returns {JQueryObj}
		 */
		RulePopupView.prototype._getFooterContent = function () {
			var footerButtons = $('<div>').addClass("btn-group").attr({
				'role': 'group',
				'name': 'ActiveButtons'
			});

			// Button to add
			this._addButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("RULESAVE")
			}).addClass("btn btn-primary");
			this._addButton.click(_.bind(function (event) {
				if (this._ruleView.validate()) {
					var ruleData = this._ruleView.getRule();
					this._addClickHandler(event, ruleData).then(_.bind(function () {
						this._popup.close();
					}, this));
				}
			}, this));

			// Button to cancel the popup
			this._cancelButton = $('<input>').attr({
				'type': 'button',
				'value': i18n.t("RULECANCEL")
			}).addClass("btn btn-default add-spacing-right");
			this._cancelButton.click(_.bind(function (event) {
				this._cancelClickHandler(event).then(_.bind(function () {
					this._popup.close();
				}, this));
			}, this));

			footerButtons.append(this._cancelButton, this._addButton);

			return footerButtons;
		};

		/**
		 * Method to reset/initalize and open the popup
		 */
		RulePopupView.prototype.show = function (data) {
			// Initialize the popup
			this._initPopup(data);
			// Open the modal popup
			this._popup.open();
		};

		/**
		 * Initialize the popup
		 */
		RulePopupView.prototype._initPopup = function (data) {
			if (this._ruleView) {
				this._ruleView.destroy();
			}

			this._ruleView = new RuleView(data.rule);
			this.setEnabled(data.enabled);

			this._popup.setContent(this._ruleView.getElement());
			this._popup.setTitle(data.title);
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RulePopupView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @private
		 */
		RulePopupView.prototype._updateEnabled = function () {
			if (this._ruleView) {
				this._ruleView.setEnabled(this._enabled);
			}
			if (this._addButton) {
				this._addButton.toggle(this._enabled);
			}
		};

		/**
		 * To add callback for add button
		 * @param {Function} callback - The callback function should return a Promise!
		 */
		RulePopupView.prototype.onSubmit = function (callback) {
			this._onAddCallbacks.push(callback);
		};

		/**
		 * To add callback for cancel button
		 * @param {Function} callback - The callback function should return a Promise!
		 */
		RulePopupView.prototype.onCancel = function (callback) {
			this._onCancelCallbacks.push(callback);
		};

		/**
		 * Clear all listeners registered for cancel/add buttons
		 */
		RulePopupView.prototype.clearListeners = function () {
			this._onAddCallbacks = [];
			this._onCancelCallbacks = [];
		};

		return RulePopupView;
	}
);