define([
	"./RuleTriggerView",
	"webcenterapi/AttributesAPI",
	"component/errorhandler/ErrorHandler",
	"component/sdrop/dataproviders/MapPromiseDataProvider",
	"component/sdrop/SDrop",
	"underscore",
	"jquery",
	"utils/webi18n!RulesTable"],
	function (
		RuleTriggerView,
		AttributesAPI,
		ErrorHandler,
		MapPromiseDataProvider,
		SDrop,
		_,
		$,
		i18n) {
		"use strict";

		/**
		 * @member {ErrorHandler}
		 * @private
		 */
		var errorHandler = new ErrorHandler();

		/**
		 * @member {AttributesAPI}
		 * @private
		 */
		var attributesAPI = new AttributesAPI(window.webcenterConfig, errorHandler);

		/**
		 * This represents a rule trigger for project attributes view
		 *
		 * @exports RuleTriggerProjectAttributeView
		 * @class
		 * @augments RuleTriggerView
		 *             
		 * @param {RuleTriggerProjectAttribute} ruleTriggerProjectAttribute - The rule trigger for a project attribute
		 * @param {Rule} rule - The rule where the trigger is part of
		 */
		var RuleTriggerProjectAttributeView = function (ruleTriggerProjectAttribute, rule) {
			RuleTriggerView.call(this, ruleTriggerProjectAttribute, rule);
			/**
			 * @member {JQueryObject} attribute select used for the sDrop
			 * @private
			 */
			this._attributeSelectWrapper = $("<div>").addClass("col-sm-6");
			this._attributeSelect = $('<select />');
			this._attributeSelectWrapper.append(this._attributeSelect);
			this._selectorWrapper.after(this._attributeSelectWrapper);

			/**
			 * @member {$.Deferred<Map.<WebCenterAttribute>>}
			 * @private
			 */
			this._attributesPromise = attributesAPI.getAttributes();

			/**
			 * @member {MapPromiseDataProvider} DataProvider for the sDrop
			 * @private
			 */
			this._dataProvider = new MapPromiseDataProvider({
				'promise': this._attributesPromise
			});

			/**
			 * @member {Object} options for the sDrop
			 * @private
			 */
			this._options = {
				enableEmptySelection: false,
				manageStyleAndWidth: false,
				onChangedByUser: _.bind(this._attributeSelected, this)
			};

			// Init
			this._attributesPromise.done(_.bind(function (attributes) {
				var currentAttributeId = this._trigger.getAttributeID();
				if (currentAttributeId && attributes.has(currentAttributeId)) {
					// only set the initial value if it is still in the attributes map, it could have been deleted 
					this._options.initialValue = currentAttributeId;
				} else if (currentAttributeId) {
					// The currentAttribute doesn't exist in the attributes map -> it has been deleted, don't save it
					this._trigger.setAttributeID(null);
				}
				this._sDrop = new SDrop(this._attributeSelect, "", this._dataProvider, this._options);
				this._updateEnabled();
			}, this)).fail(_.bind(function (jqXHR, textStatus, errorThrown) {
				this._attributeSelectWrapper.empty().text(errorThrown);
			}, this));
		};

		RuleTriggerProjectAttributeView.prototype = Object.create(RuleTriggerView.prototype);
		RuleTriggerProjectAttributeView.prototype.constructor = RuleTriggerProjectAttributeView;

		/**
		 * Executed when an attribute is selected
		 * @private
		 */
		RuleTriggerProjectAttributeView.prototype._attributeSelected = function () {
			var val = this._attributeSelect.val();
			var selected = this._sDrop.dataProvider.getById(val);
			if (selected && selected.item) {
				var attr = selected.item;
				this._trigger.setAttributeID(attr.getId());
			}
		};

		/**
		 * @override
		 */
		RuleTriggerProjectAttributeView.prototype._updateEnabled = function () {
			RuleTriggerView.prototype._updateEnabled.call(this);
			if (this._sDrop && !this._enabled) {
				this._sDrop.setDisabled(true); // there is a bug in sdrop if you set it to false, so not supported at this moment
			}
		};

		/**
		 * @override
		 */
		RuleTriggerProjectAttributeView.prototype.destroy = function () {
			RuleTriggerView.prototype.destroy.call(this);
			if (this._sDrop) {
				this._sDrop.destroy();
			}
		};

		/**
		 * @override
		 */
		RuleTriggerProjectAttributeView.prototype.validate = function () {
			if (!this._trigger.getAttributeID()) {
				window.alert(i18n.t("RULE_TRIGGER_PROJECTATTRIBUTE_REQUIRED"));
				return false;
			}
			return RuleTriggerView.prototype.validate.call(this);
		};

		return RuleTriggerProjectAttributeView;
	}
);