define([
	"./RuleFactory",
	"./RuleConstants",
	"utils/webi18n!RulesTable",
	"underscore"],
	function (
		RuleFactory,
		RuleConstants,
		i18n,
		_) {
		"use strict";

		/**
		 * This represents a rule trigger view (a single row)
		 *
		 * @exports RuleTriggerView
		 * @class
		 *             
		 * @param {RuleTrigger} trigger - The rule trigger
		 * @param {Rule} rule - The rule where the trigger is part of
		 */
		var RuleTriggerView = function (trigger, rule) {
			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {RuleTrigger}
			 * @private 
			 */
			this._trigger = trigger;

			/**
			 * @member {RuleFactory}
			 * @private 
			 */
			this._ruleFactory = new RuleFactory();

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<div>").addClass("row ruleTriggerView");

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._selectorWrapper = $("<div>").addClass("col-md-5 select");
			this._selector = $("<select>").addClass("form-control");
			_.each(RuleConstants.TriggerType, function (triggerType) {
					var option = $("<option>", {
						'value': triggerType
					}).text(i18n.t(("RULE_TRIGGER_" + triggerType).toUpperCase()));
					if (triggerType === this._trigger.getType()) {
						option.prop('selected', true);
					}
					this._selector.append(option);
				},
				this);
			this._selector.on("change", _.bind(function (event) {
				this._changeTriggerType();
			}, this));
			this._selectorWrapper.append(this._selector);

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._removeButton = $("<div>").addClass("col-md-1 remove").append(
				$("<i>").addClass("icon icon-normal icon-WCR_Filter-Remove12").click(
					_.bind(function () {
						this._removeTrigger();
					}, this)));


			this._element.append(this._removeButton).append(this._selectorWrapper);
		};

		/**
		 * Called when the trigger is changed to another type
		 * @private
		 */
		RuleTriggerView.prototype._changeTriggerType = function () {
			var newTrigger = this._ruleFactory.createTrigger(this._selector.val());
			this._rule.replaceTrigger(this._trigger, newTrigger);
		};

		/**
		 * Called when remove trigger button is clicked
		 * @private
		 */
		RuleTriggerView.prototype._removeTrigger = function () {
			this._rule.removeTrigger(this._trigger);
		};

		/**
		 * Get the element that represents this view
		 * @returns {JQueryObj}
		 */
		RuleTriggerView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the trigger view needs to be destroyed
		 */
		RuleTriggerView.prototype.destroy = function () {
			// override if needed
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RuleTriggerView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @protected
		 */
		RuleTriggerView.prototype._updateEnabled = function () {
			this._removeButton.toggle(this._enabled);
			this._selector.prop("disabled", !this._enabled);
		};


		/**
		 * Return true if all input is validated and show alert messages if needed
		 * @returns {Boolean} True if ok (default), false otherwise
		 */
		RuleTriggerView.prototype.validate = function () {
			return true;
		};

		return RuleTriggerView;
	});