define(["./RuleFactory",
		"./RuleViewFactory",
		"./RuleConstants",
		"utils/webi18n!RulesTable",
		"utils/Observer",
		"underscore",
	   	"jquery"],
	function (
		RuleFactory,
		RuleViewFactory,
		RuleConstants,
		i18n,
		Observer,
		_,
		$) {
		"use strict";

		/**
		 * This represents a rule triggers view (for adding multiple triggers)
		 *
		 * @exports RuleTriggersView
		 * @class
		 *             
		 * @param {Rule} rule - The rule to get the triggers from
		 */
		var RuleTriggersView = function (rule) {
			Observer.mixin(this);

			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {RuleFactory}
			 * @private 
			 */
			this._ruleFactory = new RuleFactory();

			/**
			 * @member {RuleViewFactory}
			 * @private 
			 */
			this._ruleViewFactory = new RuleViewFactory();

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<div>").addClass("form-entry ruleTriggers");

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._addButton = $("<div>").addClass("form-group triggerAdd").append(
				$("<a>").addClass("AddBtn btn btn-sm btn-primary").append(
					$("<i>").addClass("icon icon-WCR_Actions-Add icon-small")).click(_.bind(function () {
					this._addTrigger();
				}, this)));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._triggerViewsElement = $("<div>").addClass("form-group ruleTriggerViews");

			// Add the elements
			this._element.append(this._triggerViewsElement);
			this._element.append(this._addButton);

			// Listen
			this._observe(rule, "trigger.added", this._triggerAdded);
			this._observe(rule, "trigger.removed", this._triggerRemoved);

			/**
			 * @member {Array.<RuleTriggerView>}
			 * @private 
			 */
			this._triggerViews = [];
			if (_.size(this._rule.getTriggers()) > 0) {
				_.each(this._rule.getTriggers(), function (trigger, i) {
					this._triggerAdded(trigger, i);
				}, this);
			} else {
				// add at least 1
				this._addTrigger();
			}
		};

		/**
		 * Get the element
		 * @returns {JQueryObj}
		 */
		RuleTriggersView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the view needs to be destroyed
		 */
		RuleTriggersView.prototype.destroy = function () {
			_.each(this._triggerViews, function (triggerView) {
				triggerView.destroy();
			});
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RuleTriggersView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @private
		 */
		RuleTriggersView.prototype._updateEnabled = function () {
			this._addButton.toggle(this._enabled);

			_.each(this._triggerViews, function (triggerView) {
				return triggerView.setEnabled(this._enabled);
			}, this);
		};

		/**
		 * @private
		 */
		RuleTriggersView.prototype._triggerAdded = function (trigger, i) {
			var newTriggerView = this._ruleViewFactory.createTriggerViewFor(trigger, this._rule);
			this._triggerViews.splice(i, 0, newTriggerView);

			if (i === 0) {
				this._triggerViewsElement.prepend(newTriggerView.getElement());
			} else {
				this._triggerViewsElement.children().eq(i - 1).after(newTriggerView.getElement());
			}
		};

		/**
		 * @private
		 */
		RuleTriggersView.prototype._triggerRemoved = function (trigger, i) {
			var oldTriggerView = this._triggerViews[i];
			if (oldTriggerView) {
				oldTriggerView.destroy();
				oldTriggerView.getElement().remove();
			}
			this._triggerViews.splice(i, 1);
		};

		/**
		 * Add a new trigger
		 * @private
		 */
		RuleTriggersView.prototype._addTrigger = function () {
			this._rule.addTrigger(this._ruleFactory.createTrigger(RuleConstants.TriggerType.PROJECTATTRIBUTE));
		};

		/**
		 * Validate the entered data
		 */
		RuleTriggersView.prototype.validate = function () {
			if (_.size(this._rule.getTriggers()) <= 0) {
				window.alert(i18n.t("RULETRIGGERREQUIRED"));
				return false;
			}
			return _.every(this._triggerViews, function (triggerView) {
				return triggerView.validate();
			});
		};

		return RuleTriggersView;
	}
);