define(["./RuleTriggersView",
		"./RuleActionsView",
		"utils/webi18n!RulesTable",
		"underscore",
	   	"jquery",
	   	"utils/webcss!./ruleview-style.css"],
	function (
		RuleTriggersView,
		RuleActionsView,
		i18n,
		_,
		$) {
		"use strict";

		/**
		 * This represents a rule view
		 *
		 * @exports RuleView
		 * @class
		 * 
		 * @param {Rule} rule - The rule for which this view is
		 */
		var RuleView = function (rule) {
			/**
			 * @member {Rule}
			 * @private 
			 */
			this._rule = rule;

			/**
			 * @member {Boolean}
			 * @private 
			 */
			this._enabled = true;

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._element = $("<div>").addClass("container-fluid rule");

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleName = $("<div>").addClass("form-group");
			this._ruleNameInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"name": "ruleName",
				"maxlength": "256", // DB column
				"value": this._rule.getName()
			}).on("change", _.bind(function (e) {
				this._rule.setName($(e.target).val());
			}, this));
			this._ruleNameLabel = $("<label>", {
				"for": "ruleName",
				"class": "form-required"
			}).text(i18n.t("RULENAME"));
			this._ruleName.append(this._ruleNameLabel).append($("<div>").addClass("form-entry").append(this._ruleNameInput));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleDescription = $("<div>").addClass("form-group");
			this._ruleDescriptionInput = $("<textarea>", {
				"cols": "75",
				"rows": "3",
				"class": "form-control",
				"name": "ruleDescription",
				"maxlength": 256, //DB column
				"value": this._rule.getDescription()
			}).on("change", _.bind(function (e) {
				this._rule.setDescription($(e.target).val());
			}, this));
			this._ruleDescriptionLabel = $("<label>", {
				"for": "ruleDescription"
			}).text(i18n.t("RULEDESCRIPTION"));
			this._ruleDescription.append(this._ruleDescriptionLabel).append($("<div>").addClass("form-entry").append(this._ruleDescriptionInput));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleOrder = $("<div>").addClass("form-group");
			this._ruleOrderInput = $("<input>", {
				"type": "text",
				"class": "form-control",
				"name": "ruleOrder",
				"value": this._rule.getOrder()
			}).attr("size", "3").on("change", _.bind(function (e) {
				this._rule.setOrder($(e.target).val());
			}, this));
			this._ruleOrderLabel = $("<label>", {
				"for": "ruleOrder"
			}).text(i18n.t("RULEORDER"));
			this._ruleOrder.append(this._ruleOrderLabel).append($("<div>").addClass("form-entry").append(this._ruleOrderInput));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleEnabled = $("<div>").addClass("form-group");
			this._ruleEnabledInput = $("<input>", {
				"type": "checkbox",
				"name": "ruleEnabled"
			}).prop("checked", this._rule.isEnabled()).on("change", _.bind(function (e) {
				this._rule.setEnabled($(e.target).prop("checked"));
			}, this));
			this._ruleEnabledLabel = $("<label>", {
				"for": "ruleEnabled"
			}).text(i18n.t("RULEENABLED"));
			this._ruleEnabled.append(this._ruleEnabledLabel).append($("<div>").addClass("form-entry").append(this._ruleEnabledInput));

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleTriggersWrapper = $("<div>").addClass("form-group");
			this._ruleTriggers = $("<fieldset>");
			this._ruleTriggersWrapper.append(this._ruleTriggers);
			this._ruleTriggersView = new RuleTriggersView(this._rule);
			this._ruleTriggersLabel = $("<legend>").text(i18n.t("RULETRIGGERS"));
			this._ruleTriggers.append(this._ruleTriggersLabel).append(this._ruleTriggersView.getElement());

			/**
			 * @member {JQueryObj}
			 * @private 
			 */
			this._ruleActionsWrapper = $("<div>").addClass("form-group");
			this._ruleActions = $("<fieldset>");
			this._ruleActionsWrapper.append(this._ruleActions);
			this._ruleActionsView = new RuleActionsView(this._rule);
			this._ruleActionsLabel = $("<legend>").text(i18n.t("RULEACTIONS"));
			this._ruleActions.append(this._ruleActionsLabel).append(this._ruleActionsView.getElement());

			this._element.append(this._ruleName).append(this._ruleEnabled).append(this._ruleDescription).append(this._ruleOrder).append(this._ruleTriggersWrapper).append(this._ruleActionsWrapper);
		};

		/**
		 * Get the rule for which this view was created
		 * @returns {Rule}
		 */
		RuleView.prototype.getRule = function () {
			return this._rule;
		};

		/**
		 * Get the element
		 * @returns {JQueryObj}
		 */
		RuleView.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Called when the view needs to be destroyed
		 */
		RuleView.prototype.destroy = function () {
			this._ruleTriggersView.destroy();
			this._ruleActionsView.destroy();
		};

		/**
		 * Set the view enabled for input or not
		 * @param {Boolean} enabled - True if enabled, false if disabled
		 */
		RuleView.prototype.setEnabled = function (enabled) {
			this._enabled = enabled;
			this._updateEnabled();
		};

		/**
		 * Update the views based on the enabled flag
		 * @private
		 */
		RuleView.prototype._updateEnabled = function () {
			this._ruleNameInput.prop("disabled", !this._enabled);
			this._ruleDescriptionInput.prop("disabled", !this._enabled);
			this._ruleOrderInput.prop("disabled", !this._enabled);
			this._ruleEnabledInput.prop("disabled", !this._enabled);
			this._ruleTriggersView.setEnabled(this._enabled);
			this._ruleActionsView.setEnabled(this._enabled);
		};

		/**
		 * Validate the entered input and show alert dialogs
		 * @returns {Boolean} True if all entered data is ok, false otherwise
		 */
		RuleView.prototype.validate = function () {
			if (!this._rule.getName()) {
				window.alert(i18n.t("RULENAMEREQUIRED"));
				return false;
			}
			var order = this._rule.getOrder();
			if (order !== undefined) {
				if (!/^[1-9][0-9]*$/.test(order)) { // an invalid integer
					window.alert(i18n.t("RULEORDERINVALID"));
					return false;
				}
			} else {
				window.alert(i18n.t("RULEORDERINVALID"));
				return false;
			}
			return this._ruleTriggersView.validate() && this._ruleActionsView.validate();
		};

		return RuleView;
	});