define(["component/table/Table",
		"component/errorhandler/ErrorHandler",
		"webcenterapi/RuleEngineAPI",
		"./RuleRowView",
		"utils/webi18n!RulesTable",
		"component/pagemessagecomponent/PageMessageComponent",
		"underscore",
		"jquery"],
	function (
		Table,
		ErrorHandler,
		RuleEngineAPI,
		RuleRowView,
		i18n,
		PageMessageComponent,
		_,
		$) {
		"use strict";

		/**
		 * Constructs a Rules table component
		 * @augments Table
		 * 
		 * @param {Element} context             - the wcr-table element to work on
		 * @param {String}  itemrowSelector     - describes the selector for a row in the wcr-table-section. (Most of the time this is: "> table.ordtable > tbody > tr", but not if it is "grid" style or another representation)
		 * @param {String}  checkboxSelector    - describes the selector to find a checkbox in the "row"
		 * @param {Boolean} clickToSelectActive - whether the Table implements "click to select" 
		 * @param {String} projectID - The id of the project of this table
		 * @param {Boolean} isTemplate - True if the projectID is a template
		 */
		var RulesTable = function (context, itemrowSelector, checkboxSelector, clickToSelectActive, projectID, isTemplate) {
			Table.call(this, context, itemrowSelector, checkboxSelector, clickToSelectActive);

			/**
			 * @member {ErrorHandler}
			 * @private
			 */
			this._errorHandler = new ErrorHandler(); // this is not used in rule engine API for now, so no need to add listeners
			/**
			 * @member {RuleEngineAPI}
			 * @private
			 */
			this._ruleEngineAPI = new RuleEngineAPI(window.webcenterConfig, this._errorHandler);

			/**
			 * @member {String}
			 * @private
			 */
			this._projectID = projectID;

			/**
			 * @member {Boolean}
			 * @private
			 */
			this._isTemplate = isTemplate;

			/**
			 * @member {Array.<RuleRowView>}
			 * @private
			 */
			this._views = [];

			/**
			 * @member {JQueryObj} To show there the rules are coming from, displayed above the table
			 * @private
			 */
			this._ruleSourceLink = $("<a>").addClass("text-info actionlink");
			this._ruleSource = $("<div>").addClass("alert alert-info").text(i18n.t("RULEOWNER") + " ").append(this._ruleSourceLink);

			/**
			 * @member {JQueryObj} The select all checkbox
			 * @private
			 */
			this._selectAllInput = null;

			// Do a render
			this.render();
		};

		RulesTable.prototype = Object.create(Table.prototype);
		RulesTable.prototype.constructor = RulesTable;

		/**
		 * Returns the current maximum order
		 * @returns {Number}
		 */
		RulesTable.prototype.getMaxOrder = function () {
			if (_.size(this._views) > 0) {
				var max = _.max(this._views, function (view) {
					return view.getRule().getOrder();
				});
				return max.getRule().getOrder();
			}
			return 0;
		};

		/**
		 * Get the project ID
		 * @returns {String}
		 */
		RulesTable.prototype.getProjectID = function () {
			return this._projectID;
		};

		/**
		 * Is the table for a template?
		 * @returns {Boolean} True if template, false otherwise
		 */
		RulesTable.prototype.isTemplate = function () {
			return this._isTemplate;
		};

		/**
		 * Clear the table.
		 * This destroys all views and sets the section to loading.
		 */
		RulesTable.prototype._clear = function () {
			_.each(this._views, function (view) {
				view.destroy();
			});

			this._views = [];
			this._tablesection.empty();

			this._tablesection.text(i18n.t("RULELOADING"));
		};

		/**
		 * Set all input and actions enabled of this table or not
		 * @param {Boolean} enabled - True for enabled, false for disabled
		 */
		RulesTable.prototype.setEnabled = function (enabled) {
			if (this._selectAllInput) {
				this._selectAllInput.prop("disabled", !enabled);
			}

			_.each(this._views, function (view) {
				view.setEnabled(enabled);
			});
		};

		/**
		 * Re-render the rules table
		 */
		RulesTable.prototype.render = function () {
			this._clear();

			this._ruleEngineAPI.getRules(this.getProjectID()).then(_.bind(function (ruleSet) {
				// Update the source link
				if (!this._isTemplate && ruleSet.owner.name) {
					this._ruleSourceLink.attr("href", "projectrulesetup.jsp?menu_file=projmgmnttmplts&projectID=" + ruleSet.owner.id).text(ruleSet.owner.name);
					this._ruleSource.insertBefore(this._context);
				}


				// Update the table
				var table = $("<table>").addClass("ordtable");
				this._tablesection.empty().append(table);

				var thead = $("<thead>");
				table.append(thead);
				var tr = $("<tr>");
				thead.append(tr);

				if (this._isTemplate) {
					this._selectAllInput = $("<input>", {
						"type": "checkbox",
						"name": "selectall",
						"value": "",
						"data-toggle-all": "input[name='selectitem']",
						"alt": i18n.t("RULESELECTALL")
					});
					this._selectAllInput.prop('disabled', !this._isTemplate);
					var selectAll = $("<th>").addClass("td-checkbox").append(this._selectAllInput);
					tr.append(selectAll);
				}

				var order = $("<th>").text(i18n.t("RULEORDER"));
				var ruleName = $("<th>").text(i18n.t("RULENAME"));
				var ruleDescription = $("<th>").text(i18n.t("RULEDESCRIPTION"));
				var ruleEnabled = $("<th>").text(i18n.t("RULEENABLED"));

				tr.append(order).append(ruleName).append(ruleDescription).append(ruleEnabled);


				if (ruleSet.rules.length === 0) {
					var noResultsDiv = $("<div>").addClass("no-results").text(i18n.t("RULESNOTFOUND"));
					this._tablesection.append(noResultsDiv);
				} else {
					var tbody = $("<tbody>");
					table.append(tbody);
					_.each(ruleSet.rules, function (rule) {
						var rowView = new RuleRowView(rule, {
							enabled: this._isTemplate // only enable for templates
						});
						this._views.push(rowView);
						tbody.append(rowView.getElement());
					}, this);
				}

			}, this)).catch(_.bind(function (errorString) {
				this._tablesection.empty();
				PageMessageComponent.sendErrorMessage(i18n.t("RULELOADINGFAILED") + (errorString ? (" " + errorString) : ""));
			}, this));
		};

		/**
		 * Allow using data-init to make a RulesTable component.
		 */
		RulesTable.wcrinit = function (context) {
			if (typeof context !== $) {
				context = $(context);
			}

			// The jQuery selector to get a single row (or div)
			var itemrowSelector = context.attr("data-itemrow-selector");

			// The jQuery selector to get the checkbox inside that div
			var checkboxSelector = context.attr("data-checkbox-selector");

			// The project id
			var projectID = context.attr("data-projectid");

			// Is template?
			var isTemplate = context.attr("data-template") === "true";

			// The feedback that has to be displayed after importing
			var feedback = context.attr("data-feedback");

			// Boolean value that indicates whether there is any interesting feedback
			var isFeedbackEnabled = context.attr("data-is-feedback-enabled");

			//The type of the feedback: 0 = success, 1 = warning, 2 = error
			var feedbackType = context.attr("data-feedback-type");

			//
			// OR you can just use the defaults in most cases...
			//
			if (!itemrowSelector) {
				itemrowSelector = " > table.ordtable > tbody > tr";
				if ($("> .wcr-table-section > table.ordtable", context).hasClass("gridview")) {
					itemrowSelector = " > table.ordtable > tbody > tr > td";
				}
			}
			if (!checkboxSelector) {
				checkboxSelector = "input[type='checkbox'][name^='select']";
			}

			// Whether click to select is active
			var clickToSelectActive = context.attr("data-click-to-select") === "0" ? false : true;

			new RulesTable(context, itemrowSelector, checkboxSelector, clickToSelectActive, projectID, isTemplate);

			if (isFeedbackEnabled === "true") {

				if (feedbackType === "success") {
					PageMessageComponent.sendSuccessMessage(i18n.t("RULE_IMPORT_SUCCESS"), undefined, true);
				} else if (feedbackType === "warning") {
					PageMessageComponent.sendWarnMessage("\n" + feedback);
				} else {
					PageMessageComponent.sendErrorMessage("\n" + feedback);
				}
			}
		};


		return RulesTable;
	});