define(["require",
		"underscore",
	   	"./Rule",
		"component/errorhandler/ErrorHandler",
		"pages/rules/ImportRuleSetPopup",
		"webcenterapi/RuleEngineAPI",
	   	"utils/webi18n!RulesTable",
		"component/pagemessagecomponent/PageMessageComponent",
		"component/loadingindicator/LoadingIndicator",
	   	"jquery",
	   	"utils/webcss!./ruleview-style.css"],
	function (require,
		_,
		Rule,
		ErrorHandler,
		ImportRuleSetPopup,
		RuleEngineAPI,
		i18n,
		PageMessageComponent,
		LoadingIndicator,
		$) {
		"use strict";

		var errorHandler = new ErrorHandler(); // this is not used in rule engine API for now, so no need to add listeners
		var ruleEngineAPI = new RuleEngineAPI(window.webcenterConfig, errorHandler);

		// Loading indicator
		var loadingIndicator = $('<div>').addClass("loadingIndicator").append(new LoadingIndicator().start()).hide();

		// To be called before loading
		var loadStart = function (info) {
			info.table.getContext().before(loadingIndicator.show());
			info.table.setEnabled(false);
			info.actionmenu.setEnabled(false);
		};

		// To be called at the end of loading
		var loadEnd = function (info) {
			info.table.render(); // enables the table again
			info.actionmenu.setEnabled(true);
			loadingIndicator.hide();
		};

		/**
		 * Function which is used to execute actions
		 * 
		 * @param {ActionExecuteEvent} information about the action
		 */
		return {
			executeAction: function (info) {
				var actionName = info.action;

				var projectId = info.table.getProjectID();
				var rule;
				var popupTitle;
				var messageSuccess;
				var messageFailed;

				// execute the action
				switch (actionName) {
				case "delete_rules":
					loadStart(info);

					var ruleids = info.getActionItemsAttribute("value");
					ruleEngineAPI.deleteRules(ruleids, projectId).then(function (result) {
						PageMessageComponent.markObsolete("ruleAction");
						PageMessageComponent.sendSuccessMessage(i18n.t("RULEDELETESUCCESS"), undefined, true);

						loadEnd(info);
					}).catch(function (errorString) {
						PageMessageComponent.markObsolete("ruleAction");
						PageMessageComponent.sendErrorMessage(i18n.t("RULEDELETEFAILED") + (errorString ? (" " + errorString) : ""), undefined, "ruleAction");

						loadEnd(info);
					});
					break;
				case "new_rule":
					var nextOrder = info.table.getMaxOrder() + 1;
					rule = new Rule(undefined, undefined, undefined, '' + nextOrder, true);
					popupTitle = i18n.t("RULENEW");
					messageSuccess = "RULENEWSUCCESS";
					messageFailed = i18n.t("RULENEWFAILED");
					/* falls through */
				case "edit_rule":
					if (info.actionbutton[0] && info.actionbutton[0].rule) {
						// should be the case for edit
						rule = info.actionbutton[0].rule.copy(); // we make a copy for "cancel" support
						popupTitle = i18n.t("RULEEDIT");
						messageSuccess = "RULEEDITSUCCESS";
						messageFailed = i18n.t("RULEEDITFAILED", rule.getName());
					}

					require(["./RulePopup"], function (RulePopup) {
						if (!info.table._rulePopup) {
							info.table._rulePopup = new RulePopup();
						}
						info.table._rulePopup.clearListeners();
						info.table._rulePopup.onSubmit(function (rule) {
							loadStart(info);

							ruleEngineAPI.saveRule(rule, projectId).then(function (result) {
								PageMessageComponent.markObsolete("ruleAction");
								PageMessageComponent.sendSuccessMessage(i18n.t(messageSuccess, rule.getName()), undefined, true);

								loadEnd(info);
							}).catch(function (errorString) {
								PageMessageComponent.markObsolete("ruleAction");
								PageMessageComponent.sendErrorMessage(messageFailed + (errorString ? (" " + errorString) : ""), undefined, "ruleAction");

								loadEnd(info);
							});

							return Promise.resolve(); // we return immediately
						});
						info.table._rulePopup.open({
							"rule": rule,
							"title": popupTitle,
							"enabled": info.table.isTemplate()
						});
					});
					break;
				case "enable_rule":
					if (info.actionbutton[0] && info.actionbutton[0].rule) {
						loadStart(info);

						rule = info.actionbutton[0].rule;
						if (rule.isEnabled()) {
							// it became enabled
							messageSuccess = "RULEENABLESUCCESS";
						} else {
							messageSuccess = "RULEDISABLESUCCESS";
						}
						messageFailed = i18n.t("RULEENABLEFAILED", rule.getName());

						ruleEngineAPI.saveRule(rule, projectId).then(function (result) {
							PageMessageComponent.markObsolete("ruleAction");
							PageMessageComponent.sendSuccessMessage(i18n.t(messageSuccess, rule.getName()), undefined, true);

							loadEnd(info);
						}).catch(function (errorString) {
							PageMessageComponent.markObsolete("ruleAction");
							PageMessageComponent.sendErrorMessage(messageFailed + (errorString ? (" " + errorString) : ""), undefined, "ruleAction");

							loadEnd(info);
						});
					}
					break;
				case "import_rules":
					var importPopup = new ImportRuleSetPopup(projectId);
					importPopup.open();
					break;
				case "export_rules":
					var ruleIds = info.getActionItemsAttribute("value");
					ruleEngineAPI.exportRules(projectId, ruleIds);

					break;
				default:
					break;
				}
			}
		};
	});