define(["utils/HTTPUtils",
		"utils/XMLUtils",
		"underscore",
		"jquery"
	],
	function (HTTPUtils,
		XMLUtils,
		_,
		$) {
		"use strict";


		var LOCAL_STORAGE_KEY = "WCRI18N";
		

		/**
		 * This module provides internationalization to the wcr-static project
		 * 
		 * @param locale {string} locale which should be used
		 * @param wcrInstanceUrl {String} url for the WebCenterInstance (e.g. http://localhost:8081/wcr-web)
		 * @param timestamp {String} timestamp - this is used a cache busting parameter for requesting language strings
		 * @param {String} instanceID - this ID of the instance is used to have a unique key in the localstorage
		 */
		var I18n = function (locale, wcrInstanceUrl, timestamp, instanceID) {
			this._locale = locale ? locale : "en";
			this._timestamp = timestamp ? timestamp : 0;
			this.wcrInstanceUrl = wcrInstanceUrl;
			
			if (!instanceID){
				instanceID = window.webcenterConfig.webInstAbsPath;
			}
			this._localStorageKey = LOCAL_STORAGE_KEY + instanceID + locale;
			

			this._isLocalStorageLoaded = false;
			
			/**
			 * @member {Object} all loaded bundles
			 */
			this.bundles = {};
			/**
			 * @member {Array.<String>} list of bundles currently being fetched
			 */
			this._bundlesBeingFetched = [];
			/**
			 * @member {Array.<String>} list of bundles that should be fetched in the next fetching batch
			 */
			this._fetchQueue = [];


			this.bundleOrder = [];
			this.bundleLoadedListeners = {};
			
			this._throttledFetch = _.throttle(_.bind(this._fetchBundles, this), 50, {leading: false});
		};

		/**
		 * Get the locale of the module this object is used for
		 * @returns {String}
		 */
		I18n.prototype.getLocale = function () {
			return this._locale;
		};

		/**
		 * Load one or more bundles
		 * 
		 * If fallback is enabled, the fallback will be loaded automatically!
		 * 
		 * @param {String} bundleName - the name of the bundle
		 * @param {Function} [callback] - is called when the bundle is loaded
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		I18n.prototype.loadBundle = function (bundleName, callback, binding) {
			if (callback && binding) {
				callback = _.bind(callback, binding);
			}

			if (_.isArray(bundleName)) {
				// multiple bundles
				// make new callback, which only is executed after bundName.length-1 times
				var multiCallback = _.after(bundleName.length, _.once(callback));
				_.each(bundleName, _.bind(function (bundle) {
					this.loadSingleBundle(bundle, multiCallback);
				}, this));
			} else {
				this.loadSingleBundle(bundleName, callback);
			}
		};

		/**
		 * Load a bundle
		 * 
		 * WARNING about newlines: "\n" in the translation file is interpreted as a JavaScript newline character!! (see I18n:loadSingleBundle)
		 * 
		 * If fallback is enabled, the fallback will be loaded automatically!
		 * 
		 * @param {String} bundleName - the name of the bundle
		 * @param {Function} [callback] - is called when the bundle is loaded
		 * @param {Function} [callback] - is called when the bundle failed to load
		 */
		I18n.prototype.loadSingleBundle = function (bundleName, callback, errorcallback) {
			this._readLocalStorageIfNeeded();

			if (this.bundles[bundleName]) {
				// make sure to update the bundle order, so that the findByKey works correctly
				if (!_.contains(this.bundleOrder, bundleName)){
					this.bundleOrder.push(bundleName);
				}
				if (callback) {
					callback(this.bundles[bundleName]);
				}
			} else {
				// bundle info not yet available, register callback
				if (callback) {
					this.whenBundleLoaded(bundleName, callback);
				}

				if (!_.contains(this._bundlesBeingFetched, bundleName)) {
					this._fetchQueue.push({
						bundle: bundleName,
						errorcallback: errorcallback
					});
					this._bundlesBeingFetched.push(bundleName);
					this.bundleOrder.push(bundleName);

					this._throttledFetch();
				}


			}
		};
	
		/**
		 * Execute a batched fetch operation
		 * @private
		 */
		I18n.prototype._fetchBundles = function () {
			if (this._fetchQueue.length > 0) {
				// only fetch if there is need to fetch
				var fetchQueue = this._fetchQueue;
				var scope = _.pluck(fetchQueue, "bundle");
				var errorCallback = function (msg) {
					_.each(_.pluck(fetchQueue, "errorcallback"), function (callback) {
						if (callback) {
							callback(msg);
						}
					});
				};

				// t and lk are just for cache busting purposes
				// we use both the timestamp of reloading the languages and the user defined locale as parameters
				var url = HTTPUtils.constructURL("DownloadLocKeys.jsp", {
					scope: scope,
					t: this._timestamp,
					lk: this._locale
				}, this.wcrInstanceUrl);
				this._fetchQueue = [];

				$.ajax(url, {
					dataType: "xml",
					success: _.bind(function (data, textStatus, jqXHR) {
						// if xml, check if login failed
						if (data) {
							var code = data.getElementsByTagName("code");
							if (code[0]) {
								errorCallback(XMLUtils.textContent(code[0]));
								return;
							}

							// run over all bundles in resultset
							_.each(data.getElementsByTagName("bundle"), function (bundleEl) {
								// parse the languagestrings for the bundle
								var bundle = {};
								var bundleName = bundleEl.getAttribute("name");
								_.each(bundleEl.getElementsByTagName("lang_string"), function (lang) {
									bundle[lang.getAttribute("key")] = lang.getAttribute("value").replace(/\\n/g, '\n');
								});

								// register bundle, and notify that it is loaded
								this.bundles[bundleName] = bundle;
								this.fireBundleLoaded(bundleName, this.bundles[bundleName]);
								this._bundlesBeingFetched = _.without(this._bundlesBeingFetched, bundleName);
							}, this);

							this._saveToLocalStorage();
						}
					}, this),
					error: function (jqXHR, textStatus, errorThrown) {
						errorCallback(textStatus);
					}
				});




			}
		};

		/**
		 * Read the data from the local storage
		 * (will only be executed once)
		 * @private
		 */
		I18n.prototype._readLocalStorageIfNeeded = function () {
			if (!this._isLocalStorageLoaded) {
				try {
					var localStorageData = JSON.parse(window.localStorage.getItem(this._localStorageKey));

					if (localStorageData && localStorageData.timestamp == this._timestamp) {
						this.bundles = localStorageData.bundles;
					}
				} catch (e) {
					if (console) {
						console.log("There was a problem while reading from Local Storage", e);
					}
				}

				this._isLocalStorageLoaded = true;
			}
		};

		/**
		 * Extend the local storage information with all the bundles currently loaded
		 * @private
		 */
		I18n.prototype._saveToLocalStorage = function () {
			var localData;
			try {
				localData = JSON.parse(window.localStorage.getItem(this._localStorageKey));

			} catch (e) {
				// safe to ignore, localStorageData might not havebeen set yet
			}


			
			if (!localData || localData.timestamp !== this._timestamp || !localData.bundles) {
				// no local data is set, or the local data is corrupt or outdated
				localData = {
					timestamp: this._timestamp,
					bundles: {}
				};
			}
			
			//  copy over the bundles
			_.each(this.bundles, function (bundle, bundleName) {
				localData.bundles[bundleName] = bundle;
			}, this);


			try {
				// store in local storage
				window.localStorage.setItem(this._localStorageKey, JSON.stringify(localData));
			} catch (e) {
				if (console) {
					console.log("There was a problem while writing to Local Storage", e);
				}
			}


		};
	
	
		/**
		 * Register a callback to be fired when bundle with bundleName is loaded
		 * @param {String} bundleName 
		 * @param {Function} callback 
		 */
		I18n.prototype.whenBundleLoaded = function (bundleName, callback) {
			if (!this.bundleLoadedListeners[bundleName]) {
				this.bundleLoadedListeners[bundleName] = [];
			}
			this.bundleLoadedListeners[bundleName].push(callback);
		};

		/**
		 * Fire all callbacks that were registered 
		 * @param {String} bundleName 
		 * @param {Object} bundle     
		 */
		I18n.prototype.fireBundleLoaded = function (bundleName, bundle) {
			if (this.bundleLoadedListeners[bundleName]) {
				_.each(this.bundleLoadedListeners[bundleName], function (callback) {
					callback(bundle);
				});
				delete this.bundleLoadedListeners[bundleName];
			}
		};

		/**
		 * Try to find a value for key, respecting the order in which bundles are loaded
		 * @param {String} key
		 * @returns {String|undefined}
		 */
		I18n.prototype.findByKey = function (key) {
			for (var i = this.bundleOrder.length - 1; i >= 0; i--) {
				if (this.bundles[this.bundleOrder[i]] && this.bundles[this.bundleOrder[i]][key]) {
					return this.bundles[this.bundleOrder[i]][key];
				}
			}

			return undefined;
		};

		/**
		 * Try to find a value for a key in a certain bundle.
		 */
		I18n.prototype.findByKeyInBundle = function (key, bundleName) {
			if (this.bundles[bundleName] && this.bundles[bundleName][key]) {
				return this.bundles[bundleName][key];
			} else {
				return undefined;
			}
		};

		/**
		 * Get the translation of a certain key
		 * 
		 * @param {String} key - key from which you want the translation
		 * @param {...String} [replacements] - if extra parameters are given, they replace {0}, {1}, ... in the translated string
		 * @returns translation - if no translation is found, the key is returned
		 */
		I18n.prototype.t = function (key) {
			var value = this.findByKey(key);

			if (value !== undefined) {
				// replace all given options
				for (var i = 1; i < arguments.length; i++) {
					value = value.replace("{" + (i - 1) + "}", arguments[i]);
				}

				return value;
			}

			// we didn't find any value for key, return key
			return key;
		};

		/**
		 * Get the translation of a date based on a key
		 */
		I18n.prototype.d = function (key, value) {
			var dateFormat = this.findByKey(key);
			if (dateFormat !== undefined) {
				return value.toString(dateFormat);
			} else {
				return undefined;
			}
		};

		return I18n;
	});