define(["require", "lib/PromisePolyfill"], function (require) {
	"use strict";

	return {
		/**
		 * Require a requiremodule and get a handle to it as a promise
		 * @param {String} module
		 * @returns {Promise.<Object>}
		 */
		require: function (module) {
			return new Promise(function (resolve, reject) {
				require([module], resolve, reject);
			});
		},

		/**
		 * Turn a JQuery $.Deferred in a Promises/A compliant promise
		 * @param {$.Deferred} deferred
		 * @return Promise
		 */
		deferred: function (deferred) {
			return new Promise(function (resolve, reject) {
				deferred.done(function (data, textStatus, jqXHR) {
					resolve(data);
				}).fail(function (jqXHR, errorStatus, errorString, codeString, data) {
					reject(errorString);
				});
			});
		},

		/**
		 * Make the promise "Querable", e.g. you can now ask your promise whether it resolved, failed or was fulfilled.
		 * 
		 * @param promise
		 * 			An ordinary promise.
		 * @returns A Queryable promise with the "isResolved", "isRejected" and "isFulfilled" methods.
		 */
		makeQueryable: function (promise) {
			// Inspired from: http://stackoverflow.com/questions/21485545/is-there-a-way-to-tell-if-an-es6-promise-is-fulfilled-rejected-resolved

			// Don't create a wrapper for promises that can already be queried.
			if (promise.isResolved) return promise;

			var isResolved = false;
			var isRejected = false;

			// Observe the promise, saving the fulfillment in a closure scope.
			var result = promise.then(
				function (v) {
					isResolved = true;
					return v;
				},
				function (e) {
					isRejected = true;
					throw e;
				});
			result.isFulfilled = function () {
				return isResolved || isRejected;
			};
			result.isResolved = function () {
				return isResolved;
			};
			result.isRejected = function () {
				return isRejected;
			};
			return result;
		}
	};
});