define([], function () {

	"use strict";

	/**
	 * Date shifting tools
	 * All inputs must be ms since 1970
	 */

	var endBusinessDay = 17;
	var startBusinessDay = 9;
	var msInHour = 1000 * 3600;
	var msInDay = msInHour * 24;

	var dateShift = {

		/**
		 * Adds days and hours of WORKING TIME to a start time in ms or as a javascript date object
		 * @param startTime begin time in ms since 1970 or as a Date object (auto-detecting)
		 * @param _days: days to add, must be positive or 0
		 * @param _hours: hours to add
		 * @param leadTime: time shift in ms  days and hours will be derived from this automatically
		 *                  leadTime takes precedence over _days and _hours (should not be used together)
		 * @param dateObjFlg if true, returns a date object, otherwise returns ms
		 * @param businessHours if true, calculations happen with 8 hours in a day, so after 17:00 you go immediately to
		 *                       9:00 next business day. If false, you use all 24 hours.  Weekends are still skipped
		 * @return returns new time either in ms since 1970 or as a javascript date object depending on dateObjFlg
		 */
		addDaysHours: function (startTime, _days, _hours, leadTime, dateObjFlg, businessHours) {
			// first normalize the incoming time to ms
			var newTime;
			var hoursInDay = 24;
			if (businessHours) hoursInDay = 8;

			if (startTime.constructor == Date) {
				newTime = startTime.getTime();
			} else {
				newTime = parseInt(startTime);
			}
			try {
				var days = parseInt(_days);
				var hours = parseInt(_hours);
				if (leadTime) {
					days = Math.floor(leadTime / msInDay);
					hours = (leadTime - (days * msInDay)) / msInHour; // can be non-integer and that's ok
				}
				var fullWeeks = Math.floor(days / 5);
				var netDays = days - fullWeeks * 5;
				newTime = this.normalizeDateHour(newTime, businessHours);

				// first protect against hours > 8 or 24
				if (hours > hoursInDay) {
					days += Math.floor(hours / hoursInDay);
					hours -= Math.floor(hours / hoursInDay) * hoursInDay;
				}
				// protect against negative values
				if (days < 0 || hours < 0) throw ("No negative days or hours allowed");

				// find day of week as a number between 0 and 6
				var dayObj = new Date(newTime);
				var dayOfWeek = dayObj.getDay() - 1; // getDay gives a 0 for Sunday, now you have a 0 for Monday which is what you want
				if (dayOfWeek == -1) dayOfWeek = 6; // Move Sunday to day 6
				dayOfWeek += netDays;
				if (dayOfWeek > 4) netDays += 2; // when you touch a weekend, add 2 days
				newTime += (fullWeeks * 7 + netDays) * msInDay;

				// now the hours    
				if (businessHours) {
					// figure out whether to add a night
					var origHours = dayObj.getHours(); // this is localized, check it!
					var newHours = origHours + hours;
					if (newHours > endBusinessDay) hours += 16; // add 16 non-working hours
				}

				newTime += hours * msInHour;
				// still a risk to end on Saturday
				var newDayObj = new Date(newTime);
				dayOfWeek = newDayObj.getDay();
				if (dayOfWeek == 0 || dayOfWeek == 6) newTime += 2 * msInDay;

			} catch (err) {
				alert("Problem in  addDaysHours: " + err);
			}
			if (dateObjFlg == true) {
				return new Date(newTime);
			} else {
				return newTime;
			}
		},
		/**
		 * Subtracts days and hours of WORKING TIME to a start time in ms or as a javascript date object
		 * @param startTime begin time in ms since 1970 or as a Date object (auto-detecting)
		 * @param _days: days to add, must be positive or 0
		 * @param _hours: hours to add
		 * @param leadTime: time shift in ms  days and hours will be derived from this automatically
		 *                  leadTime takes precedence over _days and _hours (should not be used together)
		 * @param dateObjFlg if true, returns a date object, otherwise returns ms
		 * @param businessHours if true, calculations happen with 8 hours in a day, so after 17:00 you go immediately to
		 *                       9:00 next business day. If false (ro null), you use all 24 hours.  Weekends are still skipped
		 * @return returns new time either in ms since 1970 or as a javascript date object depending on dateObjFlg
		 */
		subtractDaysHours: function (startTime, _days, _hours, leadTime, dateObjFlg, businessHours) {
			// first normalize the incoming time to ms
			var newTime;
			var hoursInDay = 24;
			if (businessHours) hoursInDay = 8;
			if (startTime.constructor == Date) {
				newTime = startTime.getTime();
			} else {
				newTime = parseInt(startTime);
			}
			try {
				var days = parseInt(_days);
				var hours = parseInt(_hours);
				if (leadTime) {
					days = Math.floor(leadTime / msInDay);
					hours = (leadTime - (days * msInDay)) / msInHour; // can be non-integer and that's ok
				}
				var fullWeeks = Math.floor(days / 5);
				var netDays = days - fullWeeks * 5;
				newTime = this.normalizeDateHour(newTime, businessHours);

				// first protect against hours > 8 or 24
				if (hours > hoursInDay) {
					days += Math.floor(hours / hoursInDay);
					hours -= Math.floor(hours / hoursInDay) * hoursInDay;
				}
				// protect against negative values
				if (days < 0 || hours < 0) throw ("No negative days or hours allowed");

				// find day of week as a number between 0 and 6
				var dayObj = new Date(newTime);
				var dayOfWeek = dayObj.getDay() - 1; // getDay gives a 0 for Sunday, now you have a 0 for Monday which is what you want
				if (dayOfWeek == -1) dayOfWeek = 6; // Move Sunday to day 6
				dayOfWeek -= netDays;
				if (dayOfWeek < 0) netDays += 2; // when you touch a weekend, add 2 days to be subtracted
				newTime -= (fullWeeks * 7 + netDays) * msInDay;

				// now the hours   
				if (businessHours) {
					// figure out whether to add a night to be subtracted
					var origHours = dayObj.getHours(); // this is localized, check it!
					var newHours = origHours - hours;
					if (newHours < startBusinessDay) hours += 16; // add 16 non-working hours
				}

				newTime -= hours * msInHour;
				// still a risk to end on Sunday --> go back 2 days
				var newDayObj = new Date(newTime);
				dayOfWeek = newDayObj.getDay();
				if (dayOfWeek == 0 || dayOfWeek == 6) newTime -= 2 * msInDay;

			} catch (err) {
				alert("Problem in  subtractDaysHours: " + err);
			}
			if (dateObjFlg == true) {
				return new Date(newTime);
			} else {
				return newTime;
			}
		},
		/**
		 * @param time1 and time2 in ms since 1970 or as date objects (self-detecting)
		 * Algorithm: The working time difference is determined as follows
		 * First the real difference is calculated (including weekend days)
		 * Then the number of weekends is determined
		 * The weekend days are subtracted
		 * The code is written in such a way that you always have complete weekends (never 1 day)
		 */
		getWorkingTimeDifference: function (time1, time2, businessHours) {
			try {
				if (time1.constructor == Date) time1 = time1.getTime();
				if (time2.constructor == Date) time2 = time2.getTime();
				var diff = time2 - time1;
				var invert = false;
				if (diff < 0) {
					diff = -diff;
					invert = true;
					var tmp = time2;
					time2 = time1;
					time1 = tmp;
				}
				// now we are sure time 2 is later than time 1



				// first normalize inputs 
				time1 = this.normalizeDateHour(time1, businessHours);
				time2 = this.normalizeDateHour(time2, businessHours);
				var dateObj1 = new Date(time1);
				var dateObj2 = new Date(time2);

				var finalDiff;
				if (businessHours) {

					// part 1 is the part from time 1 until the next weekend
					var dayOfWeek = dateObj1.getDay(); // 0 to 6, starting on  Sunday
					if (dayOfWeek == 0) dayOfWeek = 7; // Put Sunday on 7
					var fullDays = 5 - dayOfWeek; // 0 for Friday, 1 for Thursday etc
					if (fullDays < 0) fullDays = 0; // probably can't happen because time1 is normalized
					var endBusinessDayObj = new Date(time1);
					endBusinessDayObj.setHours(endBusinessDay);
					endBusinessDayObj.setMinutes(0);
					var part1 = fullDays * msInDay + endBusinessDayObj.getTime() - dateObj1.getTime();

					// part 2 is the  part from the last weekend until time 2
					dayOfWeek = dateObj2.getDay(); // 0 to 6, starting on  Sunday
					if (dayOfWeek == 0) dayOfWeek = 7; // Put Sunday on 7
					fullDays = dayOfWeek - 1; // 0 for Monday, 1 for Tuesday etc
					if (fullDays > 4) fullDays = 4; // probably can't happen because time2 is normalized
					var startBusinessDayObj = new Date(time2);
					startBusinessDayObj.setHours(startBusinessDay);
					startBusinessDayObj.setMinutes(0);
					var part2 = fullDays * msInDay + dateObj2.getTime() - startBusinessDayObj.getTime();

					// part 3 is the number of full weeks
					var endFirstWeek = time1 + part1; // Friday evening
					var beginLastWeek = time2 - part2; // Monday morning
					var fullWeeks = Math.floor((beginLastWeek - endFirstWeek) / (7 * msInDay));
					finalDiff = 0;
					if (fullWeeks < 0) {
						// beginLastWeek is BEFORE endFirstWeek, so time1 and time2 are in the same week
						// then the time difference is simple
						finalDiff = dateObj2.getTime() - dateObj1.getTime();
					} else {
						finalDiff = fullWeeks * 5 * msInDay + part1 + part2;
					}
				} else { // calculate in 24 hours
					// figure out the number of weekend days.  You know that time1 and time2 are outside weekends
					var days = Math.floor((time2 - time1) / msInDay);
					var weeks = Math.floor(days / 7);
					var remainder = days - weeks * 7;
					var weekendDays = 2 * weeks;
					var dayOfWeek = dateObj1.getDay(); // 0 to 6, starting on  Sunday
					if (dayOfWeek == 0) dayOfWeek = 7; // Put Sunday on 7
					if ((dayOfWeek + remainder) > 5) weekendDays += 2;

					finalDiff = (time2 - time1) - weekendDays * msInDay;
				}

				if (invert) finalDiff = -finalDiff;
				return finalDiff;

			} catch (err) {
				alert("Problem in getWorkingTimeDifference: " + err);
			}
		},
		/* 
		 * shifts time to 
		 * @return time in ms since 1970 of the normalized date/hour
		 * @since WCR10.1
		 * @param moveToFri move to Friday instead of to Monday in case of a weekend
		 * @param defaultTime default hour to be used when moving to Friday
		 */
		normalizeDateHour: function (time, businessHours, moveToFri, defaultTime) {
			var inputType = time.constructor;
			var startHour = 0;
			var endHour = (defaultTime) ? defaultTime : 18;
			if (businessHours) {
				startHour = startBusinessDay;
				endHour = endBusinessDay;
			}
			var dateObj = null;
			if (inputType != Date) {
				dateObj = new Date(time);
			} else {
				dateObj = time;
			}
			// Move yourself out of the weekend - move to the  Monday start business day / Friday end business day
			if (dateObj.getDay() == 6) { // Saturday
				if (moveToFri) {
					dateObj = new Date(dateObj.getTime() - msInDay);
					dateObj.setHours(endHour);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
					dateObj.setMilliseconds(0);
				} else {
					dateObj = new Date(dateObj.getTime() + 2 * msInDay);
					dateObj.setHours(startHour);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
					dateObj.setMilliseconds(0);
				}
			} else if (dateObj.getDay() == 0) { // Sunday
				if (moveToFri) {
					dateObj = new Date(dateObj.getTime() - 2 * msInDay);
					dateObj.setHours(endHour);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
					dateObj.setMilliseconds(0);
				} else {
					dateObj = new Date(dateObj.getTime() + msInDay); // shift from Sunday to Monday
					dateObj.setHours(startHour);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
					dateObj.setMilliseconds(0);
				}
			}
			// if only businessHours allowed, move time from out-of-business hours to start or end businessday   
			if (businessHours) {
				if (dateObj.getHours() >= endBusinessDay) {
					dateObj.setHours(endBusinessDay);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
				}
				dateObj.setMilliseconds(0);
				if (dateObj.getHours() < startBusinessDay) {
					dateObj.setHours(startBusinessDay);
					dateObj.setMinutes(0);
					dateObj.setSeconds(0);
				}
				dateObj.setMilliseconds(0);
			}

			// return result as ms since 1970 for non-Date input and as a Date object for Date input.
			if (inputType != Date) {
				return dateObj.getTime();
			} else {
				return dateObj;
			}
		},
		/**
		 * @since WCR10.1
		 * @param _days number of lead time days
		 * @param _hours number of leat time hours
		 * @return returns the lead time in ms (days and hours), Number object
		 */
		getLeadTime: function (_days, _hours) {
			var leadTime = 0;
			try {
				var days = Number(_days);
				var hours = Number(_hours);
				leadTime = days * msInDay + hours * msInHour;
			} catch (err) {
				alert("Problem in getLeadTime: " + err);
			}
			return new Number(leadTime);
		},
		/**
		 * @since WCR10.1
		 * @param source leadTime lead time in ms
		 * @return returns the number of 'full' days
		 */
		getLeadTimeDays: function (leadTime) {
			var days = 0;
			try {
				if (leadTime > 0) {
					days = Math.floor(leadTime / msInDay);
				}
			} catch (err) {
				alert("Problem in getLeadTimeDays: " + err);
			}
			return days;
		},
		/**
		 * @since WCR10.1
		 * @param source leadTime lead time in ms
		 * @return returns the number of 'full' remaining minutes (<60)
		 */
		getLeadTimeHours: function (leadTime) {
			var hours = 0;
			try {
				if (leadTime > 0) {
					var days = this.getLeadTimeDays(leadTime);
					hours = Math.floor((leadTime - (days * msInDay)) / msInHour);
				}
			} catch (err) {
				alert("Problem in getLeadTimeHours: " + err);
			}
			return hours;
		},
		/**
		 * @since WCR10.1
		 * @param time source time in ms
		 * @return returns the time rounded to hours (in ms)
		 */
		roundTimeToHours: function (time) {
			var ret = time;
			try {
				var normtime = time + msInHour / 2;
				ret = (Math.floor(normtime / msInHour) * msInHour);
			} catch (err) {
				alert("Problem in roundTimeToHours method: " + err);
			}
			return ret;
		},
		/**
		 * @since WCR12.1
		 * @param time source time in ms
		 * @return returns the time rounded to days (in ms)
		 */
		roundTimeToDays: function (time) {
			var ret = time;
			try {
				var normtime = time + msInDay / 2;
				ret = (Math.floor(normtime / msInDay) * msInDay);
			} catch (err) {
				alert("Problem in roundTimeToDays method: " + err);
			}
			return ret;
		}
	};

	return dateShift;

});
