/**
 * @namespace ApprovalModule
 */
define(["web-viewer/core/AbstractModule",
		"web-viewer/approval/ApprovalDataExtensions",
		"web-viewer/approval/ApprovalToolbarComponent",
		"web-viewer/approval/ApprovalMobileToolbarComponent",
		"web-viewer/approval/ApprovalDetailsWidget",
		"Template",
		"tpl!web-viewer/approval/PageListApproval",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/compare/CompareConstants",
		"web-viewer/layout/LayoutConstants",
		"approval/view/ApproveRejectView",
		"underscore"
	],
	function (AbstractModule,
		ApprovalDataExtensions,
		ApprovalToolbarComponent,
		ApprovalMobileToolbarComponent,
		ApprovalDetailsWidget,
		Template,
		tpl,
		ListenerUtil,
		CompareConst,
		LayoutConstants,
		ApproveRejectView,	   
		_) {
		"use strict";

		/**
		 * Create a new instance of the approval module
		 * 
		 * @exports ApprovalModule
		 * @memberof ApprovalModule
		 * @class
		 * @augments Core.AbstractModule
		 * @constructor
		 * 
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var ApprovalModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "approval");
			ListenerUtil.mixin(this, f);

			this._allowApproval = true;
			this._showStatus = true;


			// start preloading language bundle as soon as module is loaded
			f.i18n.loadBundle("approval");

			// install the data extensions
			ApprovalDataExtensions(f);

			// publish approval buttons toolbar component 
			this._publishToolbarComponent(ApprovalToolbarComponent.toolbarName, _.bind(function (layoutType) {
				if (layoutType === LayoutConstants.LayoutType.MOBILE) {
					return new ApprovalMobileToolbarComponent(this.f, this);
				} else {
					return new ApprovalToolbarComponent(this.f, this);
				}
			}, this));

			// publish details widget
			this._publishWidget(ApprovalDetailsWidget.widgetName, _.bind(function (layoutType) {
				this._approvalWidget = new ApprovalDetailsWidget(f);
				return this._approvalWidget;
			}, this));


			this.f.getDocumentController().whenActivePageChanged(_.bind(this._whenActivePageChanged, this));
			// react on compare mode changes
			this.f.on("module.compare.changed.mode", _.bind(this._update, this));
			// make sure we update the toolbar component when compare becomes available
			this.f.whenModuleAvailable("compare", this._update, this);
		};

		// Inherit from AbstractModule
		ApprovalModule.prototype = Object.create(AbstractModule.prototype);
		ApprovalModule.prototype.constructor = ApprovalModule;


		/**
		 * When the active page has changed, react on it, and execute some actions
		 */
		ApprovalModule.prototype._whenActivePageChanged = function (oldPage, newPage) {
			if (oldPage) {
				var oldDocVersion = oldPage.getDocumentVersion();
				this._unregisterAllListenersForObject(oldDocVersion);
			}

			if (newPage) {
				var newDocVersion = newPage.getDocumentVersion();

				this._registerListener("approval.loaded", this._update, this, newDocVersion);
				this._registerListener("approval.update", this._update, this, newDocVersion);

				if (newDocVersion.isPageList()) {
					// we have a page list
					this._registerListener("approval.update", function () {
						newPage.requestPageApprovalSetup(_.bind(this._update, this));
					}, this, newDocVersion);
					// request the approval for the page
					newPage.requestPageApprovalSetup(_.bind(this._update, this));
				} else {
					// request the approval setup for the docversion
					newDocVersion.requestApprovalSetup(_.bind(this._update, this));
				}

				// listen to successful approval/rejection to show a message
				this._registerListener("approval.update", function () {
					var approveRejectModel = newPage.getApproveRejectModel();
					if (approveRejectModel) {
						approveRejectModel.approveRejectCallback = _.bind(function (success, action, error) {
							if (success) {
								this.f.emit("userMessage", "success", this.f.i18n.t("approvalSuccess" + action), true);

								if (!newPage.isBlankPage()) {
									// reload annotations when approval was successful
									var annotationModule = this.f.getModule("annotation");
									if (annotationModule) {
										annotationModule.reload();
									}
								}

								// SDK support
								// First approvalcallbackurl
								var approvalCallbackUrl = this.f.getProperty("approvalcallbackurl");
								if (approvalCallbackUrl) {
									this.f.ajax(approvalCallbackUrl, {
										dataType: undefined, // intelligent guess of what will come back
										noAjaxParam: true,
										success: _.bind(this._closeOnApprove, this), // success or error of approvalcallbackurl
										error: _.bind(this._closeOnApprove, this) // we always check if we need to close on approve
									});
								} else {
									// if we have approvalCallbackUrl, we wait until that one is finished (callbacks), otherwise we just check closeOnApprove
									this._closeOnApprove();
								}
							}
							this._emit("updated");
						}, this);
					}
				}, this, newDocVersion);
			}
		};

		/**
		 * Check if the viewer needs to be closed on approve, and do so if needed
		 */
		ApprovalModule.prototype._closeOnApprove = function () {
			// Then closeonapprove
			var closeOnApprove = this.f.getProperty('closeonapprove');
			if (closeOnApprove && closeOnApprove != "0" && closeOnApprove != "false") {
				this.f.emit("viewer.request.close", this.f.getProperty("returnurl"));
			}
		};

		/**
		 * Approval info has updated (or compare module has become available or changed)
		 */
		ApprovalModule.prototype._update = function () {
			// check which things to show based on compare mode
			// get the active document

			var compareMode = null;
			var compareModule = this.f.getModule("compare");
			if (compareModule) {
				compareMode = compareModule.getActiveMode();
			}
			this._allowApproval = (!compareMode || compareMode.getName() === CompareConst.mode.SHOW_CURRENT || compareMode.getName() === CompareConst.mode.READERSPREAD);
			this._showStatus = (!compareMode || compareMode.getName() !== CompareConst.mode.SHOW_DIFFERENCE);

			this._emit("updated");
		};


		/**
		 * Check if there are unsaved annotations, and only then execute the action
		 * @param action
		 */
		ApprovalModule.prototype._checkAndExecuteAction = function (action, event) {
			var finalAction = _.bind(function () {
				var page = this.f.getDocumentController().getActivePage();
				var approvalObj = page.getApprovalObject();
				var approveRejectModel = page.getApproveRejectModel();
				new ApproveRejectView(approvalObj, approveRejectModel).open(event, action.internalName, action.step);
			}, this);
			if (this.f.getDocumentController().getActiveDocumentVersion().isPageList()) {
				finalAction = _.bind(this.getPageListApprovalAction, this, action, event);
			}


			if (this._allowApproval) {
				var annotModule = this.f.getModule("annotation");
				if (annotModule) {
					var hasUnsavedAnnotations = annotModule.promptToSaveUnsavedAnnotations(finalAction, function () {
						// when the user cancel saving unsaved annotations we do not want to do anything...
					}, _.bind(function () {
						annotModule.reload(); //reload all the annotations, they are not saved
						finalAction();
					}, this));
					if (!hasUnsavedAnnotations) { //no prompt is shown as there are no unsaved annotations
						finalAction(); //do the final actions
					}
				} else {
					finalAction();
				}
			} else {
				// approval is not allowed, show alert
				this.f.loadBundle("viewer", function () {
					this.f.emit("userMessage", "warn", this.f.t("AppDiffModeError"));
				}, this);
			}

		};

		/**
		 * Get the list of available actions
		 * @returns {Array.<Object>} with for each action (name, action, text)
		 */
		ApprovalModule.prototype.getAvailableActions = function () {
			var page = this.f.getActivePage();
			var actions = page.getApprovalActions();

			_.each(actions.list, function (action) {
				action.action = _.bind(this._checkAndExecuteAction, this, action);
			}, this);

			return actions;
		};

		/**
		 * Is approval allowed?
		 * @returns {Boolean}
		 */
		ApprovalModule.prototype.allowApproval = function () {
			return this._allowApproval;
		};

		/**
		 * Should the approval status be shown?
		 * @returns {Boolean}
		 */
		ApprovalModule.prototype.showStatus = function () {
			return this._showStatus;
		};


		/**
		 * Extend the approval action with extra parameters for page lists
		 * @param action
		 * @param event
		 * @returns the modified action
		 */
		ApprovalModule.prototype.getPageListApprovalAction = function (action, event) {
			//check whether we want to approve this page or remaining pages:
			var approveRemaining = false;
			if (event.target.getAttribute("class") && event.target.getAttribute("class").indexOf("Remaining") > -1) {
				approveRemaining = true;
			}

			var page = this.f.getDocumentController().getActivePage();
			var approvalObj = page.getApprovalObject();
			var approveRejectModel = page.getApproveRejectModel();
			var widgetContext = $("<div/>");
			this.t = new Template(this.f.i18n, tpl);
			// set default value: 'current' when page not approved; 'all' when already approved (approve remaining pages)
			this.t.d.range = approveRemaining ? 3 : 0;
			this.t.d.approveRemaining = approveRemaining;
			this.t.setContext(widgetContext);

			// add some listeners to make the panel interactive
			this.t.bind("change", "[name=pageRange]", function (source, event) {
				this.t.d.range = parseInt(source.value, 10);
				if (this.t.d.range === 2) {
					widgetContext.find("[name=selectedPages]").removeAttr("disabled");
				} else {
					widgetContext.find("[name=selectedPages]").attr("disabled", "disabled");
				}
			}, this);

			this.t.bind("change", "[name=selectedPages]", function (source, event) {
				this.t.d.selectedRange = source.value;
			}, this);

			// rerender if marked pages have changed
			var renderPopup = _.bind(function () {
				// only render if visible (popup/visibility defined by complexapproval.js)
				if (widgetContext.css("visibility") !== 'hidden') {
					var pagelistModule = this.f.getModule("pagelist");
					if (pagelistModule) {
						this.t.d.markedPages = pagelistModule.getAllMarkedPagesAsString();
					}
					this.t.render();
				}
			}, this);

			// register listener on marked pages changed
			if (this.pageListApprovalInitialized === undefined) {
				this.f.on("doc.**.pagelistMarked.changed", renderPopup);
				this.pageListApprovalInitialized = true;
			}

			approveRejectModel.setBeforeSubmitAction(_.bind(function () {
				approvalObj.forceStep(undefined);
				var errorMsg = "";
				switch (this.t.d.range) {
				case 0:
					errorMsg = approveRejectModel.setPageRange(page.get("pageindex"));
					break;
				case 1:
					errorMsg = approveRejectModel.setPageRange(this.t.d.markedPages, true, true);
					break;
				case 2:
					errorMsg = approveRejectModel.setPageRange(this.t.d.selectedRange, true, true);
					break;
				case 3:
					errorMsg = approveRejectModel.setPageRange("all");
					break;
				}
				if(errorMsg !== ""){
					alert(i18n.t(errorMsg));
				}
			}, this));

			// Do the real approve/reject action
			approvalObj.forceStep(0);
			var dropdown = new ApproveRejectView(approvalObj, approveRejectModel).open(event, action.internalName, action.step);
			approvalObj.forceStep(undefined);
			
			$("#extraApprovalContent", dropdown).html(widgetContext);
			renderPopup();
			
		};

		/**
		 * Get the approval details widget
		 * @returns {Approval.ApprovalDetailsWidget}
		 */
		ApprovalModule.prototype.getApprovalWidget = function () {
			return this._approvalWidget;
		};

		return ApprovalModule;
	});