/**
 * @namespace AttributeCategory
 */
define(["web-viewer/core/AbstractModule",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/attributeCategory/AttributeCategoryDataExtensions",
		"web-viewer/attributeCategory/AttributeCategoryWidget",
		"attributecategory/AttributeCategory",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/FullscreenModalPopup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"underscore",
		"jquery"
	],
	function (AbstractModule,
		ListenerUtil,
		AttributeCategoryDataExtensions,
		AttributeCategoryWidget,
		AttributeCategory,
		ModalPopup,
		FullscreenModalPopup,
		ViewerToolbarButton,
		_,
		$) {
		"use strict";

		/**
		 * Create a new instance of the attribute category module. This enables a widget with the attribute category of the active document
		 *
		 * @exports AttributeCategoryModule
		 * @memberof AttributeCategoryModule
		 * @class
		 * @augments Core.AbstractModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var AttributeCategoryModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "attributeCategory");
			ListenerUtil.mixin(this, f);

			AttributeCategoryDataExtensions(f);


			this._category = null;
			this._page = null;
			this._viewList = [];
			this._activeGroupRole = null;
			this._activeView = null;

			this._viewBeforeModal = null;



			this._publishedWidget = false;

			this._registerListener("intercept.activePageChange", this._interceptActivePageChange, this);
			this._registerListener("intercept.savingCadOverlay", function (context) {
				if (this._category && this._category.hasActiveViewDocReference()) {
					if (this._page.isAttributeCategorySaveOngoing()) {
						// save operation is ongoing
						context.saveCadOverlay = false;

						// when save has finished, save CAD overlay
						this._page.once("attributeCategory.saveFinished", function (success, error) {
							// save operation for attributes has finished, save thfe CAD overlay
							context.callback();
						});


					} else if (this._page.hasUnsavedAttributeChanges()) {
						// no ongoing save operation, but unsaved attributes... prompt to save
						context.saveCadOverlay = false;
						this._promptToSaveUnsavedChanges(_.bind(function () {
							context.callback();
						}, this), _.bind(function () {
							context.callback();
						}, this), true);
					}
				}
			}, this);
			this._registerListener("newLinkedCad", function () {
				if (!this.f.getModule("compare")) {
					this.f.requestModule("compare", undefined, false);
				}
				this.resetAttributeCategory(this._activeView);
			}, this, this.f.getDocumentController());
			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._whenActivePageChangedAndParametersAvailable, this);
		};

		// Inherit from AbstractModule
		AttributeCategoryModule.prototype = Object.create(AbstractModule.prototype);
		AttributeCategoryModule.prototype.constructor = AttributeCategoryModule;


		/**
		 * When the active page has changed and the parameters are available, react on it:
		 *
		 * Load attribute category, roles and groups and display the widget if needed
		 *
		 * @param {AbstractDocumentPage} oldPage - previous active page
		 * @param {AbstractDocumentPage} newPage - new active page
		 */
		AttributeCategoryModule.prototype._whenActivePageChangedAndParametersAvailable = function (oldPage, newPage) {
			this._viewList = [];

			if (this._widget) {
				this._widget.setPage(null);
			}

			// unregister listeners
			this._unregisterAllListenersForObject(oldPage);

			newPage.whenAttributeCategoryAvailable(function (isCategoryAvailable) {
				if (isCategoryAvailable) {
					newPage.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
						var viewToActivate = null;
						if (views && _.size(views.roleSpecificViews) > 0) {
							this._viewList = _.sortBy(_.keys(views.roleSpecificViews), function (groupRole) {
								return groupRole;
							});
							this._activeGroupRole = _.first(this._viewList);
							viewToActivate = views.roleSpecificViews[this._activeGroupRole];

						} else if (newPage.hasViewerView()) {
							viewToActivate = newPage.getViewerView();
						}

						if (viewToActivate) {
							this._activateCategoryWithView(newPage, viewToActivate);
						} else {
							this._disableWidget();
						}
					}, this));
				} else {
					this._disableWidget();
				}
			}, this);
		};


		/**
		 * Activate the Category with a certain view
		 * @param {AbstractDocumentPage} page           - the page for which the attribute category should be activated
		 * @param {String}               viewToActivate - the view that should be activated on the attribute category
		 */
		AttributeCategoryModule.prototype._activateCategoryWithView = function (page, viewToActivate) {
			this._category = page.getAttributeCategory();
			this._page = page;

			if (this._aCatLoadedListener) {
				this._unregisterListener(this._aCatLoadedListener);
				delete this._aCatLoadedListener;
			}

			this._aCatLoadedListener = this._registerListener("attributeCategory.loaded", function (success, error) {
				if (!success) {
					this.f.loadBundle("viewer", function () {
						this._emit("userMessage", "error", this.f.t("AttrErrorWhileLoading", error ? error : ""));
					}, this);
				}
			}, this, page);

			this._activateViewByName(viewToActivate);
			this._displayAttributeCategory();
		};

		/**
		 * Activate a certain view on the category
		 * @param {String} viewName - view to be activated
		 */
		AttributeCategoryModule.prototype._activateViewByName = function (viewName) {
			this._activeView = viewName;
			this._category.activateViewByName(viewName);
		};

		/**
		 * Are there unsaved attributes?
		 * @Override
		 */
		AttributeCategoryModule.prototype.hasUnsavedElements = function () {
			return this._page && this._page.hasAttributeCategory() && this._page.canEditAttributes() && this._page.hasUnsavedAttributeChanges();
		};

		/**
		 * Language string for indicating what is not yet saved (hint, it returns "Attributes" :P)
		 * @returns {String}
		 * @override
		 */
		AttributeCategoryModule.prototype.getUnsavedElementsName = function () {
			// note we don't wait for the bundle to be loaded... but it's not even possible in this situation
			// this language var should normally be loaded already...
			return this.f.t("AttrUnsavedItemName");
		};

		/**
		 * Save the attribute
		 * @override
		 * @param callbackFunction
		 */
		AttributeCategoryModule.prototype.saveAllUnsavedElements = function (callbackFunction) {
			var page = this.f.getActivePage();

			if (page && page.hasAttributeCategory() && page.canEditAttributes() && page.hasUnsavedAttributeChanges()) {
				page.saveAttributes(callbackFunction);
			} else {
				callbackFunction();
			}
		};

		/**
		 * Display the attribute category in the widget (this method will publish the widget if needed)
		 */
		AttributeCategoryModule.prototype._displayAttributeCategory = function () {
			if (this._modalMode) {
				// we are displaying the attribute category in modal mode
				var options = {"allowClose":true};
				if (!this._modalPopup) {
					this._modalPopup = new FullscreenModalPopup(this.f, "attributeCategoryPopup", false, 0.95, options);
				}

				this._modalPopup.setContent(this._page.getAttributeCategoryPanel().show().addClass("wcr"));
				this._modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this._modalInfo.popupTitle));

				// save attributes and close popup				
				this._modalPopup.setOkBtnText(this._modalInfo.saveText);
				this._modalPopup.setOkAction(_.bind(function () {
					this.saveActivePageAttributes();
				}, this));
					
				if(!this._page.canEditAttributes()){
					this._modalPopup.toggleButtons(false);
				}

				// the popup is cancelled, revert changes hide popup
				this._modalPopup.setCancelAction(_.bind(function () {
					this._modalMode = false;
					this.resetAttributeCategory(this._viewBeforeModal);
					this._page.attributeCategoryFireModalPopupClosed();
				}, this));

				// show the popup
				this._modalPopup.show();
			} else {
				// display the attributes in the widget

				if (this._modalPopup) {
					this._modalPopup.hide();
				}

				if (!this._publishedWidget) {
					// the widget is not yet published

					this._publishedWidget = true;
					this._publishWidget(AttributeCategoryWidget.widgetName, _.bind(function (layoutType) {
						// widget is allowed, create it, set the page
						this._widget = new AttributeCategoryWidget(this.f, this);
						this._widget.setPage(this._page, this._activeGroupRole, this._viewList);
						this._widget.setEnabled(true);

						this._registerListener("changedGroupRole", this._activateGroupRoleView, this, this._widget);
						this._registerListener("saveAttributes", this.saveActivePageAttributes, this, this._widget);

						return this._widget;
					}, this));
				} else if (this._widget) {
					// widget already exists, just enable it and set the page
					this._widget.setEnabled(true);
					this._widget.setPage(this._page, this._activeGroupRole, this._viewList);
				}
			}

			$("#dropmenudiv").addClass("wcr").appendTo(this.f.getContext());
		};


		/**
		 * Handle the Attribute Category View request of the widget
		 * @param {String} newGroupRole - name of the group/role that should be activated
		 */
		AttributeCategoryModule.prototype._activateGroupRoleView = function (newGroupRole) {
			if (this._page) {
				this._page.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
					if (views && views.roleSpecificViews && views.roleSpecificViews[newGroupRole]) {
						// the view is avaialble on the category
						var oldGroupRole = this._activeGroupRole;

						// if there are unsaved changes, first prompt to save them
						this._promptToSaveUnsavedChanges(_.bind(function () {
							// ok, change the view
							this._activeGroupRole = newGroupRole;
							// we want to completily reload the category
							this.resetAttributeCategory(views.roleSpecificViews[newGroupRole]);

							// make sure the correct group role selection is set
							if (this._widget) {
								this._widget.updateGroupRoleSelection(newGroupRole);
							}
						}, this), _.bind(function () {
							// we didn't activate the new view
							// make sure the correct group role selection is set                        
							if (this._widget) {
								this._widget.updateGroupRoleSelection(oldGroupRole);
							}
						}, this));
					}
				}, this));
			} else {
				// This is an exceptional case, it should normally not occur... we want to log it to detect this case...
				this.f.emit("error.WARNING", "Trying to activate a view when no page is active");
			}
		};

		/**
		 * Disable the widget
		 */
		AttributeCategoryModule.prototype._disableWidget = function () {
			if (this._widget) {
				this._widget.setEnabled(false);
				this._widget.setPage(null);
			}
		};

		/**
		 * Intercept active page changes. If the page is changing, and the user has unsaved changed, we want to prompt this...
		 * @param {Object} context - context object for interception
		 */
		AttributeCategoryModule.prototype._interceptActivePageChange = function (context) {
			var activePage = context.oldActivePage;
			var newActivePage = context.newActivePage;
			if (this._page && this._page === activePage && newActivePage !== activePage && activePage.hasUnsavedAttributeChanges()) {
				// the page has changed, and we have unsaved changed

				// by default prevent the change of page
				context.newActivePage = context.oldActivePage;

				// now prompt the user what he wants to do
				this._promptToSaveUnsavedChanges(_.bind(function () {
					// the user wants to change the page (and if needed attributes have been saved)
					this.f.getDocumentController().changeActivePage(newActivePage);
				}, this), function () {
					// the user cancelled, do nothing
				});
			}
		};

		/**
		 * Validate the Password and Save the attribute for the active page
		 * @param {Function} callback - callback function called when save operation finished, two parameters: first = success, second = error if any
		 */
		AttributeCategoryModule.prototype.saveActivePageAttributes = function (callback) {
			var options = {};
			options.viewOptions = {widgetview: "viewer"};
			options.f = this.f;
			// The delay of submit is needed here to move the validation and submit (including generation of document reference "saveXML")
			// in the asynchronous calls queue after the potentially pending debounced attribute category re-paint(s), which can be triggered
			// by rules defined in attribute category.
			setTimeout(_.bind(function() {
				this._category.checkPasswordAndSave(_.bind(function() {
					this.saveAttributesChanges(callback);
				}, this, callback),options);
			}, this), 500);
		};

		/**
		 * Save the attribute for the active page
		 * 
		 * @param callback - callback function called when save operation finished, two parameters: first = success, second = error if any
		 */
		AttributeCategoryModule.prototype.saveAttributesChanges = function (callback) {
			if (this._page) {
				this._page.saveAttributes(_.bind(function (success, error) {
					// show user message for save result
					this.f.loadBundle("viewer", function () {
						if (success) {
							this.f.emit("userMessage", "success", this.f.t("AttrSaveSuccess"), true);
						} else {
							this.f.emit("userMessage", "error", this.f.t("AttrSaveError") + " " + error);
						}
					}, this);

					if (success) {
						// reload the category on success
						this.resetAttributeCategory(this._activeView);
					}

					// execute the callbacks if there are any
					if (callback) {
						callback(success, error);
					}
				}, this));
			}
		};


		/**
		 * Prompt the user the save unsaved attribute changes (if there are unsaved attribute changes)
		 * @param {Function} callbackOnOk     - called when everything is OK (attributes saved)
		 * @param {Function} callbackOnCancel - called when the user cancels the operation (attributes are not saved)
		 * @param {Boolean} [canNotBeCancelled] - indicates the action cannot be cancelled, it is either revert or save
		 * @returns {Boolean} true if a prompt is shown
		 */
		AttributeCategoryModule.prototype._promptToSaveUnsavedChanges = function (callbackOnOk, callbackOnCancel, canNotBeCancelled) {
			if (this._page && this._page.hasUnsavedAttributeChanges()) {
				this.f.loadBundle("viewer", function () {

					var modalPopup = new ModalPopup(this.f, "saveUnsavedAttributeChanges", this.f.getContext(), canNotBeCancelled);
					modalPopup.setRemoveFromDomAfterUsage(true);

					// the user wants to Save attributes before the callbackOnOk is executed
					modalPopup.setOkAction(_.bind(function () {
						// indicate to user that save operation is ongoing
						var loadingIndicator = $("<div>").css({
							'background': "url('" + requirejs.toUrl('web-viewer/layout/img/loaderTransparent.gif') + "') no-repeat 2px center",
							'padding-left': '20px',
							'line-height': '20px'
						}).addClass("unsavedItemsContent").text(this.f.t("AttrSaveOngoing"));
						modalPopup.setContent(loadingIndicator);

						//disable buttons while saving
						modalPopup.toggleButtons(false);
						modalPopup.toggleHeaderIcons(false);

						// save the attributes
						this.saveActivePageAttributes(function (success) {
							if (success) {
								// attributes are saved successfully!
								callbackOnOk();
							} else {
								// save operation failed, act like the save was cancelled, so the user wont lose data
								callbackOnCancel();
							}

							modalPopup.hide();
						});
					}, this));


					// Discard/revert changes
					var discardChanges = new ViewerToolbarButton(this.f, "exit", $("<span/>").text(this.f.t("RevertAnnChanges")), false, _.bind(function () {
						// reset the attribute category
						this.resetAttributeCategory(this._activeView);
						// execute the callback on ok
						callbackOnOk();
						modalPopup.hide();
					}, this));
					modalPopup.addButton(discardChanges);

					if (!canNotBeCancelled) {
						// cancel the pending action
						modalPopup.setCancelAction(_.bind(function () {
							callbackOnCancel();
						}, this));
						modalPopup.setCancelBtnText(this.f.i18n.t("Cancel"));
					}


					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.t("AttrSavePromptTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.t("AttrSavePromptContent")));

					modalPopup.setOkBtnText(this.f.i18n.t("Save"));

					modalPopup.show();
				}, this);
				return true;
			} else {
				// everything is OK, go on with OK action
				callbackOnOk();
				return false;
			}
		};


		/**
		 * Reset the category completily
		 * @param {String} [loadView] - if defined, the given view will be activated on the attribute category when its available (this triggers a "safe" reload), if nothing is defined, the attributes are reset, but no new data will be fetched
		 */
		AttributeCategoryModule.prototype.resetAttributeCategory = function (loadView) {
			if (this._widget) {
				this._widget.setPage(null);
			}

			this._viewList = [];

			this._activeView = null;

			var whenCategoryIsReset = _.bind(function (isCategoryAvailable) {
				if (loadView) {
					this._page.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
						this._viewList = _.sortBy(_.keys(views.roleSpecificViews), function (groupRole) {
							return groupRole;
						});
						this._activateCategoryWithView(this._page, loadView);
					}, this));
				}
			}, this);

			if (this._page) {
				this._page.resetAttributeCategory(whenCategoryIsReset);
			} else {
				this._page = this.f.getActivePage();
				whenCategoryIsReset();
			}
		};



		/**
		 * Open a modal popup with a attribute category view for the current document
		 * (Note, if the user cancels saving the current attribute changes, the modal popup won't be opened)
		 * @param {String} viewName - the name of the view you want to activate (this should be an existing view offcourse!
		 * @param {String} [popupTitle=Attributes] -title for the modal popup
		 * @parma {String} [saveText=Save Attributes] - text on save button
		 */
		AttributeCategoryModule.prototype.showViewInModalPopup = function (viewName, popupTitle, saveText) {
			// save the current active view for if the widget is already showing something
			if (!this._modalMode) {
				this._viewBeforeModal = this._activeView;
			}

			this._promptToSaveUnsavedChanges(_.bind(function () {
					this._modalMode = true;
					this._modalInfo = {
						popupTitle: popupTitle || this.f.t("AttributeCategoryWidgetTitle"),
						saveText: saveText || this.f.t("AttrSaveAttributes")
					};

					this.resetAttributeCategory(viewName);
				}, this),

				_.bind(function () {
					// the user cancelled the operation
					this._modalMode = false;
				}, this));
		};

		/**
		 * Method to get the currently active view
		 * @returns {String|Object} _activeView
		 */
		AttributeCategoryModule.prototype.getActiveView = function () {
			return this._activeView;
		};

		return AttributeCategoryModule;
	});