define(["web-viewer/core/AbstractModule",
		"web-viewer/layout/component/DocumentSelection",
		"web-viewer/documentInfo/DocumentInfoWidget",
		"web-viewer/documentInfo/InfoWidget",
		"web-viewer/documentInfo/DebugWidget",
		"web-viewer/documentInfo/DocumentInfoModel",
		"web-viewer/documentInfo/MouseLocatorTool",
		"Mousetrap",
		"web-viewer/layout/LayoutConstants",
		"underscore",
		"component/toolbar/ToolbarComponent",
		"web-viewer/core/CoreConstants"
	],
	function (AbstractModule,
		DocumentSelection,
		DocumentInfoWidget,
		InfoWidget,
		DebugWidget,
		DocumentInfoModel,
		MouseLocatorTool,
		Mousetrap,
		LayoutConstants,
		_,
		ToolbarComponent,
		CoreConstants) {
		"use strict";

		/**
		 * This module is responsible for adding the document info widget (topbar)
		 *
		 * @exports DocumentInfoModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var DocumentInfoModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "documentInfo");

			/**
			 * @member {Layout.Component.DocumentSelection} - mobile document selector, active document only
			 */
			this._documentSelection = null;

			// create the widget
			this._publishWidget(DocumentInfoWidget.widgetName, _.bind(function (layoutType) {
				this._documentInfoWidget = new DocumentInfoWidget(f);
				return this._documentInfoWidget;
			}, this));



			// create model, create tool
			this.documentInfoModel = new DocumentInfoModel(this.f, "documentInfo");
			var mouseLocatorTool = new MouseLocatorTool(f, this.documentInfoModel);
			this._publishTool(mouseLocatorTool);

			// create the Info widget
			this._publishWidget(InfoWidget.widgetName, _.bind(function (layoutType) {
				this._infoWidget = new InfoWidget(f, this.documentInfoModel);
				return this._infoWidget;
			}, this));

			// only publish the debugging-infowidget once!
			this._showDebugWidget = _.once(_.bind(function () {
				this._publishWidget(DebugWidget.widgetName, function (layoutType) {
					return new DebugWidget(f);
				});
			}, this));

			/**
			 * Toolbar Component: Document Selector
			 */
			this.f.i18n.loadBundle("viewer", function () {
				// the DocumentSelection only works correctly once the general language bundle is available
				this._publishToolbarComponent(LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR, _.bind(function (layoutType) {
					this._documentSelection = new DocumentSelection(this.f, undefined, undefined, true, true);
					var documentSelectorComponent = new ToolbarComponent(LayoutConstants.ToolbarComponent.DOCUMENTSELECTOR, f);
					documentSelectorComponent.addObject(this._documentSelection.getActiveDocumentSelector());
					return documentSelectorComponent;
				}, this));
			}, this);


			//>>excludeStart("PExclude", pragmas.productionExclude);
			// on developer systems we can just enter "d e b u g (enter)" to open the widget
			// note, this code is not included in the compiled version!
			Mousetrap.bind('d e b u g enter', this._showDebugWidget);
			Mousetrap.bind("i c o n s enter", _.bind(this._showDebugIconList, this));
			//>>excludeEnd("PExclude");	

			// if the debug parameter is provided, show the info widget
			if (this.f.getProperty("debug")) {
				this._showDebugWidget();
			}

			// Listen to changes in selection (used in e.g. measure tools) and update model accordingly
			this.f.on("selection.changed", _.bind(this.documentInfoModel.setSelection, this.documentInfoModel));

			// Keyboard shortcuts to toggle units
			Mousetrap.bind("ctrl+u", _.bind(this._setNextSizeUnit, this), "keydown");
			Mousetrap.bind("ctrl+shift+u", _.bind(this._setNextResolutionUnit, this), "keydown");
		};


		// inherit
		DocumentInfoModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		DocumentInfoModule.prototype.constructor = DocumentInfoModule;

		//>>excludeStart("PExclude", pragmas.productionExclude);
		// on developer systems we can just enter "iconlist (enter)" to open a popup with all icons
		// note, this code is not included in the compiled version!
		DocumentInfoModule.prototype._showDebugIconList = function () {
			this.f.ajax("/wcr-static/style/css/icons.css", {
				dataType: "text",
				success: function (data, textStatus, jqXHR) {
					var icons = _.sortBy(_.uniq(data.match(/\icon-WCR_[a-zA-Z0-9-]*/g), false));
					var iconList = $("<div>");
					_.each(icons, function (icon) {
						iconList.append("<div style='width: 100px; margin: 5px; border: 1px solid #cacaca; text-align: center; padding: 5px;'><p style='font-size:10px'>" + icon + "</p><i class='icon " + icon + "'/></div>");
					});

					iconList.css({
						position: "absolute",
						top: "0",
						left: "0",
						width: "100%",
						background: "white",
						"font-size": "32px",
						display: "flex",
						"flex-wrap": "wrap",
						"z-index": 100,
						"max-height": "100%",
						"overflow-y": "auto"
					});

					$(document.body).append(iconList);

					iconList.dblclick(function (element) {
						iconList.remove();
					});
				}
			});
		};
		//>>excludeEnd("PExclude");	

		/**
		 * Set the next size unit (circular)
		 * @param {Event} e - Event object
		 */
		DocumentInfoModule.prototype._setNextSizeUnit = function (e) {
			var currentSizeUnit = this.f.getUnits();
			var nextSizeUnit = (currentSizeUnit + 1) % _.size(CoreConstants.SizeUnits);
			this.f.setOption("PREFERENCES_UNIT_SIZE", nextSizeUnit, true);

			e.preventDefault();
			e.stopPropagation();
		};

		/**
		 * Set the next resolution unit (circular)
		 * @param {Event} e - Event object
		 */
		DocumentInfoModule.prototype._setNextResolutionUnit = function (e) {
			var currentResUnit = this.f.getResolutionUnits();
			var nextResUnit = (currentResUnit + 1) % _.size(CoreConstants.ResolutionUnits);
			this.f.setOption("PREFERENCES_UNIT_RESOLUTION", nextResUnit, true);
			this.f.setOption("PREFERENCES_UNIT_SCREENRULING", nextResUnit, true);
			

			e.preventDefault();
			e.stopPropagation();
		};

		return DocumentInfoModule;
	});