define(["web-viewer/layout/Layout",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/flex/modes/FlexBasicMode",
		"web-viewer/layout/flex/modes/FlexWizardMode",
		"web-viewer/layout/flex/FlexWidgetWrapper",
		"web-viewer/layout/flex/FlexSidebar",
		"web-viewer/layout/flex/FlexSection",
		"web-viewer/layout/flex/FlexDrag",
		"web-viewer/layout/flex/FlexConfiguration!restricted",
		"utils/ShapeUtils",
		"underscore",
		"jquery"
	],
	function (
		Layout,
		LayoutConstants,
		PrivateChannel,
		ListenerUtil,
		FlexBasicMode,
		FlexWizardMode,
		FlexWidgetWrapper,
		FlexSidebar,
		FlexSection,
		FlexDrag,
		FlexConfiguration,
		ShapeUtils,
		_,
		$
	) {
		"use strict";


		/**
		 * @class
		 * Icon Side Panel layout This is the view initialized by the LayoutManager that will add all the layoutcomponents
		 * to the viewer. It can generate a an object representing the current state of the viewer and can be initialized by
		 * a state-String.
		 *
		 * NOTE widget that are both MainPanelContent and ModelPanelContent are not supported at the moment!
		 *
		 * @exports FlexLayout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {LayoutModel} layoutModel
		 * @param {JQueryObj} context - context where the layout will add its components to
		 * @param {Object} layoutState
		 * @param {LoadingIndicator} loadingIndicator - the loading indicator, which was created before all requests started
         * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)
		 */
		var FlexLayout = function (f, layoutModel, context, layoutState, loadingIndicator, initialLoader) {
			Layout.call(this, f, layoutModel, context, layoutState, loadingIndicator, initialLoader);
			PrivateChannel.installTo(this, f, "layout.flex");
			ListenerUtil.mixin(this, f);

			/**
			 * Default state of the layout, used to reset the layout
			 */

			this._configuration = FlexConfiguration;

			this._defaultLayout = this._getDefaultLayout();

			/**
			 * Mapping for all the items, maps items to their section
			 */
			this._sectionMapping = {};

			/**
			 * All the layout items mapped as Name -> FlexWidgetWrapper
			 */
			this._items = {};

			/**
			 * The component that is used to display the content in the viewport: graphics, CAD, annotations,...
			 */
			this._mainContent = null;

			//a debounced invalidate, to make sure not to many invalidates are triggered
			this._debouncedInvalidate = _.debounce(_.bind(this._invalidateLayout, this), 200);

			/**
			 * In this array all items that are for the main content are kept
			 */
			this._mainContentWidgetWrappers = [];

			this._topbar = $("<div>", {
				'class': "topbar"
			});
			this._toolbar = $("<div>", {
				'class': "toolbar"
			});
			this._mainContentWrapper = $("<div>", {
				'class': "maincontentwrapper"
			});

			var isEastUndefined = typeof this._configuration.east === "undefined" || typeof this._configuration.east.iconified === "undefined";
			var isWestUndefined = typeof this._configuration.west === "undefined" || typeof this._configuration.west.iconified === "undefined";

			this._sections = {
				north: new FlexSection(this._toolbar),
				east: new FlexSidebar(this.f, "right", "right", this._mainContentWrapper, isEastUndefined ? true : this._configuration.east.iconified),
				west: new FlexSidebar(this.f, "left", "left", this._mainContentWrapper, isWestUndefined ? true : this._configuration.west.iconified)
			};

			this._centerContent = $("<div>", {
				'class': "centercontent"
			});


			this._messagebar = $("<div />").addClass("messagebar");
			this._aboveCenter = $("<div />").addClass("aboveCenter");

			this._centerContent.append(this._aboveCenter, this._loadingIndicator.element, this._messagebar);




			this._mainContentWrapper.append(this._sections.west.getTilebarContext(),
				this._sections.west.getWidgetbarContext(),
				this._centerContent,
				this._rightWidgetbar,
				this._sections.east.getWidgetbarContext(),
				this._sections.east.getTilebarContext()
			);


			// initialize the section mapping of the layoutItems
			this._initializeSectionMapping(this._layoutState);
			// initialize the sections
			this._initializeSections(this._layoutState);

			// by default load the view mode
			this._activeMode = new FlexBasicMode(this.f, this);
			this._activeMode.toggle(true);

			this._wizardMode = new FlexWizardMode(this.f, this);


			// register all modes
			this._layoutModel.registerLayoutMode(this._activeMode);
			this._layoutModel.registerLayoutMode(this._wizardMode);




			var onCSSLoaded = _.after(2, _.bind(function () {
				this._cssLoaded = true;
				this._initializeDOM();
			}, this));

			this.f.emit("layout.loadCSS", "style/css/style.css", onCSSLoaded);
			this.f.emit("layout.loadCSS", "web-viewer/layout/flex/css/flexLayout.css", onCSSLoaded);

			//listen to resize events of the window
			$(window).resize(_.bind(function (e) {
				// http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
				if (e.target === window) {
					this._debouncedInvalidate();
				}
			}, this));

			//listen to widget events, to act on resized/collapsed/...
			this._registerOnLayoutEvents();

			this._invalidateLayout(); //this one is needed to provide the correct world coordinates		

			this._updateModePanel();

			this._flexDrag = new FlexDrag(this._mainContentWrapper);
			this._flexDrag.addFlexSidebarDropzone(this._sections.west, "west", this);
			this._flexDrag.addFlexSidebarDropzone(this._sections.east, "east", this);

		};

		FlexLayout.prototype = Object.create(Layout.prototype);
		FlexLayout.prototype.constructor = FlexLayout;

		FlexLayout.prototype._initializeDOM = function () {

			this._initialLoader.remove();
			delete this._initialLoader;
			this._context.append(this._topbar, this._toolbar, this._mainContentWrapper);
			this._invalidateLayout();
		};


		/**
		 * Add an item to the layout, the items needs to already added to the items-collection.
		 * It adds the item to the corect element on the correct location (depends on the previous/default state and already added items)
		 * @param {String} itemName
		 */
		FlexLayout.prototype._addLayoutItem = function (itemName) {
			var section = this._sectionMapping[itemName]; //get the section the items belongs to
			if (section !== undefined) { //make sure there is a section to add the item to
				var sectionItems = null;
				var itemState = null;
				if (this._layoutState) {
					sectionItems = _.map(this._layoutState[section].items, function (item) {
						return item.name;
					}); //just get all the names of the items in this section, not other information
					itemState = _.find(this._layoutState[section].items, function (item) {
						return item.name === itemName;
					}); //get the state of the to be added item
				} else {
					sectionItems = this._defaultLayout[section]; //default state only holds names of the items
				}
				if (sectionItems !== null && sectionItems !== undefined) { //the section should at least hold the item itself
					var index = _.indexOf(sectionItems, itemName);
					var nextItem = null;
					if (index === -1) {
						index = _.indexOf(this._defaultLayout[section], itemName);
						nextItem = this._items[this._defaultLayout[section][++index]];
						while (index < this._defaultLayout[section].length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
							nextItem = this._items[this._defaultLayout[section][++index]];
						}
					} else {
						nextItem = this._items[sectionItems[++index]]; //get the next item, so we know infront of which item the item needs to be added to
						while (index < sectionItems.length && nextItem === undefined) { //as long nextItem is not yet added to the layout, search for a next one
							nextItem = this._items[sectionItems[++index]];
						}
					}

					if (nextItem === undefined) { //no nextItem is found, the item is (for now) the last item in the section
						this._appendItemToSection(itemName, section); //append the item to the section
					} else { //add the item in front of the found nextItem
						this._prependItemBeforeItem(nextItem.name, itemName, section);
					}
					//now be sure to set the correct state of the item (active and collapsed or not)
					if (itemState !== null && itemState !== undefined) {
						this._setActive(itemName, itemState.isActive, itemState.isCollapsed);
						var widgetWrapper = this._items[itemName];
						if (widgetWrapper !== undefined && widgetWrapper.widget) {
							if (itemState.stickyDimensions) {
								widgetWrapper.setStickyDimensions(itemState.stickyDimensions);
							} else {
								widgetWrapper.setStickyDimensions(null);
							}
						}

					}

				}
			} else {
				this.f.debug.warn("FlexLayout :: No section found for item " + itemName);
			}
		};

		/**
		 * Set a widget active
		 * @param {String} itemName
		 * @param {Boolean} collapsed
		 */
		FlexLayout.prototype._setActive = function (itemName, active, collapsed) {
			var widgetWrapper = this._items[itemName];
			if (widgetWrapper !== undefined && widgetWrapper.widget) {
				var sectionName = this._sectionMapping[itemName];
				if (sectionName) {
					var section = this._sections[sectionName];
					if (section) {
						section.toggleWidgetWrapperActive(widgetWrapper, active);
						widgetWrapper.setCollapsed(collapsed);

					}
				}
			}
		};

		/**
		 * Appends an already added item (to the items collection) to a section.
		 * @param {String} itemName
		 * @param {String} section
		 */
		FlexLayout.prototype._appendItemToSection = function (itemName, section) {
			var layoutItem = this._items[itemName];
			if (layoutItem.widget) {
				layoutItem.widget.movingStarted();
			}

			this._sections[section].appendItem(layoutItem);

			if (layoutItem.widget) {
				layoutItem.widget.movingStopped();
			}
		};

		/**
		 * Prepends an already added item (to the items collection) before the nextItem in a given section.
		 * @param {String} nextItemName
		 * @param {String} itemName
		 * @param {String} section
		 */
		FlexLayout.prototype._prependItemBeforeItem = function (nextItemName, itemName, section) {
			var layoutItem = this._items[itemName];
			if (layoutItem.widget) {
				layoutItem.widget.movingStarted();
			}


			//add item before other item in the layout
			this._sections[section].prependItemBeforeItem(layoutItem, this._items[nextItemName]);

			if (layoutItem.widget) {
				layoutItem.widget.movingStopped();
			}
		};


		/**
		 * Moves an item to another section. Is used by draggable widgets to let the layout now it is moved.
		 * @param {String} itemName
		 * @param {String} sectionName
		 * @param {String} nextItemName - item will be moved before this item
		 * @param {Boolean} resetSizeAndLocation
		 * @param {Boolean} resetCollapsed
		 */
		FlexLayout.prototype.moveItem = function (itemName, sectionName, nextItemName, resetSizeAndLocation, resetCollapsed) {
			if (itemName !== nextItemName) {
				/*	if (resetSizeAndLocation) { //resets the size and location of the item (moves from center to a side-panel)
						this._setLocation(itemName, null);
						this._setSize(itemName, null);
					}
					if (resetCollapsed) { //resets the collapsed state of the item
						this._setCollapsed(itemName, false);
					}*/
				var currentSection = this._sectionMapping[itemName];
				this._detachItem(itemName, currentSection); //removes the item from the previous section
				this._sectionMapping[itemName] = sectionName;
				if (nextItemName === undefined || this._sectionMapping[nextItemName] !== sectionName) {
					this._appendItemToSection(itemName, sectionName);
				} else {
					//check if no problems with after parameter
					this._prependItemBeforeItem(nextItemName, itemName, sectionName);
				}
			}
		};

		/**
		 * Remove an item from a section, it removes the item from the DOM, but doesn't delete the DOM-element.
		 * @param {String} itemName
		 * @param {String} sectionName
		 */
		FlexLayout.prototype._detachItem = function (itemName, sectionName) {
			var widgetWrapper = this._items[itemName];
			if (widgetWrapper) {
				if (widgetWrapper.widget) {
					widgetWrapper.widget.movingStarted();
				}

				//remove element from the DOM (doesn't delete DOM-element)
				this._items[itemName].element.detach();

				if (widgetWrapper.widget) {
					widgetWrapper.widget.movingStopped();
				}
				//the invalidate is only done after all other changes (e.g. re-add DOM-element) are done
				_.defer(_.bind(function () {
					this._invalidateLayout();
				}, this));
			}
		};

		/**
		 * Initializes the section mapping. All items will be mapped to their section so it's easier to find the item's section.
		 * Items that aren't in the layoutstate but are in the default state will also be added to the mapping.
		 *
		 * @param {Object} [layoutState] - if not defined only the default layout is used for the mapping
		 */
		FlexLayout.prototype._initializeSectionMapping = function (layoutState) {
			//first run over all the sections and items in the layoutState
			if (layoutState !== undefined && layoutState !== "" && _.size(layoutState) !== 0) {
				_.each(layoutState, function (section, sectionName) { //runs over all sections
					_.each(section.items, function (item) { //runs over all items in those sections
						this._sectionMapping[item.name] = sectionName; //map each itemName->sectionName
					}, this);
				}, this);
			}
			//run over the defaultLayout, if items of the default layout aren't added yet, add them to the mapping
			_.each(this._getDefaultLayout(), function (items, sectionName) {
				_.each(items, function (item) {
					if (this._sectionMapping[item] === undefined) { //check if item isn't added by layoutState yet
						this._sectionMapping[item] = sectionName;
					}
				}, this);
			}, this);
		};


		/**
		 * Initializes the sections, get state from the layoutState.
		 */
		FlexLayout.prototype._initializeSections = function (layoutState) {
			if (layoutState !== undefined && layoutState !== "" && _.size(layoutState) !== 0) {
				_.each(layoutState, function (section, sectionName) { //run over all sections
					if (this._sections[sectionName] !== undefined && this._sections[sectionName].restoreState) {
						this._sections[sectionName].restoreState(section);
					}
				}, this);
			}

			var onSidebarChange = _.bind(function () {
				this._emitLayoutChanged();
				this._debouncedInvalidate();
			}, this);

			this._sections.east.on("changed.*", onSidebarChange);
			this._sections.west.on("changed.*", onSidebarChange);
		};

		/**
		 * Register listeners for general layout event.
		 */
		FlexLayout.prototype._registerOnLayoutEvents = function () {
			this.f.on("layout.disableTextSelection", _.bind(function () {
				this._disableTextSelection();
			}, this));

			this.f.on("layout.enableTextSelection", _.bind(function () {
				this._enableTextSelection();
			}, this));

			this.f.on("layout.resetToDefault", _.bind(this.resetToDefault, this));

			this.f.on("layout.switchLayoutMode", _.bind(function () {
				//this._updateModePanel();
				//this._updateMainContent();
			}, this));
		};

		/**
		 * Add an item to the layout. This is called by Layout to add an item.
		 * @param {Object} layoutItem
		 * @example
		 * 	{ 	type: {LayoutConstants.LayoutType},
		 * 		name: 'itemName',
		 * 		item: {AbstractWidget/ToolbarComponent}
		 * 	}
		 */
		FlexLayout.prototype._addItem = function (layoutItem) {
			if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
				var widget = layoutItem.getItem(this.getLayoutType());
				var widgetType = widget.getWidgetType();

				if (layoutItem.name === this._activeMode.getMainPanelContent()) {
					widgetType = "maincontent";
				}

				switch (widgetType) {
				case "messagebar":
					widget.setContext(this._messagebar);

					break;
				case "topbar": //widget needs to be added to the topbar
					widget.setContext(this._topbar);
					break;
				case "wizard":
					widget.setContext(this._aboveCenter);
					this._items[widget.getID()] = widget;
					break;

				case "maincontent":
				default:
					var widgetWrapper = new FlexWidgetWrapper(widget, layoutItem.name, true);
					this._items[layoutItem.name] = widgetWrapper;


					if (widgetType === "maincontent") {
						this._centerContent.append(widgetWrapper.element);
						this._mainContentWidgetWrappers.push(widgetWrapper);
						this._updateMainContent();
					} else {
						widgetWrapper.setActive(false);
						this._flexDrag.enableDragging(widgetWrapper);
						this._addLayoutItem(layoutItem.name); //make sure the item is added on the correct location in one of the panels
					}

					this._registerListener("enabled", function (isEnabled) {
						this._toggleWidget(layoutItem.name, widgetWrapper, isEnabled);
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("visible", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("collapsed", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("dragged", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("modalResize", function () {
						this._emitLayoutChanged();
					}, this, widget);

					this._registerListener("makeVisible", _.bind(this._makeWidgetVisible, this, layoutItem.name, widgetWrapper), undefined, widget);

					if (widget.didRequestMakeVisible()) {
						this._makeWidgetVisible(layoutItem.name, widgetWrapper);
					}


					break;
				}
			} else if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
				//add a toolbarcomponent
				var toolbarItem = layoutItem.getItem(this.getLayoutType());
				toolbarItem.element[0].name = layoutItem.name;
				this._items[layoutItem.name] = toolbarItem;
				this._addLayoutItem(layoutItem.name);
				this._registerListener("changed.visibility", function () {
					this._debouncedInvalidate();
				}, this, toolbarItem);
				this._debouncedInvalidate();
			} else {
				//not supported in this layout
			}
		};




		/**
		 * Update visibility of the added items based on whether or not they are allowed in the current mode.
		 * E.g. item was added in previous mode, but should not be shown in current mode
		 */
		FlexLayout.prototype._updateItemVisibility = function () {

			_.each(this._items, _.bind(function (layoutItem) {
				if (layoutItem.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
					var toolbar = layoutItem.getItem(this.getLayoutType());
					this._navigationBar.toggleComponent(toolbar.name, this._activeMode.isItemAllowed(layoutItem.name));
				} else if (layoutItem.type === LayoutConstants.LayoutItemType.WIDGET) {
					var widgetWrapper = layoutItem.getItem(this.getLayoutType());

					widgetWrapper.setVisible(this._activeMode.isItemAllowed(layoutItem.name) && widgetWrapper.widget.isVisible());
				}
			}, this));
		};

		/**
		 * Disable text selection in the entire viewer.
		 */
		FlexLayout.prototype._disableTextSelection = function () {
			this._context.find(".enableTextSelection").toggleClass("enableTextSelection", false).toggleClass("enableTextSelectionTempDisabled", true);
		};

		/**
		 * Enable text selection in the entire viewer.
		 */
		FlexLayout.prototype._enableTextSelection = function () {
			this._context.find(".enableTextSelectionTempDisabled").toggleClass("enableTextSelection", true).toggleClass("enableTextSelectionTempDisabled", false);
		};

		/**
		 * Reset the sate of the layout to the default state.
		 */
		FlexLayout.prototype.resetToDefault = function () {

			var iconifiedWest = this._configuration.west && this._configuration.west.iconified;
			var iconifiedEast = this._configuration.east && this._configuration.east.iconified;

			this._sections.east.setIconified(iconifiedEast);
			this._sections.east.setWidgetBarWidth(null);
			this._sections.west.setIconified(iconifiedWest);
			this._sections.west.setWidgetBarWidth(null);

			_.each(this._getDefaultLayout(), function (section, sectionName) {

				if (sectionName === "west" || sectionName === "east") { //reset the width of the side panels to the default

					_.each(section, function (itemName) { //run over each item in the section
						var item = this._items[itemName];
						if (item !== undefined && this._activeMode.isItemAllowed(itemName)) { //move the item to its default location.
							this.moveItem(itemName, sectionName, undefined, true, true);
						}

						if (item && item.wrapper) {
							item.wrapper.setStickyDimensions(null);
						}
					}, this);
				}
			}, this);

			this._layoutState = this._getDefaultLayout(); //reset the state of the layout, it now is the default\

			this._updateItemVisibility();
			this._invalidateLayout();
		};

		/**
		 * Get default layout state
		 * @returns {Object}
		 */
		FlexLayout.prototype._getDefaultLayout = function () {
			return this._configuration.layout;
		};

		/**
		 * Generates the state of the layout as a string. It takes the not-visible elements into account so state of those
		 * elements is not lost.
		 * @returns {Object}
		 */
		FlexLayout.prototype.getLayoutState = function () {
			var stateObject = {};
			_.each(this._sections, _.bind(function (section, sectionName) {

				var context;
				if (sectionName === "north") {
					context = this._toolbar;
					stateObject[sectionName] = {};
				} else {
					context = this._sections[sectionName].getWidgetbarContainer();
					stateObject[sectionName] = this._sections[sectionName].getState();
				}

				stateObject[sectionName].items = [];

				_.each(context.children(), _.bind(function (item) {
					if (item !== null && item !== undefined) {
						var itemWrapper = item.wrapper;
						var itemObject = {};
						if (itemWrapper) {
							itemObject.name = itemWrapper.name;
							itemObject.isCollapsed = itemWrapper.isCollapsed;
							itemObject.isActive = itemWrapper.isActive();
							itemObject.stickyDimensions = itemWrapper.getStickyDimensions();
							stateObject[sectionName].items.push(itemObject);
						}
					}
				}, this));
			}, this));

			return stateObject;
		};

		/**
		 * Invalidates the layout. Makes sure all panels have the correct size, resize is triggered on the correct elements,...
		 */
		FlexLayout.prototype._invalidateLayout = function () {
			if (!this._cssLoaded) {
				return;
			}


			if (this._activeMainPanelContent) {
				this._activeMainPanelContent.content.css({
					"height": this._centerContent.innerHeight(),
					"width": this._centerContent.innerWidth()
				});

				if (this._activeMainPanelContent.widget !== null) {
					this._activeMainPanelContent.widget.resized();
				}

			}
			// make sure subtoolbar are moved when new items are added
			_.each(this._items, function (layoutItem, itemName) {
				if (this._layoutModel.getLayoutItemType(itemName) === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT) {
					layoutItem.moved();
				}
			}, this);


			this._sections.east.invalidateLayout();
			this._sections.west.invalidateLayout();
		};

		/**
		 * Returns the type of the layout.
		 * @override
		 * @return {Number}
		 */
		FlexLayout.prototype.getLayoutType = function () {
			return LayoutConstants.LayoutType.FLEX;
		};

		/**
		 * Get all the currently visible popups
		 * @returns {Array.<JQueryObj>}
		 */
		FlexLayout.prototype.getAllVisiblePopups = function () {
			return this._context.find(".popup:visible");
		};

		/**
		 * Enable/disable fullscreen
		 * Note: this method should be called from a user generated event... (or it won't work...)
		 * @param enabled
		 */
		FlexLayout.prototype.toggleFullscreen = function (enabled) {
			if (enabled) {
				this._requestFullscreen();
			} else {
				this._exitFullscreen();
			}
		};

		/**
		 * Check if the app is fullscreen
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.isFullscreen = function () {
			return document.fullscreenElement || // alternative standard method
				document.mozFullScreenElement || document.webkitFullscreenElement || document.msFullscreenElement;
		};

		/**
		 * Check if the browser supports the fullescren API
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.supportsFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			return viewerDiv.requestFullscreen || viewerDiv.mozRequestFullScreen || viewerDiv.webkitRequestFullscreen || viewerDiv.msRequestFullscreen;
		};

		/**
		 * Request fullscreen for the whole document.body
		 */
		FlexLayout.prototype._requestFullscreen = function () {
			var viewerDiv = this.f.getContext().get(0);
			if (viewerDiv.requestFullscreen) {
				viewerDiv.requestFullscreen();
			} else if (viewerDiv.mozRequestFullScreen) {
				viewerDiv.mozRequestFullScreen();
			} else if (viewerDiv.webkitRequestFullscreen) {
				viewerDiv.webkitRequestFullscreen();
			} else if (viewerDiv.msRequestFullscreen) {
				viewerDiv.msRequestFullscreen();
			}
		};

		/**
		 * Exit fullscreen mode
		 */
		FlexLayout.prototype._exitFullscreen = function () {
			if (document.exitFullscreen) {
				document.exitFullscreen();
			} else if (document.msExitFullscreen) {
				document.msExitFullscreen();
			} else if (document.mozCancelFullScreen) {
				document.mozCancelFullScreen();
			} else if (document.webkitExitFullscreen) {
				document.webkitExitFullscreen();
			}
		};

		/**
		 * Show or hide mode panel, depending on the active mode
		 */
		FlexLayout.prototype._updateModePanel = function () {
			//
		};

		/**
		 * Update the contents of the main panel
		 * A mode can decide to show something else than the viewport widget
		 */
		FlexLayout.prototype._updateMainContent = function () {

			var mainPanelContent = this._activeMode.getMainPanelContent();
			this._activeMainPanelContent = null;

			// run through all possible main content widget wrappers
			_.each(this._mainContentWidgetWrappers, function (wrapper) {
				if (wrapper.widget.getID() === mainPanelContent) {
					// this wrapper should be visible
					wrapper.setVisible(true);
					wrapper.setHeaderEnabled(false);

					this._activeMainPanelContent = wrapper;
				} else {
					// hide, the wrapper should not be active
					wrapper.setVisible(false);
					wrapper.setHeaderEnabled(true);
				}
			}, this);

			if (!this._activeMainPanelContent && this._items[mainPanelContent]) {
				// we didn't find the widget in the _mainContentWidgetWrappers, but it might be initialized anyway
				// append it to the center content

				this._mainContentWidgetWrappers.push(this._items[mainPanelContent]);
				this._centerContent.append(this._items[mainPanelContent].element);
				this._updateMainContent();
				return;
			}

			this._debouncedInvalidate();
		};



		/**
		 * Enable/disable a widget
		 * @param {String}        widgetName    - name of the widget
		 * @param {WidgetWrapper} widgetWrapper - widgetWrapper in which the widget is displayed
		 * @param {Boolean}       isEnabled     - should the widget be enabled
		 */
		FlexLayout.prototype._toggleWidget = function (widgetName, widgetWrapper, isEnabled) {
			if (isEnabled) {
				if (this._activeMode.isItemAllowed(widgetName)) {
					widgetWrapper.setVisible(true);
				}
			} else {
				widgetWrapper.setVisible(false);
			}
		};

		/**
		 * Returns the default layout mode for the layout.
		 * @return {LayoutMode}
		 */
		FlexLayout.prototype._getDefaultLayoutMode = function () {
			return this._layoutModel.getLayoutMode(LayoutConstants.LayoutMode.BASIC);
		};

		FlexLayout.prototype.isItemAllowed = function (itemname) {
			return !_.contains(this._configuration.disabledItems, itemname);
		};

		/**
		 * Enable/disable the sidebars
		 * @param {Boolean} enabled
		 */
		FlexLayout.prototype.setSidebarsEnabled = function (enabled) {
			this._sections.east.setEnabled(enabled);
			this._sections.west.setEnabled(enabled);
		};

		/**
		 * Hide a layout item.
		 * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
		 */
		FlexLayout.prototype.hideItem = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined && layoutItem.context !== undefined) {
				layoutItem.context.hide();
			} else if (layoutItem !== undefined && layoutItem.element !== undefined) {
				layoutItem.element.hide();
			}
		};

		/**
		 * Makes a hidden layout item visible.
		 * @param {LayoutConstants.Widget|LayoutConstants.ToolbarComponent} itemName
		 */
		FlexLayout.prototype.showItem = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined && layoutItem.context !== undefined) {
				layoutItem.context.show();
			} else if (layoutItem !== undefined && layoutItem.element !== undefined) {
				layoutItem.element.show();
			}
		};

		/**
		 * Is the layout item visible.
		 * @param {String} itemName
		 * @returns {Boolean}
		 */
		FlexLayout.prototype.isItemVisible = function (itemName) {
			var layoutItem = this._items[itemName];
			if (layoutItem !== undefined) {
				return layoutItem.isVisible();
			}
		};
		/**
		 * Make sure a widget is visible
		 * Uncollapse it if it is collapsed
		 *
		 * If the widget is modal, show it
		 * If the widget is not modal, make the widget modal if it is in a hidden/collapsed sidebar
		 * @param {String} layoutItemName - the name of the Layout Item
		 * @param {WidgetWrapper} widgetWrapper - the widgetWrapper for the widget
		 */
		FlexLayout.prototype._makeWidgetVisible = function (layoutItemName, widgetWrapper) {
			this._sections[this._sectionMapping[layoutItemName]].toggleWidgetWrapperActive(widgetWrapper, true);
			widgetWrapper.widget.setMakeVisibleRequestAccepted();
			widgetWrapper.setCollapsed(false);
		};
		return FlexLayout;
	});
