/**
 * @namespace MeasureModule
 */
define(["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/measure/RulerTool",
		"web-viewer/measure/BarcodeMeasureTool",
		"web-viewer/measure/BrailleMeasureTool",
		"web-viewer/measure/EyeDropperMeasureTool",
		"web-viewer/measure/CaliperTool",
		"web-viewer/measure/CaliperModel",
		"web-viewer/measure/ScreenRulingMeasureTool",
		"web-viewer/measure/MeasureToolbarComponent",
		"web-viewer/measure/MeasureConstants",
		"web-viewer/utils/ListenerUtil",
		"utils/XMLUtils",
		"jquery"
	],
	function (_,
		AbstractModule,
		RulerTool,
		BarcodeMeasureTool,
		BrailleMeasureTool,
		EyeDropperMeasureTool,
		CaliperTool,
		CaliperModel,
		ScreenRulingMeasureTool,
		MeasureToolbarComponent,
		MeasureConstants,
		ListenerUtil,
		XMLUtils,
		$) {

		"use strict";

		/**
		 * Create the MeasureModule
		 *
		 * @exports MeasureModule
		 * @memberof MeasureModule
		 * @class
		 * Responsible for publishing the correct tools and toolbarcomponents enabling navigation
		 * @augments Core.AbstractModule
		 * @mixes Utils.ListenerUtilMixin
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var MeasureModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "measure");
			ListenerUtil.mixin(this, f);

			$.when(this._checkBarCodeAndBrailleLicenses()).done(_.bind(function (hasBarCodeLicense, hasBrailleLicense) {
				this.tools = [];

				this.rulerTool = new RulerTool(f, "ruler");
				this._publishTool(this.rulerTool);
				this.tools.push(this.rulerTool);

				if (hasBarCodeLicense) {
					this.barcodeTool = new BarcodeMeasureTool(f, this);
					this._publishTool(this.barcodeTool);
					this.tools.push(this.barcodeTool);
				}

				if (hasBrailleLicense) {
					this.brailleTool = new BrailleMeasureTool(f, this);
					this._publishTool(this.brailleTool);
					this.tools.push(this.brailleTool);
				}

				this.eyedropperTool = new EyeDropperMeasureTool(f, "eyedropper");
				this._publishTool(this.eyedropperTool);
				this.tools.push(this.eyedropperTool);

				this.screenRulingTool = new ScreenRulingMeasureTool(f, "screenruling");
				this._publishTool(this.screenRulingTool);
				this.tools.push(this.screenRulingTool);

				this.caliperModel = new CaliperModel(f, [this.getPrivateChannelName(), "caliper", "model"]);
				this.caliperTool = new CaliperTool(f, this.caliperModel);
				this._publishTool(this.caliperTool);
				this.tools.push(this.caliperTool);

				this._publishToolbarComponent(MeasureToolbarComponent.toolbarName, _.bind(function () {
					this._measureToolbarComponent = new MeasureToolbarComponent(f, this.rulerTool, this.eyedropperTool, this.barcodeTool, this.brailleTool, this.caliperTool, this.screenRulingTool);
					return this._measureToolbarComponent;
				}, this));

				this._registerListener("module.graphics.model.changed.**", _.debounce(_.partial(this.updateMeasureTools, false), 10), this);
				this._registerListener("module.compare.referenceOffset.changed", _.partial(this.updateMeasureTools, true), this);
			}, this));
		};


		// inherit
		MeasureModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		MeasureModule.prototype.constructor = MeasureModule;

		/**
		 * Updates the data displayed in the measure tool
		 * @param {Boolean} forceUpdate - forces the update of the data in the measure tool
		 */
		MeasureModule.prototype.updateMeasureTools = function (forceUpdate) {
			_.each(this.tools, function (tool) {
				if (tool.isActive()) {
					tool.updateIfNeeded(forceUpdate);
				}
			});
		};

		/**
		 * Get the state of all the measure tools
		 * @returns {Object}
		 */
		MeasureModule.prototype.getState = function () {
			var measureState = {
				rulerState: this.rulerTool.getToolState(),
				eyedropperState: this.eyedropperTool.getToolState(),
				caliperState: this.caliperTool.getToolState(),
				sizeUnitState: this.f.getOption("PREFERENCES_UNIT_SIZE")
			};

			if (this.barcodeTool) {
				measureState.barcodeState = this.barcodeTool.getToolState();
			}
			if (this.brailleTool) {
				measureState.brailleState = this.brailleTool.getToolState();
			}

			return measureState;
		};

		/**
		 * Set the state of the measure tools
		 * @param {Object} state
		 */
		MeasureModule.prototype.setState = function (state) {
			if (state.rulerState !== undefined) {
				if (state.rulerState.isActive) {
					this.f.emit("toolManager.activateTool", this.rulerTool);
				}
				this.rulerTool.setToolState(state.rulerState);
				this.f.emit("toolManager.interruptTools", true);
			}
			if (state.barcodeState !== undefined && this.barcodeTool) {
				this.barcodeTool.setToolState(state.barcodeState);
			}
			if (state.brailleState !== undefined && this.brailleTool) {
				this.brailleTool.setToolState(state.brailleState);
			}
			if (state.eyedropperState !== undefined) {
				this.eyedropperTool.setToolState(state.eyedropperState);
			}
			if (state.caliperState !== undefined) {
				this.caliperTool.setToolState(state.caliperState);
			}
			if (state.sizeUnitState !== undefined) {
				this.f.setOption("PREFERENCES_UNIT_SIZE", state.sizeUnitState);
			}

		};

		/**
		 * Check licenses for barcode and braille
		 * @returns {$.Deferred.<Boolean, Boolean>} Two parameters where the first one if true indicates has barcode license 
		 *                                          and second one if true indicates has braille license
		 */
		MeasureModule.prototype._checkBarCodeAndBrailleLicenses = function () {
			var deferred = $.Deferred();

			$.when(this._getServerInfoFlags()).done(function (flags) {
				var hasBarCodeLicense = (flags & MeasureConstants.ServerInfoFlags.barCode) === MeasureConstants.ServerInfoFlags.barCode ||
					(flags & MeasureConstants.ServerInfoFlags.barCodeGV) === MeasureConstants.ServerInfoFlags.barCodeGV;
				var hasBrailleLicense = (flags & MeasureConstants.ServerInfoFlags.braille) === MeasureConstants.ServerInfoFlags.braille;

				deferred.resolve(hasBarCodeLicense, hasBrailleLicense);
			});

			return deferred.promise();
		};


		/**
		 * Get the ServerInfo flags (can be used for checking licenses)
		 * @return {$.Deferred.<Number>} Flags number. 0 if something went wrong.
		 */
		MeasureModule.prototype._getServerInfoFlags = function () {
			var deferred = $.Deferred();
			if (this._serverFlags !== undefined) {
				deferred.resolve(this._serverFlags);
			} else {
				this.f.ajax(this.f.getDataAdapter().ViewServer("getserverinfo.jsp"), {
					success: _.bind(function (result) {
						if (result) {
							result = XMLUtils.xml2obj(result);
							if (result && result.status && result.status.value == "0") {
								if (result.status.server) {
									this._serverFlags = parseInt(result.status.server.flags, 10);
									deferred.resolve(this._serverFlags);
									return;
								}
							}
						}

						this.f.emit("error.WARNING", "Problem while fetching server flags");
						deferred.resolve(0);
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						// Error while fetching licensing info
						this.f.emit("error.WARNING", "Problem while fetching server flags", textStatus, errorThrown);
						deferred.resolve(0);
					}, this)
				});
			}
			return deferred.promise();
		};

		return MeasureModule;
	});