/**
 * @namespace SvgModule
 */
define(["web-viewer/core/AbstractModule",
		"web-viewer/compare/CompareConstants",
		"./SvgDataExtensions",
		"./SvgLayerModel",
		"./SvgDiffLayerModel",
		"./TextSelectionTool",
		"./TextSelectionModel",
		"web-viewer/layout/LayoutConstants",
		"component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"underscore"
	],
	function (AbstractModule,
		CompareConstants,
		SvgDataExtensions,
		SvgLayerModel,
		SvgDiffLayerModel,
		TextSelectionTool,
		TextSelectionModel,
		LayoutConstants,
		ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		_) {
		"use strict";

		/**
		 * Module for enable viewing of svg files
		 * @param {ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var SvgModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "svg");
			SvgDataExtensions(f);

			/**
			 * @member {SvgLayerModel} SvgLayerModel
			 * @private
			 */
			this._layerModel = null;

			/**
			 * @member {SvgDiffLayerModel} SvgDiffLayerModel
			 * @private
			 */
			this._diffLayerModel = null;

			f.getDocumentController().whenActiveOrDifferencePageChangedAndParametersAvailable(_.bind(this._setSVGLayerModels, this));

			// Toggle the diff layer 
			this.f.on("module.compare.changed.diffMode", _.bind(function (oldMode, newMode) {
				this.f.getModule("viewport").toggleLayer("SVGDifferenceLayer", newMode === CompareConstants.diffMode.READERSPREAD);
			}, this));


			this._textSelectionModel = new TextSelectionModel(f, [this.getPrivateChannelName(), "textSelection", "model"]);

			// Add text selection tool to the toolbar
			this._textSelectionTool = new TextSelectionTool(f, this._textSelectionModel);
			this._publishTool(this._textSelectionTool);

			this._textSelectionButtonGroup = null;

			this._publishToolbarComponent(LayoutConstants.ToolbarComponent.TEXTSELECTION, _.bind(function (layoutType) {
				/**
				 * Component: Textselection
				 */
				var textSelectionToolbar = new ToolbarComponent(LayoutConstants.ToolbarComponent.TEXTSELECTION, f);


				/**
				 * Group
				 */
				var selectionGroup = new ButtonGroup("selectionGroup");
				selectionGroup.setTitle(f.i18n, "viewer", "TextBtnGroupTitle");
				textSelectionToolbar.addButtonGroup(selectionGroup);

				this._textSelectionButtonGroup = selectionGroup;

				// select button
				var textSelectionButton = new ViewerToolbarButton(f, "TextSelectionButton", "icon icon-WCR_Viewer-TextSelection", false);
				textSelectionButton.setTooltip("viewer", "TextSelectionTT");
				textSelectionButton.listenToTool(this._textSelectionTool, true);
				selectionGroup.addButton(textSelectionButton);


				return textSelectionToolbar;
			}, this));
		};

		// inherit
		SvgModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		SvgModule.prototype.constructor = SvgModule;

		/**
		 * set the correct pages for the svg layer models
		 * @param {WebCenterDocumentPage} page
		 */
		SvgModule.prototype._setSVGLayerModels = function () {
			var page = this.f.getActivePage();
			var hasSVG = page.hasSvg();
			if (hasSVG) {
				if (!this._layerModel) {
					this._layerModel = new SvgLayerModel(this.f, "SVGLayer", page);
				} else {
					this._layerModel.setPage(page);
				}

				var diffPage = this.f.getDocumentController().getDifferencePage();

				// could be that diffpage is null -> no problem
				if (!this._diffLayerModel) {
					this._diffLayerModel = new SvgDiffLayerModel(this.f, "SVGDifferenceLayer", diffPage);
				}

				if (this._textSelectionButtonGroup) {
					this._textSelectionButtonGroup.show();
				}

			} else {
				if (this._textSelectionButtonGroup) {
					this._textSelectionButtonGroup.hide();
				}
				if (this._textSelectionTool) {
					this._textSelectionTool.deactivate();
				}
			}

			// Set/Remove svg layers
			this.f.whenModuleAvailable("viewport", function (viewportModule) {
				viewportModule.whenWidgetModelAvailable(function () {
					if (hasSVG) {
						viewportModule.addLayer("SVGLayer", this._layerModel);
						viewportModule.addLayer("SVGDifferenceLayer", this._diffLayerModel);
					} else {
						viewportModule.removeLayer("SVGLayer");
						viewportModule.removeLayer("SVGDifferenceLayer");
					}


				}, this);
			}, this);
		};

		return SvgModule;

	});
