define(["web-viewer/core/AbstractModule",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils",
		"web-viewer/textContent/DynContentWidget",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/textContent/DynContentModel",
		"web-viewer/textContent/DynContentLayerModel",
		"web-viewer/textContent/DynContentPopupsLayerModel",
		"web-viewer/textContent/DynContentTool",
		"web-viewer/textContent/DynContentToolbarComponent",
		"packaging-content-management/ContentConstants",
		"underscore",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-services/elementTypeService"
	],
	function (AbstractModule,
		ListenerUtil,
		ShapeUtils,
		DynContentWidget,
		ModalPopup,
		DynContentModel,
		DynContentLayerModel,
		DynContentPopupsLayerModel,
		DynContentTool,
		DynContentToolbarComponent,
		ContentConstants,
		_,
		languageService,
		elementTypeService) {

		"use strict";
		/**
		 * This module is responsible for adding the text content widget
		 *
		 * @exports TextContentModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var TextContentModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "dyncontent");
			ListenerUtil.mixin(this, f);

			/**
			 * Layer Models
			 */
			this.layerModels = {};

			/**
			 * The dynamic content model
			 */
			this.dynContentModel = new DynContentModel(f, 'dyncontent');

			/**
			 * Component: dynamic content
			 */
			this._dynContentComponent = null; // set the annotation tool bar component as unavailable


			this.f.getDocumentController().whenActivePageChanged(_.bind(this._whenActivePageChanged, this));

			/*
			 * What to do when we need to zoom to certain dyn contents.
			 */
			this.on("zoomToDynContents", _.bind(function (dynamicContentAnnotations) {
				var options = {};
				// Constrain the zoom so it never goes lower than 2 cm
				var zoomRect;
				if (dynamicContentAnnotations && dynamicContentAnnotations.length == 1) {
					// Single annotation selected, zoom on the single annotation.
					zoomRect = dynamicContentAnnotations[0].rect.copy();
				} else {
					// Multiple annotation selected, zoom on the bounding box.
					var smallestX = Number.MAX_VALUE;
					var smallestY = Number.MAX_VALUE;
					var biggestX = Number.MIN_VALUE;
					var biggestY = Number.MIN_VALUE;
					for (var i = 0; i < dynamicContentAnnotations.length; i++) {
						var rectOrigin = dynamicContentAnnotations[i].rect.origin;
						var rectSize = dynamicContentAnnotations[i].rect.size;
						var rectBiggestX = rectOrigin.x + rectSize.width;
						var rectSmallestX = rectOrigin.x;
						var rectBiggestY = rectOrigin.y + rectSize.height;
						var rectSmallestY = rectOrigin.y;
						if (smallestX > rectSmallestX) {
							smallestX = rectSmallestX;
						}
						if (smallestY > rectSmallestY) {
							smallestY = rectSmallestY;
						}
						if (biggestX < rectBiggestX) {
							biggestX = rectBiggestX;
						}
						if (biggestY < rectBiggestY) {
							biggestY = rectBiggestY;
						}
					}
					var origin = new ShapeUtils.Point(smallestX, smallestY);
					var size = new ShapeUtils.Size(biggestX - smallestX, biggestY - smallestY);
					zoomRect = new ShapeUtils.Rect(origin, size);
				}

				var minSize = 20;

				var diff = null;
				if (Math.abs(zoomRect.size.width) < minSize) {
					diff = (minSize - zoomRect.size.width) / 2;
					zoomRect.origin.x -= diff;
					zoomRect.size.width = minSize;
				}
				if (Math.abs(zoomRect.size.height) < minSize) {
					diff = (minSize - zoomRect.size.height) / 2;
					zoomRect.origin.y -= diff;
					zoomRect.size.height = minSize;
				}

				// Convert annotation rect to world
				var transform = this.layerModels['DynContentLayer'].getTransform().copy();
				options.rect = transform.transformedRect(zoomRect);
				options.constrain = true;
				options.animated = true;

				this.dynContentModel.ensureVisibility();

				f.emit("viewport.fitInRect", options);
			}, this));

			this._docVersionID = this.f.getActiveDocument().getDocumentVersion().get("docversionid");
			this._projectID = this.f.getActiveDocument().getProjectID();
		};

		// inherit
		TextContentModule.prototype = Object.create(AbstractModule.prototype);
		TextContentModule.prototype.constructor = TextContentModule;

		/**
		 * When the dynamic contents are loaded ...
		 */
		TextContentModule.prototype.onDynContentsLoaded = function (dynContents) {
			// Check if actually dyn contents were loaded.
			if (dynContents && dynContents.length === 0) {
				return;
			}
			this._dynContents = dynContents;
			var contentStatementList = [];
			for (var i = 0; i < dynContents.length; i++) {
				if (dynContents[i].getDynContentType() == "Text") {
					var element = dynContents[i].getTextElement();
					var contentStatement = element.parentStatement;
					if ($.inArray(contentStatement, contentStatementList) < 0) {
						contentStatementList.push(contentStatement);
					}
				}
			}
			var textContentsLangs = "";
			_.each(contentStatementList, _.bind(function (contentStatement) {
				if (contentStatement && contentStatement.element && contentStatement.element.length > 0) {
					_.each(contentStatement.element, _.bind(function (csElement) {
						if (csElement && csElement.master) {
							textContentsLangs += csElement.language + ":m,";
						} else if (csElement) {
							textContentsLangs += csElement.language + ":t,";
						}
					}, this));
				}
			}, this));
			$.when(this._loadElementTypeList(), this._loadLanguages(textContentsLangs)).done(_.bind(function (csList, elementTypeList, languagesList) {

				// Check if at least one master language was selected in the project.
				var foundMasterLang = false;
				if (languagesList && languagesList.projectMasterLangs) {
					foundMasterLang = Object.keys(languagesList.projectMasterLangs).length;
				}

				// Check if a master language was found before publishing the tools and the widget.
				if (foundMasterLang) {

					// Publish the dynamic content widget.
					this._publishWidget(DynContentWidget.widgetName, _.bind(function (layoutType) {
						this._dynContentWidget = new DynContentWidget(this, this.f, contentStatementList, elementTypeList, languagesList);
						return this._dynContentWidget;
					}, this, contentStatementList, elementTypeList, languagesList));

					// Publish the dynamic content tool.
					this._selectTool = new DynContentTool(this.f, this.dynContentModel);
					this._publishTool(this._selectTool);

					// publish annotate tool bar component
					this._publishToolbarComponent(DynContentToolbarComponent.toolbarName, _.bind(function (layoutType) {
						this._dynContentComponent = new DynContentToolbarComponent(this.f, this.dynContentModel, this._selectTool);
						return this._dynContentComponent;
					}, this));

				} else {
					// Disable the dynamic content annotations.
					this.dynContentModel.disable();
				}


			}, this, contentStatementList));
		};

		/**
		 * Are there still unsaved Text?
		 * @override
		 * @returns {Boolean}
		 */
		TextContentModule.prototype.hasUnsavedElements = function () {
			return this.dynContentModel.getUnsavedTextContentsCount() > 0;
		};

		/**
		 * @override
		 * @returns {String}
		 */
		TextContentModule.prototype.getUnsavedElementsName = function () {
			return this.f.i18n.t("PCMTextWidgetTitle");
		};
		
		/**
         * Save all unsaved elements
         * @override
         * @param callbackFunction
         */
		TextContentModule.prototype.saveAllUnsavedElements = function (callbackFunction) {
            this.dynContentModel.save(null, callbackFunction);
        };

		/**
		 * Prompt the user the save unsaved text content changes (if there are unsaved text content changes)
		 * @param {Function} callbackOnOk     - called when everything is OK (text contents saved)
		 * @param {Function} callbackOnCancel - called when the user cancels the operation (text contents are not saved)
		 * @param {Boolean} [canNotBeCancelled] - indicates the action cannot be cancelled, it is either revert or save
		 * @returns {Boolean} true if a prompt is shown
		 */
		TextContentModule.prototype.promptToSaveUnsavedChanges = function (callbackOnOk, callbackOnCancel, canNotBeCancelled) {
			if (this.hasUnsavedElements()) {
				this.f.loadBundle("viewer", function () {

					var modalPopup = new ModalPopup(this.f, "saveUnsavedTextContentChanges", this.f.getContext(), canNotBeCancelled);
					modalPopup.setRemoveFromDomAfterUsage(true);

					// the user wants to Save attributes before the callbackOnOk is executed
					modalPopup.setOkAction(_.bind(function () {
						// indicate to user that save operation is ongoing
						var loadingIndicator = $("<div>").css({
							'background': "url('" + requirejs.toUrl('web-viewer/layout/img/loaderTransparent.gif') + "') no-repeat 2px center",
							'padding-left': '20px',
							'line-height': '20px'
						}).addClass("unsavedItemsContent").text(this.f.t("PCMSaveOngoing"));
						modalPopup.setContent(loadingIndicator);

						//disable buttons while saving
						modalPopup.toggleButtons(false);
						modalPopup.toggleHeaderIcons(false);

						var validatePasswordAndSaveCallback = this._dynContentWidget.getValidatePasswordAndSaveHandler();

						$.when(validatePasswordAndSaveCallback()).done(function (status) {
							if (status === ContentConstants.ServerCall.SUCCESS) {
								// text contents are saved successfully!
								callbackOnOk();
							} else {
								// save operation failed
								if (callbackOnCancel) {
									callbackOnCancel();
								}
							}
							modalPopup.hide();
						}).fail(function () {
							if (callbackOnCancel) {
								callbackOnCancel();
							}
							modalPopup.hide();
						});

					}, this));

					if (!canNotBeCancelled) {
						// cancel the pending action
						modalPopup.setCancelAction(_.bind(function () {
							if (callbackOnCancel) {
								callbackOnCancel();
							}
						}, this));
						modalPopup.setCancelBtnText(this.f.i18n.t("Cancel"));
					}

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.t("PCMUnsavedWarningTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.t("PCMUnsavedWarningMsg")));
					modalPopup.setOkBtnText(this.f.i18n.t("Save"));
					modalPopup.show();
				}, this);
				return true;
			} else {
				// everything is OK, go on with OK action
				callbackOnOk();
				return false;
			}
		};

		TextContentModule.prototype._loadElementTypeList = function () {
			var deferred = $.Deferred();

			// load element type list
			elementTypeService.getElementTypes(_.bind(function (elementTypes) {
				deferred.resolve(elementTypes);
			}, deferred));

			return deferred.promise();
		};

		TextContentModule.prototype._loadLanguages = function (textContentsLangs) {
			var deferred = $.Deferred();

			// load language list
			languageService.getLanguages(this._projectID, _.bind(function (projectMasterLangs, projectTranslationLangs, allMasterLangs, allTranslationLangs) {
				var languagesList = {};
				languagesList.projectMasterLangs = projectMasterLangs;
				languagesList.projectTranslationLangs = projectTranslationLangs;
				languagesList.allMasterLangs = allMasterLangs;
				languagesList.allTranslationLangs = allTranslationLangs;

				deferred.resolve(languagesList);
			}, deferred), textContentsLangs);

			return deferred.promise();
		};

		/**
		 * When the active page has changed, react on it:
		 *
		 * Load text content data, unregister/unregister listners and display the widget if needed
		 *
		 * @param {AbstractDocumentPage} oldPage - previous active page
		 * @param {AbstractDocumentPage} newPage - new active page
		 */
		TextContentModule.prototype._whenActivePageChanged = function (oldPage, newPage) {

			//handle new page
			if (this.layerModels['DynContentLayer'] === undefined) {
				// Create the annotations layer
				this.layerModels['DynContentLayer'] = new DynContentLayerModel(this.f, "DynContent", this.dynContentModel);
				this.layerModels['DynContentPopupsLayer'] = new DynContentPopupsLayerModel(this.f, "DynContentPopups", this.dynContentModel);

				var viewportModule = this.f.getModule("viewport");
				// Global layer
				viewportModule.addLayer("Dyncontent", this.layerModels['DynContentLayer']);
				viewportModule.addLayer("Dyncontentpopups", this.layerModels['DynContentPopupsLayer']);
			}

			// unregister listeners
			if (oldPage) {
				this._unregisterAllListenersForObject(oldPage);
			}

			if (newPage) {
				//When attribute save happens in any AttributeCategoryModule(in the Attributes - Text Content View)
				//reload the widget with updated text contents
				this._registerListener("attributeCategory.saveFinished", function () {
					this._attributeSaveHappened = true;
				}, this, newPage);

				this._registerListener("attributeCategory.modalPopupClosed", function () {
					if (this._attributeSaveHappened === true) {
						this._attributeSaveHappened = false;
						this.dynContentModel.setPage(newPage, function (dynContents) {
							this.onDynContentsLoaded(dynContents);
						}.bind(this), true);
						this.dynContentModel.modelChanged();
					}
				}, this, newPage);

				this.dynContentModel.setPage(newPage, function (dynContents) {
					this.onDynContentsLoaded(dynContents);
				}.bind(this));
				this.dynContentModel.modelChanged();
			}
		};

		/**
		 * Select all the dynamic content annotations for a given Content Statement.
		 * 
		 * @param contentStatement
		 */
		TextContentModule.prototype.selectDynContentBasedOn = function (contentStatement, langCode) {
			this.dynContentModel.selectDynContentBasedOn(contentStatement, langCode);
		};

		return TextContentModule;
	});