/**
 * @namespace View3DModule
 */
define(["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/view3D/View3DWidget",
		"web-viewer/view3D/View3DToolbarComponent",
		"jquery"
	],
	function (_,
		AbstractModule,
		View3DWidget,
		View3DToolbarComponent,
		$) {
		"use strict";
		/**
		 * Creates the View3DModule
		 *
		 * @exports View3DModule
		 * @memberof View3DModule
		 * @class
		 * Responsible for publishing a 3D Widget which can show the 3D model of ARD or Graphics if it's linked to an ARD
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var View3DModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "view3D");

			this._3DWidget = null;
			this._toolbarComponent = null;

			var boundUpdateToolbarVisibility = _.bind(this._updateToolbarVisibility, this);

			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(function (oldPage, newPage) {

				if (this._3DWidget) { //hide the widget when the active document page changed
					this._3DWidget.setEnabled(false);
					this._emit("stop3D");
				}

				// remove old listeners from old page
				if (oldPage) {
					oldPage.off("property.has3DPerspective.changed", boundUpdateToolbarVisibility);
				}

				// listen to has3DPerspective property changes
				newPage.on("property.has3DPerspective.changed", boundUpdateToolbarVisibility);

				this._updateToolbarVisibility();
			}, this);
		};


		// inherit
		View3DModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		View3DModule.prototype.constructor = View3DModule;

		/**
		 * Shows the 3D widget with the HTML5 3D Collada viewer
		 */
		View3DModule.prototype.show3D = function (position) {
			this._emit("start3D");
			if (!this._3DWidget) {
				this._publishWidget(View3DWidget.widgetName, _.bind(function (layoutType) {
					this._3DWidget = new View3DWidget(this.f);
					this._3DWidget.show(null, position);

					// React on enabled changes
					this._3DWidget.on("enabled", _.bind(function (enabled) {
						this._emit("3DWidget.enabled", enabled);
					}, this));
					this._emit("3DWidget.enabled", this._3DWidget.isEnabled());

					this._emit("3Dstarted");

					return this._3DWidget;
				}, this));
			} else {
				this._3DWidget.show(null, position);
				this._emit("3Dstarted");
			}
		};

		/**
		 * Is the 3D widget currently enabled
		 * @returns {Boolean} True if enabled, false if not enabled
		 */
		View3DModule.prototype.is3DWidgetEnabled = function () {
			if (this._3DWidget) {
				return this._3DWidget.isEnabled();
			}
			return false;
		};

		/**
		 * Initialize and shows the 3D toolbar component.
		 */
		View3DModule.prototype._show3DToolbarButton = function () {
			if (!this._toolbarComponent) {
				this._publishToolbarComponent(View3DToolbarComponent.toolbarName, _.bind(function (layoutType) {
					this._toolbarComponent = new View3DToolbarComponent(this.f, this);

					return this._toolbarComponent;
				}, this));
			} else {
				this._toolbarComponent.show();
			}
		};

		/**
		 * Update the visibility of the 3D toolbar component
		 */
		View3DModule.prototype._updateToolbarVisibility = function () {
			if (this.has3DPerspective()) {
				this._show3DToolbarButton();
				this._emit("has3DPerspective", true);
			} else {
				if (this._toolbarComponent) {
					this._toolbarComponent.hide();
				}
				this._emit("has3DPerspective", false);
			}
		};

		/**
		 * Is there a 3D perspective available? should the 3D view toolbarbutton or mode be available?
		 * @returns {Boolean}
		 */
		View3DModule.prototype.has3DPerspective = function () {
			return this.f.getActivePage().get("has3DPerspective");
		};


		/**
		 * Get the current camera position of the 3D widget if it is enabled and visible or null otherwise
		 * @returns {Object} @example 
		 *                   {scale: 1.5,
		 *                   azimuth: 46.6,
		 *                   elevation: -52}
		 */
		View3DModule.prototype.getCameraPosition = function () {
			if (this._3DWidget && this._3DWidget.isVisible() && this._3DWidget.isEnabled()) {
				return this._3DWidget.getCameraPosition();
			} else {
				return null;
			}
		};


		/**
		 * Set the current camera position of the 3D widget if it is enabled and visible
		 * Otherwise it will start loading the 3d widget
		 * @param {Object} position - 
		 *                 @example 
		 *                    {scale: 1.5,
		 *                    azimuth: 46.6,
		 *                    elevation: -52}
		 */
		View3DModule.prototype.setCameraPosition = function (position) {
			if (this._3DWidget && this._3DWidget.isVisible() && this._3DWidget.isEnabled()) {
				if (position !== undefined && position.scale !== undefined && position.azimuth !== undefined && position.elevation !== undefined) {
					this._3DWidget.setCameraPosition(position);
				}
			} else {
				this.show3D(position);
			}
		};


		return View3DModule;
	});