/**
 * @namespace WebviewerModule
 */
define(["web-viewer/core/AbstractModule",
	"web-viewer/webviewer/WebviewerWidgetModel"
],
	function (AbstractModule,
		WebviewerWidgetModel) {
		"use strict";

		/**
		 * @exports WebviewerModule
		 * @memberof WebviewerModule
		 * @class
		 * The WebviewerModule, this module installs the correct models for the viewport and load Graphics or Cad module
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var WebviewerModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "webviewer");


			this.webviewerWidgetModel = new WebviewerWidgetModel(f);
			f.getModule("viewport").setWidgetModel(this.webviewerWidgetModel);

			var docController = this.f.getDocumentController();

			this._pageChanged(undefined, this.f.getActivePage());
			this._pageChanged(undefined, docController.getCadOverlay());


			// react when the active page has changed
			docController.whenActivePageChanged(this._pageChanged, this);
			docController.whenCadOverlayChanged(this._pageChanged, this);
		};


		// inherit
		WebviewerModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		WebviewerModule.prototype.constructor = WebviewerModule;


		/**
		 * The active page or CAD overlay has changed, load the needed modules, and create a new view
		 *
		 * @param {Data.AbstractDocumentPage} oldPage
		 * @param {Data.AbstractDocumentPage} newPage
		 */
		WebviewerModule.prototype._pageChanged = function (oldPage, newPage) {
			var mainView = this.webviewerWidgetModel.getViewModel("Main");

			if (mainView === undefined || mainView === null) {
				this.webviewerWidgetModel.createView("Main", "", true);
			}

			if (newPage) {
				// create main view
				if (newPage.hasCad() || newPage.hasLinkedPage()) {
					this.f.requestModule("cad", undefined, this.silenced);
				}

				if (newPage.hasSvg()) {
					this.f.requestModule("svg", undefined, this.silenced);
				} else if (newPage.hasGraphics() || newPage.isBlankPage()) {
					this.f.requestModule("graphics", undefined, this.silenced);
				}

				// only load attribute category module when allowed and needed for the page
				if (this.f.getProperty("allowAttributeCategory") &&
					!this.f.isMobile() &&
					newPage.getAttributeCategoryID &&
					newPage.getAttributeCategoryID()) {
					this.f.requestModule("attributeCategory", undefined, this.silenced);
				}

				// load 3D module if there is a 3D perspective or a 3d perspective can be generated

				if (newPage.get("has3DPerspective")) {
					this.f.requestModule("view3D", undefined, this.silenced);
				}

				newPage.whenParametersAvailable(_.bind(function () {
					if ((newPage.hasGraphics() || newPage.isBlankPage() || newPage.hasSvg()) && newPage.getDocumentVersion().isMultiPage()) {
						this.f.requestModule("pagelist", undefined, this.silenced);
					}
				}, this));
			}
		};

		/**
		 *	Overrride the setSilenced method to also set the SVG Module as non silenced, indicating that it can publish its tools etc.
		 * 	@override
		 */
		WebviewerModule.prototype.setSilenced = function (isSilenced) {
			AbstractModule.prototype.setSilenced.call(this, isSilenced);
			if (!isSilenced) {
				var modules = ["svg", "cad", "graphics", "attributeCategory", "view3D", "pagelist"];
				modules.forEach(function (mod) {
					this.f.whenModuleAvailable(mod, function (mod) {
						mod.setSilenced(false);
					});
				}.bind(this));
			}

		};

		return WebviewerModule;
	});
